#!/bin/csh
#
#--------------------------------------------------------------------------------------------------
# NAME
#   setlocalpathc.csh
#
# SYNOPSIS
#   setlocalpath.sh
#
# DESCRIPTION
#   Set architecture dependent master path
#
# OPTIONS
#   None
#
# EXAMPLES
#   > source bin/setlocalpath.csh
#   > echo $HOMELOCALPATH
#   /home/bf/local/linux
#
# ENVIRONMENT
#   An environment variable
#     HOMELOCALPATH
#   is set.
#
# PROCEDURE
#
# WARNINGS
#
# BUGS
#
# TODO
#
# MODIFICATION HISTORY
#   2003-08-27 (B.F.) First version
#   2010-01-07 (B.F. Lyon) More cases
#--------------------------------------------------------------------------------------------------
#
#echo "Set local master path"
#echo "====================="
#
setenv MACHINE `uname -m`
#if [ "${MACHINE}" = 'i686' ]; then
#  MACMODEL=`cat /proc/cpuinfo | grep "model name" | sed -e "s/^.*: \(.*\)(.*/\1/" | sed -e "s/^\(.*[^ ]\) *$/\1/" | sed -e "s/(.*)//g" | head -1`
#else
#  MACMODEL=''
#fi
#
# === Find machine dependent path extension =======================================================
setenv MAC ''
#
if ( "${MACHINE}" == 'sv1' ) then
  # --- Cray SV1 (craSHi) -------------------------------------------------------------------------
  setenv MAC "sv1"
  #
else if ( "${MACHINE}" == 'alpha' ) then
  # --- alpha ------------------------------------------------------------------------------------
  setenv MAC "alpha"
  #
else if ( `echo ${MACHINE} | sed -e "s/^\(....\).*/\1/"` == '9000' ) then
  # --- HP 9000 -----------------------------------------------------------------------------------
  # 9000/800 (zeipel), 9000/785 (jay1)
  setenv MAC "hp"
  #
else if ( "${MACHINE}" == 'ia64' ) then
  # --- HP ia64 (Itanium) -------------------------------------------------------------------------
  setenv MAC "hpia64"
  #
else if ( "${MACHINE}" == 'SR8000' ) then
  # --- Hitachi -----------------------------------------------------------------------------------
  setenv MAC "vx1"
  #
else if ( "${MACHINE}" == 'i686' ) then
  setenv MAC "linux"
  #
else if ( "${MACHINE}" == 'x86_64' ) then
  setenv MAC "x86_64"
  #
else if ( "${MACHINE}" == 'i386' ) then
  setenv MAC "mac_i386"
  #
else if ( "${MACHINE}" == 'IBM' ) then
  # --- IBM p690 (Power4 CPUs)  -------------------------------------------------------------------
  setenv MAC "ibm"
  #
else if ( "${MACHINE}" == 'SX-5' || "${MACHINE}" == 'SX-6' || "${MACHINE}" == 'SX-8' ) then
  # --- NEC SX-5,6,8-------------------------------------------------------------------------------
  setenv MAC "nec"
  #
else if ( "${MACHINE}" == 'IP35' ) then
  # --- SGI ---------------------------------------------------------------------------------------
  setenv MAC "sgi"
  #
else if ( "${MACHINE}" == 'sun4u' || "${MACHINE}" == 'i86pc' ) then
  # --- Sun ---------------------------------------------------------------------------------------
  setenv MAC "sun"
  #
else if ( "${MACHINE}" == 'armv4l' ) then
  # --- Sharp Zaurus ------------------------------------------------------------------------------
  setenv MAC "zaurus"
  #
else
  # --- Unknown machine ---------------------------------------------------------------------------
  #echo 'Unknown architecture' ${MACHINE}
  setenv MAC "dummy"
  #
endif
#
#echo MAC ${MAC}
#
# --- Compose path ---
setenv HOMELOCALPATH "${HOME}/local/${MAC}"
#
#echo HOMELOCALPATH "${HOMELOCALPATH}"
