!   angles1:               Provide information about angles and weights used for ray system
!   angles2:               Provide information about angles and weights used for ray system
!   asnunit:               Assign information to channel unit (Crays only)
!   bicubic_coeff_1:       Compute bicubic interpolation coefficients from tabulated values in cell
!   bicubic_coeff:         Compute bicubic interpolation coefficients from tabulated values
!   bicubic_interpol_1:    Do bicubic interpolation in a single cell 
!   bicubic_interpol_gas:  Do bicubic interpolation on a regular grid, special GAS routine version
!   BnuwT:                 Bandpass weighting function
!   capco:                 Compute carbon monoxide - Planck mean opacity per gram
!   chem_rn_init:          Read chemistry data, initialize quantities
!   chem_rn_init_co_indices: Prepare indices of carbon containing chemical species C and CO
!   chem_rn_SourceStep:    Update model by applying CO source term step
!   clock:                 Return the remaining CPU time (on Hitachi)
!   CUBITc:                Perform an 1D-interpolation using cubic polynomials
!   CUBITIc:               Perform an 1D-interpolation using cubic polynomials
!   date_and_time:         Standard Fortran routine to get current date and time
!   dfoptad:               Read dust opacity table data and store it in common blocks
!   dfopta:                Read opacity table data and store it in common blocks
!   DKINIT:                Set parameters and compute Spline coefficients for C-rich chemistry
!   dust_bins_rossow78:    Compute characteristic rates for formation of Mg2SiO4 dust
!   dust_bins_SourceStep:  Update model by applying dust source term step (dust bin scheme)
!   DUSTFORM_3:            Compute rates for simplified forsterite dust formation model
!   DUSTFORM_6:            Compute condensation or evaporation rate for forsterite
!   dust_Al2O3AGBUT_TestAll: Perform unit tests of corundum dust routines
!   dust_Al2O3AGB_1Dchange: Calculate the relative amount of corundum dust at the new time level
!   dust_Al2O3AGB_1DchangeUT: Perform unit tests for dust_Al2O3AGB_1Dchange
!   dust_Al2O3AGB_1DchangeUTsimple: Perform simple unit tests for dust_Al2O3AGB_1Dchange
!   dust_Mg2SiO4AGBUT_TestAll: Perform unit tests of forsterite dust routines
!   dust_Mg2SiO4AGB_1Dchange: Calculate the relative amount of forsterite dust at the new time level
!   dust_Mg2SiO4AGB_1DchangeUT: Perform unit tests for dust_Mg2SiO4AGB_1Dchange
!   dust_Mg2SiO4AGB_1DchangeUTsimple: Perform simple unit tests for dust_Mg2SiO4AGB_1Dchange
!   eosinter_allOMP:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_all:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_logPTdTOMP:   Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_logPTdT:      Solve EOS by bicubic interpolation: rho, e -> logP, T, dTde
!   eosinter_PdP:          Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde
!   eosinter_PdPTdT:       Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde
!   eosinter_PTdT:         Solve EOS by bicubic interpolation: rho, e -> P, T, dTde
!   eosinter_PT:           Solve EOS by bicubic interpolation: rho, e -> P, T
!   eosinter:              Solve EOS by bicubic interpolation: rho, e -> P, dPdrho, dPde, T, dTde, s
!   eosinter_SwitchInfo:   Print information about used compiler switches
!   eosinter_TdT:          Solve EOS by bicubic interpolation: rho, e -> T, dTde
!   eosinter_TdTOMP:       Solve EOS by bicubic interpolation: rho, e -> T, dTde
!   eosinter_TOMP:         Solve EOS by bicubic interpolation: rho, e -> T
!   eosinter_Ts:           Solve EOS by bicubic interpolation: rho, e -> T, s
!   eostest_Check:         Check accuracy of interpolation table and speed of interpolation
!   etime:                 Many Unix systems: return the elapsed CPU time
!   flush:                 Flush output buffer
!   gas:                   EOS routine of RHD-code (or gasgas)
!   gasgas:                EOS routine of HDW-code
!   gasinter_AllDerivatives: Compute numerically 1st and 2nd derivatives for 2D array function
!   gasinter_Check:        Check accuracy of interpolation table
!   gasinter_Checkgamma:   Check accuracy of interpolation table for gamma law
!   gasinter_CheckGas:     Check accuracy of interpolation table for GAS quantities
!   gasinter_ModifyGas:    Modify precomputed set of EOS quantities
!   gasinter_ReadPar:      Read parameters from input parameter file
!   gasinter_SetUp:        Compute coefficients for bicubic interpolation of GAS/EOS quantities
!   gasinter_SetUpgamma:   Compute coeffs for bicubic interpolation of EOS qs with simple gamma law
!   gasinter_SetUpGas:     Compute coefficients for later bicubic interpolation of GAS quantities
!   gasinter:              Solve EOS by bicubic interpolation: P  , T -> rho, h, ..
!   gasinv:                Inversion of GAS routine: rho/e -> P,T, ..
!   hion_sourcestep:       Update model by applying Hydrogen-ionization source terms
!   kappadust:             Compute Rosseland dust opacity for amorph. carbon in Small-Particle Limit
!   lcfrad3:               Solve the radiative transfer equation along a number rays
!   lcfrad_ini:            Define constants and geometrical properties of the rays system.
!   lcfrad_cls:            Deallocate the arrays allocated by lcfrad_ini.
!   lfcrad_switchinfo:     Print information about used compiler switches
!   MOMENT:                Calculate the 4 moments for carbon cust at the new time level
!   MPI_AllReduce:         Perform a reduction and send result to all processes
!   MPI_Barrier:           Block untill all processes have reached this routine
!   MPI_BCast:             Broadcast a message from one process to all others in the group
!   MPI_Cart_Coords:       Determine process coordinates in Cartesian topology from rank in group
!   MPI_Cart_Create:       Create a new communicator with attached Cartesian topology information
!   MPI_Cart_Rank:         Determine process rank in communicator from Cartesian coordinates
!   MPI_Cart_Shift:        Compute the shifted source and destination ranks from direction a. amount
!   MPI_Cart_Sub:          Partition a Cartesian communicator group into lower-dimension sub groups
!   MPI_Comm_Create:       Create a new communicator
!   MPI_Comm_Free:         Free a communicator
!   MPI_Comm_Get_Attr:     Get an attribute value
!   MPI_Comm_Get_Errhandler: Retrieve error handler associated with a communicator
!   MPI_Comm_Group:        Return the group associated with a communicator
!   MPI_Comm_Rank:         Determine the rank of the calling process in the communicator
!   MPI_Comm_Set_ErrHandler: Attach a new error handler to a communicator
!   MPI_Comm_Size:         Get number of processes for given communicator
!   MPI_Comm_Split:        Split a communicator
!   MPI_Finalize:          Terminate the MPI execution environment
!   MPI_Gatherv:           Gather varying amounts of data from all processes to the root process
!   MPI_Get_Address:       Get the address of a location in memory
!   MPI_Get_Library_Version: Get the name of the MPI library version
!   MPI_Get_Processor_Name: Get processor name for current process
!   MPI_Group_Excl:        Produce a group by removing listed members
!   MPI_Group_Free:        Free a group
!   MPI_Group_Incl:        Produce a group by taking only listed members
!   MPI_Group_Rank:        Return the rank of the calling process in the given group
!   MPI_Group_Translate_Ranks: Translate the ranks of processes in one group to those in another one
!   MPI_Group_Union:       Produce a group by combining two groups
!   MPI_Init:              Initializes the MPI execution environment
!   MPI_IRecv:             Start a standard-mode non-blocking receive
!   MPI_ISend:             Start a standard-mode non-blocking send
!   MPI_Recv:              Perform a blocking receive
!   MPI_Reduce:            Perform a reduction
!   MPI_Send:              Perform a blocking send
!   MPI_SendRecv:          Perform a combined blocked send and receive
!   MPI_SSend:             Perform a blocking synchronous send
!   MPI_Type_Commit:       Commit an MPI data type
!   MPI_Type_Create_Struct: Create an MPI structured data type
!   MPI_Type_Create_Subarray: Create an MPI data type describing a sub array
!   MPI_Type_Free:         Free an MPI data type
!   MPI_Wait:              Wait for given communication to complete
!   MPI_WaitAll:           Wait for all given communications to complete
!   mscrad_cls:            Deallocate the arrays allocated by mscrad_ini
!   mscrad_geo_0:          Set up geometrical constants for MC RT (part 1)
!   mscrad_geo_1:          Set up geometrical constants for MC RT (part 2)
!   mscrad_ini:            Define constants and geometrical properties of the rays system
!   mscrad3:               Solve the radiative transfer equation with a short-characteristics method
!   mscrad_switchinfo:     Print information about used compiler switches
!   omp_get_schedule:      OpenMP system routine to get scheduling settings
!   op_dust1:              Compute dust opacity for various dust models for one vector (old version)
!   op_dust2:              Compute dust opacity for various dust models for one vector
!   op_dust:               Compute dust opacity for various dust models for 3D arrays with OpenMP
!   op_s0:                 Compute the Rosseland mean opacity in opclam
!   op_s1:                 Compute opacities and other stuff: blinci, blsurf, dipnwt, opclam, blclam
!   opta_Info:             Give stored information about current opacity table
!   opta_Modify:           Manipulate opacity data
!   opta_SwitchInfo:       Print information about used compiler switches
!   pxfstrget:             Auxiliary PXF routine (system call on Cray TS)
!   pxfstructcreate:       Auxiliary PXF routine (system call on Cray TS)
!   pxfstructfree:         Auxiliary PXF routine (system call on Cray TS)
!   pxfuname:              Get information about environment and machine (system call on Cray TS)
!   random_seed:           Set the random-generator seed
!   raysca5:               Generalized version of rayvec5 allowing to include approximate scattering
!   raytas:                Compute the optical depth increments
!   rayvec5:               Solve the equation of radiative transfer dI/dtau = (I - B)
!   readpar:               Read parameter from input parameter file
!   rhd_action_New:        Function: Change contents of action variable
!   rhd_action_InitNameAr: Initialize global name arrays
!   rhd_action_StrToRecon: Analyze string with reconstruction scheme
!   rhd_baserad_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_baserad_DirShuffle: Shuffle and revert axis directions randomly
!   rhd_baserad_PipeIndex: Compute the ray index for job pipelining
!   rhd_baserad_PipeIndexUT: Perform unit tests for rhd_baserad_PipeIndex
!   rhd_baserad_RayDirections: Create list of basic direction vectors
!   rhd_baserad_RayDirectionsUT: Perform unit tests for rhd_baserad_RayDirections
!   rhd_baserad_RaySystem: Create list of direction vectors for actual rotation matrix
!   rhd_baserad_RotationMatrixAxisAngle: Create matrix for rotation from rotation axis and one angle
!   rhd_baserad_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   rhd_baserad_RotationMatrixUT: Perform unit tests for rhd_baserad_RotationMatrix
!   rhd_baseradUT_TestAll: Perform all unit tests of basic radiation-transport routines
!   rhd_baseradUT_TestSome: Perform some unit tests of basic radiation-transport routines
!   rhd_bound_1DGhostArray: Fill ghost cells of single array according to boundary conditions
!   rhd_bound_1DGhost:     Fill ghost cells according to boundary conditions
!   rhd_bound_1DGhostOneUT: Perform unit test for rhd_bound_1DGhost
!   rhd_bound_1DGhostUT:   Perform unit tests for rhd_bound_1DGhost
!   rhd_bound_1DTransmit:  Adjust values in top or bottom cells for transmitting boundary condition
!   rhd_bound_1DTransmit_mhd: Correct values in top or bottom cells for transmitting boundary cond
!   rhd_bound_3DCenter:    Modify energy in central cells to keep entropy constant
!   rhd_bound_3DHeatBottom: Modify energy in bottom cells to keep entropy constant
!   rhd_bound_3DInOutFlow: Adjust values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DInOutFlow_mhd: Correct values in bottom cells to fulfill the open boundary condition
!   rhd_bound_3DTransmit:  Adjust density values in top cells for the open boundary condition
!   rhd_bound_3DTransmit_mhd: Correct values in cells of all sides for transmitting boundary cond
!   rhd_bound_ExtractBox:  Extract part of box, fill boundary ghost cells
!   rhd_bound_ExtractBoxMPI: Extract part of box, fill boundary ghost cells (MPI version; no MHD)
!   rhd_bound_SwitchInfo:  Print information about used compiler switches
!   rhd_box_Add:           Add (part of) box to another. Transpose, if requested
!   rhd_box_AddChange:     Add change of (part of) box into another. Transpose, if requested
!   rhd_box_AddDelta2:     Add delta of (part of) box into another
!   rhd_box_AddDelta:      Add delta of (part of) box into another. Transpose, if requested
!   rhd_box_AllAddDelta2OMP: Add change to entire box, OpenMP-parallelized if n3>m3
!   rhd_box_AllAddDeltaOMP: Add change to entire box, OpenMP-parallelized if n3>m3
!   rhd_box_AllAddOMP:     Add box to another, OpenMP-parallelized if n3>m3
!   rhd_box_AllAvgOMP:     Average two boxes, OpenMP-parallelized if n3>m3
!   rhd_box_AllCopyOMP:    Copy entire box, OpenMP-parallelized if n3>m3
!   rhd_box_AllFactorOMP:  Multiply contents of box with const. factor, OpenMP-parallelized if n3>m3
!   rhd_box_AppendBc:      Add cell-centered magnetic-field arrays to box
!   rhd_box_CheckAxes:     Check axes in box and write result to specified channel
!   rhd_box_CheckAxes2D:   Check aspect ratio of two axes
!   rhd_box_CheckAxis1D:   Check step size of 1D axis
!   rhd_box_ChunkSize:     Determine proper size of regular chunk
!   rhd_box_ChunkSizeOneUT: Perform unit test for rhd_box_ChunkSize
!   rhd_box_ChunkSizeUT:   Perform unit tests for rhd_box_ChunkSize
!   rhd_box_Copy:          Copy (part of) box into another. Transpose and shift, if requested
!   rhd_box_CopyOMP:       Copy (part of) box into another, OpenMP-parallelized if n3>m3
!   rhd_box_Delete:        Delete computational box. Deallocate memory
!   rhd_box_Exist:         Function: Test if box is already filled
!   rhd_box_Factor:        Multiply contents of box with constant factor
!   rhd_box_FillAxes:      Fill axes of box
!   rhd_box_FillGhostAxis1D: Fill ghost cells of axis vectors for real boundaries
!   rhd_box_FillGhostAxes: Fill ghost cells of box axes for real boundaries
!   rhd_box_FillTestOMP:   Set bulk arrays in box structure to some test values
!   rhd_wrk_boxGloDelete:  Delete global computational boxes. Deallocate memory
!   rhd_wrk_boxGloInit:    Initialize global computational boxes. Nullify pointers
!   rhd_box_Init:          Initialize computational box. Nullify pointers
!   rhd_box_Make:          Create computational box of requested size
!   rhd_box_MPICopy:       Copy distribution of boxes from one MPI partition to another
!   rhd_box_MPICreateType: Create MPI data type for box structure
!   rhd_box_MPIFillGhost:  Fill ghost cells by communicating with neighbor MPI processes
!   rhd_box_MPIFillGhostAxes: Fill axes in ghost layers by communicating with neighbor MPI processes
!   rhd_box_MPIFillGhostEnd: Finalize filling ghost cells
!   rhd_box_MPIFillGhostStart: Start filling "internal" ghost cells
!   rhd_box_PrintStat:     Write statistics about box to specified channel
!   rhd_box_RdData:        Read RHD model from file in UIO form
!   rhd_box_Read:          Read box from file in UIO form
!   rhd_box_RemoveBc:      Remove cell-centered magnetic-field arrays from box
!   rhd_box_RestIndices:   Compute indices of boxes surrounding small box in large box
!   rhd_box_RestIndicesOneUT: Perform unit test for rhd_box_RestIndices
!   rhd_box_RestIndicesUT: Perform unit tests for rhd_box_RestIndices
!   rhd_box_SubBoxIndices: Compute indices of box representing the section of both input boxes
!   rhd_box_SwitchInfo:    Print information about used compiler switches
!   rhd_boxUT_Test:        Do unit-test checks on arrays in box structure
!   rhd_boxUT_TestAll:     Perform all unit tests of box-handling routines
!   rhd_boxUT_TestSome:    Perform some unit tests of box-handling routines
!   rhd_box_WrData:        Write RHD model into file in UIO form
!   rhd_box_Write:         Write box with additional infos into file in UIO form
!   rhd_box_ZeroOMP:       Set bulk arrays explicitely to zero
!   rhd_dat_Add:           Add contents of two structures for additional data
!   rhd_dat_AddOverlap:    Add overlap of contents of two structures for additional data
!   rhd_datc_Add:          Add the fields in the structure for cylindrical mean data
!   rhd_datc_AddOverlap:   Add the overlap of fields in the structure for cylindrical mean data
!   rhd_datc_Copy:         Copy data structure for azimuthal means
!   rhd_datc_MPICollect:   Collect distributed azimuthal-mean data into single structure
!   rhd_datc_Delete:       Delete structure for cylindrical mean data. Deallocate memory
!   rhd_datc_Make:         Create structure of requested size for 2D cylindrical mean data
!   rhd_dat_MPICollect:    Collect distributed data from MPI processes into single structure
!   rhd_dat_MPICollectInt: Collect distributed intensities from MPI processes into single structure
!   rhd_dat_MPICollectFer: Collect distributed rad. fluxes from MPI processes into single structure
!   rhd_datc_Reset:        Set fields in structure for cylindrical mean data to zero
!   rhd_datc_SetIndices:   Set indices for quantities in structure for 2D cylindrical mean data
!   rhd_datc_Write:        Write additional RHD data into file in UIO form
!   rhd_dat_Delete:        Delete structure for additional data. Deallocate memory
!   rhd_dat_Make:          Create structure for data of requested size
!   rhd_dat_MPICreateType: Create MPI data type for dat structure
!   rhd_dat_Reset:         Set fields in structure for additional data to zero
!   rhd_dat_SwitchInfo:    Print information about used compiler switches
!   rhd_dat_Write:         Write additional RHD data into file in UIO form
!   rhd_datx_Add:          Add the fields in structure for horizontal/radial mean data
!   rhd_datx_AddOverlap:   Add the overlap of fields in structure for horizontal/radial mean data
!   rhd_datx_Axis:         Construct radial axis for datx
!   rhd_datx_MPICollectr:  Collect distributed spherical-mean data into single structure
!   rhd_datx_MPICollectx:  Collect distributed horizontal-mean data into single structure
!   rhd_datx_Copy:         Copy data structure for horizontal or spherical means
!   rhd_datx_Delete:       Delete structure for additional data. Deallocate memory
!   rhd_datx_Make:         Create structure for data of requested size
!   rhd_datx_Reset:        Set fields in structure for additional data to zero
!   rhd_datx_SetIndices:   Set indices for quantities in structure for 1D mean data
!   rhd_datx_Write:        Write additional RHD data into file in UIO form
!   rhd_dust_COSourceStep: Update model by applying CO source term step
!   rhd_dust_K3MON2SourceStep: Update model by applying dust source term step (one moment only)
!   rhd_dust_K3MONSourceStep: Update model by applying dust source term step (one moment only)
!   rhd_dust_M4C2SourceStep: Update model by applying dust source term step (4 moments)
!   rhd_dust_Mg2SiO4AGBSourceStep: Update model by applying forsterite source term step
!   rhd_dust_Mg2SiO4pAl2O3AGBSourceStep: Update model by applying forsterite+corundum source terms
!   rhd_dust_PropInit:     Initialize property structure and set e.g. number of dust components
!   rhd_dust_simpleSourceStep: Update model by applying simple (test) dust source term step
!   rhd_dust_SourceStep:   Update model by applying source term step
!   rhd_geo_Coords2mn:     Compute box boundary indices from MPI process coordinates
!   rhd_geo_Coords2mnVec:  Compute box boundary indices from MPI process coordinates
!   rhd_geo_Copy:          Copy world-geometry structure
!   rhd_geo_Init:          Initialize world-geometry structure
!   rhd_geo_Print:         Print information about geometry structure
!   rhd_geo_Toaction:      Copy boundary-condition settings from geo structure to action structure
!   rhd_grav:              Compute gravitational potential
!   rhd_hyd_ApplyGrav:     Apply gravitational acceleration
!   rhd_hyd_Coriolis:      Apply Coriolis rotation
!   rhd_hyd_CoriolisOMP:   Apply Coriolis rotation (with local OpenMP directives)
!   rhd_hyd_Dust:          Advect passive tracers
!   rhd_hyd_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_hyd_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   rhd_hyd_Fluxrhoei:     Compute 1D internal energy flux from the fluxes of individual waves
!   rhd_hyd_FluxState:     Compute 1D flux from state vector for single quantity
!   rhd_hyd_mhd:           Advect magnetic fields
!   rhd_hyd_Pred:          Compute reduced pressure as gravity correction for Roe solver
!   rhd_hyd_Roe1DMincedStep: Perform 1D hydro step in x1 direction, call Roe solver repeatedly
!   rhd_hyd_Roe1Doutput:   Extract some averaged quantities
!   rhd_hyd_Roe1DStep:     Perform 1D hydrodynamics step in x1 direction with Roe scheme
!   rhd_hyd_RoeState1Dflux: Compute 1D Roe fluxes with state reconstructions
!   rhd_hyd_RoeWave1Dflux: Compute 1D Roe fluxes with wave reconstructions
!   rhd_hyd_RoeWavealpha12: Compute 1D Roe sound-wave fluxes
!   rhd_hyd_RoeWavealpha36: Compute 1D Roe entropy-wave and ionization-energy fluxes
!   rhd_hyd_RoeWavealpha45: Compute 1D Roe transversal-momentum fluxes
!   rhd_hyd_Rossow78tfall: Calculate dust sedimentation velocity
!   rhd_hyd_sDiff:         Compute 1D energy flux down an entropy step with state reconstruction
!   rhd_hyd_SingleStep:    Perform entire (single) 2D or 3D hydrodynamics step, employ Roe scheme
!   rhd_hyd_Step:          Perform entire 3D hydro step, possibly composed of multiple time-steps
!   rhd_hyd_SwitchInfo:    Print information about used compiler switches
!   rhd_lcfrad_cls:        Deallocate arrays used by LCFrad by calling lcfrad_cls
!   rhd_lcfrad_init:       Initialize LCFrad radiative transfer equations by calling lcfrad_ini
!   rhd_lcfrad_output:     Derive some averages as additional output quantities
!   rhd_lcfrad_step:       Perform 3d radiative energy transfer step
!   rhd_lcfrad_TemperatureLimiter: Correct int. energy to prevent temp. from dropping below limit
!   rhd_mean_a2rindex:     Produce index arrays for averaging over spheres in an asymmetric box
!   rhd_mean_a2rindexOneUT: Perform one set of unit tests for rhd_mean_a2rindex
!   rhd_mean_a2rindexPairUT: Perform pairwise unit tests for rhd_mean_a2rindex
!   rhd_mean_arindex:      Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_aweight:      Produce weight array for horizontal averaging
!   rhd_mean_ax23weight:   Produce weight arrays for averaging over spheres
!   rhd_mean_c1Index:      Produce index array for azimuthal averaging over cylinders
!   rhd_mean_c1VecProj:    Project vector on radial vector perpendicular to x1 axis
!   rhd_mean_ccamean:      Average cell-centered quantity over cylinders (azimuthal mean)
!   rhd_mean_rb12mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rb13mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rb23mean:     Average boundary centered quantity over spheres in an asymmetric box
!   rhd_mean_rbmean:       Average boundary-centered quantity over spheres in a symmetric box
!   rhd_mean_rc23mean:     Average cell-centered quantity over spheres in an asymmetric box
!   rhd_mean_rcmean:       Average cell-centered quantity over spheres in a symmetric box
!   rhd_mean_rindex:       Produce index arrays for averaging over spheres in a symmetric box
!   rhd_mean_rindexOneUT:  Perform one set of unit tests for rhd_mean_rindex
!   rhd_mean_rindexUT:     Perform unit tests for rhd_mean_rindex and rhd_mean_a2rindex
!   rhd_meanUT_TestAll:    Perform unit tests of all averaging routines
!   rhd_meanUT_TestSome:   Perform unit tests of some averaging routines
!   rhd_mean_VecProj:      Project vector on radial ray
!   rhd_mean_weight:       Produce weight array for horizontal averaging
!   rhd_mean_x12mean:      Average quantity horizontally in x12 plane
!   rhd_mean_x12meanUT:    Perform unit tests of rhd_mean_x12mean
!   rhd_mean_x13mean:      Average quantity horizontally in x13 plane
!   rhd_mean_x23mean:      Average quantity horizontally in x23 plane
!   rhd_mean_x23meanUT:    Perform unit tests of rhd_mean_x23mean
!   rhd_mean_x23weight:    Produce weight arrays for averaging over spheres
!   rhd_mhd_Bb2Bc:         Compute cell-centered magnetic field from boundary values
!   rhd_mhd_CheckFlux:     Check if fluxes are save and replace by 1st order HLL fluxes if necessary
!   rhd_mhd_Constrain:     Perform constrained-transport step for magnetic fields
!   rhd_mhd_FReqvec:       Reconstruct with Frankenstein scheme and compute boundary values
!   rhd_mhd_Hancock:       Perform Hancock predictor step in 1 dimension
!   rhd_mhd_Hancock2D:     Perform Hancock predictor step in 2 dimensions
!   rhd_mhd_Hancock3D:     Perform Hancock predictor step in 3 dimensions
!   rhd_mhd_HLL1Dflux:     Compute 1D MHD HLL fluxes
!   rhd_mhd_HLL1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (HLL solver)
!   rhd_mhd_HLL2Dstep:     Perform 2D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_HLL3Dstep:     Perform 3D magnetohydrodynamics step (HLL solver)
!   rhd_mhd_PLeqvec:       Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_PLvec:         Piecewise linear reconstruction using different slope limiters
!   rhd_mhd_PPeqvec:       Reconstruct with piecewise-parabolic scheme and compute boundary values
!   rhd_mhd_PPvec:         Reconstruct with piecewise-parabolic scheme and compute boundary values
!   rhd_mhd_Reconstruct:   Reconstruct quantities in cells and compute boundary values
!   rhd_mhd_Res2DStep:     Perform 2D resistivity step
!   rhd_mhd_Res3Doutput:   Extract some averaged quantities of resistivity step
!   rhd_mhd_Res3DStep:     Perform 3D resistivity step
!   rhd_mhd_Roe1Dflux:     Compute 1D MHD Roe fluxes
!   rhd_mhd_Roe1Doutput:   Extract some averaged quantities
!   rhd_mhd_Roe1Dstep:     Perform 1D magnetohydrodynamics step in x1 direction (Roe solver)
!   rhd_mhd_SingleStep:    Perform single magnetohydrodynamic step
!   rhd_mhd_Step:          Perform entire 2D or 3D magneto-hydrodynamics step, employ HLLE scheme
!   rhd_mhd_SwitchInfo:    Print information about used compiler switches
!   rhd_mhd_UnsplitStep:   Perform single unsplit magnetohydrodynamic step
!   rhd_MPI_AllReduceReals: Reduce and broadcast a number of real scalars in one go
!   rhd_MPI_boxRead:       Read box from file in UIO form and distribute it to other processes
!   rhd_MPI_boxWrite:      Write box with additional information into file in UIO form
!   rhd_MPI_BCastReals:    Broadcast a number of real scalars in one go
!   rhd_MPI_CartSetup:     Set up Cartesian MPI communicator and put data into geometry variable
!   rhd_MPI_DimsCreate:    Find and limit number of MPI processes per dimension
!   rhd_MPI_DtimeErrorDist: Distribute time step, error code, and output string among processes
!   rhd_MPI_ErrorDist:     Distribute error code and output string among processes
!   rhd_MPI_PrintInfo:     Write infomation about MPI environment to output channel
!   rhd_mscrad_cls:        Deallocate arrays used by MSCrad by calling mscrad_cls
!   rhd_mscrad_init:       Initialize MSCrad radiative transfer equations by calling mscrad_ini
!   rhd_mscrad_output:     Derive some averages as additional output quantities
!   rhd_mscrad_step:       Perform 3d radiative energy transfer step
!   rhd_mscrad_TemperatureLimiter: Correct int. energy to prevent temp. from dropping below limit
!   rhd_OMP_PrintInfo:     Write infomation about OpenMP environment to output channel
!   rhd_par_PrintInfo:     Print information about RHD parameters
!   rhd_par_RdData:        Read parameters from parameter file in UIO form for RHD code
!   rhd_par_Read:          Read parameters from parameter file and put them into global variable
!   rhd_par_Toaction:      Prepare action variable from input parameters
!   rhd_par_ToSubpar:      Set values in parameter sub structure based on input parameters
!   rhd_par_Write:         Write RHD parameter into file in UIO form
!   rhd_prop_Init:         Initialize property structure
!   rhd_prop_MPIBCast:     Broadcast some data in the prop structure to other MPI processes
!   rhd_prop_MPICreateType: Create MPI data type for prop structure
!   rhd_prop_MPICreateTypex: Create MPI data type for axes in the prop structure
!   rhd_prop_Print:        Print information about property structure
!   rhd_rad3d_BaseAxisSystem: Generate random or unity basic orthonormal axis system
!   rhd_rad3d_diffdeep:    Compute fluxes with diffusion approximation in deeper layers
!   rhd_rad3d_diffthick:   Compute fluxes with diffusion approximation in optically thick part
!   rhd_rad3d_dir:         Compute radiative energy flux for one direction field
!   rhd_rad3d_dirper:      Compute radiative energy flux for one direction field, periodic bnds.
!   rhd_rad3d_formal:      Compute formal solution: 3D radiative energy flux
!   rhd_rad3d_fromray:     Put fluxes and energy change from ray back to hydro grid
!   rhd_rad3d_lambda:      Do Lambda* iteration
!   rhd_rad3d_projectarea1: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectarea: Compute fraction of area of parallelogram within rectangle
!   rhd_rad3d_projectarea_llw: Compute length of interval [xp0,xp1] that lies within [xr0,xr1]
!   rhd_rad3d_projectarea_sort: Sort real vector y in ascending order
!   rhd_rad3d_projectonplane1: Project point along given direction on plane
!   rhd_rad3d_projectonplane: Project point along given direction on plane
!   rhd_rad3d_RayDirections: Create list of basic direction vectors
!   rhd_rad3d_raylhd:      Solve the radiative transfer equation for one ray by Feautrier scheme
!   rhd_rad3d_RotationMatrix: Create matrix for rotation in 3D from 3 angles
!   rhd_rad3d_smooth:      Smooth 3D array
!   rhd_rad3d_solveadapt:  Perform radiative energy transfer for single ray on refined grid
!   rhd_rad3d_solveeq:     Perform radiative energy transfer for single ray on equidistant grid
!   rhd_rad3d_solve:       Perform radiative energy transfer step for single ray
!   rhd_rad3d_StepOutput:  Derive some averages as additional output quantities
!   rhd_rad3d_Step:        Perform 3d radiative energy transfer step
!   rhd_rad3d_SwitchInfo:  Print information about used compiler switches
!   rhd_rad3d_toray:       Interpolate rho, P, and T onto ray
!   rhd_rad3d_Tvis:        Compute additional "viscous" radiative fluxes in optically thin p
!   rhd_rec_ConstantEq:    Reconstruct partial wave (Constant), compute flux
!   rhd_rec_FluxalphaEqFRweno2: Reconstruct partial wave (FRweno2), compute flux on equidistant grid
!   rhd_rec_FluxalphaEqHBweno: Reconstruct partial wave (HBweno), compute flux on equidistant grid
!   rhd_rec_FluxalphaEq:   Reconstruct partial wave and compute flux on equidistant grid
!   rhd_rec_FluxalphaEqVAweno: Reconstruct partial wave (VAweno), compute flux on equidistant grid
!   rhd_rec_FluxalphaHBweno: Reconstruct partial wave (HBweno), compute flux on general grid
!   rhd_rec_Fluxalpha:     Reconstruct partial wave and compute flux on general grid
!   rhd_rec_FluxalphaVAweno: Reconstruct partial wave (VAweno), compute flux on general grid
!   rhd_rec_FRcontEq:      Reconstruct partial wave (FRcont), compute flux of wave on equidist. grid
!   rhd_rec_FRmonoEq:      Reconstruct partial wave (FRmono), compute flux of wave on equidist. grid
!   rhd_rec_FRwenoEq:      Reconstruct partial wave (FRweno), compute flux of wave on equidist. grid
!   rhd_rec_HBwenoSmoothness: Compute smoothness indicator
!   rhd_rec_MinmodEq:      Reconstruct partial wave (Minmod), compute flux of wave, equidistant grid
!   rhd_rec_Minmod:        Reconstruct partial wave (Minmod), compute flux of wave, general grid
!   rhd_rec_PPAWEq:        Reconstruct partial wave (PPAW), compute flux of wave on equidistant grid
!   rhd_rec_PPAW:          Reconstruct partial wave (PPAW), compute flux of wave on general grid
!   rhd_rec_PPEq:          Reconstruct partial wave (PP), compute flux of wave on equidistant grid
!   rhd_rec_PPmimoEq:      Reconstruct partial wave (PPmimo), compute flux of wave on equidist. grid
!   rhd_rec_PPmimo:        Reconstruct partial wave (PPmimo), compute flux of wave on general grid
!   rhd_rec_PP:            Reconstruct partial wave (PP), compute flux of wave on general grid
!   rhd_rec_SuperBeeEq:    Reconstruct partial wave (SuperBee), compute flux on equidistant grid
!   rhd_rec_SuperBee:      Reconstruct partial wave (SuperBee), compute flux on general grid
!   rhd_recUT_OneRec:      Perform unit tests for one reconstruction type
!   rhd_recUT_TestAll:     Perform unit tests of reconstruction routines
!   rhd_recUT_Vector:      Fill arrays with test values
!   rhd_recUT_Wrap:        Reconstruct and compute boundary values will different methods
!   rhd_rec_vanLeerEq:     Reconstruct partial wave (vanLeer), compute flux on equidistant grid
!   rhd_rec_vanLeer:       Reconstruct partial wave (vanLeer), compute flux on general grid
!   rhd_rec_VecEqFRcont:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqFRmono2:  Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRmono:   Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEqFRweno2:  Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqFRweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqHBweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecEqPPmimo:   Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecEqPP:       Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecEq:         Reconstruct input vector and compute boundary values on equidistant grid
!   rhd_rec_VecEqSimple:   Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_rec_VecEqVAweno:   Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRcont:     Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecFRmono2:    Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRmono:     Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_VecFRweno2:    Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecFRweno:     Reconstruct vector, compute boundary values with 2nd-order scheme
!   rhd_rec_VecPPmimo:     Reconstruct vector, compute boundary values with 1st-order mono. scheme
!   rhd_rec_VecPP:         Reconstruct vector, compute boundary values with 2nd-order mono. scheme
!   rhd_rec_Vec:           Reconstruct input vector and compute boundary values on general grid
!   rhd_rec_VecSimple:     Reconstruct vector, compute bnd values with 0th or 1st-order mono. scheme
!   rhd_scale_Final:       Finalization of module
!   rhd_scale_Init:        Initialization of module
!   rhd_scale_Step:        Scale model for one time step
!   rhd_scale_SwitchInfo:  Print information about used compiler switches
!   rhd_shortrad_Dir1:     Compute radiative energy flux for one direction field, main direction 1
!   rhd_shortrad_Dir2:     Compute radiative energy flux for one direction field, main direction 2
!   rhd_shortrad_Dir3:     Compute radiative energy flux for one direction field, main direction 3
!   rhd_shortrad_DirSimple1: Compute radiative energy flux, simple direction field, main direction 1
!   rhd_shortrad_DirSimple2: Compute radiative energy flux, simple direction field, main direction 2
!   rhd_shortrad_DirSimple3: Compute radiative energy flux, simple direction field, main direction 3
!   rhd_shortrad_dtauop:   Compute contribution to radiative time scale from optical thickness
!   rhd_shortrad_EOS:      Compute EOS quantities needed for radiative transfer
!   rhd_shortrad_EOSTest:  Compute EOS quantities with various subroutines and measure the timings
!   rhd_shortrad_Formal:   Compute formal solution: 3D radiative energy flux and change
!   rhd_shortrad_LambdadeiTune: Tune energy correction step size
!   rhd_shortrad_LambdaFluct: Compute measures for local fluctuations
!   rhd_shortrad_LambdaIter: Do accelerated Lambda iteration to get radiative energy change
!   rhd_shortrad_LambdaSmoothdei: Perform spatial smoothing of energy update
!   rhd_shortrad_LambdaWriteInfo: Print detailed information about step size of Lambda iteration
!   rhd_shortrad_LocalSource: Perform radiation update step employing local approximation
!   rhd_shortrad_MPIDir2Int1: Transfer x1 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir2Int3: Transfer x3 boundary intensity between processes for x2 integration
!   rhd_shortrad_MPIDir3Int1: Transfer x1 boundary intensity between processes for x3 integration
!   rhd_shortrad_MPIDir3Int2: Transfer x2 boundary intensity between processes for x3 integration
!   rhd_shortrad_operator: Perform short-characteristics step for pair of grid cells
!   rhd_shortrad_OpTa:     Compute opacities and source function needed for radiative transfer
!   rhd_shortrad_OpTaTest: Compute opacities with various subroutines and measure the timings
!   rhd_shortrad_RayWeight: Compute weights for incoming rays
!   rhd_shortrad_StepOutput: Derive some averages as additional output quantities
!   rhd_shortrad_Step:     Perform 3D radiation step w. short-characteristics or local-source scheme
!   rhd_shortrad_SubStep:  Perform 3d radiative energy transfer step using short characteristics
!   rhd_shortrad_SwitchInfo: Print information about used compiler switches
!   rhd_shortrad_tauCouple1: Compute tau coupling to get radiative time scale, direction 1
!   rhd_shortrad_tauCouple2: Compute tau coupling to get radiative time scale, direction 2
!   rhd_shortrad_tauCouple3: Compute tau coupling to get radiative time scale, direction 3
!   rhd_tweak_Final:       Finalization of tweak module
!   rhd_tweak_Init:        Initialization of tweak module
!   rhd_tweak_Step:        Tweak model for one time step
!   rhd_tweak_SwitchInfo:  Print information about used compiler switches
!   rhdUT_master_TestAll:  Perform all unit tests for RHD code
!   rhd_vis_2dStep:        Update model by applying 2D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_3dStep:        Update model by applying 3D tensor viscosity (incl. turbulent mixing)
!   rhd_vis_p2p2dStep:     Update model by applying 2D point-to-point (tensor) viscosity
!   rhd_vis_p2p3dStep:     Update model by applying 3D point-to-point (tensor) viscosity
!   rhd_vis_p2pStepOutput: Put some additional output quantities into dat structure
!   rhd_vis_Step:          Complete viscosity step
!   rhd_vis_StepOutput:    Put some additional output quantities into dat structure
!   rhd_vis_SwitchInfo:    Print information about used compiler switches
!   rhd_wg_Init:           Initialize global collection of world-geometry structures
!   rsys2d:                Solve the two-dimensional radiation-transfer problem
!   rsys2d_mcq_v:          MC RT for Qrad on vertical rays 
!   rsys2d_mcq_x:          MC RT for Qrad on inclined rays (phi average)
!   rsys2d_mcq_vxa:        MC RT for Qrad, averaging over all mu angles
!   rsys2d_mcu_v:          MC RT for Urad on vertical rays 
!   rsys2d_mcu_x:          MC RT for Urad on inclined rays (phi average)
!   rsys2d_mcu_vxa:        MC RT for Urad, averaging over all mu angles
!   rsys2d_mcd_v:          MC RT for Drad=Lambda* on vertical rays 
!   rsys2d_mcd_x:          MC RT for Drad=Lambda* on inclined rays (phi average)
!   rsys2d_mcd_vxa:        MC RT for Drad=Lambda*, averaging over all mu angles
!   str_count:             Increment string counter
!   str_cut:               Cut input string into two parts at first blank
!   strf_IsEqualCI:        Function: test if two strings are equal ignoring case
!   strf_IsEqualCIUT:      Perform unit tests for strf_IsEqualCI
!   str_form:              Remove all characters after first blank and substitute some substrings
!   str_identcompose:      Compose variable identifier in standard form
!   str_identparse:        Parse variable identifier
!   str_int2str:           Transform a positive integer into a string
!   str_lowercase:         Transform all uppercase letters into lowercase form
!   str_lowercaseUT:       Perform unit tests for str_lowercase
!   str_pathandfile:       Merge path name and file name
!   str_pathandfileUT:     Perform unit tests for str_pathandfile
!   str_str2int:           Transform a string into a positive integer
!   str_sub:               Substitute every occurrence of substring in string by another
!   str_uppercase:         Transform all lowercase letters into uppercase form
!   str_uppercaseUT:       Perform unit tests for str_uppercase
!   strUT_TestAll:         Perform all unit tests of string-handling routines
!   strUT_TestSome:        Perform some unit tests of string-handling routines
!   sub_xkaros3lin_logPT2kappa: Compute opacities with linear interpolation
!   sub_xkaros3vec22_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros3vec2_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros3veclin2_logPT2kappa: Compute opacities with linear interpolation, different loops
!   sub_xkaros3veclin_logPT2kappa: Compute opacities with linear interpolation, different loops
!   sub_xkaros3vec_logPT2kappa: Compute opacities with cubic interpolation, different loops
!   sub_xkaros_logPT2kappa: Interpolate opacity per mass for given logP, T, and nu bin (1D,2D,3D)
!   sub_xkaros_logPT2rhokap: Interpolate opacity per volume for given logP, T, and nu bin (1D,2D,3D)
!   supergas:              Extended gasgas routine: P, T -> rho, h, ..
!   supergasinv:           Extended inverted gasgas routine: rho, e -> P, T, ..
!   system_clock:          Standard Fortran routine to access the system clock
!   system:                Spawn UNIX script (system call on, e.g., Suns)
!   tabinter_delete:       Deallocate EOS table completely
!   tabinter_getinfo:      Give requested information about EOS interpolation table
!   tabinter_info:         Print information about interpolation table
!   tabinter_rdcoeff:      Read coefficients for bicubic interpolation of EOS quantities from file
!   tabinter_reset:        Set all quantities in EOS table to zero
!   tabinter_select:       Select table
!   tabinter_wrcoeff:      Write coefficients for bicubic interpolation of GAS quantities into file
!   timing_GetHandle:      Get list number from block name
!   timing_Init:           Initialize arrays for timing routines
!   timing_Job:            Return the remaining CPU or wall-clock time
!   timing_SelfTest:       Measure time needed for call of timing routines
!   timing_start:          Start timer for specified block
!   timing_stop:           Stop timer for specified block
!   timing_SwitchInfo:     Print information about used compiler switches
!   timing_Write:          Write results of counting process into table
!   tremain:               CRAY: return the remaining CPU time
!   uio_adkeyw:            Add one keyword to term table with keyword-value or default
!   uio_chconv:            Actualize list of conversion types for all channels
!   uio_chpos:             Give current file position or jump to specified position
!   uio_chunit:            Initialize, store and actualize a list of unit numbers
!   uio_closrd:            Close file after reading
!   uio_closwr:            Close file after writing
!   uio_cpentr:            Copy entry (header + data) from one file to another
!   uio_dattim:            Get string with current date and time
!   uio_deform:            Determine the default output format for numbers
!   uio_dim2st:            Compose dimension string
!   uio_err:               Handle UIO errors
!   uio_ex1trm:            Extract one term from the input line
!   uio_exkeyw:            Extract value of keyword from table
!   uio_exmtrm:            Transform a list of items into its components
!   uio_filcon:            Determine file contents: list of all entries with its positions
!   uio_getenv:            Get information about environment
!   uio_init:              Initialization of UIO routines
!   uio_me1trm:            Merge the input term: 'keyword', 'value' -> 'keyword=value'
!   uio_memtrm:            Merge a list of terms (keywords and their values) into a line table
!   uio_mkcvls:            Make list with possible conversion types (e.g.: 'native' or 'ieee_4')
!   uio_openrd:            Open file for reading, read header
!   uio_openwr:            Open file for writing, write header
!   uio_ppkeyw:            Print keywords with values in pretty form
!   uio_pptrmt:            Print term table in pretty form
!   uio_qmaadd:            Add quotation marks to string if necessary
!   uio_qmadel:            Parse string "inline" and remove quotation marks if necessary
!   uio_rdfifo:            Read file header
!   uio_rdhdln:            Read header, output: line table
!   uio_rdhd:              Read header of variable and extract keywords
!   uio_rdlabl:            Read label
!   uio_rd:                Read integer/real/double-real/complex/character scalar or array
!   uio_rdtab:             Read table of integer, real, and/or character data from file
!   uio_skipda:            Skip data block
!   uio_srhd:              Search entry header in file or in entry list
!   uio_st2dim:            Parse dimension string
!   uio_SwitchInfo:        Print information about used compiler switches
!   uio_tabaskhead:        Get data from table entry header or general data about table
!   uio_tabcopy:           Copy uio table (operator: =)
!   uio_tabdelete:         Delete uio table (deallocate pointers...)
!   uio_tabget:            Get table header/1d integer/real/character array from table
!   uio_tabmake:           Create or enlarge uio table
!   uio_tabnull:           Initialize uio table (nullify pointers...)
!   uio_tabput:            Put table header/1d integer/real/character array into table
!   uio_uclose:            Close file with special handling for conversion type
!   uio_uopen:             Open a file with special handling for conversion type
!   uio_var_3Dto4D:        Transform array in uio flexibel variable from 3D (or scalar) to 4D
!   uio_var_askhead:       Check header information of uio flexible variable
!   uio_var_changehead:    Change header information of uio flexible variable
!   uio_var_copy:          Copy uio flexible variable. Account for pointers properly
!   uio_var_cut:           Cut block out of (4D) array
!   uio_var_delete:        Delete uio flexible variable
!   uio_varfile_askhead:   Get some information about uio varfile
!   uio_varfile_copy:      Copy uio varfile
!   uio_varfile_delete:    Delete uio varfile. Deallocate pointers
!   uio_varfile_get:       Get uio flexible variable from uio varfile via copying
!   uio_varfile_goto:      Change position in uio varfile
!   uio_varfile_insert:    Insert uio varfile into another one
!   uio_varfile_make:      Make uio varfile of requested size or enlarge file array
!   uio_varfile_move:      Move uio varfile
!   uio_varfile_null:      Initialize uio varfile. Nullify pointers
!   uio_varfile_pointget:  Get uio flexible variable from uio varfile via pointing
!   uio_varfile_pointput:  Put uio flexible variable into uio varfile via pointing
!   uio_varfile_put:       Put uio flexible variable into uio varfile via copying
!   uio_varfile_rd:        Read uio varfile from external file
!   uio_varfile_search:    Search entry in uio varfile
!   uio_varfile_wr:        Write uio varfile into external file
!   uio_var_get:           Get data set from uio flexible variable
!   uio_var_glue:          Enlarge (4D) array in uio flexible variable
!   uio_var_null:          Set uio flexible variable to zero (initialization)
!   uio_var_pointget:      Point to data set in uio flexible variable
!   uio_var_pointput:      Put data set into uio flexible variable via pointing
!   uio_var_printhead:     Print header of uio flexible variable
!   uio_var_putfifo:       Put fileform data set into uio flexible variable
!   uio_var_putlabl:       Put label into uio flexible variable
!   uio_var_put:           Put data set into uio flexible variable
!   uio_var_puttab:        Put table into uio flexible variable
!   uio_var_rd:            Read uio file entry into uio flexible variable
!   uio_var_real:          Change type of uio flexibel variable to real
!   uio_var_wr:            Write uio flexible variable into uio file
!   uio_vnanrm:            Transform a string into one that contains a correct name of a variable
!   uio_wf2rf:             Produce from write format string read format string
!   uio_wrfifo:            Write file header
!   uio_wrhdln:            Write header of variable, input: line table
!   uio_wrhd:              Write header of variable, input: term table
!   uio_wrlabl:            Write label
!   uio_wrtab:             Write table of integer, real, and/or character data to file
!   uio_wr:                Write integer/real/double-real/complex/character scalar or array
!   uname:                 Get information about environment and machine (system call on Cray XMP)
!   UT_Assert:             Check array or relation between two arrays
!   UT_DemoExampleFailAll: Call a number of assert routines to perform tests that all fail
!   UT_DemoExamplePassAll: Call a number of assert routines to perform tests that are all passed
!   UT_Init:               Initialize unit-test counters and control variables
!   UT_PrintArray:         Print a 1D/2D/3D array
!   UT_PrintSummary:       Print values of unit-test counters and summarize tests
!   UT_StrPassFail:        Function: produce status string corresponding to the input state
!   wrtab:                 Write table of integer, real, and/or character data to file
!   xbnu0:                 Compute the fraction of the Planck function B_nu, scalar version
!   xbnu1:                 Compute the fraction of the Planck function B_nu, 1D version
!   xbnu2:                 Compute the fraction of the Planck function B_nu, 2D version
!   xbnu3:                 Compute the fraction of the Planck function B_nu, 3D version
!   xBnu:                  Compute the fraction of the Planck function B_nu
!   xBnuwT0:               Compute the fraction of B_nu and of dB_nu/dT, scalar version
!   xBnuwT1:               Compute the fraction of B_nu and of dB_nu/dT, 1D version
!   xBnuwT2:               Compute the fraction of B_nu and of dB_nu/dT, 2D version
!   xBnuwT3:               Compute the fraction of B_nu and of dB_nu/dT, 3D version
!   xco:                   Compute LTE carbon monoxide fraction
