!******************************************************************************
!       QUANTITY PROPERTIES Routines
!       Fortran90 version
!       28. 1.1999
!******************************************************************************
! Non-standard FORTRAN:
!   Hopefully none!
!******************************************************************************
! Program:
!   qprop
!******************************************************************************


program qprop
!------------------------------------------------------------------------------
! NAME:
!   qprop ('quantity_properties')
!
! PURPOSE:
!   Read uio table and
!   write program segment with determines properties of quantities.
!
! CATEGORY:
!   EOS
!
! VARIABLES:
!   ncpri:  ('number_channel_print') integer, logical number for output
!   infile: ('input_file') character, input file name
!   outfile:('output_file') character, output file name
!
! SUBROUTINES:
!   uio_init:         Initialization of uio-routines
!
! MODULES:
!   uio modules
!
! PROCEDURE:
!   Read file 'qprop.txt' with table of quantities, their names and units.
!   Write data in Fortran90 format into file 'quant.f90'
!
! MODIFICATION HISTORY:
!    1. 4.98 Written by Bernd Freytag, Astrophysik Kiel
!   28. 1.99 (B.F. Copenhagen) New path for infile
!------------------------------------------------------------------------------
use uio_nam_module
use uio_siz_module
use uio_table_module
!
implicit none
!
! --- Variables ---
integer, parameter                      :: ncpri=6
character*(*), parameter                :: infile='../qprop.txt', &
                                           outfile='quant.f90', &
                                           space=''
integer                                 :: nc, nlin, ncol, i
character*(let)                         :: type, ident
type(uio_table_type)                    :: tab
character*(let)                         :: termt(2,nttmx)
integer                                 :: ntt
character*(80), dimension(:), pointer   :: quantlist, namelist, unitlist
!------------------------------------------------------------------------------
!
write(ncpri,*)
write(ncpri,*) ' QPROP'
write(ncpri,*) ' ====='
write(ncpri,*)
!
! --- Initialize uio routines ---
call uio_init(progrm='qprop')
!
!
! --- Open input file ---
write(ncpri,'(A)') 'Read ' // trim(infile)
call uio_openrd(nc, file=infile)
!
! --- Search and read table ---
call uio_srhd(nc, termt, ntt, type=tabna, ident='qprop')
call uio_tabnull(tab)
call uio_rdtab(nc, termt, ntt, tab, ident)
!
! --- Close input file ---
call uio_closrd(nc)
!
!
! --- Extract data from ---
write(ncpri,'(A)') 'Extract data from ' // trim(infile)
call uio_tabaskhead(tab, nlin=nlin, ncol=ncol)
allocate(quantlist(nlin), namelist(nlin), unitlist(nlin))
call uio_tabget(tab, 'quantity',quantlist)
call uio_tabget(tab, 'english', namelist)
call uio_tabget(tab, 'unit',    unitlist)
!
!
! --- Write table contents into file in Fortran90 format ---
call uio_chunit('get', nc1out=nc)
write(ncpri,'(A,I3)') 'Make ' // trim(outfile) // ' on unit ', nc
open(nc, file=outfile, form='formatted', status='unknown')
!
do i=1,nlin
  write(nc,'(A)') space // 'case('''  // trim(quantlist(i)) // ''')'
  write(nc,'(A)') space // '  name=''' // trim(namelist(i)) // ''''
  write(nc,'(A)') space // '  unit=''' // trim(unitlist(i)) // ''''
end do ! i
!
close(nc)
call uio_chunit('return', nc1in=nc)
!
!
deallocate(quantlist, namelist, unitlist)
!
end program qprop
