      SUBROUTINE EOSCND( NEQ, XDN, YDN)
********************************************************************************
*
* Version: 20-MAY-1992 12:20:58
*
* Author : HGL
*
* Description: EOSCND gives the equations which govern the different ionization
*   and dissoziation processes in the EOS.
*   EOSJCB determines the Jacobi-matrix of the system of equations:
*   YJACOB( i, j) = d y( j) / d x( i)
* 
*
* Arguments, access, type, dimension, description:
*   NEQ    R I : Number of equations actually regarded of calling routine
*   XDN    R D1: Array containing the number densities of various species.
*   YDN    W D1: Array containing the deviations of the constrain equations
*
*
* Common blocks (access): EOSPAR(R)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: The routine is designed in DOUBLE PRECISION.
*   Some variables are passed to EOSCND by common block EOSPAR. This is done to
*   ensure the correct form of the parameter list of the SUBROUTINE statement.
*   BROYDN assumes such a simple form of the external vector function.
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      INCLUDE 'EOSPAR.INC'

      DOUBLE PRECISION XDN(*), YDN(*), YJACOB( MEQ, *), 
     >                 DYDNDT(*), DYDNDP(*)

***   Constants
      DOUBLE PRECISION C10, C20, C100
      PARAMETER( C10 = 1.0, C20 = 2.0, C100 = 10.0)

***   Definition of symbolic names for XDN array elements:
***   Somewhat slow but better to understand ...
***   Here EQUIVALENCE would be better but it doesn't work since XDN is a
***   formal parameter.
      DOUBLE PRECISION DNA, DNH2, DNHN, DNHP, DNHEN, DNHEP, DNHEPP, 
     >                 DNEL, DNM
      PARAMETER( INA=1, INH2=2, INHN=3, INHP=4, INHEN=5, INHEP=6, 
     >           INHEPP=7, INEL=8, INM=9)
      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
***   Modify DNEL: From numerical reasons a minimum electron density is adopted
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

***   Here the constrain equations ...
***   (see Gigas, Dissertation, p.272)
      YDN(1) = C20*DNH2 + DNHN + DNHP - X*DNA 
      YDN(2) = DNHEN + DNHEP + DNHEPP - Y*DNA 
      YDN(3) = DNA + DNEL - DNH2 - DNTOT
      YDN(4) = DNHP + DNHEP + C20*DNHEPP - DNEL
      YDN(5) = DNHN*DNHN*GT - DNH2
      YDN(6) = DNHN/DNEL*F1T - DNHP
      YDN(7) = DNHEN/DNEL*F2T - DNHEP
      YDN(8) = DNHEP/DNEL*F3T - DNHEPP
      YDN(9) = DNM - Z*DNA

      RETURN

      ENTRY EOSJCB( XDN, YJACOB)
***   ==========================
***   In this section the Jacobi-matrix of the system of constrain equations is 
***   computed:
***   XDN    R D1: Array containing the number densities of various species.
***   YJACOB W D2: Jacobi-matrix

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      YJACOB(1, 1)=-X
      YJACOB(2, 1)=C20
      YJACOB(3, 1)=1.0
      YJACOB(4, 1)=1.0
      YJACOB(5, 1)=0.0
      YJACOB(6, 1)=0.0
      YJACOB(7, 1)=0.0
      YJACOB(8, 1)=0.0
      YJACOB(9, 1)=0.0
      YJACOB(1, 2)=-Y
      YJACOB(2, 2)=0.0
      YJACOB(3, 2)=0.0
      YJACOB(4, 2)=0.0
      YJACOB(5, 2)=1.0
      YJACOB(6, 2)=1.0
      YJACOB(7, 2)=1.0
      YJACOB(8, 2)=0.0
      YJACOB(9, 2)=0.0
      YJACOB(1, 3)=1.0
      YJACOB(2, 3)=-1.0
      YJACOB(3, 3)=0.0
      YJACOB(4, 3)=0.0
      YJACOB(5, 3)=0.0
      YJACOB(6, 3)=0.0
      YJACOB(7, 3)=0.0
      YJACOB(8, 3)=1.0
      YJACOB(9, 3)=0.0
      YJACOB(1, 4)=0.0
      YJACOB(2, 4)=0.0
      YJACOB(3, 4)=0.0
      YJACOB(4, 4)=1.0
      YJACOB(5, 4)=0.0
      YJACOB(6, 4)=1.0
      YJACOB(7, 4)=C20
      YJACOB(8, 4)=-1.0
      YJACOB(9, 4)=0.0
      YJACOB(1, 5)=0.0
      YJACOB(2, 5)=-1.0
      YJACOB(3, 5)=2.0*DNHN*GT
      YJACOB(4, 5)=0.0
      YJACOB(5, 5)=0.0
      YJACOB(6, 5)=0.0
      YJACOB(7, 5)=0.0
      YJACOB(8, 5)=0.0
      YJACOB(9, 5)=0.0
      YJACOB(1, 6)=0.0
      YJACOB(2, 6)=0.0
      YJACOB(3, 6)=F1T/DNEL
      YJACOB(4, 6)=-1.0
      YJACOB(5, 6)=0.0
      YJACOB(6, 6)=0.0
      YJACOB(7, 6)=0.0
      YJACOB(8, 6)=-(DNHN*F1T)/DNEL/DNEL
      YJACOB(9, 6)=0.0
      YJACOB(1, 7)=0.0
      YJACOB(2, 7)=0.0
      YJACOB(3, 7)=0.0
      YJACOB(4, 7)=0.0
      YJACOB(5, 7)=F2T/DNEL
      YJACOB(6, 7)=-1.0
      YJACOB(7, 7)=0.0
      YJACOB(8, 7)=-(DNHEN*F2T)/DNEL/DNEL
      YJACOB(9, 7)=0.0
      YJACOB(1, 8)=0.0
      YJACOB(2, 8)=0.0
      YJACOB(3, 8)=0.0
      YJACOB(4, 8)=0.0
      YJACOB(5, 8)=0.0
      YJACOB(6, 8)=F3T/DNEL
      YJACOB(7, 8)=-1.0
      YJACOB(8, 8)=-(DNHEP*F3T)/DNEL/DNEL
      YJACOB(9, 8)=0.0
      YJACOB(1, 9)=-Z
      YJACOB(2, 9)=0.0
      YJACOB(3, 9)=0.0
      YJACOB(4, 9)=0.0
      YJACOB(5, 9)=0.0
      YJACOB(6, 9)=0.0
      YJACOB(7, 9)=0.0
      YJACOB(8, 9)=0.0
      YJACOB(9, 9)=1.0

      RETURN

      ENTRY EOSDDT( XDN, DYDNDT)
***   ==========================
***   This section computes the temperature derivatives DYDNDT of every
***   constrain equation.

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      DYDNDT(1)=0.0
      DYDNDT(2)=0.0
      DYDNDT(3)=DNTOT/DT
      DYDNDT(4)=0.0
      DYDNDT(5)=DNHN**2*DGTDT
      DYDNDT(6)=(DNHN*DF1TDT)/DNEL
      DYDNDT(7)=(DNHEN*DF2TDT)/DNEL
      DYDNDT(8)=(DNHEP*DF3TDT)/DNEL
      DYDNDT(9)=0.0

      RETURN

      ENTRY EOSDDP( XDN, DYDNDP)
***   ==========================
***   This section computes the pressure derivatives DYDNDP of every
***   constrain equation.

      DNA    = XDN( INA)
      DNH2   = XDN( INH2)
      DNHN   = XDN( INHN)
      DNHP   = XDN( INHP)
      DNHEN  = XDN( INHEN)
      DNHEP  = XDN( INHEP)
      DNHEPP = XDN( INHEPP)
      DNEL   = XDN( INEL)
      DNM    = XDN( INM)
      DNEL = SIGN( C10, DNEL) * MAX( PRECIS( INEL) / C100, ABS( DNEL))

      DYDNDP(1)=0.0
      DYDNDP(2)=0.0
      DYDNDP(3)=-DNTOT/DP
      DYDNDP(4)=0.0
      DYDNDP(5)=0.0
      DYDNDP(6)=0.0
      DYDNDP(7)=0.0
      DYDNDP(8)=0.0
      DYDNDP(9)=0.0

      RETURN

      END
