      SUBROUTINE FCSAHA( T, FACGT, FACF1T, FACF2T, FACF3T, 
     >                   EIH, EIHE, EIHEP, EDH2, EDH2DT,
     >                   QH2T, DLQDLT, GT, F1T, F2T, F3T, 
     >                   DGTDT, DF1TDT, DF2TDT, DF3TDT)
********************************************************************************
*
* Version: Fri Jan 16 14:28:06 MET 1998
*
* Author : HGL
*
* Description: FCSAHA is an auxiliary routine of the GASTST program which
*   computes some particle density independent factors which are used in the 
*   Saha-equations.
* 
*
* Arguments, access, type, dimension, description:
*   T      R D : Temperature
*   FACGT  R D : Control factor of GT
*   FACF1T R D : Control factor of F1T
*   FACF2T R D : Control factor of F2T
*   FACF3T R D : Control factor of F3T
*   EIH    R D : Ionization energy H
*   EIHE   R D : Ionization energy He
*   EIHEP  R D : Ionization energy He+
*   EDH2   R D : Dissoziation energy H_2
*   EDH2DT R D : Temperature derivative of H_2 dissoziation energy
*   QH2T   R D : q(T)
*   DLQDLT R D : d ln q(T) / d ln T
*   GT     W D : g(T)
*   F1T    W D : f1(T)
*   F2T    W D : f2(T)
*   F3T    W D : f3(T)
*   DGTDT  W D : d g(T) / d T
*   DF1TDT W D : d f1(T) / d T
*   DF2TDT W D : d f2(T) / d T
*   DF3TDT W D : d f3(T) / d T
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: Designed in DOUBLE PRECISION.
*
* Modification history:
*   08/05/92/hgl: coded
*   16/01/98/hgl: eih, eihe, eihep, edh2 become variable
*   19/01/98/hgl: edh2dt
* 
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
      DOUBLE PRECISION T, FACGT, FACF1T, FACF2T, FACF3T, QH2T, DLQDLT,
     >                 GT, F1T, F2T, F3T, DGTDT, DF1TDT, DF2TDT, DF3TDT

      INCLUDE 'matnat_23jan98.inc'

      DOUBLE PRECISION EIH, EIHE, EIHEP, EDH2, EDH2DT

***   Auxiliary paramters
      DOUBLE PRECISION AUX1, R15, R20, R40, RO40
***   AUX1 = 2 * pi * WTUNIT / h**2
      PARAMETER( AUX1 = D20PI * WTUNIT / PLNCKH / PLNCKH)
      PARAMETER( R15 = 1.5, R20 = 2.0, R40 = 4.0, RO40 = 0.25)

      DOUBLE PRECISION KT, AUX2, AUX3

***   Temperature is measured in kT
      KT = BOLTZK * T
      AUX2 = ( AUX1 * WTEL * KT)**R15

***   F1(T)
      AUX3   = EIH / KT
      F1T    = FACF1T * AUX2 * EXP( - AUX3)
      DF1TDT = FACF1T * ( R15 + AUX3) * F1T / T

***   F2(T)
      AUX3   = EIHE / KT
      F2T    = FACF2T * R40 * AUX2 * EXP( - AUX3)
      DF2TDT = FACF2T * ( R15 + AUX3) * F2T / T

***   F3(T)
      AUX3   = EIHEP / KT
      F3T    = FACF3T * AUX2 * EXP( - AUX3)
      DF3TDT = FACF3T * ( R15 + AUX3) * F3T / T

***   G(T)
      AUX3   = EDH2 / KT
      GT     = FACGT * RO40 * ( AUX1 * WTH * KT / R20)**(-R15) * QH2T
     >       * EXP( AUX3)
      DGTDT  = FACGT * ( - R15 - AUX3 + EDH2DT/BOLTZK + DLQDLT) * GT / T
C      DGTDT  = FACGT * ( - R15 - AUX3 + DLQDLT) * GT / T

      RETURN
      END

