      SUBROUTINE GAS ( P, T, RHO, XCP, XGAM, CXT, ENTALP, S, CS,
     >                  IERROR, IFION, IFENT, KENTER, X, Y, QMOL)
********************************************************************************
*
* Version: 11-JUN-1993 11:15:02
*
* Author : HGL
*
* Description: This routine computes thermodynamic quantities needed in the
*   HD context. This routine is derived from a more complete version!
*   The following ionization equilibrium is considered:
*   H    <->  H+   + e-
*   He ionization is not taken into account in this version!
*   H_2 molecule formation is not taken into account. Nonideal effects are
*   neglected. In the routine it is assumed that the plasma is primarily
*   composed of hydrogen.
* 
*
* Arguments, access, type, dimension, description:
*   P      R R : Gas pressure
*   T      R R : Temperature
*   RHO    W R : Mass density
*   XCP    W R : HD specific heat
*   XGAM   W R : HD ratio of specific heats
*   CXT    W R : CXT = CP / XCP
*   ENTALP W R : Enthalpy per mass unit
*   S      W R : Entropy per mass unit
*   CS     W R : Sound speed
*   IERROR W I : Error indicator, 0: no error, 1: illegal IFION specified
*   IFENT  R I : Entropy requested? 1: yes, else: no
*   KENTER * I : Input : < 0 : Initialization of the routine
*                        else: Compute thermodynamic quantities
*                Output: =0
*   X      R R : Hydrogen abundance by number fraction
*   Y      R R : Helium abundance by number fraction, must be zero!
*   QMOL   R R : Mean molecular weight of neutral gas
*
* Common blocks (access): EE (VAX only)
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: None
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
********************************************************************************
***   EPSIT is the relative accuracy (related to total number density)
***   to which the electron density is computed
      PARAMETER( EPSIT = 1.0E-8)
***   EPSNTR is the limit at which the plasma is regarded as neutral
      PARAMETER( EPSNTR = 0.25 * EPSIT * EPSIT)
***
      PARAMETER( BOLTZK = 1.38054E-16, PLNCKH = 6.6256E-27)
      PARAMETER( WTUNIT = 1.6604E-24,  UGC = 8.3144E+07)
***   Atomic weights
      PARAMETER( WTHN = 1.0080, WTHEN = 4.0026, WTEL = 5.48609E-4)
      PARAMETER( WTHP = WTHN - WTEL, WTHEP = WTHEN - WTEL, 
     >           WTHEPP = WTHEP - WTEL)
***   Ionization energies
      PARAMETER( EIH = 2.1785E-11, EIHE = 3.9391E-11, 
     >           EIHEP = 8.7179E-11)
      PARAMETER( EIHEPP = EIHE + EIHEP)
***
      PARAMETER( R20PI = 6.283185307179586E+00)
***   AUX0 = 2 * pi * WTUNIT * k / h**2
      PARAMETER( AUX0 = R20PI * WTUNIT / PLNCKH * BOLTZK / PLNCKH)
***   AUX1 = 2 * pi * WTUNIT / h**2 * WTEL
      PARAMETER( AUX1 = R20PI * WTUNIT / PLNCKH / PLNCKH * WTEL)
***   Partitions functions of various species
***   Metal magnesium like ...
      PARAMETER( QHN = 2.0, QHP = 1.0, QHEN = 1.0, QHEP = 2.0,
     >           QHEPP = 1.0, QEL = 2.0, QM = 1.0)


***   VAX only: (and Sun)
      COMMON / EE / ANHN, ANHP, ANHEN, ANHEP, ANHEPP, CT

      SAVE QMA, Z, WTM, SAHN, SAHP, SAHEN, SAHEP, SAHEPP, SAEL, SAM
      SAVE P01, P02, P03, P04, P05, P06, P07, P08, P09, P10
      SAVE EPSFUL

      REAL LN10, KT
      DATA LN10 / 2.3025850929940457 /
***   Statement functions ...
      EXP10( XX) = EXP( LN10 * XX)

      IF ( KENTER .LT. 0 ) THEN
        QMA = WTUNIT * QMOL
***     Auxiliary parameters for entropy computation
***     These parameters are known before run time and could be computed
***     previously.
        SAHN   = 1.5 * LOG( AUX0 * WTHN)   + LOG( QHN)
        SAHP   = 1.5 * LOG( AUX0 * WTHP)   + LOG( QHP) 
        SAHEN  = 1.5 * LOG( AUX0 * WTHEN)  + LOG( QHEN) 
        SAHEP  = 1.5 * LOG( AUX0 * WTHEP)  + LOG( QHEP) 
        SAHEPP = 1.5 * LOG( AUX0 * WTHEPP) + LOG( QHEPP) 
        SAEL   = 1.5 * LOG( AUX0 * WTEL)   + LOG( QEL) 
***     The next statement is introduced because ANEL = ANHP in this context.
***     In fact it is bad programming style ...
        SAEL   = SAEL + SAHP
***
        Z   = 1.0 - X - Y
        IF ( Z .GT. 0.0) THEN
          WTM = ( QMOL - WTHN * X - WTHEN * Y) / Z
          SAM = 1.5 * LOG( AUX0 * WTM) + LOG( QM)
        ELSE
          Z   = 0.0
          WTM = 0.0
          SAM = 0.0
        ENDIF
        P01 = 1.0 + X
        P02 = X + 2.0 * Y
        P03 = 1.0 + X + 2.0 * Y
        P04 = X + Y
        P05 = 1.0 + X + Y
        P06 = 1.0 + Y
        P07 = 1.0 + 2.0 * Y
        P08 = - ( X * EIH + Y * EIHEPP)
        P09 = 2.0 * Y
        P10 = X / ( 1.0 + X)
        IF ( Y .NE. 0.0) THEN
          WRITE( *, *) 'GAS> Gas version in effect does not support ',
     >                 'He ionization!'
          WRITE( *, *) 'GAS> Set Y = 0 or use another version!'
          STOP
        ENDIF
        WRITE( 6, '( /, 1X, 72(''=''))')
        WRITE( 6, '( 1X, ''GAS version 6-Nov-1992 (WD) busy!'')')
        WRITE( 6, '( 1X, 72(''=''))')
        WRITE( 6, '( /)')
        EPSFUL = SQRT( EPSIT)
        IERROR = 0
        RETURN
      ENDIF

      KENTER = 0

      IF ( IFION .NE. 1) THEN
        IERROR = 1
      ELSE
        IERROR = 0
      ENDIF

***   Temperature measured in kT
      KT = BOLTZK * T
      ANTOT = P / KT


      AUX2 = ( AUX1 * KT)**1.5 / ANTOT

***   F1(T)
      AUX3   = EIH / KT
      F1T    = AUX2 * EXP( - AUX3)
      DTF1T  = ( 1.5 + AUX3) * F1T / T

***   Check whether the plasma could be regarded as neutral
      IF ( F1T .LT. EPSNTR) THEN
***     In this if block all particle densities are measured in external units!
        ANA    = ANTOT
        ANHN   = X * ANTOT
        ANEL   = 0.0
        ANHP   = 0.0
***     VAX only:
        ANHEN  = 0.0
        ANHEP  = 0.0
        ANHEPP = 0.0
***
        RHO    = ANTOT * QMA
        OHR    = 1.0 / RHO
        DTRHO  = - RHO / T 
        DPRHO  =   RHO / P 
        E      = 1.5 * P * OHR
        DTEV   = 0.0
        GOTO 30
      ENDIF

***   The prefix DT denotes the temperature derivative at constant pressure
***   The prefix DP denotes the pressure derivative at constant temperature
      DTNTOT = - 1.0 / T
      DPNTOT =   1.0 / P

***   Compute of electron pressure.
      DUM  = P01 * F1T
      DUM2 = 4.0/DUM * (F1T/DUM) * X
      IF ( DUM2 .LT. EPSFUL) THEN
***     Plasma nearly completely ionized
        DUM3 = 1.0 + 0.5 * DUM2
        ANEL = P10 * ( 1.0 - 0.25 * DUM2)
      ELSE
        DUM3 = SQRT( 1.0 + DUM2)
        ANEL = 0.5 * DUM * ( DUM3 - 1.0)
      ENDIF
      ONEL  = 1.0 / ANEL
      DUM3  = P10 / DUM3
      DTLF1T = DTF1T / F1T
      DTNEL = ANEL * DTLF1T + DUM3 * ( DTNTOT - DTLF1T)
      DPNEL = DPNTOT * DUM3

***   Compute some auxiliary parameters
C      DTNENE = DTNEL * ONEL
      F1NE   = F1T * ONEL
C      DTF1NE = DTF1T * ONEL
***   Prefix TD denotes the temperature derivative of the fraction F?/ANEL
C      TDF1NE = DTF1NE - DTNENE * F1NE
C      HTERM  = 1.0 + F1NE

***   Until now external units are used ...
***   Particle densities
      ANEL   = ANEL * ANTOT
      ANA    = ANTOT - ANEL
      ANHP   = ANEL
      ANHN   = ANEL / F1NE
***   VAX only:
C      ANHEN  = 0.0
C      ANHEP  = 0.0
C      ANHEPP = 0.0
***   Temperature derivatives of particle densities at constant pressure
      DTNTOT = DTNTOT * ANTOT
      DPNTOT = DPNTOT * ANTOT
      DTNEL  = DTNEL * ANTOT
      DPNEL  = DPNEL * ANTOT
      DTNA   = DTNTOT - DTNEL
      DTNHP  = DTNEL
      DTNHN  = X * DTNTOT - P01 * DTNEL

***   Mass density
      RHO = QMA * ANA
      OHR = 1.0 / RHO
***   Temperature derivative
      DTRHO = DTNA * QMA
***   Pressure derivative
      DPRHO = ( DPNTOT - DPNEL) * QMA
***   Internal energy
      E =( ANHP * EIH + 1.5 * P) * OHR
***   Temperature derivative of internal energy per volume at const. pressure
      DTEV = DTNHP * EIH

***   =========================================================================
***   The remaining program section is used in any case (neutral or partial
***   ionization).
***   Temperature derivative of internal energy per mass at const. pressure
30    DTEM = ( DTEV - E * DTRHO) * OHR

      POHR   = P * OHR
      DTLRHO = DTRHO * OHR

***   Enthalpy
      ENTALP = E + POHR

***   CXT
      CXT = - T * DTLRHO

***   (dS/dp)_T
      DPS = DTLRHO * OHR

***   C_p
      CP =  DTEM - DTLRHO * POHR

***   C_p(s)
      XCP = CP / CXT

      CS   = 1.0 / ( DPRHO - DTRHO * DPS / CP * T)
      XGAM = CS / P * RHO
      CS = SQRT( CS)

***   VAX only:
C      CT = 1.0 / SQRT( DPRHO)

      IF ( IFENT .EQ. 1) THEN
***     Precision converted to external units
        PRECIS = EPSIT * ANTOT
        ANM    = Z * ANA
        S = ANTOT * ( 2.5 + 1.5 * LOG( T))
        IF ( ANHN  .GT. PRECIS) S = S + ANHN  * ( SAHN  - LOG( ANHN))
C        IF ( ANHP  .GT. PRECIS) S = S + ANHP  * ( SAHP  - LOG( ANHP))
        IF ( ANEL  .GT. PRECIS) 
     >  S = S + ANEL  * ( SAEL  - 2.0 * LOG( ANEL))
        IF ( ANM   .GT. PRECIS) S = S + ANM   * ( SAM   - LOG( ANM))
        S = S * BOLTZK * OHR
      ENDIF
      RETURN
      END
