      SUBROUTINE QH2( TT, U, DUDT, D2UDT2)
********************************************************************************
*
* Version: 11-MAY-1992 13:39:58
*
* Author : HGL
*
* Description: QH2 computes approximately the partition function of the H_2
*   molecule. See Bohn & Wolf (1984), A&A Vol.130, p.202!
* 
*
* Arguments, access, type, dimension, description:
*   TT      R D : Temperature
*   U       W D : Partition function at temperature TT
*   DUDT    W D : Derivative of U 
*   D2UDT2  W D : Second derivative of U
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
*
* Comments: Designed in DOUBLE PRECISION
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
* MODIFICATION HISTORY:
*   11. 5.1992 Written by HGL
*   23. 6.1997 (B.F.) 'if (T .NE. TT) then' removed
********************************************************************************
      DOUBLE PRECISION T, TT, U, DUDT, D2UDT2
      DOUBLE PRECISION DOT, DOTDOT, F2, F4, DF2, DDF2 

***   Constants
      DOUBLE PRECISION TMIN, TMAX
      PARAMETER( TMIN=1000.0, TMAX=8000.0)
      DOUBLE PRECISION D00, D10, D20, D30, D40, D60
      PARAMETER( D00=0.0, D10=1.0, D20=2.0, D30=3.0, D40=4.0, D60=6.0)

***   Parameters are set to values adopted in GAS
      DOUBLE PRECISION A1, A2, A3      
      PARAMETER ( A1 = 5.797083E-03, A2 = 4.676132E-08, 
     >            A3 = 1.088582E-11 )
      DOUBLE PRECISION QCON1, QCON2, QCON3, QCON4, QCON5
      PARAMETER ( QCON1 =  9.55469E-01, QCON2 = 1.039473E-04,
     >            QCON3 = -7.19848E-08, QCON4 = 1.154352E-11,
     >            QCON5 = -6.403472E-16 )
      DOUBLE PRECISION D
***   This parameter is called Q2P2 in GAS
      PARAMETER ( D = 6.33262E+03)
********************************************************************************

***   Restrict temperature to interval [TMIN, TMAX]
***   Adopt boundary values outside
      T = MIN( TMAX, MAX( TMIN, TT))


      F2 = A1 + T * ( A2 + T * A3)
      F4 = QCON1 + T* ( QCON2 + T * ( QCON3 + T * ( QCON4 + T * QCON5)))
      U = T * F2 / F4

c     --- 23.6.97 (B.F.) ---
c      IF ( T .NE. TT) THEN
c        DUDT   = D00
c        D2UDT2 = D00
c        RETURN
c      ENDIF

      DDF2 = D20 * A3
      DF2  = A2 + T * DDF2
C      DF4  = QCON2 + T * ( D20*QCON3 + T * ( D30*QCON4 + T*D40*QCON5))
C      DDF4 = D20 * QCON3 + T * ( D60 * QCON4 + T * D30 * D40 * QCON5)

      DOT    = D / T
      DOTDOT = DOT * DOT

      DUDT = ( F2 * ( DOT / F4 + ( D10 - DOT)) + T * DF2) / F4
      D2UDT2 = (( DOTDOT * F2 * 
     >          ( D20 / T / F4 - D30 / T) + D20 * DOT * DF2) / F4
     >       + F2 * DOTDOT / T + D20 * ( D10 - DOT) * DF2 + T * DDF2)
     >       / F4

      RETURN
      END
