      subroutine smthlm(x0, dx, x, xs, xsd, xsdd)
*******************************************************************************
* Version: Fri Jan 23 11:01:56 MET 1998
*
* Author : HGL
*
* Description: smthlm (SMooTH LiMiter) is intended to delimit smoothly 
*   the range of function arguments. I.e. if a function is well defined
*   in a certain interval of arguments only one can work around this problem
*   by modifying the argument before it is put into that function. Effectively
*   this defines a certain extrapolation procedure. The modified value xs, 
*   and the derivatives xsd = dxs/dx, xsdd = d2xsdx^2 are
*   derived from a 4th order polynomial having the following properties:
*   
*   x <= x0:      xs   = x
*                 xsd  = 1.0 (linear)
*                 xsdd = 0.0
*   x >= x0 + dx: xs   = x0 + 0.5*dx (asymtotic value for x->inf) 
*                 xsd  = 0.0
*                 xsdd = 0.0
*   x0 < x < x0 + dx: smooth connection of other two regions with continous
*                     first and second derivative.
*
* Arguments, access, type, dimension, description:
*   x0     R D : threshold where smoothing sets in 
*   dx     R D : smoothing length, for x>x0+dx the asymtotic plateau is reached
*   x      R D : original argument
*   xs     W D : modified argument
*   xsd    W D : d xs / d x
*   xsdd   W D : d2 xs / d x^2
*
*
* Common blocks (access): None
*
* External procedures   : None
*
* File units (access)   : None
*
* 
* Preprocessor macros   : None
*
* Comments:
*   Double precision!!
*
*   The relations above are given for positive x0 and dx. actually things work
*   as well for arbitrary combinations of sign of dx and x0!
*
* Modification history:
*   21/01/98/hgl: coded
*
* Legend:
*   Access: (R)ead only, (W)rite only, (*)read/write
*   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, 
*           (L)ogical, (R)eal, (S)tring, (*)label
*
*******************************************************************************
      implicit none
      double precision x0, dx, x, xs, xsd, xsdd
      double precision xn, xn2

      xn = (x-x0)/dx

      if (xn .ge. 1.0D0) then
        xs   = x0 + 0.5D0 * dx
        xsd  = 0.0
        xsdd = 0.0
        return
      endif 

      if (xn .le. 0.0D0) then
        xs   = x
        xsd  = 1.0D0
        xsdd = 0.0D0
        return
      endif  

      xn2  = xn*xn
      xs   = dx*(0.5D0*(xn-2.0D0)*xn2+1.0D0)*xn + x0
      xsd  = (2.0D0*xn-3.0D0)*xn2 + 1.0D0
      xsdd = 6.0D0*(xn2-xn)/dx

      return
      end
