C-----------------------------------------------------------------------
C FORTRAN-Includefile PHYSKO
C
C Datei: PHYSKO.INC
C Autor: M. Feuchtinger, S.Hoefner
C Datum: 13-NOV-1992, 22-JAN-1993, 15-MAR-1993
C
C Verwendungszweck:
C
C   Parameter fuer physikalische, numerische und astronomische
C   Konstanten (cgs-System, Double Precision).
C
C-----------------------------------------------------------------------
C Numerische Konstanten:
C
C   Z0     : 0
C   Z1     : 1
C   Z2     : 2
C   Z3     : 3
C   Z4     : 4
C   Z5     : 5
C   Z12    : 1/2
C   Z32    : 3/2
C   Z13    : 1/3
C   Z23    : 2/3
C   Z43    : 4/3
C   Z53    : 5/3
C   Z14    : 1/4
C
C Physikalische Konstanten:
C
C   PI    : PI
C   CLIGHT: Lichtgeschwindigkeit            ( cm / s )
C   BOLTZ : Boltzmannkonstante              ( erg / K )
C   GRAV  : Gravitationskonstante           ( dyn * cm^2 / g )
C   STBOLZ: Stefan-Boltzmann-Konstante      ( erg / (cm^2 * s * K^4) )
C   AVOGA : Avogadro-Konstante              ( 1 / mol ) 
C   RGAS  : Ideale Gaskonstante             ( erg / (K * mol) )
C   PLK   : Planck'sches WQ                 ( erg * s )
C   ELLAD : Elementarladung                 ( E.E.Q )
C   PMAS  : Protonenmasse                   ( g )
C
C Zusammengesetzte Konstanten:
C
C   Z4PI  : 4. * PI
C   Z4PIQ : (4. * PI)**2
C   Z4PIC : 4. * PI     / CLIGHT
C   Z4STC : 4. * STBOLZ / CLIGHT (Strahlungsdruckkonstante A)
C   STPI  : STBOLZ / PI
C
C Astronomische Konstanten:
C
C   SOLM  : Sonnenmasse                     ( g )
C   SOLL  : Sonnenleuchtkraft               ( erg / s )
C   SOLR  : Sonnenradius                    ( cm )
C   SJAHR : Jahr in Sekunden                ( s )
C   STAG  : Tag in Sekunden                 ( s )
C   PARSC : Parsec                          ( cm )
C   AUCM  : Astronom. Einheit               ( cm )
C 
C Parameter fuer Staubbildung:
C
C   ZNL     :  N_lower fuer Momente
C   HEH     :  Elementhaeufigkeit von He
C
C-----------------------------------------------------------------------
C***********************************************************************
C Beginn von PHYSKO.INC:
C
C-----------------------------------------------------------------------
C Double Precision:

      DOUBLE PRECISION
     # CLIGHT,BOLTZ,GRAV,RGAS,STBOLZ,AVOGA,PMAS,ELLAD,PLK,
     # SOLM,SOLL,SOLR,PARSC,SJAHR,STAG,AUCM,
     # Z0,Z1,Z2,Z3,Z4,Z5,Z12,Z32,Z13,Z23,Z43,Z53,Z14,
     # PI,Z4PI,Z4PIQ,Z4PIC,Z4STC,STPI,
     # ZNL,ZNL13,ZNL23,HEH,SMASS

C-----------------------------------------------------------------------
C Numerische Konstanten:

      PARAMETER (Z0=0., Z1=1., Z2=2., Z3=3., Z4=4., Z5=5.)
      PARAMETER (Z12=Z1/Z2, Z32=Z3/Z2, Z13=Z1/Z3, Z23=Z2/Z3, 
     #           Z43=Z4/Z3, Z53=Z5/Z3, Z14=Z1/Z4)
    
C Physikalische Konstanten:

      PARAMETER (PI     = 3.1415926535898,
     #           GRAV   = 6.6732E-08,
     #           BOLTZ  = 1.38062E-16,
     #           CLIGHT = 2.99792456E+10,
     #           PMAS   = 1.6726E-24,
     #           ELLAD  = 4.80325E-10,
     #           PLK    = 6.6262E-27,
     #           STBOLZ = 5.66961E-05,
     #           RGAS   = 8.31434E+07,
     #           AVOGA  = 6.022169E+23)

C Zusammengesetzte Konstante:

      PARAMETER (Z4PI   = Z4 * PI,
     #           Z4PIQ  = Z4PI**2,
     #           Z4PIC  = Z4PI / CLIGHT,
     #           Z4STC  = Z4 * STBOLZ / CLIGHT,
     #           STPI   = STBOLZ / PI)

C Astronomische Konstanten:

      PARAMETER (SOLM   = 1.989E+33,
     #           SOLR   = 6.9598E+10,
     #           SOLL   = 3.826E+33,
     #           SJAHR  = 3.1557E+7,
     #           STAG   = 8.64E+04,
     #           PARSC  = 3.0856E+18,
     #           AUCM   = 1.4959789E+13)

C Materialkonstante und Parameter fuer Staubbildung:

      PARAMETER ( ZNL   = 1.D+03, 
     #            ZNL13 = 1.D+01,
     #            ZNL23 = 1.D+02,
     #            HEH   = 1.D-01,
     #            SMASS = PMAS *( 1.0D00 + HEH*Z4 ) )

C-----------------------------------------------------------------------
C Ende von PHYSKO.INC
C-----------------------------------------------------------------------
