!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_shortrad_bound01 ('rhd_short-characteristics_radiation_boundary_01')
!
! PURPOSE:
!   Compute infalling intensity.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   include 'rhd_shortrad_bound01.f90'
!
! MODIFICATION HISTORY:
!   2014-03-07 (B.F. Uppsala) Put into separate include file
!   2014-03-12 (B.F.) Sign variable RdOut added
!   2017-07-21 (B.F.) Account for negative dxOut
!   2017-07-24 (B.F.) Experiment with simpler formula for Dout
!   2018-11-30 (B.F.) Rename variable RcosmuOut -> RcosthetaOut
!---------------------------------------------------------------------------------------------------
            ROut(1)     =0.5*(xb1(i1old)+xb1(i1old+1))
            ROut(2)     =0.5*(xb2(i2old)+xb2(i2old+1))
            ROut(3)     =0.5*(xb3(i3old)+xb3(i3old+1))
            RabsOut     =sqrt(  ROut(1)**2       + ROut(2)**2       + ROut(3)**2      )
            RcosthetaOut=RdOut*(ROut(1)*r_dir(1) + ROut(2)*r_dir(2) + ROut(3)*r_dir(3))
            ! \label{f90:c_radhtautop__use1}
            Dout        =sqrt(RcosthetaOut**2 + par%C_radHtautop**2 + &
                              2.0*RabsOut*par%C_radHtautop) - &
                         RcosthetaOut
            !Dout        =par%C_radHtautop   ! --- Simple version (for testing) ---
            tauOut      =2.0*dtauhalf(i1old, i2old, i3old)/(mu*abs(dxOut)) * Dout
            !
            if (dtauhalf(i1old, i2old, i3old) > dtauthick_limit) then
              I(         i1old, i2old, i3old)=S(i1old, i2old, i3old) * (    - exp(-tauOut))
            else
              I(         i1old, i2old, i3old)=S(i1old, i2old, i3old) * (1.0 - exp(-tauOut))
            endif
