!***************************************************************************************************
!
!   #####  #    # #####         #    # ######   ##   #    # #    # # #    # #####  
!   #    # #    # #    #        ##  ## #       #  #  ##   # #   #  # ##   # #    # 
!   #    # ###### #    #        # ## # #####  #    # # #  # ####   # # #  # #    # 
!   #####  #    # #    #        #    # #      ###### #  # # #  #   # #  # # #    # 
!   #   #  #    # #    #        #    # #      #    # #   ## #   #  # #   ## #    # 
!   #    # #    # #####  ###### #    # ###### #    # #    # #    # # #    # ##### _module
!
!   Radiation-Hydrodynamics-Code: kind parameters for averaging module
!
!***************************************************************************************************
!   Fortran90
!   Bernd Freytag:         Uppsala
!   2017-09-10
!***************************************************************************************************
! MODULES:
!   rhd_meankind_module:   Kind parameters for averaging routines
!
!***************************************************************************************************

!# /* --- Default values for preprocessor macros ------------------------------------------------ */
!#
!# /* --- MPI flag activation --- */
# ifndef rhd_mpi_l01
#   define rhd_mpi_l01 0
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*******************--------------------------------------------------------------------------
module rhd_meankind_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhd_meankind_module ('rhd_mean_kind_module')
!
! PURPOSE:
!   Provide kind parameters for averaging routines for rhd.
!
! CATEGORY:
!   Hydrodynamics, averaging
!
! CALLING SEQUENCE:
!   use rhd_meankind_module
!
! VARIABLES:
!   mArk:         ('mean_Array_real_kind') kind of internal mean data arrays
!   mOrk:         ('mean_Output_real_kind') kind of mean arrays written to file
!   mArkMPI:      ('mean_Array_real_kind_MPI') MPI data type of internal mean data arrays
!
! ROUTINES: (contained)
!   None
!
! MODULES:
!   MPI:                   Standard MPI routines and definitions
!
! RESTRICTIONS:
!   mArk and mArkMPI have to be consistent (both single or both double precision).
!
! MODIFICATION HISTORY:
!   2015-03-21 (B.F. Uppsala) Written
!   2015-09-03 (B.F.) Change mArk from 0.0E+00 to 0.0D+00
!   2017-09-10 (B.F.) Add MPI data type corresponding to mArk
!---------------------------------------------------------------------------------------------------
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
use MPI
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
implicit none
!
integer, parameter                      :: mArk=kind(0.0D+00)
integer, parameter                      :: mOrk=kind(0.0E+00)
!
!#
# if (rhd_mpi_l01 == 1)
!#                      /* MPI activated ----------------- */
!type(MPI_DataType), parameter           :: mArkMPI=MPI_DOUBLE_PRECISION
integer, parameter                      :: mArkMPI=MPI_DOUBLE_PRECISION
!#
# endif
!#      /* rhd_mpi_l01 ----------------------------------- */
!
end module rhd_meankind_module
