!***************************************************************************************************
!
!                        #     # #######                     
!   #####  #    # #####  #     #    #
!   #    # #    # #    # #     #    #
!   #    # ###### #    # #     #    #
!   #####  #    # #    # #     #    #
!   #   #  #    # #    # #     #    #
!   #    # #    # #####   #####     #
!
!   Radiation-Hydrodynamics-Code: unit tests
!
!***************************************************************************************************
!   Fortran90
!   2017-08-26
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Uppsala, Montpellier, Lyon
!***************************************************************************************************
! MODULES:
!   rhdUT_master_module:   Master unit-test routine for RHD code
!
! PROGRAM:
!   rhdUT:                 Perform unit tests for RHD code
!
!***************************************************************************************************

!# /* --- Set default values for some preprocessor macros --------------------------------------- */
!#
!# /* --- Select which modules to include in the test (default: all) --- */
!#
# ifndef ut_switch_rhd_baseradUT_module
#   ifndef ut_switch_testall
#     define ut_switch_rhd_baseradUT_module 2
#   else
#     define ut_switch_rhd_baseradUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_rhd_boxUT_module
#   ifndef ut_switch_testall
#     define ut_switch_rhd_boxUT_module 2
#   else
#     define ut_switch_rhd_boxUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_rhd_meanUT_module
#   ifndef ut_switch_testall
#     define ut_switch_rhd_meanUT_module 2
#   else
#     define ut_switch_rhd_meanUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_dust_Al2O3AGBUT_module
#   ifndef ut_switch_testall
#     define ut_switch_dust_Al2O3AGBUT_module 1
#   else
#     define ut_switch_dust_Al2O3AGBUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_dust_Mg2SiO4AGBUT_module
#   ifndef ut_switch_testall
#     define ut_switch_dust_Mg2SiO4AGBUT_module 1
#   else
#     define ut_switch_dust_Mg2SiO4AGBUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_rhd_recUT_module
#   ifndef ut_switch_testall
#     define ut_switch_rhd_recUT_module 1
#   else
#     define ut_switch_rhd_recUT_module ut_switch_testall
#   endif
# endif
!#
# ifndef ut_switch_strUT_module
#   ifndef ut_switch_testall
#     define ut_switch_strUT_module 2
#   else
#     define ut_switch_strUT_module ut_switch_testall
#   endif
# endif
!#
!# /* ------------------------------------------------------------------------------------------- */


!------*******************--------------------------------------------------------------------------
module rhdUT_master_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdUT_master_module ('radiation_hydrodynamics_Unit_Test_master_module')
!
! PURPOSE:
!   Provide master unit-test routines for RHD code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   use rhdUT_master_module
!
! TYPES:
!
! VARIABLES:
!
! ROUTINES: (contained)
!   rhdUT_master_TestAll:  Perform all unit tests for RHD code
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-02-26 (B.F.) More comments
!   2015-03-06 (B.F.) Split single file into several
!   2015-03-11 (B.F.) Test first averaging routines
!   2017-08-26 (B.F. Lyon) Add option for partial unit tests
!---------------------------------------------------------------------------------------------------
implicit none
!
contains

!----------********************---------------------------------------------------------------------
subroutine rhdUT_master_TestAll()
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdUT_master_TestAll ('radiation_hydrodynamics_Unit_Test_master_Test_All')
!
! PURPOSE:
!   Perform all unit tests for rhd code.
!
! CATEGORY:
!   Testing
!
! CALLING SEQUENCE:
!   call rhdUT_master_TestAll()
!
! INPUT:
!   None
!
! OUTPUT:
!   None
!
! ROUTINES:
!   dust_Al2O3AGBUT_TestAll: Perform unit tests of corundum dust routines
!   dust_Mg2SiO4AGBUT_TestAll: Perform unit tests of forsterite dust routines
!   rhd_action_InitNameAr: Initialize global name arrays
!   rhd_baseradUT_TestAll: Perform all unit tests of basic radiation-transport routines
!   rhd_baseradUT_TestSome: Perform some unit tests of basic radiation-transport routines
!   rhd_boxUT_TestAll:     Perform all unit tests of box-handling routines
!   rhd_boxUT_TestSome:    Perform some unit tests of box-handling routines
!   rhd_meanUT_TestAll:    Perform unit tests of all averaging routines
!   rhd_meanUT_TestSome:   Perform unit tests of some averaging routines
!   rhd_recUT_TestAll:     Perform unit tests of reconstruction routines
!   strUT_TestAll:         Perform all unit tests of string-handling routines
!   strUT_TestSome:        Perform some unit tests of string-handling routines
!   UT_Init:               Initialize unit-test counters and control variables
!   UT_PrintSummary:       Print values of unit-test counters and summarize tests
!
! MODULES:
!   dust_Al2O3AGBUT_module: Unit-test routines for Al2O3 dust routines for AGB stars
!   dust_Mg2SiO4AGBUT_module: Unit-test routines for Mg2SiO4 dust routines for AGB stars
!   rhd_action_module:     Routines to handle the control structure 'action'
!   rhd_baseradUT_module:  Unit-test routines for base radiation-transport routines
!   rhd_boxUT_module:      Unit-test routines for box-handling routines
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   rhd_meanUT_module:     Unit-test routines for averaging routines
!   rhd_recUT_module:      Unit-test routines for reconstruction routines
!   strUT_module:          Unit-test routines for string-handling routines
!   dust_Al2O3AGBUT_module: Unit-test routines for Al2O3 dust routines for AGB stars
!   dust_Mg2SiO4AGBUT_module: Unit-test routines for Mg2SiO4 dust routines for AGB stars
!   rhd_baseradUT_module:  Unit-test routines for base radiation-transport routines
!   rhd_boxUT_module:      Unit-test routines for box-handling routines
!   rhd_meanUT_module:     Unit-test routines for averaging routines
!   rhd_recUT_module:      Unit-test routines for reconstruction routines
!   strUT_module:          Unit-test routines for string-handling routines
!   UT_module:             Global variables and basic routines for unit testing
!
! SIDE EFFECTS:
!   Set global variable UT.
!   Print data.
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Initialize the unit-test routines: set counters to zero.
!   Call the individual routines for all unit-test groups.
!   Print the result.
!
! EXAMPLE:
!   call rhdUT_master_TestAll()
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-06 (B.F.) Call routines from separate files
!   2016-04-18 (B.F.) Add unit tests for box-handling routines
!   2016-04-29 (B.F.) Add unit tests for string-handling routines
!   2017-05-17 (B.F.) Add unit tests for Mg2SiO4-dust routines
!   2017-05-19 (B.F.) Add unit tests for Al2O3-dust routines
!   2017-08-26 (B.F. Lyon) Add routine calls for partial unit tests
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use rhd_action_module
use UT_module
!#
# if (ut_switch_rhd_baseradUT_module >= 1)
!#
use rhd_baseradUT_module
!#
# endif
!#
!#
# if (ut_switch_rhd_boxUT_module >= 1)
!#
use rhd_boxUT_module
!#
# endif
!#
!#
# if (ut_switch_rhd_meanUT_module >= 1)
!#
use rhd_meanUT_module
!#
# endif
!#
!#
# if (ut_switch_dust_Al2O3AGBUT_module >= 1)
!#
use dust_Al2O3AGBUT_module
!#
# endif
!#
!#
# if (ut_switch_dust_Mg2SiO4AGBUT_module >= 1)
!#
use dust_Mg2SiO4AGBUT_module
!#
# endif
!#
!#
# if (ut_switch_rhd_recUT_module >= 1)
!#
use rhd_recUT_module
!#
# endif
!#
!#
# if (ut_switch_strUT_module >= 1)
!#
use strUT_module
!#
# endif
!#
!
implicit none
!---------------------------------------------------------------------------------------------------
!
! --- Initialize global name arrays ---
call rhd_action_InitNameAr(-1)
!
! --- Initialize unit-test counters ---
call UT_Init()
UT%AllTestPrint_flag  =.true.
UT%AllVecInPrint_flag =.true.
UT%AllVecOutPrint_flag=.true.
!
!#
# if (ut_switch_rhd_baseradUT_module == 1)
!#
! --- Test some basic radiation-transport routines ---
call rhd_baseradUT_TestSome()
!#
# elif (ut_switch_rhd_baseradUT_module >= 2)
!#
! --- Test basic radiation-transport routines ---
call rhd_baseradUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_rhd_boxUT_module == 1)
!#
! --- Test box handling routines ---
call rhd_boxUT_TestSome()
!#
# elif (ut_switch_rhd_boxUT_module >= 2)
!#
! --- Test box handling routines ---
call rhd_boxUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_rhd_meanUT_module == 1)
!#
! --- Test some averaging routines ---
call rhd_meanUT_TestSome()
!#
# elif (ut_switch_rhd_meanUT_module >= 2)
!#
! --- Test averaging routines ---
call rhd_meanUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_dust_Al2O3AGBUT_module >= 1)
!#
! --- Test dust scheme ---
call dust_Al2O3AGBUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_dust_Mg2SiO4AGBUT_module >= 1)
!#
! --- Test dust scheme ---
call dust_Mg2SiO4AGBUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_rhd_recUT_module >= 1)
!#
! --- Test reconstruction schemes ---
call rhd_recUT_TestAll()
!#
# endif
!#
!
!#
# if (ut_switch_strUT_module == 1)
!#
! --- Test some string routines ---
call strUT_TestSome()
!#
# elif (ut_switch_strUT_module >= 2)
!#
! --- Test string routines ---
call strUT_TestAll()
!#
# endif
!#
!
! --- Print test summary ---
call UT_PrintSummary()
!
end subroutine rhdUT_master_TestAll

end module rhdUT_master_module


!-------*****---------------------------------------------------------------------------------------
program rhdUT
!---------------------------------------------------------------------------------------------------
! NAME:
!   rhdUT ('radiation_hydrodynamics_Unit_Test')
!
! PURPOSE:
!   Perform unit tests for RHD code.
!
! CATEGORY:
!   Testing
!
! VARIABLES:
!
! ROUTINES:
!   rhdUT_master_TestAll:  Perform all unit tests for RHD code
!
! MODULES:
!   rhd_gl_module:         Global type definitions and parameters for RHD
!   UT_module:             Global variables and basic routines for unit testing
!   rhdUT_master_module:   Master unit-test routine for RHD code
!
! PROCEDURE:
!   Initialize global variables and call unit-test master routine.
!
! MODIFICATION HISTORY:
!   2015-02-23 (B.F. Uppsala) First version
!   2015-03-01 (B.F.) Rename rhd_UnitTest -> rhdUT
!---------------------------------------------------------------------------------------------------
use rhd_gl_module
use UT_module
use rhdUT_master_module
!
implicit none
!---------------------------------------------------------------------------------------------------
!
! --- Set important global parameters ---
UT%nc_p        =6                           ! --- Channel nc_p for text output of unit tests ---
gl%nc_p        =6                           ! --- Channel nc_p for text output  ---
gl%nc_m        =6                           ! --- Channel nc_p for text output  ---
gl%lw_p        =84                          ! --- Line width for text output    ---
!gl%parFile     ='rhd.par'                   ! --- Name of parameter input file  ---
!gl%StopFile    ='rhd.stop'                  ! --- Name of STOP request file     ---
!gl%ContFile    ='rhd.cont'                  ! --- Name of CONTINUE request file ---
!gl%StatusFile  ='rhd.done'                  ! --- Name of status output file    ---
!gl%DumpFile    ='rhd.dump'                  ! --- Name of dump request file     ---
!gl%SnapshotFile='rhd.snap'                  ! --- Name of snapshot output file  ---
!gl%version     ='RHD 003.00.2015.02.23'     ! --- RHD code version              ---
!
!call UT_DemoExamplePassAll()
!call UT_DemoExampleFailAll()
!
call rhdUT_master_TestAll()
!
end program rhdUT
