function cosinebell, x, epsilon, delta, xf=xf
;
;------------------------------------------------------------------------------
;+
; NAME:
;   cosinebell ('cosine_bell')
;
; PURPOSE:
;   Compute the cosine bell function.
;
; CATEGORY:
;   Mathematical functions

; CALLING SEQUENCE:
;   a=cosinebell(x [, epsilon, delta, xf=xf])
;
; INPUT:
;   x:       Argument of function
;   epsilon: Sharpness parameter: 0.0 < epsilon <= 0.5  (default=0.5)
;   delta:   Inset parameter: 0.0 <= delta < epsilon      (default=0.0)
;
; OUTPUT:
;   cosinebell: cosine bell of x
;
; KEYWORDS:
;   xf:      Normalized x axis (Output for test purposes)
;
; EXAMPLE:
;   x=findgen(100)/100.0
;   cb=cosinebell(x, 0.4, 0.1)
;   plot, x, cb, ystyle=3
;
; MODIFICATION HISTORY:
;    7.11.93 Written (B.F.)
;   13.12.93 Parameter DELTA added
;    1. 4.04 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
; --- Default value for epsilon
if (n_params() lt 2) then epsilon=0.5
; --- Default value for delta
if (n_params() lt 3) then delta=0.0
;
xf=abs((abs(x)-float(fix(abs(x)+0.5)))) < epsilon    ; x -> xf in [0.0,0.5]
xf=((xf>delta)-delta)/(epsilon-delta)
return, 0.5-0.5*cos(!PI*xf)                  ; Cosine bell
;
end ; cosinebell
