pro FFTAXIS, t, nu=nu, omega=omega, tnorm=tnorm
;
;+
; NAME:
;   FFTAXIS
;
; PURPOSE:
;   Generation of a frequency axis from the time axis, used for FFT analysis.
;
; CATEGROY:
;   Analysis
;
; CALLING SEGQUENCE:
;   FFTAXIS, t, nu=nu [, omega=omega, tnorm=tnorm]
;   
; INPUT:
;   t     - Vector with time (or x) axis
;
; OUTPUT:
;   none
;
; KEYWORDS:
;   nu    - Frequency axis (Output)
;   omega - Vector with angular frequency (Output)
;   tnorm - Vector with normalized time axis (Output)
;
; PROCEDURE:
;   Simple
;
; MODIFICATION HISTORY:
;   25. 6.93 Written by Bernd Freytag, Astrophysik Kiel
;    2.12.96 (B.F.) 
;-
;
nt=n_elements(t)
tmin=min(t,max=tmax)
;
tnorm=t-tmin
;
nu=findgen(nt)/(tmax-tmin)
omega=2.0*!pi*nu
;
end ; FFTAXIS
