function REPLICATE1, ele, num
;+
; NAME:
;   REPLICATE1
;
; PURPOSE:
;   Replicate array (extension of REPLICATE)
;
; CATEGORY:
;   
; CALLING SEQUENCE:
;
; INPUTS:
;   ele - Input array
;   num - Number of elements of output vector
;         (NOT number of replications of array!)
; OUTPUTS:
;   replicate1 - Output vector (NOT an array)
;
; KEYWORDS:
;   None
;
; SIDE EFFECTS:
;   None
;
; EXAMPLES:
;   print, REPLICATE1([1,2],5)
;   1       2       1       2       1
;   print, REPLICATE1([1,2],1)
;   1
;   print, REPLICATE1([[1,2],[3,4]],5)
;   1       2       3       4       1
;
; MODIFICATION HISTORY:
;   Written: 7.4.94 (Bernd Freytag, Astrophysik Kiel)
;            23.10.94
;-
;
if (num ge 1) then begin
  ; --- Number of elements of input vector ---
  elenum=n_elements(ele)
  ; --- Output with enough elements with the right type ---
  out=replicate(ele(0),(num/elenum+1)*elenum)
  ; --- Copy the input array multiply into output array ---
  for n=0,num/elenum do out(n*elenum)=ele(*)
  ; --- Reduce size of output array to exact desired size ---
  out=out(0:num-1)
endif else begin
  out=ele
endelse
;
return, out
;
end ; REPLICATE1
