pro addbox, box1, box2
;
;------------------------------------------------------------------------------
;+
; NAME:
;   addbox ('add_box')
;
; PURPOSE:
;   Add all quantities in second box to first.
;
; CATEGORY:
;   Box handling
;
; CALLING SEQUENCE:
;   addbox, box1, box2
;
; INPUT:
;   box2:    ('box_2') data structure with CO5BOLD data
;
; INPUT/OUTPUT:
;   box1:    ('box_1') data structure with CO5BOLD data
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   This routine does essentially
;     "box1 = box1 + box2"
;   Most quantities are added (rho,ei,v1,v2,v3,time...).
;   Axes and header info are kept from box1.
;   time_out_full_last and time_out_mean_last are taken from box2.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   24. 7.03 Written by Bernd Freytag, Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Header ---
box1.modeltime =box1.modeltime +box2.modeltime
box1.modelitime=box1.modelitime+box2.modelitime
box1.dtime     =box1.dtime     +box2.dtime
box1.time_out_full_last=box2.time_out_full_last
box1.time_out_mean_last=box2.time_out_mean_last
;
; --- Contents of box "Z" ---
box1.z.time =box1.z.time +box2.z.time
box1.z.itime=box1.z.itime+box2.z.itime
box1.z.rho  =box1.z.rho  +box2.z.rho
box1.z.ei   =box1.z.ei   +box2.z.ei
box1.z.v1   =box1.z.v1   +box2.z.v1
box1.z.v2   =box1.z.v2   +box2.z.v2
box1.z.v3   =box1.z.v3   +box2.z.v3
;
tags=tag_names(box1)
;
; --- Contents of box "EOS" ---
if (total(tags eq 'EOS') gt 0.0) then begin
  box1.eos.s     =box1.eos.s     +box2.eos.s
  box1.eos.T     =box1.eos.T     +box2.eos.T
  box1.eos.P     =box1.eos.P     +box2.eos.P
  box1.eos.dPdrho=box1.eos.dPdrho+box2.eos.dPdrho
  box1.eos.dPdei =box1.eos.dPdei +box2.eos.dPdei
  box1.eos.dTdei =box1.eos.dTdei +box2.eos.dTdei
endif
;
; --- Contents of box "OPA" ---
if (total(tags eq 'OPA') gt 0.0) then begin
  box1.opa.kappa =box1.opa.kappa +box2.opa.kappa
endif
;
end ; addbox
