pro divbox, box, num
;
;------------------------------------------------------------------------------
;+
; NAME:
;   divbox ('divide_box')
;
; PURPOSE:
;   Divide all quantities in box by number.
;
; CATEGORY:
;   Box handling
;
; CALLING SEQUENCE:
;   divbox, box, num
;
; INPUT:
;   num:    ('number') Number to devide by
;
; INPUT/OUTPUT:
;   box:    ('box') data structure with CO5BOLD data
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   This routine does essentially
;     "box1 = box1/num"
;   Most quantities are divided by number (rho,ei,v1,v2,v3,time...).
;   Axes, header info, time_out_full_last and time_out_mean_last are not touched
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   24. 7.03 Written by Bernd Freytag, Uppsala
;-
;------------------------------------------------------------------------------
;
fnum=float(num)
lnum=long(num)
if (lnum eq 0) then lnum=1L
;
; --- Header ---
box.modeltime =box.modeltime /fnum
box.modelitime=box.modelitime/lnum
box.dtime     =box.dtime     /fnum
;
; --- Contents of box "Z" ---
box.z.time =box.z.time /fnum
box.z.itime=box.z.itime/fnum
box.z.rho  =box.z.rho  /fnum
box.z.ei   =box.z.ei   /fnum
box.z.v1   =box.z.v1   /fnum
box.z.v2   =box.z.v2   /fnum
box.z.v3   =box.z.v3   /fnum
;
tags=tag_names(box)
;
; --- Contents of box "EOS" ---
if (total(tags eq 'EOS') gt 0.0) then begin
  box.eos.s     =box.eos.s     /fnum
  box.eos.T     =box.eos.T     /fnum
  box.eos.P     =box.eos.P     /fnum
  box.eos.dPdrho=box.eos.dPdrho/fnum
  box.eos.dPdei =box.eos.dPdei /fnum
  box.eos.dTdei =box.eos.dTdei /fnum
endif
;
; --- Contents of box "OPA" ---
if (total(tags eq 'OPA') gt 0.0) then begin
  box.opa.kappa =box.opa.kappa /fnum
endif
;
end ; divbox
