function mdopta, opa0, Name=Name0, Comment=Comment0, $
  Tpoints=Tpoints, Ppoints=Ppoints, Wpoints=Wpoints
;
;------------------------------------------------------------------------------
;+
; NAME:
;   mdopta ('modify_opacity_table')
;
; PURPOSE:
;   Modify opacity table.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   result=mdopta(opa0, Name=Name0, Comment=Comment0, $
;                 Tpoints=Tpoints, Ppoints=Ppoints, Wpoints=Wpoints)
;
; INPUT:
;   opa0:         ('opacities_0') Structure with input opacity data
;
; OUTPUT:
;   result:       Structure with output opacity data
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   monocubic:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-21 (B.F. Garching)
;-
;------------------------------------------------------------------------------

; --- Check optional keywords and set default values ---
if (keyword_set(Name0)) then begin
  Name=Name0
endif else begin
  Name='mdopta_opacity_table'
endelse
if (keyword_set(Comment0)) then begin
  Comment=Comment0
endif else begin
  Comment=''
endelse

; --- Copy entire opacity table ---
opaOut=opa0

; --- Modify header ---
TimeCreate=systime(0)
opaOut.Header[1]='*     Opacity table name: ' + Name
opaOut.Header[2]='*     Created           : ' + TimeCreate
opaOut.Header[5]='*     '                     + Comment
opaOut.OptaNa   =Name
opaOut.DateX    =TimeCreate

if (keyword_set(Tpoints)) then begin
  ; --- Create temperature-dependent weight mask ---
  ;
  logTpoints=alog10(Tpoints > 1.0)
  Tmask     =monocubic(Wpoints, logTpoints, opa0.tabT)
  ;
  for iBand=1,opa0.nBand do begin
    for iP=0,opa0.nP-1 do begin
      opaOut.xopta[*, iP, iBand]=         Tmask * opaOut.xopta[*, iP, iBand] + $
                                 (1.0D+00-Tmask)* opaOut.xopta[*, iP, 0    ]
    endfor ; iP
  endfor ; nBand
  ;
endif else begin
  ; --- Create pressure-dependent weight mask ---
  ;
  logPpoints=alog10(Ppoints > 1.0E-10)
  Pmask     =monocubic(Wpoints, logPpoints, opa0.tabP)
  ;
  for iBand=1,opa0.nBand do begin
    for iT=0,opa0.nT-1 do begin
      opaOut.xopta[iT, *, iBand]=         Pmask * opaOut.xopta[iT, *, iBand] + $
                                 (1.0D+00-Pmask)* opaOut.xopta[iT, *, 0    ]
    endfor ; iP
  endfor ; nBand
  ;
endelse

; --- Return result structure ---
return, opaOut

end ; mdopta
