function qshock, command, gamma, rho
;
;------------------------------------------------------------------------------
;+
; NAME:
;   qshock ('quantity_shock')
;
; PURPOSE:
;   Compute pressure ratio from gamma and density ratio for shock
;
; CATEGORY:
;   EOS
;
; CALLING SEQUENCE:
;   out=qshock(command, gamma, rho)
;
; INPUT:
;   command: ('command') string with requested quantity
;   gamma:   ('gamma') ratio of specific heats
;   rho:     ('rho') density ratio
;
; OUTPUT:
;   out:     pressure ratio
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; SEE:
;   http://en.wikipedia.org/wiki/Rankine–Hugoniot_conditions
;
; MODIFICATION HISTORY:
;   2012-06-18 (B.F. Lyon) Written
;-
;------------------------------------------------------------------------------

cm=strlowcase(command)

if (cm eq 'p') then begin
  out=(rho*(gamma+1.0) -           (gamma-1.0)) / $
      (    (gamma+1.0) - rho*      (gamma-1.0))
endif else if (cm eq 'e') then begin
  out=(    (gamma+1.0) - 1.0/rho * (gamma-1.0)) / $
      (    (gamma+1.0) - rho*      (gamma-1.0))
endif else if (cm eq 'porhotgamma') then begin
  out=(rho*(gamma+1.0) -           (gamma-1.0)) / $
      (    (gamma+1.0) - rho*      (gamma-1.0)) * $
      rho^(-gamma)
endif else if (cm eq 's') then begin
  out=(rho*(gamma+1.0) -           (gamma-1.0)) / $
      (    (gamma+1.0) - rho*      (gamma-1.0)) * $
      rho^(-gamma)
  out=!con.Rgas/(gamma-1.0) * alog(out)
endif else begin
  message, 'Command ' + cm + ' is not known.'
endelse

return, out


end ; qshock
