pro rdopta_find_line_containing, string, unit
   text = ' '
   while (strpos(text, string) lt 0) do readf, unit, format='(A)', text
return
end

pro rdopta, filename, opa
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rdopta ('read_opacity_table')
;
; PURPOSE:
;   Read opacity table from file and put it into common block and structure
;
; CATEGORY:
;   I/O, opacities
;
; CALLING SEQUENCE:
;   rdopta, filename, opa
;
; INPUT:
;   filename: ('file_name') String with file name
;
; OUTPUT:
;   opa:      ('opacities') Structure with output data
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   opta_common: nt, np, nband, topta, popta, xopta
;
; ROUTINES:
;   rdopta_find_line_containing
;
; SIDE EFFECTS:
;   The contents of the common block opta_common is changed.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1998-11-18 (HGL) dfopta.pro
;   2008-05-07 (B.F. Lyon) dfopta.pro -> rdopta.pro, put data into structure
;   2009-07-10 (B.F. Paris) More comments
;   2009-10-30 (B.F. Lyon) Weights wtbbd put into common block
;-
;------------------------------------------------------------------------------

common opta_common, nt, np, nband, topta, popta, xopta, wtbbd

dum      = strarr(1)

; --- Open file ---
get_lun, unit
openr, unit, filename

; --- Read header (lines starting with star between lines starting with 8 stars) ---
rdopta_find_line_containing, '********', unit
text=''
nheadermax=1000L
header=strarr(nheadermax)
iheader=0L
endheader_flag=0
repeat begin
  readf, unit, format='(A)', text
  if (strmid(text,0,1) eq '*') then begin
    if (strmid(text,0,8) eq '********') then begin
      ; --- End of header block reached. ---
      endheader_flag=1
    endif else begin
      ; --- Put comment line into header array ---
      header[iheader]=text
      iheader=(iheader+1L) < (nheadermax-1L)
    endelse
  endif else begin
    ; --- Line without leading * reached. That should not happen within header. ---
    close, unit
    free_lun, unit
    message, 'Reading beyond header.'
  endelse
endrep until (endheader_flag)
if (iheader gt 0L) then begin
  header=header[0:iheader-1L]
endif else begin
  header=['*']
endelse

idvers= 0L
idmean= 0L
optana= ""
datex = ""
nt    = 0L
ntbn  = 0L
np    = 0L
npbn  = 0L
nband = 0L
rdopta_find_line_containing, 'IDVERS', unit
readf, unit, idvers
rdopta_find_line_containing, 'OPTANA', unit
readf, unit, optana
rdopta_find_line_containing, 'DATEX', unit
readf, unit, datex
rdopta_find_line_containing, 'IDMEAN', unit
readf, unit, idmean
rdopta_find_line_containing, 'NT', unit
readf, unit, nt
rdopta_find_line_containing, 'NTBN', unit
readf, unit, ntbn
rdopta_find_line_containing, 'NP', unit
readf, unit, np
rdopta_find_line_containing, 'NPBN', unit
readf, unit, npbn
rdopta_find_line_containing, 'NBAND', unit
readf, unit, nband

mstep =0L
nulo  =0L
nuhi  =0L
numnu =0L
intmdx=0L
rdopta_find_line_containing, 'MSTEP', unit
readf, unit, mstep
rdopta_find_line_containing, 'NULO', unit
readf, unit, nulo
rdopta_find_line_containing, 'NUHI', unit
readf, unit, nuhi
rdopta_find_line_containing, 'NUMNU', unit
readf, unit, numnu
rdopta_find_line_containing, 'INTMDX', unit
readf, unit, intmdx
;help, nt, np, nband, nulo, nuhi, numnu

topta =fltarr(nt)
tabtbn=fltarr(ntbn)
idxtbn=lonarr(ntbn)
tabdtb=fltarr(ntbn-1)
rdopta_find_line_containing, 'TABT', unit
readf, unit, topta
rdopta_find_line_containing, 'TABTBN', unit
readf, unit, tabtbn
rdopta_find_line_containing, 'IDXTBN', unit
readf, unit, idxtbn
rdopta_find_line_containing, 'TABDTB', unit
readf, unit, tabdtb

popta = fltarr(np)
tabpbn=fltarr(npbn)
idxpbn=lonarr(npbn)
tabdpb=fltarr(npbn-1)
rdopta_find_line_containing, 'TABP', unit
readf, unit, popta
rdopta_find_line_containing, 'TABPBN', unit
readf, unit, tabpbn
rdopta_find_line_containing, 'IDXPBN', unit
readf, unit, idxpbn
rdopta_find_line_containing, 'TABDPB', unit
readf, unit, tabdpb

xopta = fltarr(nt,np,nband+1) ; Rosseland averages in band 0
dum   = fltarr(nt)
idum  = 1
for i = 0, np-1 do begin
for j = 0, nband do begin
  rdopta_find_line_containing, 'log10 P', unit
  readf, unit, idum
  readf, unit, dum
  xopta[*,i,j] = dum
endfor ; j
endfor ; i

wtbbd=fltarr(nt,nband+1)
dum   = fltarr(nt)
for j = 0, nband do begin
  rdopta_find_line_containing, 'IBAND', unit
  readf, unit, idum
  readf, unit, dum
  wtbbd[*,j]=dum
endfor ; j
if (nband gt 1) then begin
  ; --- Normalize WTBBD fractions ---
  wtbbdtot=rebin(reform(total(wtbbd[*,1:*], 2), nt,1), nt,nband)
  wtbbd[*,1:*]=wtbbd[*,1:*]/wtbbdtot
endif

inband=bytarr(numnu)
rdopta_find_line_containing, 'INBAND', unit
readf, unit, inband

close, unit
free_lun, unit

opa={ $
  header: header, $
  idvers: idvers, $
  optana: optana, $
  datex:  datex, $
  idmean: idmean, $
  nt:     nt, $
  ntbn:   ntbn, $
  np:     np, $
  npbn:   npbn, $
  nband:  nband, $
  mstep:  mstep, $  
  nulo:   nulo, $
  nuhi:   nuhi, $
  numnu:  numnu, $
  intmdx: intmdx, $
  ;
  tabt:   topta, $
  tabtbn: tabtbn, $
  idxtbn: idxtbn, $
  tabdtb: tabdtb, $
  ;
  tabp:   popta, $
  tabpbn: tabpbn, $
  idxpbn: idxpbn, $
  tabdpb: tabdpb, $
  ;
  xopta:  xopta, $
  ;
  wtbbd:  wtbbd, $
  inband: inband $
    }

end ; rdopta
