function refinebox, command, box, num1, num2, num3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   refinebox ('refine_box')
;
; PURPOSE:
;   Interpolate box quantities onto finer or coarser grid.
;
; CATEGORY:
;   EOS
;
; CALLING SEQUENCE:
;   out=refinebox(command, box)
;
; INPUT:
;   command: ('command') string: 'central'
;   box:     ('box') data structure with CO5BOLD data (EOS and opacity data are
;                    ignored)
;
; OUTPUT:
;   out:     ('output') 3D array
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   ; --- Assume: modelfile is defined ---
;
;   ; --- Load data set ---
;   ful=uio_dataset_rd(modelfile)
;
;   ; --- Interpolate model onto grid with 127^3 points ---
;   sta=refinebox('central', ful, 127,127,127)
;
; MODIFICATION HISTORY:
;    2003-07-01 Written by Bernd Freytag, Uppsala
;    2009-01-29 (B.F.) quc
;    2010-02-25 (B.F.) Bb
;-
;------------------------------------------------------------------------------

rhd_manipulate_parsebox, box, indexstart=command, $
  m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;help, Bb1, Bb2, Bb3

rhd_manipulate_interpolate, num1,num2,num3, $
  m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
  indexstart=command
;help, Bb1, Bb2, Bb3

rhd_manipulate_assure3d, $
  m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3,          Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;help, Bb1, Bb2, Bb3

rhd_manipulate_makebox, out, $
  m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3, $
  inbox=box

return, out

end ; refinebox
