pro rhd_manipulate_assure3d, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, $
      Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_assure3d ('rhd_manipulate_assure_3_dimensions')
;
; PURPOSE:
;   Assure that all axes and bulk data arrays are 3D.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_assure3d, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;
; INPUT/OUTPUT:
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;   Bb1:    ('B_boundary_1') real 3D array, magnetic field, 1st coordinate
;   Bb2:    ('B_boundary_2') real 3D array, magnetic field, 2nd coordinate
;   Bb3:    ('B_boundary_3') real 3D array, magnetic field, 3rd coordinate
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2001-12-11 (B.F. Uppsala) Written
;   2004-07-29 (B.F. Montpellier) Indices long integer
;   2010-02-25 (B.F.) BbX parameters
;   2010-03-15 (B.F. Naples) Use "reform" with keyword "overwrite"
;-
;------------------------------------------------------------------------------

; --- Axes ---
xc1=reform(xc1, n1-m1+1, 1      , 1      , /overwrite)
xb1=reform(xb1, n1-m1+2, 1      , 1      , /overwrite)
xc2=reform(xc2, 1      , n2-m2+1, 1      , /overwrite)
xb2=reform(xb2, 1      , n2-m2+2, 1      , /overwrite)
xc3=reform(xc3, 1      , 1      , n3-m3+1, /overwrite)
xb3=reform(xb3, 1      , 1      , n3-m3+2, /overwrite)

; --- Bulk arrays ---
rho=reform(rho, n1-m1+1, n2-m2+1, n3-m3+1, /overwrite)
ei =reform(ei , n1-m1+1, n2-m2+1, n3-m3+1, /overwrite)
v1 =reform(v1 , n1-m1+1, n2-m2+1, n3-m3+1, /overwrite)
v2 =reform(v2 , n1-m1+1, n2-m2+1, n3-m3+1, /overwrite)
v3 =reform(v3 , n1-m1+1, n2-m2+1, n3-m3+1, /overwrite)
if (keyword_set(Bb1)) then Bb1=reform(Bb1, n1-m1+2, n2-m2+1, n3-m3+1, /overwrite)
if (keyword_set(Bb2)) then Bb2=reform(Bb2, n1-m1+1, n2-m2+2, n3-m3+1, /overwrite)
if (keyword_set(Bb3)) then Bb3=reform(Bb3, n1-m1+1, n2-m2+1, n3-m3+2, /overwrite)

; --- Indices ---
m1=long(m1) & n1=long(n1)
m2=long(m2) & n2=long(n2)
m3=long(m3) & n3=long(n3)

end ; rhd_manipulate_assure3d
