pro rhd_manipulate_new1dgridgen, ful, z1, P1, $
  j1max=j1max, z1bot=z1bot, z1top=z1top, dza=dza, dzb=dzb, $
  dist_const_top=dist_const_top, $
  aspect_min=aspect_min0, aspect_max=aspect_max0, dzchange=dzchange0, $
  input_type=input_type0, z0in=z0in, dxbin=dxbin
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_new1dgridgen
;
; PURPOSE:
;   Modify grid to make smooth grid size changes.
;
; CATEGORY:
;   Computation (special)
;
; CALLING SEQUENCE:
;   rhd_manipulate_new1dgridgen
;
; INPUT:
;   ful:   ('full') Structure with complete model data
;   j1max: (j_1_maximum') integer with number of points of new axis
;
; OUTPUT:
;   z1:    ('z_1') real vector with new z axis
;   P1:    ('P_1') real vector with pressure along new axis
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   smooth_super - For smoothing
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Not a universal routine for proper grid spacing smoothing of every grid
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1995-04-12 (B.F. Kiel) Written
;   1997-03-25 (B.F.) dzz: j1max -> j0max elements
;   2002-11-05 (B.F.) gridchange.pro -> rhd_manipulate_new1dgridgen.pro
;   2004-07-29 (B.F. Montpellier) aspect_min/max, dzchange
;   2007-11-15 (B.F. Lyon) Keywords input_type, z0in, dxbin
;   2014-12-09 (B.F. Uppsala) Some cleaning up
;-
;------------------------------------------------------------------------------
;
if (not keyword_set(input_type0)) then begin
  ; --- Properties of input axis ---
  q=combox('tau3r', ful)
  tau0=10.0^(avg(avg(alog10(q),0),0))
  q=combox('p', ful)
  P0=10.0^(avg(avg(alog10(q),0),0))
  q=0.0
  z0=reform(ful.z.xc3)
  dx=(reform(ful.z.xb1))[1]-(reform(ful.z.xb1))[0]
  ;
  j0max=n_elements(P0)
  logP0=alog(P0)
  Hp0=-deriv(logP0, z0)
endif else begin
  z0=z0in
  dx=dxbin
  j0max=n_elements(z0)
endelse
;
; --- Properties of output axis ---
if (n_elements(dzchange0) eq 0) then begin
  dzchange=0.05
endif else begin
  dzchange=dzchange0 > 0.0
endelse
if (n_elements(j1max) eq 0) then j1max=j0max
if (n_elements(z1top) eq 0) then z1top=z0[j0max-1]
if (n_elements(z1bot) eq 0) then z1bot=z0[0]
z1total=z1top-z1bot
z1=replicate(z1bot, j1max)
;
; --- Parameters for iteration ---
dzMultMin=0.01
dzMultMax=10.0
eps=5.0e-6*z1total
nIter=0
nIterMax=50
;
if (not keyword_set(input_type0)) then begin
  if (not keyword_set(dza)) then begin
    ; --- dz ---
    dza=replicate(0.20, j0max)
    ;
    if (keyword_set(dist_const_top)) then begin
      ind=where((z0(j0max-1)-z0) lt dist_const_top, nind)
      if (nind ne 0) then dza[ind]=0.10
    endif else begin
      ind=where(tau0 lt 100.0, nind)
      if (nind ne 0) then dza[ind]=0.10
      ind=where(tau0 lt 5.0, nind)
      if (nind ne 0) then dza[ind]=0.08
      ind=where(tau0 lt 0.01, nind)
      if (nind ne 0) then dza[ind]=0.15
    endelse
  endif
  ;
  if (not keyword_set(dzb)) then begin 
    dzb=dza*Hp0
  endif
endif
;
; --- Restrict the aspectratio ---
if (keyword_set(aspect_min0)) then begin
  aspect_min=aspect_min0
endif else begin
  aspect_min=0.30
endelse
if (keyword_set(aspect_max0)) then begin
  aspect_max=aspect_max0
endif else begin
  aspect_max=1.4
endelse
;
; --- Iteration to find new z axis ---
repeat begin
  nIter=nIter+1
  dzMult=sqrt(dzMultMax*dzMultMin)
  ;
  dzx=dzMult*dzb
  ;
  dzy=((aspect_min*dx) > dzx < (aspect_max*dx))          ; --- 2004-07-29 ---
;  dzy=((0.30*dx) > dzx < (1.4*dx))          ; --- 2002-11-05 ---
;  dzy=((0.35*dx) > dzx < (1.6*dx))
;  dzy=((0.35*dx) > dzx < (2.5*dx))
;  dzy=((0.50*dx) > dzx < (1.6*dx))         ; --- 21.1.96 ---
  ;
  ; --- Smoothing (double: 2004-03-23) ---
  dzz=smooth_super(dzy, 3, 1)
  dzz=smooth_super(dzz, 3, 1)
  ;
  ; --- Integrate ---
  z1[j1max-1]=z1top
  for j=j1max-1,1,-1 do begin
    if (keyword_set(dist_const_top)) then begin
      if ((z0[j0max-1]-z1[j]) lt dist_const_top) then begin
        dz=dzz[n_elements(dzz)-1]
      endif else begin
        dz=interpol[dzz,z0,z1[j]]
      endelse
    endif else begin
      dz=interpol(dzz, z0, z1[j])
    endelse
;    if (j lt (j1max-1)) then dz=((0.95*dzold) > dz < (1.05*dzold))
    if (j lt (j1max-1)) then begin
      if ((dzold/dx) lt (0.1*aspect_max+0.9*aspect_min)) then begin
        dzchsasp=dzchange*0.5
        dz=(((1.0-dzchsasp)*dzold) > dz < ((1.0+dzchsasp)*dzold))
      endif else if ((dzold/dx) lt (0.5*aspect_max+0.5*aspect_min)) then begin
;if ((j eq 80) or (j eq 50)) then print,' ', j, dz/dx, 0.5*(aspect_max+aspect_min), dz, (((1.0-dzchange)*dzold) > dz < ((1.0+dzchange)*dzold))
        dz=(((1.0-dzchange)*dzold) > dz < ((1.0+dzchange)*dzold))
      endif else begin
;if ((j eq 80) or (j eq 50)) then print,'*', j, dz/dx, 0.5*(aspect_max+aspect_min), dz, (((1.0-dzchlasp)*dzold) > dz < ((1.0+dzchlasp)*dzold))
        dzchlasp=dzchange*2.0
        dz=(((1.0-dzchlasp)*dzold) > dz < ((1.0+dzchlasp)*dzold))
      endelse
    endif
    dzold=dz
    z1[j-1]=z1[j]-dz
    ;print,j,z1[j],dz,z1[j-1]
  endfor ; j
  ;
  if (z1[0] lt z1bot) then dzMultMax=dzMult $
                      else dzMultMin=dzMult
  print, nIter, '/', nIterMax, ':', dzMultMin, dzMultMax, dzMult, z1[0], z1bot, z1[0]-z1bot, $
         format='(I3,A,I3,A,3F10.6,2X,3E13.5)'
endrep until (abs(z1[0]-z1bot) lt eps) or (nIter ge nIterMax)
;
; --- Interpolate pressure on new axis ---
if (not keyword_set(input_type0)) then P1=exp(monocubic(logP0, z0, z1))
;
end ; rhd_manipulate_new1dgridgen
