pro rhd_manipulate_rempoints, addn, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_rempoints ('rhd_manipulate_remove_points')
;
; PURPOSE:
;   Remove number of points on all sides of simulation data.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_rempoints, addn, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   addn:   ('add_number') integer, number of points to be removed on all sides
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   11.12.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   10.06.03 (B.F.) Keyword rhofac added
;   2007-04-02 (B.F. Lyon) Taken from addpoints
;-
;------------------------------------------------------------------------------

; --- New index values ---
m1new=m1+addn
n1new=n1-addn
m2new=m2+addn
n2new=n2-addn
m3new=m3+addn
n3new=n3-addn

; --- Put old values into new vectors ---
xc1=reform(xc1(addn:n1new-m1new+addn  ))
xb1=reform(xb1(addn:n1new-m1new+addn+1))
xc2=reform(xc2(addn:n2new-m2new+addn  ))
xb2=reform(xb2(addn:n2new-m2new+addn+1))
xc3=reform(xc3(addn:n3new-m3new+addn  ))
xb3=reform(xb3(addn:n3new-m3new+addn+1))

; --- Bulk arrays: copy old part ---
rho=rho(addn:n1new-m1new+addn, addn:n2new-m2new+addn, addn:n3new-m3new+addn)
ei =ei( addn:n1new-m1new+addn, addn:n2new-m2new+addn, addn:n3new-m3new+addn)
v1 =v1( addn:n1new-m1new+addn, addn:n2new-m2new+addn, addn:n3new-m3new+addn)
v2 =v2( addn:n1new-m1new+addn, addn:n2new-m2new+addn, addn:n3new-m3new+addn)
v3 =v3( addn:n1new-m1new+addn, addn:n2new-m2new+addn, addn:n3new-m3new+addn)

; --- New indices ---
m1=m1new
n1=n1new
m2=m2new
n2=n2new
m3=m3new
n3=n3new

end ; rhd_manipulate_rempoints
