pro rhd_manipulate_rotate, axis, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_rotate ('rhd_manipulate_rotate')
;
; PURPOSE:
;   Rotate simulation data arrays.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_rotate, axis, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   axis:   ('axis') integer
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Doesn't do anything yet.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   11.12.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Transpose/rotate for cube ---
rho= reverse(rho, 1, /overwrite)
ei = reverse(ei , 1, /overwrite)
v1 =-reverse(v1 , 1, /overwrite)
v2 = reverse(v2 , 1, /overwrite)
v3 = reverse(v3 , 1, /overwrite)
;
;rho0=rho
;ei0 =ei
;v10 =v1
;v20 =v2
;v30 =v3
;l3=n_elements(xc3)-1
;for i=0,l3 do begin &$
;  rho(*,*,i)=rotate(reform(rho0(*,*,l3-i)),2) &$
;  ei( *,*,i)=rotate(reform( ei0(*,*,l3-i)),2) &$
;  v1( *,*,i)=rotate(reform(-v10(*,*,l3-i)),2) &$
;  v2( *,*,i)=rotate(reform(-v20(*,*,l3-i)),2) &$
;  v3( *,*,i)=rotate(reform(-v30(*,*,l3-i)),2) &$
;endfor
;
end ; rhd_manipulate_rotate
