pro rhd_manipulate_spinup, period, $
      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_manipulate_spinup ('rhd_manipulate_spin_up')
;
; PURPOSE:
;   Add velocity field to data to induce bulk rotation around x3 axis.
;
; CATEGORY:
;   Radiation hydrodynamics, data handling
;
; CALLING SEQUENCE:
;   rhd_manipulate_spinup, period, $
;     m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
; INPUT:
;   period:   ('period') real number to specify rotation period
;
; INPUT/OUTPUT:
;   m1:     ('m_1') integer, lower index, 1st dimension
;   n1:     ('n_1') integer, upper index, 1st dimension
;   m2:     ('m_2') integer, lower index, 2nd dimension
;   n2:     ('n_2') integer, upper index, 2nd dimension
;   m3:     ('m_3') integer, lower index, 3rd dimension
;   n3:     ('n_3') integer, upper index, 3rd dimension
;   xc1:    ('x_center_1') real, 1. coordinate of cell centers
;   xc2:    ('x_center_2') real, 2. coordinate of cell centers
;   xc3:    ('x_center_3') real, 3. coordinate of cell centers
;   xb1:    ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:    ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:    ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:    ('rho') real 3D array, mass density
;   ei:     ('energy_internal') real 3D array, internal energy density per mass
;   v1:     ('v_1') real 3D array, velocity, 1st coordinate
;   v2:     ('v_2') real 3D array, velocity, 2nd coordinate
;   v3:     ('v_3') real 3D array, velocity, 3rd coordinate
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   16.04.02 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Circular frequency of rotation ---
omega=2.0*!pi/period
;
; --- Make 3D arrays with axis values ---
xc1_3d=rebin(reform(xc1, n1-m1+1, 1      , 1),  n1-m1+1,  n2-m2+1,  n3-m3+1)
xc2_3d=rebin(reform(xc2, 1,       n2-m2+1, 1),  n1-m1+1,  n2-m2+1,  n3-m3+1)
;
; --- Add rotational velocity field
v1_rot =-omega*xc2_3d
v2_rot = omega*xc1_3d
;
print, "minmax(v1_rot)= ", minmax(v1_rot)
print, "minmax(v2_rot)= ", minmax(v2_rot)
;
v1=v1 + v1_rot
v2=v2 + v2_rot
;
end ; rhd_manipulate_spinup
