pro rhd_wrboxdata, command, nc_p, ncout, file=file, form=form, conv=conv, double_flag=double_flag, $
                   ndes=ndes, description=description, $
                   nhis=nhis, history=history, $
                   version=version, $
                   unit_time=unit_time, $
                   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                   unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
                   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                   itime=itime, time=time, dtime=dtime, $
                   out_time_full_last=out_time_full_last, $
                   out_time_mean_last=out_time_mean_last, $
                   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei, $
                   quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_wrboxdata ('rhd_write_box_data')
;
; PURPOSE:
;   Write RHD model box into file in uio form.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rhd_wrboxdata, command, nc_p, ncout, file=file, form=form, conv=conv, $
;                   ndes=ndes, description=description, $
;                   nhis=nhis, history=history, $
;                   version=version, $
;                   unit_time=unit_time, $
;                   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
;                   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
;                   unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
;                   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
;                   itime=itime, time=time, dtime=dtime, $
;                   out_time_full_last=out_time_full_last, $
;                   out_time_mean_last=out_time_mean_last, $
;                   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
;                   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei, $
;                   quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;
; INPUT:
;   command: ('command') character, command string:
;            'open', 'model, 'box', 'endmo', 'close' (see below)
;   nc_p:    ('number_channel_print') integer, unit number for text output
;
; INPUT/OUTPUT:
;   ncout:   ('number_channel_output') integer, unit number for data output
;
; KEYWORDS:
;   file:    ('file') character, name of data file
;   form:    ('format') character, name of data file
;   conv:    ('conversion') character, conversion type of data file
;   ndes:    ('number_description') integer, number of lines of description
;   description:('description') character, dimension(:), description of model
;   nhis:    ('number_history') integer, number of lines of history
;   history: ('history') character, dimension(:), history of model
;   version: ('version') character, string with RHD version
;   time_unit:('time_unit') character, unit of time (e.g. 's')
;   x1_unit: ('x1_unit') character, unit of 1. space coordinate (e.g. 'cm')
;   x2_unit: ('x2_unit') character, unit of 2. space coordinate (e.g. 'cm')
;   x3_unit: ('x3_unit') character, unit of 3. space coordinate (e.g. 'cm')
;   rho_unit:('rho_unit') character, unit of density (e.g. 'g/cm^3')
;   e_unit:  ('energy_unit') character, unit of energy per mass (e.g. 'erg/g')
;   v_unit:  ('velocity_unit') character, unit of velocity (e.g. 'cm/s')
;   p_unit:  ('pressure_unit') character, unit of pressure (e.g. 'dyn/cm^2')
;   box_id:  ('box_identifier') character, identifier for box, default='z'
;   m1:      ('m_number_1') integer, 1. dimension, start index
;   n1:      ('number_1')   integer, 1. dimension, end index
;   m2:      ('m_number_2') integer, 2. dimension, start index
;   n2:      ('number_2')   integer, 2. dimension, end index
;   m3:      ('m_number_3') integer, 3. dimension, start index
;   n3:      ('number_3')   integer, 3. dimension, end index
;   itime:   ('index_time') integer, time step number
;   time:    ('time') real, time
;   dtime:   ('delta_time') real, time step
;   time_out_full_last: ('time_output_full_last') real, time of last output of full model
;   time_out_mean_last: ('time_output_mean_last') real, time of last output of averaged data
;   xc1:     ('x_center_1') real, 1. coordinate of cell centers
;   xc2:     ('x_center_2') real, 2. coordinate of cell centers
;   xc3:     ('x_center_3') real, 3. coordinate of cell centers
;   xb1:     ('x_boundary_1') real, 1. coordinate of cell boundaries
;   xb2:     ('x_boundary_2') real, 2. coordinate of cell boundaries
;   xb3:     ('x_boundary_3') real, 3. coordinate of cell boundaries
;   rho:     ('rho') real, dimension(:,:,:), density
;   ei:      ('energy_internal') real, dimension(:,:,:), internal energy density per mass
;   v1:      ('v_1') real, dimension(:,:,:), velocity, 1. coordinate
;   v2:      ('v_2') real, dimension(:,:,:), velocity, 2. coordinate
;   v3:      ('v_3') real, dimension(:,:,:), velocity, 3. coordinate
;   quc:     ('quantity_center')  real, dimension(:,:,:,:), list of centered variables
;   Bc1:     ('B_center_1') real, dimension(:,:,:), cell centered magnetic field, 1. component
;   Bc2:     ('B_center_2') real, dimension(:,:,:), cell centered magnetic field, 2. component
;   Bc3:     ('B_center_3') real, dimension(:,:,:), cell centered magnetic field, 3. component
;   Bb1:     ('B_boundary_1') real, dimension(:,:,:), cell boundary magnetic field, 1. component
;   Bb2:     ('B_boundary_2') real, dimension(:,:,:), cell boundary magnetic field, 2. component
;   Bb3:     ('B_boundary_3') real, dimension(:,:,:), cell boundary magnetic field, 3. component
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   Write data into file.
;
; RESTRICTIONS:
;   uio routines have to be initialized.
;
; PROCEDURE:
;   The behaviour of the routines depends on the contents of the command string.
;   It may contain one or more of the following substrings:
;   'open':
;     The requested file is opened (in uio format). The file parameter has to
;     be specified. The form and conv parameter may are optional.
;     The file_id and - if specified - the file description and history are
;     written into the file.
;     In the case of the 'open' command a new file is created (old data is overwritten).
;   'model':
;     Dataset is opened. General model data is written into the file:
;     dataset label, modelboxtype, dimension, time, time step number.
;   'box':
;     The box data are written into the file with the unit ncout.
;     In this case all box data have to be specified.
;   'endmo':
;     Dataset is closed: enddataset label.
;   'close':
;     The file is closed again.
;   One or more models or boxes per model may be written into the file by
;   repeated calls of the routine.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    1. 7.97 Written by Bernd Freytag, Astrophysik Kiel
;    2.10.97 (B.F.) Energy: eik
;   15.10.97 (B.F.) Parameters dtime and out_time_full_last added
;   25. 3.99 (B.F. Copenhagen) px -> vx, eik -> ei, new labels
;    9. 4.99 (B.F.) Command 'endmo' added
;   24. 2.00 (B.F.) Parameter out_time_mean_last added
;    4.11.02 (B.F. Uppsala) Parameters unit_quc, name_quc, quc,
;             unit_B, Bb1, Bb2, Bb3 added
;   2007-10-23 (B.F. Lyon) Write double precision time (time_db)
;   2008-07-29 (B.F.) Parameter double_flag added
;   2010-03-15 (B.F.) Shift indices for magnetic fields added
;-
;------------------------------------------------------------------------------
@uio_filedefinc
;
if ((strpos(' '+command,'open') gt 0) or (strpos(' '+command,'append') gt 0)) then begin
  ;
  if (strpos(' '+command,'append') gt 0) then begin
    ; --- Open file for writing in uio-form, append data ---
    uio_openwr, ncout, file, outstr,ierr, prog='rhd_wrboxdata', $
      form=form, conv=conv, position='append'
  endif else begin
    ; --- Open file for writing in uio-form ---
    uio_openwr, ncout, file, outstr,ierr, prog='rhd_wrboxdata', $
      form=form, conv=conv
  endelse
  ;
  printf, nc_p, format='(A,I3)', 'Model file ''' + strcompress(file,/rem) + $
                                 ''' opened on channel ', ncout
  ;
  ; --- Write heading information ---
  ;
  ; --- Write file_id ---
  uio_wr, ncout, 'rhd-data', file_id_ident, name='File identification'
  ;
  if (keyword_set(version)) then begin
    uio_wr, ncout, version, file_version_ident, name='Version'
  endif
  ;
  if (keyword_set(description) and keyword_set(ndes)) then begin
    ; --- Write file description ---
    uio_wr, ncout, description(0:ndes-1), file_description_ident, name='File description'
  endif
  ;
  if (keyword_set(history) and keyword_set(nhis)) then begin
    ; --- Write file history ---
    uio_wr, ncout, history(0:nhis-1), file_history_ident, name='File history'
  endif
endif
;
if (strpos(' '+command,'model') gt 0) then begin
  ;
  ; --- Write general model data ---
  ;
  ; --- Model label ---
  uio_wrlabl, ncout, dataset_ident, date='now'
  ;
  ; --- block_id ---
  uio_wr, ncout, 'single_box', dataset_id_ident, name='Type of box hierarchy'
  ;
  ; --- Time ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(time), 'modeltime', name='time', unit=unit_time
  endif else begin
    uio_wr, ncout, time, 'modeltime', name='time', unit=unit_time
  endelse
  uio_wr, ncout, double(time), 'modeltime_db', name='time', unit=unit_time
  ;
  ; --- Time step number ---
  uio_wr, ncout, itime, 'modelitime', name='time step number', unit='1'
  ;
  ; --- Time step ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(dtime), 'dtime', name='time step', unit=unit_time
  endif else begin
    uio_wr, ncout, dtime, 'dtime', name='time step', unit=unit_time
  endelse
  ;
  ; --- Time of last output of full model ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(out_time_full_last), 'time_out_full_last', $
            name='Time of last output of full model', unit=unit_time
  endif else begin
    uio_wr, ncout, out_time_full_last, 'time_out_full_last', $
            name='Time of last output of full model', unit=unit_time
  endelse
  ;
  ; --- Time of last output of mean data ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(out_time_mean_last), 'time_out_mean_last', $
            name='Time of last output of mean data', unit=unit_time
  endif else begin
    uio_wr, ncout, out_time_mean_last, 'time_out_mean_last', $
            name='Time of last output of mean data', unit=unit_time
  endelse
  ;
endif
;

; --- Write additional data (boundary conditions ...) ---

;
if (strpos(' '+command,'box') gt 0) then begin
  ;
  ; --- Write bulk model data (from box) ---
  ;
  ; --- Start block label ---
  uio_wrlabl, ncout, box_ident, date='now'
  ;
  ; --- block_id ---
  uio_wr, ncout, 'z', box_id_ident, name='Box identification'
  ;
  ; --- Block dimension ---
  uio_wr, ncout, [[m1,n1],[m2,n2],[m3,n3]], 'dimension', format='I6'
  ;
  ; --- Scalars ---
  ;
  ; --- Time ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(time), 'time', name='time', unit=unit_time
  endif else begin
    uio_wr, ncout, time, 'time', name='time', unit=unit_time
  endelse
  uio_wr, ncout, double(time), 'time_db', name='time', unit=unit_time
  ;
  ; --- Time step number ---
  uio_wr, ncout, itime, 'itime', name='time step number', unit='1'
  ;
  ; --- Arrays: ---
  ;
  ; --- Grid: 1D vectors: spread -> 3D ---
  uio_dim2st, [[0,0], [0,1], [0,1]], shistr
  ntt=0
  uio_adkey2, termt,ntt,'ds',shistr
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, reform(double(xc1),n1-m1+1,1,1), 'xc1', $
            iarr=[m1,m2,m3], $
            name='x1 coordinates of cell centers', unit=unit_x1, $
            ntt=ntt, termt=termt
  endif else begin
    uio_wr, ncout, reform(xc1,n1-m1+1,1,1), 'xc1', $
            iarr=[m1,m2,m3], $
            name='x1 coordinates of cell centers', unit=unit_x1, $
            ntt=ntt, termt=termt
  endelse
  uio_dim2st, [[0,1], [0,0], [0,1]], shistr
  ntt=0
  uio_adkey2, termt,ntt,'ds',shistr
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, reform(double(xc2),1,n2-m2+1,1), 'xc2', $
            iarr=[m1,m2,m3], $
            name='x2 coordinates of cell centers', unit=unit_x2, $
            ntt=ntt, termt=termt
  endif else begin
    uio_wr, ncout, reform(xc2,1,n2-m2+1,1), 'xc2', $
            iarr=[m1,m2,m3], $
            name='x2 coordinates of cell centers', unit=unit_x2, $
            ntt=ntt, termt=termt
  endelse
  uio_dim2st, [[0,1], [0,1], [0,0]], shistr
  ntt=0
  uio_adkey2, termt,ntt,'ds',shistr
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, reform(double(xc3),1,1,n3-m3+1), 'xc3', $
            iarr=[m1,m2,m3], $
            name='x3 coordinates of cell centers', unit=unit_x3, $
            ntt=ntt, termt=termt
  endif else begin
    uio_wr, ncout, reform(xc3,1,1,n3-m3+1), 'xc3', $
            iarr=[m1,m2,m3], $
            name='x3 coordinates of cell centers', unit=unit_x3, $
            ntt=ntt, termt=termt
  endelse
  uio_dim2st, [[0,1], [0,1], [0,1]], shistr
  ntt=0
  uio_adkey2, termt,ntt,'ds',shistr
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, reform(double(xb1),n1-m1+2,1,1), 'xb1', $
            iarr=[m1,m2,m3], $
            name='x1 coordinates of cell boundaries', unit=unit_x1, $
            ntt=ntt, termt=termt
    uio_wr, ncout, reform(double(xb2),1,n2-m2+2,1), 'xb2', $
            iarr=[m1,m2,m3], $
            name='x2 coordinates of cell boundaries', unit=unit_x2, $
            ntt=ntt, termt=termt
    uio_wr, ncout, reform(double(xb3),1,1,n3-m3+2), 'xb3', $
            iarr=[m1,m2,m3], $
            name='x3 coordinates of cell boundaries', unit=unit_x3, $
            ntt=ntt, termt=termt
  endif else begin
    uio_wr, ncout, reform(xb1,n1-m1+2,1,1), 'xb1', $
            iarr=[m1,m2,m3], $
            name='x1 coordinates of cell boundaries', unit=unit_x1, $
            ntt=ntt, termt=termt
    uio_wr, ncout, reform(xb2,1,n2-m2+2,1), 'xb2', $
            iarr=[m1,m2,m3], $
            name='x2 coordinates of cell boundaries', unit=unit_x2, $
            ntt=ntt, termt=termt
    uio_wr, ncout, reform(xb3,1,1,n3-m3+2), 'xb3', $
            iarr=[m1,m2,m3], $
            name='x3 coordinates of cell boundaries', unit=unit_x3, $
            ntt=ntt, termt=termt
  endelse
  ;
  ; --- Volume data ---
  ;
  ; --- Density ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(rho), 'rho', $
            iarr=[m1,m2,m3], $
            name='Density', unit=unit_rho
  endif else begin
    uio_wr, ncout, rho, 'rho', $
            iarr=[m1,m2,m3], $
            name='Density', unit=unit_rho
  endelse
  ;
  ; --- Energy ---
  if (keyword_set(double_flag)) then begin
    uio_wr, ncout, double(ei), 'ei', $
            iarr=[m1,m2,m3], $
            name='Internal energy', unit=unit_e
  endif else begin
    uio_wr, ncout, ei, 'ei', $
            iarr=[m1,m2,m3], $
            name='Internal energy', unit=unit_e
  endelse
  ;
  ; --- Velocity ---
  if (keyword_set(v1)) then begin
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(v1), 'v1', $
              iarr=[m1,m2,m3], $
              name='Velocity 1', unit=unit_v
    endif else begin
      uio_wr, ncout, v1, 'v1', $
              iarr=[m1,m2,m3], $
              name='Velocity 1', unit=unit_v
    endelse
  endif
  if (keyword_set(v2)) then begin
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(v2), 'v2', $
              iarr=[m1,m2,m3], $
              name='Velocity 2', unit=unit_v
    endif else begin
      uio_wr, ncout, v2, 'v2', $
              iarr=[m1,m2,m3], $
              name='Velocity 2', unit=unit_v
    endelse
  endif
  if (keyword_set(v3)) then begin
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(v3), 'v3', $
              iarr=[m1,m2,m3], $
              name='Velocity 3', unit=unit_v
    endif else begin
      uio_wr, ncout, v3, 'v3', $
              iarr=[m1,m2,m3], $
              name='Velocity 3', unit=unit_v
    endelse
  endif
  ;
  ; --- Additional quantities (quc001, quc002, ...) ---
  if (keyword_set(quc)) then begin
    nquc=n_elements(quc(0,0,0,*))
    if (not keyword_set(unit_quc)) then begin
      unit_quc=strarr(nquc)
    endif
    ;
    for iquc=1,nquc do begin
      ident='00' + strtrim(string(iquc, format='(I3)'),2)
      ident='quc' + strmid(ident,strlen(ident)-3,3)
      if (keyword_set(name_quc)) then begin
        name=name_quc(iquc-1)
      endif else begin
        name='Additional quantity ' + ident
      endelse
      if (keyword_set(double_flag)) then begin
        uio_wr, ncout, reform(double(quc[*,*,*,iquc-1]), n1-m1+1, n2-m2+1, n3-m3+1), ident, $
                iarr=[m1,m2,m3], $
                name=name, unit=unit_quc(iquc-1)
      endif else begin
        uio_wr, ncout, reform(quc[*,*,*,iquc-1], n1-m1+1, n2-m2+1, n3-m3+1), ident, $
                iarr=[m1,m2,m3], $
                name=name, unit=unit_quc(iquc-1)
      endelse
    endfor
  endif
  ;
  ; --- Magnetic field ---
  if (keyword_set(Bb1)) then begin
    uio_dim2st, [[0,1], [0,0], [0,0]], shistr
    ntt=0
    uio_adkey2, termt,ntt,'ds',shistr
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(Bb1), 'bb1', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 1', unit=unit_B, $
              ntt=ntt, termt=termt
    endif else begin
      uio_wr, ncout, Bb1, 'bb1', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 1', unit=unit_B, $
              ntt=ntt, termt=termt
    endelse
  endif
  if (keyword_set(Bb2)) then begin
    uio_dim2st, [[0,0], [0,1], [0,0]], shistr
    ntt=0
    uio_adkey2, termt,ntt,'ds',shistr
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(Bb2), 'bb2', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 2', unit=unit_B, $
              ntt=ntt, termt=termt
    endif else begin
      uio_wr, ncout, Bb2, 'bb2', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 2', unit=unit_B, $
              ntt=ntt, termt=termt
    endelse
  endif
  if (keyword_set(Bb3)) then begin
    uio_dim2st, [[0,0], [0,0], [0,1]], shistr
    ntt=0
    uio_adkey2, termt,ntt,'ds',shistr
    if (keyword_set(double_flag)) then begin
      uio_wr, ncout, double(Bb3), 'bb3', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 3', unit=unit_B, $
              ntt=ntt, termt=termt
    endif else begin
      uio_wr, ncout, Bb3, 'bb3', $
              iarr=[m1,m2,m3], $
              name='Magnetic field 3', unit=unit_B, $
              ntt=ntt, termt=termt
    endelse
  endif
  ;
  ; --- End box label ---
  uio_wrlabl, ncout, endbox_ident
endif
;
if (strpos(' '+command,'endmo') gt 0) then begin
  ; --- End dataset label ---
  uio_wrlabl, ncout, enddataset_ident, date='now'
endif
;
if (strpos(' '+command,'close') gt 0) then begin
  ;
  ; --- Close file ---
  uio_closwr, ncout
endif
;
end ; rhd_wrboxdata
