pro tabinter_rdcoeff, file, eos
;
;------------------------------------------------------------------------------
;+
; NAME:
;   tabinter_rdcoeff ('table_interpolation_read_coefficients')
;
; PURPOSE:
;   Read coefficients for bicubic interpolation of EOS quantities from file.
;
; CATEGORY:
;   EOS, Input/Output
;
; CALLING SEQUENCE:
;   
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   21. 4.99 Written by Bernd Freytag, Astrophysik Kiel
;   23.10.01 (B.F. Uppsala) Dimensions of eos.box.c changed
;   14.11.03 (B.F.) C_shift: intarr -> fltarr
;                   x1_offset, x2_offset, C_offset added
;   2014-08-29 (B.F. Uppsala) square brackets, single/double precision arrays
;-
;------------------------------------------------------------------------------
;
; --- Read file ---
tab=uio_struct_rd(file)
;
head=create_struct( $
      'file_id',      tab.file_id, $
      'description',  tab.description, $
      'history',      tab.history, $
      'file',         file, $
      'variables',    tab.variables, $
      'interpolation',tab.interpolation, $
      'source',       tab.source, $
      'abux',         tab.abux, $
      'abuy',         tab.abuy, $
      'abuz',         tab.abuz, $
      'qmol',         tab.qmol, $
      'gamma',        tab.gamma, $
      'ifion',        tab.ifion)
;
lnx11d=reform(alog(tab.x1+tab.x1shift))
nx1   =tab.dimension[1,1]-tab.dimension[0,1]+1
lnx21d=reform(alog(tab.x2+tab.x2shift))
nx2   =tab.dimension[1,2]-tab.dimension[0,2]+1
nc    =tab.dimension[1,0]-tab.dimension[0,0]+1
ny    =tab.number_variables
CoeffType=size(tab.x1shift, /type)
box=create_struct( $
      'dimension', tab.dimension, $
      'nx',        2L, $
      'ny',        ny, $
      'nc',        nc, $
      'nx1',       nx1, $
      'nx2',       nx2, $
      'x1min',     tab.x1[0,0,0] , $
      'x1max',     tab.x1[0,nx1,0], $
      'lnx1off',   lnx11d[0], $
      'lnx1fac',   float(nx1)/(lnx11d[nx1]-lnx11d[0]), $
      'x1_shift',  tab.x1shift, $
      'x1_offset', 0.0*tab.x1shift, $
      'lnx11d',    lnx11d, $
      'x2min',     tab.x2[0,0,0] , $
      'x2max',     tab.x2[0,0,nx2], $
      'lnx2off',   lnx21d[0], $
      'lnx2fac',   float(nx2)/(lnx21d[nx2]-lnx21d[0]), $
      'x2_shift',  tab.x2shift, $
      'x2_offset', 0.0*tab.x1shift, $
      'lnx21d',    lnx21d, $
      'c_shift',   make_array(tab.number_variables, type=CoeffType), $
      'c_offset',  make_array(tab.number_variables, type=CoeffType), $
      'c',         make_array(nc,nx1,nx2,ny,        type=CoeffType))
;      'c',         fltarr(nc,nx1,nx2,ny))
;      'c',         fltarr(nc,ny,nx1,nx2))
;
box.c_shift=[tab.c1shift, tab.c2shift, tab.c3shift]
box.c[*,*,*,0]=tab.c1
box.c[*,*,*,1]=tab.c2
box.c[*,*,*,2]=tab.c3
tags=tag_names(tab)
if (total(tags eq 'X1OFFSET') gt 0.0) then begin
  box.x1_offset=tab.x1offset
endif
if (total(tags eq 'X2OFFSET') gt 0.0) then begin
  box.x2_offset=tab.x2offset
endif
if (total(tags eq 'C1OFFSET') gt 0.0) then begin
  box.c_offset=[tab.c1offset, tab.c2offset, tab.c3offset]
endif
;
eos=create_struct('head', head, 'box', box)
;
end ; tabinter_rdcoeff
