function xbnu, t, iband, grid = grid, clip=clip
;------------------------------------------------------------------------------
;+
; NAME:
;   xbnu ('x_B_nu')
;
; PURPOSE:
;   Interpolate band fractions from data in opta_common
;
; CATEGORY:
;   Opacities, Interpolation
;
; CALLING SEQUENCE:
;   result=xbnu(t, iband, grid = grid, clip=clip)
;
; INPUTS:
;   T:     ('Temperature') real array with temperature data
;   iband: ('index_band') integer (always 0)
;
; OUTPUTS:
;   result: Band fractions
;
; KEYWORDS:
;   grid:  ('grid')
;   clip:  ('clip') if not set in case of extrapolation the outermost values are used (0 or n-1)
;                   if set the values 1 gridpoint away from the boundary are used (1 or n-2)
;
; COMMON BLOCKS:
;   opta_common
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   First the index and then weights are interpolated.
;   Not identical to what is done in opta_routines.F90
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;            Written by Hans-Guenter Ludwig
;   30.06.99 (B.F. Copenhagen) keyword clip added
;   11.10.99 (B.F.) Return data in same format as T
;   30.10.09 (B.F. Lyon) Derived from xkaros
;-
;------------------------------------------------------------------------------
;

common opta_common

; Determine index coordinates
it = interpol( indgen(nt), topta, alog10(t))

; Delimit coordinates
;it = max([min([it, float(nt-1)]),0.0])
if (keyword_set(clip)) then begin
  it = 1.0 > it < float(nt-2)
endif

xbn=T

if not keyword_set(grid) then begin
  xbn[*]=interpolate( wtbbd[*,iband], it)
endif else begin
  xbn[*]=interpolate( wtbbd[*,iband], it, /grid)
endelse

return, xbn

end ; xbnu
