pro bubble1d, uarr
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble1d ('bubble_1_dimension')
;
; PURPOSE:
;   Solve "bubble equation" in 1D
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   bubble1d, uarr
;
; INPUTS:
;
; OUTPUTS:
;   uarr:   ('u_array') 2D real array with solution u(x1,time)
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
n1=400
nt=7200
;
C0=0.2
C2=1.0
C4=1.0
dtdx=0.1
;
v=0.5*(-cos(findgen(n1)/float(n1)*2.0*!pi)+1.0)
;
u=fltarr(n1)
uarr=fltarr(n1,nt)
;
u=randomu(seed,n1)
uarr(*,0)=u
;
for i=1,nt-1 do begin
  dudxp  =shift(u     ,-1)-u
  dudxm  =u               -shift(u     , 1)
  dudx   =0.5*(dudxp+dudxm)
  d2udx2 =dudxp           -dudxm
  d3udx3p=shift(d2udx2,-1)-d2udx2
  d3udx3m=d2udx2          -shift(d2udx2, 1)
  d4udx4 =d3udx3p         -d3udx3m
  ;
  u=u - dtdx * (v*dudx + C4*d4udx4 + C2*d2udx2 + C0*u^3)
  ;
  uarr(*,i)=u
endfor
;
tvwin, rebin(uarr,n1,nt/8), /sample, fa=1
;
end ; bubble1d
