pro bubble2d_start, uarr, v1=v1, v2=v2
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble2d ('bubble_2_dimension')
;
; PURPOSE:
;   Solve "bubble equation" in 1D
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   bubble2d, uarr
;
; INPUTS:
;
; OUTPUTS:
;   uarr:   ('u_array') 3D real array with solution u(x1,x2,time)
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
nt=501
;
if ((n_elements(v1) ne 0) and (n_elements(v2) ne 0)) then begin
  n1=n_elements(v1(*,0))
  n2=n_elements(v2(*,0))
endif else begin
  n1=127
  n2=127
  ;
  ;v=0.5*(-cos(findgen(n1)/float(n1)*2.0*!pi)+1.0)
  ;v1=1.0 & v2=0.1
  ;v1=0.0 & v2=0.0
  x1=rebin(reform(findgen(n1)/float(n1),n1, 1),n1,n2)
  x2=rebin(reform(findgen(n2)/float(n2), 1,n2),n1,n2)
  ;v1=sin( 2.0*!pi*x2)*sin(!pi*x1)
  ;v2=sin(-2.0*!pi*x1)*sin(!pi*x2)
  v1=(sin( 2.0*!pi*x2)+0.5*sin( 2.0*!pi*x1)) * sin(!pi*x1)
  v2=(sin(-2.0*!pi*x1)+0.5*sin( 2.0*!pi*x2)) * sin(!pi*x2)
endelse
;
u=fltarr(n1,n2)
uarr=fltarr(n1,n2,nt)
;
; --- Initialize bubble array ---
u=randomu(seed,n1,n2)
for i=1,100 do begin
  bubble2d, u
endfor
;
uarr(*,*,0)=u
;
for i=1,nt-1 do begin
  if (i mod 250) eq 0 then print, i
  ;
  bubble2d, u, v1=v1, v2=v2
  ;
  uarr(*,*,i)=u
endfor
;
tvwin, reform(uarr(*,*,nt-1)), sample=0, fa=3
;
end ; bubble2d_start
