pro bubble3d_init, n1, n2, n3, u
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bubble3d_init ('bubble_3_dimension_init')
;
; PURPOSE:
;   Prepare initial bubble configuration for "bubble equation" in 3D.
;
; CATEGORY:
;   Hydrodynamics, Plotting.
;
; CALLING SEQUENCE:
;   bubble3d_init, n1, n2, n3, u
;
; INPUT:
;   n1:     ('number_1') integer with number of x1 axis values
;   n2:     ('number_2') integer with number of x2 axis values
;   n3:     ('number_3') integer with number of x3 axis values
;
; OUTPUT:
;   u:      ('u') 3D real array with bubble amplitude
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   bubble3d_operator:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   26. 3.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Parameters ---
;n_iterinit=100
n_iterinit=40
;
; --- Initial random amplitude ---
;u=randomu(seed, n1, n2, n3)
u=3.6*randomu(seed, n1, n2, n3) - 0.3
u=smooth(u,3)
;
; --- Iterate bubble equation to give a relaxed state ---
for i=1,n_iterinit do begin
  bubble3d_operator, u
print,i, minmax(u)
endfor ; i
;
end ; bubble3d_init
