pro modctv, dev
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   modctv ('modify_color_table_variable')
;
; PURPOSE:
;   Modify the color-table variable ctv when switching from one device to another.
;
; CATEGORY:
;   Plotting
;
; CALLING SEQUENCE:
;   modctv
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-07-11 (B.F.) Written, Astronomical Observatory Uppsala
;   2016-04-25 (B.F.) Experiment with darker shades of some colors
;-
;---------------------------------------------------------------------------------------------------
;
if ((strlowcase(dev) eq 'ps') or (strlowcase(dev) eq 'eps')) then begin
  ncol=fix(!D.TABLE_SIZE)
  if (ncol le 17) then c=0 else c=1
  ra=[0,!D.TABLE_SIZE-1]
  ;
  !ctv.black  =0
  !ctv.gray   =ncol-1-8*c
  !ctv.white  =ncol-1
  !ctv.magenta=ncol-1-6*c
  !ctv.red    =ncol-1-5*c
  !ctv.orange =ncol-1-4*c
  !ctv.yellow =ncol-1-3*c
  !ctv.green  =ncol-1-2*c
  !ctv.cyan   =ncol-1-7*c
  !ctv.blue   =ncol-1-1*c
  ;
  tvlct, red, green, blue, /GET                          ; Load color table
  ;
  red(  !ctv.gray)=128                                       ; Gray
  green(!ctv.gray)=128 
  blue( !ctv.gray)=128 
  ;
  red(  !ctv.red)=255                                        ; Red
  green(!ctv.red)=0 
  blue( !ctv.red)=0 
  ;
  red(  !ctv.orange)=255                                     ; Orange
  green(!ctv.orange)=140 
  blue( !ctv.orange)=0 
  ;
;  red(  !ctv.yellow)=255                                     ; Yellow
;  green(!ctv.yellow)=255 
;  blue( !ctv.yellow)=0 
  ;
;  red(  !ctv.green)=0                                        ; Green
;  green(!ctv.green)=255 
;  blue( !ctv.green)=0 
  ;
;  red(  !ctv.cyan)=0                                         ; Cyan
;  green(!ctv.cyan)=255 
;  blue( !ctv.cyan)=255 
  ;
  red(  !ctv.yellow)=230                                     ; Yellow
  green(!ctv.yellow)=220 
  blue( !ctv.yellow)=0 
  ;
  red(  !ctv.green)=0                                        ; Green
  green(!ctv.green)=230 
  blue( !ctv.green)=0 
  ;
  red(  !ctv.cyan)=0                                         ; Cyan
  green(!ctv.cyan)=230 
  blue( !ctv.cyan)=230 
  ;
  red(  !ctv.blue)=0                                         ; Blue
  green(!ctv.blue)=0 
  blue( !ctv.blue)=255 
  ;
  red(  !ctv.magenta)=255                                    ; Magenta
  green(!ctv.magenta)=0 
  blue( !ctv.magenta)=255 
  ;
  tvlct, red, green, blue                                  ; Put color table
  ;
endif else begin
  ;
  !ctv.black  =   0 + 256L * (  0 + 256L *   0)
  !ctv.gray   = 128 + 256L * (128 + 256L * 128)
  !ctv.white  = 255 + 256L * (255 + 256L * 255)
  !ctv.magenta= 255 + 256L * (  0 + 256L * 255)
  !ctv.red    = 255 + 256L * (  0 + 256L *   0)
  !ctv.orange = 255 + 256L * (140 + 256L *   0)
; !ctv.yellow = 255 + 256L * (255 + 256L *   0)
; !ctv.green  =   0 + 256L * (255 + 256L *   0)
; !ctv.cyan   =   0 + 256L * (255 + 256L * 255)
  !ctv.yellow = 230 + 256L * (220 + 256L *   0)
  !ctv.green  =   0 + 256L * (230 + 256L *   0)
  !ctv.cyan   =   0 + 256L * (230 + 256L * 230)
  !ctv.blue   =   0 + 256L * (  0 + 256L * 255)
  ;
endelse
;
end ; modctv
