pro setctv
;
;+
; NAME:
;   SETCTV (Set color table variable)
;
; PURPOSE:
;   Define a new system variable which contains information about the
;   currently loaded color tables
;
; CALING SEQUENCE:
;   setctv
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORD PARAMETERS:
;   None
;
; SIDE EFFECTS:
;   Color tables are changed. A new system variable '!CTV' is created.
;
; SUBROUTINES:
;   normct: Load and normalize color table into the specified range
;
; RESTRICTIONS:
;   The system variable !D.TABLE_SIZE has to defined. Therefore at least one
;   window must have been opened before calling setctv.
;
; EXAMPLE:
;   setctv
;   help, /struct, !CTV
;   plot, [1,2,3,2,4,5,2], color=!CTV.BLUE
;
; MODIFICATION HISTORY:
;   26.8.93 Written by Bernd Freytag, Astrophysik Kiel
;    6.12.93 Modification for True Color terminal
;    7.12.93
;   13.12.93 Standard color table -2
;   14.12.93 COLORRANGE0 added
;    3. 3.94 Standard colortables (!ctv.ctXX) for true color terminal
;            added
;    5. 4.95 (B.F.) no window opened at start 
;    3.12.96 (B.F.) For black/white terminal set all colors except black to whi.
;   29. 9.97 (B.F.) Reduce maximum line length for IDL5 
;   12.11.97 (B.F.) ct10 - ct29 removed
;    9.10.98 (B.F.) ct10 - ct29 reintroduced, batch file written as procedure
;-
;
;window, /free, xsize=100, ysize=10, /pixmap
;wdelete, !D.WINDOW
;
if (!D.N_COLORS gt 256) then true=3 else true=0
;
if (true eq 3) then begin
  print, 'True color terminal'
  ; --- Determine number of available colors ---
  ncol=fix(!D.TABLE_SIZE)
  print, '!D.TABLE_SIZE=', ncol
  ;
;  loadct,6
  normct,-2,/nonorm
  tvlct, /get, redprism, greenprism, blueprism
  color1=bytarr(1024,3)
  color1(*,0)=rebin(redprism,1024)
  color1(*,1)=rebin(greenprism,1024)
  color1(*,2)=rebin(blueprism,1024)
  ;
  loadct,0
  tvlct, /get, redgray, greengray, bluegray
  color2=bytarr(256,3)
  color2(*,0)=redgray
  color2(*,1)=greengray
  color2(*,2)=bluegray
  ;
  ; --- Define structure for new system variable ---
  ctv1={ctvtyp, colorrange0:color2, $
                colorrange1:color1, $
                colorrange2:color2, $
                ct0:color2, $
                ct1:color2, $
                ct2:color2, $
                ct3:color2, $
                ct4:color2, $
                ct5:color2, $
                ct6:color2, $
                ct7:color2, $
                ct8:color2, $
                ct9:color2, $
                ct10:color2, $
                ct11:color2, $
                ct12:color2, $
                ct13:color2, $
                ct14:color2, $
                ct15:color2, $
                ct16:color2, $
                ct17:color2, $
                ct18:color2, $
                ct19:color2, $
                ct20:color2, $
                ct21:color2, $
                ct22:color2, $
                ct23:color2, $
                ct24:color2, $
                ct25:color2, $
                ct26:color2, $
                ct27:color2, $
                ct28:color2, $
                ct29:color2, $
                black :    0 + 256L * (  0 + 256L *   0), $
                gray:    128 + 256L * (128 + 256L * 128), $
                white:   255 + 256L * (255 + 256L * 255), $
                magenta: 255 + 256L * (  0 + 256L * 255), $
                red:     255 + 256L * (  0 + 256L *   0), $
                orange:  255 + 256L * (140 + 256L *   0), $
                yellow:  255 + 256L * (255 + 256L *   0), $
                green:     0 + 256L * (255 + 256L *   0), $
                cyan:      0 + 256L * (255 + 256L * 255), $
                blue  :    0 + 256L * (  0 + 256L * 255)   }
  ; --- Define new system variable ---
  defsysv, '!ctv', ctv1
endif
;
if (true eq 0) then begin
  ; --- Determine number of available colors ---
  ncol=fix(!D.TABLE_SIZE)
  print, '!D.TABLE_SIZE=', ncol
  if (ncol le 17) then c=0 else c=1
  ra=[0,!D.TABLE_SIZE-1]
  ;
  ; --- Define structure for new system variable ---
  ctv1={ctvtyp, colorrange0:[1            , ncol-10   ], $
                colorrange1:[1            ,(ncol-10)/2], $
                colorrange2:[(ncol-10)/2+1, ncol-10   ], $
                ct0:ra, ct1:ra, ct2:ra, ct3:ra, ct4:ra, $
                ct5:ra, ct6:ra, ct7:ra, ct8:ra, ct9:ra, $
                ct10:ra, ct11:ra, ct12:ra, ct13:ra, ct14:ra, $
                ct15:ra, ct16:ra, ct17:ra, ct18:ra, ct19:ra, $
                ct20:ra, ct21:ra, ct22:ra, ct23:ra, ct24:ra, $
                ct25:ra, ct26:ra, ct27:ra, ct28:ra, ct29:ra, $
                black :0,  gray:ncol-1-8*c,   white:ncol-1,             $
                magenta:ncol-1-6*c, red :ncol-1-5*c, orange:ncol-1-4*c, $
                yellow:ncol-1-3*c, green:ncol-1-2*c,  cyan:ncol-1-7*c, $
                blue  :ncol-1-1*c}
  ; --- Define new system variable ---
  defsysv, '!ctv', ctv1
endif
;
if (true eq 3) then begin
  ;
  loadct,0
  tvlct, /get, red, green, blue
  !ctv.ct0(*,0)=red
  !ctv.ct0(*,1)=green
  !ctv.ct0(*,2)=blue
  ;
  loadct,1
  tvlct, /get, red, green, blue
  !ctv.ct1(*,0)=red
  !ctv.ct1(*,1)=green
  !ctv.ct1(*,2)=blue
  ;
  loadct,2
  tvlct, /get, red, green, blue
  !ctv.ct2(*,0)=red
  !ctv.ct2(*,1)=green
  !ctv.ct2(*,2)=blue
  ;
  loadct,3
  tvlct, /get, red, green, blue
  !ctv.ct3(*,0)=red
  !ctv.ct3(*,1)=green
  !ctv.ct3(*,2)=blue
  ;
  loadct,4
  tvlct, /get, red, green, blue
  !ctv.ct4(*,0)=red
  !ctv.ct4(*,1)=green
  !ctv.ct4(*,2)=blue
endif
if (true eq 3) then begin
  loadct,5
  tvlct, /get, red, green, blue
  !ctv.ct5(*,0)=red
  !ctv.ct5(*,1)=green
  !ctv.ct5(*,2)=blue
  ;
  loadct,6
  tvlct, /get, red, green, blue
  !ctv.ct6(*,0)=red
  !ctv.ct6(*,1)=green
  !ctv.ct6(*,2)=blue
  ;
  loadct,7
  tvlct, /get, red, green, blue
  !ctv.ct7(*,0)=red
  !ctv.ct7(*,1)=green
  !ctv.ct7(*,2)=blue
  ;
  loadct,8
  tvlct, /get, red, green, blue
  !ctv.ct8(*,0)=red
  !ctv.ct8(*,1)=green
  !ctv.ct8(*,2)=blue
  ;
  loadct,9
  tvlct, /get, red, green, blue
  !ctv.ct9(*,0)=red
  !ctv.ct9(*,1)=green
  !ctv.ct9(*,2)=blue
  ;
  loadct,0
endif
if (true eq 3) then begin
  ;
  loadct,10
  tvlct, /get, red, green, blue
  !ctv.ct10(*,0)=red
  !ctv.ct10(*,1)=green
  !ctv.ct10(*,2)=blue
  ;
  loadct,11
  tvlct, /get, red, green, blue
  !ctv.ct11(*,0)=red
  !ctv.ct11(*,1)=green
  !ctv.ct11(*,2)=blue
  ;
  loadct,12
  tvlct, /get, red, green, blue
  !ctv.ct12(*,0)=red
  !ctv.ct12(*,1)=green
  !ctv.ct12(*,2)=blue
  ;
  loadct,13
  tvlct, /get, red, green, blue
  !ctv.ct13(*,0)=red
  !ctv.ct13(*,1)=green
  !ctv.ct13(*,2)=blue
  ;
  loadct,14
  tvlct, /get, red, green, blue
  !ctv.ct14(*,0)=red
  !ctv.ct14(*,1)=green
  !ctv.ct14(*,2)=blue
endif
if (true eq 3) then begin
  loadct,15
  tvlct, /get, red, green, blue
  !ctv.ct15(*,0)=red
  !ctv.ct15(*,1)=green
  !ctv.ct15(*,2)=blue
  ;
  loadct,16
  tvlct, /get, red, green, blue
  !ctv.ct16(*,0)=red
  !ctv.ct16(*,1)=green
  !ctv.ct16(*,2)=blue
  ;
  loadct,17
  tvlct, /get, red, green, blue
  !ctv.ct17(*,0)=red
  !ctv.ct17(*,1)=green
  !ctv.ct17(*,2)=blue
  ;
  loadct,18
  tvlct, /get, red, green, blue
  !ctv.ct18(*,0)=red
  !ctv.ct18(*,1)=green
  !ctv.ct18(*,2)=blue
  ;
  loadct,19
  tvlct, /get, red, green, blue
  !ctv.ct19(*,0)=red
  !ctv.ct19(*,1)=green
  !ctv.ct19(*,2)=blue
  ;
  loadct,20
  tvlct, /get, red, green, blue
  !ctv.ct20(*,0)=red
  !ctv.ct20(*,1)=green
  !ctv.ct20(*,2)=blue
  ;
  loadct,21
  tvlct, /get, red, green, blue
  !ctv.ct21(*,0)=red
  !ctv.ct21(*,1)=green
  !ctv.ct21(*,2)=blue
  ;
  loadct,22
  tvlct, /get, red, green, blue
  !ctv.ct22(*,0)=red
  !ctv.ct22(*,1)=green
  !ctv.ct22(*,2)=blue
  ;
  loadct,23
  tvlct, /get, red, green, blue
  !ctv.ct23(*,0)=red
  !ctv.ct23(*,1)=green
  !ctv.ct23(*,2)=blue
  ;
  loadct,24
  tvlct, /get, red, green, blue
  !ctv.ct24(*,0)=red
  !ctv.ct24(*,1)=green
  !ctv.ct24(*,2)=blue
  ;
  loadct,25
  tvlct, /get, red, green, blue
  !ctv.ct25(*,0)=red
  !ctv.ct25(*,1)=green
  !ctv.ct25(*,2)=blue
  ;
  loadct,26
  tvlct, /get, red, green, blue
  !ctv.ct26(*,0)=red
  !ctv.ct26(*,1)=green
  !ctv.ct26(*,2)=blue
  ;
  loadct,27
  tvlct, /get, red, green, blue
  !ctv.ct27(*,0)=red
  !ctv.ct27(*,1)=green
  !ctv.ct27(*,2)=blue
  ;
  loadct,28
  tvlct, /get, red, green, blue
  !ctv.ct28(*,0)=red
  !ctv.ct28(*,1)=green
  !ctv.ct28(*,2)=blue
  ;
  loadct,29
  tvlct, /get, red, green, blue
  !ctv.ct29(*,0)=red
  !ctv.ct29(*,1)=green
  !ctv.ct29(*,2)=blue
  ;
  loadct,0
endif
;
if ((true eq 0) and (ncol gt 16)) then begin
  ; --- Load two color tables into subranges ---
;  normct, colorrange=!ctv.colorrange1, 6, mu=0.95
  normct, colorrange=!ctv.colorrange1, -2
  normct, colorrange=!ctv.colorrange2, 0
  ;
  ; --- Modify single colors in the color table ---
  tvlct, red, green, blue, /GET                           ; Load color table
  ;
  red(!ctv.gray)=128                                      ; Gray
  green(!ctv.gray)=128
  blue(!ctv.gray)=128
  ;
  red(!ctv.red)=255                                       ; Red
  green(!ctv.red)=0
  blue(!ctv.red)=0
  ;
  red(!ctv.orange)=255                                    ; Orange
  green(!ctv.orange)=140
  blue(!ctv.orange)=0
  ;
  red(!ctv.yellow)=255                                    ; Yellow
  green(!ctv.yellow)=255
  blue(!ctv.yellow)=0
  ;
  red(!ctv.green)=0                                       ; Green
  green(!ctv.green)=255
  blue(!ctv.green)=0
  ;
  red(!ctv.cyan)=0                                        ; Cyan
  green(!ctv.cyan)=255
  blue(!ctv.cyan)=255
  ;
  red(!ctv.blue)=0                                        ; Blue
  green(!ctv.blue)=0
  blue(!ctv.blue)=255
  ;
  red(!ctv.magenta)=255                                   ; Magenta
  green(!ctv.magenta)=0
  blue(!ctv.magenta)=255
  ;
  tvlct, red, green, blue                                 ; Put color table
endif
;
if ((true eq 0) and (ncol le 16)) then begin
  ; --- Load two color tables into subranges ---
  !ctv.colorrange0=[0,!D.TABLE_SIZE-1]
  !ctv.colorrange1=[0,!D.TABLE_SIZE-1]
  !ctv.colorrange2=[0,!D.TABLE_SIZE-1]
  normct, colorrange=!ctv.colorrange0, 0
  ;
endif
;
;help, /structure, !ctv
;
;delvar, red,green,blue, redprism,greenprism,blueprism, $
;        redgray,greengray, bluegray, color1, color2
;delvar, ctv1, ncol, ra, true, c
;
end ; setctv
