; pro SETCTV
;
;+
; NAME:
;   SETCTV (Set color table variable)
;
; PURPOSE:
;   Define a new system variable which contains information about the
;   currently loaded color tables
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORD PARAMETERS:
;   None
;
; SIDE EFFECTS:
;   Color tables are changed. A new system variable '!CTV' is created.
;   Some temporary variables are used and deleted.
;
; SUBROUTINES:
;   normct: Load and normalize color table into the specified range
;
; EXAMPLE:
;   @setctv
;   help, /struct, !CTV
;   plot, [1,2,3,2,4,5,2], color=!CTV.BLUE
;
; MODIFICATION HISTORY:
;   26.8.93 Written by Bernd Freytag, Astrophysik Kiel
;    6.12.93 Modification for True Color terminal
;    7.12.93
;   13.12.93 Standard color table -2
;   14.12.93 COLORRANGE0 added
;    3. 3.94 Standard colortables (!ctv.ctXX) for true color terminal
;            added
;    5. 4.95 (B.F.) no window opened at start 
;    3.12.96 (B.F.) For black/white terminal set all colors except black to whi.
;   29. 9.97 (B.F.) Reduce maximum line length for IDL5 
;   12.11.97 (B.F.) ct10, ct29 removed
;-
;
;window, /free, xsize=100, ysize=10, /pixmap
;wdelete, !D.WINDOW
;
device, decomposed=1

if (!D.N_COLORS gt 256) then true=3 else true=0

help, 1, true

if (true eq 3) then begin &$
  print, 'True color terminal' &$
  ncol=fix(!D.TABLE_SIZE) &$
  print, '!D.TABLE_SIZE=', ncol &$
  normct,-2,/nonorm &$
  tvlct, /get, redprism, greenprism, blueprism &$
  color1=bytarr(1024,3) &$
  color1(*,0)=rebin(redprism,1024) &$
  color1(*,1)=rebin(greenprism,1024) &$
  color1(*,2)=rebin(blueprism,1024) &$
  loadct,0 &$
  tvlct, /get, redgray, greengray, bluegray &$
  color2=bytarr(256,3) &$
  color2(*,0)=redgray &$
  color2(*,1)=greengray &$
  color2(*,2)=bluegray &$
endif

help, 2, true

if (true eq 3) then begin &$
  defctv_true, color1, color2, ctv1 &$
  defsysv, '!ctv', ctv1 &$
endif

!P.COLOR=!ctv.white

help, 3, true

if (true eq 0) then begin &$
  ncol=fix(!D.TABLE_SIZE) &$
  print, '!D.TABLE_SIZE=', ncol &$
  if (ncol le 17) then c=0 else c=1 &$
  ra=[0,!D.TABLE_SIZE-1] &$
  defctv_pseudo, ctv1 &$
  defsysv, '!ctv', ctv1 &$
endif
;
if (true eq 3) then begin &$
  loadct,0 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct0(*,0)=red &$
  !ctv.ct0(*,1)=green &$
  !ctv.ct0(*,2)=blue &$
  loadct,1 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct1(*,0)=red &$
  !ctv.ct1(*,1)=green &$
  !ctv.ct1(*,2)=blue &$
  loadct,2 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct2(*,0)=red &$
  !ctv.ct2(*,1)=green &$
  !ctv.ct2(*,2)=blue &$
  loadct,3 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct3(*,0)=red &$
  !ctv.ct3(*,1)=green &$
  !ctv.ct3(*,2)=blue &$
  loadct,4 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct4(*,0)=red &$
  !ctv.ct4(*,1)=green &$
  !ctv.ct4(*,2)=blue &$
endif

help, 4, true

if (true eq 3) then begin &$
  loadct,5 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct5(*,0)=red &$
  !ctv.ct5(*,1)=green &$
  !ctv.ct5(*,2)=blue &$
  loadct,6 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct6(*,0)=red &$
  !ctv.ct6(*,1)=green &$
  !ctv.ct6(*,2)=blue &$
  loadct,7 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct7(*,0)=red &$
  !ctv.ct7(*,1)=green &$
  !ctv.ct7(*,2)=blue &$
  loadct,8 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct8(*,0)=red &$
  !ctv.ct8(*,1)=green &$
  !ctv.ct8(*,2)=blue &$
  loadct,9 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct9(*,0)=red &$
  !ctv.ct9(*,1)=green &$
  !ctv.ct9(*,2)=blue &$
  loadct,0 &$
endif
;if (true eq 3) then begin &$
;  ;
;  loadct,10 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct10(*,0)=red &$
;  !ctv.ct10(*,1)=green &$
;  !ctv.ct10(*,2)=blue &$
;  ;
;  loadct,11 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct11(*,0)=red &$
;  !ctv.ct11(*,1)=green &$
;  !ctv.ct11(*,2)=blue &$
;  ;
;  loadct,12 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct12(*,0)=red &$
;  !ctv.ct12(*,1)=green &$
;  !ctv.ct12(*,2)=blue &$
;  ;
;  loadct,13 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct13(*,0)=red &$
;  !ctv.ct13(*,1)=green &$
;  !ctv.ct13(*,2)=blue &$
;  ;
;  loadct,14 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct14(*,0)=red &$
;  !ctv.ct14(*,1)=green &$
;  !ctv.ct14(*,2)=blue &$
;endif
;if (true eq 3) then begin &$
;  loadct,15 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct15(*,0)=red &$
;  !ctv.ct15(*,1)=green &$
;  !ctv.ct15(*,2)=blue &$
;  ;
;  loadct,16 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct16(*,0)=red &$
;  !ctv.ct16(*,1)=green &$
;  !ctv.ct16(*,2)=blue &$
;  ;
;  loadct,17 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct17(*,0)=red &$
;  !ctv.ct17(*,1)=green &$
;  !ctv.ct17(*,2)=blue &$
;  ;
;  loadct,18 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct18(*,0)=red &$
;  !ctv.ct18(*,1)=green &$
;  !ctv.ct18(*,2)=blue &$
;  ;
;  loadct,19 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct19(*,0)=red &$
;  !ctv.ct19(*,1)=green &$
;  !ctv.ct19(*,2)=blue &$
;endif
;if (true eq 3) then begin &$
;  ;
;  loadct,20 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct20(*,0)=red &$
;  !ctv.ct20(*,1)=green &$
;  !ctv.ct20(*,2)=blue &$
;  ;
;  loadct,21 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct21(*,0)=red &$
;  !ctv.ct21(*,1)=green &$
;  !ctv.ct21(*,2)=blue &$
;  ;
;  loadct,22 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct22(*,0)=red &$
;  !ctv.ct22(*,1)=green &$
;  !ctv.ct22(*,2)=blue &$
;  ;
;  loadct,23 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct23(*,0)=red &$
;  !ctv.ct23(*,1)=green &$
;  !ctv.ct23(*,2)=blue &$
;  ;
;  loadct,24 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct24(*,0)=red &$
;  !ctv.ct24(*,1)=green &$
;  !ctv.ct24(*,2)=blue &$
;endif
;if (true eq 3) then begin &$
;  loadct,25 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct25(*,0)=red &$
;  !ctv.ct25(*,1)=green &$
;  !ctv.ct25(*,2)=blue &$
;  ;
;  loadct,26 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct26(*,0)=red &$
;  !ctv.ct26(*,1)=green &$
;  !ctv.ct26(*,2)=blue &$
;  ;
;  loadct,27 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct27(*,0)=red &$
;  !ctv.ct27(*,1)=green &$
;  !ctv.ct27(*,2)=blue &$
;  ;
;  loadct,28 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct28(*,0)=red &$
;  !ctv.ct28(*,1)=green &$
;  !ctv.ct28(*,2)=blue &$
;  ;
;  loadct,29 &$
;  tvlct, /get, red, green, blue &$
;  !ctv.ct29(*,0)=red &$
;  !ctv.ct29(*,1)=green &$
;  !ctv.ct29(*,2)=blue &$
;  ;
;endif
if (true eq 3) then begin &$
  loadct,31 &$
  tvlct, /get, red, green, blue &$
  !ctv.ct31(*,0)=red &$
  !ctv.ct31(*,1)=green &$
  !ctv.ct31(*,2)=blue &$
  loadct,0 &$
endif
;
if ((true eq 0) and (ncol gt 16)) then begin &$
  normct, colorrange=!ctv.colorrange1, -2 &$
  normct, colorrange=!ctv.colorrange2, 0 &$
  tvlct, red, green, blue, /GET &$
  red(!ctv.gray)=128 &$           
  green(!ctv.gray)=128 &$
  blue(!ctv.gray)=128 &$
  red(!ctv.red)=255 &$            
  green(!ctv.red)=0 &$
  blue(!ctv.red)=0 &$
  red(!ctv.orange)=255 &$         
  green(!ctv.orange)=140 &$
  blue(!ctv.orange)=0 &$
  red(!ctv.yellow)=255 &$         
  green(!ctv.yellow)=255 &$
  blue(!ctv.yellow)=0 &$
  red(!ctv.green)=0 &$            
  green(!ctv.green)=255 &$
  blue(!ctv.green)=0 &$
  red(!ctv.cyan)=0 &$             
  green(!ctv.cyan)=255 &$
  blue(!ctv.cyan)=255 &$
  red(!ctv.blue)=0 &$             
  green(!ctv.blue)=0 &$
  blue(!ctv.blue)=255 &$
  red(!ctv.magenta)=255 &$        
  green(!ctv.magenta)=0 &$
  blue(!ctv.magenta)=255 &$
  tvlct, red, green, blue &$      
endif
;
if ((true eq 0) and (ncol le 16)) then begin &$
  normct, colorrange=!ctv.colorrange0, 0 &$
endif
;
;help, /structure, !ctv
;
delvar, red,green,blue, redprism,greenprism,blueprism, $
        redgray,greengray, bluegray, color1, color2
delvar, ctv1, ncol, ra, true, c
;
; end ; SETCTV
