pro setctv_ms
;============================================================================
; pro SETCTV
;
;+
; NAME:
;   SETCTV (Set color table variable)
; PURPOSE:
;   Define a new system variable which contains information about the
;   currently loaded color tables
; INPUTS:
;   None
; OUTPUTS:
;   None
; KEYWORD PARAMETERS:
;   None
; SIDE EFFECTS:
;   Color tables are changed. A new system variable '!CTV' is created.
;   Some temporary variables are used and deleted.
; SUBROUTINES:
;   normct: Load and normalize color table into the specified range
; EXAMPLE:
;   @setctv
;   help, /struct, !CTV
;   plot, [1,2,3,2,4,5,2], color=!CTV.BLUE
; MODIFICATION HISTORY:
;   Written: 26.8.93 (B.F.)
;            6.12.93 Modification for True Color terminal
;            7.12.93
;           13.12.93 Standard color table -2
;           14.12.93 COLORRANGE0 added
;             3.3.94 Standard colortables (ctv1.ctXX) for true color terminal
;                    added
;-
;
window, /free, xsize=100, ysize=10, /pixmap
wdelete, !D.WINDOW
;
if (!D.N_COLORS gt 256) then true=3 else true=0
;
if (true eq 3) then begin 
  print, 'True color terminal'
  ; --- Determine number of available colors ---
  ncol=fix(!D.TABLE_SIZE) 
  print, '!D.TABLE_SIZE=', ncol 
  ;
;  loadct,6  
  normct,-2,/nonorm  
  tvlct, /get, redprism, greenprism, blueprism  
  color1=bytarr(1024,3)  
  color1(*,0)=rebin(redprism,1024)  
  color1(*,1)=rebin(greenprism,1024)  
  color1(*,2)=rebin(blueprism,1024)  
  ;
  loadct,0  
  tvlct, /get, redgray, greengray, bluegray  
  color2=bytarr(256,3)  
  color2(*,0)=redgray  
  color2(*,1)=greengray  
  color2(*,2)=bluegray  
endif
if (true eq 3) then begin  
  ; --- Define structure for new system variable ---
  ctv1={ctvtyp, colorrange0:color2, $
                colorrange1:color1, $
                colorrange2:color2, $
                ct0:color2, $
                ct1:color2, $
                ct2:color2, $
                ct3:color2, $
                ct4:color2, $
                ct5:color2, $
                ct6:color2, $
                ct7:color2, $
                ct8:color2, $
                ct9:color2, $
                ct10:color2, $
                ct11:color2, $
                ct12:color2, $
                ct13:color2, $
                ct14:color2, $
                ct15:color2, $
                ct16:color2, $
                ct17:color2, $
                ct18:color2, $
                ct19:color2, $
                ct20:color2, $
                ct21:color2, $
                ct22:color2, $
                ct23:color2, $
                ct24:color2, $
                ct25:color2, $
                ct26:color2, $
                ct27:color2, $
                ct28:color2, $
                ct29:color2, $
                black :    0 + 256L * (  0 + 256L *   0), $
                gray:    128 + 256L * (128 + 256L * 128), $
                white:   255 + 256L * (255 + 256L * 255), $
                magenta: 255 + 256L * (  0 + 256L * 255), $
                red:     255 + 256L * (  0 + 256L *   0), $
                orange:  255 + 256L * (140 + 256L *   0), $
                yellow:  255 + 256L * (255 + 256L *   0), $
                green:     0 + 256L * (255 + 256L *   0), $
                cyan:      0 + 256L * (255 + 256L * 255), $
                blue  :    0 + 256L * (  0 + 256L * 255)   }  
endif
;
if (true eq 0) then begin  
  ; --- Determine number of available colors ---
  ncol=fix(!D.TABLE_SIZE)  
  print, '!D.TABLE_SIZE=', ncol  
  if (ncol lt 12) then message, 'Not enough colours available'  
  ra=[0,!D.TABLE_SIZE-1]  
  ;
  ; --- Define structure for new system variable ---
  ctv1={ctvtyp, colorrange0:[1            , ncol-10   ], $
                colorrange1:[1            ,(ncol-10)/2], $
                colorrange2:[(ncol-10)/2+1, ncol-10   ], $
                ct0:ra, ct1:ra, ct2:ra, ct3:ra, ct4:ra, $
                ct5:ra, ct6:ra, ct7:ra, ct8:ra, ct9:ra, $
                ct10:ra, ct11:ra, ct12:ra, ct13:ra, ct14:ra, $
                ct15:ra, ct16:ra, ct17:ra, ct18:ra, ct19:ra, $
                ct20:ra, ct21:ra, ct22:ra, ct23:ra, ct24:ra, $
                ct25:ra, ct26:ra, ct27:ra, ct28:ra, ct29:ra, $
                black :0,  gray:ncol-9,   white:ncol-1,             $
                magenta:ncol-7, red  :ncol-6, orange:ncol-5, $
                yellow:ncol-4, green:ncol-3,  cyan:ncol-8, $
                blue  :ncol-2}  
endif
;
if (true eq 3) then begin  
  ;
  loadct,0  
  tvlct, /get, red, green, blue  
  ctv1.ct0(*,0)=red  
  ctv1.ct0(*,1)=green  
  ctv1.ct0(*,2)=blue  
  ;
  loadct,1  
  tvlct, /get, red, green, blue  
  ctv1.ct1(*,0)=red  
  ctv1.ct1(*,1)=green  
  ctv1.ct1(*,2)=blue  
  ;
  loadct,2  
  tvlct, /get, red, green, blue  
  ctv1.ct2(*,0)=red  
  ctv1.ct2(*,1)=green  
  ctv1.ct2(*,2)=blue  
  ;
  loadct,3  
  tvlct, /get, red, green, blue  
  ctv1.ct3(*,0)=red  
  ctv1.ct3(*,1)=green  
  ctv1.ct3(*,2)=blue  
  ;
  loadct,4  
  tvlct, /get, red, green, blue  
  ctv1.ct4(*,0)=red  
  ctv1.ct4(*,1)=green  
  ctv1.ct4(*,2)=blue  
  ;
  loadct,5  
  tvlct, /get, red, green, blue  
  ctv1.ct5(*,0)=red  
  ctv1.ct5(*,1)=green  
  ctv1.ct5(*,2)=blue  
  ;
  loadct,6  
  tvlct, /get, red, green, blue  
  ctv1.ct6(*,0)=red  
  ctv1.ct6(*,1)=green  
  ctv1.ct6(*,2)=blue  
  ;
  loadct,7  
  tvlct, /get, red, green, blue  
  ctv1.ct7(*,0)=red  
  ctv1.ct7(*,1)=green  
  ctv1.ct7(*,2)=blue  
  ;
  loadct,8  
  tvlct, /get, red, green, blue  
  ctv1.ct8(*,0)=red  
  ctv1.ct8(*,1)=green  
  ctv1.ct8(*,2)=blue  
  ;
  loadct,9  
  tvlct, /get, red, green, blue  
  ctv1.ct9(*,0)=red  
  ctv1.ct9(*,1)=green  
  ctv1.ct9(*,2)=blue  
  ;
  loadct,10  
  tvlct, /get, red, green, blue  
  ctv1.ct10(*,0)=red  
  ctv1.ct10(*,1)=green  
  ctv1.ct10(*,2)=blue  
  ;
  loadct,11  
  tvlct, /get, red, green, blue  
  ctv1.ct11(*,0)=red  
  ctv1.ct11(*,1)=green  
  ctv1.ct11(*,2)=blue  
  ;
  loadct,12  
  tvlct, /get, red, green, blue  
  ctv1.ct12(*,0)=red  
  ctv1.ct12(*,1)=green  
  ctv1.ct12(*,2)=blue  
  ;
  loadct,13  
  tvlct, /get, red, green, blue  
  ctv1.ct13(*,0)=red  
  ctv1.ct13(*,1)=green  
  ctv1.ct13(*,2)=blue  
  ;
  loadct,14  
  tvlct, /get, red, green, blue  
  ctv1.ct14(*,0)=red  
  ctv1.ct14(*,1)=green  
  ctv1.ct14(*,2)=blue  
  ;
  loadct,15  
  tvlct, /get, red, green, blue  
  ctv1.ct15(*,0)=red  
  ctv1.ct15(*,1)=green  
  ctv1.ct15(*,2)=blue  
  ;
  loadct,16  
  tvlct, /get, red, green, blue  
  ctv1.ct16(*,0)=red  
  ctv1.ct16(*,1)=green  
  ctv1.ct16(*,2)=blue  
  ;
  loadct,17  
  tvlct, /get, red, green, blue  
  ctv1.ct17(*,0)=red  
  ctv1.ct17(*,1)=green  
  ctv1.ct17(*,2)=blue  
  ;
  loadct,18  
  tvlct, /get, red, green, blue  
  ctv1.ct18(*,0)=red  
  ctv1.ct18(*,1)=green  
  ctv1.ct18(*,2)=blue  
  ;
  loadct,19  
  tvlct, /get, red, green, blue  
  ctv1.ct19(*,0)=red  
  ctv1.ct19(*,1)=green  
  ctv1.ct19(*,2)=blue  
  ;
  loadct,20  
  tvlct, /get, red, green, blue  
  ctv1.ct20(*,0)=red  
  ctv1.ct20(*,1)=green  
  ctv1.ct20(*,2)=blue  
  ;
  loadct,21  
  tvlct, /get, red, green, blue  
  ctv1.ct21(*,0)=red  
  ctv1.ct21(*,1)=green  
  ctv1.ct21(*,2)=blue  
  ;
  loadct,22  
  tvlct, /get, red, green, blue  
  ctv1.ct22(*,0)=red  
  ctv1.ct22(*,1)=green  
  ctv1.ct22(*,2)=blue  
  ;
  loadct,23  
  tvlct, /get, red, green, blue  
  ctv1.ct23(*,0)=red  
  ctv1.ct23(*,1)=green  
  ctv1.ct23(*,2)=blue  
  ;
  loadct,24  
  tvlct, /get, red, green, blue  
  ctv1.ct24(*,0)=red  
  ctv1.ct24(*,1)=green  
  ctv1.ct24(*,2)=blue  
  ;
  loadct,25  
  tvlct, /get, red, green, blue  
  ctv1.ct25(*,0)=red  
  ctv1.ct25(*,1)=green  
  ctv1.ct25(*,2)=blue  
  ;
  loadct,26  
  tvlct, /get, red, green, blue  
  ctv1.ct26(*,0)=red  
  ctv1.ct26(*,1)=green  
  ctv1.ct26(*,2)=blue  
  ;
  loadct,27  
  tvlct, /get, red, green, blue  
  ctv1.ct27(*,0)=red  
  ctv1.ct27(*,1)=green  
  ctv1.ct27(*,2)=blue  
  ;
  loadct,28  
  tvlct, /get, red, green, blue  
  ctv1.ct28(*,0)=red  
  ctv1.ct28(*,1)=green  
  ctv1.ct28(*,2)=blue  
  ;
  loadct,29  
  tvlct, /get, red, green, blue  
  ctv1.ct29(*,0)=red  
  ctv1.ct29(*,1)=green  
  ctv1.ct29(*,2)=blue  
  ;
  loadct,0  
endif
;
if (true eq 0) then begin  
  ; --- Load two color tables into subranges ---
;  normct, colorrange=ctv1.colorrange1, 6, mu=0.95  
  normct, colorrange=ctv1.colorrange1, -2  
  normct, colorrange=ctv1.colorrange2, 0  
  ;
  ; --- Modify single colors in the color table ---
  tvlct, red, green, blue, /GET                             ; Load color table
  ;
  red(ctv1.gray)=128                                        ; Gray
  green(ctv1.gray)=128  
  blue(ctv1.gray)=128  
  ;
  red(ctv1.red)=255                                         ; Red
  green(ctv1.red)=0  
  blue(ctv1.red)=0  
  ;
  red(ctv1.orange)=255                                      ; Orange
  green(ctv1.orange)=140  
  blue(ctv1.orange)=0  
  ;
  red(ctv1.yellow)=255                                      ; Yellow
  green(ctv1.yellow)=255  
  blue(ctv1.yellow)=0  
  ;
  red(ctv1.green)=0                                         ; Green
  green(ctv1.green)=255  
  blue(ctv1.green)=0  
  ;
  red(ctv1.cyan)=0                                          ; Cyan
  green(ctv1.cyan)=255  
  blue(ctv1.cyan)=255  
  ;
  red(ctv1.blue)=0                                          ; Blue
  green(ctv1.blue)=0  
  blue(ctv1.blue)=255  
  ;
  red(ctv1.magenta)=255                                     ; Magenta
  green(ctv1.magenta)=0  
  blue(ctv1.magenta)=255  
  ;
  tvlct, red, green, blue                                   ; Put color table
endif
;
; --- Define new system variable ---
defsysv, '!ctv', ctv1
;
;help, /structure, ctv1
;
;delvar, red, green, blue, redprism, greenprism, blueprism, redgray,greengray, $
;        bluegray, ctv1, color1, color2, ncol, ra, true
;
end ; SETCTV
;============================================================================
