function dbdtnuplanck, nu, T
;
;------------------------------------------------------------------------------
;+
; NAME:
;   bnuplanck ('black_body_nu_planck')
;
; PURPOSE:
;   Compute the temperature derivative of the black body source function.
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   Bnu=dbdtnuplanck(nu, T)
;
; INPUT:
;   nu: ('nu') frequency (vector) [1/s]
;   T:  ('Temperature') real scalar with temperature [K]
;
; OUTPUT:
;   Bnu: ('Black_body_nu') [erg/cm^2/T] or [erg/(cm^2 T s Hz)]
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   For very large or small frequencies there might be over-/underflows.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-05-07 Written by Bernd Freytag, CRAL-ENS Lyon
;-
;------------------------------------------------------------------------------

nud=double(nu)
Td=double(T)

hnuoverkT=(!con.hPlanck/(!con.kBoltz*Td))*nud

dBdTnu=(2.0D+00*!con.hPlanck/!con.cLight^2) * nud^3/(exp(hnuoverkT)-1.0D+00) * hnuoverkT/(1.0-exp(-hnuoverkT))/T

return, dBdTnu

end ; dbdtnuplanck
