; pro dust_opatable
;
;------------------------------------------------------------------------------
;+
; NAME:
;   dust_opatable ('dust_opacity_table')
;
; PURPOSE:
;   Compute various averages from frequency-dependent dust opacity table
;
; CATEGORY:
;   dust, opacities
;
; CALLING SEQUENCE:
;   @dust_opatable
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;   Before first use type:
;     delvar, init
;
; PROCEDURE:
;
; EXAMPLE:
;   @dust_opatable
;
; MODIFICATION HISTORY:
;   2008-05-06 Written by Bernd Freytag, CRAL-ENS Lyon
;-
;------------------------------------------------------------------------------

; --- Selected dust grain radius index ---
irp=15

; --- Specify source file with dust opacities ---
;DustOpaSourceFile='Mg2SiO4.table.PER_GRAIN.extinc_edited'
DustOpaSourceFile='Mg2SiO4.table2.PER_GRAIN.extinc_edited'

; --- Gas opacities ---
opafile='t1800g550mm00_phoenix_idmean3RT3_fit.opta'
opadisk=getenv('HOME') + '/for/opa/dat'

dustopaoutfile=strmid(opafile,0,strpos(opafile, '.')) + '_dust01' + strmid(opafile,strpos(opafile, '.'))

;nT=3 & Tarr=1300.0+200.0*findgen(nT)
;nT=1 & Tarr=1500.0+200.0*findgen(nT)
nT=0

; --- Wavelength interval(s) to be considered ---
lambdaRange=[1.0E-20, 1.0]
;lambdaRange=[1.0E-20, 4.5E-04, 1.0]
;lambdaRange=[1.0E-20, 1.0E-04, 4.5E-04, 4.0E-03, 1.0]
;lambdaRange=[1.0E-20, 8.0E-05, 3.0E-04, 7.0E-04, 3.0E-03, 1.0]

BandColor=[!ctv.white,!ctv.red]

if (not keyword_set(init)) then begin &$
  ; --- Read gas opacity ---
  rdopta, opadisk + '/' + opafile, opa &$
  ;
  ; --- Read dust opacities from source file ---
  ; --- kappa:  opacity [cm^2]
  ; --- r:      grain radius [mu]
  ; --- lambda: wavelength [A]
  rdarray, DustOpaSourceFile, a &$
  OpaDust={ $
    file  : DustOpaSourceFile, $
    lambda: reform(a[1:*,0]), $
    nu    : reform(a[1:*,0])*0.0, $
    dnu   : reform(a[1:*,0])*0.0, $
    r     : reform(a[0,1:*]), $
    kappa : a[1:*,1:*] $
    } &$
  help, OpaDust.kappa,OpaDust.lambda,OpaDust.r &$
  nlambda=n_elements(OpaDust.lambda) &$
  nr     =n_elements(OpaDust.r) &$
  ;
  ; --- Transform to cgs ---
  OpaDust.lambda=OpaDust.lambda*1.0E-08 &$
  OpaDust.r     =OpaDust.r     *1.0E-04 &$
  ;
  ; --- Fill up zeros in opacity for large grains and apply median filter ---
  for ir=0,nr-1 do begin &$
    ind=where(OpaDust.kappa[*,ir] eq 0.0, nind) &$
    if (nind gt 0L) then begin &$
      print, ir, OpaDust.r[ir], nind &$
      OpaDust.kappa[ind,ir]=OpaDust.kappa[max(ind)+1L,ir] &$
    endif &$
    OpaDust.kappa[*,ir]=median(OpaDust.kappa[*,ir], 3) &$
  endfor &$
  ;
  ; --- Normalize ---
  area=2.0*!pi*OpaDust.r^2 &$
  areaarr=rebin(reform(area,1,nr),nlambda,nr) &$
  OpaDust.kappa=OpaDust.kappa/areaarr &$
  ;
  init=1 &$
endif

if (nT eq 0) then begin &$
  nT=opa.nT &$
  Tarr=10.0^opa.tabT &$
endif

; --- Compute frequency and delta-frequency axis ---
nu=!con.cLight/OpaDust.lambda
dnu=OpaDust.nu
dnu[0]        =nu[1]-nu[0]
dnu[nlambda-1]=nu[nlambda-1]-nu[nlambda-2]
dnu[1:nlambda-2]=0.5*(nu[2:nlambda-1]-nu[0:nlambda-3])
OpaDust.nu =     nu
OpaDust.dnu=abs(dnu)

; --- Plot dust opacities for all radii ---
plotlines, alog10(OpaDust.lambda), alog10(OpaDust.kappa>1.0E-20), $
  title='Opa!Dgas!N=' + opafile + $
        ',  Opa!Ddust!N=' + DustOpaSourceFile + $
        ',  Radius!Ddust!N=' + strtrim(string(OpaDust.r[irp]*1.0E4,format='(E9.2)'),2) + 'micron', $
  xrange=[-5.1,-1.0], drange=[-6.0,2.0], linecol=!ctv.gray

caldat, systime(/julian), mm,dd,yyyy,hh,mmin,ss 
DateString=strmid('000'+strtrim(string(yyyy),2),3,4,/rev) + '-' + $
           strmid('0'+strtrim(string(mm),2),1,2,/rev) + '-' + $
           strmid('0'+strtrim(string(dd),2),1,2,/rev) + ' ' + $
           strmid('0'+strtrim(string(hh),2),1,2,/rev) + ':' + $
           strmid('0'+strtrim(string(mmin),2),1,2,/rev) + ':' + $
           strmid('0'+strtrim(string(ss),2),1,2,/rev)

OpaDustHeader=[ $
'*', $
'*     Opacity table name: ' + dustopaoutfile, $
'*     Created           : ' + DateString, $
'*     Contents          : Grey Rosseland mean', $
'*                         + Band Planck means', $
'*', $
'*     Dust opacity source', $
'*     -------------------', $
'*     Dust opacity file: ' + DustOpaSourceFile, $
'*', $
'*     r-T domain definition parameters (log cgs units)', $
'*     ------------------------------------------------', $
'*     ntbn        =' + string(opa.ntbn, format='(I3)'), $
'*     TABTBN      =' + strjoin(string(opa.tabtbn, format='(F8.3)')), $
'*     TABDTB      =' + strjoin(string(opa.tabdtb, format='(F8.3)')), $
'*     nrbn        =' + string(2, format='(I3)'), $
'*     TABRBN      =' + strjoin(string([alog10(OpaDust.r[0]), alog10(OpaDust.r[n_elements(OpaDust.r)-1])], format='(F8.3)')), $
'*     TABDRB      =' + strjoin(string([alog10(OpaDust.r[1])-alog10(OpaDust.r[0])], format='(F8.3)')), $
'*', $
'*     ..................................................................' $
]

; --- Define structure for output quantities ---
nlambdaRange=n_elements(lambdaRange)-1
OpaDustband={ $
  header    : [OpaDustHeader,opa.header], $
  idvers    : opa.idvers, $
  optana_gas: opa.optana, $
  optana    : dustopaoutfile, $
  datex_gas : opa.datex, $
  datex     :  DateString, $
  idmean    : opa.idmean, $
  nt:     opa.nt, $
  ntbn:   opa.ntbn, $
;  np:     opa.np, $
;  npbn:   opa.npbn, $
  nband:  opa.nband, $
  mstep:  opa.mstep, $  
  nulo:   opa.nulo, $
  nuhi:   opa.nuhi, $
  numnu:  opa.numnu, $
  intmdx: opa.intmdx, $
  ;
  tabt:   opa.tabt, $
  tabtbn: opa.tabtbn, $
  idxtbn: opa.idxtbn, $
  tabdtb: opa.tabdtb, $
  ;
;  tabp:   opa.tabp, $
;  tabpbn: opa.tabpbn, $
;  idxpbn: opa.idxpbn, $
;  tabdpb: opa.tabdpb, $
  ;
  xopta_gas:  opa.xopta, $
  ;
  wtbbd:  opa.wtbbd, $
  inband: opa.inband, $
  ;
  dustfile: DustOpaSourceFile, $
  gasfile : opafile, $
  lambda  : OpaDust.lambda, $
  nu      : OpaDust.nu, $
  dnu     : OpaDust.dnu, $
  r       : OpaDust.r, $
  kappa     : OpaDust.kappa, $
  kappaP    : fltarr(nr,nT), $
  kappaPband: fltarr(nr,nT,opa.nBand,nlambdaRange), $
  kappaR    : fltarr(nr,nT), $
  kappaRband: fltarr(nr,nT,opa.nBand,nlambdaRange), $
  ;
  nr      : n_elements(OpaDust.r), $
  nrbn    : 2, $
  tabr    : alog10(OpaDust.r), $
  tabrbn  : [alog10(OpaDust.r[0]), alog10(OpaDust.r[n_elements(OpaDust.r)-1])], $
  idxrbn  : [1L, n_elements(OpaDust.r)], $
  tabdrb  : [alog10(OpaDust.r[1])-alog10(OpaDust.r[0])], $
  xopta   : fltarr(nr,nT,opa.nBand+1) $
  }

; --- Compute Planck mean ---
for iT=0,nT-1 do begin &$
  Bnu   =bnuplanck(OpaDust.nu, Tarr[iT]) &$
  Bnudnu=Bnu*OpaDust.dnu &$
  B     =total(Bnudnu) &$
  ;
  for ir=0,nr-1 do begin &$
    OpaDustband.kappaP[ir,iT]=total(Bnudnu*reform(OpaDust.kappa[*,ir]))/B &$
    ;
    print, ir, OpaDust.r[ir], OpaDustband.kappaP[ir,iT] &$
    if (ir eq irp) then begin &$
      plotlines, alog10(OpaDust.lambda), alog10(OpaDust.kappa[*,ir]>1.0E-20), /over &$
      plotlines, alog10(OpaDust.lambda), OpaDust.lambda*0.0+alog10(OpaDustband.kappaP[ir,iT]), /over, thick=2, linecol=!ctv.yellow &$
    endif &$
    ;
    for iBand=0,opa.nBand-1 do begin &$
      for ilambdaRange=0,nlambdaRange-1 do begin &$
        ind=where((opa.inband eq (iBand+1)) and $
                  (OpaDust.lambda gt lambdaRange[ilambdaRange  ]) and $
                  (OpaDust.lambda le lambdaRange[ilambdaRange+1]), nind) &$
        if (nind gt 0) then begin &$
          OpaDustband.kappaPband[ir,iT,iBand,ilambdaRange]=total((Bnudnu*reform(OpaDust.kappa[*,ir]))[ind])/total(Bnudnu[ind]) &$
          if (ir eq irp) then begin &$
            plotlines, alog10(OpaDust.lambda[ind]), OpaDust.lambda[ind]*0.0+alog10(OpaDustband.kappaPband[ir,iT,iBand,ilambdaRange]), $
              /over, psym=3, linecol=!ctv.green &$
          endif &$
        endif &$
      endfor &$ ; ilambdarange
    endfor &$
  endfor &$
endfor

; --- Compute Rosseland mean ---
for iT=0,nT-1 do begin &$
  dBdTnu   =dbdtnuplanck(OpaDust.nu, Tarr[iT]) &$
  dBdTnudnu=dBdTnu*OpaDust.dnu &$
  dBdT     =total(dBdTnudnu) &$
  ;
  for ir=0,nr-1 do begin &$
    OpaDustband.kappaR[ir,iT]=dBdT/total(dBdTnudnu/reform(OpaDust.kappa[*,ir])) &$
;    OpaDustband.kappaR[ir,iT]=total(dBdTnudnu*reform(OpaDust.kappa[*,ir]))/dBdT &$ ; --- Rosseland weights but linear (not inverse) kappa ---
    ;
    print, ir, OpaDust.r[ir], OpaDustband.kappaR[ir,iT] &$
    if (ir eq irp) then begin &$
      plotlines, alog10(OpaDust.lambda), alog10(OpaDust.kappa[*,ir]>1.0E-20), /over &$
      plotlines, alog10(OpaDust.lambda), OpaDust.lambda*0.0+alog10(OpaDustband.kappaR[ir,iT]), /over, thick=2, linecol=!ctv.magenta &$
    endif &$
    ;
    for iBand=0,opa.nBand-1 do begin &$
      for ilambdaRange=0,nlambdaRange-1 do begin &$
        ind=where((opa.inband eq (iBand+1)) and $
                  (OpaDust.lambda gt lambdaRange[ilambdaRange  ]) and $
                  (OpaDust.lambda le lambdaRange[ilambdaRange+1]), nind) &$
        if (nind gt 0) then begin &$
          OpaDustband.kappaRband[ir,iT,iBand,ilambdaRange]=total(dBdTnudnu[ind])/total((dBdTnudnu/reform(OpaDust.kappa[*,ir]))[ind]) &$
          if (ir eq irp) then begin &$
            plotlines, alog10(OpaDust.lambda[ind]), OpaDust.lambda[ind]*0.0+alog10(OpaDustband.kappaRband[ir,iT,iBand,ilambdaRange]), $
              /over, psym=3, linecol=!ctv.red &$
          endif &$
        endif &$
      endfor &$ ; ilambdarange
    endfor &$ ; iBand
  endfor &$ ; ir
endfor ; iT

OpaDustband.xopta[*,*,0  ]=alog10(OpaDustBand.kappaR)
OpaDustband.xopta[*,*,1:*]=alog10(OpaDustBand.kappaPband)

; --- Overplot opacity bands and Planck curve ---
for ilambdaRange=0,nlambdaRange-1 do begin &$
  ind=where((OpaDust.lambda gt lambdaRange[ilambdaRange]  ) and $
            (OpaDust.lambda le lambdaRange[ilambdaRange+1]), nind) &$
  plotlines, alog10(OpaDust.lambda[ind]), opa.inband[ind]*0.2+1.0+randomn(seed,nind)*0.02, $
             linecolor=BandColor[ilambdaRange mod n_elements(BandColor)], psym=3, /over &$
endfor ; ilambdarange
plotlines, alog10(OpaDust.lambda), alog10(bnuplanck(OpaDust.nu, Tarr[0]))+7, linecol=!ctv.blue, /over

; --- Plot averaged opacity over radius, together with fits ---
;lc=[!ctv.white,!ctv.green,!ctv.yellow,!ctv.orange]
;plotlines,alog10(OpaDust.r),alog10(OpaDustband.kappaP), xtitle='log(r/[cm])', ytitle='cross!Dgrain!N/2!4p!3r!U2!N', linecol=lc
;plotlines,alog10(OpaDust.r),alog10(OpaDustband.kappaR), /over, linecol=lc, linestyle=1
;plotlines,alog10(OpaDust.r),alog10( (OpaDust.r/1.0E-4)<1.0 ),/over,linecol=!ctv.blue
;plotlines,alog10(OpaDust.r),alog10( ((OpaDust.r/6.0E-3)>((OpaDust.r/6.0E-05)^3))<1.0 ),/over,linecol=!ctv.red,thick=3

print, 'Write result into ' + dustopaoutfile
wropta, dustopaoutfile, OpaDustBand, mode=2

; end ; dust_opatable
