pro psatc1, TEMP, PSAT, DPSATT

; adapted from
;----------------------------------------------------------------------
; File  :    PSATC1.FOR
; Autor :    Susanne Hoefner
; Datum :    28.6.91                   letzte Aenderung : 3.7.91
;
; Verwendungszweck :
;     
;     Berechnung des Saettigungsdruckes von C ueber Graphit
;......................................................................
;
;  Parameter :
;
;     TEMP       : Temperatur
;     PSAT       : Saettigungsdruck
;     DPSATT     : Abl. von Psat nach der Temperatur
;.......................................................................
;
;  verwendete Unterprogramme :
;     *** keine ***
;.......................................................................
;
;  weitere Kommentare :
;
;     Naeherungsformel fuer PSAT von C ueber Graphit 
;     (nach Gail & Sedlmayr, Paper IV, A+A 1988) 
;     gueltig im Temperaturbereich 500 K <= T <= 1500 K
;
;-----------------------------------------------------------------------

;      APSAT  = 86300D00
;      BPSAT  = 32.89D00

;       Formel fuer ln(Psat) nach G&S (Paper IV)

;        XLNPS     = - APSAT / TEMP + BPSAT
;        DLNPST    =   APSAT / TEMP^2

;       Saettigungsdruck

;        PSAT   = EXP (XLNPS)
;        DPSATT = PSAT * DLNPST

; saturation pressure of fosterite (MgFeSiO_4)
         a      =  42.179466259637
         b      = -2.94064111396886e4
         c      = -4.52457333892826e7
         d      = 2.00362363225697e10
         psat   = exp( a + b/temp + c/temp^2 + d/temp^3)
         dpsatt = 0.0
end


pro dustform4, rho, t, xndmax, xnctot, taui, ratek3, equk3, rgrain, v_settle, c_dust04=c_dust04
      default, c_dust04, 1.0 ; dust velocity factor

      kboltz = 1.38070e-16
      pmass  = 1.6726e-24
      qmol   = 1.3018    ; standard kiel mixture at solar metallicity
      abuc   = 3.0e-5 ; Fe-abundance

      Z03    = 1.0 / 3.0
      Z4PI   = 4.0 * !PI

;      Carbon
;      AGW    = 12.01115D00      ; atomic weight of carbon
;      RHOD   = 2.25D00          ; density of grain material

;     Fosterite
      AGW    = 172.25           ; atomic weight of fosterite
      RHOD   = 3.8
      STICK1 = 0.37             ; sticking coeff. [0-1]
      grav   = 1.0e5

      R1DUST = ( 3.0 * AGW * PMASS / ( Z4PI * RHOD ) )^Z03
      print, 'R1DUST = ', r1dust
      A1DUST = Z4PI * R1DUST^2

      xnctot = rho/(qmol*pmass)*abuc
      psatc1, t, psat, dum

      equK3  = (XNCTOT - PSAT/(KBOLTZ*T)) > 0.0

      dustk3 = equk3
      dustk0 = xnctot/xndmax
      ngrain = dustk3/dustk0
      xncgas = xnctot-dustk3
      rgrain = r1dust*ngrain^z03

      VTH    = SQRT( KBOLTZ * T / (2.0 * !PI * PMASS * AGW) )
      TAUI   = A1DUST * STICK1 * VTH * (XNCGAS-PSAT/(KBOLTZ*T))
      rateK3 = A1DUST * STICK1 * VTH * ((1.0E-09*DUSTK3)^2 * DUSTK0)^Z03 * 1.0E+06
      v_settle =C_dust04 * ngrain^(2.0/3.0) * grav/sqrt(T)
end

function equk3, rho, t, fraction=fraction
; computed the equilibrium value of the 3rd moment of the grain size distribution
  kboltz = 1.38070e-16
  pmass  = 1.6726e-24
  qmol   = 1.3018    ; standard kiel mixture at solar metallicity
  abuc   = 4.246e-04 ; number fraction of carbon nuclei (relative to total number of nuclei)

  xnctot = rho/(qmol*pmass)*abuc
  psatc1, t, psat, dum

  if keyword_set(fraction) then return, ((xnctot-psat/(kboltz*t))>0.0)/xnctot else return, (xnctot-psat/(kboltz*t))>0.0
end
