pro seqavgdiffusion, seq, D, mtime=mtime, ntime=ntime, $
  vfall=vfall, qucall=qucall, fqucall=fqucall, ffall=ffall, fsum=fsum, cderiv=cderiv, $
  quc001=quc001, quc002=quc002, fquc001=fquc001, fquc002=fquc002, rho=rho
;
;------------------------------------------------------------------------------
;+
; NAME:
;   seqavgdiffusion ('sequence_average_diffusion')
;
; PURPOSE:
;   Compute diffusion coefficient from concentratrion gradient and settling velocity
;
; CATEGORY:
;   RHD, dust
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   seqavg:
;   rhd_hyd_rossow78tfall:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; COMMENTS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2008-08-06 (B.F.) Written
;-
;------------------------------------------------------------------------------

; --- Gravity and density of dust particles ---
grav=seq.par.grav
rop =seq.par.C_dust01

; --- Background stratification ---
p      =seqavg('p'      , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
T      =seqavg('T'      , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
rho    =seqavg('rho'    , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
z      =seqavg('xc3'    , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')

; --- Partical radius and settling speed ---
radius =seqavg('rdust'  , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
vfall  =rhd_hyd_rossow78tfall(p, T, rho, grav, radius, rop)

; --- Dust densities and fluxes ---
quc001 =seqavg('quc001' , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
quc002 =seqavg('quc002' , seq, mtime=mtime, ntime=ntime, unitsystem='cgs')
fquc001=seqavg('fquc001', seq, mtime=mtime, ntime=ntime, unitsystem='cgs', /intertoxc)
fquc002=seqavg('fquc002', seq, mtime=mtime, ntime=ntime, unitsystem='cgs', /intertoxc)

; --- Sums ---
qucall    =quc001 +quc002
qucallorho=qucall/rho
fqucall=fquc001+fquc002
ffall  =-quc002*vfall
fsum   =-fqucall + ffall
;fsum   =ffall

; --- Diffusion coefficient ---
cderiv=rho*deriv(z, qucallorho)
D     =fsum/cderiv
;cderiv=rho*deriv(z, quc001/rho)
;D     =-fquc001/cderiv
;cderiv=rho*deriv(z, quc002/rho)
;D     =(-fquc002+ffall)/cderiv

; --- Set questionable points to dummy value ---
;epsrhoratio=2.0D-08
;Ddummy     =1.0D+00
;ind=where(abs(deriv(qucallorho)) lt epsrhoratio, nind)
;if (nind gt 0) then D[ind]=Ddummy

; --- Single precision output ---
D=float(D)

end ; pro seqavgdiffusion
