; blueshift_fufi_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   blueshift_fufi_fit ('blue_shift_function_find_fit')
;
; PURPOSE:
;   Fit blue shift as function of Teff, logg, FeH by single function
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @blueshift_fufi_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @blueshift_fufi_fit
;
; MODIFICATION HISTORY:
;   2011-09-15 (B.F.) Written by Bernd Freytag, CRAL-ENS Lyon
;   2011-10-19 (B.F.) New version of the data
;   2012-07-05 (B.F.) New plot layout
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Select device ---
  dev='ps' &$
  jpg_flag=0 &$
  png_flag=0 &$
  CMYK_flag=0 &$
  ;
  ; --- Data file name  ---
  ;InputFileName='callende_tmp_20110915.txt' &$
  ;InputFileName='callende_tmp_20111019.txt' &$
  InputFileName='callende_tmp_full_20111019.txt' &$
  ;
  ; --- Control for function finding with fufi ---
  ;tid='Polynomials' & fufiTermMask=[[0]] &$         ; --- polynomials (in all variables)
  tid='PolyExp' & fufiTermMask=[[0],[1]] &$         ; --- polynomials + exponential
  ;tid='test' & fufiTermMask=[[0],[2]] &$         ; --- polynomials (in all variables)
  ;tid='ConIdExp' & fufiTermMask=[[4],[2]] &$     ; --- const + identity + exponential
  ;
  fufiDepth=5 &$
  fufiNextraRandom=0 &$
  ;fufiNextraRandom=16 &$
  ;
  file_id='BlueShift_' + tid + '_nDepth' + trim(string(fufiDepth,format='(I3)'),2) &$
  ;
  ;RestoreOptFile='' &$
  RestoreOptFile=file_id + '.sav' &$
  SaveOptFile='' &$
  ;SaveOptFile=file_id + '.sav' &$
endif

; --- Plot file names ---
Mreg=3.0
Met_id='_Met' + strtrim(string(Mreg, format='(I1)'),2)
jpg_file=file_id + Met_id + '.jpg'
png_file=file_id + Met_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + Met_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + Met_id + '.ps' &$
endelse

; --- Read input data ---
rdarray, InputFileName, b0, s
b=b0

if (InputFileName eq 'callende_tmp_full_20111019.txt') then begin &$
  modellist0=reform(s[*,0]) &$
  ;modellist =modellist[sort(modellist0)] &$
  ;modellist =modellist[uniq(modellist )] &$
  indm=sort(modellist0) &$
  indm=uniq(modellist0, indm) &$
  modellist=modellist0[indm] &$
  ;
  vavg =fltarr(n_elements(modellist)) &$
  vdrms=fltarr(n_elements(modellist)) &$
  ;
  for i=0,n_elements(modellist)-1 do begin &$
    model=modellist[i] &$
    ind=where(model eq modellist0) &$
    vavg[ i]=mean(b[ind,6]) &$
    vdrms[i]=sqrt(mean((b[ind,6]-vavg[i])^2)/n_elements(ind)) &$
  endfor &$
  ;
  b1=[[b0[indm,0]], [b0[indm,1]], [b0[indm,2]], [b0[indm,3]], [b0[indm,4]], [b0[indm,5]], [vavg], [vdrms]] &$
  ;
  b=b1 &$
endif

; --- Restrict data ---
ind=where(b[*,2] lt 65000.0)
;ind=where(b[*,2] lt 6500.0)
;ind=where((b[*,2] lt 6500.0) and (b[*,4] ge -2.0))
;ind=where((b[*,4] eq 1.0))
;ind=where((b[*,3] ge 4.0) and (b[*,4] le 1.0))
;ind=where((abs(b[*,2]-5000.0) le 800.0) and (b[*,4] le 3.0))

; --- Create empty arrays for independent and dependent variables ---
;nx=n_elements(b[*,0])
nx=n_elements(ind)
;x =fltarr(nx,2)
x =fltarr(nx,3)
y =fltarr(nx)
dy=fltarr(nx)
w =replicate(0.02,nx)

; --- Put input data into fufi input arrays ---
x[*,0]=alog10(b[ind,2]/1000.0) ; --- log T ---
x[*,1]=b[ind,3]                ; --- log g ---
x[*,2]=b[ind,4]                ; --- log M ---
y[*]  =b[ind,6]                ; --- avg shift ---
w[*]  =1.0/(b[ind,7] > 1.0E-2) ; --- drms shift ---
dy[*] =1.0/w

; --- Find fitting function ---
; --- input: x,y; output: Fstropt, aopt ---
; --- control: TermMask, npar, depth, ... ---
npar=30 & nextramax=4
if (keyword_set(RestoreOptFile)) then begin &$
  restore, RestoreOptFile, /verbose &$
  fufiStart=fufiOpt &$
  functionfind, x, y, w, $
    Fstrstart=fufiStart.Fstr, PDERstrstart=fufiStart.PDERstr, astart=fufiStart.a, $
      allowtermstart=fufiStart.allowterm, varytermstart=fufiStart.varyterm, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=[[0]], npar=npar, depth=0, nextramax=nextramax, nextrara=fufiNextraRandom &$
endif else begin &$
  functionfind, x, y, w, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=fufiTermMask, npar=npar, depth=fufiDepth, nextramax=nextramax, nextrara=fufiNextraRandom &$
endelse

; --- Compute the values of the fit function at the given points ---
funct_string, x, aopt, yfufit
;print, yfufit
;print, 'Total rms deviation: ', rms(w*(y-yfufit), /nonorm)/(total(w)/n_elements(w)), max(abs(w*(y-yfufit)))
fit_rms=sqrt(avg((y-yfufit)^2))
fit_maxabs=max(abs((y-yfufit)))
print, 'Total rms/max deviation: ', sqrt(avg((y-yfufit)^2)), max(abs((y-yfufit)))

if (keyword_set(SaveOptFile)) then begin &$
  ; --- Save optimum solution into structure and write it to file ---
  fufiOpt=create_struct( $
                'x'        , x, $
                'y'        , y, $
                'w'        , w, $
                'qid'      , 'blueshift', $
                'tid'      , tid, $
                'TermMask' , fufiTermMask, $
                'npar'     , npar, $
                'depth'    , fufiDepth, $
                'nextramax', nextramax, $
                'Fstr'     , Fstropt, $
                'PDerstr'  , PDERstropt, $
                'A'        , Aopt, $
                'allowterm', allowtermopt, $
                'varyterm' , varytermopt, $
                'rms'      , fit_rms, $
                'maxabs'   , fit_maxabs $
          ) &$
  save, fufiOpt, file=SaveOptFile, /verbose &$
endif

; --- Write an IDL function ---
FunctionName=file_id
VarName=['logTeff', 'logg', 'M']
yName='Shift'
print, 'Create IDL function ' + FunctionName
fufi_wridl, FunctionName, varName, [0.0,0.0,0.0], [1.0,1.0,1.0], 0.0, 1.0, dname=yname

; --- Plot settings ---
;xsize=8.8 & ysize=7.0
xsize=8.8 & ysize=6.5
if (strupcase(dev) eq 'X') then begin & $
  wfactor=1000.0 &$
  set_plot, 'X' & $
  win_id=0 &$
  !P.CHARSIZE=0.0 &$
  ;window, win_id, xsize=xsize/18.0*wfactor, ysize=ysize/18.0*wfactor, retain=2 &$
  !P.FONT=-1 &$
endif else if (strupcase(dev) eq 'PS') then begin & $
  set_plot, 'ps' & $
  device, filename=ps_file, $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.6 &$
endif else begin &$ 
  set_plot, 'ps' & $
  device, filename=ps_file, $
          /encapsulated, CMYK=CMYK_flag, $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.7 &$
endelse
modctv, dev
;normct, 0

; --- Set global plot parameters ---
Pbak=!P & Xbak=!X & Ybak=!Y
!X.MARGIN=[6,1] & !Y.MARGIN=[4,1]

;; --- Plot original and fitted data over one quantity ---
;ix=0
;plotlines,reform(x[*,ix]),reform(y)   ,psym=1,linecol=!ctv.green,symsize=3
;plotlines,reform(x[*,ix]),reform(y+dy),psym=psymuser(132),linecol=!ctv.green,/over
;plotlines,reform(x[*,ix]),reform(y-dy),psym=psymuser(136),linecol=!ctv.green,/over
;plotlines,transpose([[reform(x[*,ix])],[reform(x[*,ix])]]),transpose([[reform(y)],[reform(yfufit)]]),/over,linestyle=1
;plotlines,reform(x[*,ix]),reform(yfufit),psym=102,/over,linecol=!ctv.red      

; --- Make 2D plot ---

nreg=100L
xreg=fltarr(nreg^2,3)
Treg   =(4.0*findgen(nreg)/float(nreg-1)+3.3)*1000.0
logTreg=alog10(Treg)-3.0
loggreg=       4.0*findgen(nreg)/float(nreg-1)+1.0
xreg[*,0]=rebin(reform(logTreg,nreg,1),nreg,nreg)
xreg[*,1]=rebin(reform(loggreg,1,nreg),nreg,nreg)
xreg[*,2]=Mreg

funct_string, xreg, aopt, yreg
; plotfield,/tv,logTreg,loggreg,reform(yreg,nreg,nreg),/oldasp,yRange=[max(loggreg),min(loggreg)],xRange=[max(logTreg),min(logTreg)],dRange=[-1,1]*0.35

; --- Change sign ---
yreg  =-yreg
y     =-y
yfufit=-yfufit

;dRange=[round(min(y)*10.)/10.-0.1,round(max(y)*10.)/10.+0.1]
;nlevels=5*fix((max(dRange)-min(dRange))/0.1)+1
dRange=[-0.32,0.2]
nlevels=27
levels=min(dRange)+findgen(nlevels)*0.02
levels[where(abs(levels) le 1.0e-6)]=0.0
;anno='!5' + strmid(string(levels, format='(F3.1)'),0,4) + '!X'
;anno='!5' + strtrim(string(levels, format='(F5.2)'),2) + '!X'
anno='!5' + strtrim(string(round(levels*1000.0), format='(I5)'),2) + '!X'
anno[bindgen(1+nlevels/5)*5+0]=''
anno[bindgen(nlevels/5)*5+2]=''
anno[bindgen(nlevels/5)*5+3]=''
anno[bindgen(nlevels/5)*5+4]=''
plotfield, Treg,loggreg,reform(yreg,nreg,nreg)<0.299, /oldasp, $
;  title=file_id + Met_id, $
  xtitle='T!Deff!N [K]', ytitle='log g', $
  xRange=[max(Treg),min(Treg)], yRange=[max(loggreg),min(loggreg)], dRange=[-0.6,0.33], $
;  /tv, $
  /contour, c_levels=levels, c_anno=anno, c_charsize=!P.CHARSIZE*0.92, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;plotfield, Treg,loggreg,reform(yreg,nreg,nreg)<0.299, /oldasp, $
;  xtitle='T!Deff!N [K]', ytitle='log g', $
;  xRange=[max(Treg),min(Treg)], yRange=[max(loggreg),min(loggreg)], dRange=[-0.6,0.33], $
;  /tv, $
;  dev=dev, psformat='nochange', /psnoinit, /noclose

xyouts, 0.2, 0.85, '[M/H]=' + strtrim(string(-Mreg,format='(I2)'),2), /norm, ali=0.5

for ix=0,nx-1 do begin &$
  if (abs(x[ix,2]-Mreg) le 0.1) then begin &$
    if (yfufit[ix] gt y[ix]) then begin &$
      xycolor=!ctv.green &$
    endif else begin &$
      xycolor=!ctv.red &$
    endelse &$
    ;
    xyouts, 10.0^[x[ix,0]+3.0], [x[ix,1]], $
      strtrim(string(y[ix]*1000.0, format='(I5)'),2), $
      charsize=!P.CHARSIZE*0.85, ali=0.5, color=xycolor &$
  endif &$
endfor

; --- Finish plot ---
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    ; --- Write PNG file ---
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    ; --- Write JPEG file ---
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

; end ; blueshift_fufi_fit
 
