; colorcorrect_fufi_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   colorcorrect_fufi_fit ('color_correction_function_find_fit')
;
; PURPOSE:
;   Fit color correction as function of Teff, logg by single function
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @colorcorrect_fufi_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @colorcorrect_fufi_fit
;
; MODIFICATION HISTORY:
;   2011-09-15 (B.F.) Written by Bernd Freytag, CRAL-ENS Lyon
;   2013-06-10 (B.F.) Adapted from blueshift_fufi_fit.pro
;   2017-01-27 (B.F. Uppsala) Adapted from wdcorr_fufi_fit.pro
;   2017-01-30 (B.F.) Compose complex function 'Combi01'
;   2017-02-02 (B.F.) Lower the weight of some models
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Select device ---
  dev='x' &$
  jpg_flag=0 &$
  png_flag=0 &$
  CMYK_flag=0 &$
  ;
  ; --- Specify data file name and local path ---
  InputBasePath=getenv('HOME') + '/dat/atm/ColorCorrections' &$
  ;InputDirName='UBVRIJHKHPBTVT_d3fits' & InputFileName='UBVRIJHKHPBTVT_d3m30' & InputFileSuffix='fits' &$
  InputDirName='UBVRIJHKHPBTVT_d3fits' & InputFileName='UBVRIJHKHPBTVT_d3m[0123]0' & InputFileSuffix='fits' &$
  ;
  ; --- Control function finding with fufi ---
  ;ColCorr_id  ='DUB_LHD' & tid='Polynomials'    & fufiTermMask=[[0]] &$      ; --- polynomials (in all variables)
  ;ColCorr_id  ='DUB_LHD' & tid='PolyExpGauss2D' & fufiTermMask=[[4,2,0]] &$  ; --- polynomials + 2D gaussian + exp
  ColCorr_id  ='DUB_LHD' &$
  ;ColCorr_id  ='DBV_LHD' &$
  ;ColCorr_id  ='DVR_LHD' &$
  ;ColCorr_id  ='DVI_LHD' &$
  ;ColCorr_id  ='DVK_LHD' &$
  ;ColCorr_id  ='DJK_LHD' &$
  ;ColCorr_id  ='DVHP_LHD' &$
  ;ColCorr_id  ='DBTVT_LHD' &$
                            tid='Polynomials'    & fufiTermMask=[[0]] &$      ; --- polynomials (in all variables)
  ;                          tid='PolyExp'        & fufiTermMask=[[1,0]] &$    ; --- polynomials + exp
  ;                          tid='PolyGauss2D'    & fufiTermMask=[[4,0]] &$    ; --- polynomials + 2D gaussian
  ;                          tid='PolyGauss1D'    & fufiTermMask=[[13,0]] &$   ; --- polynomials + 1D gaussian
  ;                          tid='PolyExpGauss2D' & fufiTermMask=[[4,2,0]] &$  ; --- polynomials + exp + 2D gaussian
  ;                          tid='Combi01'        & fufiTermMask=[[0]] &$   ; --- 
  ;
  fufiDepth=3 &$
  fufiNextraRandom=0 &$
  ;fufiNextraRandom=4 &$
  ;fufiNextraRandom=16 &$
  ;
  ; --- Compose file identifier ---
  IFN0=strreplaceBF(InputFileName, '[', '') &$
  IFN0=strreplaceBF(IFN0         , ']', '') &$
  IFN0=strreplaceBF(IFN0         , '?', '') &$
  IFN0=strreplaceBF(IFN0         , '*', '') &$
  IFN0=strreplaceBF(IFN0         , ',', '') &$
  file_id='ColCor_' + IFN0 + '_' + ColCorr_id + '_' + tid + '_nDepth' + trim(string(fufiDepth,format='(I3)'),2) &$
  ;
  ; --- Set file names for snapshots ---
  RestoreOptFile='' &$
  ;RestoreOptFile='ColCorUBVRIJHKHPBTVT_d3m00_DUB_LHD_Combi01_nDepth4.sav' &$
  ;RestoreOptFile=file_id + '.sav' &$
  ;SaveOptFile='' &$
  SaveOptFile=file_id + '.sav' &$
  ;
endif

; --- Create plot file names ---
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Read input data ---
FileList=file_search(InputBasePath + '/' + InputDirName + '/' + InputFileName + '.' + InputFileSuffix)
;
for iFile=0,n_elements(FileList)-1 do begin &$
  InputPathAndFile=FileList[iFile] &$
  print, iFile, ' Read ', InputPathAndFile &$
  b=mrdfits(InputPathAndFile, 1, Header) &$
  ;
  ; --- Put input data into fufi input arrays ---
  Teff0   =b.Teff &$
  logg0   =b.glog &$
  Met0    =b.Met &$
  execute_str='ColCorr0=b.' + ColCorr_id &$
  result=execute(execute_str) &$
  ;
  if (iFile eq 0) then begin &$
    Teff   =Teff0 &$
    logg   =logg0 &$
    Met    =Met0 &$
    ColCorr=ColCorr0 &$
  endif else begin &$
    Teff   =[Teff   , Teff0   ] &$
    logg   =[logg   , logg0   ] &$
    Met    =[Met    , Met0    ] &$
    ColCorr=[ColCorr, ColCorr0] &$
  endelse &$
endfor
nDim=(iFile+1) < 3

; --- Create first weight array ---
nx=n_elements(Teff)
w =fltarr(nx)
w[*  ]=1.0

; --- Restrict data ---
indloww=where(((abs(Teff - 6456.0) lt 1.0) and (abs(logg - 4.5) lt 0.01) and (abs(Met -   0.0 ) lt 0.01)) or $
              ((abs(Teff - 6484.0) lt 1.0) and (abs(logg - 4.0) lt 0.01) and (abs(Met -   0.0 ) lt 0.01)) or $
              ((abs(Teff - 6503.0) lt 1.0) and (abs(logg - 4.0) lt 0.01) and (abs(Met - (-1.0)) lt 0.01)) or $
              (     Teff le 3900.0) or $
              (     logg le 1.6   ), $
              nindloww)
if (nindloww gt 0) then begin &$
  w[indloww]=w[indloww]*0.01 &$
  indhighw=where(w ge 1.0, nindhighw) &$
  if (nindhighw gt 0) then begin &$
    Teff   =Teff[   indhighw] &$
    logg   =logg[   indhighw] &$
    Met    =Met[    indhighw] &$
    ColCorr=ColCorr[indhighw] &$
  endif &$
endif

; --- Create empty arrays for independent and dependent variables ---
nx=n_elements(Teff)
x =fltarr(nx, nDim)
y =fltarr(nx)

; --- Put input data into fufi input arrays ---
x[*,0]=[Teff-5000.0]/1000.0
x[*,1]=logg-3.0
if (ndim ge 3) then x[*,2]=Met
y     =ColCorr
;
help, nx, nindloww, indloww, Teff, logg, ColCorr, execute_str

; --- Find fitting function ---
; --- input: x,y,w; output: Fstropt, aopt ---
; --- control: TermMask, npar, depth, ... ---
npar=30 & nextramax=4
if (keyword_set(RestoreOptFile)) then begin &$
  restore, RestoreOptFile, /verbose &$
  fufiStart=fufiOpt &$
  functionfind, x, y, w, $
    Fstrstart=fufiStart.Fstr, PDERstrstart=fufiStart.PDERstr, astart=fufiStart.a, $
      allowtermstart=fufiStart.allowterm, varytermstart=fufiStart.varyterm, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=[[0]], npar=npar, depth=0, nextramax=nextramax, nextrara=fufiNextraRandom &$
  ;
endif else begin &$
  if (tid eq 'Combi01') then begin &$
    print, tid + ' poly 1' &$
    functionfind, x, y, w, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=[[14]], npar=npar, depth=1, $
        nextramax=0, nextrara=0 &$
    ;
    print &$
    print, tid + ' poly 2' &$
    FstrStart     =Fstropt &$
    PDERstrStart  =PDERstropt &$
    aStart        =aopt &$
    allowtermStart=allowtermopt &$
    varytermStart =varytermopt &$
    allowtermStart[*]=0 &$
    allowtermStart[0]=2 &$
    functionfind, x, y, w, $
      Fstrstart=FstrStart, PDERstrstart=PDERstrStart, astart=aStart, $
        allowtermstart=allowtermStart, varytermstart=varytermStart, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=[[13]], npar=npar, depth=5, $
        nextramax=0, nextrara=0 &$
    ;
    print &$
    print, tid + ' poly 3' &$
    FstrStart     =Fstropt &$
    PDERstrStart  =PDERstropt &$
    aStart        =aopt &$
    allowtermStart=allowtermopt &$
    varytermStart =varytermopt &$
    allowtermStart[*]=0 &$
    allowtermStart[1]=2 &$
    functionfind, x, y, w, $
      Fstrstart=FstrStart, PDERstrstart=PDERstrStart, astart=aStart, $
        allowtermstart=allowtermStart, varytermstart=varytermStart, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=[[13]], npar=npar, depth=2, $
        nextramax=0, nextrara=0 &$
    ;
    print &$
    print, tid + ' exp 1' &$
    FstrStart     =Fstropt &$
    PDERstrStart  =PDERstropt &$
    aStart        =aopt &$
    allowtermStart=allowtermopt &$
    varytermStart =varytermopt &$
    allowtermStart[*]=0 &$
    allowtermStart[0]=2 &$
    functionfind, x, y, w, $
      Fstrstart=FstrStart, PDERstrstart=PDERstrStart, astart=aStart, $
        allowtermstart=allowtermStart, varytermstart=varytermStart, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=[[15]], npar=npar, depth=1, $
        nextramax=nextramax, nextrara=4 &$
    ;
    print &$
    print, tid + ' poly 4' &$
    FstrStart     =Fstropt &$
    PDERstrStart  =PDERstropt &$
    aStart        =aopt &$
    allowtermStart=allowtermopt &$
    varytermStart =varytermopt &$
    allowtermStart[*]=0 &$
    allowtermStart[9]=2 &$
    aStart[       11]=-1.2 &$
    varytermStart[11]=0 &$
    functionfind, x, y, w, $
      Fstrstart=FstrStart, PDERstrstart=PDERstrStart, astart=aStart, $
        allowtermstart=allowtermStart, varytermstart=varytermStart, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=[[0]], npar=npar, depth=3, $
        nextramax=nextramax, nextrara=4 &$
    ;
  endif else begin &$
    functionfind, x, y, w, $
      Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
        allowtermopt=allowtermopt, varytermopt=varytermopt, $
      print_blank=0, print_F=1, print_a=0, print_iter=0, $
      TermMask=fufiTermMask, npar=npar, depth=fufiDepth, $
        nextramax=nextramax, nextrara=fufiNextraRandom &$
    ;
  endelse &$
endelse

; --- Compute the values of the fit function at the given points ---
funct_string, x, aopt, yfufit
;print, yfufit
;print, 'Total rms deviation: ', rms(w*(y-yfufit), /nonorm)/(total(w)/n_elements(w)), max(abs(w*(y-yfufit)))
fit_maxabs=max(abs(  (y-yfufit)))
fit_rms   =sqrt(mean((y-yfufit)^2))
y_rms     =sqrt(mean((y       )^2))
yfufit_rms=sqrt(mean((  yfufit)^2))
print, 'Total rms/max deviation, y_rms, yfit_rms: ', sqrt(avg((y-yfufit)^2)), max(abs((y-yfufit))), y_rms, yfufit_rms

if (keyword_set(SaveOptFile)) then begin &$
  ; --- Save optimum solution into structure and write it to file ---
  fufiOpt=create_struct( $
                'x'        , x, $
                'y'        , y, $
                'w'        , w, $
                'qid'      , 'colorcorrect', $
                'tid'      , tid, $
                'TermMask' , fufiTermMask, $
                'npar'     , npar, $
                'depth'    , fufiDepth, $
                'nextramax', nextramax, $
                'Fstr'     , Fstropt, $
                'PDerstr'  , PDERstropt, $
                'A'        , Aopt, $
                'allowterm', allowtermopt, $
                'varyterm' , varytermopt, $
                'rms'      , fit_rms, $
                'maxabs'   , fit_maxabs $
          ) &$
  save, fufiOpt, file=SaveOptFile, /verbose &$
endif

; --- Write an IDL function ---
FunctionName=file_id
yName='ColorCorrection'
print, 'Create IDL function ' + FunctionName
if (ndim le 2) then begin &$
  VarName=['Teff', 'logg'] &$
  fufi_wridl, FunctionName, varName, [5000.0, 3.0], [1000.0, 1.0], 0.0, 1.0, dname=yname &$
endif else begin &$
  VarName=['Teff', 'logg', 'Met'] &$
  fufi_wridl, FunctionName, varName, [5000.0, 3.0, 0.0], [1000.0, 1.0, 1.0], 0.0, 1.0, dname=yname &$
endelse


; end ; colorcorrect_fufi_fit
