; demo_fufi_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   demo_fufi_fit ('demonstration_function_find_fit')
;
; PURPOSE:
;   Create a 2D or 3D data set and fit it with FuFi.
;
; CATEGORY:
;   Function fitting
;
; CALLING SEQUENCE:
;   @demo_fufi_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @demo_fufi_fit
;
; MODIFICATION HISTORY:
;   2011-09-15 (B.F.) Written by Bernd Freytag, CRAL-ENS Lyon
;   2013-06-10 (B.F.) Adapted from blueshift_fufi_fit.pro
;   2017-01-27 (B.F. Uppsala) Adapted from wdcorr_fufi_fit.pro
;   2017-01-30 (B.F.) Compose complex function 'Combi01'
;   2017-02-02 (B.F.) Lower the weight of some models
;   2018-02-13 (B.F.) Adapted from colorcorrect_fufi_fit.pro
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Select device ---
  dev='x' &$
  jpg_flag=0 &$
  png_flag=0 &$
  CMYK_flag=0 &$
  ;
  ; --- Specify data file name and local path ---
  ;InputPath=getenv('HOME') + '/dat' &$
  InputFileName='Dummy01' &$
  ;
  ; --- Control function finding with fufi ---
  nDim=2 & Test_id  ='test2d01' &$
  ; tid='Polynomials'    & fufiTermMask=[[0]] &$      ; --- polynomials (in all variables)
  tid='PolyExp'        & fufiTermMask=[[2,0]] &$    ; --- polynomials + exp
  ; tid='PolyGauss2D'    & fufiTermMask=[[4,0]] &$    ; --- polynomials + 2D gaussian
  ; tid='PolyExpGauss2D' & fufiTermMask=[[4,2,0]] &$  ; --- polynomials + exp + 2D gaussian
  ; tid='Polyterms'    & fufiTermMask=[[13,14]] &$      ; --- polynomials (in all variables)
  ;
  ; nDim=3 & Test_id  ='test3d01' &$
  ; tid='Polynomials'    & fufiTermMask=[[0]] &$      ; --- polynomials (in all variables)
  ; tid='PolyExp'        & fufiTermMask=[[1,0]] &$    ; --- polynomials + exp
  ; tid='Polyterms'    & fufiTermMask=[[2,3,4]] &$      ; --- polynomials (in all variables)
  ;
  fufiDepth=2 &$
  fufiNextraRandom=0 &$
  ;fufiNextraRandom=4 &$
  NextraMax=1 &$
  ;
  ; --- Compose file identifier ---
  IFN0=strreplaceBF(InputFileName, '[', '') &$
  IFN0=strreplaceBF(IFN0         , ']', '') &$
  IFN0=strreplaceBF(IFN0         , '?', '') &$
  IFN0=strreplaceBF(IFN0         , '*', '') &$
  IFN0=strreplaceBF(IFN0         , ',', '') &$
  File_id='Demo' + strtrim(string(nDim),2) + 'D_' + IFN0 + '_' + Test_id + '_' + $
          tid + '_nDepth' + trim(string(fufiDepth,format='(I3)'),2) &$
  ;
  ; --- Set file names for snapshots ---
  RestoreOptFile='' &$
  ;RestoreOptFile='Previous.sav' &$
  ;RestoreOptFile=File_id + '.sav' &$
  ;SaveOptFile='' &$
  SaveOptFile=File_id + '.sav' &$
  ;
endif

; --- Create plot file names ---
jpg_file=File_id + '.jpg'
png_file=File_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =File_id + '.eps' &$
endif else begin &$
  ps_file  =File_id + '.ps' &$
endelse

; === Read input data ===
;FileList=file_search(InputBasePath + '/' + InputDirName + '/' + InputFileName + '.' + InputFileSuffix)
;...

; === Construct input data ===
; --- Set number of data points ---
np1 =5
np2 =5
np3 =((nDim le 2) ? 1 : 4)
nx  =np1*np2*np3
; --- Compute number of dimensions ---
;nDim=2 + (np3 gt 1)
; --- Create and fill weight array ---
w   =fltarr(nx)
w[*]=1.0
; --- Create empty arrays for independent and dependent variables ---
x     =fltarr(nx, nDim)
y     =fltarr(nx)
; --- Put data into fufi input arrays ---
x[  *,0]=reform(rebin(reform(findgen(np1), np1,  1,  1), np1,np2,np3))
x[  *,1]=reform(rebin(reform(findgen(np2),   1,np2,  1), np1,np2,np3))
; --- Produce some test data ---
if (nDim le 2) then begin &$
  y     =x[*,0]^2 + x[*,1]^4 &$
endif else begin &$
  x[*,2]=reform(rebin(reform(findgen(np3),   1,  1,np3), np1,np2,np3)) &$
  y     =x[*,0]^2 + x[*,1]^4 + x[*,2]^3 &$
endelse
;
help, nx, x, y, w

; --- Find fitting function ---
; --- input: x,y,w; output: Fstropt, aopt ---
; --- control: TermMask, npar, depth, ... ---
npar=30
if (keyword_set(RestoreOptFile)) then begin &$
  ; --- Start from state from restore file ---
  restore, RestoreOptFile, /verbose &$
  fufiStart=fufiOpt &$
  functionfind, x, y, w, $
    Fstrstart=fufiStart.Fstr, PDERstrstart=fufiStart.PDERstr, astart=fufiStart.a, $
      allowtermstart=fufiStart.allowterm, varytermstart=fufiStart.varyterm, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=[[0]], npar=npar, depth=0, nextramax=NextraMax, nextrara=fufiNextraRandom &$
  ;
endif else begin &$
  ; --- Start from scratch ---
  functionfind, x, y, w, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=1, print_iter=0, $
    TermMask=fufiTermMask, npar=npar, depth=fufiDepth, $
      nextramax=NextraMax, nextrara=fufiNextraRandom &$
    ;
endelse

; --- Compute the values of the fit function at the given points ---
funct_string, x, aopt, yfufit
;print, yfufit
;print, 'Total rms deviation: ', rms(w*(y-yfufit), /nonorm)/(total(w)/n_elements(w)), max(abs(w*(y-yfufit)))
fit_maxabs=max(abs(  (y-yfufit)))
fit_rms   =sqrt(mean((y-yfufit)^2))
y_rms     =sqrt(mean((y       )^2))
yfufit_rms=sqrt(mean((  yfufit)^2))
print, 'Total rms/max deviation, y_rms, yfit_rms: ', sqrt(avg((y-yfufit)^2)), max(abs((y-yfufit))), y_rms, yfufit_rms

if (keyword_set(SaveOptFile)) then begin &$
  ; --- Save optimum solution into structure and write it to file ---
  fufiOpt=create_struct( $
                'x'        , x, $
                'y'        , y, $
                'w'        , w, $
                'qid'      , 'demo' + strtrim(string(nDim),2) + 'd', $
                'tid'      , tid, $
                'TermMask' , fufiTermMask, $
                'npar'     , npar, $
                'depth'    , fufiDepth, $
                'nextramax', NextraMax, $
                'Fstr'     , Fstropt, $
                'PDerstr'  , PDERstropt, $
                'A'        , Aopt, $
                'allowterm', allowtermopt, $
                'varyterm' , varytermopt, $
                'rms'      , fit_rms, $
                'maxabs'   , fit_maxabs $
          ) &$
  save, fufiOpt, file=SaveOptFile, /verbose &$
endif

; --- Write an IDL function ---
FunctionName=File_id
yName='VarOut'
print, 'Create IDL function ' + FunctionName
if (ndim le 2) then begin &$
  VarName=['Var1', 'Var2'] &$
  fufi_wridl,  FunctionName, varName, [0.0, 0.0], [1.0, 1.0], 0.0, 1.0, dname=yname &$
  fufi_wrfort, FunctionName, varName, [0.0, 0.0], [1.0, 1.0], 0.0, 1.0 &$
endif else begin &$
  ;VarName=['Teff', 'logg', 'Met'] &$
  VarName=['Var1', 'Var2', 'Var3'] &$
  fufi_wridl,  FunctionName, varName, [0.0, 0.0, 0.0], [1.0, 1.0, 1.0], 0.0, 1.0, dname=yname &$
  fufi_wrfort, FunctionName, varName, [0.0, 0.0, 0.0], [1.0, 1.0, 1.0], 0.0, 1.0 &$
endelse


; end ; demo_fufi_fit
