; ewn_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   ewn_fit ('equivalent_width_non_local_thermal_equilibrium_fit')
;
; PURPOSE:
;   Fit equivalent width as function of Teff, logg, FeH by single function
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @ewn_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @ewn_fit
;
; MODIFICATION HISTORY:
;   2009-04-17 Written by Bernd Freytag, CRAL-ENS Lyon
;   2009-04-30 (B.F.) "exact" effective temperatures for 3D included
;   2009-05-04 (B.F.) Corrected effective temperatures for 3D
;-
;------------------------------------------------------------------------------

; --- Restore data ---
restore,'EW_NLTE.idlsave',/verb
ewneltemp=create_struct(ewn[0], 'model', '', 'Teffexact', 0.0, 'Teffrms', 0.0)
ewnext=replicate(ewneltemp, n_elements(ewn.Teff))
struct_assign, ewn, ewnext
ewn=ewnext
ewnadd=[ $
create_struct('model', 'd3t55g35mm20n01', 'Teffexact', 5505.13, 'Teffrms', 14.0759), $
create_struct('model', 'd3t55g40mm20n01', 'Teffexact', 5472.38, 'Teffrms', 7.45108), $
create_struct('model', 'd3t55g45mm20n01', 'Teffexact', 5478.73, 'Teffrms', 5.86229), $
create_struct('model', 'd3t59g35mm20n01', 'Teffexact', 5860.75, 'Teffrms', 11.7534), $
create_struct('model', 'd3t59g40mm20n02', 'Teffexact', 5855.68, 'Teffrms', 6.98436), $
create_struct('model', 'd3t59g40mm30n02', 'Teffexact', 5845.96, 'Teffrms', 7.76349), $
create_struct('model', 'd3t59g45mm20n01', 'Teffexact', 5923.39, 'Teffrms', 7.16144), $
create_struct('model', 'd3t59g45mm30n01', 'Teffexact', 5923.90, 'Teffrms', 7.00000), $
create_struct('model', 'd3t63g35mm20n01', 'Teffexact', 6286.53, 'Teffrms', 13.3011), $
create_struct('model', 'd3t63g40mm20n01', 'Teffexact', 6278.45, 'Teffrms', 11.1918), $
create_struct('model', 'd3t63g40mm30n01', 'Teffexact', 6269.19, 'Teffrms', 9.78769), $
create_struct('model', 'd3t63g45mm20n01', 'Teffexact', 6322.99, 'Teffrms', 10.4541), $
create_struct('model', 'd3t63g45mm30n01', 'Teffexact', 6272.29, 'Teffrms', 9.49944), $
create_struct('model', 'd3t65g40mm20n01', 'Teffexact', 6533.98, 'Teffrms', 12.7246), $
create_struct('model', 'd3t65g40mm30n01', 'Teffexact', 6408.38, 'Teffrms', 10.7966), $
create_struct('model', 'd3t65g45mm20n01', 'Teffexact', 6532.88, 'Teffrms', 8.22062), $
create_struct('model', 'd3t65g45mm30n01', 'Teffexact', 6555.97, 'Teffrms', 13.9100)]
ewn.Teffexact=ewnadd.Teffexact
ewn.Teffrms  =ewnadd.Teffrms
ewn.model    =ewnadd.model
; --- Correct Teff for one model ---
ewn[where(ewn.model eq 'd3t63g40mm30n01')].Teff=6270.0

; 2009-05-04:
; d3t55g35mm20n01   Teff [K]      5505.13 RMS:       14.0759
; d3t55g40mm20n01   Teff [K]      5472.38 RMS:       7.45108
; d3t55g45mm20n01   Teff [K]      5478.73 RMS:       5.86229
; d3t59g35mm20n01   Teff [K]      5860.75 RMS:       11.7534
; d3t59g40mm20n02   Teff [K]      5855.68 RMS:       6.98436
; d3t59g40mm30n02   Teff [K]      5845.96 RMS:       7.76349
; d3t59g45mm20n01   Teff [K]      5923.39 RMS:       7.16144
; d3t59g45mm30n01   Teff [K]      5923.90 RMS:       7.00
; d3t63g35mm20n01   Teff [K]      6286.53 RMS:       13.3011
; d3t63g40mm20n01   Teff [K]      6278.45 RMS:       11.1918
; d3t63g40mm30n01   Teff [K]      6269.19 RMS:       9.78769
; d3t63g45mm20n01   Teff [K]      6322.99 RMS:       10.4541
; d3t63g45mm30n01   Teff [K]      6272.29 RMS:       9.49944
; d3t65g40mm20n01   Teff [K]      6533.98 RMS:       12.7246
; d3t65g40mm30n01   Teff [K]      6408.38 RMS:       10.7966
; d3t65g45mm20n01   Teff [K]      6532.88 RMS:       8.22062
; d3t65g45mm30n01   Teff [K]      6555.97 RMS:       13.91
;
; OLD:
; d3t55g35mm20n01   Teff [K]      5505.13 RMS:       14.0759
; d3t55g40mm20n01   Teff [K]      5472.38 RMS:       7.45108
; d3t55g45mm20n01   Teff [K]      5478.73 RMS:       5.86229
; d3t59g35mm20n01   Teff [K]      5860.75 RMS:       11.7534
; d3t59g40mm20n02   Teff [K]      5860.75 RMS:       11.7534
; d3t59g40mm30n02   Teff [K]      5855.68 RMS:       6.98436
; d3t59g45mm20n01   Teff [K]      5923.39 RMS:       7.16144
; d3t59g45mm30n01                 5920.00            0.0
; d3t63g35mm20n01   Teff [K]      6286.53 RMS:       13.3011
; d3t63g40mm20n01   Teff [K]      6278.45 RMS:       11.1918
; d3t63g40mm30n01   Teff [K]      6269.19 RMS:       9.78769
; d3t63g45mm20n01   Teff [K]      6322.99 RMS:       10.4541
; d3t63g45mm30n01   Teff [K]      6272.29 RMS:       9.49944
; d3t65g40mm20n01   Teff [K]      6533.98 RMS:       12.7246
; d3t65g40mm30n01   Teff [K]      6408.38 RMS:       10.7966
; d3t65g45mm20n01   Teff [K]      6532.88 RMS:       8.22062
; d3t65g45mm30n01   Teff [K]      6500.00 RMS:       0.0

; --- Ew1, Ew3, Ew1I, Ew3I ---
qid='Ew1'

; --- Possibly restrict list of models ---
;ewn=ewn[where(ewn.Teff lt 5700.0)]
;ewn=ewn[where(abs(ewn.Teff-6000.0) lt 200.0)]
;ewn=ewn[where(ewn.Teff ne 6240.0)]
;ewn=ewn[where(ewn.model ne 'd3t63g45mm30n01')]
;ewn=ewn[where(ewn.model ne 'd3t65g40mm20n01')]

; --- Fiddling with model Teff ---
;ewn[10].Teff=ewn[10].Teff+30.0

; --- Number of models ---
n=n_elements(ewn.Teff)

v     =reform(ewn[*].LiAbu)
v     =v[sort(v)]
xLiAbu=v[uniq(v)]

; --- Empty arrays for independent and dependent variables ---
x=fltarr(3,n)
y=fltarr(n)
;y3=fltarr(n)

; --- Loop over all models to get independent and dependent variables ---
for i=0,n-1 do begin &$
;  x[0,i]=alog10(ewn[i].Teff) &$
  x[0,i]=alog10(ewn[i].Teffexact) &$
  x[1,i]=       ewn[i].logg &$
  x[2,i]=       ewn[i].FeH &$
  dummy=execute('q=alog10(ewn[i].' + qid + ')') &$
  ;
  xq=ewn[i].LiAbu &$
  ;ind=where(abs(ewn[i].LiAbu-2.1) le 1.0E-02) & y[i]=q[ind] &$
  y[i]=reform(interpol(xq, q, [1.3])) &$
  print, i, x[*,i], y[i], format='(I3,1X,3E11.3,2X,E13.6)' &$
  ;
  plotlines, xq, q, thick=1, over=(i gt 0), $
;    xrange=[0.8, 3.4], drange=[-1.2,2.1] &$
    xrange=[-1.1, 1.1], drange=[0.2,2.1] &$
  ;
  ;cPoly=poly_fit(xq, q, 5, /double, yfit=yfitpoly) &$
  ; plotlines, xq, yfitpoly, /over, linecol=!ctv.green &$
  ;plotlines, xLiAbu, poly(xLiAbu, cPoly), /over, linecol=!ctv.green &$
  ; xinfl=-cpoly[2]/(3.0*cpoly[3]) &$
  ; yinfl=poly(xinfl,cpoly) &$
  ; print, cpoly, xinfl, yinfl, format='(4X,4E12.4,2x,2E12.4)' &$
  ; plotlines, [xinfl], [yinfl], psym=1, /over, linecol=!ctv.green &$
endfor ; i

; --- Do a multi-D linear regression ---
coeff=regress(x, y, const=coeff0, yfit=yfit) & yfit=reform(yfit)
; yfit=reform(coeff#x)+coeff0

; === Find fitting function ===
functionfind, transpose(x), y, $
  termmask=[[0]], npar=17, depth=2, nextramax=1, nextraran=1, $
  Fstropt=Fstropt1, PDERstropt=PDERstropt1, aopt=aopt1
;    allowtermopt=allowtermopt1, varytermopt=varytermopt1
funct_string, transpose(x), aopt1, yfufit
print,yfufit

for i=0,n-1 do begin &$
  dummy=execute('q=alog10(ewn[i].' + qid + ')') &$
  plotlines, ewn[i].LiAbu-yfufit[i], q, $
;  plotlines, ewn[i].LiAbu-yfit[i], q, $
;  plotlines, ewn[i].LiAbu-y[i], q, $
;  plotlines, ewn[i].LiAbu, q-yfit[i], $
;  plotlines, ewn[i].LiAbu, q-yfufit[i], $
    /over, linecol=!ctv.red &$
endfor ; i

; --- Plot the error over Teff ---
;plot, ewn.Teff, y-yfufit, psym=1, xtitle='Teff', ytitle='Fit error'
print, 'Total rms deviation: ', rms(y-yfufit)

; plotlines,alog10(ewn.ew1I),alog10(ewn.ew1/ewn.ew1i),psym=1
; plotlines,alog10(ewn.ew3I),alog10(ewn.ew3/ewn.ew3i),psym=1,/over,linecol=!ctv.blue
; plotlines,alog10(ewn.ew1XI),alog10(ewn.ew1XF/ewn.ew1xi),psym=1,/over,linecol=!ctv.red


;plotlines,ewn.logg,y3,psym=1
;plotlines,ewn.logg,reform(y3fit),psym=psymuser(102),/over,linecol=!ctv.green
;plotlines,ewn.Teff,y3,psym=1
;plotlines,ewn.Teff,reform(y3fit),psym=psymuser(102),/over,linecol=!ctv.green
;plotlines,ewn.Feh,y3,psym=1
;plotlines,ewn.Feh,reform(y3fit),psym=psymuser(102),/over,linecol=!ctv.green 

; end ; ewn_fit
