; ewn_fufi_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   ewn_fufi_fit ('equivalent_width_n_function_find_fit')
;
; PURPOSE:
;   Fit equivalent width as function of Teff, logg, FeH by single function
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @ewn_fufi_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @ewn_fufi_fit
;
; MODIFICATION HISTORY:
;   2009-04-21 Written by Bernd Freytag, CRAL-ENS Lyon
;   2009-05-01 (B.F.) "exact" effective temperatures for 3D included
;   2009-05-04 (B.F.) Corrected effective temperatures for 3D
;   2009-05-10 (B.F.) Separate main control into extra block, write protocol file
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Data file name  ---
  ;RestoreFileName='EW_NLTE.idlsave' &$
  RestoreFileName='EW_combi.sav' &$
  ;
  ; --- Type of equivalent width: Ew1, Ew3, Ew1I, Ew3I, Ew1XF, Ew1XI; Ewl1 ---
  qid='Ewl1XF' &$
  ;
  ; --- What is input, what is output ---
  ;FitSense='logEw_to_logAbu' &$
  ;FitSense='logEw_to_dlogAbu' &$
  FitSense='logAbu_to_logEW' &$
  ;
  ; --- Control for function finding with fufi ---
  ;tid='polynomials' & fufiTermMask=[[0]] &$         ; --- polynomials (in all variables)
  ;fufiNextraRandom=0 &$
  ;tid='c+id+exp(x)' & fufiTermMask=[[4],[2]] &$     ; --- const + identity + exponential
  tid='loglog-absrpt-2p' & fufiTermMask=[[8],[2]] &$ ; --- loglog absorption model (2 params)
  ;tid='loglog-absrpt-3p' & fufiTermMask=[[9],[2]] &$ ; --- loglog absorption model (3 params)
  ;tid='c+exp(x)-3p' & fufiTermMask=[[10],[2]] &$     ; --- const + exponential
  fufiDepth=5 &$
  fufiNextraRandom=16 &$
  ;
  RestoreOptFile='' &$
  ;RestoreOptFile='ewn_Ew3_opt_01.sav' &$
  SaveOptFile='' &$
  ;SaveOptFile='ewn_' + qid + '_opt_' + strtrim(string(fufiTermMask[0]),2) + '-' + strtrim(string(fufiDepth),2) + '.sav' &$
  ;SaveOptFile='ewn_Ew3_opt_8-4.sav' &$
endif

if (RestoreFileName eq 'EW_combi.sav') then begin &$
  ; --- Simply restore the combined data ---
  ;
  restore, RestoreFileName, /verbose &$
endif else begin &$
  ; --- Produce the combined data from the three individual data sets ---
  ;
  ; --- Restore NLTE data ---
  restore, RestoreFileName, /verbose &$
  ; --- Number of models ---
  n   =n_elements(ewn.Teff) &$
  nabu=n_elements(ewn[0].LiAbu) &$
  ; --- Load LTE data ---
  restore, '20090505_Luca/EW_LTE.idlsave', /verbose &$
  ; --- Create single array entry and complete array with combined data ---
  ewneltemp=create_struct(ewn[0], $
                          'model', '', 'Teffexact', 0.0, 'Teffrms', 0.0, $
                          'ewl3' , fltarr(nabu), 'ewl1' , fltarr(nabu), 'ewl1xf', fltarr(nabu), $
                          'ewl3i', fltarr(nabu), 'ewl1i', fltarr(nabu), 'ewl1xi', fltarr(nabu)) &$
  ewnext=replicate(ewneltemp, n_elements(ewn.Teff)) &$
  ; --- Copy NLTE data over ---
  struct_assign, ewn, ewnext &$
  ewn=ewnext &$
  ; --- Put additional data into structure ---
  ewnadd=[ $
  create_struct('model', 'd3t55g35mm20n01', 'Teffexact', 5505.13, 'Teffrms', 14.0759), $
  create_struct('model', 'd3t55g40mm20n01', 'Teffexact', 5472.38, 'Teffrms', 7.45108), $
  create_struct('model', 'd3t55g45mm20n01', 'Teffexact', 5478.73, 'Teffrms', 5.86229), $
  create_struct('model', 'd3t59g35mm20n01', 'Teffexact', 5860.75, 'Teffrms', 11.7534), $
  create_struct('model', 'd3t59g40mm20n02', 'Teffexact', 5855.68, 'Teffrms', 6.98436), $
  create_struct('model', 'd3t59g40mm30n02', 'Teffexact', 5845.96, 'Teffrms', 7.76349), $
  create_struct('model', 'd3t59g45mm20n01', 'Teffexact', 5923.39, 'Teffrms', 7.16144), $
  create_struct('model', 'd3t59g45mm30n01', 'Teffexact', 5923.90, 'Teffrms', 7.00000), $
  create_struct('model', 'd3t63g35mm20n01', 'Teffexact', 6286.53, 'Teffrms', 13.3011), $
  create_struct('model', 'd3t63g40mm20n01', 'Teffexact', 6278.45, 'Teffrms', 11.1918), $
  create_struct('model', 'd3t63g40mm30n01', 'Teffexact', 6269.19, 'Teffrms', 9.78769), $
  create_struct('model', 'd3t63g45mm20n01', 'Teffexact', 6322.99, 'Teffrms', 10.4541), $
  create_struct('model', 'd3t63g45mm30n01', 'Teffexact', 6272.29, 'Teffrms', 9.49944), $
  create_struct('model', 'd3t65g40mm20n01', 'Teffexact', 6533.98, 'Teffrms', 12.7246), $
  create_struct('model', 'd3t65g40mm30n01', 'Teffexact', 6408.38, 'Teffrms', 10.7966), $
  create_struct('model', 'd3t65g45mm20n01', 'Teffexact', 6532.88, 'Teffrms', 8.22062), $
  create_struct('model', 'd3t65g45mm30n01', 'Teffexact', 6555.97, 'Teffrms', 13.9100)] &$
  ; --- Copy additional data and LTE data ---
  ewn.Teffexact=ewnadd.Teffexact &$
  ewn.Teffrms  =ewnadd.Teffrms &$
  ewn.model    =ewnadd.model &$
  ewn.ewl3     =ewl.ew3 &$
  ewn.ewl1     =ewl.ew1 &$
  ewn.ewl1XF   =ewl.ew1XF &$
  ewn.ewl3i    =ewl.ew3i &$
  ewn.ewl1i    =ewl.ew1i &$
  ewn.ewl1XI   =ewl.ew1XI &$
  ; --- Correct Teff for one model ---
  ewn[where(ewn.model eq 'd3t63g40mm30n01')].Teff=6270.0 &$
endelse

; --- Save result file ---
;save, ewn, filename='EW_combi.sav'

; --- Possibly restrict list of models ---
;ewn=ewn[where(ewn.Teff lt 6500.0)]
;ewn=ewn[where(abs(ewn.Teff-6000.0) lt 200.0)]
;ewn=ewn[where(ewn.Teff gt 6500.0)]
;ewn=ewn[where(ewn.Teff ne 6240.0)]
;ewn=ewn[where(ewn.logg eq 3.5)]
;ewn=ewn[where(ewn.FeH eq -2.0)]
;ewn=ewn[13]

; --- Fiddling with model Teff ---
;ewn[ 5].Teffexact=ewn[ 5].Teffexact-10.0
;ewn[13].Teffexact=ewn[13].Teffexact+10.0
;ewn[ 0].Teff=ewn[ 0].Teff-12.0
;ewn[ 6].Teff=ewn[ 6].Teff+10.0
;ewn[ 8].Teff=ewn[ 8].Teff+10.0
;ewn[10].Teff=ewn[10].Teff+30.0
;ewn[12].Teff=ewn[12].Teff+10.0

; --- Number of models ---
n   =n_elements(ewn.Teff)
nabu=n_elements(ewn[0].LiAbu)

; --- Empty arrays for independent and dependent variables ---
x=fltarr(4,n*nabu)
y=fltarr(n*nabu)
w=replicate(1.0,nabu,n)
qidl=strlowcase(qid)
;if (qidl eq 'ew3') then begin &$
;  ind=where(ewn.model eq 'd3t65g40mm20n01', count) &$
;  if (count ge 1) then w[6,ind]=0.0 &$
;endif
;w[5:*,*]=0.0
;w[*,10]=0.0
w=reform(w, n*nabu)

xfactor=[1.0, 1.0, 1.0,  1.0] & yfactor=1.0
xoffset=[0.0, 0.0, 0.0,  0.0] & yoffset=0.0
;xoffset=[2.0, 3.5, 4.0, -2.5] & yoffset=1.3
; --- Loop over all models to get independent and dependent variables ---
for i=0,n-1 do begin &$
  ; --- Use different source for effective temperature for LHD models (containing 'x' in the name) ---
  if (strpos(qidl, 'x') eq -1) then begin &$
    x[1,i*nabu:i*nabu+nabu-1]=alog10(ewn[i].Teffexact)-xoffset[1] &$
  endif else begin &$
    x[1,i*nabu:i*nabu+nabu-1]=alog10(ewn[i].Teff     )-xoffset[1] &$
  endelse &$
  ;
  ; --- Gravity and metalicity ---
  x[2,i*nabu:i*nabu+nabu-1]  =       ewn[i].logg      -xoffset[2] &$
  x[3,i*nabu:i*nabu+nabu-1]  =       ewn[i].FeH       -xoffset[3] &$
  ;
  ; --- EW and abundance ---
  if (FitSense eq 'logAbu_to_logEW') then begin &$
    x[0,i*nabu:i*nabu+nabu-1]=       ewn[i].LiAbu-xoffset[0] &$
    ;x[0,i*nabu:i*nabu+nabu-1]=   10.0^ewn[i].LiAbu &$
    ;dummy=execute('q=ewn[i].' + qid) &$
    dummy=execute('q=alog10(ewn[i].' + qid + ')') &$
    y[  i*nabu:i*nabu+nabu-1]=       q-yoffset &$
    ;
    plotlines, reform(x[0,i*nabu:i*nabu+nabu-1])+xoffset[0], y[i*nabu:i*nabu+nabu-1]+yoffset, $
      thick=1, xrange=[0.8, 3.4], drange=[0.2,2.5], over=(i gt 0) &$
    ;plotlines, alog10(reform(x[0,i*nabu:i*nabu+nabu-1])), alog10(y[i*nabu:i*nabu+nabu-1]), thick=1, $
       ;xrange=10.0^[0.8, 3.4], drange=10.0^[0.2,2.1], over=(i gt 0) &$
    ;
  endif else if (FitSense eq 'logEw_to_logAbu') then begin &$
    y[  i*nabu:i*nabu+nabu-1]=       ewn[i].LiAbu-yoffset[0] &$
    dummy=execute('q=alog10(ewn[i].' + qid + ')') &$
    x[0,i*nabu:i*nabu+nabu-1]=       q-xoffset[0] &$
    ;
    plotlines, y[i*nabu:i*nabu+nabu-1]+yoffset, reform(x[0,i*nabu:i*nabu+nabu-1])+xoffset[0], $
      thick=1, xrange=[0.8, 3.4], drange=[0.2,2.5], over=(i gt 0) &$
    ;
  endif else if (FitSense eq 'logEw_to_dlogAbu') then begin &$
    ;y[  i*nabu:i*nabu+nabu-1]=       ewn[i].LiAbu-yoffset[0] &$
    logAbu=ewn[i].LiAbu &$
    logEw =alog10(ewn[i].Ew1XF ) &$
    loglEw=alog10(ewn[i].Ewl1XF) &$
    dlogEwdlogAbu=deriv(logAbu, 0.5*(logEw+loglEw)) &$
    y[  i*nabu:i*nabu+nabu-1]=(loglEw-logEw)/dlogEwdlogAbu-yoffset[0] &$
    ;
    dummy=execute('q=alog10(ewn[i].' + qid + ')') &$
    x[0,i*nabu:i*nabu+nabu-1]=       q-xoffset[0] &$
    ;
    plotlines, y[i*nabu:i*nabu+nabu-1]+yoffset, reform(x[0,i*nabu:i*nabu+nabu-1])+xoffset[0], $
      thick=1, xrange=[-0.07, 0.07], drange=[0.2,2.5], over=(i gt 0) &$
    ;
  endif else begin &$
    print, 'Unknown FitSense: ' + FitSense &$
    stop &$
  endelse &$
  ;
  print, i, x[*,i*nabu], y[i*nabu], format='(I3,1X,4E11.3,2X,E13.6)' &$
  ;
endfor ; i

; --- Modify weights ---
;w=w/y^6

; --- Do a multi-D linear regression (just as sanity check; it is not needed for fufi) ---
;coeff=regress(x, y, const=coeff0, yfit=yfit) & yfit=reform(yfit)
; yfit=reform(coeff#x)+coeff0

; --- Find fitting function ---
; --- input: x,y; output: Fstropt, aopt ---
; --- control: TermMask, npar, depth, ... ---
npar=30 & nextramax=4
if (keyword_set(RestoreOptFile)) then begin &$
  restore, RestoreOptFile, /verbose &$
  fufiStart=fufiOpt &$
  functionfind, transpose(x), y, w, $
    Fstrstart=fufiStart.Fstr, PDERstrstart=fufiStart.PDERstr, astart=fufiStart.a, $
      allowtermstart=fufiStart.allowterm, varytermstart=fufiStart.varyterm, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=[[0]], npar=npar, depth=0, nextramax=nextramax, nextrara=fufiNextraRandom &$
endif else begin &$
  functionfind, transpose(x), y, w, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=fufiTermMask, npar=npar, depth=fufiDepth, nextramax=nextramax, nextrara=fufiNextraRandom &$
endelse

; --- Compute the values of the fit function at the given points ---
funct_string, transpose(x), aopt, yfufit
print, yfufit
;print, 'Total rms deviation: ', rms(w*(y-yfufit), /nonorm)/(total(w)/n_elements(w)), max(abs(w*(y-yfufit)))
fit_rms=sqrt(avg((y-yfufit)^2))
fit_maxabs=max(abs((y-yfufit)))
print, 'Total rms deviation: ', sqrt(avg((y-yfufit)^2)), max(abs((y-yfufit)))

if (keyword_set(SaveOptFile)) then begin &$
  ; --- Save optimum solution into structure and write it to file ---
  fufiOpt=create_struct( $
                'x'        , transpose(x), $
                'y'        , y, $
                'w'        , w, $
                'qid'      , qid, $
                'tid'      , tid, $
                'TermMask' , fufiTermMask, $
                'npar'     , npar, $
                'depth'    , fufiDepth, $
                'nextramax', nextramax, $
                'Fstr'     , Fstropt, $
                'PDerstr'  , PDERstropt, $
                'A'        , Aopt, $
                'allowterm', allowtermopt, $
                'varyterm' , varytermopt, $
                'rms'      , fit_rms, $
                'maxabs'   , fit_maxabs $
          ) &$
  save, fufiOpt, file=SaveOptFile, /verbose &$
endif

if (FitSense eq 'logAbu_to_logEW') then begin &$
  plotlines, reform(x[0,*])+xoffset[0], yfufit+yoffset, /over, psym=102, linecol=!ctv.red &$
  ;plotlines, alog10(reform(x[0,*])), alog10(yfufit), /over, psym=102, linecol=!ctv.red
endif else begin &$
  plotlines, yfufit+yoffset, reform(x[0,*])+xoffset[0], /over, psym=102, linecol=!ctv.red &$
endelse

; --- Write an IDL function ---
if (FitSense eq 'logAbu_to_logEW') then begin &$
  functionname='ewnfit_' + strlowcase('LiAbu') + '_to_' + qidl &$
  varname=['logLiAbu', 'logTeff', 'logg', 'logFeH'] &$
  yname  = 'log' + qid &$
  print, 'Create IDL function ' + functionname + '.pro' &$
  ;fufi_wridl, functionname, varname, xoffset, xfactor, yoffset, yfactor, dname=yname &$
  ;.run functionname &$
  ;yres=call_function(functionname+'', reform(x[0,*]), reform(x[1,*]), reform(x[2,*]), reform(x[3,*])) &$
  ;plotlines, reform(x[0,*])+xoffset[0], yres, /over, psym=1, linecol=!ctv.green &$
endif else if (FitSense eq 'logEW_to_logAbu') then begin &$
  functionname='ewnfit_' + qidl + '_to_' + strlowcase('LiAbu') &$
  varname=['log' + qid, 'logTeff', 'logg', 'logFeH'] &$
  yname  = 'logLiAbu' &$
  print, 'Create IDL function ' + functionname + '.pro' &$
  ;fufi_wridl, functionname, varname, xoffset, xfactor, yoffset, yfactor, dname=yname &$
  ;.run functionname &$
  ;yres=call_function(functionname+'', double(reform(x[0,*])), double(reform(x[1,*])), double(reform(x[2,*])), double(reform(x[3,*]))) &$
  ;plotlines, yres, reform(x[0,*])+xoffset[0], /over, psym=1, linecol=!ctv.green &$
endif else begin &$
  functionname='ewnfit_' + qidl + '_to_' + strlowcase('dLiAbu') &$
  varname=['log' + qid, 'logTeff', 'logg', 'logFeH'] &$
  yname  = 'dlogLiAbu' &$
  print, 'Create IDL function ' + functionname + '.pro' &$
  fufi_wridl, functionname, varname, xoffset, xfactor, yoffset, yfactor, dname=yname &$
endelse

; --- Write into protocel file ---
openw, nc, 'ewn_protocol.txt', /append, /get_lun
if (keyword_set(RestoreOptFile)) then begin &$
  printf, nc, $
          qid, FitSense, fufiStart.tid, 0, fufiNextraRandom, $
          n_elements(Aopt), fit_rms, fit_maxabs, fufiStart.TermMask[*], $
          format='(A7, 1X, A15, 1X, A18, I3, 1X, I4, 2X, I5, 2F9.6, 3X, 16I3)' &$
endif else begin &$
  printf, nc, $
          qid, FitSense, tid, fufiDepth, fufiNextraRandom, $
          n_elements(Aopt), fit_rms, fit_maxabs, fufiTermMask[*], $
          format='(A7, 1X, A15, 1X, A18, I3, 1X, I4, 2X, I5, 2F9.6, 3X, 16I3)' &$
endelse
close, nc
free_lun, nc

;i=7 & xx=EWN[i].LiAbu & yy=alog10(EWN[i].EW3) & plotlines,xx,alog10(-(yy-yy[0]-(xx-xx[0]))+0.01)
; --- Plot the error over Teff ---
;plotlines, reform(x[1,*]), y-yfufit, psym=1, xtitle='Teff', ytitle='Fit error'

; lc=[!ctv.white,!ctv.yellow,!ctv.orange,!ctv.red,!ctv.magenta,!ctv.blue,!ctv.cyan,!ctv.green]
; plotlines, findgen(nabu,n), reform(y-yfufit,nabu,n), psym=1,linecol=lc

; a0=-0.55D+00 & a1=2.9D+00 & x=findgen(7)*0.3+1.0
; a0=-0.55D+00 & a1=findgen(21)*0.1+1.0 & x=2.0
; y=a0+a1+ alog10(1.0D+00-exp(-exp(2.30259*(x-a1))))
; dyda1=1.0 + exp(2.30259*(x-a1))/(exp(exp(2.30259*(x-a1)))-1.0)
; plotlines,x,y,/over,linecol=!ctv.yellow

; end ; ewn_fufi_fit
