pro fufi_actind, termold, nparterm, nold, nact, termnew
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_ACTIND (FUnction_FInd_ACtual_INDex) 
;
; PURPOSE:
;   Change indices of parameters in input string
;
; CATEGORY:
;   String manipulation
;
; CALLING SEQUENCE:
;   fufi_actind, termold, nparterm, nold, nact, termnew
;
; INPUTS:
;   termold - String with input term with parameters
;             A(00), A(01), .., A(nparterm-1)
;   nparterm - Number of free parameters 'A(..)' of input term
;   nold - Number of parameters of current function
;   nact - Number of actual parameter
;
; OUTPUTS:
;   termnew - String with term with actualized parameter indices
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   'term(A(00),A(01),..,A(nparterm-1)'
;        --->
;   'term(A(nact),A(nold),..,A(nold+parterm-2)'
;
; EXAMPLE:
;   fufi_actind, 'A(00)+A(01)*sin(A(02)+A(03)*X)', 4, 6, 2, termnew
;   print,termnew
;   > A(02)+A(06)*sin(A(07)+A(08)*X) 
;
; MODIFICATION HISTORY:
;   7. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;  14. 9.95 'AA' -> 'A'
;-
;------------------------------------------------------------------------------
;
; --- Error handling ---
;on_error, 2
;
varname='A'
;
termnew=termold
;
; --- 'A(00)' -> 'A(nact)' ---
parstrold=varname + '(' + fufi_ind2str(0) + ')'
parindnew=fufi_ind2str(nact)
pos=strpos(termold, parstrold, 0)
while (pos ge 0) do begin
  strput, termnew, parindnew, pos+2
  pos=strpos(termold, parstrold, pos+5)
endwhile
;
; --- 'A(01)' -> 'A(nold)', 'A(02)' -> 'A(nold+1)', ... ---
for n=1,nparterm do begin
  parstrold=varname + '(' + fufi_ind2str(n) + ')'
  parindnew=fufi_ind2str(nold+n-1)
  pos=strpos(termold, parstrold, 0)
  while (pos ge 0) do begin
    strput, termnew, parindnew, pos+2
    pos=strpos(termold, parstrold, pos+5)
  endwhile
endfor
;
end ; fufi_actind
