function fufi_calldfpmin, X, Y, W, A, Function_Name = Function_Name1, $
  itermax=itermax, Amask=Amask0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_calldfpmin (FUnction_FInd_CALL_Davidon_Fletcher_Powell_MINimization)
;
; PURPOSE:
;   Fit function by call of NR_DFPMIN
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_dfpmin - Additional information about set of parameters
;
; ROUTINES:
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  27. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- Additional information about set of parameters ---
common fufi_dfpmin, Afullold, Amask, Aind, nfree, function_name
;
; --- Error handling ---
on_error, 2
;
; --- Put actual function name into common block variable ---
function_name=function_name1
;
; --- Put set of parameters into common block variable ---
Afullold=A
;
; --- Look which parameters are to be varied ---
if (not keyword_set(Amask)) then begin
  Amask=replicate(1,n_elements(A))
endif else begin
  Amask=Amask0
endelse
Aind=where(Amask eq 1)
Ared=A(Aind)
;
; --- Number of parameters ---
NTERMS = N_ELEMENTS(Ared)
; --- Degrees of freedom ---
NFREE = ((N_ELEMENTS(Y)<N_ELEMENTS(X(*,0)))-NTERMS) > 1
;
; --- DFP ---
Gtol=1e-5
eps=3.0e-6
;eps=3.0e-12
;stepmax=100.0
stepmax=1.0
itermax=800
;tolx=4.0*eps
tolx=1.0e-2
;NR_DFPMIN, Ared, Gtol, qopt, 'fufi_func', 'fufi_dfunc', $
;  iter=iternum, itmax=itermax, eps=eps, stepmax=stepmax, tolx=tolx
;
; --- Newton ---
Ared=NR_NEWT(Ared, 'fufi_dfunc')
;
; --- Powell
;dirmat=fltarr(nterms,nterms) & for n=0,nterms-1 do dirmat(n,n)=1.0
;Ftol=1e-4
;itermax=600
;NR_POWELL, Ared, dirmat, Ftol, qopt, 'fufi_func', itmax=itermax
;
; --- Broydn ---
;Ared=NR_BROYDN(Ared, 'fufi_dfunc')
; --- ---
;
result=check_math()
;
; --- Complete set of parameters ---
A(Aind)=Ared
;
call_procedure, function_name, X, A, Yfit
return, Yfit
;
end ; fufi_calldfpmin
