function fufi_dfunc, A
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_dfunc (FUnction_FInd_Derivative_FUNCtion)
;
; PURPOSE:
;   Compute derivative of function to be minimized with NR_DFPMIN.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   fufi_dfunc, A
;
; INPUTS:
;   A - Vector of parameters to be varied
;
; OUTPUTS:
;   fufi_dfunc - Vector with gradient of function at A.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_common - X/Y values to be fitted (with weights)
;   fufi_dfpmin - Additional information about set of parameters
;
; ROUTINES:
;   The routine defined by the string variable function_name0 is called
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   This procedure is usually called by NR_DFPMIN. It is used by FUNCTIONFIND.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  27. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- X/Y values to be fitted (with weights) ---
common fufi_common, Xvals, Yvals, Wvals
; --- Additional information about set of parameters ---
common fufi_dfpmin, Afullold, Amask, Aind, nfree, function_name
;
; --- Complete set of parameters with parameters not to be varied ---
Afull=Afullold
Afull(Aind)=A
;
; --- Call function ---
call_procedure, function_name, Xvals, Afull, Yfit, PDER, Amask=Amask
;
;print,Afull
;print,total(Wvals*(Yvals-Yfit)^2)/nfree
;
; --- Compute gradient of scalar function
;     and return it as vector.            ---
return, ((Wvals*(Yvals-Yfit))#PDER) * (-2.0/nfree)
;
end ; fufi_dfunc
