function fufi_func, A
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_func (FUnction_FInd_FUNCtion)
;
; PURPOSE:
;   Compute function to be minimized with NR_DFPMIN.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   fufi_func, A
;
; INPUTS:
;   A - Vector of parameters to be varied
;
; OUTPUTS:
;   fufi_func - Scalar function value at A.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   fufi_common - X/Y values to be fitted (with weights)
;   fufi_dfpmin -
;
; ROUTINES:
;   The routine in function_name0 is called
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   This procedure is usually called by fufi_recur
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  27. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- X/Y values to be fitted (with weights) ---
common fufi_common, Xvals, Yvals, Wvals
; --- Additional information about set of parameters ---
common fufi_dfpmin, Afullold, Amask, Aind, nfree, function_name
;
; --- Complete set of parameters with parameters not to be varied ---
Afull=Afullold
Afull(Aind)=A
;
; --- Call function ---
call_procedure, function_name, Xvals, Afull, Yfit
;
; --- Compute deviation^2 between Y values and fit function
;     and return it as scalar value to be minimized.        ---
return, total(Wvals*(Yvals-Yfit)^2)/nfree
;
end ; fufi_func
