pro fufi_insert, Fstrold, nold, nact, term, nparterm, Fstrnew
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_INSERT (FUnction_FInd_INSERT) 
;
; PURPOSE:
;   Insert new term into equation string.
;
; CATEGORY:
;   String manipulation
;
; CALLING SEQUENCE:
;   fufi_insert, Fstrold, nold, nact, term, nparterm, Fstrnew
;
; INPUTS:
;   Fstrold - Input equation string
;   nold - Number of parameters of input equation string
;   nact - Number of parameter to be substituted by new term (0 .. nold-1)
;   term - New term string
;   nparterm - Number of parameters of term
;
; OUTPUTS:
;   Fstrnew - Output equation string
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   Simple
;
; EXAMPLE:
;   fufi_insert, 'A(00)+A(01)*sin(A(02)+A(03)*X)', 4, 1, 'A(00)+A(01)*X', 2, $
;      Fstrnew
;   print, Fstrnew
;   > A(00)+(A(01)+A(04)*X)*sin(A(02)+A(03)*X)
;
; MODIFICATION HISTORY:
;   7. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;  14. 9.95 (B.F.) 'AA' -> 'A'
;-
;------------------------------------------------------------------------------
;
; --- Error handling ---
;on_error, 2
;
varname='A'
;
; --- Set indices of new term to correct values ---
fufi_actind, term, nparterm, nold, nact, termnew;, nnew
;
Fstrnew=Fstrold
; --- Parameter to be exchanged ---
parstrold=varname + '(' + fufi_ind2str(nact) + ')'
pos=strpos(Fstrnew, parstrold, 0)
; --- Insert new term instead of old parameter at all its positions ---
while (pos ge 0) do begin
  Fstrnew=strmid(Fstrnew, 0    , pos) + '(' + termnew  + ')' + $
          strmid(Fstrnew, pos+5, 30000)
  pos=strpos(Fstrnew, parstrold, pos-5+strlen(termnew))
endwhile
;
end ; fufi_insert
