pro fufi_recur, Fstr, PDERstr, aold, allowterm, varyterm, nold, depth, $
  quality, Fstropt, PDERstropt, aopt, allowtermopt, varytermopt
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_RECUR ('FUnction_FInd_RECURsive')
;
; PURPOSE:
;   Find the optimum fitting function with optimum parameters
;
; CATEGORY:
;   Curve fitting
;
; CALLING SEQUENCE:
;   fufi_recur, Fstr, PDERstr, aold, allowterm, varyterm, nold, depth, $
;               quality, Fstropt, PDERstropt, aopt, allowtermopt, varytermopt
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;   fufi_curvefit:
;   fufi_recur:    This routine is recursive !!!
;   fufi_wrfu:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   7. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;   8. 9.95 (B.F.)
;  10. 9.95 (B.F.) New form of terms (structure)
;  11. 9.95 (B.F.)
;  12. 9.95 (B.F.) Procedure fifu_wrfu added
;  13. 9.95 (B.F.)
;  19. 9.95 (B.F.) Handling of 'varyterm' added. Correct?
;-
;------------------------------------------------------------------------------
;
; === Common blocks ===
; --- Function and partial derivatives for CURVEFIT & FUNCT_STRING
common funct_string_common, F_string, PDER_strings, a_out
; --- X/Y values to be fitted (with weights) ---
common fufi_common, Xvals, Yvals, Wvals
; --- Arrays describing the extension posibilities ---
common fufi_recur, varname, idterm, terms00, terms0, nterms0, termmask0, nmask
; --- Arrays with global optimization quantities ---
common fufi_opt, nparmax, depthmax, qopt_eps, qopt_global, function_name0, $
       nextramax, nextrarandom
; --- Common block with runtime data ---
common fufi_time, time_fufi, time_cv, time_funct
; --- Common block with temporary file counter ---
common fufi_wrfu, ntemp
; --- Common block with I/O switches
common fufi_io, print_terms, print_blank, print_F, print_PDER, print_a, $
       print_time, print_iter, $
       print_init, plot_test, plot_fit
;
; === Restrict set of possible extension terms ===
termmask=termmask0(*,depth < (nmask-1))
termmask=termmask(where(termmask ge 0))
nterms  =n_elements(termmask)
terms1  =terms0(termmask)
nvary   =total(varyterm)
;
; === Terminal output ===
if (print_blank) then print
if (print_F) then print, string(depth,format='(X,I2)'), ' ', Fstr
if (print_PDER) then for n=0,nold-1 do print, string(depth,format='(X,I2)'), $
  string(n,format='(X,I2)'), ' ', PDERstr(n)
;
; === Make fit for given function ===
; --- Put actual function into common block for CURVEFIT & FUNCT_STRING ---
F_string    =Fstr
PDER_strings=PDERstr
; --- 'funct_temp' or 'funct_string' ---
if (function_name0 ne 'funct_temp') then begin
  function_name=function_name0
endif else begin
  if (n_elements(ntemp) eq 0) then ntemp=0L
  ntemp=ntemp+1L
  function_name=function_name0 + strcompress(string(ntemp),/remove_all)
  fufi_wrfu, function_name
endelse
;
aoldopt=aold
if ((Fstr eq '0.0*X+A(00)') or (Fstr eq '0.0*X(*,0)+A(00)')) then begin
  ; --- Trivial case: function=constant ---
  ; === Terminal output ===
  if (print_a) then print, depth, '   ',    aold   , format='(X,I2,A15     ,2X,20(X,E12.5))'
  if (varyterm(0) eq 1) then begin
    ; --- Fit constant ---
    aoldopt=total(Yvals*Wvals)/total(Wvals)
  endif else begin
    ; --- Don't fit constant ---
    aoldopt=aold
  endelse
  Yvals_fit_opt=Yvals*0.0+aoldopt
  q=total(Wvals*(Yvals-Yvals_fit_opt)^2)/((n_elements(Yvals)-nvary)>1)
  ; === Terminal output ===
  if (print_a) then print, depth, ' ->', q, aoldopt, $
    format='(X,I2,A3,E12.5,2X,20(X,E12.5))'
endif else begin
  ; --- Determine number of sets of initial values ---
  asize=size(aold)
  if (asize(0) le 1) then begin
    namax=1
  endif else begin
    namax=asize(2)
  endelse
  qmin=1e35
  seed=17
  ; --- Loop over all sets of initial values ---
  for na=0,namax-1+nextrarandom do begin
    if (na lt namax) then begin
      ; --- Initial values from table of initial value sets ---
      aoldtest=aold(*,na)
    endif else begin
      ; --- Initial values with increasingly larger random fluctuations ---
      if (namax eq 1) then begin
        varyamplitude=(avg(aold^2)-avg(aold)^2)>0.0
      endif else begin
        varyamplitude=(avg(aold^2,1)-avg(aold,1)^2)>0.0
      endelse
;print
;print, aold
;print, varyamplitude
      aoldtest=aold(*,na mod namax) + $
               float(varyterm)*0.1*(na-namax+1)*randomn(seed,nold)*varyamplitude
;               float(varyterm)*0.1*(na-namax+1)*randomn(seed,nold)
;               float(varyterm)*0.01*(na-namax+1)*randomn(seed,nold)
    endelse
    ; === Terminal output ===
    if (print_a) then print, depth, '   ', aoldtest, $
      format='(X,I2,A15     ,2X,20(X,E12.5))'
    time_cv_start=systime(1)
    ;
    ; --- Call FUFI_CURVEFIT with actual function & parameters ---
    Yvals_fit=fufi_curvefit(Xvals, Yvals, Wvals, aoldtest, sigmaa, $
      Function_Name=function_name, itermax=100, chieps=1e-14, dchieps=1e-16, $
      Amask=varyterm)
;      Function_Name=function_name, itermax=100, chieps=1e-14, dchieps=2.0e-10, $
;      Function_Name=function_name, itermax=60, chieps=1e-14, dchieps=2.0e-6, $

;      Function_Name=function_name, itermax=60, chieps=1e-14, dchieps=5.0e-5, $
;      Function_Name=function_name, itermax=60, chieps=1e-14, dchieps=1e-16, $
    ;
    ; --- Call FUFI_CALLDFPMIN with actual function & parameters
;    Yvals_fit=fufi_calldfpmin(Xvals, Yvals, Wvals, aoldtest, $
;      Function_Name=function_name, itermax=300, $
;      Amask=varyterm)
    ;
    time_cv=time_cv+(systime(1)-time_cv_start)
    q=total(Wvals*(Yvals-Yvals_fit)^2)/((n_elements(Yvals)-nvary)>1)
;    q=total(Wvals*(Yvals-Yvals_fit)^2)/((n_elements(Yvals))) ; --- 31.10.97 ---
    ; --- Look for best result ---
    if (q le qmin) then begin
      qmin=q
      aoldopt=aoldtest
      Yvals_fit_opt=Yvals_fit
      ; === Terminal output ===
      if (print_a) then print, depth, '==>', q, aoldtest, $
        format='(X,I2,A3,E12.5,2X,20(X,E12.5))'
    endif else begin
      ; === Terminal output ===
      if (print_a) then print, depth, '  >', q, aoldtest, $
        format='(X,I2,A3,E12.5,2X,20(X,E12.5))'
    endelse
  endfor ; na
  q=qmin
endelse
; --- Determine quality of fit ---
qopt_global =qopt_global<q
Fstropt     =Fstr
PDERstropt  =PDERstr
aopt        =aoldopt
allowtermopt=allowterm
varytermopt =varyterm
;
; === Plot ===
if (plot_test) then oplot, Xvals, Yvals_fit_opt
;
; --- Lowest depth -> output quality = quality of function of this depth ---
quality=q
;
if ((depth lt depthmax) and (nvary lt nparmax)) then begin
  ; --- Go through all lower depths and find best quality ---
  qmin=q
  ; --- Loop over all partial derivatives (=all parameters) ---
  for np=0,nold-1 do begin
    ; --- Loop over all extension terms ---
    for nt=0,nterms-1 do begin 
      ; --- Check if this extension is allowed ---
      if ( (allowterm(np) eq 2) or $
          ((allowterm(np) eq 1) and (termmask(nt) ne 0))) then begin
        ; --- New number of parameters ---
        nnew=nold+terms1(nt).npar-1
        ;
        ; --- Parameters to be varied ---
        varytermnew=[varyterm(*),terms1(nt).vary(1:terms1(nt).npar-1)]
          ;
        ; --- Skip new recursion step? ---
        if ((qmin ge qopt_eps) and (total(varytermnew)le nparmax)) then begin
          ;
          ; === Insert term into equation ===
          fufi_insert, Fstr, nold, np, terms1(nt).term, terms1(nt).npar, $
            Fstrnew
          ;
          ; === Allowed extension terms ===
          allowtermnew=[allowterm(*),terms1(nt).allow(1:terms1(nt).npar-1)]
          ;print,allowtermnew
          ;
          ; === New initial values ===
          nextra=terms1(nt).nextra < nextramax
          anew=replicate(aold(0),nnew,nextra)
          anew(0:nold-1,*)=rebin([aoldopt],nold,nextra)
          ;
          ; === Insert term into partial derivatives ===
          PDERstrnew=strarr(nnew)
          ; --- Old parameters ---
          for n=0,nold-1 do begin
            fufi_insert, PDERstr(n), nold, np, terms1(nt).term, $
              terms1(nt).npar, PDERstrnewhelp
            PDERstrnew(n)=PDERstrnewhelp
            if (n eq np) then begin
              ; --- Multiply old partial derivative with derivative of new term --
              termmult0=terms1(nt).termpders(0)
              if (termmult0 ne idterm) then begin 
                fufi_actind, termmult0, terms1(nt).npar, nold, np, termmult
                PDERstrnew(n)=PDERstrnew(n) + '*' + termmult
              endif
              ; --- If term=straight line forbid this substitution on sublevel -
              allowtermnew(n)=allowtermnew(n) < terms1(nt).allow(0)
              ; --- Same substitution mask as before ---
              ;
              ; --- New initial value ---
              for na=0,nextra-1 do begin
                 anew(n,na)=aoldopt(np)*terms1(nt).aextra(1,0,na)+ $
                                        terms1(nt).aextra(0,0,na)
              endfor
            endif
          endfor
          ; --- New parameters ---
          fufi_insert, PDERstr(np), nold, np, terms1(nt).term, $
            terms1(nt).npar, PDERstrnewhelp
          for n=nold,nnew-1 do begin
            PDERstrnew(n)=PDERstrnewhelp
            ; --- Multiply old partial derivative with derivative of new term --
            termmult0=terms1(nt).termpders(n-nold+1)
            if (termmult0 ne idterm) then begin
              fufi_actind, termmult0, terms1(nt).npar, nold, np, termmult
              PDERstrnew(n)=PDERstrnew(n) + '*' + termmult
            endif
            ; --- Substitutions on sublevel allowed from new term ---
            ;
            ; --- New initial value ---
             for na=0,nextra-1 do begin
               anew(n,na)=aoldopt(np)*terms1(nt).aextra(1,n-nold+1,na)+ $
                                      terms1(nt).aextra(0,n-nold+1,na)
              endfor
          endfor
          ;
          ; === Recursive call ===
          fufi_recur, Fstrnew, PDERstrnew, anew, allowtermnew, varytermnew, nnew, $
            depth+1, $
            qnew, Fstropt1, PDERstropt1, aopt1, allowopt1, varytermnew1
          ; --- New optimum? ---
          if (qnew lt qmin) then begin
            qmin=qnew
            Fstropt=Fstropt1
            PDERstropt=PDERstropt1
            aopt=aopt1
            allowtermopt=allowopt1
            varytermopt=varytermnew1
          endif
        endif
      endif
    endfor ; nt
  endfor ; np
  quality=qmin
endif
;
end ; fufi_recur
