pro fufi_wrfort, filename, varname, xoffset, xfactor, doffset, dfactor
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_wrfort (FUnction_FInd_WRite_FORTran)
;
; PURPOSE:
;   Create a file with FORTRAN function
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   fufi_wrfort, filename, varname, xoffset, xfactor, doffset, dfactor
;
; INPUTS:
;   filename - Name of file to be written ('.f' omitted).
;   varname - String vector with names of variables
;   xoffset - Real vector with x offsets
;   xfactor - Real vector with x factors
;   doffset - Real vector with data offsets
;   dfactor - Real vector with data factors
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   funct_string_common
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Create a file with a FORTRAN function
;
; RESTRICTIONS:
;   The variables in funct_string_common have to be defined properly.
;
; PROCEDURE:
;   Simple: write the function given by F_string with the parameters given
;   by a_out into file in a FORTRAN format.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  16.11.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Common block with function and partial derivatives ---
common funct_string_common, F_string, PDER_strings, a_out
;
; --- Number of dimensions ---
nvar=n_elements(varname)
; --- Number of parameters ---
nA=n_elements(PDER_strings)
;
; --- Open new file for FORTRAN routine ---
get_lun, cunit
openw, cunit, filename + '.f'
;
c='      '
;
; --- Print headline ---
vars=strcompress(varname(0),/rem)
for n=1,nvar-1 do vars =vars  + ', ' + strcompress(varname(n),/rem)
vars0=strcompress(varname(0),/rem) + '0'
for n=1,nvar-1 do vars0=vars0 + ', ' + strcompress(varname(n),/rem) + '0'
printf, cunit, c, 'function ' + filename + '(' + vars + ')'
;
printf, cunit, c, 'real ' + filename
printf, cunit, 'C --- ' + systime(0) + ' --- B.F. ---'
;
; --- Print list of variable declarations ---
printf, cunit, c, 'real ' + vars
printf, cunit, c, 'real ' + vars0
printf, cunit, c, 'real f0'
;
; --- Declaration of parameter array ---
printf, cunit, c, 'real A(0:' + strcompress(string(nA-1),/rem) + ')'
; --- Set parameter array ---
for n=0,nA-1 do printf, cunit, c, 'A(' + strcompress(string(n),/rem) + ')=' + $
  strcompress(string(a_out(n)),/rem) 
for n=0,nvar-1 do printf, cunit, c, $
  varname(n) + '0=(' + varname(n) + $
                  '-' + strcompress(string(xoffset(n)),/rem) + $
                 ')/' + strcompress(string(xfactor(n)),/rem)
;
Fstrnew=F_string
for n=0,nvar-1 do begin
  if (nvar eq 1) then parstrold='X' $
                 else parstrold='X(*,' + string(n,format='(I1)') + ')'
  pos=strpos(Fstrnew, parstrold, 0)
  ; --- Insert new variable instead of old term at all its positions ---
  termnew=strcompress(varname(n),/rem) + '0'
  while (pos ge 0) do begin
    Fstrnew=strmid(Fstrnew, 0    , pos) + termnew  + $
            strmid(Fstrnew, pos+strlen(parstrold), 30000)
    pos=strpos(Fstrnew, parstrold, pos-strlen(parstrold)+strlen(termnew))
  endwhile
endfor
printf, cunit, c + 'f0=' +  Fstrnew
;
printf, cunit, c, filename + '=f0*' + strcompress(dfactor,/rem) + $
                            '+' + strcompress(doffset,/rem)
;
; --- End of function ---
printf, cunit, c, 'end'
;
; --- Close file ---
free_lun, cunit
close, cunit
;
end ; fufi_wrfort
