pro fufi_wrfu, filename
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUFI_wrfu (FUnction_FInd_WRite_FUnction)
;
; PURPOSE:
;   Create a file with procedure
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   fufi_wrfu, filename
;
; INPUTS:
;   filename - Name of file to be written ('.pro' omitted).
;
; OUTPUTS:
;   None
;
; OPTIONAL OUTPUT PARAMETERS:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   funct_string_common
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Create a file
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  12. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;  20. 9.95 (B.F.) Amask introduced
;-
;------------------------------------------------------------------------------
;
; --- Common block with function and partial derivatives ---
common funct_string_common, F_string, PDER_strings, a_out
;
; --- Number of parameters ---
nA=n_elements(PDER_strings)
; --- Number of elements ---
nX=n_elements(Xvals)
;
get_lun, cunit
openw, cunit, filename + '.pro'
;
printf, cunit, 'pro ' + filename + ', X, A, F, PDER, Amask=Amask'
;
printf, cunit, 'common fufi_time, time_fufi, time_cv, time_funct'
printf, cunit, 'time_funct_start=systime(1)'
;
printf, cunit, 'nA=' + strcompress(string(nA),/rem)
;printf, cunit, 'nX=n_elements(X)'
printf, cunit, 'nX=n_elements(X(*,0))'
;
printf, cunit, 'F=' +  F_string
printf, cunit, 'if (n_params() gt 3) then begin'
;
printf, cunit, 'if (not keyword_set(Amask)) then Amask=replicate(1,nA)'
printf, cunit, 'Aind=where(Amask eq 1)'
printf, cunit, 'nAind=n_elements(Aind)
printf, cunit, 'Aref=replicate(-1,nA) & Aref(Aind)=indgen(nAind)'
printf, cunit, 'PDER=replicate(A(0),nX,nAind)'
for n=0,nA-1 do begin
  printf, cunit, 'if (Amask(' + strcompress(string(n),/rem) + $
                 ') eq 1) then $'
  printf, cunit, '  PDER(*,Aref(' + strcompress(string(n),/rem) + '))=' + $
                 PDER_strings(n)
endfor
printf, cunit, 'endif'
;
printf, cunit, 'cm=check_math(0,1)'
;
printf, cunit, 'time_funct=time_funct + (systime(1)-time_funct_start)'
;
printf, cunit, 'end'
;
free_lun, cunit
close, cunit
;
end ; fufi_wrfu
