pro FUNCT_2EXP, X, A, F, Pder
;
;------------------------------------------------------------------------------
;+
; NAME:
;   FUNCT_2EXP (FUNCTion_2_EXPonentials)
;
; PURPOSE:
;   Evaluate the sum of two exponentials and optionally evaluate its
;   partial derivatives. Normally, this function is used by CURVEFIT.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   FUNCT, X, A, F [, Pder]
;
; INPUTS:
;   X -	The values of the independent variable.
;   A - Three parameters of the equation described in PROCEDURE below.
;
; OUTPUTS:
;   F - The value of the function at each X(i).
;
; OPTIONAL OUTPUT PARAMETERS:
;   Pder - An array of the size (N_ELEMENTS(X),3) that contains the
;          partial derivatives.  Pder(i,j) represents the derivative
;          at the i'th point with respect to j'th parameter.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   F=A(0)*(exp(A(1)*X)-exp(A(2)*X))
;
; EXAMPLE:
;   x=findgen(100)/10.
;   a=[1.0,0.0,-1.0]
;   funct_2exp,x,a,f,pder
;   plot, x, f
;
; MODIFICATION HISTORY:
;   1.11.94 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Compute both exponentials ---
e1=exp(A(1)*X)
e2=exp(A(2)*X)
;
; --- Compute function ---
F=A(0)*(e1-e2)
;
if (n_params() gt 3) then begin
  ; --- Compute partial derivatives ---
  PDER=replicate(A(0),n_elements(X),3)
  PDER(*,0)=e1-e2
  PDER(*,1)=A(0)*X*e1
  PDER(*,2)=-A(0)*X*e2
endif
;
end ; FUNCT_2EXP
