pro funct_linexp, X, A, F, Pder
;
;------------------------------------------------------------------------------
;+
; NAME:
;   funct_linexp (FUNCTion_LINear_EXPonential)
;
; PURPOSE:
;   Evaluate the sum of a straight line and an exponential and optionally
;   evaluate its partial derivatives.
;   Normally, this function is used by CURVEFIT.
;
; CATEGORY:
;   Curve and surface fitting
;
; CALLING SEQUENCE:
;   funct_linexp, X, A, F [, Pder]
;
; INPUTS:
;   X -	The values of the independent variable.
;   A - Three parameters of the equation described in PROCEDURE below.
;
; OUTPUTS:
;   F - The value of the function at each X(i).
;
; OPTIONAL OUTPUT PARAMETERS:
;   Pder - An array of the size (N_ELEMENTS(X),4) that contains the
;          partial derivatives.  Pder(i,j) represents the derivative
;          at the i'th point with respect to j'th parameter.
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   None
;
; PROCEDURE:
;   F=A(0) + A(1)*X + A(2)*exp(A(3)*X)
;;   F=A(0) + A(1)*X + A(2)*exp(A(3)*X) - (A(2) + A(3)*X)
;
; EXAMPLE:
;   x=findgen(100)/10.
;   a=[0.0,0.1,1.0,0.2]
;   funct_linexp,x,a,f,pder
;   plot, x, f
;
; MODIFICATION HISTORY:
;   4. 9.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Compute exponential ---
e1=exp(A(3)*X)
;
; --- Compute function ---
F=A(0) + A(1)*X + A(2)*e1
;F=A(0) + A(1)*X + A(2)*(e1 - (1.0 + A(3)*X))
;
if (n_params() gt 3) then begin
  ; --- Compute partial derivatives ---
  PDER=replicate(A(0),n_elements(X),4)
  PDER(*,0)=1.0
  PDER(*,1)=X
  PDER(*,2)=e1
  PDER(*,3)=A(2)*X*e1
;  PDER(*,0)=1.0
;  PDER(*,1)=X
;  PDER(*,2)=(e1 - (1.0 + A(3)*X))
;  PDER(*,3)=A(2)*(X*e1 - X)
endif
;
end ; FUNCT_linexp
