; wdcorrect_fufi_fit
;
;------------------------------------------------------------------------------
;+
; NAME:
;   wdcorrect_fufi_fit ('blue_shift_function_find_fit')
;
; PURPOSE:
;   Fit blue shift as function of Teff, logg, FeH by single function
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @wdcorrect_fufi_fit
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot. It's a script.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @wdcorrect_fufi_fit
;
; MODIFICATION HISTORY:
;   2011-09-15 (B.F.) Written by Bernd Freytag, CRAL-ENS Lyon
;   2011-10-19 (B.F.) New version of the data
;   2012-07-05 (B.F.) New plot layout
;   2013-06-10 (B.F.) Adapted from blueshift_fufi_fit.pro
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Select device ---
  dev='x' &$
  jpg_flag=0 &$
  png_flag=0 &$
  CMYK_flag=0 &$
  ;
  ; --- Data file name  ---
  ;InputFileName='3D_corr_ML1.7' &$
  ;InputFileName='3D_corr2_ML1.7' &$
  ;InputFileName='3D_corr_ML1.8' &$
  ;
  ; --- Control for function finding with fufi ---
  ;Corr_id='dTeff' & tid='Polynomials' & fufiTermMask=[[0]] &$        ; --- polynomials (in all variables)
  ;Corr_id='dTeff' & tid='PolyGauss2D' & fufiTermMask=[[4,0]] &$    ; --- polynomials + 2D gaussian
  ;Corr_id='dTeff' & tid='PolyGauss1D' & fufiTermMask=[[12,0]] &$     ; --- polynomials + 1D gaussian
  ;Corr_id='dTeff' & tid='PolyGauss1Da' & fufiTermMask=[[12],[0]] &$   ; --- polynomials + 1D gaussian
  ;Corr_id='dlogg' & tid='Polynomials' & fufiTermMask=[[0]] &$        ; --- polynomials (in all variables)
  ;Corr_id='dlogg' & tid='PolyGauss2D' & fufiTermMask=[[4,0]] &$    ; --- polynomials + 2D gaussian
  Corr_id='dlogg' & tid='PolyExpGauss2D' & fufiTermMask=[[4,2,0]] &$    ; --- polynomials + 2D gaussian + exp
  ;Corr_id='dlogg' & tid='PolyGauss1D' & fufiTermMask=[[12,0]] &$     ; --- polynomials + 1D gaussian
  ;Corr_id='dlogg' & tid='PolyGauss1D' & fufiTermMask=[[11,0]] &$     ; --- polynomials + 2D gaussian
  ;Corr_id='dlogg' & tid='PolyExpGauss1D' & fufiTermMask=[[11,2,0]] &$ ; --- polynomials + 2D gaussian + exp
  ;Corr_id='dlogg' & tid='PolyExp' & fufiTermMask=[[2,0]] &$           ; --- polynomials + exp
  ;tid='test' & fufiTermMask=[[0],[2]] &$         ; --- polynomials (in all variables)
  ;tid='ConIdExp' & fufiTermMask=[[4],[2]] &$     ; --- const + identity + exponential
  ;
  fufiDepth=4 &$
  ;fufiNextraRandom=0 &$
  fufiNextraRandom=4 &$
  ;fufiNextraRandom=16 &$
  ;
  file_id='WD' + InputFileName + '_' + Corr_id + '_' + tid + '_nDepth' + trim(string(fufiDepth,format='(I3)'),2) &$
  ;
  RestoreOptFile='' &$
  ;RestoreOptFile=file_id + '.sav' &$
  ;SaveOptFile='' &$
  SaveOptFile=file_id + '.sav' &$
endif

; --- Plot file names ---
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Read input data ---
rdarray, InputFileName, b0, s
b0=b0[2:*,0:4]
b =b0

; --- Add fake values ---
v1a=[3000.0, 3500.0, 4000.0, 4500.0]
v2a=[7.0, 7.5, 8.0, 8.5, 9.0]
n1a=n_elements(v1a)
n2a=n_elements(v2a)
ba=fltarr(n1a*n2a,5)
ba[*,0]=replicate1(v1a,n1a*n2a)
ba[*,1]=reform(rebin(reform(v2a,1,n2a),n1a,n2a),n1a*n2a)
ba[*,4]=1.0
b=[b,ba]

; --- Adjust weights ---
ind=where((b[*,0] gt 4500.0) and (b[*,0] le 15000.0) and ((b[*,1] eq 7.5) or (b[*,1] eq 8.5)))
b[ind,4]=b[ind,4]* 1.0/sqrt(2.0)^2
ind=where((b[*,0] gt 4500.0) and (b[*,0] le 15000.0) and ((b[*,1] eq 7.0) or (b[*,1] eq 9.0)))
b[ind,4]=b[ind,4]* 1.0/2.0^2

; --- Restrict data ---
;ind=where(b[*,0] lt 6500.0)
if (InputFileName eq '3D_corr_ML1.7') then begin &$
  ind=where((b[*,0] ne 6500.0) or (b[*,1] ne 9.0)) &$
endif else if (InputFileName eq '3D_corr_ML1.8') then begin &$
  ind=where((b[*,0] ne 13500.0) or (b[*,1] ne 8.0)) &$
endif else begin &$
  ind=where(b[*,0] gt 0.0) &$
endelse

; --- Create empty arrays for independent and dependent variables ---
;nx=n_elements(b[*,0])
nx=n_elements(ind)
x =fltarr(nx,2)
;x =fltarr(nx,3)
y =fltarr(nx)
w =fltarr(nx)

; --- Put input data into fufi input arrays ---
Teff=b[ind,0]
logg=b[ind,1]
x[*,0]=((b[ind,0]-10000.0)/1000.0)     ; --- T ---
x[*,1]=b[ind,1]-8.0                    ; --- log g ---
if (Corr_id eq 'dTeff') then begin &$
  yFactor=1000.0 &$
  y[*]  =b[ind,2]/yFactor &$           ; --- dTeff ---
endif else begin &$
  yFactor=1.0 &$
  y[*]  =b[ind,3]/yFactor &$           ; --- dlogg ---
endelse
w[*]  =b[ind,4] ; --- weight ---

; --- Find fitting function ---
; --- input: x,y,w; output: Fstropt, aopt ---
; --- control: TermMask, npar, depth, ... ---
npar=30 & nextramax=4
if (keyword_set(RestoreOptFile)) then begin &$
  restore, RestoreOptFile, /verbose &$
  fufiStart=fufiOpt &$
  functionfind, x, y, w, $
    Fstrstart=fufiStart.Fstr, PDERstrstart=fufiStart.PDERstr, astart=fufiStart.a, $
      allowtermstart=fufiStart.allowterm, varytermstart=fufiStart.varyterm, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=[[0]], npar=npar, depth=0, nextramax=nextramax, nextrara=fufiNextraRandom &$
endif else begin &$
  functionfind, x, y, w, $
    Fstropt=Fstropt, PDERstropt=PDERstropt, aopt=aopt, $
      allowtermopt=allowtermopt, varytermopt=varytermopt, $
    print_blank=0, print_F=1, print_a=0, print_iter=0, $
    TermMask=fufiTermMask, npar=npar, depth=fufiDepth, nextramax=nextramax, nextrara=fufiNextraRandom &$
endelse

; --- Compute the values of the fit function at the given points ---
funct_string, x, aopt, yfufit
;print, yfufit
;print, 'Total rms deviation: ', rms(w*(y-yfufit), /nonorm)/(total(w)/n_elements(w)), max(abs(w*(y-yfufit)))
fit_rms=sqrt(avg((y-yfufit)^2))
fit_maxabs=max(abs((y-yfufit)))
print, 'Total rms/max deviation: ', sqrt(avg((y-yfufit)^2)), max(abs((y-yfufit)))

if (keyword_set(SaveOptFile)) then begin &$
  ; --- Save optimum solution into structure and write it to file ---
  fufiOpt=create_struct( $
                'x'        , x, $
                'y'        , y, $
                'w'        , w, $
                'qid'      , 'wdcorrect', $
                'tid'      , tid, $
                'TermMask' , fufiTermMask, $
                'npar'     , npar, $
                'depth'    , fufiDepth, $
                'nextramax', nextramax, $
                'Fstr'     , Fstropt, $
                'PDerstr'  , PDERstropt, $
                'A'        , Aopt, $
                'allowterm', allowtermopt, $
                'varyterm' , varytermopt, $
                'rms'      , fit_rms, $
                'maxabs'   , fit_maxabs $
          ) &$
  save, fufiOpt, file=SaveOptFile, /verbose &$
endif

; --- Write an IDL function ---
FunctionName=file_id
VarName=['Teff', 'logg']
yName='Shift'
print, 'Create IDL function ' + FunctionName
fufi_wridl, FunctionName, varName, [10000.0,8.0], [1000.0,1.0], 0.0, yFactor, dname=yname

; --- Plot settings ---
;xsize=8.8 & ysize=7.0
xsize=8.8 & ysize=6.5
if (strupcase(dev) eq 'X') then begin & $
  wfactor=1000.0 &$
  set_plot, 'X' & $
  win_id=0 &$
  !P.CHARSIZE=0.0 &$
  ;window, win_id, xsize=xsize/18.0*wfactor, ysize=ysize/18.0*wfactor, retain=2 &$
  !P.FONT=-1 &$
endif else if (strupcase(dev) eq 'PS') then begin & $
  set_plot, 'ps' & $
  device, filename=ps_file, $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.6 &$
endif else begin &$ 
  set_plot, 'ps' & $
  device, filename=ps_file, $
          /encapsulated, CMYK=CMYK_flag, $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.7 &$
endelse
modctv, dev
;normct, 0

; --- Set global plot parameters ---
Pbak=!P & Xbak=!X & Ybak=!Y
!X.MARGIN=[6,1] & !Y.MARGIN=[4,1]

;; --- Plot original and fitted data over one quantity ---
;ix=0
;plotlines,reform(x[*,ix]),reform(y)   ,psym=1,linecol=!ctv.green,symsize=3
;plotlines,transpose([[reform(x[*,ix])],[reform(x[*,ix])]]),transpose([[reform(y)],[reform(yfufit)]]),/over,linestyle=1
;plotlines,reform(x[*,ix]),reform(yfufit),psym=102,/over,linecol=!ctv.red      

; --- Make 2D plot ---

nreg=100L
xreg=fltarr(nreg^2,2)
Treg   =(16.0*findgen(nreg)/float(nreg-1)+2.0)*1000.0
loggreg=       4.0*findgen(nreg)/float(nreg-1)+6.0
xreg[*,0]=rebin(reform((   Treg-10000.0)/1000.0,nreg,1),nreg,nreg)
xreg[*,1]=rebin(reform( loggreg-8.0            ,1,nreg),nreg,nreg)
funct_string, xreg, aopt, yreg
;plotfield,/tv,Treg,loggreg,reform(yreg,nreg,nreg),/oldasp,yRange=[max(loggreg),min(loggreg)],xRange=[max(Treg),min(Treg)],dRange=[-1,1]*0.35

; --- Change sign ---
;yreg  =-yreg
;y     =-y
;yfufit=-yfufit

;dRange=[round(min(y)*10.)/10.-0.1,round(max(y)*10.)/10.+0.1]
;nlevels=5*fix((max(dRange)-min(dRange))/0.1)+1
dRange=[-0.42,0.4]
nlevels=84
levels=min(dRange)+findgen(nlevels)*0.01
levels[where(abs(levels) le 1.0e-6)]=0.0
;anno='!5' + strmid(string(levels, format='(F3.1)'),0,4) + '!X'
;anno='!5' + strtrim(string(levels, format='(F5.2)'),2) + '!X'
anno='!5' + strtrim(string(round(levels*1000.0), format='(I5)'),2) + '!X'
anno[bindgen(1+nlevels/5)*5+0]=''
anno[bindgen(  nlevels/5)*5+2]=''
anno[bindgen(  nlevels/5)*5+3]=''
anno[bindgen(  nlevels/5)*5+4]=''
plotfield, Treg,loggreg,reform(yreg,nreg,nreg)<0.299, /oldasp, $
;  title=file_id + Met_id, $
  xtitle='T!Deff!N [K]', ytitle='log g', $
  xRange=[max(Treg),min(Treg)], yRange=[max(loggreg),min(loggreg)], dRange=[-0.6,0.33], $
;  /tv, $
  /contour, c_levels=levels, c_anno=anno, c_charsize=!P.CHARSIZE*0.92, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;plotfield, Treg,loggreg,reform(yreg,nreg,nreg)<0.299, /oldasp, $
;  xtitle='T!Deff!N [K]', ytitle='log g', $
;  xRange=[max(Treg),min(Treg)], yRange=[max(loggreg),min(loggreg)], dRange=[-0.6,0.33], $
;  /tv, $
;  dev=dev, psformat='nochange', /psnoinit, /noclose

;xyouts, 0.2, 0.85, '[M/H]=' + strtrim(string(-Mreg,format='(I2)'),2), /norm, ali=0.5

for ix=0,nx-1 do begin &$
    if (yfufit[ix] gt y[ix]) then begin &$
      xycolor=!ctv.green &$
    endif else begin &$
      xycolor=!ctv.red &$
    endelse &$
    ;
    xyouts, Teff[ix], logg[ix], $
      strtrim(string(y[ix]*1000.0, format='(I5)'),2), $
      charsize=!P.CHARSIZE*0.85, ali=0.5, color=xycolor &$
endfor

; --- Finish plot ---
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    ; --- Write PNG file ---
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    ; --- Write JPEG file ---
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

; end ; wdcorrect_fufi_fit
 
