pro ima3_shell, d, dshell2, $
  dlevel=dlevel0, dsmoothbox=dsmoothbox0, dsmoothlevel=dsmoothlevel0, $
  dmask0=dmask0, dshell0=dshell0, dmask1=dmask1, dvalley=dvalley, dmask2=dmask2
;------------------------------------------------------------------------------
;+
; NAME:
;   ima3_shell ('imaga_analysis_3D_shell')
;
; PURPOSE:
;   Compute surface shell and distinguish between convex and concave regions.
;
; CATEGORY:
;   3D image analysis
;
; CALLING SEQUENCE:
;   ima3_shell, d, dshell2 $
;     [, dlevel=dlevel0, dsmoothbox=dsmoothbox0, dsmoothlevel=dsmoothlevel0, $
;      dmask0=dmask0, dshell0=dshell0, dmask1=dmask1, dvalley=dvalley,
;      dmask2=dmask2]
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   29. 7.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;    4. 8.03 (B.F.) More keywords
;   11. 8.02 (B.F.) One-pixel bug in healing step removed
;-
;------------------------------------------------------------------------------
;
; --- Default values for optional parameters ---
if (keyword_set(dlevel0)) then begin
  dlevel=dlevel0
endif else begin
  dmin=min(d, max=dmax)
  dlevel=0.5*(dmin+dmax)
endelse
if (keyword_set(dsmoothbox0)) then begin
  dsmoothbox=dsmoothbox0
endif else begin
  dsmoothbox=7
endelse
if (keyword_set(dsmoothlevel0)) then begin
  dsmoothlevel=dsmoothlevel0
endif else begin
  dsmoothlevel=0.53
endelse
;
message, /info, 'dlevel= ' + strtrim(string(dlevel),2) + $
              ', dsmoothbox=' + strtrim(string(dsmoothbox),2) + $
              ', dsmoothlevel=' + strtrim(string(dsmoothlevel),2)
;
; --- Basic masking step: all points with value above threshold ---
dmask0=d ge dlevel
;
if (keyword_set(dshell0)) then begin
  ; --- Take just the surface shell ---
  dmasksmooth=smooth(float(dmask0), 3, /edge_truncate)
  dshell0=(dmasksmooth lt 1.0) and dmask0
endif
;
; --- Make mask "more convex" by filling up valleys partly ---
dmask1=dmask0
imax=40
i=0
repeat begin
; st28gm06n03 (171^3, extended): 7000K:  7/0.49
; st35gm04n30 (127^3):           12000K: 7/0.53
;  dmasksmooth=smooth(float(dmask1), 5, /edge_truncate)
;  dmask1n     =(dmasksmooth ge 0.53) or dmask1
;  dmasksmooth=smooth(smooth(float(dmask1), 3, /edge_truncate), 3, /edge_truncate)
;  dmasksmooth=convolsym(float(dmask1), exp(-(0.5*findgen(3))^2), /edge_truncate)
  dmasksmooth=smooth(float(dmask1), dsmoothbox, /edge_truncate)
  dmask1n     =(dmasksmooth ge dsmoothlevel) or dmask1
  maskdiff=total(abs(dmask1n-dmask1))
  print,i,maskdiff
  dmask1=dmask1n
  i=i+1
endrep until ((i ge imax) or (maskdiff eq 0.0))
;
; --- Remove single layer from valleys only
;dmasksmooth=smooth(float(dmask1), 3, /edge_truncate)
;dmask2=(dmasksmooth ge 0.99) or dmask0
dmask2=dmask1
;

; --- Healing step ---
; --- Valeys only ---
dvalley=dmask2 and (not dmask0)
;dvalley=smooth(float(dvalley), 3, /edge_trun) ge 0.04
;dvalley=(     (smooth(float(dvalley), 3, /edge_trun) ge 0.04) ) $
;        or $
;        (  (smooth(float(dvalley), 3, /edge_trun) lt 0.04) $
;           and $
;        (  (smooth(float(dmask0 ), 3, /edge_trun) ge 0.96)))
dvalley=(smooth(float(dvalley), 3, /edge_trun) ge 0.04) $
        or $
        (smooth(float(dmask0 ), 3, /edge_trun) ge 0.96)
; --- Composite ---
dmask2=dmask0 or dvalley
;

dvalley=dmask2 and (not dmask0)
;
; --- Take just the surface shell ---
dmasksmooth=smooth(float(dmask2), 3, /edge_truncate)
dshell2=(dmasksmooth lt 1.0) and dmask2
;
end ; ima3_shell
