function ima3_smallestbox, mask, indexlist=indexlist, border=border0, $
  mvec=mvec, nvec=nvec, outindexlist=outindexlist
;
;------------------------------------------------------------------------------
;+
; NAME:
;   ima3_smallestbox ('image_analysis_3D_smallest_box')
;
; PURPOSE:
;   Give smallest box that still contains all relevant points.
;
; CATEGORY:
;   3D image analysis
;
; CALLING SEQUENCE:
;   smallmask=ima3_smallestbox(mask [, indexlist=indexlist, border=border0, $
;               mvec=mvec, nvec=nvec, outindexlist=outindexlist])
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    6. 8.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; --- Check parameters, set default values for keywords ---
if (n_params() ne 1) then begin
  message, 'One parameter is needed'
endif
size_mask=size(mask)
if (not keyword_set(indexlist)) then begin
  if (array_equal(mask, 0B)) then begin
    message, 'Input array contains only zeros'
  endif
endif
if (keyword_set(border0)) then begin
  border=long(border0) > 0L
endif else begin
  border=0L
endelse
if ((size_mask)(0) ne 3) then begin
  message, 'Input array must have exactly three dimensions'
endif
;
nmvec=size(mask, /dimensions)
mvec=nmvec
nvec=nmvec
;
if (not keyword_set(indexlist)) then begin
  ; --- Look for relevant range by directly checking the array ---
  ;
  a=total(total(mask,2),2) ne 0.0
  a=long(a) * (lindgen(nmvec(0))+1L)
  a=a(where (a ne 0L))
  mvec(0)=(min(a)-1L-border) > 0L
  nvec(0)=(max(a)-1L+border) < (nmvec(0)-1L)
;  help, mvec(0),nvec(0)
  ;
  a1=total(mask,1)
  a=total(a1,2) ne 0.0
  a=long(a) * (lindgen(nmvec(1))+1L)
  a=a(where (a ne 0L))
  mvec(1)=(min(a)-1L-border) > 0L
  nvec(1)=(max(a)-1L+border) < (nmvec(1)-1L)
;  help, mvec(1),nvec(1)
  ;
  a=total(a1,1) ne 0.0
  a=long(a) * (lindgen(nmvec(2))+1L)
  a=a(where (a ne 0L))
  mvec(2)=(min(a)-1L-border) > 0L
  nvec(2)=(max(a)-1L+border) < (nmvec(2)-1L)
;  help, mvec(2),nvec(2)
  ;
endif else begin
  ; --- Transform indexlist into set of spatial indices and use these ---
  ;
  vectorindex=ima3_vectorindex(indexlist, nmvec)
  ;
  mvec=nmvec
  nvec=nmvec
  for i=0,n_elements(nmvec)-1 do begin
    mvec(i)=(min(vectorindex(*,i), max=maxv)-border) > 0L
    nvec(i)=(maxv                           +border) < (nmvec(i)-1L)
;    help, mvec(i), nvec(i)
    vectorindex(*,i)=vectorindex(*,i)-mvec(i)
  endfor
  ;
  outindexlist=ima3_scalarindex(vectorindex, nvec-mvec+1L)
  ;
endelse
;
out=reform(mask(mvec(0):nvec(0)  , mvec(1):nvec(1)  , mvec(2):nvec(2)  ), $
                nvec(0)-mvec(0)+1, nvec(1)-mvec(1)+1, nvec(2)-mvec(2)+1)
;
return, out
;
end ; ima3_smallestbox
