function ima3_vectorindex, scalarindex, nvec
;
;------------------------------------------------------------------------------
;+
; NAME:
;   ima3_vectorindex ('image_analysis_3D_vector_index')
;
; PURPOSE:
;   Transform index number into vector index.
;
; CATEGORY:
;   3D image analysis
;
; CALLING SEQUENCE:
;   vectorindex=ima3_vectorindex(scalarindex, nvec)
;
; INPUT:
;   scalarindex: ('scalar_index') Long integer vector with array index numbers,
;                resulting e.g. from "where(array eq value)" operation
;   nvec:        ('number_vector') (Long) integer vector with array size,
;                e.g. from "size(array, /dimension)"
;
; OUTPUT:
;   vectorindex: Long integer 2-D array with
;                  (n_elements(scalarindex), n_elements(nvec))
;                elements containing the dimensional array indices
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Straightforward: employ appropriate DIV and MOD operations to get indices.
;
; EXAMPLE:
;   scalarindex=[1L, 4L, 5L]
;   nvec=[3,4]
;   vectorindex=ima3_vectorindex(scalarindex, nvec)
;   print, vectorindex
;     1           1           2
;     0           1           1
;
; SEE ALSO:
;   ima3_scalarindex
;
; MODIFICATION HISTORY:
;    6. 8.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
if (n_params() lt 2) then begin
  message, 'Two parameters are required.'
endif
;
; --- Number of spatial dimensions ---
ndim=n_elements(nvec)
;
; --- Create output array ---
vectorindex=lonarr(n_elements(scalarindex), ndim)
;
; --- Transform indices ---
factor=1L
for i=0,ndim-1 do begin
  vectorindex(*,i)=(scalarindex / factor) mod nvec(i)
  factor=factor*nvec(i)
endfor
;
return, vectorindex
;
end ; ima3_vectorindex
