; pro f20_2_uio
;
;------------------------------------------------------------------------------
;+
; NAME:
;   f20_2_uio ('F20_to_UIO')
;
; PURPOSE:
;   Extract data from f20 files, write them in file in uio form.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   @f20_2_uio
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rdf20:
;   uio_wrf20:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, modellist
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    2. 2.98 Written by Bernd Freytag, Astrophysik Kiel
;   19. 2.98 (B.F.)
;-
;------------------------------------------------------------------------------
;
;disk='/astrotmp1/supas024/f20/'
;outdisk='/astrotmp1/supas024/par/'
;gmodisk='/astro/home/bf/idl/out/'
;disk='/astro/p3/bf/hd/f20/'
;counterdisk='/astro/p3/bf/hd/counter/'
gmodisk=getenv('HOME') + '/idl/out/'
disk=getenv('HOME') + '/dat/hdw/f20/'
counterdisk=getenv('HOME') + '/dat/hdw/counter/'
outdisk=''
;
; --- HD data -----------------------------------------------------------------
;
; --- Define list of models ---
modellist=['ot63g40n2']
; modellist=['gt57g44n1', 'gt70g44n2', 'ot57g44n3', 'zt134g80n1']
;if (n_elements(modellist) eq 0) then begin &$
;  rdgmodels, gmodisk + 'g_models.txt', g_models, modellist &$
;endif
;
;
nlist=n_elements(modellist)
;
;
; --- Read f20 parameter file, derive EOS and opacity file ---
; --- Write output data into parameter file in UIO format  ---
for n=0,nlist-1 do begin &$
  filenames=findfile(disk + modellist(n) + '.f20_*') &$
  if (filenames(0) eq '') then begin &$
    print, 'No F20-file for ' + modellist(n) &$
  endif else begin &$
    rdf20, filenames(n_elements(filenames)-1), $
      istop=istop, jstop=jstop, $
      ibound=ibound, ibotm=ibotm, igas=igas, iparsp=iparsp, $
      ifco=ifco, $
      lmax=lmax, mxbcor=mxbcor, iflcor=iflcor, mside=mside, $
      dr=dr, logg=logg, Teff=Teff, $
      Fimplc=Fimplc, Srgam=Srgam, $
      Hpbot=Hpbot, Hptop=Hptop, Htau0=Htau0, Tsurf=Tsurf, tservo=tservo, $
      abuX=abuX, abuY=abuY, Qmol=Qmol &$
    hddata_analyse, modellist(n), eosfile=eosfile, optafile=opafile &$
    uio_wrf20, outdisk + modellist(n) + '.par', $
      form='formatted', conv='ieee_4', $
      model=modellist(n), f20file=filenames(n_elements(filenames)-1), $
      istop=istop, jstop=jstop, $
      ibound=ibound, ibotm=ibotm, igas=igas, iparsp=iparsp, $
      ifco=ifco, $
      lmax=lmax, mxbcor=mxbcor, iflcor=iflcor, mside=mside, $
      dr=dr, logg=logg, Teff=Teff, $
      Fimplc=Fimplc, Srgam=Srgam, $
      Hpbot=Hpbot, Hptop=Hptop, Htau0=Htau0, Tsurf=Tsurf, tservo=tservo, $
      abuX=abuX, abuY=abuY, Qmol=Qmol, $
      eosfile=eosfile, opafile=opafile &$
  endelse &$
endfor
;
;
; --- Append counter file if it exists ---
for n=0,nlist-1 do begin &$
  parfiles=findfile(outdisk + modellist(n) + '.par') &$
  if (parfiles(0) ne '') then begin &$
    counterfiles=findfile(counterdisk + modellist(n) + '.counter') &$
    if (counterfiles(0) ne '') then begin &$
      print, 'cat ' + counterfiles(0) + ' >> ' + parfiles(0) &$
      spawn, 'cat ' + counterfiles(0) + ' >> ' + parfiles(0) &$
    endif else begin &$
      print, modellist(n) + ': no counter file' &$
    endelse &$
  endif &$
endfor
;
; end ; f20_2_uio
