pro IO4050, MODE, NCH, IDIM, JDIM, IIO, JIO, $
            KTIME, TIME1, DT, IDATE, ICLOCK, $
            UXX0, SXX0, DZX, VXX0, PXX0, DMDVX0, DFDPX0, $
            BFLX, RXXX, VBT0, DMDVB0, FBOTAV, FTOPAV, $
            SXIB, ZOX, TOX, POX, U, V, P, T
;+
; NAME:
;   IO4050
;
; PURPOSE:
;   Input and output of FOR40 and FOR50 files
;
; CATEGORY:
;   Input/Output
;
; MODIFICATION HISTORY:
;   16. 7.93: Translation: FORTRAN -> IDL (Bernd Freytag, Astrophysik Kiel)
;   10. 4.95: (B.F.) Writing is now possible
;   22. 5.95: (B.F.) Mode 'read2head' added
;    1. 6.95: (B.F.)
;    3. 6.99: (B.F. Copenhagen) Read date and time
;   
;-
;
FORMAT900='(4X, E18.11, 1X, E18.11, 1X, E18.11)'
FORMAT910='(I6, E19.12, 1X, E19.12, 5X, A9, 2X, A9)'
FORMAT920='(4E20.13)'
;
IF (strupcase(MODE) EQ 'READ1') THEN begin
  ;
  ; --- Read in geometrical depth scale and horizontally averaged model ---
  ;
  tmp=fltarr(3,jdim)
  ;  READ (NCH,900) tmp
  READf, NCH, format=format900, tmp
  ZOX=reform(tmp(0,*),jdim)
  TOX=reform(tmp(1,*),jdim)
  POX=reform(tmp(2,*),jdim)
ENDIF
;
IF (strupcase(MODE) EQ 'READ2HEAD') THEN begin
  ;
  ; --- Read in two-dimensional model to be advanced in time --- 
  ;
  IDATE='Date     '
  ICLOCK='Time     '
  ktime=0L
  time1=0.0
  dt=0.0
  uxx0=0.0
  sxx0=0.0
  dzx=0.0
  READf, NCH, format=format910, KTIME, TIME1, DT, IDATE, ICLOCK
  READf, NCH, format=format920, UXX0, SXX0, DZX
  READf, NCH, format=format920, VXX0, PXX0, DMDVX0, DFDPX0
;  READf, NCH, format=format920, BFLX, RXXX, VBT0, DMDVB0
  VBT0=0.0
  DMDVB0=0.0
  READf, NCH, format=format920, BFLX, RXXX
  SXIB=fltarr(IDIM)
  READf, NCH, format=format920, SXIB
endif
;
IF (strupcase(MODE) EQ 'READ2') THEN begin
  ;
  ; --- Read in two-dimensional model to be advanced in time --- 
  ;
  IDATE='Date     '
  ICLOCK='Time     '
  ktime=0L
  time1=0.0
  dt=0.0
  uxx0=0.0
  sxx0=0.0
  dzx=0.0
  READf, NCH, format=format910, KTIME, TIME1, DT, IDATE, ICLOCK
  READf, NCH, format=format920, UXX0, SXX0, DZX
  READf, NCH, format=format920, VXX0, PXX0, DMDVX0, DFDPX0
;  READf, NCH, format=format920, BFLX, RXXX, VBT0, DMDVB0
  VBT0=0.0
  DMDVB0=0.0
  READf, NCH, format=format920, BFLX, RXXX
  SXIB=fltarr(IDIM)
  READf, NCH, format=format920, SXIB
;  tmp=fltarr(4)
;  for i=0,iio-1,4 do begin
;    read, nch, format=format920, tmp
;    for ii=0,3 do begin
;      iii=i+ii
;      if (iii lt iio) then sxib(iii)=tmp(ii)
;    endif
;  endfor
;
  tmp=fltarr(4,iio)
  u=fltarr(idim,jdim)
  v=fltarr(idim,jdim)
  P=fltarr(idim,jdim)
  T=fltarr(idim,jdim)
  for j=0,jio-1 do begin
    readf, nch, format=format920, tmp
    u(0:iio-1,j)=tmp(0,*)
    v(0:iio-1,j)=tmp(1,*)
    p(0:iio-1,j)=tmp(2,*)
    T(0:iio-1,j)=tmp(3,*)
  endfor
  ;
  fbotav=0.0 & ftopav=0.0
  readf, nch, format=format920, fbotav, ftopav
ENDIF
;
IF (strupcase(MODE) EQ 'WRITE') THEN begin  
  ; --- Write geometrical depth scale and horizontally averaged model ---
  tmp=fltarr(3,jdim)
  tmp(0,*)=ZOX
  tmp(1,*)=TOX
  tmp(2,*)=POX
  printf, nch, format=format900, tmp
  ; --- Write some scalars and a vector ---
  printf, nch, format=format910, KTIME, TIME1, DT, IDATE, ICLOCK
  printf, nch, format=format920, UXX0, SXX0, DZX
  printf, nch, format=format920, VXX0, PXX0, DMDVX0, DFDPX0
  printf, nch, format=format920, BFLX, RXXX, VBT0, DMDVB0
  printf, nch, format=format920, SXIB
  ; --- Write bulk data ---
  tmp=fltarr(4,iio)
  for j=0,jio-1 do begin
    tmp(0,*)=u(0:iio-1,j)
    tmp(1,*)=v(0:iio-1,j)
    tmp(2,*)=p(0:iio-1,j)
    tmp(3,*)=T(0:iio-1,j)
    printf, nch, format=format920, tmp
  endfor
  ; --- Write two scalars ---
  printf, nch, format=format920, fbotav, ftopav
endif
;
end ; IO4050
