pro ioes, filename, iversion, nheader, header, nalpha, nteff, ngrav, nvars, $
          alpha, teff, grav, es, idstring, formatted=formatted, write=write
; IOES reads and writes files in the es-data format.
; It is able to handle formatted or xdr-unformatted data.
; Default action is reading a xdr-unformatted file.
; This action can be modified by setting the keyword 'formatted' and 'write'.
; e.g. ioes, ..., /write, /formatted 
;      writes an formatted es file.


print, 'IOES: file = ', filename

get_lun, unit

if not keyword_set(write) then begin

iversion = 0
nheader  = 0
nalpha   = 0
nteff    = 0
ngrav    = 0
nvars    = 0

if keyword_set(formatted) then begin
  openr, unit, filename
  readf, unit, iversion, nheader
  header = strarr(nheader)
  readf, unit, header
  for i = 0, nheader-1 do print, header(i)

  readf, unit, nalpha, nteff, ngrav, nvars
  alpha    = fltarr(nalpha)
  teff     = fltarr(nteff)
  grav     = fltarr(ngrav)
  idstring = strarr(nvars) 
  es       = fltarr(nalpha, nteff, ngrav, nvars)
  readf, unit, alpha
  readf, unit, teff
  readf, unit, grav
  readf, unit, idstring
  readf, unit, es
endif else begin
  ; Read XDR file
  openr, unit, filename, /xdr
  readu, unit, iversion, nheader
  header = strarr(nheader)
  readu, unit, header
  for i = 0, nheader-1 do print, header(i)

  readu, unit, nalpha, nteff, ngrav, nvars
  alpha    = fltarr(nalpha)
  teff     = fltarr(nteff)
  grav     = fltarr(ngrav)
  idstring = strarr(nvars) 
  es       = fltarr(nalpha, nteff, ngrav, nvars)
  readu, unit, alpha
  readu, unit, teff
  readu, unit, grav
  readu, unit, idstring
  readu, unit, es
endelse

close, unit
free_lun, unit

idstring = strcompress(idstring,/remove_all)

endif else begin
; Writing of a file 
if keyword_set(formatted) then begin
  ; Write formatted
  openw, unit, filename
  printf, unit, iversion, nheader
  for i = 0, nheader-1 do printf, unit, header(i)
  printf, unit, nalpha, nteff, ngrav, nvars
  printf, unit, alpha
  printf, unit, teff
  printf, unit, grav
  for i = 0, nvars-1 do printf, unit, idstring(i)
  printf, unit, es
endif else begin
  ; Write XDR file
  openw,  unit, filename, /xdr
  writeu, unit, iversion, nheader
  writeu, unit, header
  writeu, unit, nalpha, nteff, ngrav, nvars
  writeu, unit, alpha
  writeu, unit, teff
  writeu, unit, grav
  writeu, unit, idstring
  writeu, unit, es
endelse

close, unit
free_lun, unit

endelse

return
end
