pro phoenix_opac_filterwavelength, filespec, nww, ww, wt
; Filters the wavelength grid from fort.73 files
; Returns the sorted wavelength ww
; and integration weights wt
; MODIFICATION HISTORY:
;   2007       (HGL)
;   2007-09-20 (BF) Routine renamed
;   2007-10-16 (BF) weights wt -> wt*0.5

  n_OpacLayer= 128
  wskip      = 8 + n_OpacLayer*7*4; offset in byte per data block
  ww         = dblarr(4000000L,/nozero)
  wl         = double(0.0)

  files = findfile(filespec, count=nfiles)
  get_lun, unit

  nww = 0L

  for ifile=0,nfiles-1 do begin
    openr, unit, files[ifile], /swap_if_big_endian
    print, '%filter_wavelength: doing ', ifile, files[ifile], format='(A, I3, 1X, A)'
    nskip = 0L
    wlMinusOneCounter=0
    while ((not eof(unit)) and (wlMinusOneCounter lt 2)) do begin
      readu, unit, wl & ww(nww) = wl
      if (wl eq -1.0D+00) then wlMinusOneCounter=wlMinusOneCounter+1
      nww   = nww+1L
      nskip = nskip+1L
      point_lun, unit, nskip*wskip
    endwhile
    close, unit
  endfor

  free_lun, unit 
  
  ; --- Remove trailing zeros ---
  ww = temporary(ww[0:nww-1])
  ; --- Sort ---
  ww = temporary(ww(sort(ww)))
  ; --- Remove double entries ---
  ww = temporary(ww(uniq(ww)))
  ; --- Remove first entry (-1.0) ---
  ww = temporary(ww[1:*])
  nww= n_elements(ww) 

  ; --- Compute weights ---
  wt       =(shift(ww,-1)-shift(ww,1))
  wt[0]    =ww[1]-ww[0]
  wt[nww-1]=ww[nww-1]-ww[nww-2]
  wt       =0.5*wt

end ; phoenix_opac_filterwavelength
