pro rd_sort_opac, filespec, gfile, dfile

; Full version
;  lgtmin     = 2.5
;  lgtmax     = 4.0
;  tstep      = 0.04
;  lgpmin     = -4.0
;  lgpmax     = 10.0
;  pstep      = 0.2

  lgtmin     = 2.5
  lgtmax     = 4.0
  tstep      = 0.1
  lgpmin     = 0.0
  lgpmax     = 10.0
  pstep      = 0.5

  nt         = long(nint((lgtmax-lgtmin)/tstep)+1)
  np         = long(nint((lgpmax-lgpmin)/pstep)+1)
  opac_layer = 128
  maxwl      = 100000L

  opacity_wl_block = {opac_data, wl:double(0.0), $
       xc:fltarr(opac_layer), sc:fltarr(opac_layer), $ 
       xd:fltarr(opac_layer), sd:fltarr(opac_layer), $ 
       xal:fltarr(opac_layer), xml:fltarr(opac_layer), $ 
       xnlte:fltarr(opac_layer)}
  owlsize = 8+opac_layer*7*4   ; bytes

  xsblock = {wl:double(-1.0),wt:0.0, x:fltarr(np,nt)-99.0}
  xssize  = 8L + 4L + nt*np*4L    ; bytes
 
  data  = replicate(opacity_wl_block,maxwl)
  ndata = 0L

; Setup temperature, pressure, and wavelength grids
  logt = lgtmin+findgen(nt)*tstep
  logp = lgpmin+findgen(np)*pstep
  filter_wavelength, filespec, nwl, wl, wt
  wt = float(wt)
  print, '% rd_sort_opac: number of detected wavelengths ', nwl
  print, '% rd_sort_opac: data file size (bytes) ', nwl*xssize

  get_lun, unit
  get_lun, gunit
  get_lun, dunit

  openw, gunit, gfile, /swap_if_little_endian
  openw, dunit, dfile, /swap_if_little_endian

; Write empty output files
  for iwl=0L,nwl-1 do begin
    xsblock.wl = wl(iwl)
    xsblock.wt = wt(iwl)
    writeu, gunit, xsblock
    writeu, dunit, xsblock 
  endfor

  files = findfile(filespec, count=nfiles)

  for ifile=0,nfiles-1 do begin
    openr, unit, files(ifile), /swap_if_big_endian
;    openr, unit, files(ifile)
    while not eof(unit) do begin
      readu, unit, opacity_wl_block

      if (opacity_wl_block.wl lt 0.0) then begin
;       Block containing thermodynamic data is encountered
;       Setup temperature pressure indices
        it = round((opacity_wl_block.xc-lgtmin)/tstep) 
        ip = round((opacity_wl_block.sc-lgpmin)/pstep)
        ii = ip + it*np ; 1D index
;       Check whether 1D mapping forms a consecutive sequence
;        if max(ii-shift(ii,1)) gt 1 then begin
;          print, '% rd_sort_opac: error: ii non consecutive'
;          stop
;        endif
        lgrho = opacity_wl_block.xd

      endif else begin
;       Opacity block encountered. It is assumed that ii is set.
        iw = (where(opacity_wl_block.wl eq wl))(0)
        print, '% rd_sort_opac: ii(0), iw', ii(0), iw
        xgas =(alog10(opacity_wl_block.xc  + opacity_wl_block.sc $
                   +  opacity_wl_block.xal + opacity_wl_block.xml $
                   +  opacity_wl_block.xnlte) - lgrho) > (-99.0)
        xdust= (alog10(opacity_wl_block.xd  + opacity_wl_block.sd) $
                      - lgrho) > (-99.0)
        pos = long(xssize)*iw + ii(0)*4L + 12L; bytes
        point_lun, gunit, pos
        writeu, gunit, xgas
        point_lun, dunit, pos
        writeu, dunit, xdust
      endelse

    endwhile
    close, unit
  endfor

  close, gunit, dunit
  free_lun, unit, gunit, dunit
end
