pro RDDIFUZ1D, lfilename, title0=title0, $
  nnd=nnd, numel=numel, istp=istp, ielzc1=ielzc1, ielzc2=ielzc2, $
  nt=nt, nsteps=nsteps, iset=iset, iaccrad=iaccrad, ivisc=ivisc, $
  itherm=itherm, iburn=iburn, ivdzc=ivdzc, izc=izc, iturb=iturb, $
  imdot=imdot, iacret=iacret, n_type=ntype, nr=nr, mixmod=mixmod, $
  conct=conct, theta=theta, timein=timein, toan=toan, conc0=conc0, $
  mstar=mstar, emdot=emdot, ve=ve, dmmmin=dmmmin, dmmmax=dmmmax, $
  aa=aa, bb=bb, rstar=rstar, rzc1=rzc1, rzc2=rzc2, $
  lstar=lstar, teff=teff, ztr=ztr, atr=atr, concx=concx, $
  concxzc=concxzc, cism0=cism, cismfac=cismfac, qq=qq, sn=sn, $
  rsol=rsol, acrate=acrate, zcmass=zcmass, Hpzc=Hpzc, $
  r0=r0, dmm0=dmm0, w0=w0, t0=t0, rho0=rho0, $
  vset0=vset0, vdth0=vdth0, vrad0=vrad0, vdif0=vdif0, $
  d12org=d12org, dturb=dturb, d12tot=d12tot, $  
  dmmall=dmm, timeall=time, abzc=abzc, concall=concall
;
;+
; NAME:
;   RDDIFUZ1D
; PURPOSE:
;   Read in data from the FORTRAN program DIFUZ1D.F which solves 1d diffusion
;   problems
; CALLING SEQUENCE:
;   rddifuz1d, ...
; INPUTS:
; OUTPUTS:
; KEYWORDS:
; PROCEDURE:
;   Straightforward. Data are expected to exist in 8 byte format (double
;   precision, 2*long)
; EXAMPLE:
; MODIFICATION HISTORY:
;   16.5.94 Adopted from the routine PARTRANS.PRO from Paul Charbonneau
;           by Bernd Freytag
;   19.5.94 New data format for DIFUZ1D.F (B.F.)
;   21.5.94 Data format modified (B.F.)
;   22.5.94 Hpzc into input list (B.F.)
;-
;
; --- Get free channel number and open file ---
get_lun, ncl
openr, ncl, /f77_unformatted, lfilename
;
; --- Read information strings ---
version=string(' ',format='(a80)') 
title0=string(' ',format='(a80)') 
date=string(' ',format='(a20)') 
time=string(' ',format='(a20)') 
ctime=1.e0
readu, ncl, version
print, version
readu, ncl, title0
print, title0
readu, ncl, date, time
print, date, time
;
il=2
;
; --- Read list of inter values (ignore every 2nd because of 8 byte length) ---
ldum=0L
num_integer=0L
readu, ncl, num_integer
help, num_integer
;idum=replicate(2L,num_integer)
;readu, ncl, idum
;idum=reform(idum(0,*))
;print,idum
;help,idum
;nnd   =idum(0) &numel =idum(1) &istp  =idum(2) &ielzc1=idum(3) &ielzc2=idum(4)
;nt    =idum(5) &nsteps=idum(6) &iset  =idum(7) &iaccrad=idum(8)&ivisc =idum(9)
;itherm=idum(10)&iburn =idum(11)&ivdzc =idum(12)&izc   =idum(13)&iturb =idum(14)
;imdot =idum(15)&iacret=idum(16)&ntype =idum(17)&nr    =idum(18)&mixmod=idum(19)
nnd   =0L &numel =0L &istp  =0L &ielzc1=0L &ielzc2=0L
nt    =0L &nsteps=0L &iset  =0L &iaccrad=0L&ivisc =0L
itherm=0L&iburn =0L&ivdzc =0L&izc   =0L&iturb =0L
imdot =0L&iacret=0L&ntype =0L&nr    =0L&mixmod=0L
;readu, ncl, nnd, numel, istp, ielzc1, ielzc2, $
;            nt, nsteps, iset, iaccrad, ivisc, $
;            itherm, iburn, ivdzc, izc, iturb, $
;            imdot, iacret, ntype, nr, mixmod
readu, ncl, nnd
readu, ncl, numel
readu, ncl, istp
readu, ncl, ielzc1
readu, ncl, ielzc2
readu, ncl, nt
readu, ncl, nsteps
readu, ncl, iset
readu, ncl, iaccrad
readu, ncl, ivisc
readu, ncl, itherm
readu, ncl, iburn
readu, ncl, ivdzc
readu, ncl, izc
readu, ncl, iturb
readu, ncl, imdot
readu, ncl, iacret
readu, ncl, ntype
readu, ncl, nr
readu, ncl, mixmod
for i=21,num_integer do readu, ncl, ldum
help, nnd, numel, istp, ielzc1, ielzc2
;
; --- Read list of real values ---
num_real=0L
readu, ncl, num_real
help, num_real
fdum=dblarr(num_real)
readu, ncl, fdum
conct =fdum(0) &theta =fdum(1) &timein=fdum(2) &toan  =fdum(3) &conc0 =fdum(4)
mstar =fdum(5) &emdot =fdum(6) &ve    =fdum(7) &dmmmin=fdum(8) &dmmmax=fdum(9)
aa    =fdum(10)&bb    =fdum(11)&rstar =fdum(12)&rzc1  =fdum(13)&rzc2  =fdum(14)
lstar =fdum(15)&teff  =fdum(16)&ztr   =fdum(17)&atr   =fdum(18)&concx =fdum(19)
concxzc=fdum(20)&cism =fdum(21)&cismfac=fdum(22)&qq   =fdum(23)&sn    =fdum(24)
rsol  =fdum(25)&acrate=fdum(26)&zcmass=fdum(27)&Hpzc  =fdum(28)
;
; --- Read list of vectors describing the stellar model ---
num_vector=0L
len_vector=0L
;readu, ncl, num_vector, ldum, len_vector, ldum
;readu, ncl, num_vector, len_vector
readu, ncl, num_vector
readu, ncl, len_vector
help, num_vector, len_vector
r0=dblarr(len_vector)   & readu, ncl, r0
dmm0=dblarr(len_vector) & readu, ncl, dmm0
w0=dblarr(len_vector)   & readu, ncl, w0
t0=dblarr(len_vector)   & readu, ncl, t0
rho0=dblarr(len_vector) & readu, ncl, rho0
vset0=dblarr(len_vector)& readu, ncl, vset0
vdth0=dblarr(len_vector)& readu, ncl, vdth0
vrad0=dblarr(len_vector)& readu, ncl, vrad0
vdif0=dblarr(len_vector)& readu, ncl, vdif0
d12org=dblarr(len_vector)& readu, ncl, d12org
dturb=dblarr(len_vector) & readu, ncl, dturb
d12tot=dblarr(len_vector)& readu, ncl, d12tot
;
;
; --- Read in relative mass coordinate ---
;readu, ncl, num_vector, ldum, len_vector, ldum
;readu, ncl, num_vector, len_vector
readu, ncl, num_vector 
readu, ncl, len_vector
help, num_vector, len_vector
xx=dblarr(len_vector)
readu, ncl, xx
dmm=float(10.^(aa-bb*xx))
;
; --- End of header
;readu, ncl, num_vector, ldum, len_vector, ldum
;readu, ncl, num_vector, len_vector
readu, ncl, num_vector
readu, ncl, len_vector
help, num_vector, len_vector
;
; --- Read main data field ---
typestring='12345678'
readu, ncl, typestring
print, typestring
len_list=0L
;readu, ncl, len_list,ldum, num_integer,ldum, num_real,ldum, num_vector,ldum, $
;            len_vector,ldum
;readu, ncl, len_list, num_integer, num_real, num_vector, $
;            len_vector
readu, ncl, len_list 
readu, ncl, num_integer
readu, ncl, num_real
readu, ncl, num_vector
readu, ncl, len_vector
help, len_list, num_integer, num_real, num_vector, len_vector
;
if (typestring ne 'long    ') then message, 'Unsupported file type'
;
; --- Define type of some quantities ---
time =fltarr(nt)
abzc =fltarr(nt)
conc =dblarr(nnd)
;
concall=fltarr(nnd,nt)
;
ctime=1.d0
cabzc=1.d0
time(0:0)=0.
abzc(0:0)=conc0
;
idum=lonarr(2)
;
; --- Loop over timesteps ---
for it=0,nt-1 do begin
  ; --- Read results for omega and bphi, for each solution
;  readu, ncl, idum
;  istep=fix(idum(0))
  readu, ncl, ldum
  istep=fix(ldum)
  readu, ncl, ctime, cabzc
  readu, ncl, conc
  print, istep, ctime, cabzc
  time(it:it)=float(ctime/3600./365.25/24./1.e6)
  abzc(it:it)=float(cabzc)
  concall(*,it)=conc/conc0
endfor
;
; --- Close data file ---
close, ncl
free_lun, ncl
;
;--------------------------------------------------------------------------
;
; --- Get free channel number and open file ---
;get_lun, ncs
;openr, ncs, /f77_unformatted, sfilename
;;
;title0=string(' ',format='(a64)') 
;;date=string(' ',format='(a8)') 
;;time=string(' ',format='(a8)') 
;readu, ncs, title0
;;readu, ncs, date, time
;;
;idum=lonarr(26)
;readu, ncs, idum
;nnd=idum(0)
;;
;fdum=dblarr(12)
;readu, ncs, fdum
;print, title0
;;
;xx   =dblarr(nnd)
;;
;readu, ncs, xx
;;
;idum=lonarr(2)
;readu, ncs, idum
;nt=idum(0)
;time =dblarr(nt)
;abzc =dblarr(nt)
;readu, ncs, time
;readu, ncs, abzc
;abzc=abzc/conc0
;time=time/3600./24./365.25
;
end ; RDDIFUZ1D




