; pro RDDIFUZ1DSTART
;
;+
; NAME:
;   RDDIFUZ1DSTART
;
; PURPOSE:
;   Read in data from the FORTRAN program DIFUZ1D.F which solves 1d diffusion
;   problems by calling RDDIFUZ1D
;
; CALLING SEQUENCE:
;   @RDDIFUZ1DSTART
;
; INPUTS, OUTPUTS, KEYWORDS:
;   None
;
; PROCEDURE:
;   Call RDDIFUZ1D, ...
;
; SUBROUTINES:
;   rddifuz1d: Read data file generated by the FORTRAN program DIFUZ1D.F
;   plotlines: Plot a 2D array as a set of lines
;
; SIDE EFFECTS:
;   A lot
;
; EXAMPLE:
;   @RDDIFUZ1DSTART
;
; MODIFICATION HISTORY:
;   16.5.94 Written (Bernd Freytag)
;   19.5.94 Modification to account for new DIFUZ1D.F data format
;   20.5.94 New header
;   21.5.94 6 plots (B.F.)
;-
;
; --- Define filename with model data ---
; --- Ca ---
;lfilename='../diff/l_ca_7500_n1.i3e'
;lfilename='../diff/l_ca_7500_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/lca_8000_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/lca_8000_mm3_qq14_sn8_n2.i3e'
;lfilename='../diff/l_ca_8000_mm2_qq14_sn8_n1.i3e'
;lfilename='../diff/l_ca_8000_mm1_qq14_sn8_n1.i3e'
;lfilename='../diff/l_ca_8000_mm2_qq12_sn8_n1.i3e'
;lfilename='../diff/l_ca_8000_mm3_qq14_sn8_n3.i3e'
;lfilename='../diff/l_ca_8000_mm3_qq14_sn8_n4.i3e'
;lfilename='../diff/l_ca_8000_n1.i3e'
;lfilename='../diff/l_ca_8000_n2.i3e'
;lfilename='../diff/l_ca_8000_mm2em13_n1.i3e'      ; --- Accretion ---
;lfilename='../diff/l_ca_8000_mm1em13_n1.i3e'
;lfilename='../diff/l_ca_8000_mm1em12_n1.i3e'
;lfilename='../diff/l_ca_8000_mm1em13_n2.i3e'
;lfilename='../diff/l_ca_8000_mm5em14_n1.i3e'
;lfilename='../diff/l_ca_8000_mm2em14_n1.i3e'
;lfilename='../diff/l_ca_8000_mm1em14_n1.i3e'
;lfilename='../diff/l_ca_8000_mm5em15_n1.i3e'
;lfilename='../diff/l_ca_8000_mp5em14_n1.i3e'      ; --- Mass loss: ERROR ---
;lfilename='../diff/l_ca_8000_mp5em14_n2.i3e'
;lfilename='../diff/l_ca_8000_mp5em14_n3.i3e'
;lfilename='../diff/l_ca_8000_mp5em14_n4.i3e'
;lfilename='../diff/l_ca_8000_mp5em14_n5.i3e'
;lfilename='../diff/l_ca_8000_mp5em14_n6.i3e'
;lfilename='../diff/l_ca_8000_mp1em13_n1.i3e'
;lfilename='../diff/l_ca_8500_n1.i3e'
;lfilename='../diff/l_ca_8500_mm3_qq14_sn8_n1.i3e'
; --- Ti ---
;lfilename='../diff/l_ti_7500_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/l_ti_7500_n1.i3e'
;lfilename='../diff/l_ti_8000_mm3_qq14_sn8_n1.i3e'   ; <---
lfilename='../diff/l_ti_8000_n1.i3e'
;lfilename='../diff/l_ti_8500_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/l_ti_8500_n1.i3e'
; --- Eu ---
;lfilename='../diff/l_eu_7500_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/l_eu_7500_n1.i3e'
;lfilename='../diff/l_eu_8000_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/l_eu_8000_mm3_qq14_sn8_n2.i3e'
;lfilename='../diff/l_eu_8000_n1.i3e'
;lfilename='../diff/l_eu_8500_mm3_qq14_sn8_n1.i3e'
;lfilename='../diff/l_eu_8500_n1.i3e' ; ERROR
;lfilename='../diff/l_eu_8500_n2.i3e'
;
dev='x'
;
; --- Read in model data ---
if (n_elements(conc) eq 0) then $
 RDDIFUZ1D, lfilename, title0=title0, $
  nnd=nnd, numel=numel, istp=istp, ielzc1=ielzc1, ielzc2=ielzc2, $
  nt=nt, nsteps=nsteps, iset=iset, iaccrad=iaccrad, ivisc=ivisc, $
  itherm=itherm, iburn=iburn, ivdzc=ivdzc, izc=izc, iturb=iturb, $
  imdot=imdot, iacret=iacret, n_type=ntype, nr=nr, mixmod=mixmod, $
  conct=conct, theta=theta, timein=timein, toan=toan, conc0=conc0, $
  mstar=mstar, emdot=emdot, dmmmin=dmmmin, dmmmax=dmmmax, $
  aa=aa, bb=bb, rstar=rstar, rzc1=rzc1, rzc2=rzc2, $
  lstar=lstar, teff=teff, ztr=ztr, atr=atr, $
  cism0=cism, cismfac=cismfac, qq=qq, sn=sn, $
  rsol=rsol, zcmass=zcmass, Hpzc=Hpzc, $
  r0=r0, dmm0=dmm0, w0=w0, t0=t0, rho0=rho0, $
  vset0=vset0, vdth0=vdth0, vrad0=vrad0, vdif0=vdif0, $
  d12org=d12org, dturb=dturb, d12tot=d12tot, $  
  dmmall=dmm, timeall=time, abzc=abzc, concall=conc
;
; --- Header ---
head=strarr(8)
head(0)='!6' + title0 + '!X'
head(1)='File: ' + lfilename
;loggstar=alog10(mstar/rstar^2*10.0^4.44)
loggstar=alog10(mstar/rstar^2*10.0^4.40) ; 27.6.94 (B.F.)
head(2)= $
   'T!Deff!N = ' + strcompress(string(fix(Teff),format='(I5)'),/rem) + $
               ' K' + $
    '   logg = ' + strcompress(string(loggstar,format='(F7.3)'),/rem) + $
  '   M!D*!N = ' + strcompress(string(Mstar,format='(F7.3)'),/rem) + $
               ' M!D!9n!X!N' + $
  '   L!D*!N = ' + strcompress(string(Lstar,format='(F7.3)'),/rem) + $
               ' L!D!9n!X!N' + $
  '   R!D*!N = ' + strcompress(string(Rstar,format='(F7.3)'),/rem) + $
               ' R!D!9n!X!N'
head(3)='Time: ' + $
    strcompress(string(min(time),format='(F6.2)'),/remove) + $
                   ' ... ' + $
    strcompress(string(max(time),format='(F6.2)'),/remove) + $
                   ' Myr' + $
    '   Mass: ' + $
    '10!U'+strcompress(string(alog10(dmmmin),format='(F6.2)'),/remove)+'!N'+ $
                ' ... ' + $
    '10!U'+strcompress(string(alog10(dmmmax),format='(F6.2)'),/remove)+'!N'+ $
               ' M!D*!N'
head(4)= $
     'izc=' + strcompress(string(izc,format='(I1)'),/rem) + $
 '  iburn=' + strcompress(string(iburn,format='(I1)'),/rem) + $
 '  iturb=' + strcompress(string(iturb,format='(I1)'),/rem) + $
 '  iset=' + strcompress(string(iset,format='(I1)'),/rem) + $
 '  itherm=' + strcompress(string(itherm,format='(I1)'),/rem) + $
 '  iaccrad=' + strcompress(string(iaccrad,format='(I1)'),/rem) + $
 '  imdot=' + strcompress(string(imdot,format='(I1)'),/rem) + $
 '  iacret=' + strcompress(string(iacret,format='(I1)'),/rem)
head(5)= $
     'A = ' + strcompress(string(fix(atr),format='(I3)'),/rem) + $
  '   Z = ' + strcompress(string(fix(ztr),format='(I3)'),/rem)
  if (emdot gt 0.0) then begin &$
    head(5)=head(5) + '   Mass loss: ' + $
      strcompress(string(emdot,format='(E10.2)'),/rem) + ' M!D!9n!X!N/yr' &$
  endif else if (emdot lt 0.0) then begin &$ 
    head(5)=head(5) + '   Accretion Rate: ' + $
      strcompress(string(-emdot,format='(E10.2)'),/rem) + ' M!D!9n!X!N/yr' + $
      '   c!DISM!N/c!D0!N = ' + $
      strcompress(string(cismfac,format='(E10.2)'),/rem) &$
  endif
head(6)='Mass!Dcz,He!N = ' + '10!U' + $
  strcompress(string(alog10(dmm(ielzc1*istp)),format='(F6.2)'),/rem) + '!N' + $
  ' M!D*!N' + $
  '   Mass!Dcz,H!N = ' + '10!U' + $
  strcompress(string(alog10(dmm(ielzc2*istp)),format='(F6.2)'),/rem) + '!N' + $
  ' M!D*!N' + $
   '   H!DP,cz!N = ' + $
  strcompress(string(Hpzc*rsol/1e5,format='(F7.0)'),/rem) + ' km'
if (iturb eq 1) then begin &$
  head(7)='Convective Mixing(' + $
    strcompress(string(mixmod,format='(I2)'),/rem) + $
    '): D!Dturb!N(z)=10!U' + $
    strcompress(string(qq,format='(F8.2)'),/rem) + '!N*' &$
  if (mixmod eq 1) then begin &$
    head(7)=head(7) + '(!4q!X!Dcz!N/!4q!X(r))!U' &$
  endif else if (mixmod eq 2) then begin &$
    head(7)=head(7) + '(p!Dcz!N/p(r))!U' &$
  endif else if (mixmod eq 3) then begin &$
    head(7)=head(7) + 'e!U((r-r!Icz!U)/H!Ip,cz!U)*' &$
  endif &$
  head(7)=head(7) + strcompress(string(sn,format='(F8.2)'),/rem) + '!N' &$
endif
;
concmin=min(alog10(conc),cminsub)
concmax=max(alog10(conc),cmaxsub)
;concrange=[concmin < (-1.0), concmax > 1.0]
concrange=[-4.0,3.0]
dmmrange=[alog10(dmmmin),alog10(dmmmax)]
;dmmrange=[alog10(dmmmax),alog10(dmmmin)]
;
normct,-3,color=!ctv.colorrange2,drange=[concmin < (-0.1),concmax > 0.1], $
  rgbmask=[[-1.,-1.,-1.],[1.,1.,1.]]
;
if (strlowcase(dev) eq 'ps') then begin &$
  set_plot, 'ps', /copy &$
  device, /color, bits_per_pixel=8 &$
  ; --- Portrait, DIN A 4, USA --- &$
  device, /portrait , xsize=19.3, ysize=27.0,  xoffset=0.5, yoffset=0.5 &$
endif
;
!X.MARGIN=[12,3]
!P.CHARSIZE=1.6
!P.MULTI=[8,2,5,0,0]
;
plotlines, head=head, /date, dev=dev, /psnoinit, /noclose
;
; --- Plot 1 ---
!Y.RANGE=concrange
!X.RANGE=dmmrange
indranc=fix((findgen(10)+1.0)/10.0*nt - 1.0)
plotlines, alog10(dmm), alog10(conc(*,indranc)), $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='log!D10!N(c/c!D0!N)', $
  title='Evolution of Concentration', $
;  linecolor=(float(indranc)/nt*(!ctv.colorrange1(1)-!ctv.colorrange1(0)) + $
;             !ctv.colorrange1(0)), $
  dev=dev, /psnoinit, /noclose
oplot, [alog10(dmm(ielzc1*istp)), alog10(dmm(ielzc1*istp))], $
       [0.8*concrange(1),2.0*concrange(1)], linestyle=1
oplot, [alog10(dmm(ielzc2*istp)), alog10(dmm(ielzc2*istp))], $
       [0.8*concrange(1),2.0*concrange(1)], linestyle=1
!X.RANGE=0
;
; --- Plot 2 ---
;!X.RANGE=[min(time),max(time)+(max(time)-min(time))*0.1]
;indran=fix(findgen(16)/15.0*nnd)
indran=[nnd-1, cminsub mod nnd, cmaxsub mod nnd]
plotlines, time, alog10(transpose(conc(indran,*))), $
  xtitle='Time t [Myr]', ytitle='log!D10!N(c/c!D0!N)', $
  title='Concentration', $
  thick=[2,1,1], $
  dev=dev, /psnoinit, /noclose
;  /label, legitems=string(alog10(dmm(indran)),format='(F6.2)')
;
!Y.RANGE=0
!X.RANGE=0
;
; --- Plot 3 ---
!X.RANGE=dmmrange
!Y.RANGE=0
plotfield, alog10(dmm), time, alog10(conc), /oldaspect, $
  /tv, t_color=!ctv.colorrange2, $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='Time t [Myr]', $
  title='Concentration', xstyle=3, ystyle=3, $
  dev=dev, /psnoinit, /noclose
!X.RANGE=0
;
; --- Plot 4 ---
r=interpol(r0,dmm0,dmm)
!Y.RANGE=concrange
plotlines, r/rstar, alog10(conc(*,indranc)), $
  xtitle='r/R!D*!N', ytitle='log!D10!N(c/c!D0!N)', $
  title='Evolution of Concentration', $
  dev=dev, /psnoinit, /noclose
;  linecolor=(float(indranc)/nt*(!ctv.colorrange1(1)-!ctv.colorrange1(0)) + $
;             !ctv.colorrange1(0))
oplot, [r(ielzc1*istp), r(ielzc1*istp)]/rstar, $
       [0.8*concrange(1),2.0*concrange(1)], linestyle=1
oplot, [r(ielzc2*istp), r(ielzc2*istp)]/rstar, $
       [0.8*concrange(1),2.0*concrange(1)], linestyle=1
;
; --- Plot 5 ---
!X.RANGE=dmmrange
yval=alog10([[d12tot],[d12org],[dturb]]+1e-2)
!Y.RANGE=[0.0,max(yval)]
plotlines, alog10(dmm0), yval, $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='log!D10!N(D [cm!U2!N/s])', $
  title='Diffusion Coefficient', $
  thick=[2,1,1], $
  dev=dev, /psnoinit, /noclose
!Y.RANGE=0
;
; --- Plot 6 ---
!Y.RANGE=[(-max(abs(vdif0))) < (-1e-6), max(abs(vdif0)) > 1e-6]
plotlines, alog10(dmm0), [[vdif0],[vset0],[vdth0],[vrad0]], $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='v [cm/s]', $
  title='Diffusion Velocity', $
  thick=[2,1,1,1], $
  dev=dev, /psnoinit, /noclose
!Y.RANGE=0
;
; --- Plot 7 ---
plotlines, alog10(dmm0), alog10(rho0), $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='log!D10!N(!4q!X [g/cm!U3!N])', $
  title='Density', $
  dev=dev, /psnoinit, /noclose
;
; --- Plot 8 ---
plotlines, alog10(dmm0), t0, $
  xtitle='log!D10!N(!4D!XM/M!D*!N)', ytitle='T [K]', $
  title='Temperature', $
  dev=dev, /psnoinit
;
!P.CHARSIZE=1.6
!P.MULTI=0
!X.RANGE=0
!Y.RANGE=0
!X.MARGIN=[12,3]
;
; end ; RDDIFUZ1DSTART




