pro rdevosal, filelist, modellist=modellist, nmodelinter=nmodelinter, $
    alpha=alphaarr, age=agearr, Teff=Teffarr, Lum=Lumarr, Mass=Massarr, $
    Radiusarr=Radiusarr, gravarr=gravarr, $
    nage=nagearr
;------------------------------------------------------------------------------
;+
; NAME:
;   rdevosal ('read_evolution_salaris')
;
; PURPOSE:
;   Read evolutionary tracks
;
; CATEGORY:
;   Stellar Evolution, I/O
;
; CALLING SEQUENCE:
;   rdevosal, 
;
; INPUTS:
;   filelist:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   ;   Before first use type
;     delvar, init
;     uio_data, mode='allclose'  ; --- Attention ---
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   31.10.97 Written by Bernd Freytag, Astrophysik Kiel
;    6.11.97 (B.F.) Shift age to reach same luminosity (L=10) at same age
;-
;------------------------------------------------------------------------------
;
nmodels=n_elements(filelist) 
if (n_elements(modellist) eq 0) then begin
  modellist='evo' + strtrim(indgen(nmodels))
endif
;
; --- Initialize arrays ---
nage_max=1000 
agearr   =fltarr(nage_max,nmodels) 
Teffarr  =agearr   
Lumarr   =agearr 
Radiusarr=agearr 
gravarr  =agearr 
Massarr  =fltarr(nmodels) 
alphaarr =Massarr 
nagearr  =intarr(nmodels) 
;
; --- Read list of evolutionary models into arrays ---
for i=0,nmodels-1 do begin 
  ;
  ; --- Open file from list ---
  uio_data, mo='open', file=filelist(i), fam=modellist(i)
  ;
  ; --- Read some entries ---
  alpha=    uio_d('alpha'        ) 
  age  =    uio_d('sequence.age' ) 
  Lum  =    uio_d('sequence.l'   )
  Teff =    uio_d('sequence.teff') 
  Mass =avg(uio_d('sequence.m'   )) 
  ;
  ; --- Put quantities into arrays ---
  n=n_elements(age) 
  nagearr(i)        =n 
  agearr(0:n-1,i)   =age 
  Lumarr(0:n-1,i)   =Lum
  Teffarr(0:n-1,i)  =Teff
  Massarr(i)        =Mass
  alphaarr(i)       =alpha
endfor 
;
; --- Interpolation to age axis with equal values ---
if (n_elements(nmodelinter) eq 0) then begin
  nin=-1
endif else begin
  nin=nmodelinter < (nmodels-1)
endelse
;
if (nin ge 0) then begin
  ; --- Shift age -> same age = same luminosity ---
  ; --- Maximum start luminosity ---
;  Lum_start=max(Lumarr(0,*), imax)
  Lum_start=10.0
  age_shift=fltarr(nmodels)
  for i=0,nmodels-1 do begin
;    nage=min(nagearr)<50
    nage=nagearr(i)
    age_shift(i)=interpol(agearr(0:nage-1,i), Lumarr(0:nage-1,i), Lum_start)
  endfor
  for i=0,nmodels-1 do begin
    agearr(0:nagearr(i)-1,i)=agearr(0:nagearr(i)-1,i)- $
                             age_shift(i)+age_shift(nin)
  endfor
  print, lum_start
  print, age_shift
  ;
  ; --- Maximum start age ---
  age_start=max(agearr(0,*))
  ; --- Minimum end age ---
  age_end_arr=fltarr(nmodels)
  for i=0,nmodels-1 do age_end_arr(i)=agearr(nagearr(i)-1,i)
  age_end=min(age_end_arr)
  help, age_start, age_end
  ;
  ; --- Common age range (avoid extrapolation)
  ind=where((agearr(0:nagearr(nin)-1,nin) ge age_start) and $
            (agearr(0:nagearr(nin)-1,nin) le age_end  ), count )
  agearr( 0:count-1,nin)=agearr(ind,nin)
  Lumarr( 0:count-1,nin)=Lumarr(ind,nin)
  Teffarr(0:count-1,nin)=Teffarr(ind,nin)
  help,count
  ;
  ; --- Linear Interpolation ---
  for i=0,nmodels-1 do begin
    if (i ne nin) then begin
      Teffarr(0:count-1,i)=interpol(Teffarr(0:nagearr(i)-1,i), $
                                     agearr(0:nagearr(i)-1,i), $
                                     agearr(0:count-1,nin)  )
      Lumarr( 0:count-1,i)=interpol(Lumarr( 0:nagearr(i)-1,i), $
                                    agearr( 0:nagearr(i)-1,i), $
                                    agearr( 0:count-1,nin)  )
      agearr( 0:count-1,i)=agearr(0:count-1,nin)
    endif
  endfor
  ;
  nagearr(*)=count
  agearr( count:*,*)=0.0
  Lumarr( count:*,*)=0.0
  Teffarr(count:*,*)=0.0
endif
;
for i=0,nmodels-1 do begin
  ; --- Compute radius and surface gravity ---  
  n=nagearr(i)
  Radiusarr(0:n-1,i)=sqrt(Lumarr(0:n-1,i)*!CON.LSUN / $
                         (!CON.SIGMASB*Teffarr(0:n-1,i)^4 * 4.0*!PI))
  gravarr(0:n-1,i)  =!CON.G*Mass*!CON.MSUN/Radiusarr(0:n-1,i)^2
endfor
;
end ; rdevosal
