pro RDF06, filename, ktimearr=ktimearr, time1arr=time1arr, taur=taur, fr=fr
;
;+
; NAME:
;   RDF06
;
; PURPOSE:
;   Read data from F06 file written by HD code.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rdf06, filename [, ktimearr=ktimearr, time1arr=time1arr, taur=taur, fr=fr]
;
; INPUTS:
;   filename - Name of the input file
;
; OUTPUTS:
;
; KEYWORDS:
;
; EXAMPLE:
;   RDF06 
;
; MODIFICATION HISTORY:
;    29.7.94 Written (Bernd Freytag, Astrophysik, Kiel)
;-
;
print, 'Loading: ', filename
;
; --- Initializiation ---
jmax=300
nmax=100
ktime=0L
time1=0.0
ktimearr=lonarr(nmax)
time1arr=fltarr(nmax)
alogtaur=fltarr(jmax,nmax)
ifr=lonarr(jmax,nmax)
line=' '
oldline='1'
;
; --- Open file on free channel ---
get_lun, cunit
openr, cunit, filename
;
n=0
; --- Loop over all data sets ---
while (not EOF(cunit)) do begin
  ;
  ; --- Look for KTIME and read time data ---
  ; --- Skip invalid lines ---
  while ((not EOF(cunit)                     ) and $
         ((strmid(line,1,7) ne 'KTIME =') or $ 
          (strmid(line,18,7) ne 'TIME1 =') or $
          (line eq oldline)                  )) do begin
    readf, cunit, line
  endwhile
  ; --- Extract relevant data from line ---
  if (not EOF(cunit)) then begin
    oldline=line
    reads, line, format='(9X,I5,11X,G11.4)', ktime, time1
    print, 'KTIME=', ktime, '  time1=', time1
    ktimearr(n)=ktime
    time1arr(n)=time1
    ;
    ; --- Read effective temperature and total flux ---
    while ((not EOF(cunit)) and (strmid(line,1,6) ne 'TEFF =')) do begin
      readf, cunit, line
    endwhile
    reads, line, format='(7X,F8.1,13X,G12.5)', Teff, Fref
    print, 'Teff=', Teff, '  Fref=', Fref
    ;
    ; --- Skip 3 lines ---
    readf, cunit, line
    readf, cunit, line
    readf, cunit, line
    ;
    ; --- Read table with stratification data ---
    staur=' '
    ssrho=' '
    sfm=' '
    svm=' '
    iflux=lonarr(8)
    j=0
    jstop=j
    endtabflag=0
    repeat begin
      readf, cunit, format='(X,I2,4(F10.3,X,A1),3X,8I7)', $
        j, lgtaur,staur, lsrho,ssrho, lgfm,sfm, lgvm,svm, iflux
      if (j eq 1) then endtabflag=1
      jstop=jstop > j
      alogtaur(j-1,n)=lgtaur
      ifr(j-1,n)=iflux(2)
    ;  print, j, lgtaur, iflux(2)
    endrep until endtabflag
    ; --- Next time step ---
    n=n+1
  endif
endwhile
;
; --- Shrink arrays to proper size ---
ktimearr=ktimearr(0:n-1)
time1arr=time1arr(0:n-1)
taur=10.0^alogtaur(0:jstop-1,0:n-1)
fr=float(ifr(0:jstop-1,0:n-1))*1.0E-04
;
; --- Close channel ---
close, cunit
free_lun, cunit
;
end ; RDF06
