pro RDF15, nn, ifend, istop, jstop, ibound, ibplt, $
           mgeom, mside, ktime, time1, teff, glog, htau0, $
           r, z, tox, pox, u, v, p, t, d, $
           hdate, htime, htext, cnumber
;
; Version: 24-jun-1993
;
;***********************************************************************
;
;   ==================================
;   Last Update:   9-NOV-1992 16:59:03 
;   ==================================
;
;   Die Subroutine liest einen FOR15-File ein. Fuer nn = 0 wird der
;   erste Teil eingelesen (allgemeine Angaben, gemittelte Anfangs-
;   schichtung), fuer nn > 0 ein Modell fuer den darauffolgenden 
;   Zeitpunkt.
;
;   Der Output-Parameter ifend wird standardmaessig = 0 gesetzt. Wird
;   beim Versuch des Daten-Inputs das Ende des Files erreicht, wird
;   ifend = 1.
;
;   (isave spezifiziert, wo die gelesenen Datensaetze - nicht der erste 
;         Teil - abgelegt werden.)
;***********************************************************************
;
;+
; NAME:
;   RDF15
;
; PURPOSE:
;   Read in a data set from a F15 file.
;
; INPUTS:
;   nn - Input mode: 0 or 1
;   cnumber - Number of input channel
;
; OUTPUTS:
;
; MODIFICATION HISTORY:
;   Transformed from FORTRAN to IDL: (B.F.)
;   13.9.93 (B.F.)
;
;-
; --- save icod15 ---						???
common icod15_block, icod15
;
print, 'RDF15'
;
;--- Verzweigung: Lesen des Headers/Lesen der Daten ---
if (nn ne 0) then goto, label_1
;
;=======================================================================
;--- Lesen des Headers ---
;=======================================================================
;
;--- Header des Input-Files lesen ---
;**   First check whether file structure is 1, 2, or 3
chtst1='char1'
chtst2='char4'
readf, cnumber, format='(A1, 16X, A4)', chtst1, chtst2
;print, 'chtst1=',chtst1
;print, 'chtst2=',chtst2
;
; --- rewind(cnumber) ---
point_lun, cnumber, 0
;
istop=0
jstop=0
ibound=0
ibplt=0
mgeom=0
mside=0
hdate='char*9   '
htime='char*9   '
icod15=0
;
if (chtst1 eq '#') then begin & $
  ;**   read header according file structure 3: & $
  readf, cnumber, format='$(1x, 4i4, 2i2, 1x, a9, 2x, a9, 1x, i2)', $
        istop, jstop, ibound, ibplt, mgeom, mside, hdate, htime, icod15  & $
endif else if (chtst2 eq '    ') then begin & $
  ;**   read header according file structure 1: & $
  readf, cnumber, format='$(1x, 4i4, 5x, a9, 2x, a9)', $
    istop, jstop, ibound, ibplt, hdate, htime & $
  ;**   cylindrical case ... & $
  mgeom  = 1 &  mside  = 1 &  icod15 = 1 & $
endif else begin & $
  ;**   read header according file structure 2: & $
  readf, cnumber, format='$(1x, 4i4, 2i2, 1x, a9, 2x, a9)', $
        istop, jstop, ibound, ibplt, mgeom, mside, hdate, htime & $
  icod15 = 2 & $
endelse
;
print, hdate
;
idim=istop
jdim=jstop
;
;--- Dimensionsangaben ueberpruefen -----
;if (istop gt idim) then
;         write(*, '(1x, ''? ISTOP greater than '', i3)')idim
;         call EXIT
;      else if (jstop .gt. jdim) then
;         write(*, '(1x, ''? JSTOP greater than '', i3)')jdim
;         call EXIT
;      end if
;
r=fltarr(idim)
z=fltarr(jdim)
readf, cnumber, format='$(1x, 5e10.3)', r
readf, cnumber, format='$(1x, 5e10.3)', z
;
tmparr=fltarr(2,jstop)
readf, cnumber, tmparr
tox=tmparr(0,*)
pox=tmparr(1,*)
;
ifend = 0
goto, end_label
;
;
;=======================================================================
;--- Lesen der Daten ---
;=======================================================================
;
;--- Datenteil des Input-Files lesen ---
label_1:
;
ktime=0L
if (eof(cnumber)) then goto, eof_label
;
readf, cnumber, format='$(1x, i5, 1x, e10.3)', ktime, time1
readf, cnumber, format='$(1x, 6e11.4)', uxx, vxx, pxx, axx, sxx, dxx
readf, cnumber, format='$(1x, 6e11.4)', scdata
;
;--- Einlesen der Integer-Hilfsgroessen und ---
;--- Umwandlung der Integer-Groessen in gesuchte Daten ---
;
idim=istop
jdim=jstop
iu0=intarr(jdim)
iv0=intarr(jdim)
ip0=intarr(jdim)
it0=intarr(jdim)
id0=intarr(jdim)
u=fltarr( idim, jdim)
v=fltarr( idim, jdim)
p=fltarr( idim, jdim)
t=fltarr( idim, jdim)
d=fltarr( idim, jdim)
; --- Horizontal velocity ---
for i=0,istop-1 do begin & $
  readf, cnumber, format='$(1x,10i5)', iu0 & $
  u(i,*) = uxx * float(iu0) / scdata & $
endfor
; --- Vertical velocity ---
for i=0,istop-1 do begin & $
  readf, cnumber, format='$(1x,10i5)', iv0 & $
  v(i,*) = vxx * float(iv0) / scdata & $
endfor
; --- Pressure ---
for i=0,istop-1 do begin & $
  readf, cnumber, format='$(1x,10i5)', ip0 & $
  if (pxx lt 0.0) then begin & $
    p(i,*) = pox*abs(pxx)*(0.5 > float(ip0))/scdata & $
  endif else if ((pxx ge 0.0) and (icod15 le 3)) then begin & $
    p(i,*) = pxx * (0.5 > float(ip0))/scdata & $
  endif else begin & $
    p(i,*) = pox * exp(2.30258509299405*pxx*float(ip0)/scdata) & $
  endelse & $
endfor
; --- Temperature ---
for i=0,istop-1 do begin & $
  readf, cnumber, format='$(1x,10i5)', it0 & $
  t(i,*) = axx * float(it0) / scdata & $
endfor
; --- Radiative damping ---
for i=0,istop-1 do begin & $
  readf, cnumber, format='$(1x,10i5)', id0 & $
  d(i,*) = dxx * float(id0) / scdata & $
endfor
;
;--- Schlussteil des Datenfiles einlesen ---
;
htext='char9'
readf, cnumber, format='$(a80)', htext
if (icod15 lt 3) then begin & $
  readf, cnumber, format='$(f8.2, f6.2, f8.2, f9.2)', $
            teff, glog, rsu, reflam & $
  hp00 =-(z(jstop-1)-z(jstop-2-1))/alog(pox(jstop-1)/pox(jstop-2-1)) & $
  htau0= 0.75 * hp00 & $
endif else begin & $
  readf, cnumber, format='$(f8.2, f6.2, f8.2, f9.2, e13.5)', $
            teff, glog, rsu, reflam, htau0 & $
end
readf, cnumber, format='$(f6.2)', tur
if (tur lt 0.0) then begin & $
  xi=fltarr(jdim) & $
  readf, cnumber, format='(5f7.2)', xi & $
endif
mode=0
ionst=0
iprint=0
kprint=0
lpr1=0
lpr2=0
lprd=0
nlaks=0
readf, cnumber, format='$(7i4, /, i4)', mode, ionst, iprint, kprint, lpr1, lpr2, $
                                  lprd, nlaks
if (nlaks gt 0) then begin & $
  olam=fltarr(nlaks) & $
  readf, cnumber, format='$(5f10.2)', olam & $
endif
;NLAM = 60
;      if (nlaks .gt. nlam) then
;         write(*, '(1x, ''? Number of wavelength points greater than '',
;     >          i3)')nlam
;      end if
lcontr=0
readf, cnumber, format='$(i3)', lcontr
;
ifend = 0
goto, end_label
;
;
;=======================================================================
;--- END-OF-FILE beim Lesen erreicht ---
;=======================================================================
;
eof_label:
;close, cnumber
ifend = 1
;
end_label:
;
end ; RDF15
