pro RDF16, filename, tab=tab, time1arr=time1arr, ktimearr=ktimearr, $
  uarr=uarr, varr=varr, Tarr=Tarr, Parr=Parr, Darr=Darr, datearr=datearr
;
;------------------------------------------------------------------------------
;+
; NAME:
;   RDF16
;
; PURPOSE:
;   Read data from F16 file written by HD code.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   rdf16, filename
;
; INPUTS:
;   filename - Name of the input file
;
; OUTPUTS:
;   Only output keywords
;
; KEYWORDS:
;   tab - Main data array
;   time1arr - Real output vector with time data [sec]
;   ktimearr - Real output vector with time step number.
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   3. 4.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
print, 'RDF16: Loading: ', filename
;
; --- Open file on free channel ---
get_lun, cunit
openr, cunit, filename
;
; --- Initializiation ---
time1=0.0
ktime=0.0
nquant=0
quanttype=intarr(6)
jmax=0
date=''
filetype=''
line=''
;
; --- Save start position ---
point_lun, -cunit, startposition
;
; --- Read first line ---
readf, cunit, format='(X,2F10.2,7I2,I5,2X,17A,2X,9A)', $
  time1, ktime, nquant, quanttype, jmax, date, filetype
;
; --- Create arrays with enough elements ---
nmax=20000
tab=fltarr(nquant*jmax,nmax)
temp=fltarr(nquant*jmax)
time1arr=fltarr(nmax)
ktimearr=fltarr(nmax)
datearr=strarr(nmax)
;
; --- Number of lines to skip after first line of data block ---
iheadline=1+(jmax*6)/80
;
; --- Restore start position ---
point_lun, cunit, startposition
;
; --- Read file and extract all data blocks ---
n=0
while (not EOF(cunit) and (n lt nmax)) do begin
  ; --- Read time data ---
  readf, cunit, format='(X,2F10.2,7I2,I5,2X,17A,2X,9A)', $
    time1, ktime, nquant, quanttype, jmax, date, filetype
;  readf, cunit, format='(X,2F10.2)', time1, ktime
  time1arr(n)=time1
  ktimearr(n)=ktime
  datearr(n)=strmid(date,0,17)
  ; --- Skip line(s) ---
  for i=1,iheadline do readf, cunit, line
  ; --- Read data ---
  readf, cunit, temp, format='(6(E12.5))'
  tab(*,n)=temp
  ; --- Increment time index ---
  n=n+1
endwhile
;
; --- Shrink arrays to proper size ---
time1arr=time1arr(0:n-1)
ktimearr=ktimearr(0:n-1)
datearr=datearr(0:n-1)
tab=tab(*,0:n-1)
;
; --- Close channel ---
close, cunit
free_lun, cunit
;
; --- Extract from array TAB various quantities ---
quanttypenorm=quanttype ne 0
indarr=indgen(jmax)*nquant
;
if (quanttype(0) ne 0) then begin
  offset=total(quanttypenorm(0:0))-1
  uarr=tab(offset+indarr,*)
endif
if (quanttype(1) ne 0) then begin
  offset=total(quanttypenorm(0:1))-1
  varr=tab(offset+indarr,*)
endif
if (quanttype(2) ne 0) then begin
  offset=total(quanttypenorm(0:2))-1
  Parr=tab(offset+indarr,*)
endif
if (quanttype(3) ne 0) then begin
  offset=total(quanttypenorm(0:3))-1
  Tarr=tab(offset+indarr,*)
endif
if (quanttype(5) ne 0) then begin
  offset=total(quanttypenorm(0:5))-1
  Darr=tab(offset+indarr,*)
endif
;
end ; RDF16
