pro RDMARCS, filename, r, head=head, Teff=Teff, grav=grav
;
;+
; NAME:
;   RDMARCS
;
; PURPOSE:
;   Load data array from a .mod file produced by the MARCS program.
;
; CATEGORY:
;   Input/Output
;
; INPUT:
;   filename - Name of data file (including path)
;
; OUTPUT:
;   r - Array with output data, r(17,jstop)
;       r(0,*) - Level
;       r(1,*) - log tau_R
;       r(2,*) - log tau_5
;       r(3,*) - depth [cm], shifted
;       r(4,*) - TEMP [K]
;       r(5,*) - Electron  pressure
;       r(6,*) - Gas       pressure 
;       r(7,*) - Radiation pressure
;       r(8,*) - Turbulent pressure 
;       r(9,*) - Level
;       r(10,*) - log tau_R
;       r(11,*) - kappa_Ross [cm^2/g]
;       r(12,*) - Density [cm^-3]
;       r(13,*) - molecular weight mu
;       r(14,*) - VCONV [CM/S]
;       r(15,*) - Fconv/F
;       r(16,*) - RHOX [cm^-2]
;
; KEYWORDS:
;   head - Output: string array with some information
;   jstop - 
;
; MODIFICATION HISTORY:
;   04.11.2005 Written by M. Steffen, AIP
;-
;
print, 'Loading: ', filename
;
; --- Get free unit number ---
get_lun, cunit
; --- Open file ---
openr, cunit, filename
;
; --- Create input string variable ---
line=' '
; --- Find Teff ---
Teff=0.0
iflag=-1
while (iflag eq -1) do begin
   readf, cunit, line
   iflag=strpos(line, 'Teff')
   if (iflag ne -1) then begin
      reads, line, Teff
   endif
endwhile
; --- Find grav ---
grav=0.0
iflag=-1
while (iflag eq -1) do begin
   readf, cunit, line
   iflag=strpos(line, 'Surface gravity')
   if (iflag ne -1) then begin
      reads, line, grav
   endif
endwhile
; --- Find number of depth points ---
jstop=0
iflag=-1
while (iflag eq -1) do begin
   readf, cunit, line
   iflag=strpos(line, 'Number of depth points')
   if (iflag ne -1) then begin
      reads, line, jstop
      readf, cunit, line
   endif
endwhile
;
; --- Create data array ---
r =fltarr(17,jstop)
r1=fltarr(9,jstop)
r2=fltarr(8,jstop)
; --- Create string array for header data ---
head=strarr(2)
head1=''
head2=''
;
readf, cunit, head1
readf, cunit, r1
readf, cunit, head2
readf, cunit, r2
r(0:8, *)=r1
r(9:16,*)=r2
;
; --- Close file ---
close, cunit
; --- Free unit ---
free_lun, cunit
;
end ; RDMARCS
