pro rdpaths, filename, paths
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rdpaths ('read_paths')
;
; PURPOSE:
;   Read path data from ASCII file.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   
;
; INPUTS:
;   filename: ('filename') character, name of file(s) with array,
;             wildscards are possible
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A channel unit number is fetched and returned, a file is opened and closed.
;
; RESTRICTIONS:
;   The routine is slow.
;
; PROCEDURE:
;
; EXAMPLES:
;  plotlines,reform(paths(1,*,*))/!con.y,(reform(paths(11,*,*)/(paths(11,*,*)+paths(12,*,*))))(*,*)
;
; MODIFICATION HISTORY:
;   28. 7.03 (B.F.)
;-
;------------------------------------------------------------------------------
;
; --- Get free channel number ---
get_lun, nc
;
openr, nc, filename
headline=""
readf, nc, headline, format="(A)"
print, headline
;
number_of_constants=0L
if (headline eq " Dust formation along COBOLD particle paths") then begin
  readf, nc, number_of_constants, format="(I13)"
  print, "Number of constants:", number_of_constants
  array_of_constants=fltarr(number_of_constants)
  tmp_constant=0.0
   name_of_constant=""
  for i=0,number_of_constants-1L do begin
    readf, nc, tmp_constant, name_of_constant, format="(E13.6,4X,A)"
    print, name_of_constant + ": ", tmp_constant
    array_of_constants(i)=tmp_constant
  endfor
  n_time=0L
  n_col=0L
  n_ind=0L
  readf, nc, n_time, format="(I13)"
  readf, nc, n_col , format="(I13)"
  readf, nc, n_part ,format="(I13)"
  tableheader=""
  readf, nc, tableheader, format="(A)"
  print, tableheader
  paths=fltarr(n_col,n_time*n_part)
  readf, nc, paths, format="(1(F5.1,12(1X,E13.6)))"
  paths=reform(paths, n_col, n_time, n_part)
endif
;
; --- Close file ---
close, nc
;
; --- Return channel number ---
free_lun, nc
;
end ; rdpaths
