pro RDSPEC, listname, spec, freq, ntime=ntime
;
;+
; NAME:
;   RDSPEC
; PURPOSE:
;   Read in list of spectra
; INPUTS:
;   listname - 
; EXAMPLE:
;   RDSPEC, '/tmp_mnt/home/venus/supas048/spesyn/L69D07.SCN_*', spec, freq, $
;            ntime=39
;   tvscl, spec(*,*,0) 
; MODIFICATION HISTORY:
;   Written: 22.6.93 (B.F.)
;-
;
; --- Define input channel numbers ---
ncspec=10
ncfinali=11
;
if (not keyword_set(ntime)) then ntime=39
;
if (listname ne 'rdspecpar.txt') then begin
  ; --- Generate list of filenames from filename with wildcard ---
  fifi=findfile(listname)
  openw, ncfinali, 'rdspecpar.txt'
  lastindex,fifi,nlistlast
  for nlist=0,nlistlast-1 do printf, ncfinali, fifi(nlist)
  close, ncfinali
endif
;
; --- Open file with list of filenames which include the spectra ---
openr, ncfinali, 'rdspecpar.txt'
;
; --- Initialisation of some variables and counters ---
nfile=0  
nspec=0
infostring=strarr(4)
info=''
filename=''
nfreq=1
freqmin=0.0
freqmax=0.0
freqcen=0.0
delfreq=0.0
fclam=0.0
;
; --- Read in list of files with spectra ---
repeat begin
  ; --- Read name of next file, if no next file then filename='' ---
  if (EOF(ncfinali)) then begin
    filename=''
  endif else begin
    readf, ncfinali, filename
    filename=strtrim(strcompress(filename))
  endelse
  ;
  if (filename ne '') then begin
    nfile=nfile+1
    print, nfile, ': ',filename
    ;
    ; --- Read in data which represent the spectrum ---
    openr, ncspec, filename
    ;
    readf, ncspec, istop
    for i=0,3 do begin
      readf, ncspec, info
      infostring(i)=info
    endfor
    if (nspec eq 0) then print, infostring
    ;
    for i=0,istop-1 do begin
      readf, ncspec, nfreq, freqmin, freqmax, freqcen, delfreq, fclam
      factor=fclam/10000.0
      ;
      if ((nspec eq 0) and (i eq 0)) then begin
        spec=fltarr(nfreq,istop,ntime)
        tmpspec=fltarr(2,nfreq)
      endif
      readf, ncspec, tmpspec
      if ((nspec eq 0) and (i eq 0)) then freq=reform(tmpspec(0,*),nfreq)
      spec(*,i,nspec)=tmpspec(1,*)*factor
    endfor
    nspec=nspec+1
    ;
    close, ncspec
  endif
endrep until ((filename eq '') or (nspec ge ntime))
;
close, ncfinali                             ; Close file with list of filenames
;
end ; RDSPEC
