pro find_line_containing, string, unit
   text = ' '
   while (strpos(text, string) lt 0) do readf, unit, format='(A)', text
return
end

pro rdqf15, filename, idstring, z, logtau, yqz, y1qtau

   text     = ' '
   nz       = 0
   ntau     = 0
   nqy      = 0
   nqey     = 0
   idstring = ['U', 'V', 'P', 'T', 'R', 'D', 'S', 'RHO', 'FM', 'FH']

   get_lun, unit
   openr, unit, filename

   find_line_containing, '*ID   NZ', unit
   readf, unit, format='(A)', text
   readf, unit, nz

   find_line_containing, '*ID   NTAU', unit
   readf, unit, format='(A)', text
   readf, unit, ntau

   find_line_containing, '*ID   NQY', unit
   readf, unit, format='(A)', text
   readf, unit, nqy

   find_line_containing, '*ID   NQEY', unit
   readf, unit, format='(A)', text
   readf, unit, nqey

   find_line_containing, '*ID   PUNY', unit
   readf, unit, format='(A)', text
   puny = fltarr(nqy) 
   readf, unit, puny

   find_line_containing, '*ID   TBLZ', unit
   readf, unit, format='(A)', text
   z = fltarr(nz) 
   readf, unit, z

   find_line_containing, '*ID   TABTAU', unit
   readf, unit, format='(A)', text
   logtau = fltarr(ntau) 
   readf, unit, logtau

   find_line_containing, '*ID   Y1QZ', unit
   yqz  = fltarr(nz, nqy, 2) 
   dum  = fltarr(nz)
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqz( *, iq, 0) = dum * puny(iq)
   endfor

   find_line_containing, '*ID   Y2QZ', unit
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqz( *, iq, 1) = dum * puny(iq) * puny(iq)  
   endfor

;   print, 'Tau averages from first moment (extrapolated)!'
   y1qtau = fltarr(ntau, nqey, 3) 
   dum    = fltarr(ntau)
   find_line_containing, '*ID   Y1QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     y1qtau( *, iq, 0) = dum * puny(iq)
   endfor
   find_line_containing, '*ID   Y2QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     y1qtau( *, iq, 1) = dum * puny(iq)
   endfor
   find_line_containing, '*ID   Y3QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     y1qtau( *, iq, 2) = dum * puny(iq)
   endfor

close, unit

free_lun, unit
;idstring = strcompress(idstring,/remove_all)

return
end

pro rdsqf15, filename, data, m4=m4, help=help

   if keyword_set(help) then begin
     print, 'RDSQF15 reads qf15-data from file <filename> into (anonymous) structure <data>'
     print, 'M4: set keyword if actually 4th moment is stored in file'
     return
   endif

   if keyword_set(m4) then begin
     print, 'rdsqf15: 4th moments!'
     pow3=4
   endif else begin
     print, 'rdsqf15: 3rd moments!'
     pow3=3
   endelse

   text     = ' '
   tbase    = 0.0
   sbase    = 0.0
   itimq    = 0
   nz       = 0
   ntau     = 0
   nqy      = 0
   nqey     = 0
   nr       = 0
   itgrbo   = 0
   ipgrto   = 0
   idstring = ['U', 'V', 'P', 'T', 'R', 'D', 'S', 'RHO', 'FM', 'FH']

   get_lun, unit
   openr, unit, filename

   find_line_containing, '*ID   TBASE', unit
   readf, unit, format='(A)', text
   readf, unit, tbase

   find_line_containing, '*ID   SBASE', unit
   readf, unit, format='(A)', text
   readf, unit, sbase

   find_line_containing, '*ID   ITIMQ', unit
   readf, unit, format='(A)', text
   readf, unit, itimq

   find_line_containing, '*ID   NZ', unit
   readf, unit, format='(A)', text
   readf, unit, nz

   find_line_containing, '*ID   NTAU', unit
   readf, unit, format='(A)', text
   readf, unit, ntau

   find_line_containing, '*ID   NQY', unit
   readf, unit, format='(A)', text
   readf, unit, nqy

   find_line_containing, '*ID   NQEY', unit
   readf, unit, format='(A)', text
   readf, unit, nqey

   find_line_containing, '*ID   NR', unit
   readf, unit, format='(A)', text
   readf, unit, nr

   find_line_containing, '*ID   ITGRBO', unit
   readf, unit, format='(A)', text
   readf, unit, itgrbo

   find_line_containing, '*ID   IPGRTO', unit
   readf, unit, format='(A)', text
   readf, unit, itgrbo

   find_line_containing, '*ID   PUNY', unit
   readf, unit, format='(A)', text
   puny = dblarr(nqy) 
   readf, unit, puny

   find_line_containing, '*ID   TBLZ', unit
   readf, unit, format='(A)', text
   tblz = dblarr(nz) 
   readf, unit, tblz

   find_line_containing, '*ID   FRAZ', unit
   readf, unit, format='(A)', text
   fraz = dblarr(nz) 
   readf, unit, fraz

   find_line_containing, '*ID   TABTAU', unit
   readf, unit, format='(A)', text
   logtau = dblarr(ntau) 
   readf, unit, logtau

   find_line_containing, '*ID   FRATAU', unit
   readf, unit, format='(A)', text
   fratau = dblarr(ntau) 
   readf, unit, fratau

   find_line_containing, '*ID   FRATAE', unit
   readf, unit, format='(A)', text
   fratae = dblarr(ntau) 
   readf, unit, fratae

   find_line_containing, '*ID   Y1QZ', unit
   yqz  = dblarr(nz, nqy, 3) 
   dum  = dblarr(nz)
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqz( *, iq, 0) = dum * puny(iq)
   endfor

   find_line_containing, '*ID   Y2QZ', unit
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqz( *, iq, 1) = dum * puny(iq) * puny(iq)  
   endfor

   find_line_containing, '*ID   Y3QZ', unit
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqz( *, iq, 2) = dum * puny(iq)^pow3  
   endfor

   find_line_containing, '*ID   Y1QTAU', unit
   yqtau = dblarr(ntau, nqy, 3) 
   dum  = dblarr(ntau)
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtau( *, iq, 0) = dum * puny(iq)
   endfor

   find_line_containing, '*ID   Y2QTAU', unit
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtau( *, iq, 1) = dum * puny(iq) * puny(iq)  
   endfor

   find_line_containing, '*ID   Y3QTAU', unit
   for iq = 0, nqy-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtau( *, iq, 2) = dum * puny(iq)^pow3  
   endfor

   yqtae = dblarr(ntau, nqey, 3) 
   dum    = dblarr(ntau)
   find_line_containing, '*ID   Y1QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtae( *, iq, 0) = dum * puny(iq)
   endfor
   find_line_containing, '*ID   Y2QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtae( *, iq, 1) = dum * puny(iq)
   endfor
   find_line_containing, '*ID   Y3QTAE', unit
   for iq = 0, nqey-1 do begin
     readf, unit, format='(A)', text
     readf, unit, dum
     yqtae( *, iq, 2) = dum * puny(iq)^pow3
   endfor

close, unit

free_lun, unit
;idstring = strcompress(idstring,/remove_all)

u_z   = { m1: yqz(*,0,0), m2: yqz(*,0,1), m3: yqz(*,0,2)}  
v_z   = { m1: yqz(*,1,0), m2: yqz(*,1,1), m3: yqz(*,1,2)}  
p_z   = { m1: yqz(*,2,0), m2: yqz(*,2,1), m3: yqz(*,2,2)}  
t_z   = { m1: yqz(*,3,0), m2: yqz(*,3,1), m3: yqz(*,3,2)}  
r_z   = { m1: yqz(*,4,0), m2: yqz(*,4,1), m3: yqz(*,4,2)}  
d_z   = { m1: yqz(*,5,0), m2: yqz(*,5,1), m3: yqz(*,5,2)}  
s_z   = { m1: yqz(*,6,0), m2: yqz(*,6,1), m3: yqz(*,6,2)}  
rho_z = { m1: yqz(*,7,0), m2: yqz(*,7,1), m3: yqz(*,7,2)}  
fm_z  = { m1: yqz(*,8,0), m2: yqz(*,8,1), m3: yqz(*,8,2)}  
fh_z  = { m1: yqz(*,9,0), m2: yqz(*,9,1), m3: yqz(*,9,2)}  

u_tau   = { m1: yqtau(*,0,0), m2: yqtau(*,0,1), m3: yqtau(*,0,2)}  
v_tau   = { m1: yqtau(*,1,0), m2: yqtau(*,1,1), m3: yqtau(*,1,2)}  
p_tau   = { m1: yqtau(*,2,0), m2: yqtau(*,2,1), m3: yqtau(*,2,2)}  
t_tau   = { m1: yqtau(*,3,0), m2: yqtau(*,3,1), m3: yqtau(*,3,2)}  
r_tau   = { m1: yqtau(*,4,0), m2: yqtau(*,4,1), m3: yqtau(*,4,2)}  
d_tau   = { m1: yqtau(*,5,0), m2: yqtau(*,5,1), m3: yqtau(*,5,2)}  
s_tau   = { m1: yqtau(*,6,0), m2: yqtau(*,6,1), m3: yqtau(*,6,2)}  
rho_tau = { m1: yqtau(*,7,0), m2: yqtau(*,7,1), m3: yqtau(*,7,2)}  
fm_tau  = { m1: yqtau(*,8,0), m2: yqtau(*,8,1), m3: yqtau(*,8,2)}  
fh_tau  = { m1: yqtau(*,9,0), m2: yqtau(*,9,1), m3: yqtau(*,9,2)}  

u_tae   = { m1: yqtae(*,0,0), m2: yqtae(*,0,1), m3: yqtae(*,0,2)}  
v_tae   = { m1: yqtae(*,1,0), m2: yqtae(*,1,1), m3: yqtae(*,1,2)}  
p_tae   = { m1: yqtae(*,2,0), m2: yqtae(*,2,1), m3: yqtae(*,2,2)}  
t_tae   = { m1: yqtae(*,3,0), m2: yqtae(*,3,1), m3: yqtae(*,3,2)}  

qz = { u: u_z, $
       v: v_z, $ 
       p: p_z, $ 
       t: t_z, $ 
       r: r_z, $ 
       d: d_z, $ 
       s: s_z, $ 
       rho: rho_z, $ 
       fm: fm_z, $ 
       fh: fh_z}

qtau ={u: u_tau, $
       v: v_tau, $ 
       p: p_tau, $ 
       t: t_tau, $ 
       r: r_tau, $ 
       d: d_tau, $ 
       s: s_tau, $ 
       rho: rho_tau, $ 
       fm: fm_tau, $ 
       fh: fh_tau}

qtae ={u: u_tae, $
       v: v_tae, $ 
       p: p_tae, $ 
       t: t_tae} 

data = {file:   filename, $ 
        tbase:  tbase, $ 
        sbase:  sbase, $
        itimq:  itimq, $
        nz:     nz, $
        ntau:   ntau, $
        nqy:    nqy, $
        nqey:   nqey, $
        nr:     nr, $
        itgrbo: itgrbo, $
        ipgrto: ipgrto, $
        puny:   puny, $
        z:      tblz, $
        fraz:   fraz, $
        logtau: logtau, $
        fratau: fratau, $
        fratae: fratae, $
        yqz:    yqz, $ 
        yqtau:  yqtau, $
        yqtae:  yqtae, $
        qz:     qz, $
        qtau:   qtau, $
        qtae:   qtae}

return
end
