pro RDTAB, filename, r, istop, jstop, infoname=infoname, rinfo=rinfo
;
;+
; NAME:
;   RDTAB
; PURPOSE:
;   Read in a data set in dat/txt-format.
; CALLING SEQUENCE:
;   RDTAB, filename, r [, istop, jstop] [,infoname=infoname, rinfo=rinfo]
; INPUTS:
;   filename - Name of input data file
; OUTPUTS:
;   r - Output array
;   istop -
;   jstop -
; KEYWORD PARAMETERS:
;   infoname - Name of input info file
;   rinfo - String array containing infos about r
; EXAMPLE:
;   To read in the data in 'greyat.txt':
;     RDTAB, 'greyat.txt', atmarray
; MODIFICATION HISTORY:
;   7. 9.93 Written by B.F.
;  12. 9.93 (B.F.) Keywords infoname and rinfo added
;   1.11.93 (B.F.) Binary files with suffix '.lon' can be read.
;  21. 7.03 (B.F.) swap_if_little_endian

;-
;
print, 'RDTAB: ', filename
;
; --- Load data file ---
get_lun, nc
istop=0L
jstop=0L
if (strpos(filename,'.dat') ne -1) then begin
  openr, nc, filename, /f77_unformatted, /swap_if_little_endian
  readu, nc, istop, jstop
  r=fltarr(jstop,istop)
  point_lun, nc, 0
  readu, nc, istop, jstop, r
  r=transpose(reform(r,istop,jstop))
endif else if (strpos(filename,'.lon') ne -1) then begin
  idum=0L
  jdum=0L
  openr, nc, filename, /f77_unformatted, /swap_if_little_endian
  readu, nc, istop, idum, jstop, jdum
;  readu, nc, istop, jstop
  r=dblarr(jstop,istop)
  point_lun, nc, 0
  readu, nc, istop, idum, jstop, jdum, r
;  readu, nc, istop, jstop, r
  r=transpose(reform(r,istop,jstop))
endif else begin
  openr, nc, filename
  readf, nc, istop, jstop
  r=fltarr(jstop,istop)
  readf, nc, r
endelse
close, nc

; --- Load INFO file ---
if keyword_set(infoname) then begin
  rinfohelp=strarr(2*istop)
  openr, nc, infoname
  readf, nc, rinfohelp
  close, nc
  rinfo=strarr(istop)
  for i=0,istop-1 do rinfo(i)=rinfohelp(2*i+1)
endif
;
free_lun, nc
;
end ; RDTAB
