pro wrfftcoeff, filename, FFTcoeff, n_FFTcoeff, time, time_zero, meanradius
;
;------------------------------------------------------------------------------
;+
; NAME:
;   wrfftcoeff ('write_FFT_coefficients')
;
; PURPOSE:
;   Write Fourier coefficients into file.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   wrfftcoeff
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;   Opens file and writes into it.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2005-02-22 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   
;-
;------------------------------------------------------------------------------

factor_normalize=2.0/float(n_elements(FFTcoeff))

; --- Write to text file ---
get_lun, nc
openw, nc, filename

printf, nc, "# Table with Fourier coefficients " + systime(0)
printf, nc, "# Quantity: relative 'surface' radius position"
printf, nc, "# R=sum_i  A_i cos((2*pi/(Total_time)) * i * time - delta_i)"
printf, nc, n_FFTcoeff, "Number of coefficients,", format='(I13,4X,A)'
printf, nc, 3,       "Number of columns", format='(I13,4X,A)'
printf, nc, time*!con.y,    "Total time [s]", format='(E13.6,4X,A)'
printf, nc, meanradius*!con.Rsun, "Mean radius [cm]", format='(E13.6,4X,A)'
printf, nc, "# frequency     amplitude phase_shift"
printf, nc, "#         i           A_i     delta_i"
for i=1,n_FFTcoeff do begin &$
  printf, nc, float(i), $
              factor_normalize*abs(FFTcoeff(i)), $
              atan(FFTcoeff(i),/phase) - time_zero*2.0*!pi/time*float(i), $
              format='(I11,1X,F13.10,1X,F11.7)' &$
endfor

close, nc
free_lun, nc

end ; wrfftcoeff
