pro wrisosal, disk, metid, alphalist, agelist, $
    Teff=Teffarr, Lum=Lumarr, Mass=Massarr, alpha=alphaarr, suffix=suffix
;------------------------------------------------------------------------------
;+
; NAME:
;   wrisosal ('write_isochrone_salaris')
;
; PURPOSE:
;   Write evolutionary tracks in Maurizio Salaris format
;
; CATEGORY:
;   Stellar Evolution, I/O
;
; CALLING SEQUENCE:
;   wrisosal, disk, metid, alphalist, agelist, $
;     Teff=Teffarr, Lum=Lumarr, Mass=Massarr, $
;     suffix=suffix
;
; INPUTS:
;   disk:     ('disk') directory where files are located
;   metid:    ('metallicity_identifier') string to identify isochrone file set
;             with proper metallicity:
;               met='mm10': metid='183'
;               met='mm20': metid='184'
;   alphalist:('alpha_list') real 1D vector with alpha values,
;             e.g. alphalist=[1.2, 1.5, 1.8, 2.1]
;   agelist:  ('age_list') real 1D vector with age values [Gyr],
;             e.g. agelist  =[9.0, 10.0, 11.0, 12.0, 13.0, 14.0]
;   Teff:     ('temperature_effective')
;   Lum:      ('luminosity')
;   Mass:     ('mass')
;   suffix:   ('suffix') string, suffix for file names (default: 'norma')
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   22. 5.98 Written by Bernd Freytag, Astrophysik Kiel
;   27. 8.98 (B.F. Copenhagen) rdisosal -> wrisosal
;-
;------------------------------------------------------------------------------
;
nalphas=n_elements(alphalist)
nages  =n_elements(agelist  )
nmasses=n_elements(Massarr(*,0,0))
;
if (not keyword_set(metid)) then metid=''
if (not keyword_set(suffix)) then suffix='norma'
;
for i=0,nalphas-1 do begin
  alpha_str=strtrim(string(alphalist(i)*10.0, format='(I2)'),2)
  if (strlen(alpha_str) eq 1) then alpha_str='0' + alpha_str
  for j=0,nages-1 do begin
    age_str=strtrim(string(agelist(j), format='(I2)'),2)
    if (strlen(age_str) eq 1) then age_str='0' + age_str
    file='I' + age_str + metid + 'a' + alpha_str + suffix
    print, 'Write file ''' + file + ''''
    get_lun, nc
    openw, nc, disk+file
    if (keyword_set(alphaarr)) then begin
      for n=0,nmasses-1 do begin
        printf, nc, $
          alog10(Lumarr(n,j,i)), alog10(Teffarr(n,j,i)), Massarr(n,j,i), $
          alphaarr(n,j,i), $
          format='(4X,F7.4,1X,F10.7,1X,F8.6,1X,F9.5)'
      endfor ; n
    endif else begin
      for n=0,nmasses-1 do begin
        printf, nc, $
          alog10(Lumarr(n,j,i)), alog10(Teffarr(n,j,i)), Massarr(n,j,i), $
          format='(4X,F7.4,1X,F10.7,1X,F8.6)'
      endfor ; n
    endelse
    close, nc
    free_lun, nc
  endfor ; j
endfor ; i
;
end ; wrisosal
