; pro lawe_plot
;------------------------------------------------------------------------------
;+
; NAME:
;   lawe_plot ('lawe_plot')
;
; PURPOSE:
;   Plot lawe data
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @lawe_plot
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2006-06-28 Written by Bernd Freytag, L.A.
;   2006-07-18 (B.F.) Legend with time-scales
;-
;------------------------------------------------------------------------------

; === Manual selection ===========
if (not keyword_set(external_control)) then begin &$
  ; --- Choose device: 'x', 'ps' ---
  dev='ps' &$
  jpg_flag=0 &$
  png_flag=0 &$
  ;
  ; --- Model ---
  ;model='lc0gf' &$
  ;model='lc0hg' &$
  model='lc0gg' &$
  ;model='lf0gg' &$
  ;
  ;suffix='_mode_cm03_nz-1000_gridopt-opt2_var-vx,vz.sav' &$
  suffix='_mode_cm04_nz-1000_gridopt-opt2_var-vx,vz.sav' &$
  ;
  ; --- Quantity to analyse ---
  ;qid='vz' & qtitle='v!Dvert!N' & qrange=0 &$
  ;qid='log10_vz' & qtitle='log(v!Dvert!N)' & qrange=[-65.0,1.0] &$
  qid='log10b_vz' & qtitle='log(v!Dvert!N)' & qrange=[-6.0,0.0] &$
  ;qid='log10_omegabv' & qtitle='log(abs(!4x!X!DBV!N/s!U-1!N))' & qrange=0 &$
  ;
  xsize=5.7 & ysize= 5.0 & wfactor=1200.0 &$
  ;xsize=8.2 & ysize= 6.5 & wfactor=1200.0 &$
  ;xsize=17.4 & ysize= 9.0 & wfactor=600.0 &$
  ;
  colorliststr='colorlist=[!P.COLOR]' &$
  ;colorliststr='colorlist=[!P.COLOR, !ctv.green, !ctv.red, !ctv.blue, !ctv.magenta, !ctv.gray, !ctv.orange, !P.COLOR]' &$
  ;linestylelist=[0,5,2] &$  
  ;linestylelist=[0,2,1] &$  
  linestylelist=[0] &$  
  thicklist=[1] &$
  ;thicklist=[1,4] &$
  ;
  yannotype='yes' &$
endif
; ================================

; --- Fixed settings ---
x_fac=1.0E-08 & x_title='y [Mm]' & x_unit='Mm'

; --- File names for plot output ---
file_id='lawe_' + model + '_' + qid
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Choose color table ---
normct

; --- Set margins ---
if (yannotype eq 'no') then begin &$
  !X.MARGIN=[1,0] &$
  xsize=xsize-1.0 &$
endif else begin &$
  !X.MARGIN=[6,1] &$
endelse
!Y.MARGIN=[3,1]

if (strupcase(dev) eq 'X') then begin & $
  set_plot, 'X' & $
  !P.CHARSIZE=1.6 &$
  !P.FONT=-1 &$
  win_id=0 &$
  window, win_id, xsize=xsize/10.0*wfactor, ysize=ysize/10.0*wfactor, retain=2 &$
endif else begin & $
  set_plot, 'ps' & $
  !P.CHARSIZE=0.65 &$
  !P.FONT=0 &$
  device, filename=ps_file, $
          encapsulated=(strupcase(dev) eq 'EPS'), $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
endelse

; --- List of colors ---
dummy=execute(colorliststr)

; --- Set global plot parameters ---
;!P.COLOR=!ctv.black & !P.BACKGROUND=!ctv.white
;!P.COLOR=colorlist[0] & !P.BACKGROUND=colorlist[1]
;!X.TICKLEN=-0.02
;!Y.TICKLEN=-0.02

; === Load data ===
savefile=model + suffix
print, 'Restore file ' + savefile
restore, savefile

y_title=qtitle
x=reform(mo.z)*x_fac

ind=where(mo.mode.nnode eq 0)
ind=ind[0:7]

; === Plot ===

if (qid eq 'vz') then begin &$
  q=mo.mode.vz[*,ind] &$
  for i=0,n_elements(ind)-1 do q[*,i]=q[*,i]/max(abs(q[*,i]),/NAN) &$
  plotlines, x, q, drange=qrange, $
             thick=thicklist, linecol=colorlist, linestyle=linestylelist, $
             xtitle=x_title, $
             ytitle=transfont(y_title), $
             dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif else if ((qid eq 'log10_vz') or (qid eq 'log10b_vz')) then begin &$
  q=mo.mode.vz[*,ind] &$
  for i=0,n_elements(ind)-1 do q[*,i]=q[*,i]/max(abs(q[*,i]),/NAN) &$
  q=alog10(q) &$
  plotlines, x, q, drange=qrange, $
             thick=thicklist, linecol=colorlist, linestyle=linestylelist, $
             xtitle=x_title, $
             ytitle=transfont(y_title), $
             dev=dev, psformat='nochange', /psnoinit, /noclose &$
  legend, string(1.0/(2.0*!pi*mo.mode.frequency[ind]), format='(I5)') + ' s', $
          thick=thicklist, color=colorlist, linestyle=linestylelist, $
          xali=0.5, yali=0.0, charsize=!P.CHARSIZE*0.85, box=0, margin=0.1, spacing=0.5, pspacing=1 &$
endif else if (qid eq 'log10_omegabv') then begin &$
  q=mo.omegaBVsqrograv*mo.grav &$
  q=alog10([[sqrt(abs(q))], [sqrt(-q)]]) &$
  plotlines, x, q, drange=qrange, $
             thick=thicklist, linecol=colorlist, linestyle=linestylelist, $
             xtitle=x_title, $
             ytitle=transfont(y_title), $
             dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

; --- Legend ---
;legend, ['total', 'convection', 'g-modes', 's-modes', 'p-modes'], $
;        color=[colorlist[0], colorlist[1], colorlist[3], colorlist[6], colorlist[2]], $
;        linestyle=0, thick=[3,1,1,1,1], $
;        xali=1.0, yali=1.0, charsize=!P.CHARSIZE*0.85, box=0, margin=0.1, spacing=0.5, pspacing=1

if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    ; --- Write PNG file ---
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    ; --- Write JPEG file ---
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak
if (yannotype eq 'no') then begin &$
  xsize=xsize+1.0 &$
endif

; end ; lawe_plot
