pro opac_cluster, $
  MetaRaw=MetaRaw0, Tt=Tt, Pt=Pt, rhot=rhot, nu=nu, dnu=dnu0, kappanu=kappanu, $
  MetaAtm=MetaAtm0, x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
  ParOpac=Par, $
    OutputTemplate=OutputTemplate0, OutputMarker=OutputMarker0, Author=Author0, $
    kCluster=kCluster0, $
    ModeInit=ModeInit0, $
      InputIndCluster=InputIndCluster, ParInit=ParInit, $
    ModeMean=ModeMean0, ModexLength=ModexLength0, $
      xLengthPar=xLengthPar0, CDeltaT=CDeltaT0, $
    nTop=nTop0, ExprhoWeight=ExprhoWeight0, $
    ModeSmall=ModeSmall0, $
      nnuPointInitial=nnuPointInitial0, nnuStepInitial=nnuStepInitial0, $
      CMaskSmall=CMaskSmall0, rIterFactor=rIterFactor0, nIterControl=nIterControl0, $
      mChunk=mChunk0, nChunk=nChunk0, $
      ExitClusterChangeNumber=ExitClusterChangeNumber0, $
      ExitClusterChangeFraction=ExitClusterChangeFraction0, $
      ExitSysTime=ExitSysTime0, ExitRelError=ExitRelError0, $
    PlotSelect=PlotSelect, $
  IndCluster=IndCluster, Result=Result, Opa=Opa
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_cluster ('opacity_cluster')
;
; PURPOSE:
;   Collect frequency points into clusters and compute cluster mean opacities along 1D stratification.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   opac_cluster, $
;     MetaRaw=MetaRaw, Tt=Tt, Pt=Pt, rhot=rhot, nu=nu, dnu=dnu, kappanu=kappanu, $
;     MetaAtm=MetaAtm, x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
;     Par=Par, $
;       OutputTemplate=OutputTemplate, OutputMarker=OutputMarker, Author=Author, $
;       kCluster=kCluster, $
;       ModeInit=ModeInit, $
;         InputIndCluster=InputIndCluster, $
;        ModeMean=ModeMean, ModexLength=ModexLength, $
;         xLengthPar=xLengthPar, CDeltaT=CDeltaT, $
;       nTop=nTop, ExprhoWeight=ExprhoWeight, $
;       ModeSmall=ModeSmall, $
;         nnuPointInitial=nnuPointInitial, nnuStepInitial=nnuStepInitial, $
;         CMaskSmall=CMaskSmall, rIterFactor=rIterFactor, nIterControl=nIterControl, $
;         mChunk=mChunk, nChunk=nChunk, $
;         ExitClusterChangeNumber=ExitClusterChangeNumber, $
;         ExitClusterChangeFraction=ExitClusterChangeFraction, $
;         ExitSysTime=ExitSysTime, ExitRelError=ExitRelError, $
;       PlotSelect=PlotSelect, $
;     IndCluster=IndCluster, Result=Result, Opa=Opa
;
; INPUT:
;   None (only via keywords)
;
; OUTPUT:
;   None (only via keywords)
;
; KEYWORDS (INPUT):
;   --- Raw opacities ---
;   MetaRaw:      ('Meta_Raw') structure with meta data of raw opacities
;                 This structure is put into the output structure.
;                 Part of the information (all tags except 'Description') is used for the header
;                 of the opacity file. E.g.:
;                   MetaRaw= { $
;                     Description: 'Meta data of raw opacities', $
;                     FileOriginal:      OpaRawFiles + ' + ' + OpSRawFiles, $
;                     FileSource:        OpTSavFile, $
;                     Code:              'COMA', $
;                     CodeVersion:       'COMA 11a, B.Aringer 2012', $
;                     Author:            'Bernhard Aringer', $
;                     AbundanceID:       'solar', $
;                     ElementSymbols:    strupcase(EleSym), $
;                     ElementAbundances: EleAbu, $
;                     abuX:              abuX, $
;                     abuY:              abuY, $
;                     abuZ:              abuZ, $
;                     X:                 X, $
;                     Y:                 Y, $
;                     Z:                 Z, $
;                     Qmol:              Qmol, $
;                     Vmicro:            2.5 $
;                            }
;                 E.g.:
;                 help, /str, Res.MetaRaw
;                 ** Structure <Anonymous>, 19 tags, data length=668:
;                    DESCRIPTION     STRING    'Meta data of raw opacities'
;                    FILEORIGINAL    STRING    'kapnu_solabu_h2obt2_p[123] + kapsc_solabu_p[1'...
;                    FILESOURCE      STRING    'coma_kapto_solabu_h2obt2.sav'
;                    FILEPATH        STRING    '/Users/befre677/dat/opa/BernhardAringer/solab'...
;                    CODE            STRING    'COMA'
;                    CODEVERSION     STRING    'COMA 11a, B.Aringer 2012'
;                    AUTHOR          STRING    'Bernhard Aringer'
;                    ABUNDANCESORCE  STRING    'solar (Paola)'
;                    ABUNDANCEID     STRING    'Paola'
;                    ELEMENTSYMBOLS  STRING    Array[75]
;                    ELEMENTABUNDANCES FLOAT   Array[75]
;                    ABUX            FLOAT           0.90988
;                    ABUY            FLOAT           0.08892
;                    ABUZ            FLOAT           0.00121
;                    X               FLOAT           0.70877
;                    Y               FLOAT           0.27503
;                    Z               FLOAT           0.01620
;                    QMOL            FLOAT           1.29401
;                    VMICRO          FLOAT           2.50000
;   Tt:           ('Temperature_table') 1D real vector (nT elements) with temperature axis
;                 of opacity data block
;   Pt:           ('Pressure_table') 1D real vector (nP elements) with pressure axis
;                 of opacity data block
;   rhot:         ('rho_table') 2D real array (nT x nP elements) with density values
;                 corresponding to T/P axis values
;   nu:           ('nu') 1D real vector (nnu elements) with frequency axis of opacity data block
;   dnu:          ('delta_nu') optional 1D real vector (nnu elements) with frequency intervals
;                 of opacity data block. The default is to compute this vector from nu.
;   kappanu:      ('kappa_nu') 3D real array (nT x nP x nnu) with frequency-dependent opacities (large)
;                 of the form kappa(T, P, nu) as function of temperature, pressure, and frequency
;   --- 1D reference atmosphere ---
;   MetaAtm:      ('Meta_Atmosphere') structure with meta data of 1D reference atmosphere.
;                 This structure is put into the output result structure.
;                 Part of the information (all tags except 'Description') is used for the header
;                 of the opacity file. E.g.:
;                   MetaAtm= { $
;                     Description: 'Meta data of 1D reference atmosphere', $
;                     Model:             Model, $
;                     FileSource:        AtmFile, $
;                     Code:              'CO5BOLD', $
;                     Teff:              (udb_extract('Teff'   , [Model], basDB, Default=0.0))[0], $
;                     Teffout:           (udb_extract('Teffout', [Model], basDB, Default=0.0))[0], $
;                     grav:              (udb_extract('grav'   , [Model], basDB, Default=0.0))[0], $
;                     Author:            (udb_extract('Author' , [Model], modDB, Default='' ))[0] $
;                            }
;                 E.g. (with lots of additional non-mandatory data):
;                 GDL> help, /str, Res.MetaAtm
;                 ** Structure <Anonymous>, 21 tags, data length=84:
;                    DESCRIPTION     STRING    'Meta data of 1D reference atmosphere'
;                    MODEL           STRING    'st28gm06n25'
;                    FILEDATA        STRING    'st28gm06n25_ring_a08-11.sav'
;                    FILESOURCE      STRING    'st28gm06n25_atm_ring_02.sav'
;                    CODE            STRING    'CO5BOLD'
;                    TEFF            FLOAT           2800.00
;                    TEFFOUT         FLOAT           2727.05
;                    TEFFTARGET      FLOAT           2800.00
;                    GRAV            FLOAT           0.19679
;                    AUTHOR          STRING    'Bernd Freytag'
;                    MX1             INT             20
;                    NX1             INT             -5
;                    MR1             INT              0
;                    NR1             INT              0
;                    MF1             INT             20
;                    NF1             INT             -1
;                    NPLUS           INT             35
;                    MDELTA          INT             15
;                    NDELTA          INT              5
;                    NPERTAU10       INT             15
;                    SCALEFACTOR     FLOAT           0.52494
;   x1D:          ('x_1_Dimension') 1D real vector with x coordinates of points in stratification
;   T1D:          ('Temperature_1_Dimension') 1D real vector with temperature values of stratification
;   P1D:          ('Pressure_1_Dimension') 1D real vector with pressure values of stratification
;   rho1D:        ('rho_1_Dimension') 1D real vector with density values of stratification
;   --- Opacity-clustering control: everything ---
;   Par:          ('Parameter') structure with all the control parameters below.
;                 An explicitely (separately) specified values takes preference over the value
;                 in the Par structure which takes preference over the build-in default value.
;   --- Opacity-clustering control: meta data ---
;   OutputTemplate: ('Output_Template') string, optionally with placeholders, to construct OutputID
;                 Default: 't(teff)g(logground)mm00_(rawcode)_oc_(modemean)_b(kcluster)_(datetime)'
;   Author:       ('Author') string with name of person running the opacity-clustering code
;   --- Opacity-clustering control: number of clusters (or bands) ---
;   kCluster:     ('k_Cluster') integer with number of clusters to sort the frequency points into
;                   1      : Just compute a grey table without sorting into clusters (default)
;                   2      : Minimum number of clusters to activate the sorting
;                   3..5   : Compute tables with a small sensible number of clusters
;                   >20..30: Use a large number of clusters to get very good results on the cost
;                            of very long iteration times
;   --- Opacity-clustering control: cluster initialization ---
;   ModeInit:     ('Mode_Initialization') string passed to opac_cluster_init.pro with initial
;                 clustering pattern:
;                   'FrequencySplitEvenBlock' : Split the frequency interval into blocks with
;                                               even number of points
;                   'FrequencySplitRoundRobin': Split the frequency interval in a
;                                               round-robin fashion
;                   'dtauSplitEvenBlock'      : Sort the clusters according to their
;                                               reference tau value
;   InputIndCluster: ('Input_Index_Cluster') integer vector (nnu elements) with the
;                 initial cluster membership of each frequency point.
;                 This array is used for ModeInit='Input'.
;                 Often, this vector is from the output of a previous run of opac_cluster.pro.
;                 Value:
;                   -3           : Keep point on hold until the final 'Dry' run if ModeSmall ne 'No'
;                   -2           : Point is on hold but might be activated in any 'Big' step
;                   -1           : Point is waiting to be sorted into cluster in the next 'Big' step
;                   0..kCluster-1: Cluster, the point is member of
;   --- Opacity-clustering control: which opacity mean to use ---
;   ModeMean:     ('Mode_Mean') string with type of opacity mean.
;                 The value is used in opac_com_mean.pro where the complete list can be found.
;                 Only 'proper' local means can be used to produce the final opacity table 'opa'.
;                 'Improper' detailed-intensity-dependent means can only be used for the sorting.
;                 Examples:
;                   --- 'Proper' local means ---
;                   'P'   :  Planck mean
;                   'Psf' :  small-fluctuation Planck mean
;                   'R'   :  Rosseland mean
;                   'RP'  :  Blend of Rosseland and Planck mean (default) (xLength, CDeltaT)
;                   'RPsf':  Blend of Rosseland and small-fluctuation Planck mean (xLength)
;                   'BmB' :  Sort-of non-local Rosseland mean with weight B(far)-B(at_point)
;                            (xLength, CDeltaT)
;                   'Mix1':  Mix of RP and BmB mean (xLength, CDeltaT)
;                   'Mix2':  Mix of RP and BmB mean (xLength, CDeltaT)
;                   --- 'Improper' means using the intensity in the reference atmosphere ---
;                   'IumId': Flux-like (Chandrasekhar-type) mean with weight I_up-I_down
;                   'IupId': J-like mean with weight I_up+I_down
;                   'J'    : J-like mean with weight from discrete angles I_up+I_down+4B
;                   'IumB' : Upward-intensity-change mean with weight I_up-B
;                   'BmId' : Downward-intensity-change mean with weight B-I_down
;   ModexLength:  ('Mode_x_Length') string with unit of xLength parameter:
;                   'Depth': Total model depth (xLengthPar around 0.2)
;                   'Hp'   : Pressure scale height (xLengthPar around 3.0)
;                   'cm'   : Direct length specification (xLength gives length scale directly in cm)
;   xLengthPar:   ('x_Length_Parameter') scalar real with factor for typical length scale.
;                 The interpretation depends on keyword ModexLength.
;   CDeltaT:      ('Constant_Delta_Temperature') scalar real with temperature fraction.
;                   0.2: Default
;   --- Opacity-clustering control: which part of the atmosphere to optimize with which weight ---
;   nTop:         ('number_Top') integer with last element of input vectors to consider.
;                   -4: Default (ignore just a few points at the top)
;   ExprhoWeight: ('Exponent_rho_Weight') real scalar with density exponent to use in
;                 weight function.
;                   0.0  ... 1.0 : Reasonable values
;                   1.0          : Optimize error of heating per gram
;                   0.25 ... 0.75: Typical values
;                   0.0          : Default value (but values around 0.5 or above are more sensible).
;                                  Optimize error of heating per volume
;   --- Opacity-clustering control: iteration control and exit conditions ---
;   ModeSmall     ('Mode_Small') string with mode of treatment of small-contribution frequency points
;                 during final 'Dry'-run iteration step:
;                   'No'            : No special treatment of small-contribution points, i.e.,
;                                     don't split them off and add them later (default)
;                   'LastCluster'   : Put small-contribution points at end into last cluster
;                   'MaxtauCluster' : Put small-contribution points at end into cluster
;                                     with max tau_Ref
;                   'MintauCluster' : Put small-contribution points at end into cluster
;                                     with min tau_Ref
;                   'SorttauCluster': Sort small-contribution points at end into clusters
;                                     according to tau_Ref (a sensible choice)
;   nnuPointInitial: ('number_nu_Point_Initial') integer with approximate initial number
;                 of frequency points per cluster.
;                 nnuStepInitial is chosen as power of 2, so that the initial number of
;                 frequency points per cluster is close to nnuPointInitial. Choices:
;                   40..300: Reasonable values
;                   100    : Default
;   nnuStepInitial: ('number_nu_Step_Initial') integer with initial frequency-axis coarsening factor,
;                 typically a power of two.
;                 Based on this value as a starting point, the number of frequency points will
;                 be refined by a factor of two in every 'Big' iteration step, controlled with
;                 nIterControl.
;                   0 : Compute nnuStepInitial from nnuPointInitial (recommended)
;                   >0: Overwrite value computed from nnuPointInitial
;                   8 : Default
;   CMaskSmall:   ('Constant_Mask_Small') real with fraction of contribution of frequency point
;                 below which it is considered small and treated as requested by parameter ModeSmall.
;                   1.0D-14: Default
;   rIterFactor;  ('real_Iteration_Factor') real scalar aplied to the default array nIterControl
;                 computed from the number of clusters kCluster
;   nIterControl: ('number_Iteration_Control') integer vector with sequence of maximum number of
;                 iteration 'Sub' steps between 'Big' steps in which frequency-axis refinement
;                 takes place:
;                   [20,15,10,5]: Example: use 20 iterations for the first group, 15 for the next,
;                                 10 for all of the following except the last which uses 5
;                   [-1]        : No iteration: useful to keep the initial cluster-memberships
;                   [0]         : Use the default
;   mChunk:       ('m_Chunk') integer with initial search-stencil size (default: 24).
;                The search stencil is the chunk of points that is processed in one sub step.
;                The frequency point with the largest (negative) contribution to the error change
;                is moved from its source cluster to the optimal target cluster.
;                Smaller stencils are more efficient. However, larger stencils improve the quality
;                of the point that is actually moved.
;   nChunk:       ('number_Chunk') integer with final search-stencil size (default: 48)
;   ExitClusterChangeNumber: ('Exit_Cluster_Change_Number') integer with number of
;                 cluster-membership changes per full iteration step below which to stop.
;                   0  : Ensure convergence (default).
;                   > 0: Exit already if the number of changes becomes small (but non-zero)
;                        to improve performany on the cost of accuracy.
;   ExitClusterChangeFraction: ('Exit_Cluster_Change_Fraction') real with fraction of
;   ExitSysTime:  ('Exit_System_Time') real with number of seconds after which to stop.
;                   Default: very large.
;   ExitRelError: ('Exit_Relative_Error') real with relative change in relative error between
;                 full iterations steps below which to stop.
;                 A small value gives the best results but the longest run times.
;                   0.0    : Iterate to full convergence (or another criterion is met)
;                   1.0E-04: Typical value (default)
;   --- Plot control ---
;   PlotSelect:   ('Plot_Select') integer 2D array passed to opac_cluster_multiplot.pro
;                 to select the frames to plot during and at the end of the optimization.
;
; KEYWORDS (OUTPUT):
;   IndCluster:   ('Index_Cluster') 1D integer vector (nnu) with the cluster
;                 each frequency points belongs to
;   Result:       ('Result') structure with collected results
;   Opa:          ('Opacity') structure with opacity data in OPTA format
;
; LOCAL VARIABLES:
;   nnuCluster:   ('number_nu_Cluster') 1D integer vector (kCluster) with the number of
;                 frequency points in each cluster
;   IndnuCluster: ('Index_nu_Cluster) 2D integer array (nnu, kCluster) with the
;                 frequency points in each cluster
;   kappaCluster: ('kappa_Cluster') double-precision 2D array (nx1D, kCluster) with
;                 1D opacity profile for each cluster
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   opac_com_Iintegrate:   Integrate upward and downward intensities along 1D ray
;   opac_com_Mean:         Compute the weights necessary to get opacity means
;   opac_com_StratInterpolate: Interpolate opacity and source function onto 1D stratification
;   opac_com_tau:          Integrate the optical-depth scale of 1D stratification
;   opac_cluster_CreateOpTa: Assemble the standard opacity-table structure
;   opac_cluster_Init:     Initialize the cluster membership
;   opac_cluster_Mean:      Compute clustered/binned opacity means.
;   opac_cluster_MultiPlot: Plot results produced by opac_cluster
;   plotlines:             Make multi-line plots
;   strreplacebf:          Function to replace a substring by another
;
; GLOBAL VARIALBES (used):
;   !ctv:         Color-table variable (for plotting with plotlines.pro in opac_cluster_multiplot.pro)
;   (!con:         Some constants)
;
; SIDE EFFECTS:
;   Global variables controlling plots are changed by opac_cluster_MultiPlot.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Input:
;     1) Raw opacities kappa(T, P, nu) together with corresponding meta data
;     2) A reference atmosphere (1D) with corresponding meta data
;     3) Control parameters, e.g.:
;          kCluster: number of clusters to search for
;          ModeMean: the opacity mean to use
;          xLengthPar: to construct a typical length scale
;          parameter(s) for the weight function of the intensity-change error
;          hints for the initial configuration of the clusters
;          iteration-control
;   Output:
;     1) An integer array IndCluster(nnu) with the cluster each frequency points belongs to
;     2) A 'result' structure with lots of log and meta data:
;        a) The cluster-membership integer array IndCluster(nnu)
;        b) Mean opacities (of the chosen type) within each cluster for the 1D stratification
;        c) Lots of additional information about the process and result of the optimization,
;           which can be displayed, e.g., with opac_cluster_multiplot.pro.
;     3) An 'opa' structure that can be written to a file in standard opta format with wropta.pro:
;        a) Temperature and pressure axes
;        b) Total Rosseland opacity table
;        c) Chosen-mean opacity tables for each cluster
;        d) Some meta data
;   Basic ingredients of the algorithm:
;     1) A general opacity mean applicable in the optically thin and thick
;     2) A metric based on the difference between intensity changes due to
;        mean and total frequency-dependent opacities
;     3) A k-means-cluster-analysis-like optimization scheme (not entirely unlike Lloyds algorithm)
;        based on the above metric
;   Steps:
;     1) Initialize, create arrays
;     2) Interpolate data onto input 1D T/P stratification
;     3) Integrate optical depth and intensity for all frequency points
;     4) Initialize the cluster membership
;     5) Find clusters by iteration:
;        a) Occasionally: add frequency points to the pool
;        b) Compute the mean opacity in each cluster
;        c) Solve the radiative-energy-transfer equation for each cluster
;        d) Sum the total intensity changes for all frequency points in each cluster
;        e) For all clusters, for all frequency points in the list
;             (if new points have been added, consider only them.
;                Otherwise, consider all active points.)
;             Compute the new combined mean opacity
;             Solve the radiative-energy-transfer equation for each combination
;             Determine the relative change in intensity-change error
;             Find the cluster with the best change and put the frequency point there
;             Adjust the total error
;             Count the number of cluster-membership changes
;        f) Plot the results
;     6) Sort the clusters according to mean opacity (from continuum to strong-line cores)
;     7) Compute mean opacities for the entire (T, P) table within each cluster
;     8) Deliver final results in two structures
;   Behavior:
;     The algorithm is able to find a local minimum.
;     For small frequency-point and cluster numbers, the algorithm can actually converge
;       in finite time (minutes).
;     For large frequency-point and cluster numbers, complete convergence would need prohibitively
;       long time spans (many days?).
;     The local minimum the algorithm approaches depends on initial conditions and
;       iteration-control details.
;     The larger the number of requested clusters, the larger the difference in the outcomes
;       with different settings.
;     The local minimum the algorithm approaches is very likely not the global minimum.
;
; EXAMPLE:
;   See opac_cluster_start.pro!
;   
;   opac_cluster, MetaRaw=MetaRaw, Tt=T, Pt=P, rhoT=rho, kappanu=kappanu, nu=nu, $
;     MetaAtm=MetaAtm, x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
;     kCluster=3, $
;     result=res, opa=opa
;   opac_cluster_MultiPlot, result=res, PlotSelect=[20,30]
;   ;
;   nuTest=nu[0:*:200]
;   kappaTest=kappanu[*,*,0:*:200] & kappaTest[*]=1.0 & kappaTest[*,*,0:*:2]=0.1 & kappaTest=kappaTest+0.001*randomu(seed, 72, 71, n_elements(nuTest))
;
; TODO:
;   Expand cluster-membership initialization to approximate "classical" sorting schemes.
;   Write demo routine with toy data.
;   Compute the stellar flux, at least as final check.
;     ... Later ...:
;   Improve the results for large cluster numbers (> 16 or so):
;     Compute distance between clusters.
;     Split or merge clusters.
;     Prefer small clusters or small-error clusters while sorting.
;   Use fuzzy cluster membership, represented by a vector of reals for each cluster (instead
;     of a single integer).
;   Allow more than one reference structure (not so easy...).
;
; MODIFICATION HISTORY:
;   2015-07-07 (B.F. Uppsala) First sketch
;   2015-07-18 (B.F.) First (very slowly, buggy) working version
;   2015-07-21 (B.F.) First (very slowly) working version
;   2015-07-22 (B.F.) Sub steps and various different opacity means
;   2015-07-23 (B.F.) New parameter ModeMean
;   2015-07-25 (B.F.) Compute global means and reference tau scale
;   2015-07-27 (B.F.) New single-frequency-point sub-step mode
;   2015-07-29 (B.F.) Sub iteration not over clusters but frequency points
;   2015-07-31 (B.F.) Change the iteration control to allow more flexibility
;   2015-08-03 (B.F.) Tune the performance
;   2015-08-04 (B.F.) Allow variable chunk sizes
;   2015-08-05 (B.F.) Change result structure
;   2015-08-06 (B.F.) Add more plot frames and exit conditions
;   2015-08-10 (B.F.) Handle tiny contributions: experiment with boost factor, add ModeSmall parameter
;   2015-08-11 (B.F.) Do some clean-up; new exit condition; first version of Opa output structure
;   2015-08-13 (B.F.) Add meta data support
;   2015-08-14 (B.F.) Compose output ID, complete header of OPA structure
;   2015-08-16 (B.F.) Compute full clustered/binned opacity tables (vs. 0.1.2015-08-16)
;   2015-08-21 (B.F.) Add more comments and change default-handling of some keywords
;   2015-08-25 (B.F.) Sort clusters after final sorting -- before 'Dry' -- step
;   2015-08-27 (B.F.) Vs. 0.2.2015-08-27
;   2015-09-06 (B.F.) Some cleanup
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

Program='Opac_Cluster'
Version='0.2.2015-09-06'

; --- Get initial time ---
time0=systime(/SECONDS)

; --- Compute additional axes ---
;lambda    =!con.clight/nu
if (keyword_set(dnu0)) then begin
  dnu=double(dnu0)
endif else begin
  dnu=deriv(double(nu))
endelse

; --- Define abbreviations for array sizes ---
nTt =n_elements(Tt )
nPt =n_elements(Pt )
nnu =n_elements(nu )
nx1D=n_elements(x1D)
if ((nx1D ne n_elements(P1D)) or $
    (nx1D ne n_elements(T1D))     ) then begin
  message, 'Number of elements of 1D stratification arrays do not agree.'
endif

; --- Check for structure with raw-opacity meta data ---
if (keyword_set(MetaRaw0)) then begin
  MetaRaw    =MetaRaw0
endif else begin
  MetaRaw= { $
    Description: 'Meta data of raw opacities' $
           }
endelse
MetaRawTags=tag_names(MetaRaw)

; --- Check for structure with atmosphere meta data ---
if (keyword_set(MetaAtm0)) then begin
  MetaAtm    =MetaAtm0
endif else begin
  MetaAtm= { $
    Description: 'Meta data of raw opacities' $
           }
endelse
MetaAtmTags=tag_names(MetaAtm)

; --- Check for structure with collected input parameters ---
if (keyword_set(Par)) then begin
  ParTags=tag_names(Par)
endif else begin
  ParTags=['']
endelse

; --- Get template for output file name ---
OutputTemplate='t(teff)g(logground)mm00_(rawcode)_oc_(modemean)_b(kcluster)_(datetime)'
if ((where(ParTags eq strupcase('OutputTemplate')))[0] ne -1) then begin
  if (keyword_set(Par.OutputTemplate)) then begin
    OutputTemplate=string(Par.OutputTemplate)
  endif
endif
if (keyword_set(OutputTemplate0)) then begin
  OutputTemplate=string(OutputTemplate0)
endif
print, 'OutputTemplate=           ', OutputTemplate, format='(A,A)'
OutputTemplateLC=strlowcase(OutputTemplate)

; --- Get output ID for output file name ---
OutputMarker=''
if ((where(ParTags eq strupcase('OutputMarker')))[0] ne -1) then begin
  if (keyword_set(Par.OutputMarker)) then begin
    OutputMarker=string(Par.OutputMarker)
  endif
endif
if (keyword_set(OutputMarker0)) then begin
  OutputMarker=string(OutputMarker0)
endif
print, 'OutputMarker=             ', OutputMarker, format='(A,A)'
OutputMarkerLC=strlowcase(OutputMarker)

; --- Get author ---
Author='-'
if ((where(ParTags eq strupcase('Author')))[0] ne -1) then begin
  if (keyword_set(Par.Author)) then begin
    Author=string(Par.Author)
  endif
endif
if (keyword_set(Author0)) then begin
  Author=string(Author0)
endif
print, 'Author=                   ', Author, format='(A,A)'

; --- Get mode of cluster-membership initialization ---
ModeInit='tauSplitEvenBlock'
if ((where(ParTags eq strupcase('ModeInit')))[0] ne -1) then begin
  if (keyword_set(Par.ModeInit)) then begin
    ModeInit=string(Par.ModeInit)
  endif
endif
if (keyword_set(ModeInit0)) then begin
  ModeInit=string(ModeInit0)
endif
print, 'ModeInit=                 ', ModeInit, format='(A,A)'
ModeInitLC=strlowcase(ModeInit)

; --- Get mode of opacity mean ---
ModeMean='RP'
if ((where(ParTags eq strupcase('ModeMean')))[0] ne -1) then begin
  if (keyword_set(Par.ModeMean)) then begin
    ModeMean=string(Par.ModeMean)
  endif
endif
if (keyword_set(ModeMean0)) then begin
  ModeMean=string(ModeMean0)
endif
print, 'ModeMean=                 ', ModeMean, format='(A,A)'
ModeMeanLC=strlowcase(ModeMean)

; --- Get mode of interpretation of xLength parameter ---
ModexLength='Hp'
if ((where(ParTags eq strupcase('ModexLength')))[0] ne -1) then begin
  if (keyword_set(Par.ModexLength)) then begin
    ModexLength=string(Par.ModexLength)
  endif
endif
if (keyword_set(ModexLength0)) then begin
  ModexLength=string(ModexLength0)
endif
print, 'ModexLength=              ', ModexLength, format='(A,A)'
ModexLengthLC=strlowcase(ModexLength)

; --- Get mode of treatment of small contributions ---
ModeSmall='No'
if ((where(ParTags eq strupcase('ModeSmall')))[0] ne -1) then begin
  if (keyword_set(Par.ModeSmall)) then begin
    ModeSmall=string(Par.ModeSmall)
  endif
endif
if (keyword_set(ModeSmall0)) then begin
  ModeSmall=string(ModeSmall0)
endif
print, 'ModeSmall=                ', ModeSmall, format='(A,A)'
ModeSmallLC=strlowcase(ModeSmall)

; --- Get number of clusters ---
kCluster=1 < nnu
if ((where(ParTags eq strupcase('kCluster')))[0] ne -1) then begin
  if (keyword_set(kCluster)) then begin
    kCluster=(long(Par.kCluster) > 1) < nnu
  endif
endif
if (keyword_set(kCluster0)) then begin
  kCluster=(long(kCluster0) > 1) < nnu
endif
print, 'kCluster=                 ', kCluster, format='(A,I12)'

; --- Get typical length scale ---
if (ModexLengthLC eq 'depth') then begin
  xLengthPar=0.17D+00
  ;
endif else if (ModexLengthLC eq 'hp') then begin
  xLengthPar=3.4D+00
  ;
endif else if (ModexLengthLC eq 'cm') then begin
  xLengthPar=5.0D+07
  ;
endif else begin
  message, 'Unknow mode of xLength specification'
  ;
endelse
if ((where(ParTags eq strupcase('xLengthPar')))[0] ne -1) then begin
  if (keyword_set(Par.xLengthPar)) then begin
    xLengthPar=abs(double(Par.xLengthPar))
  endif
endif
if (keyword_set(xLengthPar0)) then begin
  xLengthPar=abs(double(xLengthPar0))
endif
if (ModexLengthLC eq 'depth') then begin
  xLength=xLengthPar * (max(x1D)-min(x1D))
  ;
endif else if (ModexLengthLC eq 'hp') then begin
  xLength=xLengthPar * (max(x1D)-min(x1D))/(alog(max(P1D))-alog(min(P1D)))
  ;
endif else if (ModexLengthLC eq 'cm') then begin
  xLength=xLengthPar
  ;
endif
print, 'xLengthPar=               ', xLengthPar, format='(A,E13.6)'

; --- Get temperature fraction ---
CDeltaT=0.2D+00
if ((where(ParTags eq strupcase('CDeltaT')))[0] ne -1) then begin
  if (keyword_set(Par.CDeltaT)) then begin
    CDeltaT=double(Par.CDeltaT)
  endif
endif
if (keyword_set(CDeltaT0)) then begin
  CDeltaT=double(CDeltaT0)
endif
print, 'CDeltaT=                  ', CDeltaT, format='(A,E13.6)'

; --- Get density exponent for weights ---
ExprhoWeight=0.0D+00
if ((where(ParTags eq strupcase('ExprhoWeight')))[0] ne -1) then begin
  if (keyword_set(Par.ExprhoWeight)) then begin
    ExprhoWeight=double(Par.ExprhoWeight)
  endif
endif
if (keyword_set(ExprhoWeight0)) then begin
  ExprhoWeight=double(ExprhoWeight0)
endif
print, 'ExprhoWeight=             ', ExprhoWeight, format='(A,E13.6)'

; --- Set number of points to ignore at bottom ---
mBot=2

; --- Get number of points to ignore at top (-1: use all points) ---
nTop=-4
if ((where(ParTags eq strupcase('nTop')))[0] ne -1) then begin
  if (keyword_set(Par.nTop)) then begin
    nTop=(-nX1D) > long(Par.nTop) < (nX1D-1)
  endif
endif
if (keyword_set(nTop0)) then begin
  nTop=(-nX1D) > long(nTop0) < (nX1D-1)
endif
print, 'mBot=                     ', mBot, format='(A,I12)'
print, 'nTop=                     ', nTop, format='(A,I12)'

; --- Get initial cluster size ---
nnuPointInitial=100
if ((where(ParTags eq strupcase('nnuPointInitial')))[0] ne -1) then begin
  if (keyword_set(Par.nnuPointInitial)) then begin
    nnuPointInitial=long(Par.nnuPointInitial) > 1
  endif
endif
if (keyword_set(nnuPointInitial0)) then begin
  nnuPointInitial=long(nnuPointInitial0) > 1
endif

; --- Get initial step size on frequency axis (initial coarsening factor) ---
nnuStepInitial=2^round(alog10(nnu/kCluster/nnuPointInitial)/alog10(2.0))
if ((where(ParTags eq strupcase('nnuStepInitial')))[0] ne -1) then begin
  if (keyword_set(Par.nnuStepInitial)) then begin
    nnuStepInitial =long(Par.nnuStepInitial) > 1
    nnuPointInitial=nnu/kCluster/nnuStepInitial
  endif
endif
if (keyword_set(nnuStepInitial0)) then begin
  nnuStepInitial =long(nnuStepInitial0) > 1
  nnuPointInitial=nnu/kCluster/nnuStepInitial
endif
if (kCluster eq 1) then begin
  nnuStepInitial =1
  nnuPointInitial=nnu/kCluster/nnuStepInitial
endif
print, 'nnuPointInitial=          ', nnuPointInitial, format='(A,I12)'
print, 'nnuStepInitial=           ', nnuStepInitial, format='(A,I12)'
nnuStepCur=nnuStepInitial

; --- Get small-contribution fraction ---
CMaskSmall=1.0D-14
if ((where(ParTags eq strupcase('CMaskSmall')))[0] ne -1) then begin
  if (keyword_set(par.CMaskSmall)) then begin
    CMaskSmall=double(Par.CMaskSmall) > 0.0D+00
  endif
endif
if (keyword_set(CMaskSmall0)) then begin
  CMaskSmall=double(CMaskSmall0) > 0.0D+00
endif
print, 'CMaskSmall=               ', CMaskSmall, format='(A,E13.6)'

; --- Get factor for iteration-control array ---
rIterFactor=1.0D+00
if ((where(ParTags eq strupcase('rIterFactor')))[0] ne -1) then begin
  if (keyword_set(par.rIterFactor)) then begin
    rIterFactor=double(Par.rIterFactor) > 0.0D+00
  endif
endif
if (keyword_set(rIterFactor0)) then begin
  rIterFactor=double(rIterFactor0) > 0.0D+00
endif
print, 'rIterFactor=              ', rIterFactor, format='(A,E13.6)'

; --- Get iteration-control array ---
nIterControl=round(rIterFactor * float(2*kCluster+20) * [1.0, 0.92, 0.84, 0.76, 0.68])
if ((where(ParTags eq strupcase('nIterControl')))[0] ne -1) then begin
  if (keyword_set(Par.nIterControl)) then begin
    nIterControl=long(Par.nIterControl)
  endif
endif
if (keyword_set(nIterControl0)) then begin
  nIterControl=long(nIterControl0)
endif
if (kCluster eq 1) then begin
  nIterControl=[-1]
endif
print, 'nIterControl=          ', nIterControl, format='(A,16I5)'

; --- Get lower limit for number of changed cluster to cause a refinement or the exit ---
ExitClusterChangeNumber=0
if ((where(ParTags eq strupcase('ExitClusterChangeNumber')))[0] ne -1) then begin
  if (keyword_set(Par.ExitClusterChangeNumber)) then begin
    ExitClusterChangeNumber=long(Par.ExitClusterChangeNumber) > 0
  endif
endif
if (keyword_set(ExitClusterChangeNumber0)) then begin
  ExitClusterChangeNumber=long(ExitClusterChangeNumber0) > 0
endif
print, 'ExitClusterChangeNumber=  ', ExitClusterChangeNumber, format='(A,I12)'

; --- Get lower limit for fraction of changed cluster to cause a refinement or the exit ---
ExitClusterChangeFraction=0.0D+00
if ((where(ParTags eq strupcase('ExitClusterChangeFraction')))[0] ne -1) then begin
  if (keyword_set(Par.ExitClusterChangeFraction)) then begin
    ExitClusterChangeFraction=double(Par.ExitClusterChangeFraction) > 0.0D+00
  endif
endif
if (keyword_set(ExitClusterChangeFraction0)) then begin
  ExitClusterChangeFraction=double(ExitClusterChangeFraction0) > 0.0D+00
endif
print, 'ExitClusterChangeFraction=', ExitClusterChangeFraction, format='(A,E13.6)'

; --- Get limit for relative error between steps to cause a refinement or the exit ---
ExitRelError=1.0D-04
if ((where(ParTags eq strupcase('ExitRelError')))[0] ne -1) then begin
  ExitRelError=double(Par.ExitRelError) > 0.0D+00
endif
if (n_elements(ExitRelError0) gt 0) then begin
  ExitRelError=double(ExitRelError0)
endif
print, 'ExitRelError=             ', ExitRelError, format='(A,E13.6)'

; --- Get upper limit for wall-clock time to use until exit ---
ExitSysTime=2.0D+05
if ((where(ParTags eq strupcase('ExitSysTime')))[0] ne -1) then begin
  if (keyword_set(Par.ExitSysTime)) then begin
    ExitSysTime=double(Par.ExitSysTime) > 0.0D+00
  endif
endif
if (keyword_set(ExitSysTime0)) then begin
  ExitSysTime=double(ExitSysTime0)
endif
print, 'ExitSysTime=              ', ExitSysTime, ' s', format='(A,E13.6,A)'

; --- Get initial (small) chunk size ---
mChunk=32
if ((where(ParTags eq strupcase('mChunk')))[0] ne -1) then begin
  if (keyword_set(Par.mChunk)) then begin
    mChunk=long(Par.mChunk) > 1
  endif
endif
if (keyword_set(mChunk0)) then begin
  mChunk=long(mChunk0) > 1
endif
print, 'mChunk=                   ', mChunk, format='(A,I12)'

; --- Get final (large) chunk size ---
nChunk=2*mChunk
if ((where(ParTags eq strupcase('nChunk')))[0] ne -1) then begin
  if (keyword_set(Par.nChunk)) then begin
    nChunk=long(Par.nChunk) > mChunk
  endif
endif
if (keyword_set(nChunk0)) then begin
  nChunk=long(nChunk0) > mChunk
endif
print, 'nChunk=                   ', nChunk, format='(A,I12)'


; === Interpolate data onto input 1D T/P stratification ============================================

opac_com_StratInterpolate, time0=time0, $
                           Pt=Pt, Tt=Tt, rhot=rhot, nu=nu, $
                           kappanu=kappanu, $
                           x1D=x1D, T1D=T1D, P1D=P1D, rho1D=rho1D, $
                           kappa1D=kappa1D, B1D=B1D, dBdT1D=dBdT1D

; --- Construct height-dependent weight function based on density ---
rhoMid=sqrt(double(rho1D[0])*double(rho1D[nTop]))
Wei1D =(double(rho1D)/rhoMid)^ExprhoWeight * rhoMid


; === Integrate optical depth scale and intensity ==================================================

print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, tau integration', $
       format='(A,F8.2,A)'

; --- Integrate optical depth scale and compute additional quntities ---
opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappa1D, $
              dxInt=dxInt, dtauInt=dtauInt, tauInt=tauInt, tauTopnu=tauTopnu, HrhoTop=HrhoTop
;dx1D=2.0D+00*rebin(dxInt, nx1D)
tauInt=tauInt[1:*:2,*]
tauIntTop=reform(tauInt[-1,*])

; --- Compute corresponding intensities ---
opac_com_Iintegrate, time0=time0, $
                     x1D=x1D, rho1D=rho1D, B1D=B1D, $
                     dtauInt=dtauInt, tauTopnu=tauTopnu, $
                     dIIntu=dIIntu, IIntu=IIntu, dIIntd=dIIntd, IIntd=IIntd
dIIntu=2.0D+00*rebin(dIIntu, nx1D, nnu)
dIIntd=2.0D+00*rebin(dIIntd, nx1D, nnu)
IIntu =IIntu[1:*:2, *]
IIntd =IIntd[1:*:2, *]
dtauInt=0


; === Create and fill arrays to be used for mean-opacity calculations ==============================

opac_com_Mean, ModeMean=ModeMean, xLength=xLength, CDeltaT=CDeltaT, $
               nu=nu, dnu=dnu, $
               rho1D=rho1D, T1D=T1D, B1D=B1D, dBdT1D=dBdT1D, kappa1D=kappa1D, $
               IIntu=IIntu, IIntd=IIntd, $
               wHat1v=wHat1, wHat1kappa=wHat1kappa, wHat2v=wHat2, wHat2kappa=wHat2kappa, $
               DualwHat_flag=DualwHat_flag


; === Compute global (grey and frequency-integrated) quantities ====================================

dnuAr        =rebin(reform(dnu, 1, nnu), nx1D, nnu)

; --- Sum weight functions and compute opacity means ---
wHat1kappaAll=total(wHat1kappa, 2)
wHat1All     =total(wHat1     , 2)
if (DualwHat_flag eq 0) then begin
  kappaAll     =wHat1kappaAll/wHat1All
endif else begin ; (DualwHat_flag)
  wHat2kappaAll=total(wHat2kappa, 2)
  wHat2All     =total(wHat2     , 2)
  kappaAll=2.0D+00/(wHat1All/wHat1kappaAll + $
                    wHat2All/wHat2kappaAll)
endelse ; (DualwHat_flag)
kappaAll=reform(kappaAll, nx1D, 1)

; --- Compute some global means ---
dnuAll      =total(         dnu     )
BAll        =total(B1D    * dnuAr, 2) / dnuAll
dIIntuSumAll=total(dIIntu * dnuAr, 2) / dnuAll
dIIntdSumAll=total(dIIntd * dnuAr, 2) / dnuAll
IIntuSumAll =total(IIntu  * dnuAr, 2) / dnuAll
IIntdSumAll =total(IIntd  * dnuAr, 2) / dnuAll

; --- Integrate optical depth scale and compute additional quntities ---
opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappaAll, $
              dxInt=dxIntAll, dtauInt=dtauIntAll, tauInt=tauIntAll, $
              tauTopnu=tauTopnuAll
tauIntAll=tauIntAll[1:*:2,*]

; --- Integrate intensity for global mean ---
opac_com_Iintegrate, time0=time0, $
                     x1D=x1D, rho1D=rho1D, B1D=BAll, $
                     dtauInt=dtauIntAll, tauTopnu=tauTopnuAll, $
                     dIIntu=dIIntuAll, IIntu=IIntuAll, dIIntd=dIIntdAll, IIntd=IIntdAll
dIIntuAll  =2.0D+00*rebin(dIIntuAll, nx1D)
dIIntdAll  =2.0D+00*rebin(dIIntdAll, nx1D)
IIntuAll   =IIntuAll[1:*:2,*]
IIntdAll   =IIntdAll[1:*:2,*]
IIntuAllTop=reform(IIntuAll[-1,*])

; --- Compute the normalized intensity difference for the global mean ----------
ddIIuAll=(((dIIntuAll - $
            dIIntuSumAll ) $
          )/Wei1D)^2        *  dnuAll
ddIIdAll=(((dIIntdAll - $
            dIIntdSumAll ) $
          )/Wei1D)^2        *  dnuAll
dIIAllTotal=total((ddIIuAll+ddIIdAll)[mBot:nTop])
print, 'dIIAllTotal=', dIIAllTotal


; === Find reference optical depth(s) tau_greyall(tau_nu=tau_Reference) ============================
tauRefAll   =[1.0, 0.1, 0.01, 5.0]
logtauRefAll    =alog10(tauRefAll)
logtauIntAll    =alog10(tauIntAll)
tauRefInt       =dblarr(n_elements(logtauRefAll), nnu)
kappaRefInt     =dblarr(n_elements(logtauRefAll), nnu)
dkappadtauRefInt=dblarr(n_elements(logtauRefAll), nnu)
BRefInt         =dblarr(n_elements(logtauRefAll), nnu)
RRefInt         =dblarr(n_elements(logtauRefAll), nnu)
for inu=0,nnu-1 do begin
  logtauInt=alog10(tauInt[*, inu])
  tauRefInt[  *, inu]=10.0D+00^interpol(logtauIntAll           , logtauInt, logtauRefAll)
  dkappadtau=deriv(logtauInt, reform(alog10(kappa1D[*, inu])))
  ;dkappadtau=deriv(logtauIntAll, reform(alog10(kappa1D[*, inu])))
  dkappadtauRefInt[*, inu]=interpol(dkappadtau             , logtauInt, logtauRefAll)
  ;dkappadtauRefInt[*, inu]=interpol(dkappadtau             , logtauIntAll, logtauRefAll)
  ;kappaRefInt[*, inu]=10.0D+00^interpol(alog10(kappa1D[*, inu]), logtauInt, logtauRefAll)
  ;kappaRefInt[*, inu]=10.0D+00^interpol(alog10(kappa1D[*, inu]), logtauIntAll, logtauRefAll)
  BRefInt[    *, inu]=10.0D+00^interpol(alog10(B1D[    *, inu]), logtauInt, logtauRefAll)
  RRefInt[    *, inu]=interpol((IIntu[*, inu])/ $
                               (IIntd[*, inu])                 , logtauInt, logtauRefAll)
  ;RRefInt[    *, inu]=interpol(-(IIntu[*, inu]-B1D[*, inu])/ $
  ;                              (IIntd[*, inu]-B1D[*, inu]) , logtauInt, logtauRefAll)
  ;RRefInt[    *, inu]=interpol((IIntu[*, inu]-B1D[*, inu])/ $
  ;                                            B1D[*, inu]  , logtauInt, logtauRefAll)
endfor ; inu
; --- Limit extrapolated small-tau points to minimum value of reference tau scale ---
tauRefInt=tauRefInt > min(tauIntAll)


; === Handle small-contribution frequency points ===================================================
; --- Set reference-depth point for masking ---
ntauMask=nTop-20
if (ModeSmallLC eq strlowcase('No')) then begin
  ; --- Don't treat small contributions differently'
  IndClusterMaskSmall=-1
  nIndTooSmall       =0
endif else begin
  ; --- Create mask for points with too small contibution ---
  qMask   =reform(wHat1kappa[ntauMask, *])
  qMaskTot=total(qMask)
  ;
  IndClusterMaskSmall=where((qMask/qMaskTot) lt CMaskSmall, nIndTooSmall)
  ;
  help, ntauMask, CMaskSmall, nIndTooSmall
endelse


dnuAr     =0


; === Create and initialize some logging arrays and counters =======================================
nFLog       =10000000
dIITotalFLog=dblarr(nFLog)
iFLog       =0
nCLog             =10000
dIITotalCLog      =dblarr(nCLog)
NClusterChangeCLog=lonarr(nCLog)
SysTimeCLog       =dblarr(nCLog)
ModeIterClog      =strarr(nCLog)
iFlogCLog         =lonarr(nCLog)
iCLog             =0


; === Initialize the cluster membership ============================================================

; --- Create the array for the cluster each frequency points belongs to ---
IndCluster=replicate(-2, nnu)
if (nIndTooSmall gt 0) then IndCluster[IndClusterMaskSmall]=-3
; --- Create the array for the number of frequency points in each cluster ---
nnuCluster=lonarr(kCluster)
; --- Create the array for the frequency points in each cluster ---
IndnuCluster=reform(lonarr(nnu, kCluster), nnu, kCluster)

; --- Fill the arrays with initial cluster membership of the frequency points ---
if (ModeInitLC eq 'input') then begin
  if (not keyword_set(InputIndCluster)) then begin
    message, 'No input cluster-membership array "InputIndCluster" specified.'
    ;
  endif else if (n_elements(InputIndCluster) ne nnu) then begin
    message, 'Input cluster-membership array "InputIndCluster" has wrong number of elements.'
    ;
  endif else begin
    ; --- Copy the input array ---
    IndCluster=long(InputIndCluster)
    ;
    ; --- Update the other frequency-index arrays ---
    for iCluster=0,kCluster-1 do begin
      Ind                          =where(IndCluster eq iCluster, n)
      IndnuCluster[0:n-1, iCluster]=Ind
      nnuCluster[         iCluster]=n
    endfor ; iCluster
  endelse
endif else begin ; (ModeInitLC)
  opac_cluster_Init, Mode=ModeInit, Par=ParInit, $
                     nnuStep=nnuStepCur, $
                     nu=nu, dnu=dnu, IIntuAllTop=IIntuAllTop, tauIntRef=reform(tauRefInt[0,*]), $
                     Indcluster=Indcluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster
endelse ; (ModeInitLC)
ddIISource=dblarr(nnu)

; --- Create some cluster arrays ---
DataClusterHasChanged_flags=replicate(1,   kCluster)
wHat1kappaCluster          =dblarr(  nx1D, kCluster)
wHat1Cluster               =dblarr(  nx1D, kCluster)
wHat2kappaCluster          =dblarr(  nx1D, kCluster)
wHat2Cluster               =dblarr(  nx1D, kCluster)
kappaCluster               =dblarr(  nx1D, kCluster)
dnuCluster                 =dblarr(        kCluster)
BCluster                   =dblarr(  nx1D, kCluster)
IIntuSumCluster            =dblarr(  nx1D, kCluster)
IIntdSumCluster            =dblarr(  nx1D, kCluster)
dIIntuSumCluster           =dblarr(  nx1D, kCluster)
dIIntdSumCluster           =dblarr(  nx1D, kCluster)
ddIIuCluster               =dblarr(  nx1D, kCluster)
ddIIdCluster               =dblarr(  nx1D, kCluster)

OnlySinglePointMove_flag   =0
IndClusterSourceSinglePoint=-1
IndClusterTargetSinglePoint=-1
IndnuSinglePoint           =-1

print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, finish initialization', $
       format='(A,F8.2,A)'

; ### Main iteration loop ##########################################################################
iIter             =0
iBigIter          =0
nMaxIter          =1000000
nIterNextBig      =iIter + nIterControl[iBigIter] + 1
iChunk            =mChunk
if ((min(nIterControl) lt 0) and (nnuStepCur eq 1)) then begin
  ; --- No iteration requested: just do one dry run ---
  ExitStatus      ='One single dry run performed'
  ModeIteration   ='Dry'
endif else begin
  ExitStatus      =''
  ModeIteration   =''
endelse
repeat begin
  ;
  print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, start main loop', $
         format='(A,F8.2,A)'
  ;
  ; === (Possibly) add frequency points to the pool and set current iteration mode =================
  if (ModeIteration eq 'Dry') then begin
    ; --- Preparte for final 'Dry' run without any resorting ---------------------------------------
    MaxSubIter   =1
    ;
    if (ModeSmallLC ne strlowcase('No')) then begin
      ; --- Compute reference tau value for each cluster ---
      logtauClusterRef=dblarr(kCluster)
      for iCluster=0,kCluster-1 do begin
        logtauClusterRef[iCluster]=interpol(logtauIntAll, $
                                            alog10(reform(tauCluster[*,iCluster])), $
                                            [0.0D+00])
      endfor ; iCluster
      tauClusterRef=10.0D+00^logtauClusterRef
      print, 'logtauClusterRef: ', logtauClusterRef
    endif ; (ModeSmallLC)
    ;
    ; --- Insert the so far withheld frequency points with (IndCluster eq -3) ---
    if (ModeSmallLC eq strlowcase('LastCluster')) then begin
      ; --- Insert the remaining frequency points into last cluster ---
      IndCand=where(IndCluster eq -3, nIndCand)
      if (nIndCand gt 0) then begin
        IndCluster[IndCand]=kCluster-1
      endif
      ;
    endif else if (ModeSmallLC eq strlowcase('MaxtauCluster')) then begin
      ; --- Insert the remaining frequency points into cluster with largest reference-tau value ---
      Dummy=max(tauClusterRef, iClusterMaxtau)
      ;
      IndCand=where(IndCluster eq -3, nIndCand)
      if (nIndCand gt 0) then begin
        IndCluster[IndCand]=iClusterMaxtau
      endif
      ;
    endif else if (ModeSmallLC eq strlowcase('MintauCluster')) then begin
      ; --- Insert the remaining frequency points into cluster with smallest reference-tau value ---
      dummy=min(tauClusterRef, iClusterMintau)
      ;
      IndCand=where(IndCluster eq -3, nIndCand)
      if (nIndCand gt 0) then begin
        IndCluster[IndCand]=iClusterMintau
      endif
      ;
    endif else if (ModeSmallLC eq strlowcase('SorttauCluster')) then begin
      ; --- Sort the remaining frequency points into cluster with nearest reference-tau value ---
      IndCand=where(IndCluster eq -3, nIndCand)
      if (nIndCand gt 0) then begin
        for iIndCand=0,nIndCand-1 do begin
          Dummy=min((logtauClusterRef-alog10(tauRefInt[0, IndCand[iIndCand]]))^2, iMin)
          IndCluster[IndCand[iIndCand]]=iMin
        endfor ; iIndCand        
      endif
      ;
    endif ; (ModeSmallLC)
    ;
    ; --- Update the other frequency-index arrays ---
    for iCluster=0,kCluster-1 do begin
      Ind                          =where(IndCluster eq iCluster, n)
      IndnuCluster[0:n-1, iCluster]=Ind
      nnuCluster[         iCluster]=n
    endfor ; iCluster
    ;
  endif else if ((nnuStepCur gt 1) and (iIter eq nIterNextBig)) then begin
    ; --- Mark some so far unused frequency points as active ---------------------------------------
    ;
    nnuStepCur=nnuStepCur/2
    ;
    print, '  Add points: iBigIter=', iBigIter, ' nnuStepCur=', nnuStepCur, format='(A,I2,A,I4)'
    ;
    ;IndCluster[0:*:nnuStepCur]=-1 > IndCluster[0:*:nnuStepCur]
    IndCand=where(IndCluster[0:*:nnuStepCur] eq -2, nIndCand)
    if (nIndCand gt 0) then begin
      IndCluster[IndCand*nnuStepCur]=-1
    endif
    ;
    ModeIteration='Big'
    MaxSubIter   =1
    iBigIter     =iBigIter+1
    if (nnuStepCur le 1) then begin
      ; --- This is the last big step ---
      nIterNextBig      =iIter + nIterControl[-1] + 1
      nMaxIter          =nIterNextBig
    endif else begin
      nIterNextBig=iIter + nIterControl[0 > (iBigIter < (n_elements(nIterControl)-2))] + 1
    endelse
    ;
  endif else begin
    ; --- Don't add any new points and just request a number of sub steps --------------------------
    ;
    ModeIteration='Sub'
    MaxSubIter   =nnu
    ;
  endelse ; (ModeInitLC)
  ;
  ; --- Initialize the counter for points with changed cluster membership during current step ---
  NClusterChangeTotal=0
  ;
  DataClusterHasChanged_flags[*]=1
  OnlySinglePointMove_flag      =0
  IndClusterSourceSinglePoint   =-1
  IndClusterTargetSinglePoint   =-1
  IndnuSinglePoint              =-1
  ;
  ; ### Loop over sub steps ########################################################################
  iSubIter       =0
  EndSubIter_flag=0
  repeat begin
    ;
    ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, start sub loop', $
    ;       format='(A,F8.2,A)'
    ;
    ; === Set frequency points currently to be checked =============================================
    IndnuAllActive=where(IndCluster ge 0, nnuAllActive)
    if (ModeIteration eq 'Big') then begin
      ; --- Use all points in the pool without cluster membership ----------------------------------
      ;
      IndnuAllCur=where(IndCluster eq -1, nnuAllCur)
      ;
      iChunk=mChunk-1
      ;
      ; --- Use all active points ---
      ;IndnuAllCur=IndnuAllActive
      ;nnuAllCur  =nnuAllActive
      ;
    endif else if (ModeIteration eq 'Dry') then begin
      ; --- Use all active points ------------------------------------------------------------------
      ;
      IndnuAllCur=IndnuAllActive
      nnuAllCur  =nnuAllActive
      ;
    endif else begin ; (ModeIteration)
      ; --- Prepare (almost) evenly sized distributed chunks ---------------------------------------
      ;
      ; --- Use round robin frequency chunks ---
      IndnuAllCur=IndnuAllActive[iSubIter:*:nnuAllActive/iChunk]
      ;
      nnuAllCur=n_elements(IndnuAllCur)
      ;
      if (iSubIter ge (nnuAllActive/iChunk-1)) then begin
        EndSubIter_flag=1
      endif else begin
        EndSubIter_flag=0
      endelse
      ;
      ;
      ; --- Use chunks of consecutive points --------------------------------------------- 
      ;m=(nnuAllActive* iSubIter   ) / (nnuAllActive/iChunk)
      ;n=(nnuAllActive*(iSubIter+1)) / (nnuAllActive/iChunk) - 1
      ;
      ; --- Use (almost) evenly sized dtau chunks ---
      ;if (iSubIter eq 0) then begin
      ;   ; --- Sort the active points (only at first sub iteration step) ---
      ;   IndnuAllActiveSort=reverse(sort(reform(tauRefInt[0, IndnuAllActive])))
      ;endif
      ;IndnuAllCur=IndnuAllActive[IndnuAllActiveSort[m:n]]
      ;
      ; --- Use (almost) evenly sized frequency chunks ---
      ;IndnuAllCur=IndnuAllActive[m:n]
      ;
      ;nnuAllCur=n-m+1
      ;
      ;if (n ge (nnuAllActive-1)) then begin
      ;  EndSubIter_flag=1
      ;endif else begin
      ;  EndSubIter_flag=0
      ;endelse
      ;
      ;
      ; --- Use all points in current cluster (iSubIter loops over clusters) -------------
      ;IndnuAllCur=where((IndCluster eq -1) or (IndCluster eq (iSubIter mod kCluster)), nnuAllCur)
      ;IndnuAllCur=where(IndCluster eq (iSubIter mod kCluster), nnuAllCur)
      ;
    endelse ; (ModeIteration)
    ;
    if (nnuAllCur gt 0) then begin
      ;
      ; === Compute the mean opacity in each cluster ===============================================
      for iCluster=0,kCluster-1 do begin
        if (DataClusterHasChanged_flags[iCluster] eq 1) then begin
          ; --- Compute frequency indices and some totals for current cluster ---
          nnuCurClus=nnuCluster[iCluster]
          if (nnuCurClus gt 0) then begin
            if (OnlySinglePointMove_flag eq 1) then begin
              ; --- Update the cluster opacity by applying changes ---
              ;
              if (IndClusterSourceSinglePoint eq iCluster) then begin
                ; --- Remove contribution from single frequency point ---
                f=-1.0D+00
              endif else begin
                ; --- Add contribution from single frequency point ---
                f= 1.0D+00
              endelse
              ;
              dnuClusterNew        =dnuCluster[iCluster] + f*dnu[IndnuSinglePoint]
              BCluster[*, iCluster]=(BCluster[*, iCluster        ]  *dnuCluster[iCluster] + $
                                     B1D[     *, IndnuSinglePoint]*f*dnu[IndnuSinglePoint]    ) / $
                                     dnuClusterNew
              ;
              ; --- Sum weight functions and compute opacity means ---
              wHat1kappaCluster[*, iCluster]=wHat1kappaCluster[*, iCluster] + f*wHat1kappa[*, IndnuSinglePoint]
              wHat1Cluster[     *, iCluster]=wHat1Cluster[     *, iCluster] + f*wHat1[     *, IndnuSinglePoint]
              if (DualwHat_flag eq 0) then begin
                kappaCluster[     *, iCluster]=wHat1kappaCluster[*, iCluster]/ $
                                               wHat1Cluster[     *, iCluster]
              endif else begin ; (DualwHat_flag)
                wHat2kappaCluster[*, iCluster]=wHat2kappaCluster[*, iCluster] + f*wHat2kappa[*, IndnuSinglePoint]
                wHat2Cluster[     *, iCluster]=wHat2Cluster[     *, iCluster] + f*wHat2[     *, IndnuSinglePoint]
                kappaCluster[     *, iCluster]=2.0D+00/ (wHat1Cluster[     *, iCluster]/ $
                                                         wHat1kappaCluster[*, iCluster] + $
                                                         wHat2Cluster[     *, iCluster]/ $
                                                         wHat2kappaCluster[*, iCluster]    )
              endelse ; (DualwHat_flag)
              ;
              ; --- Update the intensity changes summed over all frequency points ---
              dIIntuSumCluster[*, iCluster]=(dIIntuSumCluster[*, iCluster]          *dnuCluster[iCluster] + $
                                             dIIntu          [*, IndnuSinglePoint]*f*dnu[IndnuSinglePoint]   ) / $
                                            dnuClusterNew
              dIIntdSumCluster[*, iCluster]=(dIIntdSumCluster[*, iCluster]          *dnuCluster[iCluster] + $
                                             dIIntd          [*, IndnuSinglePoint]*f*dnu[IndnuSinglePoint]   ) / $
                                            dnuClusterNew
              ;
              dnuCluster[iCluster]=dnuClusterNew
              ;
            endif else begin ; (OnlySinglePointMove_flag)
              ; --- Compute the cluster opacity by summing over all contributions ---
              ;
              Indnu     =IndnuCluster[0:nnuCurClus-1, iCluster]
              dnuAr     =rebin(reform(dnu[Indnu],1, nnuCurClus), nx1D, nnuCurClus)
              dnuCluster[          iCluster]=total(dnuAr[0, *])
              BCluster[         *, iCluster]=total(reform(B1D[*, Indnu] * dnuAr, nX1D, nnuCurClus), 2)/ $
                                             dnuCluster[iCluster]
              ;
              ; --- Sum weight functions and compute opacity means ---
              wHat1kappaCluster[*, iCluster]=total(reform(wHat1kappa[*, Indnu], nX1D, nnuCurClus), 2)
              wHat1Cluster[     *, iCluster]=total(reform(wHat1[     *, Indnu], nX1D, nnuCurClus), 2)
              if (DualwHat_flag eq 0) then begin
                kappaCluster[     *, iCluster]=wHat1kappaCluster[*, iCluster]/ $
                                               wHat1Cluster[     *, iCluster]
              endif else begin ; (DualwHat_flag)
                wHat2kappaCluster[*, iCluster]=total(reform(wHat2kappa[*, Indnu], nX1D, nnuCurClus), 2)
                wHat2Cluster[     *, iCluster]=total(reform(wHat2[     *, Indnu], nX1D, nnuCurClus), 2)
                kappaCluster[     *, iCluster]=2.0D+00/ (wHat1Cluster[     *, iCluster]/ $
                                                         wHat1kappaCluster[*, iCluster] + $
                                                         wHat2Cluster[     *, iCluster]/ $
                                                         wHat2kappaCluster[*, iCluster]    )
              endelse ; (DualwHat_flag)
              ;
              ; --- Sum the intensity changes summed over all frequency points in each cluster ---
              dIIntuSumCluster[*, iCluster]=total(reform(dIIntu[*, Indnu] * dnuAr, nX1D, nnuCurClus), 2) / $
                                            dnuCluster[iCluster]
              dIIntdSumCluster[*, iCluster]=total(reform(dIIntd[*, Indnu] * dnuAr, nX1D, nnuCurClus), 2) / $
                                            dnuCluster[iCluster]
              ;
            endelse ; (OnlySinglePointMove_flag)
            ;
          endif else begin ; (nnuCurClus eq 0)
            dnuCluster[iCluster]=0.0D+00
            ;
            ; --- Cluster is empty: zero the opacity-weight arrays and intensity changes ---
            wHat1kappaCluster[*,iCluster]=0.0D+00
            wHat1Cluster[     *,iCluster]=0.0D+00
            ;
            dIIntuSumCluster[*, iCluster]=0.0D+00
            dIIntdSumCluster[*, iCluster]=0.0D+00
            ;
          endelse ; (nnuCurClus)
          ;
        endif ; (DataClusterHasChanged_flags[iCluster])
      endfor ; iCluster
      ;
      if (ModeIteration eq 'Dry') then begin
        for iCluster=0,kCluster-1 do begin
          nnuCurClus=nnuCluster[iCluster]
          if (nnuCurClus gt 0) then begin
            Indnu     =IndnuCluster[0:nnuCurClus-1, iCluster]
            dnuAr     =rebin(reform(dnu[Indnu],1, nnuCurClus), nx1D, nnuCurClus)
            dnuCluster[iCluster]=total(dnuAr[0, *])
            ;
            ; --- Sum the intensity changes for all frequency points in each cluster ---
            IIntuSumCluster[*, iCluster]=total(reform(IIntu[*, Indnu] * dnuAr, nX1D, nnuCurClus), 2) / $
                                         dnuCluster[iCluster]
            IIntdSumCluster[*, iCluster]=total(reform(IIntd[*, Indnu] * dnuAr, nX1D, nnuCurClus), 2) / $
                                         dnuCluster[iCluster]
          endif else begin ; (nnuCurClus eq 0)
            dnuCluster[iCluster]=0.0D+00
            ;
            ; --- Cluster is empty: zero the intensity arrays ---
            IIntuSumCluster[*, iCluster]=0.0D+00
            IIntdSumCluster[*, iCluster]=0.0D+00
            ;
          endelse ; (nnuCurClus)
        endfor ; iCluster
      endif ; (ModeIteration)
      dnuAr=0
      ;
      ; --- Integrate optical depth scale for all clusters ---
      ;
      ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, cluster: tau integration', $
      ;       format='(A,F8.2,A)'
      ;
      ; --- Integrate optical depth scale and compute additional quntities ---
      opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappaCluster, $
                    dxInt=dxIntCluster, dtauInt=dtauCluster, $
                    tauTopnu=tauTopnuCluster
      ;tauCluster=tauCluster[1:*:2,*]
      ;
      ; --- Integrate intensity for all clusters ---
      opac_com_Iintegrate, time0=time0, $
                           x1D=x1D, rho1D=rho1D, B1D=BCluster, $
                           dtauInt=dtauCluster, tauTopnu=tauTopnuCluster, $
                           dIIntu=dIIntuCluster, IIntu=IIntuCluster, $
                           dIIntd=dIIntdCluster, IIntd=IIntdCluster
      dIIntuCluster =2.0D+00*rebin(dIIntuCluster, nx1D, kCluster)
      dIIntdCluster =2.0D+00*rebin(dIIntdCluster, nx1D, kCluster)
      if (ModeIteration eq 'Dry') then begin
        IIntuCluster=IIntuCluster[1:*:2,*]
        IIntdCluster=IIntdCluster[1:*:2,*]
      endif else begin
        IIntuCluster=0
        IIntdCluster=0
      endelse
      dtauCluster=0
      ;
      ; --- Compute the normalized intensity-change difference in each cluster ---
      for iCluster=0,kCluster-1 do begin
        if (DataClusterHasChanged_flags[iCluster] eq 1) then begin
          ddIIuCluster[*, iCluster]=(((dIIntuCluster[   *, iCluster] - $
                                       dIIntuSumCluster[*, iCluster]    ) $
                                     )/Wei1D)^2                            *  dnuCluster[iCluster]
          ddIIdCluster[*, iCluster]=(((dIIntdCluster[   *, iCluster] - $
                                       dIIntdSumCluster[*, iCluster]    ) $
                                     )/Wei1D)^2                            *  dnuCluster[iCluster]
        endif ; (DataClusterHasChanged_flags[iCluster])
      endfor ; iCluster
      dIITotal           =total((ddIIuCluster+ddIIdCluster)[mBot:nTop,*])
      dIITotalFLog[iFLog]=dIITotal
      ;
      ;
      ; --- For all frequency points in current stencil: ---
      ; === Determine the change in intensity-change error when removing a point ===================
      ;
      ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, cluster: determine ddIIu and d', $
      ;       format='(A,F8.2,A)'
      ;
      for iCluster=0,kCluster-1 do begin
        ; --- Get frequency indices of all points in current stencil and current cluster ---
        Indnu=where(IndCluster[IndnuAllCur] eq iCluster, nnuCurClus)
        ;
        if (nnuCurClus gt 0) then begin
          Indnu=IndnuAllCur[Indnu]
          dnuAr=rebin(reform(dnu[Indnu], 1   , nnuCurClus), nx1D, nnuCurClus)
          WeiAr=rebin(reform(Wei1D     , nx1D, 1         ), nx1D, nnuCurClus)
          ;
          ; --- Compute mean opacity of all currently active frequency points and current cluster ---
          wHat1kappaCurClus=-wHat1kappa[*, Indnu] + rebin(wHat1kappaCluster[*, iCluster], nx1D, nnuCurClus)
          wHat1CurClus     =-wHat1[     *, Indnu] + rebin(wHat1Cluster[     *, iCluster], nx1D, nnuCurClus)
          if (DualwHat_flag eq 0) then begin
            kappa1DCurClus   =wHat1kappaCurClus/wHat1CurClus
          endif else begin ; (DualwHat_flag)
            wHat2kappaCurClus=-wHat2kappa[*, Indnu] + rebin(wHat2kappaCluster[*, iCluster], nx1D, nnuCurClus)
            wHat2CurClus     =-wHat2[     *, Indnu] + rebin(wHat2Cluster[     *, iCluster], nx1D, nnuCurClus)
            kappa1DCurClus   =2.0D+00/(wHat1CurClus/wHat1kappaCurClus + wHat2CurClus/wHat2kappaCurClus)
            wHat2kappaCurClus=0
            wHat2CurClus     =0
          endelse ; (DualwHat_flag)
          wHat1kappaCurClus=0
          wHat1CurClus     =0
          BCurClus           =(-B1D[      *, Indnu]*dnuAr + $
                               rebin(BCluster[*, iCluster]*dnuCluster[iCluster], nx1D, nnuCurClus)) / $
                               (-dnuAr+dnuCluster[iCluster])
          ;
          ; --- Integrate optical depth scale and compute additional quantities ---
          opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappa1DCurClus, $
                        dxInt=dxIntCurClus, dtauInt=dtauIntCurClus, $
                        tauTopnu=tauTopnuCurClus
          ;tauIntCurClus=tauIntCurClus[1:*:2,*]
          ;
          ; --- Integrate intensity for all active frequency points ---
          opac_com_Iintegrate, time0=time0, $
                               x1D=x1D, rho1D=rho1D, B1D=BCurClus, $
                               dtauInt=dtauIntCurClus, tauTopnu=tauTopnuCurClus, $
                               dIIntu=dIIntuCurClus, dIIntd=dIIntdCurClus
          dIIntuCurClus =2.0D+00*rebin(dIIntuCurClus, nx1D, nnuCurClus)
          dIIntdCurClus =2.0D+00*rebin(dIIntdCurClus, nx1D, nnuCurClus)
          ;IIntuCurClus=IIntuCurClus[1:*:2,*]
          ;IIntdCurClus=IIntdCurClus[1:*:2,*]
          ;
          ddIIuNew=((       dIIntuCurClus                        - $
                     (rebin(dIIntuSumCluster[*, iCluster    ] *  dnuCluster[iCluster], nx1D, nnuCurClus) - $
                            dIIntu[          *, Indnu       ] *                         dnuAr               ) / $
                                                                (dnuCluster[iCluster] - dnuAr) $
                    )/WeiAr)^2                                * (dnuCluster[iCluster] - dnuAr) 
          ddIIdNew=((       dIIntdCurClus                        - $
                     (rebin(dIIntdSumCluster[*, iCluster    ] *  dnuCluster[iCluster], nx1D, nnuCurClus) - $
                            dIIntd[          *, Indnu       ] *                         dnuAr               ) / $
                                                                (dnuCluster[iCluster] - dnuAr) $
                    )/WeiAr)^2                                * (dnuCluster[iCluster] - dnuAr) 
          ddIIuOld=rebin(ddIIuCluster[*, iCluster], nx1D, nnuCurClus)
          ddIIdOld=rebin(ddIIdCluster[*, iCluster], nx1D, nnuCurClus)
          ;
          ddIISource[Indnu]=total((ddIIuNew-ddIIuOld+ddIIdNew-ddIIdOld)[mBot:nTop,*], 1)
          ;
        endif ; (nnuCurClus gt 1)
      endfor ; iCluster
      ;
      dtauIntCurClus=0
      BCurClus      =0
      dnuAr   =0
      WeiAr   =0
      ddIIuNew=0
      ddIIuOld=0
      ddIIdNew=0
      ddIIdOld=0
      ;
      ;
      ; --- For all frequency points in current stencil: ----------------
      ; === Determine the total change in intensity-change error ===================================
      ;
      ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, cluster: determine ddIITot', $
      ;       format='(A,F8.2,A)'
      ;
      ddIITot=dblarr(nnuAllCur, kCluster)
      ;
      dnuAr=rebin(reform(dnu[IndnuAllCur], 1   , nnuAllCur), nx1D, nnuAllCur)
      WeiAr=rebin(reform(Wei1D           , nx1D, 1        ), nx1D, nnuAllCur)
      ;
      for iCluster=0,kCluster-1 do begin
        ; --- Compute mean opacity of all currently active frequency points and current cluster ---
        ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, iter: what', $
        ;       format='(A,F8.2,A)'
        ;
        wHat1kappaAllCur=wHat1kappa[*, IndnuAllCur] + rebin(wHat1kappaCluster[*, iCluster], nx1D, nnuAllCur)
        wHat1AllCur     =wHat1[     *, IndnuAllCur] + rebin(wHat1Cluster[     *, iCluster], nx1D, nnuAllCur)
        if (DualwHat_flag eq 0) then begin
          kappa1DAllCur   =wHat1kappaAllCur/wHat1AllCur
        endif else begin ; (DualwHat_flag)
          wHat2kappaAllCur=wHat2kappa[*, IndnuAllCur] + rebin(wHat2kappaCluster[*, iCluster], nx1D, nnuAllCur)
          wHat2AllCur     =wHat2[     *, IndnuAllCur] + rebin(wHat2Cluster[     *, iCluster], nx1D, nnuAllCur)
          kappa1DAllCur   =2.0D+00/(wHat1AllCur/wHat1kappaAllCur + wHat2AllCur/wHat2kappaAllCur)
          wHat2kappaAllCur=0
          wHat2AllCur     =0
        endelse ; (DualwHat_flag)
        wHat1kappaAllCur=0
        wHat1AllCur     =0
        BAllCur         =(B1D[      *, IndnuAllCur]*dnuAr + $
                          rebin(BCluster[*, iCluster]*dnuCluster[iCluster], nx1D, nnuAllCur)) / $
                           (dnuAr+dnuCluster[iCluster])
        ;
        ; --- Integrate optical depth scale ---
        ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, iter: tau integration', $
        ;       format='(A,F8.2,A)'
        ;
        ; --- Integrate optical depth scale and compute additional quantities ---
        opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappa1DAllCur, $
                      dxInt=dxIntAllCur, dtauInt=dtauIntAllCur, $
                      tauTopnu=tauTopnuAllCur
        ;tauIntAllCur=tauIntAllCur[1:*:2,*]
        ;
        ; --- Integrate intensity for all current frequency points ---
        ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, iter: intensity integration', $
        ;       format='(A,F8.2,A)'
        ;
        opac_com_Iintegrate, time0=time0, $
                             x1D=x1D, rho1D=rho1D, B1D=BAllCur, $
                             dtauInt=dtauIntAllCur, tauTopnu=tauTopnuAllCur, $
                             dIIntu=dIIntuAllCur, dIIntd=dIIntdAllCur
        dIIntuAllCur =2.0D+00*rebin(dIIntuAllCur, nx1D, nnuAllCur)
        dIIntdAllCur =2.0D+00*rebin(dIIntdAllCur, nx1D, nnuAllCur)
        ;IIntuAllCur=IIntuAllCur[1:*:2,*]
        ;IIntdAllCur=IIntdAllCur[1:*:2,*]
        ;
        ; --- Determine the relative change in intensity-change error:          ---
        ; ---   ((dI_ClusterMeanNew - (dI_ClusterSummedOrg + dI_NewPoint))^2 -  ---
        ; ---    (dI_ClusterMeanOrg - dI_ClusterSummedOrg                )^2    ) ---
        ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, iter: determine ddII', $
        ;       format='(A,F8.2,A)'
        ;
        ddIIuNew=((       dIIntuAllCur                        - $
                   (rebin(dIIntuSumCluster[*, iCluster   ] *  dnuCluster[iCluster], nx1D, nnuAllCur) + $
                          dIIntu[          *, IndnuAllCur] *                         dnuAr              ) / $
                                                             (dnuCluster[iCluster] + dnuAr) $
                  )/WeiAr)^2                               * (dnuCluster[iCluster] + dnuAr) 
        ddIIdNew=((       dIIntdAllCur                        - $
                   (rebin(dIIntdSumCluster[*, iCluster   ] *  dnuCluster[iCluster], nx1D, nnuAllCur) + $
                          dIIntd[          *, IndnuAllCur] *                         dnuAr              ) / $
                                                             (dnuCluster[iCluster] + dnuAr) $
                  )/WeiAr)^2                               * (dnuCluster[iCluster] + dnuAr) 
        ddIIuOld=rebin(ddIIuCluster[*, iCluster], nx1D, nnuAllCur)
        ddIIdOld=rebin(ddIIdCluster[*, iCluster], nx1D, nnuAllCur)
        ;
        ; --- Combine all source and target terms for up and down contributions ---
        ddIITot[*, iCluster]=(total((ddIIuNew-ddIIuOld+ddIIdNew-ddIIdOld)[mBot:nTop,*], 1) + $
                              ddIISource[IndnuAllCur]) * $
                             (IndCluster[IndnuAllCur] ne iCluster)
        ;
      endfor ; iCluster
      ;
      dtauIntAllCur=0
      BAllCur      =0
      dnuAr   =0
      WeiAr   =0
      ddIIuNew=0
      ddIIuOld=0
      ddIIdNew=0
      ddIIdOld=0
      ;
      ; --- Normalize ddIITot ---
      ;ddIITotNorm=rebin(reform(total(abs(ddIITot), 2), nnuAllCur, 1), nnuAllCur, kCluster)
      ;ddIITot0   =ddIITot/ddIITotNorm
      ;
      ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, cluster: change cluster membership', $
      ;       format='(A,F8.2,A)'
      ;
      ; --- Find the cluster with the best change ---
      ddIITotMin   =min(transpose(ddIITot), IndNewCluster, dim=1)
      IndNewCluster=IndNewCluster mod kCluster
      IndOldCluster=IndCluster[IndnuAllCur]
      ;
      ; --- Sort the candidates and take only the most promising one(s) ---
      if (ModeIteration eq 'Sub') then begin
        IndLost=where(IndOldCluster ne IndNewCluster, nLost)
        if (nLost gt 0) then begin
          ; --- Find the one candidate with the minimum total intensity change ---
          ddIICandMin=min(ddIITotMin[IndLost], IndCand)
          IndCand    =IndLost[IndCand]
          ;
          if (nnuCluster[IndOldCluster[IndCand]] le 1) then begin
            ; --- Do not remove the last remaining point from this cluster ---
            ;
            OnlySinglePointMove_flag   =1
            IndClusterSourceSinglePoint=-1
            IndClusterTargetSinglePoint=-1
            IndnuSinglePoint           =-1
            ;
            print, '  Out of ', nLost, $
                   ', don''t move point from cluster ', IndOldCluster[IndCand], $
                   ' to ', IndNewCluster[IndCand], $
                   format='(A,I5,A,I4,A,I4)'
            ;
            NClusterChange             =0
            DataClusterHasChanged_flags[*]=0
            ;
          endif else begin
            ; --- Move frequency point to new cluster ---
            ;
            OnlySinglePointMove_flag   =1
            IndClusterSourceSinglePoint=IndOldCluster[IndCand]
            IndClusterTargetSinglePoint=IndNewCluster[IndCand]
            IndnuSinglePoint           =IndnuAllCur[  IndCand]
            ;
            print, '  From, to get ', nLost, $
                   '        '       , ddIICandMin, $
                   ', move point '  , IndnuSinglePoint, $
                   ' from cluster ' , IndOldCluster[IndCand], ' to ', IndNewCluster[IndCand], $
                   format='(A,I5,A,E15.6,A,I6,A,I4,A,I4)'
            ;
            ; --- Count the number of cluster-membership changes (easy: it's 1) ---
            NClusterChange=1
            ;
            DataClusterHasChanged_flags[*]=0
            ;
            ; --- Associate the frequency point to the best-fit cluster  ---
            IndCluster[IndnuAllCur[IndCand]]=IndNewCluster[IndCand]
            ;
            ; --- Update the other frequency-index arrays ---
            iCluster                             =IndOldCluster[IndCand]
            if (iCluster ge 0) then begin
              Ind                                  =where(IndCluster eq iCluster, n)
              IndnuCluster[        0:n-1, iCluster]=Ind
              nnuCluster[                 iCluster]=n
              DataClusterHasChanged_flags[iCluster]=1
            endif ; (iCluster)
            ;
            iCluster                             =IndNewCluster[IndCand]
            Ind                                  =where(IndCluster eq iCluster, n)
            IndnuCluster[        0:n-1, iCluster]=Ind
            nnuCluster[                 iCluster]=n
            DataClusterHasChanged_flags[iCluster]=1
            ;
          endelse
          ;
        endif else begin ; (nLost)
          ; --- No candidate point found: don't change anything ---
          ;
          OnlySinglePointMove_flag   =1
          IndClusterSourceSinglePoint=-1
          IndClusterTargetSinglePoint=-1
          IndnuSinglePoint           =-1
          ;
          NClusterChange=0
          DataClusterHasChanged_flags[*]=0
        endelse ; (nLost)
        ;
      endif else if (ModeIteration eq 'Dry') then begin
        ; --- Do a dry run: skip any resorting ---
        ;
        OnlySinglePointMove_flag   =1
        IndClusterSourceSinglePoint=-1
        IndClusterTargetSinglePoint=-1
        IndnuSinglePoint           =-1
        ;
        NClusterChange=0
        DataClusterHasChanged_flags[*]=0
        ;
      endif else begin ; (ModeIteration)
        ; --- Big step: likely lots of points have been added --- 
        ;
        OnlySinglePointMove_flag   =0
        IndClusterSourceSinglePoint=-1
        IndClusterTargetSinglePoint=-1
        IndnuSinglePoint           =-1
        ;
        IndCluster[IndnuAllCur]=IndNewCluster
        ;
        ; --- Update the other frequency-index arrays ---
        for iCluster=0,kCluster-1 do begin
          Ind                          =where(IndCluster eq iCluster, n)
          IndnuCluster[0:n-1, iCluster]=Ind
          nnuCluster[         iCluster]=n
        endfor ; iCluster
        ;
        DataClusterHasChanged_flags[*]=1
        ;
        ; --- Count the number of cluster-membership changes ---
        IndnuClusterChange=where(IndOldCluster ne IndNewCluster, NClusterChange)
        ;
        ;for iCluster=0,kCluster-1 do begin
          ;ddIITotDiff=ddIITotMin-ddIITot[*, iCluster]
          ;IndLost    =where((IndOldCluster eq iCluster) and (IndNewCluster ne iCluster), nLost)
          ;print, minmax(ddIITotDiff)
          ;if (nLost gt 0) then print, minmax(ddIITotDiff[IndLost])
          ;IndLost=where((IndOldCluster eq iCluster) and (IndNewCluster ne iCluster) and (ddIITotDiff ge -0.01), nLost)
          ;if (ModeIteration eq 'Big') then begin
          ;  ;if (nLost gt (0.04*nnuCluster[iCluster])) then begin
          ;  ;  print, '  Limit iCluster: ', iCluster, nLost, long(0.96*nLost)
          ;  ;  IndSort=sort(ddIITotDiff[IndLost])
          ;  ;  IndSort=IndLost[reverse(IndSort)]
          ;  ;  IndNewCluster[IndSort[0:long(0.96*nLost)-1]]=iCluster
          ;  ;endif ; (nLost)
          ;endif else begin
          ;  if (nLost gt 1) then begin
          ;    print, '  Limit iCluster: ', iCluster, nLost, nLost-1
          ;    IndSort=sort(ddIITotDiff[IndLost])
          ;    IndSort=IndLost[reverse(IndSort)]
          ;    IndNewCluster[IndSort[0:-2]]=iCluster
          ;  endif ; (nLost)
          ;endelse ; (ModeIteration)
          ;
        ;endfor ; iCluster
        ;
        ;DataClusterHasChanged_flags[*]=0
        ;;
        ;; --- Count the number of cluster-membership changes ---
        ;IndnuClusterChange=where(IndOldCluster ne IndNewCluster, NClusterChange)
        ;;
        ;if (NClusterChange gt 0) then begin
        ;  for iCluster=0,kCluster-1 do begin
        ;    Indtmp=where(IndOldCluster[IndnuClusterChange] eq iCluster, n)
        ;    ;
        ;    if (n gt 0) then begin
        ;      DataClusterHasChanged_flags[iCluster]=1
        ;    endif
        ;  endfor ; iCluster
        ;  ;
        ;endif
        ;;
        ;; --- Associate the frequency points to the best-fit cluster  ---
        ;IndCluster[IndnuAllCur]=IndNewCluster
        ;;
        ;; --- Update the other frequency-index arrays ---
        ;for iCluster=0,kCluster-1 do begin
        ;  Ind                          =where(IndCluster eq iCluster, n)
        ;  IndnuCluster[0:n-1, iCluster]=Ind
        ;  nnuCluster[         iCluster]=n
        ;endfor ; iCluster
        ;
      endelse ; (ModeIteration)
      ;
      NClusterChangeTotal=NClusterChangeTotal + NClusterChange
      ;
      ;print, DataClusterHasChanged_flags, format='(120I2)'
      ;
      ; === Print iteration-step summary =======================================
      print, strmid(ModeIteration, 0, 1), iBigIter, '-', iIter, '-', $
             iSubIter, nnuAllCur, NClusterChange, dIITotal, nnuCluster, $
             format='(A,I2,A,I4,A,I4,2(1X,I6),2X,1E15.6,3X,120I7)'
      print, long(total((ddIIuCluster+ddIIdCluster)[mBot:nTop,*], 1)/dIITotal*1.0E+06), $
             format='(47X,120I7)'
      print, alog10(total((ddIIuCluster+ddIIdCluster)[mBot:nTop,*], 1)), $
             format='(47X,120F7.3)'
      ;
    endif ; (nnuAllCur gt 0)
    ;
    iSubIter=iSubIter+1
    iFLog   =(iFLog+1) < (nFLog-1)
    ;
  endrep until ((iSubIter gt (MaxSubIter-1)) or (EndSubIter_flag eq 1))
  ;
  ;
  ; --- Integrate optical depth scale and compute additional quntities ---
  opac_com_tau, x1D=x1D, rho1D=rho1D, kappa1D=kappaCluster, $
                dxInt=dxIntCluster, dtauInt=dtauCluster, tauInt=tauCluster, $
                tauTopnu=tauTopnuCluster
  tauCluster=tauCluster[1:*:2,*]
  ;
  ; --- Find reference optical depth(s) tau_Cluster(tau_nu=tau_Reference) ---
  tauRefCluster  =dblarr(n_elements(logtauRefAll), kCluster)
  kappaRefCluster=dblarr(n_elements(logtauRefAll), kCluster)
  ;BRefCluster    =dblarr(n_elements(logtauRefAll), kCluster)
  ;RRefCluster    =dblarr(n_elements(logtauRefAll), kCluster)
  logtauCluster   =alog10(tauCluster)
  for iCluster=0,kCluster-1 do begin
    tauRefCluster[  *, iCluster]=10.0D+00^interpol(logtauIntAll, $
                                                   logtauCluster[*, iCluster], logtauRefAll)
    kappaRefCluster[*, iCluster]=10.0D+00^interpol(alog10(kappaCluster[*, iCluster]), $
                                                   logtauCluster[*, iCluster], logtauRefAll)
    ;
    ;BRefCluster[  *, iCluster]=10.0D+00^interpol(alog10(BCluster[*, iCluster]), $
    ;                                                   alog10(tauCluster[*, iCluster]), logtauRefAll)
    ;RRefCluster[  *, iCluster]=         interpol( (IIntuCluster[*, iCluster])/ $
    ;                      (IIntdCluster[*, iCluster]), alog10(tauCluster[*, iCluster]), logtauRefAll)
  endfor ; iCluster
  ;
  ;kappaRefInt=dblarr(n_elements(logtauRefAll), nnu)
  ;for inuAllActive=0,nnuAllActive-1 do begin
  ;  inu     =IndnuAllActive[inuAllActive]
  ;  iCluster=IndCluster[    inu         ]
  ;  kappaRefInt[*, inu]=10.0D+00^interpol(alog10(kappa1D[*, inu]), $
  ;                                        logtauCluster[*, iCluster], logtauRefAll)
  ;  ;kappaRefInt[*, inu]=10.0D+00^interpol(alog10(kappa1D[*, inu]), alog10(tauInt[*, inu]), logtauRefAll)
  ;  ;kappaRefInt[*, inu]=10.0D+00^interpol(alog10(kappa1D[*, inu]), logtauIntAll          , logtauRefAll)
  ;endfor ; inuAllActive
  ;
  ;
  ; === Plot =======================================================================================
  ;
  ;print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, cluster: plot', $
  ;       format='(A,F8.2,A)'
  ;
  opac_cluster_MultiPlot, PlotSelect=PlotSelect, nu=nu, rho1D=rho1D, $
      kCluster=kCluster, IndCluster=IndCluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster, $
        dnuCluster=dnuCluster, kappaCluster=kappaCluster, $
        dIIntuCluster=dIIntuCluster, dIIntdCluster=dIIntdCluster, $
        dIIntuSumCluster=dIIntuSumCluster, dIIntdSumCluster=dIIntdSumCluster, $
        kappaRefCluster=kappaRefCluster, tauRefCluster=tauRefCluster, $
      tauIntAll=tauIntAll, $
        tauRefAll=tauRefAll, tauRefInt=tauRefInt, dkappadtauRefInt=dkappadtauRefInt, $
      iFlog=iFlog, dIITotalFLog=dIITotalFLog
  ;
  ;
  iIter=iIter+1
  ;
  ; --- Put some data into coarse log vectors ---
  dIITotalCLog[      iCLog]=dIITotal
  NClusterChangeCLog[iCLog]=NClusterChangeTotal
  SysTimeCLog[       iCLog]=systime(/SECONDS)-time0
  ModeIterCLog[      iCLog]=ModeIteration
  iFlogCLog[         iClog]=iFLog-1
  ;
  if (iChunk ge nChunk) then begin
    iChunk=mChunk > (nChunk-2)
  endif else begin
    iChunk=mChunk > (iChunk+1) < nChunk
  endelse
  ;
  ; === Check exit conditions ======================================================================
  if (ModeIteration eq 'Dry') then begin
    ; --- This was already a final dry run: exit now ---
    Exit_flag=1
    ;
  endif else begin ; (ModeIteration)
    Exit_flag=0
    ;
    if (nnuStepCur gt 1) then begin
      ; --- The finest frequency level has not been reached: check for refinement condition ---
      ;
      if (NClusterChangeTotal eq 0) then begin
        ; --- The scheme has converged -- but not on the finest frequency level: ---
        ; --- Initiate refinement                                                ---
        nIterNextBig=iIter
        ;
      endif else if (NClusterChangeTotal le ExitClusterChangeNumber) then begin
        ; --- The scheme has almost converged -- but not on the finest frequency level ---
        ; --- Initiate refinement                                                      ---
        nIterNextBig=iIter
        ;
      endif else if (double(NClusterChangeTotal)/double(nnuAllActive > 1) le $
                     ExitClusterChangeFraction                                ) then begin
        ; --- The scheme has almost converged -- but not on the finest frequency level: ---
        ; --- Initiate refinement                                                       ---
        nIterNextBig=iIter
        ;
      endif else if ((ExitRelError                           ge 0.0D+00                ) and $
                     (iCLog                                  ge 1                      ) and $
                     (ModeIteration                          eq 'Sub'                  ) and $
                     (ModeIterCLog[(iClog-1) > 0]            eq 'Sub'                  ) and $
                     ((dIITotalClog[(iClog-1) > 0]-dIITotal) le (ExitRelError*dIITotal))      ) then begin
        ; --- The scheme has almost converged -- but not on the finest frequency level: ---
        ; --- Initiate refinement                                                       ---
        nIterNextBig=iIter
        ;
      endif
    endif else begin ; (nnuStepCur eq 1)
      ; --- The finest frequency level has been reached: check for exit condition ---
      ;
      if (NClusterChangeTotal eq 0) then begin
        ; --- No cluster has changed: initiate a dry run ---
        ExitStatus   ='Convergence: no cluster has changed during last step'
        ModeIteration='Dry'
        ;
      endif else if (NClusterChangeTotal le ExitClusterChangeNumber) then begin
        ; --- Number of changed clusters below limit: initiate a dry run ---
        ExitStatus   ='Almost convergence: number of changed clusters below limit'
        ModeIteration='Dry'
        ;
      endif else if (double(NClusterChangeTotal)/double(nnuAllActive > 1) le $
                     ExitClusterChangeFraction                                ) then begin
        ; --- Number of changed clusters below limit: initiate a dry run ---
        ExitStatus   ='Almost convergence: fraction of changed clusters below limit'
        ModeIteration='Dry'
        ;
      endif else if ((ExitRelError                           ge 0.0D+00                ) and $
                     (iCLog                                  ge 1                      ) and $
                     (ModeIteration                          eq 'Sub'                  ) and $
                     (ModeIterCLog[(iClog-1) > 0]            eq 'Sub'                  ) and $
                     ((dIITotalClog[(iClog-1) > 0]-dIITotal) le (ExitRelError*dIITotal)) ) then begin
        ; --- Relative error below limit: initiate a dry run ---
        ExitStatus   ='Almost convergence: relative error below limit'
        ModeIteration='Dry'
        ;
      endif else if (iIter ge nMaxIter) then begin
        ; --- The maximum number of iterations has been reached: initiate a dry run ---
        ExitStatus   ='Maximum number of iterations reached'
        ModeIteration='Dry'
        ;
      endif else if ((ExitSysTime ge 0.0D+00                  ) and $
                     ((systime(/SECONDS)-time0) ge ExitSysTime)      ) then begin
        ; --- The maximum wall-clock time has been reached: initiate a dry run ---
        ExitStatus   ='Maximum wall-clock time reached'
        ModeIteration='Dry'
        ;
      endif
      ;
    endelse ; (nnuStepCur)
  endelse ; (ModeIteration)
  help, iIter, nMaxIter, nnuStepCur, NClusterChangeTotal, Exit_flag, ModeIteration
  ;
  if ((ModeIteration eq 'Dry') and (Exit_flag eq 0)) then begin
    ; --- Sort the clusters according to reference optical depths (from continuum to line core) ---
    ;
    IndSort        =reverse(sort(reform(tauRefCluster[0, *])))
    IndnuClusterNew=IndnuCluster
    IndClusterNew  =IndCluster
    nnuClusterNew  =nnuCluster
    ;
    print, '  Sort Clusters: ', IndSort
    ;
    for iCluster=0,kCluster-1 do begin
      ; --- Move IndSort[iCluster] to iCluster ---
      nnuCurClus                                =nnuCluster[IndSort[iCluster]]
      nnuClusterNew[                   iCluster]=nnuCurClus
      IndnuClusterNew[0:nnuCurClus-1 , iCluster]=IndnuCluster[0:nnuCurClus-1, IndSort[iCluster]]
      IndClusterNew[IndnuClusterNew[0:nnuCurClus-1, iCluster]]=iCluster
    endfor ; iCluster
    ;
    IndnuCluster=IndnuClusterNew
    IndCluster  =IndClusterNew
    nnuCluster  =nnuClusterNew
    ;
  endif ; (ModeIteration)
  ;
  iCLog   =(iCLog+1) < (nCLog-1)
  ;
endrep until (Exit_flag eq 1)


; === Compose output ID ============================================================================
; --- Construct date and date/time string ---
caldat, systime(/julian), mm,dd,yyyy,hh,mmin,ss
ss=round(ss)
yyyy=strmid('000'+strtrim(string(yyyy),2),3,4,/rev)
mm  =strmid('0'  +strtrim(string(mm  ),2),1,2,/rev)
dd  =strmid('0'  +strtrim(string(dd  ),2),1,2,/rev)
hh  =strmid('0'  +strtrim(string(hh  ),2),1,2,/rev)
mmin=strmid('0'  +strtrim(string(mmin),2),1,2,/rev)
ss  =strmid('0'  +strtrim(string(ss  ),2),1,2,/rev)
DateString      =                   yyyy + '-' + mm   + '-' + dd
DateTimeString  =DateString + ' ' + hh   + ':' + mmin + ':' + ss
DateShString    =                   yyyy       + mm         + dd
DateTimeShString=DateShString     + hh         + mmin       + ss

; --- Replace some terms in round brackets in templates to get the output ID ---
OutputID=OutputTemplate
OutputID=strreplacebf(OutputID, '(outputmarker)', strlowcase(    OutputMarker       )         )
OutputID=strreplacebf(OutputID, '(rawcode)'     , strlowcase(    MetaRaw.Code       )         )
if (max(nIterControl) le 0) then begin
  IterID='0'
endif else begin
  IterID='1'
endelse
OutputID=strreplacebf(OutputID, '(iterid)'      , strlowcase(    IterID             )         )
OutputID=strreplacebf(OutputID, '(modemean)'    , strlowcase(    ModeMean           )         )
OutputID=strreplacebf(OutputID, '(abundanceid)' , strlowcase(    MetaRaw.AbundanceID)         )
OutputID=strreplacebf(OutputID, '(kcluster)'    , strtrim(string(kCluster , FORMAT='(I8)'), 2))
if ((where(MetaAtmTags eq strupcase('Teff')))[0] ne -1) then begin
  TeffString=strtrim(string(round(MetaAtm.Teff), FORMAT='(I8)'), 2)
  OutputID  =strreplacebf(OutputID, '(teff)'    , TeffString)
endif else begin
  TeffString=''
endelse
if ((where(MetaAtmTags eq strupcase('Teffout')))[0] ne -1) then begin
  TeffOutString=strtrim(string(round(MetaAtm.Teffout), FORMAT='(I8)'), 2)
  OutputID     =strreplacebf(OutputID, '(teffout)', TeffOutString)
endif else begin
  TeffOutString=''
endelse
if ((where(MetaAtmTags eq strupcase('grav')))[0] ne -1) then begin
  logg=alog10(MetaAtm.grav)
  ;
  loggValue   =round(abs(logg)*100.0)
  loggString  =strtrim(string(loggValue, FORMAT='(I8)'), 2)
  loggString  =strmid('00' + loggString, strlen(loggString), 3, /REVERSE)
  loggString  =((logg le 0.0) ? 'm' + loggString : loggString)
  OutputID    =strreplacebf(OutputID, '(logg)'  , loggString)
  ;
  loggValue   =50*round(abs(logg)*2.0)
  loggString  =strtrim(string(loggValue, FORMAT='(I8)'), 2)
  loggString  =strmid('00' + loggString, strlen(loggString), 3, /REVERSE)
  loggString  =((logg le 0.0) ? 'm' + loggString : loggString)
  OutputID    =strreplacebf(OutputID, '(logground)'  , loggString)
  ;
  loggwPString=string(logg, FORMAT='(F6.3)')
endif else begin
  loggString  =''
  loggwPString=''
endelse
OutputID=strreplacebf(OutputID, '(date)'    , DateShString      )
OutputID=strreplacebf(OutputID, '(datetime)', DateTimeShString  )

; --- Compute further quantities to be put into result structure ---
dnuAr     =rebin(reform(dnuCluster, 1, kCluster), nx1D, kCluster)
ddIIntTdnu=((dIIntuCluster-dIIntuSumCluster) + (dIIntdCluster-dIIntdSumCluster)) * dnuAr
if (kCluster gt 1) then begin
  ddIIntTdnu=total(ddIIntTdnu, 2)
endif


; === Put the results into the output structure ====================================================
Result={ $
Par:               { $
  Description:       'Control parameters', $
  Program:           Program, $
  Version:           Version, $
  OutputTemplate:    OutputTemplate, $
  OutputMarker:      OutputMarker, $
  ModeInit:          ModeInit, $
  ModeMean:          ModeMean, $
  ModexLength:       ModexLength, $
  ModeSmall:         ModeSmall, $
  kCluster:          kCluster, $
  xLengthPar:        xLengthPar, $
  xLength:           xLength, $
  CDeltaT:           CDeltaT, $
  ExprhoWeight:      ExprhoWeight, $
  mBot:              mBot, $
  nTop:              nTop, $
  nnuPointInitial:   nnuPointInitial, $
  nnuStepInitial:    nnuStepInitial, $
  ntauMask:          ntauMask, $
  CMaskSmall:        CMaskSmall, $
  rIterFactor:       rIterFactor, $
  nIterControl:      nIterControl, $
  ExitClusterChangeNumber: ExitClusterChangeNumber, $
  ExitClusterChangeFraction: ExitClusterChangeFraction, $
  ExitRelError:      ExitRelError, $
  ExitSysTime:       ExitSysTime, $
  mChunk:            mChunk, $
  nChunk:            nChunk, $
  Author:            Author $
                   }, $
  ;
MetaRaw:           MetaRaw, $
  ;
MetaAtm:           MetaAtm, $
  ;
Atm:               { $
  Description:       '1D reference atmosphere', $
  x:                 x1D, $
  T:                 T1D, $
  P:                 P1D, $
  rho:               rho1D, $
  Wei:               Wei1D $
                   }, $
  ;
Status:            { $
  Description:       'Information about iterations and final status', $
  OutputID:          OutputID, $
  ExitStatus:        ExitStatus, $
  DateTime:          DateTimeString, $
  Date:              DateString, $
  nIter:             iIter, $
  nMaxIter:          nMaxIter, $
  nBigIter:          iBigIter, $
  SysTime:           systime(/SECONDS)-time0, $
  dIITotal:          dIITotal, $
  dIIGreyTotal:      dIIAllTotal $
                   }, $
  ;
CLog:              { $
  Description:       'Coarse log', $
  dIITotal:          dIITotalCLog[      0: iCLog-1], $
  NClusterChange:    NClusterChangeCLog[0: iCLog-1], $
  SysTime:           SysTimeCLog[       0: iCLog-1], $
  ModeIteration:     ModeIterCLog[      0: iCLog-1], $
  iFlog:             iFlogCLog[         0: iCLog-1] $
                   }, $
  ;
FLog:              { $
  Description:       'Fine log', $
  dIITotal:          dIITotalFLog[0: iFLog-1] $
                   }, $
  ;
All:               { $
  Description:       'Reference values characterizing each frequency point', $
  nnu:               nnu, $
  nu:                nu, $
  dnu:               dnu, $
  tauRef:            tauRefInt, $
  dkappadtauRef:     dkappadtauRefInt, $
  BRef:              BRefInt, $
  RRef:              RRefInt, $
  IIntuTop:          reform(IIntu[-1,*]), $
  tauTop:            tauIntTop $
                   }, $
  ;
Cluster:           { $
  Description:       'Means and other information for all clusters', $
  nnu:               nnuCluster, $
  dnu:               dnuCluster, $
  tauRef:            tauRefCluster, $
  kappaRef:          kappaRefCluster, $
  Ind:               IndCluster, $
  Indnu:             IndnuCluster, $
  kappa:             kappaCluster, $
  IIntuSum:          IIntuSumCluster, $
  IIntdSum:          IIntdSumCluster, $
  IIntu:             IIntuCluster, $
  IIntd:             IIntdCluster, $
  dIIntuSum:         dIIntuSumCluster, $
  dIIntdSum:         dIIntdSumCluster, $
  dIIntu:            dIIntuCluster, $
  dIIntd:            dIIntdCluster, $
  tau:               tauCluster, $
  ddIIu:             ddIIuCluster, $
  ddIId:             ddIIdCluster, $
  ddIIntTdnu:        ddIIntTdnu $
                   }, $
  ;
Grey:              { $
  Description:       'Means and other information for overall grey case', $
  nnu:               1, $
  dnu:               dnuAll, $
  tauRef:            tauRefAll, $
  kappa:             reform(kappaAll), $
  IIntuSum:          IIntuSumAll, $
  IIntdSum:          IIntdSumAll, $
  IIntu:             IIntuAll, $
  IIntd:             IIntdAll, $
  dIIntuSum:         dIIntuSumAll, $
  dIIntdSum:         dIIntdSumAll, $
  dIIntu:            dIIntuAll, $
  dIIntd:            dIIntdAll, $
  tau:               tauIntAll, $
  ddIIu:             ddIIuAll, $
  ddIId:             ddIIdAll $
                   } $
       }


; === Create the structure for the binned-(clustered)-opacity table ================================

opac_cluster_CreateOpTa, Tt=Tt, Pt=Pt, rhot=rhot, kappanu=kappanu, $
                         Result=Result, $
                         Opa=Opa


; ==================================================================================================

print, '% opac_cluster: ', systime(/SECONDS)-time0, ' s, finish', $
       FORMAT='(A,F8.2,A)'

print
print, '=== ', ExitStatus, ' ==='

end ; opac_cluster
