pro opac_cluster_MultiPlot, PlotSelect=PlotSelect0, dev=dev0, $
      Result=Result, nu=nu, rho1D=rho1D, $
      kCluster=kCluster, IndCluster=IndCluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster, $
        dnuCluster=dnuCluster, kappaCluster=kappaCluster, $
        dIIntuCluster=dIIntuCluster, dIIntdCluster=dIIntdCluster, $
        dIIntuSumCluster=dIIntuSumCluster, dIIntdSumCluster=dIIntdSumCluster, $
        kappaRefCluster=kappaRefCluster, tauRefCluster=tauRefCluster, $
      tauIntAll=tauIntAll, $
        tauRefAll=tauRefAll, tauRefInt=tauRefInt, dkappadtauRefInt=dkappadtauRefInt, $
      iFLog=iFLog, dIITotalFLog=dIITotalFLog
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   opac_cluster_MultiPlot ('opacity_cluster_Multi_Plot')
;
; PURPOSE:
;   Plot results produced by opac_cluster.
;
; CATEGORY:
;   Opacities, plotting
;
; CALLING SEQUENCE:
;   opac_cluster_MultiPlot, PlotSelect=PlotSelect, dev=dev, $
;     Result=Result, nu=nu, rho1D=rho1D, $
;     kCluster=kCluster, IndCluster=IndCluster, nnuCluster=nnuCluster, IndnuCluster=IndnuCluster, $
;       dnuCluster=dnuCluster, kappaCluster=kappaCluster, $
;       dIIntuCluster=dIIntuCluster, dIIntdCluster=dIIntdCluster, $
;       dIIntuSumCluster=dIIntuSumCluster, dIIntdSumCluster=dIIntdSumCluster, $
;       kappaRefCluster=kappaRefCluster, tauRefCluster=tauRefCluster, $
;     tauIntAll=tauIntAll, $
;       tauRefAll=tauRefAll, tauRefInt=tauRefInt, dkappadtauRefInt=dkappadtauRefInt, $
;     iFLog=iFLog, dIITotalFLog=dIITotalFLog
;
; INPUT:
;   None (only via keywords)
;
; OUTPUT:
;   None (only via keywords)
;
; KEYWORDS (INPUT):
;   IndCluster:   ('Index_Cluster') 1D integer vector (nnu) with the cluster
;                 each frequency points belongs to
;   kappaCluster: ('kappa_Cluster') double-precision 2D array with 1D opacity profile for
;                 each cluster
;
; KEYWORDS (OUTPUT):
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   plotlines:             Make multi-line plots
;   replicate1:            Extend vector by replication
;
; GLOBAL VARIALBES (used):
;   !ctv:         Color-table variable (for plotting with plotlines.pro, initialized with setctv)
;
; SIDE EFFECTS:
;   Global variables controlling plotting are changed.
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   ; --- Make multi-panel plot using result structure previously produced by opac_cluster ---
;   opac_cluster_MultiPlot, result=result
;   ; --- Make plot producing only one panel ---
;   opac_cluster_MultiPlot, result=result, PlotSelect=[30]
;
; MODIFICATION HISTORY:
;   2015-08-17 (B.F. Uppsala) Split from opac_cluster.pro and put into own file
;   2015-08-18 (B.F.) Add keyword PlotSelect
;   2015-08-30 (B.F.) Add keyword dev
;-
;---------------------------------------------------------------------------------------------------

; --- Set some compiler options ---
compile_opt defint32, strictarr


; === Initialize ===================================================================================

;print, '% opac_cluster_MultiPlot: ', systime(/SECONDS)-time0, ' s, cluster: plot', $
;       format='(A,F8.2,A)'

if (keyword_set(Result)) then begin
  ; --- Take all input arrays from structure 'Result' ---
  ;
  nu=Result.All.nu
  ;
  rho1D=Result.Atm.rho
  Wei1D=Result.Atm.Wei
  ;
  kCluster=Result.Par.kCluster
  ;
  IndCluster      =Result.Cluster.Ind
  nnuCluster      =Result.Cluster.nnu
  IndnuCluster    =Result.Cluster.Indnu
  dnuCluster      =Result.Cluster.dnu
  kappaCluster    =Result.Cluster.kappa
  dIIntuCluster   =Result.Cluster.dIIntu
  dIIntdCluster   =Result.Cluster.dIIntd
  dIIntuSumCluster=Result.Cluster.dIIntuSum
  dIIntdSumCluster=Result.Cluster.dIIntdSum
  kappaRefCluster =Result.Cluster.kappaRef
  tauRefCluster   =Result.Cluster.tauRef
  ;
  tauIntAll=Result.Grey.tau
  tauRefAll=Result.Grey.tauRef
  ;
  tauRefInt       =Result.All.tauRef
  dkappadtauRefInt=Result.All.dkappadtauRef
  ;
  dIITotalFLog=Result.FLog.dIITotal
  iFLog       =n_elements(dIITotalFLog)
  ;
endif ; (keyword_set(Result))

nX1D=n_elements(rho1D)

; --- Get frame-selection array ---
PlotSelect=[[30, 40, 21, 50],[13, 41, 20, 22]]
if (keyword_set(PlotSelect0)) then begin
  PlotSelect=string(PlotSelect0)
endif
;print, 'PlotSelect=               ', PlotSelect, format='(A,20I3)'

; --- Get plot device ---
dev='X'
if (keyword_set(dev0)) then begin
  dev=string(dev0)
endif
;help, dev

; --- Save global plot parameters ---
Pbak=!P & Xbak=!X & Ybak=!Y

; === Set plot parameters ==========================================================================

; --- Construct file name(s) for plot output ---
if (strupcase(dev) eq 'EPS') then begin
  ps_file=Result.Status.OutputID + '.eps'
endif else if (strupcase(dev) eq 'PS') then begin
  ps_file=Result.Status.OutputID + '.ps'
endif

; --- Set multi-plot parameters ---
nPlot=n_elements(PlotSelect)
if (size(PlotSelect, /N_DIMENSIONS) eq 0) then begin
  !P.MULTI=[0, 0, 0, 0, 0]
endif else if (size(PlotSelect, /N_DIMENSIONS) eq 1) then begin
  !P.MULTI=[0, (size(PlotSelect, /DIMENSIONS))[0], 1, 0, 0]
endif else begin
  !P.MULTI=[0, (size(PlotSelect, /DIMENSIONS))[0], (size(PlotSelect, /DIMENSIONS))[1], 0, 0]
endelse

; --- Choose color table ---
normct

; --- Prepare plotting device ---
if (strupcase(dev) eq 'X') then begin
  set_plot, 'X'
  !P.FONT=-1

  ;  win_id=0
  ;  window, win_id, retain=2
endif else begin ; (dev)
  set_plot, 'ps'
  if (total(!P.MULTI) ge 3) then begin
    !P.CHARSIZE=1.4
  endif else begin
    !P.CHARSIZE=0.7
  endelse
  ;!P.FONT=0
  !P.SYMSIZE=1
  ;device, filename=ps_file, $
  ;        encapsulated=(strupcase(dev) eq 'EPS'), $
  ;        /portrait, xsize=18.0, ysize=12.0, xoffset=0.4, yoffset=0.4, /color, bits=8
  device, filename=ps_file, $
          encapsulated=(strupcase(dev) eq 'EPS'), $
          /portrait, xsize=20.0, ysize=29.0, xoffset=0.4, yoffset=0.4, /color
  ;device, filename=ps_file, $
  ;        encapsulated=(strupcase(dev) eq 'EPS'), $
  ;        /landscape, xsize=29.0, ysize=14.0, xoffset=0.4, yoffset=0.4, /color
  modctv, 'PS'
  ;
  !P.BACKGROUND=!ctv.white
  !P.COLOR     =!ctv.black
endelse ; (dev)

;help, /str, !P

; --- Set line colors ---
lc=[!P.COLOR, !ctv.yellow, !ctv.orange, !ctv.red, !ctv.magenta, !ctv.blue, !ctv.cyan, !ctv.green, !ctv.gray]
;print, 'lc= ', lc
;lcStr=['!ctv.white', '!ctv.yellow', '!ctv.orange', '!ctv.red', '!ctv.magenta', '!ctv.blue', '!ctv.cyan', '!ctv.green', '!ctv.gray']
;
;ic=(1.0+findgen(kCluster))/(kCluster+0.0001) & lc=long(ic^0.25*2^8) + long(ic^2*2^8)*2L^8 + long(ic^8*2^8)*2L^16
;lcb0=float([ $
;  [ 80,   0,  60], $
;  [128,   0,   0], $
;  [192,  32,   0], $
;  [255,  80,   0], $
;  [255, 128,   0], $
;  [255, 192,  32], $
;  [255, 255,  80], $
;  [255, 255, 255], $
;  [128, 128, 255], $
;  [ 64,  64, 255], $ 
;  [  0,   0, 255], $ 
;  [  0,  96, 255], $ 
;  [  0, 192, 255], $ 
;  [  0, 255, 255], $ 
;  [  0, 255, 192], $ 
;  [  0, 255, 128], $ 
;  [  0, 255,  64], $ 
;  [  0, 255,   0], $ 
;  [  0,  80,   0], $ 
;  [108, 128,   0] $ 
;          ])
;lcb0=float([ $
;  [180, 128, 140], $
;  [192,  48, 164], $
;  [232,  32,  80], $
;  [255,   0,   0], $
;  [255, 128,   0], $
;  [255, 255,   0], $
;  [255, 255, 128], $
;  [255, 255, 255], $
;  [128, 255,   0], $
;  [  0, 255,   0], $
;  [  0, 255, 128], $
;  [  0, 255, 255], $
;  [  0, 128, 255], $
;  [  0,   0, 255], $
;  [ 83,  83, 255] $
;          ])
;ic0 =findgen(n_elements(lcb0[0, *])) / float(n_elements(lcb0[0, *])-1)
;lcb1=fltarr(3, kCluster)
;ic1 =findgen(kCluster)/float(kCluster-1)
;for i=0,2 do begin
;  lcb1[i, *]=interpol(reform(lcb0[i, *]), ic0, ic1)
;endfor ; i
;lcb2=long(0.0 > lcb1 < 255.0)
;lcb3=lcb2[0, *] + lcb2[1, *]*256 + lcb2[2, *]*65536
;lc=lcb3


; --- Reshuffle cluster color scale so that clusters appear ordered ---
; --- according to reference tau value                              ---
itau=0
lcv            =replicate1(lc, kCluster)
IndSort        =reverse(sort(reform(tauRefCluster[itau, *])))
lcSort         =lcv
lcSort[IndSort]=lcv

; --- Compute some plotting ranges ---
itau=0
;itau1=1 & itau2=0
logtauRange       =[1.2, -5.8]+alog10(tauRefAll[itau])
;logkappaRatioRange=[-2.0, 1.0]
;dkappadtauRange   =[0.9, -0.6]
dkappadtauRange   =[4.0, -4.0]
lognuRange        =alog10([nu[0], nu[-1]])
logkappaMin       =min(alog10(kappaCluster), max=logkappaMax)
logkappaMargin    =0.1
logkapparange     =[logkappaMin-logkappaMargin, logkappaMax+logkappaMargin]

; --- Set axis titles ---
lognuTitle     =transfont('log(!4m!X/[Hz])')
logkappaTitle  =transfont('log(!4j!X!DCluster!N)')
logtauTitle    =transfont('log(!4s!X)')
IndClusterTitle=transfont('i!DCluster!N')

; === Loop over all plots ==========================================================================
for iPlot=0,nPlot-1 do begin
  if (PlotSelect[iPlot] eq 10) then begin
    ; --- Plot cluster index over log frequency ----------------------------------------------------
    plotlines, alog10(nu), $
               IndCluster, $
               psym=3, $
               xRange=lognuRange, xTitle=lognuTitle, $
               dRange=[-3.2,kCluster-0.8], yTitle=IndClusterTitle, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, alog10(nu[Ind]), $
                 IndCluster[Ind], $
                 /over, $
                 psym=1, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    ;
  endif else if (PlotSelect[iPlot] eq 11) then begin
    ; --- Plot cluster index over log frequency ----------------------------------------------------
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, alog10(nu[Ind]), $
                 IndCluster[Ind], $
                 over=(iCluster gt 0), $
                 xRange=lognuRange, xTitle=lognuTitle, $
                 dRange=[-0.2,kCluster-0.8], yTitle=IndClusterTitle, $
                 psym=1, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    ;
  endif else if (PlotSelect[iPlot] eq 13) then begin
    ; --- Plot cluster index with shift by log(tau_ref) over log frequency -------------------------
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, alog10(nu[Ind]), $
                 IndCluster[Ind] - 0.15*alog10(reform(tauRefInt[itau, Ind])), $
                 over=(iCluster gt 0), $
                 xRange=lognuRange, xTitle=lognuTitle, $
                 dRange=[-0.49,kCluster-0.01], $
                 psym=3, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    ;
  endif else if (PlotSelect[iPlot] eq 20) then begin
    ; --- Plot reference optical depth vs. log frequency -------------------------------------------
    yRange=!Y.RANGE
    !Y.RANGE=logtauRange
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, alog10(nu[Ind]), $
                 alog10(reform(tauRefInt[itau, Ind])), $
                 over=(iCluster gt 0), $
                 xRange=lognuRange, xTitle=lognuTitle, $
                 yTitle=logtauTitle, $
                 psym=3, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    !Y.RANGE=yRange
    ;
  endif else if (PlotSelect[iPlot] eq 21) then begin
    ; --- Plot reference opacity gradient vs. log frequency ----------------------------------------
    yRange=!Y.RANGE
    !Y.RANGE=dkappadtauRange
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, alog10(nu[Ind]), $
                 reform(dkappadtauRefInt[itau, Ind]), $
                 over=(iCluster gt 0), $
                 xRange=lognuRange, xTitle=lognuTitle, $
                 psym=3, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    !Y.RANGE=yRange
    ;
  endif else if (PlotSelect[iPlot] eq 22) then begin
    ; --- Plot reference optical depth vs. reference opacity gradient ------------------------------
    yRange=!Y.RANGE
    !Y.RANGE=logtauRange
    for iCluster=0,kCluster-1 do begin
      n  =nnuCluster[         iCluster]
      Ind=IndnuCluster[0:n-1, iCluster]
      ;
      plotlines, reform(dkappadtauRefInt[itau, Ind]), $
                 alog10(reform(tauRefInt[itau, Ind])), $
                 over=(iCluster gt 0), $
                 xRange=dkappadtauRange, $
                 yTitle=logtauTitle, $
                 psym=3, linecolor=lcSort[iCluster], $
                 dev=dev, /psnoinit, psformat='nochange', /noclose
    endfor ; iCluster
    !Y.RANGE=yRange
    ;
  endif else if (PlotSelect[iPlot] eq 30) then begin
    ; --- Plot cluster opacities over grey tau scale -----------------------------------------------
    yRange=!Y.RANGE
    !Y.RANGE=logkappaRange
    plotlines, alog10(tauIntAll), $
               alog10(kappaCluster), $
               xTitle=logtauTitle, $
               yTitle=logkappaTitle, $
               /zeroline, linecol=lcSort, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauRefCluster[itau, *]), $
               alog10(kappaRefCluster[itau, *]), $
               /over, $
               psym=1, linecol=lcSort, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    !Y.RANGE=yRange
    ;
  endif else if (PlotSelect[iPlot] eq 40) then begin
    ; --- Plot cooling per volume over grey tau scale ----------------------------------------------
    dnuAr    =rebin(reform(dnuCluster, 1, kCluster), nx1D, kCluster)
    ddIIdnuu=(dIIntuCluster-dIIntuSumCluster) * dnuAr
    ddIIdnud=(dIIntdCluster-dIIntdSumCluster) * dnuAr
    if (kCluster gt 1) then begin
      ddIIdnu =total(ddIIdnuu+ddIIdnud, 2)
    endif else begin
      ddIIdnu =ddIIdnuu+ddIIdnud
    endelse
    ;ddIIRange=[min(ddIIdnuu) < min(ddIIdnud), max(ddIIdnuu) > max(ddIIdnud)]
    ddIIRange=[min(ddIIdnu), max(ddIIdnu)]
    plotlines, alog10(tauIntAll), ddIIdnu, $
               xTitle=logtauTitle, $
               drange=ddIIRange, $
               linecol=!ctv.gray, linestyle=0, thick=5, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIdnuu, $
               /over, $
               linecol=lcSort, linestyle=0, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIdnud, $
               /over, $
               linecol=lcSort, linestyle=5, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    ;
  endif else if (PlotSelect[iPlot] eq 41) then begin
    ; --- Plot heating per mass over grey tau scale ------------------------------------------------
    dnuAr    =rebin(reform(dnuCluster, 1   , kCluster), nx1D, kCluster)
    rhoAr    =rebin(reform(rho1D     , nx1D, 1       ), nx1D, kCluster)
    ddIIdnuu=(dIIntuCluster-dIIntuSumCluster) * dnuAr
    ddIIdnud=(dIIntdCluster-dIIntdSumCluster) * dnuAr
    if (kCluster gt 1) then begin
      ddIIdnu =total(ddIIdnuu+ddIIdnud, 2)
    endif else begin
      ddIIdnu =ddIIdnuu+ddIIdnud
    endelse
    ddIIorhodnuu=ddIIdnuu/rhoAr
    ddIIorhodnud=ddIIdnud/rhoAr
    ddIIorhodnu =ddIIdnu /rhoAr
    ddIIorhoRange=[min(ddIIorhodnu), max(ddIIorhodnu)]
    plotlines, alog10(tauIntAll), ddIIorhodnu, $
               xTitle=logtauTitle, $
               drange=ddIIorhoRange, $
               linecol=!ctv.gray, linestyle=0, thick=5, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIorhodnuu, $
               /over, $
               linecol=lcSort, linestyle=0, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIorhodnud, $
               /over, $
               linecol=lcSort, linestyle=5, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    ;
  endif else if (PlotSelect[iPlot] eq 42) then begin
    ; --- Plot heating scaled with weight over grey tau scale --------------------------------------
    dnuAr    =rebin(reform(dnuCluster, 1   , kCluster), nx1D, kCluster)
    WeiAr    =rebin(reform(Wei1D     , nx1D, 1       ), nx1D, kCluster)
    ddIIdnuu=(dIIntuCluster-dIIntuSumCluster) * dnuAr
    ddIIdnud=(dIIntdCluster-dIIntdSumCluster) * dnuAr
    if (kCluster gt 1) then begin
      ddIIdnu =total(ddIIdnuu+ddIIdnud, 2)
    endif else begin
      ddIIdnu =ddIIdnuu+ddIIdnud
    endelse
    ddIIoweidnuu=ddIIdnuu/WeiAr
    ddIIoweidnud=ddIIdnud/WeiAr
    ddIIoweidnu =ddIIdnu /WeiAr
    ddIIoweiRange=[min(ddIIoweidnu), max(ddIIoweidnu)]
    plotlines, alog10(tauIntAll), ddIIoweidnu, $
               xTitle=logtauTitle, $
               drange=ddIIoweiRange, $
               linecol=!ctv.gray, linestyle=0, thick=5, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIoweidnuu, $
               /over, $
               linecol=lcSort, linestyle=0, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    plotlines, alog10(tauIntAll), ddIIoweidnud, $
               /over, $
               linecol=lcSort, linestyle=5, thick=1, $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    ;
  endif else if (PlotSelect[iPlot] eq 50) then begin
    ; --- Plot log error vs. log substep iteration number ------------------------------------------
    plotlines, alog10(findgen(iFLog)+1.0), $
               alog10(dIITotalFLog[0:iFLog-1]), $
               xRange=[0.0, alog10(iFlog)+0.1], $
               dev=dev, /psnoinit, psformat='nochange', /noclose
    ;
  endif
  ;
endfor ; iPlot


;yRange=!Y.RANGE
;!Y.RANGE=logkappaRatioRange
;!P.MULTI=[6,4,2,0,0]
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(nu[Ind]), $
;             alog10(reform(kappaRefInt[itau1, Ind]/kappaRefInt[itau2, Ind])), $
;             over=(iCluster gt 0), $
;             xRange=lognuRange, $
;             psym=3, linecolor=lcSort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange


;yRange=!Y.RANGE
;!Y.RANGE=logkappaRatioRange
;!P.MULTI=[5,4,2,0,0]
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(reform(tauRefInt[itau, Ind])), $
;             alog10(reform(kappaRefInt[itau1, Ind]/kappaRefInt[itau2, Ind])), $
;             over=(iCluster gt 0), $
;             xRange=logtauRange, $
;             psym=3, linecolor=lcSort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange


;yRange=!Y.RANGE
;!Y.RANGE=dkappadtauRange
;!P.MULTI=[5,4,2,0,0]
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(reform(tauRefInt[itau, Ind])), $
;             reform(dkappadtauRefInt[itau, Ind]), $
;             over=(iCluster gt 0), $
;             xRange=logtauRange, $
;             psym=3, linecolor=lcsort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange


;yRange=!Y.RANGE
;!Y.RANGE=logtauRange
;!P.MULTI=[1,4,2,0,0]
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(reform(kappaRefInt[itau1, Ind]/kappaRefInt[itau2, Ind])), $
;             alog10(reform(tauRefInt[itau, Ind])), $
;             over=(iCluster gt 0), $
;             xRange=logkappaRatioRange, $
;             psym=3, linecolor=lcSort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange

;yRange=!Y.RANGE
;!Y.RANGE=logtauRange
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(reform(tauRefInt[0, Ind]/tauRefInt[1, Ind])), alog10(reform(tauRefInt[0, Ind])), $
;             over=(iCluster gt 0), $
;             xRange=[0.3, 1.3], $
;             psym=1, linecolor=lcSort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange
;
;logBRange=alog10([max(BRefInt), min(BRefInt)])
;logBRange=[-5.0, -8.0]
;RRange=[max(RRefInt), min(RRefInt)]
;logRRange=alog10([max(RRefInt[0,*]), min(RRefInt[0,*])])
;RRange=[0.1,10.0]
;yRange=!Y.RANGE
;!Y.RANGE=logRRange
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(nu[Ind]), alog10(reform(RRefInt[0, Ind])), $
;             over=(iCluster gt 0), $
;             xRange=alog10([nu[0], nu[-1]]), $
;             psym=1, linecolor=lcSort[iCluster]
;endfor ; iCluster
;!Y.RANGE=yRange

; --- Plot the individual frequency dependent opacities ---
;for iCluster=0,kCluster-1 do begin
;  n  =nnuCluster[         iCluster]
;  Ind=IndnuCluster[0:n-1, iCluster]
;  ;
;  plotlines, alog10(kappa1D[*, Ind]), drange=[-4,4], linecol=lcsort[iCluster], over=(iCluster > 0)
;endfor ; iCluster
;
;plotlines, ddIITot0, linecol=lcSort, drange=[-1.1,1.1]


; === Write image files, close PS device ===
if (strupcase(dev) eq 'X') then begin
;  if (png_flag) then begin
;    write_png, png_file, tvrd(true=1)
;  endif
;  if (jpg_flag) then begin
;    write_jpeg, jpg_file, tvrd(true=3), true=3
;  endif
endif else begin
  ; --- Close postscript device ---
  device, /close
  set_plot, 'X'
  modctv, 'X'
endelse

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak


end ; opac_cluster_MultiPlot
