pro plotfield, xaxis, yaxis, datain, datamask=datamask, $
  imagemask=imagemask, $
  overplot=overplot, $
    aspectratio=aspectratio, oldaspect=oldaspect, drange=drange , $
  device=device, $
    psfilename=psfilename, psformat=psformat, psnoinit=psnoinit, noclose=noclose, $
  xtitle=xtitle0, ytitle=ytitle0, title=title0, $
    xthick=xthick0, ythick=ythick0, $
    xstyle=xstyle0, ystyle=ystyle0, $
    xrange=xrange0, yrange=yrange0, $
  meansub=meansub, $
  legend=legend, legitems=legitems, head=head, date=date, $
  tv=tv, $
    t_colorrange=t_colorrange, t_notrue=t_notrue, t_sample=t_sample, t_dpcm=t_dpcm0, $
  contour=contour, $
    c_nlevels=c_nlevels0, c_levels=c_levels0, $
    c_annotation=c_annotation, c_labels=c_labels, c_fill=c_fill, $
    c_charsize=c_charsize, c_thick=c_thick0, $
    c_color=c_color, c_linestyle=c_linestyle0, c_smooth=c_smooth, $
  xvec=xvec, yvec=yvec, $
  vectors=vectors, $
    v_columns=v_columns, v_color=v_color, v_length=v_length, $
  streamlines=streamlines, $
    s_randomstart=s_randomstart, s_rows=s_rows0, s_columns=s_columns0, $
    s_timestep=s_timestep, s_nsteps=s_nsteps0, s_norder=s_norder, $
    s_thick=s_thick0, s_color=s_color0, s_arrowsize=s_arrowsize
;
;------------------------------------------------------------------------------
;+
; NAME:
;   plotfield ('plot_field')
;
; PURPOSE:
;   Plot a 2D array as grey scale or contour plot possibly with additional streamlines.
;
; CATEGORY:
;   Plotting
;
; CALLING SEQUENCE:
;   PLOTFIELD, xaxis, yaxis, datain, $
;     overplot=overplot, $
;       aspectratio=aspectratio, oldaspect=oldaspect, drange=drange , $
;     device=device, $
;       psfilename=psfilename, psformat=psformat, psnoinit=psnoinit, noclose=noclose, $
;     xtitle=xtitle, ytitle=ytitle, title=title, $
;       xthick=xthick, ythick=ythick, $
;       xstyle=xstyle, ystyle=ystyle, $
;       xrange=xrange, yrange=yrange, $
;     meansub=meansub, $
;     legend=legend, legitems=legitems, head=head, date=date, $
;     tv=tv, $
;       t_colorrange=t_colorrange, t_notrue=t_notrue, t_sample=t_sample, t_dpcm=t_dpcm, $
;     contour=contour, $
;       c_nlevels=c_nlevels, c_levels=c_levels, $
;       c_annotation=c_annotation, c_labels=c_labels, c_fill=c_fill, $
;       c_charsize=c_charsize, c_thick=c_thick, $
;       c_color=c_color, c_linestyle=c_linestyle, c_smooth=c_smooth, $
;     xvec=xvec, yvec=yvec, $
;     vectors=vectors, $
;       v_columns=v_columns, v_color=v_color, v_length=v_length, $
;     streamlines=streamlines, $
;       s_randomstart=s_randomstart, s_rows=s_rows, s_columns=s_columns, $
;       s_timestep=s_timestep, s_nsteps=s_nsteps, s_norder=s_norder, $
;       s_thick=s_thick, s_color=s_color, s_arrowsize=s_arrowsize
;   
; INPUT:
;   xaxis:      ('x_axis') x axis values
;   yaxis:      ('y_axis') y axis values
;   datain:     ('data_in') Data array (1D or 2D)
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   overplot:   ('over_plot') logical (0/1): if set, the old coordinate system
;               and labeling is used
;   aspectratio:('aspect_ratio') real, if specified this values is used instead the
;               ratio of the lengths of the x- and the y-axis
;   oldaspect:  ('old_aspect_ratio') If set, the standard IDL frame size is used
;   drange:     ('data_range') 2-element real vector specifying the data range.
;               Smaller and larger values are clipped.
;   device:     ('device') string: name of output device:
;               'X':  graphic display or
;               'PS': postscript file
;   psfilename: ('postscript_file_name) string, name of postscript file.
;               If keyword DEVICE is set a postscript file with this name is generated.
;   psformat:   ('postscript_format') list of formattings for postscript:
;                 'ldina4':    Landscape, DIN A 4
;                 'pdina4':    Portrait, DIN A 4
;                 'pdina4_16': Portrait, DIN A 4, small version
;                 'nochange':  No change of plot window size and offset
;                 default:     Portrait, DIN A 4
;   noclose:    ('no_close') logical (0/1): if set and device='ps' the
;               output postscript file is not closed after plotting
;               (to allow overplotting)
;   psnoinit:   ('postscript_no_init') logical (0/1): if set no new output
;               postscript file is generated.
;               It is assumed that one is opened already.
;   xtitle:     ('x_title') string, title of x axis
;   ytitle:     ('y_title') string, title of y axis
;   title:      ('title') string, head line on top of plot
;   xthick:     ('x_thick') x thickness of frame (see !X.THICK)
;   ythick:     ('y_thick') y thickness of frame (see !Y.THICK)
;   xstyle:     ('x_style') x style of frame (see !X.STYLE)
;   ystyle:     ('y_style') y style of frame (see !Y.STYLE)
;   xrange:     ('x_range') 2-element vector with x-axis range 
;               (default: from input data)
;   yrange:     ('y_range') 2-element vector with y-axis range 
;               (default: from input data)
;   meansub:    ('mean_subtract') If set, the horizontal mean (first dimension)
;               is subtracted
;   legend:     ('legend') logical (0/1): if set draw a legend
;   legitems:   ('legend_items') list of strings to label items in legend
;   head:       ('head') vector of strings to be plotted on top of plot
;   date:       ('date') logical (0/1): if set the current time and date is plotted
;               in the upper right corner
;   tv:         ('tv') If set, make gray scale or color plot
;     t_colorrange: ('t_color_range') A two element vector which defines
;                 the boundaries of the desired color table
;     t_sample:   ('t_sample') If set nearest neighbor sampling instead of
;                 bilinear interpolation is used
;     t_dpcm:     ('t_dots_per_centimeter') Dots per centimeter for postscript output,
;                 default: 50
;     t_notrue:   ('t_notrue') Use only 8 bit colour tables (even on 24 bit system)
;   contour:    ('contour') If set, plot contour lines
;     c_nlevels:  ('contour_number_levels') integer, number of levels (default=10)
;     c_levels:   ('contour_levels') real vector with level values (default=automatic)
;     c_annotation: ('contour_annotation') character vector with the labels
;                 to be drawn on the contours (default=value)
;     c_charsize: ('contour_character_size') real, the size of the characters
;                 used to annotate contour labels
;     c_labels:   ('contour_labels') specifies which contour levels should be labeled
;     c_fill:     ('contour_fill') logical (0/1), if set -> filled contour
;     c_color:    ('contour_colour') integer vector (or scalar) with the color indices
;                 used to draw the contours (default=!P.COLOR)
;     c_linestyle: ('contour_line_style') integer vector with the linestyles
;                 used to draw the contours
;     c_thick:    ('contour_thick') integer vector of line thickness values
;     c_smooth:   ('contour_smooth') integer  to activate smoothing of contour lines
;                 useful for MPEG movie production, default=0, reasonable=1,2,3
;   xvec:       ('x_vector') real 2D array with x-component
;               of optional vector field (Use keywords vectors or streamlines)
;   yvec:       ('y_vector') real 2D array with y-component
;               of optional vector field (Use keywords vectors or streamlines)
;   vectors:    ('vectors') logical (0/1), if set, plot vector field with plotfield_velovect
;     v_columns:  ('vector_columns') integer, horizontal number of vectors (default=70)
;     v_color:    ('vector_color') integer, color of vectors (default=!P.COLOR)
;     v_length:   ('vector_length') real, vector length factor
;                 (default=1 -> longest vector=cell length)
;   streamlines: ('stream_lines') logical (0/1),
;                if set, plot vector field with STREAMLINES
;     s_randomstart: ('stream_lines_random_start') real, start with
;                 random initial positions
;     s_rows:     ('stream_lines_rows') integer, number of tracer particle rows
;     s_columns:  ('stream_lines_columns') integer, number of tracer particle columns
;     s_timestep: ('stream_lines_time_step') real, timestep
;     s_nsteps:   ('stream_lines_number_steps') integer, number of points
;                 for one streamline (typical 4)
;     s_norder:   ('stream_lines_number_order') integer, order of integration (1 or 2)
;     s_thick:    ('stream_lines_thick') integer with thickness of streamlines (and arrows)
;     s_color:    ('stream_lines_color') integer with color index of streamlines
;     s_arrowsize:('stream_lines_arrow_size') real with size of arrow head
;
; SUBROUTINES:
;   replicate1:            Extension of REPLICATE: Replicate an array
;   legend:                Draw a legend
;   psymuser:              Draw additional symbols (used by legend)
;   plotfield_rectpix:     Resize 2D array to equidistant gridded axes
;   plotfield_velovect:    Plot a vector field using straight vectors
;   plotfield_streamlines: Integrate pseudo streamlines from a vector field
;   plotfield_arrow:       Replacement of standard IDL routine 'arrow'
;   smooth2d:              Smooth images
;
; SIDE EFFECTS:
;   Plotting into window or postscript file
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;  15. 4.94 Written (Bernd Freytag, Astrophysik Kiel)
;  25. 4.94 (B.F.) Keyword LANDSCAPE substituted by PSFORMAT
;  22. 5.94 (B.F.) Keyword OLDASPECT added
;  23. 5.94 (B.F.) x and y axis may decrease
;  12. 9.94 (B.F.) TV and postscript output: Now 50x50 pixel per cm^2
;  28.12.94 (B.F.) ticklen changed
;   7. 4.95 (B.F.) Attention: Keywords may be changed from 0 to ? during execution (removed)
;   5. 5.95 (B.F.) Space for header smaller, (contour annotation with P.FONT=-1)
;  10. 4.96 (B.F.) Specification of drange -> cut
;  23.10.96 (B.F.) Contour plot labels only with vector fonts
;  14. 3.97 (B.F.) call set_plot, 'X' only if !D.NAME ne 'X', image scaling reprogrammed
;  15.10.99 (B.F.) Keywords xrange, yrange
;  24. 9.04 (B.F. Montpellier) +1 added
;  21.10.04 (B.F.) set_plot, dev, copy=(true eq 0)
;  15. 5.05 (B.F. LA) Screen output changed
;   6. 7.05 (B.F.) Keyword c_smooth added
;  23. 9.09 (B.F.) Keyword t_dpcm added
;   4. 2.11 (B.F. Napels) Handling of keyword datamask debugged (was added earlier)
;  2014-05-27 (B.F. Uppsala) Replace avg() with mean(); some ()->[]
;-
;------------------------------------------------------------------------------

if (n_params() eq 1) then begin
  data=xaxis
endif else if (n_params() eq 2) then begin
  xax=xaxis
  data=yaxis
endif else begin
  xax=xaxis
  yax=yaxis
  data=datain
endelse
;
; --- Check field dimensions ---
datasize=size(data)
if (datasize[0] eq 0) then begin
  if (datasize[1] eq 0) then begin
    message, 'Data field is undefined.'
  endif else begin
    ix=1
    iy=1
  endelse
endif else if (datasize[0] eq 1) then begin
  ix=datasize[1]
  iy=1
endif else if (datasize[0] eq 2) then begin
  ix=datasize[1]
  iy=datasize[2]
endif else begin
  message, 'Data field has more than 2 dimensions.'
endelse
;print, 'Field dimensions:', ix, ' x', iy
;
; --- Check x axis ---
if (n_elements(xax) ne 0) then begin
  xaxissize=size(xax)
  if (xaxissize[0] eq 0) then begin
    if (ix ne 1) then begin
      message, 'x axis has too many elements.'
    endif
  endif else if (xaxissize[0] gt 2) then begin
    message, 'x axis has too many dimensions.'
  endif else if (xaxissize[1] ne ix) then begin
    message, 'x axis has an incorrect number of elements.'
  endif
endif else begin
  xax=findgen(ix)
endelse
;
; --- Check y axis ---
if (n_elements(yax) ne 0) then begin
  yaxissize=size(yax)
  if (yaxissize[0] eq 0) then begin
    if (iy ne 1) then begin
      message, 'y axis has too many elements.'
    endif
  endif else if (yaxissize[0] gt 2) then begin
    message, 'y axis has too many dimensions.'
  endif else if (yaxissize[1] ne iy) then begin
    message, 'y axis has an incorrect number of elements.'
  endif
endif else begin
  yax=findgen(iy)
endelse
;
; --- Subtract mean ---
if (keyword_set(meansub)) then begin
  ;dataavg=avg(data, 0)
  dataavg=mean(data, dim=1)
  for n=0,iy-1 do data[*,n]=data[*,n]-dataavg[n]
endif
;
; --- Choose device ---
if ((!D.N_COLORS gt 256) and not keyword_set(t_notrue)) then true=3 $
                                                        else true=0

;help,true

if keyword_set(device) then dev=device else dev='X'
if (not keyword_set(overplot) and (not keyword_set(psnoinit))) then begin
  if (strupcase(dev) eq 'X') then begin
    if (!D.NAME ne 'X') then set_plot, 'X'
  endif else begin
    set_plot, dev, copy=(true eq 0)               ; --- Select postscript output device ---
    if (n_elements(psfilename) ne 0) then device, filename=psfilename
    if (keyword_set(psformat)) then begin
      if (strlowcase(psformat) eq 'ldina4') then begin
        ; --- Landscape, DIN A 4 ---
        device, /landscape, xsize=29.0, ysize=19.3, xoffset=1.0, yoffset=29.7
      endif else if (strlowcase(psformat) eq 'pdina4') then begin
        ; --- Portrait, DIN A 4 ---
;        device, /portrait , xsize=19.3, ysize=28.0,  xoffset=1.0, yoffset=1.0
        device, /portrait , xsize=19.3, ysize=27.0,  xoffset=1.0, yoffset=1.5
      endif else if (strlowcase(psformat) eq 'pdina4_16') then begin
        ; --- Portrait, DIN A 4, small version ---
        device, /portrait , xsize=16.0, ysize=28.0,  xoffset=1.0, yoffset=1.0
      endif else if (strlowcase(psformat) eq 'pdina4_18x12') then begin
        ; --- Portrait, DIN A 4, small version ---
        device, /portrait , xsize=18.0, ysize=12.0,  xoffset=1.0, yoffset=1.0
      endif else if (strlowcase(psformat) eq 'pdina4_18x6') then begin
        ; --- Portrait, DIN A 4, small version ---
        device, /portrait , xsize=18.0, ysize=6.0,  xoffset=1.0, yoffset=1.0
      endif else if (strlowcase(psformat) eq 'ldina3') then begin
        ; --- Landscape, DIN A 3 ---
        device, /landscape, xsize=39.6, ysize=28, xoffset=0.5, yoffset=40.8
      endif else if (strlowcase(psformat) eq 'nochange') then begin
        ; --- No change of plot window size and offset ---
        print, 'Old device settings'
      endif else begin
        ; --- Default: Portrait, DIN A 4 ---
        device, /portrait , xsize=19.3, ysize=28.0,  xoffset=1.0, yoffset=1.0
      endelse
    endif
    if (keyword_set(c_color)) then begin         ; --- Enable color output ---
      if (min(c_color) ge 0) then device, /color, bits_per_pixel=8
    endif
    if keyword_set(tv) then begin
      ;if (true eq 3) then begin
      ;  device, /color, /decomposed
        ;modctv, dev
      ;endif else begin
        device, /color, bits_per_pixel=8
        modctv, dev
      ;endelse
    endif
    ;help, /device 
  endelse
endif
;
; --- Title ---
if (not keyword_set(xtitle0)) then xtitle=!X.TITLE else xtitle=xtitle0
if (not keyword_set(ytitle0)) then ytitle=!Y.TITLE else ytitle=ytitle0
if (not keyword_set( title0)) then  title=!P.TITLE else  title=title0
;
; --- Thickness ---
if (not keyword_set(xthick0)) then $
   if (!X.THICK eq 0) then xthick=2 else xthick=!X.THICK else xthick=xthick0
if (not keyword_set(ythick0)) then $
   if (!Y.THICK eq 0) then ythick=2 else ythick=!Y.THICK else ythick=ythick0
if (not keyword_set(c_thick0)) then c_thick=!P.THICK else c_thick=c_thick0
numc_thick=n_elements(c_thick)
;
; --- Style ---
if (not keyword_set(xstyle0)) then $
  if (!X.STYLE eq 0) then xstyle=1 else xstyle=!X.STYLE else xstyle=xstyle0
if (not keyword_set(ystyle0)) then $
  if (!Y.STYLE eq 0) then ystyle=1 else ystyle=!Y.STYLE else ystyle=ystyle0
;
; --- Range ---
if (keyword_set(xrange0)) then begin
  xaxmin=xrange0[0]
  xaxmax=xrange0[1]
endif else begin
  xaxmin=xax[0]                        ; --- x axis ---
  xaxmax=xax[ix-1]
endelse
if (keyword_set(yrange0)) then begin
  yaxmin=yrange0[0]
  yaxmax=yrange0[1]
endif else begin
  yaxmin=yax[0]                        ; --- y axis ---
  yaxmax=yax[iy-1]
endelse
if (keyword_set(drange)) then begin    ; --- data ---
  datamin=min(drange, max=datamax)
  if (datamin eq datamax) then datamin=min(data, max=datamax)
endif else begin
  datamin=min(data, max=datamax)
endelse
;
; --- Color ---
if (n_elements(c_color) eq 0) then begin
  lc=!P.COLOR
endif else if (size(c_color, /type) eq 7) then begin
  lc=c_color
endif else if (min(c_color) lt 0) then begin
  lc=!P.COLOR
endif else begin
  lc=c_color
endelse
numc_color=n_elements(lc)
;
; --- Linestyle ---
if (not keyword_set(c_linestyle0)) then c_linestyle=!P.LINESTYLE $
                                   else c_linestyle=c_linestyle0
numc_linestyle=n_elements(c_linestyle)
;
; --- Levels ---
if (not keyword_set(c_nlevels0)) then c_nlevels=10 else c_nlevels=c_nlevels0
if (not keyword_set(c_levels0)) then $
  c_levels=(findgen(c_nlevels)+1.0)/float(c_nlevels+1)*(datamax-datamin) + $
           datamin $
  else c_levels=c_levels0
;
; --- Fill with colors ---
if (not keyword_set(c_fill)) then c_fill=0
;
; --- Size of contour label characters ---
if (not keyword_set(c_charsize)) then c_charsize=0
;
; --- Annotation ---
if (not keyword_set(c_annotation)) then begin
  c_ann=''
endif else if (c_annotation(0) eq 'auto') then begin
  c_ann=strcompress(string(c_levels))
endif else begin
  c_ann=c_annotation
endelse
if (not keyword_set(c_labels)) then begin
  c_lab=replicate(0, 30)
endif else begin
  c_lab=replicate1(c_labels, 30)
endelse
;
; --- Header --
ymargin=!Y.MARGIN
if (keyword_set(head) and not keyword_set(overplot)) then begin
  headnum=n_elements(head)
  if (!P.CHARSIZE eq 0) then headcharsize=1.0 else headcharsize=!P.CHARSIZE
;  ymargin(1)=!Y.MARGIN(1) > float((2.0+headnum*1.8)*headcharsize)
  ymargin(1)=!Y.MARGIN(1) > float((2.0+headnum*1.8))
endif
;
; --- Find largest possible frame ---
if (not keyword_set(overplot)) then begin
  ; --- Make unvisible testplot ---
  plot, [xaxmin,xaxmax], [yaxmin,yaxmax], /nodata, $
    xrange=[xaxmin,xaxmax], yrange=[yaxmin,yaxmax], $
    ymargin=ymargin, $
    xstyle=8*(xstyle/8)+4+(xstyle mod 4), ystyle=8*(ystyle/8)+4+(ystyle mod 4)
   ;help, xstyle, ystyle, 8*(xstyle/8)+4+(xstyle mod 4), 8*(ystyle/8)+4+(ystyle mod 4)
   ;print, ymargin
endif
; --- Find coordinates of frame ---
px = !X.WINDOW * !D.X_VSIZE  ; Get size of window in device units (frame size)
py = !Y.WINDOW * !D.Y_VSIZE
xleft0 =px[0]
ybot0  =py[0]
xright0=px[1]
ytop0  =py[1]
;help, xleft0, ybot0, xright0, ytop0
frameaspect=(xright0-xleft0)/(ytop0-ybot0)
if (not keyword_set(overplot)) then begin
  ; --- Find coordinates of plotting area ---
  coco=convert_coord([xaxmin,xaxmax],[yaxmin,yaxmax], /data, /to_device)
  xleft=coco(0,0)              ; Get size of plot area (<= frame size)
  ybot=coco(1,0)
  xright=coco(0,1)
  ytop=coco(1,1)+1.0    ; --- 2004-09-24: +1.0 added
  plotaspect=(xright-xleft)/(ytop-ybot)
  dataaspect=(xaxmax-xaxmin)/(yaxmax-yaxmin)
  ;
  ; --- Transform coordinates of frames edges to get correct aspect ratio ---
  if (not keyword_set(aspectratio)) then aspectratio=dataaspect
  if (keyword_set(oldaspect)) then aspectratio=frameaspect
  if (abs(aspectratio) gt plotaspect) then begin
    ytop0=(ytop0-ybot0)*plotaspect/abs(aspectratio)+ybot0
  endif else begin
    xright0=(xright0-xleft0)*abs(aspectratio)/plotaspect+xleft0
  endelse
  ;
  ; --- Fix plotting coordinate system with correct aspect ratio ---
  plot, [xaxmin,xaxmax], [yaxmin,yaxmax], /nodata, $
    xrange=[xaxmin,xaxmax], yrange=[yaxmin,yaxmax], $
    position=[xleft0,ybot0,xright0,ytop0], /device, $
    /noerase, $
    xstyle=8*(xstyle/8)+4+(xstyle mod 4), ystyle=8*(ystyle/8)+4+(ystyle mod 4)
endif else begin
  if (not keyword_set(aspectratio)) then aspectratio=frameaspect
endelse


if (keyword_set(contour) and (strupcase(dev) eq 'X') and (true eq 3) and (keyword_set(c_smooth))) then begin
  ; --- Determine plot window origin and size ---
  coco=convert_coord([xaxmin,xaxmax],[yaxmin,yaxmax], /data, /to_device)
  xleft=coco(0,0)
  ybot=coco(1,0)
  xright=coco(0,1)
  ytop=coco(1,1)
  swx=xright-xleft
  swy=ytop-ybot
  imagetmp=tvrd(xleft, ybot, swx, swy, true=3)
  tv, imagetmp*0B, xleft, ybot, xsize=swx, ysize=swy, true=3
  contour, data, xax, yax, /overplot, levels=c_levels, $
    xstyle=xstyle, ystyle=ystyle, $
    c_thick=c_thick+2*c_smooth
  imagemask=(tvrd(xleft, ybot, swx, swy, true=3) ne 0B)
  tv, imagetmp, xleft, ybot, xsize=swx, ysize=swy, true=3
endif


; --- Grey scale or color plot ---
if (keyword_set(tv)) then begin
  ; --- Determine plot window origin and size ---
  coco=convert_coord([xaxmin,xaxmax],[yaxmin,yaxmax], /data, /to_device)
  xleft=coco(0,0)
  ybot=coco(1,0)
  xright=coco(0,1)
  ytop=coco(1,1)
  swx=xright-xleft
  swy=ytop-ybot
  ;help, xleft, ybot, xright, ytop
  ;if ((!D.N_COLORS gt 256) and not keyword_set(t_notrue)) then true=3 $
  ;                                                        else true=0
  ;help, !D.N_COLORS, t_notrue, true
  if (not keyword_set(t_sample)) then t_sample=0
  if (not keyword_set(t_dpcm0)) then begin
    t_dpcm=50.0
  endif else begin
    t_dpcm=1 > t_dpcm0 < 2000
  endelse
  ;
  ; --- Produce rectangular pixels ---
;print,  '(!D.FLAGS and 1):', (!D.FLAGS and 1)
  if (!D.FLAGS and 1) ne 0 then begin          ; Scalable pixels
;    image=plotfield_rectpix(data,xax,yax,xfac=800./float(ix),sample=t_sample, $
;            aspectratio=abs(aspectratio))
    ; --- 12.9.94: 50 pixel per cm ---
    ; --- 23.7.09: parameter for pixel per cm ---
    image=plotfield_rectpix(data,xax,yax, $
            xfac=(xright0-xleft0)/float(!D.X_PX_CM)*t_dpcm/float(ix), $
            sample=t_sample, aspectratio=abs(aspectratio))
    if (keyword_set(datamask)) then begin
      image_datamask=plotfield_rectpix(datamask,xax,yax, $
                       xfac=(xright0-xleft0)/float(!D.X_PX_CM)*t_dpcm/float(ix), $
                       sample=t_sample, aspectratio=abs(aspectratio))
      image_datamask=0.0 > image_datamask < 1.0
;help, image_datamask
;print, minmax(image_datamask)
    endif
  endif else begin                             ; Not scalable pixels
    image=plotfield_rectpix(data,xax,yax,xfac=swx/float(ix),sample=t_sample, $
            aspectratio=abs(aspectratio))
    if (keyword_set(datamask)) then begin
      image_datamask=plotfield_rectpix(datamask,xax,yax,xfac=swx/float(ix),sample=t_sample, $
                       aspectratio=abs(aspectratio))
      image_datamask=0.0 > image_datamask < 1.0
;help, image_datamask
;print, minmax(image_datamask)
    endif
  endelse
  ;
  ; --- Rotate if neccessary ---
  if (xaxmin gt xaxmax) then begin
    if (yaxmin gt yaxmax) then direction=2 else direction=5
  endif else begin
    if (yaxmin gt yaxmax) then direction=7 else direction=0
  endelse
  image=rotate(image,direction)
  ;
  ; --- Reduce data range to specified limits (10.4.96) ---
  image=datamin > image < datamax
  ;
  ; --- Check display type ---
  ;help, true
  if (true eq 3) then begin
    ; --- True color terminal (24 bit) ---
    ;
    ; --- Transform from data values to colors ---
    if (n_elements(t_colorrange) eq 0) then begin
      tvlct, ctred, ctgreen, ctblue, /GET
      color_fi=0
      color_ra=n_elements(ctred)-1
    endif else begin
      ctred  =t_colorrange[*,0]
      ctgreen=t_colorrange[*,1]
      ctblue =t_colorrange[*,2]
      color_fi=0
      color_ra=n_elements(t_colorrange[*,0])-1
    endelse
;help, image
;print, minmax(image)
    image=(image-datamin)*(float(color_ra)/float(datamax-datamin))
;help, image
;print, minmax(image)
    image=byte(( 0 > image < color_ra) + color_fi)
;help, image
;print, minmax(image)
    if (keyword_set(datamask)) then begin
      image=byte([[[ctred[image]*image_datamask]], [[ctgreen[image]*image_datamask]], [[ctblue[image]*image_datamask]]]) 
    endif else begin
      image=[[[ctred[image]]], [[ctgreen[image]]], [[ctblue[image]]]]
    endelse
;help, image
;print, minmax(image)
    tv, image, true=3, xleft, ybot, xsize=swx, ysize=swy
    ;
    ; --- Display image ---
;    tv, ctred[  image], xleft, ybot, xsize=swx, ysize=swy, channel=1
;    tv, ctgreen[image], xleft, ybot, xsize=swx, ysize=swy, channel=2
;    tv, ctblue[ image], xleft, ybot, xsize=swx, ysize=swy, channel=3
;    image_3=bytarr(n_elements(image[*,0]), n_elements(image[0,*]), 3)
;    image_3[*,*,0]=ctred[  image]
;    image_3[*,*,1]=ctgreen[image]
;    image_3[*,*,2]=ctblue[ image]
;    tv, image_3, true=3, xleft, ybot, xsize=swx, ysize=swy
;    image_3=[[[ctred[image]]], [[ctgreen[image]]], [[ctblue[image]]]]
;    print,min(image_3),max(image_3)
;    tv, image_3, true=3, xleft, ybot, xsize=swx, ysize=swy

  endif else begin
    ; --- Pseudo color terminal (8 bit) ---
    ;
    ; --- Transform from data values to colors ---
    if (n_elements(t_colorrange) eq 0) then begin           ; Full color range
      color_ra=!D.TABLE_SIZE-1
      color_fi=0
    endif else if (n_elements(t_colorrange) eq 1) then begin ; Cut color range
      color_ra=t_colorrange
      color_fi=0
    endif else begin                                     ; Shifted color range
      color_ra=t_colorrange(1)-t_colorrange(0)
      color_fi=t_colorrange(0)
    endelse
    image=(image-datamin)*(float(color_ra)/float(datamax-datamin))
    image=byte(( 0 > image < color_ra) + color_fi)
    ;
    ; --- Display image ---
    tv, image, xleft, ybot, xsize=swx, ysize=swy

;    bar=rebin(reform(findgen(swy)*(float(color_ra)/(swy-1.)),1,swy),20,swy)
;    tv, bar, xleft+swx+10, ybot, xsize=20, ysize=swy

  endelse
endif
;
; --- Plot contours ---
if (keyword_set(contour)) then begin
  pfont=!P.FONT
  !P.FONT=-1
  contour, data, xax, yax, /overplot, levels=c_levels, $
    xstyle=xstyle, ystyle=ystyle, $
    c_thick=c_thick, c_color=lc, c_linestyle=c_linestyle, fill=c_fill, $
    c_annotation=c_ann, c_labels=c_lab, c_charsize=c_charsize
  if ((strupcase(dev) eq 'X') and (true eq 3) and (keyword_set(c_smooth))) then begin
    ; --- Determine plot window origin and size ---
    coco=convert_coord([xaxmin,xaxmax],[yaxmin,yaxmax], /data, /to_device)
    xleft=coco(0,0)
    ybot=coco(1,0)
    xright=coco(0,1)
    ytop=coco(1,1)
    swx=xright-xleft
    swy=ytop-ybot
    imageorg           =tvrd(xleft, ybot, swx, swy, true=3)
    imagecontour       =imageorg
    imagecontour(*,*,0)=smooth2d(imagecontour(*,*,0), q_smooth=c_smooth)
    imagecontour(*,*,1)=smooth2d(imagecontour(*,*,1), q_smooth=c_smooth)
    imagecontour(*,*,2)=smooth2d(imagecontour(*,*,2), q_smooth=c_smooth)
    imagecontour=imagemask*imagecontour + (1B-imagemask)*imageorg
    tv, imagecontour, xleft, ybot, xsize=swx, ysize=swy, true=3
  endif

  !P.FONT=pfont
endif
;
if (keyword_set(date)) then xyouts, 0.99, 0.98, /norm, ali=1.0, charsize=0.7, $
                                    systime(0)
;
; --- Plot vector field ---
if (keyword_set(vectors) and keyword_set(xvec) and keyword_set(yvec)) then begin
  if (not keyword_set(v_columns)) then v_columns=70.0
  if (n_elements(v_color) eq 0) then begin
    v_color=!P.COLOR
  endif else if (min(v_color) lt 0) then begin
    v_color=!P.COLOR
  endif
  if (not keyword_set(v_length)) then v_length=1.0
  imxvec=plotfield_rectpix(xvec,xax,yax,xfac=v_columns/float(ix), $
           xnew=xaxnew, ynew=yaxnew)
  imyvec=plotfield_rectpix(yvec,xax,yax,xfac=v_columns/float(ix))
  plotfield_velovect, imxvec, imyvec, xaxnew, yaxnew, /overplot, noclip=0, $
    color=v_color, length=v_length
endif
;
; --- Plot streamlines ---
if (keyword_set(streamlines) and keyword_set(xvec) and keyword_set(yvec)) $
  then begin
  if (not keyword_set(s_randomstart)) then s_randomstart=0
  if (not keyword_set(s_rows0)) then s_rows=20 else s_rows=s_rows0
  if (not keyword_set(s_columns0)) then s_columns=70 else s_columns=s_columns0 
  if (not keyword_set(s_timestep)) then s_timestep=0
  if (not keyword_set(s_nsteps0)) then s_nsteps=4 else s_nsteps=s_nsteps0
  if (not keyword_set(s_norder)) then s_norder=0
  if (not keyword_set(s_thick0)) then s_thick=!P.THICK else s_thick=s_thick0
  if (n_elements(s_color0) eq 0) then begin
    s_color=!P.COLOR
  endif else if (min(s_color0) lt 0) then begin
    s_color=!P.COLOR
  endif else begin
    s_color=s_color0
  endelse
  if (n_elements(s_arrowsize) eq 0) then s_arrowsize=-1.0
  plotfield_streamlines, xvec, yvec, xaxis=xax, yaxis=yax, /overplot, /noclose, $
    randomstart=s_randomstart, nrow=s_rows, ncolumn=s_columns, $
    timestep=s_timestep, nsteps=s_nsteps, norder=s_norder, thick=s_thick, $
    velcolor=s_color, arrowsize=s_arrowsize
endif
;
; --- Plot frame ---
if (not keyword_set(overplot)) then begin
  frameaspect=(xright0-xleft0)/(ytop0-ybot0)
  if (!X.TICKLEN eq 0.0) then xticklen=0.02 $
                         else xticklen=!X.TICKLEN
  if (!Y.TICKLEN eq 0.0) then yticklen=xticklen/frameaspect $
                         else yticklen=!Y.TICKLEN
  plot, [xaxmin,xaxmax], [yaxmin,yaxmax], $
    xrange=[xaxmin,xaxmax], yrange=[yaxmin,yaxmax], $
    position=[xleft0,ybot0,xright0,ytop0], /device, /nodata, $
    /noerase, xstyle=xstyle, ystyle=ystyle, $
    xtitle=xtitle, ytitle=ytitle, title=title, xthick=xthick, ythick=ythick, $
    xticklen=xticklen, yticklen=yticklen
endif
;
; --- Header --
if (keyword_set(head) and not keyword_set(overplot)) then begin
  for n=0,headnum-1 do begin
;    linestep=1.5*float(!D.Y_CH_SIZE)/float(!D.Y_SIZE)
;    xyouts, linestep, 0.98-linestep*(n+1), /norm, head(n)
;    linestep=1.8*float(!D.Y_CH_SIZE)/float(!D.Y_SIZE)*headcharsize
;    xyouts, linestep, 0.98-linestep*(n+1), /norm, head(n), charsize=headcharsize
    linestep=1.8*float(!D.Y_CH_SIZE)*headcharsize
    xyouts, 3*!D.X_CH_SIZE, !D.Y_SIZE-linestep*(n+0.8), /device, head(n), $
            charsize=headcharsize
  endfor
endif
;
; --- Legend ---
if (not keyword_set(legitems)) then begin
  items=strcompress(string(c_levels))
endif else begin
  items=replicate1(legitems,iy)
endelse
if (keyword_set(legend)) then begin
  legend, items, $
     linestyle=replicate1(c_linestyle,c_nlevels),color=replicate1(lc,c_nlevels)
endif
;
; --- Reset variables ---
if (strupcase(dev) ne 'X' and not keyword_set(noclose)) then begin
  device, /close 
  set_plot, 'X'
  modctv, 'X'
endif
;
end ; PLOTFIELD
