; pro QGREY_PLOT
;
;+
; NAME:
;   QGREY_PLOT
;
; PURPOSE:
;   Plot the function q_grey and grey temperature stratification.
;
; CALLING SEQUENCE:
;   @qgrey_plot
;
; SIDE EFFECTS:
;   A lot.
;
; MODIFICATION HSITORY:
;   22.2.95 Written by Bernd Freytag, Astrophysik, Kiel
;-
;
; --- Number of elements ---
jmax=120
;
; --- Make optical depth axis ---
tau=exp(findgen(jmax)/8.0-8.0)
;
exact='unsoeld'
;app='eddington'
;app='chandrasekhar2'
app='mihalas'
; --- Compute q_grey(tau) ---
qgreyuns=qgrey(tau,app=exact)
; --- Approximate q_grey by constant ---
qgreyedd=qgrey(tau,app=app)
;
; --- Compute T(tau)/T_eff ---
Tempuns=tempgreytau(tau,app=exact)
Tempedd=tempgreytau(tau,app=app)
;
; --- Select device ---
dev='x'
;
; --- Make device settings ---
if (strupcase(dev) eq 'X') then begin & $
  set_plot, 'X' & $
endif else begin & $
  set_plot, 'ps' & $
  device, filename='qgrey.ps', $
          /portrait , xsize=14.5, ysize=25.0,  xoffset=0.4, yoffset=0.4 &$
  help, /device & $
endelse
;
!X.MARGIN=[12,3]
!P.CHARSIZE=1.6
!X.THICK=2.0
!Y.THICK=2.0
!P.MULTI=[0,0,3,0,0]
!X.RANGE=[-6.75,3.75]
;
; --- Make plot # 1 ---
plot, alog10(tau), qgreyuns, $
  xtitle='log!D10!N(!4s!X)', xstyle=1,  $
  ytitle='q!Dgrey!N', ystyle=3, yrange=[0.56,0.72], $
  title=exact + ' - ' + app
oplot, alog10(tau), qgreyedd, linestyle=2
;oplot, alog10(tau0(1:*)), q0(1:*), psym=1
;
;xyouts, alog10(tau(jmax-1)), qgreyuns(jmax-1), $
;  ' q!Dgrey!N !96!X ' + string(qgreyuns(jmax-1),format='(F6.4)')
;xyouts, alog10(tau(jmax-1)), qgreyedd(jmax-1), $
;  ' q!Dgrey!N !9A!X ' + string(qgreyedd(jmax-1),format='(F6.4)')
;xyouts, alog10(tau(jmax-1)), qgreyuns(0), $
;  ' q!Dgrey!N !96!X ' + string(qgreyuns(0),format='(F6.4)')
xyouts, alog10(tau(0)), qgreyuns(jmax-1), ali=1, $
  ' q!Dgrey!N !96!X ' + string(qgreyuns(jmax-1),format='(F6.4)') + ' '
;xyouts, alog10(tau(0)), qgreyedd(jmax-1), ali=1, $
;  ' q!Dgrey!N !9A!X ' + string(qgreyedd(jmax-1),format='(F6.4)') + ' '
xyouts, alog10(tau(0)), qgreyuns(0), ali=1, $
  ' q!Dgrey!N !96!X ' + string(qgreyuns(0),format='(F6.4)') + ' '
;
; --- Make plot # 2 ---
plot, alog10(tau), Tempuns, $
  xtitle='log!D10!N(!4s!X)', xstyle=1, $
  ytitle='T/T!Deff!N', ystyle=3
oplot, alog10(tau), Tempedd, linestyle=2
;xyouts, alog10(tau(jmax-1)), Tempuns(0), $
;  ' T/T!Deff!N !96!X ' + string(Tempuns(0),format='(F6.4)')
xyouts, alog10(tau(0)), Tempuns(0), ali=1, $
  ' T/T!Deff!N !96!X ' + string(Tempuns(0),format='(F6.4)') + ' '
;
; --- Make plot # 3 ---
plot, alog10(tau), Tempedd-Tempuns, $
  xtitle='log!D10!N(!4s!X)', xstyle=1, $
  ytitle='(T-T!DUns'+!con.oes+'ld!N)/T!Deff!N', ystyle=3

;
; --- Close plot device ---
if (strupcase(dev) eq 'PS') then device, /close 
set_plot, 'X'
;
; --- Reset all user plotting variables ---
!X.MARGIN=[12,3]
!P.CHARSIZE=0
!X.THICK=0
!Y.THICK=0
!P.MULTI=0
!X.RANGE=0
;
; end ; QGREY_PLOT
