pro rhd_ring_data, ful, ring, i, $
      n_ring, ring_index, $
      n_tau, logtau, tau
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_ring_data ('radiation_hydrodynamics_loop')
;
; PURPOSE:
;   Collect data from full file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   rhd_ring_data, ful, ring, i, $
;     n_ring, ring_index, $
;     n_tau, logtau, tau
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-05-17 (B.F. Copenhagen) Written
;   2001-04-03 (B.F. Uppsala) rhd_plot -> rhd_data -> rhd_data
;   2014-05-20 (B.F.) Some updates, added ring.ei_mean, ring.rhovv_mean
;-
;------------------------------------------------------------------------------
;
nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
;
ring.itime[i]  =ful.z.itime
ring.time[ i]  =ful.z.time
ring.itime_last=ful.z.itime
;
;print, '    ', systime(0), $
;      total(ring.tau1[*,*,i] ge tau_limit), $
;      total(ring.tau2[*,*,i] ge tau_limit), $
;      total(ring.tau3[*,*,i] ge tau_limit), $
;      ring.n_ge_T_limit[i]
;
T =combox('T'     , ful)
;T4=temporary(T)^4
P =combox('P'     , ful)
x =combox('xc1a'  , ful)
r =combox('radius', ful)
;
taul=combox('tau1l', ful)
;Tl  =interbox(T        , taul, x_regular=tau, ori=1, boundary=1)
;T2l =interbox(T^2      , taul, x_regular=tau, ori=1, boundary=1)
T4l =interbox(T^4      , taul, x_regular=tau, ori=1, boundary=1)
Pl  =interbox(P        , taul, x_regular=tau, ori=1, boundary=1)
xl  =interbox(x        , taul, x_regular=tau, ori=1, boundary=1)
rl  =interbox(r        , taul, x_regular=tau, ori=1, boundary=1)
rhol=interbox(ful.z.rho, taul, x_regular=tau, ori=1, boundary=1)
eil =interbox(ful.z.ei , taul, x_regular=tau, ori=1, boundary=1)
vl  =interbox(ful.z.v1 , taul, x_regular=tau, ori=1, boundary=1)
taur=combox('tau1r', ful)
;Tr  =interbox(T        , taur, x_regular=tau, ori=1, boundary=1)
;T2r =interbox(T^2      , taur, x_regular=tau, ori=1, boundary=1)
T4r =interbox(T^4      , taur, x_regular=tau, ori=1, boundary=1)
Pr  =interbox(P        , taur, x_regular=tau, ori=1, boundary=1)
xr  =interbox(x        , taur, x_regular=tau, ori=1, boundary=1)
rr  =interbox(r        , taur, x_regular=tau, ori=1, boundary=1)
rhor=interbox(ful.z.rho, taur, x_regular=tau, ori=1, boundary=1)
eir =interbox(ful.z.ei , taur, x_regular=tau, ori=1, boundary=1)
vr  =interbox(ful.z.v1 , taur, x_regular=tau, ori=1, boundary=1)
for j=1,n_ring do begin
  ring_mask=rebin(reform(float(ring_index eq (j-1)), 1, nxc2, nxc3), n_tau, nxc2, nxc3)
  n_column=total(ring_index eq (j-1)) > 1.0
  ring.T_mean[    *, j-1, 0, i]=          total(total(ring_mask*sqrt(sqrt(T4l)),2),2)/n_column
  ring.T_mean[    *, j-1, 1, i]=          total(total(ring_mask*sqrt(sqrt(T4r)),2),2)/n_column
  ring.T_mean2[   *, j-1, 0, i]=     sqrt(total(total(ring_mask*     sqrt(T4l) ,2),2)/n_column)
  ring.T_mean2[   *, j-1, 1, i]=     sqrt(total(total(ring_mask*     sqrt(T4r) ,2),2)/n_column)
  ring.T_mean4[   *, j-1, 0, i]=sqrt(sqrt(total(total(ring_mask*          T4l  ,2),2)/n_column))
  ring.T_mean4[   *, j-1, 1, i]=sqrt(sqrt(total(total(ring_mask*          T4r  ,2),2)/n_column))
  ring.P_mean[    *, j-1, 0, i]=          total(total(ring_mask*          Pl   ,2),2)/n_column
  ring.P_mean[    *, j-1, 1, i]=          total(total(ring_mask*          Pr   ,2),2)/n_column
  ring.x_mean[    *, j-1, 0, i]=          total(total(ring_mask*          xl   ,2),2)/n_column
  ring.x_mean[    *, j-1, 1, i]=          total(total(ring_mask*          xr   ,2),2)/n_column
  ring.x_mean2[   *, j-1, 0, i]=     sqrt(total(total(ring_mask*          xl^2 ,2),2)/n_column)
  ring.x_mean2[   *, j-1, 1, i]=     sqrt(total(total(ring_mask*          xr^2 ,2),2)/n_column)
  ring.r_mean[    *, j-1, 0, i]=          total(total(ring_mask*          rl   ,2),2)/n_column
  ring.r_mean[    *, j-1, 1, i]=          total(total(ring_mask*          rr   ,2),2)/n_column
  ring.rho_mean[  *, j-1, 0, i]=          total(total(ring_mask*          rhol ,2),2)/n_column
  ring.rho_mean[  *, j-1, 1, i]=          total(total(ring_mask*          rhor ,2),2)/n_column
  ring.ei_mean[   *, j-1, 0, i]=          total(total(ring_mask*          eil  ,2),2)/n_column
  ring.ei_mean[   *, j-1, 1, i]=          total(total(ring_mask*          eir  ,2),2)/n_column
  ring.v_mean[    *, j-1, 0, i]=          total(total(ring_mask*          vl   ,2),2)/n_column
  ring.v_mean[    *, j-1, 1, i]=          total(total(ring_mask*          vr   ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 0, i]=          total(total(ring_mask*     rhol*vl^2 ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 1, i]=          total(total(ring_mask*     rhor*vr^2 ,2),2)/n_column
  ring.v_mean2[   *, j-1, 0, i]=     sqrt(total(total(ring_mask*          vl^2 ,2),2)/n_column)
  ring.v_mean2[   *, j-1, 1, i]=     sqrt(total(total(ring_mask*          vr^2 ,2),2)/n_column)
endfor
;
x =combox('xc2a', ful)
;
taul=combox('tau2l', ful)
;Tl  =transpose(interbox(T        , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
;T2l =transpose(interbox(T^2      , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
T4l =transpose(interbox(T^4      , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
Pl  =transpose(interbox(P        , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
xl  =transpose(interbox(x        , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
rl  =transpose(interbox(r        , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
rhol=transpose(interbox(ful.z.rho, taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
eil =transpose(interbox(ful.z.ei , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
vl  =transpose(interbox(ful.z.v2 , taul, x_regular=tau, ori=2, boundary=1), [1,0,2])
taur=combox('tau2r', ful)
;Tr  =transpose(interbox(T        , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
;T2r =transpose(interbox(T^2      , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
T4r =transpose(interbox(T^4      , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
Pr  =transpose(interbox(P        , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
xr  =transpose(interbox(x        , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
rr  =transpose(interbox(r        , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
rhor=transpose(interbox(ful.z.rho, taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
eir =transpose(interbox(ful.z.ei , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
vr  =transpose(interbox(ful.z.v2 , taur, x_regular=tau, ori=2, boundary=1), [1,0,2])
for j=1,n_ring do begin
  ring_mask=rebin(reform(float(ring_index eq (j-1)), 1, nxc2, nxc3), n_tau, nxc2, nxc3)
  n_column=total(ring_index eq (j-1)) > 1.0
  ring.T_mean[    *, j-1, 2, i]=          total(total(ring_mask*sqrt(sqrt(T4l)),2),2)/n_column
  ring.T_mean[    *, j-1, 3, i]=          total(total(ring_mask*sqrt(sqrt(T4r)),2),2)/n_column
  ring.T_mean2[   *, j-1, 2, i]=     sqrt(total(total(ring_mask*     sqrt(T4l) ,2),2)/n_column)
  ring.T_mean2[   *, j-1, 3, i]=     sqrt(total(total(ring_mask*     sqrt(T4r) ,2),2)/n_column)
  ring.T_mean4[   *, j-1, 2, i]=sqrt(sqrt(total(total(ring_mask*          T4l  ,2),2)/n_column))
  ring.T_mean4[   *, j-1, 3, i]=sqrt(sqrt(total(total(ring_mask*          T4r  ,2),2)/n_column))
  ring.P_mean[    *, j-1, 2, i]=          total(total(ring_mask*          Pl   ,2),2)/n_column
  ring.P_mean[    *, j-1, 3, i]=          total(total(ring_mask*          Pr   ,2),2)/n_column
  ring.x_mean[    *, j-1, 2, i]=          total(total(ring_mask*          xl   ,2),2)/n_column
  ring.x_mean[    *, j-1, 3, i]=          total(total(ring_mask*          xr   ,2),2)/n_column
  ring.x_mean2[   *, j-1, 2, i]=     sqrt(total(total(ring_mask*          xl^2 ,2),2)/n_column)
  ring.x_mean2[   *, j-1, 3, i]=     sqrt(total(total(ring_mask*          xr^2 ,2),2)/n_column)
  ring.r_mean[    *, j-1, 2, i]=          total(total(ring_mask*          rl   ,2),2)/n_column
  ring.r_mean[    *, j-1, 3, i]=          total(total(ring_mask*          rr   ,2),2)/n_column
  ring.rho_mean[  *, j-1, 2, i]=          total(total(ring_mask*          rhol ,2),2)/n_column
  ring.rho_mean[  *, j-1, 3, i]=          total(total(ring_mask*          rhor ,2),2)/n_column
  ring.ei_mean[   *, j-1, 2, i]=          total(total(ring_mask*          eil  ,2),2)/n_column
  ring.ei_mean[   *, j-1, 3, i]=          total(total(ring_mask*          eir  ,2),2)/n_column
  ring.v_mean[    *, j-1, 2, i]=          total(total(ring_mask*          vl   ,2),2)/n_column
  ring.v_mean[    *, j-1, 3, i]=          total(total(ring_mask*          vr   ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 2, i]=          total(total(ring_mask*     rhol*vl^2 ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 3, i]=          total(total(ring_mask*     rhor*vr^2 ,2),2)/n_column
  ring.v_mean2[   *, j-1, 2, i]=     sqrt(total(total(ring_mask*          vl^2 ,2),2)/n_column)
  ring.v_mean2[   *, j-1, 3, i]=     sqrt(total(total(ring_mask*          vr^2 ,2),2)/n_column)
endfor
;
x =combox('xc3a', ful)
;
taul=combox('tau3l', ful)
;Tl  =transpose(interbox(T        , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
;T2l =transpose(interbox(T^2      , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
T4l =transpose(interbox(T^4      , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
Pl  =transpose(interbox(P        , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
xl  =transpose(interbox(x        , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
rl  =transpose(interbox(r        , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
rhol=transpose(interbox(ful.z.rho, taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
eil =transpose(interbox(ful.z.ei , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
vl  =transpose(interbox(ful.z.v3 , taul, x_regular=tau, ori=3, boundary=1), [2,1,0])
taur=combox('tau3r', ful)
;Tr  =transpose(interbox(T        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
;T2r =transpose(interbox(T^2      , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
T4r =transpose(interbox(T^4      , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
Pr  =transpose(interbox(P        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
xr  =transpose(interbox(x        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
rr  =transpose(interbox(r        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
rhor=transpose(interbox(ful.z.rho, taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
eir =transpose(interbox(ful.z.ei , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
vr  =transpose(interbox(ful.z.v3 , taur, x_regular=tau, ori=3, boundary=1), [2,1,0])
for j=1,n_ring do begin
  ring_mask=rebin(reform(float(ring_index eq (j-1)), 1, nxc2, nxc3), n_tau, nxc2, nxc3)
  n_column=total(ring_index eq (j-1)) > 1.0
  ring.T_mean[    *, j-1, 4, i]=          total(total(ring_mask*sqrt(sqrt(T4l)),2),2)/n_column
  ring.T_mean[    *, j-1, 5, i]=          total(total(ring_mask*sqrt(sqrt(T4r)),2),2)/n_column
  ring.T_mean2[   *, j-1, 4, i]=     sqrt(total(total(ring_mask*     sqrt(T4l) ,2),2)/n_column)
  ring.T_mean2[   *, j-1, 5, i]=     sqrt(total(total(ring_mask*     sqrt(T4r) ,2),2)/n_column)
  ring.T_mean4[   *, j-1, 4, i]=sqrt(sqrt(total(total(ring_mask*          T4l  ,2),2)/n_column))
  ring.T_mean4[   *, j-1, 5, i]=sqrt(sqrt(total(total(ring_mask*          T4r  ,2),2)/n_column))
  ring.P_mean[    *, j-1, 4, i]=          total(total(ring_mask*          Pl   ,2),2)/n_column
  ring.P_mean[    *, j-1, 5, i]=          total(total(ring_mask*          Pr   ,2),2)/n_column
  ring.x_mean[    *, j-1, 4, i]=          total(total(ring_mask*          xl   ,2),2)/n_column
  ring.x_mean[    *, j-1, 5, i]=          total(total(ring_mask*          xr   ,2),2)/n_column
  ring.x_mean2[   *, j-1, 4, i]=     sqrt(total(total(ring_mask*          xl^2 ,2),2)/n_column)
  ring.x_mean2[   *, j-1, 5, i]=     sqrt(total(total(ring_mask*          xr^2 ,2),2)/n_column)
  ring.r_mean[    *, j-1, 4, i]=          total(total(ring_mask*          rl   ,2),2)/n_column
  ring.r_mean[    *, j-1, 5, i]=          total(total(ring_mask*          rr   ,2),2)/n_column
  ring.rho_mean[  *, j-1, 4, i]=          total(total(ring_mask*          rhol ,2),2)/n_column
  ring.rho_mean[  *, j-1, 5, i]=          total(total(ring_mask*          rhor ,2),2)/n_column
  ring.ei_mean[   *, j-1, 4, i]=          total(total(ring_mask*          eil  ,2),2)/n_column
  ring.ei_mean[   *, j-1, 5, i]=          total(total(ring_mask*          eir  ,2),2)/n_column
  ring.v_mean[    *, j-1, 4, i]=          total(total(ring_mask*          vl   ,2),2)/n_column
  ring.v_mean[    *, j-1, 5, i]=          total(total(ring_mask*          vr   ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 4, i]=          total(total(ring_mask*     rhol*vl^2 ,2),2)/n_column
  ring.rhovv_mean[*, j-1, 5, i]=          total(total(ring_mask*     rhor*vr^2 ,2),2)/n_column
  ring.v_mean2[   *, j-1, 4, i]=     sqrt(total(total(ring_mask*          vl^2 ,2),2)/n_column)
  ring.v_mean2[   *, j-1, 5, i]=     sqrt(total(total(ring_mask*          vr^2 ,2),2)/n_column)
endfor
;
end ; rhd_ring_data
