pro rhd_seq_put2struct, ful, seq, par=par, $
  InitStruct=InitStruct, modelname=modelname, modelident=modelident, n_timestep=n_timestep, $
  snapshot=snapshot, file=file, i_timestep=i_timestep, date=date, version=version, $
  ReadMetaData=ReadMetaData
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq_put2struct ('rhd_sequence_put_to_structure')
;
; PURPOSE:
;   Collect data from CO5BOLD mean files.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   rhd_seq_put2struct
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   combox
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-05-17 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   2001-04-03 (B.F. Uppsala) rhd_plot -> rhd_collect
;   2003-10-10 (B.F.) rhd_collect -> rhd_seq_collect
;   2005-06-15 (B.F. LA) Adaption from rhd_seq_collect
;   2005-06-16 (B.F.) Changes
;   2005-06-20 (B.F.) Put into separate subroutine
;   2006-05-03 (B.F.) (i) -> [i]
;   2006-05-04 (B.F.) v2 statistics added
;   2006-05-12 (B.F.) quc's added
;   2008-02-26 (B.F.) rage_stat_put2struct.pro -> rhd_seq_put2struct.pro
;   2008-03-06 (B.F. Lyon) Trms, Prms, rhorms, srms added
;   2008-09-22 (B.F.) Template for data structure first created as string, saves memory
;   2008-11-06 (B.F.) (i)time_first,last added
;   2009-07-17 (B.F.) v2 entries added
;   2009-10-10 (B.F.) Keyword date added; tags JulianDate, Version added
;   2009-10-13 (B.F.) Keyword ReadMetaData added, tags reordered, tags rhototalX removed
;   2009-10-16 (B.F.) Create temporary structure for qucs
;   2009-11-10 (B.F.) Initialize structure also for central potential
;   2010-03-22 (B.F. Naples) Magnetic field data
;   2010-12-03 (B.F. Lyon) Split long execute string into three, because of IDL 8
;   2012-12-10 (B.F.) ful.z.(i)time -> ful.z3.(i)time
;   2013-02-20 (B.F.) Append '_XMEAN' in check for existence of qucs
;   2013-07-08 (B.F.) Moment of inertia and angular momentum
;   2015-01-13 (B.F. Uppsala) few changes
;   2016-10-26 (B.F.) Add rhov3v3
;   2016-11-01 (B.F.) Add rhov1v3, rhov2v3, frhov1, frhov2, frhov3
;   2016-11-07 (B.F.) Add rhov1v1, rhov1v2, rhov2v2
;   2018-06-24 (B.F. Kiel) Add spherical wave amplitudes rhovbl???m????
;   2018-06-25 (B.F.) Add rhotil, rhovbrms
;-
;------------------------------------------------------------------------------

compile_opt defint32

; --- Array sizes ---
;nxc1 =n_elements(ful.z.xc1)
;nxc2 =n_elements(ful.z.xc2)
;nxc3 =n_elements(ful.z.xc3)
;x1tot=ful.z.xb1[nxc1]-ful.z.xb1[0]
nr=n_elements(ful.rad.xc3)
nxc1=n_elements(ful.rad.xc1)
nxc2=n_elements(ful.rad.xc2)
nxc3=n_elements(ful.rad.xc3)

;ind=where(strmid(tag_names(ful.z), 0, 3) eq 'QUC', nquc)
;ind=where(strmatch(tag_names(ful.z), 'QUC[0-9][0-9][0-9]'), nquc)
;ind=where(strmatch(tag_names(ful.z), 'FQUC[0-9][0-9][0-9]'), nfquc)
ind=where(strmatch(tag_names(ful.z3), 'QUC[0-9][0-9][0-9]_XMEAN'), nquc)
ind=where(strmatch(tag_names(ful.z3), 'FQUC[0-9][0-9][0-9]_XMEAN'), nfquc)
ind=where(strmatch(tag_names(ful.z3), 'BC3_XMEAN'), nB) & nB=nB < 1
isCentral_flag=(strmatch(par.grav_mode, 'central' ,/fold))

if (isCentral_flag) then begin
  ind=where(strmatch(tag_names(ful.r), 'RHOVBL[0-9][0-9][0-9]M?[0-9][0-9][0-9]_XMEAN'), nwave)
endif else begin
  nwave=0
endelse

if (keyword_set(InitStruct)) then begin
  ; === Create structure that will hold all the averaged data ===
  ;
  ;print, 'rhd_seq_put2struct: nquc=', nquc
  ;help, nr, nxc1, nxc2, nxc3
  ;
  ;print, 'Model ' + modelname + '  with ', $
  ; nxc1, ' x', nxc2, ' x', nxc3, ' points  in ', n_timestep, ' files', $
  ; format='(A,I6,A,I6,A,I6,A,I6,A)'
  ;
  ; --- Create the structure ---
  if (keyword_set(ReadMetaData)) then begin
    ; --- Create structure including metadata ---
    nstr='__NAME'
    ustr='__UNIT'
    ;
    if (isCentral_flag) then begin
      tags=tag_names(ful.r)
    endif else begin
      tags=tag_names(ful.z3)
    endelse
    ;
    seqstr="seq=create_struct(" + $
      "'model'               , modelname," + $
      "'model"      +nstr+ "', 'Name of model'," + $
      "'modelident'          , modelident," + $
      "'modelident" +nstr+ "', 'Placeholder for model sequence'," + $
      "'mean'                , ful," + $
      "'mean"       +nstr+ "', 'Complete mean structure'," + $
      "'par'                 , par," + $
      "'par"        +nstr+ "', 'Parameter file structure'," + $
      "'xc1'                 , float(reform(ful.rad.xc1))," + $
      "'xc1"        +nstr+ "', ful.rad.xc1" +nstr+ "," + $
      "'xc1"        +ustr+ "', ful.rad.xc1" +ustr+ "," + $
      "'xc2'                 , float(reform(ful.rad.xc2))," + $
      "'xc2"        +nstr+ "', ful.rad.xc2" +nstr+ "," + $
      "'xc2"        +ustr+ "', ful.rad.xc2" +ustr+ "," + $
      "'xc3'                 , float(reform(ful.rad.xc3))," + $
      "'xc3"        +nstr+ "', ful.rad.xc3" +nstr+ "," + $
      "'xc3"        +ustr+ "', ful.rad.xc3" +ustr+ "," + $
      "'xb1'                 , float(reform(ful.rad.xb1))," + $
      "'xb1"        +nstr+ "', ful.rad.xb1" +nstr+ "," + $
      "'xb1"        +ustr+ "', ful.rad.xb1" +ustr+ "," + $
      "'xb2'                 , float(reform(ful.rad.xb2))," + $
      "'xb2"        +nstr+ "', ful.rad.xb2" +nstr+ "," + $
      "'xb2"        +ustr+ "', ful.rad.xb2" +ustr+ "," + $
      "'xb3'                 , float(reform(ful.rad.xb3))," + $
      "'xb3"        +nstr+ "', ful.rad.xb3" +nstr+ "," + $
      "'xb3"        +ustr+ "', ful.rad.xb3" +ustr+ "," + $
      "'binfactor'           , 1L," + $
      "'binfactor"  +nstr+ "', 'Binning factor'," + $
      "'itime'               , lonarr(     n_timestep)," + $
      "'itime"      +nstr+ "', ful.z3.itime" +nstr+ "," + $
      "'itime_first'         ,lonarr(     n_timestep)," + $
      "'itime_first"+nstr+ "', 'First time step number of bin'," + $
      "'itime_last'          , lonarr(     n_timestep)," + $
      "'itime_last" +nstr+ "', 'Last time step number of bin'," + $
      "'time'                , fltarr(     n_timestep)," + $
      "'time"       +nstr+ "', ful.z3.time" +nstr+ "," + $
      "'time"       +ustr+ "', ful.z3.time" +ustr+ "," + $
      "'time_first'          , fltarr(     n_timestep)," + $
      "'time_first" +nstr+ "', 'First time value of bin'," + $
      "'time_first" +ustr+ "', ful.z3.time" +ustr+ "," + $
      "'time_last'           , fltarr(     n_timestep)," + $
      "'time_last"  +nstr+ "', 'Last time value of bin'," + $
      "'time_last"  +ustr+ "', ful.z3.time" +ustr+ "," + $
      "'JulianDate'          , dblarr(     n_timestep)," + $
      "'JulianDate" +nstr+ "', 'Julian date'," + $
      "'JulianDate" +ustr+ "', 'd'," + $
      "'ifile'               , lonarr(     n_timestep)," + $
      "'ifile"      +nstr+ "', 'mean file number'," + $
      "'file'                , strarr(     n_timestep)," + $
      "'file"       +nstr+ "', 'mean file name'," + $
      "'version'             , strarr(     n_timestep)," + $
      "'s_inflow'            , fltarr(     n_timestep)," + $
      "'s_inflow"   +nstr+ "', par.s_inflow" +nstr+ "," + $
      "'s_inflow"   +ustr+ "', par.s_inflow" +ustr+ "," + $
      "'c_schange'           , fltarr(     n_timestep)," + $ 
      "'c_schange"  +nstr+ "', par.c_schange" +nstr+ "," + $
      "'c_schange"  +ustr+ "', par.c_schange" +ustr+ "," + $
      "'c_hptopfactor'       , fltarr(     n_timestep)," + $ 
      "'c_hptopfactor"+nstr+ "', par.c_hptopfactor" +nstr+ "," + $
      "'c_hptopfactor"+ustr+ "', par.c_hptopfactor" +ustr+ "," + $
      "'I3r'                 , fltarr(     n_timestep)," + $
      "'I3r"        +nstr+ "', ful.rad.Intb3_r" +nstr+ "," + $
      "'I3r"        +ustr+ "', ful.rad.Intb3_r" +ustr
    seqstr=seqstr + ")"
    ;print, seqstr
    ;
    dummy=execute(seqstr)
    ;
    seq2str="seq2=create_struct(" + $
      "'v1'                  , fltarr(nr  ,n_timestep)," + $
      "'v1"         +nstr+ ((isCentral_flag) ? "', 'Tangential velocity'," $
                                             : "', ful.z3.v1_xmean" +nstr+ ",") + $
      "'v1"         +ustr+ "', ful.z3.v1_xmean" +ustr+ "," + $
      "'v1drms'              , fltarr(nr  ,n_timestep)," + $
      "'v1drms"     +nstr+ ((isCentral_flag) ? "', 'root-mean-square of tangential velocity fluctuations'," $
                                             : "', 'root-mean-square of horizontal velocity fluctuations 1',") + $
      "'v1drms"     +ustr+ "', ful.z3.v1_xmean" +ustr+ "," + $
      "'v1rms'               , fltarr(nr  ,n_timestep)," + $
      "'v1rms"      +nstr+ ((isCentral_flag) ? "', 'root-mean-square of tangential velocity'," $
                                             : "', 'root-mean-square of horizontal velocity 1',") + $
      "'v1rms"      +ustr+ "', ful.z3.v1_xmean" +ustr+ "," + $
      "'rhov1'               , fltarr(nr  ,n_timestep)," + $
      "'rhov1"      +nstr+ "', ful.z3.rhov1_xmean" +nstr+ "," + $
      "'rhov1"      +ustr+ "', ful.z3.rhov1_xmean" +ustr+ "," + $
      "'rhov1v1'             , fltarr(nr  ,n_timestep)," + $
      "'rhov1v1"    +nstr+ ((total(tags eq 'RHOV1V1_XMEAN') gt 0.0) ? "', ful.z3.rhov1v1_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', ''," $
                                                                                        : "', 'momentum x1 flux x1 direction',")) + $
      "'rhov1v1"    +ustr+ ((total(tags eq 'RHOV1V1_XMEAN') gt 0.0) ? "', ful.z3.rhov1v1_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'rhov1v2'             , fltarr(nr  ,n_timestep)," + $
      "'rhov1v2"    +nstr+ ((total(tags eq 'RHOV1V2_XMEAN') gt 0.0) ? "', ful.z3.rhov1v2_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', ''," $
                                                                                        : "', 'momentum x1 flux x2 direction',")) + $
      "'rhov1v2"    +ustr+ ((total(tags eq 'RHOV1V2_XMEAN') gt 0.0) ? "', ful.z3.rhov1v2_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'rhov1v3'             , fltarr(nr  ,n_timestep)," + $
      "'rhov1v3"    +nstr+ ((total(tags eq 'RHOV1V3_XMEAN') gt 0.0) ? "', ful.z3.rhov1v3_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', ''," $
                                                                                        : "', 'momentum x1 flux x3 direction',")) + $
      "'rhov1v3"    +ustr+ ((total(tags eq 'RHOV1V3_XMEAN') gt 0.0) ? "', ful.z3.rhov1v3_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'v2'                  , fltarr(nr  ,n_timestep)," + $
      "'v2"         +nstr+ "', ful.z3.v2_xmean" +nstr+ "," + $
      "'v2"         +ustr+ "', ful.z3.v2_xmean" +ustr+ "," + $
      "'v2drms'              , fltarr(nr  ,n_timestep)," + $
      "'v2drms"     +nstr+ "', 'root-mean-square of horizontal velocity fluctuations 2'," + $
      "'v2drms"     +ustr+ "', ful.z3.v2_xmean" +ustr+ "," + $
      "'v2rms'               , fltarr(nr  ,n_timestep)," + $
      "'v2rms"      +nstr+ "', 'root-mean-square of horizontal velocity 2'," + $
      "'v2rms"      +ustr+ "', ful.z3.v2_xmean" +ustr+ "," + $
      "'rhov2'               , fltarr(nr  ,n_timestep)," + $
      "'rhov2"      +nstr+ "', ful.z3.rhov2_xmean" +nstr+ "," + $
      "'rhov2"      +ustr+ "', ful.z3.rhov2_xmean" +ustr+ "," + $
      "'rhov2v2'             , fltarr(nr  ,n_timestep)," + $
      "'rhov2v2"    +nstr+ ((total(tags eq 'RHOV2V2_XMEAN') gt 0.0) ? "', ful.z3.rhov2v2_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', ''," $
                                                                                        : "', 'momentum x2 flux x2 direction',")) + $
      "'rhov2v2"    +ustr+ ((total(tags eq 'RHOV2V2_XMEAN') gt 0.0) ? "', ful.z3.rhov2v2_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'rhov2v3'             , fltarr(nr  ,n_timestep)," + $
      "'rhov2v3"    +nstr+ ((total(tags eq 'RHOV2V3_XMEAN') gt 0.0) ? "', ful.z3.rhov2v3_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', ''," $
                                                                                        : "', 'momentum x2 flux x3 direction',")) + $
      "'rhov2v3"    +ustr+ ((total(tags eq 'RHOV2V3_XMEAN') gt 0.0) ? "', ful.z3.rhov2v3_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'v3'                  , fltarr(nr  ,n_timestep)," + $
      "'v3"         +nstr+ ((isCentral_flag) ? "', 'Radial velocity'," $
                                             : "', ful.z3.v3_xmean" +nstr+ ",") + $
      "'v3"         +ustr+ "', ful.z3.v3_xmean" +ustr+ "," + $
      "'v3drms'              , fltarr(nr  ,n_timestep)," + $
      "'v3drms"     +nstr+ ((isCentral_flag) ? "', 'root-mean-square of radial velocity fluctuations'," $
                                             : "', 'root-mean-square of vertical velocity fluctuations',") + $
      "'v3drms"     +ustr+ "', ful.z3.v3_xmean" +ustr+ "," + $
      "'v3rms'               , fltarr(nr  ,n_timestep)," + $
;      "'v3rms"      +nstr+ "', ful.z3.v3_xmean2" +nstr+ "," + $
;      "'v3rms"      +ustr+ "', ful.z3.v3_xmean2" +ustr+ "," + $
      "'v3rms"      +nstr+ ((isCentral_flag) ? "', 'root-mean-square of radial velocity'," $
                                             : "', 'root-mean-square of vertical velocity',") + $
      "'v3rms"      +ustr+ "', ful.z3.v3_xmean" +ustr+ "," + $
      "'rhov3'               , fltarr(nr  ,n_timestep)," + $
      "'rhov3"      +nstr+ "', ful.z3.rhov3_xmean" +nstr+ "," + $
      "'rhov3"      +ustr+ "', ful.z3.rhov3_xmean" +ustr+ "," + $

      "'rhov3v3'             , fltarr(nr  ,n_timestep)," + $
      "'rhov3v3"    +nstr+ ((total(tags eq 'RHOV3V3_XMEAN') gt 0.0) ? "', ful.z3.rhov3v3_xmean" +nstr+ "," $
                                                                    : ((isCentral_flag) ? "', 'radial turbulent pressure'," $
                                                                                        : "', 'vertical turbulent pressure',")) + $
      "'rhov3v3"    +ustr+ ((total(tags eq 'RHOV3V3_XMEAN') gt 0.0) ? "', ful.z3.rhov3v3_xmean" +ustr+ "," $
                                                                    : "', ful.z3.rhoek_xmean" +ustr+ ",") + $

      "'rho'                 , fltarr(nr  ,n_timestep)," + $
      "'rho"        +nstr+ "', ful.z3.rho_xmean" +nstr+ "," + $
      "'rho"        +ustr+ "', ful.z3.rho_xmean" +ustr+ "," + $
      "'rhorms'              , fltarr(nr  ,n_timestep)," + $
      "'rhorms"     +nstr+ "', 'root-mean-square of density'," + $
      "'rhorms"     +ustr+ "', ful.z3.rho_xmean" +ustr+ "," + $
      "'ei'                  , fltarr(nr  ,n_timestep)," + $
      "'ei"         +nstr+ "', ful.z3.ei_xmean" +nstr+ "," + $
      "'ei"         +ustr+ "', ful.z3.ei_xmean" +ustr+ "," + $
      "'rhoei'               , fltarr(nr  ,n_timestep)," + $
      "'rhoei"      +nstr+ "', ful.z3.rhoei_xmean" +nstr+ "," + $
      "'rhoei"      +ustr+ "', ful.z3.rhoei_xmean" +ustr+ "," + $
      "'rhoek'               , fltarr(nr  ,n_timestep)," + $
      "'rhoek"      +nstr+ "', ful.z3.rhoek_xmean" +nstr+ "," + $
      "'rhoek"      +ustr+ "', ful.z3.rhoek_xmean" +ustr+ "," + $
      "'rhoeg'               , fltarr(nr  ,n_timestep)," + $
      "'rhoeg"      +nstr+ "', ful.z3.rhoeg_xmean" +nstr+ "," + $
      "'rhoeg"      +ustr+ "', ful.z3.rhoeg_xmean" +ustr
    seq2str=seq2str + ")"
    ;print, seq2str
    ;
    dummy=execute(seq2str)
    seq=create_struct(seq, temporary(seq2))
    ;
    seq3str="seq3=create_struct(" + $
      "'T'                   , fltarr(nr  ,n_timestep)," + $
      "'T"          +nstr+ "', ful.z3.T_xmean" +nstr+ "," + $
      "'T"          +ustr+ "', ful.z3.T_xmean" +ustr+ "," + $
      "'Tdrms'               , fltarr(nr  ,n_timestep)," + $
      "'Tdrms"      +nstr+ "', 'root-mean-square of temperature fluctuations'," + $
      "'Tdrms"      +ustr+ "', ful.z3.T_xmean" +ustr+ "," + $
      "'Trms'                , fltarr(nr  ,n_timestep)," + $
      "'Trms"       +nstr+ "', 'root-mean-square of temperature'," + $
      "'Trms"       +ustr+ "', ful.z3.T_xmean" +ustr+ "," + $
      "'T_xmean4'            , fltarr(nr  ,n_timestep)," + $
      "'T_xmean4"   +nstr+ "', '4th root of mean of 4th power of temperature'," + $
      "'T_xmean4"   +ustr+ "', ful.z3.T_xmean" +ustr+ "," + $
      "'T_xmeankapparho'          , fltarr(nr  ,n_timestep)," + $
      "'T_xmeankapparho" +nstr+ "', 'opacity average of temperature'," + $
      "'T_xmeankapparho" +ustr+ "', ful.z3.T_xmean" +ustr+ "," + $
      "'P'                   , fltarr(nr  ,n_timestep)," + $
      "'P"          +nstr+ "', ful.z3.P_xmean" +nstr+ "," + $
      "'P"          +ustr+ "', ful.z3.P_xmean" +ustr+ "," + $
      "'Prms'                , fltarr(nr  ,n_timestep)," + $
      "'Prms"       +nstr+ "', 'root-mean-square of pressure'," + $
      "'Prms"       +ustr+ "', ful.z3.P_xmean" +ustr+ "," + $
      "'s'                   , fltarr(nr  ,n_timestep)," + $
      "'s"          +nstr+ "', ful.z3.s_xmean" +nstr+ "," + $
      "'s"          +ustr+ "', ful.z3.s_xmean" +ustr+ "," + $
      "'srms'                , fltarr(nr  ,n_timestep)," + $
      "'srms"       +nstr+ "', 'root-mean-square of entropy'," + $
      "'srms"       +ustr+ "', ful.z3.s_xmean" +ustr+ "," + $
      "'rhos'                , fltarr(nr  ,n_timestep)," + $
      "'rhos"       +nstr+ "', ful.z3.rhos_xmean" +nstr+ "," + $
      "'rhos"       +ustr+ "', ful.z3.rhos_xmean" +ustr+ "," + $
      "'gamma1'              , fltarr(nr  ,n_timestep)," + $
      "'gamma1"     +nstr+ "', ful.z3.gamma1_xmean" +nstr+ "," + $
      "'gamma1"     +ustr+ "', ful.z3.gamma1_xmean" +ustr+ "," + $
      "'gamma3'              , fltarr(nr  ,n_timestep)," + $
      "'gamma3"     +nstr+ "', ful.z3.gamma3_xmean" +nstr+ "," + $
      "'gamma3"     +ustr+ "', ful.z3.gamma3_xmean" +ustr+ "," + $
      "'delta'               , fltarr(nr  ,n_timestep)," + $
      "'delta"      +nstr+ "', ful.z3.delta_xmean" +nstr+ "," + $
      "'delta"      +ustr+ "', ful.z3.delta_xmean" +ustr+ "," + $
      "'kapparho'            , fltarr(nr  ,n_timestep)," + $
      "'kapparho"   +nstr+ "', ful.z3.kapparho_xmean" +nstr+ "," + $
      "'kapparho"   +ustr+ "', ful.z3.kapparho_xmean" +ustr+ "," + $
      "'gravrad'             , fltarr(nr  ,n_timestep)," + $
      "'gravrad"    +nstr+ "', ful.z3.gravrad_xmean" +nstr+ "," + $
      "'gravrad"    +ustr+ "', ful.z3.gravrad_xmean" +ustr+ "," + $

      "'rhotil'              , fltarr(nr+1,n_timestep)," + $
      "'rhotil"     +nstr+ ((total(tags eq 'RHOTIL_XMEAN') gt 0.0) ? "', ful.z3.rhotil_xmean" +nstr+ "," $
                                                                   : "', ful.z3.rho_xmean"    +nstr+ ",") + $
      "'rhotil"     +ustr+ ((total(tags eq 'RHOTIL_XMEAN') gt 0.0) ? "', ful.z3.rhotil_xmean" +ustr+ "," $
                                                                   : "', ful.z3.rho_xmean"    +ustr+ ",") + $

      "'rhovb'               , fltarr(nr+1,n_timestep)," + $
      "'rhovb"      +nstr+ "', ful.z3.rhovb_xmean" +nstr+ "," + $
      "'rhovb"      +ustr+ "', ful.z3.rhovb_xmean" +ustr+ "," + $

      "'rhovbrms'            , fltarr(nr+1,n_timestep)," + $
      "'rhovbrms"   +nstr+ ((total(tags eq 'RHOVB_XMEAN2') gt 0.0) ? "', ful.z3.rhovb_xmean2" +nstr+ "," $
                                                                   : "', ful.z3.rhovb_xmean"  +nstr+ ",") + $
      "'rhovbrms"   +ustr+ ((total(tags eq 'RHOVB_XMEAN2') gt 0.0) ? "', ful.z3.rhovb_xmean2" +ustr+ "," $
                                                                   : "', ful.z3.rhovb_xmean"  +ustr+ ",") + $

      "'frhov1b'             , fltarr(nr+1,n_timestep)," + $
      "'frhov1b"    +nstr+ ((total(tags eq 'FRHOV13B_XMEAN') gt 0.0) ? "', ful.z3.frhov13b_xmean" +nstr+ "," $
                                                                     : ((isCentral_flag) ? "', 'Flux of momentum x1 flux in x3 direction'," $
                                                                                         : "', '',")) + $
      "'frhov1b"    +ustr+ ((total(tags eq 'FRHOV13B_XMEAN') gt 0.0) ? "', ful.z3.frhov13b_xmean" +ustr+ "," $
                                                                     : "', ful.z3.rhoek_xmean" +ustr+ ",") + $

      "'frhov2b'             , fltarr(nr+1,n_timestep)," + $
      "'frhov2b"    +nstr+ ((total(tags eq 'FRHOV23B_XMEAN') gt 0.0) ? "', ful.z3.frhov23b_xmean" +nstr+ "," $
                                                                     : ((isCentral_flag) ? "', 'Flux of momentum x2 flux in x3 direction'," $
                                                                                         : "', '',")) + $
      "'frhov2b"    +ustr+ ((total(tags eq 'FRHOV23B_XMEAN') gt 0.0) ? "', ful.z3.frhov23b_xmean" +ustr+ "," $
                                                                     : "', ful.z3.rhoek_xmean" +ustr+ ",") + $

      "'frhov3b'             , fltarr(nr+1,n_timestep)," + $
      "'frhov3b"    +nstr+ ((total(tags eq 'FRHOV33B_XMEAN') gt 0.0) ? "', ful.z3.frhov33b_xmean" +nstr+ "," $
                                                                     : ((isCentral_flag) ? "', 'Flux of momentum x3 flux in x3 direction'," $
                                                                                         : "', '',")) + $
      "'frhov3b"    +ustr+ ((total(tags eq 'FRHOV33B_XMEAN') gt 0.0) ? "', ful.z3.frhov33b_xmean" +ustr+ "," $
                                                                     : "', ful.z3.rhoek_xmean" +ustr+ ",") + $
      "'feipb'               , fltarr(nr+1,n_timestep)," + $
      "'feipb"      +nstr+ "', ful.z3.feipb_xmean" +nstr+ "," + $
      "'feipb"      +ustr+ "', ful.z3.feipb_xmean" +ustr+ "," + $

      "'fekb'                , fltarr(nr+1,n_timestep)," + $
      "'fekb"       +nstr+ "', ful.z3.fekb_xmean" +nstr+ "," + $
      "'fekb"       +ustr+ "', ful.z3.fekb_xmean" +ustr+ "," + $
      "'fegb'      , fltarr(nr+1,n_timestep)," + $
      "'fegb"       +nstr+ "', ful.z3.fegb_xmean" +nstr+ "," + $
      "'fegb"       +ustr+ "', ful.z3.fegb_xmean" +ustr+ "," + $
      "'fepb'      , fltarr(nr+1,n_timestep)," + $
      "'fepb"       +nstr+ "', ful.z3.fepb_xmean" +nstr+ "," + $
      "'fepb"       +ustr+ "', ful.z3.fepb_xmean" +ustr+ "," + $
      "'fevb'      , fltarr(nr+1,n_timestep)," + $
      "'fevb"       +nstr+ "', ful.z3.fevb_xmean" +nstr+ "," + $
      "'fevb"       +ustr+ "', ful.z3.fevb_xmean" +ustr+ "," + $
      "'fewb'      , fltarr(nr+1,n_timestep)," + $
      "'fewb"       +nstr+ "', ful.z3.fevp2pb_xmean" +nstr+ "," + $
      "'fewb"       +ustr+ "', ful.z3.fevp2pb_xmean" +ustr+ "," + $
      "'ferb'      , fltarr(nr+1,n_timestep)," + $
      "'ferb"       +nstr+ "', ful.z3.ferb_xmean" +nstr+ "," + $
      "'ferb"       +ustr+ "', ful.z3.ferb_xmean" +ustr
    seq3str=seq3str + ")"
    ;print, seq3str
    ;
    dummy=execute(seq3str)
    seq=create_struct(seq, temporary(seq3))
    ;
    if (nquc gt 0) then begin
      ; --- Create separate structure for additional density fields and append it ---
      seqqstr="seqq=create_struct("
      for iquc=0,nquc-1 do begin
        qucid ='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
        qucmid=qucid + '_xmean'
        print, 'Add quantity ' + qucid + ' to structure.'
        if (iquc ge 1) then seqqstr=seqqstr + ","
        seqqstr=seqqstr +  "'" + qucid      + "', fltarr(nxc3,n_timestep)" + $
                          ",'" + qucid +nstr+ "', ful.z3." + qucmid +nstr + $
                          ",'" + qucid +ustr+ "', ful.z3." + qucmid +ustr
      endfor
      ;
      for iquc=0,nfquc-1 do begin
        fqucid ='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
        fqucmid=fqucid + '_xmean'
        ;print, 'Add quantity ' + fqucid + ' to structure.'
        seqqstr=seqqstr + ",'" + fqucid + "'," + "fltarr(nxc3+1,n_timestep)" + $
                          ",'" + fqucid +nstr+ "', ful.z3." + fqucmid +nstr + $
                          ",'" + fqucid +ustr+ "', ful.z3." + fqucmid +ustr
      endfor
      ;
      seqqstr=seqqstr + ")"
      ;print, seqqstr
      ;
      dummy=execute(seqqstr)
      seq=create_struct(seq, temporary(seqq))
      ;
    endif ; (nquc)
    ;
    if (nB gt 0) then begin
      ; --- Create separate structure for additional magnetic field data and append it ---
      seqqstr="seqq=create_struct("
      ;
      seqqstr=seqqstr +  "'" + 'bc1' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc1' +nstr+ "', ful.z3." + 'bc1_xmean' +nstr + $
                        ",'" + 'bc1' +ustr+ "', ful.z3." + 'bc1_xmean' +ustr
      seqqstr=seqqstr + ",'" + 'bc1rms' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc1rms' +nstr+ "', ful.z3." + 'bc1_xmean2' +nstr + $
                        ",'" + 'bc1rms' +ustr+ "', ful.z3." + 'bc1_xmean2' +ustr
      seqqstr=seqqstr + ",'" + 'bc1abs' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc1abs' +nstr+ "', ful.z3." + 'bc1_xabsmean' +nstr + $
                        ",'" + 'bc1abs' +ustr+ "', ful.z3." + 'bc1_xabsmean' +ustr

      ;
      seqqstr=seqqstr + ",'" + 'bc2' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc2' +nstr+ "', ful.z3." + 'bc2_xmean' +nstr + $
                        ",'" + 'bc2' +ustr+ "', ful.z3." + 'bc2_xmean' +ustr
      seqqstr=seqqstr + ",'" + 'bc2rms' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc2rms' +nstr+ "', ful.z3." + 'bc2_xmean2' +nstr + $
                        ",'" + 'bc2rms' +ustr+ "', ful.z3." + 'bc2_xmean2' +ustr
      seqqstr=seqqstr + ",'" + 'bc2abs' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc2abs' +nstr+ "', ful.z3." + 'bc2_xabsmean' +nstr + $
                        ",'" + 'bc2abs' +ustr+ "', ful.z3." + 'bc2_xabsmean' +ustr

      ;
      seqqstr=seqqstr + ",'" + 'bc3' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc3' +nstr+ "', ful.z3." + 'bc3_xmean' +nstr + $
                        ",'" + 'bc3' +ustr+ "', ful.z3." + 'bc3_xmean' +ustr
      seqqstr=seqqstr + ",'" + 'bc3rms' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc3rms' +nstr+ "', ful.z3." + 'bc3_xmean2' +nstr + $
                        ",'" + 'bc3rms' +ustr+ "', ful.z3." + 'bc3_xmean2' +ustr
      seqqstr=seqqstr + ",'" + 'bc3abs' + "'," + "fltarr(nxc3,n_timestep)" + $
                        ",'" + 'bc3abs' +nstr+ "', ful.z3." + 'bc3_xabsmean' +nstr + $
                        ",'" + 'bc3abs' +ustr+ "', ful.z3." + 'bc3_xabsmean' +ustr
      ;
      if (total(tags eq 'FEMB_XMEAN'     ) gt 0.0) then begin
        seqqstr=seqqstr + "," + "'femb'      , fltarr(nr+1,n_timestep)" + $
                          "," + "'femb"       +nstr+ "', ful.z3.femb_xmean" +nstr+ $
                          "," + "'femb"       +ustr+ "', ful.z3.femb_xmean" +ustr+ $
                          "," + "'feyb'      , fltarr(nr+1,n_timestep)" + $
                          "," + "'feyb"       +nstr+ "', ful.z3.feyb_xmean" +nstr+ $
                          "," + "'feyb"       +ustr+ "', ful.z3.feyb_xmean" +ustr
      endif
      ;
      seqqstr=seqqstr + ")"
      ;print, seqqstr
      ;
      dummy=execute(seqqstr)
      seq=create_struct(seq, temporary(seqq))
      ;
    endif ; (nB)
    ;
    if (isCentral_flag) then begin
      if (total(tags eq 'MOMENTOFINERTIA3_XMEAN') gt 0.0) then begin
        dummy=execute("MomentOfInertia3_name=ful.r.MomentOfInertia3_xmean"+nstr)
        dummy=execute("MomentOfInertia3_unit=ful.r.MomentOfInertia3_xmean"+ustr)
        dummy=execute("AngularMomentum3_name=ful.r.AngularMomentum3_xmean"+nstr)
        dummy=execute("AngularMomentum3_unit=ful.r.AngularMomentum3_xmean"+ustr)
      endif else begin
        MomentOfInertia3_name='Moment of inertia 3'
        MomentOfInertia3_unit='g/cm'
        AngularMomentum3_name='Angular momentum 3'
        AngularMomentum3_unit='g/cm/s'
      endelse
      ;
      ; --- Create separate structure for quantities used for central potential and append it ---
      seqqstr="seqq=create_struct(" + $
        "'xc3_radial'          , float(reform(ful.r.xc3))," + $
        "'xc3_radial" +nstr+ "', ful.r.xc3" +nstr+ "," + $
        "'xc3_radial" +ustr+ "', ful.r.xc3" +ustr+ "," + $
        "'xc3_volume'          , float(reform(ful.r.xc3_volume))," + $
        "'xc3_volume" +nstr+ "', ful.r.xc3_volume" +nstr+ "," + $
        "'xc3_volume" +ustr+ "', ful.r.xc3_volume" +ustr+ "," + $
        "'xc3_surface'         , float(reform(ful.r.xc3_surface))," + $
        "'xc3_surface"+nstr+ "', ful.r.xc3_surface" +nstr+ "," + $
        "'xc3_surface"+ustr+ "', ful.r.xc3_surface" +ustr+ "," + $
        "'xb3_radial'          , float(reform(ful.r.xb3))," + $
        "'xb3_radial" +nstr+ "', ful.r.xb3" +nstr+ "," + $
        "'xb3_radial" +ustr+ "', ful.r.xb3" +ustr+ "," + $
        "'xb3_volume'          , float(reform(ful.r.xb3_volume))," + $
        "'xb3_volume" +nstr+ "', ful.r.xb3_volume" +nstr+ "," + $
        "'xb3_volume" +ustr+ "', ful.r.xb3_volume" +ustr+ "," + $
        "'xb3_surface'         , float(reform(ful.r.xb3_surface))," + $
        "'xb3_surface"+nstr+ "', ful.r.xb3_surface" +nstr+ "," + $
        "'xb3_surface"+ustr+ "', ful.r.xb3_surface" +ustr+ "," + $
        "'MOMENTOFINERTIA3'                   , fltarr(nr  ,n_timestep)," + $
        "'MOMENTOFINERTIA3"          +nstr+ "', MomentOfInertia3_name," + $
        "'MOMENTOFINERTIA3"          +ustr+ "', MomentOfInertia3_unit," + $
        "'ANGULARMOMENTUM3'                   , fltarr(nr  ,n_timestep)," + $
        "'ANGULARMOMENTUM3"          +nstr+ "', AngularMomentum3_name," + $
        "'ANGULARMOMENTUM3"          +ustr+ "', AngularMomentum3_unit," + $
        "'luminositypervolume' , fltarr(n_timestep)," + $ 
        "'luminositypervolume" +nstr+ "', ful.rad.luminositypervolume" +nstr+ "," + $
        "'luminositypervolume" +ustr+ "', ful.rad.luminositypervolume" +ustr+ "," + $
        "'rhototal1'           , fltarr(     n_timestep)," + $
        "'rhototal1"  +nstr+ "', ful.z3.rho_xmean" +nstr+ "," + $
        "'rhototal1"  +ustr+ "', ful.z3.rho_xmean" +ustr+ "," + $
        "'rhototal2'           , fltarr(     n_timestep)," + $
        "'rhototal2"  +nstr+ "', ful.z3.rho_xmean" +nstr+ "," + $
        "'rhototal2"  +ustr+ "', ful.z3.rho_xmean" +ustr+ "," + $
        "'rhototal3'           , fltarr(     n_timestep)," + $
        "'rhototal3"  +nstr+ "', ful.z3.rho_xmean" +nstr+ "," + $
        "'rhototal3"  +ustr+ "', ful.z3.rho_xmean" +ustr+ "," + $
        "'x1center'            , fltarr(     n_timestep)," + $
        "'x1center"   +nstr+ "', 'x1 coordinate of center of mass'," + $
        "'x1center"   +ustr+ "', ful.rad.xc1" +ustr+ "," + $
        "'x2center'            , fltarr(     n_timestep)," + $
        "'x2center"   +nstr+ "', 'x2 coordinate of center of mass'," + $
        "'x2center"   +ustr+ "', ful.rad.xc2" +ustr+ "," + $
        "'x3center'            , fltarr(     n_timestep)," + $
        "'x3center"   +nstr+ "', 'x3 coordinate of center of mass'," + $
        "'x3center"   +ustr+ "', ful.rad.xc3" +ustr+ "," + $
        "'v1center'            , fltarr(     n_timestep)," + $
        "'v1center"   +nstr+ "', 'v1 velocity of center of mass'," + $
        "'v1center"   +ustr+ "', ful.z3.v1_xmean" +ustr+ "," + $
        "'v2center'            , fltarr(     n_timestep)," + $
        "'v2center"   +nstr+ "', 'v2 velocity of center of mass'," + $
        "'v2center"   +ustr+ "', ful.z3.v2_xmean" +ustr+ "," + $
        "'v3center'            , fltarr(     n_timestep)," + $
        "'v3center"   +nstr+ "', 'v3 velocity of center of mass'," + $
        "'v3center"   +ustr+ "', ful.z3.v3_xmean" +ustr+ "," + $
        "'I1l'                 , fltarr(     n_timestep)," + $
        "'I1l"        +nstr+ "', ful.rad.Intb1_l" +nstr+ "," + $
        "'I1l"        +ustr+ "', ful.rad.Intb1_l" +ustr+ "," + $
        "'I1r'                 , fltarr(     n_timestep)," + $
        "'I1r"        +nstr+ "', ful.rad.Intb1_r" +nstr+ "," + $
        "'I1r"        +ustr+ "', ful.rad.Intb1_r" +ustr+ "," + $
        "'I2l'                 , fltarr(     n_timestep)," + $
        "'I2l"        +nstr+ "', ful.rad.Intb2_l" +nstr+ "," + $
        "'I2l"        +ustr+ "', ful.rad.Intb2_l" +ustr+ "," + $
        "'I2r'                 , fltarr(     n_timestep)," + $
        "'I2r"        +nstr+ "', ful.rad.Intb2_r" +nstr+ "," + $
        "'I2r"        +ustr+ "', ful.rad.Intb2_r" +ustr+ "," + $
        "'I3l'                 , fltarr(     n_timestep)," + $
        "'I3l"        +nstr+ "', ful.rad.Intb3_l" +nstr+ "," + $
        "'I3l"        +ustr+ "', ful.rad.Intb3_l" +ustr+ "," + $
        "'F1l'                 , fltarr(     n_timestep)," + $
        "'F1l"        +nstr+ "', ful.rad.Ferb1_l" +nstr+ "," + $
        "'F1l"        +ustr+ "', ful.rad.Ferb1_l" +ustr+ "," + $
        "'F1r'                 , fltarr(     n_timestep)," + $
        "'F1r"        +nstr+ "', ful.rad.Ferb1_r" +nstr+ "," + $
        "'F1r"        +ustr+ "', ful.rad.Ferb1_r" +ustr+ "," + $
        "'F2l'                 , fltarr(     n_timestep)," + $
        "'F2l"        +nstr+ "', ful.rad.Ferb2_l" +nstr+ "," + $
        "'F2l"        +ustr+ "', ful.rad.Ferb2_l" +ustr+ "," + $
        "'F2r'                 , fltarr(     n_timestep)," + $
        "'F2r"        +nstr+ "', ful.rad.Ferb2_r" +nstr+ "," + $
        "'F2r"        +ustr+ "', ful.rad.Ferb2_r" +ustr+ "," + $
        "'F3l'                 , fltarr(     n_timestep)," + $
        "'F3l"        +nstr+ "', ful.rad.Ferb3_l" +nstr+ "," + $
        "'F3l"        +ustr+ "', ful.rad.Ferb3_l" +ustr+ "," + $
        "'F3r'                 , fltarr(     n_timestep)," + $
        "'F3r"        +nstr+ "', ful.rad.Ferb3_r" +nstr+ "," + $
        "'F3r"        +ustr+ "', ful.rad.Ferb3_r" +ustr
      ;
      seqqstr=seqqstr + ")"
      ;print, seqqstr
      ;
      dummy=execute(seqqstr)
      seq=create_struct(seq, temporary(seqq))
      ;
      if (nwave gt 0) then begin
        ; --- Create separate structure for wave amplitudes and append it ---
        seqqstr="seqq=create_struct("
        for iwave=0,nwave-1 do begin
          lwave=floor(sqrt(float(iwave)+0.1))
          mwave=iwave - lwave^2 - lwave
          ;
          lwaveStr=strmid('000' + strtrim(string(    lwave) , 2), 2, /reverse)
          mwaveStr=strmid('000' + strtrim(string(abs(mwave)), 2), 2, /reverse)
          if (mwave lt 0) then begin
            mwaveStr='m' + mwaveStr
          endif else begin
            mwaveStr='p' + mwaveStr
          endelse
          ;
          waveid ='rhovb' + 'l' + lwaveStr + 'm' + mwaveStr
          wavemid=waveid + '_xmean'
          print, iwave, lwave, mwave, ' ('+lwaveStr+')', ' ('+mwaveStr+')'
          print, 'Add quantity ' + waveid + ' to structure.'
          ;
          if (iwave ge 1) then seqqstr=seqqstr + ","
          seqqstr=seqqstr +  "'" + waveid      + "', fltarr(nr+1,n_timestep)" + $
                            ",'" + waveid +nstr+ "', ful.r." + wavemid +nstr + $
                            ",'" + waveid +ustr+ "', ful.r." + wavemid +ustr
        endfor ; (iwave)
        ;
        seqqstr=seqqstr + ")"
        ;print, seqqstr
        ;
        dummy=execute(seqqstr)
        seq=create_struct(seq, temporary(seqq))
        ;
      endif ; (nwave)
      ;
    endif ; (isCentral_flag)
    ;
  endif else begin ; (not keyword_set(ReadMetaData))
    ; --- Create structure without metadata ---
    seqstr="seq=create_struct(" + $
      "'model'     , modelname," + $
      "'modelident', modelident," + $
      "'mean'      , ful," + $
      "'par'       , par," + $
      "'xc1'       , float(reform(ful.rad.xc1))," + $
      "'xc2'       , float(reform(ful.rad.xc2))," + $
      "'xc3'       , float(reform(ful.rad.xc3))," + $
      "'xb1'       , float(reform(ful.rad.xb1))," + $
      "'xb2'       , float(reform(ful.rad.xb2))," + $
      "'xb3'       , float(reform(ful.rad.xb3))," + $
      "'binfactor' , 1L," + $
      "'itime'     , lonarr(     n_timestep)," + $
      "'itime_first',lonarr(     n_timestep)," + $
      "'itime_last', lonarr(     n_timestep)," + $
      "'time'      , fltarr(     n_timestep)," + $
      "'time_first', fltarr(     n_timestep)," + $
      "'time_last' , fltarr(     n_timestep)," + $
      "'JulianDate', dblarr(     n_timestep)," + $
      "'ifile'     , lonarr(     n_timestep)," + $
      "'file'      , strarr(     n_timestep)," + $
      "'version'   , strarr(     n_timestep)," + $
      "'s_inflow'  , fltarr(     n_timestep)," + $
      "'c_schange' , fltarr(     n_timestep)," + $ 
      "'c_hptopfactor', fltarr(  n_timestep)," + $ 
      "'I3r'       , fltarr(     n_timestep)," + $
      "'v1'        , fltarr(nr  ,n_timestep)," + $
      "'v1drms'    , fltarr(nr  ,n_timestep)," + $
      "'v1rms'     , fltarr(nr  ,n_timestep)," + $
      "'rhov1'     , fltarr(nr  ,n_timestep)," + $
      "'rhov1v1'   , fltarr(nr  ,n_timestep)," + $
      "'rhov1v2'   , fltarr(nr  ,n_timestep)," + $
      "'rhov1v3'   , fltarr(nr  ,n_timestep)," + $
      "'v2'        , fltarr(nr  ,n_timestep)," + $
      "'v2drms'    , fltarr(nr  ,n_timestep)," + $
      "'v2rms'     , fltarr(nr  ,n_timestep)," + $
      "'rhov2'     , fltarr(nr  ,n_timestep)," + $
      "'rhov2v2'   , fltarr(nr  ,n_timestep)," + $
      "'rhov2v3'   , fltarr(nr  ,n_timestep)," + $
      "'v3'        , fltarr(nr  ,n_timestep)," + $
      "'v3drms'    , fltarr(nr  ,n_timestep)," + $
      "'v3rms'     , fltarr(nr  ,n_timestep)," + $
      "'rhov3'     , fltarr(nr  ,n_timestep)," + $
      "'rhov3v3'   , fltarr(nr  ,n_timestep)," + $
      "'rho'       , fltarr(nr  ,n_timestep)," + $
      "'rhorms'    , fltarr(nr  ,n_timestep)," + $
      "'ei'        , fltarr(nr  ,n_timestep)," + $
      "'rhoei'     , fltarr(nr  ,n_timestep)," + $
      "'rhoek'     , fltarr(nr  ,n_timestep)," + $
      "'rhoeg'     , fltarr(nr  ,n_timestep)," + $
      "'T'         , fltarr(nr  ,n_timestep)," + $
      "'Tdrms'     , fltarr(nr  ,n_timestep)," + $
      "'Trms'      , fltarr(nr  ,n_timestep)," + $
;      "'T_xmean4'  , fltarr(nr  ,n_timestep)," + $
;      "'T_xmeankapparho', fltarr(nr  ,n_timestep)," + $
      "'P'         , fltarr(nr  ,n_timestep)," + $
      "'Prms'      , fltarr(nr  ,n_timestep)," + $
      "'s'         , fltarr(nr  ,n_timestep)," + $
      "'srms'      , fltarr(nr  ,n_timestep)," + $
      "'rhos'      , fltarr(nr  ,n_timestep)," + $
      "'gamma1'    , fltarr(nr  ,n_timestep)," + $
      "'gamma3'    , fltarr(nr  ,n_timestep)," + $
      "'delta'     , fltarr(nr  ,n_timestep)," + $
      "'kapparho'  , fltarr(nr  ,n_timestep)," + $
      "'gravrad'   , fltarr(nr  ,n_timestep)," + $
      "'rhotil'    , fltarr(nr+1,n_timestep)," + $
      "'rhovb'     , fltarr(nr+1,n_timestep)," + $
      "'rhovbrms'  , fltarr(nr+1,n_timestep)," + $
      "'frhov1b'   , fltarr(nr+1,n_timestep)," + $
      "'frhov2b'   , fltarr(nr+1,n_timestep)," + $
      "'frhov3b'   , fltarr(nr+1,n_timestep)," + $
      "'feipb'     , fltarr(nr+1,n_timestep)," + $
      "'fekb'      , fltarr(nr+1,n_timestep)," + $
      "'fegb'      , fltarr(nr+1,n_timestep)," + $
      "'fepb'      , fltarr(nr+1,n_timestep)," + $
      "'femb'      , fltarr(nr+1,n_timestep)," + $
      "'feyb'      , fltarr(nr+1,n_timestep)," + $
      "'fevb'      , fltarr(nr+1,n_timestep)," + $
      "'fewb'      , fltarr(nr+1,n_timestep)," + $
      "'ferb'      , fltarr(nr+1,n_timestep)"
    ;
    for iquc=0,nquc-1 do begin
      qucid='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      print, 'Add quantity ' + qucid + ' to structure.'
;      seq=create_struct(seq, qucid, fltarr(nxc3,n_timestep))
      seqstr=seqstr + ",'" + qucid + "'," + "fltarr(nxc3,n_timestep)"
    endfor
    ;
    for iquc=0,nfquc-1 do begin
      fqucid='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      ;print, 'Add quantity ' + fqucid + ' to structure.'
;      seq=create_struct(seq, fqucid, fltarr(nxc3+1,n_timestep))
      seqstr=seqstr + ",'" + fqucid + "'," + "fltarr(nxc3+1,n_timestep)"
    endfor
    ;
    seqstr=seqstr + ")"
    ;print, seqstr
    ;
    dummy=execute(seqstr)
    ;
    if (isCentral_flag) then begin
      ; --- Create separate structure for quantities used for central potential and append it ---
      seqqstr="seqq=create_struct(" + $
        "'xc3_radial'          , float(reform(ful.r.xc3))," + $
        "'xc3_volume'          , float(reform(ful.r.xc3_volume))," + $
        "'xc3_surface'         , float(reform(ful.r.xc3_surface))," + $
        "'xb3_radial'          , float(reform(ful.r.xb3))," + $
        "'xb3_volume'          , float(reform(ful.r.xb3_volume))," + $
        "'xb3_surface'         , float(reform(ful.r.xb3_surface))," + $
        "'MOMENTOFINERTIA3'    , fltarr(nr  ,n_timestep)," + $
        "'ANGULARMOMENTUM3'    , fltarr(nr  ,n_timestep)," + $
        "'luminositypervolume' , fltarr(     n_timestep)," + $ 
        "'rhototal1'           , fltarr(     n_timestep)," + $
        "'rhototal2'           , fltarr(     n_timestep)," + $
        "'rhototal3'           , fltarr(     n_timestep)," + $
        "'x1center'            , fltarr(     n_timestep)," + $
        "'x2center'            , fltarr(     n_timestep)," + $
        "'x3center'            , fltarr(     n_timestep)," + $
        "'v1center'            , fltarr(     n_timestep)," + $
        "'v2center'            , fltarr(     n_timestep)," + $
        "'v3center'            , fltarr(     n_timestep)," + $
        "'I1l'                 , fltarr(     n_timestep)," + $
        "'I1r'                 , fltarr(     n_timestep)," + $
        "'I2l'                 , fltarr(     n_timestep)," + $
        "'I2r'                 , fltarr(     n_timestep)," + $
        "'I3l'                 , fltarr(     n_timestep)," + $
        "'F1l'                 , fltarr(     n_timestep)," + $
        "'F1r'                 , fltarr(     n_timestep)," + $
        "'F2l'                 , fltarr(     n_timestep)," + $
        "'F2r'                 , fltarr(     n_timestep)," + $
        "'F3l'                 , fltarr(     n_timestep)," + $
        "'F3r'                 , fltarr(     n_timestep)"
      ;
      seqqstr=seqqstr + ")"
      ;print, seqqstr
      ;
      dummy=execute(seqqstr)
      seq=create_struct(seq, temporary(seqq))
      ;
      if (nwave gt 0) then begin
        ; --- Create separate structure for wave amplitudes and append it ---
        seqqstr="seqq=create_struct("
        for iwave=0,nwave-1 do begin
          lwave=floor(sqrt(float(iwave)+0.1))
          mwave=iwave - lwave^2 - lwave
          ;
          lwaveStr=strmid('000' + strtrim(string(    lwave) , 2), 2, /reverse)
          mwaveStr=strmid('000' + strtrim(string(abs(mwave)), 2), 2, /reverse)
          if (mwave lt 0) then begin
            mwaveStr='m' + mwaveStr
          endif else begin
            mwaveStr='p' + mwaveStr
          endelse
          ;
          waveid ='rhovb' + 'l' + lwaveStr + 'm' + mwaveStr
          print, 'Add quantity ' + waveid + ' to structure.'
          ;
          if (iwave ge 1) then seqqstr=seqqstr + ","
          seqqstr=seqqstr +  "'" + waveid      + "', fltarr(nr+1,n_timestep)"
        endfor ; (iwave)
        ;
        seqqstr=seqqstr + ")"
        ;print, seqqstr
        ;
        dummy=execute(seqqstr)
        seq=create_struct(seq, temporary(seqq))
        ;
      endif ; (nwave)
      ;
    endif ; (isCentral_flag)
  endelse ; (keyword_set(ReadMetaData))
  ;
endif else begin ; (not keyword_set(InitStruct))
  ; === Put snapshot data into structure ===
  ;
  file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1)
  i=i_timestep
  ;
  JulianDate=0.0D+00
  if (keyword_set(date)) then begin
    if (strmatch(date, '[0-9][0-9].[0-9][0-9].[0-9][0-9][0-9][0-9] [0-9][0-9]:[0-9][0-9]:[0-9][0-9]?[0-9][0-9][0-9]*')) then begin
      JulianDate=julday(long(strmid(date, 3,2)), long(strmid(date, 0,2)), long(strmid(date, 6,4)), $
                    long(strmid(date,11,2)), long(strmid(date,14,2)), long(strmid(date,17,2))) + $
             double(strmid(date,20,3))/86400000.0D+00
    endif
  endif
  ;
  seq.itime[i]      =ful.z3.itime
  seq.itime_first[i]=ful.z3.itime
  seq.itime_last[i] =ful.z3.itime
  seq.time[i]       =ful.z3.time
  seq.time_first[i] =ful.z3.time
  seq.time_last[i]  =ful.z3.time
  seq.JulianDate[i] =JulianDate
  seq.ifile[i]      =snapshot
  seq.file[i]       =file_tmp
  if (keyword_set(version)) then seq.version[i]=version
  seq.s_inflow[i]   =par.s_inflow
  seq.c_schange[i]  =par.c_schange
  seq.c_hptopfactor[i]=par.c_hptopfactor
  seq.I3r[i]        = avg(ful.rad.Intb3_r)
  ;
  if (isCentral_flag) then begin
    ; --- Central potential ---
    ;
    tags=tag_names(ful.r)
    ;
    seq.luminositypervolume[i]=ful.rad.luminositypervolume
    seq.rhototal1[i] =avg(ful.z1.rho_xmean)
    seq.rhototal2[i] =avg(ful.z2.rho_xmean)
    seq.rhototal3[i] =avg(ful.z3.rho_xmean)
    seq.x1center[i]  =avg(ful.z1.rho_xmean*ful.z1.xc1)/avg(ful.z1.rho_xmean)
    seq.x2center[i]  =avg(ful.z2.rho_xmean*ful.z2.xc2)/avg(ful.z2.rho_xmean)
    seq.x3center[i]  =avg(ful.z3.rho_xmean*ful.z3.xc3)/avg(ful.z3.rho_xmean)
    seq.v1center[i]  =avg(ful.z1.rhov1_xmean)/avg(ful.z1.rho_xmean)
    seq.v2center[i]  =avg(ful.z2.rhov2_xmean)/avg(ful.z2.rho_xmean)
    seq.v3center[i]  =avg(ful.z3.rhov3_xmean)/avg(ful.z3.rho_xmean)
    seq.I1l[i]       =-avg(ful.rad.Intb1_l)
    seq.I1r[i]       = avg(ful.rad.Intb1_r)
    seq.I2l[i]       =-avg(ful.rad.Intb2_l)
    seq.I2r[i]       = avg(ful.rad.Intb2_r)
    seq.I3l[i]       =-avg(ful.rad.Intb3_l)
    seq.F1l[i]       =-avg(ful.rad.Ferb1_l)
    seq.F1r[i]       = avg(ful.rad.Ferb1_r)
    seq.F2l[i]       =-avg(ful.rad.Ferb2_l)
    seq.F2r[i]       = avg(ful.rad.Ferb2_r)
    seq.F3l[i]       =-avg(ful.rad.Ferb3_l)
    seq.F3r[i]       = avg(ful.rad.Ferb3_r)
    ;
    if (total(tags eq 'V3_XMEAN'       ) gt 0.0) then seq.v3[*,i]             =reform(ful.r.v3_xmean)
    if (total(tags eq 'V3_XMEAN2'      ) gt 0.0) then seq.v3drms[*,i]         =reform(sqrt((ful.r.v3_xmean2^2-ful.r.v3_xmean^2)>0.0))
    if (total(tags eq 'V3_XMEAN2'      ) gt 0.0) then seq.v3rms[*,i]          =reform(ful.r.v3_xmean2)
    if (total(tags eq 'V1_XMEAN'       ) gt 0.0) then seq.v1[*,i]             =reform(ful.r.v1_xmean)
    if ((total(tags eq 'V1_XMEAN2'      ) gt 0.0) and $
        (total(tags eq 'V1_XMEAN'       ) gt 0.0)) then seq.v1drms[*,i]      =reform(sqrt((ful.r.v1_xmean2^2-ful.r.v1_xmean^2)>0.0))
    if (total(tags eq 'V1_XMEAN2'      ) gt 0.0) then seq.v1rms[*,i]          =reform(ful.r.v1_xmean2)
    ;seq.v2[*,i]      =0.0
    ;seq.v2drms[*,i]  =0.0
    ;seq.v2rms[*,i]   =0.0
    ;
    if (total(tags eq 'RHOV1_XMEAN'    ) gt 0.0) then seq.rhov1[*,i]          =reform(ful.r.rhov1_xmean)
    if (total(tags eq 'RHOV1V1_XMEAN'  ) gt 0.0) then seq.rhov1v1[*,i]        =reform(ful.r.rhov1v1_xmean)
    ;
    if (total(tags eq 'RHOV3_XMEAN'    ) gt 0.0) then seq.rhov3[*,i]          =reform(ful.r.rhov3_xmean)
    if (total(tags eq 'RHOV3V3_XMEAN'  ) gt 0.0) then seq.rhov3v3[*,i]        =reform(ful.r.rhov3v3_xmean)
    seq.rho[*,i]     =reform(ful.r.rho_xmean)
    if (total(tags eq 'RHO_XMEAN2'     ) gt 0.0) then seq.rhorms[*,i]         =reform(ful.r.rho_xmean2)
    ;
    if (total(tags eq 'EI_XMEAN'       ) gt 0.0) then seq.ei[*,i]             =reform(ful.r.ei_xmean)
    seq.rhoei[*,i]   =reform(ful.r.rhoei_xmean)
    seq.rhoek[*,i]   =reform(ful.r.rhoek_xmean)
    seq.rhoeg[*,i]   =reform(ful.r.rhoeg_xmean)
    ;
    seq.T[*,i]       =reform(ful.r.T_xmean)
    if (total(tags eq 'T_XMEAN2'       ) gt 0.0) then seq.Tdrms[*,i]          =reform(sqrt((ful.r.T_xmean2^2-ful.r.T_xmean^2)>0.0))
    if (total(tags eq 'T_XMEAN2'       ) gt 0.0) then seq.Trms[*,i]           =reform(ful.r.T_xmean2)
    if (total(tags eq 'T_XMEAN4'       ) gt 0.0) then seq.T_xmean4[*,i]       =reform(ful.r.T_xmean4)
    if (total(tags eq 'T_XMEANKAPPARHO') gt 0.0) then seq.T_xmeankapparho[*,i]=reform(ful.r.T_xmeankapparho)
    ;
    seq.P[*,i]       =reform(ful.r.P_xmean)
    if (total(tags eq 'P_XMEAN2'       ) gt 0.0) then seq.Prms[*,i]           =reform(ful.r.P_xmean2)
    if (total(tags eq 'S_XMEAN'        ) gt 0.0) then seq.s[*,i]              =reform(ful.r.s_xmean)
    if (total(tags eq 'S_XMEAN2'       ) gt 0.0) then seq.srms[*,i]           =reform(ful.r.s_xmean2)
    seq.rhos[*,i]    =reform(ful.r.rhos_xmean)
    ;
    seq.Gamma1[*,i]  =reform(ful.r.Gamma1_xmean)
    seq.Gamma3[*,i]  =reform(ful.r.Gamma3_xmean)
    seq.delta[*,i]   =reform(ful.r.delta_xmean)
    seq.kapparho[*,i]=reform(ful.r.kapparho_xmean)
    seq.gravrad[*,i] =reform(ful.r.gravrad_xmean)
    if (total(tags eq 'RHOTIL_XMEAN'   ) gt 0.0) then seq.rhotil[*,i]         =reform(ful.r.rhotil_xmean)
    seq.rhovb[*,i]   =reform(ful.r.rhovb_xmean)
    if (total(tags eq 'RHOVB_XMEAN2'   ) gt 0.0) then seq.rhovbrms[*,i]       =reform(ful.r.rhovb_xmean2)
    ;
    if (total(tags eq 'MOMENTOFINERTIA3_XMEAN'     ) gt 0.0) then begin
      seq.MomentOfInertia3[*,i]           =reform(ful.r.MomentOfInertia3_xmean)
      seq.AngularMomentum3[*,i]           =reform(ful.r.AngularMomentum3_xmean)
    endif
    ;
    seq.feipb[*,i]   =reform(ful.r.feipb_xmean)
    seq.fekb[*,i]    =reform(ful.r.fekb_xmean)
    seq.fegb[*,i]    =reform(ful.r.fegb_xmean)
    seq.fepb[*,i]    =reform(ful.r.fepb_xmean)
    if (total(tags eq 'FEMB_XMEAN'     ) gt 0.0) then seq.femb[*,i]           =reform(ful.r.femb_xmean   )
    if (total(tags eq 'FEYB_XMEAN'     ) gt 0.0) then seq.feyb[*,i]           =reform(ful.r.feyb_xmean   )
    if (total(tags eq 'FEVB_XMEAN'     ) gt 0.0) then seq.fevb[*,i]           =reform(ful.r.fevb_xmean   )
    if (total(tags eq 'FEVP2PB_XMEAN'  ) gt 0.0) then seq.fewb[*,i]           =reform(ful.r.fevp2pb_xmean)
    seq.ferb[*,i]    =reform(ful.r.ferb_xmean)
    ;  
    for iquc=0,nquc-1 do begin
      qucsid='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      qucmid='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse) + '_xmean'
      dummy=execute('seq.' + qucsid + '[*,i]=reform(ful.r.' + qucmid +')')
    endfor ; iquc
    ;
    for iquc=0,nfquc-1 do begin
      fqucsid='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      fqucmid='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse) + '_xmean'
      dummy=execute('seq.' + fqucsid + '[*,i]=reform(ful.r.' + fqucmid +')')
    endfor ; iquc
    ;
    if (total(tags eq 'BC1_XMEAN'      ) gt 0.0) then seq.bc1[   *,i]         =reform(ful.r.bc1_xmean)
    if (total(tags eq 'BC1_XMEAN2'     ) gt 0.0) then seq.bc1rms[*,i]         =reform(ful.r.bc1_xmean2)
    if (total(tags eq 'BC1_XABSMEAN'   ) gt 0.0) then seq.bc1abs[*,i]         =reform(ful.r.bc1_xabsmean)
    if (total(tags eq 'BC2_XMEAN'      ) gt 0.0) then seq.bc2[   *,i]         =reform(ful.r.bc2_xmean)
    if (total(tags eq 'BC2_XMEAN2'     ) gt 0.0) then seq.bc2rms[*,i]         =reform(ful.r.bc2_xmean2)
    if (total(tags eq 'BC2_XABSMEAN'   ) gt 0.0) then seq.bc2abs[*,i]         =reform(ful.r.bc2_xabsmean)
    if (total(tags eq 'BC3_XMEAN'      ) gt 0.0) then seq.bc3[   *,i]         =reform(ful.r.bc3_xmean)
    if (total(tags eq 'BC3_XMEAN2'     ) gt 0.0) then seq.bc3rms[*,i]         =reform(ful.r.bc3_xmean2)
    if (total(tags eq 'BC3_XABSMEAN'   ) gt 0.0) then seq.bc3abs[*,i]         =reform(ful.r.bc3_xabsmean)
    ;
    for iwave=0,nwave-1 do begin
      ; --- Store wave amplitudes ---
      lwave=floor(sqrt(float(iwave)+0.1))
      mwave=iwave - lwave^2 - lwave
      ;
      lwaveStr=strmid('000' + strtrim(string(    lwave) , 2), 2, /reverse)
      mwaveStr=strmid('000' + strtrim(string(abs(mwave)), 2), 2, /reverse)
      if (mwave lt 0) then begin
        mwaveStr='m' + mwaveStr
      endif else begin
        mwaveStr='p' + mwaveStr
      endelse
      ;
      waveid ='rhovb' + 'l' + lwaveStr + 'm' + mwaveStr
      wavemid=waveid + '_xmean'
      ;
      dummy=execute('seq.' + waveid + '[*,i]=reform(ful.r.' + wavemid +')')
    endfor ; (iwave)
    ;
  endif else begin ; (not isCentral_flag)
    ; --- Plane-parallel geometry ---
    ;
    tags=tag_names(ful.z3)
    ;
    if (total(tags eq 'V3_XMEAN'       ) gt 0.0) then seq.v3[*,i]             =reform(ful.z3.v3_xmean)
    if (total(tags eq 'V3_XMEAN2'      ) gt 0.0) then seq.v3drms[*,i]         =reform(sqrt((ful.z3.v3_xmean2^2-ful.z3.v3_xmean^2)>0.0))
    if (total(tags eq 'V3_XMEAN2'      ) gt 0.0) then seq.v3rms[*,i]          =reform(ful.z3.v3_xmean2)
    if (total(tags eq 'V1_XMEAN'       ) gt 0.0) then seq.v1[*,i]             =reform(ful.z3.v1_xmean)
    if (total(tags eq 'V1_XMEAN2'      ) gt 0.0) then seq.v1drms[*,i]         =reform(sqrt((ful.z3.v1_xmean2^2-ful.z3.v1_xmean^2)>0.0))
    if (total(tags eq 'V1_XMEAN2'      ) gt 0.0) then seq.v1rms[*,i]          =reform(ful.z3.v1_xmean2)
    if (total(tags eq 'V2_XMEAN'       ) gt 0.0) then seq.v2[*,i]             =reform(ful.z3.v2_xmean)
    if (total(tags eq 'V2_XMEAN2'      ) gt 0.0) then seq.v2drms[*,i]         =reform(sqrt((ful.z3.v2_xmean2^2-ful.z3.v2_xmean^2)>0.0))
    if (total(tags eq 'V2_XMEAN2'      ) gt 0.0) then seq.v2rms[*,i]          =reform(ful.z3.v2_xmean2)
    ;
    if (total(tags eq 'RHOV1_XMEAN'    ) gt 0.0) then seq.rhov1[*,i]          =reform(ful.z3.rhov1_xmean)
    if (total(tags eq 'RHOV1V1_XMEAN'  ) gt 0.0) then seq.rhov1v1[*,i]        =reform(ful.z3.rhov1v1_xmean)
    if (total(tags eq 'RHOV1V2_XMEAN'  ) gt 0.0) then seq.rhov1v2[*,i]        =reform(ful.z3.rhov1v2_xmean)
    if (total(tags eq 'RHOV1V3_XMEAN'  ) gt 0.0) then seq.rhov1v3[*,i]        =reform(ful.z3.rhov1v3_xmean)
    if (total(tags eq 'RHOV2_XMEAN'    ) gt 0.0) then seq.rhov2[*,i]          =reform(ful.z3.rhov2_xmean)
    if (total(tags eq 'RHOV2V2_XMEAN'  ) gt 0.0) then seq.rhov2v2[*,i]        =reform(ful.z3.rhov2v2_xmean)
    if (total(tags eq 'RHOV2V3_XMEAN'  ) gt 0.0) then seq.rhov2v3[*,i]        =reform(ful.z3.rhov2v3_xmean)
    if (total(tags eq 'RHOV3_XMEAN'    ) gt 0.0) then seq.rhov3[*,i]          =reform(ful.z3.rhov3_xmean)
    if (total(tags eq 'RHOV3V3_XMEAN'  ) gt 0.0) then seq.rhov3v3[*,i]        =reform(ful.z3.rhov3v3_xmean)
    seq.rho[*,i]      =reform(ful.z3.rho_xmean)
    if (total(tags eq 'RHO_XMEAN2'     ) gt 0.0) then seq.rhorms[*,i]         =reform(ful.z3.rho_xmean2)
    ;
    if (total(tags eq 'EI_XMEAN'       ) gt 0.0) then seq.ei[*,i]             =reform(ful.z3.ei_xmean)
    seq.rhoei[*,i]    =reform(ful.z3.rhoei_xmean)
    seq.rhoek[*,i]    =reform(ful.z3.rhoek_xmean)
    seq.rhoeg[*,i]    =reform(ful.z3.rhoeg_xmean)
    ;
    seq.T[*,i]        =reform(ful.z3.T_xmean)
    if (total(tags eq 'T_XMEAN2'       ) gt 0.0) then seq.Tdrms[*,i]          =reform(sqrt((ful.z3.T_xmean2^2-ful.z3.T_xmean^2)>0.0))
    if (total(tags eq 'T_XMEAN2'       ) gt 0.0) then seq.Trms[*,i]           =reform(ful.z3.T_xmean2)
    if (total(tags eq 'T_XMEAN4'       ) gt 0.0) then seq.T_xmean4[*,i]       =reform(ful.z3.T_xmean4)
    if (total(tags eq 'T_XMEANKAPPARHO') gt 0.0) then seq.T_xmeankapparho[*,i]=reform(ful.z3.T_xmeankapparho)
    ;
    seq.P[*,i]        =reform(ful.z3.P_xmean)
    if (total(tags eq 'P_XMEAN2'       ) gt 0.0) then seq.Prms[*,i]           =reform(ful.z3.P_xmean2)
    if (total(tags eq 'S_XMEAN'        ) gt 0.0) then seq.s[*,i]              =reform(ful.z3.s_xmean)
    if (total(tags eq 'S_XMEAN2'       ) gt 0.0) then seq.srms[*,i]           =reform(ful.z3.s_xmean2)
    seq.rhos[*,i]     =reform(ful.z3.rhos_xmean)
    ;
    seq.Gamma1[*,i]   =reform(ful.z3.Gamma1_xmean)
    seq.Gamma3[*,i]   =reform(ful.z3.Gamma3_xmean)
    seq.delta[*,i]    =reform(ful.z3.delta_xmean)
    seq.kapparho[*,i] =reform(ful.z3.kapparho_xmean)
    seq.gravrad[*,i]  =reform(ful.z3.gravrad_xmean)
    if (total(tags eq 'RHOTIL_XMEAN'   ) gt 0.0) then seq.rhotil[*,i]         =reform(ful.z3.rhotil_xmean)
    seq.rhovb[*,i]    =reform(ful.z3.rhovb_xmean)
    if (total(tags eq 'RHOVB_XMEAN2'   ) gt 0.0) then seq.rhovbrms[*,i]       =reform(ful.z3.rhovb_xmean2)
    ;
    if (total(tags eq 'FRHOV13B_XMEAN' ) gt 0.0) then seq.frhov1b[*,i] =reform(ful.z3.frhov13b_xmean)
    if (total(tags eq 'FRHOV23B_XMEAN' ) gt 0.0) then seq.frhov2b[*,i] =reform(ful.z3.frhov23b_xmean)
    if (total(tags eq 'FRHOV33B_XMEAN' ) gt 0.0) then seq.frhov3b[*,i] =reform(ful.z3.frhov33b_xmean)
    ;
    seq.feipb[*,i]    =reform(ful.z3.feipb_xmean)
    seq.fekb[*,i]     =reform(ful.z3.fekb_xmean)
    seq.fegb[*,i]     =reform(ful.z3.fegb_xmean)
    seq.fepb[*,i]     =reform(ful.z3.fepb_xmean)
    if (total(tags eq 'FEMB_XMEAN'     ) gt 0.0) then seq.femb[*,i]    =reform(ful.z3.femb_xmean   )
    if (total(tags eq 'FEYB_XMEAN'     ) gt 0.0) then seq.feyb[*,i]    =reform(ful.z3.feyb_xmean   )
    if (total(tags eq 'FEVB_XMEAN'     ) gt 0.0) then seq.fevb[*,i]    =reform(ful.z3.fevb_xmean   )
    if (total(tags eq 'FEVP2PB_XMEAN'  ) gt 0.0) then seq.fewb[*,i]    =reform(ful.z3.fevp2pb_xmean)
    seq.ferb[*,i]     =reform(ful.z3.ferb_xmean)
    ;
    for iquc=0,nquc-1 do begin
      qucsid='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      qucmid='quc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse) + '_xmean'
      dummy=execute('seq.' + qucsid + '[*,i]=reform(ful.z3.' + qucmid +')')
    endfor ; iquc
    ;
    for iquc=0,nfquc-1 do begin
      fqucsid='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse)
      fqucmid='fquc' + strmid('000' + strtrim(string(iquc+1), 2), 2, /reverse) + '_xmean'
      dummy=execute('seq.' + fqucsid + '[*,i]=reform(ful.z3.' + fqucmid +')')
    endfor ; iquc
    ;
    if (total(tags eq 'BC1_XMEAN'      ) gt 0.0) then seq.bc1[   *,i]         =reform(ful.z3.bc1_xmean)
    if (total(tags eq 'BC1_XMEAN2'     ) gt 0.0) then seq.bc1rms[*,i]         =reform(ful.z3.bc1_xmean2)
    if (total(tags eq 'BC1_XABSMEAN'   ) gt 0.0) then seq.bc1abs[*,i]         =reform(ful.z3.bc1_xabsmean)
    if (total(tags eq 'BC2_XMEAN'      ) gt 0.0) then seq.bc2[   *,i]         =reform(ful.z3.bc2_xmean)
    if (total(tags eq 'BC2_XMEAN2'     ) gt 0.0) then seq.bc2rms[*,i]         =reform(ful.z3.bc2_xmean2)
    if (total(tags eq 'BC2_XABSMEAN'   ) gt 0.0) then seq.bc2abs[*,i]         =reform(ful.z3.bc2_xabsmean)
    if (total(tags eq 'BC3_XMEAN'      ) gt 0.0) then seq.bc3[   *,i]         =reform(ful.z3.bc3_xmean)
    if (total(tags eq 'BC3_XMEAN2'     ) gt 0.0) then seq.bc3rms[*,i]         =reform(ful.z3.bc3_xmean2)
    if (total(tags eq 'BC3_XABSMEAN'   ) gt 0.0) then seq.bc3abs[*,i]         =reform(ful.z3.bc3_xabsmean)
    ;
  endelse ; (isCentral_flag)
  ;
endelse ; (keyword_set(InitStruct))

end ; rhd_seq_put2struct
