;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   coma_opac_extract ('coma_opacity_extract')
;
; PURPOSE:
;   Read COMA raw opacity data and put them into IDL save file together with some meta data.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   @coma_opac_extract
;
; EXAMPLE:
;   @coma_opac_extract
;
; MODIFICATION HISTORY:
;   2015-08-27 (B.F. Uppsala) Put into it's own routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set the raw-data file names ---
;vmicro=2.5 & OpaRawDir='solabu/'
;             OpaRawFiles='kapnu_solabu_p[123]' & OpSRawFiles='kapsc_solabu_p[123]'
;             OpaSavFile='coma_kapnu_solabu.sav' & OpSSavFile='coma_kapsc_solabu.sav'
vmicro=2.5 & OpaRawDir='solabu_h2obt2/'
             OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='kapsc_solabu_p[123]'
             OpaSavFile='coma_kapnu_solabu_h2obt2.sav' & OpSSavFile='coma_kapsc_solabu.sav'
;vmicro=20.0 & OpaRawDir='solabu_h2obt2_xi20/'
;              OpaRawFiles='kapnu_solabu_h2obt2_xi20_p[123]' & OpSRawFiles='kapsc_solabu_h2obt2_xi20_p[123]'
;              OpaSavFile='coma_kapnu_solabu_h2obt2_xi20.sav' & OpSSavFile='coma_kapsc_solabu_h2obt2_xi20.sav'

; --- Set the path ---
OpaRawPath=getenv('HOME') + '/dat/opa/BernhardAringer/' + OpaRawDir

; --- Decide, which ones to get (absorption or scattering) ---
RawFiles=OpaRawFiles & SavFile=OpaSavFile
;RawFiles=OpSRawFiles & SavFile=OpSSavFile


; --- Load the raw data ---
coma_opac_rd, OpaRawPath, RawFiles, P=P, T=T, rho=rho, kappanu=kappanu, kappaRoss=kappaRoss, kappaPlan=kappaPlan, lambda=lambda, nu=nu, EleSym=EleSym, EleAbu2=EleAbu


; --- Set file name for atomic-weight data and load it ---
EleID  ='IUPAC_2013_Atomic_Weights'
EleOut =EleID + '.sav'
restore, EleOut, /verbose

; --- Compute abundances --- 
EleInd=strCrossCmp(strtrim(EleSym, 2), Ele.Symbol, v2=Ele.Weight, Default=0.0, v1=EleWeight, /FOLD_CASE)
;
EA =10.0^EleAbu
EAT=total(EA)
abuX=EA[0]/EAT
abuY=EA[1]/EAT
abuZ=total(EA[2:*])/EAT
help, abuX, abuY, abuZ, abuX+abuY+abuZ
;
EW =EA * EleWeight
EWT=total(EW)
X=EW[0]/EWT
Y=EW[1]/EWT
Z=total(EW[2:*])/EWT
Qmol=EWT/EAT
help, X, Y, Z, X+Y+Z, Qmol

;; --- Compute abundances with a mean molecular weight for the metals---
;wX=1.0080 & wY=4.0026 & wZ=17.8351
;help, wX,wY,wZ
;;
;abuTot=total(10.0^EleAbu)
;abuX=10.0^EleAbu[0]/abuTot
;abuY=10.0^EleAbu[1]/abuTot
;abuZ=1.0-abuX-abuY
;help, abuX, abuY, abuZ
;Qmol=abuX*wX+abuY*wY+abuZ*wZ
;help, Qmol
;;
;X=abuX*wX/Qmol
;Y=abuY*wY/Qmol
;Z=abuZ*wZ/Qmol
;help, X, Y, Z

; --- Assemble some meta data ---
MetaRaw= { $
  Description: 'Meta data of raw opacities', $
  FileOriginal:      RawFiles, $
  FileSource:        SavFile, $
  FilePath:          OpaRawPath, $
  Code:              'COMA', $
  CodeVersion:       'COMA 11a, B.Aringer 2012', $
  Author:            'Bernhard Aringer', $
  AbundanceSorce:    'solar (Paola)', $
  AbundanceID:       'Paola', $
  ElementSymbols:    strupcase(EleSym), $
  ElementAbundances: EleAbu, $
  abuX:              abuX, $
  abuY:              abuY, $
  abuZ:              abuZ, $
  X:                 X, $
  Y:                 Y, $
  Z:                 Z, $
  Qmol:              Qmol, $
  Vmicro:            vmicro $
         }


; --- Write results into save file ---
save, file=SavFile, MetaRaw, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu, /verbose



; --- Define input opacity file(s) ---
;OpaSuffix=''
;OpaSuffix='_05'
;OpaSuffix='_10'

;;OpaRawFiles='kapnu_solabu_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu.sav' & OpSSavFile='coma_kapsc_solabu.sav'
;;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu_h2obt_10.sav' & OpSSavFile=''
;;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu_h2obt_10.sav' & OpSSavFile='coma_kapsc_solabu_10.sav'

;kappanu=kappanu[*,*,0:*:10] & nu=nu[0:*:10] & lambda=lambda[0:*:10]
;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu

;OpaRawFiles='kapnu_solabu_p[123]' & OpSRawFiles='kapsc_solabu_p[123]' & OpaSavFile='coma_kapsc_solabu_10.sav'
;kappanu=kappanu[*,*,0:*:10] & nu=nu[0:*:10] & lambda=lambda[0:*:10]
;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu
;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu

;if (OpSSavFile ne '') then begin &$
;  restore, OpSSavFile, /verbose &$
;  kappaSc=temporary(kappanu) &$
;endif

;restore, OpaSavFile, /verbose
;kappaAb=kappanu
;if (OpSSavFile ne '') then begin &$
;  kappanu=kappanu+kappaSc &$
;endif
;kappaSc=0
;kappaAb=0


;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='kapsc_solabu_p[123]' & OpaSavFile='coma_kapnu_solabu_h2obt' + OpaSuffix + '.sav' & OpSSavFile='coma_kapsc_solabu' + OpaSuffix + '.sav'
;OpTSavFile='coma_kapto_solabu_h2obt' + OpaSuffix + '.sav' 

; --- Load input opacity data data ---
;restore, OpTSavFile, /verbose
;if (n_elements(EleSym) eq 0) then EleSym=EleNam

; --- Optionally: skip some frequency points ---
;i=3 & kappanu=kappanu[*,*,i:*:10] & nu=nu[i:*:10] & lambda=lambda[i:*:10]
;i=0 & iB=2 & kappanu=kappanu[*,*,i:*:iB] & nu=nu[i:*:iB] & lambda=lambda[i:*:iB]
;i=3 & iB=5 & kappanu=kappanu[*,*,i:*:iB] & nu=nu[i:*:iB] & lambda=lambda[i:*:iB]
;OpTSavFile='coma_kapto_solabu_h2obt_05.sav'

