;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   coma_opac_process ('coma_opacity_process')
;
; PURPOSE:
;   Read COMA raw opacity data and put them into IDL save file together with some meta data.
;
; CATEGORY:
;   Opacities
;
; CALLING SEQUENCE:
;   @coma_opac_process
;
; EXAMPLE:
;   @coma_opac_process
;
; MODIFICATION HISTORY:
;   2015-08-27 (B.F. Uppsala) Put into it's own routine
;-
;---------------------------------------------------------------------------------------------------

; --- Set the raw-data file names ---
;vmicro=2.5 & OpaSavFile='coma_kapnu_solabu.sav' & OpSSavFile='coma_kapsc_solabu.sav' & OpTSavFile='coma_kapto_solabu.sav'
vmicro=2.5 & OpaSavFile='coma_kapnu_solabu_h2obt2.sav' & OpSSavFile='coma_kapsc_solabu.sav' & OpTSavFile='coma_kapto_solabu_h2obt2.sav' 
;vmicro=20.0 & OpaSavFile='coma_kapnu_solabu_h2obt2_xi20.sav' & OpSSavFile='coma_kapsc_solabu_h2obt2_xi20.sav' & OpTSavFile='coma_kapto_solabu_h2obt2_xi20.sav' 


; --- Load scattering data ---
if (OpSSavFile ne '') then begin &$
  restore, OpSSavFile, /verbose &$
  kappaSc=temporary(kappanu) &$
  MetaRawSC=MetaRaw &$
endif

; --- Load absorption data ---
restore, OpaSavFile, /verbose
MetaRawAb=MetaRaw
;kappaAb=kappanu


; --- Add absorption and scattering ---
if (OpSSavFile ne '') then begin &$
  kappanu=kappanu+kappaSc &$
endif
kappaSc=0
;kappaAb=0

; --- Combine meta data ---
MetaRaw.FileOriginal=MetaRawAb.FileOriginal + ' + ' + MetaRawSc.FileOriginal
MetaRaw.FileSource  =OpTSavFile

;MetaRaw= { $
;  Description: 'Meta data of raw opacities', $
;  FileOriginal:      RawFiles, $
;  FileSource:        SavFile, $
;  FilePath:          OpaRawPath, $
;  Code:              'COMA', $
;  CodeVersion:       'COMA 11a, B.Aringer 2012', $
;  Author:            'Bernhard Aringer', $
;  AbundanceSorce:    'solar (Paola)', $
;  AbundanceID:       'Paola', $
;  ElementSymbols:    strupcase(EleSym), $
;  ElementAbundances: EleAbu, $
;  abuX:              abuX, $
;  abuY:              abuY, $
;  abuZ:              abuZ, $
;  X:                 X, $
;  Y:                 Y, $
;  Z:                 Z, $
;  Qmol:              Qmol, $
;  Vmicro:            vmicro $
;         }


; --- Write results into save file ---
SavFile=OpTSavFile
save, file=SavFile, MetaRaw, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu, /verbose

; --- Resample data ---
kappanu=kappanu[*,*,0:*:5] & nu=nu[0:*:5] & lambda=lambda[0:*:5]

; --- Write results into save file ---
SavFile=strReplaceBF(OpTSavFile, '.sav', '_5.sav')
save, file=SavFile, MetaRaw, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu, /verbose

; --- Resample data (once again) ---
kappanu=kappanu[*,*,0:*:2] & nu=nu[0:*:2] & lambda=lambda[0:*:2]

; --- Write results into save file ---
SavFile=strReplaceBF(OpTSavFile, '.sav', '_10.sav')
save, file=SavFile, MetaRaw, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu, /verbose



; --- Define input opacity file(s) ---
;OpaSuffix=''
;OpaSuffix='_05'
;OpaSuffix='_10'

;;OpaRawFiles='kapnu_solabu_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu.sav' & OpSSavFile='coma_kapsc_solabu.sav'
;;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu_h2obt_10.sav' & OpSSavFile=''
;;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='' & OpaSavFile='coma_kapnu_solabu_h2obt_10.sav' & OpSSavFile='coma_kapsc_solabu_10.sav'

;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu

;OpaRawFiles='kapnu_solabu_p[123]' & OpSRawFiles='kapsc_solabu_p[123]' & OpaSavFile='coma_kapsc_solabu_10.sav'
;kappanu=kappanu[*,*,0:*:10] & nu=nu[0:*:10] & lambda=lambda[0:*:10]
;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu
;save, file=OpaSavFile, OpaRawFiles, OpSRawFiles, kappanu, P, T, rho, kappaRoss, kappaPlan, lambda, nu, EleSym, EleAbu


;OpaRawFiles='kapnu_solabu_h2obt2_p[123]' & OpSRawFiles='kapsc_solabu_p[123]' & OpaSavFile='coma_kapnu_solabu_h2obt' + OpaSuffix + '.sav' & OpSSavFile='coma_kapsc_solabu' + OpaSuffix + '.sav'
;OpTSavFile='coma_kapto_solabu_h2obt' + OpaSuffix + '.sav' 

; --- Load input opacity data data ---
;restore, OpTSavFile, /verbose
;if (n_elements(EleSym) eq 0) then EleSym=EleNam

; --- Optionally: skip some frequency points ---
;i=3 & kappanu=kappanu[*,*,i:*:10] & nu=nu[i:*:10] & lambda=lambda[i:*:10]
;i=0 & iB=2 & kappanu=kappanu[*,*,i:*:iB] & nu=nu[i:*:iB] & lambda=lambda[i:*:iB]
;i=3 & iB=5 & kappanu=kappanu[*,*,i:*:iB] & nu=nu[i:*:iB] & lambda=lambda[i:*:iB]
;OpTSavFile='coma_kapto_solabu_h2obt_05.sav'

