;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   elements_extract ('elements_extract')
;
; PURPOSE:
;   Read a file with atomic weights of the elements and put the data into a structure.
;
; CATEGORY:
;   Abundances
;
; CALLING SEQUENCE:
;   @elements_extract
;
; EXAMPLE:
;   @elements_extract
;
; MODIFICATION HISTORY:
;   2015-08-26 (B.F. Uppsala) Written
;-
;---------------------------------------------------------------------------------------------------

; --- Define file name and give some meta data ---
EleDisk=getenv('HOME') + '/dat/abu/IUPAC/'
EleID  ='IUPAC_2013_Atomic_Weights'
EleFile=EleID + '.tab'
EleWeb ='http://www.chem.qmul.ac.uk/iupac/AtWt/'
EleRef ='Pure Appl. Chem., 2013, 85, 1047-1078'
EleOut =EleID + '.sav'

; --- Read the file with the element abundances ---
RdArray, EleDisk + EleFile, a, s, /notransform
help, s
print, s[0, *]

; --- Put the data into a structure ---
Ele= { $
  ID:        EleID, $
  File:      EleFile, $
  Path:      EleDisk, $
  Webpage:   EleWeb, $
  Reference: EleRef, $
  Date:      strDate(systime(/JULIAN)), $
  Z:          long(reform(s[1:*, 0])), $
  Symbol:          reform(s[1:*, 1]), $
  Name:            reform(s[1:*, 2]), $
  Weight:    float(reform(s[1:*, 3])) $
     }

help, /str, Ele

; --- Save the output structure to a file ---
save, file=EleOut, Ele

; --- Compute abundances with a mean molecular weight for the metals---
wX=1.0080 & wY=4.0026 & wZ=17.8351
help, wX,wY,wZ
;
abuTot=total(10.0^EleAbu)
abuX=10.0^EleAbu[0]/abuTot
abuY=10.0^EleAbu[1]/abuTot
abuZ=1.0-abuX-abuY
help, abuX, abuY, abuZ
Qmol=abuX*wX+abuY*wY+abuZ*wZ
help, Qmol
;
X=abuX*wX/Qmol
Y=abuY*wY/Qmol
Z=abuZ*wZ/Qmol
help, X, Y, Z

; --- Compute abundances --- 
EleInd=strCrossCmp(strtrim(EleSym, 2), Ele.Symbol, v2=Ele.Weight, Default=0.0, v1=EleWeight, /FOLD_CASE)
;
EA =10.0^EleAbu
EAT=total(EA)
abuX=EA[0]/EAT
abuY=EA[1]/EAT
abuZ=total(EA[2:*])/EAT
help, abuX, abuY, abuZ, abuX+abuY+abuZ
;
EW =EA * EleWeight
EWT=total(EW)
X=EW[0]/EWT
Y=EW[1]/EWT
Z=total(EW[2:*])/EWT
Qmol=EWT/EAT
help, X, Y, Z, X+Y+Z, Qmol

