; pro greyintegrate_start
;
;------------------------------------------------------------------------------
;+
; NAME:
;   greyinstart ('grey_integrate_start')
;
; PURPOSE:
;   Integrate a grey atmosphere in radiative equilibrium by calling greyintegrate.
;
; CATEGORY:
;   Computations (Special)
;
; CALLING SEQUENCE:
;   @greyintegrate_start
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   tabinter_rdcoeff;
;   dfopta:
;   greyintegrate:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   Before first use type
;     delvar, init
;
; PROCEDURE:
;   Call greyintegrate
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   28.10.94 Written by Bernd Freytag, Astrophysik Kiel
;    2.11.94 (B.F.)
;   28. 4.99 (B.F. Copenhagen) greyinstart -> greyintegrate_start
;-
;------------------------------------------------------------------------------
;
; --- Define EOS/OPA paths and files ---
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
gasfile='gas_mm00_l.eos' & abuX0='sun' & gasname='gas'
;opafile='g2va.opta'
;opafile='big_grey.opta'
opafile='phoenix_opal_grey.opta'
;
; --- Define model data ---
;model='sun1d'
;Teff=5770.
;logg=4.44
;Ptop=4000.0
;Pbot=3.5E+05
;dlntau=0.15
;approximation='singleray'     ; --- 'eddington', 'mihalas' --
;
;model='astar1d'
;Teff=8500.
;logg=4.44
;;Ptop=20.0 & Pbot=2.0E+06
;Ptop=1.0E-01 & Pbot=2.0E+06
;dlntau=0.15
;npoints=55
;aspect=1.0
;approximation='singleray'     ; --- 'eddington', 'mihalas' --
;
;model='astar1d'
;Teff=8500.
;logg=4.44
;Ptop=1.0E-01 & Pbot=2.0E+06
;dlntau=0.15
;npoints=200
;aspect=4.0
;approximation='mihalas'     ; --- 'eddington', 'mihalas' --

;model='at70g43n01'
;Teff=7000.
;Tmax=130000.0
;logg=4.30
;Ptop=3.0E+00 & Pbot=62.0E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=0.2E+00
;approximation='mihalas'

;model='at71g43n01'
;Teff=7100.
;Tmax=130000.0
;logg=4.30
;Ptop=1.0E+00 & Pbot=61.0E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=0.2E+00
;approximation='mihalas'

;model='at73g43n01'
;Teff=7300.
;Tmax=130000.0
;logg=4.30
;Ptop=1.0E+00 & Pbot=60.0E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=0.2E+00
;approximation='mihalas'

;model='at75g44n01'
;Teff=7500.
;Tmax=100000.0
;logg=4.40
;Ptop=3.0E+00 & Pbot=21.0E+06
;dlntau=0.15
;npoints=350
;n1=300 & n2=1
;aspect=3.0
;vampl=1.0E+04
;dtime=0.2E+00
;approximation='mihalas'

;model='at77g44n01'
;Teff=7700.
;Tmax=80000.0
;logg=4.40
;Ptop=3.0E+00 & Pbot=7.0E+06
;dlntau=0.15
;npoints=200
;n1=200 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+01
;approximation='mihalas'

;model='at77g44n02'
;Teff=7700.
;Tmax=85000.0
;logg=4.40
;Ptop=3.0E+00 & Pbot=10.0E+06
;dlntau=0.15
;npoints=300
;n1=300 & n2=1
;aspect=3.0
;vampl=1.0E+04
;dtime=0.3E+00
;approximation='mihalas'

;model='at77g44n03'
;Teff=7700.
;Tmax=100000.0
;logg=4.40
;Ptop=3.0E+00 & Pbot=20.3E+06
;dlntau=0.1
;npoints=400
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=0.1E+00
;approximation='mihalas'

model='at90g40n01'
Teff=9000.
Tmax=70000.0
logg=4.00
Ptop=0.5E+00 & Pbot=2.0E+06
dlntau=0.1
npoints=300
n1=300 & n2=1
aspect=2.0
vampl=1.0E+04
dtime=0.1E+00
approximation='mihalas'

;model='d1t36gm08n01'
;Teff=3600.
;Tmax=20000.0
;logg=-0.8
;Ptop=1.0E-02 & Pbot=5.0E+04
;dlntau=0.15
;npoints=100
;aspect=4.0
;dtime=1.0E+05
;approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
; model='d1t36gm08n02'
; Teff=3600.
; Tmax=200000.0
; logg=-0.8
; Ptop=2.0E-03 & Pbot=2.0E+04
; dlntau=0.15
; npoints=100
; aspect=4.0
; dtime=0.25E+05
; approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
; model='gt35gm04n01'
; Teff=3500.
; Tmax=30000.0
; logg=-0.4
; Ptop=5.0E+00 & Pbot=1.2E+03
; dlntau=0.15
; npoints=100
; n1=120 & n2=120
; aspect=1.0
; dtime=1.0E+02
; approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
;  model='gt35gm04n02'
;  Teff=3500.
;  Tmax=100000.0
;  logg=-0.4
;  Ptop=1.0E-03 & Pbot=3.80E+04
;  dlntau=0.15
;  npoints=118
;  n1=118 & n2=1
;  aspect=1.0
;  vampl=1.0E+04
;  dtime=1.0E+02
;  approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
; model='gt35gm04n04'
; Teff=3500.
; Tmax=100000.0
; logg=-0.4
; Ptop=1.0E-03 & Pbot=3.80E+04
; dlntau=0.15
; npoints=470
; n1=235 & n2=1
; aspect=2.0
; vampl=1.0E+04
; dtime=1.0E+02
; approximation='mihalas'     ; --- 'eddington', 'mihalas' --

;model='gt48g10n01'
;Teff=4800.0
;Tmax=100000.0
;logg=1.0
;Ptop=1.0E-04 & Pbot=9.0E+05
;dlntau=0.15
;npoints=600
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+01
;approximation='mihalas'

;model='gt49g20n01'
;Teff=4900.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-05 & Pbot=8.5E+06
;dlntau=0.10
;npoints=650
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt50g10n01'
;Teff=5000.0
;Tmax=130000.0
;logg=1.0
;Ptop=1.0E-04 & Pbot=2.3E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+01
;approximation='mihalas'

;model='gt50g20n01'
;Teff=5000.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-05 & Pbot=8.5E+06
;dlntau=0.10
;npoints=650
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt51g20n01'
;Teff=5100.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-05 & Pbot=8.0E+06
;dlntau=0.15
;npoints=650
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt52g20n01'
;Teff=5200.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=8.0E+06
;dlntau=0.15
;npoints=600
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt53g20n01'
;Teff=5300.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=7.5E+06
;dlntau=0.15
;npoints=600
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt54g20n01'
;Teff=5400.0
;Tmax=130000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=7.0E+06
;dlntau=0.15
;npoints=600
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt56g20n01'
;Teff=5600.0
;Tmax=100000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=2.2E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt58g20n01'
;Teff=5800.0
;Tmax=100000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=1.9E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt60g20n01'
;Teff=6000.0
;Tmax=100000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=1.9E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;model='gt62g20n01'
;Teff=6200.0
;Tmax=100000.0
;logg=2.0
;Ptop=1.0E-03 & Pbot=1.7E+06
;dlntau=0.15
;npoints=500
;n1=600 & n2=1
;aspect=2.0
;vampl=1.0E+04
;dtime=1.0E+00
;approximation='mihalas'

;  model='bt300g40n01'
;  Teff=30000.
;  Tmax=200000.0
;  logg=4.0
;  Ptop=5.0E+00 & Pbot=1.0E+06
;  dlntau=0.15
;  npoints=100
;  n1=120 & n2=1
;  aspect=2.0
;  vampl=1.0E+02
;  dtime=0.02
;  approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
; --- Load EOS and opacity data ---
if (n_elements(init) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + gasfile, gas &$
  dfopta, opadisk + opafile &$
  init=1 &$
endif
;
; --- Integrate grey statrification ---
print, 'Start integration'
greyintegrate, Teff=Teff, logg=logg, $
  gas=gas, $
  Ptop=Ptop, Pbot=Pbot, Tmax=Tmax, $
  dlntau=dlntau, approximation=approximation, $
  tauout=tau0, Tout=T0, Pout=P0, zout=z0
print, 'Integration done'
; --- Remove uppermost point ---
n0=n_elements(T0)-1
tau0=tau0(0:n0-1)
T0  =T0(  0:n0-1)
P0  =P0(  0:n0-1)
z0  =z0(  0:n0-1)
;
; --- Interpolate on equidistant z scale ---
if (1) then begin &$
  n=npoints &$
  z  =min(z0)+(max(z0)-min(z0))*findgen(n)/float(n-1) &$
  T  =          monocubic(     T0   , z0, z) &$
;  T  =sqrt(sqrt(monocubic(     T0^4 , z0, z))) &$
  P  =      exp(monocubic(alog(P0  ), z0, z)) &$
  tau=      exp(monocubic(alog(tau0), z0, z)) &$
endif else begin &$
  n=n_elements(T0) &$
  z  =z0 &$
  T  =T0 &$
  P  =P0 &$
  tau=tau0 &$
endelse
;
; --- Compute rho and ei ---
gasinter, P, T, rho, xcp, xgam, cxt, h, s, csound, eos=gas, ierror=ierror
ei=h-P/rho
;
!P.MULTI=[0,2,2,0,0]
plot, z, alog10(rho), psym=-1
plot, alog10(tau), alog10(rho), psym=-1
plot, z, T, psym=-1
plot, alog10(tau), T, psym=-1
!P.MULTI=0
;
filename=model + '.sta' & fileform='unformatted' & fileconv='ieee_4'
version='greyintegrate 2009.09.17'
description=['Grey hydrostatic atmosphere', $
             strtrim(systime(0),2)]
history=['Produced by greyintegrate.pro: '+strtrim(systime(0),2)]

dx3=z(1)-z(0)
dx1=aspect*(z(1)-z(0))
dx2=aspect*(z(1)-z(0))
help, dx1, dx2, dx3

rhd_wrboxstrat, $
  filename=filename, fileform=fileform, fileconv=fileconv, $
  description=description, $
  history=history, $
  version=version, $
  zarr=z, rhoarr=rho, eiarr=ei, vampl=vampl, $
  dx1=dx1, dx2=dx2, dtime=dtime, n1=n1, n2=n2
;
;print, max(z)-min(z)
;print,max(ful.z.xc3)
;
;j=n_elements(T)
;plothdinit, /init, $
;  xname='x', yname='z', zname='ti', $
;  abuX=abuX0, $
;  pin=reform(p,1,j), Tin=reform(T,1,j), $
;  uin=fltarr(1,j), vin=fltarr(1,j), $
;  xin=[0.0], yin=z, zin=[0.0], optafile=opadisk+opafile, gasname=gasname, $
;  modelname=model, Teff=Teff, logg=logg
;plothd, '', xtosystem='kmgs', ytosystem='kmgs', t_color=!ctv.colorrange1, $
;            xvectosys='kmgs', yvectosys='kmgs', date=0, $
;            language='english', /save
;plothd, 't', /trans

;
; end ; greyintegrate_start
