; pro opta_manipulate
;
;------------------------------------------------------------------------------
;+
; NAME:
;   opta_manipulate ('opacity_table_manipulate')
;
; PURPOSE:
;   Load, manipulate, and write an opacity table.
;
; CATEGORY:
;   opacities
;
; CALLING SEQUENCE:
;   @opta_manipulate
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-02-11 (B.F. Uppsala) 
;-
;------------------------------------------------------------------------------

opaFileIn='phoenix_opal_grey.opta'
OpaNameOut='phoenix_opal_greynodust01'

opaFileOut=OpaNameOut + '.opta'
outDisk=getenv('HOME') + '/idlwrk/'
;eosDisk=getenv('HOME') + '/for/eos/dat/'
opaDisk=getenv('HOME') + '/for/opa/dat/'

; --- Read input opacity table ---
rdopta, opaDisk + opaFileIn, opaIn

; --- Find approximate curve in P/T index plane that separates dusty and dust-free region ---
nP=n_elements(opaIn.tabp)
xP=findgen(nP)
yT=round(18.0 + 0.06*xP + 6.0E-4*xP^2)


plotfield, /tv, /oldasp, reform(opaIn.xopta[*,*,0])
plotlines, round(yT), xP, /over


; --- Copy table ---
opaOut=opaIn

; --- Modify table in low-temperature region to suppress contribution of dust ---
for ip=0,nP-1 do begin &$
  iT=yt[iP] &$
  print, iP, iT, opaIn.xopta[iT,iP,0] &$
  opaOut.xopta[0:iT-1,iP,0]=opaIn.xopta[iT,iP,0] &$
  opaOut.xopta[0:iT-1,iP,1]=opaIn.xopta[iT,iP,1] &$
endfor ; ip

; --- Modify header ---
opaOut.DATEX =systime(0)
opaOut.OPTANA=OpaNameOut


; --- Write output opacity table ---
wropta, outDisk + opaFileOut, opaOut, mode=1

wait, 1
plotfield, /tv, /oldasp, reform(opaOut.xopta[*,*,0])
plotlines, round(yT), xP, /over



; end ; opta_manipulate
