; pro rhd_addboxes
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_addboxes ('radiation_hydrodynamics_add_boxes')
;
; PURPOSE:
;   Add models arrays.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_addboxes
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos, par
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   24. 7.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
;modelident=''
;model='st35gm04n28' & modelident='_??' & parmodelident='_55'
model='st35gm04n29' & modelident='_??' & parmodelident='_30'
;model='st35gm04n30' & modelident='_??' & parmodelident='_15'
outdisk='/p4/bf/idl/wrk/'
outfile=outdisk + model + '_sum.full'
;
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;eosdisk=getenv('HOME') + '/for/eos/dat/'
;opadisk=getenv('HOME') + '/for/opa/dat/'
;
modelfile=modeldisk + model + modelident + '.full'
;
; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')
;
; --- Read EOS and opacity file (not active: you need the appropriate files) ---
;if (n_elements(EOS) eq 0) then begin &$
;  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;  dfopta, opadisk + par.opafile &$
;endif
;f0=!con.sigmasb*par.Teff^4
;
n_timestep=10000    ; --- Some huge value to get everything. Reduce for tests! ---
ierr=0
i=0
ibox=0
iboxtot=16
delvar,listdata
;
; --- Loop over all datasets ---
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    print, '--- ', i, ibox, '/', iboxtot, ful.z.itime, ful.z.time, ' ---' , $
           format='(A,I4,I4,A,I4,I8,E15.8,A)' &$
    ;
    ; --- Compute EOS and kappa (not active) ---
    ; eosbox, ful, eos=EOS, /opa, ierror=ierror & help,ierror &$
    ;
    ; --- Add boxes ---
    if (ibox eq 0) then begin &$
      fulsum=temporary(ful) &$
      ibox=ibox+1 &$
    endif else begin &$
      addbox, fulsum, ful &$
      ibox=ibox+1 &$
      if (ibox eq iboxtot) then begin &$
        divbox, fulsum, ibox &$
        wrbox, fulsum, file=outfile, position='append' &$
        ibox=0 &$
      endif &$
    endelse &$
    ;
    i=i+1 &$
  endif &$
endwhile
;
if (ibox ne 0) then begin &$
  divbox, fulsum, ibox &$
  wrbox, fulsum, file=outfile, position='append' &$
endif

;
; end ; rhd_addboxes
