; pro rhd_area_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_area_collect ('radiation_hydrodynamics_area_collect')
;
; PURPOSE:
;   Collect data from full file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_area_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    9.10.03 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
; ================================
;
tauharm_limit=2./3.
T_limit      =5500.0        ; --- par.Teff*1.5, par.Teff*1.06358, 4000.0
;
; ================================
;
;model='st33gm06n03' & modelident='_[0-6]?' & parmodelident='_01'
;model='st33gm06n03' & modelident='_17' & parmodelident='_17'
;model='st35gm06n02' & modelident='_??' & parmodelident='_20'
;model='st35gm04n01' & modelident='_??' & parmodelident='_35'
;model='st35gm04n04' & modelident='_??' & parmodelident='_38' & n_timestep=162
;model='st47g20n05' & modelident='_??' & parmodelident='_01' & n_timestep=20
;model='st35gm04n32' & modelident='_15' & parmodelident='_15'
;
;modeldisk='/p1' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
modelfile=modeldisk + model + modelident + '.full'
;
; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')
;
; --- Read EOS file ---
;if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS
  dfopta, opadisk + par.opafile
;endif
f0=!con.sigmasb*par.Teff^4
;
delvar, listdata
ful=uio_datasetlist_rd(modelfile, listdata=listdata, blacklist=['v1','v2','v3'], ierr=ierr)
uio_closrd, listdata.channel
;
nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
time_first=ful.modeltime
;
uio_openrd, nc, listdata.file(listdata.total_number-1)
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, ierr, outstr, time_last &$
  endif &$
endwhile
uio_closrd, nc
;
n_timestep_first=round(time_first/par.dtime_out_full)
n_timestep_last =round(time_last/par.dtime_out_full)
n_timestep=n_timestep_last-n_timestep_first+1
n_timestep=n_timestep
;n_timestep=10   ; 319, 376, 256, 97, 105, 96
help, time_first, time_last, par.dtime_out_full, n_timestep_first, n_timestep_last, n_timestep
;
delvar, listdata
;
;n_tau =101
;logtau=findgen(n_tau)/10.0 - 5.0
;tau=10.0^logtau
;
area=create_struct( $
  'model'             , model, $
  'modelident'        , modelident, $
  'T_limit'           , T_limit, $
  'tauharm_limit'     , tauharm_limit, $
  'xc1'               , reform(ful.z.xc1), $
  'xc2'               , reform(ful.z.xc2), $
  'xc3'               , reform(ful.z.xc3), $
  'xb1'               , reform(ful.z.xb1), $
  'xb2'               , reform(ful.z.xb2), $
  'xb3'               , reform(ful.z.xb3), $
  'dx1'               , ful.z.xb1(1)-ful.z.xb1(0), $
  'dx2'               , ful.z.xb2(1)-ful.z.xb2(0), $
  'dx3'               , ful.z.xb3(1)-ful.z.xb3(0), $
  'itime'             , lonarr(            n_timestep), $
  'time'              , fltarr(            n_timestep), $
  'ifile'             , lonarr(            n_timestep), $
  'file'              , strarr(            n_timestep), $
  'itime_first'       , ful.z.itime, $
  'itime_last'        , -10000L, $
  'tau1'              , fltarr(nxc2, nxc3, n_timestep), $
  'tau2'              , fltarr(nxc1, nxc3, n_timestep), $
  'tau3'              , fltarr(nxc1, nxc2, n_timestep), $
  'masktauharm1'      , bytarr(nxc2, nxc3, n_timestep), $
  'masktauharm2'      , bytarr(nxc1, nxc3, n_timestep), $
  'masktauharm3'      , bytarr(nxc1, nxc2, n_timestep), $
  'n_ge_tauharm_limit', lonarr(            n_timestep), $
  'n_ge_tauharm_0100' , lonarr(            n_timestep), $
  'n_ge_tauharm_0666' , lonarr(            n_timestep), $
  'n_ge_tauharm_1000' , lonarr(            n_timestep), $
  'n_ge_tauharm_2000' , lonarr(            n_timestep), $
  'maskT1'            , bytarr(nxc2, nxc3, n_timestep), $
  'maskT2'            , bytarr(nxc1, nxc3, n_timestep), $
  'maskT3'            , bytarr(nxc1, nxc2, n_timestep), $
  'n_ge_T_limit'      , lonarr(            n_timestep), $
  'n_ge_T_3500'       , lonarr(            n_timestep), $
  'n_ge_T_4000'       , lonarr(            n_timestep), $
  'n_ge_T_5000'       , lonarr(            n_timestep), $
  'n_ge_T_6000'       , lonarr(            n_timestep) $
                  )
;
.run rhd_area_data
;
ierr=0
i=0
file=''
systime0=systime(1)
;ful=0
;
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful=0 &$
  ful=uio_datasetlist_rd(modelfile, listdata=listdata, blacklist=['v1','v2','v3'], ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=listdata.file(listdata.open_number) &$
    ;ful=temporary(ful0) &$
    if (ful.modelitime gt (area.itime_last+1)) then begin &$
      print, '--- ', i, ful.z.itime, ful.z.time, systime(1)-systime0, ' ---' , $
        format='(A,I4,X,I6,E15.6,1X,F8.1,A)' &$
      ;
      eosbox, ful, eos=EOS, /opa, out_quantities=1 &$
      ;
      file_tmp     =strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
      area.ifile(i)=listdata.open_number &$
      area.file(i) =file_tmp &$
      ;
      rhd_area_data, ful, area, i, $
        tauharm_limit, T_limit &$
      ;
      i=i+1 &$
    endif &$
  endif &$
endwhile
;
delvar,ful
;
save, filename=model + '_area.sav', area
;

;plotlines,(3.0/(4.0*!pi)*area.N_GE_TAUHARM_1000)^(0.333333)*area.dx1/!con.Rsun
;plotlines,alog10(7.5/((3.0/(4.0*!pi)*area.N_GE_TAUHARM_1000)^(0.333333)*area.dx1/!con.Rsun)^2*27500.)
;print,(69500./720.^2)^0.25*5770.
;plotlines, seq.LUMINOSITYPERVOLUME*(4.0*!pi/3.0*(1.0D0*par.r0_grav)^3)/!con.Lsun
;print,(46000./640.^2)^0.25*5770.
;print,(44000./630.^2)^0.25*5770.
;print,alog10(27500.0*(par.mass_star/!con.Msun)/640.^2)
;print,alog10(27500.0*(par.mass_star/!con.Msun)/630.^2)

; end ; rhd_area_collect
