; pro rhd_atm2box
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_atm2box ('grey_integrate_start')
;
; PURPOSE:
;   Read a 1D atmosphere model and generate RHD start model from it.
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   @rhd_atm2box
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   rdphoenix:  Load data array from atmosphere file produced by the PHOENIX program
;   tabinter_rdcoeff;
;   dfopta:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   Before first use type
;     delvar, init
;
; PROCEDURE:
;   Read 1D model atmosphere data,
;   compute rho, ei,
;   generate vertical grid
;   generate 3D output arrays and axes
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2007-11-15 (B.F. Lyon) Derived from greyintegrate_start.pro
;   2007-11-29 (B.F.)
;   2008-01-31 (B.F.) mt18g55mm00n02
;   2009-03-10 (B.F.) Use column 16 (height1) instead of column 15 (height) with rdphoenix20
;   2009-10-07 (B.F.) Optional smoothing of input structure (smooth_q0) added
;-
;------------------------------------------------------------------------------

; ==================================================
if (not keyword_set(external_control)) then begin &$
  ;stop &$
  ;
  smooth_q0=0 &$
  nhighsub=0 &$
  ;
  ;atmtype='phoenix' & atmfile='phoenix_atm_T1600g500_20071010.txt' &$
  ;model='mt16g50mm00n01' & ndeepadd=178 & dxb=90000.0 & n1=3 & n2=1 & n3=230 &$
  ;
  ;atmtype='phoenix' & atmfile='lte18-5.0-0.0.AMES-Rainout.txt' &$
  ;model='mt18g50mm00n05' &$
  ;
  ;atmtype='phoenixout' & atmfile='lte07-5.0-0.0.AMES-dusty.out' &$
  ;model='mt07g50mm00n01' & ndeepadd=195 & rhohighadd=8.0 & dxb=50000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenixout' & atmfile='lte08-5.0-0.0.AMES-dusty.out' &$
  ;model='mt08g50mm00n01' & ndeepadd=195 & rhohighadd=8.0 & dxb=55000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte09-3.5-0.0.AMES-dusty.20' &$
  ;atmtype='phoenix20' & atmfile='lte09-3.5-0.0.AMES-Cond-2000.20' &$
  ;model='mt09g35mm00n01' & ndeepadd=190 & rhohighadd=10.0 & dxb=2400000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte09-5.0-0.0.AMES-dusty.20' &$
  ;model='mt09g50mm00n01' & ndeepadd=195 & rhohighadd=3.0 & dxb=55000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte09-5.0-0.0.AMES-Cond-2000.20' &$
  ;model='mt09g50mm00n02' & ndeepadd=195 & rhohighadd=3.0 & dxb=55000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte10-5.0-0.0.AMES-dusty.20' &$
  ;model='mt10g50mm00n01' & ndeepadd=190 & rhohighadd=3.0 & dxb=60000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte11-4.0-0.0.AMES-dusty.20' &$
  ;model='mt11g40mm00n01' & ndeepadd=300 & rhohighadd=6.0 & dxb=950000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte11-5.0-0.0.AMES-dusty.20' &$
  ;model='mt11g50mm00n01' & ndeepadd=180 & rhohighadd=3.0 & dxb=65000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte11-5.0-0.0.AMES-dusty.20' &$
  ;model='mt11g50mm00n02' & ndeepadd=180 & rhohighadd=3.0 & dxb=65000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte12-4.0-0.0.AMES-dusty.20' &$
  ;model='mt12g40mm00n01' & ndeepadd=185 & rhohighadd=2.0 & dxb=1000000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte12-4.5-0.0.AMES-dusty.20' &$
  ;model='mt12g45mm00n01' & ndeepadd=175 & rhohighadd=3.0 & dxb=300000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte12-5.0-0.0.AMES-dusty.20' &$
  ;model='mt12g50mm00n01' & ndeepadd=165 & rhohighadd=3.0 & dxb=70000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte13-3.5-0.0.AMES-dusty.20' &$
  ;model='mt13g35mm00n01' & ndeepadd=170 & rhohighadd=24.0 & dxb=3200000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte13-4.0-0.0.AMES-dusty.20' &$
  ;model='mt13g40mm00n01' & ndeepadd=170 & rhohighadd=24.0 & dxb=1000000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte13-4.5-0.0.AMES-dusty.20' &$
  ;model='mt13g45mm00n01' & ndeepadd=180 & rhohighadd=15.0 & dxb=320000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte13-5.0-0.0.AMES-dusty.20' &$
  ;model='mt13g50mm00n01' & ndeepadd=170 & rhohighadd=3.0 & dxb=75000.0 & n1=400 & n2=1 & n3=270 &$
  ;atmtype='phoenix20' & atmfile='lte13-5.5-0.0.AMES-dusty.20' &$
  ;model='mt13g55mm00n01' & ndeepadd=220 & rhohighadd=22.0 & dxb=32000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte13-5.5-0.0.AMES-dusty.20' &$
  ;model='mt13g55mm00n02' & ndeepadd=220 & rhohighadd=22.0 & dxb=32000.0 & n1=300 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte14-5.0-0.0.AMES-dusty.20' &$
  ;model='mt14g50mm00n01' & ndeepadd=177 & rhohighadd=3.0 & dxb=80000.0 & n1=400 & n2=1 & n3=270 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte15-4.0-0.0.AMES-dusty.20' &$
  ;model='mt15g40mm00n01' & ndeepadd=200 & rhohighadd=15.0 & dxb=1150000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte15-5.0-0.0.AMES-dusty.20' &$
  ;model='mt15g50mm00n01' & ndeepadd=185 & rhohighadd=3.0 & dxb=85000.0 & n1=400 & n2=1 & n3=270 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte16-4.0-0.0.AMES-dusty.20' &$
  ;model='mt16g40mm00n01' & ndeepadd=240 & rhohighadd=8.0 & dxb=920000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte16-5.0-0.0.AMES-dusty.20' &$
  ;model='mt16g50mm00n03' & ndeepadd=192 & rhohighadd=3.0 & dxb=90000.0 & n1=400 & n2=1 & n3=270 &$
  ;atmtype='phoenix20' & atmfile='lte16-5.0-0.0.AMES-dusty.20' &$
  ;model='mt16g50mm00n06' & ndeepadd=221 & rhohighadd=8.0 & dxb=88000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte17-5.0-0.0.AMES-dusty.20' &$
  ;model='mt17g50mm00n01' & ndeepadd=199 & dxb=100000.0 & n1=400 & n2=1 & n3=250 &$
  ;atmtype='phoenix20' & atmfile='lte17-5.0-0.0.AMES-dusty.20' &$
  ;model='mt17g50mm00n02' & ndeepadd=212 & dxb=95000.0 & rhohighadd=8.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte18-3.5-0.0.AMES-dusty.20' &$
  ;model='mt18g35mm00n01' & ndeepadd=230 & rhohighadd=18.0 & dxb=4400000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte18-4.0-0.0.AMES-dusty.20' &$
  ;model='mt18g40mm00n01' & ndeepadd=204 & rhohighadd=12.0 & dxb=1350000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte18-4.5-0.0.AMES-dusty.20' &$
  ;model='mt18g45mm00n01' & ndeepadd=204 & rhohighadd=14.0 & dxb=420000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte18-5.0-0.0.AMES-dusty.20' &$
  ;model='mt18g50mm00n06' & ndeepadd=204 & dxb=100000.0 & n1=400 & n2=1 & n3=260 &$
  ;atmtype='phoenix20' & atmfile='lte18-5.0-0.0.AMES-dusty.20' &$
  ;model='mt18g50mm00n07' & ndeepadd=204 & rhohighadd=8.0 & dxb=100000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte18-5.5-0.0.AMES-dusty.20' &$
  ;model='mt18g55mm00n33' & ndeepadd=220 & rhohighadd=14.0 & dxb=42000.0 & n1=300 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte19-5.0-0.0.AMES-dusty.20' &$
  ;model='mt19g50mm00n01' & ndeepadd=211 & rhohighadd=3.0 & dxb=105000.0 & n1=400 & n2=1 & n3=260 &$
  ;atmtype='phoenix20' & atmfile='lte19-5.0-0.0.AMES-dusty.20' &$
  ;model='mt19g50mm00n02' & ndeepadd=211 & rhohighadd=8.0 & dxb=105000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte20-5.0-0.0.AMES-dusty.20' &$
  ;model='mt20g50mm00n01' & ndeepadd=215 & rhohighadd=3.0 & dxb=110000.0 & n1=400 & n2=1 & n3=260 &$
  ;atmtype='phoenix20' & atmfile='lte20-5.0-0.0.AMES-dusty.20' &$
  ;model='mt20g50mm00n02' & ndeepadd=215 & rhohighadd=8.0 & dxb=110000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte20-5.0-0.0.AMES-dusty.20' &$
  ;model='mt20g50mm00n03' & ndeepadd=188 & rhohighadd=8.0 & dxb=95000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte21-5.0-0.0.AMES-dusty.20' &$
  ;model='mt21g50mm00n01' & ndeepadd=219 & dxb=115000.0 & n1=400 & n2=1 & n3=260 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte22-4.0-0.0.AMES-dusty.20' &$
  ;model='mt22g40mm00n01' & ndeepadd=260 & rhohighadd=12.0 & dxb=1600000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte22-5.0-0.0.AMES-dusty.20' &$
  ;model='mt22g50mm00n01' & ndeepadd=223 & rhohighadd=3.0 & dxb=120000.0 & n1=400 & n2=1 & n3=260 &$
  ;atmtype='phoenix20' & atmfile='lte22-5.0-0.0.AMES-dusty.20' &$
  ;model='mt22g50mm00n02' & ndeepadd=219 & rhohighadd=8.0 & dxb=120000.0 & n1=400 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte22-5.0-0.0.AMES-dusty.20' &$
  ;model='mt22g50mm00n04' & ndeepadd=171 & rhohighadd=8.0 & dxb=100000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte24-4.0-0.0.AMES-dusty.20' &$
  ;model='mt24g40mm00n01' & ndeepadd=230 & rhohighadd=10.0 & dxb=1500000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte24-5.0-0.0.AMES-dusty.20' &$
  ;model='mt24g50mm00n01' & ndeepadd=152 & rhohighadd=8.0 & dxb=105000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  atmtype='phoenix20' & atmfile='lte26-3.5-0.0.AMES-dusty.20' &$
  model='mt26g35mm00n01' & ndeepadd=260 & rhohighadd=35.0 & dxb=5200000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte26-4.0-0.0.AMES-dusty.20' &$
  ;model='mt26g40mm00n01' & ndeepadd=230 & rhohighadd=24.0 & dxb=1600000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte26-4.5-0.0.AMES-dusty.20' &$
  ;model='mt26g45mm00n01' & ndeepadd=200 & rhohighadd=24.0 & dxb=500000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte26-5.0-0.0.AMES-dusty.20' &$
  ;model='mt26g50mm00n01' & ndeepadd=132 & rhohighadd=8.0 & dxb=105000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte28-4.0-0.0.AMES-dusty.20' &$
  ;model='mt28g40mm00n01' & ndeepadd=200 & rhohighadd=13.0 & dxb=1600000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte28-5.0-0.0.AMES-dusty.20' &$
  ;model='mt28g50mm00n01' & ndeepadd=110 & rhohighadd=8.0 & dxb=110000.0 & n1=400 & n2=1 & n3=0 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte30-4.0-0.0.AMES-dusty.20' &$
  ;model='mt30g40mm00n01' & ndeepadd=140 & rhohighadd=3.0 & dxb=1600000.0 & n1=300 & n2=1 & n3=310 &$
  ;atmtype='phoenix20' & atmfile='lte30-5.0-0.0.AMES-dusty.20' &$
  ;model='mt30g50mm00n01' & ndeepadd=110 & rhohighadd=8.0 & dxb=160000.0 & n1=300 & n2=1 & n3=0 &$
  ;atmtype='phoenix20' & atmfile='lte30-5.0-0.0.AMES-dusty.20' &$
  ;model='mt30g50mm00n03' & ndeepadd=200 & rhohighadd=8.0 & dxb=180000.0 & n1=300 & n2=1 & n3=260 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte32-4.0-0.0.AMES-dusty.20' &$
  ;model='mt32g40mm00n01' & ndeepadd=180 & rhohighadd=3.0 & dxb=1800000.0 & n1=300 & n2=1 & n3=320 &$
  ;atmtype='phoenix20' & atmfile='lte32-5.0-0.0.AMES-dusty.20' &$
  ;model='mt32g50mm00n01' & ndeepadd=190 & rhohighadd=3.0 & dxb=180000.0 & n1=300 & n2=1 & n3=280 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte33-2.5-0.0.AMES-Cond-2000.20' &$
  ;model='mt33g25mm00n01' & ndeepadd=2000 & rhohighadd=0.0 & nhighsub=3 & dxb=1.0E+08 & n1=300 & n2=1 & n3=400 &$
  ;atmtype='phoenix20' & atmfile='lte33.0-2.50-0.0.BT-NextGen.20' &$
  ;model='mt33g25mm00n02' & ndeepadd=2000 & rhohighadd=0.0 & nhighsub=3 & dxb=1.0E+08 & n1=300 & n2=1 & n3=400 &$
  ;atmtype='phoenix20' & atmfile='lte33-3.0-0.0.AMES-Cond-2000.20' &$
  ;model='mt33g30mm00n01' & ndeepadd=1600 & rhohighadd=3.0 & dxb=26000000.0 & n1=300 & n2=1 & n3=400 &$
  ;atmtype='phoenix20' & atmfile='lte33-3.0-0.0.AMES-Cond-2000.20' &$
  ;model='mt33g30mm00n02' & ndeepadd=1600 & rhohighadd=0.0 & nhighsub=5 & dxb=26000000.0 & n1=300 & n2=1 & n3=400 &$
  ;atmtype='phoenix20' & atmfile='lte33-3.5-0.0.AMES-Cond-2000.20' &$
  ;model='mt33g35mm00n01' & ndeepadd=1100 & rhohighadd=3.0 & dxb=8500000.0 & n1=300 & n2=1 & n3=340 &$
  ;atmtype='phoenix20' & atmfile='lte33-4.0-0.0.AMES-dusty.20' &$
  ;model='mt33g40mm00n01' & ndeepadd=360 & rhohighadd=3.0 & dxb=2500000.0 & n1=300 & n2=1 & n3=295 &$
  ;atmtype='phoenix20' & atmfile='lte33-4.0-0.0.AMES-dusty.20' &$
  ;model='mt33g40mm00n02' & ndeepadd=480 & rhohighadd=4.0 & dxb=2500000.0 & n1=300 & n2=1 & n3=320 &$
  ;atmtype='phoenix20' & atmfile='lte33-4.5-0.0.AMES-dusty.20' &$
  ;model='mt33g45mm00n01' & ndeepadd=450 & rhohighadd=4.0 & dxb=800000.0 & n1=300 & n2=1 & n3=310 &$
  ;atmtype='phoenix20' & atmfile='lte33-5.0-0.0.AMES-dusty.20' &$
  ;model='mt33g50mm00n01' & ndeepadd=340 & rhohighadd=3.0 & dxb=250000.0 & n1=300 & n2=1 & n3=260 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte35-4.0-0.0.AMES-dusty.20' &$
  ;model='mt35g40mm00n01' & ndeepadd=180 & rhohighadd=3.0 & dxb=1900000.0 & n1=300 & n2=1 & n3=330 &$
  ;atmtype='phoenix20' & atmfile='lte35-4.0-0.0.AMES-dusty.20' &$
  ;model='mt35g40mm00n02' & ndeepadd=400 & rhohighadd=5.0 & dxb=2200000.0 & n1=300 & n2=1 & n3=345 &$
  ;atmtype='phoenix20' & atmfile='lte35-4.0-0.0.AMES-dusty.20' &$
  ;model='mt35g40mm00n04' & ndeepadd=900 & rhohighadd=6.0 & dxb=2600000.0 & n1=300 & n2=1 & n3=370 &$
  ;atmtype='phoenix20' & atmfile='lte35-5.0-0.0.AMES-dusty.20' &$
  ;model='mt35g50mm00n01' & ndeepadd=190 & rhohighadd=3.0 & dxb=190000.0 & n1=300 & n2=1 & n3=320 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte36.0-0.00-0.0.BT-NextGen.20' &$
  ;model='mt36g00mm00n01' & ndeepadd=1600 & rhohighadd=0.0 & nhighsub=0 & dxb=3.0E+10 & n1=300 & n2=1 & n3=400 &$
  ;atmtype='phoenix20' & atmfile='lte36.0-3.00-0.0.BT-NextGen.20' &$
  ;model='mt36g30mm00n01' & ndeepadd=1600 & rhohighadd=0.0 & nhighsub=0 & dxb=3.0E+07 & n1=300 & n2=1 & n3=400 &$
  ;
  ;atmtype='phoenix20' & atmfile='lte40-5.0-0.0.AMES-dusty.20' &$
  ;model='mt40g50mm00n01' & ndeepadd=330 & rhohighadd=0.0 & dxb=260000.0 & n1=300 & n2=1 & n3=380 &$
  ;atmtype='phoenix20' & atmfile='lte40-5.0-0.0.AMES-dusty.20' &$
  ;model='mt40g50mm00n02' & ndeepadd=500 & rhohighadd=0.0 & dxb=340000.0 & n1=300 & n2=1 & n3=350 & smooth_q0=3 &$
  ;
  ;atmtype='phoenix' & atmfile='phoenix_atm_T1800g550_20071010.txt' &$
  ;model='mt18g55mm00n01' & n1=3 & n2=1 & n3=300 &$
  ;model='mt18g55mm00n02' &$
  ;model='mt18g55mm00n13' & ndeepadd=320 & dxb=25000.0 & n1=3 & n2=1 & n3=300 &$
  ;model='mt18g55mm00n22' & ndeepadd=188 & dxb=30000.0 & n1=3 & n2=1 & n3=260 &$
  ;
  vampl=1.0E+02 &$
  ;dtime=1.0E-03 &$
  dtime=3.0E-02 &$
  ;dtime=2.0E+00 &$
  ms =140.71*!con.muAtom &$                         ; (2.0*24.31+1.0*28.09+4.0*16.0)
  dustfrac=6.0324E-04/(2.0*24.31)*ms/!con.muAtom &$ ; AGS abu, Mg2SiO4&$
  eps=1.55935e-05 &$
  ;
endif
; ==================================================

if (strmatch(atmfile, '*AMES-Dusty*', /fold_case)) then begin &$
  atmdisk=getenv('HOME') + '/dat/atm/phoenix/AMES-Dusty/' &$
endif else if (strmatch(atmfile, '*AMES-Cond*', /fold_case)) then begin &$
  atmdisk=getenv('HOME') + '/dat/atm/phoenix/AMES-Cond/' &$
endif else if (strmatch(atmfile, '*BT-NextGen*', /fold_case)) then begin &$
  atmdisk=getenv('HOME') + '/dat/atm/phoenix/BT-NextGen/' &$
endif else begin &$
  message, 'Unknown class of atmosphere', /info &$
  stop &$
endelse

; --- Define EOS/OPA paths and files ---
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
eosfile='eos_mm00_l5.eos' & gasfile='gas_mm00_l.eos' & abuX0='sun' & gasname='gas'
;opafile='g2va.opta'
;opafile='big_grey.opta'
opafile='phoenix_opal_grey.opta'

; --- Load EOS and opacity data ---
if (n_elements(init) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + gasfile, GAS &$
  tabinter_rdcoeff, eosdisk + eosfile, EOS &$
  dfopta, opadisk + opafile &$
  init=1 &$
endif

if (atmtype eq 'phoenix') then begin &$
  rdphoenix, atmdisk+atmfile, r, Teff=Teff, grav=grav &$
  logg=alog10(grav) &$
  T0  =reverse(reform(r[ 2,*])) &$
  p0  =reverse(reform(r[ 3,*])) &$
  z0  =reverse(reform(r[13,*])) &$
  ;tau0=reverse(reform(r[12,*])) &$
endif else if (atmtype eq 'phoenix20') then begin &$
  rdphoenix20, atmdisk+atmfile, r, Teff=Teff, grav=grav &$
  logg=alog10(grav) &$
  T0  =reverse(reform(r[ 1,*])) &$
  p0  =reverse(reform(r[ 8,*])) &$
  ;z0  =reverse(reform(r[15,*])) &$
  z0  =reverse(reform(r[16,*])) &$
  ;tau0=reverse(reform(r[0,*])) &$
endif else if (atmtype eq 'phoenixout') then begin &$
  rdphoenixout, atmdisk+atmfile, r, Teff=Teff, grav=grav &$
  logg=alog10(grav) &$
  T0  =reverse(reform(r[ 2,*])) &$
  p0  =reverse(reform(r[ 3,*])) &$
  z0  =reverse(reform(r[13,*])) &$
  ;tau0=reverse(reform(r[12,*])) &$
endif else begin &$
  message, 'Type of atmosphere not known' &$
endelse

; --- Remove uppermost point ---
if (nhighsub ge 1) then begin &$
  n0=n_elements(T0)-nhighsub &$
;  tau0=tau0(0:n0-nhighsub) &$
  T0  =T0(  0:n0-nhighsub) &$
  P0  =P0(  0:n0-nhighsub) &$
  z0  =z0(  0:n0-nhighsub) &$
endif

; --- Smooth ---
if (smooth_q0 gt 0) then begin &$
  T0=smooth(T0, smooth_q0) &$
  P0=exp(smooth(alog(P0), smooth_q0)) &$
endif

; --- Compute rho and ei ---
gasinter, P0, T0, rho0, xcp0, xgam0, cxt0, h0, s0, csound0, eos=GAS, ierror=ierror
ei0=h0-P0/rho0
n0=n_elements(z0   )

; --- Point at the top ---
if (keyword_set(rhohighadd)) then begin &$
  Hptop=P0[n0-1]/(grav*rho0[n0-1]) &$
  help, Hptop &$
  rhohigh=[rho0[n0-1]/rhohighadd] &$
  eihigh=[ei0[n0-1]] &$
  zhigh=[z0[n0-1]+Hptop*alog(rhohighadd)] &$
  nhigh=1 &$
endif else begin &$
  nhigh=0 &$
endelse

; --- Integrate hydrostatic stratification with constant entropy ---
stratintegrate, reform(ei0[0]), reform(rho0[0]), reform(z0[0]), eideep, rhodeep, zdeep, $
                dz=0.1*reform(z0[0]-z0[1]), grav=grav, npoint=ndeepadd, eos=EOS

; --- Merge atm and deep and high stratification ---
ndeep=n_elements(zdeep)
n    =n0+ndeep+nhigh-1
rho=fltarr(n)
ei =fltarr(n)
z  =fltarr(n)
rho[0:ndeep-1]=rhodeep
ei[ 0:ndeep-1]=eideep
z[  0:ndeep-1]=zdeep
rho[ndeep-1:n0+ndeep-2]=rho0
ei[ ndeep-1:n0+ndeep-2]=ei0
z[  ndeep-1:n0+ndeep-2]=z0
if (nhigh > 0) then begin &$
  rho[n0+ndeep-1:*]=rhohigh &$
  ei[ n0+ndeep-1:*]=eihigh &$
  z[  n0+ndeep-1:*]=zhigh &$
endif

; --- Compute EOS for merged stratification ---
eosinter, rho, ei, P, dPdrho, dPdei, T, dTdei, s, eos=EOS

; --- Modify grid to smooth out grid size changes ---
;dzb=deriv(-alog(P),z)/25.0
;help,p,z,dzb
;dzb=replicate(0.4,n_elements(z))
;dzb[where(T lt Teff*0.8)]=0.45
;dzb[where(T lt Teff*0.75)]=0.5
;;dzb[where((T lt Teff*1.10) and (T gt Teff*0.83))]=0.4
;;dzb[where(T gt Teff*1.1)]=0.45
;;dzb[where((T lt Teff*1.2) and (T gt Teff*0.78))]=0.4
;dzb[where(T gt Teff*1.15)]=0.45
;dzb[where(T gt Teff*1.5)]=0.5
;dzb[where(T gt Teff*1.8)]=0.55
;dzb[where(T gt Teff*1.2)]=1.0
;dzb[where(T gt Teff*2.1)]=0.6
;dzb=dzb*dxb
;dzchange=0.015
;dzchange=0.008
;rhd_manipulate_new1dgridgen, dummy, zout, Pout, $
;  aspect_min=0.4, aspect_max=1.0, dzchange=dzchange, $
;  j1max=n3, z1bot=min(z), z1top=max(z), dzb=dzb, $
;  input_type=1, z0in=z, dxbin=dxb
; --- Constant spacing R=0.4 ---
;n3=long((max(z)-min(z))/(0.4*dxb))
;zout=findgen(n3)*(0.4*dxb)+z[0]
; --- Constant spacing R=0.3 ---
n3=long((max(z)-min(z))/(0.3*dxb))
zout=findgen(n3)*(0.3*dxb)+z[0]

; --- Interpolate on new grid ---
rhoout=exp(monocubic(alog(rho),z,zout))
Pout  =exp(monocubic(alog(P  ),z,zout))
eiout =    monocubic(     ei  ,z,zout)
sout  =    monocubic(     s   ,z,zout)
Tout  =    monocubic(     T   ,z,zout)

; --- Additional densities ---
delvar, name_quc, unit_quc, qucout
name_quc =['rho_monomer', 'rho_dustr1']
unit_quc =['g/cm^3'     , 'g/cm^3'    ]
;name_quc =['rho_monomer', 'rho_dustr1', 'rho_dustr2', 'rho_dustr3', 'rho_dustr4', 'rho_dustr5']
;unit_quc =['g/cm^3'     , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    ]
;name_quc =['rho_monomer', 'rho_dustr1', 'rho_dustr2', 'rho_dustr3', 'rho_dustr4', 'rho_dustr5', 'rho_dustr6', 'rho_dustr7']
;unit_quc =['g/cm^3'     , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    ]
nquc=n_elements(name_quc)
qucout=fltarr(n3,nquc)
; --- Compute saturation pressure and density, limit density of monomers to saturation density in very high layers ---
.run equk3
psatc1, Tout, Psat
rhosat=eps*ms/(!con.kBoltz*Tout)*Psat
qucout[*,0]=(rhoout*dustfrac)
;ind=where(Tout lt 1900.0)
;qucout[ind,0]=0.01*((rhoout[ind]*dustfrac) < rhosat[ind])
;qucout[*,0]=qucout[*,0]*float((1.0 < (Tout/1900.0D+00))^120)
;qucout[*,0]=qucout[*,0]*float((1.0 < (Tout/1600.0D+00))^6)
;qucout[*,0]=(qucout[*,0]*float((1.0 < (Tout/1300.0D+00))^80)) > (qucout[*,0] < rhosat)
;qucout[*,0]=(qucout[*,0]*float((1.0 - (((Tout-1300.0D+00)/400.0-1.0) < 0.0)^4))) > (qucout[*,0] < rhosat)
;qucout[*,0]=(qucout[*,0]*float((1.0 - (((Tout-1650.0D+00)/400.0-1.0) < 0.0)^4))) > (qucout[*,0] < 1.01*rhosat)
;
;qucout[*,1]=((rhoout*dustfrac)*1.0E-10 > (((qucout[*,0]-rhosat) > 0.0)*1.0E-3)) < (0.5*qucout[*,0])
;qucout[*,1]=((rhoout*dustfrac)*1.0E-10 > (((qucout[*,0]-rhosat) > 0.0)*1.0E-2)) < (0.5*qucout[*,0])
qucout[*,1]=((rhoout*dustfrac)*1.0E-10 > (((qucout[*,0]-rhosat) > 0.0)*0.5)) < (0.5*qucout[*,0])
;
qucout[*,0]=qucout[*,0]-qucout[*,1]
;print, 'No dust arrays.' & delvar,qucout

;print,minmax(t)
plotlines, z,ei
plotlines, zout,eiout,psym=-1, /over, linecol=!ctv.red
;plotlines,z,dzb/dxb, drange=[0.0,1.0] & plotlines, zout, deriv(zout)/dxb, psym=-1,/over
plotlines, zdeep, eideep, psym=-1, /over, linecol=!ctv.red

;!P.MULTI=[0,2,2,0,0]
;plotlines, zout, alog10(rhoout), psym=-1
;plotlines, alog10(Pout), alog10(rhoout), psym=-1
;plotlines, zout, Tout, psym=-1
;plotlines, alog10(Pout), Tout, psym=-1
;!P.MULTI=0

filename=model + '.sta' & fileform='unformatted' & fileconv='ieee_4' & double_flag=0
version='rhd_atm2box 2010.04.21'

description=['RHD model derived from 1D model atmosphere', $
             atmtype + ": " + atmfile, $
             strtrim(systime(0),2)]
history=['Produced by rhd_atm2box.pro: ' + strtrim(systime(0),2)]
rhd_wrboxstrat, $
  filename=filename, fileform=fileform, fileconv=fileconv, double_flag=double_flag, $
  description=description, $
  history=history, $
  version=version, $
  zarr=zout, rhoarr=rhoout, eiarr=eiout, $
  qucarr=qucout, unit_quc=unit_quc, name_quc=name_quc, vampl=vampl, $
  dx1=dxb, dx2=dxb, dtime=dtime, n1=n1, n2=n2

;sta=uio_dataset_rd(filename) & eosbox, sta, eos=eos, /opa
;plotlines,alog10(reform((combox('dtau3',sta))[0,0,*])),/zeroline

; end ; rhd_atm2box
