; pro rhd_chunks_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_chunks_collect ('rhd_chunks_collect')
;
; PURPOSE:
;   Collect chunks from RHD data files.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_chunks_collect
;
; COMMON BLOCKS:
;
; ROUTINES:
;   combox
;   rd_datasetlist
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    3. 4.01 (B.F. Uppsala) rhd_plot -> rhd_collect
;   10.10.03 (B.F.) rhd_collect -> rhd_seq_collect
;   2005-06-15 (B.F. LA) Adaption from rhd_seq_collect
;   2005-06-16 (B.F.) Adaption from rage_seq_collect
;   2005-07-27 (B.F.) Use rd_datasetlist for reading of models
;   2005-07-28 (B.F.) rage_layer_collect -> rage_cube_collect
;   2005-09-14 (B.F.) Some modifications
;   2006-03-28 (B.F.) Analysis of FLASH data
;   2009-02-11 (B.F.) Analysis of CO5BOLD data (mt28g50mm00n02),
;                     put par and datatype into cube structure
;   2010-07-09 (B.F.) Derived from rage_cube_collect
;-
;-------------------------------------------------------------------------------

.run rhdmodeldata
rhdmodeldata, modeltable

; === Manual selection ===========
if (not keyword_set(external_control)) then begin &$
  ;stop &$
  ;datatype='rage' &$
  ;model='lc0gg' &$
  ;datatype='flash' &$
  ;model='lf0gg/lf0gg-2d_hdf5_plt_cnt_*' &$
  datatype='uio' &$
  ;model0='d3t57g44b0n002' & ModelIdent='_???' & parmodelident='' &$
  model0='d3t57g44b0n032' & ModelIdent='_?1?' & parmodelident='' &$
  ;model0='d3t57g44b1n016' & ModelIdent='_???' &$
  ;model0='d3t57g44b1n019' & ModelIdent='_???' &$
  ;model0='gt58g20n02' & modelident='_??' & parmodelident='_07' &$
  ;model0='mt09g50mm00n08' & modelident='_0[3-6]' & parmodelident='_06' &$
  ;model0='mt09g50mm00n09' & modelident='_??' & parmodelident='' &$
  ;model0='mt12g50mm00n07' & modelident='_1[2-5]' & parmodelident='_15' &$
  ;model0='mt15g50mm00n07' & modelident='_1[45]' & parmodelident='_15' &$
  ;model0='mt15g50mm00n09' & modelident='_??' & parmodelident='_20' &$
  ;model0='mt18g55mm00n03' & modelident='_0?' &$
  ;model0='mt28g50mm00n02' & modelident='_0[67]' & parmodelident='_06' &$
  ;
  single_quantity='' &$
  m_timestep=0L &$
  fileincrement=1 &$
  ;InputSuffix='full' &$
  InputSuffix='chu3' &$
  ;i2=0 &$
  ;chunk_flag=0 &$
  ;chunk_flag=1 & m1=0L & n1=10000L & m2=0L & n2=10000L & m3=230 & n3=230 &$
  chunk_flag=1 & m1=0L & n1=10000L & m2=0L & n2=10000L & m3=0L & n3=0L &$
  ;chunk_flag=1 & m1=0L & n1=10000L & m2=0L & n2=10000L & m3=125 & n3=125 &$
  ;chunk_flag=1 & m1=0L & n1=10000L & m3=200 & n3=299 &$
  ;chunk_flag=1 & m1=0L & n1=10000L & m3=0 & n3=499 &$
endif
; ================================

if (parmodelident eq '') then begin &$
  ; --- Find "most representative" parameter file from model table ---
  ii=(where(modeltable.model eq model0))[0] &$
  if (ii eq -1L) then begin &$
    ; --- No entry found: try default ---
    parmodelident='_01' &$
  endif else begin &$
    ; --- Take entry from model table ---
    parmodelident=modeltable[ii].parmodelident &$
  endelse &$
endif

if (datatype eq 'uio') then begin &$
  model=model0 + modelident &$
  modelbasepath=getenv('HOME') + '/dat/rhd/d' + model0 + '/' &$
  ;modelwithpath=modelbasepath+model+'.full' &$
  modelwithpath=modelbasepath+model + '.' + InputSuffix &$
  ;
  ; --- Read parameter file ---
  parfile=modelbasepath + model0 + parmodelident + '.par' &$
  par=uio_struct_rd(parfile) &$
endif else begin &$
  modelbasepath=getenv('HOME') + '/' + datatype + '/dat/' &$
  modelwithpath=modelbasepath+model &$
  ;
  ; --- Set default parameters for RAGE/FLASH runs ---
  par={grav: 10.0^7.695, Gamma1: 1.6666666} &$
endelse

; --- Read first file ---
delvar, listdata
ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                   single_quantity=single_quantity, $
                   listdata=listdata, ierr=ierr)
if (ierr ne 0) then begin &$
  print, 'Error reading data files' &$
  stop &$
endif
if (listdata.total_number eq 0) then begin &$
  print, 'No data files found' &$
  stop &$
endif
if (m_timestep gt 0) then begin &$
  listdata.open_number=long(m_timestep)-fileincrement &$
  ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                     single_quantity=single_quantity, $
                     listdata=listdata, ierr=ierr) &$
endif
if (datatype eq 'uio') then begin &$
  time_first=ful.modeltime &$
  ;
  uio_openrd, nc, listdata.file(listdata.total_number-1) &$
  outstr='found' &$
  while (outstr eq 'found') do begin &$
    uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
    if (ierr eq 0) then begin &$
      uio_rd, nc, termt,ntt, time_last,ident &$
      help, ierr, outstr, time_last &$
    endif &$
   endwhile &$
  uio_closrd, nc &$
  ;
  ;n_timestep_first=round(time_first/par.dtime_out_full) &$
  ;n_timestep_last =round(time_last/par.dtime_out_full) &$
  ;n_timestep=n_timestep_last-n_timestep_first+1 &$
  ;;n_timestep=n_timestep+n_timestep_adjust &$
  ;help, time_first, time_last, par.dtime_out_full, n_timestep_first, n_timestep_last, n_timestep &$
  ;
  n_timestep_first=round(time_first/par.dtime_out_mean) &$
  n_timestep_last =round(time_last/par.dtime_out_mean) &$
  n_timestep=n_timestep_last-n_timestep_first+1 &$
  ;n_timestep=n_timestep+n_timestep_adjust &$
  help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep &$
endif else begin &$
  n_timestep=(listdata.total_number-listdata.open_number)/fileincrement &$
endelse

; --- Set to something small for tests (and comment out otherwise) ---
;n_timestep=5L

; --- Array sizes ---
nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
time_first=ful.z.time

if (chunk_flag eq 0) then begin &$
  m1=0L &$
  n1=nxc1-1L &$
  m2=0L &$
  n2=nxc2-1L &$
  m3=0L &$
  n3=nxc3-1L &$
endif else begin &$
  n1=n1 < (nxc1-1) &$
  n2=n2 < (nxc2-1) &$
  n3=n3 < (nxc3-1) &$
endelse
dim=[[m1,n1], [m2,n2], [m3,n3]]
ful=chunkbox(ful, dim, /ZeroStart)

print, 'Model ' + model + '  with ', nxc1, ' x ', nxc2, ' x ', nxc3, ' points and ', n_timestep, ' time steps'
print, m1,n1, m2,n2, m3,n3

; --- Create structure that will hold the averaged data ---
chul=replicate(ful, n_timestep)
help, chul

ierr=0
i=0
itime_last=-10000L
systime0=systime(1)

while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  if (i ge 1) then begin &$
    ful=0 &$
    ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                       single_quantity=single_quantity, $
                       listdata=listdata, ierr=ierr) &$
    ful=chunkbox(ful, dim, /ZeroStart) &$
  endif &$
  if (ierr eq 0) then begin &$
    file=listdata.file(listdata.open_number) &$
    file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
    ;
    print, '--- ', file_tmp, i, ful.z.itime, ful.z.time, ' ---' , format='(A,A,I5,I7,E15.8,A)' &$
    ;
    ;chul[i]=ful &$
    tmp=chul[i] &$
    struct_assign, ful, tmp &$
    chul[i]=tmp &$
    ;
    i=i+1 &$
  endif &$
endwhile

print, 'Total time: ', systime(1)-systime0, ' sec'

slashpos=strpos(model, '/')
if (slashpos eq -1) then begin &$
  model_id=model &$
  if (datatype eq 'uio') then model_id=model0 &$
endif else begin &$
  model_id=strmid(model, 0, slashpos) &$
endelse
if (chunk_flag) then begin &$
  outfile=model_id + '_chul_' + InputSuffix + '_' + trim(string(m3),2)  + '-' + trim(string(n3),2) + '.sav' &$
endif else begin &$
  outfile=model_id + '_chul_' + InputSuffix + '_' + trim(string(i2),2) + '.sav' &$
endelse

print, 'Save dataset (chul) into ''' + outfile + ''''
save, filename=outfile, chul

; end ; rhd_chunks_collect
