; pro rhd_intstat_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_intstat_collect ('radiation_hydrodynamics_intstat_collect')
;
; PURPOSE:
;   Collect data from mean file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_intstat_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   26. 9.03 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;-
;------------------------------------------------------------------------------
;
;modelident='' & n_timestep_adjust=0
;model='st26gm09n02' & modelident='_??' & parmodelident='_07' 
;model='st28gm06n01' & modelident='_??' & parmodelident='_08'
;model='st28gm06n02' & modelident='_??' & parmodelident='_21'
;model='st33gm06n03' & modelident='_00-60' & parmodelident='_01'
;model='st35gm06n01' & modelident='_02-03' & parmodelident='_02'
;model='st35gm06n02' & modelident='_??' & parmodelident='_20'
;model='st35gm04n01' & modelident='_??' & parmodelident='_50' & n_timestep=228
;model='st35gm04n02' & modelident='_??' & parmodelident='_27' & n_timestep=415
;model='st35gm04n03' & modelident='_??' & parmodelident='_26' & n_timestep=518
;model='st35gm04n04' & modelident='_??' & parmodelident='_38' & n_timestep=648 ; dtime=0.193298E+09
;model='st35gm04n05' & modelident='_??' & parmodelident='_50' & n_timestep=307
;model='st35gm04n06' & modelident='_??' & parmodelident='_15' & n_timestep=235
;model='st35gm04n07' & modelident='_??' & parmodelident='_41' & n_timestep=242
;model='st35gm04n08' & modelident='_??' & parmodelident='_01'
;model='st35gm04n20' & modelident='_??' & parmodelident='_04'
;model='st35gm04n21' & modelident='_??' & parmodelident='_13'
;model='st35gm04n22' & modelident='_??' & parmodelident='_42'
;model='st35gm04n23' & modelident='_0[1-7]' & parmodelident='_07'
;model='st35gm04n24' & modelident='_??' & parmodelident='_12'
;model='st35gm04n25' & modelident='_??' & parmodelident='_27'
;model='st35gm04n26' & modelident='_??' & parmodelident='_75'
;model='st35gm04n27' & modelident='_??' & parmodelident='_08'
;model='st35gm04n28' & modelident='_??' & parmodelident='_55' & n_timestep_adjust=-15
;model='st35gm04n29' & modelident='_??' & parmodelident='_25'
;model='st35gm04n30' & modelident='_??' & parmodelident='_20'
;model='st35gm04n31' & modelident='_??' & parmodelident='_12'
;model='st35gm04n32' & modelident='_??' & parmodelident='_10' & n_timestep_adjust=-1
;model='st47g20n04' & modelident='_??' & parmodelident='_01'
;model='st47g20n05' & modelident='_??' & parmodelident='_01'
;model='st57g44n08'  & modelident='_??' & parmodelident='_13'
;model='st57g44n09'  & modelident='??' & parmodelident='az'
;model='rhd' & modelident='' & parmodelident=''
;
;modeldisk='/u64/bf' + '/dat/rhd/d' + model + '/'
;modeldisk='/p3/bf' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/job/jc/'
;modeldisk='/export/gusers/bf/dat/job/j2/bak/'
;modeldisk=getenv('HOME') + '/dat/rhd/tmp/'
;
modelfile=modeldisk + model + modelident + '.mean'
;modelfile=modeldisk + '*' + modelident + '.mean'
;
;smoothsigmapar=0.0 & sigma_id=''
;smoothsigmapar=0.04 & sigma_id='_sm004'
;
; ================================
;
; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')
;
delvar, listdata
ful=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr)
uio_closrd, listdata.channel
;
nr=n_elements(ful.r.xc3_volume)
nxc1=n_elements(ful.z1.xc1)
nxc2=n_elements(ful.z2.xc2)
nxc3=n_elements(ful.z3.xc3)
smoothsigma=float(nxc1)*smoothsigmapar
time_first=ful.modeltime
;
uio_openrd, nc, listdata.file(listdata.total_number-1)
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, ierr, outstr, time_last &$
  endif &$
endwhile
uio_closrd, nc
;
;n_timestep_first=ceil(time_first/par.dtime_out_mean)
;n_timestep_last =floor(time_last/par.dtime_out_mean)
;n_timestep=long((time_last-time_first)/par.dtime_out_mean+0.5)+1
n_timestep_first=round(time_first/par.dtime_out_mean)
n_timestep_last =round(time_last/par.dtime_out_mean)
n_timestep=n_timestep_last-n_timestep_first+1
n_timestep=n_timestep+n_timestep_adjust
help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep
;
delvar, listdata
;
.run rhd_intstat_data
;
itime              =lonarr(n_timestep)
time               =fltarr(n_timestep)
ifile              =lonarr(n_timestep)
file               =strarr(n_timestep)
luminositypervolume=fltarr(n_timestep)
;
I3l=create_struct( $
  'mean'    , fltarr(n_timestep), $
  'min'     , fltarr(n_timestep), $
  'max'     , fltarr(n_timestep), $
  'drms'    , fltarr(n_timestep), $
  'npts'    , fltarr(n_timestep), $
  'x'       , fltarr(n_timestep), $
  'y'       , fltarr(n_timestep), $
  'r'       , fltarr(n_timestep), $
  'r2'      , fltarr(n_timestep) $
                 )
I1l=I3l
I1r=I3l
I2l=I3l
I2r=I3l
I3l=I3l
I3r=I3l
I3lmask=I3l
I3rmask=I3l
;
ierr=0
i=0
itime_last=-10000L
newfile=''
systime0=systime(1)
;
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
 ful0=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr) &$
 if (ierr eq 0) then begin &$
 oldfile=newfile &$
 newfile=listdata.file(listdata.open_number) &$
 ful=temporary(ful0) &$
 if (ful.modelitime gt (itime_last+1)) then begin &$
  print, '--- ', i, ful.z.itime, ful.z.time, ' ---' , format='(A,I4,I7,E15.8,A)' &$
  ;
  if (newfile ne oldfile) then par=uio_struct_rd(strmid(newfile,0,strlen(newfile)-4) + 'par') &$
  file_tmp=strmid(newfile, strpos(newfile,'/',/REVERSE_SEARCH)+1) &$
  tags=tag_names(ful.r) &$
  ;
  itime(i)     =ful.z.itime &$
  time(i)      =ful.z.time &$
  ifile(i)     =listdata.open_number &$
  file(i)      =file_tmp &$
  luminositypervolume(i)=ful.rad.luminositypervolume &$
  ;
  n1=n_elements(ful.rad.xc1) &$
  n2=n_elements(ful.rad.xc2) &$
  n3=n_elements(ful.rad.xc3) &$
  ;
  ; --- I1l ---
  Intb=abs(reform(ful.rad.Intb1_l)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc2), reform(ful.rad.xc3), smoothsigma, i, I1l &$
  ;
  ; --- I1r ---
  Intb=abs(reform(ful.rad.Intb1_r)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc2), reform(ful.rad.xc3), smoothsigma, i, I1r &$
  ;
  ; --- I2l ---
  Intb=abs(reform(ful.rad.Intb2_l)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc3), smoothsigma, i, I2l &$
  ;
  ; --- I2r ---
  Intb=abs(reform(ful.rad.Intb2_r)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc3), smoothsigma, i, I2r &$
  ;
  mask=(ful.z.ei ge 3.0E+12) &$
  negmask=(ful.z.ei lt 3.0E+12) &$
  ;
  ; --- I3l ---
  Intb=abs(reform(ful.rad.Intb3_l)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc2), smoothsigma, i, I3l &$
  ;
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc2), 0.0, i, I3lmask, mask=mask, negmask=negmask &$
  ;
  ; --- I3r ---
  Intb=abs(reform(ful.rad.Intb3_r)) &$
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc2), smoothsigma, i, I3r &$
  ;
  rhd_intstat_data, ful, Intb, reform(ful.rad.xc1), reform(ful.rad.xc2), 0.0, i, I3rmask, mask=mask, negmask=negmask &$
  ;
  i=i+1 &$
 if (ierr eq 0) then itime_last=ful.modelitime &$
 endif &$
 endif &$
endwhile
;
print, 'Total time: ', systime(1)-systime0, ' sec'
;
intstat=create_struct( $
  'model'     , model, $
  'modelident', modelident, $
  'smoothsigmapar', smoothsigmapar, $
  'itime'              , itime, $
  'time'               , time, $
  'ifile'              , ifile, $
  'file'               , file, $
  'luminositypervolume', luminositypervolume, $ 
  'I1l'       , I1l, $
  'I1r'       , I1r, $
  'I2l'       , I2l, $
  'I2r'       , I2r, $
  'I3l'       , I3l, $
  'I3r'       , I3r, $
  'I3lmask'   , I3lmask, $
  'I3rmask'   , I3rmask $
                      )

print, 'Save dataset (intstat) into ''' + model + sigma_id + '_intstat.sav' + ''''
save, filename=model + sigma_id + '_intstat.sav', intstat
;
;exit
;
; end ; rhd_intstat_collect
