; pro rhd_load
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_load ('radiation_hydrodynamics_load')
;
; PURPOSE:
;   Load model data for single time step, solve EOS, compute opacity.
;
; CATEGORY:
;   RHD, I/O
;
; CALLING SEQUENCE:
;   @rhd_load
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, EOS
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    7. 6.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
modelident=''

; --- Set model name ---
;model='ahht82g42n01' & modelident='_01'
;model='ahht100g42n01' & modelident='_01'
;model='at71g43n01' & modelident='_06'
;model='at73g43n01' & modelident='_01'
;model='d3t36g10mm00n02' & modelident='.15'
;model='d3t75g40mm00n1' & modelident='.f9'
;model='at75g44n01' & modelident='_02'
;model='at77g44n01' & modelident='_01'
;model='at80g40n11_3D' & modelident='ez'
;model='at80g40n20' & modelident='_03'
;model='at80g40n21' & modelident='_35'
;model='at80g40n22' & modelident='_03nomassflux'
;model='at80g44n10_3D' & modelident='bz'
;model='at85g44n13' & modelident='_12'
;model='at85g44n14' & modelident='_02'
;model='at85g44n15' & modelident='_04'
;model='at85g40n01' & modelident='_07'
;model='at85g40n02' & modelident='_11'

;model='bt300g40n01' & modelident='_08'

;model='gt35gm04n02' & modelident='_02'
;model='gt35gm04n03' & modelident='_01'
;model='gt35gm04n04' & modelident='_02'
;model='gt35gm04n05' & modelident='_04'

;model='gt43g15n01' & modelident='_01'

;model='gt48g10n01' & modelident='_01'
;model='gt50g10n01' & modelident='_01'
model='gt50g20n01' & modelident='_40'
;model='gt51g20n01' & modelident='_14'
;model='gt52g20n01' & modelident='_12'
;model='gt53g20n01' & modelident='_10'
;model='gt53g20n02' & modelident='_01'
;model='gt54g20n01' & modelident='_06'
;model='gt54g20n03' & modelident='_43'
;model='gt54g20n04' & modelident='_12'
;model='gt54g20n05' & modelident='_80'
;model='gt55g20n01' & modelident='_22'
;model='gt56g20n01' & modelident='_09'
;model='gt56g20n02' & modelident='_48'
;model='gt56g20n03' & modelident='_20'
;model='gt56g20n03' & modelident='_20'
;model='gt56g20n04' & modelident='_52'
;model='gt58g20n01' & modelident='_02'
;model='gt58g20n04' & modelident='_03'
;model='gt60g20n01' & modelident='_02'
;model='gt60g20n04' & modelident='_60'
;model='gt62g20n01' & modelident='_05'

;model='st26gm09n02' & modelident='_07'
;model='st28gm07n002' & modelident='_050'
;model='st28gm06n01' & modelident='_10'
;model='st28gm06n02' & modelident='_29'
;model='st28gm06n03' & modelident='_08'
;model='st28gm06n04' & modelident='_02'
;model='st28gm06n05' & modelident='_01'
;model='st28gm06n06' & modelident='_23'
;model='st28gm06n06' & modelident='_19_shortrad_pressureall_1'
;model='st28gm06n07' & modelident='_03d'
;model='st28gm06n11' & modelident='_01'
;model='st28gm06n12' & modelident='_011'
;model='st28gm06n13' & modelident='_080'
;model='st28gm06n14' & modelident='_100'
;model='st28gm06n15' & modelident='_001'
;model='st28gm06n16' & modelident='_010'
;model='st28gm06n18' & modelident='_014'
;model='st28gm06n19' & modelident='_040'
;model='st28gm06n22' & modelident='_002'
;model='st28gm06n23' & modelident='_010'
;model='st28gm06n25' & modelident='_100'
;model='st28gm06n26' & modelident='_080'
;model='st28gm06n27' & modelident='_005'
;model='st28gm06n28' & modelident='_005'
;model='st28gm06n031' & modelident='_080'
;model='st28gm06n032' & modelident='_020'
;model='st28gm06n033' & modelident='_012'
;model='st28gm06n034' & modelident='_002'
;model='st28gm06n038' & modelident='_200'
;model='st28gm05n006' & ModelIdent='_037'
;model='st28gm05n010' & ModelIdent='_300'
;model='st28gm05n011' & ModelIdent='_130'
;model='st28gm05n012' & ModelIdent='_250'
;model='st28gm05n014' & ModelIdent='_001'
;model='st28gm05n017' & ModelIdent='_064'
;model='st28gm05n020' & ModelIdent='_010'
;model='st28gm05n023' & ModelIdent='_300'

;model='st33gm06n02' & modelident='_21'
;model='st33gm06n03' & modelident='_01'
;model='st35gm06n02' & modelident='_01'

;model='st35gm04n01' & modelident='_50'
;model='st35gm04n02' & modelident='_20'
;model='st35gm04n03' & modelident='_20'
;model='st35gm04n04' & modelident='_33'
;model='st35gm04n06' & modelident='_12'
;model='st35gm04n07' & modelident='_29'
;model='st35gm04n08' & modelident='_03'
;model='st35gm04n22' & modelident='_32'
;model='st35gm04n23' & modelident='_08'
;model='st35gm04n24' & modelident='_13'
;model='st35gm04n26' & modelident='_75'
;model='st35gm04n27' & modelident='_08'
;model='st35gm04n28' & modelident='_50'
;model='st35gm04n29' & modelident='_26'
;model='st35gm04n29' & modelident='_sum'
;model='st35gm04n30' & modelident='_01'
;model='st35gm04n31' & modelident='_22'
;model='st35gm04n32' & modelident='_10'
;model='st35gm04n34' & modelident='_01'
;model='st35gm04n36' & modelident='_048'
;model='st35gm04n37' & modelident='_137'
;model='st35gm04n41' & modelident='_999'
;model='st35gm04b0n001' & modelident='_009'
;model='st35gm04b1n001' & modelident='_446'
;model='st35gm04b1n002' & modelident='_850'
;model='st35gm04b1n003' & modelident='_500'
;model='st35gm04b1n004' & modelident='_140'
;model='st35gm04b1n005' & modelident='_184'

;model='st35gm03n01' & modelident='_01'
;model='st35gm03n02' & modelident='_23'
;model='st35gm03n04' & modelident='_01'
;model='st35gm03n06' & modelident='_73t-20'
;model='st35gm03n07' & modelident='_150'
;model='st35gm03n09' & modelident='_80'
;model='st35gm03n10' & modelident='_70'
;model='st35gm03n11' & modelident='_06'
;model='st35gm03n13' & modelident='_008'
;model='st35gm03n14c' & modelident='_001'
;model='st35gm03n15' & modelident='_008'
;model='st35gm03n16' & modelident='_020'
;model='st35gm03n17' & modelident='_200'
;model='st35gm03n19' & modelident='_100'

;model='st35gm00n01' & modelident='_001'

;model='st36g00n01' & modelident='_017'
;model='st36g00n05' & modelident='_257'

;model='st40gm03n01' & modelident='_17'
;model='st40gm03n02' & modelident='_30'
;model='st40gm03n03' & modelident='_05'
;model='st40gm03n04' & modelident='_96'
;model='st45gm03n01' & modelident='_08'
;model='st45gm03n02' & modelident='_04'

;model='st57g44n08' & modelident='_06'
;model='st57g44n10f' & modelident='_01'

;model='st22g35n07' & modelident='_150'
;model='st22g35n08' & modelident='_043'
;model='st22g40n01' & modelident='_003'

;model='st26g35n03' & modelident='_100'
;model='st26g35n04' & modelident='_034'
;model='st26g35n05' & modelident='_007'
;model='st26g35n06' & modelident='_020'
;model='st26g35n09' & modelident='_055'

;model='st36g45n04'     & ModelIdent='_23'
;model='st36g45n05'     & ModelIdent='_001'
;model='st36g45n08'     & ModelIdent='_100'
;model='st36g45n13'     & ModelIdent='_259'
;model='st36g45n15'     & ModelIdent='_028'
;model='st36g45n16'     & ModelIdent='_300'
;model='st36g45n17'     & ModelIdent='_067'
;model='st36g45n18'     & ModelIdent='_100'
;model='st36g45n20'     & ModelIdent='_013'
;model='st36g45n21'     & ModelIdent='_015'
;model='st36g45n23'     & ModelIdent='_008'
;model='st36g45n24'     & ModelIdent='_071'
;model='st36g45n26'     & ModelIdent='_018'
;model='st36g45n27'     & ModelIdent='_003'
;model='st36g45n29'     & ModelIdent='_006'
;model='st36g45n30'     & ModelIdent='_018'
;model='st36g45n31'     & ModelIdent='_002'
;model='st36g45n32'     & ModelIdent='_002'
;model='st36g45n36'     & ModelIdent='_069'
;model='st36g45n40'     & ModelIdent='_040'
;model='st36g45n44'     & ModelIdent='_010'
;model='st36g45n47'     & ModelIdent='_008'
;model='st36g45n48'     & ModelIdent='_011'
;model='st36g45n49a'     & ModelIdent='_019'
;model='st36g45n49b'     & ModelIdent='_001'
;model='st36g45n49c'     & ModelIdent='_025'
;model='st36g45n49d'     & ModelIdent='_020'
;model='st36g45n49f'     & ModelIdent='_005'
;model='st36g45n49g'     & ModelIdent='_002'
;model='st36g45n04rot'  & ModelIdent='_01'
;model='st36g45n04rot2' & ModelIdent='_14'
;model='st36g45n04hll'     & ModelIdent='_13'

;model='gt57g44n35dust' & modelident='_01'
;model='gt57g44b50n10_3D' & modelident='ae'
;model='d3t65g40mm00n01x8' & modelident='_051f'
;model='d3gt57g44n59' & modelident='.181'
;model='gt57g44n004_2D_B80' & modelident='_110'
;model='d3t33g50mm00w1hrlg' & modelident='.a4'
;model='d3t55g25mm00n01' & modelident='.037nl'
;model='d3t57g44b0n001' & modelident='_072'
;model='d3t57g44b0n002' & modelident='_090'
;model='d3t57g44b0n015' & modelident='_018'
;model='d3t57g44b0n018' & modelident='_018'
;model='d3t57g44b0n020' & modelident='_017'
;model='d3t57g44b0n025' & modelident='_010'
;model='d3t57g44b0n029' & modelident='_017'
;model='d3t57g44b0n032' & modelident='_009'
;model='d3t57g44b1n002' & modelident='_090'
;model='d3t57g44b1n013' & modelident='_075'
;model='d3t57g44b1n018' & modelident='_030'
;model='d3t57g44b1n022' & modelident='_001'
;model='d3gt57g44n59new3' & modelident='.60'
;model='d3gt57g44b000n59' & modelident='.67'
;model='d3gt57g44b010n59' & modelident='.60'
;model='sunmhd3D100' & modelident='_n018'
;model='sunmhd3D101' & modelident='_n007'
;model='job3Dh50' & modelident='_030'
;model='chro01' & modelident='l'
;model='chro2D03co08' & modelident='_01'
;model='radgreens3d02' & modelident='_21'
;model='radgreens' & modelident='3d01'

;model='d3t040g80wd08_RungeKutta2' & modelident=''
;model='d3t040g80wd08c' & modelident='_b'
;model='d3t040g80wd08e' & modelident='_c'
;model='d3t040g80wd08g' & modelident='_c'
;model='d3t120g80wd08_mhd000' & modelident='_025'
;model='d3t180g80db02' & modelident='_125'

;model='d2t18g50mm00du2' & modelident='.06'
;model='d2t18g50mm00du3' & modelident='.02'
;model='d2t18g50mm00du8' & modelident='_03'
;model='d2t18g50mm00dun07' & modelident='_05'
;model='mt07g50mm00n02' & modelident='_15'
;model='mt07g50mm00n03' & modelident='_40'
;model='mt13g50mm00n04' & modelident='_10'
;model='mt13g50mm00n07' & modelident='_10'
;model='mt13g50mm00n13' & modelident='_01'
;model='mt17g50mm00n03' & modelident='_100'
;model='mt18g50mm00n01' & modelident='_01'
;model='mt18g50mm00n02' & modelident='_06'
;model='mt18g50mm00n03' & modelident='_02'
;model='mt18g50mm00n04' & modelident='_04'
;model='dustg-t05-30n01' & modelident='_01'
;
;model='mt18g55mm00n19' & modelident='_03'
;model='mt18g55mm00n26' & modelident='_01'
;model='mt18g55mm00n32' & modelident='_03'

;model='mt07g50mm00n02' & modelident='_08'
;model='mt08g50mm00n03' & modelident='_25'
;model='mt09g35mm00n01' & modelident='_13'
;model='mt09g50mm00n02' & modelident='_40'
;model='mt10g50mm00n12' & modelident='_01'
;model='mt10g50mm00n14' & modelident='_20'
;model='mt11g50mm00n03' & modelident='_30'
;model='mt13g35mm00n01' & modelident='_02'
;model='mt13g40mm00n01' & modelident='_05'
;model='mt13g45mm00n01' & modelident='_03'
;model='mt13g50mm00n03' & modelident='_01'
;model='mt13g50mm00n05' & modelident='_01'
;model='mt13g50mm00n14' & modelident='_03'
;model='mt15g40mm00n01' & modelident='_02'
;model='mt15g50mm00n04' & modelident='_08'
;model='mt15g50mm00n06' & modelident='_18'
;model='mt15g50mm00n08' & modelident='_45'
;model='mt15g50mm00n11' & modelident='_01'
;model='mt15g50mm00n13' & modelident='_084'
;model='mt15g50mm00n14' & modelident='_035'
;model='mt15g50mm00n15' & modelident='_011'
;model='mt15g50mm00n16' & modelident='_001'
;model='mt16g40mm00n01' & modelident='_02'
;model='mt16g50mm00n02' & modelident='_01'
;model='mt18g45mm00n01' & modelident='_01'
;model='mt18g50mm00n07' & modelident='_20'
;model='mt18g50mm00n08' & modelident='_19'
;model='mt18g50mm00n10' & modelident='_01'
;model='mt18g50mm00n11' & modelident='_200'
;model='mt18g50mm00n13' & modelident='_026'
;model='mt18g55mm00n33b' & modelident='_05'
;model='mt20g50mm00n11' & modelident='_013'
;model='mt22g50mm00n06' & modelident='_03'
;model='mt22g50mm00n13' & modelident='_023'
;model='mt24g40mm00n01' & modelident='_01'
;model='mt24g50mm00n02' & modelident='_011'
;model='mt26g35mm00n01' & modelident='_10'
;model='mt26g45mm00n01' & modelident='_09'
;model='mt26g50mm00n03' & modelident='_009'
;model='mt26g50mm00n05' & modelident='_064'
;model='mt28g40mm00n01' & modelident='_03'
;model='mt28g50mm00n02' & modelident='_02'
;model='mt28g50mm00n03' & modelident='_200'
;model='mt30g40mm00n01' & modelident='_02'
;model='mt30g50mm00n06' & modelident='_05'
;model='mt30g50mm00n08' & modelident='_01'
;model='mt30g50mm00n10' & modelident='_023'
;model='mt32g40mm00n01' & modelident='_03'
;model='d3t32g45' & ModelIdent='_005'
;model='mt33g25mm00n01' & modelident='_01'
;model='mt33g30mm00n01' & modelident='_01'
;model='mt33g30mm00n02' & modelident='_01'
;model='mt33g40mm00n01' & modelident='_05'
;model='mt33g45mm00n02' & modelident='_042'
;model='d3t33g50mm00w1hrlg' & modelident='.a4'
;model='mt33g50mm00n01' & modelident='_04'
;model='mt33g50mm00n03' & modelident='_018'
;model='mt33g50mm00n04' & modelident='_040'
;model='mt33g50mm00n05' & modelident='_012'
;model='mt33g50mm00n08' & modelident='_100'
;model='mt33g50mm00n10' & modelident='_050'
;model='mt33g50mm00n13' & modelident='_040'
;model='mt35g40mm00n01' & modelident='_06'
;model='mt35g50mm00n01' & modelident='_06'
;model='mt35g50mm00n02' & modelident='_004'
;model='mt40g50mm00n04' & modelident='_004'

;model='rhd' & modelident=''

;model='Orszag-Tang_n05' & modelident=''
;model='Magnetoconvection' & modelident=''
;model='sedov3d_vanleer' & modelident='_08'

; --- Set some paths (model master directory, EOS data, opacities) ---
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/bak08/'
;modeldisk=getenv('HOME') + '/dat/rhd/hartmut_holweger/'
;modeldisk=getenv('HOME') + '/dat/rhd/mhd/'
;modeldisk='/u77/bf/dat/rhd/d' + model + '/'
;modeldisk='/mnt/sdc1/bf/dat/rhd/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/job/j1/bak/'
;modeldisk=getenv('HOME') + '/dat/job/j1/wrk/'
;modeldisk=getenv('HOME') + '/dat/job/j1/bak/bak20/'
;modeldisk='/u77/bf/dat/job/j7/bak/'
;modeldisk='/home/bf/idl/wrk/'
;modeldisk=getenv('HOME') + '/dat/rhd/tests/dsedov/'
;modeldisk=getenv('HOME') + '/dat/rhd/tests/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/rhd/tmp/'
EOSdisk=getenv('HOME') + '/for/eos/dat/'
;EOSdisk=getenv('HOME') + '/for/eos/ddat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
; --- Select the snapshot ---
;modelfile=modeldisk + model + modelident + '.chu1' & n=10
;modelfile=modeldisk + model + modelident + '.chu2' & n=2
;modelfile=modeldisk + model + modelident + '.chu3' & n=2
modelfile=modeldisk + model + modelident + '.end'  & n=0
;modelfile=modeldisk + model + modelident + '.full' & n=0
;modelfile=modeldisk + model + modelident + '.mcyl'  & n=25
;modelfile=modeldisk + model + modelident + '.mean'  & n=0
;modelfile=modeldisk + model + modelident + '.sta'  & n=0
;modelfile=modeldisk + model + modelident + '.snap'  & n=0
;modelfile=modeldisk + 'bak01/' + model + modelident + '.end'  & n=0
;
;
; --- Read parameter file ---
ierr=0
parfile=modeldisk + model + modelident + '.par'
par=uio_struct_rd(parfile, ierr=ierr)
if (ierr gt 0) then stop
;
; --- Read EOS and opacity file ---
if (not keyword_set(EOS)) then begin &$
  tabinter_rdcoeff, EOSdisk + par.EOSfile, EOS &$
  if (par.opafile ne '') then begin &$
    rdopta, opadisk + par.opafile, opadat &$
    opa=1 &$
  endif else begin &$
    opa=0 &$
  endelse &$
endif
f0=!con.sigmasb*par.Teff^4
;
; --- Load the model data ---
ierr=0
print, 'Loading ''' + modelfile + ''''
ful=uio_dataset_rd(modelfile, n=n, /HeadRead, ierr=ierr)
;ful=uio_dataset_rd(modelfile, n=n, blacklist=['v1','v2'], ierr=ierr)
;ful=uio_dataset_rd(modelfile, n=n, blacklist=['v1','v2','rho','ei'], ierr=ierr)
if (ierr gt 0) then stop

; --- Compute EOS and opacities ---
eosbox, ful, eos=EOS, opa=opa, out_q=0, ierror=ierror & if (ierror ne 0) then help,ierror
;eosbox, ful, eos=EOS, opa=0, out_q=1, ierror=ierror & if (ierror ne 0) then help,ierror
;
help, /str, ful
;
;x1=reform(ful.z.xc1) & x3=reform(ful.z.xc3) & v1=reform(ful.z.v1) & v3=reform(ful.z.v3) & q=alog10(reform(ful.eos.s))
;x1=reform(ful.z.xc1) & x3=reform(ful.z.xc3) & v1=reform(ful.z.v1) & v3=reform(ful.z.v3) & q=reform(combox('s',ful,/meansub))
;i=63 & x1=reform(ful.z.xc1) & x3=reform(ful.z.xc3) & v1=reform(ful.z.v1[*,i,*]) & v3=reform(ful.z.v3[*,i,*]) & q=alog10(reform(ful.eos.T[*,i,*]))
;i=63 & x2=reform(ful.z.xc2) & x3=reform(ful.z.xc3) & v2=reform(ful.z.v2[i,*,*]) & v3=reform(ful.z.v3[i,*,*]) & q=alog10(reform(ful.eos.T[i,*,*]))
;i=135 & x1=reform(ful.z.xc1) & x2=reform(ful.z.xc2) & v1=reform(ful.z.v1[*,*,i]) & v2=reform(ful.z.v2[*,*,i]) & q=alog10(reform(ful.eos.T[*,*,i]))

;m1=0 & n1=299 & i2=0 & m3=0 & n3=199
;x1=(reform(ful.z.xc1))[m1:n1] & x3=(reform(ful.z.xc3))[m3:n3] & v1=reform(ful.z.v1[m1:n1,i2,m3:n3]) & v3=reform(ful.z.v3[m1:n1,i2,m3:n3]) & q=reform((combox('s',ful,/meansub))[m1:n1,i2,m3:n3])

;m1=0 & n1=99 & m2=100 & n2=159 & i3=135
;x1=(reform(ful.z.xc1))[m1:n1] & x2=(reform(ful.z.xc2))[m2:n2] & v1=reform(ful.z.v1[m1:n1,m2:n2,i3]) & v2=reform(ful.z.v2[m1:n1,m2:n2,i3]) & q=alog10(reform((combox('t',ful,meansub=0))[m1:n1,m2:n2,i3]))

;x1=reform(ful.z.xc1) & x3=reform(ful.z.xc3) & v1=reform(ful.z.v1) & v3=reform(ful.z.v3) & q=reform(combox('s',ful,/meansub))
;plotfield, x1, x3, q, /tv, t_col=!ctv.ct3, xvec=v1, yvec=v3, /str, s_colu=120, s_row=40

;x1=reform(ful.z.xc1) & x2=reform(ful.z.xc2) & v1=reform(ful.z.v1) & v2=reform(ful.z.v2) & q=alog10(reform(ful.eos.T))
;plotfield, x1, x2, q, /tv, t_col=!ctv.ct3, xvec=v1, yvec=v2, /str, s_colu=50, s_row=50

;x2=reform(ful.z.xc2) & x3=reform(ful.z.xc3) & v2=reform(ful.z.v2) & v3=reform(ful.z.v3) & q=alog10(reform(ful.eos.T))
;plotfield, x2, x3, q, /tv, t_col=!ctv.ct3, xvec=v2, yvec=v3, /str, s_colu=50, s_row=50

; plotlines,transpose(reform(alog10(ful.z.rho(80:120,20:40,127)))),off=0,psym=-1
; x1=reform(ful.z.xc1(80:120)) & x3=reform(ful.z.xc3(10:50)) &
; v1=reform(ful.z.v1(80:120,10:50,127)) & v3=reform(ful.z.v2(80:120,10:50,127))
; & q=alog10(reform(ful.z.rho(80:120,10:50,127)))

;kernel=[[1.0, 2.0, 1.0], [2.0, 4.0, 2.0], [1.0, 2.0, 1.0]] & kernel=kernel/total(kernel)
;plotfield,median(convol(abs(reform(ful.rad.intb3_r)),kernel, /edge_truncate, /center),3),/tv

;rdragebox,'/home/bf/rage/dat/', 'lcb01', 9, ful
;x=reform(ful.z.xc1) & y=reform(ful.z.xc3) & q=reform(combox('s',ful)) & vx=reform(combox('v1',ful)) & vy=reform(combox('v3',ful))
;plotfield,x,y,q,/tv, xvec=vx,yvec=vy, /str,s_colu=60,s_row=30,s_time=10.0,s_nst=20,s_ar=5

;rdflashbox,'/u/bf/flash/dat/heat7/','rbeq-2d_hdf5_plt_cnt_',100,ful

;logtaur=-findgen(121)/120.0*10.0+6.0
; Tr=reform(interbox(ful.eos.T, alog10(combox('tau3r',ful)), x_reg=logtaur, orient=3, bound=1))
;v3r=reform(interbox(ful.z.v3, alog10(combox('tau3r',ful)), x_reg=logtaur, orient=3, bound=1))
