; pro rhd_mean_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_mean_collect ('radiation_hydrodynamics_loop')
;
; PURPOSE:
;   Collect data from full file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_mean_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    3. 4.01 (B.F. Uppsala) rhd_plot -> rhd_collect -> rhd_collect
;-
;------------------------------------------------------------------------------

; ================================
if (not(keyword_set(external_control))) then begin &$
  model='at80g40n22' & modelident='_03nomassflux' & parmodelident='_04' &$
  out_id=modelident &$
endif
; ================================

modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'

;modelfile=modeldisk + model + modelident + '.full'
modelfile=modeldisk + model + modelident + '.end'

outfile=model + out_id+ '_meantab.txt'

; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')

; --- Read EOS file ---
;if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS
  dfopta, opadisk + par.opafile
;endif
f0=!con.sigmasb*par.Teff^4

delvar, listdata
ful=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr)
uio_closrd, listdata.channel

nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
time_first=ful.modeltime

uio_openrd, nc, listdata.file(listdata.total_number-1)
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, ierr, outstr, time_last &$
  endif &$
endwhile
uio_closrd, nc

n_timestep_first=round(time_first/par.dtime_out_full)
n_timestep_last =round(time_last/par.dtime_out_full)
n_timestep=n_timestep_last-n_timestep_first+1
n_timestep=n_timestep
;n_timestep=2   ; 319, 376, 256, 97, 105, 96
help, time_first, time_last, par.dtime_out_full, n_timestep_first, n_timestep_last, n_timestep

delvar, listdata

; --- Generate regular optical depth scale ---
n_tau =97
logtau=findgen(n_tau)/10.0 - 6.0
tau=10.0^logtau

mean=create_struct( $
  'model'             , model, $
  'modelident'        , modelident, $
  'xc1'               , reform(ful.z.xc1), $
  'xc2'               , reform(ful.z.xc2), $
  'xc3'               , reform(ful.z.xc3), $
  'xb1'               , reform(ful.z.xb1), $
  'xb2'               , reform(ful.z.xb2), $
  'xb3'               , reform(ful.z.xb3), $
  'dx1'               , ful.z.xb1(1)-ful.z.xb1(0), $
  'dx2'               , ful.z.xb2(1)-ful.z.xb2(0), $
  'dx3'               , ful.z.xb3(1)-ful.z.xb3(0), $
  'itime'             , lonarr(            n_timestep), $
  'time'              , fltarr(            n_timestep), $
  'ifile'             , lonarr(            n_timestep), $
  'file'              , strarr(            n_timestep), $
  'itime_first'       , ful.z.itime, $
  'itime_last'        , -10000L, $
  'surface_id'        , ['3r'], $
  'tau_mean'      , 10.0^logtau                   , $
  'P_mean'        , fltarr(n_tau, n_timestep), $
  'rho_mean'      , fltarr(n_tau, n_timestep), $
  'T_mean'        , fltarr(n_tau, n_timestep), $
  'T_mean2'       , fltarr(n_tau, n_timestep), $
  'T_mean4'       , fltarr(n_tau, n_timestep), $
  'v_mean'        , fltarr(n_tau, n_timestep), $
  'v_mean2'       , fltarr(n_tau, n_timestep), $
  'x_mean'        , fltarr(n_tau, n_timestep), $
  'x_mean2'       , fltarr(n_tau, n_timestep) $
                  )

;.run rhd_mean_data

ierr=0
i=0L
file=''
systime0=systime(1)

while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful0=0 &$
  ful0=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=listdata.file(listdata.open_number) &$
    ful=temporary(ful0) &$
    if (ful.modelitime gt (mean.itime_last+1)) then begin &$
      print, '--- ', i, ful.z.itime, ful.z.time, systime(1)-systime0, ' ---' , $
        format='(A,I4,X,I6,E15.6,1X,F8.1,A)' &$
      ;
      eosbox, ful, eos=EOS, /opa, out_quantities=1 &$
      ;
      file_tmp     =strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
      mean.ifile(i)=listdata.open_number &$
      mean.file(i) =file_tmp &$
      ;
      mean.itime[i]=ful.z.itime &$
      mean.time[i] =ful.z.time &$
      mean.itime_last=ful.z.itime &$
      ;
      T=combox('T',ful) &$
      P=combox('P',ful) &$
      x=combox('xc3a', ful) &$
      taur=combox('tauint', ful, par=par) &$
      T4r =transpose(interbox(T^4      , taur, x_regular=tau, ori=3, boundary=1), [2,1,0]) &$
      Pr  =transpose(interbox(P        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0]) &$
      xr  =transpose(interbox(x        , taur, x_regular=tau, ori=3, boundary=1), [2,1,0]) &$
      rhor=transpose(interbox(ful.z.rho, taur, x_regular=tau, ori=3, boundary=1), [2,1,0]) &$
      vr  =transpose(interbox(ful.z.v3 , taur, x_regular=tau, ori=3, boundary=1), [2,1,0]) &$
      ;
      mean.T_mean[  *, i]=          avg(avg(sqrt(sqrt(T4r)),1),1) &$
      mean.T_mean2[ *, i]=     sqrt(avg(avg(     sqrt(T4r) ,1),1)) &$
      mean.T_mean4[ *, i]=sqrt(sqrt(avg(avg(          T4r  ,1),1))) &$
      mean.P_mean[  *, i]=          avg(avg(          Pr   ,1),1) &$
      mean.x_mean[  *, i]=          avg(avg(          xr   ,1),1) &$
      mean.x_mean2[ *, i]=     sqrt(avg(avg(          xr^2 ,1),1)) &$
      mean.rho_mean[*, i]=          avg(avg(          rhor ,1),1) &$
      mean.v_mean[  *, i]=          avg(avg(          vr   ,1),1) &$
      mean.v_mean2[ *, i]=     sqrt(avg(avg(          vr^2 ,1),1)) &$
      ;
      i=i+1L &$
    endif &$
  endif &$
endwhile

save, filename=model + '_mean' + '.sav', mean

i=i-1L
x3arr =reform(mean.x_mean[*,i])
tauarr=reform(mean.tau_mean)
Tarr  =reform(mean.T_mean4)
Parr  =reform(mean.P_mean)
n_z  =n_elements(tauarr)
n_col=4
; --- Write to text file ---
get_lun, nc
openw, nc, outfile
printf, nc, "# Table with data averaged on tau iso surfaces: " + systime(0)
printf, nc, n_z,    "Number of height points", format='(I13,4X,A)'
printf, nc, n_col,  "Number of columns", format='(I13,4X,A)'
printf, nc, 1,      "Number of groups", format='(I13,4X,A)'
printf, nc, "z             tau           T             P"
for j=0L,n_z-1L do begin &$
  printf, nc, x3arr[j], tauarr[j], Tarr[j], Parr[j], format='(E13.6,3(1X,E13.6))' &$
endfor
close, nc
free_lun, nc

; end ; rhd_mean_collect
