; rhd_mlt_alphafromstratplot
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_mlt_alphafromstratplot ('alpha_mixing-length-theory_alpha_from_stratification')
;
; PURPOSE:
;   Compute mixing-length alpha from stratplotification of RHD models
;
; CATEGORY:
;   RHD, analysis
;
; CALLING SEQUENCE:
;   @rhd_mlt_alphafromstratplot
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2012-04-25 (B.F. Lyon) First version
;   2012-04-26 (B.F.) Compute nabla_el
;-
;------------------------------------------------------------------------------

; === Manual selection ===========
; --- Choose device: 'x', 'ps' ---
dev='ps' 
file_prefix='aa2012rot_' 
jpg_flag=0 
png_flag=0 
;
; --- Plot Id ---
id='alpha_CoNum' 
;
; --- Plot size ---
xsize=8.8 & ysize=6.5
;
ModelList=['st36g45n04', 'st36g45n04rot', 'st36g45n04rot2']
CommentList=['!4a!X!DV!N ' + ModelList[0], $
             '!4a!X!DF!N ' + ModelList[0], $
             '!4a!X!DV!N ' + ModelList[1], $
             '!4a!X!DF!N ' + ModelList[1], $
             '!4a!X!DV!N ' + ModelList[2], $
             '!4a!X!DF!N ' + ModelList[2]]
; ================================

; --- Output file prefix ---
file_id=file_prefix + id
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Device settings ---
if (strupcase(dev) eq 'X') then begin & $
  wfactor=1000.0 &$
  set_plot, 'X' &$
  !p.font=-1 &$
  win_id=0 &$
  window, win_id, xsize=xsize/18.0*wfactor, ysize=ysize/18.0*wfactor, retain=2 &$
endif else begin &$
  set_plot, 'ps' &$
  !p.font=0 &$
  device, filename=ps_file, $
          /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
;          /portrait, xsize=18.0, ysize=24.0,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
;          /portrait, xsize=17.5, ysize=12.5,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
;  help, /device & $
endelse
modctv, dev

; --- Save and set global plot parameters ---
ColorList=[!P.COLOR,!P.COLOR,!ctv.red,!ctv.red,!ctv.green,!ctv.green]
LineStyleList=[1,0,1,0,1,0]
Pbak=!P & Xbak=!X & Ybak=!Y
!P.CHARSIZE=0.75
;!P.BACKGROUND=!ctv.black
;!X.STYLE=5
;!Y.STYLE=5
;!Y.TICKNAME=' '
;!X.MARGIN=[1,1]
!X.MARGIN=[6,1]
;!Y.MARGIN=[2,2]
!Y.MARGIN=[4,1]

external_control=1

; --- Set input model name 1 -----------------------------------------------------------
Model=ModelList[0]  & m=75 & n=130

; --- Load data and compute alphas and Coriolis number (and lots of other things) ---
@rhd_mlt_alphafromstrat

; --- Plot alphas ---
plotlines, Co[m:n], alphaV[m:n], $
  xrange=[0.0, 10.5], drange=[0.0,5.0], $
  xtitle=transfont('Coriolis number'), ytitle=transfont('!4a!X!DMLT!N'), $
  linestyle=LineStyleList[0], linecolor=ColorList[0], $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, Co[m:n], alphaF[m:n], $
  /over, $
  linestyle=LineStyleList[1], linecolor=ColorList[1], $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Set input model name 2 -----------------------------------------------------------
Model=ModelList[1] & m=25 & n=118

; --- Load data and compute alphas and Coriolis number (and lots of other things) ---
@rhd_mlt_alphafromstrat

; --- Plot alphas ---
plotlines, Co[m:n], alphaV[m:n], $
  /over, $
  linestyle=LineStyleList[2], linecolor=ColorList[2], $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, Co[m:n], alphaF[m:n], $
  /over, $
  linestyle=LineStyleList[3], linecolor=ColorList[3], $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Set input model name 3 -----------------------------------------------------------
Model=ModelList[2] & m=45 & n=125

; --- Load data and compute alphas and Coriolis number (and lots of other things) ---
@rhd_mlt_alphafromstrat

; --- Plot alphas ---
plotlines, Co[m:n], alphaV[m:n], $
  /over, $
  linestyle=LineStyleList[4], linecolor=ColorList[4], $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, Co[m:n], alphaF[m:n], $
  /over, $
  linestyle=LineStyleList[5], linecolor=ColorList[5], $
  dev=dev, psformat='nochange', /psnoinit, /noclose

; --------------------------------------------------------------------------------------

external_control=0

CoVec=findgen(200)/10.0
plotlines, CoVec, 3.0*exp(-CoVec/12.0), $\
  /over, $
  linecolor=!ctv.blue

; --- Make legend ---
for i=0,n_elements(CommentList)-1 do begin &$
  CommentList[i]=transfont(CommentList[i]) &$
endfor
legend, CommentList, /nFromItems, $
  linestyle=LineStyleList, $
  color=ColorList, $
  xali=0.97, yali=0.97, box=0, margin=0.1, spacing=0.8, pspacing=2, charsize=!P.CHARSIZE*0.9

; --- Finalize plot ---
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    ; --- Write PNG file ---
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    ; --- Write JPEG file ---
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

; end ; rhd_mlt_alphafromstratplot
