;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_PrintTable_AGB ('radiation_hydrodynamcis_Print_Table')
;
; PURPOSE:
;   Extract data from RHD model databases and print them in LaTeX table format.
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   .run rhd_printtable_agb
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_extract:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   .run rhd_printtable_agb
;
; MODIFICATION HISTORY:
;   2015-09-09 (B.F. Uppsala) Written
;   2016-04-27 (B.F.) Add pulsation period and sigma
;-
;---------------------------------------------------------------------------------------------------

rhdmodeldata, modDB
restore, 'allbasdb.sav'
restore, 'allpardb.sav'
restore, 'list_period_AGB.sav' & perDB=per_db

; === Manually select some parameters ====
if (not keyword_set(external_control)) then begin
  ;
  ;MLID='AGB_Grid' & ModelList=['st28gm06n02', 'st28gm06n03', 'st28gm06n05', 'st28gm06n06', 'st28gm06n13', 'st28gm06n16', 'st28gm06n18', 'st28gm06n24', 'st28gm06n25', 'st28gm06n29', 'st28gm06n30', 'st28gm07n002', 'st28gm07n004', 'st28gm07n003', 'st28gm07n001', 'st28gm07n005', 'st26gm07n002', 'st26gm07n001', 'st28gm06n036', 'st28gm06n26', 'st29gm06n001',  'st27gm06n001', 'st28gm05n002', 'st28gm05n001', 'st28gm05n003', 'st29gm04n001', 'st28gm05n004', 'st28gm05n005', 'st28gm05n008', 'st28gm06n037', 'st28gm05n009', 'st28gm05n010', 'st28gm05n011', 'st28gm05n012', 'st28gm05n007', 'st28gm05n006']
  MLID='AGB_all' & ModelList=udb_ListAll(modDB) & ModelList=ModelList[where(strmatch(ModelList, 'st2?gm*'))] & ModelList=ModelList(sort(ModelList))
  ;MLID='AGB_2dust' & ModelList=['st28gm06n25', 'st28gm06n032', 'st28gm06n033', 'st28gm06n034', 'st28gm06n038', 'st28gm06n039']
  ;MLID='AGB_2dust' & ModelList=['st28gm05n012', 'st28gm05n013', 'st28gm05n014']
  x_fac=1.0/!con.RSun & x_unit='$R_\sun$' & time_fac=1.0/!con.y & time_unit='yr'
  ;
  qList=      list(list('model'            , 'model'                      , ''            , 'A12' , 0     , 0            ))
  qList=qList+list(list('opafile'          , 'opacities'                  , ''            , 'A70' , 0     , 0            ))
  qList=qList+list(list('nband'            , '\!$n_\mathrm{band}$'        , ''            , 'I2'  , 0     , 0            ))
  qList=qList+list(list('mass_star'        , '$M_\star$'                  , '$M_\sun$'   , 'F3.1', 1.0E-1, 1.0/!con.MSun ))
;  qList=qList+list(list('mass_envelope'    , '$M_\mathrm{env}$'           , '$M_\sun$'   , 'F5.3', 1.0E-3, 1.0/!con.MSun ))
  qList=qList+list(list('mass_envelopefrac', '$M_\mathrm{env}$'           , '$M_\star$'  , 'F5.3', 1.0E-3, 0             ))
  qList=qList+list(list('LuminosityOut'    , '$L_\star$'                  , '$L_\sun$'   , 'I5'  , 1.0   , 1.0/!con.LSun ))
  qList=qList+list(list('nx'               , '\!\!\!$n_x$$\times$$n_y$$\times$$n_z$\!', ''    , 'I3'  , 0     , 0            ))
;  qList=qList+list(list('nx1'              , '$n_\mathrm{grid}$'          , ''            , 'I3'  , 0     , 0            ))
  qList=qList+list(list('r0_grav'          , '$R_{0,\mathrm{box}}$'       , x_unit        , 'I3'  , 1.0   , x_fac        ))
  qList=qList+list(list('xb1tot'           , '$x_\mathrm{box}$'           , x_unit        , 'I4'  , 1.0   , x_fac        ))
;  qList=qList+list(list('xb3tot'           , '$z_\mathrm{box}$'           , x_unit        , 'I4'  , 1.0   , x_fac        ))
;  qList=qList+list(list('P_rotation'       , '$P_\mathrm{rot}$'           , time_unit     , 'I2'  , 1.0   , time_fac     ))
  qList=qList+list(list('timeavg'          , '$t_\mathrm{avg}$'           , time_unit     , 'F5.2', 1.0E-2, time_fac     ))
  qList=qList+list(list('Radius'           , '$R_\star$'                  , x_unit        , 'I4'  , 1.0   , x_fac        ))
  qList=qList+list(list('TeffOut'          , '$T_\mathrm{eff}$'           , 'K'           , 'I4'  , 1.0   , 0            ))
  qList=qList+list(list('log10_grav'       , '$\log g$'                   , '(cgs)'       , 'F5.2', 1.0E-2, 0            ))
  qList=qList+list(list('s_core'           , '$s\mathrm{core}$'           , 'G erg/K/g'   , 'F6.3', 1.0E-3,  1.0E-9      ))
;  qList=qList+list(list('period'           , '$P_\mathrm{puls}$'          , time_unit     , 'F5.3', 1.0E-3, time_fac     ))
;  qList=qList+list(list('sigma'            , '$\sigma_\mathrm{puls}$'     , time_unit     , 'F5.3', 1.0E-3, time_fac     ))
  ;
;  MLID='M-dwarf_3D_logg=5_allnew' & ModelList=['mt33g45mm00n02', 'mt33g45mm00n03', 'mt33g45mm00n04', 'mt40g50mm00n04', 'mt40g50mm00n05', 'mt35g50mm00n02', 'mt35g50mm00n03', 'mt33g50mm00n02', 'mt33g50mm00n03', 'mt33g50mm00n04', 'mt33g50mm00n05', 'mt33g50mm00n06', 'mt33g50mm00n07', 'mt33g50mm00n08', 'mt33g50mm00n09', 'mt33g50mm00n10', 'mt33g50mm00n11', 'mt33g50mm00n12', 'mt33g50mm00n13', 'mt33g50mm00n14', 'mt32g50mm00n02', 'mt32g50mm00n03', 'mt30g50mm00n10', 'mt30g50mm00n11', 'mt30g50mm00n12', 'mt28g50mm00n03', 'mt28g50mm00n04', 'mt28g50mm00n05', 'mt26g50mm00n05', 'mt26g50mm00n07', 'mt24g50mm00n02', 'mt24g50mm00n03', 'mt22g50mm00n13', 'mt22g50mm00n14', 'mt20g50mm00n11', 'mt20g50mm00n12', 'mt18g50mm00n13', 'mt18g50mm00n14', 'mt17g50mm00n03', 'mt16g50mm00n08', 'mt15g50mm00n13', 'mt15g50mm00n14', 'mt15g50mm00n15', 'mt15g50mm00n17', 'mt14g50mm00n03']
;  x_fac=1.0E-05 & x_unit='km' & time_fac=1.0/3600.0 & time_unit='h'
;  qList=      list(list('model'        , 'model'                      , ''            , 'A15' , 0     , 0            ))
;  qList=qList+list(list('opafile'      , 'opacities'                  , ''            , 'A50' , 0     , 0            ))
;  qList=qList+list(list('nx'           , '$n_x$$\times$$n_y$$\times$$n_z$', ''            , 'I3'  , 0     , 0            ))
;  qList=qList+list(list('xb1tot'       , '$x_\mathrm{box}$'           , x_unit        , 'I4'  , 1.0   , x_fac        ))
;  qList=qList+list(list('xb3tot'       , '$z_\mathrm{box}$'           , x_unit        , 'I4'  , 1.0   , x_fac        ))
;  qList=qList+list(list('s_inflow'     , '$s_\mathrm{inflow}$'        ,'$10^9$erg/K/g', 'F6.4', 1.0E-4, 1.0E-9       ))
;  qList=qList+list(list('log10_grav'   , '$\log g$'                   , '(cgs)'       , 'F5.2', 1.0E-2, 0            ))
;  qList=qList+list(list('timeavg'      , '$t_\mathrm{avg}$'           , time_unit     , 'F5.2', 1.0E-2, time_fac     ))
;  qList=qList+list(list('TeffOut'      , '$T_\mathrm{eff}$'           , 'K'           , 'I4'  , 1.0   , 0            ))

  ;
endif

;print, ModelList

; --- Set the indices of the various entries in each sub list ---
iID=0
iNa=1
iUn=2
iFo=3
iRo=4
iFa=5

; --- Construct empty table as 2D string array ---
TabOut=strarr(n_elements(ModelList), n_elements(qList))

; --- Assemble the data table ---
for iq=0, n_elements(qList)-1 do begin
  qID=(qList[iq])[iID]
  if (strmid(qID, 0, 6) eq 'log10_') then begin
    qID=strmid(qID, 6, 200)
    log10_flag=1
  endif else begin
    log10_flag=0
  endelse
  ;print, iq, ' ', qID
  ;
  if (qID eq 'nx') then begin
    ; --- Create composed string describing the grid size of the model box ---
    nx1=udb_extract('nx1', ModelList, basDB)
    nx2=udb_extract('nx2', ModelList, basDB)
    nx3=udb_extract('nx3', ModelList, basDB)
    ;
    qStr=string(nx1, '(' + (qList[iq])[iFo] + ')') + '$\times$' + $
         string(nx2, '(' + (qList[iq])[iFo] + ')') + '$\times$' + $
         string(nx3, '(' + (qList[iq])[iFo] + ')')
    ;
    Ind=where((nx1 eq nx2) and (nx2 ne nx3), n)
    if (n gt 0) then begin
      qStr[Ind]=string(nx1[Ind], '(' + (qList[iq])[iFo] + ')') + '$^2$$\times$' + $
                string(nx3[Ind], '(' + (qList[iq])[iFo] + ')')
    endif
    ;
    Ind=where((nx1 eq nx2) and (nx2 eq nx3), n)
    if (n gt 0) then begin
      qStr[Ind]=string(nx1[Ind], '(' + (qList[iq])[iFo] + ')') + '$^3$'
    endif
    ;
  endif else if (qID eq 'timeavg') then begin
    ; --- Compute the time span used for the averaging ---
    time_first=udb_extract('time_first', ModelList, basDB)
    time_last =udb_extract('time_last' , ModelList, basDB)
    ;
    q=time_last-time_first
    if ((qList[iq])[iFa] ne 0) then q=q*(qList[iq])[iFa]
    if (log10_flag           ) then q=alog10(q)
    if ((qList[iq])[iRo] ne 0) then q=round(q/(qList[iq])[iRo])*(qList[iq])[iRo]
    ;
    qStr=string(q, '(' + (qList[iq])[iFo] + ')')
    ;
  endif else if (qID eq 'mass_envelopefrac') then begin
    ; --- Compute the envelope mass fraction ---
    mass_envelope=udb_extract('mass_envelope', ModelList, basDB)
    mass_star    =udb_extract('mass_star'    , ModelList, parDB)
    ;
    q=mass_envelope/mass_star
    ;
    if ((qList[iq])[iFa] ne 0) then q=q*(qList[iq])[iFa]
    if (log10_flag           ) then q=alog10(q)
    if ((qList[iq])[iRo] ne 0) then q=round(q/(qList[iq])[iRo])*(qList[iq])[iRo]
    ;
    qStr=string(q, '(' + (qList[iq])[iFo] + ')')
    ;
  endif else if (qID eq 'P_rotation') then begin
    ; --- Compute the rotation period ---
    nu_rotation=udb_extract('nu_rotation', ModelList, parDB)
    ;
    q=nu_rotation*0.0
    Ind=where(nu_rotation gt 0.0, n)
    if (n gt 0) then q[Ind]=1.0/nu_rotation[Ind]
    ;
    if ((qList[iq])[iFa] ne 0) then q=q*(qList[iq])[iFa]
    if (log10_flag           ) then q=alog10(q)
    if ((qList[iq])[iRo] ne 0) then q=round(q/(qList[iq])[iRo])*(qList[iq])[iRo]
    ;
    qStr=string(q, '(' + (qList[iq])[iFo] + ')')
    ;
  endif else if ((qID eq 'period') or (qID eq 'sigma')) then begin
    ; --- Compute the pulsation period and sigma ---
    q=udb_extract(qID, ModelList, perDB)
    ;
    q=q*!con.y
    ;
    if ((qList[iq])[iFa] ne 0) then q=q*(qList[iq])[iFa]
    if (log10_flag           ) then q=alog10(q)
    if ((qList[iq])[iRo] ne 0) then q=round(q/(qList[iq])[iRo])*(qList[iq])[iRo]
    ;
    qStr=string(q, '(' + (qList[iq])[iFo] + ')')
    ;
  endif else begin
    ; --- Look for normal quantity ---
    if (qID eq 'model') then begin
      q=ModelList
    endif else begin
      q=udb_extract(qID, ModelList, basDB, parDB)
    endelse
    ;
    if ((qList[iq])[iFa] ne 0) then q=q*(qList[iq])[iFa]
    if (log10_flag           ) then q=alog10(q)
    if ((qList[iq])[iRo] ne 0) then q=round(q/(qList[iq])[iRo])*(qList[iq])[iRo]
    ;if (strmid((qList[iq])[iFo],0,1) eq 'A') then q='{\small\verb|' + q + '|}'
    if (qID eq 'opafile') then begin
      for iM=0, n_elements(ModelList)-1 do begin
        ;q[iM]='{\footnotesize\verb|' + strmid(q[iM],0,strlen(q[iM])-5) + '|}'
        q[iM]='{\footnotesize\verb|' + strreplacebf(strreplacebf(strreplacebf(strreplacebf(q[iM], '_paola_oc1rp_b3_n02', ''), '.opta', ''), '_phoenix', ''), '_fit', '') + '|}\!\!'
      endfor ; iM
    endif
    ;
    qStr=string(q, '(' + (qList[iq])[iFo] + ')')
    ;
  endelse
  ;
  ;print, qStr
  TabOut[*, iq]=qStr
endfor ; iq

; --- Print the final table ---
; --- Loop over all models; negative indices indicate header lines ---
for iM=-2, n_elements(ModelList)-1 do begin
  StrOut=''
  ;
  for iq=0, n_elements(qList)-1 do begin
    if (iM eq -2) then begin
      StrOut=StrOut + (qList[iq])[iNa]
    endif else if (iM eq -1) then begin
      StrOut=StrOut + (qList[iq])[iUn]
    endif else begin
      StrOut=StrOut + TabOut[iM, iq]
    endelse
    ;
    if (iq lt (n_elements(qList)-1)) then begin
      StrOut=StrOut + ' & '
    endif else begin
      StrOut=StrOut + ' \\'
    endelse
  endfor ; iq
  ;
  print, StrOut
  ;
endfor ; iM

end ; rhd_printtable
