; pro rhd_pulsationcycle_plot
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_pulsationcycle_plot ('rhd_pulsation_cycle_plot')
;
; PURPOSE:
;   Compute and plot work integral from RHD data.
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   .run rhd_pulsationcycle_plot
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, seq
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2016-11-04 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

;model='d3t57g44b0n026'
;model='d3t57g44b0n027'
;model='d3t57g44b0n030'
;model='d3t57g44b0n031'
;model='d3t57g44b0n032'
;model='d3t57g44b0n033'

model='gt50g20n01'
;model='gt51g20n01'
;model='gt51g20n02'
;model='gt52g20n01'
;model='gt52g20n02'
;model='gt52g20n03'
;model='gt53g20n01'
;model='gt53g20n02'
;model='gt54g20n01'
;model='gt54g20n02'
;model='gt54g20n03'
;model='gt54g20n04'
;model='gt54g20n05'
;model='gt55g20n01'
;model='gt56g20n01'
;model='gt56g20n02'
;model='gt56g20n03'
;model='gt56g20n04'
;model='gt58g20n01'
;model='gt58g20n02'
;model='gt58g20n03'
;model='gt58g20n04'
;model='gt58g20n05'
;model='gt60g20n01'
;model='gt60g20n02'
;model='gt60g20n03'
;model='gt60g20n04'
;model='gt62g20n01'
;model='gt62g20n02'

sav_id=''


; --- Load data if necessary ---
if (n_elements(seq) eq 0) then begin
  LoadNewFile_flag=1
endif else begin
  LoadNewFile_flag=(seq.Model ne Model)
endelse
;
if (LoadNewFile_flag) then begin
  file=Model + '_seq' + sav_id + '.sav'
  print, 'Restore data from ''', file, ''''
  restore, file
endif else begin
  print, 'Reuse data for model ', seq.Model
endelse

; --- Load table with model data ---
;.run rhdmodeldata
rhdmodeldata, modDB

; --- Extract/set averaging interval ---
;mtime=10000L
;ntime=99999L
;mtime=600L
;ntime=800L
;mtime=8003L
;ntime=8792L
;mtime=8791L
;ntime=9507L
;mtime=409L
;ntime=807L
; --- model='gt58g20n03'
;  mtime=1020L
;  ntime=2010L
;  mtime=996L
;  ntime=1988L
mtime=(udb_extract('mMeanAvg', [Model], modDB, default=0L    ))[0]
ntime=(udb_extract('nMeanAvg', [Model], modDB, default=99999L))[0]
help, Model, mtime, ntime

; --- Extract some quantities from data structure ---
time   =seq.time[(0L > (mtime < (n_elements(seq.time)-1L))):(0L > (ntime < (n_elements(seq.time)-1L)))]
xc3    = seqavg('xc3'    , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
;
dxb3a  = seqavg('dxb3a'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
;
rho    = seqavg('rho'    , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhov3  = seqavg('rhov3'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhoeg  =-seqavg('rhoeg'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhoei  = seqavg('rhoei'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')

; --- Print information about some arrays ---
print, 'rho:    ', size(rho    , /dimensions), minmax(rho    ), format='(A10,2X,2I6,2X,2E14.6)'
print, 'rhov3:  ', size(rhov3  , /dimensions), minmax(rhov3  ), format='(A10,2X,2I6,2X,2E14.6)'

; --- Compute further quantities ---
rhoek_Mode  = 0.5*rhov3^2/rho

; --- Integrate some quantities over height ---
rho_Int3      =reverse(total(reverse(rho       *dxb3a, 1), 1, /cumulative), 1)
rhov3_Int3    =reverse(total(reverse(rhov3     *dxb3a, 1), 1, /cumulative), 1)
rhoeg_Int3    =reverse(total(reverse(rhoeg     *dxb3a, 1), 1, /cumulative), 1)
rhoei_Int3    =reverse(total(reverse(rhoei     *dxb3a, 1), 1, /cumulative), 1)
rhoek_ModeInt3=reverse(total(reverse(rhoek_Mode*dxb3a, 1), 1, /cumulative), 1)

; --- Take total values ---
rho_Tot3      =reform(rho_Int3[      0,*])
rhov3_Tot3    =reform(rhov3_Int3[    0,*])
rhoeg_Tot3    =reform(rhoeg_Int3[    0,*])
rhoei_Tot3    =reform(rhoei_Int3[    0,*])
rhoek_ModeTot3=reform(rhoek_ModeInt3[0,*])



; --- Compute effective values ---
v3_EffTot3 =rhov3_Tot3/rho_Tot3
;ek3_EffTot3=0.5*abs(v3_EffTot3)*rhov3_Tot3/rho_Tot3



; ======================================

!P.MULTI=[0,2,2,0,0]
!P.CHARSIZE=1.6

; --- Plot 1 ---------------------------

plotlines, time, rhoeg_Tot3


; --- Plot 2 ---------------------------

plotlines, time, v3_EffTot3


; --- Plot 3 ---------------------------

plotlines, rhoeg_Tot3, v3_EffTot3


; --- Plot 4 ---------------------------

plotlines, rhoei_Tot3, v3_EffTot3


; --- ------ ---------------------------

!P.MULTI=0

end ; rhd_pulsationcycle_plot.pro
