; pro rhd_radius_collect_demo
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_radius_collect_demo ('radiation_hydrodynamics_radius_collect_demo')
;
; PURPOSE:
;   Collect data from full file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_radius_collect_demo
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-10-09 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2005-03-10 (B.F.) Demo version
;-
;------------------------------------------------------------------------------

; ================================
;
rhoei_limit1=1.0E+04
rhoei_limit2=1.0E+02
rhoei_limit3=1.0E+00
;
n_timestep=500
model='st35gm04n26' & modelident='_??' & parmodelident='_75'
;model='st35gm04n29' & modelident='_??' & parmodelident='_30'
;model='st35gm04n32' & modelident='_??' & parmodelident='_15'
;
; --- Path to data (local and remote) ---
modeldisk='/u77/bf/dat/rhd/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk='/p1' + '/dat/rhd/d' + model + '/'
;
; ================================

modelfile=modeldisk + model + modelident + '.full'

; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')

; --- Constants ---
year=365.2425*24.0*3600.0
Rsun=6.98000e+10

; --- Initial (fake) data structure ---
radius=create_struct( $
  'itime_last'        , -10000L $
       )

; --- Plot empty frame with correct axes ---
plot, [-1.0, 25.0], [0.0, 850.0], /nodata, $
  xtitle='time [yrs]'       , xstyle=1, xthick=2, $
  ytitle='Radius [R!DSun!N]', ystyle=1, ythick=2, $
  thick=2, charsize=1.8

; --- Initialize loop variables,... ---
delvar, listdata
ierr=0
i=0
file=''
systime0=systime(1)

; --- Loop over all requested time steps ---
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful0=0 &$
  ful0=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=listdata.file(listdata.open_number) &$
    ful=temporary(ful0) &$
    if (ful.modelitime gt (radius.itime_last+1)) then begin &$
      if (i eq 0) then begin &$
        radius=create_struct( $
          'model'              , model, $
          'modelident'         , modelident, $
          'rhoei_limit1'       , rhoei_limit1, $
          'rhoei_limit2'       , rhoei_limit2, $
          'rhoei_limit3'       , rhoei_limit3, $
          'xc1'                , reform(ful.z.xc1), $
          'xc2'                , reform(ful.z.xc2), $
          'xc3'                , reform(ful.z.xc3), $
          'xb1'                , reform(ful.z.xb1), $
          'xb2'                , reform(ful.z.xb2), $
          'xb3'                , reform(ful.z.xb3), $
          'dx1'                , ful.z.xb1(1)-ful.z.xb1(0), $
          'dx2'                , ful.z.xb2(1)-ful.z.xb2(0), $
          'dx3'                , ful.z.xb3(1)-ful.z.xb3(0), $
          'itime'              , lonarr(            n_timestep), $
          'time'               , fltarr(            n_timestep), $
          'ifile'              , lonarr(            n_timestep), $
          'file'               , strarr(            n_timestep), $
          'itime_first'        , ful.z.itime, $
          'itime_last'         , -10000L, $
          'radius_rhoei_limit1', fltarr(            n_timestep), $
          'radius_rhoei_limit2', fltarr(            n_timestep), $
          'radius_rhoei_limit3', fltarr(            n_timestep) $
                  ) &$
        dVol=double(ful.z.xb1(1)-ful.z.xb1(0)) * (ful.z.xb2(1)-ful.z.xb2(0)) * (ful.z.xb3(1)-ful.z.xb3(0)) &$
      endif &$
      ;
      nxc1=n_elements(ful.z.xc1) &$
      nxc2=n_elements(ful.z.xc2) &$
      nxc3=n_elements(ful.z.xc3) &$
      ;
      file_tmp       =strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
      radius.ifile(i)=listdata.open_number &$
      radius.file(i) =file_tmp &$
      radius.time(i) =ful.z.time &$
      radius.itime(i)=ful.z.itime &$
      radius.itime_last=ful.z.itime &$
      ;
      datasize=1.0*n_elements(ful.z.rho)*(5.0*4.0/float(2L^20)) &$
      ;
      print, i, (radius.time(i)-radius.time(0))/year, ' yr  ', systime(1)-systime0, ' s' , datasize, ' MByte', $
        format='(I5,1X,F7.3,A,F8.1,A,F6.1,A)' &$
      ;
      rhoei=ful.z.ei*ful.z.rho &$
      ;
      radius.radius_rhoei_limit1(i)=(float(n_elements(where(rhoei ge rhoei_limit1))) * 3.0/(4.0*!PI))^(1.0/3.0) * radius.dx1 &$
      radius.radius_rhoei_limit2(i)=(float(n_elements(where(rhoei ge rhoei_limit2))) * 3.0/(4.0*!PI))^(1.0/3.0) * radius.dx1 &$
      radius.radius_rhoei_limit3(i)=(float(n_elements(where(rhoei ge rhoei_limit3))) * 3.0/(4.0*!PI))^(1.0/3.0) * radius.dx1 &$
      ;
      oplot, [(radius.time(i)-radius.time(0))/year], [radius.radius_rhoei_limit1(i)/Rsun], psym=3, symsize=1.5 &$
      oplot, [(radius.time(i)-radius.time(0))/year], [radius.radius_rhoei_limit2(i)/Rsun], psym=2, symsize=1.5 &$
      oplot, [(radius.time(i)-radius.time(0))/year], [radius.radius_rhoei_limit3(i)/Rsun], psym=3, symsize=1.5 &$
      ;
      i=i+1 &$
    endif &$
  endif &$
endwhile

uio_closrd, listdata.channel

save, filename=model + '_radius_demo.sav', radius

; end ; rhd_radius_collect_demo
