; pro rhd_ring_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_ring_collect ('radiation_hydrodynamics_loop')
;
; PURPOSE:
;   Collect data from full file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_ring_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-05-17 (B.F. Copenhagen) Written
;   2001-04-03 (B.F. Uppsala) rhd_plot -> rhd_collect -> rhd_ring_collect
;   2014-05-20 (B.F.) Some updates, added ring.ei_mean, ring.rhovv_mean
;-
;------------------------------------------------------------------------------
;
; ================================
;
;n_ring=21 & ring_id='_a10-21'
;n_ring=21 & ring_id='_a08-21'
n_ring=11 & ring_id='_a08-11'
;n_ring=21 & ring_id='_a05-21'
;
; ================================
;
;model='st28gm06n18' & modelident='_???' & parmodelident='_027'
model='st28gm06n25' & modelident='_???' & parmodelident='_020'
;model='st28gm06n24' & modelident='_002' & parmodelident='_057'
;model='st33gm06n03' & modelident='_[0-6]?' & parmodelident='_01'
;model='st33gm06n03' & modelident='_17' & parmodelident='_17'
;model='st35gm06n02' & modelident='_??' & parmodelident='_20'
;model='st35gm04n01' & modelident='_??' & parmodelident='_35'
;model='st35gm04n04' & modelident='_??' & parmodelident='_38' & n_timestep=162
;model='st35gm04n26' & modelident='_75' & parmodelident='_75' & n_timestep=1
;model='st47g20n05' & modelident='_??' & parmodelident='_01' & n_timestep=20
;model='st35gm04n32' & modelident='_15' & parmodelident='_15'
;
;modeldisk='/p1' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
modelfile=modeldisk + model + modelident + '.full'
;modelfile=modeldisk + model + modelident + '.end'
;
; --- Read parameter file ---
par=uio_struct_rd(modeldisk + model + parmodelident + '.par')
;
; --- Read EOS file ---
;if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS
  dfopta, opadisk + par.opafile
;endif
f0=!con.sigmasb*par.Teff^4
;
delvar, listdata
ful=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr)
uio_closrd, listdata.channel
;
nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
time_first=ful.modeltime
;
uio_openrd, nc, listdata.file(listdata.total_number-1)
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, ierr, outstr, time_last &$
  endif &$
endwhile
uio_closrd, nc
;
n_timestep_first=round(time_first/par.dtime_out_full)
n_timestep_last =round(time_last/par.dtime_out_full)
n_timestep=n_timestep_last-n_timestep_first+1
;n_timestep=n_timestep
;n_timestep=2   ; 319, 376, 256, 97, 105, 96
help, time_first, time_last, par.dtime_out_full, n_timestep_first, n_timestep_last, n_timestep
;
delvar, listdata
;
; --- Define tau scale ---
;n_tau =101
;logtau=findgen(n_tau)/10.0 - 5.0
n_tau =111
logtau=findgen(n_tau)/10.0 - 6.0
tau=10.0^logtau
;
ring_dist=shift(dist(nxc2,nxc3), nxc2/2, nxc3/2)
ring_dist2=ring_dist^2
ring_radiusb =fltarr(n_ring+1)
ring_radiusc =fltarr(n_ring)
ring_radiusc2=fltarr(n_ring)
ring_radiusb(n_ring)=max(ring_dist)*1.00001
;
;ring_radiusb[0:n_ring-1]=findgen(n_ring)/float(n_ring-1) * float((nxc2 < nxc3))/2.0 
;ring_radiusb[0:n_ring-1]=(findgen(n_ring)/float(n_ring-1))^0.8 * float((nxc2 < nxc3))/2.0
;ring_radiusb[0:n_ring-1]=(findgen(n_ring)/float(n_ring-1))^0.8 * float((nxc2 < nxc3))/4.0
ring_radiusb[0:n_ring-1]=(findgen(n_ring)/float(n_ring-1))^0.8 * float((nxc2 < nxc3))/5.0
;ring_radiusb[0:n_ring-1]=sqrt(findgen(n_ring)/float(n_ring-1)) * float((nxc2 < nxc3))/2.0 
;
ring_index=lonarr(nxc2,nxc3)
dx=ful.z.xb1[1]-ful.z.xb1[0]
for i=1,n_ring do begin &$
  ind=where((ring_dist ge ring_radiusb[i-1]) and (ring_dist lt ring_radiusb[i])) &$
  ring_index[ind] = i-1 &$
  ring_radiusc[i-1]=avg(ring_dist[ind]) &$
  ring_radiusc2[i-1]=sqrt(avg(ring_dist2[ind])) &$
endfor
;
ring=create_struct( $
  'model'             , model, $
  'modelident'        , modelident, $
  'par'               , par, $
  'xc1'               , reform(ful.z.xc1), $
  'xc2'               , reform(ful.z.xc2), $
  'xc3'               , reform(ful.z.xc3), $
  'xb1'               , reform(ful.z.xb1), $
  'xb2'               , reform(ful.z.xb2), $
  'xb3'               , reform(ful.z.xb3), $
  'dx1'               , ful.z.xb1[1]-ful.z.xb1[0], $
  'dx2'               , ful.z.xb2[1]-ful.z.xb2[0], $
  'dx3'               , ful.z.xb3[1]-ful.z.xb3[0], $
  'itime'             , lonarr(            n_timestep), $
  'time'              , fltarr(            n_timestep), $
  'ifile'             , lonarr(            n_timestep), $
  'file'              , strarr(            n_timestep), $
  'itime_first'       , ful.z.itime, $
  'itime_last'        , -10000L, $
  'surface_id'        , ['1l', '1r', '2l', '2r', '3l', '3r'], $
  'dist'          , ring_dist                     , $
  'index'         , ring_index                    , $
  'radiusb'       , ring_radiusb                  , $
  'radiusc'       , ring_radiusc                  , $
  'radiusc2'      , ring_radiusc2                 , $
  'tau_mean'      , 10.0^logtau                   , $
  'P_mean'        , fltarr(n_tau, n_ring, 6, n_timestep), $
  'rho_mean'      , fltarr(n_tau, n_ring, 6, n_timestep), $
  'ei_mean'       , fltarr(n_tau, n_ring, 6, n_timestep), $
  'T_mean'        , fltarr(n_tau, n_ring, 6, n_timestep), $
  'T_mean2'       , fltarr(n_tau, n_ring, 6, n_timestep), $
  'T_mean4'       , fltarr(n_tau, n_ring, 6, n_timestep), $
  'v_mean'        , fltarr(n_tau, n_ring, 6, n_timestep), $
  'rhovv_mean'    , fltarr(n_tau, n_ring, 6, n_timestep), $
  'v_mean2'       , fltarr(n_tau, n_ring, 6, n_timestep), $
  'x_mean'        , fltarr(n_tau, n_ring, 6, n_timestep), $
  'x_mean2'       , fltarr(n_tau, n_ring, 6, n_timestep), $
  'r_mean'        , fltarr(n_tau, n_ring, 6, n_timestep) $
                  )
;
.run rhd_ring_data
;
ierr=0
i=0
file=''
systime0=systime(1)
;
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful0=0 &$
  ful0=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=listdata.file(listdata.open_number) &$
    ful=temporary(ful0) &$
    if (ful.modelitime gt (ring.itime_last+1)) then begin &$
      print, '--- ', i, ful.z.itime, ful.z.time, systime(1)-systime0, ' ---' , $
        format='(A,I4,X,I6,E15.6,1X,F8.1,A)' &$
      ;
      eosbox, ful, eos=EOS, /opa, out_quantities=1 &$
      ;
      file_tmp     =strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
      ring.ifile[i]=listdata.open_number &$
      ring.file[i] =file_tmp &$
      ;
      rhd_ring_data, ful, ring, i, $
        n_ring, ring_index, $
        n_tau, logtau, tau &$
      ;
      i=i+1 &$
    endif &$
  endif &$
endwhile
;
save, filename=model + '_ring' + ring_id + '.sav', ring
;
; end ; rhd_ring_collect
