; pro rhd_seq2tab
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq2tab ('radiation_hydrodynamcis_path_to_table')
;
; PURPOSE:
;   Read sequence file and put selected data into ASCII table
;
; CATEGORY:
;   RHD, data processing
;
; CALLING SEQUENCE:
;   @rhd_seq2tab
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, seq
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-05-21 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2006-04-19 (B.F. LA) rhd_path2tab -> rage_path2tab
;   2006-04-28 (B.F.) rage_path2tab -> rhd_seq2tab
;-
;------------------------------------------------------------------------------

; === Manual selection ===========
if (not keyword_set(external_control)) then begin &$
  ;
  ; --- Model ---
  ;model='st35gm04n26' &$
  ;model='st35gm04n29' &$
  ;model='st35gm04n30' &$
  ;model='st35gm04n32' &$
  model='st35gm04n34' &$
  ;
  ; --- Input file with path data ---
  seqfile=model + '_seq.sav' &$
  ;
  ; --- Output files ---
  UIOfile=model + '_intseq_01.tab' &$
  outfile=model + '_intseq_01.txt' &$
  ;
endif
; ================================

delvar,seq
if (n_elements(seq) eq 0) then begin &$
  restore, seqfile, /verbose &$
  n_time=n_elements(seq.time) &$
endif

; --- Create UIO table ---
delvar, tab
uio_tab0, tab, outstr, ierr, nlinmx=n_time, $
  ntaimx=2, ntarmx=8, ntasmx=1

uio_tabw, '', 'intseq', tab, format='1X', $
  name='Table with particle paths', $
  date='now', /tableheader

timearr=seq.time
uio_tabw, timearr, 'time', tab, format='E13.6', $
  unit='s', name='Time'

Inormfac=avg([avg(abs(seq.I1l)), avg(abs(seq.I2l)), avg(abs(seq.I3l)), $
              avg(abs(seq.I1r)), avg(abs(seq.I2r)), avg(abs(seq.I3r))])

I1larr=abs(seq.I1l)/Inormfac
uio_tabw, I1larr, 'I1l', tab, format='E13.6', $
  name='Intensity, left face 1'
I2larr=abs(seq.I2l)/Inormfac
uio_tabw, I2larr, 'I2l', tab, format='E13.6', $
  name='Intensity, left face 2'
I3larr=abs(seq.I3l)/Inormfac
uio_tabw, I3larr, 'I3l', tab, format='E13.6', $
  name='Intensity, left face 3'

I1rarr=abs(seq.I1r)/Inormfac
uio_tabw, I1rarr, 'I1r', tab, format='E13.6', $
  name='Intensity, right face 1'
I2rarr=abs(seq.I2r)/Inormfac
uio_tabw, I2rarr, 'I2r', tab, format='E13.6', $
  name='Intensity, right face 2'
I3rarr=abs(seq.I3r)/Inormfac
uio_tabw, I3rarr, 'I3r', tab, format='E13.6', $
  name='Intensity, right face 3'


; --- Write to UIO file ---
uio_openwr, nc, UIOfile, $
            progrm='rhd_seq2tab', $
            form='formatted', conv='default'
uio_wrtab, nc, tab
uio_closwr, nc


; --- Write to text file ---
get_lun, nc
openw, nc, outfile
printf, nc, "# Table with intensity sequences: " + systime(0)
printf, nc, n_time, "Number of time steps", format='(I13,4X,A)'
printf, nc, 7,      "Number of columns", format='(I13,4X,A)'
printf, nc, 1,      "Number of groups", format='(I13,4X,A)'
printf, nc, " time          I1l           I2l           I3l           I1r           I2r           I3r"
for i=0L,n_time-1L do begin &$
  printf, nc, timearr(i), $
              I1larr(i),  I2larr(i),  I3larr(i), $
              I1rarr(i),  I2rarr(i),  I3rarr(i), $
              format='(E13.6,6(1X,E13.6))' &$
endfor
close, nc
free_lun, nc

;
; end ; rhd_seq2tab
