; pro rhd_seq2tab_cepheid
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq2tab_cepheid ('radiation_hydrodynamcis_sequence_to_table_z')
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @rhd_seq2tab_cepheid
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, seq
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-05-21 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2006-04-19 (B.F. LA) rhd_path2tab -> rage_path2tab
;   2006-08-09 (B.F. LA) rage_path2tab -> rhd_seq2tabz
;   2007-11-19 (B.F. Lyon) rhd_seq2tabz -> rhd_seq2tab_cepheid
;   2008-02-04 (B.F.) Models gt50g20n01, gt51g20n01 added
;-
;------------------------------------------------------------------------------

@modelplotdata_cepheid

; === Manual selection =============================
if (not keyword_set(external_control)) then begin &$
  ;
  ; --- Model ---
  pd=[gt50g20n01_pd, gt51g20n01_pd, gt52g20n01_pd, gt52g20n02_pd, gt52g20n03_pd, gt53g20n01_pd, gt54g20n01_pd, gt54g20n02_pd, gt56g20n01_pd, gt58g20n01_pd, gt58g20n02_pd, gt60g20n01_pd, gt60g20n02_pd, gt62g20n01_pd, gt62g20n02_pd] &$
  ;pd=[gt50g20n01_pd, gt51g20n01_pd] &$
  ;pd=[gt54g20n01_pd, gt62g20n02_pd] &$
  ;
  ; --- Output quantities ---
  qidlist=['xc3', 'rho', 'ei', 'T', 'P', 's', 'v1rms', 'v3rms', 'feipkv0', 'fek0', 'fer'] &$
  ;
endif
; ==================================================

iqs=(where(qidlist eq 's'))
iqP=(where(qidlist eq 'P'))
iqfeipkv0=(where(qidlist eq 'feipkv0'))
iqfek0=(where(qidlist eq 'fek0'))
iqfer=(where(qidlist eq 'fer'))

nfile=n_elements(pd)
nq   =n_elements(qidlist)

delvar, tab

print, 'model', 'mtime', 'ntime1', 'ntime', 'time', 'par.Teff', 'Teffreal', 'Frad_bot', 'Frad_top', 'min(Fkin)', 'max(Fconv)', 'P(s_low)', 'P(s_up)', $
  format='(A11,3A7,A11,2X,2A9,2X,4A13,2X,2A13)'

; --- Loop over all model files ---
for ifile=0,nfile-1 do begin &$
  ; --- Read averaged model data ---
  model=pd[ifile].model &$
  mtime=pd[ifile].mmeanavg &$
  ntime=pd[ifile].nmeanavg &$
  ;print, ifile, ' ', model, mtime, ntime &$
  seq=0 &$
  restore, model + '_seq.sav', verbose=0 &$
  n3=n_elements(seq.v3[*,0]) &$
  ;
  ; --- Create arrays ---
  rtab=fltarr(nq, n3) &$
  nametab=strarr(nq) &$
  unittab=strarr(nq) &$
  ;
  ; --- Loop over all output quantities ---
  for iq=0,nq-1 do begin &$
    q=seqavg(qidlist[iq], seq, mtime=mtime, ntime=ntime, unitsystem='cgs', uniteflux='cgs', intertoxc=1, $
             qname=qname, qunit=qunit) &$
    rtab[   iq,*]=q &$
    nametab[iq]  =qname &$
    unittab[iq]  =qunit &$
  endfor &$
  ;
  par=uio_struct_rd(getenv('HOME') + '/dat/rhd/d' + model + '/' + model + pd[ifile].parmodelident + '.par') &$
  ;plotlines,reform(rtab[0,*]),reform(rtab[5,*]),over=(ifile gt 0),drange=[2.0e+09,3.2e+09] &$
  ;plotlines,reform(rtab[0,*]),reform(rtab[iqfeipkv0,*]),over=(ifile gt 0),drange=[-4.0e+10,4e+10] &$
  plotlines,reform(rtab[0,*]),reform(rtab[iqs,*]),over=(ifile gt 0),drange=[1.8e+9,3.6e+9] &$
  ;plotlines,reform(rtab[0,*]),reform(rtab[7,*]),over=(ifile gt 0),xrange=[-6.0E11,1.5E11] &$
  ;plotlines,reform(rtab[0,*]),reform(rtab[8,*])/(par.Teff^4*!con.sigmasb),over=(ifile gt 0) &$
  ntime1=ntime<n_elements(seq.time) &$
  s=reform(rtab[iqs,*]) &$
  P=reform(rtab[iqP,*]) &$
  ind_slow=(where(deriv(s) lt 0.0))[0] &$
  ind_sup=(where(deriv(reverse(s)) gt 0.0))[0] &$
  print, model, mtime, ntime1, ntime, seq.time(ntime1-1)-seq.time(mtime), $
    par.Teff, sqrt(sqrt(rtab[iqfer,n3-1]/!con.sigmasb)), $
    rtab[iqfer,0], rtab[iqfer,n3-1], min(rtab[iqfek0,*]), max(rtab[iqfeipkv0,*]), $
    P[ind_slow], (reverse(P))[ind_sup], $
    format='(A11,3I7,F11.0,2X,2F9.2,2X,4E13.5,2X,2E13.5)' &$
  ;
  ; --- Write into output table: UIO ---
  UIOfile=model + '_seqtab.tab' &$
  ; --- Create UIO table ---
  uio_tab0, tab, outstr, ierr, nlinmx=n3, $
    ntaimx=2, ntarmx=12, ntasmx=1 &$
  uio_tabw, '', 'seq', tab, format='1X', $
    name='Table with horizontal averaged data', $
    date='now', /tableheader &$
  ; --- Loop over all output quantities ---
  for iq=0,nq-1 do begin &$
    uio_tabw, reform(rtab[iq,*]), qidlist[iq], tab, format='E13.6', $
      unit=unittab[iq], name=nametab[iq] &$
  endfor &$
  ; --- Write to UIO file ---
  uio_openwr, nc, UIOfile, $
              progrm='rhd_seq2tab_cepheid', $
              form='formatted', conv='default' &$
  uio_wrtab, nc, tab &$
  uio_closwr, nc &$
  ;
  ; --- Write into output table: ASCII ---
  outfile=model + '_seqtab.txt' &$
  ; --- Write to text file ---
  get_lun, nc &$
  openw, nc, outfile &$
  printf, nc, '# Table with horizontally averaged data: ' + systime(0) &$
  printf, nc, '#', n3,  "Number of height points", format='(A,I13,4X,A)' &$
  printf, nc, '#', nq,  "Number of columns", format='(A,I13,4X,A)' &$
  printf, nc, '#', 1,   "Number of groups", format='(A,I13,4X,A)' &$
  printf, nc, '#', qidlist, format='(A,A12,' + strtrim(string(nq-1)) + '(1X,A13))' &$
  rtabformat='(E13.6,' + strtrim(string(nq-1)) + '(1X,E13.6))' &$
  for i3=0,n3-1 do begin &$
    printf, nc, rtab[*,i3], format=rtabformat &$
   endfor &$
  close, nc &$
  free_lun, nc &$
  ;
endfor

; end ; rhd_seq2tab_cepheid
