; pro rhd_seq2tabz
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq2tabz ('radiation_hydrodynamcis_sequence_to_table_z')
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @rhd_seq2tabz
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, seq
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2003-05-21 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2006-04-19 (B.F. LA) rhd_path2tab -> rage_path2tab
;   2006-08-09 (B.F. LA) rage_path2tab -> rhd_seq2tabz
;-
;------------------------------------------------------------------------------

; === Manual selection ===========
if (not keyword_set(external_control)) then begin &$
  ;
  ; --- Model ---
  model='at80g40n22' & modelident='_??' & parmodelident='_04' &$
  mtime=30 & ntime=10000L &$
  ;
  ; --- Input file with seq data ---
  seqfile=model + '_seq.sav' &$
  ;
  ; --- Output files ---
  UIOfile=model + '_seqtab.tab' &$
  outfile=model + '_seqtab.txt' &$
  ;
endif
; ================================

; --- Restore data ---
if (n_elements(seq) eq 0) then begin &$
  restore, seqfile, /verbose &$
  n_z=n_elements(seq.v3[*,0]) &$
endif

; --- Create UIO table ---
n_col=4
delvar, tab
uio_tab0, tab, outstr, ierr, nlinmx=n_z, $
  ntaimx=2, ntarmx=8, ntasmx=1

uio_tabw, '', 'seq', tab, format='1X', $
  name='Table with horizontal averaged data', $
  date='now', /tableheader

x3arr=seqavg('xc3', seq, mtime=mtime, ntime=ntime) 
uio_tabw, x3arr, 'x3', tab, format='E13.6', $
  unit='Mm', name='x3 coordinate'

tauarr=seqavg('tau', seq, mtime=mtime, ntime=ntime) 
uio_tabw, tauarr, 'tau', tab, format='E13.6', $
  unit='1', name='Rosseland optical depth'

Tarr=seqavg('T', seq, mtime=mtime, ntime=ntime)
uio_tabw, Tarr, 't', tab, format='E13.6', $
  unit='K', name='Temperature'

Parr=seqavg('P', seq, mtime=mtime, ntime=ntime)
uio_tabw, Parr, 'p', tab, format='E13.6', $
  unit='dyn/cm^2', name='Pressure'

; --- Write to UIO file ---
uio_openwr, nc, UIOfile, $
            progrm='rhd_seq2tabz', $
            form='formatted', conv='default'
uio_wrtab, nc, tab
uio_closwr, nc

; --- Write to text file ---
get_lun, nc
openw, nc, outfile
printf, nc, "# Table with horizontally averaged data: " + systime(0)
printf, nc, n_z,    "Number of height points", format='(I13,4X,A)'
printf, nc, n_col,  "Number of columns", format='(I13,4X,A)'
printf, nc, 1,      "Number of groups", format='(I13,4X,A)'
printf, nc, "z              tau           T             P"
;printf, nc, "iseq          time            x1            x2            x3           rho            ei             t"
for i=0L,n_z-1L do begin &$
  printf, nc, x3arr[i], tauarr[i], Tarr[i], Parr[i], format='(E13.6,3(1X,E13.6))' &$
endfor
close, nc
free_lun, nc

;
; end ; rhd_seq2tabz
