; pro rhd_seq_collect
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq_collect ('radiation_hydrodynamics_sequence_collect')
;
; PURPOSE:
;   Collect data from mean files into single structure.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_seq_collect
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES/FUNCTIONS:
;   rhd_seq_put2struct:
;   seqrebin:
;   udb_Extract:
;   uio_closrd:
;   uio_datasetlist_rd:
;   uio_openrd:
;   uio_slhd:
;   uio_struct_rd:
;
; SIDE EFFECTS:
;   Files are read and written. Variables are created.
;
; RESTRICTIONS:
;   Some specifications in the header (e.g., the name of the model) have to be edited by hand.
;
; PROCEDURE:
;   Loop over all datasets in all specified mean files of the specified model,
;   extract part of the mean data (spherical averages), and put them into a structure,
;   where the individual bulk entries have a radius and a time coordinate.
;   Optionally, also create a structure with surface intensity maps.
;
; EXAMPLE:
;   @rhd_seq_collect
;
; MODIFICATION HISTORY:
;   1999-05-17 (B.F. Copenhagen) Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   2001-04-03 (B.F. Uppsala) rhd_plot -> rhd_collect
;   2003-10-10 (B.F.) rhd_collect -> rhd_seq_collect
;   2009-11-10 (B.F.) Call rhd_seq_put2struct for initialization
;   2018-08-10 (B.F.) Clean up and add comments
;-
;---------------------------------------------------------------------------------------------------
;
;ModelIdent=''
n_timestep_adjust=0
BinFactor=20L

 ;Model='st22g35n01' & ModelIdent='_???' & parModelIdent='_016'
 ;Model='st22g35n02' & ModelIdent='_???' & parModelIdent='_042'
 ;Model='st22g35n03' & ModelIdent='_???' & parModelIdent='_009'
 ;Model='st22g35n04' & ModelIdent='_???' & parModelIdent='_030'
 ;Model='st22g35n05' & ModelIdent='_???' & parModelIdent='_080'
 ;Model='st22g35n06' & ModelIdent='_???' & parModelIdent='_070'
 ;Model='st22g35n07' & ModelIdent='_???' & parModelIdent='_170'
 ;Model='st22g35n08' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st22g40n01' & ModelIdent='_???' & parModelIdent='_014'

 ;Model='st26g35n01' & ModelIdent='_???' & parModelIdent='_011'
 ;Model='st26g35n02' & ModelIdent='_???' & parModelIdent='_400'
 ;Model='st26g35n03' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st26g35n04' & ModelIdent='_???' & parModelIdent='_060'
 ;Model='st26g35n05' & ModelIdent='_???' & parModelIdent='_003'
 ;Model='st26g35n06' & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st26g35n07' & ModelIdent='_???' & parModelIdent='_070'
 ;Model='st26g35n08' & ModelIdent='_???' & parModelIdent='_070'
 ;Model='st26g35n09' & ModelIdent='_???' & parModelIdent='_070'
 ;Model='st26g35n10' & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st26gm09n01' & ModelIdent='_??' & parModelIdent='_03'
 ;Model='st26gm09n02' & ModelIdent='_??' & parModelIdent='_07' & n_timestep_adjust=-1
 ;Model='st26gm07n001' & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st26gm07n002' & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st27gm06n001' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm07n001' & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st28gm07n002' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm07n003' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm07n004' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm07n005' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n01' & ModelIdent='_??' & parModelIdent='_11'
 ;Model='st28gm06n02' & ModelIdent='_??' & parModelIdent='_29'
 ;Model='st28gm06n03' & ModelIdent='_??' & parModelIdent='_17'
 ;Model='st28gm06n05' & ModelIdent='_??' & parModelIdent='_24'
 ;Model='st28gm06n06' & ModelIdent='_??' & parModelIdent='_23'
 ;Model='st28gm06n07' & ModelIdent='_??' & parModelIdent='_07'
 ;Model='st28gm06n09e' & ModelIdent='_??' & parModelIdent='_28' & n_timestep_adjust=-8
 ;Model='st28gm06n09t4' & ModelIdent='_??' & parModelIdent='_55' & n_timestep_adjust=0
 ;Model='st28gm06n10d' & ModelIdent='_??' & parModelIdent='_02'
 ;Model='st28gm06n11' & ModelIdent='_??' & parModelIdent='_01'
 ;Model='st28gm06n12' & ModelIdent='_???' & parModelIdent='_003'
 ;Model='st28gm06n13' & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st28gm06n14' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n15' & ModelIdent='_???' & parModelIdent='_130'
 ;Model='st28gm06n16' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-40
 ;Model='st28gm06n17' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n18' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n19' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n20' & ModelIdent='_???' & parModelIdent='_080'
 ;Model='st28gm06n21' & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st28gm06n22' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n23' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n24' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n25' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n26' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n27' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n28' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n29' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n30' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm06n031' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n032' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n033' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n034' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n036' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n037' & ModelIdent='_???' & parModelIdent='_250'
 ;Model='st28gm06n038' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm06n039' & ModelIdent='_???' & parModelIdent='_200'
se ;Model='st28gm06n040' & ModelIdent='_???' & parModelIdent='_040'
 ;Model='st28gm06n041' & ModelIdent='_???' & parModelIdent='_040'
 Model='st28gm06n042' & ModelIdent='_???' & parModelIdent='_040'

 ;Model='st28gm05n001' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm05n002' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st28gm05n003' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm05n004' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm05n005' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n006' & ModelIdent='_???' & parModelIdent='_350'
 ;Model='st28gm05n007' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n008' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm05n009' & ModelIdent='_???' & parModelIdent='_250'
 ;Model='st28gm05n010' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n011' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n012' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n013' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st28gm05n014' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n015' & ModelIdent='_???' & parModelIdent='_090'
 ;Model='st28gm05n016' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n017' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n018' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n019' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n020' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n021' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n022' & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st28gm05n023' & ModelIdent='_???' & parModelIdent='_340'
 ;Model='st29gm06n001' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st29gm04n001' & ModelIdent='_???' & parModelIdent='_100'

 ;Model='st33gm06n02' & ModelIdent='_??' & parModelIdent='_21'
 ;old;Model='st33gm06n03' & ModelIdent='_00-60' & parModelIdent='_01'
 ;Model='st33gm06n03' & ModelIdent='_??' & parModelIdent='_60'

 ;old;Model='st35gm06n01' & ModelIdent='_02-03' & parModelIdent='_02'
 ;Model='st35gm06n01' & ModelIdent='_??' & parModelIdent='_03'
 ;Model='st35gm06n02' & ModelIdent='_??' & parModelIdent='_20'

 ;Model='st35gm04n01' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=228
 ;Model='st35gm04n02' & ModelIdent='_??' & parModelIdent='_27' & n_timestep=415
 ;Model='st35gm04n03' & ModelIdent='_??' & parModelIdent='_26' & n_timestep=518
 ;Model='st35gm04n04' & ModelIdent='_??' & parModelIdent='_38' & n_timestep=648 ; dtime=0.193298E+09
 ;Model='st35gm04n05' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=307
 ;Model='st35gm04n06' & ModelIdent='_??' & parModelIdent='_15' & n_timestep=384
 ;Model='st35gm04n07' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=381
 ;Model='st35gm04n08' & ModelIdent='_??' & parModelIdent='_04' & n_timestep_adjust=-2
 ;Model='st35gm04n20' & ModelIdent='_??' & parModelIdent='_04'
 ;Model='st35gm04n21' & ModelIdent='_??' & parModelIdent='_13'
 ;Model='st35gm04n22' & ModelIdent='_??' & parModelIdent='_42' & n_timestep_adjust=-2
 ;old;Model='st35gm04n23' & ModelIdent='_0[1-7]' & parModelIdent='_07'
 ;Model='st35gm04n23' & ModelIdent='_??' & parModelIdent='_07'
 ;Model='st35gm04n24' & ModelIdent='_??' & parModelIdent='_13'
 ;Model='st35gm04n25' & ModelIdent='_??' & parModelIdent='_27'
 ;Model='st35gm04n26' & ModelIdent='_??' & parModelIdent='_75'
 ;Model='st35gm04n27' & ModelIdent='_??' & parModelIdent='_08'
 ;Model='st35gm04n28' & ModelIdent='_??' & parModelIdent='_55' & n_timestep_adjust=-15
 ;Model='st35gm04n29' & ModelIdent='_??' & parModelIdent='_30'
 ;Model='st35gm04n30' & ModelIdent='_??' & parModelIdent='_20'
 ;Model='st35gm04n31' & ModelIdent='_??' & parModelIdent='_40'
 ;Model='st35gm04n32' & ModelIdent='_??' & parModelIdent='_15' & n_timestep_adjust=-1
 ;Model='st35gm04n33' & ModelIdent='_??' & parModelIdent='_26'
 ;Model='st35gm04n34' & ModelIdent='_??' & parModelIdent='_36'
 ;Model='st35gm04n36' & ModelIdent='_???' & parModelIdent='_090'
 ;Model='st35gm04n37' & ModelIdent='_???' & parModelIdent='_180'
 ;Model='st35gm04n38' & ModelIdent='_???' & parModelIdent='_027'
 ;Model='st35gm04n41' & ModelIdent='_???' & parModelIdent='_999' & n_timestep_adjust=-3
 ;Model='st35gm04n42' & ModelIdent='_???' & parModelIdent='_255'
 ;Model='st35gm04n43' & ModelIdent='_???' & parModelIdent='_244'
 ;Model='st35gm04b0n001' & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st35gm04b0n002' & ModelIdent='_???' & parModelIdent='_082'
 ;Model='st35gm04b1n001' & ModelIdent='_???' & parModelIdent='_446'
 ;Model='st35gm04b1n002' & ModelIdent='_???' & parModelIdent='_999'
 ;Model='st35gm04b1n003' & ModelIdent='_???' & parModelIdent='_606'
 ;Model='st35gm04b1n004' & ModelIdent='_???' & parModelIdent='_141'
 ;Model='st35gm04b1n005' & ModelIdent='_???' & parModelIdent='_184'

 ;Model='st35gm03n01' & ModelIdent='_??' & parModelIdent='_04'
 ;Model='st35gm03n02' & ModelIdent='_??' & parModelIdent='_30' & n_timestep_adjust=-26
 ;Model='st35gm03n03' & ModelIdent='_??' & parModelIdent='_09'
 ;Model='st35gm03n04' & ModelIdent='_??' & parModelIdent='_50'
 ;Model='st35gm03n05' & ModelIdent='_??' & parModelIdent='_30'
 ;Model='st35gm03n06' & ModelIdent='_??' & parModelIdent='_80' & n_timestep_adjust=-1
 ;Model='st35gm03n07' & ModelIdent='_???' & parModelIdent='_161'
 ;Model='st35gm03n08' & ModelIdent='_??' & parModelIdent='_21'
 ;Model='st35gm03n09' & ModelIdent='_??' & parModelIdent='_80'
 ;Model='st35gm03n10' & ModelIdent='_??' & parModelIdent='_70' & n_timestep_adjust=-3
 ;Model='st35gm03n11' & ModelIdent='_??' & parModelIdent='_75' & n_timestep_adjust=-1
 ;Model='st35gm03n13' & ModelIdent='_???' & parModelIdent='_708' & n_timestep_adjust=-301
 ;Model='st35gm03n14c' & ModelIdent='_[01]??' & parModelIdent='_199'
 ;Model='st35gm03n15' & ModelIdent='_???' & parModelIdent='_008'
 ;Model='st35gm03n16' & ModelIdent='_???' & parModelIdent='_080'
 ;Model='st35gm03n17' & ModelIdent='_???' & parModelIdent='_200'
 ;Model='st35gm03n18' & ModelIdent='_???' & parModelIdent='_018'
 ;Model='st35gm03n19' & ModelIdent='_???' & parModelIdent='_100'

 ;Model='st35gm00n01' & ModelIdent='_???' & parModelIdent='_029'
 ;Model='st35gm00n01_tweak' & ModelIdent='_???' & parModelIdent='_024'

 ;Model='st36g00n01' & ModelIdent='_???' & parModelIdent='_029'
 ;Model='st36g00n03' & ModelIdent='_???' & parModelIdent='_012'
 ;Model='st36g00n04' & ModelIdent='_???' & parModelIdent='_071'
 ;Model='st36g00n05' & ModelIdent='_???' & parModelIdent='_303' & n_timestep_adjust=-250

 ;Model='st36g45n04'     & ModelIdent='_??' & parModelIdent='_24' & n_timestep_adjust=250
 ;Model='st36g45n04rot'  & ModelIdent='_??' & parModelIdent='_23' & n_timestep_adjust=250
 ;Model='st36g45n04rot2' & ModelIdent='_??' & parModelIdent='_14' & n_timestep_adjust=250
 ;Model='st36g45n04rot3' & ModelIdent='_??' & parModelIdent='_29'
 ;Model='st36g45n04rot4' & ModelIdent='_??' & parModelIdent='_29'
 ;Model='st36g45n04rot5' & ModelIdent='_??' & parModelIdent='_29'
 ;Model='st36g45n04hll'  & ModelIdent='_??' & parModelIdent='_13'
 ;Model='st36g45n05'     & ModelIdent='_???' & parModelIdent='_225'
 ;Model='st36g45n06'     & ModelIdent='_???' & parModelIdent='_031'
 ;Model='st36g45n07'     & ModelIdent='_???' & parModelIdent='_034'
 ;Model='st36g45n08'     & ModelIdent='_???' & parModelIdent='_130'
 ;Model='st36g45n09'     & ModelIdent='_???' & parModelIdent='_150'
 ;Model='st36g45n10'     & ModelIdent='_???' & parModelIdent='_150'
 ;Model='st36g45n11'     & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st36g45n12'     & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st36g45n13'     & ModelIdent='_???' & parModelIdent='_400'
 ;Model='st36g45n14'     & ModelIdent='_???' & parModelIdent='_400'
 ;Model='st36g45n15'     & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st36g45n16'     & ModelIdent='_???' & parModelIdent='_300'
 ;Model='st36g45n17'     & ModelIdent='_???' & parModelIdent='_267'
 ;Model='st36g45n18'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n19'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n20'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n21'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n22'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n23'     & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st36g45n24'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n25'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n26'     & ModelIdent='_???' & parModelIdent='_127'
 ;Model='st36g45n27'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n28'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n29'     & ModelIdent='_???' & parModelIdent='_150'
 ;Model='st36g45n30'     & ModelIdent='_???' & parModelIdent='_060'
 ;Model='st36g45n31'     & ModelIdent='_???' & parModelIdent='_120'
 ;Model='st36g45n32'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n33'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n34'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n35'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n36'     & ModelIdent='_???' & parModelIdent='_080'
 ;Model='st36g45n37'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n38'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n39'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n40'     & ModelIdent='_???' & parModelIdent='_040'
 ;Model='st36g45n41'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n42'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n43'     & ModelIdent='_???' & parModelIdent='_100'
 ;Model='st36g45n44'     & ModelIdent='_???' & parModelIdent='_030'
 ;Model='st36g45n45'     & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st36g45n46'     & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st36g45n47'     & ModelIdent='_???' & parModelIdent='_025'
 ;Model='st36g45n48'     & ModelIdent='_???' & parModelIdent='_030'
 ;Model='st36g45n49a'    & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st36g45n49b'    & ModelIdent='_???' & parModelIdent='_001'
 ;Model='st36g45n49c'    & ModelIdent='_???' & parModelIdent='_025'
 ;Model='st36g45n49d'    & ModelIdent='_???' & parModelIdent='_040'
 ;Model='st36g45n49e'    & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st36g45n49f'    & ModelIdent='_???' & parModelIdent='_020'
 ;Model='st36g45n49g'    & ModelIdent='_???' & parModelIdent='_030'

 ;Model='st40gm03n01' & ModelIdent='_??' & parModelIdent='_17' & n_timestep_adjust=-37
 ;Model='st40gm03n02' & ModelIdent='_??' & parModelIdent='_39' & n_timestep_adjust=-9
 ;Model='st40gm03n03' & ModelIdent='_??' & parModelIdent='_22'
 ;Model='st40gm03n04' & ModelIdent='_??' & parModelIdent='_99'
 ;Model='st45gm03n01' & ModelIdent='_??' & parModelIdent='_09' & n_timestep_adjust=-5
 ;Model='st45gm03n02' & ModelIdent='_??' & parModelIdent='_07'

 ;Model='st47g20n04'  & ModelIdent='_??' & parModelIdent='_01'
 ;Model='st47g20n05'  & ModelIdent='_??' & parModelIdent='_01'
 ;Model='st57g44n08'  & ModelIdent='_??' & parModelIdent='_13'
 ;Model='st57g44n09'  & ModelIdent='??'  & parModelIdent='az'
 ;Model='st57g44n11a' & ModelIdent='_??' & parModelIdent='01'
 ;Model='rhd' & ModelIdent='' & parModelIdent=''
 ;
 ;allModel='st28gm06n0[5,6]_??'

; --- Read meta data (name and unit of tags) ---
ReadMetaData=1
; --- Read intensity data into separate structure ---
ReadIntens_flag=1

; --- Set path to model master directory ---
ModelDisk=getenv('HOME') + '/dat/rhd/d' + Model + '/'
;ModelDisk='/u64/bf' + '/dat/rhd/d' + Model + '/'
;ModelDisk='/p3/bf' + '/dat/rhd/d' + Model + '/'
;ModelDisk='/u77/bf' + '/dat/rhd/d' + Model + '/'
;ModelDisk=getenv('HOME') + '/dat/job/j5/bak/'
;ModelDisk=getenv('HOME') + '/dat/rhd/tmp/'
ModelFile=ModelDisk + Model + ModelIdent + '.mean'
;ModelFile=ModelDisk + allModel + '.mean'
;ModelFile=ModelDisk + '*' + ModelIdent + '.mean'

; --- Read parameter file ---
par=uio_struct_rd(ModelDisk + Model + parModelIdent + '.par', ReadMetaData=ReadMetaData, /HeadRead)

; --- Read first data set, create file list, and extract some information ---
delvar, ListData
ful=uio_datasetlist_rd(ModelFile, ListData=ListData, ReadMetaData=ReadMetaData, /HeadRead, date=date, ierr=ierr)
uio_closrd, ListData.channel

nr  =n_elements(ful.r.xc3_volume)
nxc1=n_elements(ful.z1.xc1)
nxc2=n_elements(ful.z2.xc2)
nxc3=n_elements(ful.z3.xc3)
time_first=ful.modeltime

; --- Read last data set and extract some information
uio_openrd, nc, listdata.file[listdata.total_number-1]
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, ierr, outstr, time_last &$
  endif &$
endwhile
uio_closrd, nc

n_timestep_first=round(time_first/par.dtime_out_mean)
n_timestep_last =round(time_last/par.dtime_out_mean)
n_timestep      =n_timestep_last-n_timestep_first+1
n_timestep      =n_timestep+n_timestep_adjust
help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep

delvar, listdata

; --- Create output structure for time sequence of mean data ---
delvar,seq
rhd_seq_put2struct, ful, seq, /InitStruct, par=par, $
  ModelName=Model, ModelIdent=ModelIdent, n_timestep=n_timestep, $
  ReadMetaData=ReadMetaData

; --- Optionally, create output structure for time sequence of intensity maps ---
if (ReadIntens_flag) then begin &$
  intens=create_struct( $
    'Model'     , Model, $
    'ModelIdent', ModelIdent, $
    'mean'      , ful, $
    'itime'     , lonarr(            n_timestep), $
    'time'      , fltarr(            n_timestep), $
    'ifile'     , lonarr(            n_timestep), $
    'file'      , strarr(            n_timestep), $
;    'I1l'       , fltarr(nxc1, nxc2, n_timestep), $
;    'I1r'       , fltarr(nxc1, nxc2, n_timestep), $
;    'I2l'       , fltarr(nxc1, nxc3, n_timestep), $
;    'I2r'       , fltarr(nxc1, nxc3, n_timestep), $
;    'I3l'       , fltarr(nxc2, nxc3, n_timestep), $
    'I3r'       , fltarr(nxc2, nxc3, n_timestep) $
                     ) &$
endif ; (ReadIntens_flag)

; --- Intialize counters and timers ---
ierr      =0
i         =0L
itime_last=-10000L
file_last =''
systime0  =systime(1)

; --- Loop over all data sets ---
while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ; --- Read next data set ---
  ful=uio_datasetlist_rd(ModelFile, listdata=listdata, date=date, /HeadRead, ierr=ierr) &$
  ;
  if (ierr eq 0) then begin &$
    if (ful.modelitime gt (itime_last+1)) then begin &$
      ; --- Handle data set if no error occurred and timestep increase is large enough ---
      ;
      print, '--- ', i, '/', n_timestep, ful.z3.itime, ful.z3.time, date, ' ---' , $
             format='(A,I5,A,I5,1X,I8,E15.8,2X,A,A)' &$
      ;
      if (total(strcmp(tag_names(ful.head), 'VERSION')) ge 1.0) then begin &$
        version=ful.head.version &$
      endif else begin &$
        version   ='' &$
      endelse &$
      ;
      file=listdata.file[listdata.open_number] &$
      ;
      ; --- Read parameter file, if new ---
      if (file ne file_last) then par=uio_struct_rd(strmid(file,0,strlen(file)-4) + 'par') &$
      ;
      rhd_seq_put2struct, ful, seq, par=par, $
        snapshot=ListData.open_number, file=file, i_timestep=i, date=date, version=version &$
      ;
      if (ReadIntens_flag) then begin &$
        intens.itime[  i]=ful.z3.itime &$
        intens.time[   i]=ful.z3.time &$
        intens.ifile[  i]=ListData.open_number &$
        intens.file[   i]=strmid(file, strpos(file, '/', /REVERSE_SEARCH)+1) &$
;        intens.I1l[*,*,i]=-(ful.rad.Intb1_l) &$
;        intens.I1r[*,*,i]= (ful.rad.Intb1_r) &$
;        intens.I2l[*,*,i]=-(ful.rad.Intb2_l) &$
;        intens.I2r[*,*,i]= (ful.rad.Intb2_r) &$
;        intens.I3l[*,*,i]=-(ful.rad.Intb3_l) &$
        intens.I3r[*,*,i]= (ful.rad.Intb3_r) &$
      endif &$ ; (ReadIntens_flag) &$
      ;
      file_last =file &$
      itime_last=ful.modelitime &$
      i         =i+1 &$
      ;
    endif &$ ; (ful.modelitime gt (itime_last+1))
  endif &$ ; (ierr eq 0)
endwhile

; --- Check if all time steps are filled ---
ind=where(seq.file eq '', nCount)
if (nCount gt 0) then print, 'WARNING: seq structure not filled completely! ####################### ' + string(nCount)

print, 'Total time: ', systime(1)-systime0, ' sec'

; --- Optionally, safe intensity data ---
if (ReadIntens_flag) then begin &$
  print, 'Save dataset (intens) into ''' + Model + '_intens.sav' + '''' &$
  save, filename=Model + '_intens.sav', intens &$
endif

; --- Save raw mean data ---
savefile=Model + '_seq.sav'
print, 'Save dataset (seq) into ''' + savefile + ''''
save, filename=savefile, seq

; --- Optionally, average mean data over time ---
.run rhdmodeldata
rhdmodeldata, modeltable
mtime=(udb_Extract('mMeanAvg', [Model], modeltable))[0]
ntime=(udb_Extract('nMeanAvg', [Model], modeltable))[0]
if (ntime gt 0) then begin &$
  seq=seqrebin(seq, 0, mtime=mtime, ntime=ntime) &$
  sav_id='_0' &$
  savefile=Model + '_seq' + sav_id + '.sav' &$
  print, 'Save dataset (seq) into ''' + savefile + '''' &$
  save, filename=savefile, seq &$
endif

; --- Optionally, bin mean data ---
if (BinFactor gt 1) then begin &$
  savefile=Model + '_seq.sav' &$
  restore, savefile &$
  sav_id='_' + strtrim(string(BinFactor, format='(I9)'), 2) &$
  savefile=Model + '_seq' + sav_id + '.sav' &$
  seq=seqrebin(seq, BinFactor) &$
  print, 'Save dataset (seq) into ''' + savefile + '''' &$
  save, filename=savefile, seq &$
endif

savefile=Model + '_seq.sav'
restore, savefile

;exit
;stop
;
; --- Some sample plot commands ---
;plotlines, seq.time/!con.y, sqrt(seq.x1center^2+seq.x2center^2+seq.x3center^2)/!con.Rsun
;plotlines, seq.time/!con.y, sqrt(seq.v1center^2+seq.v2center^2+seq.v3center^2)
;plotlines, seq.time/!con.y, seq.i1l+seq.i2l+seq.i3l+seq.i1r+seq.i2r+seq.i3r
;plotlines, seq.time/!con.y, seq.f1l+seq.f2l+seq.f3l+seq.f1r+seq.f2r+seq.f3r
;
;xcenter=sqrt(seq.x1center^2+seq.x2center^2+seq.x3center^2)
;plotlines, seq.time/!con.y,  xcenter/!con.Rsun, dran=[-max(xcenter),max(xcenter)]/!con.Rsun
;plotlines, seq.time/!con.y, -xcenter/!con.Rsun, /over
;plotlines, seq.time/!con.y, (seq.x1center)/!con.Rsun, /over,linecol=!ctv.red
;plotlines, seq.time/!con.y, (seq.x2center)/!con.Rsun, /over,linecol=!ctv.green
;plotlines, seq.time/!con.y, (seq.x3center)/!con.Rsun, /over,linecol=!ctv.blue
;
;vcenter=sqrt(seq.v1center^2+seq.v2center^2+seq.v3center^2)
;plotlines, seq.time/!con.y,  vcenter, dran=[-max(vcenter), max(vcenter)]
;plotlines, seq.time/!con.y, -vcenter, /over
;plotlines, seq.time/!con.y, (seq.v1center), /over,linecol=!ctv.red
;plotlines, seq.time/!con.y, (seq.v2center), /over,linecol=!ctv.green
;plotlines, seq.time/!con.y, (seq.v3center), /over,linecol=!ctv.blue
;
;i=80 & a=4.0D0*!DPI*(reform(seq.mean.r.xb3_surface))(i)^2 & L=reform((seq.feipb+seq.fekb+seq.ferb)(i,*))*a/!con.Lsun
;i=20 & a=4.0D0*!DPI*(reform(seq.mean.r.xc3_volume))(i)^2 &
;Lmass=reform((seq.rhov3/seq.rho*(seq.rhoei+seq.P))(i,*))*a/!con.Lsun
;
;plotlines, seq.time/!con.y,seq.rhototal1*double(seq.mean.z.xb2(n_elements(seq.mean.z.xb2)-1)-seq.mean.z.xb2(0))^3/!con.Msun
;
;w=rebin(reform(seq.xc3_volume,315,1),315,n_elements(seq.time))
;plotlines,seq.time,alog10(total(w*(seqavg('rhoem',seq,qavg=-1)),1))
; plotlines,seq1.time,alog10(total(0.5*w1*(seq1.bc1rms^2+seq1.bc3rms^2),1)),xrange=[0,9e8],drange=[11,20]
; plotlines,seq2.time,alog10(total(w2*(seqavg('rhoek',seq2,qavg=-1)),1)),/over,linecol=!ctv.blue
; plotlines,seq2.time,alog10(total(0.5*w2*(seq2.bc1rms^2+seq2.bc3rms^2),1)),/over,linecol=!ctv.red       
;
; end ; rhd_seq_collect
