; CALLING SEQUENCE:
;   .run rhd_seq_extractpar
;
; MODIFICATION HISTORY:
;   2009-10-17 (B.F. Lyon) Written
;   2010-01-08 (B.F.) New tag 'DateAdded'
;   2013-10-11 (B.F.) Modify formula for radius determination for global models
;   2015-12-16 (B.F. Uppsala) Replace avg() by mean()
;   2017-10-23 (B.F.) Add s_core to basDB
;   2017-10-31 (B.F.) Add RadiusT8K to basDB
;   2018-04-16 (B.F.) Add nbin to basDB

eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'

; --- Choose list file to read and output idendifier ---
OutId=''   & seqSaveFileId   ='*seq_0.sav'     & intensSaveFileId='*intens.sav'

;OutId='mt'      & seqSaveFileId   ='mt?0*seq_0.sav'    & intensSaveFileId='mt?0*intens.sav'
;OutId='mt12'    & seqSaveFileId   ='mt12*seq_0.sav'    & intensSaveFileId='mt12*intens.sav'
;OutId='st28'    & seqSaveFileId   ='st28*seq_0.sav'    & intensSaveFileId='st28*intens.sav'
;OutId='st36g45' & seqSaveFileId   ='st36g45*seq_0.sav' & intensSaveFileId='st36g45*intens.sav'
;OutId='swe'     & seqSaveFileId   ='d3*w1*seq_0.sav'   & intensSaveFileId='d3*w1*intens.sav'
;OutId='Test1'   & seqSaveFileId   ='*seq_0.sav'        & intensSaveFileId='*intens.sav'

DateAdded=systime(/julian)

; --- Load current model data base ---------------------------------------------
;.run rhdmodeldata
rhdmodeldata, modDB


; --- Load database with horizontally averaged data ----------------------------
; --- Clear existing seq db ---
nFile=n_elements(seqDB)
if (nFile gt 0) then print, 'Clear old seq data'
for iFile=0,nFile-1 do begin
  if (ptr_valid(seqDB[iFile])) then ptr_free, seqDB[iFile]
endfor

; --- Get list of files that match the key ---
seqSaveFileList=file_search(seqSaveFileId, count=nFile)
if (nFile eq 0) then begin
  print, 'No files found'
  stop
endif

; --- Restore seq files and put data into db ---
print, '=== Load all seq data: create seqDB database ================================================'
seqDB=ptrarr(nFile, /allocate_heap)
for iFile=0,nFile-1 do begin
  seq=0
  file=seqSaveFileList[iFile]
  print, 'Restore data from ''', file, ''''
  restore, file
  seq=create_struct(seq, $
                    'DateAdded', DateAdded)
  *seqDB[iFile]=temporary(seq)
endfor ; iFile

; --- Save it ---
save, seqDB, file='all' + OutID + 'seqdb.sav', /verbose


; --- Extract parameter database -----------------------------------------------
; --- Clear existing par db ---
npar=n_elements(pardb)
if (npar gt 0) then print, 'Clear old par data'
for i=0,npar-1 do begin
  if (ptr_valid(pardb[i])) then ptr_free, pardb[i]
endfor ; i

; --- Extract the par structure from seq structure ---
print, '=== Load all par data from seq data: create parDB database ==================================='
pardb=ptrarr(nFile, /allocate_heap)
for iFile=0,nFile-1 do begin
  print, iFile, ' ', (*seqDB[iFile]).model
  *pardb[iFile]=create_struct('model'      , (*seqDB[iFile]).model, $
                              'DateAdded'  , DateAdded, $
                              (*seqDB[iFile]).par)
endfor ; iFile

; --- Save it ---
save, pardb, file='all' + OutID + 'pardb.sav'


; --- Extract basic model data from seq structures ------------------------------
print, '=== Extract basic model data from seq structures: create basDB database ======================'
basDB=replicate( $
        create_struct('model'        , '', $
                      'DateAdded'    , DateAdded, $
                      'grav_mode'    , "", $
                      'eosfile'      , "", $
                      'opafile'      , "", $
                      'nBand'        , 0L, $
                      'mtime'        , 0L, $
                      'ntime'        , 0L, $
                      'itime_first'  , 0L, $
                      'itime_last'   , 0L, $
                      'time_first'   , 0.0, $
                      'time_last'    , 0.0, $
                      'Radius'       , 0.0, $
                      'RadiusT8K'    , 0.0, $
                      'mass_star'    , 0.0, $
                      'mass_envelope', 0.0, $
                      'Teff'         , 0.0, $
                      'TeffOut'      , 0.0, $
                      'LuminosityOut', 0.0D+00, $
                      's_inflow'     , 0.0, $
                      's_core'       , 0.0, $
                      'grav'         , 0.0, $
                      'abuX'         , 0.0, $
                      'abuY'         , 0.0, $
                      'abuZ'         , 0.0, $
                      'Qmol'         , 0.0, $
                      'xb1tot'       , 0.0, $
                      'xb2tot'       , 0.0, $
                      'xb3tot'       , 0.0, $
                      'nquc'         , 0L, $
                      'precision'    , 0L, $
                      'dimension'    , 0L, $
                      'nx1'          , 0L, $
                      'nx2'          , 0L, $
                      'nx3'          , 0L $
                     ), $
        nFile)
;
LastEosFile='No'
LastOpaFile='No'
;
for iFile=0,nFile-1 do begin
  basDB[iFile].model      =(*seqDB[iFile]).model
  grav_mode               =strlowcase((*pardb[iFile]).grav_mode)
  basDB[iFile].mtime=long((udb_extract('mMeanAvg', [(*seqDB[iFile]).model], modDB))[0])
  basDB[iFile].ntime=long((udb_extract('nMeanAvg', [(*seqDB[iFile]).model], modDB))[0])
  basDB[iFile].itime_first=(*seqDB[iFile]).itime_first
  basDB[iFile].itime_last =(*seqDB[iFile]).itime_last
  basDB[iFile].time_first =(*seqDB[iFile]).time_first
  basDB[iFile].time_last  =(*seqDB[iFile]).time_last
  basDB[iFile].nx1        =n_elements((*seqDB[iFile]).xc1)
  basDB[iFile].nx2        =n_elements((*seqDB[iFile]).xc2)
  basDB[iFile].nx3        =n_elements((*seqDB[iFile]).xc3)
  basDB[iFile].dimension  =(basDB[iFile].nx1 gt 1) + (basDB[iFile].nx2 gt 1) + (basDB[iFile].nx3 gt 1)
  basDB[iFile].xb1tot     =max((*seqDB[iFile]).xb1)-min((*seqDB[iFile]).xb1)
  basDB[iFile].xb2tot     =max((*seqDB[iFile]).xb2)-min((*seqDB[iFile]).xb2)
  basDB[iFile].xb3tot     =max((*seqDB[iFile]).xb3)-min((*seqDB[iFile]).xb3)
  if (size((*seqDB[iFile]).mean.modeltime, /type) eq (size(1.0, /type))) then begin
    basDB[iFile].precision=4
  endif else begin
    basDB[iFile].precision=8
  endelse
  indquctag=where(strmatch(tag_names((*seqDB[iFile])), 'QUC[0-9][0-9][0-9]'), nquc)
  basDB[iFile].nquc       =nquc
  ;
  basDB[iFile].Teff       =(*pardb[iFile]).Teff
  basDB[iFile].s_inflow   =(*pardb[iFile]).s_inflow
  basDB[iFile].grav_mode  =grav_mode
  ;
  if (grav_mode eq 'central') then begin
    ; === Central potential ===
    ;
    q=seqavg('lumr', (*seqDB[iFile]))
    luminosityout=mean(q[n_elements(q)-11:n_elements(q)-1])
    T =(*seqDB[iFile]).T
    x =(*seqDB[iFile]).xc3_volume
    xb=(*seqDB[iFile]).xb1
    ;
    edge=(max(xb)-min(xb))
    lumofradius=!con.sigmasb * T^4 * 4.0D+00*!DPI*(x*1.0D+00)^2/!con.Lsun
    ; --- Find maximum (probably close to core) ---
    dummy=max(lumofradius, i_max)
    ;dummy=min(lumofradius[i_max:*], i_min)
    ;
    SmoothLumOfRadius=smooth(lumofradius,11)
    ; --- Add a bit ---
    i_max=(i_max+7) > 16L
    dlumdi=deriv(SmoothLumOfRadius[i_max:*])
    ind=where(dlumdi gt 0.0, ncount)
    if (ncount eq 0) then begin
      i_min=n_elements(x[i_max:*])-1L
    endif else begin
      i_min=(reform(ind[0])+3L) < (n_elements(x[i_max:*])-1L)
    endelse
    ;
    i_min=i_min+i_max
    dummy=min((lumofradius[i_max:i_min]-luminosityout)^2, i_est)
    i_est=i_est+i_max
    ; --- Get radius ---
    extremum, x=x, lumofradius-luminosityout, 0, Radius, lradius, i0=i_est, irange=2, degree=2, verbose=0
    ; --- Compute effective temperature ---
    Teff=(1.0D+00*luminosityout*!con.Lsun/(4.0D+00*!DPI*(Radius*1.0D+00)^2)/!con.sigmasb)^0.25
    if (total(tag_names(*parDB[iFile]) eq 'R2_GRAV') gt 0.0) then begin
      r2_grav=(*parDB[iFile]).r2_grav
    endif else begin
      r2_grav=0.0
    endelse
    ; --- Compute surface gravity ---
    grav=abs(spheregrav(Radius, r0=(*parDB[iFile]).r0_grav, r1=(*parDB[iFile]).r1_grav, r2=r2_grav, C=!con.G*(*parDB[iFile]).mass_star))
    ;
    TRadius=8000.0
    if (max(T) ge TRadius) then begin
      i_est=max(where(T ge TRadius))
      extremum, x=x, T-TRadius, 0, RadiusT8K, TRadiusT8K, i0=i_est, irange=2, degree=2, verbose=0
    endif else begin
      RadiusT8K=0.0
    endelse
    ;
    print, (*seqDB[iFile]).model, ': Teff, logg, L, R: ', $
      Teff, alog10(grav), luminosityout, Radius/!con.Rsun, RadiusT8K/!con.Rsun, format='(A18,A,5E15.6)'
    ;
    basDB[iFile].Radius       =Radius
    basDB[iFile].RadiusT8K    =RadiusT8K
    basDB[iFile].TeffOut      =Teff
    basDB[iFile].grav         =grav
    basDB[iFile].LuminosityOut=1.0D+00*luminosityout*!con.Lsun
    basDB[iFile].mass_star    =(*pardb[iFile]).mass_star
    basDB[iFile].mass_envelope=mean(aarsg_seqavg_data('mass1', (*seqDB[iFile]), qavg=-1))*!con.MSun
  endif else begin
    ; === Constant gravity ===
    ;
    ; --- Get Teff from radiative flux at the top ---
    basDB[iFile].Teffout      =sqrt(sqrt((*seqDB[iFile]).ferb[ $
                               n_elements((*seqDB[iFile]).ferb)-1L]/!con.sigmasb))
    ; --- Get the gravity directly from the parameter file ---
    basDB[iFile].grav         =(*pardb[iFile]).grav
    ;
    print, (*seqDB[iFile]).model, ": Teff, logg      : ", basDB[iFile].Teffout, alog10(basDB[iFile].grav), format='(A18,A,2E15.6)'
  endelse
  ;
  ; --- Compute core or bottom entropy by averaging over 5 layers ---
  q=seqavg('s', (*seqDB[iFile]), units='cgs')
  s_core=mean(q[0:4])
  basDB[iFile].s_core=s_core
  ;
  ; --- Load EOS file (if different from previous one) and extract some data ---
  basDB[iFile].eosfile    =(*pardb[iFile]).eosfile
  if (LastEosFile ne basDB[iFile].eosfile) then begin
    print, 'Loading ' + basDB[iFile].eosfile
    opafile=file_search(eosdisk+basDB[iFile].eosfile)
    tabinter_rdcoeff, eosdisk+basDB[iFile].eosfile, EOS
    LastEosFile=basDB[iFile].eosfile
  endif
  basDB[iFile].abuX       =eos.head.abuX
  basDB[iFile].abuY       =eos.head.abuY
  basDB[iFile].abuZ       =eos.head.abuZ
  basDB[iFile].Qmol       =eos.head.Qmol
  ;
  ; --- Load opacity file (if different from previous one) and extract some data ---
  basDB[iFile].opafile    =(*pardb[iFile]).opafile
  if (LastOpaFile ne basDB[iFile].opafile) then begin
    print, 'Loading ' + basDB[iFile].opafile
    OpaFile=file_search(opadisk+basDB[iFile].opafile)
    if (OpaFile ne '') then begin
      rdopta, OpaFile, OPA
      LastOpaFile=basDB[iFile].opafile
    endif else begin
      print, '  ... not found.'
      LastOpaFile='No'
    endelse
  endif
  if (LastOpaFile ne 'No') then begin
    if ((*pardb[iFile]).N_radBand eq 1) then begin
      basDB[iFile].nBand=1
    endif else begin
      basDB[iFile].nBand=OPA.nBand
    endelse
  endif
  ;
endfor ; iFile

; --- Save it ---
save, basDB, file='all' + OutID + 'basdb.sav'

; delvar seqDB
;exit
stop ; --------------------------


; --- Analyse intensity maps ---------------------------------------------------
intensSaveFileList=file_search(intensSaveFileId, count=nFile)
if (nFile eq 0) then begin
  print, 'No files with intensity data found'
  exit
endif
; --- Analyse intensities ---
print, '=== Analyse intensities =============================================================='
; --- Create array of structures ---
intDB=replicate( $
        create_struct('model', '', $
                      'DateAdded'  , DateAdded, $
                      'mtime'      , 0L, $
                      'ntime'      , 0L, $
                      'itime_first', 0L, $
                      'itime_last' , 0L, $
                      'time_first' , 0.0, $
                      'time_last'  , 0.0, $
                      'intmean'    , 0.0, $
                      'intadrms'   , 0.0, $
                      'inttdrms'   , 0.0, $
                      'xm1intfft'  , 0.0, $
                      'xp1intfft'  , 0.0 $
                     ), $
        nFile)
for iFile=0,nFile-1 do begin
  intens=0
  file=intensSaveFileList[iFile]
  restore, file
  ;model=strmid(file,0,strlen(file)-11)
  model=intens.model
  print, 'Restore data for ''' + model + ''' from ''' + file + ''''
  if (intens.time[n_elements(intens.time)-1L] eq 0.0) then print, '### Warning: last time entry empty #########################'
  ; --- Get averaging interval ---
  mtime=long((udb_extract('mMeanAvg', [model], modDB))[0])
  ntime=long((udb_extract('nMeanAvg', [model], modDB))[0])
  mtime=0L > mtime < (n_elements(intens.time)-1L)
  if (ntime eq 0L) then ntime=999999L
  ntime=mtime > ntime < (n_elements(intens.time)-1L)
  ; --- Dimensions of data set ---
  nx1=n_elements(intens.I3r[*,0,0])
  nx2=n_elements(intens.I3r[0,*,0])
  nt =ntime-mtime+1L
  ;
  ; --- Determine average and contrast ---
  grav_mode=strlowcase((udb_extract('grav_mode', [model], basDB))[0])
  intenxavg=fltarr(nt)
  intrmsatotal=0.0
  if (grav_mode eq 'central') then begin
    ; --- Central potential ---
    Mask=shift(dist(nx1,nx1),nx1/2,nx2/2) lt (nx1/4 < nx2/4)
    nMask=total(Mask)
    for it=mtime,ntime do begin
      intenxavg[it-mtime]=total(double(Mask*intens.I3r[*,*,it]))
    endfor ; it
    intenxavg=intenxavg/nMask
    intenaavg=mean(intenxavg)
    ;
    for it=mtime,ntime do begin
      intrmsatotal=intrmsatotal + total(Mask*(intens.I3r[*,*,it]-intenaavg)^2)
    endfor ; it
    intrmsaa=sqrt(intrmsatotal/float(nMask*nt))/intenaavg
  endif else begin
    ; --- Constant gravity ---
    for it=mtime,ntime do begin
      intenxavg[it-mtime]=mean(double(intens.I3r[*,*,it]))
    endfor ; it
    intenaavg=mean(intenxavg)
    ;
    for it=mtime,ntime do begin
      intrmsatotal=intrmsatotal + total((intens.I3r[*,*,it]-intenaavg)^2)
    endfor ; it
    intrmsaa=sqrt(intrmsatotal/float(nx1*nx2*nt))/intenaavg
  endelse
  intrmsta=sqrt(mean( (intenxavg -intenaavg)^2) )/intenaavg
  ;
  ; --- Put results into database ---
  intDB[iFile].model      =model
  intDB[iFile].mtime      =mtime
  intDB[iFile].ntime      =ntime
  intDB[iFile].itime_first=intens.itime[mtime]
  intDB[iFile].itime_last =intens.itime[ntime]
  intDB[iFile].time_first =intens.time[mtime]
  intDB[iFile].time_last  =intens.time[ntime]
  intDB[iFile].intmean    =intenaavg
  intDB[iFile].intadrms   =intrmsaa
  intDB[iFile].inttdrms   =intrmsta
  ;
  ; --- FFT ---
  ; --- Masks ---
  x1tot=intens.mean.rad.xb1[nx1]-intens.mean.rad.xb1[0]
  xp1mask=sqrt(rebin(reform((findgen(nx1)-((2.0*findgen(nx1)-nx1) > 0.0))^2, nx1, 1   ), nx1, nx2) + $
               rebin(reform((findgen(nx2)-((2.0*findgen(nx2)-nx2) > 0.0))^2, 1   , nx2), nx1, nx2)  )
  xm1mask=fltarr(nx1, nx2)
  ind=where(xp1mask ne 0.0)
  xm1mask[ind]=1.0/xp1mask[ind]
  fftq=fltarr(nx1,nx2)
  for it=mtime,ntime do begin
    q=intens.I3r[*,*,it]-intenaavg
    fftq=fftq+abs(fft(q))^2
  endfor ; it
  totalfftq=total(fftq)
  if (totalfftq ne 0.0) then begin
    totalfftq=1.0/totalfftq
  endif
  fftq=fftq*totalfftq
  xp1qfft=total(xp1mask*fftq)
  if (xp1qfft ne 0.0) then begin
    xp1qfft=x1tot/xp1qfft
  endif
  intDB[iFile].xm1intfft=x1tot*total(xm1mask*fftq)
  intDB[iFile].xp1intfft=xp1qfft
  ;
  ; --- Delete structure (free the memory) ---
  intens=0
endfor ; iFile

; --- Save it ---
save, intDB, file='all' + OutID + 'intdb.sav'

end ; rhd_seq_extractpar
