; pro rhd_seq_q_time_plot
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seq_q_time_plot ('rhd_sequence_quantity_time_plot')
;
; PURPOSE:
;   Plot quantity over time from sequence data
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   @rhd_seq_q_time_plot
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, modellist_old
;
; PROCEDURE:
;
; EXAMPLE:
;
; TOOLS:
;   print,'[' + strjoin("'" + reverse(modellist) + "', ") + ']'
;
; MODIFICATION HISTORY:
;   2003-09-04 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   2004-10-12 (B.F.) xc3v -> xc3
;   2005-06-21 (B.F. LA) RAGE version
;   2006-05-26 (B.F.) mtime -> mtimelist, ntime -> ntimelist
;   2006-08-09 (B.F. Uppsala) rage_seqavgall_plot -> prag06_seqavgall_plot
;   2008-02-25 (B.F. Lyon) prag06_seqavgall_plot -> lyon08_seqavgall_plot
;   2008-05-13 (B.F.) lyon08_seqavgall_plot -> aabd_seqavgall_plot
;   2008-05-21 (B.F.) aabd1_seqavgall_plot -> seqavgstat_plot
;   2008-10-28 (B.F.) Restore from (and save) rebinned seq files
;   2009-01-22 (B.F.) Parameter file_prefix added
;   2011-02-08 (B.F. Naples) seqavgstat_plot.pro -> nap11_seq_q_time_plot.pro
;   2011-03-25 (B.F. Lyon) nap11_seq_q_time_plot.pro -> rhd_seq_q_time_plot.pro
;-
;---------------------------------------------------------------------------------------------------

; ================================
if (not keyword_set(external_control)) then begin &$
  ; --- Choose device: 'x', 'ps' ---
  dev='x' &$
  jpg_flag=0 &$
  png_flag=0 &$
  ;
  ; --- Read everything into (possibly large) pointer array. Otherwise: reread every time ---
  readall_flag  =1 &$
  recompute_flag=1 &$
  BinFactor=1L &$
  ;
  ; --- List of models ---
  ;mlid='Sun' & modellist=['d3gt57g44n58', 'd3gt57g44n94', 'd3t57g44b0n001', 'd3t57g44b0n002', 'd3t57g44b0n003', 'd3t57g44b0n004', 'd3t57g44b0n005', 'd3t57g44b0n006', 'd3t57g44b0n007', 'd3t57g44b0n011', 'd3t57g44b1n002', 'd3t57g44b1n003'] &$
  ;mlid='Sun' & modellist=['d3t57g44b1n002', 'd3t57g44b1n003'] &$
  ;mlid='Sun' & modellist=['d3t57g44b1n004', 'd3t57g44b1n012', 'd3t57g44b1n013'] &$
  mlid='Sun' & modellist=['d3t57g44b1n021', 'd3t57g44b1n022', 'd3t57g44b1n023', 'd3t57g44b1n024', 'd3t57g44b1n025', 'd3t57g44b1n026', 'd3t57g44b1n027', 'd3t57g44b1n028', 'd3t57g44b1n031'] &$
  ;mlid='Sun' & modellist=['d3t57g44b1n016', 'd3t57g44b1n017', 'd3t57g44b1n021', 'd3t57g44b1n022', 'd3t57g44b1n023', 'd3t57g44b1n024'] &$
  file_prefix='rhd_' &$
  ;
  commentlist=modellist &$
  ;
  ;colorliststr='colorlist=replicate1([!ctv.black],99)' &$
  ;colorliststr='colorlist=[!ctv.magenta, !ctv.red, !ctv.orange, !ctv.yellow, !ctv.green, !ctv.cyan, !ctv.gray]' &$
  colorliststr='colorlist=replicate1([!ctv.black, !ctv.blue, !ctv.magenta, !ctv.red, !ctv.orange, !ctv.yellow, !ctv.green, !ctv.cyan, !ctv.gray],99)' &$
  ;colorliststr='colorlist=replicate1([!ctv.black, !ctv.blue, !ctv.red, !ctv.orange, !ctv.yellow, !ctv.green, !ctv.cyan, !ctv.gray],99)' &$
  ;colorliststr='colorlist=replicate1([!ctv.black, !ctv.black, !ctv.blue, !ctv.blue, !ctv.red, !ctv.red, !ctv.green, !ctv.green],99)' &$
  ;colorliststr='colorlist=[!P.COLOR, !ctv.green, !ctv.orange, !ctv.red, !ctv.magenta, !ctv.magenta, !ctv.blue, !ctv.gray]' &$
  ;colorliststr='colorlist=[!P.COLOR, !ctv.green, !ctv.yellow, !ctv.orange, !ctv.red, !ctv.magenta, !ctv.blue, !ctv.gray]' &$
  ;colorliststr='colorlist=[!P.COLOR, !ctv.green, !ctv.red, !ctv.blue, !ctv.magenta, !ctv.gray, !ctv.orange, !P.COLOR]' &$
  ;colorliststr='colorlist=[!P.COLOR, !ctv.red, !ctv.green, !ctv.blue, !ctv.red, !ctv.green, !ctv.blue, !P.COLOR]' &$
  ;colorliststr='colorlist=[!ctv.black, !ctv.red, !ctv.green, !ctv.blue, !ctv.red, !ctv.green, !ctv.blue, !P.COLOR]' &$
  ;colorliststr='colorlist=[!P.BACKGROUND, !ctv.green, !ctv.red, !ctv.magenta, !ctv.blue, !ctv.gray, !ctv.orange, !P.BACKGROUND]' &$
  psymlist=replicate1([1],99) &$
  linestylelist=replicate1([0],99) &$
  ;linestylelist=[0,2,4,0,2,4,0,2] &$
  ;linestylelist=replicate1([0,0,0,0,0,0,0,0,0,5,5,5,5,5,5,5,5,5,2,2,2,2,2,2,2,2,2],99) &$
  ;linestylelist=replicate1([0,0,0,0,0,0,0,0,0,0,5,5,5,5,5,5,5,5,5,5,5,5,2,2,2,2,2,2,2,2],99) &$
  ;linestylelist=replicate1([2,0],99) &$
  ;linestylelist=replicate1([0,0,0,5,5,5,3,3,3,2,2,2],99) &$
  ;thicklist=[2,2,2,2,1,1,1,1] &$
  thicklist=replicate1([2],99) &$
  ;thicklist=replicate1([3,2],99) &$
  ;thicklist=replicate1([3,2,1],99) &$
  ;thicklist=[2,2,2,2,2,2,2,2] &$
  ;thicklist=[4,4,4,4,2,2,2,2] &$
  ;
  ;xsize=4.5 & ysize=4.0 & wfactor=1400.0 &$
  ;xsize=16.0 & ysize=9.0 & wfactor=500.0 &$
  ;xsize=30.0 & ysize=14.0 & wfactor=500.0 &$
  ;xsize=7.6 & ysize=4.5 & wfactor=900.0 &$
  ;xsize=7.7 & ysize=6.5 & wfactor=900.0 &$
  ;xsize=8.0 & ysize=7.0 & wfactor=900.0 &$
  ;xsize=8.8 & ysize=7.0 & wfactor=1100.0 &$
  ;xsize=9.0 & ysize=8.0 & wfactor=700.0 &$
  ;xsize=12.5 & ysize=8.0 & wfactor=800.0 &$
  ;xsize=12.5 & ysize=9.0 & wfactor=1200.0 &$
  ;xsize=13.4 & ysize=9.6 & wfactor=900.0 &$
  ;xsize=15.7 & ysize=5.5 & wfactor=600.0 &$
  xsize=18.0 & ysize=8.0 & wfactor=900.0 &$
  ;
  ;plot_id='talk_'+modellist[0]+'_' &$
  ;plot_id='proc_'+modellist[0]+'_' &$
  plot_id=modellist[0]+'_' &$
  ;
  ; --- Flux unit: from Teff or Frtop ---
  uniteflux='Frtop' &$
  ;
  ; --- x, xc3, log10_P, log10_tau / '', smin ---
  qcommand='log10_intrhoem' &$
  ;qcommand=q2bqcmd &$
  ;xcommand=q1bqcmd &$
  ;qcommand='' &$
  ;xcommand='' &$
  ;
  ; --- Mode of data range determination: 'auto', 'set' (use qrange0) ---
  qrangemode='auto' &$
  ;qrange0=[-1.0,1.0]*1.0e-5 &$
  ;qrange0=[-0.1,1]*2e-1 &$
  ;qrange0=[-0.5,1.5] &$
  ;qrange0=[7.0,2.8] &$
  ;qrange0=[-14.0,-2.5] &$
  qrange0=[3600.0,7000.0] &$
  qtitle0='auto' &$
  qtitle0='log(!4q!Xe!Dmagnetic,total!N)' &$
  ;
  ; --- Mode of x axis range determination: 'auto', 'set' (use xrange0) ---
  xrangemode='auto' &$
  ;xrange0=[6300.0,4900.0] &$
  ;xrange0=[800.0,2700.0] &$
  xrange0=[2800.0,900.0] &$
  ;xrange0=[1.5784,1.5800] &$
  ;xrange0=[0.0,13.0] &$
  ;xrange0=[-1.0,1.0] &$
  ;xtitle0='auto' &$
  xtitle0='time [s]' &$
  ;
  ; --- Number of cells to ignore at each boundary ---
  nr=0 &$
  ;
  ; --- Smoothing ---
  qsmooth=0 &$
  ;
  ; --- Legend ---
  ;xali=0.0 &$
  ;xali=0.5 &$
  xali=0.97 &$
  ;yali=-1.0 &$
  yali=0.01 &$
  ;yali=0.05 &$
  ;yali=0.97 &$
  ;
  ; --- Make line plots, overplot, keep device open ---
  plotline_flag=1 &$
  overplot_flag=0 &$
  keepopen_flag=0 &$
  ;
endif
; ================================


;if (modelgroupid eq 'ceph') then unitsystem='Gm,s,km/s'
;if (modelgroupid eq 'bd') then unitsystem='km,s'
;if (modelgroupid eq 'bd') then unitsystem='cgs'
;if (modelgroupid eq 'bd') then unitsystem='cgssE9'
unitsystem='Mm,s,km/s'
;unitsystem='km,s,cm/s'
;unitsystem='cgs'

; --- Load table with model data ---
rhdmodeldata, mt
modeltable=mt

nfile=n_elements(modellist)
mtimelist=replicate(     0L, nfile)
ntimelist=replicate(999999L, nfile)

; --- Find models in model table and extract averaging interval ---
;for i=0,nfile-1 do begin &$
;  im=(where(modeltable[*].model eq modellist[i]))[0] &$
;  if (im ne -1) then begin &$
;   mtimelist[i]=modeltable[im].mmeanavg &$
;    ntimelist[i]=modeltable[im].nmeanavg &$
;  endif &$
;  print, modellist[i], mtimelist[i], ntimelist[i] &$
;endfor

if (n_elements(commentlist) eq 0) then commentlist=strarr(n_elements(modellist))

; --- File names for plot output ---
;file_id='aa' + modelgroupid + '_seqavgstat_' + plot_id + q1aqcmd
;file_id='aa' + modelgroupid + '_seqavgstat_' + plot_id + qcommand
;file_id='pamol08_seqavgstat_' + plot_id + qcommand
;file_id='pamol08p_seqavgstat_' + plot_id + qcommand
file_id=file_prefix + plot_id + qcommand
;if (xcommand ne '') then file_id=file_id + '_' + xcommand
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
if (not(overplot_flag)) then begin & Pbak=!P & Xbak=!X & Ybak=!Y & endif

; --- Choose color table ---
normct

; --- Prepare plotting device ---
if (not(overplot_flag)) then begin &$
  if (strupcase(dev) eq 'X') then begin &$
    set_plot, 'X' &$
    ;!P.CHARSIZE=1.4 &$
    !P.CHARSIZE=1.6 &$
    ;!P.CHARSIZE=1.8 &$
    !P.FONT=-1 &$
    !P.SYMSIZE=3 &$
    win_id=0 &$
    window, win_id, xsize=xsize/10.0*wfactor, ysize=ysize/10.0*wfactor, retain=2 &$
  endif else begin & $
    set_plot, 'ps' & $
    !P.CHARSIZE=0.7 &$
    ;!P.CHARSIZE=1.0 &$
    ;!P.CHARSIZE=1.2 &$
    !P.FONT=0 &$
    !P.SYMSIZE=1 &$
    device, filename=ps_file, $
            encapsulated=(strupcase(dev) eq 'EPS'), $
            /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
    modctv, 'PS' &$
  endelse &$
endif

; --- List of colors ---
dummy=execute(colorliststr)

; --- Set global plot parameters ---
!P.THICK=2
;!X.MARGIN=[7,1] & !Y.MARGIN=[3,1]
!X.MARGIN=[9,1] & !Y.MARGIN=[4,1]
;!X.MARGIN=[11,1] & !Y.MARGIN=[4,1]
!P.COLOR=!ctv.black & !P.BACKGROUND=!ctv.white
;!X.STYLE=5     & !Y.STYLE=5

; --- Load data ---
read_flag=0
if (readall_flag) then begin &$
  if (n_elements(modellist_old) eq 0) then begin &$
    read_flag=1 &$
  endif else begin &$
    read_flag=not (array_equal(modellist_old, modellist)) &$
  endelse &$
  if (read_flag) then begin &$
    for i=0,n_elements(seqa)-1 do begin &$
      if (ptr_valid(seqa[i])) then ptr_free, seqa[i] &$
    endfor &$
    modellist_old=modellist &$
    seqa=ptrarr(nfile, /allocate_heap) &$
    print, 'Load all data' &$
    for i=0,nfile-1 do begin &$
      seq=0 &$
      sav_id='_' + strtrim(string(BinFactor, format='(I9)'), 2) &$
      file=modellist[i] + '_seq' + sav_id + '.sav' &$
      if (((BinFactor eq 0) or (BinFactor gt 1)) and file_test(file)) then begin &$
        print, 'Restore data from ''', file, '''' &$
        restore, file &$
      endif else begin &$
        file=modellist[i] + '_seq' + '.sav' &$
        print, 'Restore data from ''', file, '''' &$
        restore, file &$
        if (BinFactor gt 1) then begin &$
          seq=seqrebin(seq, BinFactor) &$
          file=modellist[i] + '_seq' + sav_id + '.sav' &$
        endif &$
      endelse &$
      *seqa[i]=temporary(seq) &$
    endfor &$
  endif else begin &$
    print, 'Reuse existing data' &$
  endelse &$
endif else begin &$
  for i=0,n_elements(seqa)-1 do begin &$
    if (ptr_valid(seqa[i])) then ptr_free, seqa[i] &$
  endfor &$
  modellist_old=modellist &$
endelse

; --- Compute average ---
.run seqavg
if (recompute_flag) then begin &$
  nxmax=16000L &$
  qa=fltarr(nxmax+1,nfile) &$
  xa=fltarr(nxmax+1,nfile) &$
  nqa=lonarr(nfile) &$
  nxa=lonarr(nfile) &$
  Teffa=fltarr(nfile) &$
  grava=fltarr(nfile) &$ 
  simina=lonarr(nfile) &$
  q1arr=fltarr(nfile) &$
  q2arr=fltarr(nfile) &$ 
endif
if (recompute_flag) then begin &$
  for i=0,nfile-1 do begin &$
    if (readall_flag) then begin &$
      seq=*seqa[i] &$
    endif else begin &$
      seq=0 &$
      sav_id='_' + strtrim(string(BinFactor, format='(I9)'), 2) &$
      file=modellist[i] + '_seq' + sav_id + '.sav' &$
      if (((BinFactor eq 0) or (BinFactor gt 1)) and file_test(file)) then begin &$
        print, 'Restore data from ''', file, '''' &$
        restore, file &$
      endif else begin &$
        file=modellist[i] + '_seq' + '.sav' &$
        print, 'Restore data from ''', file, '''' &$
        restore, file &$
        if (BinFactor gt 1) then begin &$
          seq=seqrebin(seq, BinFactor) &$
          file=modellist[i] + '_seq' + sav_id + '.sav' &$
        endif &$
      endelse &$
    endelse &$
    ;
    ;grava[i]=seq.par.grav &$
    ;Teffa[i]=sqrt(sqrt(fer(n_elements(fer)-1L)/!con.sigmasb)) &$
    ;
    q=seqavg(qcommand, seq, mtime=mtimelist[i], ntime=ntimelist[i], $
             unitsystem=unitsystem, uniteflux=uniteflux, $
             qavg=-1, qsmooth=qsmooth, qtitle=qtitle, /intertoxc) &$
    ; --- Take bottom value ---
    q=reform(q[0,*]) &$
    nqa[i]=n_elements(q) &$
    qa[0L:nqa[i]-1L,i]=q &$
    ;
    ;if (xcommand ne '') then begin &$
    ;  x=seqavg(xcommand, seq, mtime=mtimelist[i], ntime=ntimelist[i], unitsystem=unitsystem,$
    ;                          qtitle=xtitle) &$
      x=seq.time &$
      nxa[i]          =n_elements(x) &$
      xa[0L:nxa[i]-1L,i]=x &$
    ;endif &$
    ;
  endfor &$
endif

;restore, 'allpardb.sav'
;hdscheme=udb_extract('hdscheme', modellist, parDB)
;ModDimension=udb_extract('dimension', modellist, basDB)
;for i=0,nfile-1 do begin &$
;  commentlist[i]=commentlist[i] + ': T!Deff!N=' + strtrim(string(Teffa[i], format='(I6)'),2) + 'K' &$
;  ;commentlist[i]=commentlist[i] + ': T!Deff!N=' + strtrim(string(Teffa[i], format='(I6)'),2) + 'K ' + strtrim(string(ModDimension[i], format='(I1)'),2)+ 'D' &$
;  ;commentlist[i]=commentlist[i] + ', ' + hdscheme[i] &$
;  ;commentlist[i]='T!Deff!N=' + strtrim(string(Teffa[i], format='(I6)'),2) + 'K' &$
;endfor &$

; --- Find data range ---
if (qrangemode eq 'auto') then begin &$
  qrange=[1.0E+38,-1.0e38] &$
  if (plotline_flag) then begin &$
    for i=0,nfile-1 do begin &$
      qrange[0]=qrange[0] < min(qa[nr:nqa[i]-1-nr,i]) &$
      qrange[1]=qrange[1] > max(qa[nr:nqa[i]-1-nr,i]) &$
    endfor &$
  endif else begin &$
    qrange[0]=min(q2arr) &$
    qrange[1]=max(q2arr) &$
  endelse &$
endif else begin &$
  qrange=qrange0 &$
endelse
if (qtitle0 ne 'auto') then begin &$
  qtitle=transfont(qtitle0) &$
endif

; --- Find x axis range and title ---
if (xrangemode eq 'auto') then begin &$
  xrange=[1.0E+38,-1.0e38] &$
  if (plotline_flag) then begin &$
    for i=0,nfile-1 do begin &$
      xrange[0]=xrange[0] < min(xa[nr:nxa[i]-1-nr,i]) &$
      xrange[1]=xrange[1] > max(xa[nr:nxa[i]-1-nr,i]) &$
    endfor &$
  endif else begin &$
    xrange[0]=min(q1arr) &$
    xrange[1]=max(q1arr) &$
  endelse &$
  xrange[0]=xrange[0] -  5.0E-03*(xrange[1]-xrange[0]) &$
endif else begin &$
  xrange=xrange0 &$
endelse
if (xtitle0 ne 'auto') then begin &$
  xtitle=transfont(xtitle0) &$
endif


; === Plot data ===
!X.RANGE=xrange
!Y.RANGE=qrange
if (not keyword_set(noplot_flag)) then begin &$
 for i=0,nfile-1 do begin &$
  if (plotline_flag) then begin &$
    x=xa[nr:nxa[i]-1-nr,i] &$
    q=qa[nr:nqa[i]-1-nr,i] &$
    plotlines, x, q, $
      xtitle=xtitle, $
      ytitle=qtitle, $
      linecolor=[colorlist[i],colorlist[i]], linestyle=linestylelist[i], thick=thicklist[i], $
      over=((i ge 1) or (overplot_flag)), $
      dev=dev,  /psnoinit, psformat='nochange', /noclose &$
  endif &$
 endfor &$
endif

; === Diagonal ===
if (0) then begin &$
  plotlines, q1arr, q1arr, /over, $
    dev=dev,  /psnoinit, psformat='nochange', /noclose &$
endif

; === Legend ===
if (yali ge 0.0) then begin &$
  legend, commentlist, /nfromitems, $
          linestyle=linestylelist, $
          color=colorlist, $
          thick=thicklist, $
          xali=xali, yali=yali, charsize=!P.CHARSIZE*0.8, box=0, margin=0.1, spacing=0.7, pspacing=2 &$
;          xali=xali, yali=yali, charsize=!P.CHARSIZE*0.85, box=0, margin=0.1, spacing=0.5, pspacing=2 &$
;          xali=xali, yali=yali, charsize=!P.CHARSIZE*0.85, box=0, margin=0.1, spacing=0.5, pspacing=1 &$
endif


; === Write image files, close PS device ===
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else if (not keepopen_flag) then begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endif

; --- Reset global plot parameters ---
if (not keepopen_flag) then begin & !P=Pbak & !X=Xbak & !Y=Ybak & endif

; end ; rhd_seq_q_time_plot
