; pro rhd_seqpp_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_seqpp_collect ('radiation_hydrodynamics_sequence_plane-parallel_collect')
;
; PURPOSE:
;   Collect data from mean file.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_seqpp_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    3. 4.01 (B.F. Uppsala) rhd_plot -> rhd_collect
;   10.10.03 (B.F.) rhd_collect -> rhd_seq_collect
;   17. 3.04 (B.F.) Modification of rhd_seq_collect -> rhd_seqpp_collect
;   2007-09-27 (B.F. Lyon) v1 added to seq structure
;   2008-02-26 (B.F.) Initialization and handling of structure seq put into subroutine
;   2009-10-10 (B.F.) Read meta data into par structure; some names put into CamelCase
;   2016-11-01 (B.F.) Account for non-existing modeltime_db entry
;-
;------------------------------------------------------------------------------

;ModelIdent=''
;n_timestep_adjust=0
BinFactor=20L

;model='d3gt57g44n58' & ModelIdent='.??' & parModelIdent='.00'
;model='d3gt57g44n59' & ModelIdent='.???' & parModelIdent='.181'
;model='d3gt57g44b000n59' & ModelIdent='.??' & parModelIdent='.60'
;model='d3gt57g44b010n59' & ModelIdent='.??' & parModelIdent='.60'
;model='gt57g44n67_3D' & ModelIdent='??' & parModelIdent='kz' 
;model='d3gt57g44n94' & ModelIdent='.{4[6-9],[5-9]?}' & parModelIdent='.00'
;model='gt57g44n004_2D_RHD' & ModelIdent='_??' & parModelIdent='_10'
;model='gt57g44n004_2D_B00' & ModelIdent='_??' & parModelIdent='_10'
;model='gt57g44n004_2D_B10' & ModelIdent='_??' & parModelIdent='_18'
;model='gt57g44n004_2D_B20' & ModelIdent='_??' & parModelIdent='_25'
;model='gt57g44n004_2D_B40' & ModelIdent='_??' & parModelIdent='_41'
;model='gt57g44n004_2D_B80' & ModelIdent='_???' & parModelIdent='_110'

;model='d3t57g44b0n001' & ModelIdent='_???' & parModelIdent='_072'
;model='d3t57g44b0n002' & ModelIdent='_???' & parModelIdent='_090'
;model='d3t57g44b0n003' & ModelIdent='_???' & parModelIdent='_062'
;model='d3t57g44b0n004' & ModelIdent='_???' & parModelIdent='_045'
;model='d3t57g44b0n005' & ModelIdent='_???' & parModelIdent='_024'
;model='d3t57g44b0n006' & ModelIdent='_???' & parModelIdent='_090'
;model='d3t57g44b0n007' & ModelIdent='_???' & parModelIdent='_075'
;model='d3t57g44b0n008' & ModelIdent='_???' & parModelIdent='_034'
;model='d3t57g44b0n009' & ModelIdent='_???' & parModelIdent='_027'
;model='d3t57g44b0n010' & ModelIdent='_???' & parModelIdent='_006'
;model='d3t57g44b0n011' & ModelIdent='_???' & parModelIdent='_061'
;model='d3t57g44b0n012' & ModelIdent='_???' & parModelIdent='_012'
;model='d3t57g44b0n013' & ModelIdent='_???' & parModelIdent='_031'
;model='d3t57g44b0n014' & ModelIdent='_???' & parModelIdent='_054' & n_timestep_adjust=1
;model='d3t57g44b0n015' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n016' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n017' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n018' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n019' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b0n020' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n021' & ModelIdent='_???' & parModelIdent='_050'
;model='d3t57g44b0n022' & ModelIdent='_???' & parModelIdent='_050'
;model='d3t57g44b0n023' & ModelIdent='_???' & parModelIdent='_040'
;model='d3t57g44b0n024' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n025' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n026' & ModelIdent='_???' & parModelIdent='_060'
;model='d3t57g44b0n027' & ModelIdent='_???' & parModelIdent='_008'
;model='d3t57g44b0n029' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n030' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b0n031' & ModelIdent='_???' & parModelIdent='_050'
;model='d3t57g44b0n032' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b0n033' & ModelIdent='_???' & parModelIdent='_100'
;
;model='d3t57g44b1n002' & ModelIdent='_???' & parModelIdent='_090'
;model='d3t57g44b1n003' & ModelIdent='_???' & parModelIdent='_090'
;model='d3t57g44b1n004' & ModelIdent='_???' & parModelIdent='_090'
;model='d3t57g44b1n005' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n006' & ModelIdent='_???' & parModelIdent='_057'
;model='d3t57g44b1n007' & ModelIdent='_???' & parModelIdent='_061'
;model='d3t57g44b1n008' & ModelIdent='_???' & parModelIdent='_036'
;model='d3t57g44b1n009' & ModelIdent='_???' & parModelIdent='_015'
;model='d3t57g44b1n010' & ModelIdent='_???' & parModelIdent='_016'
;model='d3t57g44b1n011' & ModelIdent='_???' & parModelIdent='_121'
;model='d3t57g44b1n012' & ModelIdent='_???' & parModelIdent='_061'
;model='d3t57g44b1n013' & ModelIdent='_???' & parModelIdent='_075'
;model='d3t57g44b1n015' & ModelIdent='_???' & parModelIdent='_002'
;model='d3t57g44b1n016' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n017' & ModelIdent='_???' & parModelIdent='_018'
;model='d3t57g44b1n018' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n019' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n020' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n021' & ModelIdent='_???' & parModelIdent='_030'
;model='d3t57g44b1n022' & ModelIdent='_???' & parModelIdent='_040'
;model='d3t57g44b1n023' & ModelIdent='_???' & parModelIdent='_160'
;model='d3t57g44b1n024' & ModelIdent='_???' & parModelIdent='_110'
;model='d3t57g44b1n025' & ModelIdent='_???' & parModelIdent='_120'
;model='d3t57g44b1n026' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t57g44b1n027' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t57g44b1n028' & ModelIdent='_???' & parModelIdent='_121'
;model='d3t57g44b1n029' & ModelIdent='_???' & parModelIdent='_001'
;model='d3t57g44b1n030' & ModelIdent='_???' & parModelIdent='_001'
;model='d3t57g44b1n031' & ModelIdent='_???' & parModelIdent='_004'

;model='d3t25g50mm00w1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-330
;model='d3t25g50mm00w1hr' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-19
;model='d3t25g50mm00w1hrt' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t25g50mm00w1hrcb' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t28g50mm00w1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=0
;model='d3t28g50mm00w1hr' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-150
;model='d3t28g50mm00w1x2' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=0
;model='d3t31g50mm00w1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-10
;model='d3t33g30mm00w1' & ModelIdent='.???' & parModelIdent='.00'
;model='d3t33g30mm00w1hr' & ModelIdent='.???' & parModelIdent='.00' & n_timestep_adjust=-800
;model='d3t33g35mm00w1' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t33g35mm00w1lg' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t33g35mm00w1hrlg' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t33g40mm00w1' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t33g40mm00w1hr' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t33g45mm00w1' & ModelIdent='.???' & parModelIdent='.00' & n_timestep_adjust=-1175
;model='d3t33g45mm00w1hr' & ModelIdent='.???' & parModelIdent='.00' & n_timestep_adjust=-441
;model='d3t33g50mm00w1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=299
;model='d3t33g50mm00w1hr' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-10
;model='d3t33g50mm00w1hrlg' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=0
;model='d3t35g50mm00w1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-409
;model='d3t35g50mm00w1lg' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-496
;model='d3t35g50mm00w1hr' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-1676
;model='d3t38g49mm00w1' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t38g49mm00w1hr' & ModelIdent='.??' & parModelIdent='.00'

;model='d3t28g50mm00du4' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t28g50mm00du5' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t36g00mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t36g00mm00n02' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-2
;model='d3t36g00mm00n03' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t36g00mm00n04' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t36g10mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t36g10mm00n02' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-1
;model='d3t38g40mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t38g40mm00n2' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t40g15mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t40g15mm00n02' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-10
;model='d3t40g45mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t45g25mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t45g45mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t46g32mm00n01' & ModelIdent='.[89a]?' & parModelIdent='.00'
;model='d3t50g25mm00n01' & ModelIdent='.[m-z]?' & parModelIdent='.00'
;model='d3t50g35mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t50g40mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t50g45mm00n01' & ModelIdent='.[89]?' & parModelIdent='.00'
;model='d3t50g45mm00n02' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t50g45mm00n03' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t50g45mm00n04' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t55g35mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t55g40mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t55g45mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t59g35mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t59g40mm00n01' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-10
;model='d3t59g45mm00n01' & ModelIdent='.[4-7]?' & parModelIdent='.00'
;model='d3t63g40mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t63g40mm00n02' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t63g45mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t65g40mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t65g45mm00n01' & ModelIdent='.??' & parModelIdent='.00'
;model='d3t65g45mm00n02' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-15
;model='d3t68g43mm00n01' & ModelIdent='.[a-z]?' & parModelIdent='.00'
;model='d3t75g40mm00n1' & ModelIdent='.??' & parModelIdent='.00' & n_timestep_adjust=-5
;model='d3t116g80wd2' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t116g80wd4' & ModelIdent='_???' & parModelIdent='_399'
;model='d3t120g80wd2' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t120g80wd4' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t124g80wd2' & ModelIdent='_???' & parModelIdent='_150'
;model='d3t124g80wd4' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t128g80wd2' & ModelIdent='_???' & parModelIdent='_150'
;model='d3t128g80wd4' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-12
;
;model='d3t040g80wd08_RungeKutta2' & ModelIdent='' & parModelIdent=''
;model='d3t040g80wd08e' & ModelIdent='_?' & parModelIdent='_c'
;model='d3t040g80wd08f' & ModelIdent='' & parModelIdent=''
;model='d3t040g80wd08g' & ModelIdent='_?' & parModelIdent='_c'

;model='d3t060g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-623
;model='d3t070g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-677
;model='d3t080g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-636
;model='d3t090g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-387
;model='d3t095g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-373
;model='d3t100g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-305
;model='d3t105g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-338
;model='d3t110g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-614
;model='d3t115g80wd08' & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-481
;model='d3t120g80wd08bin8'  & ModelIdent='_???' & parModelIdent='_100'
;model='d3t120g80wd08bin11' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t125g80wd08bin8'  & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-501
;model='d3t125g80wd08bin11' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t130g80wd08bin8'  & ModelIdent='_???' & parModelIdent='_100' & n_timestep_adjust=-339
;model='d3t130g80wd08bin11' & ModelIdent='_???' & parModelIdent='_100'
;model='d3t140g80db02' & ModelIdent='_{025,050,075,100}' & parModelIdent='_100'
;model='d3t180g80db02' & ModelIdent='_{100,125}' & parModelIdent='_125'
;model='d3t220g80db02' & ModelIdent='_???' & parModelIdent='_100'
model='d3t340g80db02' & ModelIdent='_050' & parModelIdent='_050'
;
;model='ahht82g42n01' & ModelIdent='_01' & parModelIdent='01' & n_timestep=2933 ; 2126
;model='ahht90g42n01' & ModelIdent='_01' & parModelIdent='01' & n_timestep=3912
;model='ahht100g42n01' & ModelIdent='_01' & parModelIdent='01' & n_timestep=2862
;
;model='at70g43n01' & ModelIdent='_??' & parModelIdent='_13'
;model='at71g43n01' & ModelIdent='_??' & parModelIdent='_15'
;model='at73g43n01' & ModelIdent='_??' & parModelIdent='_10'
;model='at75g44n01' & ModelIdent='_??' & parModelIdent='_03'
;model='at77g44n01' & ModelIdent='_??' & parModelIdent='_01'
;model='at77g44n02' & ModelIdent='_??' & parModelIdent='_03'
;model='at77g44n03' & ModelIdent='_??' & parModelIdent='_03'
;model='at80g40n20' & ModelIdent='_??' & parModelIdent='_04'
;model='at80g40n21' & ModelIdent='_??' & parModelIdent='_36' 
;model='at80g40n22' & ModelIdent='_??' & parModelIdent='_24'
;model='at80g44n10_3D' & ModelIdent='??' & parModelIdent='bz' 
;model='at85g40n01' & ModelIdent='_??' & parModelIdent='_07'
;model='at85g40n02' & ModelIdent='_??' & parModelIdent='_11'
;model='at85g44n10' & ModelIdent='_??' & parModelIdent='_02' ; ? 
;model='at85g44n11' & ModelIdent='_??' & parModelIdent='_04' ; ?
;model='at85g44n13' & ModelIdent='_??' & parModelIdent='_24'
;model='at85g44n14' & ModelIdent='_??' & parModelIdent='_12'
;model='at85g44n15' & ModelIdent='_0[2-4]' & parModelIdent='_04' ; --- file 01 is corrupted ---
;model='at85g44n16' & ModelIdent='_{0[1-6],0[8-9],10,1[2-9],20}' & parModelIdent='_20' & n_timestep_adjust=-60
;
;model='bt300g40n01' & ModelIdent='_??' & parModelIdent='_08' 
;
;model='gt35gm04n02' & ModelIdent='_??' & parModelIdent='_02' 
;model='gt35gm04n03' & ModelIdent='_??' & parModelIdent='_01' 
;model='gt35gm04n04' & ModelIdent='_??' & parModelIdent='_02' 
;model='gt35gm04n05' & ModelIdent='_??' & parModelIdent='_04'
;model='gt43g15n01' & ModelIdent='_??'
;model='gt50g10n01' & ModelIdent='_??' & parModelIdent='_01'

;model='gt50g20n01' & ModelIdent='_??' & parModelIdent='_40' & n_timestep_adjust=-2
;model='gt51g20n01' & ModelIdent='_??' & parModelIdent='_44'
;model='gt51g20n02' & ModelIdent='_??' & parModelIdent='_50'
;model='gt52g20n01' & ModelIdent='_??' & parModelIdent='_18'
;model='gt52g20n02' & ModelIdent='_??' & parModelIdent='_03'
;model='gt52g20n03' & ModelIdent='_??' & parModelIdent='_15'
;model='gt53g20n01' & ModelIdent='_??' & parModelIdent='_13'
;model='gt53g20n02' & ModelIdent='_??' & parModelIdent='_40'
;model='gt54g20n01' & ModelIdent='_??' & parModelIdent='_06'
;model='gt54g20n02' & ModelIdent='_??' & parModelIdent='_10'
;model='gt54g20n03' & ModelIdent='_??' & parModelIdent='_43'
;model='gt54g20n04' & ModelIdent='_??' & parModelIdent='_70'
;model='gt54g20n05' & ModelIdent='_??' & parModelIdent='_80'
;model='gt55g20n01' & ModelIdent='_??' & parModelIdent='_70'
;model='gt56g20n01' & ModelIdent='_??' & parModelIdent='_14'
;model='gt56g20n02' & ModelIdent='_??' & parModelIdent='_50'
;model='gt56g20n03' & ModelIdent='_??' & parModelIdent='_50'
;model='gt56g20n04' & ModelIdent='_??' & parModelIdent='_99'
;model='gt58g20n01' & ModelIdent='_??' & parModelIdent='_02'
;model='gt58g20n02' & ModelIdent='_??' & parModelIdent='_07'
;model='gt58g20n03' & ModelIdent='_??' & parModelIdent='_50'
;model='gt58g20n04' & ModelIdent='_??' & parModelIdent='_50'
;model='gt58g20n05' & ModelIdent='_??' & parModelIdent='_60'
;model='gt60g20n01' & ModelIdent='_??' & parModelIdent='_03'
;model='gt60g20n02' & ModelIdent='_??' & parModelIdent='_08'
;model='gt60g20n03' & ModelIdent='_??' & parModelIdent='_99'
;model='gt60g20n04' & ModelIdent='_??' & parModelIdent='_60'
;model='gt62g20n01' & ModelIdent='_??' & parModelIdent='_05'
;model='gt62g20n02' & ModelIdent='_??' & parModelIdent='_09'

;model='d3t25g50mm00w1' & ModelIdent='.??' & parModelIdent='.54'

;model='mt07g50mm00n01' & ModelIdent='_??' & parModelIdent='_13'
;model='mt07g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt07g50mm00n03' & ModelIdent='_??' & parModelIdent='_40'

;model='mt08g50mm00n01' & ModelIdent='_??' & parModelIdent='_16'
;model='mt08g50mm00n02' & ModelIdent='_??' & parModelIdent='_30'
;model='mt08g50mm00n03' & ModelIdent='_??' & parModelIdent='_25'

;model='mt09g35mm00n01' & ModelIdent='_??' & parModelIdent='_13'
;model='mt09g50mm00n01' & ModelIdent='_??' & parModelIdent='_21'
;model='mt09g50mm00n02' & ModelIdent='_??' & parModelIdent='_40'
;model='mt09g50mm00n03' & ModelIdent='_??' & parModelIdent='_40'
;model='mt09g50mm00n04' & ModelIdent='_??' & parModelIdent='_40'
;model='mt09g50mm00n05' & ModelIdent='_??' & parModelIdent='_40'
;model='mt09g50mm00n06' & ModelIdent='_??' & parModelIdent='_40'
;model='mt09g50mm00n07a' & ModelIdent='_??' & parModelIdent='_01'
;model='mt09g50mm00n07b' & ModelIdent='_??' & parModelIdent='_01'
;model='mt09g50mm00n08' & ModelIdent='_??' & parModelIdent='_20'
;model='mt09g50mm00n09' & ModelIdent='_??' & parModelIdent='_10'
;model='mt09g50mm00n10' & ModelIdent='_??' & parModelIdent='_20'
;model='mt09g50mm00n11' & ModelIdent='_??' & parModelIdent='_20'
;model='mt09g50mm00n12' & ModelIdent='_??' & parModelIdent='_10'
;model='mt09g50mm00n13' & ModelIdent='_??' & parModelIdent='_12'
;model='mt09g50mm00n14' & ModelIdent='_??' & parModelIdent='_30'

;model='mt10g50mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n03' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n04' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n05' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n06' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n07' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n08' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n09' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n10' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n11' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n12' & ModelIdent='_??' & parModelIdent='_02'
;model='mt10g50mm00n13' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n14' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n15' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n16' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n17' & ModelIdent='_??' & parModelIdent='_20'
;model='mt10g50mm00n18' & ModelIdent='_??' & parModelIdent='_20'

;model='mt11g40mm00n01' & ModelIdent='_??' & parModelIdent='_09'
;model='mt11g40mm00n02' & ModelIdent='_??' & parModelIdent='_25'
;model='mt11g50mm00n01' & ModelIdent='_??' & parModelIdent='_13'
;model='mt11g50mm00n02' & ModelIdent='_??' & parModelIdent='_11'
;model='mt11g50mm00n03' & ModelIdent='_??' & parModelIdent='_35'
;model='mt11g50mm00n04' & ModelIdent='_??' & parModelIdent='_12'
;model='mt11g50mm00n05' & ModelIdent='_??' & parModelIdent='_20'
;model='mt11g50mm00n06' & ModelIdent='_??' & parModelIdent='_20'

;model='mt12g50mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt12g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt12g50mm00n03' & ModelIdent='_??' & parModelIdent='_21'
;model='mt12g50mm00n04' & ModelIdent='_??' & parModelIdent='_17'
;model='mt12g50mm00n05' & ModelIdent='_??' & parModelIdent='_17'
;model='mt12g50mm00n06' & ModelIdent='_??' & parModelIdent='_17'
;model='mt12g50mm00n07' & ModelIdent='_??' & parModelIdent='_15'
;model='mt12g50mm00n08' & ModelIdent='_??' & parModelIdent='_12'
;model='mt12g50mm00n09' & ModelIdent='_??' & parModelIdent='_99'
;model='mt12g50mm00n10' & ModelIdent='_??' & parModelIdent='_20'
;model='mt12g50mm00n11' & ModelIdent='_??' & parModelIdent='_20'

;model='mt13g35mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g40mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g40mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g45mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n03' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n04' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n05' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n06' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g50mm00n07' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n08' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n09' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n10' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n11' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n12' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n13' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n14' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n15' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n16' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n17' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g50mm00n18' & ModelIdent='_??' & parModelIdent='_20'
;model='mt13g55mm00n01' & ModelIdent='_??' & parModelIdent='_10'
;model='mt13g55mm00n02' & ModelIdent='_??' & parModelIdent='_20'

;model='mt14g50mm00n01' & ModelIdent='_??' & parModelIdent='_15'
;model='mt14g50mm00n02' & ModelIdent='_??' & parModelIdent='_17'
;model='mt14g50mm00n03' & ModelIdent='_???' & parModelIdent='_100'
;model='d2t14g50mm00n01' & ModelIdent='_??' & parModelIdent='_06'

;model='mt15g40mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt15g40mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt15g50mm00n01' & ModelIdent='_??' & parModelIdent='_15'
;model='mt15g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt15g50mm00n03' & ModelIdent='_??' & parModelIdent='_20'
;model='mt15g50mm00n04' & ModelIdent='_??' & parModelIdent='_15'
;model='mt15g50mm00n05' & ModelIdent='_??' & parModelIdent='_20'
;model='mt15g50mm00n06' & ModelIdent='_???' & parModelIdent='_150'
;model='mt15g50mm00n07' & ModelIdent='_??' & parModelIdent='_15'
;model='mt15g50mm00n08' & ModelIdent='_??' & parModelIdent='_45'
;model='mt15g50mm00n09' & ModelIdent='_??' & parModelIdent='_20'
;model='mt15g50mm00n10' & ModelIdent='_???' & parModelIdent='_034'
;model='mt15g50mm00n11' & ModelIdent='_??' & parModelIdent='_10'
;model='mt15g50mm00n12' & ModelIdent='_??' & parModelIdent='_10'
;model='mt15g50mm00n13' & ModelIdent='_???' & parModelIdent='_300'
;model='mt15g50mm00n14' & ModelIdent='_???' & parModelIdent='_100'
;model='mt15g50mm00n15' & ModelIdent='_???' & parModelIdent='_100'
;model='mt15g50mm00n16' & ModelIdent='_???' & parModelIdent='_010'
;model='mt15g50mm00n17' & ModelIdent='_???' & parModelIdent='_100'

;model='mt16g40mm00n01' & ModelIdent='_??' & parModelIdent='_02'
;model='mt16g50mm00n01a' & ModelIdent='_??' & parModelIdent='_01'
;model='mt16g50mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt16g50mm00n03' & ModelIdent='_??' & parModelIdent='_15'
;model='mt16g50mm00n04' & ModelIdent='_??' & parModelIdent='_08'
;model='mt16g50mm00n05' & ModelIdent='_??' & parModelIdent='_12'
;model='mt16g50mm00n06' & ModelIdent='_??' & parModelIdent='_20'
;model='mt16g50mm00n07' & ModelIdent='_??' & parModelIdent='_12'
;model='mt16g50mm00n08' & ModelIdent='_???' & parModelIdent='_100'

;model='mt17g50mm00n01' & ModelIdent='_??' & parModelIdent='_14'
;model='mt17g50mm00n02' & ModelIdent='_??' & parModelIdent='_20'
;model='mt17g50mm00n03' & ModelIdent='_???' & parModelIdent='_110'

;model='mt18g35mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt18g40mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt18g40mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt18g45mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='d2t18g50mm00du3' & ModelIdent='.0?' & parModelIdent='.01'
;model='d2t18g50mm00du3' & ModelIdent='_03' & parModelIdent='_03'
;model='d2t18g50mm00du4' & ModelIdent='_03' & parModelIdent='_03'
;model='d2t18g50mm00du5' & ModelIdent='_03' & parModelIdent='_03'
;model='d2t18g50mm00due' & ModelIdent='_03' & parModelIdent='_03'
;model='mt18g50mm00n02' & ModelIdent='_??' & parModelIdent='_05'
;model='mt18g50mm00n03' & ModelIdent='_??' & parModelIdent='_08'
;model='mt18g50mm00n04' & ModelIdent='_??' & parModelIdent='_08'
;model='mt18g50mm00n06' & ModelIdent='_??' & parModelIdent='_15'
;model='mt18g50mm00n07' & ModelIdent='_??' & parModelIdent='_20'
;model='mt18g50mm00n08' & ModelIdent='_??' & parModelIdent='_20'
;model='mt18g50mm00n09a' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g50mm00n09b' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g50mm00n10' & ModelIdent='_[2-4]?' & parModelIdent='_40'
;model='mt18g50mm00n10' & ModelIdent='_??' & parModelIdent='_40'
;model='mt18g50mm00n11' & ModelIdent='_???' & parModelIdent='_200'
;model='mt18g50mm00n12' & ModelIdent='_??' & parModelIdent='_10'
;model='mt18g50mm00n13' & ModelIdent='_???' & parModelIdent='_100'
;model='mt18g50mm00n14' & ModelIdent='_???' & parModelIdent='_120'
;model='mt18g55mm00n02d' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n02e' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n03' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n04' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n05' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n06' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n07' & ModelIdent='_??' & parModelIdent='_04'
;model='mt18g55mm00n08' & ModelIdent='_??' & parModelIdent='_07'
;model='mt18g55mm00n09' & ModelIdent='_??' & parModelIdent='_09'
;model='mt18g55mm00n10' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n11' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n13c' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n14' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n15' & ModelIdent='_??' & parModelIdent='_05'
;model='mt18g55mm00n16' & ModelIdent='_??' & parModelIdent='_03'
;model='mt18g55mm00n17' & ModelIdent='_??' & parModelIdent='_03'
;model='mt18g55mm00n18' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n19' & ModelIdent='_??' & parModelIdent='_03'
;model='mt18g55mm00n20' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n21' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n22a' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n22b' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n23' & ModelIdent='_??' & parModelIdent='_15'
;model='mt18g55mm00n24' & ModelIdent='_??' & parModelIdent='_10'
;model='mt18g55mm00n25' & ModelIdent='_??' & parModelIdent='_02'
;model='mt18g55mm00n26' & ModelIdent='_??' & parModelIdent='_15'
;model='mt18g55mm00n27' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n28' & ModelIdent='_??' & parModelIdent='_06'
;model='mt18g55mm00n29' & ModelIdent='_??' & parModelIdent='_09'
;model='mt18g55mm00n29a' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29b' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29c' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29d' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29e' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29f' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n29g' & ModelIdent='_??' & parModelIdent='_01'
;model='mt18g55mm00n30' & ModelIdent='_??' & parModelIdent='_07'
;model='mt18g55mm00n31' & ModelIdent='_??' & parModelIdent='_03'
;model='mt18g55mm00n32' & ModelIdent='_??' & parModelIdent='_39'
;model='mt18g55mm00n33' & ModelIdent='_??' & parModelIdent='_11'
;model='mt18g55mm00n33b' & ModelIdent='_??' & parModelIdent='_05'
;model='mt18g55mm00n33c' & ModelIdent='_??' & parModelIdent='_05'

;model='mt19g50mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt19g50mm00n02' & ModelIdent='_[12]?' & parModelIdent='_23'
;model='mt19g50mm00n03' & ModelIdent='_??' & parModelIdent='_20'

;model='mt20g50mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt20g50mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt20g50mm00n03' & ModelIdent='_??' & parModelIdent='_09'
;model='mt20g50mm00n04' & ModelIdent='_??' & parModelIdent='_04'
;model='mt20g50mm00n05' & ModelIdent='_{0[3-9],1[0-9],20}' & parModelIdent='_20'
;model='mt20g50mm00n06' & ModelIdent='_??' & parModelIdent='_01'
;model='mt20g50mm00n07' & ModelIdent='_??' & parModelIdent='_01'
;model='mt20g50mm00n08' & ModelIdent='_??' & parModelIdent='_10'
;model='mt20g50mm00n09' & ModelIdent='_???' & parModelIdent='_200'
;model='mt20g50mm00n10' & ModelIdent='_??' & parModelIdent='_10'
;model='mt20g50mm00n11' & ModelIdent='_???' & parModelIdent='_100'
;model='mt20g50mm00n12' & ModelIdent='_???' & parModelIdent='_100'

;model='mt22g40mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt22g50mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt22g50mm00n02' & ModelIdent='_??' & parModelIdent='_08'
;model='mt22g50mm00n03' & ModelIdent='_??' & parModelIdent='_02'
;model='mt22g50mm00n04' & ModelIdent='_??' & parModelIdent='_09'
;model='mt22g50mm00n05' & ModelIdent='_??' & parModelIdent='_15'
;model='mt22g50mm00n06' & ModelIdent='_??' & parModelIdent='_20'
;model='mt22g50mm00n07' & ModelIdent='_??' & parModelIdent='_40'
;model='mt22g50mm00n08' & ModelIdent='_??' & parModelIdent='_44'
;model='mt22g50mm00n09' & ModelIdent='_???' & parModelIdent='_300'
;model='mt22g50mm00n10' & ModelIdent='_???' & parModelIdent='_200'
;model='mt22g50mm00n11' & ModelIdent='_???' & parModelIdent='_200'
;model='mt22g50mm00n12' & ModelIdent='_??' & parModelIdent='_10'
;model='mt22g50mm00n13' & ModelIdent='_???' & parModelIdent='_100'
;model='mt22g50mm00n14' & ModelIdent='_???' & parModelIdent='_100'

;model='mt24g40mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt24g50mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt24g50mm00n02' & ModelIdent='_???' & parModelIdent='_100'
;model='mt24g50mm00n03' & ModelIdent='_???' & parModelIdent='_100'

;model='mt26g35mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt26g40mm00n01' & ModelIdent='_??' & parModelIdent='_13'
;model='mt26g45mm00n01' & ModelIdent='_??' & parModelIdent='_09'
;model='mt26g45mm00n02' & ModelIdent='_??' & parModelIdent='_06'
;model='mt26g50mm00n01' & ModelIdent='_??' & parModelIdent='_15'
;model='mt26g50mm00n02' & ModelIdent='_??' & parModelIdent='_15'
;model='mt26g50mm00n03' & ModelIdent='_???' & parModelIdent='_200'
;model='mt26g50mm00n04' & ModelIdent='_??' & parModelIdent='_10'
;model='mt26g50mm00n05' & ModelIdent='_???' & parModelIdent='_140'
;model='mt26g50mm00n06a' & ModelIdent='_???' & parModelIdent='_001'
;model='mt26g50mm00n07' & ModelIdent='_???' & parModelIdent='_100'

;model='mt28g40mm00n01' & ModelIdent='_??' & parModelIdent='_11'
;model='mt28g50mm00n01' & ModelIdent='_??' & parModelIdent='_15'
;model='mt28g50mm00n02' & ModelIdent='_??' & parModelIdent='_10'
;model='mt28g50mm00n03' & ModelIdent='_???' & parModelIdent='_200'
;model='mt28g50mm00n04' & ModelIdent='_???' & parModelIdent='_100'
;model='mt28g50mm00n05' & ModelIdent='_???' & parModelIdent='_100'

;model='mt30g40mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt30g50mm00n01' & ModelIdent='_??' & parModelIdent='_20'
;model='mt30g50mm00n02' & ModelIdent='_??' & parModelIdent='_06'
;model='mt30g50mm00n03' & ModelIdent='_??' & parModelIdent='_05'
;model='mt30g50mm00n04' & ModelIdent='_??' & parModelIdent='_05'
;model='mt30g50mm00n05' & ModelIdent='_??' & parModelIdent='_05'
;model='mt30g50mm00n06' & ModelIdent='_??' & parModelIdent='_05'
;model='mt30g50mm00n07' & ModelIdent='_??' & parModelIdent='_07'
;model='mt30g50mm00n08' & ModelIdent='_??' & parModelIdent='_05'
;model='mt30g50mm00n09' & ModelIdent='_??' & parModelIdent='_07'
;model='mt30g50mm00n10' & ModelIdent='_???' & parModelIdent='_150'
;model='mt30g50mm00n11' & ModelIdent='_???' & parModelIdent='_100'
;model='mt30g50mm00n12' & ModelIdent='_???' & parModelIdent='_100'
;
;model='mt32g40mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt32g40mm00n02' & ModelIdent='_??' & parModelIdent='_06'
;model='mt32g40mm00n03' & ModelIdent='_??' & parModelIdent='_05'
;model='d3t32g45' & ModelIdent='_??[2-5]' & parModelIdent='_005'
;model='mt32g50mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt32g50mm00n02' & ModelIdent='_???' & parModelIdent='_120'
;model='mt32g50mm00n03' & ModelIdent='_???' & parModelIdent='_100'
;
;model='mt33g30mm00n01' & ModelIdent='_??' & parModelIdent='_01'
;model='mt33g30mm00n02' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g35mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g40mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g40mm00n02' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g40mm00n03' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g40mm00n04' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g45mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g45mm00n02' & ModelIdent='_???' & parModelIdent='_140'
;model='mt33g45mm00n03' & ModelIdent='_???' & parModelIdent='_180'
;model='mt33g45mm00n04' & ModelIdent='_???' & parModelIdent='_100'
;model='mt33g50mm00n01' & ModelIdent='_??' & parModelIdent='_05'
;model='mt33g50mm00n02' & ModelIdent='_???' & parModelIdent='_080'
;model='mt33g50mm00n03' & ModelIdent='_???' & parModelIdent='_050'
;model='mt33g50mm00n04' & ModelIdent='_???' & parModelIdent='_170'
;model='mt33g50mm00n05' & ModelIdent='_???' & parModelIdent='_100'
;model='mt33g50mm00n06' & ModelIdent='_???' & parModelIdent='_150'
;model='mt33g50mm00n07' & ModelIdent='_???' & parModelIdent='_200'
;model='mt33g50mm00n08' & ModelIdent='_???' & parModelIdent='_200'
;model='mt33g50mm00n09' & ModelIdent='_???' & parModelIdent='_100'
;model='mt33g50mm00n10' & ModelIdent='_???' & parModelIdent='_100'
;model='mt33g50mm00n11' & ModelIdent='_???' & parModelIdent='_120'
;model='mt33g50mm00n12' & ModelIdent='_???' & parModelIdent='_120'
;model='mt33g50mm00n13' & ModelIdent='_???' & parModelIdent='_120'
;model='mt33g50mm00n14' & ModelIdent='_???' & parModelIdent='_150'
;
;model='mt35g40mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt35g40mm00n02' & ModelIdent='_??' & parModelIdent='_06'
;model='mt35g40mm00n03' & ModelIdent='_??' & parModelIdent='_05'
;model='mt35g40mm00n04' & ModelIdent='_??' & parModelIdent='_05'
;model='mt35g50mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt35g50mm00n02' & ModelIdent='_???' & parModelIdent='_120'
;model='mt35g50mm00n03' & ModelIdent='_???' & parModelIdent='_100'
;
;model='mt40g50mm00n01' & ModelIdent='_??' & parModelIdent='_06'
;model='mt40g50mm00n02' & ModelIdent='_??' & parModelIdent='_05'
;model='mt40g50mm00n03' & ModelIdent='_??' & parModelIdent='_05'
;model='mt40g50mm00n04' & ModelIdent='_???' & parModelIdent='_100'
;model='mt40g50mm00n05' & ModelIdent='_???' & parModelIdent='_069'
;
;model='rhd' & ModelIdent='' & parModelIdent=''

; --- Read meta data (name and unit of tags) ---
ReadMetaData_flag=1
; --- Read intensity data into separate structure ---
ReadIntens_flag=0

ModelDisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;ModelDisk=getenv('HOME') + '/dat/job/jc/'
;ModelDisk='/export/gusers/bf/dat/job/j2/bak/'
;ModelDisk=getenv('HOME') + '/dat/rhd/tmp/'
;ModelDisk=getenv('HOME') + '/dat/rhd/hartmut_holweger/'

ModelFile=ModelDisk + model + ModelIdent + '.mean'
;ModelFile=ModelDisk + '*' + ModelIdent + '.mean'

; --- Read parameter file ---
par=uio_struct_rd(ModelDisk + model + parModelIdent + '.par', ReadMetaData=ReadMetaData_flag, outstr=outstr, ierr=ierr)
if (ierr gt 0) then begin &$
  message, 'Error reading parameter file: ' + outstr &$
endif

delvar, ListData
ful=uio_datasetlist_rd(ModelFile, ListData=ListData, ReadMetaData=ReadMetaData_flag, date=date, ierr=ierr)
uio_closrd, ListData.channel

;nr=n_elements(ful.r.xc3_volume)
;nr=n_elements(ful.rad.xc3)
nxc1=n_elements(ful.rad.xc1)
nxc2=n_elements(ful.rad.xc2)
nxc3=n_elements(ful.rad.xc3)
time_first=ful.modeltime

uio_openrd, nc, ListData.file[ListData.total_number-1]
;outstr='0found'
outstr='found'
i_timestep=0L
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    i_timestep=i_timestep+1L &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    help, time_last &$
  endif &$
endwhile
uio_closrd, nc

if (par.dtime_out_mean gt 0.0) then begin &$
  ;n_timestep_first=ceil(time_first/par.dtime_out_mean) &$
  ;n_timestep_last =floor(time_last/par.dtime_out_mean) &$
  ;n_timestep=long((time_last-time_first)/par.dtime_out_mean+0.5)+1 &$
  n_timestep_first=round(time_first/par.dtime_out_mean) &$
  n_timestep_last =round(time_last/par.dtime_out_mean) &$
  n_timestep=n_timestep_last-n_timestep_first+1 &$
  n_timestep=n_timestep+n_timestep_adjust &$
  help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep &$
endif else begin &$
  n_timestep=i_timestep-1L &$
  help, par.dtime_out_mean, n_timestep &$
endelse
;n_timestep=23*24+12
delvar, ListData

;ind=where(strmid(tag_names(ful.z), 0, 3) eq 'QUC', nquc)
;ind=where((strmid(tag_names(ful.z), 0, 3) eq 'QUC') and (strlen(tag_names(ful.z)) eq 6), nquc)
ind=where((strmid(tag_names(ful.z3), 0, 3) eq 'QUC') and (strlen(tag_names(ful.z3)) eq 11), nquc)
print, 'rhd_seqpp_collect: nquc=', nquc

delvar,seq
rhd_seq_put2struct, ful, seq, /InitStruct, par=par, $
  ModelName=model, ModelIdent=ModelIdent, n_timestep=n_timestep, $
  ReadMetaData=ReadMetaData_flag

if (ReadIntens_flag) then begin &$
  intens=create_struct( $
    'model'     , model, $
    'ModelIdent', ModelIdent, $
    'mean'      , ful, $
    'itime'     , lonarr(            n_timestep), $
    'time'      , fltarr(            n_timestep), $
    'time_db'   , dblarr(            n_timestep), $
    'ifile'     , lonarr(            n_timestep), $
    'file'      , strarr(            n_timestep), $
    'I3r'       , fltarr(nxc1, nxc2, n_timestep) $
                       ) &$
endif

ierr=0L
i=0L
itime_last=-10000L
time_last=-1.0D+09
file=''
systime0=systime(1)
version=''

while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful0=uio_datasetlist_rd(ModelFile, ListData=ListData, date=date, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=ListData.file[ListData.open_number] &$
    file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
    ful=temporary(ful0) &$
    if (total(strcmp(tag_names(ful.head), 'VERSION')) ge 1.0) then begin &$
      version=ful.head.version &$
    endif &$
    ;
    if (total(strcmp(tag_names(ful), 'MODELTIME_DB')) ge 1.0) then begin &$
      modeltime=ful.modeltime_db &$
    endif else begin &$
      modeltime=ful.modeltime &$
    endelse &$
    ;
    ;if (1) then begin &$
    ;if ((ful.modelitime gt (itime_last+3)) and (modeltime gt (time_last+par.dtime_out_mean*0.2))) then begin &$
    if ((ful.modelitime gt (itime_last+1)) and (modeltime gt (time_last+par.dtime_out_mean*0.2))) then begin &$
    ;if ((ful.modelitime gt (itime_last+1)) and $
    ;    ((file_tmp ne 'gt50g20n01_24.mean') or (ful.modeltime le 4.26230e+07)) and $
    ;    ((file_tmp ne 'gt54g20n02_07.mean') or (ful.modeltime ge 6345000.0  ))) then begin &$
      print, '--- ', i, '/', n_timestep, ful.modelitime, modeltime, date, ' ---' , format='(A,I5,A,I5,1X,I8,E15.8,2X,A,A)' &$
      ;
      if (file ne oldfile) then par=uio_struct_rd(strmid(file,0,strlen(file)-4) + 'par') &$
      ;
      rhd_seq_put2struct, ful, seq, par=par, $
        snapshot=ListData.open_number, file=file, i_timestep=i, date=date, version=version &$
;print,minmax(ful.z3.ei),minmax(seq.ei) &$
      ;
      if (ReadIntens_flag) then begin &$
        intens.itime[i]  =ful.modelitime &$
        intens.time[i]   =ful.modeltime &$
        intens.time_db[i]=modeltime &$
        intens.ifile[i]  =ListData.open_number &$
        intens.file[i]   =file_tmp &$
        intens.I3r[*,*,i]=reform(ful.rad.Intb3_r) &$
      endif &$
      ;
;      if (i gt 0) then begin &$
;        if ((seq.time[i]-seq.time[i-1]) gt (1.5*par.dtime_out_mean)) then begin &$
;          print, '  Time step problem at ', seq.time[i] &$
;          ;
;          seq.time[i]=0.5*(ful.modeltime+seq.time[i-1]) &$
;          if (ReadIntens_flag) then intens.time[i]=0.5*(ful.modeltime+intens.time[i-1]) &$
;          ;
;          i=i+1 &$
;          ;
;          rhd_seq_put2struct, ful, seq, par=par, $
;            snapshot=ListData.open_number, file=file, i_timestep=i &$
;          ;
;          if (ReadIntens_flag) then begin &$
;            intens.itime[i]  =ful.modelitime &$
;            intens.time[i]   =modeltime &$
;            intens.file[i]   =file_tmp &$
;            intens.I3r[*,*,i]=reform(ful.rad.Intb3_r) &$
;          endif &$
;        endif &$
;      endif &$
      ;
      i=i+1 &$
      if (ierr eq 0) then begin &$
        itime_last=ful.modelitime &$
        time_last =modeltime &$
      endif &$
    endif &$
  endif &$
endwhile

ind=where(seq.file eq '', nCount)
if (nCount gt 0) then print, 'WARNING: seq structure not filled completely! ####################### ' + string(nCount)

print, 'Total time: ', systime(1)-systime0, ' sec'

if (ReadIntens_flag) then begin &$
  print, 'Save dataset (intens) into ''' + model + '_intens.sav' + '''' &$
  save, filename=model + '_intens.sav', intens &$
endif

savefile=model + '_seq.sav'
print, 'Save dataset (seq) into ''' + savefile + ''''
save, filename=savefile, seq

.run rhdmodeldata
rhdmodeldata, modeltable
mtime=(udb_Extract('mMeanAvg', [model], modeltable))[0]
ntime=(udb_Extract('nMeanAvg', [model], modeltable))[0]
if (ntime gt 0) then begin &$
  savefile=model + '_seq.sav' &$
  restore, savefile &$
  seq=seqrebin(seq, 0, mtime=mtime, ntime=ntime) &$
  sav_id='_0' &$
  savefile=model + '_seq' + sav_id + '.sav' &$
  print, 'Save dataset (seq) into ''' + savefile + '''' &$
  save, filename=savefile, seq &$
endif

if (BinFactor gt 1) then begin &$
  savefile=model + '_seq.sav' &$
  restore, savefile &$
  sav_id='_' + strtrim(string(BinFactor, format='(I9)'), 2) &$
  savefile=model + '_seq' + sav_id + '.sav' &$
  seq=seqrebin(seq, BinFactor) &$
  print, 'Save dataset (seq) into ''' + savefile + '''' &$
  save, filename=savefile, seq &$
endif

savefile=model + '_seq.sav'
restore, savefile

; drange=minmax(intens.i3r[*,*,0]) & for i=0,n_elements(intens.time)-1 do plotfield,/tv,reform(intens.i3r[*,*,i]),over=(i gt 0),drange=drange

;plotlines, seq.time, seq.i3r
;plotlines,sqrt(avg(avg(intens.i3r^2,0),0)-avg(avg(intens.i3r,0),0)^2)/avg(avg(intens.i3r,0),0)

;logL=[2.713,2.782] & Teff=[5526.0,6241.0] & M=5.0 & print,-logL + (4.0*alog10(Teff/!con.TeffSun)) + alog10(M) + alog10(!con.gravsun)
;logL=[4.053,4.080] & Teff=[4680.0,5544.0] & M=5.0 & print,-logL + (4.0*alog10(Teff/!con.TeffSun)) + alog10(M) + alog10(!con.gravsun)

;savefile=model + '_seq.sav'
;restore, savefile
;;mTimeMass=892
;;mTimeMass=1452
;mTimeMass=min(where(seq.C_HpTopFactor lt 0.60))
;x=seq.c_hptopfactor[mTimeMass:*]
;;y=reform((seqavg('v3mass',seq,qavg=-1))[299,mTimeMass:*])
;y=reform(seq.rhovb[200,mTimeMass:*])
;plotlines,x+(randomu(seed,n_elements(x))-0.5)*0.001,y,psym=1
;cPoly=poly_fit(x, y, 1, /double, yfit=yfitpoly)
;xfit=findgen(20000)*0.0001 & yfit=poly(xfit,cPoly)
;plotlines,xfit,yfit,/over,/zeroline
;print,min(abs(yfit),ind)
;print,xfit[ind]

; end ; rhd_seqpp_collect
