; pro rhd_surface_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_surface_collect ('rhd_surface_collect')
;
; PURPOSE:
;   Collect data from RAGE HDF files.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_surface_collect
;
; COMMON BLOCKS:
;
; ROUTINES:
;   combox
;   rdragebox
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    3. 4.01 (B.F. Uppsala) rhd_plot -> rhd_collect
;   10.10.03 (B.F.) rhd_collect -> rhd_seq_collect
;   2005-06-15 (B.F. LA) Adaption from rhd_seq_collect
;   2005-06-16 (B.F.) Adaption from rage_seq_collect
;   2005-07-27 (B.F.) Use rd_datasetlist for reading of models
;   2005-07-28 (B.F.) rage_layer_collect -> rage_cube_collect
;   2005-09-14 (B.F.) Some modifications
;   2006-03-28 (B.F.) Analysis of FLASH data
;   2009-02-11 (B.F.) Analysis of CO5BOLD data (mt28g50mm00n02),
;                     put par and datatype into cube structure
;   2009-03-16 (B.F.) rage_cube_collect.pro -> rhd_surface_collect.pro
;-
;-------------------------------------------------------------------------------

.run rhdmodeldata
rhdmodeldata, modeltable

; === Manual selection ===========
if (not keyword_set(external_control)) then begin &$
  ;stop &$
  ;datatype='rage' & single_quantity=q_id &$
  ;model='lc0gg' &$
  ;datatype='flash' & single_quantity=q_id &$
  ;model='lf0gg/lf0gg-2d_hdf5_plt_cnt_*' &$
  datatype='uio' & single_quantity='' &$
  ;model0='gt50g20n01' & modelident='_??' & parmodelident='_40' &$
  ;model0='gt51g20n01' & modelident='_44' & parmodelident='_44' &$
  ;model0='gt58g20n02' & modelident='_07' & parmodelident='_07' &$
  model0='mt09g50mm00n10' & modelident='_??' & parmodelident='' &$
  ;model0='mt09g50mm00n12' & modelident='_??' & parmodelident='' &$
  ;model0='mt15g50mm00n06' & modelident='_???' & parmodelident='' &$
  ;model0='mt15g50mm00n07' & modelident='_1[45]' & parmodelident='_15' &$
  ;model0='mt15g50mm00n09' & modelident='_??' & parmodelident='_20' &$
  ;model0='mt18g55mm00n03' & modelident='_0?' &$
  ;model0='mt22g50mm00n08' & modelident='_??' & parmodelident='' &$
  ;model0='mt28g50mm00n02' & modelident='_0[67]' & parmodelident='_06' &$
  ;model0='mt30g50mm00n01' & modelident='_??' & parmodelident='_20' &$
  ;
  ;layermode='i3' & q_id='v3' & i3=100 &$
  ;layermode='belowsmin' & q_id='v3mass' & logPfac=1.0 &$
  ;layermode='belowsmin' & q_id='s' &
  logPfac=1.0 &$
  m_timestep=0L &$
  fileincrement=1 &$
endif
; ================================

if (parmodelident eq '') then parmodelident=modeltable[(where(modeltable.model eq model0))[0]].parmodelident

if (datatype eq 'uio') then begin &$
  model=model0 + modelident &$
  modelbasepath=getenv('HOME') + '/dat/rhd/d' + model0 + '/' &$
  ;modelwithpath=modelbasepath+model+'.end' &$
  modelwithpath=modelbasepath+model+'.full*' &$
  ;
  ; --- Read parameter file ---
  parfile=modelbasepath + model0 + parmodelident + '.par' &$
  par=uio_struct_rd(parfile) &$
  ;
  ; --- Read EOS file ---
  eosdisk=getenv('HOME') + '/for/eos/dat/' &$
  ;opadisk=getenv('HOME') + '/for/opa/dat/' &$
  if (n_elements(EOS) eq 0) then begin &$
    tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
    ;dfopta, opadisk + par.opafile &$
  endif &$
endif else begin &$
  modelbasepath=getenv('HOME') + '/' + datatype + '/dat/' &$
  modelwithpath=modelbasepath+model &$
  ;
  ; --- Set default parameters for RAGE/FLASH runs ---
  par={grav: 10.0^7.695, Gamma1: 1.6666666} &$
endelse

; --- Read first file ---
delvar, listdata
ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                   single_quantity=single_quantity, $
                   listdata=listdata, ierr=ierr)
if (ierr ne 0) then begin &$
  print, 'Error reading data files' &$
  stop &$
endif
if (listdata.total_number eq 0) then begin &$
  print, 'No data files found' &$
  stop &$
endif
if (m_timestep gt 0) then begin &$
  listdata.open_number=long(m_timestep)-fileincrement &$
  ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                     single_quantity=single_quantity, $
                     listdata=listdata, ierr=ierr) &$
endif
if (datatype eq 'uio') then begin &$
  time_first=ful.modeltime &$
  ;
  uio_openrd, nc, listdata.file(listdata.total_number-1) &$
  outstr='found' &$
  while (outstr eq 'found') do begin &$
    uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
    if (ierr eq 0) then begin &$
      uio_rd, nc, termt,ntt, time_last,ident &$
      help, ierr, outstr, time_last &$
    endif &$
   endwhile &$
  uio_closrd, nc &$
  ;
  n_timestep_first=round(time_first/par.dtime_out_full) &$
  n_timestep_last =round(time_last/par.dtime_out_full) &$
  n_timestep=n_timestep_last-n_timestep_first+1 &$
  ;n_timestep=n_timestep+n_timestep_adjust &$
  help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep &$
endif else begin &$
  n_timestep=(listdata.total_number-listdata.open_number)/fileincrement &$
endelse

; --- Set to something small for tests (and comment out otherwise) ---
;n_timestep=150L

; --- Array sizes ---
nxc1=n_elements(ful.z.xc1)
nxc2=n_elements(ful.z.xc2)
nxc3=n_elements(ful.z.xc3)
time_first=ful.z.time

m1=0L
n1=nxc1-1L
m2=0L
n2=nxc2-1L
m3=0L
n3=nxc3-1L

print, 'Model ' + model + '  with ', nxc1, ' x ', nxc2, ' x ', nxc3, ' points and ', n_timestep, ' time steps'
print, m1,n1, m2,n2, m3,n3

; --- Create structure that will hold the averaged data ---
q=fltarr(n1-m1+1, n2-m2+1, n_timestep)
help, q
zbox=create_struct( $
  'dimension' , reform([[m1,n1], [m2,n2], [0L,n_timestep-1L]], 2, 3), $
  'layerdimension' , reform([[m1,n1], [m2,n2], [0L,0L]], 2, 3), $
  'itime'     , lonarr(     n_timestep), $
  'time'      , fltarr(     n_timestep), $
  'xc1'       , reform(ful.z.xc1[m1:n1  , 0      , 0      ], n1-m1+1, 1       , 1      ), $
  'xc2'       , reform(ful.z.xc2[0      , m2:n2  , 0      ], 1      , n2-m2+1 , 1      ), $
  'xc3'       , reform(ful.z.xc3[0      , 0      , m3:n3  ], 1      , 1       , n3-m3+1), $
  'xb1'       , reform(ful.z.xb1[m1:n1+1, 0      , 0      ], n1-m1+2, 1       , 1      ), $
  'xb2'       , reform(ful.z.xb2[0      , m2:n2+1, 0      ], 1      , n2-m2+2 , 1      ), $
  'xb3'       , reform(ful.z.xb3[0      , 0      , m3:n3+1], 1      , 1       , n3-m3+2), $
  'i3'        , 0L, $
  'Pfac'      , 1.0, $
  'mass0'     , fltarr(     n_timestep), $
  'rho0'      , fltarr(     n_timestep), $
  'ei0'       , fltarr(     n_timestep), $
  'P0'        , fltarr(     n_timestep), $
  'T0'        , fltarr(     n_timestep), $
  'x30'       , fltarr(     n_timestep), $
  'Hp0'       , fltarr(     n_timestep), $
  's0'        , fltarr(     n_timestep), $
  'massref'   , fltarr(     n_timestep), $
  'rhoref'    , fltarr(     n_timestep), $
  'eiref'     , fltarr(     n_timestep), $
  'Pref'      , fltarr(     n_timestep), $
  'Tref'      , fltarr(     n_timestep), $
  'x3ref'     , fltarr(     n_timestep), $
  'Hpref'     , fltarr(     n_timestep), $
  'sref'      , fltarr(     n_timestep), $
  'v3mass'    , fltarr(n1-m1+1, n2-m2+1, n_timestep), $
  's'         , fltarr(n1-m1+1, n2-m2+1, n_timestep))

surf=create_struct( $
  'model'     , model0, $
  'modelident', modelident , $
  'datatype'  , datatype, $
  'par'       , par, $
  'itime'     , lonarr(     n_timestep), $
  'time'      , fltarr(     n_timestep), $
  'ifile'     , lonarr(     n_timestep), $
  'file'      , strarr(     n_timestep), $
  'z'         , temporary(zbox))

ierr=0
i=0L
itime_last=-10000L
systime0=systime(1)

while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  if (i ge 1) then begin &$
    ful=0 &$
    ful=rd_datasetlist(modelwithpath, datatype=datatype, fileincrement=fileincrement, $
                       single_quantity=single_quantity, $
                       listdata=listdata, ierr=ierr) &$
  endif &$
  if (ierr eq 0) then begin &$
    file=listdata.file(listdata.open_number) &$
    ;
    print, '--- ', i, ful.z.itime, ful.z.time, ' ---' , format='(A,I5,1X,I7,1X,E15.8,A)' &$
    ;
;    if (strlowcase(layermode) eq 'belowsmin') then begin &$
      ; --- Find vertical velocity at "surface" ---
      ;
      ; --- Compute EOS ---
      eosbox,ful,eos=eos &$
      rho  =combox('rho', ful) &$
      ; --- Compute entropy and find minimum ---
      s    =combox('s', ful) &$
      ; --- Find minimum entropy for each vertical column ---
      smin =avg(min(s, ismin, dimension=3)) &$
      smean=float(avg(avg(double(s),0),0)) &$
;      smin0=min(smean, ismin) &$
;      extremum, (smean-smin0)/1.0E+09, 1, degree=2, i0=ismin, irange=1, ismin, smin, verbose=0 &$
;      smin=smin*1.0E+09+smin0 &$
      ; --- Find pressure at (or below) entropy minimum ---
      ;Pmean=float(rebin(reform(avg(avg(double(P),0),0),1,1,nxc3),nxc1,nxc2,nxc3)) &$
;      logP0=(interpol(alog(Pmean),findgen(nxc3),[ismin]))[0] & P0=exp(logP0) &$
      mass        =combox('mass3r', ful) &$
      massmean    =float(avg(avg(double(mass),0),0)) &$
      mass0       =meanmedian(mass[ismin]) & logmass0=alog(mass0) &$
      logmass     =alog(temporary(mass)) &$
      ;
      ; --- Compute mass density reference value ---
      if (i le 1) then begin &$
        logmassref=     logmass0+logPfac &$
      endif else begin &$
        logmassref=0.1*(logmass0+logPfac) + (1.0-0.1)*logmassref &$
      endelse &$
      ; --- Layers of quantities at reference level ---
      logmassdiff=min((logmass-logmassref)^2, iind, dimension=3) &$
      iindcor=where(logmass[iind] lt logmassref, nindcor) &$
      if (nindcor gt 0) then iind[iindcor]=iind[iindcor]-nxc1*nxc2 &$
      imind =iind &$
      ipind =imind+nxc1*nxc2 &$
      dimind=abs(logmass[imind]-logmassref) &$
      dipind=abs(logmass[ipind]-logmassref) &$
      wimind=dipind/(dimind+dipind) &$
      wipind=1.0-wimind &$
print,mean(wimind), mean(wipind), mean(float((array_indices(logmass,imind))[2,*])) &$
      ; --- Additional scalar quantities at reference level ---
      surf.z.rho0[    i]=meanmedian(rho[ismin]) &$
      surf.z.rhoref[  i]=mean(rho[imind]*wimind + rho[ipind]*wipind) &$
      surf.z.s0[      i]=meanmedian(s[ismin]) &$
      srefl             =s[imind]*wimind + s[ipind]*wipind &$
      surf.z.sref[    i]=mean(srefl) &$
      P                 =combox('p', ful) &$
      surf.z.P0[     i]=meanmedian(P[ismin]) &$
      surf.z.Pref[   i]=mean(P[imind]*wimind + P[ipind]*wipind) &$
      qa               =P/(rho*par.grav) &$
      surf.z.Hp0[    i]=meanmedian(qa[ismin]) &$
      surf.z.Hpref[  i]=mean(qa[imind]*wimind + qa[ipind]*wipind) &$
      qa               =combox('ei', ful) &$
      surf.z.ei0[    i]=meanmedian(qa[ismin]) &$
      surf.z.eiref[  i]=mean(qa[imind]*wimind + qa[ipind]*wipind) &$
      qa               =combox('T', ful) &$
      surf.z.T0[     i]=meanmedian(qa[ismin]) &$
      surf.z.Tref[   i]=mean(qa[imind]*wimind + qa[ipind]*wipind) &$
      qa               =combox('xc3a', ful) &$
      surf.z.x30[    i]=meanmedian(qa[ismin]) &$
      surf.z.x3ref[  i]=mean(qa[imind]*wimind + qa[ipind]*wipind) &$
      ; --- Save memory: s=0 & P=0 &$
      ; --- Get vertical velocity at layer from mass flux and density ---
;      rhov3=combox('rhov3', ful) &$
;      rhov3=rhov3[imind]*wimind + rhov3[ipind]*wipind &$
;      rho  =combox('rho', ful) &$
;      rho  =rho[  imind]*wimind + rho[  ipind]*wipind &$
;      rhov3=rhov3-mean(rhov3) &$
;      q    =rhov3/mean(rho) &$
;      if (strlowcase(q_id) eq 'v3mass') then begin &$
        rhov3=combox('rhov3', ful) &$
        rhov3m=rhov3[imind] &$
        rhov3m=rhov3m-mean(rhov3m) &$
        rhov3p=rhov3[ipind] &$
        rhov3p=rhov3p-mean(rhov3p) &$
        rhom  =rho[  imind] &$
        rhop  =rho[  ipind] &$
        surf.z.v3mass[ *,*,i]=rhov3m/mean(rhom)*wimind + rhov3p/mean(rhop)*wipind &$
;      endif else if (strlowcase(q_id) eq 's') then begin &$
        surf.z.s[ *,*,i] =srefl &$
;      endif else begin &$
;        qa    =combox(q_id, ful) &$
;        qam   =qa[   imind] &$
;        qap   =qa[   ipind] &$
;        q     =qam*wimind + qap*wipind &$
;      endelse &$
      ;
;   endif else if (q_id eq 'v3surf2') then begin &$
;      ; --- Find vertical velocity at "surface" ---
;      ;
;      ; --- Compute entropy and find minimum ---
;      eosbox,ful,eos=eos &$
;      s=combox('s', ful) &$
;      smin=min(s, ismin) &$
;      ; --- Find pressure at (or below) entropy minimum ---
;      logP=alog(combox('p', ful)) &$
;      logPsmin=(reform(logP[ismin]))[0] &$
;      logPref=logPsmin+2.0 &$
;      logPdiff=min((logP-logPref)^2, iind, dimension=3) &$
;      ; --- Get vertical velocity at layer from mass flux and density ---
;      rhov3=(combox('rhov3', ful))[iind] &$
;      rho  =(combox('rho'  , ful))[iind] &$
;      rhov3=rhov3-avg(rhov3) &$
;      q    =rhov3/rho &$
;      ;
;    endif else if (q_id eq 'v3surf1') then begin &$
;      ; --- Find vertical velocity at "surface" ---
;      ;
;      ; --- Compute entropy ---
;      eosbox,ful,eos=eos &$
;      s=combox('s', ful) &$
;      ; --- Find minimum entropy for each vertical column ---
;      smin=min(s, ismin, dimension=3) &$
;      iasmin=array_indices(s, ismin) &$
;      iasmin[2,*]=(iasmin[2,*]-20L) > 0L &$
;      iind=reform(iasmin[0,*] + nxc1*iasmin[1,*] + nxc1*nxc2*iasmin[2,*]) &$
;      ; --- Get vertical velocity at layer from mass flux and density ---
;      rhov3=(combox('rhov3', ful))[iind] &$
;      rho  =(combox('rho'  , ful))[iind] &$
;      rhov3=rhov3-avg(rhov3) &$
;      q    =rhov3/rho &$
;      ;
;    endif else begin &$
;      ; --- Any quantity available via combox ---
;      ;
;      q=(combox(q_id, ful))[m1:n1,m2:n2,i3] &$
;      surf.z.i3=i3 &$
;      ;
;    endelse &$
    ;
    file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
    if (datatype eq 'uio') then begin &$
      surf.ifile[i]  =listdata.open_number &$
    endif else begin &$
      surf.ifile[i]  =listdata.snapshot(listdata.open_number) &$
    endelse &$
    surf.file[i]     =file_tmp &$
    surf.itime[i]    =ful.z.itime &$
    surf.time[i]     =ful.z.time &$
    surf.z.itime[i]  =ful.z.itime &$
    surf.z.time[i]   =ful.z.time &$
    surf.z.Pfac      =exp(logPfac) &$
    ;
    surf.z.mass0[ i] =mass0 &$
    surf.z.massref[i]=exp(logmassref) &$
    ;
    i=i+1 &$
  endif &$
endwhile

if (datatype eq 'uio') then begin &$
  uio_closrd, listdata.channel &$
endif

print, 'Total time: ', systime(1)-systime0, ' sec'

slashpos=strpos(model, '/')
if (slashpos eq -1) then begin &$
  model_id=model &$
  if (datatype eq 'uio') then model_id=model0 &$
endif else begin &$
  model_id=strmid(model, 0, slashpos) &$
endelse
;if (strlowcase(layermode) eq 'belowsmin') then begin &$
;  outfile=model_id + '_surf_' + q_id + '_' + layermode + '_' + trim(string(logPfac*10.0,format='(I4)'),2) + '.sav' &$
;endif else begin &$
;  outfile=model_id + '_surf_' + q_id + '_' + layermode + '_' + trim(string(i3),2)                         + '.sav' &$
;endelse
outfile=model_id + '_surf_' + 'Pfac' + trim(string(logPfac*10.0,format='(I4)'),2) + '.sav' &$

print, 'Save dataset (surf) into ''' + outfile + ''''
save, filename=outfile, surf

;vis3d,findgen(size(surf.z.v3mass,/dim)),surf.z.v3mass

; end ; rhd_surface_collect
