;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_teffgransize_plot ('RHD_temperature_effective_granule_size_plot')
;
; PURPOSE:
;   Plot granule size over effective temperature
;
; CATEGORY:
;   RHD, plotting
;
; CALLING SEQUENCE:
;   @rhd_teffgransize_plot
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-19 (B.F. Garching)
;-
;------------------------------------------------------------------------------

dev='x'
file_prefix='rhd_'
TRange=[8600.0,800.0]
;loggRange=[5.6,1.3]
;logIntdrmsRange=[-2.3,-0.2]
RelGranSizeRange=[0.0,16.0]
;RelGranSizeRange=[0.0,4.0]
colorliststr='colorlist=replicate1([!ctv.black, !ctv.blue, !ctv.magenta, !ctv.red, !ctv.orange, !ctv.yellow, !ctv.green, !ctv.cyan, !ctv.gray],99)' 
xsize=12.5 & ysize=8.0 & wfactor=700

; --- Load current model data base ---------------------------------------------
.run rhdmodeldata
rhdmodeldata, modDB
restore, 'allpardb.sav'
restore, 'allbasdb.sav'
restore, 'allintdb.sav'
;;restore, 'allseqdb.sav'

; --- Define model list ---
modellist=modDB.model
modellist=modellist[where(modellist ne 'd3gt57g44n58')]
modellist=modellist[where(udb_extract('Quality'   , modellist, modDB) ge 0.5           )]
modellist=modellist[where(udb_extract('dustscheme', modellist, parDB) ne 'dust_bins_01')]
modellist=modellist[where(udb_extract('C_radDinci', modellist, parDB) eq 0.0           )]
modellist=modellist[where(udb_extract('Teffout'   , modellist, basDB) ne 0.0           )]
;modellist=modellist[where(udb_extract('grav'      , modellist, basDB) ge 4.0           )]
;modellist=modellist[where(udb_extract('INTADRMS'  , modellist, intDB) ne 0.0           )]
modellist=modellist[where(udb_extract('XM1INTFFT', modellist, intDB) ne 0.0)]
nModels=n_elements(modellist)

; --- Extract plot data ---
;Teffin   =udb_extract('Teff', modellist, basDB)
Teff     =udb_extract('Teffout', modellist, basDB)
logg     =alog10(udb_extract('grav', modellist, basDB))
dimension=udb_extract('dimension', modellist, basDB)
;logIntDrms=alog10(udb_extract('INTADRMS', modellist, intDB))
GranSize=udb_extract('XM1INTFFT', modellist, intDB)
;GranSize=udb_extract('XP1INTFFT', modellist, intDB)
RelGranSize=GranSize/1.0E+08 / (Teff/5770.0) * (10.0^logg/10.0^4.44)

; --- File names for plot output ---
file_id=file_prefix + 'Teffgransize'
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
if (not keyword_set(overplot_flag)) then begin & Pbak=!P & Xbak=!X & Ybak=!Y & endif

; --- Choose color table ---
normct

; --- Prepare plotting device ---
if (not keyword_set(overplot_flag)) then begin &$
  if (strupcase(dev) eq 'X') then begin &$
    set_plot, 'X' &$
    ;!P.CHARSIZE=1.4 &$
    !P.CHARSIZE=1.6 &$
    ;!P.CHARSIZE=1.8 &$
    !P.FONT=-1 &$
    !P.SYMSIZE=3 &$
    SymSizeFactor=3 &$
    win_id=0 &$
    window, win_id, xsize=xsize/10.0*wfactor, ysize=ysize/10.0*wfactor, retain=2 &$
  endif else begin & $
    set_plot, 'ps' & $
    !P.CHARSIZE=0.7 &$
    ;!P.CHARSIZE=1.0 &$
    ;!P.CHARSIZE=1.2 &$
    !P.FONT=0 &$
    !P.SYMSIZE=1 &$
    SymSizeFactor=1 &$
    device, filename=ps_file, $
            encapsulated=(strupcase(dev) eq 'EPS'), $
            /portrait, xsize=xsize, ysize=ysize,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
    modctv, 'PS' &$
  endelse &$
endif

; --- List of colors ---
dummy=execute(colorliststr)

; --- Set global plot parameters ---
!P.THICK=2
!X.MARGIN=[9,1] & !Y.MARGIN=[4,1]
!P.COLOR=!ctv.black & !P.BACKGROUND=!ctv.white
;!X.STYLE=5     & !Y.STYLE=5
!X.RANGE=TRange
!Y.RANGE=RelGranSizeRange

PsymList=replicate(136, nModels)
PsymList[where(dimension eq 3)]=144
;SymSizeList=replicate(0.6, nModels)
SymSizeList=0.5*(6.5-logg) > 0.5
SymFillList=replicate(0, nModels)
ind=where((Teff le 5800.0) and (Teff ge 5750.0) and (logg ge 4.40) and (logg le 4.5), nCount)
if (nCount ge 0) then begin &$
  PsymList[ind]=104 &$
  SymFillList[ind]=1 &$
endif
SymColorList=replicate(ColorList[0], nModels)

plotlines, Teff, RelGranSize, /nodata, psym=1, $
  xRange=TeffRange, xtitle='T!Deff!N [K]', $
  drange=RelGranSizeRange, ytitle='Normalized Granule Size [solar Mm]', $
  dev=dev,  /psnoinit, psformat='nochange', /noclose

for i=0,nModels-1 do begin &$
  plotlines, [Teff[i]], [RelGranSize[i]], /over, $
    psym=psymuser(PsymList[i], fill=SymFillList[i]), $
    symsize=SymSizeFactor*SymSizeList[i], linecolor=SymColorList[i], $
    dev=dev,  /psnoinit, psformat='nochange', /noclose &$
endfor ; i

; === Write image files, close PS device ===
if (strupcase(dev) eq 'X') then begin &$
  if (keyword_set(png_flag)) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (keyword_set(jpg_flag)) then begin &$
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else if (not keyword_set(keepopen_flag)) then begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endif

; --- Reset global plot parameters ---
if (not keyword_set(keepopen_flag)) then begin & !P=Pbak & !X=Xbak & !Y=Ybak & endif

; end ; rhd_teffgransize_plot
