; pro rhd_tpgrid
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_tpgrid ('radiation_hydrodynamics_temperature_pressure_grid')
;
; PURPOSE:
;
; CATEGORY:
;   RHD, start model creation
;
; CALLING SEQUENCE:
;   @rhd_tpgrid
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2007-11-12 (B.F. Lyon) Copied from rhd_manipulate.pro
;-
;------------------------------------------------------------------------------

; ==================================================
if (not keyword_set(external_control)) then begin &$
  ;stop &$
  ;
  ; --- Model name ---
  ;model='dustg-t05-30n01' & modelident='_01' & dustfrac=1.0E-05 ; First test &$
  model='dustg-t05-30n02' & modelident='_01' & dustfrac=6.0324E-04/(2.0*24.31)*(2.0*24.31+1.0*28.09+4.0*16.0) ; AGS abu, Mg2SiO4&$
  ;
  ; --- Temperature/pressurt/dustbin grid ---
  nT=101   & Tmin=500.0  & Tmax=3000.0 &$
  nlogP=29 & logPmin=1.0 & logPmax=8.0 &$
  ndust=6 &$
  ; --- Spatial grid spacing, time step ---
  dx=1.0E+05 & dtime=1.0 &$
  ; --- Fraction: dust density/gas density ---
  ;
endif
; ==================================================

outmodel    =model & outmodelident=modelident
outmodeldisk='~/idlwrk/'
outmodelfile=outmodeldisk + outmodel + outmodelident + '.sta'

eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
gasfile='gas_mm00_l.eos' & abuX0='sun' & gasname='gas'

; --- Read EOS file ---
par=uio_struct_rd(parfile)
if (n_elements(EOS) eq 0) then begin &$
  ;tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
  tabinter_rdcoeff, eosdisk + gasfile, gas &$
  dfopta, opadisk + par.opafile &$
endif
;f0=!con.sigmasb*par.Teff^4

; --- Physical units ---
unit_time='s'
unit_x1  ='cm'
unit_x2  =unit_x1
unit_x3  =unit_x1
unit_rho ='g/cm^3'
unit_v   ='cm/s'
unit_e   ='erg/g'
name_quc =['rho_monomer', 'rho_dustr1', 'rho_dustr2', 'rho_dustr3', 'rho_dustr4', 'rho_dustr5']
unit_quc =['g/cm^3'     , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    , 'g/cm^3'    ]
;unit_B   ='G'


; --- Start processing --------------------------------------------------------

; --- Indices ---
m1=1L
n1=long(nT)
m2=1L
n2=long(ndust)
m3=1L
n3=long(nlogP)

; --- Time information ---
itime=0L
time =0.0
time_out_full_last=-1.0E9
time_out_mean_last=time_out_full_last

; --- Axes ---
xb1=findgen(n1-m1+2)*dx
xb2=findgen(n2-m2+2)*dx
xb3=findgen(n3-m3+2)*dx
xc1=0.5*(xb1[0:n1-m1]+xb1[1:n1-m1+1])
xc2=0.5*(xb2[0:n2-m2]+xb2[1:n2-m2+1])
xc3=0.5*(xb3[0:n3-m3]+xb3[1:n3-m3+1])

; --- Velocities ---
v1=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)
v2=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)
v3=fltarr(n1-m1+1, n2-m2+1, n3-m3+1)

; --- Create 2D temperature/pressure arrays ---
T=     rebin(reform(Tmin   +(Tmax   -Tmin   )/float(nT   -1L)*findgen(nT   ), nT,1    ), nT,nlogP)
P=10.0^rebin(reform(logPmax+(logPmin-logPmax)/float(nlogP-1L)*findgen(nlogP), 1 ,nlogP), nT,nlogP)

; --- Compute 2D rho and ei arrays via EOS ---
gasinter, P, T, rho, xcp, xgam, cxt, h, s, csound, eos=gas, ierror=ierror
ei=h-P/rho

; --- Make 3D arrays ---
rho4D=    reform(rho, nT,1,nlogP,1)*dustfrac
rho=rebin(reform(rho, nT,1,nlogP), nT,ndust,nlogP)
ei =rebin(reform(ei , nT,1,nlogP), nT,ndust,nlogP)

; --- Dust ---
quc=fltarr(n1-m1+1, n2-m2+1, n3-m3+1, ndust)
for iquc=0,ndust-1 do begin &$
  quc[*,iquc,*,iquc]=rho4D &$
endfor

help, itime, time, dtime, v1, T, P, rho, ei, quc
help, m1,n1, m2,n2, m3,n3
print, 'minmax(rho,ei,v1,v2,v3,quc)'
print, minmax(rho) & print, minmax(ei) & print, minmax(v1) & print, minmax(v2) & print, minmax(v3) & print, minmax(quc)

; --- End processing ----------------------------------------------------------


rhd_manipulate_assure3d, m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3
;
help, m1,n1, m2,n2, m3,n3
print, 'minmax(rho,ei,v1,v2,v3)'
print, minmax(rho) & print, minmax(ei) & print, minmax(v1) & print, minmax(v2) & print, minmax(v3)
;
; --- Model information ---
fileform='unformatted' & fileconv='ieee_4'
version    ='rhd_tpgrid 2007-11-12'
description=['Temperature/pressure grid with dust/monomer density in different bins', $
             'Tmin   =' + string(Tmin   ,format='(F9.2)') + '  Tmax   =' + string(Tmax   ,format='(F9.2)'), $
             'logPmin=' + string(logPmin,format='(F9.2)') + '  logPmax=' + string(logPmax,format='(F9.2)')]
history    =['Produced by rhd_tpgrid: ' + strtrim(systime(0),2)] &$
;
; stop ; ----------------------------
;
;rhd_manipulate_makebox, ful1, $
;      m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3, quc=quc
;
; stop ; ----------------------------
;
; --- Write data into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
   -1, ncout, file=outmodelfile, form=fileform, conv=fileconv, $
   ndes=n_elements(description), description=description, $
   nhis=n_elements(history), history=history, $
   version=version, $
   unit_time=unit_time, $
   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
   unit_quc=unit_quc, name_quc=name_quc, unit_B=unit_B, $
   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
   itime=itime, time=time, dtime=dtime, $
   out_time_full_last=time_out_full_last, out_time_mean_last=time_out_mean_last, $
   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei, $
   quc=quc, Bb1=Bb1, Bb2=Bb2, Bb3=Bb3
;
; end ; rhd_tpgrid
