; pro rhd_v3d
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_v3d ('radiation_hydrodynamics_visualize_3_dimension')
;
; PURPOSE:
;   Load model data for single time step, solve EOS, compute opacity.
;
; CATEGORY:
;   RHD, visualization
;
; CALLING SEQUENCE:
;   @rhd_v3d
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos, par, ful, mean
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    4. 5.02 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
modelident=''
;model='mt15g50mm00n06' & modelident='_18'
;model='st22g35n07' & modelident='_038'
;model='st28gm06n01' & modelident='_06'
;model='st28gm06n02' & modelident='_10'
;model='st28gm06n05' & modelident='_03'
;model='st28gm06n13' & modelident='_003'
;model='st28gm06n033' & modelident='_017' & nmeanID=9
;model='st28gm06n034' & modelident='_001' & nmeanID=9
;model='st28gm06n034' & modelident='_003' & nmeanID=9
;model='st28gm06n034' & modelident='_084' & nmeanID=9
;model='st28gm05n010' & modelident='_194' & nmeanID=0
;model='st28gm05n011' & modelident='_153' & nmeanID=0
model='st28gm05n012' & modelident='_139' & nmeanID=0
;model='st28gm05n013' & modelident='_017' & nmeanID=0
;model='st33gm06n02' & modelident='_18'
;model='st33gm06n03' & modelident='_01'
;model='st35gm03n06' & modelident='_24'
;model='st35gm03n13' & modelident='_708'
;model='st35gm04n01' & modelident='_50'
;model='st35gm04n02' & modelident='_20'
;model='st35gm04n03' & modelident='_20'
;model='st35gm04n04' & modelident='_33'
;model='st35gm04n06' & modelident='_15'
;model='st35gm04n07' & modelident='_29'
;model='st35gm04n08' & modelident='_03'
;model='st35gm04n22' & modelident='_30'
;model='st35gm04n24' & modelident='_08'
;model='st35gm04n25' & modelident='_18'
;model='st35gm04n26' & modelident='_75'
;model='st35gm04n32' & modelident='_10'
;model='st35gm04n33' & modelident='_22'
;model='st35gm04n34' & modelident='_36'
;model='st35gm06n02' & modelident='_01'
;model='st36g00n04' & modelident='_069'
;model='st36g45n25' & modelident='_013'
;model='chro01' & modelident='l'
;model='radgreens3d02' & modelident='_21'
;model='rhd' & modelident=''
;
;modeldisk='/u64/bf' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk='/home/bf/idl/wrk/'
;modeldisk=getenv('HOME') + '/dat/rhd/tests/dsedov/'
;modeldisk=getenv('HOME') + '/dat/rhd/tests/d' + model + '/'
;modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/tacsg2/job1/'
;modeldisk=getenv('HOME') + '/dat/job/jc/'
;modeldisk=getenv('HOME') + '/dat/rhd/tmp/'
;modeldisk=getenv('HOME') + '/dat/job/copenhagen/p3/job1/'
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
modelfile=modeldisk + model + modelident + '.full' & n=0
;modelfile=modeldisk + model + modelident + '.sta' & n=0
;modelfile=modeldisk + model + modelident + '.end' & n=0
meanfile=modeldisk + model + modelident + '.mean' & nmean=nmeanID
;
parfile=modeldisk + model + modelident + '.par'
;
; --- Read parameter, EOS, and opacity file ---
if (n_elements(par) eq 0) then par=uio_struct_rd(parfile)
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
  dfopta, opadisk + par.opafile &$
endif
f0=!con.sigmasb*par.Teff^4
;
if (n_elements(ful) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful=uio_dataset_rd(modelfile, ndataset=n) &$
  ;ful=uio_dataset_rd(modelfile, ndataset=n, blacklist=['v1','v2','v3']) &$
  eosbox, ful, eos=EOS, opa=1, ierror=ierror & help,ierror &$
  ;eosbox, ful, eos=EOS, opa=1, out_q=1, ierror=ierror & help,ierror &$
endif
if (n_elements(mean) eq 0) then mean=uio_dataset_rd(meanfile, ndataset=nmean)
;
print, ful.z.time, mean.z.time
;print, ful.z.time
;

;c1=combox('xc3a', ful)
;d1=combox('quc002orho', ful)

;c1=combox('radius'        , ful )/!con.Rsun
;c1=combox('tauharm', ful)
;c1=!con.sigmasb*combox('T', ful)^4 / f0
;d1=!con.sigmasb*combox('T', ful)^4 / f0
;c1=!con.sigmasb*(0.001*(combox('T', ful)-3300.0)+3300.0)^4 / f0
;c1=combox('T', ful)
;d1=combox('tauharm', ful)
;d1=combox('T', ful)/1000.
;
;c2=c1
;c2=combox('tauharm', ful)
;c2=combox('dtau3', ful)
;c2=combox('dtau3', ful) * (combox('tauharm', ful) le 1.0E+01)
;d2=d1^4
;c2=map2box('int'          , mean) / f0
;d2=combox('radiuscubenorm', ful )*(1.0-c2*(0.05/max(c2)))
;d2=alog10(combox('p', ful))
c2=combox('v_radial'      , ful)/1.0E+05
d1=( c2 > 0.0) * (combox('tauharm', ful) gt 1.0)
d2=(-c2 > 0.0) * (combox('tauharm', ful) gt 1.0)
;
;c3=c1
;c3=combox('v_radial'       , ful )
;c3=combox('radius'        , ful )/(max(ful.z.xc3))
;d3=combox('radius'        , ful )/(max(ful.z.xc3))
;d3=combox('s'             , ful ) * (d1 gt 1.0)
;d3=-combox('v_radial'       , ful ) * (combox('tauharm', ful) gt 1.0)
;d3=d2
;d3=float(get_cellmask5(ful.eos.t,ful.eos.p, SHELLTHI=9, NEIGH=5, MINVOX=62, TMIN=12000.0, PMAX=3.34))
;quc001=combox('quc001', ful)/40.0E+06
;quc002=combox('quc002', ful)/40.0E+06
;d3=quc001+quc002
;c3=alog10(0.1 > ((quc001>1E-6)/(quc002>1E-6)) < 10.0)
;

;vis3d, d1,c1, $
;  IDLPAL=[3], viewcolor=[0,0,0], lights=[0], $
;  /no_colorbar, scale=1.4, xsize=700, ysize=500, decimation=[100]

;vis3d, c2,d1, oUser1=oPolyHigh, isovalue=[1.0], $
;  IDLPAL=[3], viewcolor=[255,255,255], lights=[0], $
;  /no_colorbar, scale=1.4, xsize=500, ysize=500, decimation=[20]

;vis3d, c1,d1, c2,d1, isovalue=[1.0, 1.0], $
;  IDLPAL=[3,3], viewcolor=[0,0,128], lights=[0,0], $
;  /no_colorbar, scale=1.4, xsize=450, ysize=450, decimation=[10,10], renderer=0

vis3d, d1,c2, d2,c2, isovalue=[0.95, 0.95], $
  IDLPAL=[64,64], viewcolor=[20,20,20], lights=[0,0], $
  /no_colorbar, scale=2.2, xsize=900, ysize=800

;vis3d, d1,c1, d3,c3, isovalue=[1.0, 0.0], $
;  IDLPAL=[3,76], viewcolor=[20,20,20], lights=[0,0], $
;  /no_colorbar, scale=1.4, xsize=800, ysize=600

;vis3d, d1,c1, d3,c3, isovalue=[1.0, 0.0], $
;  IDLPAL=[3,76], viewcolor=[20,20,20], lights=[0,0], $
;  /no_colorbar, scale=1.2, xsize=900, ysize=700

;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[1.0, 0.95, 0.0], $
;  IDLPAL=[3,3,64], viewcolor=[20,20,20], lights=[0,0,0], $
;  /no_colorbar, scale=1.4,xsize=700, ysize=700

;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[1.0, 0.94, 3.0e9],scale=2.2,xsize=300, ysize=300
;vis3d, d1,c1, d3,c3, isovalue=[1.0, 3.0e9],scale=2.0,xsize=500, ysize=500

;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[1.0, 0.94, 2.8e9], $
;  IDLPAL=[3,3,0], viewcolor=[0,150,150], lights=[0,0,1], $
;  /no_colorbar, scale=2.0,xsize=400, ysize=400

;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[1.0, 0.4, 2.8e9], $
;  IDLPAL=[3,3,0], viewcolor=[0,150,150], lights=[0,0,1], $
;  /no_colorbar, scale=2.2,xsize=400, ysize=400
;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[12000.0, 25000.0, 0.5], $
;  IDLPAL=[3,3,3], viewcolor=[0,150,150], lights=[1,1,1], $
;  /no_colorbar, scale=2.0,xsize=400, ysize=400, decimation=[4,4,4]
;
;vis3d, d1,c1, /no_colorbar, scale=2.0,xsize=400, ysize=400
