; pro rhd_v3d_2local
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_v3d_2local ('radiation_hydrodynamics_visualize_3_dimension_2_models_local')
;
; PURPOSE:
;   Load model data for single time step, solve EOS, compute opacity.
;
; CATEGORY:
;   RHD, visualization
;
; CALLING SEQUENCE:
;   @rhd_v3d_2local
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos, ful
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    2002-05-04 Written by Bernd Freytag, Astronomical Observatory Uppsala
;    2006-09-19 (B.F. Lyon) Version for local models
;-
;------------------------------------------------------------------------------
;
;model='mt15g50mm00n06' & ModelIdent='_047'
;model='mt15g50mm00n06' & ModelIdent='_067'
;model='mt15g50mm00n10' & ModelIdent='_035'
;model1='mt22g50mm00n11' & ModelIdent1='_001' & model2='mt22g50mm00n11' & ModelIdent2='_002'
;model='at80g40n22' & ModelIdent='_07'
;model='job3Dh50' & ModelIdent='_030'
;model='jobtest20100326' & ModelIdent='_01'
;model='d3gt57g44b0n001' & ModelIdent='_044'
;model='d3t57g44b0n002' & ModelIdent='_041'
;model='d3t57g44b0n008' & ModelIdent='_019'
;model1='d3t57g44b0n009' & ModelIdent1='_027'
;model2='d3t57g44b0n013' & ModelIdent2='_024'
;model='d3t57g44b1n002' & modelident='_089'
;model='d3t57g44b1n003' & modelident='_052'
;model='d3t57g44b1n005' & modelident='_030'
;model='d3t57g44b1n006' & modelident='_049'
;model='d3t57g44b1n007' & modelident='_061'
;model='d3t57g44b1n008' & modelident='_036'
;model1='d3t57g44b1n009' & modelident1='_002' & model2='d3t57g44b1n009' & modelident2='_015'
;model1='d3t57g44b1n010' & modelident1='_001' & model2='d3t57g44b1n010' & modelident2='_007'
;model1='d3t57g44b1n011' & modelident1='_114' & model2='d3t57g44b1n011' & modelident2='_114'
;model1='d3t57g44b1n012' & modelident1='_020' & model2='d3t57g44b1n015' & modelident2='_001'
;model1='d3t57g44b1n013' & modelident1='_075' & model2='d3t57g44b1n013' & modelident2='_038'
;model1='d3t57g44b1n010' & modelident1='_015' & model2='d3t57g44b1n016' & modelident2='_015'
;model1='d3t57g44b1n016' & modelident1='_018' & model2='d3t57g44b1n016' & modelident2='_018'
;model1='d3t57g44b1n016' & modelident1='_018' & model2='d3t57g44b1n017' & modelident2='_018'
;model1='d3t57g44b1n020' & modelident1='_010' & model2='d3t57g44b1n020' & modelident2='_030'
;model1='d3t57g44b1n021' & modelident1='_006' & model2='d3t57g44b1n021' & modelident2='_009'
;model1='d3t57g44b1n022' & modelident1='_008' & model2='d3t57g44b1n022' & modelident2='_002'
;model1='d3t57g44b1n023' & modelident1='_160' & model2='d3t57g44b1n025' & modelident2='_040'
;model1='d3t57g44b1n025' & modelident1='_057' & model2='d3t57g44b1n027' & modelident2='_007'
;model1='d3t57g44b1n027' & modelident1='_069' & model2='d3t57g44b1n028' & modelident2='_069'
model1='d3t57g44b1n031' & modelident1='_004' & model2='d3t57g44b1n034' & modelident2='_004'
;model1='st35gm03n14a' & modelident1='_001' & model2='st35gm03n14b' & modelident2='_001'
;model='sunmhd3D100' & modelident='_n018'
;model='sunmhd3D101' & modelident='_n007'
;model='rhd' & ModelIdent=''
;
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'

model=model1 & ModelIdent=ModelIdent1
;
;modeldisk='/u64/bf' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk='/home/bf/idl/wrk/'
;
modelfile=modeldisk + model + ModelIdent + '.full' & n=0
;modelfile=modeldisk + model + ModelIdent + '.sta' & n=0
;modelfile=modeldisk + model + ModelIdent + '.end' & n=0
;meanfile=modeldisk + model + ModelIdent + '.mean' & nmean=4*n+2
;
parfile=modeldisk + model + ModelIdent + '.par'
;
; --- Read parameter, EOS, and opacity file ---
par1=uio_struct_rd(parfile)
delvar, EOS
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par1.eosfile, EOS &$
  dfopta, opadisk + par1.opafile &$
endif
f0=!con.sigmasb*par1.Teff^4
;
if (n_elements(ful1) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful1=uio_dataset_rd(modelfile, ndataset=n) &$
  Lx1=max(ful1.z.xb1)-min(ful1.z.xb1) &$
  Lx3=max(ful1.z.xb3)-min(ful1.z.xb3) &$
  nx1=n_elements(ful1.z.xc1) &$
  ;nx1=240L &$
  ;nx1=250L &$
  ;;nx1=300L &$
  nx3=round(Lx3/Lx1*nx1) &$
  ;ful1=refinebox('0', ful1, nx1, nx1, nx3) &$
  eosbox, ful1, eos=EOS, opa=1, out_q=0, ierror=ierror & help,ierror &$
endif

print, ful1.z.time

n1=n_elements(ful1.z.xc1)
n2=n_elements(ful1.z.xc2)
n3=n_elements(ful1.z.xc3)

c1=combox('bc3', ful1)/(combox('bc',ful1)>1e-9)
d1=combox('Pmag', ful1)^2/combox('P',ful1)

;d1=combox('bc', ful1)^2/combox('rho',ful1)^(1.0/1.0)
;;c1=combox('bc3', ful)
;d1=rebin(reform(c1, n1,n2,n3,1), n1,n2,n3,3)
;d1[*,*,*,0]=combox('bc1', ful)
;d1[*,*,*,1]=combox('bc2', ful)
;d1[*,*,*,2]=combox('bc3', ful)

;d1=(combox('bc', ful))^2
;c1=combox('helv', ful1)
;d1=(abs(c1))
;c1=c1/((combox('v', ful)^2)>1.0)
;c1=(combox('v3',ful1)/(combox('v', ful1)>1.0))
;d1=(combox('v', ful1))^2

;c1=combox('xc3a', ful)
;c1=c1-min(c1) & c1=c1/max(c1) & c1=1.0-c1
;;d1=combox('quc002', ful)
;d1=combox('quc002orho', ful)
;d1=d1/max(d1)

;d1=combox('rho', ful)+5.0*combox('rho', ful, /meansub)
;d1=combox('T', ful)+10.0*combox('T', ful, /meansub)
;d1=combox('T', ful, /meansub) * (combox('T', ful) gt 2000.0)
;d1=combox('T', ful, /meansub) * (combox('s', ful) lt (par.s_inflow+1.0e-7))

;d2=combox('s', ful)-par.s_inflow
;d2=d2*(combox('s', ful) lt (par.s_inflow+1.0e-7))
;d2=d2/max(abs(d2))
;d2=-(d2<0.0)
;d2=d2/max(abs(d2))

;d1=d1+1.0
;d1=4.0+alog10(d1 > 1.0E-04)
;d1=8.0+alog10(d1 > 1.0E-8)
;d1=4.0*d1
;d1=d1 mod 1.0
;c2=combox('s', ful, /meansub)
;c2=c2/max(abs(c2))
;d2=combox('dtau3', ful)
;ind=where(c2 gt 0.0)
;d2[ind]=0.1*d2[ind]

;d2=combox('T', ful)/1000.0
;c2=alog10(combox('rho', ful))
;c2=alog10(combox('rho', ful)) & d2=d2-min(d2)
;d2=combox('rho', ful, /meansub) & d2=d2>0.0
;c2=combox('xc3a', ful)

;c1=combox('T', ful)/1000.0
;d1=-combox('v3', ful)*1.0E-05
;c1=alog10(combox('p', ful))

;d1=combox('tau_int', ful)
;c2=combox('dtau3', ful)
;d2=alog10(combox('p', ful))

model=model2 & ModelIdent=ModelIdent2
;
;modeldisk='/u64/bf' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk='/home/bf/idl/wrk/'
;
modelfile=modeldisk + model + ModelIdent + '.full' & n=0
;modelfile=modeldisk + model + ModelIdent + '.sta' & n=0
;modelfile=modeldisk + model + ModelIdent + '.end' & n=0
;meanfile=modeldisk + model + ModelIdent + '.mean' & nmean=4*n+2
;
parfile=modeldisk + model + ModelIdent + '.par'
;
; --- Read parameter, EOS, and opacity file ---
par2=uio_struct_rd(parfile)
delvar, EOS
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par2.eosfile, EOS &$
  dfopta, opadisk + par2.opafile &$
endif
f0=!con.sigmasb*par2.Teff^4
;
if (n_elements(ful2) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful2=uio_dataset_rd(modelfile, ndataset=n) &$
  Lx1=max(ful2.z.xb1)-min(ful2.z.xb1) &$
  Lx3=max(ful2.z.xb3)-min(ful2.z.xb3) &$
  nx1=n_elements(ful2.z.xc1) &$
  ;nx1=240L &$
  ;nx1=250L &$
  ;;nx1=300L &$
  nx3=round(Lx3/Lx1*nx1) &$
  ;ful2=refinebox('0', ful2, nx1, nx1, nx3) &$
  eosbox, ful2, eos=EOS, opa=1, out_q=0, ierror=ierror & help,ierror &$
endif

print, ful2.z.time

n1=n_elements(ful2.z.xc1)
n2=n_elements(ful2.z.xc2)
n3=n_elements(ful2.z.xc3)

c2=combox('bc3', ful2)/(combox('bc',ful2)>1e-9)
d2=(combox('Pmag', ful2))^2/combox('P',ful2)

;d2=(combox('bc', ful2))^2/combox('rho',ful2)^(1.0/1.0)
;c2=combox('bc2', ful1)/(combox('bc',ful1)>1e-9)
;d2=combox('bc', ful1)^2
;c2=combox('bc2', ful2)/(combox('bc',ful2)>1e-9)
;d2=combox('bc', ful2)^2
;;c1=combox('bc3', ful)
;d1=rebin(reform(c1, n1,n2,n3,1), n1,n2,n3,3)
;d1[*,*,*,0]=combox('bc1', ful)
;d1[*,*,*,1]=combox('bc2', ful)
;d1[*,*,*,2]=combox('bc3', ful)

;c2=combox('helv', ful2)
;d2=(abs(c2))
;c2=(combox('v3',ful2)/(combox('v', ful2)>1.0))
;d2=(combox('v', ful2))^2
;
;M=[[     0.75282,     0.65809,    -0.01293,   -59.28635],$
;   [    -0.29550,     0.35545,     0.88675,   106.13383],$
;   [     0.58817,    -0.66375,     0.46206,    26.78621],$
;   [     0.00000,     0.00000,     0.00000,     1.00000]]
;M=[[     0.70575,     0.70844,     0.00402,   -45.49843],$
;   [    -0.18668,     0.18049,     0.96569,    75.41313],$
;   [     0.68341,    -0.68229,     0.25963,    26.53138],$
;   [     0.00000,     0.00000,     0.00000,     1.00000]]
M=[[     0.77192,     0.63552,     0.01585,   -51.38434],$
   [    -0.18124,     0.19610,     0.96369,    82.65643],$
   [     0.60934,    -0.74676,     0.26656,    47.54708],$
   [     0.00000,     0.00000,     0.00000,     1.00000]]

;save, d1, c1, file='d1c1.sav'
;save, d1, c1, d2, c2, file='d1c1d2c2.sav'

; ---
vis3d, d1,c1, d2, c2, $
  IDLPAL=[63,63], viewcolor=[0,0,0], no_colorbar=1, $
  scale=1.3,xsize=900, ysize=600, decimation=[100]

; --- Magnetic field ---
;vis3d, d1,c1, d2,c2, $
;  IDLPAL=[63,0], viewcolor=[0,0,0], $
;  scale=1.3,xsize=800, ysize=600, decimation=[100]

; ---
;vis3d, d1,c1, d2,c2, $
;;  viewcolor=[0,0,0], IDLPAL=[64,64], paletterange=[[-1.3e-6,1.3e-6],[-0.12,0.12]], $
;  viewcolor=[0,0,0], IDLPAL=[64,64], paletterange=[[-1e5,1e5],[-0.12,0.12]], $
;  xsize=600, ysize=400

; --- Vol: Scale: 0.7; HiZxyx: 00111, Animation: angle: 45, cos, 400 frames
;vis3d, d1,c1, $
;  IDLPAL=[3], viewcolor=[0,0,0], lights=[0], rotatematrix=M, $
;  /no_colorbar, scale=1.7,xsize=900, ysize=486, decimation=[100]

; --- Vol: Scale: 0.7; HiZxyx: 00111, Animation: angle: 45, cos, 400 frames
;vis3d, d1,c1, d2,c1, $
;  IDLPAL=[3,5], viewcolor=[0,0,0], lights=[0], rotatematrix=M, $
;  /no_colorbar, scale=1.4,xsize=680, ysize=600, decimation=[100,100]

; --- Vol: Scale: 0.025
;vis3d, d1,c1, isovalue=[5.0], $
;  IDLPAL=[3], viewcolor=[0,0,0], paletterange=[0.0,33.0], lights=[0], $
;  /no_colorbar, scale=1.4,xsize=800, ysize=600, decimation=[10]

;vis3d, c1,d1, c2,d1, isovalue=[1.0, 1.0], $
;  IDLPAL=[3,3], viewcolor=[0,0,128], lights=[0,0], $
;  /no_colorbar, scale=1.4,xsize=450, ysize=450, decimation=[10,10]
