; pro rhd_v3d_compare3
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_v3d_compare3 ('radiation_hydrodynamics_visualize_3_dimension_compare_3')
;
; PURPOSE:
;   Load model data for single time step, solve EOS, compute opacity.
;
; CATEGORY:
;   RHD, visualization
;
; CALLING SEQUENCE:
;   @rhd_v3d
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos, par, ful, mean
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    4. 5.02 Written by Bernd Freytag, Astronomical Observatory Uppsala
;-
;------------------------------------------------------------------------------
;
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
; === Model 0 ===
;model='st35gm04n30' & modelident='_01a' & n=0         ; (n_org=8)
model='st35gm04n30' & modelident='_01' & n=8
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
modelfile=modeldisk + model + modelident + '.full' & n=n
parfile=modeldisk + model + modelident + '.par'
;
; --- Read parameter, EOS, and opacity file ---
if (n_elements(par0) eq 0) then par0=uio_struct_rd(parfile)
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par0.eosfile, EOS &$
  dfopta, opadisk + par0.opafile &$
endif
f0=!con.sigmasb*par0.Teff^4
;
if (n_elements(ful0) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful0=uio_dataset_rd(modelfile, n=n) &$
  ful0=refinebox('central', ful0, 171,171,171) &$
  eosbox, ful0, eos=EOS, opa=1, ierror=ierror & help,ierror &$
endif
;
print, ful0.z.time
;
; === Model 1 ===
model='st35gm04n26' & modelident='_57' & n=3
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
modelfile=modeldisk + model + modelident + '.full' & n=n
parfile=modeldisk + model + modelident + '.par'
;
; --- Read parameter, EOS, and opacity file ---
if (n_elements(par1) eq 0) then par1=uio_struct_rd(parfile)
;
if (n_elements(ful1) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful1=uio_dataset_rd(modelfile, n=n) &$
  eosbox, ful1, eos=EOS, opa=1, ierror=ierror & help,ierror &$
endif
;
print, ful1.z.time
;
; === Model 2 ===
;model='st35gm04n29' & modelident='_03a' & n=0   ; (n_org=2)
model='st35gm04n29' & modelident='_03' & n=2
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
modelfile=modeldisk + model + modelident + '.full' & n=n
parfile=modeldisk + model + modelident + '.par'
;
; --- Read parameter, EOS, and opacity file ---
if (n_elements(par2) eq 0) then par2=uio_struct_rd(parfile)
;
if (n_elements(ful2) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful2=uio_dataset_rd(modelfile, n=n) &$
  ful2=refinebox('central', ful2, 171,171,171) &$
  eosbox, ful2, eos=EOS, opa=1, ierror=ierror & help,ierror &$
endif
;
print, ful2.z.time
;
;c0=!con.sigmasb*combox('T', ful0)^4 / f0
;d0=combox('tauharm', ful0)
;c1=!con.sigmasb*combox('T', ful1)^4 / f0
;d1=combox('tauharm', ful1)
;
c0=combox('radius' , ful0)/(max(ful0.z.xc3))
m =combox('tauharm', ful0)
d0=combox('s'      , ful0) * (m gt 1.0)
d0(0:84,*,*)=0.0
c1=combox('radius' , ful1)/(max(ful1.z.xc3))
m =combox('tauharm', ful1)
d1=combox('s'      , ful1) * (m gt 1.0)
d1(0:84,*,*)=0.0
c2=combox('radius' , ful2)/(max(ful2.z.xc3))
m =combox('tauharm', ful2)
d2=combox('s'      , ful2) * (m gt 1.0)
d2(0:84,*,*)=0.0
;
;vis3d, d1,c1, d2,c2, d3,c3, isovalue=[1.0, 0.4, 2.8e9], $
;  IDLPAL=[3,3,0], viewcolor=[0,150,150], lights=[0,0,1], $
;  /no_colorbar, scale=2.2,xsize=400, ysize=400
;
;vis3d, d0,c0, d1,c1, isovalue=[1.0, 1.0], $
;  IDLPAL=[3,3], viewcolor=[0,150,150], smooth_on=0, $
;  /no_colorbar, xsize=400, ysize=400
;
vis3d, d0,c0, d1,c1, d2,c2, isovalue=[1.0,1.0,1.0]*3.22E+09, $
  IDLPAL=3, viewcolor=[0,100,100], lights=1, smooth_on=0, $
  /no_colorbar, scale=2.0,xsize=400, ysize=400
