; pro rhd_v3d_local
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_v3d_local ('radiation_hydrodynamics_visualize_3_dimension_local')
;
; PURPOSE:
;   Load model data for single time step, solve EOS, compute opacity.
;
; CATEGORY:
;   RHD, visualization
;
; CALLING SEQUENCE:
;   @rhd_v3d_local
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos, ful
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    2002-05-04 Written by Bernd Freytag, Astronomical Observatory Uppsala
;    2006-09-19 (B.F. Lyon) Version for local models
;-
;------------------------------------------------------------------------------
;
ModelIdent=''
;model='mt15g50mm00n06' & ModelIdent='_047'
;model='mt15g50mm00n06' & ModelIdent='_067'
;model='mt15g50mm00n10' & ModelIdent='_035'
;model='mt24g50mm00n03' & ModelIdent='_100'
;model='mt28g50mm00n04' & ModelIdent='_046'
;model='mt33g50mm00n07' & ModelIdent='_200'
;model='mt40g50mm00n04' & ModelIdent='_030'
;model='at80g40n22' & ModelIdent='_07'
;model='job3Dh50' & ModelIdent='_030'
;model='job3Dtest1' & ModelIdent='_060'
;model='jobtest20100326' & ModelIdent='_01'
;model='d3gt57g44b0n001' & ModelIdent='_044'
;model='d3t57g44b0n002' & ModelIdent='_041'
;model='d3t57g44b0n006' & ModelIdent='_090'
;model='d3t57g44b0n008' & ModelIdent='_019'
model='d3t57g44b0n025' & ModelIdent='_015'
;model='d3t57g44b0n011' & ModelIdent='_013'
;model='d3t57g44b0n013' & ModelIdent='_007'
;model='d3t57g44b0n014' & ModelIdent='_005'
;model='d3t57g44b1n002' & modelident='_089'
;model='d3t57g44b1n003' & modelident='_052'
;model='d3t57g44b1n004' & modelident='_040'
;model='d3t57g44b1n005' & modelident='_030'
;model='d3t57g44b1n006' & modelident='_049'
;model='d3t57g44b1n007' & modelident='_061'
;model='d3t57g44b1n008' & modelident='_006'
;model='d3t57g44b1n013' & modelident='_075'
;model='d3t57g44b1n021' & modelident='_009'
;model='sunmhd3D100' & modelident='_n018'
;model='sunmhd3D101' & modelident='_n007'
;model='rhd' & ModelIdent=''
;
;modeldisk='/u64/bf' + '/dat/rhd/d' + model + '/'
modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;modeldisk='/home/bf/idl/wrk/'
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;
modelfile=modeldisk + model + ModelIdent + '.full' & n=0
;modelfile=modeldisk + model + ModelIdent + '.sta' & n=0
;modelfile=modeldisk + model + ModelIdent + '.end' & n=0
;meanfile=modeldisk + model + ModelIdent + '.mean' & nmean=4*n+2
;
parfile=modeldisk + model + ModelIdent + '.par'
;
; --- Read parameter, EOS, and opacity file ---
par=uio_struct_rd(parfile)
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
  dfopta, opadisk + par.opafile &$
endif
f0=!con.sigmasb*par.Teff^4
;
if (n_elements(ful) eq 0) then begin &$
  print, 'Loading ''' + modelfile + '''' &$
  ful=uio_dataset_rd(modelfile, ndataset=n, /HeadRead) &$
  ful=chunkbox(ful, [[400,599], [600,799], [150,349]], /ZeroStart) &$
  Lx1=max(ful.z.xb1)-min(ful.z.xb1) &$
  Lx3=max(ful.z.xb3)-min(ful.z.xb3) &$
  nx1=n_elements(ful.z.xc1) &$
  ;nx1=240L &$
  ;nx1=250L &$
  ;;nx1=300L &$
  nx3=round(Lx3/Lx1*nx1) &$
  ;ful=refinebox('0', ful, nx1, nx1, nx3) &$
  eosbox, ful, eos=EOS, opa=1, out_q=0, ierror=ierror & help,ierror &$
endif

;stop

print, ful.z.time

n1=n_elements(ful.z.xc1)
n2=n_elements(ful.z.xc2)
n3=n_elements(ful.z.xc3)

;c1=combox('bc3', ful)/(combox('bc',ful)>1e-9)
;;c1=combox('bc3', ful)
;d1=rebin(reform(c1, n1,n2,n3,1), n1,n2,n3,3)
;d1[*,*,*,0]=combox('bc1', ful)
;d1[*,*,*,1]=combox('bc2', ful)
;d1[*,*,*,2]=combox('bc3', ful)

;d1=(combox('bc', ful))^2
;d1=combox('helv', ful)
;c1=combox('helv', ful)
;d1=(abs(c1))^0.5
;c1=c1/((combox('v', ful)^2)>1.0)
;d1=(combox('v', ful))^2
;d1=-((combox('v3', ful))<0.0)
;c1=combox('v3', ful)*1e-5

;c1=combox('xc3a', ful)
;c1=c1-min(c1) & c1=c1/max(c1) & c1=1.0-c1
;c1=combox('T', ful)
;d1=alog10(combox('P', ful))
;;d1=combox('quc002', ful)
;d1=combox('quc002orho', ful)
;d1=d1/max(d1)
;c1=(combox('s', ful)-par.s_inflow) * 1.0E-09
;c1=combox('s', ful, /meansub) * 1.0E-09

;d1=combox('rho', ful)+5.0*combox('rho', ful, /meansub)
;d1=(combox('T', ful)+40.0*combox('T', ful, /meansub)) > 0.0
;d1=combox('T', ful, /meansub) * (combox('T', ful) gt 2000.0)
;d1=combox('T', ful, /meansub) * (combox('s', ful) lt (par.s_inflow+1.0e-7))
;d1=combox('rho', ful, /meansub) >0.0

;d2=combox('s', ful)-par.s_inflow
;d2=d2*(combox('s', ful) lt (par.s_inflow+1.0e-7))
;d2=d2/max(abs(d2))
;d2=-(d2<0.0)
;d2=d2/max(abs(d2))

;d1=d1+1.0
;d1=4.0+alog10(d1 > 1.0E-04)
;d1=8.0+alog10(d1 > 1.0E-8)
;d1=4.0*d1
;d1=d1 mod 1.0
;c2=combox('s', ful, /meansub)
;c2=c2/max(abs(c2))
;d2=combox('dtau3', ful)
;ind=where(c2 gt 0.0)
;d2[ind]=0.1*d2[ind]

;c2=combox('helv', ful)
;d2=(abs(c2))^2
;d2=(abs(combox('helv', ful)))
;d2=(abs(combox('helv', ful)))^2
d2=(combox('absrotv', ful))^2
;d2=combox('T', ful)/1000.0
;c2=alog10(combox('rho', ful))
;c2=alog10(combox('rho', ful)) & d2=d2-min(d2)
;d2=combox('rho', ful, /meansub) & d2=d2>0.0
;c2=combox('xc3a', ful)

c1=(combox('T', ful)/1000.0)^4
;d1=-combox('v3', ful)*1.0E-05
c2=combox('v3', ful)*1.0E-05
;c1=alog10(combox('p', ful))

d1=combox('tau_int', ful, par=par)
;d1(where(d1 gt 1.5))=0.0
;d1(where(d1 lt 0.01))=0.0
;c2=combox('dtau3', ful)
;d2=alog10(combox('p', ful))

;c3=combox('qucallm1', ful)*1.0E11
;c3=combox('qucallm1orho', ful)*1e4
;d3=c3


;M=[[     0.75282,     0.65809,    -0.01293,   -59.28635],$
;   [    -0.29550,     0.35545,     0.88675,   106.13383],$
;   [     0.58817,    -0.66375,     0.46206,    26.78621],$
;   [     0.00000,     0.00000,     0.00000,     1.00000]]
;M=[[     0.70575,     0.70844,     0.00402,   -45.49843],$
;   [    -0.18668,     0.18049,     0.96569,    75.41313],$
;   [     0.68341,    -0.68229,     0.25963,    26.53138],$
;   [     0.00000,     0.00000,     0.00000,     1.00000]]
M=[[     0.77192,     0.63552,     0.01585,   -51.38434],$
   [    -0.18124,     0.19610,     0.96369,    82.65643],$
   [     0.60934,    -0.74676,     0.26656,    47.54708],$
   [     0.00000,     0.00000,     0.00000,     1.00000]]

;save, d1, c1, file='d1c1.sav'
;save, d1, c1, d2, c2, file='d1c1d2c2.sav'

; ---
;vis3d, d1,c1, $
;  IDLPAL=[63], viewcolor=[0,0,0], lights=[0], $
;  scale=1.4,xsize=900, ysize=600, decimation=[100]

;vis3d, d1,c1, d2,c2, $
;  IDLPAL=[0,63], viewcolor=[0,0,0], no_colorbar=0, $
;  scale=1.3,xsize=1000, ysize=700, decimation=[100]


;restore, 'vis3d_object_d3t57g44b0n25_T4tau1.sav'
restore, 'vis3d_object_d3t57g44b0n25_T4vol.sav'
vis3d, d1,c1, d2,c2, oUser2=oModel, $
  IDLPAL=[0,63], viewcolor=[0,0,0], no_colorbar=0, $
  scale=1.3, xsize=1000, ysize=800, decimation=[100]

; --- Entropy and vertical velocity ---
;vis3d, d1,c1, d2,c2, $
;  IDLPAL=[8,64], viewcolor=[0,0,0], no_colorbar=0, $
;  scale=1.3,xsize=1000, ysize=700, decimation=[100]

; --- Entropy and vertical velocity and dust ---
;vis3d, d1,c1, d2,c2, d3,c3, $
;  IDLPAL=[8,64,3], viewcolor=[0,0,0], no_colorbar=1, $
;  scale=1.3,xsize=800, ysize=700, decimation=[100]

; ---
;vis3d, d1,c1, d2,c2, $
;;  viewcolor=[0,0,0], IDLPAL=[64,64], paletterange=[[-1.3e-6,1.3e-6],[-0.12,0.12]], $
;  viewcolor=[0,0,0], IDLPAL=[64,64], paletterange=[[-1e5,1e5],[-0.12,0.12]], $
;  xsize=600, ysize=400

; --- Vol: Scale: 0.7; HiZxyx: 00111, Animation: angle: 45, cos, 400 frames
;vis3d, d1,c1, $
;  IDLPAL=[3], viewcolor=[0,0,0], lights=[0], rotatematrix=M, $
;  /no_colorbar, scale=1.7,xsize=900, ysize=486, decimation=[100]

; --- Vol: Scale: 0.7; HiZxyx: 00111, Animation: angle: 45, cos, 400 frames
;vis3d, d1,c1, d2,c1, $
;  IDLPAL=[3,5], viewcolor=[0,0,0], lights=[0], rotatematrix=M, $
;  /no_colorbar, scale=1.4,xsize=680, ysize=600, decimation=[100,100]

; --- Vol: Scale: 0.025
;vis3d, d1,c1, isovalue=[5.0], $
;  IDLPAL=[3], viewcolor=[0,0,0], paletterange=[0.0,33.0], lights=[0], $
;  /no_colorbar, scale=1.4,xsize=800, ysize=600, decimation=[10]

;vis3d, c1,d1, c2,d1, isovalue=[1.0, 1.0], $
;  IDLPAL=[3,3], viewcolor=[0,0,128], lights=[0,0], $
;  /no_colorbar, scale=1.4,xsize=450, ysize=450, decimation=[10,10]
