; pro rhd_workintegral_plot
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   rhd_workintegral_plot ('rhd_work_integral_plot')
;
; PURPOSE:
;   Compute and plot work integral from RHD data.
;
; CATEGORY:
;   RHD data analysis
;
; CALLING SEQUENCE:
;   .run rhd_workintegral_plot
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, seq
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2016-10-26 (B.F. Uppsala) First version
;-
;---------------------------------------------------------------------------------------------------

;model='d3t57g44b0n026'
;model='d3t57g44b0n027'
;model='d3t57g44b0n030'
;model='d3t57g44b0n031'
;model='d3t57g44b0n032'
;model='d3t57g44b0n033'

;model='gt50g20n01'
;model='gt51g20n01'
;model='gt51g20n02'
;model='gt52g20n01'
;model='gt52g20n02'
;model='gt52g20n03'
;model='gt53g20n01'
;model='gt53g20n02'
;model='gt54g20n01'
;model='gt54g20n02'
;model='gt54g20n03'
;model='gt54g20n04'
;model='gt54g20n05'
;model='gt55g20n01'
;model='gt56g20n01'
;model='gt56g20n02'
;model='gt56g20n03'
;model='gt56g20n04'
model='gt58g20n01'
;model='gt58g20n02'
;model='gt58g20n03'
;model='gt58g20n04'
;model='gt58g20n05'
;model='gt60g20n01'
;model='gt60g20n02'
;model='gt60g20n03'
;model='gt60g20n04'
;model='gt62g20n01'
;model='gt62g20n02'

sav_id=''


; --- Load data if necessary ---
if (n_elements(seq) eq 0) then begin
  LoadNewFile_flag=1
endif else begin
  LoadNewFile_flag=(seq.Model ne Model)
endelse
;
if (LoadNewFile_flag) then begin
  file=Model + '_seq' + sav_id + '.sav'
  print, 'Restore data from ''', file, ''''
  restore, file
endif else begin
  print, 'Reuse data for model ', seq.Model
endelse

; --- Load table with model data ---
;.run rhdmodeldata
rhdmodeldata, modDB

; --- Extract/set averaging interval ---
;mtime=0L
;ntime=99999L
;mtime=600L
;ntime=800L
;mtime=8003L
;ntime=8792L
;mtime=8791L
;ntime=9507L
;mtime=409L
;ntime=807L
; --- model='gt58g20n03'
;  mtime=1020L
;  ntime=2010L
;  mtime=996L
;  ntime=1988L
mtime=(udb_extract('mMeanAvg', [Model], modDB, default=0L    ))[0]
ntime=(udb_extract('nMeanAvg', [Model], modDB, default=99999L))[0]
help, Model, mtime, ntime

; --- Extract some quantities from data structure ---
time =seq.time[(0L > (mtime < (n_elements(seq.time)-1L))):(0L > (ntime < (n_elements(seq.time)-1L)))]
xc3    =seqavg('xc3'    , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
;
dxb3a  =seqavg('dxb3a'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
grava  =seqavg('grava'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
;
v3m    =seqavg('v3'     , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhov3  =seqavg('rhov3'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhov3v3=seqavg('rhov3v3', seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rho    =seqavg('rho'    , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
rhoek  =seqavg('rhoek'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
P      =seqavg('P'      , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
;
frhob  =seqavg('rhovb'  , seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
frhov3b=seqavg('frhov3b', seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs')
frhov3c=seqavg('frhov3b', seq, mtime=mtime, ntime=ntime, qavg=-1, unitsystem='cgs', uniteflux='cgs', /intertoxc)
;
if ((min(rhov3v3) eq max(rhov3v3)) and (min(frhov3c) ne max(frhov3c))) then begin
  print, 'Compute rhov3v3 from frhov3c'
  rhov3v3=frhov3c-P
endif

; --- Print information about some arrays ---
print, 'rhov3:  ', size(rhov3  , /dimensions), minmax(rhov3  ), format='(A10,2X,2I6,2X,2E14.6)'
print, 'P:      ', size(P      , /dimensions), minmax(P      ), format='(A10,2X,2I6,2X,2E14.6)'
print, 'rhov3v3:', size(rhov3v3, /dimensions), minmax(rhov3v3), format='(A10,2X,2I6,2X,2E14.6)'
print, 'frhov3c:', size(frhov3c, /dimensions), minmax(frhov3c), format='(A10,2X,2I6,2X,2E14.6)'
print, 'frhov3b:', size(frhov3b, /dimensions), minmax(frhov3b), format='(A10,2X,2I6,2X,2E14.6)'

; --- Compute further quantities ---
rhoek_Mode  = 0.5*rhov3^2/rho
rhoek3      = 0.5*rhov3v3
v3          = rhov3/rho
v3b         = fltarr(n_elements(xc3)+1, n_elements(time))
v3b[1:-2,*] = 0.5*(v3[0:-2,*]+v3[1:-1,*])
Pb          = fltarr(n_elements(xc3)+1, n_elements(time))
Pb[1:-2,*]  = 0.5*(P[0:-2,*]+P[1:-1,*])
Pb[ 0,*]    = P[1 ,*]
Pb[-1,*]    = P[-2,*]

dv3         = (v3b[1:-1,*]-v3b[0:-2,*])/dxb3a


; --- Compute work-integral contributions ---
W_v3drhov3v3pP = -v3 * (frhov3b[1:-1,*]-frhov3b[0:-2,*])/dxb3a
W_rhov3grav    = -rhov3 * grava
W_v3v3drhov3   = 0.5*v3^2 * (frhob[1:-1,*]-frhob[0:-2,*])/dxb3a

W_dv3rhov3v3pP = -((v3b*frhov3b)[1:-1,*]-(v3b*frhov3b)[0:-2,*])/dxb3a
W_rhov3v3pPdv3 = 0.5*(frhov3b[1:-1,*]+frhov3b[0:-2,*]) * (v3b[1:-1,*]-v3b[0:-2,*])/dxb3a

W_v3dP         = -v3 * (Pb[1:-1,*]-Pb[0:-2,*])/dxb3a
W_Pdv3         = P * (v3b[1:-1,*]-v3b[0:-2,*])/dxb3a

W_v3drhov3v3   = -v3 * ((frhov3b-Pb)[1:-1,*]-(frhov3b-Pb)[0:-2,*])/dxb3a
W_rhov3v3dv3   = 0.5*((frhov3b-Pb)[1:-1,*]+(frhov3b-Pb)[0:-2,*]) * (v3b[1:-1,*]-v3b[0:-2,*])/dxb3a

W_Tot1         = W_v3drhov3v3pP + W_rhov3grav + W_v3v3drhov3

; --- Average over time ---
rho_Avgt      =mean(rho       , dim=2)
rhoek_Avgt    =mean(rhoek     , dim=2)
rhoek3_Avgt   =mean(rhoek3    , dim=2)
rhoek_ModeAvgt=mean(rhoek_Mode, dim=2)

; --- Average over height ---
rhoek_Avg3    =mean(rhoek     *dxb3a, dim=1)
rhoek3_Avg3   =mean(rhoek3    *dxb3a, dim=1)
rhoek_ModeAvg3=mean(rhoek_Mode*dxb3a, dim=1)

; --- Compute further quantities from averages ---
vrhoek_Avgt    =sqrt(2.0*rhoek_Avgt/rho_Avgt)
vrhoek3_Avgt   =sqrt(2.0*rhoek3_Avgt/rho_Avgt)
vrhoek_ModeAvgt=sqrt(2.0*rhoek_ModeAvgt/rho_Avgt)


; ======================================

!P.MULTI=[0,3,3,0,0]
!P.CHARSIZE=3.2

; --- Plot 1 ---------------------------

plotlines, time, rhoek_Avg3, $
  drange=[0, max(rhoek_Avg3)]

plotlines, time, rhoek3_Avg3, /over, $
  linecol=!ctv.green

plotlines, time, rhoek_ModeAvg3, /over, $
  linecol=!ctv.red


;plotlines, rhoek_Avg3, $
;  drange=[0, max(rhoek_Avg3)]

;plotlines, rhoek3_Avg3, /over, $
;  linecol=!ctv.green

;plotlines, rhoek_ModeAvg3, /over, $
;  linecol=!ctv.red


; --- Plot 2 ---------------------------

plotlines, xc3, rhoek_Avgt, $
  drange=[0, max(rhoek_Avgt)]

plotlines, xc3, rhoek3_Avgt, /over, $
  linecol=!ctv.green

plotlines, xc3, rhoek_ModeAvgt, /over, $
  linecol=!ctv.red


; --- Plot 3 ---------------------------

plotlines, xc3, vrhoek_Avgt, $
  drange=[0, max(vrhoek_Avgt)]

plotlines, xc3, vrhoek3_Avgt, /over, $
  linecol=!ctv.green

plotlines, xc3, vrhoek_ModeAvgt, /over, $
  linecol=!ctv.red


; --- Plot 4 ---------------------------

W_Tot1Avg=mean(W_Tot1, dim=2)

drange=[min(W_Tot1Avg), max(W_Tot1Avg)]

plotlines, xc3, W_tot1Avg, $
  drange=drange

plotlines, xc3, smooth(W_tot1Avg, 13), /over, $
  linecol=!ctv.yellow, thick=3


; --- Plot 5 ---------------------------

W_Tot1Avg=mean(W_Tot1, dim=2)

drange=[min(W_Tot1Avg), max(W_Tot1Avg)]

plotlines, xc3, W_tot1Avg, $
  drange=drange

plotlines, xc3, mean(W_v3drhov3v3pP, dim=2), /over, $
  linecol=!ctv.red

plotlines, xc3, mean(W_rhov3grav, dim=2), /over, $
  linecol=!ctv.green

plotlines, xc3, mean(W_v3v3drhov3, dim=2), /over, $
  linecol=!ctv.yellow


; --- Plot 6 ---------------------------

drange=[min(mean(W_v3drhov3v3pP, dim=2)) < min(mean(W_dv3rhov3v3pP, dim=2)) < min(mean(W_rhov3v3pPdv3, dim=2)), $
        max(mean(W_v3drhov3v3pP, dim=2)) > max(mean(W_dv3rhov3v3pP, dim=2)) > max(mean(W_rhov3v3pPdv3, dim=2))   ]

plotlines, xc3, mean(W_v3drhov3v3pP, dim=2), $
  drange=drange

plotlines, xc3, mean(W_dv3rhov3v3pP, dim=2), /over, $
  linecol=!ctv.red

plotlines, xc3, mean(W_rhov3v3pPdv3, dim=2), /over, $
  linecol=!ctv.green

;plotlines, xc3, mean(W_dv3rhov3v3pP + W_rhov3v3pPdv3, dim=2), /over, $
;  linecol=!ctv.blue


; --- Plot 7 ---------------------------

;drange=[min(mean(W_Pdv3, dim=2)), max(mean(W_Pdv3, dim=2))]
drange=[min(mean(W_v3dP, dim=2)), max(mean(W_v3dP, dim=2))]

plotlines, xc3, mean(W_v3dP, dim=2), $
  drange=drange

plotlines, xc3, mean(W_Pdv3, dim=2), /over, $
  linecol=!ctv.blue


; --- Plot 8 ---------------------------


plotlines, xc3, mean(W_v3drhov3v3, dim=2), $
  drange=drange

plotlines, xc3, mean(W_rhov3v3dv3, dim=2), /over, $
  linecol=!ctv.blue


; --- ------ ---------------------------

!P.MULTI=0

end ; rhd_workintegral_plot
