; pro rhd_z_collect
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_z_collect ('radiation_hydrodynamics_z_collect')
;
; PURPOSE:
;   Collect data structures from rhd files into one array of structures.
;
; CATEGORY;
;   RHD, data handling
;
; CALLING SEQUENCE:
;   @rhd_z_collect
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   @rhd_z_collect
;   v1=reform(mean(fula[*].rhov1_xmean,dim=4)/mean(fula[*].rho_xmean,dim=4))
;   v2=reform(mean(fula[*].rhov2_xmean,dim=4)/mean(fula[*].rho_xmean,dim=4))
;   v3=reform(mean(fula[*].rhov3_xmean,dim=4)/mean(fula[*].rho_xmean,dim=4))
;   plotfield, /tv, v2, drange=[-1,1]*5e5, xvec=v1, yvec=v3,/str, s_colu=30, s_row=60
;
; MODIFICATION HISTORY:
;   2014-06-12 (B.F. Uppsala) Adapt from rhd_seq_collect.pro
;-
;------------------------------------------------------------------------------
;
;ModelIdent=''
n_timestep_adjust=0
;BinFactor=20L

;model='mt16g50mm00n08' & ModelIdent='_{09?,10?}' & parModelIdent='_100'
;model='mt17g50mm00n03' & ModelIdent='_1??' & parModelIdent='_110'

;Model='st22g35n01' & ModelIdent='_???' & parModelIdent='_016'
;Model='st22g35n02' & ModelIdent='_???' & parModelIdent='_042'
;Model='st22g35n03' & ModelIdent='_???' & parModelIdent='_009'
;Model='st22g35n04' & ModelIdent='_???' & parModelIdent='_030'
;Model='st22g35n05' & ModelIdent='_07?' & parModelIdent='_080'
;Model='st22g35n06' & ModelIdent='_???' & parModelIdent='_070'
;Model='st22g35n07' & ModelIdent='_1[0123]?' & parModelIdent='_120' ; '_170'
;Model='st22g35n08' & ModelIdent='_???' & parModelIdent='_019'
;Model='st22g40n01' & ModelIdent='_???' & parModelIdent='_014'

;Model='st26g35n01' & ModelIdent='_???' & parModelIdent='_011'
;Model='st26g35n02' & ModelIdent='_???' & parModelIdent='_400'
;Model='st26g35n03' & ModelIdent='_???' & parModelIdent='_200'
;Model='st26g35n04' & ModelIdent='_???' & parModelIdent='_060'
;Model='st26g35n05' & ModelIdent='_???' & parModelIdent='_003'
;Model='st26g35n06' & ModelIdent='_???' & parModelIdent='_020'
;Model='st26g35n07' & ModelIdent='_???' & parModelIdent='_070'
;Model='st26g35n08' & ModelIdent='_???' & parModelIdent='_070'
;Model='st26g35n09' & ModelIdent='_???' & parModelIdent='_070'
;Model='st26g35n10' & ModelIdent='_???' & parModelIdent='_020'
;Model='st26gm09n01' & ModelIdent='_??' & parModelIdent='_03'
;Model='st26gm09n02' & ModelIdent='_??' & parModelIdent='_07' & n_timestep_adjust=-1
;Model='st28gm06n01' & ModelIdent='_??' & parModelIdent='_11'
;Model='st28gm06n02' & ModelIdent='_??' & parModelIdent='_29'
;Model='st28gm06n03' & ModelIdent='_??' & parModelIdent='_17'
;Model='st28gm06n05' & ModelIdent='_??' & parModelIdent='_24'
;Model='st28gm06n06' & ModelIdent='_??' & parModelIdent='_23'
;Model='st28gm06n07' & ModelIdent='_??' & parModelIdent='_07'
;Model='st28gm06n09e' & ModelIdent='_??' & parModelIdent='_28' & n_timestep_adjust=-8
;Model='st28gm06n09t4' & ModelIdent='_??' & parModelIdent='_55' & n_timestep_adjust=0
;Model='st28gm06n10d' & ModelIdent='_??' & parModelIdent='_02'
;Model='st28gm06n11' & ModelIdent='_??' & parModelIdent='_01'
;Model='st28gm06n12' & ModelIdent='_???' & parModelIdent='_003'
;Model='st28gm06n13' & ModelIdent='_???' & parModelIdent='_120'
;Model='st28gm06n14' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n15' & ModelIdent='_???' & parModelIdent='_130'
;Model='st28gm06n16' & ModelIdent='_???' & parModelIdent='_040' & n_timestep_adjust=-40
;Model='st28gm06n17' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n18' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n19' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n20' & ModelIdent='_???' & parModelIdent='_080'
;Model='st28gm06n21' & ModelIdent='_???' & parModelIdent='_120'
;Model='st28gm06n22' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n23' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n24' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n25' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n26' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n29' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n30' & ModelIdent='_???' & parModelIdent='_100'
;Model='st28gm06n29' & ModelIdent='_{0[3-9]?,100}' & parModelIdent='_100'
;;Model='st28gm06n30' & ModelIdent='_{0[3-9]?,100}' & parModelIdent='_100'
Model='st28gm06n038' & ModelIdent='_11[0-7]' & parModelIdent='_200'

;Model='st33gm06n02' & ModelIdent='_??' & parModelIdent='_21'
;old;Model='st33gm06n03' & ModelIdent='_00-60' & parModelIdent='_01'
;Model='st33gm06n03' & ModelIdent='_??' & parModelIdent='_60'

;old;Model='st35gm06n01' & ModelIdent='_02-03' & parModelIdent='_02'
;Model='st35gm06n01' & ModelIdent='_??' & parModelIdent='_03'
;Model='st35gm06n02' & ModelIdent='_??' & parModelIdent='_20'

;Model='st35gm04n01' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=228
;Model='st35gm04n02' & ModelIdent='_??' & parModelIdent='_27' & n_timestep=415
;Model='st35gm04n03' & ModelIdent='_??' & parModelIdent='_26' & n_timestep=518
;Model='st35gm04n04' & ModelIdent='_??' & parModelIdent='_38' & n_timestep=648 ; dtime=0.193298E+09
;Model='st35gm04n05' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=307
;Model='st35gm04n06' & ModelIdent='_??' & parModelIdent='_15' & n_timestep=384
;Model='st35gm04n07' & ModelIdent='_??' & parModelIdent='_50' & n_timestep=381
;Model='st35gm04n08' & ModelIdent='_??' & parModelIdent='_04' & n_timestep_adjust=-2
;Model='st35gm04n20' & ModelIdent='_??' & parModelIdent='_04'
;Model='st35gm04n21' & ModelIdent='_??' & parModelIdent='_13'
;Model='st35gm04n22' & ModelIdent='_??' & parModelIdent='_42' & n_timestep_adjust=-2
;old;Model='st35gm04n23' & ModelIdent='_0[1-7]' & parModelIdent='_07'
;Model='st35gm04n23' & ModelIdent='_??' & parModelIdent='_07'
;Model='st35gm04n24' & ModelIdent='_??' & parModelIdent='_13'
;Model='st35gm04n25' & ModelIdent='_??' & parModelIdent='_27'
;Model='st35gm04n26' & ModelIdent='_??' & parModelIdent='_75'
;Model='st35gm04n27' & ModelIdent='_??' & parModelIdent='_08'
;Model='st35gm04n28' & ModelIdent='_??' & parModelIdent='_55' & n_timestep_adjust=-15
;Model='st35gm04n29' & ModelIdent='_??' & parModelIdent='_30'
;Model='st35gm04n30' & ModelIdent='_??' & parModelIdent='_20'
;Model='st35gm04n31' & ModelIdent='_??' & parModelIdent='_40'
;Model='st35gm04n32' & ModelIdent='_??' & parModelIdent='_15' & n_timestep_adjust=-1
;Model='st35gm04n33' & ModelIdent='_??' & parModelIdent='_26'
;Model='st35gm04n34' & ModelIdent='_??' & parModelIdent='_36'
;Model='st35gm04n36' & ModelIdent='_???' & parModelIdent='_090'
;Model='st35gm04n37' & ModelIdent='_???' & parModelIdent='_180'
;Model='st35gm04n38' & ModelIdent='_???' & parModelIdent='_027'
;model='st35gm04n41' & ModelIdent='_[7-9]??' & parModelIdent='_999'
;model='st35gm04n42' & ModelIdent='_???' & parModelIdent='_255'
;Model='st35gm04b0n001' & ModelIdent='_???' & parModelIdent='_100'
;Model='st35gm04b0n002' & ModelIdent='_???' & parModelIdent='_082'
;Model='st35gm04b1n001' & ModelIdent='_???' & parModelIdent='_446'
;Model='st35gm04b1n002' & ModelIdent='_???' & parModelIdent='_999'
;Model='st35gm04b1n003' & ModelIdent='_???' & parModelIdent='_606'
;Model='st35gm04b1n004' & ModelIdent='_???' & parModelIdent='_141'
;Model='st35gm04b1n005' & ModelIdent='_???' & parModelIdent='_184'

;Model='st35gm03n01' & ModelIdent='_??' & parModelIdent='_04'
;Model='st35gm03n02' & ModelIdent='_??' & parModelIdent='_30' & n_timestep_adjust=-26
;Model='st35gm03n03' & ModelIdent='_??' & parModelIdent='_09'
;Model='st35gm03n04' & ModelIdent='_??' & parModelIdent='_50'
;Model='st35gm03n05' & ModelIdent='_??' & parModelIdent='_30'
;Model='st35gm03n06' & ModelIdent='_??' & parModelIdent='_80' & n_timestep_adjust=-1
;Model='st35gm03n07' & ModelIdent='_???' & parModelIdent='_161'
;Model='st35gm03n08' & ModelIdent='_??' & parModelIdent='_21'
;Model='st35gm03n09' & ModelIdent='_??' & parModelIdent='_80'
;Model='st35gm03n10' & ModelIdent='_??' & parModelIdent='_70' & n_timestep_adjust=-3
;Model='st35gm03n11' & ModelIdent='_??' & parModelIdent='_75' & n_timestep_adjust=-1
;Model='st35gm03n13' & ModelIdent='_???' & parModelIdent='_708' & n_timestep_adjust=-301
;Model='st35gm03n14c' & ModelIdent='_[01]??' & parModelIdent='_199'
;Model='st35gm03n15' & ModelIdent='_???' & parModelIdent='_008'
;Model='st35gm03n16' & ModelIdent='_???' & parModelIdent='_080'
;Model='st35gm03n17' & ModelIdent='_???' & parModelIdent='_200'
;Model='st35gm03n18' & ModelIdent='_???' & parModelIdent='_018'
;Model='st35gm03n19' & ModelIdent='_???' & parModelIdent='_100'

;Model='st35gm00n01' & ModelIdent='_???' & parModelIdent='_115'
;Model='st35gm00n01_tweak' & ModelIdent='_???' & parModelIdent='_024'

;Model='st36g00n01' & ModelIdent='_???' & parModelIdent='_029'
;Model='st36g00n03' & ModelIdent='_???' & parModelIdent='_012'
;Model='st36g00n04' & ModelIdent='_???' & parModelIdent='_071'
;Model='st36g00n05' & ModelIdent='_???' & parModelIdent='_303' & n_timestep_adjust=-250

;Model='st36g45n04'     & ModelIdent='_??' & parModelIdent='_24' & n_timestep_adjust=250
;Model='st36g45n04rot'  & ModelIdent='_??' & parModelIdent='_23' & n_timestep_adjust=250
;Model='st36g45n04rot2' & ModelIdent='_??' & parModelIdent='_14' & n_timestep_adjust=250
;Model='st36g45n04rot3' & ModelIdent='_??' & parModelIdent='_29'
;Model='st36g45n04rot4' & ModelIdent='_??' & parModelIdent='_29'
;Model='st36g45n04rot5' & ModelIdent='_??' & parModelIdent='_29'
;Model='st36g45n04hll'   & ModelIdent='_??' & parModelIdent='_13'
;Model='st36g45n05'     & ModelIdent='_???' & parModelIdent='_225'
;Model='st36g45n06'     & ModelIdent='_???' & parModelIdent='_031'
;Model='st36g45n07'     & ModelIdent='_???' & parModelIdent='_034'
;Model='st36g45n08'     & ModelIdent='_???' & parModelIdent='_130'
;Model='st36g45n09'     & ModelIdent='_???' & parModelIdent='_150'
;Model='st36g45n10'     & ModelIdent='_???' & parModelIdent='_150'
;Model='st36g45n11'     & ModelIdent='_???' & parModelIdent='_300'
;Model='st36g45n12'     & ModelIdent='_???' & parModelIdent='_300'
;Model='st36g45n13'     & ModelIdent='_???' & parModelIdent='_400'
;Model='st36g45n14'     & ModelIdent='_???' & parModelIdent='_400'
;Model='st36g45n15'     & ModelIdent='_???' & parModelIdent='_300'
;Model='st36g45n16'     & ModelIdent='_???' & parModelIdent='_300'
;Model='st36g45n17'     & ModelIdent='_???' & parModelIdent='_267'
;Model='st36g45n18'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n19'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n20'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n21'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n22'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n23'     & ModelIdent='_11?' & parModelIdent='_120'
;Model='st36g45n24'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n25'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n26'     & ModelIdent='_10?' & parModelIdent='_127'
;Model='st36g45n27'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n28'     & ModelIdent='_09?' & parModelIdent='_100'
;Model='st36g45n29'     & ModelIdent='_1??' & parModelIdent='_150'
;Model='st36g45n30'     & ModelIdent='_0[3-6]?' & parModelIdent='_060'
;Model='st36g45n31'     & ModelIdent='_1??' & parModelIdent='_120'
;Model='st36g45n32'     & ModelIdent='_{0[8-9]?,100}' & parModelIdent='_100'
;Model='st36g45n33'     & ModelIdent='_0{[5-9]?,100}' & parModelIdent='_100'
;Model='st36g45n34'     & ModelIdent='_0[4-9]?' & parModelIdent='_082'
;Model='st36g45n35'     & ModelIdent='_0[3-5]?' & parModelIdent='_055'
;Model='st36g45n37'     & ModelIdent='_0[2-4]?' & parModelIdent='_041'
;Model='st36g45n38'     & ModelIdent='_0[12]?' & parModelIdent='_011'
;Model='st36g45n39'     & ModelIdent='_0[3-9]?' & parModelIdent='_050'
;Model='st36g45n41'     & ModelIdent='_0??' & parModelIdent='_026'
;Model='st36g45n42'     & ModelIdent='_0[4-9]?' & parModelIdent='_068'
;Model='st36g45n43'     & ModelIdent='_{0[7-9]?,100}' & parModelIdent='_100'

;Model='st40gm03n01' & ModelIdent='_??' & parModelIdent='_17' & n_timestep_adjust=-37
;Model='st40gm03n02' & ModelIdent='_??' & parModelIdent='_39' & n_timestep_adjust=-9
;Model='st40gm03n03' & ModelIdent='_??' & parModelIdent='_22'
;Model='st40gm03n04' & ModelIdent='_??' & parModelIdent='_99'
;Model='st45gm03n01' & ModelIdent='_??' & parModelIdent='_09' & n_timestep_adjust=-5
;Model='st45gm03n02' & ModelIdent='_??' & parModelIdent='_07'

;Model='st47g20n04' & ModelIdent='_??' & parModelIdent='_01'
;Model='st47g20n05' & ModelIdent='_??' & parModelIdent='_01'
;Model='st57g44n08'  & ModelIdent='_??' & parModelIdent='_13'
;Model='st57g44n09'  & ModelIdent='??' & parModelIdent='az'
;Model='st57g44n11a'  & ModelIdent='_??' & parModelIdent='01'
;Model='rhd' & ModelIdent='' & parModelIdent=''
;
;allModel='st28gm06n0[5,6]_??'

; --- Read meta data (name and unit of tags) ---
ReadMetaData=0

;ModelSuffix='chu1'
;ModelSuffix='mcyl'
ModelSuffix='mean'

ModelDisk=getenv('HOME') + '/dat/rhd/d' + Model + '/'
eosDisk=getenv('HOME') + '/for/eos/dat/'
opaDisk=getenv('HOME') + '/for/opa/dat/'


ModelFile=ModelDisk + Model + ModelIdent + '.' + ModelSuffix

; --- Read parameter file ---
par=uio_struct_rd(ModelDisk + Model + parModelIdent + '.par', ReadMetaData=1)
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosDisk + par.eosfile, EOS &$
  dfopta          , opaDisk + par.opafile &$
endif

delvar, ListData
ful=uio_datasetlist_rd(ModelFile, ListData=ListData, ReadMetaData=ReadMetaData, HeadRead=0, date=date, ierr=ierr)
uio_closrd, ListData.channel

; --- Process model ---
; --- Take slice out of model ---
ful=slicebox(ful, NormalId='0+0', Slope=0.6)
; --- Compute EOS data and opacities ---
eosbox, ful, eos=EOS, /opa, ierror=ierror & help,ierror

time_first=ful.Modeltime

uio_openrd, nc, listdata.file[listdata.total_number-1]
outstr='found'
while (outstr eq 'found') do begin &$
  uio_slhd, nc, ['real', 'modeltime'], termt, ntt, outstr, ierr &$
  if (ierr eq 0) then begin &$
    uio_rd, nc, termt,ntt, time_last,ident &$
    print, 'ierr, time_last=', ierr, time_last &$
  endif &$
endwhile
uio_closrd, nc

;n_timestep_first=ceil(time_first/par.dtime_out_mean)
;n_timestep_last =floor(time_last/par.dtime_out_mean)
;n_timestep=long((time_last-time_first)/par.dtime_out_mean+0.5)+1
n_timestep_first=round(time_first/par.dtime_out_mean)
n_timestep_last =round(time_last/par.dtime_out_mean)
n_timestep=n_timestep_last-n_timestep_first+1
n_timestep=n_timestep+n_timestep_adjust
help, time_first, time_last, par.dtime_out_mean, n_timestep_first, n_timestep_last, n_timestep

delvar, listdata

; --- Create (possibly large) array of structures ---
;fula=make_array(n_timestep, value=ful.z)
fula=make_array(n_timestep, value=ful)

ierr=0
i=0L
itime_last=-10000L
file=''
systime0=systime(1)
;version=''

while ((ierr eq 0) and (i lt n_timestep)) do begin &$
  ful0=uio_datasetlist_rd(ModelFile, listdata=listdata, ReadMetaData=ReadMetaData, HeadRead=0, date=date, ierr=ierr) &$
  if (ierr eq 0) then begin &$
    oldfile=file &$
    file=listdata.file[listdata.open_number] &$
    file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
    ful=temporary(ful0) &$
    ;if (total(strcmp(tag_names(ful.head), 'VERSION')) ge 1.0) then begin &$
    ;  version=ful.head.version &$
    ;endif &$
    if (ful.Modelitime gt (itime_last+1)) then begin &$
      print, '--- ', i, '/', n_timestep, ful.z.itime, ful.z.time, date, ' ---' , format='(A,I5,A,I5,1X,I8,E15.8,2X,A,A)' &$
      ;
      ;if (file ne oldfile) then par=uio_struct_rd(strmid(file,0,strlen(file)-4) + 'par') &$
      file_tmp=strmid(file, strpos(file,'/',/REVERSE_SEARCH)+1) &$
      ;
      ; --- Process model ---
      ; --- Take slice out of model ---
      ful=slicebox(ful, NormalId='0+0', Slope=0.6) &$
      ; --- Compute EOS data and opacities ---
      eosbox, ful, eos=EOS, /opa, ierror=ierror &$
      ;
      ; --- Put selected sub structure into array of structures ---
      ;fula[i]=ful.z &$
      fula[i]=ful &$
      ;
      i=i+1 &$
      if (ierr eq 0) then itime_last=ful.Modelitime &$
    endif &$
  endif &$
endwhile

if (i lt n_timestep) then print, 'WARNING: structure not filled completely! ####################### ' + string(n_timestep-i)

print, 'Total time: ', systime(1)-systime0, ' sec'

;SaveFile=Model + '_' + ModelSuffix + '.sav'
SaveFile=Model + '_' + ModelSuffix + '_slice060' + '.sav'
print, 'Save dataset into ''' + SaveFile + ''''
save, filename=SaveFile, fula

; v1=reform(mean(fula[*].v1_xmean,dim=4)) & v2=reform(mean(fula[*].v2_xmean,dim=4)) & v3=reform(mean(fula[*].v3_xmean,dim=4)) & s=reform(mean(fula[*].s_xmean,dim=4))

; rho=reform(mean(fula[*].rho_xmean,dim=4)) & v1=reform(mean(fula[*].rhov1_xmean,dim=4))/rho & v2=reform(mean(fula[*].rhov2_xmean,dim=4))/rho & v3=reform(mean(fula[*].rhov3_xmean,dim=4))/rho & s=reform(mean(fula[*].s_xmean,dim=4)) & T=reform(mean(fula[*].t_xmean,dim=4))

; loadct,64,file='colors1.tbl' & plotfield, /tv, reform(v2), drange=[-1,1]*4e5, xvec=reform(v1), yvec=reform(v3), /str, s_time=20e-5, s_colu=30, s_row=60 & plotfield, /contour, T, c_levels=[2800.0,3600.0], /over

; normct, 8 & plotfield, /tv, reform(s), xvec=reform(v1), yvec=reform(v3), drange=1.363e9+[-1,0.15]*3e7, /str, s_time=30e-5, s_colu=30, s_row=60                                                                                      

; end ; rhd_z_collect
