; pro sphereintegrate_start
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sphereinstart ('shpere_integrate_start')
;
; PURPOSE:
;   Integrate a spheric stratification by calling sphereintegrate.
;
; CATEGORY:
;   Computations (Special)
;
; CALLING SEQUENCE:
;   @sphereintegrate_start
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   tabinter_rdcoeff:
;   dfopta:
;   sphereintegrate:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   Before first use type
;     delvar, init
;
; PROCEDURE:
;   Call sphereintegrate
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1999-06-09 (B.F. Copenhagen) Written by Bernd Freytag, Astronomical Observatory Copenhagen
;   2007-02-10 (B.F. Lyon) st57g44n11, out_time_mean_last
;   2012-05-14 (B.F.) Parameter approximation for each model
;   2014-07-08 (B.F. Uppsala) Add parameter r2_grav
;   2018-04-19 (B.F.) Add parameter gravDeep
;   2018-04-25 (B.F.) Rename parameter gravDeep -> gravCorePotDepth
;-
;------------------------------------------------------------------------------
;
; --- Define EOS/OPA paths and files ---
eosdisk=getenv('HOME') + '/for/eos/dat/'
opadisk=getenv('HOME') + '/for/opa/dat/'
;eosdisk='/astro/b/bf/eos/dat/'
;opadisk='/astro/b/bf/opa/dat/'
;eosdisk='~/eos/dat/'
;opadisk='~/opa/dat/'
gasfile='gas_mm00_l.eos' & abuX0='sun' & gasname='gas' & eosfile='eos_mm00_l5.eos'
;opafile='g2va.opta'
;opafile='big_grey.opta'
opafile='phoenix_opal_grey.opta'
;outdisk='/astro/p4/bf/hd/job1/sta/'
outdisk=getenv('HOME') + '/idlwrk/'
;
; --- Default values (may be overwritten for any model) ---
sp_integrator='gas'
approximation='mihalas'     ; --- 'eddington', 'mihalas' --
RSurface     =0.0
r2_grav      =0.0
gravCorePotDepth     =0.0

; --- Define model data ---
;model='betelgeuse1'
;  Teff=4000.
;  mass_star=1. * !con.Msun
;  Ptop=0.05
;  Radius=1200.*!con.Rsun
;  r0_grav=Radius/8.0
;  Center=0.0
;  smax=2.70E+09
;  boxsize=Radius
;  npoint=31
;  vampl=10.0

;model='betelgeuse2'
;  Teff=3000.
;  mass_star=1. * !con.Msun
;  Ptop=0.0002
;  Radius=1200.*!con.Rsun
;  r0_grav=Radius/10.0
;  Center=0.0
;  smax=3.10E+09
;  boxsize=Radius
;  npoint=31
;  vampl=10.0

;model='betelgeuse3'
;  Teff=3000.
;  mass_star=1. * !con.Msun
;  Ptop=1.0E-5
;  Radius=1200.*!con.Rsun
;  r0_grav=Radius/10.0
;  Center=0.0
;  smax=3.10E+09
;  boxsize=Radius
;  npoint=31
;  vampl=10.0

;model='betelgeuse4'
;  Teff=3000.
;  mass_star=1. * !con.Msun
;  Ptop=1.0E-5
;  Radius=1200.*!con.Rsun
;  r0_grav=Radius/10.0
;  Center=0.0
;  smax=3.10E+09
;  boxsize=Radius
;  npoint=31
;  vampl=1.0E5

;model='betelgeuse5'
;  Teff     =3500.
;  mass_star=1. * !con.Msun
;  Ptop     =1.0E-4
;  Radius   =1200.*!con.Rsun
;  r0_grav  =Radius/12.0
;  Center   =0.0
;  smax     =3.10E+09
;  boxsize  =Radius
;  npoint   =51
;  vampl    =3.0E5

;model='betelgeuse6'
;  Teff     =3500.
;  mass_star=1. * !con.Msun
;  Ptop     =1.0E-4
;  Radius   =1200.*!con.Rsun
;  r0_grav  =Radius/14.0
;  Center   =0.0
;  smax     =3.3E+09
;  boxsize  =Radius
;  npoint   =51
;  vampl    =3.0E5

;model='betelgeuse7'
;  Teff     =3500.
;  mass_star=0.3 * !con.Msun
;  Ptop     =5.0E+00
;  Radius   =1200.*!con.Rsun
;  r0_grav  =Radius/14.0
;  Center   =0.0
;  smax     =3.4E+09
;  boxsize  =Radius
;  npoint   =51
;  vampl    =1.0E5

;model='betelgeuse99'
;  Teff=3000.
;  mass_star=2.5 * !con.Msun
;  Ptop=1.0E-10
;  Radius=1200.*!con.Rsun
;  r0_grav=Radius/10.0
;  Center=0.0
;  smax=3.10E+09
;  boxsize=Radius
;  npoint=31
;  vampl=10.0

;model='sunsphere'
;  Teff=5770.
;  mass_star=!con.Msun
;  r0_grav=0.0
;  Ptop=4000.0
;  Radius=!con.Rsun
;  Center=Radius-1.0E8
;  smax=1.73E+09
;  vampl=10.0
;
;model='sunmini'
;  Teff     =5770.
;  mass_star=(0.5*0.003)^2 * !con.Msun
;  Ptop     =5.0E+02
;  Radius   =0.003*!con.Rsun
;  r0_grav  =Radius/10.0
;  Center   =0.0
;  smax     =1.73E+09
;  boxsize  =Radius
;  npoint   =51
;  vampl    =1.0E5
;
;model='sunmini2'
;  Teff     =5770.
;  mass_star=(0.5*0.0035)^2 * !con.Msun
;  Ptop     =1.0E+02
;  Radius   =0.0035*!con.Rsun
;  r0_grav  =Radius/10.0
;  Center   =0.0
;  smax     =1.73E+09
;  boxsize  =Radius
;  npoint   =63
;  vampl    =1.0E5

;model='st22g35n01'
;  date     ='2013-04-08'
;  dlntau   =0.01
;  Teff     =2200.
;  Radius   =0.015 * !con.Rsun
;  RSurface =Radius
;  mass_star=2.58335e-05 * !con.Msun  ; 0.015^2*10.0^(3.5-4.44)
;  Ptop     =0.75E-03
;  Center   =0.0
;  smax     =0.92843e+09  ; 0.9290e+09
;  facsmax  =0.985
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.57
;  npoint   =315
;  vampl    =100.0
;  dtime    =1.0E+00
;  opafile='t1800g550mm00_phoenix_idmean3RT3_fit.opta'
;  sp_integrator='eos'
;  approximation='holmu'

;model='st22g35n02'
;  date     ='2013-04-12'
;  dlntau   =0.01
;  Teff     =2200.
;  Radius   =0.012 * !con.Rsun
;  RSurface =Radius
;  mass_star=1.65334e-05 * !con.Msun  ; 0.012^2*10.0^(3.5-4.44)
;  Ptop     =4.7E-02
;  Center   =0.0
;  smax     =0.928435e+09  ; 0.9290e+09
;  facsmax  =0.985
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.57
;  npoint   =375
;  vampl    =100.0
;  dtime    =1.0E+00
;  opafile='t1800g550mm00_phoenix_idmean3RT3_fit.opta'
;  sp_integrator='eos'
;  approximation='holmu'

;model='st22g35n08test'
;  date     ='2014-07-08'
;  dlntau   =0.002
;  Teff     =2200.
;  Radius   =0.012 * !con.Rsun
;  RSurface =Radius
;  mass_star=(10.0^3.5/!con.gravsun) * (Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =4.0E-08
;  Center   =0.0
;  smax     =0.9288e+09  ; 0.9290e+09
;  facsmax  =0.985
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.3
;  r1_grav  =0.0
;  r2_grav  =Radius*3.75
;  npoint   =375
;  vampl    =100.0
;  dtime    =1.0E+00
;  opafile='t1800g550mm00_phoenix_idmean3RT3_fit.opta'
;  sp_integrator='eos'
;  approximation='holmu'

;model='st22g40n01'
;  date     ='2013-04-02'
;  dlntau   =0.03
;  Teff     =2200.
;  Radius   =0.007 * !con.Rsun
;  RSurface =Radius
;  mass_star=1.77908e-05 * !con.Msun
;  Ptop     =2.0E-07
;  Center   =0.0
;  smax     =0.8851e+09
;  facsmax  =0.987
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.57
;  npoint   =315
;  vampl    =100.0
;  dtime    =1.0E+00
;  opafile='t1800g550mm00_phoenix_idmean3RT3_fit.opta'
;  sp_integrator='eos'
;  approximation='holmu'

;model='st26gm09n01'
;  Teff     =2600.
;  Radius   =3.26E+13
;  mass_star=1.0 * !con.Msun
;  Ptop     =3.0E-4
;  Center   =0.0
;  smax     =3.50E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*1.51
;  npoint   =127
;  vampl    =0.1E5
;  dtime=1.0E+04

;model='st26gm09n02'
;  Teff     =2600.
;  Radius   =3.26E+13
;  mass_star=1.0 * !con.Msun
;  Ptop     =5.0E-5
;  Center   =0.0
;  smax     =3.60E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*1.51
;  npoint   =127
;  vampl    =0.1E5
;  dtime=1.0E+04

;model='st26gm09n03'
;  Teff     =2600.
;  Radius   =3.26E+13
;  mass_star=1.0 * !con.Msun
;  Ptop     =5.0E-6
;  Center   =0.0
;  smax     =3.30E+09
;  facsmax  =0.95
;  boxsize  =3.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/14.4
;  r1_grav  =boxsize*1.51
;  npoint   =127
;  vampl    =0.1E5
;  dtime=1.0E+04

;model='st26g35n01'
;  date     ='2010-10-07'
;  dlntau   =0.003
;  Teff     =2600.
;  Radius   =0.03 * !con.Rsun
;  mass_star=0.000104 * !con.Msun
;  Ptop     =1.0E-03
;  Center   =0.0
;  smax     =1.0837e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.55
;  npoint   =255
;  vampl    =0.1
;  dtime    =1.0E+00
;  sp_integrator='eos'

;model='st26g35n02'
;  date     ='2010-12-17'
;  dlntau   =0.03
;  Teff     =2600.
;  Radius   =0.02 * !con.Rsun
;  mass_star=4.62222e-05 * !con.Msun
;  Ptop     =1.0E-01
;  Center   =0.0
;  smax     =1.0837e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.55
;  npoint   =255
;  vampl    =0.1
;  dtime    =1.0E+00
;  sp_integrator='eos'

;model='st26g35n06'
;  date     ='2012-05-14'
;  dlntau   =0.03
;  Teff     =2600.
;  Radius   =0.02 * !con.Rsun
;  RSurface =Radius
;  mass_star=4.62222e-05 * !con.Msun
;  Ptop     =1.0E-04
;  Center   =0.0
;  smax     =1.0837e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.50
;  r1_grav  =boxsize*0.57
;  npoint   =255
;  vampl    =100.0
;  dtime    =1.0E+00
;  sp_integrator='eos'
;  approximation='holmu'

;model='st28gm06n01'
;  Teff     =2800.
;  Radius   =2.48E+13
;  mass_star=1.0 * !con.Msun
;  Ptop     =1.0E-8
;  Center   =0.0
;  smax     =3.50E+09
;  facsmax  =0.9
;  boxsize  =3.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/16.0
;  r1_grav  =boxsize*1.51
;  npoint   =127
;  vampl    =0.1E5
;  dtime    =1.0E+03

;model='st28gm06n08'
;  Teff     =2800.
;  dlntau   =0.01
;  Radius   =2.48E+13
;  mass_star=1.0 * !con.Msun
;  Ptop     =4.0E-02
;  Center   =0.0
;  smax     =8.50E+09
;  facsmax  =1.2
;  boxsize  =4.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize*(3.5/4.0)/16.0
;  r1_grav  =boxsize*(3.5/4.0)*1.51
;  npoint   =171
;  vampl    =0.1E5
;  dtime    =1.0E+03
;;  sp_integrator='eos'


;model='st28gm05n014test'
;  date     ='2018-04-19'
;  dlntau   =0.005
;  Teff     =2900.
;  Radius   =280.0 * !con.Rsun
;  mass_star=1.5 * !con.Msun
;  Ptop     =1.4E-12
;  Center   =0.0
;  smax     =2.89E+09
;  facsmax  =0.4
;  boxsize  =2.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  ;r0_grav  =4.38679e+12
;  r0_grav  =Radius*0.30
;  r1_grav  =boxsize*1.5
;  r2_grav  =0.0
;  ;gravCorePotDepth=0.0
;  gravCorePotDepth=2.0
;  ;gravCorePotDepth=10.0
;  npoint   =127
;  vampl    =0.1
;  dtime    =500.0E+00
;  sp_integrator='eos'
;  ; envelope mass: org:0.272440     r0_grav:0.224756     gravCorePotDepth:0.373162      ratio=1.36970

;model='st28gm05n015test'
;  date     ='2018-05-02'
;  dlntau   =0.005
;  Teff     =2900.
;  Radius   =280.0 * !con.Rsun
;  mass_star=1.5 * !con.Msun
;  Ptop     =1.4E-12
;  Center   =0.0
;  smax     =2.89E+09
;  facsmax  =0.4
;  boxsize  =2.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  ;r0_grav  =4.38679e+12
;  r0_grav  =85.0 * !con.Rsun
;  ;r0_grav  =Radius*0.34
;  r1_grav  =boxsize*1.5
;  r2_grav  =0.0
;  ;gravCorePotDepth=1.0
;  gravCorePotDepth=1.275
;  ;gravCorePotDepth=2.0
;  npoint   =127
;  vampl    =0.1
;  dtime    =500.0E+00
;  sp_integrator='eos'
;  ; envelope mass: seq:0.37395542   org:0.37506543   gravCorePotDepth:0.50482785   ratio=1.34597
;  ; envelope mass: seq:0.22251514   org:0.272440     r0_grav:0.22251514     gravCorePotDepth:0.27210067      ratio=0.998755

;model='st28gm05n017test'
;  date     ='2018-05-15'
;  dlntau   =0.005
;  Teff     =2900.
;  Radius   =280.0 * !con.Rsun
;  mass_star=1.5 * !con.Msun
;  Ptop     =1.4E-12
;  Center   =0.0
;  smax     =2.89E+09
;  facsmax  =0.4
;  boxsize  =2.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =85.0 * !con.Rsun
;  r1_grav  =boxsize*1.5
;  r2_grav  =0.0
;  ;gravCorePotDepth=1.275
;  gravCorePotDepth=2.5
;  npoint   =127
;  vampl    =0.1
;  dtime    =500.0E+00
;  sp_integrator='eos'
;  ; envelope mass: seq:0.27296063   org:0.27210067   gravCorePotDepth:0.40943175   ratio=1.50471

;model='st28gm05n020test'
;  date     ='2018-06-29'
;  dlntau   =0.005
;  Teff     =2900.
;  Radius   =280.0 * !con.Rsun
;  mass_star=1.5 * !con.Msun
;  Ptop     =1.6E-12
;  Center   =0.0
;  smax     =2.89E+09
;  facsmax  =0.4
;  boxsize  =2.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =85.0 * !con.Rsun
;  r1_grav  =boxsize*1.5
;  r2_grav  =0.0
;  ;gravCorePotDepth=2.5
;  gravCorePotDepth=7.0
;  npoint   =127
;  vampl    =0.1
;  dtime    =500.0E+00
;  sp_integrator='eos'
;  ; envelope mass: seq:0.41067703   org:0.41053958   gravCorePotDepth:0.61922612   ratio=1.50832

;model='st28gm05n021test'
;  date     ='2018-07-03'
;  dlntau   =0.005
;  Teff     =2900.
;  Radius   =280.0 * !con.Rsun
;  mass_star=1.5 * !con.Msun
;  Ptop     =1.6E-12
;  Center   =0.0
;  smax     =2.89E+09
;  facsmax  =0.4
;  boxsize  =2.0*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =85.0 * !con.Rsun
;  r1_grav  =boxsize*1.5
;  r2_grav  =0.0
;  ;gravCorePotDepth=2.5
;  gravCorePotDepth=3.0
;  npoint   =127
;  vampl    =0.1
;  dtime    =500.0E+00
;  sp_integrator='eos'
;  ; envelope mass: seq:0.41066954   org:0.41053958   gravCorePotDepth:0.44758527   ratio=1.09024

model='st28gm05n024NotDone'
  date     ='2018-07-25'
  dlntau   =0.005
  Teff     =2700.
  Radius   =290.0 * !con.Rsun
  mass_star=1.0 * !con.Msun
  Ptop     =3.0E-09
  Center   =0.0
  smax     =2.645E+09
  facsmax  =0.4
  boxsize  =2.0*Radius
  Rtop     =sqrt(3.0)*boxsize/2.0
  r0_grav  =4.38679E+12
  r1_grav  =boxsize*1.5
  r2_grav  =0.0
  ;gravCorePotDepth=1.0
  gravCorePotDepth=1.4
  npoint   =127
  vampl    =0.1
  dtime    =500.0E+00
  sp_integrator='eos'
  ; envelope mass: seq:0.46330074   org:0.46558282   gravCorePotDepth:0.60599402   ratio=1.30158

;model='st33gm06n01'
;  Teff     =3300.
;  Radius   =5.56E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =2.0E-8
;  Center   =0.0
;  smax     =3.55E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.57
;  npoint   =127
;  vampl    =0.0E5
;  dtime=1.0E+04

;model='st33gm06n02'
;  Teff     =3300.
;  Radius   =5.56E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =5.0E-6
;  Center   =0.0
;  smax     =3.65E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.51
;  npoint   =127
;  vampl    =0.0E5
;  dtime=4.0E+04

;model='st35gm06n01'
;  Teff     =3300.
;  Radius   =5.56E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =10.0E-6
;  Center   =0.0
;  smax     =3.85E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.51
;  npoint   =127
;  vampl    =0.0E5
;  dtime=2.0E+04
;

;model='st35gm04n03'
;  Teff     =3300.
;  Radius   =5.56E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =10.0E-6
;  Center   =0.0
;  smax     =3.85E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.51
;  npoint   =127
;  vampl    =0.0E5
;  dtime=2.0E+04
;
;model='st35gm04n03'
;  Teff     =3500.
;  Radius   =4.54E+13
;  mass_star=5.0 * !con.Msun
;  Ptop     =10.0E-10
;  Center   =0.0
;  smax     =3.85E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.62
;  npoint   =127
;  vampl    =0.1
;  dtime    =1.0E+04

;model='st35gm03n01'
;  date     ='2004-03-05'
;  Teff     =3500.
;  ;Radius   = 810.0     * !con.Rsun
;  Radius   = 810.99512 * !con.Rsun
;  mass_star= 12.0 * !con.Msun
;  Ptop     =10.0E-13
;  Center   =0.0
;  smax     =3.85E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.50
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04

; model='st35gm03n01'
;   date     ='2004-03-05'
;   Teff     =3500.
;   ;Radius   = 810.0     * !con.Rsun
;   Radius   = 810.99512 * !con.Rsun
;   mass_star= 12.0 * !con.Msun
;   Ptop     =10.0E-13
;   Center   =0.0
;   smax     =3.40E+09
;   facsmax  =0.95
;   boxsize  =2.5*Radius
;   Rtop     =sqrt(3.0)*boxsize/2.0
;   r0_grav  =boxsize/15.0
;   r1_grav  =boxsize*0.50
;   npoint   =171
;   vampl    =0.1
;   dtime    =1.0E+04
;   sp_integrator='eos'

;model='st35gm03n02'
;  date     ='2004-03-22'
;  Teff     =3500.
;  ;Radius   = 810.0     * !con.Rsun
;  Radius   = 810.99512 * !con.Rsun
;  mass_star= 12.0 * !con.Msun
;  Ptop     =10.0E-13
;  Center   =0.0
;  smax     =3.50E+09
;  facsmax  =0.96
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.50
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04
;  sp_integrator='eos'

;model='st36g00n01'
;  date     ='2009-10-23'
;  Teff     =3600.
;  Radius   =400.0 * !con.Rsun
;  mass_star=6.0 * !con.Msun
;  Ptop     =1.0E-15
;  Center   =0.0
;  smax     =3.2E+09
;  facsmax  =0.96
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.55
;  npoint   =255
;  vampl    =0.1
;  dtime    =1.0E+03
;  sp_integrator='eos'

;model='st36gm05n02'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=10.0 * !con.Msun
;  Ptop     =5.0E-5
;  Center   =0.0
;  smax     =3.00E+09
;  facsmax  =0.98
;  boxsize  =2.5*Radius
;  Rtop     =boxsize/2.0
;  r0_grav  =boxsize/10.0
;  npoint   =159
;  vampl    =1.0E5
;  dtime=1.0E+04

;model='st36gm08n01'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=6.0 * !con.Msun
;;  Ptop     =2.0E-2 & Rtop     =boxsize/2.0
;  Ptop     =3.0E-3
;  Center   =0.0
;  smax     =3.10E+09
;  facsmax  =0.995
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/16.0
;  r1_grav  =boxsize*0.5
;  npoint   =159
;  vampl    =1.0E5
;  dtime=1.0E+04

;model='st36gm08n02'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =3.0E-3
;  Center   =0.0
;  smax     =3.10E+09
;  facsmax  =0.6
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/16.0
;  r1_grav  =boxsize*0.5
;  npoint   =159
;  vampl    =1.0E5
;  dtime=2.0E+04

;model='st36gm08n03'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =2.0E-2
;  Center   =0.0
;  smax     =3.70E+09
;  facsmax  =0.99
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.46
;  npoint   =127
;  vampl    =1.0E5
;  dtime=2.0E+04

;model='st36gm08n04'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =1.0E-6
;  Center   =0.0
;  smax     =3.45E+09
;  facsmax  =0.99
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.60
;  npoint   =127
;  vampl    =1.0E5
;  dtime=2.0E+04

;model='st36gm08n05'
;  Teff     =3600.
;  Radius   =7.0E+13
;  mass_star=6.0 * !con.Msun
;  Ptop     =1.0E-6
;  Center   =0.0
;  smax     =3.40E+09
;  facsmax  =2.0
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*0.60
;  npoint   =127
;  vampl    =0.0E5
;  dtime=2.0E+04

;model='st40gm03n01'
;  date     ='2004-03-05'
;  Teff     =4000.
;  ;Radius   = 810.0    * !con.Rsun
;  Radius   = 806.09267* !con.Rsun
;  mass_star=12.0 * !con.Msun
;  Ptop     =10.0E-12
;  Center   =0.0
;  smax     =4.95E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.50
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04

;model='st40gm03n01'
;  date     ='2004-03-10'
;  Teff     =4000.
;  ;Radius   = 810.0    * !con.Rsun
;  Radius   = 806.09267* !con.Rsun
;  mass_star=12.0 * !con.Msun
;  Ptop     =10.0E-12
;  Center   =0.0
;  smax     =4.95E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.50
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04
;  sp_integrator='eos'

;model='st45gm03n01'
;  Teff     =4500.
;  ;Radius   = 810.0     * !con.Rsun
;  Radius   = 808.98780 * !con.Rsun
;  mass_star=12.0 * !con.Msun
;  Ptop     =10.0E-12
;  Center   =0.0
;  smax     =4.95E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.54
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04

;model='st45gm03n01'
;  date     ='2004-03-10'
;  Teff     =4500.
;  ;Radius   = 810.0     * !con.Rsun
;  Radius   = 808.98780 * !con.Rsun
;  mass_star=12.0 * !con.Msun
;  Ptop     =10.0E-12
;  Center   =0.0
;  smax     =4.95E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/15.0
;  r1_grav  =boxsize*0.54
;  npoint   =171
;  vampl    =0.1
;  dtime    =1.0E+04
;  sp_integrator='eos'

;model='st47g20n01'
;  Teff     =4700.
;  Radius   =1.0E+11
;  mass_star=(Radius/!con.Rsun)^2 * (100.0/!con.gravsun) * !con.Msun
;  Ptop     =2.0E00
;  Center   =0.0
;  smax     =2.52E+09
;  facsmax  =0.95
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/14.0
;  r1_grav  =boxsize*0.5
;  npoint   =127
;  vampl    =1.0E5
;  dtime=2.0E+02

;model='st57g44n01'
;  Teff     =5770.
;  Radius   =2.0E+08
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =3.0E+01
;  Center   =0.0
;  smax     =1.73E+09
;  facsmax  =0.98
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/8.0
;  npoint   =127
;  vampl    =1.0E5
;  dtime=0.4E+00

;model='st57g44n08'
;  Teff     =5770.
;; --- Radius=5.0E+08 & Ptop=3.0E-06 & dlntau=0.0015
;; --- Radius=4.0E+08 & Ptop=3.0E-04 & dlntau=0.0015 & smax=1.73E+09
;  Radius   =4.0E+08
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =3.0E-04
;  Center   =0.0
;  smax     =1.73E+09
;  facsmax  =0.99
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*1.51
;  npoint   =171
;  vampl    =1.0E5
;  dtime=0.4E+00
;  sp_integrator='eos'

;model='st57g44n10'
;  Teff     =5770.
;; --- Radius=5.0E+08 & Ptop=3.0E-06 & dlntau=0.0015
;; --- Radius=4.0E+08 & Ptop=3.0E-04 & dlntau=0.0015 & smax=1.73E+09
;  Radius   =2.0E+08
;  dlntau=0.01
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =1.0E-01
;  Center   =0.0
;  smax     =1.5E+09
;  facsmax  =0.99
;  boxsize  =2.5*Radius
;  Rtop     =2.0*boxsize
;  r0_grav  =boxsize/12.0
;  r1_grav  =boxsize*5.0
;  npoint   =171
;  vampl    =0.0E5
;  dtime=0.3E+00
;  sp_integrator='eos'

;model='st57g44n11'
;  Teff     =5770.
;  ; --- Radius=4.0E+08 & Ptop=3.0E-04 & dlntau=0.0015 & smax=1.73E+09
;  Radius   =4.0E+08
;  dlntau   =0.01
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =5.0E-04
;  Center   =0.0
;  smax     =1.73E+09
;  facsmax  =0.97
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize/5.0
;  r1_grav  =boxsize*5.0
;  npoint   =171
;  vampl    =1.0E5
;  dtime    =2.0E+00
;  sp_integrator='eos'

;model='st57g44n22'
;  Teff     =5770.
;  ; --- Radius=4.0E+08 & Ptop=3.0E-04 & dlntau=0.0015 & smax=1.73E+09
;  Radius   =4.0E+08
;  dlntau   =0.01
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =5.0E-04
;  Center   =0.0
;  smax     =1.73E+09
;  facsmax  =0.97
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =boxsize*0.125
;  r1_grav  =boxsize*5.0
;  npoint   =171
;  vampl    =1.0E5
;  dtime    =2.0E+00
;  sp_integrator='eos'

;model='st57g44n24'
;  Teff     =5770.
;  ; --- Radius=4.0E+08 & Ptop=3.0E-04 & dlntau=0.0015 & smax=1.73E+09
;  Radius   =5.0E+08
;  dlntau   =0.01
;  mass_star=(Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =5.0E-05
;  Center   =0.0
;  smax     =1.73E+09
;  facsmax  =0.97
;  boxsize  =2.4*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.5
;  r1_grav  =boxsize*5.0
;  npoint   =255
;  vampl    =1.0E5
;  dtime    =2.0E+00
;  sp_integrator='eos'

;model='st36g45n01'
;  date     ='2011-01-25'
;  dlntau   =0.01
;  Teff     =3600.
;  Radius   =0.003 * !con.Rsun
;  mass_star=(10.0^4.5/!con.gravsun) * (Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =3.0E-01
;  Center   =0.0
;  smax     =1.363e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.25
;  r1_grav  =boxsize*0.63
;  npoint   =255
;  vampl    =0.1
;  dtime    =1.0E+00
;  sp_integrator='eos'

;model='st36g45n36compatible'
;  date     ='2014-07-08'
;  dlntau   =0.01
;  Teff     =3600.
;  Radius   =0.003 * !con.Rsun
;  mass_star=(10.0^4.5/!con.gravsun) * (Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =8.0E-05
;  Center   =0.0
;  smax     =1.363e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.44
;  r1_grav  =0.0
;  r2_grav  =Radius*3.75
;  npoint   =255
;  vampl    =0.1
;  dtime    =0.5E+00
;  sp_integrator='eos'

;model='st36g45n36'
;  date     ='2014-07-08'
;  dlntau   =0.002
;  Teff     =3600.
;  Radius   =0.0042 * !con.Rsun
;  mass_star=(10.0^4.5/!con.gravsun) * (Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =2.0E-08
;  Center   =0.0
;  smax     =1.363e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.35
;  r1_grav  =0.0
;  r2_grav  =Radius*3.75
;  npoint   =365
;  vampl    =0.1
;  dtime    =0.5E+00
;  sp_integrator='eos'

;model='st36g45n40'
;  date     ='2014-07-21'
;  dlntau   =0.002
;  Teff     =3600.
;  Radius   =0.0042 * !con.Rsun
;  mass_star=(10.0^4.5/!con.gravsun) * (Radius/!con.Rsun)^2 * !con.Msun
;  Ptop     =2.0E-08
;  Center   =0.0
;  smax     =1.363e+09
;  facsmax  =0.8
;  boxsize  =2.5*Radius
;  Rtop     =sqrt(3.0)*boxsize/2.0
;  r0_grav  =Radius*0.35
;  r1_grav  =0.0
;  r2_grav  =Radius*3.75
;  npoint   =401
;  vampl    =0.1
;  dtime    =0.3E+00
;  sp_integrator='eos'

;-------------------------------------------------------------------------------------

;dlntau=0.01
;dlntau=0.02
;dlntau=0.05
;dlntau=0.10
;approximation='mihalas'     ; --- 'eddington', 'mihalas' --
;
; --- Load EOS and opacity data ---
if (n_elements(init) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + gasfile, gas &$
  tabinter_rdcoeff, eosdisk + eosfile, eos &$
  dfopta, opadisk + opafile &$
  init=1 &$
endif
;
; --- Integrate grey stratification ---
if (sp_integrator eq 'eos') then begin &$
  sphereintegraterhoei, $
    Teff=Teff, $
    r0_grav=r0_grav, r1_grav=r1_grav, r2_grav=r2_grav, gravCorePotDepth=gravCorePotDepth, mass_star=mass_star, $
    gas=gas, eos=eos, $
    Ptop=Ptop, Radius=Rtop, RSurface=Radius, Center=Center, smax=smax, facsmax=facsmax, $
    dlntau=dlntau, approximation=approximation, $
    tauout=tau0, Tout=T0, Pout=P0, zout=z0, gout=g0, kapout=kappa0, rhoout=rho0, eiout=ei0 &$
endif else begin &$
  sphereintegrate, $
    Teff=Teff, $
    r0_grav=r0_grav, r1_grav=r1_grav, r2_grav=r2_grav, mass_star=mass_star, $
    gas=gas, $
    Ptop=Ptop, Radius=Rtop, Center=Center, smax=smax, facsmax=facsmax, $
    dlntau=dlntau, approximation=approximation, $
    tauout=tau0, Tout=T0, Pout=P0, zout=z0, gout=g0, kapout=kappa0 &$
endelse
;
; --- Remove uppermost point ---
n0=n_elements(T0)
;n0=n_elements(T0)-1
;tau0  =tau0(  0:n0-1)
;T0    =T0(    0:n0-1)
;P0    =P0(    0:n0-1)
;z0    =z0(    0:n0-1)
;g0    =g0(    0:n0-1)
;kappa0=kappa0(0:n0-1)
;
; --- Interpolate on equidistant z scale ---
;n=100
;z  =min(z0)+(max(z0)-min(z0))*findgen(n)/float(n-1)
;T  =    monocubic(     T0    , z0, z)
;tau=exp(monocubic(alog(tau0) , z0, z))
;P  =exp(monocubic(alog(P0  ) , z0, z))
;g  =    monocubic(     g0    , z0, z)
;kappa=monocubic(     kappa0  , z0, z)
n=n_elements(T0)
tau=tau0
z=z0
T=T0
P=P0
g=g0
kappa=kappa0
ei=ei0
rho=rho0
;
; --- Compute rho and ei ---
if (sp_integrator eq 'eos') then begin &$
  eosinter, rho, ei, Pdummy, dPdrho, dPdei, Tdummy, dTdei, s, eos=eos &$
  print, 'eosinter: rho,ei -> ...' &$
endif else begin &$
  gasinter, P, T, rho, xcp, xgam, cxt, h, s, csound, eos=gas, ierror=ierror &$
  ei=h-P/rho &$
  print, 'gasinter: P,T -> ...' &$
endelse
;
z1d  =z
rho1d=rho
ei1d =ei
;
!P.MULTI=[0,2,2,0,0]
!X.MARGIN=[10,2]
!Y.MARGIN=[3,1]
;
plot, z, s/1.0E+09, psym=-1, /ynoz
oplot, [0.5*boxsize,0.5*boxsize], !Y.CRANGE, linestyle=2
oplot, [Radius,Radius], !Y.CRANGE, linestyle=2, color=!ctv.red
oplot, [r0_grav, r0_grav], !Y.CRANGE, linestyle=2
oplot, seqavg('xc', seq, units='cgs'), seqavg('s', seq), color=!ctv.green
;
plot, z, alog10(T), psym=-1, /ynoz
oplot, [0.5*boxsize,0.5*boxsize], !Y.CRANGE, linestyle=2
oplot, [Radius,Radius], !Y.CRANGE, linestyle=2, color=!ctv.red
oplot, [r0_grav, r0_grav], !Y.CRANGE, linestyle=2
oplot, seqavg('xc', seq, units='cgs'), seqavg('log10_T', seq), color=!ctv.green
;
plot, z, alog10(rho), psym=-1
oplot, [0.5*boxsize,0.5*boxsize], !Y.CRANGE, linestyle=2
oplot, [Radius,Radius], !Y.CRANGE, linestyle=2, color=!ctv.red
oplot, [r0_grav, r0_grav], !Y.CRANGE, linestyle=2
oplot, seqavg('xc', seq, units='cgs'), seqavg('log10_rho', seq), color=!ctv.green
;
;plot, z, alog10(tau), psym=-1
;oplot, [0.5*boxsize,0.5*boxsize], !Y.CRANGE, linestyle=2
;oplot, [Radius,Radius], !Y.CRANGE, linestyle=2, color=!ctv.red
;oplot, [r0_grav, r0_grav], !Y.CRANGE, linestyle=2
;
plot, z, alog10(P), psym=-1
oplot, [0.5*boxsize,0.5*boxsize], !Y.CRANGE, linestyle=2
oplot, [Radius,Radius], !Y.CRANGE, linestyle=2, color=!ctv.red
oplot, [r0_grav, r0_grav], !Y.CRANGE, linestyle=2
oplot, seqavg('xc', seq, units='cgs'), seqavg('log10_P', seq), color=!ctv.green
;
; plot, z, p/rho/g/z

!P.MULTI=0
!X.MARGIN=[12,3]
!Y.MARGIN=[4,2]
;
print, 'minmax(T, P, rho, ei)'
print, minmax(T), minmax(P), minmax(rho), minmax(ei)
Luminosity=1.0D00*!con.sigmasb*Teff^4*(4.0D00*!DPi*double(Radius)^2)
Radius1=sqrt(Luminosity/(!con.sigmasb * Teff^4 * 4.0D00*!DPi))
luminositypervolume=Luminosity/(4.0D00/3.0D00*!DPI*double(r0_grav)^3)
grav_surf=!con.G*mass_star/(radius^2)
print, 'Luminosity/L_Sun:     ', Luminosity/!con.Lsun
print, 'luminositypervolume:  ', luminositypervolume
print, 'Surface gravity logg: ', alog10(grav_surf), (interpol(alog10(g),z,[Radius]))[0]
print, 'Surface temp Teff [K]:', (interpol(T,z,[Radius]))[0]
print, 'Mass star [g]:        ', mass_star
print, 'r0_grav [cm]:         ', r0_grav
print, 'r1_grav [cm]:         ', r1_grav
print, 'r2_grav [cm]:         ', r2_grav
print, 'Radius, Radius1 [cm]: ', Radius, Radius1
print, 'Entropy smax, xbot:   ', smax, s[0]
;
;stop ; --------------------------------------------------------------
;
filename   =model + '.sta' & fileform='unformatted' & fileconv='ieee_4'
;version    ='sphereintegraterhoei 2003-03-10'
;version    ='sphereintegrate 2004-03-22'
;version    ='sphereintegrate 2012-05-14'
;version    ='sphereintegrate 2014-07-21'
version    ='sphereintegrate 2018-04-19'
description=['Spherical atmosphere                            ', $
             strtrim(systime(0),2), $
             'Teff=                ' + string(Teff                , format='(E13.6)'), $
             'mass_star=           ' + string(mass_star           , format='(E13.6)'), $
             'Radius=              ' + string(Radius              , format='(E13.6)'), $
             'r0_grav=             ' + string(r0_grav             , format='(E13.6)'), $
             'r1_grav=             ' + string(r1_grav             , format='(E13.6)'), $
             'r2_grav=             ' + string(r2_grav             , format='(E13.6)'), $
             'gravCorePotDepth=    ' + string(gravCorePotDepth    , format='(E13.6)'), $
             'R_top=               ' + string(Rtop                , format='(E13.6)'), $
             'P_top=               ' + string(Ptop                , format='(E13.6)'), $
             'v_ampl=              ' + string(vampl               , format='(E13.6)'), $
             's_limfac=            ' + string(facsmax             , format='(E13.6)'), $
             's_inflow=            ' + string(smax                , format='(E13.6)'), $
             'luminositypervolume= ' + string(luminositypervolume , format='(E13.6)') $
            ]
history    =['Produced by sphereintegrate.pro: ' + strtrim(systime(0),2)]
;
; --- Physical units ---
unit_time='s'
unit_x1  ='cm'
unit_x2  =unit_x1
unit_x3  =unit_x1
unit_rho ='g/cm^3'
unit_v   ='cm/s'
unit_e   ='erg/g'
;
; --- Box size ---
;m1=-((npoint+1)/2)
m1=-(npoint/2)
n1= m1+npoint-1
;m2=-((npoint+1)/2)
m2=-(npoint/2)
n2= m2+npoint-1
;m2=0
;n2=0
;m3=-((npoint+1)/2)
m3=-(npoint/2)
n3= m3+npoint-1
;
; --- Time information ---
itime=0
time=0.0
out_time_full_last=-1.0E20
;
; --- Axes ---
xfac=boxsize/float(n1-m1+1)
xc1=xfac*(findgen(n1-m1+1)-float(n1-m1  )*0.5)
xb1=xfac*(findgen(n1-m1+2)-float(n1-m1+1)*0.5)
xc2=xfac*(findgen(n2-m2+1)-float(n2-m2  )*0.5)
xb2=xfac*(findgen(n2-m2+2)-float(n2-m2+1)*0.5)
xc3=xfac*(findgen(n3-m3+1)-float(n3-m3  )*0.5)
xb3=xfac*(findgen(n3-m3+2)-float(n3-m3+1)*0.5)
;
if (m2 eq n2) then begin &$
  distance=sqrt( $
           rebin(reform(xc1,n1-m1+1,1      ,1      ),n1-m1+1,n2-m2+1,n3-m3+1)^2 + $
           rebin(reform(xc3,1      ,1      ,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)^2 ) < Rtop &$
endif else begin &$
  distance=sqrt( $
           rebin(reform(xc1,n1-m1+1,1      ,1      ),n1-m1+1,n2-m2+1,n3-m3+1)^2 + $
           rebin(reform(xc2,1      ,n2-m2+1,1      ),n1-m1+1,n2-m2+1,n3-m3+1)^2 + $
           rebin(reform(xc3,1      ,1      ,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)^2 ) < Rtop &$
endelse
;
print, 'Interpolate density'
rho=interpol(rho1d,z1d,distance)
print, 'Interpolate internal energy'
ei =interpol(ei1d ,z1d,distance)
print, 'Set random velocities'
v1 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
v2 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
v3 =fltarr(     n1-m1+1, n2-m2+1, n3-m3+1)
noff=2
v1[noff:n1-m1-noff,noff:n2-m2-noff,noff:n3-m3-noff] = $
                       2.0*vampl*(randomu(seed,n1-m1+1-2*noff, n2-m2+1-2*noff, n3-m3+1-2*noff)-0.5)
v2[noff:n1-m1-noff,noff:n2-m2-noff,noff:n3-m3-noff] = $
                       2.0*vampl*(randomu(seed,n1-m1+1-2*noff, n2-m2+1-2*noff, n3-m3+1-2*noff)-0.5)
v3[noff:n1-m1-noff,noff:n2-m2-noff,noff:n3-m3-noff] = $
                       2.0*vampl*(randomu(seed,n1-m1+1-2*noff, n2-m2+1-2*noff, n3-m3+1-2*noff)-0.5)
v1=smooth(v1,2*noff+1)
v2=smooth(v2,2*noff+1)
v3=smooth(v3,2*noff+1)
;v1=vampl*fin06.z.v1
;v2=vampl*fin06.z.v2
;v3=vampl*fin06.z.v3
rho=reform(rho, n1-m1+1, n2-m2+1, n3-m3+1)
ei =reform(ei , n1-m1+1, n2-m2+1, n3-m3+1)
v1 =reform(v1 , n1-m1+1, n2-m2+1, n3-m3+1)
v2 =reform(v2 , n1-m1+1, n2-m2+1, n3-m3+1)
v3 =reform(v3 , n1-m1+1, n2-m2+1, n3-m3+1)

;rhd_manipulate_spinup, 1800.0, m1,n1, m2,n2, m3,n3, xc1,xc2,xc3, xb1,xb2,xb3, rho, ei, v1,v2,v3

help, xc1, xb1, xc2, xb2, xc3, xb3, rho, ei, v1, v2, v3
;
print, 'Envelope mass:       ', 1.0D0 * total(rho) * double(xfac)^3 / !con.MSun, ' M_Sun'
print, 'Envelope mass (seq): ', 1.0D0 * max(seqavg('massintdownb',seq))/!con.Msun, ' M_Sun'
print, 'dx:                  ', xb1[1]-xb1[0], ' cm'
;
stop ; --------------------------------------------------------------
;
; --- Write data into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
   -1, ncout, file=outdisk + filename, form=fileform, conv=fileconv, $
   ndes=n_elements(description), description=description, $
   nhis=n_elements(history), history=history, $
   version=version, $
   unit_time=unit_time, $
   unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
   unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
   m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
   itime=itime, time=time, dtime=dtime, $
   out_time_full_last=out_time_full_last, out_time_mean_last=out_time_full_last, $
   xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
   rho=rho, v1=v1, v2=v2, v3=v3, ei=ei
;
print, 'Data written into file ''', filename, ''' in UIO format for RHD'
;
;rhd_wrboxstrat, $
;  filename=filename, fileform=fileform, fileconv=fileconv, $
;  description=description, $
;  history=history, $
;  version=version, $
;  zarr=z, rhoarr=rho, eiarr=ei
;
; end ; sphereintegrate_start
