; pro START
;
;+
; NAME:
;   START
;
; PURPOSE:
;   Initialization
;
; CATEGORY:
;   Initialization
;
; CALLING SEQUENCE:
;   @start
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; KEYWORD PARAMETERS:
;   None
;
; SIDE EFFECTS:
;   The new system variables !CTV and !CON are created.
;   The color table is modified.
;   The number of available colors is set.
;   Mofify !X.MARGIN.
;
; SUBROUTINES:
;   setctv - Create new system variable !CTV and modify color table.
;   setcon - Create new system variable !CON with some usefull constants.
;
; PROCEDURE:
;   Straightforward:
;
; EXAMPLE:
;   @start
;
; MODIFICATION HISTORY:
;   25. 2.94 Written by Bernd Freytag, Astrophysik Kiel
;    9. 3.95 (B.F.) Path expansion
;    9. 5.95 (B.F.) Modification for Garching
;   11.10.96 (B.F.) Path for uio added
;    5. 2.97 (B.F.) Version for 'IDL' and 'WAVE'
;   16.10.97 (B.F.) Initialize uio-routines
;   24.07.98 (B.F.) Copenhagen version
;    9.10.98 (B.F.) setctv and setcon are now procedures
;   24. 3.05 (B.F.) idl/wrk changed to idlwrk
;   15. 6.05 (B.F.) Save plot settings
;-
;
; --- Try to determine language ---
if (n_elements(!X.TICKV) eq 150) then langua='WAVE' else langua='IDL'
;
; --- Set number of colors, define color table ---
;device, pseudo=8
window, 0, xsize=100, ysize=100
;window, 0, color=220, xsize=100, ysize=100
;window, 0, color=12, xsize=100, ysize=100
;window, 0, color=2, xsize=100, ysize=100
wdelete, 0
@setctv_idl4
;@setctv_gdl
;
; --- Define physical constants ---
setcon
;
; --- Change default value of graphic system variable ---
!X.MARGIN=[12,3]
;
; --- Add UIO IDL directory to search path ---
;addpath=expand_path('+' + getenv('UIOPATH') + '/idl') &$
;print, addpath
;if strtrim(addpath,2) ne '' then !path=addpath + ':' + !path
;
; --- Add user IDL directory to search path ---
;addpath=expand_path('+' + getenv('HOME') + '/idl') &$
;print, addpath
;if strtrim(addpath,2) ne '' then !path=addpath + ':' + !path
;
; delvar, addpath
;
; --- Goto IDL home directory ---
cd, getenv('HOME') + '/idlwrk'
;
; --- Initialize uio-routines ---
uio_init, progrm='by hand'
;
; --- Save last 200 commands in buffer (has to be in auto startup file) ---
;!EDIT_INPUT=200
;
; --- Save plot settings ---
Porg=!P & Xorg=!X & Yorg=!Y
;!P=Porg & !X=Xorg & !Y=Yorg
;
; end ; START
