; pro stratintegrate_start
;
;----------------------------------------------------------------------------------------------------
;+
; NAME:
;   stratinstart ('strat_integrate_start')
;
; PURPOSE:
;   Integrate an adiabatically stratified atmosphere by calling stratintegrate.
;
; CATEGORY:
;   Computations (Special)
;
; CALLING SEQUENCE:
;   @stratintegrate_start
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   tabinter_rdcoeff;
;   stratintegrate:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   Before first use type
;     delvar, init
;
; PROCEDURE:
;   Call stratintegrate
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2014-09-21 (B.F. Uppsala) Adapted from greyintegrate_start
;-
;---------------------------------------------------------------------------------------------------
;
; --- Define EOS path ---
eosdisk=getenv('HOME') + '/for/eos/dat/'
;
; --- Define model data ---
model='d1strat01blob00'
eosfile='eos_mm00_l6.eos' & gasname='eos'
logg=4.5
eiTop=-2.74e11
rhoTop=1.0e-7
n1=1
n2=1
n3=160
dx3=-28.0E+05
aspect=1.0
;
nzFac=10L
;
; --- Load EOS and opacity data ---
if (n_elements(init) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + eosfile, EOS &$
  init=1 &$
endif
;
; --- Integrate grey statrification ---
print, 'Start integration'
stratintegrate, eiTop, rhoTop, 0.0, $
  ei, rho, z, $
  dz=dx3/nzFac, grav=10.0^logg, npoint=nzFac*(n3-1)+1, EOS=EOS
print, 'Integration done'
;
; --- Use only coarse z scale ---
ei =ei[ 0:*:nzfac]
rho=rho[0:*:nzfac]
z  =z[  0:*:nzfac]
;
; --- Compute EOS quantities ---
eosinter, rho, ei, P, dPdrho, dPdei, T, dTdei, s, EOS=EOS
;
; --- Plot some quantities ---
!P.MULTI=[0,2,2,0,0]
plotlines, z, alog10(rho), psym=0, ytitle='log(rho)'
plotlines, z, ei, psym=0, ytitle='ei'
plotlines, z, T, psym=0, ytitle='T'
plotlines, z, s, psym=0, ytitle='s'
!P.MULTI=0
;
; --- Write output stratification ---
filename=model + '.sta' & fileform='unformatted' & fileconv='ieee_4'
version='stratintegrate 2014-09-21'
description=['Adiabatic hydrostatic atmosphere', $
             strtrim(systime(0),2)]
history=['Produced by stratintegrate.pro: '+strtrim(systime(0),2)]
;
dx1=aspect*abs(dx3)
dx2=dx1
help, dx1, dx2, dx3
;
rhd_wrboxstrat, $
  filename=filename, fileform=fileform, fileconv=fileconv, $
  description=description, $
  history=history, $
  version=version, $
  zarr=z, rhoarr=rho, eiarr=ei, vampl=0.0, $
  dx1=dx1, dx2=dx2, dtime=dtime, n1=n1, n2=n2
;
ful=uio_dataset_rd(filename,/headread)
eosbox, ful, EOS=EOS
;
; end ; greyintegrate_start
