; pro sketch_hd2dtrans
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sketch_hd2dtrans ('sketch_hydrodynamics_2_dimension_transversal')
;
; PURPOSE:
;   Sketch 2D advection
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @sketch_hd2dtrans
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Before first use type
;     delvar, external_control
;
; EXAMPLE:
;   @sketch_hd2dtrans
;
; MODIFICATION HISTORY:
;   2012-12-12 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

; ================================
if (not keyword_set(external_control)) then begin &$
  ; --- Choose device: 'x', 'ps', 'eps' ---
  dev='x' &$
  jpeg_flag=0 &$
  png_flag=0 &$
  ;
endif
; ================================


file_id='sketch_hd2dtrans'
jpeg_file=file_id + '.jpg'
png_file =file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Color table ---
normct

aspect_ratio=1.0
window_xsize=700
ps_xsize=10.4
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'x' &$
  xsize=window_xsize & ysize=round(window_xsize*aspect_ratio) &$
  win_id=0 &$
  window, win_id, xsize=xsize, ysize=ysize, retain=2 &$
  !P.FONT=-1 &$
  !P.CHARSIZE=2.1 &$
  !P.SYMSIZE=2.5 &$
endif else if (strupcase(dev) eq 'PS') then begin & $
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endif else begin &$
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /encapsulated, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endelse

; --- Plot settings ---
linecolor=[!ctv.red, !ctv.blue, !ctv.black]
linethick=3
thick_arrow=2
!P.BACKGROUND=!ctv.white
!P.COLOR=!ctv.black
!X.MARGIN=[1,1] & !Y.MARGIN=[1,1]
!P.MULTI=[0,2,2,0,0]
!P.THICK=linethick
!X.TICKLEN=0.0000001
!Y.TICKLEN=0.0000001
!X.TICKNAME=replicate(' ', 8)
!Y.TICKNAME=replicate(' ', 8)

; --- Define initial array ---
nx=12
ny=nx
q0=fltarr(nx,ny)
q0[3:nx-2:4,3:ny-2:4]=1.0


; --- Shift up ---
q=q0
ix0=findgen(nx)
iy0=[0.0,findgen(ny-1)+0.5]
ix=ix0
iy=iy0
q=bilinear(q, ix, iy)
q=bilinear(q, ix, iy)
q=q[1:nx-2,1:ny-2]

plotfield, /tv, /t_sample, -q, $
  drange=[-1.0,0.0], $
  xtitle='', ytitle='', $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Shift diagonally ---
q=q0
ix0=[0.0,findgen(nx-1)+0.5]
iy0=[0.0,findgen(ny-1)+0.5]
ix=ix0
iy=iy0
q=bilinear(q, ix, iy)
q=bilinear(q, ix, iy)
q=q[1:nx-2,1:ny-2]

plotfield, /tv, /t_sample, -q, $
  drange=[-1.0,0.0], $
  xtitle='', ytitle='', $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Unshifted array ---
q=q0
q=q[1:nx-2,1:ny-2]

plotfield, /tv, /t_sample, -q, $
  drange=[-1.0,0.0], $
  xtitle='', ytitle='', $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Shift to the right ---
q=q0
ix0=[0.0,findgen(nx-1)+0.5]
iy0=findgen(ny)
ix=ix0
iy=iy0
q=bilinear(q, ix, iy)
q=bilinear(q, ix, iy)
q=q[1:nx-2,1:ny-2]

plotfield, /tv, /t_sample, -q, $
  drange=[-1.0,0.0], $
  xtitle='', ytitle='', $
  dev=dev, psformat='nochange', /psnoinit, /noclose


; --- Arrows ---
arrow1, 0.42, 0.25, 0.58, 0.25, $
  /norm, thick=thick_arrow, dev=dev
arrow1, 0.25, 0.42, 0.25, 0.58, $
  /norm, thick=thick_arrow, dev=dev
arrow1, 0.42, 0.42, 0.58, 0.58, $
  /norm, thick=thick_arrow, dev=dev



; === Finish plot ===
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpeg_flag) then begin &$
    write_jpeg, jpeg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

!P=Pbak & !X=Xbak & !Y=Ybak

; end ; sketch_hd2dtrans
