; pro sketch_hrd
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sketch_hrd ('sketch_Hertzsprung-Russell_diagram')
;
; PURPOSE:
;   Draw a sketch to illustrate the HRD
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @sketch_hrd
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Before first use type
;     delvar, external_control
;
; EXAMPLE:
;   @sketch_hrd
;
; SOURCES:
;   HRD: http://www.atlasoftheuniverse.com/hr.html
;
; MODIFICATION HISTORY:
;   2009-05-19 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

; ================================
if (not keyword_set(external_control)) then begin &$
  ; --- Choose device: 'x', 'ps', 'eps' ---
  dev='x' &$
  jpeg_flag=0 &$
  png_flag=0 &$
  ;
  pid='' & plotAbsSize_flag=0 & plotRelSize_flag=0 & plotRelCephSize_flag=0 &plotIsoRadius_flag=0 &$
  ;pid='_AbsSize' & plotAbsSize_flag        =1 &$
  ;pid='_RelSize' & plotRelSize_flag        =1 &$
  ;pid='_RelCephSize' & plotRelCephSize_flag=1 &$
  pid='_IsoRadius' & plotIsoRadius_flag    =1 &$
  ;
  plotHayashi_flag=0 &$
  plotInstStrip_flag=1 &$
endif
; ================================

; --- http://en.wikipedia.org/wiki/Main_sequence ---
; Table of main sequence stellar parameters[20]
; Stellar
; Class Radius	Mass	Luminosity	Temperature	Examples[21]
; R/Rsun M/Msun	L/Lsun	K
; O5	18	40	500,000	38,000	Zeta Puppis
; B0	7.4	18	20,000	30,000	Phi1 Orionis
; B5	3.8	6.5	800	16,400	Pi Andromedae A
; A0	2.5	3.2	80	10,800	Alpha Coronae Borealis A
; A5	1.7	2.1	20	8,620	Beta Pictoris
; F0	1.4	1.7	6	7,240	Gamma Virginis
; F5	1.2	1.29	2.5	6,540	Eta Arietis
; G0	1.05	1.10	1.26	6,000	Beta Comae Berenices
; G2	1.00	1.00	1.00	5,920	Sun
; G5	0.93	0.93	0.79	5,610	Alpha Mensae
; K0	0.85	0.78	0.40	5,150	70 Ophiuchi A
; K5	0.74	0.69	0.16	4,640	61 Cygni A
; M0	0.63	0.47	0.063	3,920	Gliese 185
; M5	0.32	0.21	0.0079	3,120	EZ Aquarii A
; M8	0.13	0.10	0.0008	-	Van Biesbroeck's star[24]
MSwiki=[ $
create_struct("Type", "O5",  "Radius",  18.0,  "Mass", 40.0 ,  "Luminosity", 500000.0   ,  "Teff", 38000.0,  "example", "Zeta Puppis"), $
create_struct("Type", "B0",  "Radius",  7.4,   "Mass", 18.0 ,  "Luminosity",  20000.0   ,  "Teff", 30000.0,  "example", "Phi1 Orionis"), $
create_struct("Type", "B5",  "Radius",  3.8,   "Mass",  6.5 ,  "Luminosity",    800.0   ,  "Teff", 16400.0,  "example", "Pi Andromedae A"), $
create_struct("Type", "A0",  "Radius",  2.5,   "Mass",  3.2 ,  "Luminosity",     80.0   ,  "Teff", 10800.0,  "example", "Alpha Coronae Borealis A"), $
create_struct("Type", "A5",  "Radius",  1.7,   "Mass",  2.1 ,  "Luminosity",     20.0   ,  "Teff",  8620.0,  "example", "Beta Pictoris"), $
create_struct("Type", "F0",  "Radius",  1.4,   "Mass",  1.7 ,  "Luminosity",      6.0   ,  "Teff",  7240.0,  "example", "Gamma Virginis"), $
create_struct("Type", "F5",  "Radius",  1.2,   "Mass",  1.29,  "Luminosity",      2.5   ,  "Teff",  6540.0,  "example", "Eta Arietis"), $
create_struct("Type", "G0",  "Radius",  1.05,  "Mass",  1.10,  "Luminosity",      1.26  ,  "Teff",  6000.0,  "example", "Beta Comae Berenices"), $
;create_struct("Type", "G2",  "Radius",  1.00,  "Mass",  1.00,  "Luminosity",      1.00  ,  "Teff",  5920.0,  "example", "Sun"), $
create_struct("Type", "G5",  "Radius",  0.93,  "Mass",  0.93,  "Luminosity",      0.79  ,  "Teff",  5610.0,  "example", "Alpha Mensae"), $
create_struct("Type", "K0",  "Radius",  0.85,  "Mass",  0.78,  "Luminosity",      0.40  ,  "Teff",  5150.0,  "example", '70 Ophiuchi A'), $
create_struct("Type", "K5",  "Radius",  0.74,  "Mass",  0.69,  "Luminosity",      0.16  ,  "Teff",  4640.0,  "example", '61 Cygni A'), $
create_struct("Type", "M0",  "Radius",  0.63,  "Mass",  0.47,  "Luminosity",      0.063 ,  "Teff",  3920.0,  "example", "Gliese 185"), $
create_struct("Type", "M5",  "Radius",  0.32,  "Mass",  0.21,  "Luminosity",      0.0079,  "Teff",  3120.0,  "example", "EZ Aquarii A")]
;create_struct("Type", "M8",  "Radius",  0.13,  "Mass",  0.10,  "Luminosity",      0.0008,  "Teff",  1000.0,  "example", "Van Biesbroeck's star")]

; --- Individual stars ---
stars_Sun= $
  create_struct("Type", "G2",  "Radius",   1.00,  "Mass",  1.00,  "Luminosity",      1.00  ,  "Teff",  5775.0)
stars_Betelgeuse= $
  create_struct("Type", "MI",  "Radius", 800.00,  "Mass", 15.00,  "Luminosity",      0.00  ,  "Teff",  3700.0)
stars_Betelgeuse.Luminosity=stars_Sun.Luminosity * (stars_Betelgeuse.Radius/stars_Sun.Radius)^2 * (stars_Betelgeuse.Teff/stars_Sun.Teff)^4
stars_AGB= $
  create_struct("Type", "M",  "Radius", 333.00,  "Mass", 1.50,  "Luminosity",      0.00  ,  "Teff",  2937.0)
stars_AGB.Luminosity=stars_Sun.Luminosity * (stars_AGB.Radius/stars_Sun.Radius)^2 * (stars_AGB.Teff/stars_Sun.Teff)^4

; --- My Cepheid + A-type star list ---
;CepheidBFtemplate=create_struct("Radius",  0.0,  "Mass", 0.0 ,  "Luminosity", 1000.0   ,  "Teff", 0.0, "logg", 2.0)
;CepheidsBF=replicate(CepheidBFtemplate, 3)
;CepheidsBF.Teff=[5000.0, 5600.0, 6200.0]
;CepheidsBF.Radius=sqrt(CepheidsBF.Luminosity/(CepheidsBF.Teff/stars_Sun.Teff)^4)
CepheidsBF=[ $
create_struct("Radius",  0.0,  "Mass", 0.0 ,  "Luminosity", 1000.0,  "Teff", 5300.0, "logg", 2.0, "g", 0.0), $
create_struct("Radius",  0.0,  "Mass", 0.0 ,  "Luminosity",    5.0,  "Teff", 7300.0, "logg", 4.3, "g", 0.0)]
CepheidsBF.Radius=sqrt(CepheidsBF.Luminosity/(CepheidsBF.Teff/stars_Sun.Teff)^4)
CepheidsBF.g=10.0^CepheidsBF.logg
CepheidsBF.Mass=CepheidsBF.g*(CepheidsBF.Radius*!con.Rsun)^2/!con.G/!con.Msun
stars_Ceph53=CepheidsBF[0]

; --- Kippenhahn & Weigert 1991 "Stellar Structure and Evolution", Fig. 22.1 ---
ZAMSkw=[ $
create_struct('Mass',  0.1 , 'logL',  -3.03, 'Luminosity', 0.0,  'logTeff',  3.423, 'Teff', 0.0), $
create_struct('Mass',  0.2 , 'logL',  -2.4 , 'Luminosity', 0.0,  'logTeff',  3.47 , 'Teff', 0.0), $
create_struct('Mass',  0.3 , 'logL',  -2.03, 'Luminosity', 0.0,  'logTeff',  3.50 , 'Teff', 0.0), $
create_struct('Mass',  0.4 , 'logL',  -1.75, 'Luminosity', 0.0,  'logTeff',  3.525, 'Teff', 0.0), $
create_struct('Mass',  0.5 , 'logL',  -1.55, 'Luminosity', 0.0,  'logTeff',  3.548, 'Teff', 0.0), $
create_struct('Mass',  0.65, 'logL',  -0.96, 'Luminosity', 0.0,  'logTeff',  3.63 , 'Teff', 0.0), $
create_struct('Mass',  1.0 , 'logL',  -0.2 , 'Luminosity', 0.0,  'logTeff',  3.74 , 'Teff', 0.0), $
create_struct('Mass',  1.3 , 'logL',   0.4 , 'Luminosity', 0.0,  'logTeff',  3.82 , 'Teff', 0.0), $
create_struct('Mass',  2.0 , 'logL',   1.2 , 'Luminosity', 0.0,  'logTeff',  3.96 , 'Teff', 0.0), $
create_struct('Mass',  3.0 , 'logL',   1.85, 'Luminosity', 0.0,  'logTeff',  4.085, 'Teff', 0.0), $
create_struct('Mass',  5.0 , 'logL',   2.72, 'Luminosity', 0.0,  'logTeff',  4.23 , 'Teff', 0.0), $
create_struct('Mass',  7.0 , 'logL',   3.25, 'Luminosity', 0.0,  'logTeff',  4.32 , 'Teff', 0.0), $
create_struct('Mass', 10.0 , 'logL',   3.75, 'Luminosity', 0.0,  'logTeff',  4.403, 'Teff', 0.0), $
create_struct('Mass', 20.0 , 'logL',   4.65, 'Luminosity', 0.0,  'logTeff',  4.54 , 'Teff', 0.0), $
create_struct('Mass', 30.0 , 'logL',   5.1,  'Luminosity', 0.0,  'logTeff',  4.61 , 'Teff', 0.0), $
create_struct('Mass', 40.0 , 'logL',   5.3 , 'Luminosity', 0.0,  'logTeff',  4.64 , 'Teff', 0.0)]
ZAMSkw.Luminosity=10.0^ZAMSkw.logL
ZAMSkw.Teff=10.0^ZAMSkw.logTeff

; --- Hayashi tracks: Kippenhahn & Weigert 1991 "Stellar Structure and Evolution", Fig. 24.3 (Ezer, Cameron 1967) ---
HayashiKW=[ $
create_struct('Mass',  1.0, 'logL', [1.0 , 2.8], 'Luminosity', [0.0,0.0],  'logTeff', [3.68, 3.58], 'Teff', [0.0,0.0]), $
create_struct('Mass',  2.0, 'logL', [1.3 , 3.4], 'Luminosity', [0.0,0.0],  'logTeff', [3.70, 3.57], 'Teff', [0.0,0.0]), $
create_struct('Mass',  5.0, 'logL', [2.7 , 3.9], 'Luminosity', [0.0,0.0],  'logTeff', [3.69, 3.59], 'Teff', [0.0,0.0]), $
create_struct('Mass', 10.0, 'logL', [3.35, 4.5], 'Luminosity', [0.0,0.0],  'logTeff', [3.69, 3.58], 'Teff', [0.0,0.0])]
HayashiKW.Luminosity=10.0^HayashiKW.logL
HayashiKW.logTeff=HayashiKW.logTeff-0.06
HayashiKW.Teff=10.0^HayashiKW.logTeff

; --- Instability strip; Cox 1980 "Theory of Stellar Pulsation", Fig. 3.1 ---
InstStripCox=create_struct('Mv', [-6.0, 3.0, 3.0, -6.0], 'Luminosity', [0.0,0.0,0.0,0.0], 'Teff', [5400.0, 7100.0, 8500.0, 6000.0])
; --- Unsoeld Eq. (4.4.23) ---
InstStripCox.Luminosity=10.0^((4.74-InstStripCox.Mv)/2.5)

file_id='sketch_hrd' + pid
jpeg_file=file_id + '.jpg'
png_file =file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Color table ---
normct

aspect_ratio=1.0
window_xsize=700
ps_xsize=10.0
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'x' &$
  xsize=window_xsize & ysize=round(window_xsize*aspect_ratio) &$
  win_id=0 &$
  window, win_id, xsize=xsize, ysize=ysize, retain=2 &$
  !P.FONT=-1 &$
  !P.CHARSIZE=2.1 &$
  !P.SYMSIZE=2.5 &$
endif else if (strupcase(dev) eq 'PS') then begin & $
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endif else begin &$
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /encapsulated, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endelse

; --- Plot settings ---
linecolor=[!ctv.red, !ctv.blue, !ctv.black]
linethick=3
thick_arrow=2
!P.BACKGROUND=!ctv.white
!P.COLOR=!ctv.black
!X.MARGIN=[6,1] &!Y.MARGIN=[4,1]
!P.THICK=linethick
color_MS=!ctv.black
color_Betelgeuse=!ctv.red
color_AGB       =!ctv.red


; === Quantities to plot ===

plotlines, alog10(MSwiki.Teff), alog10(MSwiki.Luminosity), /nodata, $
  xrange=[ 4.01, 3.39], $
  drange=[-2.01, 6.3], $
  xtitle='log(T!Deff!N/K)', xstyle=0, xthick=2, $
  ytitle='log(L/L!DSun!N)', ystyle=1, ythick=2, $
  linecolor=color_MS, thick=1, $
  dev=dev, psformat='nochange', /psnoinit, /noclose

; --- Instability strip ---
if (plotInstStrip_flag) then begin &$
  polyfill, alog10(InstStripCox.Teff), alog10(InstStripCox.Luminosity), $
    /data, color=!ctv.blue &$
  ;plotlines, alog10(CepheidsBF.Teff), alog10(CepheidsBf.Luminosity), /over, $
  ;  linecolor=!ctv.black, psym=psymuser(102, /fill), symsize=1.0*!P.SYMSIZE, $
  ;  dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

; --- Main sequence ---
n=100
logTeff=findgen(n)/float(n-1)*(max(alog10(ZAMSkw.Teff))-min(alog10(ZAMSkw.Teff))) + min(alog10(ZAMSkw.Teff))
logL   =interpol(alog10(ZAMSkw.Luminosity), alog10(ZAMSkw.Teff), logTeff, /quadratic)
plotlines, logTeff, logL, /over, $
  linecolor=color_MS, thick=2, psym=0, $
  dev=dev, psformat='nochange', /psnoinit, /noclose

; --- Hayashi tracks ---
if (plotHayashi_flag) then begin &$
  plotlines, alog10(HayashiKW.Teff), alog10(HayashiKW.Luminosity), /over, $
    linecolor=!ctv.black, thick=2, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

; --- Sun ---
plotlines, [alog10(stars_Sun.Teff)], [alog10(stars_Sun.Luminosity)], /over, $
  linecolor=!ctv.yellow, psym=psymuser(102, /fill), symsize=1.4*!P.SYMSIZE, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, [alog10(stars_Sun.Teff)], [alog10(stars_Sun.Luminosity)], /over, $
  linecolor=!ctv.black, thick=2, psym=psymuser(104, /fill), symsize=1.4*!P.SYMSIZE, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
; --- Betelgeuse ---
plotlines, [alog10(stars_Betelgeuse.Teff)], [alog10(stars_Betelgeuse.Luminosity)], /over, $
  linecolor=color_Betelgeuse, psym=psymuser(380, /fill), symsize=2.2*!P.SYMSIZE, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
; --- AGB ---
plotlines, [alog10(stars_AGB.Teff)], [alog10(stars_AGB.Luminosity)], /over, $
  linecolor=color_AGB, psym=psymuser(364, /fill), symsize=1.5*!P.SYMSIZE, $
  dev=dev, psformat='nochange', /psnoinit, /noclose

if (plotAbsSize_flag) then begin &$
  ; --- Absolute iso-granule-sizes ---
  ; --- Solar mass ---
  nTeffAr=6 &$
  nSizeAr=4 &$
  logTeffArSun=rebin(reform(findgen(nTeffAr)/float(nTeffAr-1)*(-0.43)+3.95, nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArSun=rebin(reform(findgen(nSizeAr)/float(nSizeAr-1)*  3.0 -1.0, 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArSun=logSizeArSun + 3.0*(logTeffArSun-alog10(stars_Sun.Teff)) + alog10(stars_Sun.Mass) &$
  plotlines, logTeffArSun, logLumiArSun, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArSun[0,*]),format='(F6.1)'),2), $
    linecolor=!ctv.black, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
  ; --- Betelgeuse mass ---
  nTeffAr=5 &$
  nSizeAr=4 &$
  logTeffArBetelgeuse=rebin(reform(findgen(nTeffAr)/float(nTeffAr-1)*(-0.36)+3.85, nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArBetelgeuse=rebin(reform(findgen(nSizeAr)/float(nSizeAr-1)*  3.0 +1.0, 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArBetelgeuse=logSizeArBetelgeuse + 3.0*(logTeffArBetelgeuse-alog10(stars_Betelgeuse.Teff)) + alog10(stars_Betelgeuse.Mass) &$
  plotlines, logTeffArBetelgeuse, logLumiArBetelgeuse, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArBetelgeuse[0,*]),format='(I5)'),2), $
    linecolor=color_Betelgeuse, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

if (plotRelSize_flag) then begin &$
  ; --- Relative iso-granule-sizes ---
  ; --- Solar mass ---
  nTeffAr=6 &$
  nSizeAr=4 &$
  SizeGranSun=1.0E+08 &$
  logTeffArSun=rebin(reform( findgen(nTeffAr)     /float(nTeffAr-1)*(-0.43)+3.95                        , nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArSun=rebin(reform((findgen(nSizeAr)-1.0)/float(nSizeAr-1)*  1.5 +alog10(SizeGranSun/!con.Rsun), 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArSun=2.0*(logSizeArSun-alog10(SizeGranSun/!con.Rsun)) + 2.0*(logTeffArSun-alog10(stars_Sun.Teff)) + 2.0*alog10(stars_Sun.Mass) &$
  plotlines, logTeffArSun, logLumiArSun, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArSun[0,*]),format='(F6.4)'),2), $
    linecolor=!ctv.black, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
  ; --- Betelgeuse mass ---
  nTeffAr=5 &$
  nSizeAr=4 &$
  logTeffArBetelgeuse=rebin(reform(findgen(nTeffAr)/float(nTeffAr-1)*(-0.36)+3.85, nTeffAr,1), nTeffAr, nSizeAr) &$
;  logSizeArBetelgeuse=rebin(reform((findgen(nSizeAr))/float(nSizeAr-1)*  1.5 +alog10(SizeGranSun/!con.Rsun), 1,nSizeAr), nTeffAr, nSizeAr) &$
  logSizeArBetelgeuse=rebin(reform((findgen(nSizeAr))/float(nSizeAr-1)*  1.5 -2.5                          , 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArBetelgeuse=2.0*(logSizeArBetelgeuse-alog10(SizeGranSun/!con.Rsun)) + 2.0*(logTeffArBetelgeuse-alog10(stars_Betelgeuse.Teff)) + 2.0*alog10(stars_Betelgeuse.Mass) &$
  plotlines, logTeffArBetelgeuse, logLumiArBetelgeuse, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArBetelgeuse[0,*]),format='(F6.4)'),2), $
    linecolor=color_Betelgeuse, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

if (plotRelCephSize_flag) then begin &$
  ; --- Relative iso-granule-sizes (relative to 5300K Cepheid model) ---
  ; --- Cepheid mass ---
  nTeffAr=6 &$
  nSizeAr=4 &$
  SizeGranCeph53=1.0E+12 &$
  logTeffArCeph53=rebin(reform( findgen(nTeffAr)     /float(nTeffAr-1)*(-0.42)+3.95                           , nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArCeph53=rebin(reform((findgen(nSizeAr)-1.0)/float(nSizeAr-1)*  1.5 +alog10(SizeGranCeph53/(!con.Rsun*stars_Ceph53.Radius)), 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArCeph53=2.0*(logSizeArCeph53-alog10(SizeGranCeph53/(!con.Rsun*stars_Ceph53.Radius))) + 2.0*(logTeffArCeph53-alog10(stars_Ceph53.Teff)) + alog10(stars_Ceph53.Luminosity) &$
  plotlines, logTeffArCeph53, logLumiArCeph53, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArCeph53[0,*]),format='(F7.4)'),2), $
    linecolor=!ctv.black, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
  ; --- Betelgeuse mass ---
  nTeffAr=5 &$
  nSizeAr=3 &$
  logTeffArBetelgeuse=rebin(reform(findgen(nTeffAr)/float(nTeffAr-1)*(-0.36)+3.85, nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArBetelgeuse=rebin(reform((findgen(nSizeAr))/float(nSizeAr-1)*  1.0-0.2                          , 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArBetelgeuse=2.0*(logSizeArBetelgeuse-alog10(SizeGranCeph53/(!con.Rsun*stars_Ceph53.Radius))) + 2.0*(logTeffArBetelgeuse-alog10(stars_Ceph53.Teff)) + 2.0*(alog10(stars_Betelgeuse.Mass)-alog10(stars_Ceph53.Mass)) + alog10(stars_Ceph53.Luminosity) &$
  plotlines, logTeffArBetelgeuse, logLumiArBetelgeuse, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArBetelgeuse[0,*]),format='(F6.4)'),2), $
    linecolor=color_Betelgeuse, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif

if (plotIsoRadius_flag) then begin &$
  ; --- Stellar radius ---
  nTeffAr=6 &$
  nSizeAr=4 &$
  logTeffArSun=rebin(reform(findgen(nTeffAr)/float(nTeffAr-1)*(-1.92)+5.4, nTeffAr,1), nTeffAr, nSizeAr) &$
  logSizeArSun=rebin(reform(findgen(nSizeAr)                            , 1,nSizeAr), nTeffAr, nSizeAr) &$
  logLumiArSun=2.0*logSizeArSun + 4.0*(logTeffArSun-alog10(stars_Sun.Teff)) &$
  plotlines, logTeffArSun, logLumiArSun, /over, $
    label=1, legitems=' '+strtrim(string(reform(10.0^logSizeArSun[0,*]),format='(I6)'),2), $
    linecolor=!ctv.black, thick=1, $
    dev=dev, psformat='nochange', /psnoinit, /noclose &$
endif


;; --- Bins ---
;plotlines, alog10(radiusb[4:*]), qucb[4:*], /nodata, $
;  xrange=[min(alog10(radiusb)-0.2), max(alog10(radiusb))], $
;  drange=[0,max(qucb)*1.16], $
;  xtitle='log(radius)', xstyle=0, xthick=2, $
;  ytitle='density'    , ystyle=1, ythick=2, $
;  linecolor=color_dust, thick=4, $
;  dev=dev, psformat='nochange', /psnoinit, /noclose
;polyfill, alog10(radiusb[4:*]), qucb[4:*], $
;  /data, color=!ctv.yellow
;plotlines, alog10(radiusb[4:*]), qucb[4:*], /over, $
;  xrange=[min(alog10(radiusb)-0.2), max(alog10(radiusb))], $
;  drange=[0,max(qucb)*1.16], $
;  xtitle='log(radius)', xstyle=0, xthick=2, $
;  ytitle='density'    , ystyle=1, ythick=2, $
;  linecolor=color_dust, thick=4, $
;  dev=dev, psformat='nochange', /psnoinit, /noclose
;;
;polyfill, alog10(radiusb[0:3]), qucb[0:3], $
;  /data, color=!ctv.blue
;plotlines, alog10(radiusb[0:3]), qucb[0:3], /over, $
;  linecolor=color_momomer, thick=4, $
;  dev=dev, psformat='nochange', /psnoinit, /noclose
;xyouts, alog10(radiusm), qucm+0.01, 'MOMOMERS', $
;  alignment=0.5, color=color_momomer, charsize=1.1*!P.CHARSIZE
;xyouts, alog10(radiusd[(nqucd)/2]), qucm+0.01, 'DUST', $
;  alignment=0.5, color=color_dust, charsize=1.1*!P.CHARSIZE
;
;; --- Nucleation ---
;y=qucd[0]-0.03
;arrow1, alog10(radiusm), y, alog10(radiusd[0]), y, $
;  /data, thick=thick_arrow, dev=dev, $
;  color=color_nucl
;xyouts, 0.5*(alog10(radiusm)+alog10(radiusd[0])), y+0.01, 'Nucleation', $
;  alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5
;xyouts, 0.5*(alog10(radiusm)+alog10(radiusd[0])), y+0.01, 'Nucleation', $
;  alignment=0.5, color=color_nucl, charsize=1.1*!P.CHARSIZE
;
;; --- Coagulation ---
;for i=0,nqucd-2 do begin &$
;  arrow1, alog10(radiusd[i]*1.15), 0.03, alog10(radiusd[i+1]/1.15), 0.03, $
;    /data, thick=thick_arrow, dev=dev, $
;    color=color_coag &$
;  if (i eq 2) then begin &$
;    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.04, 'Coagulation', $
;      alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5 &$
;    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.04, 'Coagulation', $
;      alignment=0.5, color=color_coag, charsize=1.1*!P.CHARSIZE &$
;   endif &$
;endfor
;
;; --- Condensation ---
;for i=0,nqucd-2 do begin &$
;  print, i &$
;  arrow1, alog10(radiusd[i]*1.15), 0.5*(qucd[i]+qucd[i+1]), alog10(radiusd[i+1]/1.15), 0.5*(qucd[i]+qucd[i+1]), $
;    /data, thick=thick_arrow, dev=dev, color=color_cond &$
;  arrow1, alog10(radiusd[i+1]/1.15), 0.5*(qucd[i]+qucd[i+1]), alog10(radiusd[i]*1.15), 0.5*(qucd[i]+qucd[i+1]), $
;    /data, thick=thick_arrow, dev=dev, color=color_cond &$
;  arrow1, alog10(radiusm), qucm-0.02, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.002, $
;    /data, thick=1, dev=dev, color=color_cond &$
;  arrow1, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.002, alog10(radiusm), qucm-0.02, $
;    /data, thick=1, dev=dev, color=color_cond &$
;  if (i eq 2) then begin &$
 ;   xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.01, 'Condensation', $
;      alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5 &$
;    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.01, 'Condensation', $
;      alignment=0.5, color=color_cond, charsize=1.1*!P.CHARSIZE &$
;   endif &$
;endfor


; === Finish plot ===
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpeg_flag) then begin &$
    write_jpeg, jpeg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

!P=Pbak & !X=Xbak & !Y=Ybak

; end ; sketch_hrd
