; pro sketch_hrd
;
;------------------------------------------------------------------------------
;+
; NAME:
;   sketch_hrd ('sketch_Hertzsprung-Russell_diagram')
;
; PURPOSE:
;   Draw a sketch to illustrate the HRD
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @sketch_hrd
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Before first use type
;     delvar, external_control
;
; EXAMPLE:
;   @sketch_hrd
;
; SOURCES:
;   HRD: http://www.atlasoftheuniverse.com/hr.html
;
; MODIFICATION HISTORY:
;   2009-05-19 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

; ================================
if (not keyword_set(external_control)) then begin &$
  ; --- Choose device: 'x', 'ps', 'eps' ---
  dev='x' &$
  jpeg_flag=0 &$
  png_flag=1 &$
  ;
endif
; ================================

; --- http://en.wikipedia.org/wiki/Main_sequence ---
; Table of main sequence stellar parameters[20]
; Stellar
; Class Radius	Mass	Luminosity	Temperature	Examples[21]
; R/Rsun M/Msun	L/Lsun	K
; O5	18	40	500,000	38,000	Zeta Puppis
; B0	7.4	18	20,000	30,000	Phi1 Orionis
; B5	3.8	6.5	800	16,400	Pi Andromedae A
; A0	2.5	3.2	80	10,800	Alpha Coronae Borealis A
; A5	1.7	2.1	20	8,620	Beta Pictoris
; F0	1.4	1.7	6	7,240	Gamma Virginis
; F5	1.2	1.29	2.5	6,540	Eta Arietis
; G0	1.05	1.10	1.26	6,000	Beta Comae Berenices
; G2	1.00	1.00	1.00	5,920	Sun
; G5	0.93	0.93	0.79	5,610	Alpha Mensae
; K0	0.85	0.78	0.40	5,150	70 Ophiuchi A
; K5	0.74	0.69	0.16	4,640	61 Cygni A
; M0	0.63	0.47	0.063	3,920	Gliese 185
; M5	0.32	0.21	0.0079	3,120	EZ Aquarii A
; M8	0.13	0.10	0.0008	-	Van Biesbroeck's star[24]
HRDwiki=[ $
create_struct("Type", "O5",  "Radius",  18.0,  "Mass", 40.0 ,  "Luminosity", 500000.0   ,  "Teff", 38000.0,  "example", "Zeta Puppis"), $
create_struct("Type", "B0",  "Radius",  7.4,   "Mass", 18.0 ,  "Luminosity",  20000.0   ,  "Teff", 30000.0,  "example", "Phi1 Orionis"), $
create_struct("Type", "B5",  "Radius",  3.8,   "Mass",  6.5 ,  "Luminosity",    800.0   ,  "Teff", 16400.0,  "example", "Pi Andromedae A"), $
create_struct("Type", "A0",  "Radius",  2.5,   "Mass",  3.2 ,  "Luminosity",     80.0   ,  "Teff", 10800.0,  "example", "Alpha Coronae Borealis A"), $
create_struct("Type", "A5",  "Radius",  1.7,   "Mass",  2.1 ,  "Luminosity",     20.0   ,  "Teff",  8620.0,  "example", "Beta Pictoris"), $
create_struct("Type", "F0",  "Radius",  1.4,   "Mass",  1.7 ,  "Luminosity",      6.0   ,  "Teff",  7240.0,  "example", "Gamma Virginis"), $
create_struct("Type", "F5",  "Radius",  1.2,   "Mass",  1.29,  "Luminosity",      2.5   ,  "Teff",  6540.0,  "example", "Eta Arietis"), $
create_struct("Type", "G0",  "Radius",  1.05,  "Mass",  1.10,  "Luminosity",      1.26  ,  "Teff",  6000.0,  "example", "Beta Comae Berenices"), $
create_struct("Type", "G2",  "Radius",  1.00,  "Mass",  1.00,  "Luminosity",      1.00  ,  "Teff",  5920.0,  "example", "Sun"), $
create_struct("Type", "G5",  "Radius",  0.93,  "Mass",  0.93,  "Luminosity",      0.79  ,  "Teff",  5610.0,  "example", "Alpha Mensae"), $
create_struct("Type", "K0",  "Radius",  0.85,  "Mass",  0.78,  "Luminosity",      0.40  ,  "Teff",  5150.0,  "example", "70 Ophiuchi A"), $
create_struct("Type", "K5",  "Radius",  0.74,  "Mass",  0.69,  "Luminosity",      0.16  ,  "Teff",  4640.0,  "example", "?"), $
create_struct("Type", "M0",  "Radius",  0.63,  "Mass",  0.47,  "Luminosity",      0.063 ,  "Teff",  3920.0,  "example", "?"), $
create_struct("Type", "M5",  "Radius",  0.32,  "Mass",  0.21,  "Luminosity",      0.0079,  "Teff",  3120.0,  "example", "EZ Aquarii A"), $
create_struct("Type", "M8",  "Radius",  0.13,  "Mass",  0.10,  "Luminosity",      0.0008,  "Teff",  1000.0,  "example", "Van Biesbroeck's star")]

stop
 
file_id='sketch_hrd'
jpeg_file=file_id + '.jpg'
png_file =file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Color table ---
normct

aspect_ratio=0.7
window_xsize=750
ps_xsize=12.0
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'x' &$
  xsize=window_xsize & ysize=round(window_xsize*aspect_ratio) &$
  win_id=0 &$
  window, win_id, xsize=xsize, ysize=ysize, retain=2 &$
  !P.FONT=-1 &$
  !P.CHARSIZE=2.1 &$
  !P.SYMSIZE=2.5 &$
endif else if (strupcase(dev) eq 'PS') then begin & $
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endif else begin &$
  set_plot, 'ps' & $
  device, filename=ps_file, $
         /encapsulated, $
         /portrait , xsize=ps_xsize, ysize=ps_xsize*aspect_ratio,  xoffset=0.4, yoffset=0.4, /color, bits=8 &$
  modctv, 'PS' &$
  !P.FONT=0 &$
  !P.CHARSIZE=0.9 &$
  !P.SYMSIZE=1.2 &$
endelse

; --- Plot settings ---
linecolor=[!ctv.red, !ctv.blue, !ctv.black]
linethick=3
thick_arrow=2
!P.BACKGROUND=!ctv.white
!P.COLOR=!ctv.black
!X.MARGIN=[5,1] &!Y.MARGIN=[4,1]
!Y.TICKNAME=replicate(' ', 30)
!X.TICKNAME=[' ', ' ']
!P.THICK=linethick
color_dust=!ctv.black


; === Quantities to plot ===

; --- Bins ---
plotlines, alog10(radiusb[4:*]), qucb[4:*], /nodata, $
  xrange=[min(alog10(radiusb)-0.2), max(alog10(radiusb))], $
  drange=[0,max(qucb)*1.16], $
  xtitle='log(radius)', xstyle=0, xthick=2, $
  ytitle='density'    , ystyle=1, ythick=2, $
  linecolor=color_dust, thick=4, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
polyfill, alog10(radiusb[4:*]), qucb[4:*], $
  /data, color=!ctv.yellow
plotlines, alog10(radiusb[4:*]), qucb[4:*], /over, $
  xrange=[min(alog10(radiusb)-0.2), max(alog10(radiusb))], $
  drange=[0,max(qucb)*1.16], $
  xtitle='log(radius)', xstyle=0, xthick=2, $
  ytitle='density'    , ystyle=1, ythick=2, $
  linecolor=color_dust, thick=4, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;
polyfill, alog10(radiusb[0:3]), qucb[0:3], $
  /data, color=!ctv.blue
plotlines, alog10(radiusb[0:3]), qucb[0:3], /over, $
  linecolor=color_momomer, thick=4, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
xyouts, alog10(radiusm), qucm+0.01, 'MOMOMERS', $
  alignment=0.5, color=color_momomer, charsize=1.1*!P.CHARSIZE
xyouts, alog10(radiusd[(nqucd)/2]), qucm+0.01, 'DUST', $
  alignment=0.5, color=color_dust, charsize=1.1*!P.CHARSIZE

; --- Nucleation ---
y=qucd[0]-0.03
arrow1, alog10(radiusm), y, alog10(radiusd[0]), y, $
  /data, thick=thick_arrow, dev=dev, $
  color=color_nucl
xyouts, 0.5*(alog10(radiusm)+alog10(radiusd[0])), y+0.01, 'Nucleation', $
  alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5
xyouts, 0.5*(alog10(radiusm)+alog10(radiusd[0])), y+0.01, 'Nucleation', $
  alignment=0.5, color=color_nucl, charsize=1.1*!P.CHARSIZE

; --- Coagulation ---
for i=0,nqucd-2 do begin &$
  arrow1, alog10(radiusd[i]*1.15), 0.03, alog10(radiusd[i+1]/1.15), 0.03, $
    /data, thick=thick_arrow, dev=dev, $
    color=color_coag &$
  if (i eq 2) then begin &$
    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.04, 'Coagulation', $
      alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5 &$
    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.04, 'Coagulation', $
      alignment=0.5, color=color_coag, charsize=1.1*!P.CHARSIZE &$
   endif &$
endfor

; --- Condensation ---
for i=0,nqucd-2 do begin &$
  print, i &$
  arrow1, alog10(radiusd[i]*1.15), 0.5*(qucd[i]+qucd[i+1]), alog10(radiusd[i+1]/1.15), 0.5*(qucd[i]+qucd[i+1]), $
    /data, thick=thick_arrow, dev=dev, color=color_cond &$
  arrow1, alog10(radiusd[i+1]/1.15), 0.5*(qucd[i]+qucd[i+1]), alog10(radiusd[i]*1.15), 0.5*(qucd[i]+qucd[i+1]), $
    /data, thick=thick_arrow, dev=dev, color=color_cond &$
  arrow1, alog10(radiusm), qucm-0.02, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.002, $
    /data, thick=1, dev=dev, color=color_cond &$
  arrow1, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.002, alog10(radiusm), qucm-0.02, $
    /data, thick=1, dev=dev, color=color_cond &$
  if (i eq 2) then begin &$
    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.01, 'Condensation', $
      alignment=0.5, color=!P.BACKGROUND, charsize=1.1*!P.CHARSIZE, charthick=5 &$
    xyouts, 0.5*(alog10(radiusd[i])+alog10(radiusd[i+1])), 0.5*(qucd[i]+qucd[i+1])+0.01, 'Condensation', $
      alignment=0.5, color=color_cond, charsize=1.1*!P.CHARSIZE &$
   endif &$
endfor


; === Finish plot ===
if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpeg_flag) then begin &$
    write_jpeg, jpeg_file, tvrd(true=3), true=3 &$
  endif &$
endif else begin &$
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endelse

!P=Pbak & !X=Xbak & !Y=Ybak

; end ; sketch_hrd
