function TRANSFONT, stin
;
;------------------------------------------------------------------------------
;+
; NAME:
;   TRANSFONT
;
; PURPOSE:
;   Transform string from !P.FONT=-1 format to !P.FONT=0 format.
;
; CATEGORY:
;   input/output
;
; CALLING SEQUENCE:
;   sout=transfont(stin)
;
; INPUTS:
;   stin - Input string
;
; OUTPUTS:
;   transfont
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Only a small subset of all characters is transformed.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   24. 4.95 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
if (!P.FONT eq -1) then return, stin
stinsize=size(stin)
if (stinsize(stinsize(0)+1) ne 7) then return, stin
;
; --- Define strings for special characters ---
AAB=string(197b)  ; Angstrom
aes=string(228b)  ; ae
AEB=string(196b)  ; AE
oes=string(246b)  ; oe
OEB=string(214b)  ; OE
ues=string(252b)  ; ue
UEB=string(220b)  ; UE
ss=string(223b)  ; ss
;
; --- Transformation arrays ---
tc=transpose([ ['3', '3', 'n'], ['5', '4', 'n'], ['X', 'X', 'n'], $
               ['4', '9', 'g'], ['7', '9', 'g'], ['9', '9', 's'] ])
tn=transpose([ [AAB, 'A'], [aes, 'ae'], [AEB, 'Ae'], [oes, 'oe'], [OEB, 'Oe'], $
     [ues, 'ue'], [UEB, 'Ue'], [ss, 'ss'] ])
tg=transpose([ ['C', 'G'], ['H', 'Q'], ['K', 'L'], ['R', 'S'], $
   ['U', 'F'], ['W', 'Y'], ['X', 'W'], $
   ['f', 'z'], ['g', 'h'], ['h', 'q'], ['j', 'k'], ['k', 'l'], $
   ['l', 'm'], ['m', 'n'], ['n', 'x'], ['q', 'r'], ['r', 's'], $
   ['s', 't'], ['t', 'u'], ['u', 'f'], ['v', 'c'], ['w', 'y'], $
   ['x', 'w'] ])
ts=transpose([ ['G', string(209b)] ]) ; --- nabla ---
;
soutarr=stin
;
for n=0,n_elements(stin)-1 do begin
;
s=stin(n)
length_s=strlen(s)
mode='n'
sout=''
;
l=0
while (l lt length_s) do begin
  c=strmid(s,l,1)
  if (c eq '!') then begin
    sout=sout+c
    if (l lt length_s-1) then begin
      l=l+1
      c=strmid(s,l,1)
      ind=where(c eq tc(*,0))
      ind=ind(0)
      if (ind gt -1) then begin
        sout=sout+tc(ind,1)
        mode=tc(ind,2)
      endif else begin
        sout=sout+c
      endelse
    endif
  endif else if (mode eq 'n') then begin
    ind=where(c eq tn(*,0))
    ind=ind(0)
    if (ind(0) gt -1) then begin
      sout=sout+tn(ind,1)
    endif else begin
      sout=sout+c
    endelse
  endif else if (mode eq 'g') then begin
    ind=where(c eq tg(*,0))
    ind=ind(0)
    if (ind(0) gt -1) then begin
      sout=sout+tg(ind,1)
    endif else begin
      sout=sout+c
    endelse
  endif else begin
    ind=where(c eq ts(*,0))
    ind=ind(0)
    if (ind(0) gt -1) then begin
      sout=sout+ts(ind,1)
    endif else begin
      sout=sout+c
    endelse
  endelse
  l=l+1
endwhile
soutarr(n)=sout
endfor
;
return, soutarr
end ; TRANSFONT
