pro rhd_makeblast, file=file0, form=form0, conv=conv0, $
  units=units0, $
  m1=m10, n1=n10, dx=dx0, x_c=x_c0, $
  dtime=dtime0, $
  ei_left=ei_left0, rho_left=rho_left0, v_left=v_left0, $
  ei_center=ei_center0, rho_center=rho_center0, v_center=v_center0, $
  ei_right=ei_right0, rho_right=rho_right0, v_right=v_right0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makeblast
;
; PURPOSE:
;   Generate a RHD start model file: Woodward double shock
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   28.10.97 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse
; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_px  ='g/cm^2/s'
  unit_e   ='erg/cm^3'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_px  ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_px  =''
  unit_e   =''
endelse
; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=0
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=0
endelse
m2=1
n2=1
m3=1
n3=1
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(x_c0)) then begin
  x_c=x_c0
endif else begin
  x_c=0.1
endelse
; --- States: left side ---
if (keyword_set(ei_left0)) then begin
  ei_left=ei_left0
endif else begin
  ei_left=1.0
endelse
if (keyword_set(rho_left0)) then begin
  rho_left=rho_left0
endif else begin
  rho_left=1.0
endelse
if (keyword_set(v_left0)) then begin
  v_left=v_left0
endif else begin
  v_left=0.0
endelse
; --- States: center ---
if (keyword_set(ei_center0)) then begin
  ei_center=ei_center0
endif else begin
  ei_center=1.0
endelse
if (keyword_set(rho_center0)) then begin
  rho_center=rho_center0
endif else begin
  rho_center=1.0
endelse
if (keyword_set(v_center0)) then begin
  v_center=v_center0
endif else begin
  v_center=0.0
endelse
; --- States: right side ---
if (keyword_set(ei_right0)) then begin
  ei_right=ei_right0
endif else begin
  ei_right=1.0
endelse
if (keyword_set(rho_right0)) then begin
  rho_right=rho_right0
endif else begin
  rho_right=1.0
endelse
if (keyword_set(v_right0)) then begin
  v_right=v_right0
endif else begin
  v_right=0.0
endelse
;
; --- Channel for comment output ---
nc_p=-1
;
; --- Description and history ---
description=['RHD test data at ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makeblast.pro at '+strtrim(systime(0),2)]
nhis=1
;
; --- Initial time ---
ntime=0
time =0.0
;
; --- Grid from 0 to 1 ---
xb1=dx*(findgen(n1-m1+2))
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)
;
xc1=0.5*(xb1(1:n1-m1+1)+xb1(0:n1-m1))
xc2=0.5*(xb2(1:n2-m2+1)+xb2(0:n2-m2))
xc3=0.5*(xb3(1:n3-m3+1)+xb3(0:n3-m3))
;
; --- Left states on all grid points ---
rho=replicate(rho_left,n1-m1+1)
p1 =replicate(v_left,  n1-m1+1) * rho_left
p2 =   fltarr(         n1-m1+1)
p3 =   fltarr(         n1-m1+1)
eik=replicate(ei_left, n1-m1+1) + (p1^2+p2^2+p3^2)*0.5/rho_left
;
xfac=(xb1(n1-m1+1)-xb1(0))
; --- Central states ---
ind=where(xc1 ge xfac*x_c and xc1 lt xfac*(1.0-x_c))
rho(ind)=rho_center
p1( ind)=v_center * rho_center
eik(ind)=ei_center + (p1(ind)^2+p2(ind)^2+p3(ind)^2)*0.5/rho_center
;
; --- Right states ---
ind=where(xc1 ge xfac*(1.0-x_c))
rho(ind)=rho_right
p1( ind)=v_right * rho_right
eik(ind)=ei_right + (p1(ind)^2+p2(ind)^2+p3(ind)^2)*0.5/rho_right
;
; --- Plot ---
help, rho_left, rho_center, rho_right
plot, xc1, eik, psym=-1
;
; --- 3D ---
rho=rebin(reform(rho,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
p1 =rebin(reform(p1 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
p2 =rebin(reform(p2 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
p3 =rebin(reform(p3 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
eik=rebin(reform(eik,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
               nc_p, ncout, file=file, form=form, conv=conv, $
               ndes=ndes, description=description, $
               nhis=nhis, history=history, $
               unit_time=unit_time, $
               unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
               unit_rho=unit_rho, unit_px=unit_px, unit_e=unit_e, $
               m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
               ntime=ntime, time=time, $
               dtime=dtime0, out_time_full_last=0.0, $
               xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
               rho=rho, p1=p1, p2=p2, p3=p3, eik=eik
;
end ; rhd_makeblast
