pro rhd_makerandom, file=file0, form=form0, conv=conv0, $
  units=units0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, $
  dtime=dtime0, $
  ei_ampl=ei_ampl0, rho_ampl=rho_ampl0, v_ampl=v_ampl0, $
  ei_mean=ei_mean0, rho_mean=rho_mean0, v_mean=v_mean0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makerandom
;
; PURPOSE:
;   Generate a RHD start model file: 1D/2D/3D Random rho/ei/v values
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   16.10.97 Written by Bernd Freytag, Astrophysik Kiel (rhd_makesedov)
;    3. 6.99 (B.F. Copenhagen)
;   18. 5.04 (B.F. Montpellier)
;-
;------------------------------------------------------------------------------
;
; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse
;
; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v  ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_px  ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_px  =''
  unit_e   =''
endelse
;
; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=0
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=0
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=0
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=0
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=0
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=0
endelse
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse
;
; --- States: inside ---
if (keyword_set(ei_ampl0)) then begin
  ei_ampl=ei_ampl0
endif else begin
  ei_ampl=10.0
endelse
if (keyword_set(rho_ampl0)) then begin
  rho_ampl=rho_ampl0
endif else begin
  rho_ampl=10.0
endelse
if (keyword_set(v_ampl0)) then begin
  v_ampl=v_ampl0
endif else begin
  v_ampl=1.0E+05
endelse
; --- States: outside ---
if (keyword_set(ei_mean0)) then begin
  ei_mean=ei_mean0
endif else begin
  ei_mean=1.0E+13
endelse
if (keyword_set(rho_mean0)) then begin
  rho_mean=rho_mean0
endif else begin
  rho_mean=1.0E-10
endelse
if (keyword_set(v_mean0)) then begin
  v_mean=v_mean0
endif else begin
  v_mean=0.0
endelse
;
; --- Channel for comment output ---
nc_p=-1
;
; --- Description and history ---
description=['Random rho/ei/v fields ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makerandom.pro at ' + strtrim(systime(0),2)]
nhis=1
;
; --- Initial time ---
itime=0
time =0.0
;
; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
xb1=dx*(findgen(n1-m1+2)+float(m1)-0.5)
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)
;
; --- Outside states on all grid points ---
ran=exp(alog(rho_ampl)*2.0*(randomu(seed,n1-m1+1,n2-m2+1,n3-m3+1)-0.5))
rho=rho_mean*ran
ran=exp(alog(ei_ampl )*2.0*(randomu(seed,n1-m1+1,n2-m2+1,n3-m3+1)-0.5))
ei =ei_mean*ran
ran=2.0*(randomu(seed,n1-m1+1,n2-m2+1,n3-m3+1)-0.5)
v1 =v_mean + v_ampl*ran
ran=2.0*(randomu(seed,n1-m1+1,n2-m2+1,n3-m3+1)-0.5)
v2 =v_mean + v_ampl*ran
ran=2.0*(randomu(seed,n1-m1+1,n2-m2+1,n3-m3+1)-0.5)
v3 =v_mean + v_ampl*ran
;
rho=rebin(reform(rho,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
                nc_p, ncout, file=file, form=form, conv=conv, $
                ndes=ndes, description=description, $
                nhis=nhis, history=history, $
                unit_time=unit_time, $
                unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                itime=itime, time=time, $
                dtime=dtime, out_time_full_last=-1.0E08, $
                xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                rho=rho, ei=ei, v1=v1, v2=v2, v3=v3
;
end ; rhd_makerandom
