; pro rhd_makesedov_start
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makesedov_start
;
; PURPOSE:
;   Generate a RHD start model file: Rieman problem
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   @rhd_makesedov_start
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Start rhd_makesedov with reasonable parameters.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-16 Written by Bernd Freytag, Astrophysik Kiel
;   1999-06-03 (B.F. Copenhagen)
;   2004-05-12 (B.F. Montpellier)
;   2011-07-19 (B.F. Lyon) New naming scheme
;-
;------------------------------------------------------------------------------
;
;rhd_makesedov, file='d2tstsedov01.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-25, n1=25, m2=0, n2=0, m3=-25, n3=25, dx=10.0e+05, ren=1.5, $
;  dtime=0.1, $
;  ei_in =1.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+07, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='d2tstsedov02.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-25, n1=25, m2=0, n2=0, m3=-25, n3=25, dx=10.0e+05, ren=1.5, $
;  dtime=0.002, $
;  ei_in =1.00e+16, rho_in =1.000e-12, v_in =0.0, $
;  ei_out=1.00e+11, rho_out=1.000e-12, v_out=0.0
;
rhd_makesedov, file='d2tstsedov03.sta', form='unformatted', conv='ieee_4', $
  units='cgs', $
  m1=-25, n1=25, m2=0, n2=0, m3=-25, n3=25, dx=10.0e+05, $
  ren=0.5, q_smooth=1, q_drop=0.25, $
  dtime=0.002, $
  ei_in =9.00e+16, rho_in =1.000e-12, v_in =0.0, $
  ei_out=1.00e+11, rho_out=1.000e-12, v_out=0.0

;rhd_makesedov, file='d3tstsedov01.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-25, n1=25, m2=-25, n2=25, m3=-25, n3=25, dx=10.0e+05, ren=1.5, $
;  dtime=0.002, $
;  ei_in =1.00e+16, rho_in =1.000e-12, v_in =0.0, $
;  ei_out=1.00e+11, rho_out=1.000e-12, v_out=0.0
;
;rhd_makesedov, file='sedov2d_9p.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-25, n1=25, m2=0, n2=0, m3=-25, n3=25, dx=10.0e+05, ren=1.5, $
;  dtime=0.05, $
;  ei_in =1.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+07, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='sedov3d.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-25, n1=25, m2=-25, n2=25, m3=-25, n3=25, dx=10.0e+05, ren=1.5, $
;  dtime=0.05, $
;  ei_in =1.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+07, rho_out=1.000e-8, v_out=0.0
;
;n=10
;rhd_makesedov, file='sedov3d.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-n, n1=n, m2=-n, n2=n, m3=-n, n3=n, dx=10.0e+05, ren=1.5, $
;  dtime=0.05, $
;  ei_in =1.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+08, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='sedov1d.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=1, n1=41, m2=1, n2=1, m3=1, n3=1, dx=1.0e+03, ren=1.5, $
;  dtime=0.001, $
;  ei_in =1.00e+02, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e-02, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='sedov2d.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=1, n1=41, m2=1, n2=41, m3=1, n3=1, dx=1.0e+03, ren=3.5, $
;  dtime=0.001, $
;  ei_in =1.00e+02, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e-01, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='sedov2dhot.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-30, n1=30, m2=0, n2=0, m3=-30, n3=30, dx=1.0e+06, ren=3.5, $
;  dtime=1.0E-04, $
;  ei_in =1.00e+15, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+11, rho_out=1.000e-8, v_out=0.0

;rhd_makesedov, file='sedov3dhot.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=-30, n1=30, m2=-30, n2=30, m3=-30, n3=30, dx=1.0e+06, ren=3.5, $
;  dtime=3.0E-03, $
;  ei_in =1.00e+15, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+11, rho_out=1.000e-8, v_out=0.0

;
;rhd_makesedov, file='radgreens2d01.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=1, n1=41, m2=1, n2=1, m3=1, n3=21, dx=1.0e+06, ren=0.5, $
;  dtime=0.1, $
;  ei_in =2.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+12, rho_out=1.000e-8, v_out=0.0
;
;rhd_makesedov, file='radgreens3d01.sta', form='unformatted', conv='ieee_4', $
;  units='cgs', $
;  m1=1, n1=41, m2=1, n2=41, m3=1, n3=21, dx=1.0e+06, ren=0.5, $
;  dtime=0.1, $
;  ei_in =2.00e+12, rho_in =1.000e-8, v_in =0.0, $
;  ei_out=1.00e+12, rho_out=1.000e-8, v_out=0.0
;
; end ; rhd_makesedov_start
