function rhd_makeshear_gauss, x, beta, z
return, exp(-beta*(x-z)^2)
end ; rhd_makeshear_gauss

function rhd_makeshear_triangle, x, gamma, g
return, 1.0-abs(gamma*(x-g))
end ; rhd_makeshear_triangle

function rhd_makeshear_ellipse, x, alpha, a
return, sqrt( (1.0-alpha^2*(x-a)^2)>0.0)
end ; rhd_makeshear_ellipse


pro rhd_makeshear, file=file0, form=form0, conv=conv0, $
  funct=funct0, units=units0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, $
  dtime=dtime0, $
  ei_in=ei_in0, rho_in=rho_in0, v_in=v_in0, $
  vampl=vampl0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makeshear
;
; PURPOSE:
;   Generate a RHD start model file: 1D/2D/3D shear flow
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   31. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;-
;------------------------------------------------------------------------------
;
; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse
;
; --- Type of function ---
if (keyword_set(funct0)) then begin
  funct=strlowcase(funct0)
endif else begin
  funct='spikes'
endelse
;
; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v   ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_v   ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_v   =''
  unit_e   =''
endelse
;
; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=-100
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=99
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=0
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=0
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=0
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=0
endelse
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(dtime0)) then begin
  dtime=dtime0
endif else begin
  dtime=1.0
endelse
;
; --- States: inside ---
if (keyword_set(ei_in0)) then begin
  ei_in=ei_in0
endif else begin
  ei_in=1.0E08
endelse
if (keyword_set(rho_in0)) then begin
  rho_in=rho_in0
endif else begin
  rho_in=1.0
endelse
if (keyword_set(v_in0)) then begin
  v_in=v_in0
endif else begin
  v_in=0.0
endelse
;
; --- Amplitude ---
if (keyword_set(vampl0)) then begin
  vampl=vampl0
endif else begin
  vampl=0.0
endelse
;
; --- Channel for comment output ---
nc_p=-1
;
; --- Description and history ---
version='MAKESHEAR 000.02.1999.05.31'
description=['RHD test data ', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makeshear.pro: '+strtrim(systime(0),2)]
nhis=1
;
; --- Initial time ---
itime=0
time =0.0
;
; --- Grid ---
xc1=dx*(findgen(n1-m1+1)+float(m1))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
xb1=dx*(findgen(n1-m1+2)+float(m1)-0.5)
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)
;
u=fltarr(n1-m1+1)
if (funct eq 'spikes') then begin
  ; --- Function with spikes ---
  a=0.5
  z=-0.7
  delta=0.005
  alpha=10.0
  beta=alog(2.0)/(36.0*delta^2)
  xnorm=xc1/dx*0.01
  for i=0,n1-m1 do begin
    x=xnorm(i)
    if ((x ge -0.8) and (x le -0.6)) then begin
      u(i)=1.0/6.0*(    rhd_makeshear_gauss(x,beta,z-delta) + $
                        rhd_makeshear_gauss(x,beta,z+delta) + $
                    4.0*rhd_makeshear_gauss(x,beta,z)          )
    endif else if ((x ge -0.4) and (x le -0.2)) then begin
      u(i)=1.0
    endif else if ((x ge  0.0) and (x le  0.2)) then begin
      u(i)=rhd_makeshear_triangle(x,10.0,0.1)
    endif else if ((x ge  0.4) and (x le  0.6)) then begin
      u(i)=1.0/6.0*(    rhd_makeshear_ellipse(x,alpha,a-delta) + $
                        rhd_makeshear_ellipse(x,alpha,a+delta) + $
                    4.0*rhd_makeshear_ellipse(x,alpha,a)          )
    endif
  endfor
  u=u*vampl
endif else if (funct eq 'needles') then begin
  xnorm=xc1/dx*0.01
  period=20
  for i=0,n1-m1-period do begin
    idiv=i/period
    imod = i mod period
    if (imod lt idiv) then u(i)=vampl
  endfor
endif else if (funct eq 'needles2') then begin
  xnorm=xc1/dx*0.01
  period=26
  for i=0,n1-m1-period do begin
    idiv=2*(i/period)
    imod = i mod period
    if (imod lt idiv) then u(i)=vampl
  endfor
endif else if (funct eq 'sine') then begin
  xnorm=xc1/dx*0.01
  u=vampl*sin(2.0*!PI*xnorm)
endif else begin

endelse
;
plot,xnorm,u, psym=-1, ystyle=3
;
u=rebin(reform(u,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- Create arrays and rebin them to 3D ---
rho=replicate(rho_in,n1-m1+1,n2-m2+1,n3-m3+1)
ei =replicate(ei_in, n1-m1+1,n2-m2+1,n3-m3+1)
v1 =   fltarr(       n1-m1+1,n2-m2+1,n3-m3+1)
v2 =   fltarr(       n1-m1+1,n2-m2+1,n3-m3+1)
v3 =replicate(v_in,  n1-m1+1,n2-m2+1,n3-m3+1)
rho=rebin(reform(rho,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,n2-m2+1,n3-m3+1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- Modulate states ---
v3 =v3*u
;
; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
                nc_p, ncout, file=file, form=form, conv=conv, $
                ndes=ndes, description=description, $
                nhis=nhis, history=history, $
                unit_time=unit_time, $
                unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
                unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
                m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
                itime=itime, time=time, $
                dtime=dtime, out_time_full_last=0.0, $
                xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
                rho=rho, ei=ei, v1=v1, v2=v2, v3=v3
;
end ; rhd_makeshear
