pro rhd_makeshock, file=file0, form=form0, conv=conv0, double_flag=double_flag, $
  units=units0, $
  m1=m10, n1=n10, m2=m20, n2=n20, m3=m30, n3=n30, dx=dx0, x_c=x_c0, $
  dtime=dtime0, $
  ei_left=ei_left0  , rho_left=rho_left0  , v_left=v_left0, $
  ei_right=ei_right0, rho_right=rho_right0, v_right=v_right0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makeshock
;
; PURPOSE:
;   Generate a RHD start model file: Rieman problem
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    5.10.97 Written by Bernd Freytag, Astrophysik Kiel
;   27.10.97 (B.F.) Shift x(nc1) to x_c (default: 0.0)
;   28.10.97 (B.F.) Some renaming
;    8. 4.99 (B.F. Copenhagen) Different basic variables in RHD data file
;   25. 5.99 (B.F.) Keywords, m2-n3 added
;-
;------------------------------------------------------------------------------
;
; --- Default values for keywords ---
; --- File properties ---
if (keyword_set(file0)) then begin
  file=file0
endif else begin
  file='rhd.sta'
endelse
if (keyword_set(form0)) then begin
  form=form0
endif else begin
  form='formatted'
endelse
if (keyword_set(conv0)) then begin
  conv=conv0
endif else begin
  conv=''
endelse
; --- Units ---
if (keyword_set(units0)) then begin
  units=strlowcase(units0)
endif else begin
  units='1'
endelse
if (units eq 'cgs') then begin
  unit_time='s'
  unit_x1  ='cm'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='g/cm^3'
  unit_v   ='cm/s'
  unit_e   ='erg/g'
endif else if (units eq '1') then begin
  unit_time='1'
  unit_x1  ='1'
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho ='1'
  unit_v   ='1'
  unit_e   ='1'
endif else begin
  unit_time=''
  unit_x1  =''
  unit_x2  =unit_x1
  unit_x3  =unit_x1
  unit_rho =''
  unit_v   =''
  unit_e   =''
endelse
; --- Grid ---
if (keyword_set(m10)) then begin
  m1=m10
endif else begin
  m1=0
endelse
if (keyword_set(n10)) then begin
  n1=n10
endif else begin
  n1=0
endelse
if (keyword_set(m20)) then begin
  m2=m20
endif else begin
  m2=1
endelse
if (keyword_set(n20)) then begin
  n2=n20
endif else begin
  n2=1
endelse
if (keyword_set(m30)) then begin
  m3=m30
endif else begin
  m3=1
endelse
if (keyword_set(n30)) then begin
  n3=n30
endif else begin
  n3=1
endelse
;
n1c=0.5*(float(m1)+float(n1))
if (keyword_set(dx0)) then begin
  dx=dx0
endif else begin
  dx=1.0
endelse
if (keyword_set(x_c0)) then begin
  x_c=x_c0
endif else begin
  x_c=0.0
endelse
; --- States: left side ---
if (keyword_set(ei_left0)) then begin
  ei_left=ei_left0
endif else begin
  ei_left=1.0
endelse
if (keyword_set(rho_left0)) then begin
  rho_left=rho_left0
endif else begin
  rho_left=1.0
endelse
if (keyword_set(v_left0)) then begin
  v_left=v_left0
endif else begin
  v_left=0.0
endelse
; --- States: right side ---
if (keyword_set(ei_right0)) then begin
  ei_right=ei_right0
endif else begin
  ei_right=1.0
endelse
if (keyword_set(rho_right0)) then begin
  rho_right=rho_right0
endif else begin
  rho_right=1.0
endelse
if (keyword_set(v_right0)) then begin
  v_right=v_right0
endif else begin
  v_right=0.0
endelse
;
; --- Channel for comment output ---
nc_p=-1
;
; --- Description and history ---
description=['RHD test data', strtrim(systime(0),2)]
ndes=2
history=['Produced by rhd_makeshock.pro: '+strtrim(systime(0),2)]
nhis=1
;
; --- Initial time ---
itime=0
time =0.0
;
; --- Grid ---
xc1=dx*(findgen(n1-m1+1)-(n1c-1.0))
xc2=dx*(findgen(n2-m2+1)+float(m2))
xc3=dx*(findgen(n3-m3+1)+float(m3))
;
xb1=dx*(findgen(n1-m1+2)-(n1c-0.5))
xb2=dx*(findgen(n2-m2+2)+float(m2)-0.5)
xb3=dx*(findgen(n3-m3+2)+float(m3)-0.5)
;
; --- Left states on all grid points ---
rho=replicate(rho_left,n1-m1+1)
v1 =replicate(v_left,  n1-m1+1)
v2 =   fltarr(         n1-m1+1)
v3 =   fltarr(         n1-m1+1)
ei =replicate(ei_left, n1-m1+1)
; --- Right states ---
if ((long(n1c)+1) le n1) then begin
  n1s=long(n1c)+1-m1
  rho(n1s:n1-m1)=rho_right
  v1( n1s:n1-m1)=  v_right
  v2( n1s:n1-m1)= 0.0
  v3( n1s:n1-m1)= 0.0
  ei( n1s:n1-m1)= ei_right
endif
; --- 3D ---
rho=rebin(reform(rho,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
v1 =rebin(reform(v1 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
v2 =rebin(reform(v2 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
v3 =rebin(reform(v3 ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
ei =rebin(reform(ei ,n1-m1+1,1,1),n1-m1+1,n2-m2+1,n3-m3+1)
;
; --- Plot ---
plot, xc1, rho, psym=-1
;
; --- Write model into file ---
rhd_wrboxdata, 'open,model,box,endmo,close', $
               nc_p, ncout, file=file, form=form, conv=conv, double_flag=double_flag, $
               ndes=ndes, description=description, $
               nhis=nhis, history=history, $
               unit_time=unit_time, $
               unit_x1=unit_x1, unit_x2=unit_x2, unit_x3=unit_x3, $
               unit_rho=unit_rho, unit_v=unit_v, unit_e=unit_e, $
               m1=m1, m2=m2, m3=m3, n1=n1, n2=n2, n3=n3, $
               itime=itime, time=time, $
               dtime=dtime0, out_time_full_last=0.0, $
               xc1=xc1, xc2=xc2, xc3=xc3, xb1=xb1, xb2=xb2, xb3=xb3, $
               rho=rho, v1=v1, v2=v2, v3=v3, ei=ei
;
end ; rhd_makeshock
