; pro rhd_makeshock_start
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makeshock_start
;
; PURPOSE:
;   Generate a RHD start model file: Rieman problem
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   @rhd_makeshock_start
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Start rhd_makeshock with reasonable parameters.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-05 Written by Bernd Freytag, Astrophysik Kiel
;   1999-04-09 (B.F. Copenhagen) Different basic quantities
;   2011-07-07 (B.F. Lyon) New naming scheme for test models
;-
;------------------------------------------------------------------------------
;
; --- Scaling factors ---
x_fac   =1.0e-5
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0e+08
e_fac   =v_fac^2
rhoe_fac=rho_fac*e_fac
p_fac   =rhoe_fac
;
;rhd_makeshock, file='sod_n_v10.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=160, dx=0.01/x_fac, $
;  dtime=0.001, $
;  ei_left =2.5/e_fac, rho_left =1.0/rho_fac  , v_left =1.0/v_fac, $
;  ei_right=2.0/e_fac, rho_right=0.125/rho_fac, v_right=1.0/v_fac
;rhd_makeshock, file='sod_r_vm10.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=160, dx=0.01/x_fac, $
;  dtime=0.001, $
;  ei_left =2.0/e_fac, rho_left =0.125/rho_fac, v_left =-1.0/v_fac, $
;  ei_right=2.5/e_fac, rho_right=1.0/rho_fac  , v_right=-1.0/v_fac

;rhd_makeshock, file='sod2d.sta', form='unformatted', conv='', $
;  units='cgs', $
;  m1=1, n1=100, m3=1, n3=5, dx=0.01/x_fac, $
;  dtime=0.001, $
;  ei_left =2.5/e_fac, rho_left =1.0/rho_fac , v_left =0.0/v_fac, $
;  ei_right=2.0/e_fac, rho_right=0.125/rho_fac, v_right=0.0/v_fac

; --- 2011-07-07 ---
rhd_makeshock, file='d1tstsod01.sta', $
  form='unformatted', conv='', units='cgs', $
  m1=1, n1=100, dx=0.01/x_fac, dtime=0.001, $
  ei_left =2.5/e_fac, rho_left =1.000/rho_fac, v_left =0.0/v_fac, $
  ei_right=2.0/e_fac, rho_right=0.125/rho_fac, v_right=0.0/v_fac

;rhd_makeshock, file='sod.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=100, dx=0.01/x_fac, $
;  dtime=0.001, $
;  ei_left =2.5/e_fac, rho_left =1.0/rho_fac  , v_left =0.0/v_fac, $
;  ei_right=2.0/e_fac, rho_right=0.125/rho_fac, v_right=0.0/v_fac

;rhd_makeshock, file='sod.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=100, dx=1.0e+03, $
;  dtime=0.001, $
;  ei_left =2.50e02, rho_left =1.000e-8, v_left =0.0, $
;  ei_right=0.25e02, rho_right=0.125e-8, v_right=0.0
;
;rhd_makeshock, file='hawley.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=100, dx=0.01/x_fac, $
;  dtime=0.01/t_fac, $
;  ei_left =1.0E-02/e_fac, rho_left =1.0E+02/rho_fac, v_left =0.0, $
;  ei_right=1.0E-07/e_fac, rho_right=1.0E+00/rho_fac, v_right=0.0
;
; --- 2011-07-08 ---
rhd_makeshock, file='d1tsthawley01.sta', $
  form='unformatted', conv='', units='cgs', $
  m1=1, n1=100, dx=0.01/x_fac, dtime=0.01/t_fac, $
  ei_left =1.0E-02/e_fac, rho_left =1.0E+02/rho_fac, v_left =0.0/v_fac, $
  ei_right=1.0E-07/e_fac, rho_right=1.0E+00/rho_fac, v_right=0.0/v_fac
;
;rhd_makeshock, file='hawley_fine.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=200, dx=0.5e+03, $
;  dtime=0.0001, $
;  ei_left =1.0e+02, rho_left =1.000e-6, v_left =0.0, $
;  ei_right=1.0e-03, rho_right=1.000e-8, v_right=0.0
;
;rhd_makeshock, file='nordlund.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=200, dx=0.5e+03, $
;  dtime=0.0005, $
;  ei_left =1.50e02, rho_left =1.000e-8, v_left =0.0, $
;  ei_right=0.15e02, rho_right=0.100e-8, v_right=0.0
;
;rhd_makeshock, file='wind.sta', form='formatted', conv='', $
;  units='cgs', $
;  m1=1, n1=100, dx=1.0e+03, $
;  dtime=0.001, $
;  ei_left =0.333e02*3.0, rho_left =0.333e-08, v_left =3.0e+05, $
;  ei_right=1.000e02*3.0, rho_right=1.000e-08, v_right=1.0e+05
;
; end ; rhd_makeshock_start
