; pro rhd_makespike_start
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_makespike_start
;
; PURPOSE:
;   Generate a RHD start model file: Rieman problem
;
; CATEGORY:
;   RHD
;
; CALLING SEQUENCE:
;   @rhd_makespike_start
;
; INPUT:
;
; OUTPUT:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;   rhd_wrboxdata: Write RHD model into file.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Start rhd_makespike with reasonable parameters.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-17 (B.F. Astrophysik Kiel)
;   1997-10-19 (B.F.) Keyword funct added
;   1999-03-25 (B.F. Copenhagen) Different basic quantities
;   2012-04-19 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

; --- Scaling factors ---
x_fac   =1.0e-5
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0e+08
e_fac   =v_fac^2
rhoe_fac=rho_fac*e_fac
p_fac   =rhoe_fac

;rhd_makespike, file='needle.sta', form='formatted', conv='', $
;  funct='needles', units='cgs', $
;  m1=-99, n1=140, dx=1.0e+03, $
;  dtime=0.004, $
;  ei_in =1.00e01, rho_in =1.000e-8, v_in =1.0e+05, $
;  ampl=1.0

;rhd_makespike, file='needle.sta', form='formatted', conv='', $
;  funct='needles2', units='cgs', $
;  m1=1, n1=200, dx=1.0e+03, $
;  dtime=0.004, $
;  rhoei_in =1.00e01, rho_in =1.000e-8, v_in =1.0e+05, $
;  ampl=999.0

rhd_makespike, file='spike.sta', form='unformatted', conv='', $
  funct='spikes', units='cgs', $
  m1=-99, n1=100, dx=1.0e+03, $
  dtime=0.004, $
  rhoei_in =1.00e01, rho_in =1.000e-8, v_in =1.0e+05, $
  ampl=1.0E2

;rhd_makespike, file='d1tstspikev101.sta', form='unformatted', conv='', $
;  funct='spikes', modulation='v1', units='cgs', $
;  RepliNumber=50, RepliFactor=sqrt(0.5), $
;  m1=1, n1=200, dx=10.0e+05, $
;  dtime=0.1, $
;  rhoei_in =1.00E+04*1.785713, rho_in =1.000E-08, v_in =2.0*10.0E+05, $
;  ampl=1.0E+00

; ful=uio_dataset_rd('/home/bf/dat/job/j4/bak/d1tstspikev101n107_1.end')
; q=(reform(ful.z.v1-sta.z.v1,200,50)) & for i=1L,49L do q[*,i]=q[*,i]*sqrt(2.0)^i
; plotlines,alog10(abs(transpose(q[100:110,*]))>1.0E1),psym=-1
; plotfield,/tv,/oldasp,t_sam=1,alog10(abs(transpose(q))>1.0E1)


; end ; rhd_makespike_start
