; pro rhd_plotblast
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_plotblast ('rhd_plot_blast')
;
; PURPOSE:
;   Plot HD-quantities from RHD simulation and analytical solution
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @rhd_plotblast
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   ;   Before first use type
;     delvar, init
;     uio_data, mode='allclose'  ; --- Attention ---
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   29.10.97 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Directories ---
rhd_dir='../hd/rhdtst/'
ref_dir=rhd_dir
;funct='woodward'
;funct='wood400'
;funct='wood800'
funct='wood1600'
;model=funct + '_constant'
;model=funct + '_minmod'
;model=funct + '_vanleer'
model=funct + '_superbee'
;model=funct + '_superbeemod'
;model=funct + '_pp'
;rhd1_file=model + '.sta'
;rhd2_file=model + '.end'
rhdpar_file=model + '_par.txt'
ref_file='wood1600' + '_superbeemod' + '.end'
;
; --- Choose plot device ---
;dev='ps'
dev='x'
;
; --- Scaling factors ---
x_fac=1.0e-5
t_fac=1.0
v_fac=x_fac/t_fac
rho_fac=1.0e+08
e_fac=v_fac^2
eps_fac=rho_fac*v_fac^2
p_fac=rho_fac*e_fac
;
if (n_elements(init) eq 0) then begin &$
  ; --- Read parameter file ---
  uio_data, mo='open', file=rhd_dir + rhdpar_file, fam=funct + 'par' &$
  reconstruction=uio_d('reconstruction') &$
  C_interaction=uio_d('c_interaction') &$
  gamma=uio_d('gamma') &$
  rhd1_file=uio_d('infile_start') &$
  rhd2_file=uio_d('outfile_end') &$
endif
;
if (n_elements(init) eq 0) then begin &$
  ; --- Read "exact" blast wave solution ---
  uio_data, mo='open', file=ref_dir+ref_file, fam='ref' &$
  x_ref    =reform(uio_d('xc1')) * x_fac &$
  xb_ref   =reform(uio_d('xb1')) * x_fac &$
  p1_ref   =reform(uio_d('p1' )) &$
  rho_ref  =reform(uio_d('rho')) &$
  v_ref    =p1_ref/rho_ref       * v_fac &$
  rho_ref  =rho_ref              * rho_fac &$
  epsik_ref=reform(uio_d('eik')) * eps_fac &$
  ei_ref=epsik_ref/rho_ref - 0.5*v_ref^2 &$
  p_ref=(gamma-1.0)*ei_ref*rho_ref &$
endif
;
if (n_elements(init) eq 0) then begin &$
  ; --- Read RHD model ---
  uio_data, mo='open', file=rhd_dir + rhd2_file, fam='rhd' &$
  x_rhd    =reform(uio_d('xc1')) * x_fac &$
  p1_rhd   =reform(uio_d('p1' )) &$
  rho_rhd  =reform(uio_d('rho')) &$
  v_rhd    =p1_rhd/rho_rhd       * v_fac &$
  rho_rhd  =rho_rhd              * rho_fac &$
  epsik_rhd=reform(uio_d('eik')) * eps_fac &$
  ei_rhd=epsik_rhd/rho_rhd - 0.5*v_rhd^2 &$
  p_rhd=(gamma-1.0)*ei_rhd*rho_rhd &$
  time=uio_d('time') &$
  ntime=uio_d('ntime') &$
  nx=n_elements(x_rhd) &$
  ;
  init=1 &$
endif
;
; --- Choose size of postscript output ---
if (dev eq 'ps') then begin &$
  set_plot, 'ps' &$
  device, filename='../hd/rhdtst/' + model + '.ps' &$
  device, /landscape, xsize=28.5, ysize=19.3, xoffset=1.0,yoffset=29.2, $
    color=0 &$
;  device, /portrait, xsize=18.0, ysize=11.0, xoffset=0.4,yoffset=0.4,color=0 &$
  color=!P.COLOR &$
endif else begin &$
  color=!ctv.green &$
endelse
;
!P.MULTI=[0,2,2,0,0]
!X.MARGIN=[7,2]
!Y.MARGIN=[2,0]
!X.THICK=2
!Y.THICK=2
!X.STYLE=1
!Y.STYLE=3
!P.FONT=0
!P.CHARSIZE=0.75
!P.SYMSIZE=0.5
psym=psymuser(102)
;
plot, x_ref, v_ref, ytitle='v', xrange=[min(xb_ref),max(xb_ref)]
oplot,x_rhd,    v_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
;
plot, x_ref, p_ref, ytitle='p', xrange=[min(xb_ref),max(xb_ref)]
oplot,x_rhd,    p_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
legend, [reconstruction, $
         "C!Dinter!N=" + strtrim(string(C_interaction,format='(E8.1)'),2), $
         "t="          + strtrim(string(time,         format='(F9.6)'),2), $
         "n!Dt!N="     + strtrim(string(ntime,        format='(I5)'  ),2), $
         "n!Dx!N="     + strtrim(string(nx,           format='(I5)'  ),2), $
         "!9g!X="      + strtrim(string(gamma,        format='(F5.3)'),2)        ], $
        box=0, xali=1.0, yali=1.0, margin=0.1, spacing=0.9
;
plot, x_ref, rho_ref, ytitle='!9r!X', xrange=[min(xb_ref),max(xb_ref)]
oplot,x_rhd,    rho_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
;
plot, x_ref, ei_ref, ytitle='!9e!X', xrange=[min(xb_ref),max(xb_ref)]
oplot,x_rhd,    ei_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
;
!P.MULTI=0
!X.MARGIN=[12,3]
!Y.MARGIN=[4,2]
!X.THICK=0
!Y.THICK=0
!X.STYLE=0
!Y.STYLE=0
!P.CHARSIZE=0
!P.SYMSIZE=0
;
if (dev eq 'ps') then begin &$
  device, /close &$
  set_plot, 'x' &$
endif
;
; end ; rhd_plotblast
