; pro rhd_ploteddy
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_ploteddy ('rhd_plot_eddy')
;
; PURPOSE:
;   Plot HD-quantities from RHD simulation and start model.
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @rhd_ploteddy
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;   a=1.0 & b=1.0 & plotlines,logx,x/(b/((1.0+x/(a*b)))^2+x)
;
; MODIFICATION HISTORY:
;   1997-11-10 Written by Bernd Freytag, Astrophysik Kiel
;   2011-08-26 (B.F. Lyon) Additional curves
;   2011-10-26 (B.F.) Based on rhd_plotsedov.pro -> rhd_ploteddy.pro
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Choose plot device ---
  ;dev='ps' &$
  dev='x' &$
  png_flag=0 &$
  jpg_flag=0 &$
  ;
  ; --- Directories ---
  ;rhd_dir='../hd/rhdtst/' &$
  rhd_dir='/home/bf/dat/job/j4/bak/' &$
  ;ref_dir=rhd_dir &$
  ;
  ; --- Problems ---
  ;funct='eddy' &$
  funct='eddygrav' &$
  ;
  ; --- Models ---
  ;modelset='d2tst' + funct + '01' &$
  modelset='d2tst' + funct + '04' &$
  model=modelset + 'n037b' &$
  modelident='_1' &$
  ;rhd1_file=model + '.sta' &$
  ;rhd2_file=model + '.end' &
  ;rhdpar_file=model + '_par.txt' &$
  ;ref_file='' &$
  ;
  ct=3 &$
endif

dnx=6

; --- Common paths ---
eosdisk   =getenv('HOME') + '/for/eos/dat/'
opadisk   =getenv('HOME') + '/for/opa/dat/'

rhdpar_file=model + modelident + '.par'

; --- Scaling factors ---
x_fac   =1.0e-5
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0
e_fac   =v_fac^2
rhoe_fac=rho_fac*e_fac
P_fac   =rhoe_fac

delvar, init

if (n_elements(init) eq 0) then begin &$
  ; --- Read parameter file ---
  par=uio_struct_rd(rhd_dir + rhdpar_file) &$
  ;
endif
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;  dfopta, opadisk + par.opafile &$
endif

if (n_elements(init) eq 0) then begin &$
  ; --- Read RHD model(s) ---
  rhd1_file=modelset + '.sta' &$
  rhd2_file=model + modelident + '.end' &$
  ;rhd2_file=model + modelident + '.full' &$
  ;
  sta=uio_dataset_rd(rhd_dir + rhd1_file) &$
  eosbox, sta, eos=EOS, ierror=ierror & help,ierror &$
  fin=uio_dataset_rd(rhd_dir + rhd2_file, n=0) &$
  eosbox, fin, eos=EOS, ierror=ierror & help,ierror &$
  ;
  init=1 &$
endif

x_rhd  =reform(fin.z.xc1) * x_fac
y_rhd  =reform(fin.z.xc3) * x_fac
r_rhd  =reform(combox('radius', fin)) * x_fac
nx     =n_elements(x_rhd)
rho_rhd=reform(fin.z.rho) * rho_fac
ei_rhd =reform(fin.z.ei)  * e_fac
P_rhd  =reform(fin.eos.P) * P_fac
;P_rhd  =(par.Gamma-1.0)*ei_rhd*rho_rhd
vr_rhd =reform(combox('v_radial',fin)) * v_fac
vt_rhd =reform(combox('v_tangential',fin)) * v_fac
machr_rhd =reform(combox('mach_radial',fin))
vx_rhd=reform(fin.z.v1)   * v_fac
vy_rhd=reform(fin.z.v3)   * v_fac
v_rhd=sqrt(vx_rhd^2+vy_rhd^2)
time   =fin.z.time        * t_fac
itime  =fin.z.itime

P_sta  =reform(sta.eos.P) * P_fac
P_staCenter=P_sta[nx/2,nx/2]
ind=sort(r_rhd)
P_sta=P_sta[ind]
r_sort=r_rhd[ind]
vt_sta=reform(combox('v_tangential',sta)) * v_fac
vt_sta=vt_sta[ind]

; --- File names for plot output ---
file_prefix=''
file_id=file_prefix + model
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Choose size of postscript output ---
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'X' &$
  !P.FONT=0 &$
;  !P.CHARSIZE=1.5 &$
;  !P.SYMSIZE=1.0 &$
  s_arrow=2000 &$
endif else begin &$
  set_plot, 'ps' &$
  !P.FONT=0 &$
;  device, filename='../hd/rhdtst/' + model + '.ps', $
;          /portrait, xsize=18.0, ysize=11.0, xoffset=0.4,yoffset=0.4,color=0 &$
  device, filename=ps_file, $
          encapsulated=(strupcase(dev) eq 'EPS'), $
          /portrait, xsize=18.0, ysize=17.0, xoffset=0.4, yoffset=0.4, color=1, bits=8 &$
;          /portrait, xsize=9.0, ysize=9.4, xoffset=0.4, yoffset=0.4, color=1, bits=8 &$
;  device, filename=ps_file, $
;          /landscape, xsize=28.0, ysize=18.0, xoffset=1.0,yoffset=29.2, $
    color=0 &$
;  !P.CHARSIZE=0.75 &$
;  !P.SYMSIZE=0.5 &$
  s_arrow=40000 &$
endelse
modctv, dev

; --- Choose color table ---
loadct, ct

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Plot settings ---
!X.MARGIN=[12,1]
!Y.MARGIN=[4,2]
!X.THICK=2
!Y.THICK=2
!X.STYLE=1
!Y.STYLE=1
!X.TICKLEN=-0.02
!y.TICKLEN=-0.02
!P.CHARSIZE=0.75
!P.SYMSIZE=0.5
!P.MULTI=[0,2,2,0,0]
psym=psymuser(102)

; --- Plot ---
P_min=min(alog10(P_rhd[dnx:nx-1-dnx,dnx:nx-1-dnx]), max=P_max)
plotfield, x_rhd, y_rhd, alog10(P_rhd), $
  title=model, $
  /tv, t_sample=1, drange=[P_min, P_max], $
  str=0, xvec=vx_rhd, yvec=vy_rhd, s_color=!ctv.green, s_colu=29, s_row=28, s_ar=2500, s_time=100.0, $
  contour=0, c_color=!ctv.white, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;c_levels=0.2501+findgen(30)*0.0001

normct, ct & modctv,dev
pcolor=!P.COLOR
!P.COLOR=!ctv.blue
!P.CHARSIZE=!P.CHARSIZE*0.8
legend, [funct, $
         '!9g!X='  + string(par.gamma, format='(F5.3)'), $
         't='      + string(time,      format='(F7.3)'), $
         'n!Dt!N=' + string(itime,     format='(I4)'  ), $
         'n!Dx!N=' + string(nx,        format='(I4)'  )  $
        ], $
box=0, xali=0.0, yali=1.0, margin=0.1, spacing=0.9, pspacing=0.1

if ( (where(tag_names(par) eq 'HDTIMEINTEGRATIONSCHEME'))[0] ne -1) then begin &$
  hdTimeIntegrationScheme=par.hdTimeIntegrationScheme &$
endif else begin &$
  hdTimeIntegrationScheme='' &$
endelse
if ( (where(tag_names(par) eq 'HDTRANSVELOMODE'))[0] ne -1) then begin &$
  hdTransVeloMode=par.hdTransVeloMode &$
endif else begin &$
  hdTransVeloMode='' &$
endelse
legend, ['Solv=' + par.hdscheme, $
         'Rec!Dx!N=' + par.reconstruction, $
         'TrV=' + hdTransVeloMode, $
         'Dir!Dx!N=' + par.hdsplit, $
         'Int!Dt!N=' + hdTimeIntegrationScheme $
        ], box=0, xali=1.0, yali=1.0, margin=0.1, spacing=0.9

legend, ['T!Ddiff,lin!N=' + string(par.C_hydTdiffLin, format='(F5.2)'), $
         'T!Ddiff,mach!N=' + string(par.C_hydTdiffMach, format='(F5.2)'), $
         'C!DSlopeRed!N=' + string(par.C_SlopeRed, format='(F5.2)'), $
         'C!DRecContShift!N=' + strtrim(string(par.C_RecContShift, format='(F8.3)'),2), $
         'C!DRecContSteep!N=' + strtrim(string(par.C_RecContSteep, format='(F8.3)'),2) $
        ], box=0, xali=0.0, yali=0.0, margin=0.1, spacing=0.9, pspacing=0.1

if ( (where(tag_names(par) eq 'C_VISP2PCOEFF'))[0] ne -1) then begin &$
  c_visp2pcoeff=par.c_visp2pcoeff &$
endif else begin &$
  c_visp2pcoeff=0.0 &$
endelse
if ( (where(tag_names(par) eq 'C_VISP2PINCL1'))[0] ne -1) then begin &$
  c_visp2pincl1=par.c_visp2pincl1 &$
endif else begin &$
  c_visp2pincl1=0.0 &$
endelse
if ( (where(tag_names(par) eq 'C_VISP2PHYPSMAGORINSKY'))[0] ne -1) then begin &$
  c_visp2phypsmagorinsky=par.c_visp2phypsmagorinsky &$
endif else begin &$
  c_visp2phypsmagorinsky=0.0 &$
endelse

legend, ['C!DVisSma!N='     + string(par.c_visSmagorinsky, format='(F4.2)'), $
         'C!DVisArt!N='     + string(par.c_visArtificial, format='(F4.2)'), $
         'C!DVisPr!N='      + string(par.c_visPrTurb, format='(F4.2)'), $
         'C!DVisP2P!N='     + string(c_visp2pCoeff, format='(F4.2)'), $
         'C!DVisP2PhSma!N=' + string(c_visp2phypsmagorinsky, format='(F4.2)'), $
         'C!DVisP2P1!N=' + string(c_visp2pIncl1, format='(F4.2)') $
        ], box=0, xali=1.0, yali=0.0, margin=0.1, spacing=0.9
!P.CHARSIZE=!P.CHARSIZE/0.8
!P.COLOR=pcolor

loadct, ct
; --- Plot ---
vt_min=min(vt_rhd[dnx:nx-1-dnx,dnx:nx-1-dnx], max=vt_max)
plotfield, x_rhd, y_rhd, (vt_rhd), $
  /tv, t_sample=1, drange=[vt_min,vt_max], $
  dev=dev, psformat='nochange', /psnoinit, /noclose
normct, ct & modctv,dev
plotfield, x_rhd, y_rhd, (vt_rhd), /over, $
  str=1, xvec=vx_rhd, yvec=vy_rhd, s_color=!ctv.green, s_colu=29, s_row=28, s_ar=s_arrow, s_time=10.0, $
  dev=dev, psformat='nochange', /psnoinit, /noclose

;  contour=0, c_color=!ctv.white, $
;c_levels=0.2501+findgen(30)*0.0001

!X.STYLE=0
!Y.STYLE=0
!X.TICKLEN=0
!y.TICKLEN=0

plotlines, r_rhd, alog10(P_rhd/P_staCenter), title='log(P)', aspect=1.0, $
  ; drange=[min(alog10([P_rhd[*],P_sta[*]])), max(alog10([P_rhd[*],P_sta[*]]))], $
  drange=[P_min < min(alog10(P_sta)), P_max > max(alog10(P_sta))]-alog10(P_staCenter), $
  xrange=[0,max(r_rhd)], $
  psym=1, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, r_sort, alog10(P_sta/P_staCenter), /over, $
  linecolor=!ctv.cyan, $
  dev=dev, psformat='nochange', /psnoinit, /noclose


vtstamax=max(vt_sta)
plotlines, r_rhd, vt_rhd, title='v!Dtangential,radial!N', aspect=1.0, $
;  drange=[min([vt_rhd[*],vr_rhd[*],vt_sta[*]]), max([vt_rhd[*],vr_rhd[*],vt_sta[*]])], $
  drange=[-vtstamax*0.1,(vtstamax > max(vt_rhd))], $
  xrange=[0,max(r_rhd)], $
  psym=1, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, r_rhd, vr_rhd, /over, $
  psym=psymuser(102), linecolor=!ctv.red, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, r_sort, vt_sta, /over, $
  linecolor=!ctv.cyan, $
  dev=dev, psformat='nochange', /psnoinit, /noclose

if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else if (dev eq 'ps') then begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endif

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

;print, total(rho_rhd), total(rho_rhd*ei_rhd)
print, 'M: ', total(combox('rho'    ,sta,par=par)), total(combox('rho'    ,fin,par=par))
;print, 'E: ', total(combox('rhoeikg',sta,par=par)), total(combox('rhoeikg',fin,par=par))
print, 'E: ', total(combox('rhoeik',sta,par=par)), total(combox('rhoeik',fin,par=par))
print, 'L: ', total(combox('l2'     ,sta,par=par)), total(combox('l2'     ,fin,par=par))

; end ; rhd_ploteddy
