; pro rhd_plotsedov
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_plotsedov ('rhd_plot_sedov')
;
; PURPOSE:
;   Plot HD-quantities from RHD simulation
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @rhd_plotsedov
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   ;   Before first use type
;     delvar, init
;     uio_data, mode='allclose'  ; --- Attention ---
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-11-10 Written by Bernd Freytag, Astrophysik Kiel
;   2011-08-26 (B.F. Lyon) Additional curves
;-
;------------------------------------------------------------------------------

if (not keyword_set(external_control)) then begin &$
  ; --- Choose plot device ---
  ;dev='ps' &$
  dev='x' &$
  png_flag=0 &$
  jpg_flag=0 &$
  ;
  ; --- Directories ---
  ;rhd_dir='../hd/rhdtst/' &$
  rhd_dir='/home/bf/dat/job/j4/bak/' &$
  ;ref_dir=rhd_dir &$
  ;
  ; --- Problems ---
  ;funct='sedov1d' &$
  ;funct='sedov2d' &$
  funct='sedov' &$
  ;
  ; --- Models ---
  ;model='d2tst' + funct + '04n0037' &$
  ;model='d2tst' + funct + '04n1002' &$
  ;model='d2tst' + funct + '04n1102' &$
  ;model='d2tst' + funct + '04n1247' &$
  ;model='d2tst' + funct + '04n1347' &$
  ;model='d2tst' + funct + '04n1447' &$
  ;
  model='d2tst' + funct + '04n1407' &$
  ;
  ;model='d2tst' + funct + '02n1331' &$
  ;model='d2tst' + funct + '02n0007' &$
  ;model='d2tst' + funct + '02n2757' &$
  ;model='d2tst' + funct + '02n2827' &$
  ;model='d2tst' + funct + '02n1047a' &$
  ;model='d2tst' + funct + '03n1047a' &$
  ;model='d2tst' + funct + '02n4627' &$
  ;model='d2tst' + funct + '02n4677' &$
  ;model='d3tst' + funct + '01n657' &$
  modelident='_1' &$
  ;model=funct + '_constant' &$
  ;model=funct + '_minmod' &$
  ;model=funct + '_vanleer' &$
  ;model=funct + '_superbee' &$
  ;model=funct + '_superbeemod' &$
  ;model=funct + '_pp' &$
  rhd1_file='d2tst' + funct + '02' + '.sta' &$
  ;rhd1_file=model + '.sta' &$
  ;rhd2_file=model + '.end' &$
  ;rhdpar_file=model + '_par.txt' &$
  ;ref_file='' &$
endif

; --- Common paths ---
eosdisk   =getenv('HOME') + '/for/eos/dat/'
opadisk   =getenv('HOME') + '/for/opa/dat/'
rhdpar_file=model + modelident + '.par'
rhd2_file=model + modelident + '.end' & nr2=0
;rhd2_file=model + modelident + '.full' & nr2=6
;rhdpar_file='rhd.par'
;rhd2_file='rhd.end'

; --- Scaling factors ---
x_fac   =1.0e-6
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0e+08
e_fac   =v_fac^2
rhoe_fac=rho_fac*e_fac
P_fac   =rhoe_fac

delvar, init

if (n_elements(init) eq 0) then begin &$
  ; --- Read parameter file ---
  ;uio_data, mo='open', file=rhd_dir + rhdpar_file, fam=funct + 'par' &$
  par=uio_struct_rd(rhd_dir + rhdpar_file) &$
  ;
  ;reconstruction=uio_d('reconstruction') &$
  ;C_interaction=uio_d('c_interaction') &$
  ;gamma=uio_d('gamma') &$
  ;rhd1_file=uio_d('infile_start') &$
  ;rhd2_file=uio_d('outfile_end') &$
  ;rhd2_file=model + modelident + '.end' &$
endif
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;  dfopta, opadisk + par.opafile &$
endif

if (n_elements(init) eq 0) then begin &$
  ; --- Read RHD model ---
;  uio_data, mo='open', file=rhd_dir + rhd2_file, fam='rhd' &$
;  x1_rhd    =reform(uio_d('xc1')) * x_fac &$
;  x2_rhd    =reform(uio_d('xc2')) * x_fac &$
;  p1_rhd   =reform(uio_d('p1' )) &$
;  p2_rhd   =reform(uio_d('p2' )) &$
;  rho_rhd  =reform(uio_d('rho')) &$
;  v1_rhd    =p1_rhd/rho_rhd      * v_fac &$
;  v2_rhd    =p2_rhd/rho_rhd      * v_fac &$
;  rho_rhd  =rho_rhd              * rho_fac &$
;  eik_rhd=reform(uio_d('eik')) * eps_fac &$
;  epsi_rhd=eik_rhd/rho_rhd - 0.5*(v1_rhd^2+v2_rhd^2) &$
;  ei_rhd=epsi_rhd*rho_rhd &$
;  p_rhd=(gamma-1.0)*ei_rhd &$
;  time=uio_d('time') &$
;  ntime=uio_d('ntime') &$
;  nx=n_elements(x_rhd) &$
  sta=uio_dataset_rd(rhd_dir + rhd1_file) &$
  eosbox, sta, eos=EOS, ierror=ierror & help,ierror &$
  fin=uio_dataset_rd(rhd_dir + rhd2_file,n=nr2) &$
  eosbox, fin, eos=EOS, ierror=ierror & help,ierror &$
  ;
  init=1 &$
endif

i2=n_elements(fin.z.xc2)/2
;i2=n_elements(fin.z.xc2)*3/4
;i2=35

x_rhd  =reform(fin.z.xc1) * x_fac
y_rhd  =reform(fin.z.xc3) * x_fac
r_rhd  =reform((combox('radius', fin))[*,i2,*]) * x_fac
nx     =n_elements(x_rhd)
rho_rhd=reform(fin.z.rho[*,i2,*]) * rho_fac
ei_rhd =reform(fin.z.ei[*,i2,*])  * e_fac
P_rhd  =reform(fin.eos.P[*,i2,*]) * P_fac
;P_rhd  =(par.Gamma-1.0)*ei_rhd*rho_rhd
vr_rhd =reform((combox('v_radial',fin))[*,i2,*]) * v_fac
vt_rhd =reform((combox('v_tangential',fin))[*,i2,*]) * v_fac
machr_rhd =reform((combox('mach_radial',fin))[*,i2,*])
vx_rhd=reform(fin.z.v1[*,i2,*])   * v_fac
vy_rhd=reform(fin.z.v3[*,i2,*])   * v_fac
v_rhd=sqrt(vx_rhd^2+vy_rhd^2)
time   =fin.z.time        * t_fac
itime  =fin.z.itime
; --- File names for plot output ---
file_prefix=''
file_id=file_prefix + model
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Choose size of postscript output ---
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'X' &$
  !P.FONT=0 &$
;  !P.CHARSIZE=1.5 &$
;  !P.SYMSIZE=1.0 &$
endif else begin &$
  set_plot, 'ps' &$
  !P.FONT=0 &$
;  device, filename='../hd/rhdtst/' + model + '.ps', $
;          /portrait, xsize=18.0, ysize=11.0, xoffset=0.4,yoffset=0.4,color=0 &$
  device, filename=ps_file, $
          encapsulated=(strupcase(dev) eq 'EPS'), $
          /portrait, xsize=18.0, ysize=9.4, xoffset=0.4, yoffset=0.4, color=1, bits=8 &$
;          /portrait, xsize=9.0, ysize=9.4, xoffset=0.4, yoffset=0.4, color=1, bits=8 &$
;  device, filename=ps_file, $
;          /landscape, xsize=28.0, ysize=18.0, xoffset=1.0,yoffset=29.2, $
    color=0 &$
;  !P.CHARSIZE=0.75 &$
;  !P.SYMSIZE=0.5 &$
endelse
modctv, dev

; --- Choose color table ---
;normct, 5
normct, 3

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Plot settings ---
!X.MARGIN=[6,1]
!Y.MARGIN=[4,1]
!X.THICK=2
!Y.THICK=2
!X.STYLE=1
!Y.STYLE=1
!X.TICKLEN=-0.02
!y.TICKLEN=-0.02
!P.CHARSIZE=0.75
!P.SYMSIZE=0.5
!P.MULTI=[0,2,2,0,0]
psym=psymuser(102)

; --- Plot ---
plotfield, x_rhd, y_rhd, alog10(ei_rhd), $
  /tv, t_sample=1, drange=[-0.08,0.008]*0.0, $
  str=0, xvec=vx_rhd, yvec=vy_rhd, s_color=!ctv.green, s_colu=52, s_row=51, s_ar=2500, s_time=100.0, $
  contour=0, c_color=!ctv.white, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;c_levels=0.2501+findgen(30)*0.0001

!P.COLOR=!ctv.white
!P.CHARSIZE=0.6
legend, [funct, $
         '!9g!X='  + string(par.gamma, format='(F5.3)'), $
         't='      + string(time,      format='(F7.3)'), $
         'n!Dt!N=' + string(itime,     format='(I4)'  ), $
         'n!Dx!N=' + string(nx,        format='(I4)'  )  $
        ], $
box=0, xali=0.0, yali=1.0, margin=0.1, spacing=0.9, pspacing=0.1


if ( (where(tag_names(par) eq 'HDTIMEINTEGRATIONSCHEME'))[0] ne -1) then begin &$
  hdTimeIntegrationScheme=par.hdTimeIntegrationScheme &$
endif else begin &$
  hdTimeIntegrationScheme='' &$
endelse
if ( (where(tag_names(par) eq 'HDTRANSVELOMODE'))[0] ne -1) then begin &$
  hdTransVeloMode=par.hdTransVeloMode &$
endif else begin &$
  hdTransVeloMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTROPYWAVEMODE'))[0] ne -1) then begin &$
  hdEntropyWaveMode=par.hdEntropyWaveMode &$
endif else begin &$
  hdEntropyWaveMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTHALPYAVGMODE'))[0] ne -1) then begin &$
  hdEnthalpyAvgMode=par.hdEnthalpyAvgMode &$
endif else begin &$
  hdEnthalpyAvgMode='' &$
endelse
;
legend, ['Solv=' + par.hdscheme, $
         'Rec!Dx!N=' + par.reconstruction, $
         'Mo=' + hdTransVeloMode+ ' / ' + hdEntropyWaveMode + ' / ' + hdEnthalpyAvgMode, $
         'Dir!Dx!N=' + par.hdsplit, $
         'Int!Dt!N=' + hdTimeIntegrationScheme $
        ], box=0, xali=1.0, yali=1.0, margin=0.1, spacing=0.9


legend, ['T!DdiffLin!N=' + string(par.C_hydTdiffLin, format='(F5.2)'), $
         'T!DdiffMach!N=' + string(par.C_hydTdiffMach, format='(F5.2)'), $
         'C!DSlopeRed!N=' + string(par.C_SlopeRed, format='(F5.3)'), $
         'C!DRecContShift!N=' + string(par.C_RecContShift, format='(F5.2)'), $
         'C!DRecContSteep!N=' + string(par.C_RecContSteep, format='(F5.3)'), $
         'C!DLowMachcsFac!N=' + string(par.C_hydLowMachcsFactor, format='(F5.2)'), $
         's!Ddiff!N=' + string(par.C_hydsDiffVelo, format='(F5.2)'), $
         'v!Ddiff!N=' + string(par.C_hydvDiffVelo, format='(F5.2)') $
        ], box=0, xali=0.0, yali=0.0, margin=0.1, spacing=0.9, pspacing=0.1


LegAr=['C!DVisSma!N='     + string(par.c_visSmagorinsky, format='(F4.2)'), $
       'C!DVisArt!N='     + string(par.c_visArtificial, format='(F4.2)'), $
       'C!DVisPr!N='      + string(par.c_visPrTurb, format='(F4.2)')]
if ( (where(tag_names(par) eq 'C_VISP2PCOEFF'))[0] ne -1) then begin &$
  LegAr=[LegAr, 'C!DVisP2P!N='     + string(par.c_visp2pcoeff, format='(F4.2)')] &$
endif
if ( (where(tag_names(par) eq 'C_VISP2PHYPSMAGORINSKY'))[0] ne -1) then begin &$
  LegAr=[LegAr, 'C!DVisP2PhSma!N=' + string(par.c_visp2phypsmagorinsky, format='(F4.2)')] &$
endif
if ( (where(tag_names(par) eq 'C_VISP2PHYPARTIFICIAL'))[0] ne -1) then begin &$
  LegAr=[LegAr, 'C!DVisP2PhArt!N=' + string(par.c_visp2phypartificial, format='(F4.2)')] &$
endif
if ( (where(tag_names(par) eq 'C_VISP2DIVRHOV'))[0] ne -1) then begin &$
  LegAr=[LegAr, 'C!DVisP2Pdiv!N=' + string(par.c_visp2pdivrhov, format='(F4.2)')] &$
endif
if ( (where(tag_names(par) eq 'C_VISP2PINCL1'))[0] ne -1) then begin &$
  LegAr=[LegAr, 'C!DVisP2P1!N=' + string(par.c_visp2pIncl1, format='(F4.2)')] &$
endif
;
legend, LegAr, box=0, xali=1.0, yali=0.0, margin=0.1, spacing=0.9

;legend, ['C!DVisSma!N='     + string(par.c_visSmagorinsky, format='(F4.2)'), $
;         'C!DVisArt!N='     + string(par.c_visArtificial, format='(F4.2)'), $
;         'C!DVisPr!N='      + string(par.c_visPrTurb, format='(F4.2)'), $
;         'C!DVisP2P!N='     + string(c_visp2pCoeff, format='(F4.2)'), $
;         'C!DVisP2PhSma!N=' + string(c_visp2phypsmagorinsky, format='(F4.2)'), $
;         'C!DVisP2PhArt!N=' + string(par.c_visp2phypartificial, format='(F4.2)'), $
;         'C!DVisP2PhExp!N=' + string(par.c_visp2phypexpansion, format='(F4.2)'), $
;         'C!DVisP2Pdiv!N=' + string(par.c_visp2pdivrhov, format='(F4.2)'), $
;         'C!DVisP2P1!N=' + string(c_visp2pIncl1, format='(F4.2)') $
;        ], box=0, xali=1.0, yali=0.0, margin=0.1, spacing=0.9

;         'VisP2PRecN=' + par.visp2phypreconstruction $

; --- Plot ---
plotfield, x_rhd, y_rhd, (vr_rhd), $
  /tv, t_sample=1, drange=[-0.08,0.008]*0.0, $
  str=0, xvec=vx_rhd, yvec=vy_rhd, s_color=!ctv.green, s_colu=52, s_row=51, s_ar=2500, s_time=100.0, $
  contour=0, c_color=!ctv.white, $
  dev=dev, psformat='nochange', /psnoinit, /noclose
;c_levels=0.2501+findgen(30)*0.0001

!X.STYLE=0
!Y.STYLE=0
plotlines, r_rhd, alog10(ei_rhd), title='log(e!Di!N)', $
  psym=1, drange=[-1.0,3.9], xrange=[0,max(r_rhd)/sqrt(2.0)], $
   dev=dev, psformat='nochange', /psnoinit, /noclose
;plotlines, r_rhd, alog10(ei_rhd_old), /over, $
;  psym=1, linecolor=!ctv.orange, $
;   dev=dev, psformat='nochange', /psnoinit, /noclose


plotlines, r_rhd, vr_rhd, title='v!Dradial!N', $
  psym=1, drange=[0.0,6.9], xrange=[0,max(r_rhd)/sqrt(2.0)], $
   dev=dev, psformat='nochange', /psnoinit, /noclose
plotlines, r_rhd, vt_rhd, /over, $
  psym=psymuser(102), linecolor=!ctv.red, $
   dev=dev, psformat='nochange', /psnoinit, /noclose
;plotlines, r_rhd, vr_rhd_old, /over, $
;  psym=1, linecolor=!ctv.orange, $
;   dev=dev, psformat='nochange', /psnoinit, /noclose

if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else if (dev eq 'ps') then begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endif

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

; end ; rhd_plotsedov
