; pro rhd_plotshock
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_plotshock ('rhd_plot_shock')
;
; PURPOSE:
;   Plot HD-quantities from RHD simulation and analytical solution
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @rhd_plotshock
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   ;   Before first use type
;     delvar, init
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   1997-10-10.97 Written by Bernd Freytag, Astrophysik Kiel
;   25. 3.99 (B.F. Copenhagen) New data format, new read routine,
;            different scaling factors
;    9. 4.99 (B.F.) New basic quantities
;   31. 1.01 (B.F.) Some modifications for case with moving rarefaction wave
;   2011-07-07 (B.F. Lyon)
;-
;------------------------------------------------------------------------------
;
if (not keyword_set(external_control)) then begin &$
  ; --- Choose plot device ---
  ;dev='x' &$
  dev='eps' &$
  png_flag=0 &$
  jpg_flag=0 &$
  ;
  ; --- Directories ---
  ;rhd_dir='/home/bf/dat/rhd/tests/rarefaction/' &$
  ;rhd_dir='/home/bf/dat/rhd/tests/dsod/' &$
  ;rhd_dir='/home/bf/dat/job/j4/bak/' &$
  ;rhd_dir=getenv('HOME') + '/dat/job/j7/wrk/' &$
  rhd_dir=getenv('HOME') + '/dat/job/j7/bak/' &$
  ;rhd_dir='/home/bf/dat/job/jf/' &$
  ;ashock_dir='/home/bf/for/hd/shock/sun/' &$
  ;ashock_dir='/home/bf/dat/rhd/tests/dhawley/' &$
  ;ashock_dir='/home/bf/dat/rhd/tests/dsod/' &$
  ;ashock_dir=getenv('HOME') + '/co5bold/resources/kiel/copenhagen/hd/shock/' &$
  ;ashock_dir=getenv('HOME') + '/co5bold/resources/crow/for/hd/shock/sun/' &$
  ashock_dir=getenv('HOME') + '/dat/job/j7/wrk/' &$
  ;
  ; --- Problems ---
  funct='sod' &$
  ;funct='sod2d' &$
  ;funct='hawley' &$
  ;funct='hawley_fine' &$
  ;funct='nordlund' &$
  ;
  ; --- Models ---
  reverse_flag=0 & shift_flag=0 &$
  ;model='d1tst' + funct + '_Constant' &$
  ;model='d1tst' + funct + '_MinMod' &$
  ;model='d1tst' + funct + '_VanLeer' &$
  ;model='d1tst' + funct + '_SuperBee' &$
  ;model='d1tst' + funct + '_PP' &$
  ;model='d1tst' + funct + '_FRmono' &$
  ;model='d1tst' + funct + '_FRmono' &$
  ;model='d1tst' + funct + '_HBweno' &$
  model='d1tst' + funct + '_VAweno' &$
  modelident='_01' &$
  ;model=funct + '_all' & 
  ;model=funct + '_constant' &$
  ;model=funct + '_minmod' &$
  ;model=funct + '_vanleer' & reverse_flag=0 & shift_flag=0 &$
  ;model=funct + '_superbee' &$
  ;model=funct + '_PP' &$
  ;model='d1tst' + funct + '_PP' &$
  ;model='d1tst' + funct + '01n2308' & reverse_flag=0 & shift_flag=0 &$
  ;model='d1tst' + funct + '01' & reverse_flag=0 & shift_flag=0 &$
  ;model=funct + '_01' & reverse_flag=0 & shift_flag=0 &$
  ;model=funct + '_n_v00_17' & reverse_flag=0 & shift_flag=0 &$
  ;model=funct + '_r_v00_01' & reverse_flag=1 & shift_flag=0 &$
  ;model=funct + '_n_v05_22' & reverse_flag=0 & shift_flag=1 &$
  ;model=funct + '_r_vm10_17' & reverse_flag=1 & shift_flag=1 &$
  ;rhd1_file=model + '.sta' &$
  ;rhd2_file=model + '.end' &$
  ;ashock_file='sod.uio' &$
endif
;
ashock_file=funct + '.uio'
rhdpar_file=model + modelident + '.par'
;
; --- Common paths ---
eosdisk   =getenv('HOME') + '/for/eos/dat/'
opadisk   =getenv('HOME') + '/for/opa/dat/'
;
; --- Scaling factors ---
x_fac   =1.0e-5
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0e+08
e_fac   =v_fac^2
rhoe_fac=rho_fac*e_fac
p_fac   =rhoe_fac
s_fac   =1.0E-09
;
delvar, init
;
if (n_elements(init) eq 0) then begin &$
  ; --- Read parameter file ---
  par=uio_struct_rd(rhd_dir + rhdpar_file) &$
  ;rhd1_file=par.infile_start &$
  ;rhd2_file=par.outfile_end &$
  rhd2_file=model + modelident + '.end' &$
  ;rhd2_file=model + modelident + '.full' &$
endif
;print, par.gamma
if (n_elements(EOS) eq 0) then begin &$
  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;  dfopta, opadisk + par.opafile &$
endif
;
if (n_elements(init) eq 0) then begin &$
  ; --- Read RHD model ---
  ;sta=uio_dataset_rd(rhd_dir + rhd1_file) &$
  fin=uio_dataset_rd(rhd_dir + rhd2_file,n=0) &$
  eosbox, fin, eos=EOS, ierror=ierror &$
  if (ierror ne 0) then help,ierror &$
  ;init=1 &$
endif
x_rhd  =reform(fin.z.xc1) * x_fac
rho_rhd=reform(fin.z.rho) * rho_fac
ei_rhd =reform(fin.z.ei)  * e_fac
s_rhd  =reform(fin.eos.s) * s_fac
v_rhd  =reform(fin.z.v1)  * v_fac
time   =fin.z.time        * t_fac
p_rhd  =(par.Gamma-1.0)*ei_rhd*rho_rhd
itime  =fin.z.itime
nx=n_elements(x_rhd)
if (shift_flag eq 1) then begin &$
  print, 'Shift: ', v_rhd(0)*time &$
  x_rhd=x_rhd-v_rhd(0)*time   &$
  v_rhd=v_rhd-v_rhd(0)        &$
endif
if (reverse_flag eq 1) then begin &$
  v_rhd=-v_rhd   &$
endif

if (n_elements(init) eq 0) then begin &$
  ; --- Read analytical shock ---
  ashock=uio_dataset_rd(ashock_dir+ashock_file) &$
;  ashock=uio_struct_rd(ashock_dir+ashock_file) &$
  eosbox, ashock, eos=EOS, ierror=ierror &$
  if (ierror ne 0) then help,ierror &$
endif
x_ashock  =reform(ashock.z.xc1)*x_fac
x_ashock  =x_ashock * (fin.z.time/ashock.z.time)
v_ashock  =reform(ashock.z.v1) *v_fac
p_ashock  =reform(ashock.z.p)  *p_fac
rho_ashock=reform(ashock.z.rho)*rho_fac
ei_ashock =reform(ashock.z.ei) *e_fac
s_ashock =reform(ashock.eos.s) *s_fac
;ei_ashock =reform(ashock.z.eik/ashock.z.rho-0.5*ashock.z.v1^2) *e_fac
if (reverse_flag eq 1) then begin &$
  x_ashock=-x_ashock   &$
endif


; --- File names for plot output ---
file_prefix='pde1d_'
file_id=file_prefix + model
jpg_file=file_id + '.jpg'
png_file=file_id + '.png'
if (strupcase(dev) eq 'EPS') then begin &$
  ps_file  =file_id + '.eps' &$
endif else begin &$
  ps_file  =file_id + '.ps' &$
endelse

; --- Save plot settings ---
Pbak=!P & Xbak=!X & Ybak=!Y

; --- Choose color table ---
normct

; --- Choose size of postscript output ---
if (strupcase(dev) eq 'X') then begin &$
  set_plot, 'X' &$
  !P.FONT=0 &$
  !P.CHARSIZE=1.5 &$
  !P.SYMSIZE=1.0 &$
  spacing=1.0 &$
endif else begin &$
  set_plot, 'ps' &$
  !P.FONT=0 &$
;  device, filename='../hd/rhdtst/' + model + '.ps', $
;          /portrait, xsize=18.0, ysize=11.0, xoffset=0.4,yoffset=0.4,color=0 &$
  device, filename=ps_file, $
          encapsulated=(strupcase(dev) eq 'EPS'), $
          /portrait, xsize=18.0, ysize=8.8, xoffset=0.4, yoffset=0.4, color=1, bits=8 &$
;  device, filename=ps_file, $
;          /landscape, xsize=28.0, ysize=18.0, xoffset=1.0,yoffset=29.2, $
    color=0 &$
  modctv, 'PS' &$
  !P.CHARSIZE=1.2 &$
  !P.SYMSIZE=0.5 &$
  spacing=0.7 &$
endelse
;
color=!ctv.green
;!P.MULTI=[0,2,2,0,0]
!P.MULTI=[0,3,2,0,0]
!X.MARGIN=[7,2]
;!Y.MARGIN=[2,0]
!Y.MARGIN=[2,1]
!X.THICK=2
!Y.THICK=2
!X.STYLE=1
;!Y.STYLE=3 & y_fac=0.0
!Y.STYLE=1 & y_fac=1.0
psym=-psymuser(102)

x_range=[-0.5,0.5]
;x_range=[-0.05,0.05]
if (funct eq 'sod') then begin &$
  v_range=[-0.05,1.05] & p_range=[ 0.05,1.05] & rho_range=[ 0.05,1.05] & e_range=[ 1.6,3.1] &$
  rhov_range=[-0.05, 0.45] & s_range=[1.97, 2.10]*1.0E+09 &$
  ;
  ;v_range=[-0.05,1.05] & p_range=[ 0.2,0.6] & rho_range=[ 0.2,0.8] & e_range=[ 1.6,2.2] &$
  ;rhov_range=[-0.05, 0.45] & s_range=[1.98, 1.99]*1.0E+09 &$
  ;v_range=[0.90,0.95] & p_range=[ 0.28,0.33] & rho_range=[ 0.05,1.05] & e_range=[ 1.6,3.1] &$
  ;rhov_range=[0.37, 0.43] & s_range=[1.97, 2.10]*1.0E+09 &$
endif else begin &$
  v_range=[-0.01,0.16] & p_range=[-0.03,0.71] & rho_range=[-5.0,110.0] & e_range=[-0.001,0.012] &$
endelse

; --- Plot velocity ---
LegendCharSize=0.55
plot, x_ashock, v_ashock, ytitle='v', xrange=x_range, yrange=v_range*y_fac
oplot,x_rhd,    v_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
legend, [funct, $
         '!9g!X='  + string(par.gamma, format='(F5.3)'), $
         't='      + string(time,      format='(F5.3)'), $
         'n!Dt!N=' + string(itime,     format='(I4)'  ), $
         'n!Dx!N=' + string(nx,        format='(I4)'  )   ], $
   box=0, xali=0.0, yali=1.0, margin=0.1, spacing=spacing, pspacing=0.1, charsize=LegendCharSize

; --- Plot mass flux ---
plot, x_ashock, rho_ashock*v_ashock, ytitle=transfont('!9r!Xv'), xrange=x_range, yrange=rhov_range
oplot,x_rhd,    rho_rhd   *v_rhd   , psym=psym, symsize=!P.SYMSIZE, color=color

; --- Plot density ---
plot, x_ashock, rho_ashock, ytitle=transfont('!9r!X'), xrange=x_range, yrange=rho_range*y_fac
oplot,x_rhd,    rho_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
if ( (where(tag_names(par) eq 'HDTIMEINTEGRATIONSCHEME'))[0] ne -1) then begin &$
  hdTimeIntegrationScheme=par.hdTimeIntegrationScheme &$
endif else begin &$
  hdTimeIntegrationScheme='' &$
endelse
if ( (where(tag_names(par) eq 'HDTRANSVELOMODE'))[0] ne -1) then begin &$
  hdTransVeloMode=par.hdTransVeloMode &$
endif else begin &$
  hdTransVeloMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTROPYWAVEMODE'))[0] ne -1) then begin &$
  hdEntropyWaveMode=par.hdEntropyWaveMode &$
endif else begin &$
  hdEntropyWaveMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTHALPYAVGMODE'))[0] ne -1) then begin &$
  hdEnthalpyAvgMode=par.hdEnthalpyAvgMode &$
endif else begin &$
  hdEnthalpyAvgMode='' &$
endelse
legend, ['Solv =' + par.hdscheme, $
         'xRec =' + par.reconstruction, $
         'xDir =' + par.hdsplit, $
         'tInt =' + hdTimeIntegrationScheme, $
         'MoEnt=' + hdEntropyWaveMode, $
         'MoEnthalpy=' + hdEnthalpyAvgMode $
        ], $
        box=0, xali=1.0, yali=1.0, margin=0.1, spacing=spacing, pspacing=0.1, charsize=LegendCharSize

; --- Plot pressure ---
plot, x_ashock, p_ashock, ytitle='p', xrange=x_range, yrange=p_range*y_fac
oplot,x_rhd,    p_rhd, psym=psym, symsize=!P.SYMSIZE, color=color
legend, ['C!DSlopeRed!N    =' + string(par.C_SlopeRed, format='(F5.2)'), $
;         'C!DRecContShift!N=' + strtrim(string(par.C_RecContShift, format='(F8.3)'),2), $
         'C!DRecContSteep!N=' + strtrim(string(par.C_RecContSteep, format='(F8.3)'),2), $
         'C!DSmallFlctFac!N=' + strtrim(string(par.C_hydSmallFluctFactor, format='(F8.3)'),2), $
         'C!DLowMachcsBas!N=' + string(par.C_hydLowMachcsBase, format='(F5.2)'), $
         'C!DLowMachcsFac!N=' + string(par.C_hydLowMachcsFactor, format='(F5.2)'), $
         'T!DdiffLin!N =' + string(par.C_hydTdiffLin, format='(F5.2)'), $
         'T!DdiffMach!N=' + string(par.C_hydTdiffMach, format='(F5.2)'), $
         's!Ddiff!N    =' + string(par.C_hydsDiffVelo, format='(F5.2)'), $
         'v!Ddiff!N    =' + string(par.C_hydvDiffVelo, format='(F5.2)') $
        ], $
        box=0, xali=1.0, yali=1.0, margin=0.1, spacing=spacing, charsize=LegendCharSize

;         string(par.C_visNeu1,    format='("C!DvisNeu1!N=",F4.2)'), $
;         string(par.C_visNeu2,    format='("C!DvisNeu2!N=",F4.2)'), $

; --- Plot internal energy ---
plot, x_ashock, ei_ashock, ytitle='e', xrange=x_range, yrange=e_range*y_fac
oplot,x_rhd,    ei_rhd, psym=psym, symsize=!P.SYMSIZE, color=color

; --- Plot entropy ---
plot, x_ashock, s_ashock, ytitle='s', xrange=x_range, yrange=s_range*s_fac
oplot,x_rhd,    s_rhd, symsize=!P.SYMSIZE, color=color
oplot,x_rhd,    s_rhd, psym=psym, symsize=!P.SYMSIZE, color=color

if (strupcase(dev) eq 'X') then begin &$
  if (png_flag) then begin &$
    write_png, png_file, tvrd(true=1) &$
  endif &$
  if (jpg_flag) then begin &$
    write_jpeg, jpg_file, tvrd(true=3), true=3 &$
  endif &$
endif else if (dev eq 'ps') then begin &$
  ; --- Close postscript device ---
  device, /close &$
  set_plot, 'X' &$
  modctv, 'X' &$
endif

; --- Reset global plot parameters ---
!P=Pbak & !X=Xbak & !Y=Ybak

; end ; rhd_plotshock
