; pro rhd_plotspike
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_plotspike ('rhd_plot_spike')
;
; PURPOSE:
;   Plot HD-quantities from RHD simulation
;
; CATEGORY:
;   Hydrodynamics, Plotting
;
; CALLING SEQUENCE:
;   @rhd_plotspike
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;   A lot
;
; RESTRICTIONS:
;   Before first use type
;     delvar, init
;     uio_data, mode='allclose'  ; --- Attention ---
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20.10.97 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Directories ---
;rhd_dir='/astro/b/bf/hd/rhd/tst/'
;eosdisk='/astro/b/bf/eos/dat/'
;funct='ionspike100vp20'
;model=funct + '_04'
;model=funct + '_constant'
;model=funct + '_minmod'
;model=funct + '_vanleer'
;model=funct + '_superbee'
;model=funct + '_pp'
;model=funct
rhd_dir=getenv('HOME') + '/dat/job/j1/bak/'
eosdisk   =getenv('HOME') + '/for/eos/dat/'
funct='d1tstspike'
; model=funct + 'n1009' + '_1'
rhd1_file=funct + '.sta'
rhd2_file=model + '.end' &$
;
rhdpar_file=model + '.par'
;
; --- Choose plot device ---
;dev='ps'
dev='x'
;
; --- Scaling factors ---
;x_fac   =1.0e-5
;t_fac   =1.0
;v_fac   =x_fac/t_fac
;rho_fac =1.0e+08
;e_fac   =v_fac^2
;rhoe_fac=rho_fac*e_fac
;p_fac   =rhoe_fac
x_fac   =1.0e-05
t_fac   =1.0
v_fac   =x_fac/t_fac
rho_fac =1.0
e_fac   =v_fac^2
rhoe_fac=1.0
p_fac   =1.0
;
;if (n_elements(init) eq 0) then begin &$
  ; --- Read parameter file ---
  par=uio_struct_rd(rhd_dir + rhdpar_file) &$
;  rhd1_file=par.infile_start &$
;  rhd2_file=par.outfile_end &$
  ;
  file=eosdisk + par.eosfile &$
  if (n_elements(eos) eq 0) then begin &$
    tabinter_rdcoeff, file, eos &$
  endif else if (file ne eos.head.file) then begin &$
    tabinter_rdcoeff, file, eos &$
  endif &$
;endif
;
;if (n_elements(init) eq 0) then begin &$
  ; --- Read start model ---
  sta=uio_dataset_rd(rhd_dir + rhd1_file) &$
  eosbox, sta, eos=eos &$
  x_1  =reform(sta.z.xc1)     * x_fac &$
  rho_1=reform(sta.z.rho)     * rho_fac &$
  v_adv=avg(reform(sta.z.v1)) * v_fac &$
  ;
  ; --- Read end model ---
  fin=uio_dataset_rd(rhd_dir + rhd2_file) &$
  eosbox, fin, eos=eos &$
  x_2  =reform(fin.z.xc1)     * x_fac &$
  rho_2=reform(fin.z.rho)     * rho_fac &$
  time =fin.z.time            * t_fac &$
  itime=fin.z.itime &$
  nx=n_elements(x_2) &$
  ;
  init=1 &$
;endif
;
; --- Choose size of postscript output ---
if (dev eq 'ps') then begin &$
  set_plot, 'ps' &$
  device, filename=model + '.ps' &$
  device, /landscape, xsize=29.0, ysize=19.3, xoffset=1.0,yoffset=29.7, $
    color=1 &$
;  device, /portrait, xsize=18.0, ysize=4.5, xoffset=0.4,yoffset=0.4,color=0 &$
;  color=!P.COLOR &$
  color=!ctv.green &$
endif else begin &$
  color=!ctv.green &$
endelse
;
;!P.MULTI=[0,2,2,0,0]
;!X.MARGIN=[7,2]
!X.MARGIN=[16,4]
;!Y.MARGIN=[2,0]
!Y.MARGIN=[3,1]
!X.THICK=2
!Y.THICK=2
!X.STYLE=1
!Y.STYLE=3
!P.FONT=0
;!P.CHARSIZE=0.75
;!P.SYMSIZE=0.5
!P.CHARSIZE=2.5
!P.SYMSIZE=1.5
psym=psymuser(102)
;
plot, x_1, rho_1, psym=-1, symsize=!P.SYMSIZE*0.7, $
                  ytitle=transfont('!4q!X')
oplot,x_2, rho_2, psym=psym, symsize=!P.SYMSIZE, color=color
;
if ( (where(tag_names(par) eq 'HDTIMEINTEGRATIONSCHEME'))[0] ne -1) then begin &$
  hdTimeIntegrationScheme=par.hdTimeIntegrationScheme &$
endif else begin &$
  hdTimeIntegrationScheme='' &$
endelse
if ( (where(tag_names(par) eq 'HDTRANSVELOMODE'))[0] ne -1) then begin &$
  hdTransVeloMode=par.hdTransVeloMode &$
endif else begin &$
  hdTransVeloMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTROPYWAVEMODE'))[0] ne -1) then begin &$
  hdEntropyWaveMode=par.hdEntropyWaveMode &$
endif else begin &$
  hdEntropyWaveMode='' &$
endelse
if ( (where(tag_names(par) eq 'HDENTHALPYAVGMODE'))[0] ne -1) then begin &$
  hdEnthalpyAvgMode=par.hdEnthalpyAvgMode &$
endif else begin &$
  hdEnthalpyAvgMode='' &$
endelse
legend, ['Solv  =' + par.hdscheme, $
         'xRec  =' + par.reconstruction, $
         'xDir  =' + par.hdsplit, $
         'tInt  =' + hdTimeIntegrationScheme, $
         'MoEntr=' + hdEntropyWaveMode, $
         'MoEnth=' + hdEnthalpyAvgMode, $
         "t     ="      + strtrim(string(time, format='(F7.3)'),2), $
         "n!Dt!N   =" + strtrim(string(itime,format='(I7)')  ,2), $
         "n!Dx!N   =   " + strtrim(string(nx,   format='(I7)')  ,2), $
         "v     ="      + strtrim(string(v_adv,format='(F4.1)'),2)  $
        ],  $
        box=0, xali=1.0, yali=1.0, margin=0.2, spacing=0.9

;
;plot, fin.z.xc1, fin.z.v1, ytitle='v', ystyle=3

;plot, fin.z.xc1, sta.z.rho*(sta.z.v1-fin.z.v1)*(sta.z.v1+fin.z.v1), $
;      ytitle=transfont('!4q!X(v!U2!N!Dend!N-v!U2!N!Dsta!N), p!Dend!N-p!Dsta!N'), ystyle=3
;oplot, fin.z.xc1, fin.z.rho*fin.z.ei - sta.z.rho*sta.z.ei, color=!ctv.red

;plot, fin.z.xc1, fin.z.v1-sta.z.v1, ytitle='v!Dend!N-v!Dsta!N', yrange=[-0.03,0.03]

;
!P.MULTI=0
!X.MARGIN=[12,3]
!Y.MARGIN=[4,2]
!X.THICK=0
!Y.THICK=0
!X.STYLE=0
!Y.STYLE=0
!P.FONT=-1
!P.CHARSIZE=0
!P.SYMSIZE=0
;
if (dev eq 'ps') then begin &$
  device, /close &$
  set_plot, 'x' &$
endif
;
; end ; rhd_plotspike
