function udb_Convert, uDBType, uDB0
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   udb_Convert ('universal_data_base_Convert')
;
; PURPOSE:
;   Merge two data bases
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_Convert(uDBType, uDB0)
;
; INPUT:
;   uDBType: ('universal_Data_Base_Type') String with target type of data base
;   uDB0:    ('universal_Data_Base_0')
;
; OUTPUT:
;   result: converted data base
;
; KEYWORDS:
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_GetTags:
;   udb_GetType:
;   udb_ListAll:
;   udb_MergeStringVectors:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Not all combinations implemented, yet.
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-12-03 (B.F. Lyon)
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

if (n_params() lt 2) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Determine type of input database ---
uDB_Type0=udb_GetType(uDB0, nElDB=nElDB0)

uDB_Type2=udb_Gettype(uDBType)
nElDB2   =nElDB0

if (uDB_Type2 eq 'ListOfPointers') then begin ;-----------------------------------------------------
  ; --- Everything can be converted to a pointer list ---
  ;
  ; --- Create empty output pointer array ---
  uDB2=ptrarr(nElDB2>1, /allocate_heap)
  ;
  ; --- Copy database ---
  if (uDB_Type0 eq 'ListOfPointers') then begin ; ................................................
    for iElDB0=0,nELDB0-1 do begin
      if (ptr_valid(uDB0[iElDB0])) then begin
        *uDB2[iElDB0]=*uDB0[iElDB0]
      endif
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; .................................
    for iElDB0=0,nElDB0-1 do begin
      *uDB2[iElDB0]=uDB0[iElDB0]
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; .................................
    ; --- Get tag names and number of data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    ; --- Create template structure with only one element (but with all tags) ---
    if (nTagsDB0 ge 1) then uDBtemplate=create_struct(TagsDB0[0], (uDB0.(0))[0])
    for iTagsDB0=1,nTagsDB0-1 do begin
      uDBtemplate=create_struct(uDBtemplate, TagsDB0[iTagsDB0], (uDB0.(iTagsDB0))[0])
    endfor ; iTagsDB0
    ;help, /str, uDBtemplate
    ;
    ; --- Copy entries individually ---
    for iElDB0=0,nELDB0-1 do begin
      uDBsingle=uDBtemplate
      for iTagsDB0=0,nTagsDB0-1 do begin
        uDBsingle.(iTagsDB0)=(uDB0.(iTagsDB0))[iElDB0<(n_elements(uDB0.(iTagsDB0))-1)]
      endfor ; iTagsDB0
      ;help, iELDB0
      ;help, /str, uDBsingle
      ;
      *uDB2[iElDB0]=uDBsingle
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'UIOTable') then begin ; ...........................................
    ; --- Get tag names and number of data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    Nstr='__NAME'
    Ustr='__UNIT'
    ; --- Create template structure with only one element (but with all tags) ---
    if (nTagsDB0 ge 1) then begin
      uio_tabr, val, ident, uDB0, incolumn=1, name=valName, unit=valUnit
      uDBtemplate=create_struct(ident, val[0], ident+Nstr, valName, ident+Ustr, valUnit)
    endif
    for iTagsDB0=1,nTagsDB0-1 do begin
      uio_tabr, val, ident, uDB0, incolumn=iTagsDB0+1, name=valName, unit=valUnit
      uDBtemplate=create_struct(uDBtemplate, ident, val[0], ident+Nstr, valName, ident+Ustr, valUnit)
    endfor ; iTagsDB0
    ;help, /str, uDBtemplate
    ;
    ; --- Copy entries individually ---
    for iElDB0=0,nELDB0-1 do begin
      uDBsingle=uDBtemplate
      for iTagsDB0=0,nTagsDB0-1 do begin
        uio_tabr, val, ident, uDB0, incolumn=iTagsDB0+1
        uDBsingle.(3*iTagsDB0)=val[iELDB0]
      endfor ; iTagsDB0
      ;help, iELDB0
      ;help, /str, uDBsingle
      ;
      *uDB2[iElDB0]=uDBsingle
    endfor ; iElDB0
    ;
  endif else begin ; .............................................................................
    message, 'Unknown uDB Type.'
    ;
  endelse ; ......................................................................................
  ;
endif else if (uDB_Type2 eq 'VectorOfStructures') then begin ; -------------------------------------
  if (uDB_Type0 eq 'VectorOfStructures') then begin ; ............................................
    uDB2=uDB0
    ;
  endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; .................................
    ; --- Get names and number of tags in data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    ; --- Create template structure with only one element (but with all tags) ---
    if (nTagsDB0 ge 1) then uDBtemplate=create_struct(TagsDB0[0], (uDB0.(0))[0])
    for iTagsDB0=1,nTagsDB0-1 do begin
      uDBtemplate=create_struct(uDBtemplate, TagsDB0[iTagsDB0], (uDB0.(iTagsDB0))[0])
    endfor ; iTagsDB0
    ;help, /str, uDBtemplate
    uDB2=replicate(uDBtemplate, nElDB2)
    ;
    ; --- Copy entries individually ---
    for iElDB0=0,nELDB0-1 do begin
      ;uDBsingle=uDBtemplate
      ;help, iELDB0
      for iTagsDB0=0,nTagsDB0-1 do begin
        ;help, iTagsDB0
        ;uDBsingle.(iTagsDB0)=uDB0.(iTagsDB0)[iElDB0<(n_elements(uDB0.(iTagsDB0))-1)]
        uDB2[iElDB0].(iTagsDB0)=uDB0.(iTagsDB0)[iElDB0<(n_elements(uDB0.(iTagsDB0))-1)]
      endfor ; iTagsDB0
      ;help, /str, uDBsingle
      ;
      ;uDB2[iElDB0]=uDBsingle
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'UIOTable') then begin ; ...........................................
    ; --- Get tag names and number of data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    Nstr='__NAME'
    Ustr='__UNIT'
    ; --- Create template structure with only one element (but with all tags) ---
    if (nTagsDB0 ge 1) then begin
      uio_tabr, val, ident, uDB0, incolumn=1, name=valName, unit=valUnit
      uDBtemplate=create_struct(ident, val[0], ident+Nstr, valName, ident+Ustr, valUnit)
    endif
    for iTagsDB0=1,nTagsDB0-1 do begin
      uio_tabr, val, ident, uDB0, incolumn=iTagsDB0+1, name=valName, unit=valUnit
      uDBtemplate=create_struct(uDBtemplate, ident, val[0], ident+Nstr, valName, ident+Ustr, valUnit)
    endfor ; iTagsDB0
    ;help, /str, uDBtemplate
    uDB2=replicate(uDBtemplate, nElDB2)
    ;
    ; --- Copy entries individually ---
    for iElDB0=0,nELDB0-1 do begin
      ;uDBsingle=uDBtemplate
      for iTagsDB0=0,nTagsDB0-1 do begin
        uio_tabr, val, ident, uDB0, incolumn=iTagsDB0+1
        ;uDBsingle.(3*iTagsDB0)=val[iElDB0]
        uDB2[iElDB0].(3*iTagsDB0)=val[iElDB0]
      endfor ; iTagsDB0
      ;help, iELDB0
      ;help, /str, uDBsingle
      ;
      ;uDB2[iElDB0]=uDBsingle
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'ListOfPointers') then begin ; .....................................
    message, 'Combination of types not implemented yet'
    ; Todo
  endif else begin ; .............................................................................
    message, 'Unknown uDB Type.'
  endelse ; ......................................................................................
  ;
endif else if (uDB_Type2 eq 'StructureOfVectors') then begin ; -------------------------------------
  if (uDB_Type0 eq 'StructureOfVectors') then begin ; ............................................
    uDB2=uDB0
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; .................................
    ; --- Get names and number of tags in data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    ;
    ; --- Create output structure ---
    for iTagsDB0=0,nTagsDB0-1 do begin
      ; --- Get current tag name and value ---
      TagName =TagsDB0[iTagsDB0]
      TagValue=uDB0.(iTagsDB0)
      ;help, TagName, TagValue
      ;
      ; --- Check if it is meta data (tag name contains '__' and tag is string array) ---
      if ((strpos(TagName, '__') ne -1) and (size(TagValue, /TName) eq 'STRING')) then begin
        ; --- Tag contains meta data ---
        TagValueSort=TagValue[    sort(TagValue    )]
        TagValueSort=TagValueSort[uniq(TagValueSort)]
        if (n_elements(TagValueSort) ge 2) then begin
          message, 'Meta data in ' + TagName + ' is not uniq.', /info
        endif else begin
          ; --- Use the first element: it is representative ---
          TagValue=TagValue[0]
        endelse
      endif
      ;help, TagValue
      ;
      ; --- Append new tag name/value pair ---
      if (iTagsDB0 eq 0) then begin
        uDB2=create_struct(      TagName, TagValue)
      endif else begin
        uDB2=create_struct(uDB2, TagName, TagValue)
      endelse
    endfor ; iTagsDB0
    ;help, /str, uDB2
    ;
  endif else if (uDB_Type0 eq 'UIOTable') then begin ; ...........................................
    ; --- Get tag names and number of data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    Nstr='__NAME'
    Ustr='__UNIT'
    ; --- Create output structure  ---
    for iTagsDB0=0,nTagsDB0-1 do begin
      uio_tabr, val, ident, uDB0, incolumn=iTagsDB0+1, name=valName, unit=valUnit
      if (iTagsDB0 eq 0) then begin
        uDB2=create_struct(      ident, val, ident+Nstr, valName, ident+Ustr, valUnit)
      endif else begin
        uDB2=create_struct(uDB2, ident, val, ident+Nstr, valName, ident+Ustr, valUnit)
      endelse
    endfor ; iTagsDB0
    ;help, /str, uDB2
    ;
  endif else if (uDB_Type0 eq 'ListOfPointers') then begin ; .....................................
    message, 'Combination of types not implemented yet'
    ; Todo
  endif else begin ; .............................................................................
    message, 'Unknown uDB Type.'
  endelse ; ......................................................................................
  ;
endif else if (uDB_Type2 eq 'UIOTable') then begin ; -----------------------------------------------
  if (uDB_Type0 eq 'UIOTable') then begin ; ......................................................
    uDB2=uDB0
    ;
  endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; .................................
    ; --- Get tag names and number of data base ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    ;help, nTagsDB0, nELDB0
    Nstr='__NAME'
    Ustr='__UNIT'
    ;
    ; --- Create output table ---
    uio_tab0, uDB2
    uio_tabw, '', 'udb', uDB2, format='1X', name='uDB in UIO format', date='now', /tableheader
    for iTagsDB0=0,nTagsDB0-1 do begin
      TagName =TagsDB0[iTagsDB0]
      TagValue=uDB0.(iTagsDB0)
      ;help, TagName, TagValue
      ;
      if ((strpos(Tagname, '__') eq -1) or (size(TagValue, /TName) ne 'STRING')) then begin
        iTagLongName=where(strcmp(TagName+Nstr, TagsDB0) eq 1)
        if (iTagLongName ne -1) then begin
          TagLongName=uDB0.(iTagLongName)[0]
        endif else begin
          TagLongName=''
        endelse
        iTagUnit=where(strcmp(TagName+Ustr, TagsDB0) eq 1)
        if (iTagUnit ne -1) then begin
          TagUnit=uDB0.(iTagUnit)[0]
        endif else begin
          TagUnit=''
        endelse
        uio_tabw, TagValue, strlowcase(TagName), uDB2, name=TagLongName, unit=TagUnit
      endif
    endfor ; iTagsDB0
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; .................................
    uDB2=udb_Convert('UIOTable', udb_Convert('StructureOfVectors', uDB0))
    ;
  endif else if (uDB_Type0 eq 'ListOfPointers') then begin ; .....................................
    message, 'Combination of types not implemented yet'
    ; Todo
  endif else begin ; .............................................................................
    message, 'Unknown uDB Type.'
  endelse ; ......................................................................................
    ;
endif else begin ; ---------------------------------------------------------------------------------
  ; --- Other uDB types ... ---
  message, 'Combination of types not implemented yet'
  ;
endelse ; ------------------------------------------------------------------------------------------

return, uDB2

end ; udb_Convert
