function udb_Copy, uDB0, mask=mask0
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   udb_Copy ('universal_data_base_Copy')
;
; PURPOSE:
;   Copy data base, optionally selecting only masked entries
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_Copy(uDB0, mask=mask)
;
; INPUT:
;   uDB0:   ('universal_Data_Base_0')
;
; OUTPUT:
;   result:   Merged universal data base
;
; KEYWORDS:
;   mask:   ('mask') Optional input vector with 0/1 values indicating which elements to copy over
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_GetTags:
;   udb_GetType:
;   udb_ListAll:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Not all combinations implemented, yet.
;   Copy:    from LoP VoS SoV UIO
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-12-10 (B.F. Lyon)
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

if (n_params() lt 1) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Determine types of databases ---
uDB_Type0=udb_GetType(uDB0, nElDB=nElDB0)

if (nElDB0 eq 0) then begin
  ; --- No elements in data base, just copy whatever it is ---
  uDB2=uDB0
  ;
endif else begin
  ;
  ; --- Set default mask if non is specified ---
  mask=replicate(1, nElDB0)
  if (keyword_set(mask0)) then begin
    mask[0:n_elements(mask0)-1]=mask0
  endif
  ;
  ; --- Get number of elements in output data base ---
  IndMask=where(mask ne 0, nElDB2)
  ;
  if (nElDB2 eq 0) then begin
    ; --- No elements selected to copy ---
    uDB2=uDB_Type0
    ;
  endif else if (uDB_Type0 eq 'ListOfPointers') then begin ;----------------------------------------
    ; --- Create empty output pointer array ---
    uDB2=ptrarr(nElDB2, /allocate_heap)
    ;
    ; --- Copy database ---
    iElDB2=0
    for iElDB0=0,nElDB0-1 do begin
      if ((ptr_valid(uDB0[iElDB0])) and (mask[iElDB0])) then begin
        *uDB2[iElDB2]=*uDB0[iElDB0]
        iElDB2=iElDB2+1
      endif
    endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'VectorOfStructures') then begin ; -----------------------------------
    uDB2=uDB0[IndMask]
    ; --- Copy database ---
    ;iElDB2=0
    ;for iElDB0=0,nELDB0-1 do begin
    ;  if (mask[iElDB0]) then begin
    ;    if (iElDB2 eq 0) then begin
    ;      uDB2=uDB0[iElDB0]
    ;    endif else begin
    ;      uDB2=create_struct(uDB2, uDB0[iElDB0])
    ;    endelse
    ;    iElDB2=iElDB2+1
    ;  endif
    ;endfor ; iElDB0
    ;
  endif else if (uDB_Type0 eq 'StructureOfVectors') then begin ; -----------------------------------
    ; --- Create tag lists ---
    TagsDB0=udb_GetTags(uDB0, nTags=nTagsDB0)
    TagsDB2=TagsDB0
    nTagsDB2=nTagsDB0
    ;
    ; --- Create template structure with tags from both input structures ---
    for iTagsDB0=0,nTagsDB0-1 do begin
      ; --- Get the value of the next tag ---
      TagValue=uDB0.(iTagsDB0)
      nValue=n_elements(TagValue)
      ;
      if (nValue eq 1) then begin
        ; --- It is only a scalar: copy it over regardless of mask (i.e. keep it) ---
      endif else begin
        ; --- Copy the selected elements ---
        TagValue=TagValue[IndMask]
      endelse
      ;
      ; --- Append new tag with value ---
      if (iTagsDB0 eq 0) then begin
        uDB2=create_struct(      TagsDB0[iTagsDB0], TagValue)
      endif else begin
        uDB2=create_struct(uDB2, TagsDB0[iTagsDB0], TagValue)
      endelse
    endfor ; iTagsDB0
  endif else if (uDB_Type0 eq 'UIOTable') then begin ; ---------------------------------------------
    uDB2=uDB0
    uio_tabc, uDB2, mode='rearrange', index=IndMask
    ;
  endif else begin ; -------------------------------------------------------------------------------
    ; --- Other uDB types ... ---
    message, 'Unknown uDB type'
    ;
  endelse ; ----------------------------------------------------------------------------------------
endelse

return, uDB2

end ; udb_Copy
