pro udb_Delete, uDB
;
;---------------------------------------------------------------------------------------------------
;+
; NAME:
;   udb_Delete ('universal_data_base_Delete')
;
; PURPOSE:
;   Get all tags (of first entry) of database.
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   udb_Delete, uDB
;
; INPUT:
;   uDB:    ('universal_Data_Base')
;
; OUTPUT:
;   result: Vector of strings with with structure tags
;
; KEYWORDS:
;   None   
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_GetType:
;
; SIDE EFFECTS:
;   If the input array is a vector of pointers, the pointer targets are deleted.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Check the type.
;   If it's a pointer list, delete the individual pointer targets.
;   Delete the entire structure by setting it to zero.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-12-03 (B.F. Lyon)
;-
;---------------------------------------------------------------------------------------------------

compile_opt defint32, strictarr

if (n_params() eq 0) then begin
  ; --- Error ---
  message, 'No database specified.'
endif

; --- Determine type of database ---
uDB_Type=udb_GetType(uDB, nElDB=nELDB)

if (uDB_Type eq 'ListOfPointers') then begin ;------------------------------------------------------
  for iElDB=0,nELDB-1 do begin
    if (ptr_valid(uDB[iElDB])) then ptr_free, uDB[iElDB]
  endfor ; iElDB
endif

; --- Set to zero to (essentially) free the memory ---
uDB=0

end ; udb_Delete
