function udb_GetTags, uDB, nElDB=nElDB, nTags=nTags
;
;------------------------------------------------------------------------------
;+
; NAME:
;   udb_GetTags ('universal_data_base_Get_Tags')
;
; PURPOSE:
;   Get all tags (of first entry) of database.
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_GetTags(uDB)
;
; INPUT:
;   uDB:    ('universal_Data_Base')
;
; OUTPUT:
;   result: Vector of strings with with structure tags
;
; KEYWORDS:
;   nElDB:  ('number_Elements_Data_Base') integer, output
;   nTags:  ('number_Tags') integer, output
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   udb_GetType:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Should number of tags of UIO table account for meta-information tags?
;   ToDo: Assume, each tag has NAME and UNIT meta-information tag
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-08 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

if (n_params() eq 0) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Get all tag names ---

; --- Determine type of data base ---
uDB_Type=udb_GetType(uDB, nElDB=nELDB)
;
if (nElDB eq 0) then begin
  ; --- Database is empty: no tags ---
  nTags=0
  uDB_Tags=['']
endif else begin
  if            (uDB_type eq 'VectorOfStructures') then begin
    uDB_Tags=tag_names(uDB[0])
  endif else if (uDB_type eq 'StructureOfVectors') then begin
    uDB_Tags=tag_names(uDB[0])
  endif else if (uDB_type eq 'ListOfPointers'    ) then begin
    uDB_Tags=tag_names((*uDB[0]))
  endif else if (uDB_type eq 'UIOTable'          ) then begin
    uDB_Tags=strupcase(reform(uDB[0].termt[0,1,1:nElDB]))
  endif else begin
    message, 'Unknown type of data base.'
  endelse
  nTags=n_elements(uDB_Tags)
endelse

return, uDB_Tags

end ; udb_GetTags
