function udb_GetType, uDB, nElDB=nElDB
;
;------------------------------------------------------------------------------
;+
; NAME:
;   udb_GetType ('universal_data_base_Get_Type')
;
; PURPOSE:
;   Determine type of data base.
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_GetType(uDB, nElDB=nElDB)
;
; INPUT:
;   uDB:    ('universal_Data_Base')
;
; OUTPUT:
;   result: String with
;             'VectorOfStructures':
;             'StructureOfVectors':
;             'ListOfPointers'    :
;             'UIOTable'          :
;
; KEYWORDS:
;   nElDB:  ('number_Elements_Data_Base') output integer with number of elements
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-06 (B.F. Lyon)
;   2009-10-25 (B.F.) New types 'StructureOfVectors', 'UIOTable'
;   2009-12-08 (B.F.) Scalar strings can represent empty databases
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

uDB_Type=''
if (n_params() eq 0) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Determine type of data base and number of elements: ---
; --- (1D array of structures, 1D array of pointers to structures, uio table) ---
uDB_size=size(uDB, /type)
if (uDB_size eq 7) then begin
  ; --- It's a string ---
  nEl=size(uDB, /n_elements)
  if (nEl eq 1) then begin
    nElDB=0
    ; --- Only one element ---
    if (           (strmatch(uDB, 'VectorOfStructures', /fold_case)) or $
                   (strmatch(uDB, 'VOS'               , /fold_case))) then begin
      uDB_Type='VectorOfStructures'
    endif else if ((strmatch(uDB, 'StructureOfVectors', /fold_case)) or $
                   (strmatch(uDB, 'SOV'               , /fold_case))) then begin
      uDB_Type='StructureOfVectors'
    endif else if ((strmatch(uDB, 'ListOfPointers'    , /fold_case)) or $
                   (strmatch(uDB, 'LOP'               , /fold_case))) then begin
      uDB_Type='ListOfPointers'
    endif else if ((strmatch(uDB, 'UIOTable'          , /fold_case)) or $
                   (strmatch(uDB, 'UIO'               , /fold_case))) then begin
      uDB_Type='UIOTable'
    endif else begin
      message, 'Unknown database type', /info
      uDB_Type=''
    endelse
  endif else begin
    message, 'Unknown database type', /info
    uDB_Type=''
  endelse
  ;
endif else if (uDB_size eq 8) then begin
  ; --- It's not a list of pointer list ---
  nEl=size(uDB, /n_elements)
  if (nEl eq 1) then begin
    ; --- Only one element: could be structure of vectors or UIO table ---
    uDB_Tags=tag_names(uDB[0])
    iMasterKey=(where(uDB_Tags eq uDB_MasterKey, nCount))[0]
    if (nCount gt 0) then begin
      ; --- There is a master-key vector present: check its length ---
      uDB_Type='StructureOfVectors'
      nElDB   =n_elements(uDB.(iMasterKey))
    endif else begin
      ; --- Could be UIO table: check for some tags ---
      iType=(where(uDB_Tags eq 'TYPE', nCount))[0]
      if (nCount gt 0) then begin
        isUIOTable_flag=strmatch(uDB[0].(iType), 'table', /fold_case)
      endif else begin
        isUIOTable_flag=0
      endelse
      iDummy=(where(uDB_Tags eq 'TERMT', nCount))[0]
      isUIOTable_flag=isUIOTable_flag and (nCount gt 0)
      iDummy=(where(uDB_Tags eq 'NLIN', nCount))[0]
      isUIOTable_flag=isUIOTable_flag and (nCount gt 0)
      if (isUIOTable_flag) then begin
        ; --- Assume it's an UIO table ---
        uDB_Type='UIOTable'
        nElDB   =uDB.nLin
      endif else begin
        ; --- Assume there is just one entry in normal structure of vectors ---
        uDB_Type='StructureOfVectors'
        nElDB   =nEL
      endelse
    endelse
  endif else begin
    uDB_Type='VectorOfStructures'
    nElDB=nEl
  endelse
  ;
endif else if (uDB_size eq 10) then begin
  uDB_Type='ListOfPointers'
  nElDB   =size(uDB, /n_elements)
  ;
endif else begin
  message, 'Unknown type of data base.'
  ;
endelse

return, uDB_Type

end ; udb_GetType
