function udb_IsHomogen, uDB
;
;------------------------------------------------------------------------------
;+
; NAME:
;   udb_IsHomogen ('universal_data_base_Is_Homogen')
;
; PURPOSE:
;   Test of database is homogen i.e. all entries have the same tags
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;   result=udb_IsHomogen(uDB)
;
; INPUT:
;   uDB:    ('universal_Data_Base')
;
; OUTPUT:
;   result: Logical
;
; KEYWORDS:
;   None
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-10-06 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_HomogenFlag=0
if (n_params() eq 0) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Check for homogeneity ---

; --- Determine type of data base ---
uDB_Type=udb_GetType(uDB, nElDB=nElDB)
;
; --- Different handling for different types ---
if (uDB_Type eq 'ListOfPointers') then begin
  ; --- A list of pointers might point to different structures: check ---
  ;
  uDB_HomogenFlag=1
  tags0=sort(tag_names((*uDB[0])))
  ; --- Loop over all elements of database ---
  for iElDB=1,nElDB-1 do begin
    if (not array_equal(tags0, sort(tag_names((*uDB[iElDB]))))) then begin
      uDB_HomogenFlag=0
      break
    endif      
  endfor ; iElDB
endif else begin
  ; --- Database is made of vectors: it is homogenous ---
  ;
  uDB_HomogenFlag=1
endelse

return, uDB_HomogenFlag

end ; udb_IsHomogen
