function udb_ListAll, uDB0, uDB1, uDB2, FoundVector=FoundVector
;
;------------------------------------------------------------------------------
;+
; NAME:
;   udb_ListAll ('universal_data_base_List_All')
;
; PURPOSE:
;   Get identifier for all entries in data base(s).
;
; CATEGORY:
;   udb, universal database handling
;
; CALLING SEQUENCE:
;
; INPUT:
;   uDB0:        ('universal_Data_Base_0') data base 0
;   uDB1:        ('universal_Data_Base_1') data base 1
;   uDB2:        ('universal_Data_Base_2') data base 2
;
; OUTPUT:
;   result:      String array with identifier (model) list
;
; KEYWORDS:
;   FoundVector: ('Found_Vector') Integer vector indicating in which database the Id was found
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   2009-12-02 (B.F. Lyon)
;-
;------------------------------------------------------------------------------

compile_opt defint32, strictarr

uDB_MasterKey='MODEL'

if (n_params() lt 1) then begin
  ; --- Error ---
  message, 'Not enough parameters specified.'
endif

; --- Number of databases to inquire ---
nDB=n_params()
; --- Number of entry identifiers ---
dummy=udb_GetType(uDB0, nElDB=nId)
if (nDB ge 2) then begin
  dummy=udb_GetType(uDB1, nElDB=nElDB)
  nId=nId+nElDB
endif
if (nDB ge 3) then begin
  dummy=udb_GetType(uDB2, nElDB=nElDB)
  nId=nId+nElDB
endif

; --- Define output arrays ---
ResultVector=strarr(nId)
FoundVector =lonarr(nId)
iId=0

; --- Loop over all data bases ---
for iDB=0,nDB-1 do begin
  ; --- Select current data base (out of 3 so far) ---
  if (iDB eq 0) then begin
    uDB=uDB0
  endif else if (iDB eq 1) then begin
    uDB=uDB1
  endif else if (iDB eq 2) then begin
    uDB=uDB2
  endif
  ;
  ; --- Determine type and tags and number of elements of data base ---
  uDB_Type=udb_GetType(uDB, nElDB=nElDB)
  ;help, iDB, uDB_Type, nElDB
  ;
  if ((uDB_Type eq 'VectorOfStructures') or $
      (uDB_Type eq 'StructureOfVectors') or $
      (uDB_Type eq 'UIOTable'          )   ) then begin
    ; === uDB is 1D array of (identical) structures or structure of vectors or UIO table =========
    ;
    uDB_Tags=udb_GetTags(uDB)
    uDB_MaskMasterKey=strcmp(uDB_Tags, uDB_MasterKey, /fold_case)
    ;help, total(uDB_MaskMasterKey)
    if (total(uDB_MaskMasterKey) ge 1.0) then begin
      ; --- Tags exist in structure: search match ---
      ;
      ; --- Number of entry of master key (there can only be one) ---
      iMasterKey=(where(uDB_MaskMasterKey))[0]
      ;help, iMasterKey
      ; --- Array of values corresponding to master key (e.g. the list of the model names) ---
      if (uDB_Type eq 'UIOTable') then begin
        uio_tabr, AllMasterValue, ''+uDB_MasterKey, uDB
      endif else begin
        AllMasterValue=uDB.(iMasterKey)
      endelse
      ;
      ResultVector[iId:iId+nElDB-1]=AllMasterValue
      FoundVector[ iId:iId+nElDB-1]=iDB+1
      ;
    endif
    iId=iId+nELdb
    ;
  endif else if (uDB_Type eq 'ListOfPointers') then begin
    ; === uDB is 1D array of pointers to structures ========================================
    ;
    ; --- Loop over all elements of list ---
    for iElDB=0,nElDB-1 do begin
      ;
      ;help,iELDB,nELDB
      if (ptr_valid(uDB[iElDB])) then begin
        ; --- Pointer is valid: go on ---
        uDB_Tags=tag_names((*uDB[iElDB]))
        uDB_MaskMasterKey=strcmp(uDB_Tags, uDB_MasterKey, /fold_case)
        if (total(uDB_MaskMasterKey) ge 1.0) then begin
          ; --- Tags exist in structure: search match ---
          ;
          ; --- Number of entry of master key (there can only be one) ---
          iMasterKey=(where(uDB_MaskMasterKey))[0]
          ; --- Value corresponding to master key (e.g. the model name) ---
          MasterValue=(*uDB[iElDB]).(iMasterKey)
          ;help, MasterValue
          ;
          ResultVector[iId]=MasterValue
          FoundVector[ iId]=iDB+1
        endif
      endif
      iId=iId+1
    endfor; iElDB
  endif
  ;
endfor ; iDB

;help, iId, nId

;; --- Check number of elements in output vector ---
;if (iId eq 0) then begin
;  ResultVector=[""]
;  FoundVector =[0]
;  message, /info, 'No valid Ids found.'
;endif else if (iId lt nId) then begin
;  ResultVector=ResultVector[0:iId-1]
;  FoundVector =FoundVector[ 0:iId-1]
;  message, /info, 'Not all data base entries contains valid entries.'
;endif

return, ResultVector

end ; udb_ListAll
