pro uio_adkey1, termt,ntt,keywrd, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_adkey1 ('uio_add_keyword_1')
;
; PURPOSE:
;   Add one keyword to term table, keyword-value=' ', no link
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_adkey1, termt,ntt,keywrd, outstr,ierr
;
; INPUT:
;   keywrd: ('keyword') character, single keyword to be added, keyword-value=''
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiosizinc
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Add scalar keyword (without value) to term table.
;
; EXAMPLE:
;                        (1)                           (2)
;   termt:  |real                         |                            |
;                        (1)
;   keywrd: |temp                                                      |
;    --->
;                        (1)                           (2)
;   termt:  |real                         |                            |
;           |temp                         |                            |
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
if (ntt eq 0) then begin
  ; --- Create new empty term table ---
  termt=strarr(2,nttmx)
endif
;
if (ntt lt nttmx) then begin
  ; --- There is still space left in table ---
  termt(0,ntt)=keywrd
  termt(1,ntt)=''
  ntt=ntt+1
  outstr='done'
  ierr=0
endif else begin
  ; --- Table is already full ---
  outstr='table is full'
  ierr=1
endelse
;
end ; uio_adkey1
