pro uio_adkey2, termt,ntt,keywrd,keyval, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_adkey2 ('uio_add_keyword_2')
;
; PURPOSE:
;   Add one keyword to term table, keyword-value=keyval
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_adkey2, termt,ntt,keywrd,keyval, outstr,ierr
;
; INPUT:
;   keywrd: ('keyword') character, single keyword to be added, keyword-value=''
;   keyval: 'keyword_value', character, value associated with keyword
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiosizinc
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Add keyword with value to term table.
;   If keyval is empty, no keyword is added.
;
; EXAMPLE:
;                        (1)                           (2)
;   termt:  |real                         |                            |
;           |temp                         |                            |
;                        (1)
;   keywrd: |name                                                      |
;   keyval: |temperature                                               |
;     --->
;                        (1)                           (2)
;   termt:  |real                         |                            |
;           |temp                         |                            |
;           |name                         |temperature                 |
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
if (ntt eq 0) then begin
  ; --- Create new empty term table ---
  termt=strarr(2,nttmx)
endif
;
if (ntt lt nttmx) then begin
  ; --- There is still space left in table ---
  if (strcompress(keyval,/rem) ne '') then begin
    termt(0,ntt)=keywrd
    termt(1,ntt)=keyval
    ntt=ntt+1
    outstr='done'
  endif else begin
    outstr='keyword value is empty'
  endelse
  ierr=0
endif else begin
  ; --- Table is already full ---
  outstr='table is full'
  ierr=1
endelse
;
end ; uio_adkey2
