pro uio_adkey3, termt,ntt,keywrd,keyval,keydef, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_adkey3 ('uio_add_keyword_3')
;
; PURPOSE:
;   Add one keyword to term table, keyword-value=keyval or default:keydef
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_adkey3, termt,ntt,keywrd,keyval,keydef, outstr,ierr
;
; INPUT:
;   keywrd: ('keyword') character, single keyword to be added, keyword-value=''
;   keyval: ('keyword_value'), character, value associated with keyword
;   keydef: ('keyword_default'), character, default value if keyval is empty
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiosizinc
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Add keyword with value to term table.
;   If keyval is empty, keydef is taken.
;
; EXAMPLE:
;                        (1)                           (2)
;   termt:  |real                         |                            |
;           |temp                         |
;                        (1)
;   keywrd: |name                                                      |
;   keyval: |temperature                                               |
;     --->
;                        (1)                           (2)
;   termt:  |real                         |                            |
;           |temp                         |                            |
;           |name                         |temperature                 |
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
if (ntt eq 0) then begin
  ; --- Create new empty term table ---
  termt=strarr(2,nttmx)
endif
;
if (ntt lt nttmx) then begin
  ; --- There is still space left in table ---
  termt(0,ntt)=keywrd
  if (not keyword_set(keyval)) then begin
    ; --- Take default ---
    termt(1,ntt)=keydef
    outstr='keyword value is empty'
  endif else if (strcompress(keyval,/rem) ne '') then begin
    ; --- Take keyword value ---
    termt(1,ntt)=keyval
    outstr='done'
  endif else begin
    ; --- Take default ---
    termt(1,ntt)=keydef
    outstr='keyword value is empty'
  endelse
  ntt=ntt+1
  ierr=0
endif else begin
  ; --- Table is already full ---
  outstr='table is full'
  ierr=1
endelse
;
end ; uio_adkey3
