pro uio_chpos, nc,mode,pos, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_chpos ('uio_channel_position')
;
; PURPOSE:
;   Give current file position or jump to specified position.
;
; CATEGORY:
;   Input/output
;
; CALLING SEQUENCE:
;   uio_chpos, nc,mode,pos, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, number of channel
;   mode:   ('mode') character: 'get', 'goto'
;
; INPUT/OUTPUT:
;   pos:    ('position') integer, file counter position
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   Change file pointer position.
;
; RESTRICTIONS:
;   Choose the file position to jump to carefully. Not every possible number
;   is an reasonable file position. It is always save to jump to a position
;   previously derived with 'get'.
;   
; PROCEDURE:
;   Use point_lun.
;   mode='get':  Give out file counter position in ncpos(nc) in common block
;                chainf
;   mode='goto': Compare current (ncpos(nc)) and desired (pos) file counter
;                position. Then go forward (read statements) or backward
;                (backspace statement).
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   26.10.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
;@uio_uiocstinc
;
if (mode eq 'goto') then begin
  ; --- Go to specified file position ---
  point_lun, nc, pos
endif else begin
  ; --- Give current file position ---
  point_lun, -nc, pos
endelse
;
outstr='done'
ierr=0
;
end ; uio_chpos
