pro uio_closrd, nc, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_closrd ('uio_close_read')
;
; PURPOSE:
;   Close a file after reading.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_closrd, nc, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   chunit: Initialize, store and actualize a list of unit numbers
;   uclose: Close file with special handling for conversion type
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Identical to uio_closwr.
;
; PROCEDURE:
;   Just calls uclose and chunit.
;
; EXAMPLE:
;   ; --- Open file, read first entry and print its header, close file again --
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   uio_pptrmt, termt,ntt
;   uio_closrd, nc, outstr,ierr
;
; MODIFICATION HISTORY:
;   20. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
; --- Close file ---
uio_uclose, nc, outstr,ierr
;
; --- Return unit number ---
uio_chunit, 'return', nc, 0, outstr,ierr
;
outstr='done'
ierr=0
;
end ; uio_closrd
