pro uio_closwr, nc, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_closwr ('uio_close_write')
;
; PURPOSE:
;   Close a file after writing.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_closwr, nc, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   chunit: Initialize, store and actualize a list of unit numbers
;   uclose: Close file with special handling for conversion type
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   A file on the specified channel unit is closed. The channel unit status
;   is set to 'free' and saved.
;
; RESTRICTIONS:
;   Identical to uio_closrd.
;
; PROCEDURE:
;   Just calls uclose and chunit.
;
; EXAMPLE:
;   conv=''
;   filename='uiotestfile.txt'
;   form='formatted'
;   uio_openwr, nc,filename, outstr,ierr, form=form, conv=conv
;   val0=sqrt(findgen(10))
;   uio_wr, nc, val0, 'number1', outstr, ierr
;   uio_closwr, nc, outstr,ierr
;     ---> file 'uiotestfile.txt':
;   fileform uio form=formatted version=0.0.1996.10.14 &
;   date='Mon Oct 14 17:48:43 1996' system=SunOS machine=saturn &
;   osrelease=4.1.3 &
;   osversion=3 hardware=sun4m language='IDL 4.0' program=IDL
;   real number1 d=(0:9) f=E13.6 p=4 b=4
;    0.000000E+00 1.000000E+00 1.414214E+00 1.732051E+00
;    2.000000E+00 2.236068E+00 2.449490E+00 2.645751E+00
;    2.828427E+00 3.000000E+00
;
; MODIFICATION HISTORY:
;   20. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
; --- Close file ---
uio_uclose, nc, outstr,ierr
;
; --- Return unit number ---
uio_chunit, 'return', nc, 0, outstr,ierr
;
outstr='done'
ierr=0
;
end ; uio_closwr
