pro uio_cpentr, nc,termt,ntt, ncout, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_cpentr ('uio_copy_entry')
;
; PURPOSE:
;   Copy entry (header + data) from one file to another.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_cpentr, nc,termt,ntt, ncout, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number for input
;   ncout:  ('number_channel_out') integer, channel unit number for output
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   rd:     Read scalar or array data of any type
;   rdfifo: Read file header
;   rdlabl: Read label
;   rdtab:  Read table of integer, real, and/or character data from file
;   wr:     Write scalar or array data of any type
;   wrfifo: Write file header
;   wrlabl: Write label
;   wrtab:  Write table of integer, real, and/or character data to file
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are read from file on unit nc and written into file on unit ncout.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Extract variable type from header, read data from one file (unit nc), write
;   header and data to another file (unit ncout).
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   30. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   13.10.96 (B.F.) Renaming: termt, corrections in call of uio_wrtab
;   15.10.96 (B.F.) rdfifo, rdlabl: Parameters -> keywords
;   16.10.96 (B.F.) wrfifo: Parameters -> keywords
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
; --- Find keywords and extract their values ---
type=termt(0,0)
;
; --- Read data from file: skip ---
if (type eq fifona) then begin
  ; --- Fileform ---
  uio_rdfifo, nc,   termt,ntt, outstr,ierr
  ; --- Ignore all old keywords from fileform ---
  uio_wrfifo, ncout, outstr,ierr, progrm='cpentr'
endif else if (type eq lablna) then begin
  ; --- Label ---
  uio_rdlabl, nc,termt,ntt, ident, outstr,ierr
  uio_wrlabl, ncout,ident, termt=termt,ntt=ntt, $
    outstr,ierr
endif else if (type eq tabna) then begin
  ; --- Table ---
;  format=''
;  name=''
;  date=''
  uio_rdtab, nc, termt,ntt, tab,ident, outstr,ierr
;    format=format, name=name, date=date
  uio_wrtab, ncout, tab, outstr,ierr
endif else begin
  ; --- Integer, real, double precision real, complex, or character data ---
  format=''
  name=''
  unit=''
  tran=''
  date=''
  uio_rd, nc, termt,ntt, val,ident, iarr=iarr, outstr,ierr, $
    format=format, name=name, unit=unit, tran=tran, date=date
;  help, val,ident,iarr,format,name,unit,tran,date
  uio_wr, ncout, val,ident, iarr=iarr, outstr,ierr, $
    format=format, name=name, unit=unit, tran=tran, date=date, $
    termt=termt,ntt=ntt
endelse
;
ierr=0
outstr='done'
;
end ; uio_cpentr
