; pro uio_data_getall
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_data_getall ('uio_data_get_all')
;
; PURPOSE:
;   Read all entries from uio file and put them into IDL variables with proper
;   names.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   @uio_data_getall
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_data:
;   uio_d:
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   A lot: A file in uio form (opened by uio_data) is read.
;   A list of variables is generated.
;   Be careful: The variable names in the file might overlap with your
;               previously defined variable names.
;
; RESTRICTIONS:
;   The file has to be opened (and selected) by uio_data.
;   You can append a praefix and a suffix string to every variable name.
;   These two strings have to be edited in this batch file.
;
; PROCEDURE:
;   Very simple
;
; EXAMPLE:
;   uio_data, mo='open', file='../eos/f90/gas2eos_err.dat', fam='err13'
;   @uio_data_getall
;
; MODIFICATION HISTORY:
;    2. 2.98 Written by Bernd Freytag, Astrophysik Kiel
;   16. 2.98 (B.F.) postfix -> suffix
;-
;------------------------------------------------------------------------------
;
; --- Define praefix and suffix strings if not already done ---
if (n_elements(praefix) eq 0) then praefix=''
if (n_elements(suffix ) eq 0) then suffix=''
;
; --- Determine entry list ---
uio_data, mode='content', outidentlist=identlist
;
; --- Read all entries in list into variables with their names ---
for i=1,n_elements(identlist(0,*)) do $
  dummy=execute(praefix + identlist(1,i-1) + suffix + $
                '=uio_d("' + identlist(1,i-1) + '")')
;
; end ; uio_data_getall
