pro uio_dim2st, iarr, dimstr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_dim2st ('uio_dimensions_to_string')
;
; PURPOSE:
;   Make dimension string.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_dim2st, iarr, dimstr
;
; INPUT:
;   iarr:   ('index_array') integer array(2,ndim)
;
; OUTPUT:
;   dimstr: ('dimension_string') character, string with dimensions
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   FORTRAN version can handle only 4 dimensions.
;
; PROCEDURE:
;   Straightforward: Put integer values with dimension specifications into
;   single string.
;   The inverse routine is uio_st2dim.
;
; EXAMPLE:
;   uio_dim2st, [[1,2],[3,4]], dimstr
;   print, dimstr
;     --->
;   (1:2,3:4)
;
; MODIFICATION HISTORY:
;    5. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Determine number of dimensions ---
ndim=n_elements(iarr)/2
;
dimstr='('
;
sarr=strcompress(string(iarr),/remove_all)
; --- Merge indices into single string ---
for n=0,ndim-2 do begin
  dimstr=dimstr+sarr(0,n)+':'+sarr(1,n)+','
endfor
dimstr=dimstr+sarr(0,ndim-1)+':'+sarr(1,ndim-1)+')'
;
end ; uio_dim2st
