pro uio_ex1trm, inline, outerm,ouline,outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_ex1trm ('uio_extract_1_term')
;
; PURPOSE:
;   Extract one term from the input line.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   ex1trm, inline, outerm,ouline,outstr,ierr
;
; INPUT:
;   inline: ('input_line') character*(*), line to be parsed
;
; OUTPUT:
;   outerm: ('output_term') character*(*)(2), first term: keyword(+value)
;   ouline: ('output_line') character*(*), rest of inline
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_qmadel: Parse string "inline" and remove quotation marks if necessary
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Search first term in inline, remove it from inline
;   and put it in seperated form (keyword (+value)) in outerm.
;
; EXAMPLE:
;                         (1)
;   inline: | name=density  unit=g/cm**3                                |
;    --->
;                         (1)                         (2)
;   outerm: |name                         |density                      |
;                         (1)
;   ouline: |  unit=g/cm**3                                             |
;
; MODIFICATION HISTORY:
;   15. 3.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
ierr=0
inl=strtrim(inline)
ouline=''
outerm=strarr(2)
;
if (inl eq '') then begin
  ; --- Input line is empty ---
  outstr='empty'
endif else begin
  ; --- Start extraction ---
  lenlin=strlen(inl)
  i1=0
  ; --- Skip blanks and link-characters ---
  while ((strmid(inl,i1,1) eq ' ') or (strmid(inl,i1,1) eq lnkchr)) do begin
    i1=i1+1
    if (i1 ge lenlin) then begin
      outstr='only '+lnkchr
      goto, endlabel
    endif
  endwhile
  ; --- Search separator equchr ---
  i2=i1
  while ((strmid(inl,i2,1) ne ' ') and (strmid(inl,i2,1) ne equchr)) do begin
    i2=i2+1
    if (i2 ge lenlin) then begin
      outerm(0)=strmid(inl,i1,i2-i1+1)
      outstr='just name'
      goto, endlabel
    endif
  endwhile
  ; --- End of lhs ---
  if (i1 eq i2) then begin
    ; --- No lhs ---
    ouline=strmid(inl,i2+1,lenlin)
    outstr='start with '+equchr
    ierr=1
  endif else if (strmid(inl,i2,1) eq ' ') then begin
    ; --- No link character ---
    outerm(0)=strmid(inl,i1,i2-i1)
    ouline=strmid(inl,i2,lenlin)
    outstr='just name'
    goto, endlabel
  endif else begin
    ; --- Find rhs ---
    outerm(0)=strmid(inl,i1,i2-i1)
    uio_qmadel, strmid(inl,i2+1,lenlin), oute,ouline,outstr,ierr
    outerm(1)=oute
    if (ierr eq 0) then begin
      outstr='done'
    endif else begin
      outstr='ex1trm:qmadel: '+outstr
      ierr=2
    endelse
  endelse
endelse
;
endlabel:
;
end ; uio_ex1trm
