pro uio_exkeyw, mode, termt,ntt, keywrd,keyval, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_exkeyw ('uio_extract_keyword')
;
; PURPOSE:
;   Extract value of keyword from table
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_exkeyw, mode, termt,ntt, keywrd,keyval, outstr,ierr
;
; INPUT:
;    mode:   ('mode') integer, search mode:
;           0X: start search form 3. element
;           1X: start search from 1. element
;            0: search only
;            1: search and delete (leave blank)
;            2: search, delete, and shift
;            3: search, delete, and reconstruct
;            4: reconstruct with search
;   keywrd: ('keyword') character, keyword to be searched for
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   keyval: 'keyword_value', character, value of found keyword, otherwise empty
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiosizinc: length of strings, size of tables (termt, linet)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Search for keyword keywrd in term table termt. If found, put the
;   corresponding value in keyval.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) More comments
;   15.10.96 (B.F.) Mode 4 implemented
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
ierr=0
;
; --- Set start point of search ---
if (mode ge 10) then begin
  nttstt=0
  mode0=mode-10
endif else begin
  nttstt=2
  mode0=mode
endelse
;
if (mode0 eq 4) then begin
  keyval=''
  outstr='only reconstruction'
  goto, reconstructlabel
endif
;
; --- Search in table until end or keyword found ---
for n=nttstt,ntt-1 do begin
  if (termt(0,n) eq keywrd) then begin
    ; --- Keyword found ---
    keyval=termt(1,n)
    outstr='found'
    ; --- Delete keyword from table ---
    if (mode0 ge 1) then termt(0,n)=''
    if (mode0 eq 2) then begin
      ; --- Shift following keywords ---
      for n1=n,ntt-2 do begin
        termt(*,n1)=termt(*,n1+1)
      endfor
      ntt=ntt-1
      goto, endlabel
    endif
    goto, reconstructlabel
  endif
endfor
;
; --- Keyword not found ---
keyval=''
outstr='not found'
;
; --- Reconstruct complete table ---
reconstructlabel:
if ((mode0 eq 3) or (mode0 eq 4)) then begin
  n1=0
  ntto=ntt
  for n2=0,ntto-1 do begin
    if (termt(0,n2) ne '') then begin
      termt(*,n1)=termt(*,n2)
      n1=n1+1
    endif else begin
      ntt=ntt-1
    endelse
  endfor
endif
;
endlabel:
;
end ; uio_exkeyw
