pro uio_exmtrm, linet,nlt, termt,ntt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_exmtrm ('uio_extract_multiple_term')
;
; PURPOSE:
;   Transform a list of items into its components
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   exmtrm, linet,nlt, termt,ntt, outstr,ierr
;
; INPUT:
;   linet:  ('line_table') character array, nltmx lines
;   nlt:    ('number_line_table') integer, number of non-empty lines of linet
;
; OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;
; INCLUDES:
;   uio_uiosizinc: length of strings, size of tables (termt,linet)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   This routine is rather time consuming: approx 1.5ms/keyword (saturn)
;
; PROCEDURE:
;   Parse all lines in linet, extract all terms (keyword (+value)), result in
;   termt.
;
; EXAMPLE:
;                         (1)
;   linet:  |real rho &                                                 |  (1)
;           | name=density unit='g/cm**3'                               |  (2)
;    --->
;                         (1)                          (2)
;   termt:  |real                         |                             |  (1)
;           |rho                          |                             |  (2)
;           |name                         |density                      |  (3)
;           |unit                         |g/cm**3                      |  (4)
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   13.10.96 (B.F.) Renaming: linet
;   14.10.96 (B.F.) Renaming: lineto
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
ierr=0
;
; --- Copy input table into output table ---
nltols=nlt<nltmx
lineto=linet(0:nltols-1)
;
ntt=0
termt=strarr(2,nttmx)
for nlto=0,nltols-1 do begin
  nexttrm: line=lineto(nlto)
  uio_ex1trm, line, outerm,ouline, outstr,ierr
  termt(*,ntt)=outerm
  lineto(nlto)=ouline
  if (termt(0,ntt) ne '') then begin
    if (ntt eq nttmx-1) then begin
      outstr='output list full'
      ierr=1
      goto, endlabel
    endif else begin
      ntt=ntt+1
      goto, nexttrm
    endelse
  endif
endfor
;ntt=ntt-1
outstr='done'
;
endlabel:
;
end ; uio_exmtrm
