;******************************************************************************
;       Universal Input/Output Routines (IDL)
;        2-Feb-1998 17:48:46
;******************************************************************************
; Routines, functions : uio_*.pro: (+: user-routine, -: comfortable, .:useful) 
; . adkey1: Add one keyword to term table, keyword-value=' ', no link character
; . adkey2: Add one keyword to term table with keyword-value
; . adkey3: Add one keyword to term table with keyword-value or default
; . chconv: Actualize list of conversion types for all channels
; + chpos:  Give current file position or jump to specified position
; . chunit: Initialize, store and actualize a list of free and occupied unit
;           numbers
; + closrd: Close file after reading
; + closwr: Close file after writing
; - cpentr: Copy entry from one file to another
; ! d:      Read data from uio-file(s) in quasi direct access mode
; ! data:   Handle uio-file(s) in quasi direct access mode
; ! data_getall: Read all entries in uio file into variables with their names
;   deform: Determine the default output format for numbers
;   dim2st: Compose dimension string
; . exkeyw: Extract value of keyword from table
;   ex1trm: Extract one term from the input line
;   exmtrm: Transform a list of items into its components
; . filcon: Determine file contents: list of all entries with its positions
;   getenv: Get information about environment
; + init:   Initialization procedure for input/output routines
;   me1trm: Merge the input term: 'keyword', 'value' -> 'keyword=value'
;   memtrm: Merge a list of terms (keywords and their values) into a line table
;   mkcvls: Make list with possible conversion types
; . nc2nt:  From column number or entry name find table entry number
; + openrd: Open file for reading, read header
; + openwr: Open file for writing, write header
; - pptrmt: Print term table in pretty form
;   qmaadd: Transform a string into a string with quotation marks if necessary
;   qmadel: Parse string "inline" and remove quotation marks if necessary
; + rd:     Reading scalar and array data of all types
; . rdfifo: Read file header
; + rdhdex: Read header of variable and extract keywords
;   rdhead: Read header
; + rdlabl: Read label
; + rdtab:  Read table of integer, real, and/or character data from file
; + skipda: Skip data block
; - slhdex: Search header of variables given by list and extract keywords
; . st2dim: Parse dimension string
; . tab0:   Create empty table structure
; + tabc:   Change and modify table contents: rearrange lines.
; + tabm:   Merge two tables in different ways
; + tabr:   Read 1d array from 2d table array (all types)
; + tabw:   Write 1d array into table (all types)
;   uclose: Close file with special handling for conversion type
;   uopen:  Open file with special handling for conversion type
;   vnanrm: Transform a string to give a correct name of a variable
;   wf2rf:  Produce from write format string corresponding read format string
; + wr:     Writing scalar and array data of all types
; . wrfifo: Write file header
;   wrhdme: Write header of variable, input: term table.
;   wrhead: Write header of variable, input: line table.
; + wrlabl: Write label
; + wrtab:  Write table of integer, real, and/or character data to file
;******************************************************************************
; Include-Files uio_*.pro:
;   uiocstinc: channel status information (common uio_chainf)
;   uiocvlinc: convert type list of current machine (common uio_cvlist)
;   uionaminc: names of types, keywords, identifiers; default formats
;              (common uio_defnam)
;   uiosizinc: length of strings, size of tables (trmtab, lintab)
;   uiotabinc: empty table structure (common uio_taborg)
;******************************************************************************
;******************************************************************************
; IDL:
; Things to do (now):
;   Keywords as in Fortran90
;   nperlin - perlin as keyword, default: reasonable guess
; Things to do (future):
;   fileform keyword: filename
;   rd: mode: read into part of larger array
;   tabm: interpolate
;   error handling (message...) ?
;   wr...: headonly/dataonly
;   open-append
;   double keyword names in termtable (adkeyX) ?
;   Use HOST_TO_IEEE and IEEE_TO_HOST or BYTEORDER
;   table: wrtab: first entry no table header entry
;******************************************************************************
; Obsolete (used only in FORTRAN77, not in IDL):
;   ppkeyw: Print keywords with values in pretty form
;   srhdex: Search header of variable and extract keywords
;******************************************************************************
; FORTRAN77:
; Things to do (now):
;   iarr: array with dimensions
;   real: 2D, 3D, 4D
; Things to do (future):
;   closrd=closwr -> ?
;   table: wrtab: first entry no table header entry
;******************************************************************************
; Fortran90:
; Things to do (now):
;   real,...: 2D, 3D, 4D
; Things to do (future):
;   Table routines more Fortran90-like: structure tab, keywords, ...
;   Universal data type
;   error handling (improvement)
;******************************************************************************
; UNIX:
;   uiolook: all types, keyword: number of keywords,
;            keyword: number of entries (file header only)
;   uiocat: start entry, end entry, 
;           compare entry, compare value, compare epsilon, period,
;           cat with fileform entry, (UNIX head, tail)
;           table entry list
;   uiocount: record count, entry count
;   uiosplit: split uio-file into list of files at selected entry
;   uiodiff: compare two files
;   uio2tex: transform uio-table into LaTeX-table
;   uiognu: start gnuplot with data from uio-table (or other entry)
;   uioprint: real r=min...max unit (name)
;******************************************************************************
