pro uio_me1trm, interm, ouline, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_me1trm ('uio_'merge_1_term')
;
; PURPOSE:
;   Merge the input term: 'keyword', 'value' -> 'keyword=value'
;
; CATEGORY:
;   String/array processing
;
; CALLING SEQUENCE:
;   uio_me1trm, interm, ouline, outstr,ierr
;
; INPUT:
;   interm: ('input_term') character array(2) containing keyword (and value)
;
; OUTPUT:
;   ouline: ('output_line') character, merged keyword, '=', and value
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_qmaadd: Transform a string into one with quotation marks if necessary
;   uio_vnanrm: Transform a string to give a correct name of a variable
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Trailing blanks in keyword value are ignored (FORTRAN standard).
;
; PROCEDURE:
;   The first element of interm gives the keyword name, the second element its
;   value. They are merged into a single string ouline.
;   Special handling for blanks and quotations marks.
;
; EXAMPLES:
;   interm(1)    interm(2)   ->    ouline
;   name         density           name=density
;   name                           name
;                density
;   greetings    hello hello       greetings='hello hello'
;   state        I'm tired         state='I''m tired'
;
; MODIFICATION HISTORY:
;   10. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
@uio_uiosizinc
;
ierr=0
;
; --- Remove leading and trailing blanks ---
inte=strtrim(interm,2)
;
if (inte(0) eq '') then begin
  ; --- Everything empty ---
  ouline=''
  outstr='empty'
endif else begin
  ; --- Make correct name of variable ---
  uio_vnanrm, inte(0), ouline,remain, outstr,ierr
  if (inte(1) eq '') then begin
    outstr='just name'
  endif else begin
    idxoul=strlen(ouline)
    if (idxoul ge lel-4) then begin
      outstr='2. term too long'
      ierr=1
    endif else begin
      ouline=ouline+equchr
      idxoul=idxoul+1
      ; --- Add quotation marks to keyword value if necessary ---
      uio_qmaadd, inte(1), value,remain, lenoul=lel-2-idxoul, outstr,ierr
      ouline=ouline+value
      if (remain eq '') then begin
        outstr='done'
      endif else begin
        outstr='2. term too long remain:'+remain
        ierr=1
      endelse
    endelse
  endelse
endelse
;
end ; uio_me1trm
