pro uio_nc2nt, tabtype, tabncol, ncol, ntai, ntar, ntas, outstr,ierr, $
    type=type, ident=ident
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_nc2nt ('uio_number_column_to_number_table')
;
; PURPOSE:
;   From column number or entry name find table entry number.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_nc2nt, tabtype, tabncol, ncol, ntai, ntar, ntas, outstr,ierr
;
; INPUT:
;   tabtype:('table_type') character array(2,tabncol), types and identifiers
;           of table entries (e.g. reform(tab.termt(0,0:1,0:tab.ncol-1)))
;   tabncol:('table_number_column') integer, total column number
;
; INPUT/OUTPUT:
;   ncol:   ('number_column') integer, number of column in complete table
;
; OUTPUT:
;   ntai:   ('number_table_integer') integer, entry number in integer table
;   ntar:   ('number_table_real') integer, entry number in real table
;   ntas:   ('number_table_character') integer, entry number in character table
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   ident:  ('identifier') character, identifier, input
;   type:   ('type') character, type of variable, input
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   If at least one of the keywords ident and type is set, the table entry is
;   searched for by identifier and/or type, respectively. Otherwise it is
;   searched for by column number ncol.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    5.10.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) More comments
;   16.10.96 (B.F.) Corrections
;   25.10.09 (B.F. Lyon) Ignore case in comparison of identifier
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
;
; --- Find table column ---
ntai=0
ntar=0
ntas=0
ncolsl=0
endsearch_flag=(tabncol eq 0)
found_flag=0
if (keyword_set(ident) or keyword_set(type)) then begin
  ; --- Search column by name ---
  endsearch_flag=(tabncol eq 0)
  while (not endsearch_flag) do begin
    type_tmp =tabtype[0,ncolsl]
    ident_tmp=tabtype[1,ncolsl]
    if (keyword_set(type)) then begin
      if (type_tmp eq type) then begin
        if (keyword_set(ident)) then begin
;          if (ident_tmp eq ident) then begin
          if (strcmp(ident_tmp, ident, /fold_case)) then begin
            found_flag=1
            endsearch_flag=1
          endif
        endif else begin
          found_flag=1
          endsearch_flag=1
        endelse
      endif
    endif else begin
;      if (ident_tmp eq ident) then begin
      if (strcmp(ident_tmp, ident, /fold_case)) then begin
        found_flag=1
        endsearch_flag=1
      endif
    endelse   
    ;
    if (not endsearch_flag) then begin
      if (type_tmp eq intena) then begin
        ntai=ntai+1
      endif else if (type_tmp eq realna) then begin
        ntar=ntar+1
      endif else if (type_tmp eq charna) then begin
        ntas=ntas+1
      endif
      ncolsl=ncolsl+1
      endsearch_flag=(ncolsl ge tabncol)
    endif  
  endwhile
endif else begin
  ; --- Search column by number ---
  newcol=0>ncol<(tabncol-1)
  while (not endsearch_flag) do begin
    endsearch_flag=(newcol eq ncolsl)
    found_flag=endsearch_flag
    type_tmp=tabtype(0,ncolsl)
    if (not endsearch_flag) then begin
      if (type_tmp eq intena) then begin
        ntai=ntai+1
      endif else if (type_tmp eq realna) then begin
        ntar=ntar+1
      endif else if (type_tmp eq charna) then begin
        ntas=ntas+1
      endif
      ncolsl=ncolsl+1
      endsearch_flag=(ncolsl ge tabncol)
    endif else begin
;      found_flag=0
    endelse
  endwhile
endelse
ncol=ncolsl
;
ierr=0
if (found_flag) then begin
  outstr='found'
endif else begin
  outstr='not found'
endelse
;
end ; uio_nt2ct
