pro uio_pptrmt, termt,ntt, tabletype=tabletype
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_pptrmt ('uio_pretty_print_term_table')
;
; PURPOSE:
;   Print term table in pretty form.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_pptrmt, termt,ntt, tabletype=tabletype
;
; INPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   tabletype: ('table_type') integer, determines interpretation of first two
;              keywords
;              0: first two keywords give variable type and identifier (default)
;              1: all entries are keywords
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,linet)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data are printed on screen.
;
; RESTRICTIONS:
;   Only the first 12 letters of the keyword and the first 64 letters of the
;   corresponding value are printed.
;
; PROCEDURE:
;   Straightforward: Just print strings in term table in (pretty) readable form.
;   Simulate call of ppkeyw by 
;     uio_pptrmt, tabletype, [[k1,v1],[k2,v2],[k3,v3]], 3
;
; EXAMPLE:
;   termt=[['real',''],['r',''],['f','F5.1'],['n','radius']]
;   ntt=n_elements(termt)/2
;   uio_pptrmt, termt,ntt
;
; MODIFICATION HISTORY:
;   23. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) Output format changed
;   15.10.96 (B.F.) tabletype becomes keyword, more comments
;    1. 1.97 (B.F.) 'X' -> '1X'
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
;fo='(1X,A12,A,1X,A64)'
fo='(2X,A12,A,1X,A64)'
;sh=' '
sh= string(' ',format='(A12)')
pad=string(' ',format='(A64)')
;
; --- Print variable type and identifier ---
n1=0
if ((not keyword_set(tabletype)) and (ntt ge 2)) then begin
  print, termt(0,0) + ' ' + termt(0,1)
  n1=2
endif
;
; --- Print term table ---
for n=n1,ntt-1 do begin
  if (termt(0,n) ne '') then begin
    if (termt(1,n) ne '') then begin
      print, termt(0,n) + sh, equchr, termt(1,n)+pad, format=fo
    endif else begin
      print, termt(0,n) + sh, format=fo
    endelse
  endif
endfor
;
end ; uio_pptrmt
