pro uio_qmaadd, inline, ouline,remain, outstr,ierr, lenouline=lenouline
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_qmaadd ('uio_quotation_mark_add')
;
; PURPOSE:
;   Transform a string 'inline' 
;   into a string 'ouline' with quotation marks if necessary.
;
; CATEGORY:
;   String processing
;
; CALLING SEQUENCE:
;   uio_qmaadd, inline, ouline,remain, outstr,ierr, lenouline=lenouline
;
; INPUT:
;   inline: ('input_line') character, input string
;
; OUTPUT:
;   ouline: ('output_line') character, output string (with quotation marks)
;   remain: ('remainder') character, remainder of inline if inline is to long
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
; KEYWORDS:
;   lenouline: ('length_output_line') integer, maximum allowed length for
;           output line, default=let-4
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt, linet)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Leading and trailing blanks are removed. If the remaining string contains
;   any blanks, the string is included in quotation marks and
;   inner quotation marks are doubled.
;
; EXAMPLE:
;    inline:                   ouline            remain
;    tired             --->    tired
;    very tired        --->    'very tired'
;    I'm tired.        --->    'I''m tired.'
;    I'm               --->    I'm           ? 'I''m'
;
; MODIFICATION HISTORY:
;    9. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uionaminc
@uio_uiosizinc
;
ierr=0
outstr='done'
remain=''
;
; --- Trim input string ---
inl=strtrim(inline,2)
if (inl eq '') then begin
  ; --- String is empty ---
  ouline=inl
  remain=''
  outstr='empty'
endif else begin
  ; --- Non-empty string ---
  idxinl=strlen(inl)
  if (n_elements(lenouline) eq 0) then begin
    lenoul=lel-4
  endif else begin
    lenoul=lenouline
  endelse
  if ((strpos(inl,' ') le -1) and (strpos(inl,qmachr) le -1)) then begin
    ; --- String without included blanks and without quotation marks ---
    if (lenoul ge idxinl) then begin
      ouline=inl
      remain=''
      outstr='just copied'
    endif else begin
      ouline=strmid(inl,0,lenoul)
      remain=strmid(inl,lenoul,80)
      outstr='rest cut'
    endelse
  endif else if (lenoul le 2) then begin
    ouline=''
    remain=inl
    outstr='output string only one or two chars'
  endif else begin
    ; --- String needs quotation marks ---
    ; --- Transform to string with quotation marks ---
    iin=0
    iout=1
    ouline=qmachr
    while ((iin lt idxinl) and (iout lt lenoul-1)) do begin
      if (iout eq lenoul-2) then begin
        ; --- End of output string nearly reached ---
        if (strmid(inl,iin,1) eq qmachr) then begin
          ; --- Input string too long ---
	  remain=strmid(inl,iin,80)
          outstr='too long, end with '+qmachr
        endif else begin
          ouline=ouline+strmid(inl,iin,1)
          if (iin lt idxinl-1) then begin
            remain=strmid(inl,iin+1,80)
            outstr='too long'
          endif else begin
            remain=''
            outstr='done'
          endelse
        endelse
      endif else if (strmid(inl,iin,1) eq qmachr) then begin
        ; --- Quotation mark found ---
        ouline=ouline+qmachr+qmachr
        iout=iout+1
        if (iout ge lenoul-2) then begin
          if (iin gt idxinl-2) then begin
            remain=''
            outstr='done'
          endif else begin
            remain=strmid(inl,iin+1,80)
            outstr='cut'
          endelse
        endif
      endif else begin
        ; --- Just character ---
        ouline=ouline+strmid(inl,iin,1)
      endelse  
      ; --- Increment counters ---
      iin=iin+1
      iout=iout+1
    endwhile
    ouline=ouline+qmachr
  endelse
endelse
;
end ; uio_qmaadd
