pro uio_rdfifo, nc,termt,ntt, outstr,ierr, $
  format=form, conversion=conv, date=date, program=progrm, $
  system=system, machine=machin, osrelease=osrels, osversion=osvers, $
  hardware=hrdwar, language=langua
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rdfifo ('uio_read_fileform')
;
; PURPOSE:
;   Read file header.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rdfifo, nc,termt,ntt, outstr,ierr, $
;     format=form, conversion=conv, date=date, program=progrm, $
;     system=system, machine=machin, osrelease=osrels, osversion=osvers, $
;     hardware=hrdwar, language=langua
;
; INPUT:
;   nc: ('number_channel') integer, channel unit number (actually not used)
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   (Output:)
;   form:   ('format') character, 'formatted' or 'unformatted'
;   conv:   ('conversion') character, conversion type
;   date:   ('date') character, empty or date information
;   system: ('system') character, operating system
;                                                        (e.g. SunOS, craSH    )
;   machin: ('machine') character, empty or machine name
;                                                    (e.g. venus, kueste, craSH)
;   osrels: ('operating_system_release') character, operating system release
;                                                        (e.g. 4.1.3, 8.0.3.0  )
;   osvers: ('operating_system_version') character, operating system version
;                                                        (e.g. 1,     awa.5    )
;   hrdwar: ('hardware') character, empty or machine hardware
;                                                      (e.g. sun4m, 'CRAY Y-MP')
;   langua: ('language') character, language of program that wrote file
;                                                      (e.g. 'FORTRAN77', 'IDL')
;   progrm: ('program_name') character, name of main program
;
; ROUTINES:
;   uio_exkeyw: Extract value of keyword from table
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   Only keyword values for keywords which are not undefined are extracted.
;   The requested keywords may be set to ''.
;
; PROCEDURE:
;   Extract information from termt for selected keywords by repeated calls of
;   uio_exkeyw. The rest of the keywords is kept in termt.
;
; EXAMPLE:
;   ; --- Assume: file on unit nc is open, current entry is fileform entry ---
;   form=''
;   conv=''
;   date=''
;   progrm=''
;   system=''
;   machin=''
;   osrels=''
;   osvers=''
;   hrdwar=''
;   langua=''
;   uio_rdfifo, nc,termt,ntt, outstr,ierr, $
;     format=form, conversion=conv, date=date, program=progrm, $
;     system=system, machine=machin, osrelease=osrels, osversion=osvers, $
;     hardware=hrdwar, language=langua
;   help, outstr,ierr, $
;     format=form, conversion=conv, date=date, program=progrm, $
;     system=system, machine=machin, osrelease=osrels, osversion=osvers, $
;     hardware=hrdwar, language=langua
;
; MODIFICATION HISTORY:
;   18. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;    9.10.96 (B.F.) Check type
;   15.10.96 (B.F.) Parameters -> keywords
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
; --- Check type ---
if (termt(0,0) ne fifona) then begin
  ierr=1
  outstr='No fileform entry, but ' + termt(0,0)
  goto, endlabel
endif
;
ierr=0
outstr='done'
;
; --- Find keywords and extract their values ---
termt(0,0:1)=''
m=(n_elements(form) ne 0)
uio_exkeyw, m,termt,ntt,formna, form,   outstr,ierr
m=(n_elements(conv) ne 0)
uio_exkeyw, m,termt,ntt,convna, conv,   outstr,ierr
m=(n_elements(vers) ne 0)
uio_exkeyw, m,termt,ntt,versna, vers,   outstr,ierr
m=(n_elements(date) ne 0)
uio_exkeyw, m,termt,ntt,datena, date,   outstr,ierr
m=(n_elements(system) ne 0)
uio_exkeyw, m,termt,ntt,systna, system, outstr,ierr
m=(n_elements(machin) ne 0)
uio_exkeyw, m,termt,ntt,machna, machin, outstr,ierr
m=(n_elements(osrels) ne 0)
uio_exkeyw, m,termt,ntt,osrena, osrels, outstr,ierr
m=(n_elements(osvers) ne 0)
uio_exkeyw, m,termt,ntt,osvena, osvers, outstr,ierr
m=(n_elements(hrdwar) ne 0)
uio_exkeyw, m,termt,ntt,hrdwna, hrdwar, outstr,ierr
m=(n_elements(langua) ne 0)
uio_exkeyw, m,termt,ntt,langna, langua, outstr,ierr
m=(n_elements(progrm) ne 0)
uio_exkeyw, m,termt,ntt,progna, progrm, outstr,ierr
uio_exkeyw, 4,termt,ntt,''    , dummy , outstr,ierr
;
endlabel:
;
end ; uio_rdfifo
