pro uio_rdhd, nc, termt,ntt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rdhd ('uio_read_head')
;
; PURPOSE:
;    Read header of variable and extract keywords.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;   outstr: ('output_string') character, EOF -> 'eof'
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   chunit: Initialize, store and actualize a list of free and occupied unit
;           numbers
;   exmtrm: Transform a list of items into its components
;   rdhdln: Read header
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_defnam in uionaminc
;
; SIDE EFFECTS:
;   Data are read from file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Read data header from file on unit nc and put keywords into table.
;
; EXAMPLE:
;   ; --- Open file, read first entry and print its header, close file again --
;   filename='uiotestfile.txt'
;   uio_openrd, nc,filename, outstr,ierr
;   uio_rdhd, nc, termt,ntt, outstr,ierr
;   uio_pptrmt, termt,ntt
;   uio_closrd, nc, outstr,ierr
;;
; MODIFICATION HISTORY:
;   18. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   14.10.96 (B.F.) Renaming: linet
;   15.10.96 (B.F.) More comments
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
; --- Get format of input file ---
uio_chunit, 'ask', nc, 0, form, ierr
;
; --- Read file header completely ---
uio_rdhdln, nc,form,linet,nlt, outstr,ierr
;
if (outstr ne 'eof') then begin
  ; --- Put keywords into table ---
  uio_exmtrm, linet,nlt, termt,ntt ; , outstr,ierr
  ;
  outstr='done'
  ierr=0
endif else begin
  ntt=0
endelse
;
end ; uio_rdhd
