pro uio_rdlabl, nc,termt,ntt, ident, outstr,ierr, $
  name=name, date=date
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_rdlabl ('uio_read_label')
;
; PURPOSE:
;   Read label.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_rdlabl, nc,termt,ntt, ident,name,date, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number (actually not used)
;
; INPUT/OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;
; OUTPUT:
;   ident:  ('identifier') character, identifier
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   name:   ('name') character, name of variable, output
;   date:   ('date') character, date (and time) information, output
;
; ROUTINES:
;   uio_exkeyw: Extract value of keyword from table
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Extract information from termt by repeated calls of uio_exkeyw.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   23. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;    4.10.96 (B.F.) Test if entry is label
;    9.10.96 (B.F.) Test in different formulation
;   15.10.96 (B.F.) name and date are now keywords
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
; --- Check type ---
if (termt(0,0) ne lablna) then begin
  ierr=1
  outstr='No label, but ' + termt(0,0)
  goto, endlabel
endif
;
; --- Find keywords and extract their values ---
ident=termt(0,1)
termt(0,0:1)=''
m=(n_elements(name) ne 0)
uio_exkeyw, m,termt,ntt,namena, name ,  outstr,ierr
m=(n_elements(date) ne 0)
uio_exkeyw, m,termt,ntt,datena, date ,  outstr,ierr
uio_exkeyw, 4,termt,ntt,''    , dummy,  outstr,ierr
;
ierr=0
outstr='done'
;
endlabel:
;
end ; uio_rdlabl
