pro uio_slhd, nc, srchls, termt,ntt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_slhd ('uio_search_list_head_extract')
;
; PURPOSE:
;   Search header of variables given by list and extract keywords.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_slhd, nc, srchls, termt,ntt, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   srchls: ('search_list') character array, list with names of
;           types and/or identifiers of variables to be searched for
;           Attention: srchls is modified if it is scalar or one-dimensional
;
; OUTPUT:
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;   outstr: ('output_string') character, EOF -> 'eof'
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_rdhd: Read next header and extract keywords
;   uio_skipda: Skip data block
;
; INCLUDES:
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   Data are skipped in file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Read header, look if variable type and identifier is in search list.
;   Exit if found. Otherwise skip data block and continue the search.
;   Simulate call of srhdex by
;     uio_slhd, nc, [[type1,ident1],[type2,ident2]], termt,ntt, outstr,ierr
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   23. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
size_srchls=size(srchls)
if (size_srchls(0) eq 0) then begin
  srchls=reform(['',srchls],2,1)
endif else if (size_srchls(0) eq 1) then begin
  srchls=reform([srchls(0),srchls(1)],2,1)
endif
nsrc=n_elements(srchls)/2
;
ierr=0
;
goto, label20
;
label10:
; --- Skip data block ---
uio_skipda, nc, termt,ntt, outstr,ierr
label20:
; --- Read next header ---
uio_rdhd, nc, termt,ntt, outstr,ierr
;
if (outstr ne 'eof') then begin
  ; --- Loop over search list until variable found or end of list ---
  n=0
  found=1
  label100:
  if (n lt nsrc) then begin
    if (srchls(0,n) eq '') then begin
      found=srchls(1,n) eq termt(0,1)
    endif else if (srchls(0,n) ne termt(0,0)) then begin
      found=0
    endif else if (srchls(1,n) eq '') then begin
      found=1
    endif else begin
      found=srchls(1,n) eq termt(0,1)
    endelse
    n=n+1
    if (not found) then goto, label100
  endif
  ;
  if (not found) then goto, label10
  outstr='found
endif
;
end ; uio_slhd
