pro uio_st2dim, dimstr, iarr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_st2dim ('uio_string_to_dimensions')
;
; PURPOSE:
;   Analyse and parse dimension string.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_st2dim, dimstr, iarr
;
; INPUT:
;   dimstr: ('dimension_string') character, string with dimensions
;
; OUTPUT:
;   iarr:   ('index_array') long integer array(2,ndim)
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;   FORTRAN version can handle only 4 dimensions -> only 4 dimensions allowed.
;
; PROCEDURE:
;   Straightforward: Parse dimension string into single dimension
;   specifications.
;   The inverse routine is uio_dim2st.
;
; EXAMPLE:
;      dimstr          ndim  indices
;   (10:20,30:40)   ->   2:  10, 20    30, 40
;   (-30:30)        ->   1: -30, 30
;   (4)             ->   1:   1,  4
;   (3,4,5)         ->   3:   1,  3     1,  4     1,  5
;
; MODIFICATION HISTORY:
;    5. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;   17.10.97 (B.F.) Search colon in term, not in dimhlp
;-
;------------------------------------------------------------------------------
;
dimhlp=strcompress(dimstr,/remove_all)
lendim=strlen(dimhlp)
;
; --- Remove brackets ---
if (strmid(dimhlp,lendim-1,1) eq ')') then dimhlp=(strmid(dimhlp,0,lendim-1))
if (strmid(dimhlp,0,1) eq '(') then dimhlp=(strmid(dimhlp,1,80))
lendim=strlen(dimhlp)
;
; --- Extract indices, result in string array ---
ndim=-1
sarr=strarr(2,4)
repeat begin
  ndim=ndim+1
  ; --- Search next comma ---
  comma=strpos(dimhlp,',')
  if (comma ge 0) then begin
    term=strmid(dimhlp,0,comma)
    dimhlp=strmid(dimhlp,comma+1,80)
  endif else begin
    term=dimhlp
  endelse
  ; --- Search next colon ---
;  colon=strpos(dimhlp,':')
  colon=strpos(term,':')
  if (colon lt 0) then begin
    sarr(0,ndim)='1'
    sarr(1,ndim)=term
  endif else begin
    sarr(0,ndim)=strmid(term,0,colon)
    sarr(1,ndim)=strmid(term,colon+1,80)
  endelse
endrep until (comma lt 0)
;
; --- Transform string array into long integer array ---
iarr=long(sarr(*,0:ndim))
;
end ; uio_st2dim
