pro uio_tabr, val,ident, type=type, tab, outstr,ierr, $
    nbytes=nbytes, $
    format=format, name=name, unit=unit, tran=tran, date=date, $
    termt=termt,ntt=ntt, outcolumn=ncolsl, incolumn=incolumn
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_tabr ('uio_table_read')
;
; PURPOSE:
;   Read 1d array from table variable.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_tabr, val,ident, tab, type=type, outstr,ierr, $
;             nbytes=nbytes, $
;             format=format, name=name, unit=unit, tran=tran, date=date, $
;             termt=termt,ntt=ntt, outcolumn=outcolumn, incolumn=incolumn
;
; INPUT:
;   tab:    ('table') structure, complete table
;
; INPUT/OUTPUT:
;   ident:  ('identifier') character, column name to look for
;
; OUTPUT:
;   val:    ('value') integer, real, or character; table column to be read
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   type:   ('type') character, type of variable, input/output
;   nbytes: ('number_bytes') integer, number of characters of variable val to
;           be written (only if val is of type character), output
;   format: ('format') character, FORTRAN style format string, output
;   name:   ('name') character, name of variable, output
;   unit:   ('unit') character, string with units, output
;   tran:   ('transformation') character, transformation, output
;   date:   ('date') character, date string, output
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines,
;           output
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           output
;   incolumn: ('input_column') integer, number of entry column, input
;   outcolumn: ('input_column') integer, number of entry column, output
;
; ROUTINES:
;   uio_exkeyw: Extract value of keyword from table
;   uio_nc2nt:  From column number or entry name find table entry number
;
; INCLUDES:
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Find variable identifier in entry table tab.termt.
;   Read corresponding keywords and extract their values.
;   Read requested 1D array from table array.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   28. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   30. 9.96 (B.F.) Keywords in/outcolumn added
;    6.10.96 (B.F.) Parameter nc removed, routine uio_nc2nt called
;    7.10.96 (B.F.) ita -> nlin
;   10.10.96 (B.F.) Renamed: uio_rt -> uio_tabr
;   13.10.96 (B.F.) Renaming
;   15.10.96 (B.F.) More comments
;   10.12.09 (B.F. Lyon) Replaced some round by square brackets
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
;
; --- Find table column ---
if (n_elements(incolumn) eq 0) then incolumn=-1
if (incolumn ge 0) then begin
  ; --- Search column by number ---
  ncolsl=0>incolumn<tab.ncol
  uio_nc2nt, reform(tab.termt[0,0:1,0:tab.ncol-1]), tab.ncol, ncolsl, $
    ntaisl, ntarsl, ntassl, outstr,ierr
endif else begin
  ; --- Search column by name ---
  ncolsl=0
  uio_nc2nt, reform(tab.termt[0,0:1,0:tab.ncol-1]), tab.ncol, ncolsl, $
    ntaisl, ntarsl, ntassl, outstr,ierr, $
    type=type, ident=ident
endelse
found_flag=(outstr eq 'found')
;help,outstr,found_flag,ncolsl
;
if (found_flag) then begin
  termt=tab.termt[*,*,ncolsl]
  ntt  =tab.ntta[ncolsl]
  type =termt[0,0]
  ident=termt[0,1]
  termt[0,0:1]=''
  uio_exkeyw, 1,termt,ntt, dimna ,dimstr, outstr,ierr
  m=(n_elements(format) ne 0)
  uio_exkeyw, m,termt,ntt, frmtna,format, outstr,ierr
;  uio_exkeyw, 1,termt,ntt, plinna,perlin, outstr,ierr
;  rfrmt='(' + perlin + '(' + format + '))'
  m=(n_elements(name) ne 0)
  uio_exkeyw, m,termt,ntt, namena,name  , outstr,ierr
  m=(n_elements(unit) ne 0)
  uio_exkeyw, m,termt,ntt, unitna,unit  , outstr,ierr
  m=(n_elements(tran) ne 0)
  uio_exkeyw, m,termt,ntt, tranna,tran  , outstr,ierr
  m=(n_elements(date) ne 0)
  uio_exkeyw, m,termt,ntt, datena,date  , outstr,ierr
  uio_exkeyw, 3,termt,ntt, bytena,bytes , outstr,ierr
  ;reads, bytes, nbytes, format='(I6)'
  nbytes=fix(bytes)
  ;
  ; --- Read 1D array from table array ---
  if (tab.nlin eq 0) then begin
    val=''
    outstr='empty column'
    ierr=0
  endif else begin
    if (type eq intena) then begin
      val=tab.i[0:tab.nlin-1,ntaisl]
    endif else if (type eq realna) then begin
      val=tab.r[0:tab.nlin-1,ntarsl]
      ; --- Transformation ---
      if (tran eq 'log10') then val=10.0^val
    endif else if (type eq charna) then begin
      val=tab.s[0:tab.nlin-1,ntassl]
    endif else begin
      val=''
    endelse
    outstr='done'
    ierr=0
  endelse
  ;
endif else begin
  ncolsl=tab.ncol
  outstr='not found'
  ierr=1
endelse
;
end ; uio_tabr
