pro uio_vnanrm, inline, ouline,remain, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_vnanrm ('uio_variable_name_norm')
;
; PURPOSE:
;   Transforms a string "inline" 
;   into a string "ouline", which now contains a correct name of a variable.
;
; CATEGORY:
;   String processing
;
; CALLING SEQUENCE:
;   uio_vnanrm, inline, ouline,remain,outstr,ierr
;
; INPUT:
;   inline: ('input_line') character
;
; OUTPUT:
;   ouline: ('output_line') character, maximum length: let-2
;   remain: ('remainder') character
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uionaminc: names of types, keywords, identifiers; default formats
;   uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   None
;
; RESTRICTIONS:
;
; PROCEDURE:
;  The input string is tested if it is a correct variable/keyword name.
;  The end of the name of a variable is determined by a following blank ' ' or
;  a "equchr" '='.
;  The rest of "inline" which does not fit in "ouline" is given out in "remain".
;  The maximum length of ouline is (let-2).
;
; EXAMPLES:
;    inline:                ouline       remain        ierr
;    'rho'                  'rho'        ''            0
;    '  rho'                'rho'        ''            0
;    'rho  rho'             'rho'        '  rho'       1
;    'rho=10.0'             'rho'        '=10.0'       1
;    '=10.0'                ''           '=10.0'       1
;    ' '                    ''           ''            1
;
; MODIFICATION HISTORY:
;    9. 2.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Include common blocks ---
@uio_uionaminc
@uio_uiosizinc
;
ierr=0
;
; --- Delete leading and trailing blanks ---
ouline=strtrim(inline,2)
if (ouline eq '') then begin
  ; --- Input string empty ---
  remain=''
  outstr='empty'
  ierr=1
endif else begin
  ; --- Input string non-empty ---
  indbl =strpos(ouline,' ')
  indequ=strpos(ouline,equchr)
  if (indequ eq 0) then begin
    remain=ouline
    ouline=''
    outstr='no name'
    ierr=1
  endif else if (indbl le -1) then begin
    if (indequ le -1) then begin
      remain=''
      outstr='done'
    endif else begin
      remain=strmid(ouline,indequ+1,let-2)
      ouline=strmid(ouline,0,indequ)
      outstr='name not correct'
    endelse
  endif else begin
    if (indequ le -1) then begin
      ind=indbl
    endif else begin
      ind=indbl < indequ
    endelse
    remain=strmid(ouline,ind+1,let-2)
    ouline=strmid(ouline,0,ind)
    outstr='more than one name'
  endelse
endelse
;
ouline=strmid(ouline,0,let-2)
;
end ; uio_vnanrm
