pro uio_wrfifo, nc, outstr,ierr, progrm=program, termt=termt,ntt=ntt
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wrfifo ('uio_write_fileform')
;
; PURPOSE:
;   Write file header.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wrfifo, nc, outstr,ierr, progrm=program, termt=termt,ntt=ntt
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   progrm: ('program') character, name of calling program, input
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           (ntt >= 0)
;
; ROUTINES:
;   uio_adkey1: Add one keyword to term table, keyword-value=' ', no link char.
;   uio_adkey2: Add one keyword to term table with keyword-value
;   uio_adkey3: Add one keyword to term table with keyword-value or default
;   uio_chunit: Initialize, store and actualize a list of unit numbers
;   uio_getenv: Get information about environment
;   uio_wrhd: Write header of variable, input: term table
;
; INCLUDES:
;   uio_uiocstinc: status and convert type for all channels
;   uio_uiocvlinc: all convert type and default format for current machine
;   uio_uionaminc: names of types, keywords, identifiers; default formats
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   uio_chainf in uio_uiocstinc
;   uio_cvlist in uio_uiocvlinc
;   uio_defnam in uio_uionaminc
;
; SIDE EFFECTS:
;   Data lines are written to file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   The header data are assembled from data about the system, merged into lines,
;   and written into file on unit nc.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   20. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   30. 9.96 (B.F.) Keyword progrm added, parameter conv removed
;    2.10.96 (B.F.) ncmin may become negative, cvnum(.) -> cvnum(.-ncmin)
;    7.10.96 (B.F.) IDL release added
;   13.10.96 (B.F.) Renaming: termt, linet
;   15.10.96 (B.F.) More comments
;   16.10.96 (B.F.) termt becomes keyword
;   26.10.96 (B.F.) Use uio_wrhd instead of uio_wrhdln.
;   11. 1.97 (B.F.) uio_getenv without date
;    1. 2.97 (B.F.) different call of uio_getenv
;   20. 2.97 (B.F.) write conversion type also for 'unformatted' file
;------------------------------------------------------------------------------
;
@uio_uiosizinc
@uio_uionaminc
@uio_uiocstinc
@uio_uiocvlinc
;
; --- Get some information about environment ---
uio_getenv, system=system, machin=machin, osrels=osrels, osvers=osvers, $
            hrdwar=hrdwar, progrm=progrm, langua=langua
;
; --- Get date/time data ---
;date=systime(0)
date=strmid(systime(0),0,24)
;
; --- Get channel unit ---
uio_chunit, 'ask',nc,0, form,ierr
;
; --- Assemble file header: standard keywords ---
ntt_tmp=0
uio_adkey1, termt_tmp,ntt_tmp,fifona,                    outstr,ierr
uio_adkey1, termt_tmp,ntt_tmp,uiona ,                    outstr,ierr
uio_adkey3, termt_tmp,ntt_tmp,formna,form  ,'formatted', outstr,ierr
;if (form eq 'unformatted') then begin
  uio_adkey2, termt_tmp,ntt_tmp,convna,cvnals(cvnum(nc-ncmin)), outstr,ierr
;endif
uio_adkey2, termt_tmp,ntt_tmp,versna,versio,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,datena,date  ,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,systna,system,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,machna,machin,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,osrena,osrels,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,osvena,osvers,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,hrdwna,hrdwar,             outstr,ierr
uio_adkey2, termt_tmp,ntt_tmp,langna,langua +' '+ !VERSION.RELEASE, outstr,ierr
uio_adkey3, termt_tmp,ntt_tmp,progna,program,progrm,     outstr,ierr
if (n_elements(ntt) eq 0) then ntt=n_elements(termt)/2
;
; --- Concatenate the two term tables ---
nttn=(ntt_tmp+ntt)<nttmx
if (nttn gt ntt_tmp) then begin
  termt_tmp(*,ntt_tmp:nttn-1)=termt(*,0:nttn-ntt_tmp-1)
endif
ntt_tmp=nttn
;
; --- Write header of variable ---
uio_wrhd, nc,form, termt_tmp,ntt_tmp, outstr,ierr
;
end ; uio_wrfifo
