pro uio_wrhd, nc,form, termt,ntt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wrhd ('write_head')
;
; PURPOSE:
;   Write header of variable, input: term table.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wrhd, nc,form, termt,ntt, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   form:   ('format') character, 'formatted' or 'unformatted'
;   termt:  ('term_table') character array (2,nttmx), 2 columns, nttmx lines
;   ntt:    ('number_term_table') integer, number of non-empty lines of termt
;           (ntt >= 0)
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   uio_memtrm: Merge a list of terms (keywords and values) into a line table
;   uio_wrhdln: Write header of variable
;
; INCLUDES:
;   None
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   Data are written into file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Merge term table (call uio_memtrm) and
;   write entry header into file (call uio_wrhdln).
;
; EXAMPLE:
;   ; --- Produce sample term table ---
;   termt=[['real',''],['r',''],['name','radius'],['unit','km']]
;   ntt=4
;   ; --- Write to standard output ---
;   uio_wrhd, -1,'formatted', termt,ntt, outstr,ierr
;     --->
;   real r name=radius unit=km
;
; MODIFICATION HISTORY:
;   26.10.96 Written by Bernd Freytag, Astrophysik Kiel
;-
;------------------------------------------------------------------------------
;
; --- Merge header ---
nlt=0
uio_memtrm, termt,ntt, linet,nlt, outstr,ierr
;
; --- Write header of variable ---
uio_wrhdln, nc,form, linet,nlt, outstr,ierr
;
end ; uio_wrhd
