pro uio_wrhdln, nc,form,linet,nlt, outstr,ierr
;------------------------------------------------------------------------------
;+
; NAME:
;   uio_wrhdln ('write_head_line')
;
; PURPOSE:
;   Write header of variable.
;
; CATEGORY:
;   Input/Output
;
; CALLING SEQUENCE:
;   uio_wrhdln, nc,form,linet,nlt, outstr,ierr
;
; INPUT:
;   nc:     ('number_channel') integer, channel unit number
;   form:   ('format') character, 'formatted' or 'unformatted'
;   linet:  ('line_table') character array, nltls (<nltmx) lines
;   nlt:    ('number_line_table') integer, number of non-empty lines of linet
;
; OUTPUT:
;   outstr: ('output_string') character
;   ierr:   ('integer_error') integer, contains exit error status
;            0: correct operation
;           >0: occurance of an error
;
; KEYWORDS:
;   None
;
; ROUTINES:
;   None
;
; INCLUDES:
;   uio_uiosizinc: length of strings, size of tables (termt,lntab)
;
; COMMON BLOCKS:
;   None
;
; SIDE EFFECTS:
;   Data are written into file on unit nc.
;
; RESTRICTIONS:
;
; PROCEDURE:
;   Write entry header line by line into file according to file format.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   16. 9.96 Written by Bernd Freytag, Astrophysik Kiel
;   13.10.96 (B.F.) Renaming: linet
;   14.10.96 (B.F.) Continuation lines are plotted differently
;   16.10.96 (B.F.) More comments, unformatted writing faster?
;-
;------------------------------------------------------------------------------
;
@uio_uiosizinc
;
pad=string(' ',format='(A'+string(lel)+')')
if (form eq 'unformatted') then begin
  ; --- Write unformatted ---
  for nltit=0,nlt-1 do begin
;    line=linet(nltit)
;    lenline=strlen(line)
;    if (lenline lt lel) then line=line+ $
;                                string(' ',format='(A'+string(lel-lenline)+')')
;    writeu, nc, line
    writeu, nc, strmid(linet(nltit)+pad,0,lel)
  endfor
endif else begin
  ; --- Write formatted ---
;  printf, nc, linet(0:nlt-1), format='(A)'
  printf, nc, linet(0), format='(A)'
;  if (nlt gt 0) then printf, nc, linet(1:nlt-1), format='(A80)'
  for n=1,nlt-1 do begin
    if (strlen(linet(n)) le (lel-2)) then pad='  ' else pad=''
    printf, nc, pad+linet(n), format='(A)'
  endfor; n
endelse
;
outstr='done'
ierr=0
;
end ; uio_wrhdln
