;-------------------------------------------------------------------------------
; Vis3D module: Axis or box
;-------------------------------------------------------------------------------
; Contains:
;-------------------------------------------------------------------------------


function v3d_create3dbox, xrange, yrange, zrange, COLOR=color, HIDE=hide
;-------------------------------------------------------------------------------
; NAME:
;   v3d_create3dbox ('v3d_create_3_dimensional_box')
;
; PURPOSE:
;   Create a 3D object, a box which shows the dataset boundaries.
;
; CATEGORY:
;   3D visualization
;
; CALLING SEQUENCE:
;   oBox=v3d_create3dbox(xrange, yrange, zrange, COLOR=color, HIDE=hide)
;
; MODIFICATION HISTORY:
;   2002-06-03 Written by Sigurdur Finnsson, Uppsala
;   2005-07-08 (B.F.) Second color for xyz axes, extension of z axis
;   2005-07-18 (B.F.) Include v3d_parameters
;   2005-09-07 (B.F.) Increase box size by about 1/2 grid point on each side
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; red is the default color 
  if (N_ELEMENTS(color) eq 0) then color=v3dpar_box_Color

  ; by default the box is hidden 
  if (N_ELEMENTS(hide) eq 0) then hide=1

  ; Some variables for more convenient typing
;  d3xmin=float(min(xrange))
;  d3xmax=float(max(xrange))
;  d3ymin=float(min(yrange))
;  d3ymax=float(max(yrange))
;  d3zmin=float(min(zrange))
;  d3zmax=float(max(zrange))
  d3xmin=float(min(xrange))-0.5001
  d3xmax=float(max(xrange))+0.5001
  d3ymin=float(min(yrange))-0.5001
  d3ymax=float(max(yrange))+0.5001
  d3zmin=float(min(zrange))-0.5001
  d3zmax=float(max(zrange))+0.5001
  d3zext=d3zmax+0.03D+00*(d3zmax-d3zmin)

  ; Create a box which contains the array within
  ; Vertices array
  box_vert=[[d3xmin,d3ymax,d3zmax],$  ;#0
            [d3xmax,d3ymax,d3zmax],$  ;#1
            [d3xmin,d3ymin,d3zmax],$  ;#2
            [d3xmax,d3ymin,d3zmax],$  ;#3
            [d3xmin,d3ymax,d3zmin],$  ;#4
            [d3xmax,d3ymax,d3zmin],$  ;#5
            [d3xmin,d3ymin,d3zmin],$  ;#6
            [d3xmax,d3ymin,d3zmin],$  ;#7
            [d3xmin,d3ymin,d3zext]]   ;#8

  box_poly=[4, 0,1,3,2, $  ;#1
            4, 1,5,7,3, $  ;#2
            4, 4,5,1,0, $  ;#3
            4, 4,0,2,6, $  ;#4
            4, 7,6,2,3, $  ;#5
            4, 5,4,6,7, $  ;#6
            2, 8,6,     $  ;#7
           -1]             ; Terminate the polygon list

  cx=transpose(v3dpar_box_Color2)
  ;cy=transpose([255,  0,255])
  ;cz=transpose([255,255, 90])
  c0=transpose(color)
  vert_colors=transpose([c0, c0, c0, c0, c0, c0, cx, c0, c0])
  ;vert_colors=transpose( $
  ;            [    c0, c0, c0, c0, $
  ;             c0, c0, c0, c0, c0, $
  ;             c0, c0, c0, c0, c0, $
  ;             c0, c0, c0, c0, cz, $
  ;             cy, c0, cx, cz, c0, $
  ;             c0, c0, cy, cx])

  ; And return the handler to the new object
  return, OBJ_NEW('IDLgrPolyline', $
                  box_vert, $
                  polylines=box_poly, $
                  color=color, $
                  vert_colors=vert_colors, $
                  HIDE=hide)

end


pro v3d_axis_createobj, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_axis_createobj ('v3d_axis_create_obj')
;
; PURPOSE:
;   Destroy axis structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_axis_createobj, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; HISTORY:
;   2005-09-09 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  (*sState)[index].oBox=v3d_create3dbox((*sState)[index].box.xrange, $
                                        (*sState)[index].box.yrange, $
                                        (*sState)[index].box.zrange, $
                                        COLOR=(*sState)[index].axis.color1, $
                                        HIDE =(*sState)[index].hideaxis)

;   xmin=float(min((*sState)[index].box.xrange))-0.5
;   xmax=float(max((*sState)[index].box.xrange))+0.5
;   ymin=float(min((*sState)[index].box.yrange))-0.5
;   ymax=float(max((*sState)[index].box.yrange))+0.5
;   zmin=float(min((*sState)[index].box.zrange))-0.5
;   zmax=float(max((*sState)[index].box.zrange))+0.5

;   (*sState)[index].oBoxEdges=obj_new('IDLgrROIGroup', HIDE=1)

;   nx   =(*sState)[index].box.nx
;   EdgeX=fltarr(3, 4*(nx+1))
;   EdgeX[0, 0:4*(nx+1)-1:4]=findgen(nx+1)/float(nx)*(xmax-xmin) + xmin
;   EdgeX[0, 1:4*(nx+1)-1:4]=findgen(nx+1)/float(nx)*(xmax-xmin) + xmin
;   EdgeX[0, 2:4*(nx+1)-1:4]=findgen(nx+1)/float(nx)*(xmax-xmin) + xmin
;   EdgeX[0, 3:4*(nx+1)-1:4]=findgen(nx+1)/float(nx)*(xmax-xmin) + xmin
;   EdgeX[1, 0:4*(nx+1)-1:4]=ymin
;   EdgeX[1, 1:4*(nx+1)-1:4]=ymin
;   EdgeX[1, 2:4*(nx+1)-1:4]=ymax
;   EdgeX[1, 3:4*(nx+1)-1:4]=ymax
;   EdgeX[2, 0:4*(nx+1)-1:4]=zmin
;   EdgeX[2, 1:4*(nx+1)-1:4]=zmax
;   EdgeX[2, 2:4*(nx+1)-1:4]=zmin
;   EdgeX[2, 3:4*(nx+1)-1:4]=zmax
;   (*sState)[index].oBoxEdgeX=obj_new('IDLgrROI', EdgeX, STYLE=0, HIDE=0)
;   (*sState)[index].oBoxEdges->Add, (*sState)[index].oBoxEdgeX

;   ny   =(*sState)[index].box.ny
;   EdgeY=fltarr(3, 4*(ny+1))
;   EdgeY[0, 0:4*(ny+1)-1:4]=xmin
;   EdgeY[0, 1:4*(ny+1)-1:4]=xmin
;   EdgeY[0, 2:4*(ny+1)-1:4]=xmax
;   EdgeY[0, 3:4*(ny+1)-1:4]=xmax
;   EdgeY[1, 0:4*(ny+1)-1:4]=findgen(ny+1)/float(ny)*(ymax-ymin) + ymin
;   EdgeY[1, 1:4*(ny+1)-1:4]=findgen(ny+1)/float(ny)*(ymax-ymin) + ymin
;   EdgeY[1, 2:4*(ny+1)-1:4]=findgen(ny+1)/float(ny)*(ymax-ymin) + ymin
;   EdgeY[1, 3:4*(ny+1)-1:4]=findgen(ny+1)/float(ny)*(ymax-ymin) + ymin
;   EdgeY[2, 0:4*(ny+1)-1:4]=zmin
;   EdgeY[2, 1:4*(ny+1)-1:4]=zmax
;   EdgeY[2, 2:4*(ny+1)-1:4]=zmin
;   EdgeY[2, 3:4*(ny+1)-1:4]=zmax
;   (*sState)[index].oBoxEdgeY=obj_new('IDLgrROI', EdgeY, STYLE=0, HIDE=0)
;   (*sState)[index].oBoxEdges->Add, (*sState)[index].oBoxEdgeY

;   nz   =(*sState)[index].box.nz
;   EdgeZ=fltarr(3, 4*(nz+1))
;   EdgeZ[0, 0:4*(nz+1)-1:4]=xmin
;   EdgeZ[0, 1:4*(nz+1)-1:4]=xmin
;   EdgeZ[0, 2:4*(nz+1)-1:4]=xmax
;   EdgeZ[0, 3:4*(nz+1)-1:4]=xmax
;   EdgeZ[1, 0:4*(nz+1)-1:4]=ymin
;   EdgeZ[1, 1:4*(nz+1)-1:4]=ymax
;   EdgeZ[1, 2:4*(nz+1)-1:4]=ymin
;   EdgeZ[1, 3:4*(nz+1)-1:4]=ymax
;   EdgeZ[2, 0:4*(nz+1)-1:4]=findgen(nz+1)/float(nz)*(zmax-zmin) + zmin
;   EdgeZ[2, 1:4*(nz+1)-1:4]=findgen(nz+1)/float(nz)*(zmax-zmin) + zmin
;   EdgeZ[2, 2:4*(nz+1)-1:4]=findgen(nz+1)/float(nz)*(zmax-zmin) + zmin
;   EdgeZ[2, 3:4*(nz+1)-1:4]=findgen(nz+1)/float(nz)*(zmax-zmin) + zmin
;   (*sState)[index].oBoxEdgeZ=obj_new('IDLgrROI', EdgeZ, STYLE=0, HIDE=0)
;   (*sState)[index].oBoxEdges->Add, (*sState)[index].oBoxEdgeZ

end ; v3d_axis_createobj


pro v3d_axis_destroy, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_axis_destroy ('v3d_axis_destroy')
;
; PURPOSE:
;   Destroy axis structure and free memory used in it.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_axis_destroy, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; PROCEDURE:
;   Do nothing.
;
; HISTORY:
;   2005-09-09 Written by Bernd Freytag, L.A.
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

end ; v3d_axis_destroy


function v3d_axis_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_axis_init ('v3d_axis_initialize')
;
; PURPOSE:
;   Initialize axis structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   axis=v3d_axis_init()
;
; MODIFICATION HISTORY:
;   2005-09-09 Written by Bernd Freytag, L.A.
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  ; Create an anonymous structure containing the basic data for the axis/box rendering
  axis={$
    color1:                v3dpar_box_Color, $
    color2:                v3dpar_box_Color2 $
       }

  return, axis

end ; v3d_axis_init
