;-------------------------------------------------------------------------------
; Vis3D module: Handling of control widget
;-------------------------------------------------------------------------------
; Contains:
;   v3d_select_event:        Handle all events originating from select widget
;   v3d_createselectwidget:  Create the select widget
;   v3d_updatecontrolwidget: Update the control widget while keeping its position
;   v3d_updatefields:        Update all sub control widgets in the control panel
;   v3d_createcontrolwidget: Create control widget from data in the state variable
;-------------------------------------------------------------------------------


function v3d_select_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_select_event ('v3d_select_event')
;
; PURPOSE:
;   Handle all mesh events originating from select widget.
;
; CATEGORY:
;   3D Visualization
;
; MODIFICATION HISTORY:
;   2005-08-21 (B.F.) Taken from v3d_dataset_event
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ;message, '', /INFORMATIONAL
  ;help, /str, sEvent
  ;help, sState, index, uval

  if (uval eq 'BUTTON_AXIS_SHOWHIDEOBJ') then begin ; --------------------------
    ; The Show/Hide reference box button
    ;
    ; Toggle the hide flag of the 3D box
    (*sState)[index].oBox->GetProperty, HIDE=hide
    (*sState)[index].oBox->SetProperty, HIDE=1-keyword_set(hide)
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
  endif else $

  if (uval eq 'BUTTON_LIGHT_SHOWHIDEOBJ') then begin ; -------------------------
    ; The Show/Hide light button
    ;
    ; Toggle the hide flag of the light objects
    (*sState)[index].oLight1->GetProperty, HIDE=hide
    (*sState)[index].oLight1->SetProperty, HIDE=1-keyword_set(hide)
    (*sState)[index].oLight2->SetProperty, HIDE=1-keyword_set(hide)
    (*sState)[index].oWindow->Draw, (*sState)[index].oView
  endif else $

  if (uval eq 'BUTTON_OUSER_SHOWHIDEOBJ') then begin ; -------------------------
    ; The Show/Hide oUser button
    ;
    ; Toggle the hide flag of the user object
    if (OBJ_VALID([(*sState)[index].oUserHigh])) then begin
      (*sState)[index].oUserHigh->GetProperty, HIDE=hide
      (*sState)[index].oUserHigh->SetProperty, HIDE=1-keyword_set(hide)
      (*sState)[index].oWindow->Draw, (*sState)[index].oView
    endif
  endif else $

  if (uval eq 'BUTTON_ISO_SHOWHIDECONTROL') then begin ; -----------------------
    ; The button to open isosurface control window
    ;
    message, 'Open iso controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlIso=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif else $

  if (uval eq 'BUTTON_COLOR_SHOWHIDECONTROL') then begin ; ---------------------
    ; The button to open color control window
    ;
    message, 'Open color-palette controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlColor=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif else $

  if (uval eq 'BUTTON_DOTS_SHOWHIDECONTROL') then begin ; -----------------------
    ; The button to open dots control window
    ;
    message, 'Open dots controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlDots=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif

  if (uval eq 'BUTTON_VOL_SHOWHIDECONTROL') then begin ; ------------------------
    ; The button to open vol control window
    ;
    message, 'Open volume controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlVol=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif

  if (uval eq 'BUTTON_SLICE_SHOWHIDECONTROL') then begin ; -----------------------
    ; The button to open slice window
    ;
    message, 'Open slice controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlSlice=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif

  if (uval eq 'BUTTON_STREAM_SHOWHIDECONTROL') then begin ; ----------------------
    ; The button to open streamline control window
    ;
    message, 'Open stream controls', /INFORMATIONAL
    ;
    (*sState)[index].hideControlStream=0
    ;
    v3d_updatecontrolwidget, sState
    ;
  endif

end ; v3d_select_event, sEvent


function v3d_createselectwidget, wBaseSelect, sState, index
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_createselectwidget ('v3d_create_select_widget')
;
; PURPOSE:
;   Create the select widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_createselectwidget(wBaseSelect, sState, index)
;
; INPUT:
;   wBaseSelect:  ('widget_Base_Select') parent widget
;   index:        ('index') current view index
;
; INPUT/OUTPUT:
;   sState:       ('structure_State') The state variable array
;
; RETURNED VALUE:
;   result:       Widget
;
; KEYWORDS:
;
; MODIFICATION HISTORY:
;   2005-08-23 (B.F.) Written
;-
;-------------------------------------------------------------------------------

compile_opt hidden, strictarr

wBaseButtons=widget_base(wBaseSelect, $
                         /COLUMN, $
                         UVALUE=index, $
                         EVENT_FUNC='v3d_select_event', $
                         XPAD=0, YPAD=0)
  ;
  (*sState)[index].wShowHideAxis=widget_button(wBaseButtons,         VALUE ='Axis', $
                                               /ALIGN_CENTER,        UVALUE='BUTTON_AXIS_SHOWHIDEOBJ', $
                                               TOOLTIP='Show/hide box/axis object')
;  if ((*sState)[index].hideAxis) then begin
;        widget_control, (*sState)[index].wShowHideAxis, $
;                        SET_VALUE='Show'
;  endif
  ;
  wShowHide=widget_button(wBaseButtons,           VALUE ='Light', $
                          /ALIGN_CENTER,          UVALUE='BUTTON_LIGHT_SHOWHIDEOBJ', $
                          TOOLTIP='Show/hide external light source')
  ;
  if (obj_valid((*sState)[index].oUser    ) or $
      obj_valid((*sState)[index].oUserHigh)) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='oUser', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_OUSER_SHOWHIDEOBJ', $
                            TOOLTIP='Show/hide user object')
  endif
  ;
  if ((*sState)[index].hideControlColor) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Color', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_COLOR_SHOWHIDECONTROL', $
                            TOOLTIP='Display color-palette controls')
  endif
  ;
  if ((*sState)[index].hideControlIso) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Iso', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_ISO_SHOWHIDECONTROL', $
                            TOOLTIP='Display iso-surface controls')
  endif
  ;
  if ((*sState)[index].hideControlDots) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Dots', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_DOTS_SHOWHIDECONTROL', $
                            TOOLTIP='Display dots volume-rendering controls')
  endif
  ;
  if ((*sState)[index].hideControlVol) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Vol', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_VOL_SHOWHIDECONTROL', $
                            TOOLTIP='Display volume-rendering controls')
  endif
  ;
  if ((*sState)[index].hideControlSlice) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Slice', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_SLICE_SHOWHIDECONTROL', $
                            TOOLTIP='Display slice controls')
  endif
  ;
  if (((*sState)[index].hideControlStream) and $
      (PTR_VALID((*sState)[index].box.vecxdataset))) then begin
    wShowHide=widget_button(wBaseButtons,         VALUE ='Stream', $
                            /ALIGN_CENTER,        UVALUE='BUTTON_STREAM_SHOWHIDECONTROL', $
                            TOOLTIP='Display stream controls')
  endif

return, wBaseButtons

end ; v3d_createselectwidget


pro v3d_updatecontrolwidget, sState
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_updatecontrolwidget ('v3d_update_contol_widgets')
;
; PURPOSE:
;   Update the control widget while keeping its position.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_updatecontrolwidget, sState
;
; INPUT/OUTPUT:
;   sState:  ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Base widget
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;   2005-09-07 (B.F.) Put into separate routine
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  geometry=widget_info((*sState)[0].wBaseControl, /GEOMETRY)
  widget_control, (*sState)[0].wBaseControl, /DESTROY

  (*sState)[*].wBaseControl=v3d_createcontrolwidget(sState)
  widget_control, (*sState)[0].wBaseControl, $
                  XOFFSET=geometry.xoffset, $
                  YOFFSET=geometry.yoffset
  widget_control, (*sState)[0].wBaseControl, /REALIZE
  geometry2=widget_info((*sState)[0].wBaseControl, /GEOMETRY)
  widget_control, (*sState)[0].wBaseControl, $
                  XOFFSET=geometry.xoffset-(geometry2.xoffset-geometry.xoffset), $
                  YOFFSET=geometry.yoffset-(geometry2.yoffset-geometry.yoffset)

  v3d_updatefields, sState

  widget_control, (*sState)[0].wBaseControl, SET_UVALUE=sState
  xmanager, 'vis3d', (*sState)[0].wBaseControl, /NO_BLOCK

end ; v3d_updatecontrolwidget


pro v3d_updatefields, sState
;-------------------------------------------------------------------------------
; NAME:
;   v3d_updatefields ('v3d_update_fields')
;
; PURPOSE:
;   Update all sub control widgets in the control panel.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_updatefields, sState
;
; INPUT/OUTPUT:
;   sState:    ('structure_State') The state variable array
;
; MODIFICATION HISTORY:
;   2005-08-23 (B.F.) Put into own routine
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  if ((*sState)[0].no_controls eq 0) then begin
    ; --- Fill text/info widgets ---
    ;
    for i=0, n_elements(*sState)-1 do begin
      ;
      ; --- Iso control ---
      ; Show initial values (isovalue and decimation) in input text widgets 
      v3d_iso_updatewidget, sState, i
      ;
      ; --- Color control ---
      ; Show initial palette range in input text widgets
      v3d_color_updatewidget, sState, i
      ;
      ; --- Dots control ---
      v3d_dots_updatewidget, sState, i
      ;
      ; --- Volume control ---
      v3d_vol_updatewidget, sState, i
      ;
      ; --- Slice control ---
      v3d_slice_updatewidget, sState, i
      ;
      ; --- Streamline control ---
      v3d_stream_updatewidget, sState, i
      ;
    endfor ; i
  endif

end ; v3d_updatefields


function v3d_createcontrolwidget, sState
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_createcontrolwidget ('v3d_create_contol_widgets')
;
; PURPOSE:
;   Create control widget from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_createcontrolwidget(sState)
;
; INPUT/OUTPUT:
;   sState:  ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Base widget
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Widget Hierarchy:
;     wBaseControl            , UVALUE=sState (via xmanager)
;       for i=0, nrofviews-1
;         wDSbaseControl
;           wBaseFields
;             wMiscCtrl
;             wColorControl
;             wIsoControl
;             wDotsControl
;             wVolControl
;             wSliceControl
;             wStreamControl
;       endfor
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-08-06 (B.F.) Collected from bits in vis3d and put into separate file
;   2005-08-18 (B.F.) Keywords separate and wBaseControl added
;   2005-08-22 (B.F.) Put into separate routine
;   2005-08-31 (B.F.) Controls for dots
;   2006-06-07 (B.F.) Controls for vol
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  nrofviews=n_elements(*sState)

  if ((*sState)[0].no_menu eq 0) then begin
    wBaseControl=widget_base(TITLE='Vis3D Control', $
                             GROUP=(*sState)[0].wBase, $
;                            /TLB_KILL_REQUEST_EVENTS, $
;                            /TLB_SIZE_EVENTS,$
                             /ROW, $
                             MBAR=wMenu, $
;                             YOFFSET=(*sState)[0].viewysize+70, $
                             YOFFSET=(*sState)[0].viewysize+36, $
                             XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
    (*sState)[*].wMenu=wMenu
  endif else begin
    wBaseControl=widget_base(TITLE='Vis3D Control', $
                             GROUP=(*sState)[0].wBase, $
;                            /TLB_KILL_REQUEST_EVENTS, $
;                            /TLB_SIZE_EVENTS,$
                             /ROW, $
                             YOFFSET=(*sState)[0].viewysize+70, $
                             XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
  endelse
  ;
  if ((*sState)[0].no_menu eq 0) then begin
    ; --- The menu on top of the graphics window ---
    ;
    v3d_createmenuwidget, wMenu, sState
    ;
  endif
  ;
  ; Loop over all views
  for i=0, nrofviews-1 do begin
    ; Generate the widgets, and pack all into a state variable
    wDSbaseControl=widget_base(wBaseControl, $
                               /COLUMN, /FRAME, $
;                               UVALUE=i, $
;                               EVENT_FUNC='v3d_draw_event', $
                               XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
      ;
      ; Base widget which contains the controls and information table for the dataset
      ;wBaseDataCtrl=widget_base(wDSbaseControl, $
      ;                          /COLUMN, $
      ;                          XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
      ;
      ; Widget for text field
      ;(*sState)[i].wInfo=v3d_createinfo(wBASE=wBaseDataCtrl)
      ;
      ; Base widget which contains the controls
      wBaseFields=widget_base(wDSbaseControl, $
                              /ROW, $
                              XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
      (*sState)[i].wBaseFields=wDSbaseControl
        ;
        wMiscCtrl=v3d_createselectwidget(wBaseFields, sState, i)
        ;
        if ((*sState)[i].hideControlColor eq 0) then begin
          (*sState)[i].wColorControl=v3d_color_createwidget(wBaseFields, sState, i)
        endif
        ;
        if ((*sState)[i].hideControlIso eq 0) then begin
          (*sState)[i].wIsoControl=v3d_iso_createwidget(wBaseFields, sState, i)
        endif
        ;
        if ((*sState)[i].hideControlDots eq 0) then begin
          (*sState)[i].wDotsControl=v3d_dots_createwidget(wBaseFields, sState, i)
        endif
        ;
        if ((*sState)[i].hideControlVol eq 0) then begin
          (*sState)[i].wVolControl=v3d_vol_createwidget(wBaseFields, sState, i)
        endif
        ;
        if ((*sState)[i].hideControlSlice eq 0) then begin
          (*sState)[i].wSliceControl=v3d_slice_createwidget(wBaseFields, sState, i)
        endif
        ;
        if ((*sState)[i].hideControlStream eq 0) then begin
          (*sState)[i].wStreamControl=v3d_stream_createwidget(wBaseFields, sState, i)
        endif
    ;
  endfor ; Loop over all views

  return, wBaseControl

end ; v3d_createcontrolwidget
