;-------------------------------------------------------------------------------
; Vis3D module: Main menu handling
;-------------------------------------------------------------------------------
; Contains:
;   v3d_menu_displayhelpintro: Display help text with general introduction to vis3D
;   v3d_menu_displayhelpdraw:  Display help text about draw window button control
;   v3d_menu_displayhelpspeed: Display help text with tips to speed up vis3D
;   v3d_makemovie:             Make movie from the data and control parameters in sState
;   v3d_drawwindow_event:      Handle all events that are NOT directly related to a dataset
;   v3d_menu_event:            Handle all main menu events events
;   v3d_createmenuwidget:      Create the main menu
;-------------------------------------------------------------------------------


pro v3d_menu_displayhelpintro, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_menu_displayhelpintro ('v3d_menu_display_help_intro')
;
; PURPOSE:
;   Display help text with general introduction to vis3D.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_menu_displayhelpintro, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-11-21 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Introduction to Vis3D', $
'=====================', $
'', $
'Vis3D is a tool for the multi-view visualization of 3D datasets.', $
'', $
'A simple call', $
'  vis3d', $
'on the IDL command line starts the program with demo data (2 pairs of datasets).', $
'After that some information can be found in the entries in the main "Help" menu', $
'and in the "Help" entries hidden under the main button of the individual tools', $
'(e.g. click on "Isosurf 1" and then "Help").', $
'', $
'Vis3D uses two main windows: The "View" window displays the data in a separate', $
'view for each pair of datasets. The "Control" window contains the menu and ', $
'one set of controls of each view. To limit the amount of space necessary for', $
'the multi-view multi-tool control the individual control panels can be closed', $
'(with the "Close" button in the top row) and reopened (with the button on the', $
'left corresponding to the tool) without changing the display of the data.', $
'', $
'The Control window can be closed with the window manager and reopened by', $
'pointing the mouse into the view window and typing "m" (see the Help entry', $
'"View Control"). Closing the view window kills the application.', $
'', $
'The view angle of the object can be adjusted by left clicking into any view', $
'panel and dragging the object (mouse button always pressed). For more options', $
'see the Help entry "View Control".', $
'', $
'The most simple non-trivial call of Vis3D would be', $
'  vis3d, array3D_1, array3D_2', $
'where the 3D array array3D_1 would be used to produce iso-surfaces', $
'and the array3D_2 (array with same dimensions as array3D_1) would be used to', $
'define the colors.']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0], $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_menu_displayhelpintro


pro v3d_menu_displayhelpdraw, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_menu_displayhelpdraw ('v3d_menu_display_help_draw')
;
; PURPOSE:
;   Display help text about draw window button control.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_menu_displayhelpdraw, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2005-11-16 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Control of views via mouse and keyboard', $
'=======================================', $
'', $
'The object view can be controlled by clicking a mouse button (and keeping it', $
'pressed) and moving the mouse:', $
'The left mouse button allows the rotation of the object', $
'The center mouse button allows zooming in or out by moving the mouse up or', $
'down, respectively.', $
'The right mouse button allows panning: the object can be moved around.', $
'', $
'Several keyboard commands modify this behaviour (the key has to be pressed', $
'once, the last key is remembered):', $
'', $
'0: Recenter the object (undo panning)', $
'1: Set the magnification to 1 (undo scaling)', $
'2: Use unity rotation matrix (undo rotation)', $
'f: A center-mouse move will not scale the object but move the far view plane', $
'm: Recreate the control window (if it has been closed)', $
'n: A center-mouse move will not scale the object but move the near view plane', $
'p: Toggle projection: perspective <--> orthogonal', $
'w: Unconstrained rotation (the default)', $
'x: Contrained rotation: x axis (view)', $
'X: Contrained rotation: x axis (model)', $
'y: Contrained rotation: y axis (view)', $
'Y: Contrained rotation: y axis (model)', $
'z: Contrained rotation: z axis (view)', $
'Z: Contrained rotation: z axis (model)']

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0] + ' Help', $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_menu_displayhelpdraw


pro v3d_menu_displayhelpspeed, group=group
;-------------------------------------------------------------------------------
; NAME:
;   v3d_menu_displayhelpspeed ('v3d_menu_display_help_speed')
;
; PURPOSE:
;   Display help text with tips to speed up vis3D.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_menu_displayhelpspeed, group=group
;
; KEYWORDS:
;   group:  ('group') Widget ID of group leader whose death kills help widget.
;
; MODIFICATION HISTORY:
;   2006-06-16 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  text=[$
'Speed improvements for Vis3D', $
'============================', $
'', $
'The visualization of 3D datasets greatly benefits from a fast CPU and a fast', $
'graphics card: the more the better.', $
'', $
'Hardware & drivers:', $
'On a given machine make sure that hardware accelerated 3D graphics rendering', $
'is supported by the operating system, the OpenGL drivers, and IDL. This', $
'acceleration greatly improves the speed of the rendering of iso-surfaces', $
'(polygons) and particles. It is of less (or no) importance for volume rendering,', $
'yet.', $
'', $
'Data preparation:', $
'Smaller (rebinned) dataset or subsets of them can be displayed faster.  ;-)', $
'The restriction to only one display panel (or the hiding of the objects in the', $
'other panels) speeds up rendering in the remaining panel.', $
'Smaller draw panels improve the render speed in some situations.', $
'', $
'Choice of visualization tool:', $
'The restriction to simple 2D slices results in the fastest rendering.', $
'The speed of rendering of polygons (for iso-surfaces or "dots") very much', $
'depends on the power of the graphics card (and on the size of the datasets)', $
'and lie between quasi-instantaneous or very sluggish.', $
'Volume rendering currently is not supported (i.e. accelerated) by graphics', $
'cards. It requires lots of operations and relies on the relatively slow CPU.', $
'The rendering of larger volumes can be VERY slow.', $
'', $
'Speedup by parameter optimization:', $
'Some tools in Vis3D (iso-surfaces and volume rendering) allow the choice of', $
'different rendering parameters during the interactive display (left mouse', $
'button pressed) and the static display (no mouse button pressed).', $
'Display quality is traded in for speed. After releasing the mouse button the', $
'display will flip (after some time) from the low- to the high-quality display.', $
'A careful tuning of these parameters allows the fast display of datasets with', $
'sizes of more than demo character.', $
'Iso-surfaces:', $
'During the interactive display a decimated (coarser) mesh can be used. By', $
'balancing speed and accuracy even large datasets can be visualized.', $
'In addition, the rendering style (filled polygons, lines, dots) and the', $
'interpolation method can be varied.', $
'Volume rendering:', $
'In the interactive display mode the number of rays, the number of steps on each', $
'ray, the interpolation type etc. can be reduced.' $
]

  xdisplayfile, '', $
                TEXT=text, $
                GROUP=group, $
                TITLE=text[0], $
                WIDTH=80, $
                HEIGHT=n_elements(text)+1, $
                DONE_BUTTON='OK'

end ; v3d_menu_displayhelpspeed


pro v3d_makemovie, sState
;-------------------------------------------------------------------------------
; NAME:
;   v3d_makemovie ('v3d_make_movie')
;
; PURPOSE:
;   Make movie from the data and control parameters in sState.
;
; CATEGORY:
;   3D Visualization
;
; INPUT:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;   2005-08-27 (B.F.) First version
;   2005-08-29 (B.F.) Put into separate routine
;   2006-06-16 (B.F.) smooth2d -> v3d_smooth2d
;   2010-03-22 (B.F.) Write optionally JPG files
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  index           =(*sState)[0].movie_index
  rotateaxis      =float((*sState)[0].movie_rotateaxis eq indgen(3))
  BufferDimensions=(*sState)[0].movie_antialiasfactor * (*sState)[0].movie_dimensions
  ;
message, 'Skip creation of MPG file',    /INFORMATIONAL
  ;
  ; Create the MPEG object
;  oMPEG=OBJ_NEW('IDLgrMPEG', $
;                FILENAME  =(*sState)[0].movie_filename, $
;                DIMENSIONS=(*sState)[0].movie_dimensions, $
;                QUALITY   =(*sState)[0].movie_quality, $
;                FORMAT    =(*sState)[0].movie_format, $
;                FRAME_RATE=(*sState)[0].movie_framerate)
  ;
;  oMPEG->GetProperty, BITRATE=bitrate
;  oMPEG->GetProperty, IFRAME_GAP=iframe_gap
;  message, 'Bit rate:   ' + string(bitrate),    /INFORMATIONAL
;  message, 'Iframe gap: ' + string(iframe_gap), /INFORMATIONAL
  ;
  ; Create an off-screen buffer to draw into (allows other resolutions to be used)
  oBuffer=OBJ_NEW('IDLgrBuffer', $
                  COLOR_MODEL=0, $
                  DIMENSIONS =BufferDimensions)
  ;
  ; Save original transformation matrix
  (*sState)[index].oModel->GetProperty, TRANSFORM=transformation_matrix
  ;
  if ((*sState)[0].movie_antialiasfactor gt 1) then begin
    (*sState)[index].oPolyhigh->GetProperty, THICK=polyhigh_thick
    (*sState)[index].oPoly    ->GetProperty, THICK=poly_thick
    (*sState)[index].oBox     ->GetProperty, THICK=box_thick
    thick=round((*sState)[0].movie_antialiasfactor * 1.501)
    (*sState)[index].oPolyhigh->SetProperty, $
                                THICK=thick
    (*sState)[index].oPoly    ->SetProperty, $
                                THICK=thick
    (*sState)[index].oBox     ->SetProperty, $
                                THICK=thick
  endif
  ;
  ; Loop over all frames
  for ii=0, (*sState)[0].movie_nframes-1 do begin
    ;
    message, 'Putting image ' + string(ii, format='(I5)') + ' into MPEG buffer.', $
             /INFORMATIONAL
    ;
    ; Compute rotation angle depending on mode and frame number
    if ((*sState)[0].movie_mode eq 0) then begin
      rotateangle=float(ii)/float((*sState)[0].movie_nframes)*360.0
    endif else if ((*sState)[0].movie_mode eq 1) then begin
      rotateangle=(*sState)[0].movie_oscillationangle * $
                  sin(float(ii)/float((*sState)[0].movie_nframes)*2.0*!PI)
    endif else begin
      rotateangle=(*sState)[0].movie_oscillationangle * $
                  cos(float(ii)/float((*sState)[0].movie_nframes)*2.0*!PI)
    endelse
    ;
    ; Rotate model
    if ((*sState)[0].movie_transsystem eq 0) then begin
      ; Rotate model: use view as reference system
      (*sState)[index].oModel->Translate, -(*sState)[index].center[0], $
                                          -(*sState)[index].center[1], $
                                          -(*sState)[index].center[2]
      (*sState)[index].oModel->Rotate,    rotateaxis, rotateangle
      (*sState)[index].oModel->Translate,  (*sState)[index].center[0], $
                                           (*sState)[index].center[1], $
                                           (*sState)[index].center[2]
    endif else begin
      ; Rotate model: use model as reference system
      (*sState)[index].oModel->Translate,  (*sState)[index].center[0], $
                                           (*sState)[index].center[1], $
                                           (*sState)[index].center[2], $
                                           /PREMULTIPLY
      (*sState)[index].oModel->Rotate,    rotateaxis, rotateangle, $
                                           /PREMULTIPLY
      (*sState)[index].oModel->Translate, -(*sState)[index].center[0], $
                                          -(*sState)[index].center[1], $
                                          -(*sState)[index].center[2], $
                                           /PREMULTIPLY
    endelse
    ;
    ; Create an IDLgrImage image of the 3D view
    oBuffer->Draw, (*sState)[index].oView
    oImage=oBuffer->Read()
    ;
    if ((*sState)[0].movie_antialiasfactor gt 1) then begin
      oImage->GetProperty, DATA=image
      ;
      ; Smooth
;      q_smooth=(*sState)[0].movie_antialiasfactor/2
      q_smooth=(*sState)[0].movie_antialiasfactor^2/4
      image[0, *, *]=byte(v3d_smooth2d(float(image[0, *, *]), q_smooth=q_smooth))
      image[1, *, *]=byte(v3d_smooth2d(float(image[1, *, *]), q_smooth=q_smooth))
      image[2, *, *]=byte(v3d_smooth2d(float(image[2, *, *]), q_smooth=q_smooth))
      ;
      ; Rebin the image to get the appropriate smaller size
      image=rebin(image, 3, (*sState)[0].movie_dimensions[0], $
                            (*sState)[0].movie_dimensions[1] )
      ;
      oImage->SetProperty, DATA=image
    endif
    ;
    if ((*sState)[0].movie_imageformat gt 0) then begin
      ; --- Write single image into file ---
      oImage->GetProperty, DATA=image
      str_i_frame='0000' + strtrim(string(ii),2)
      str_i_frame=strmid(str_i_frame,strlen(str_i_frame)-5,5)
      image_file=(*sState)[0].movie_filename + '_' + str_i_frame + '.jpg'
      write_jpeg, image_file, image, order=0, quality=(*sState)[0].movie_quality, true=1
    endif
    ;
;    oMPEG->Put, oImage
    ;
    ; Destroy the image object
;    obj_destroy, oImage
    ;
    ; Restore original transformation matrix
    (*sState)[index].oModel->SetProperty, TRANSFORM=transformation_matrix
    ;
  endfor ; ii
  ;
  ; Create the actual MPEG file from buffered frames
;  message, 'Making the movie (have patience!)', /INFORMATIONAL
;  oMPEG->Save
  ;
  ; The buffer isn't needed anymore
  obj_destroy, oBuffer
  ;
  if ((*sState)[0].movie_antialiasfactor gt 1) then begin
    (*sState)[index].oPolyhigh->SetProperty, THICK=polyhigh_thick
    (*sState)[index].oPoly    ->SetProperty, THICK=poly_thick
    (*sState)[index].oBox     ->SetProperty, THICK=box_thick
  endif

end ; v3d_makemovie


pro v3d_drawwindow_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_menu_event ('v3d_menu_event')
;
; PURPOSE:
;   Handle all events that are NOT directly related to a dataset.
;
; CATEGORY:
;   3D Visualization
;
; INPUT:
;   sEvent:  ('structure_Event') structure with information about event
;
; PROCEDURE:
;   The main event handler handles all events which are NOT directly related to
;   a dataset. Those events include KILL REQUESTS, Menu events and in general
;   events from all widgets outside the dataset views.
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-13 (B.F.) Small modifications
;   2005-07-16 (B.F.) More comments, 'Export' menu -> 'File' menu; 'Help' menu added
;   2005-08-23 (B.F.) vis3d_event -> v3d_menu_event
;   2005-08-25 (B.F.) Add menu point 'Save object'
;   2005-08-26 (B.F.) Handle alpha_channel for polygons
;   2005-08-29 (B.F.) Split from v3d_menu_event
;-------------------------------------------------------------------------------

  compile_opt hidden

  @v3d_parameters

  widget_control, sEvent.top, GET_UVALUE=sState
  widget_control, sEvent.id , GET_UVALUE=uval

  index=-1 & strindex='x'
  if (size(uval, /type) eq 7) then begin
    ; uval is a string
    lastc=strmid(uval, 0, 1, /reverse)
    if ((lastc eq '1') or (lastc eq '2') or (lastc eq '3') or (lastc eq '4')) then begin
      strindex=lastc
      index=fix(strindex)-1
      uval=strmid(uval, 0, strlen(uval)-1)
    endif
  endif

  ;message, /INFORMATIONAL
  ;help,/str,sEvent

  ; Get number of views in use
  nrofviews=N_ELEMENTS(*sState)

  ; Take care of KILL events
  if (TAG_NAMES(sEvent, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST') then begin
    ; Destroy all objects and widgets
    ;
    ; Destroy widgets
;    widget_control, sEvent.top, /DESTROY
    widget_control, (*sState)[0].wBase, /DESTROY
    ;
    ; Destroy heap variables 
    v3d_sstate_freemem, sState
    ;
    return  
  endif

  ; Take care of RESIZE events
  if (TAG_NAMES(sEvent, /STRUCTURE_NAME) eq 'WIDGET_BASE') then begin
    ; Get current main window size
    geometry=widget_info((*sState)[0].wBase, /GEOMETRY)
    ;help, /str, geometry
    wxsize=long(geometry.xsize)
    wysize=long(geometry.ysize)
    ; Get menu bar height
    ;if ((*sState)[0].wMenu ne 0L) then begin
    ;  geometry=widget_info((*sState)[0].wMenu, /GEOMETRY)
    ;  ; The menu ysize does not cover the total extent: add an empirical offset
    ;  mysize=long(geometry.ysize)+12L
    ;endif else begin
      mysize=0L
    ;endelse
    ; Get current draw window size
    xsize=(*sState)[0].viewxsize
    ysize=(*sState)[0].viewysize
    ;
    ; print, sEvent.x, sEvent.y, wxsize, wysize, mysize
    ;help, /str, sEvent
    ;
    ; Compute new draw window size. The height of the menu bar is an estimate.
    (*sState)[*].viewxsize=v3dpar_xsize_min > $
                           (xsize + (sEvent.x-wxsize)/nrofviews) < $
                           v3dpar_xsize_max
    (*sState)[*].viewysize=v3dpar_ysize_min > $
;                           (ysize + (sEvent.y-wysize) - (1-(*sState)[0].no_menu)*mysize) < $
                           (ysize + (sEvent.y-wysize))  < $
                           v3dpar_ysize_max
    ;
    message, 'Resizing draw windows: ' + $
               strtrim(string(xsize),2) + ' -> ' + strtrim(string((*sState)[0].viewxsize),2) + ' , ' + $
               strtrim(string(ysize),2) + ' -> ' + strtrim(string((*sState)[0].viewysize),2), $
             /INFORMATIONAL
    ;
    for i=0, nrofviews-1 do begin
      ; Adjust size of draw widget
      widget_control, (*sState)[i].wDraw, $
                      XSIZE=(*sState)[i].viewxsize, $
                      YSIZE=(*sState)[i].viewysize
      if ((*sState)[i].no_colorbar eq 0) then begin
        ; Adjust size of colorbar widget
        widget_control, (*sState)[i].wColorbar, $
                        XSIZE=v3dpar_colorbar_WindowWidth, $
                        YSIZE=(*sState)[i].viewysize
      endif
      ; Adjust transformation aspectratio
      v3d_getviewvolume, (*sState)[i].box, (*sState)[i].oView, /ONLYVIEWPLANE, $
                         SCALE=(*sState)[i].scale, $
                         XSIZE=(*sState)[i].viewxsize, $
                         YSIZE=(*sState)[i].viewysize
      ; Adjust trackball center
      trackCenterX=round((*sState)[i].viewxsize/2.)
      trackCenterY=round((*sState)[i].viewysize/2.)
      trackRadius =sqrt(trackcenterX^2+trackcenterY^2)
      (*sState)[i].oTrack->Reset, [trackCenterX, trackCenterY], trackRadius
    endfor ; i
    ;
    return
  endif

end ; v3d_drawwindow_event


pro v3d_menu_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_menu_event ('v3d_menu_event')
;
; PURPOSE:
;   Handle all main menu events events.
;
; CATEGORY:
;   3D Visualization
;
; INPUT:
;   sEvent:  ('structure_Event') structure with information about event
;
; PROCEDURE:
;   The main event handler handles all events which are NOT directly related to
;   a dataset. Those events include KILL REQUESTS, Menu events and in general
;   events from all widgets outside the dataset views.
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-13 (B.F.) Small modifications
;   2005-07-16 (B.F.) More comments, 'Export' menu -> 'File' menu; 'Help' menu added
;   2005-08-23 (B.F.) vis3d_event -> v3d_menu_event
;   2005-08-25 (B.F.) Add menu point 'Save object'
;   2005-08-26 (B.F.) Handle alpha_channel for polygons
;   2005-09-08 (B.F.) Allow export of sState to command line level via scope_varfetch
;-------------------------------------------------------------------------------

  compile_opt hidden

  @v3d_parameters

  widget_control, sEvent.top, GET_UVALUE=sState
  widget_control, sEvent.id , GET_UVALUE=uval

  index=-1 & strindex='x'
  if (size(uval, /type) eq 7) then begin
    ; uval is a string
    lastc=strmid(uval, 0, 1, /reverse)
    if ((lastc eq '1') or (lastc eq '2') or (lastc eq '3') or (lastc eq '4')) then begin
      strindex=lastc
      index=fix(strindex)-1
      uval=strmid(uval, 0, strlen(uval)-1)
    endif
  endif

  ;message, /INFORMATIONAL
  ;help,/str,sEvent

  ; Get number of views in use
  nrofviews=N_ELEMENTS(*sState)

  ; Take care of MENU events
  case uval of

    ; ==========================================================================

    'MENU_FILE_EXPORT2D': begin ; ----------------------------------------------
      ; The Export to 2D image format Menu item
      ;
      ; Create buttons for each view in dialog 
      n_str='1' ; It's safe to assume at least one view is present
      for i=2, nrofviews do begin
        n_str = n_str + '|' + string(i, FORMAT='(I0)')
      endfor
      if (nrofviews ge 2) then n_str=n_str + ', SET_VALUE=0'
      ;
      ; Get current resolution of view to put something
      ; initially into the resolution input fields.
      ; I use (*sState)[0] since at least one view has to be present. 
      width =string((*sState)[0].viewXsize, FORMAT='(I0)')
      height=string((*sState)[0].viewYsize, FORMAT='(I0)')
      ;
      ; The dialog box to get necessary parameters for the exporting
      export_dialog = [ $
                        '0, LABEL, Export view nr.: , LEFT', $
                         '1, BASE, , COLUMN, FRAME', $
                         '2, BUTTON,' + n_str + ', EXCLUSIVE, ROW, TAG=view', $
                       ; '1, BASE,, COLUMN', $
                       ; '2, BUTTON, Include colorbar?|..., TAG=options', $
                         '1, BASE, , COLUMN', $
                         '0, LABEL, Resolution:, LEFT', $
                         '0, LABEL, (max. 1600x1200), LEFT',$
                          '1, BASE, , ROW', $
                          '0, INTEGER,' + width +', WIDTH=4, TAG=width', $
                          '0, LABEL, x ', $
                          '2, INTEGER,' + height +', WIDTH=4, TAG=height', $
                          '1, BASE, , ROW', $
                          '0, BUTTON, Export, QUIT, TAG=ok', $
                          '2, BUTTON, Cancel, QUIT, TAG=Cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expres = CW_FORM(export_dialog, /COLUMN, TITLE='Export 2D')
      ;
      if (expres.ok eq 1) then begin
        ;
        ; Check if the dimensions are sensible
        if ( (expres.width le 1600) AND (expres.height le 1200) AND $
            (expres.width gt 0) AND (expres.height gt 0)) then begin 
          ;
          ; Create a off-screen buffer to draw into (allows other resolutions to be used)
          oBuffer=OBJ_NEW('IDLgrBuffer', $
                          COLOR_MODEL=0, $
                          DIMENSIONS=[expres.width, expres.height])
          ;
          oBuffer->Draw, (*sState)[expres.view].oView
          ;
          ; Create an IDLgrImage image of the 3D view
          oImage = oBuffer->Read()
          oImage->SetProperty, INTERLEAVE=0
          oImage->GetProperty, DATA=image
          ;
          ; The objects aren't needed anymore
          obj_destroy, oImage
          obj_destroy, oBuffer
          ;
          ; A dialog which saves images and converts them.
          ; The returned value is actually not needed, but IDL complains if it isn't included.
          saveresults=dialog_write_image(image, $
                                         DIALOG_PARENT=sEvent.top, $
                                         FILE=v3dpar_WriteImageFileName_default, $
                                         PATH=v3dpar_ExportPath_default, $
                                         TYPE=v3dpar_ExportFileType_default, $
                                         TITLE='Export to file', $
                                         /WARN_EXIST)
          ;
        endif else begin ;endif of dimension checking 
          message, 'ERROR: Resolution is outside allowed boundaries.', /CONTINUE
        endelse
        ;
      endif ; End of 'if (expres.ok eq 1)' 
      ;
    end ; 'MENU_FILE_EXPORT2D'

    'MENU_FILE_EXPORTANIM': begin ; --------------------------------------------
      ; Animation Menu item
      ;
      ; Get current resolution of view to put something
      ; initially into the resolution input fields.
      strwidth           =string((*sState)[0].viewXsize, FORMAT='(I0)')
      strheight          =string((*sState)[0].viewYsize, FORMAT='(I0)')
      ;
      strindex           =string((*sState)[0].movie_index           , FORMAT='(I0)')
      strnframes         =string((*sState)[0].movie_nframes         , FORMAT='(I0)')
      strmode            =string((*sState)[0].movie_mode            , FORMAT='(I0)')
      stroscillationangle=string((*sState)[0].movie_oscillationangle, FORMAT='(G0)')
      strrotateaxis      =string((*sState)[0].movie_rotateaxis      , FORMAT='(I0)')
      strtranssystem     =string((*sState)[0].movie_transsystem     , FORMAT='(I0)')
      strframerate       =string((*sState)[0].movie_framerate-1     , FORMAT='(I0)')
      strquality         =string((*sState)[0].movie_quality         , FORMAT='(I0)')
      strantialiasfactor =string((*sState)[0].movie_antialiasfactor , FORMAT='(I0)')
      strformat          =string((*sState)[0].movie_format          , FORMAT='(I0)')
      strimageformat     =string((*sState)[0].movie_imageformat     , FORMAT='(I0)')
      filename           =       (*sState)[0].movie_filename
      ;
      ; Create buttons for each view in dialog 
      n_str='1' ; It's safe to assume at least one view is present
      for i=2, nrofviews do begin
        n_str = n_str + '|' + string(i, FORMAT='(I0)')
      endfor
      if (nrofviews ge 2) then n_str=n_str + ', SET_VALUE=' + strindex
      ;
      ; The dialog box to get necessary parameters for the movie
      export_dialog = [ $
        '0, LABEL, Export as MPEG animation , LEFT', $
        '1, BASE, , COLUMN, FRAME', $
         '1, BASE, , ROW', $
          '0, LABEL, Use view  : , LEFT', $
          '2, BUTTON,' + n_str + ', EXCLUSIVE, ROW, TAG=index', $
       ; '1, BASE,, COLUMN', $
       ; '2, BUTTON, Include colorbar?|..., TAG=options', $
         '1, BASE, , ROW', $
          '0, LABEL, Resolution: , LEFT',$
          '0, INTEGER,' + strwidth +', WIDTH=4, TAG=width', $
          '0, LABEL, x ', $
          '0, INTEGER,' + strheight +', WIDTH=4, TAG=height', $
          '2, LABEL, (max. 1600x1200), LEFT', $
         '1, BASE, , ROW', $
          '0, LABEL, Frames    :, LEFT', $
          '2, INTEGER, ' + strnframes + ', WIDTH=6, TAG=NFRAMES', $
         '1, BASE, , ROW', $
          '0, LABEL, Mode      : , LEFT', $
          '2, BUTTON, Rotation|Oscillation (sin)|Oscillation (cos), EXCLUSIVE, ROW, ' + $
                                   'SET_VALUE=' + strmode + ' , TAG=mode', $
         '1, BASE, , ROW', $
          '0, LABEL, Angle     : , LEFT', $
          '2, FLOAT, ' + stroscillationangle + ', WIDTH=4, TAG=oscillationangle', $
         '1, BASE, , ROW', $
          '0, LABEL, Axis      : , LEFT', $
          '0, BUTTON, x|y|z, EXCLUSIVE, ROW, ' + $
                   'SET_VALUE=' + strrotateaxis + ', TAG=axis', $
          '0, LABEL, ...Reference: , LEFT', $
          '2, BUTTON, view|model, EXCLUSIVE, ROW, ' + $
                                   'SET_VALUE=' + strtranssystem + ', TAG=transsystem', $
         '1, BASE, , ROW', $
          '0, LABEL, Frame rate: , LEFT', $
          '2, BUTTON, 23.9|24|25|29.9|30|50|59.9|60, EXCLUSIVE, ROW, ' + $
                                   'SET_VALUE=' + strframerate + ', TAG=framerate', $
         '1, BASE, , ROW', $
          '0, LABEL, Quality   : , LEFT', $
          '0, INTEGER, ' + strquality + ', WIDTH=3, TAG=quality', $
          '0, LABEL, %    AntiAlias:, LEFT', $
          '2, INTEGER, ' + strantialiasfactor + ', WIDTH=2, TAG=antialiasfactor', $
         '1, BASE, , ROW', $
          '0, LABEL, Format:, LEFT', $
          '0, BUTTON, MPEG1|MPEG2, EXCLUSIVE, ROW, SET_VALUE=' + strformat + ', TAG=format', $
          '0, LABEL, Image Format:, LEFT', $
          '2, BUTTON, None|JPG, EXCLUSIVE, ROW, SET_VALUE=' + strimageformat + ', TAG=imageformat', $
         '1, BASE, , ROW', $
          '0, LABEL, Filename  :, LEFT', $
          '2, TEXT, ' + filename + ', WIDTH=40, TAG=filename', $
         '2, BASE, ', $
        '1, BASE, , ROW', $
         '0, BUTTON, Make Movie, QUIT, TAG=ok', $
         '2, BUTTON, Cancel    , QUIT, TAG=Cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expres=CW_FORM(export_dialog, /COLUMN, TITLE='Export as Animation')
      ;
      if (expres.ok eq 1) then begin
        ;
        ; Check if the dimensions are sensible
        if ((expres.width gt 1600) OR (expres.height gt 1200)) then begin 
          message, 'Reducing frame format', /INFORMATIONAL
        endif
        ;
        ; Put all parameters into state variable
        (*sState)[*].movie_dimensions      =[16 > expres.width < 1600, 16 > expres.height < 1200]
        (*sState)[*].movie_index           =expres.index
        (*sState)[*].movie_nframes         =1 > expres.nframes
        (*sState)[*].movie_mode            =expres.mode
        (*sState)[*].movie_oscillationangle=(-720.0) > expres.oscillationangle < 720.0
        (*sState)[*].movie_rotateaxis      =expres.axis
        (*sState)[*].movie_transsystem     =expres.transsystem
        (*sState)[*].movie_framerate       =expres.framerate+1
        (*sState)[*].movie_quality         =5 > expres.quality < 100
        (*sState)[*].movie_format          =expres.format
        (*sState)[*].movie_imageformat     =expres.imageformat
        (*sState)[*].movie_antialiasfactor =1 > expres.antialiasfactor < 16
        (*sState)[*].movie_filename        =expres.filename
        ;
        ; Make the movie
        v3d_makemovie, sState
        ;
      endif ; End of 'if (expres.ok eq 1)' 
      ;
    end ; 'MENU_FILE_EXPORTANIM'

    'MENU_FILE_EXPORTVRML': begin ; --------------------------------------------
      ; The Export to 2D image format Menu item
      ;
      ; Create buttons for each view in dialog 
      n_str='1' ; Its safe to assume at least one view is present
      for i=2, nrofviews do begin
        n_str = n_str + '|' + string(i, FORMAT='(I0)')
      endfor ; i
      if (nrofviews ge 2) then n_str=n_str + ', SET_VALUE=0'
      ;
      ; The dialog box to get necessary parameters for the exporting
      export_dialog = [ $
                        '0, LABEL, Export view nr.: , LEFT', $
                        '1, BASE, , COLUMN, FRAME', $
                         '2, BUTTON,' + n_str + ', EXCLUSIVE, ROW, TAG=view', $
                        '1, BASE,, COLUMN', $
                         '2, BUTTON, Decimated model?|..., TAG=options', $
                        '1, BASE, , ROW', $
                         '0, LABEL, Path+Filename:, LEFT', $
                         '2, TEXT, obj.wrl, WIDTH=20, TAG=filename', $
                        '1, BASE, , ROW', $
                         '0, BUTTON, Export, QUIT, TAG=ok', $
                         '2, BUTTON, Cancel, QUIT, TAG=Cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expvrml = CW_FORM(export_dialog, /COLUMN, TITLE='Export VRML')  
      ;
      ; If cancel pressed then skip the rest
      if (expvrml.ok eq 1) then begin 
        ;
        ; Check if any filename was given. 
        ; TODO: More checks, is filename valid...
        if (expvrml.filename ne '') then begin 
          ;
          ; The VRML object to export to.
          oVRML=OBJ_NEW('IDLgrVRML', $
                        COLOR_MODEL=0, $
                        DIMENSIONS=[(*sState)[expvrml.view].viewxsize, $
                                    (*sState)[expvrml.view].viewysize], $
                        FILENAME=expvrml.filename, $
                        WORLDINFO='Exported from Vis3D', $
                        WORLDTITLE='Star Simulation Data')
          ;
          ; Save hide state of the polygon objects
          (*sState)[expvrml.view].oPolyhigh->GetProperty, HIDE=polyhigh_hide
          (*sState)[expvrml.view].oPoly    ->GetProperty, HIDE=polylow_hide
          ;
          if (expvrml.options[0] eq 0) then begin
            ; The high-resolution mesh should be exported, make sure 
            ; it is unhidden and hide the low-resolution mesh.
            (*sState)[expvrml.view].oPoly    ->SetProperty, HIDE=1
            (*sState)[expvrml.view].oPolyhigh->SetProperty, HIDE=0
          endif else begin
            ; And vice versa, hide the high resolution mesh and
            ; show the low resolution mesh
            (*sState)[expvrml.view].oPoly    ->SetProperty, HIDE=0
            (*sState)[expvrml.view].oPolyhigh->SetProperty, HIDE=1
          endelse
          ;
          oVRML->Draw, (*sState)[expvrml.view].oView
          ;
          ; Destroy the VRML object and make it save the VRML file
          OBJ_DESTROY, oVRML 
          ;
          ; Restore hide state of the meshes objects
          (*sState)[expvrml.view].oPolyhigh->SetProperty, HIDE=polyhigh_hide
          (*sState)[expvrml.view].oPoly    ->SetProperty, HIDE=polylow_hide
          ;
        endif else begin
          message, 'ERROR: Export to VRML failed, no filename was given.', /CONTINUE
        endelse
        ;
      endif ; End of 'if (expres.ok eq 1)' 
      ;
    end ; 'MENU_FILE_EXPORTVRML'

    'MENU_FILE_SAVEOBJ': begin ; -----------------------------------------------
      ; Save object into file
      ;
      ; Create buttons for each view in dialog 
      n_str='1' ; Its safe to assume at least one view is present
      for i=2, nrofviews do begin
        n_str = n_str + '|' + string(i, FORMAT='(I0)')
      endfor ; i
      if (nrofviews ge 2) then n_str=n_str + ', SET_VALUE=0'
      ;
      ; The dialog box to get necessary parameters for the exporting
      export_dialog = [ $
                        '0, LABEL, Save object nr.: , LEFT', $
                        '1, BASE, , COLUMN, FRAME', $
                        '2, BUTTON,' + n_str + ', EXCLUSIVE, ROW, TAG=view', $
                        '1, BASE,, COLUMN', $
;                        '2, BUTTON, Decimated model?|..., TAG=options', $
;                        '1, BASE, , COLUMN', $
                        '2, LABEL, Path+Filename:, LEFT', $
                        '1, BASE, , ROW', $
                        '2, TEXT, vis3d_object.sav, WIDTH=30, TAG=filename', $
                        '1, BASE, , ROW', $
                        '0, BUTTON, Save  , QUIT, TAG=ok', $
                        '2, BUTTON, Cancel, QUIT, TAG=Cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expres = CW_FORM(export_dialog, /COLUMN, TITLE='Save object')  
      ;
      ; If cancel pressed then skip the rest
      if (expres.ok eq 1) then begin 
        ;
        ; Check if any filename was given. 
        ; TODO: More checks, is filename valid...
        if (expres.filename ne '') then begin 
          ;
          index=expres.view
          ;
;          oPolyHigh=(*sState)[index].oPolyHigh
;          iscontained=(*sState)[index].oModel->IsContained(oPolyHigh, POSITION=position)
;          (*sState)[index].oModel->Remove, oPolyHigh
;          save, FILENAME=expres.filename, oPolyHigh, /VERBOSE
;          (*sState)[index].oModel->Add, oPolyHigh, POSITION=position
          ;
          oModel=(*sState)[index].oModel
          oModel->GetProperty, TRANSFORM=rotatematrix
          oModel->SetProperty, TRANSFORM=diag_matrix(replicate(1.0, 4))
          iscontained=(*sState)[index].oWorld->IsContained(oModel, POSITION=position)
          (*sState)[index].oWorld->Remove, oModel
          save, FILENAME=expres.filename, oModel, /VERBOSE
          oModel->SetProperty, TRANSFORM=rotatematrix
          (*sState)[index].oWorld->Add, oModel, POSITION=position
          ;
        endif else begin
          message, 'ERROR: Save to file failed, no filename was given.', /CONTINUE
        endelse
        ;
      endif ; End of 'if (expres.ok eq 1)' 
      ;
    end ; 'MENU_FILE_SAVEOBJ'

    'MENU_FILE_PRINTROTMATRIX': begin ; ----------------------------------------
      (*sState)[0].oModel->GetProperty, TRANSFORM=tMatrix
      print, 'The transformation matrix:' 
      ;
      for j=0,3 do begin
        if (j eq 0) then mStr='M=[[' else mStr='   ['
        for i=0,3 do begin
          mStr=mStr + string(tMatrix[i,j], FORMAT='(F12.5)')
          if (i ne 3) then mStr=mStr + ','
        endfor
        if (j eq 3) then mStr=mStr + ']]' else mStr=mStr + '],$'
        print, mStr
      endfor
      ;
    end ; 'MENU_FILE_PRINTROTMATRIX'

    'MENU_FILE_QUIT': begin ; --------------------------------------------------
      ; Destroy all objects and widgets
      ;
      ; Destroy widgets
;      WIDGET_CONTROL, sEvent.top, /DESTROY 
      widget_control, (*sState)[0].wBase, /DESTROY
      ;
      ; Destroy heap variables 
      v3d_sstate_freemem, sState
      ;
      return
      ;
    end ; 'MENU_FILE_QUIT'

    ; ==========================================================================

    'MENU_SETTINGS_VIEWALL': begin ; -------------------------------------------
      ;
      (*sState)[0].oView->GetProperty, PROJECTION=projection
      strprojection=string(projection-1, format='(I0)')
      ;
      (*sState)[0].oModel->GetProperty, TRANSFORM=rotatematrix
      mshiftmat       =diag_matrix(replicate(1.0, 4))
      pshiftmat       =mshiftmat
      mshiftmat(3,0:2)=-(*sState)[0].center
      pshiftmat(3,0:2)= (*sState)[0].center
      centermatrix=pshiftmat#rotatematrix#mshiftmat
      strZoom=string(1.0/centermatrix(3,3), format='(F12.7)')
      ;
;      strrenderer=string((*sState)[0].renderer, format='(I0)')
      ;
      strxsize=string((*sState)[0].viewxsize, format='(I0)')
      strysize=string((*sState)[0].viewysize, format='(I0)')
      ;
      export_dialog=[ $
        '0, LABEL, Settings for all views, LEFT', $
        '1, BASE, , COLUMN, FRAME', $
         '1, BASE, , ROW', $
          '0, INTEGER, ' + strxsize + ', TAG=xsize, WIDTH=5, LABEL_LEFT=View   xsize ', $
          '2, INTEGER, ' + strysize + ', TAG=ysize, WIDTH=5, LABEL_LEFT=ysize ', $
;         '0, BUTTON, hardware|software, EXCLUSIVE, ROW, ' + $
;                     'LABEL_LEFT=Renderer, ' + $
;                     'SET_VALUE=' + strrenderer + ', TAG=renderer', $
         '1, BASE, , ROW', $
          '0, LABEL, Rotate, LEFT', $
          '2, BUTTON, Reset , TAG=resetrotate', $
         '1, BASE, , ROW', $
          '0, LABEL, Zoom.., LEFT', $
          '0, BUTTON, Reset , TAG=resetzoom', $
          '2, FLOAT, ' + strZoom + ', TAG=zoomfactor, WIDTH=12, LABEL_LEFT=factor', $
         '1, BASE, , ROW', $
          '0, LABEL, Pan..., LEFT', $
          '2, BUTTON, Reset , TAG=resetpan', $
         '2, BUTTON, orthogonal|perspective, EXCLUSIVE, ROW, ' + $
                     'LABEL_LEFT=Projection, ' + $
                     'SET_VALUE=' + strprojection + ', TAG=projection', $
        '1, BASE, , ROW, CENTER', $
         '0, BUTTON, Apply , QUIT, TAG=ok', $
         '2, BUTTON, Cancel, QUIT, TAG=cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expset=CW_FORM(export_dialog, /COLUMN, TITLE='Settings for all views')
      ;
      ; If cancel pressed then skip the rest
      if (expset.ok eq 1) then begin
        ;
;        if ((*sState)[0].renderer ne expset.renderer) then begin
;          (*sState)[*].renderer=expset.renderer
;          for i=0, nrofviews-1 do begin
;            (*sState)[i].wDraw->SetProperty, renderer=(*sState)[i].renderer
;          endfor
;        endif
        ;
        ; Change in draw window size?
        if (((*sState)[0].viewxsize ne expset.xsize) or $
            ((*sState)[0].viewysize ne expset.ysize)   ) then begin
          (*sState)[*].viewxsize=v3dpar_xsize_min > expset.xsize < v3dpar_xsize_max
          (*sState)[*].viewysize=v3dpar_ysize_min > expset.ysize < v3dpar_ysize_max
          ;
          for i=0, nrofviews-1 do begin
            ; Adjust size of draw widget
            widget_control, (*sState)[i].wDraw, $
                            XSIZE=(*sState)[i].viewxsize, $
                            YSIZE=(*sState)[i].viewysize
            if ((*sState)[i].no_colorbar eq 0) then begin
              ; Adjust size of colorbar widget
              widget_control, (*sState)[i].wColorbar, $
                              XSIZE=v3dpar_colorbar_WindowWidth, $
                              YSIZE=(*sState)[i].viewysize
            endif
            ; Adjust transformation aspectratio
            v3d_getviewvolume, (*sState)[i].box, (*sState)[i].oView, /ONLYVIEWPLANE, $
                               SCALE=(*sState)[i].scale, $
                               XSIZE=(*sState)[i].viewxsize, $
                               YSIZE=(*sState)[i].viewysize
            ; Adjust trackball center
            trackCenterX=round((*sState)[i].viewxsize/2.)
            trackCenterY=round((*sState)[i].viewysize/2.)
            trackRadius=sqrt(trackcenterX^2+trackcenterY^2)
            (*sState)[i].oTrack->Reset, [trackCenterX, trackCenterY], trackRadius
          endfor ; i
        endif
        ;
        ZoomFactor=expset.zoomfactor
        if (expset.resetzoom  ) then ZoomFactor=1.0
        centermatrix(3,3)=1.0/ZoomFactor
        if (expset.resetpan   ) then centermatrix(3  ,0:2)=0.0
        if (expset.resetrotate) then centermatrix(0:2,0:2)=diag_matrix(replicate(1.0, 3))
        rotatematrix=mshiftmat#centermatrix#pshiftmat
        for i=0, nrofviews-1 do begin
          (*sState)[i].oModel->SetProperty, TRANSFORM=rotatematrix
          ;
          (*sState)[i].oView->SetProperty, PROJECTION=expset.projection+1
          ;
          (*sState)[i].oWindow->Draw, (*sState)[i].oView
        endfor ; i
        ;
      endif
      ;
    end ; 'MENU_SETTINGS_VIEWALL'

    'MENU_SETTINGS_VIEW': begin ; ----------------------------------------------
      ;
;      (*sState)[index].oView->GetProperty, COLOR=view_color
;      strview_red  =string(view_color[0], format='(I0)')
;      strview_green=string(view_color[1], format='(I0)')
;      strview_blue =string(view_color[2], format='(I0)')
      ;
      (*sState)[index].oPolyhigh->GetProperty, STYLE=polyhigh_style
      (*sState)[index].oPoly    ->GetProperty, STYLE=poly_style
      strpolyhigh_style=string(polyhigh_style, format='(I0)')
      strpoly_style    =string(poly_style    , format='(I0)')
      ;
      (*sState)[index].oPolyhigh->GetProperty, SHADING=polyhigh_shading
      (*sState)[index].oPoly    ->GetProperty, SHADING=poly_shading
      strpolyhigh_shading=string(polyhigh_shading, format='(I0)')
      strpoly_shading    =string(poly_shading    , format='(I0)')
      ;
      (*sState)[index].oPolyhigh->GetProperty, THICK=polyhigh_thick
      (*sState)[index].oPoly    ->GetProperty, THICK=poly_thick
      strpolyhigh_thick=string(polyhigh_thick, format='(I0)')
      strpoly_thick    =string(poly_thick    , format='(I0)')
      ;
      (*sState)[index].oPolyhigh->GetProperty, ALPHA_CHANNEL=alphachannel
      str_alphachannel=string(alphachannel, format='(G0)')
      ;
      export_dialog=[ $
        '0, LABEL, Settings view ' + strindex + ', LEFT', $
;        '1, BASE, , ROW, FRAME', $
;         '0, INTEGER,' + strview_red   + $
;                     ', WIDTH=3, LABEL_LEFT=Background color: RED:, TAG=view_red', $
;         '0, INTEGER,' + strview_green + $
;                     ', WIDTH=3, LABEL_LEFT=GREEN:                , TAG=view_green', $
;         '2, INTEGER,' + strview_blue  + $
;                     ', WIDTH=3, LABEL_LEFT=BLUE:                 , TAG=view_blue', $
        '1, BASE, , COLUMN, FRAME', $
         '0, FLOAT,' + str_alphachannel + $
                     ', WIDTH=12, ' + $
                     'LABEL_LEFT=Polygon opacity        :, TAG=alphachannel', $
         '0, BUTTON, points|lines|filled, EXCLUSIVE, ROW, ' + $
                     'LABEL_LEFT=Polygon Style (HighRes):, ' + $
                     'SET_VALUE=' + strpolyhigh_style + ', TAG=polyhigh_style', $
         '0, BUTTON, points|lines|filled, EXCLUSIVE, ROW, ' + $
                     'LABEL_LEFT=Polygon Style  (LowRes):, ' + $
                     'SET_VALUE=' + strpoly_style + ', TAG=poly_style', $
         '0, BUTTON, coarse|fine, EXCLUSIVE, ROW, ' + $
                     'LABEL_LEFT=Shading       (HighRes):, ' + $
                     'SET_VALUE=' + strpolyhigh_shading + ', TAG=polyhigh_shading', $
;         '0, BUTTON, off|on, EXCLUSIVE, ROW, ' + $
;                     'LABEL_LEFT=Colorbar, ' + $
;                     'SET_VALUE=' + '1' + ', TAG=colorbar', $
         '0, BUTTON, coarse|fine, EXCLUSIVE, ROW, ' + $
                     'LABEL_LEFT=Shading        (LowRes):, ' + $
                     'SET_VALUE=' + strpoly_shading + ', TAG=poly_shading', $
         '0, INTEGER,' + strpolyhigh_thick + $
                     ', WIDTH=2, ' + $
                     'LABEL_LEFT=Polygon thickness (HR) :, TAG=polyhigh_thick', $
         '2, INTEGER,' + strpoly_thick + $
                     ', WIDTH=2, ' + $
                     'LABEL_LEFT=Polygon thickness (LR) :, TAG=poly_thick', $
        '1, BASE, , ROW, CENTER', $
         '0, BUTTON, Apply , QUIT, TAG=ok', $
         '2, BUTTON, Cancel, QUIT, TAG=cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expset=CW_FORM(export_dialog, /COLUMN, TITLE='Settings view ' + strindex)
      ;
      if (expset.ok eq 1) then begin
        ;
;        widget_control, (*sState)[index].wColorbar, map=expset.colorbar
        ;
;        (*sState)[index].oView->SetProperty, $
;          COLOR=byte(0 > [expset.view_red, expset.view_green, expset.view_blue] < 255)
        ;
        alphachannel=0.0 > float(expset.alphachannel) < 1.0
        (*sState)[index].oPolyhigh->SetProperty, ALPHA_CHANNEL=alphachannel
        (*sState)[index].oPoly    ->SetProperty, ALPHA_CHANNEL=alphachannel
        if (alphachannel le 0.4) then begin
          (*sState)[index].oPolyhigh->SetProperty, DEPTH_TEST_DISABLE=1
          (*sState)[index].oPoly    ->SetProperty, DEPTH_TEST_DISABLE=1
        endif else begin
          (*sState)[index].oPolyhigh->SetProperty, DEPTH_TEST_DISABLE=0
          (*sState)[index].oPoly    ->SetProperty, DEPTH_TEST_DISABLE=0
        endelse
        ;
        (*sState)[index].oPolyhigh->SetProperty, STYLE=expset.polyhigh_style
        (*sState)[index].oPoly    ->SetProperty, STYLE=expset.poly_style
        ;
        (*sState)[index].oPolyhigh->SetProperty, SHADING=expset.polyhigh_shading
        (*sState)[index].oPoly    ->SetProperty, SHADING=expset.poly_shading
        ;
        (*sState)[index].oPolyhigh->SetProperty, THICK=1 > expset.polyhigh_thick < 10
        (*sState)[index].oPoly    ->SetProperty, THICK=1 > expset.poly_thick     < 10
        ;
        (*sState)[index].oWindow->Draw, (*sState)[index].oView
      endif
      ;
    end ; 'MENU_SETTINGS_VIEW'

    'MENU_SETTINGS_LIGHT': begin ; ---------------------------------------------
      ;
      v3d_light_starteditor, sState, index
      ;
    end ; 'MENU_SETTINGS_LIGHT'

    ; ==========================================================================

    'MENU_TOOLS_POLYHIGHTOXOBV': begin ; ---------------------------------------
      ;
      (*sState)[index].oView->GetProperty, COLOR=view_color
      xobjview, (*sState)[index].oPolyHigh, $
                TITLE='HighRes Polygon ' + strindex, $
                GROUP=(*sState)[index].wDraw, $
                BACKGROUND=view_color, $
                RENDERER=(*sState)[index].renderer
      ;
    END ; 'MENU_TOOLS_POLYHIGHTOXOBV'

    'MENU_TOOLS_MODELTOXOBV': begin ; ------------------------------------------
      ;
      (*sState)[index].oView->GetProperty, COLOR=view_color
      xobjview, (*sState)[index].oModel, $
                TITLE='Model ' + strindex, $
                GROUP=(*sState)[index].wDraw, $
                BACKGROUND=view_color, $
                RENDERER=(*sState)[index].renderer
      ;
    END ; 'MENU_TOOLS_MODELTOXOBV'

    'MENU_TOOLS_MODELTOXVOL': begin ; -----------------------------------------
      ;
      data=bytscl(*(*sState)[index].box.isodataset)
      xvolume, data, $
               GROUP=(*sState)[index].wDraw, $
               RENDERER=(*sState)[index].renderer
      ;
    END ; 'MENU_TOOLS_MODELTOXVOL'

    'MENU_TOOLS_MODELTOSLICER': begin ; ----------------------------------------
      ;
      data=ptr_new(*(*sState)[index].box.isodataset)
      slicer3, data, $
               group=(*sState)[index].wDraw
      ;
    END ; 'MENU_TOOLS_MODELTOSLICER'

    'MENU_TOOLS_MODELTOIVOL': begin ; ------------------------------------------
      ;
      data=*(*sState)[index].box.isodataset
      (*sState)[index].oView->GetProperty, COLOR=view_color
      ivolume, data, $
               background_color=view_color
      ;
    END ; 'MENU_TOOLS_MODELTOIVOL'

    'MENU_TOOLS_PUTCOMMON': begin ; --------------------------------------------
      ;
      export_dialog=[ $
        '0, LABEL, Put state variable sState into common block, LEFT', $
        '1, BASE,  , COLUMN, FRAME', $
         '2, BUTTON, v3dcommon1|v3dcommon2, EXCLUSIVE, ROW, ' + $
                       'LABEL_LEFT=Common block name:, SET_VALUE=0, TAG=commonblocknumber', $
        '1, BASE,  , ROW, CENTER', $
         '0, BUTTON, OK    , QUIT, TAG=ok', $
         '2, BUTTON, Cancel, QUIT, TAG=cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expset=CW_FORM(export_dialog, /COLUMN, TITLE='sState variable')
      ;
      if (expset.ok eq 1) then begin
        ;
        v3d_sstate_putcommon, sState, expset.commonblocknumber+1
        ;
      endif
      ;
    end ; 'MENU_TOOLS_PUTCOMMON'

    'MENU_TOOLS_EXPORTTOIDL': begin ; ------------------------------------------
      ;
      export_dialog=[ $
        '0, LABEL, Put state variable sState into variable accessible from command line, LEFT', $
        '1, BASE,  , ROW, FRAME', $
         '0, LABEL, Variable name:, LEFT', $
         '2, TEXT, sState, WIDTH=30, TAG=varname', $
        '1, BASE,  , ROW, CENTER', $
         '0, BUTTON, OK    , QUIT, TAG=ok', $
         '2, BUTTON, Cancel, QUIT, TAG=cancel']
      ;
      ; Create a modal dialog, the results are saved in the returned variable
      expset=CW_FORM(export_dialog, /COLUMN, TITLE='sState variable')
      ;
      if ((expset.ok eq 1) and (expset.varname ne '')) then begin
        ;
        ; Export the state variable to the command line under the name given by expset.varname
        (scope_varfetch(expset.varname, /ENTER, LEVEL=1))=sState
        ;
        message, 'Now try and type "help, ' + expset.varname + '".', /INFORMATIONAL
        ;
      endif
      ;
    end ; 'MENU_TOOLS_EXPORTTOIDL'

    ; ==========================================================================

    'MENU_HELP_ABOUT': begin ; -------------------------------------------------
        ; 'About' entry in help menu
        ;
        ; Use a simple export dialog until I know better.
        export_dialog=[ $
          '1, BASE,, COLUMN', $
           '0, LABEL, .                Vis3D                  .  , LEFT', $
           '0, LABEL, Tool for the visualization of 3D arrays    , LEFT', $
           '0, LABEL,                                            , LEFT', $
           '0, LABEL,                                            , LEFT', $
           '0, LABEL, Written by                                 , LEFT', $
           '0, LABEL, Sigurdur Finnsson and Bernd Freytag        , LEFT', $
           '0, LABEL, Uppsala\, 2002\, ...                       , LEFT', $
           '2, BUTTON, OK, QUIT, TAG=OK']
        ;
        ; Create a modal dialog
        expres = CW_FORM(export_dialog, /COLUMN, TITLE='About Vis3D')  
      end ; 'MENU_HELP_ABOUT'

    'MENU_HELP_DOCLIBRARY': begin ; --------------------------------------------
        ; 'doc_library' entry in help menu
        ;
        doc_library, 'vis3d'
      end ; 'MENU_HELP_DOCLIBRARY'

    'MENU_HELP_INTRO': begin ; -------------------------------------------------
        ; Give general introduction to vis3D
        ;
        v3d_menu_displayhelpintro, group=(*sState)[0].wBaseControl
      end ; 'MENU_HELP_INTRO'

    'MENU_HELP_VIEW': begin ; --------------------------------------------------
        ; Give informations about view control with mouse buttons and keys
        ;
        v3d_menu_displayhelpdraw, group=(*sState)[0].wBaseControl
      end ; 'MENU_HELP_VIEW'

    'MENU_HELP_SPEED': begin ; -------------------------------------------------
        ; Give informations about performance improvements
        ;
        v3d_menu_displayhelpspeed, group=(*sState)[0].wBaseControl
      end ; 'MENU_HELP_SPEED'

    'MENU_HELP_XDL': begin ; ---------------------------------------------------
        ; 'xdl' entry in help menu
        ;
        xdl, 'vis3d', group=sEvent.id
      end ; 'MENU_HELP_XDL'

    else: dummy=1 ; ------------------------------------------------------------

  endcase

end ; v3d_menu_event


pro v3d_createmenuwidget, wMenu, sState
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_createmenuwidget ('v3d_create_menu_widget')
;
; PURPOSE:
;   Create the main menu.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_createmenuwidget(wMenu, sState)
;
; INPUT:
;   wMenu:        ('widget_Menu') parent widget
;   sState:       ('structure_State') The state variable array
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Widget Hierarchy:
;     wBase                   , UVALUE=sState (via xmanager)
;       wMenu
;         wExportMenuRoot
;         wSettingsMenuRoot
;         wHelpMenuRoot
;
; MODIFICATION HISTORY:
;   2005-08-23 (B.F.) Put into separate file
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  nrofviews=n_elements(*sState)

  ; The the file (export) menu
  wFileMenuRoot =widget_button(wMenu, $
                                 VALUE='File', $
                                 /MENU, $
                                 EVENT_PRO='v3d_menu_event', $
                                 UVALUE=sState, $
                                 TOOLTIP='File and export menu')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Export as 2D image',$
                                 UVALUE='MENU_FILE_EXPORT2D')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Export as Animation',$
                                 UVALUE='MENU_FILE_EXPORTANIM')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Export as VRML', $
                                 UVALUE='MENU_FILE_EXPORTVRML')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Save object into file', $
                                 UVALUE='MENU_FILE_SAVEOBJ')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Print Rotation Matrix', $
                                 UVALUE='MENU_FILE_PRINTROTMATRIX')
    wFileMenuEntry=widget_button(wFileMenuRoot, $
                                 VALUE='Quit', $
                                 /SEPARATOR, $
                                 UVALUE='MENU_FILE_QUIT')
  ;
  ; The settings menu
  wSettingsMenuRoot=widget_button(wMenu, $
                                    VALUE='Settings', $
                                    /MENU, $
                                 EVENT_PRO='v3d_menu_event', $
                                 UVALUE=sState, $
                                    TOOLTIP='Change settings here')
    wSettingsMenuEntry=widget_button(wSettingsMenuRoot, $
                                    VALUE='Settings All Views', $
                                    UVALUE='MENU_SETTINGS_VIEWALL')
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wSettingsMenuEntry=widget_button(wSettingsMenuRoot, $
                                       VALUE='Settings View ' + strindex, $
                                       UVALUE='MENU_SETTINGS_VIEW' + strindex)
    endfor ; i
    ;
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wSettingsMenuEntry=widget_button(wSettingsMenuRoot, $
                                       VALUE='Settings Light ' + strindex, $
                                       UVALUE='MENU_SETTINGS_LIGHT' + strindex)
    endfor ; i
  ;
  ; The tools menu
  wToolsMenuRoot=widget_button(wMenu, $
                                 VALUE='Tools', $
                                 /MENU, $
                                 EVENT_PRO='v3d_menu_event', $
                                 UVALUE=sState, $
                                 TOOLTIP='Some Tools')
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                    VALUE='Call XOBJVIEW with HighRes Polygon ' + strindex, $
                                    UVALUE='MENU_TOOLS_POLYHIGHTOXOBV' + strindex)
    endfor ; i
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                    VALUE='Call XOBJVIEW with model ' + strindex, $
                                    UVALUE='MENU_TOOLS_MODELTOXOBV' + strindex)
    endfor ; i
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                    VALUE='Call XVOLUME with iso dataset ' + strindex, $
                                    UVALUE='MENU_TOOLS_MODELTOXVOL' + strindex)
    endfor ; i
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                    VALUE='Call SLICER3 with iso dataset ' + strindex, $
                                    UVALUE='MENU_TOOLS_MODELTOSLICER' + strindex)
    endfor ; i
    for i=0, nrofviews-1 do begin
      strindex=string(i+1, format='(I0)')
      wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                    VALUE='Call iVOLUME with iso dataset ' + strindex, $
                                    UVALUE='MENU_TOOLS_MODELTOIVOL' + strindex)
    endfor ; i
    wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                  VALUE='Put sState into common block', $
                                  /SEPARATOR, $
                                  UVALUE='MENU_TOOLS_PUTCOMMON')
    wToolsMenuEntry=widget_button(wToolsMenuRoot, $
                                  VALUE='Export sState to IDL', $
                                  UVALUE='MENU_TOOLS_EXPORTTOIDL')
  ;
  ; The help menu
  wHelpMenuRoot =widget_button(wMenu, $
                                 VALUE='Help', $
                                 /MENU, $
                                 EVENT_PRO='v3d_menu_event', $
                                 UVALUE=sState, $
                                 TOOLTIP='Help and information')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='About Vis3D',$
                                 UVALUE='MENU_HELP_ABOUT')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='Help: Introduction to Vis3D',$
                                 UVALUE='MENU_HELP_INTRO')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='Help: View control',$
                                 UVALUE='MENU_HELP_VIEW')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='Help: Speed improvements',$
                                 UVALUE='MENU_HELP_SPEED')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='doc_library, "vis3d"',$
                                 UVALUE='MENU_HELP_DOCLIBRARY')
    wHelpMenuEntry=widget_button(wHelpMenuRoot, $
                                 VALUE='xdl, "vis3d"',$
                                 UVALUE='MENU_HELP_XDL')

end ; v3d_createmenuwidget
