;-------------------------------------------------------------------------------
; Vis3D module: Handling of draw widget
;-------------------------------------------------------------------------------
; Contains:
;   v3d_draw_event:            Handle events for dataset draw widgets
;   v3d_draw_createviewwidget: Create the draw widget
;   v3d_draw_createwidget:     Create draw and colorbar widget
;-------------------------------------------------------------------------------


function v3d_draw_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_draw_event ('v3d_draw_event')
;
; PURPOSE:
;   Handle events for dataset draw widgets.
;
; CATEGORY:
;   3D Visualization
;
; INPUT:
;   sEvent:  ('structure_Event') structure with information about event
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-16 (B.F.) More comments
;   2005-08-19 (B.F.) Controls for 'ISO' moved into v3d_select_event
;   2005-08-21 (B.F.) Controls for 'MISCCTRL' moved into v3d_select_event
;   2005-08-22 (B.F.) Controls for 'CHCOLOR' moved into v3d_color_event
;   2005-09-13 (B.F.) Renamed: v3d_dataset_event -> v3d_draw_event
;   2006-05-31 (B.F.) Call 'v3d_updatecontrolwidget, sState' after re-creating the widget
;-------------------------------------------------------------------------------

  compile_opt hidden

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  widget_control, sEvent.id     , GET_UVALUE=uval

  ; Check if event came from wrong place (without giving the sState structure)
  if (size(*sState, /type) ne 8) then return, 1

;  if (TAG_NAMES(sEvent, /STRUCTURE_NAME) eq 'WIDGET_DRAW') then begin
;    print, sEvent.type, sEvent.ch, sEvent.press, sEvent.release, (*sState)[index].rotzoompan
;  endif

  case uval of

    ; Check if this was a DRAW event,
    ; check if the trackball needs updating and update the view.
    'DRAW': begin ; ============================================================

;       oNear=(*sState)[index].oBoxEdges->PickRegion((*sState)[index].oWindow, $
;                                                    (*sState)[index].oView, $
;                                                    [sEvent.X, sEvent.Y])
;       help, oNear, (*sState)[index].oBoxEdgeX
;       if (oNear eq (*sState)[index].oBoxEdgeX) then begin
;         jNear=(*sState)[index].oBoxEdgeX->PickVertex((*sState)[index].oWindow, $
;                                                      (*sState)[index].oView, $
;                                                      [sEvent.X, sEvent.Y])
;         print, 'x', jNear, jNear/4
;       endif else if (oNear eq (*sState)[index].oBoxEdgeY) then begin
;         jNear=(*sState)[index].oBoxEdgeY->PickVertex((*sState)[index].oWindow, $
;                                                      (*sState)[index].oView, $
;                                                      [sEvent.X, sEvent.Y])
;         print, 'Y', jNear, jNear/4
;       endif else begin
;         jNear=(*sState)[index].oBoxEdgeZ->PickVertex((*sState)[index].oWindow, $
;                                                      (*sState)[index].oView, $
;                                                      [sEvent.X, sEvent.Y])
;         print, 'Z', jNear, jNear/4
;       endelse

      if (sEvent.type eq 0) then begin
        ; Trackball updates
        if (sEvent.press eq 1) then (*sState)[*].rotzoompan=0
        if (sEvent.press eq 2) then (*sState)[*].rotzoompan=1
        if (sEvent.press eq 4) then (*sState)[*].rotzoompan=2
      endif
      if ((*sState)[index].rotzoompan eq 0) then begin
        bHaveTransform1=(*sState)[index].oTrack->Update(sEvent, $
                                                        TRANSFORM=qmat1, $
                                                        MOUSE    =1, $
                                                        TRANSLATE=0)
      endif else if ((*sState)[index].rotzoompan eq 1) then begin
        bHaveTransform1=(*sState)[index].oTrack->Update(sEvent, $
                                                        TRANSFORM=qmat1, $
                                                        MOUSE    =2, $
                                                        TRANSLATE=1)
      endif else begin
        bHaveTransform1=(*sState)[index].oTrack->Update(sEvent, $
                                                        TRANSFORM=qmat1, $
                                                        MOUSE    =4, $
                                                        TRANSLATE=1)
      endelse
      if (bHaveTransform1 ne 0) then begin
        ; print, 'xxx ', sEvent.type, sEvent.press, sEvent.release, (*sState)[index].rotzoompan
        ; This is a little messy, but it's faster than using procedures since they
        ; really are terribly slow.
        ;
        ; First rotate the current dataset (where the mouse cursor is). It feels 
        ; more responsive if that one finishes rotating first.
        ;
        ; Construct total transformation matrix,
        ; assume that the centers for all view are identical.
        mshiftmat       =diag_matrix(replicate(1.0, 4))
        pshiftmat       =mshiftmat
        mshiftmat(3,0:2)=-(*sState)[index].center
        pshiftmat(3,0:2)= (*sState)[index].center
        ; Type of transformation (Rotate, Zoom, Pan)
        if ((*sState)[index].rotzoompan eq 0) then begin
          ; Rotate
          qmat=qmat1
        endif else if ((*sState)[index].rotzoompan eq 1) then begin
          ; Zoom
          qmat       =diag_matrix(replicate(1.0, 4))
          if ((*sState)[index].key eq 102) then begin
            (*sState)[index].oView->GetProperty, ZCLIP=zclip
            zclip1=zclip[1]-qmat1(3,1)*float((*sState)[index].box.xrange[1]+ $
                                             (*sState)[index].box.xrange[0] )*0.5
            zclip0=zclip[0] > (zclip1+3.0)
            ;print, 'f: ', zclip[0], ' -> ', zclip0, zclip[1], ' -> ', zclip1
            zclip=[zclip0, zclip1]
            for i=0, n_elements(*sState)-1 do begin
              (*sState)[i].oView->SetProperty, ZCLIP=zclip
            endfor ; i
          endif else if ((*sState)[index].key eq 110) then begin
            (*sState)[index].oView->GetProperty, ZCLIP=zclip
            zclip0=zclip[0]-qmat1(3,1)*float((*sState)[index].box.xrange[1]+ $
                                             (*sState)[index].box.xrange[0] )*0.5
            zclip1=zclip[1] < (zclip0-3.0)
            ;print, 'n: ', zclip[0], ' -> ', zclip0, zclip[1], ' -> ', zclip1
            zclip=[zclip0, zclip1]
            for i=0, n_elements(*sState)-1 do begin
              (*sState)[i].oView->SetProperty, ZCLIP=zclip
            endfor ; i
          endif else begin
            qmat(3,3)  =(qmat(3,3)-qmat1(3,1)*1.0) > 0.1
          endelse
        endif else begin
          ; Pan
          qmat=qmat1
          ;qmat(3,0:2)=qmat(3,0:2)*10.0
          qmat(3,0:2)=qmat(3,0:2)*float((*sState)[index].box.xrange[1]- $
                                        (*sState)[index].box.xrange[0] )*0.25
        endelse
        qmat=mshiftmat#qmat#pshiftmat
        ;
        ; Get and multiply transformation matrices
        (*sState)[index].oModel->GetProperty, TRANSFORM=oModel_t
        if (((*sState)[index].key eq 88) or $
            ((*sState)[index].key eq 89) or $
            ((*sState)[index].key eq 90)) then begin
          (*sState)[index].oModel->SetProperty, TRANSFORM=qmat#oModel_t
        endif else begin
          (*sState)[index].oModel->SetProperty, TRANSFORM=oModel_t#qmat
        endelse
        ;
        (*sState)[index].oWindow->Draw, (*sState)[index].oView
        ;
        ; Now rotate all the other datasets 
        for i=0, n_elements(*sState)-1 do begin
          if (i ne index) then begin
            (*sState)[i].oModel->GetProperty, TRANSFORM=oModel_t
            if (((*sState)[index].key eq 88) or $
                ((*sState)[index].key eq 89) or $
                ((*sState)[index].key eq 90)) then begin
              (*sState)[i].oModel->SetProperty, TRANSFORM=qmat#oModel_t
            endif else begin
              (*sState)[i].oModel->SetProperty, TRANSFORM=oModel_t#qmat
            endelse
            ;
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endif
        endfor ; i
        ;
        ; Could be skipped but it saves few nanoseconds to do this.
        return, 1 ; Make the function swallow the event
      endif

      case sEvent.type of

        0: begin ; -------------------------------------------------------------
          ;
          ; Catch mouse-down events to ensure correct resolution of object is shown
          ;
          for i=0, n_elements(*sState)-1 do begin
            (*sState)[i].oPoly    ->GetProperty, HIDE=hidepolylow
            (*sState)[i].oPolyhigh->GetProperty, HIDE=hidepolyhigh
            if ((hidepolylow eq 0) or (hidepolyhigh eq 0)) then begin
              (*sState)[i].oPoly    ->SetProperty, HIDE=0
              (*sState)[i].oPolyhigh->SetProperty, HIDE=1
            endif
            ; 
            if (obj_valid((*sState)[i].oUser) and obj_valid((*sState)[i].oUserHigh)) then begin
              (*sState)[i].oUser    ->GetProperty, HIDE=hideuserlow
              (*sState)[i].oUserhigh->GetProperty, HIDE=hideuserhigh
              if ((hideuserlow eq 0) or (hideuserhigh eq 0)) then begin
                (*sState)[i].oUser    ->SetProperty, HIDE=0
                (*sState)[i].oUserhigh->SetProperty, HIDE=1
              endif
            endif
            ;
            if (obj_valid((*sState)[i].oVol)) then begin
              (*sState)[i].oVol->GetProperty, HIDE=hidevol
              if (hidevol eq 0) then begin
                v3d_vol_createobj, sState, i, /LOW
              endif
            endif
            ;
          endfor
        end ; End of sEvent.type=0

        1: begin ; -------------------------------------------------------------
          ; Catch mouse-up events to make high resolution objects being drawn
          if (sEvent.release eq 2) then (*sState)[*].rotzoompan=0
          if (sEvent.release eq 4) then (*sState)[*].rotzoompan=0
          for i=0, n_elements(*sState)-1 do begin
            (*sState)[i].oPoly    ->GetProperty, HIDE=hidepolylow
            (*sState)[i].oPolyhigh->GetProperty, HIDE=hidepolyhigh
            if ((hidepolylow eq 0) or (hidepolyhigh eq 0)) then begin
              (*sState)[i].oPoly    ->SetProperty, HIDE=1
              (*sState)[i].oPolyhigh->SetProperty, HIDE=0
            endif
            ;
            if (obj_valid((*sState)[i].oUser) and obj_valid((*sState)[i].oUserHigh)) then begin
              (*sState)[i].oUser    ->GetProperty, HIDE=hideuserlow
              (*sState)[i].oUserhigh->GetProperty, HIDE=hideuserhigh
              if ((hideuserlow eq 0) or (hideuserhigh eq 0)) then begin
                (*sState)[i].oUser    ->SetProperty, HIDE=1
                (*sState)[i].oUserhigh->SetProperty, HIDE=0
              endif
            endif else begin
              hideuserlow=1
              hideuserhigh=1
            endelse
            ;
            if (obj_valid((*sState)[i].oVol)) then begin
              (*sState)[i].oVol->GetProperty, HIDE=hidevol
              if (hidevol eq 0) then begin
                v3d_vol_createobj, sState, i
              endif
            endif else begin
              hidevol=1
            endelse
            ;
            if ((hidepolylow eq 0) or (hidepolyhigh eq 0) or $
                (hideuserlow eq 0) or (hideuserhigh eq 0) or $
                (hidevol eq 0)                              ) then begin
              (*sState)[i].oWindow->Draw, (*sState)[i].oView
            endif
          endfor
        end ; End of sEvent.type=1

        4: begin ; -------------------------------------------------------------
          ; Exposing of the draw widget
          ;
          ; Redraw all views, some bug in IDL starts without exposing view 2 (sometimes).
          ; This should prevent that to happen.
          for i=0, n_elements(*sState)-1 do begin
            (*sState)[i].oWindow->Draw, (*sState)[i].oView
          endfor
        end ; End of sEvent.type=4 

        5: begin ; -------------------------------------------------------------
          if (sEvent.press eq 1) then begin
            ; Keyboard events
            ;
            (*sState)[*].key=sEvent.ch
            key          =sEvent.ch
            message, 'Set key ' + string(byte(key)) + ' ' + strtrim(string(fix(key))), /INFORMATIONAL
            ;
            if ((key eq  87) or (key eq  88) or (key eq  89) or (key eq  90) or $
                (key eq 119) or (key eq 120) or (key eq 121) or (key eq 122)   ) then begin
              ; (Un)constrained rotation
              constrain=0
              axis     =0
              if ((key eq 88) or (key eq 120)) then begin
                constrain=1
                axis     =0
              endif else if ((key eq 89) or (key eq 121)) then begin
                constrain=1
                axis     =1
              endif else if ((key eq 90) or (key eq 122)) then begin
                constrain=1
                axis     =2
              endif
              trackCenterX=round((*sState)[index].viewxsize/2.)
              trackCenterY=round((*sState)[index].viewysize/2.)
              trackRadius=sqrt(trackcenterX^2+trackcenterY^2)
              for i=0, n_elements(*sState)-1 do begin
                (*sState)[i].oTrack->Reset, [trackCenterX, trackCenterY], trackRadius, $
                                            CONSTRAIN=constrain, $
                                            AXIS=axis
              endfor ; i
              ;
            endif else if ((key eq 48) or (key eq 49) or (key eq 50)) then begin
              ; Scaling -> 1
              (*sState)[0].oModel->GetProperty, TRANSFORM=rotatematrix
              mshiftmat       =diag_matrix(replicate(1.0, 4))
              pshiftmat       =mshiftmat
              mshiftmat(3,0:2)=-(*sState)[0].center
              pshiftmat(3,0:2)= (*sState)[0].center
              centermatrix=pshiftmat#rotatematrix#mshiftmat
              ;
              if (key eq 48) then centermatrix(3  ,0:2)=0.0
              if (key eq 49) then centermatrix(3  ,3  )=1.0
              if (key eq 50) then centermatrix(0:2,0:2)=diag_matrix(replicate(1.0, 3))
              ;
              rotatematrix=mshiftmat#centermatrix#pshiftmat
              for i=0, n_elements(*sState)-1 do begin
                (*sState)[i].oModel->SetProperty, TRANSFORM=rotatematrix
                (*sState)[i].oWindow->Draw, (*sState)[i].oView
              endfor ; i
              ;
            endif else if (key eq 109) then begin
              ; (Re)create control widget
              ;
              if (not widget_info((*sState)[0].wBaseControl, /VALID_ID)) then begin
                (*sState)[0].no_controls=0
                wBaseControl=v3d_createcontrolwidget(sState)
                (*sState)[*].wBaseControl=wBaseControl
                widget_control, (*sState)[0].wBaseControl, /REALIZE
                ;
                widget_control, wBaseControl, SET_UVALUE=sState
                xmanager, 'vis3d', wBaseControl, /NO_BLOCK
                ;
                v3d_updatecontrolwidget, sState
              endif
              ;
            endif else if (key eq 112) then begin
              ; Toggle projection: perspective <--> orthogonal
              ;
              (*sState)[0].oView->GetProperty, PROJECTION=projection
              ;
              for i=0, n_elements(*sState)-1 do begin
                (*sState)[i].oView->SetProperty, PROJECTION=3-projection
                (*sState)[i].oWindow->Draw, (*sState)[i].oView
              endfor ; i
              ;
            endif
          endif
        end

        else: dummy=1

      endcase ; End of case sEvent.type

    end ; 'DRAW'

    ; Catching events from the colorbar
    'COLORBAR': begin ; ========================================================
      if (sEvent.type eq 4) then begin
        (*sState)[index].oCbarDrawable->Draw, (*sState)[index].oCbarView
      endif
    end ; 'COLORBAR'

    else: dummy=1 ; ============================================================

  endcase ; End case uval 

  ; Return a constant value (not a structure), which makes the function "swallow" the event.
  return, 1

end ; v3d_draw_event 


function v3d_draw_createviewwidget, wBase, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_draw_createviewwidget ('v3d_draw_create_view_widget')
;
; PURPOSE:
;   Create the draw widget.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_draw_createviewwidget(wBase, sState, index)
;
; INPUT:
;   wBase:  ('widget_Base') handle of the base widget
;
; RETURN VALUE:
;   result: handle to the draw widget
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-16 (B.F.) More comments
;   2005-07-26 (B.F.) Put into separate file
;   2005-07-29 (B.F.) Keyword renderer added
;   2005-09-13 (B.F.) Put into v3d_draw module file, create only view widget
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  wDraw=widget_draw(wBase, $
                    UVALUE         ='DRAW', $
                    XSIZE          =(*sState)[index].viewxsize, $
                    YSIZE          =(*sState)[index].viewysize, $
                    RENDERER       =(*sState)[index].renderer, $
                    RETAIN         =0, $
                    GRAPHICS_LEVEL =2, $
                    KEYBOARD_EVENTS=1, $
                    /EXPOSE_EVENTS, $
                    /BUTTON_EVENTS)
  widget_control, wDraw, /DRAW_MOTION_EVENTS

  return, wDraw

end ; v3d_draw_createviewwidget


function v3d_draw_createwidget, sState
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_draw_createwidget ('v3d_draw_create_widget')
;
; PURPOSE:
;   Create the widget hierarchy from data in the state variable.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   result=v3d_draw_createwidget(sState)
;
; INPUT/OUTPUT:
;   sState:  ('structure_State') The state variable array
;
; OUTPUT:
;   result:  Base widget
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;   Widget Hierarchy:
;     wBase                   , UVALUE=sState (via xmanager)
;       for i=0, nrofviews-1
;         wDSbase             , UVALUE=i,       EVENT_FUNC='v3d_draw_event'
;           wColorbar
;           wDraw
;       endfor
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-08-06 (B.F.) Collected from bits in vis3d and put into separate file
;   2005-08-18 (B.F.) Keywords separate and wBaseControl added
;   2005-08-22 (B.F.) Creation of control widget put into separate routine
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  nrofviews=n_elements(*sState)

  ; --- GROUP LEADER WIDGET AND OTHER BASE WIDGETS ---
  ; The base window/widget which holds the drawables (Views)
;  if ((*sState)[0].no_menu eq 0) then begin
;    wBase=widget_base(TITLE=(*sState)[0].title, $
;                      /TLB_KILL_REQUEST_EVENTS, $
;                      /TLB_SIZE_EVENTS, $
;                      /ROW, $
;                      MBAR=wMenu, $
;                      XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
;    (*sState)[*].wMenu=wMenu
;  endif else begin
    wBase=widget_base(TITLE=(*sState)[0].title, $
                      /TLB_KILL_REQUEST_EVENTS, $
                      /TLB_SIZE_EVENTS,$
                      /ROW, $
                      XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
;  endelse
  (*sState)[*].wBase=wBase
  ;
;  if ((*sState)[0].no_menu eq 0) then begin
;    ; --- The menu on top of the graphics window ---
;    ;
;    v3d_createmenuwidget, wMenu, sState
;    ;
;  endif
  ;
  ; Loop over all views
  for i=0, nrofviews-1 do begin
    ; Generate the widgets, and pack all into a state variable
    ;
    ; Base for both drawables (3D view and Colorbar) 
    wDSbase=widget_base(wBase, $
                        /ROW, /FRAME, $
                        UVALUE=i, $
                        EVENT_FUNC='v3d_draw_event', $
                        XPAD=v3dpar_Widget_xpad, YPAD=v3dpar_Widget_ypad)
    ;
    ; Next two widgets, colorbar and draw, must be created in the order: colorbar, draw
    ; since it will affect how they are placed in the window.
    if ((*sState)[i].no_colorbar eq 0) then begin
      (*sState)[i].wColorbar=v3d_cbar_createwidget(wDSbase, sState, i)
    endif
    ;
    ; Create the draw widget which contains the drawable
    (*sState)[i].wDraw=v3d_draw_createviewwidget(wDSbase, sState, i)
    ;
  endfor ; Loop over all views

  return, wBase

end ; v3d_draw_createwidget
