pro v3d_exportimage, sState, nrofviews, $
           NO_COLORBAR=no_colorbar, $
           EXPORT_FILENAME=export_filename, $
           EXPORT_FILENR=export_filenr, $
           EXPORT_PATH=export_path, $
           EXPORT_TYPE=export_type, $
           EXPORT_DIMENSIONS=export_dimensions, $
           EXPORT_SHOW=export_show, $
           EXPORT_GETIMAGE=export_getimage
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_exportimage ('v3d_export_image')
;
; PURPOSE:
;   Export image to file in non-interactive mode.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_exportimage, sState, nrofviews, $
;           NO_COLORBAR=no_colorbar, $
;           EXPORT_FILENAME=export_filename, $
;           EXPORT_FILENR=export_filenr, $
;           EXPORT_PATH=export_path, $
;           EXPORT_TYPE=export_type, $
;           EXPORT_DIMENSIONS=export_dimensions, $
;           EXPORT_SHOW=export_show, $
;           EXPORT_GETIMAGE=export_getimage   
;
; INPUT:
;   nrofviews: ('number_of_views') integer with number of view windows
;
; INPUT/OUTPUT:
;   sState: ('s_State') array of structures with state information,
;           e.g. handles to all graphics objects or palette information
;
; KEYWORDS:
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2005-07-26 (B.F.) Put into separate file, set some defaults
;-
;-------------------------------------------------------------------------------

  compile_opt hidden

  @v3d_parameters

  ; --- Check keywords ---

  if (NOT KEYWORD_SET(export_filename)) then export_filename=v3dpar_ExportFileName_default

  if (NOT KEYWORD_SET(export_type)) then export_type=v3dpar_ExportFileType_default

  if (NOT KEYWORD_SET(export_dimensions)) then begin
    export_dimensions=[v3dpar_xsize_default, v3dpar_ysize_default]
  endif 

  ; Use default path if EXPORT_PATH not set
  if (NOT KEYWORD_SET(export_path)) then begin
    export_path=v3dpar_ExportPath_default
  endif 

  ; If export_filenr is not set, default to zero length string
  if (N_ELEMENTS(export_filenr) eq 0) then begin
    export_filepostfix=''
  endif else begin
    export_filepostfix='_' + string(export_filenr, FORMAT='(I0)')
  endelse
  
  ; Abbreviations
  ExpWidth=export_dimensions[0]
  ExpHeight=export_dimensions[1]

  if (NOT KEYWORD_SET(no_colorbar)) then begin
    oExpBuffer_c=OBJ_NEW('IDLgrBuffer', $
                         COLOR_MODEL=0, $
                         DIMENSIONS=[v3dpar_colorbar_WindowWidth, ExpHeight])
  endif

  ; Create image array which contains the total final image to be exported
  colorbarWidth=KEYWORD_SET(no_colorbar) ? 0 : v3dpar_colorbar_WindowWidth
  export_getimage=MAKE_ARRAY(3, nrofviews * (ExpWidth + colorbarWidth), ExpHeight, /BYTE ) 

  oExpBuffer_i=OBJ_NEW('IDLgrBuffer', $
                       COLOR_MODEL=0, $
                       DIMENSIONS=export_dimensions)

  ; will be an index into exported image array
  x_pos=0

  for i=0, nrofviews-1 do begin
    oExpBuffer_i->Draw, (*sState)[i].oView
    ;
    oExpImage=oExpBuffer_i->Read()  
    oExpImage->SetProperty, INTERLEAVE=0
    oExpImage->GetProperty, DATA=expView
    obj_destroy, oExpImage
    ;
    if (NOT KEYWORD_SET(no_colorbar)) then begin
      oExpBuffer_c->Draw, (*sState)[i].oCbarView
      oExpColorbar=oExpBuffer_c->Read()
      oExpColorbar->SetProperty, INTERLEAVE=0
      oExpColorbar->GetProperty, DATA=expColorbar
      obj_destroy, oExpColorbar
      ;
      export_getimage[*, x_pos:(x_pos + v3dpar_colorbar_WindowWidth - 1), *] = expColorbar
      x_pos = x_pos + v3dpar_colorbar_WindowWidth 
    endif
    ;
    ; Copy image to exported image
    export_getimage[*, x_pos:(x_pos+expWidth-1), *] = expView 
    x_pos = x_pos + expWidth            
  endfor

  full_filename=export_path + export_filename + export_filepostfix + '.' + strlowcase(export_type)

  case strupcase(export_type) of 
    'BMP':begin
        write_bmp, full_filename, export_getimage
      end
    'JPEG':begin
        write_jpeg, full_filename, export_getimage, TRUE=1 
      end
    'JPG':begin
        write_jpeg, full_filename, export_getimage, TRUE=1 
      end
    'PNG':begin
        write_png, full_filename, export_getimage
      end
    'TIFF':begin
        write_tiff, full_filename, export_getimage
      end
    'NOFILE':begin
      end
    else: begin
        message, 'ERROR: Not supported image type for exporting.'
      end
  endcase

  ; Destroy the buffer and image objects
  obj_destroy, oExpBuffer_i

  if (NOT KEYWORD_SET(no_colorbar)) then begin
    obj_destroy, oExpBuffer_c
  endif

  print, 'Exported ' + full_filename
  message, 'Finished exporting.', /INFORMATIONAL

end ; exportimage
