;-------------------------------------------------------------------------------
; Vis3D module: Light
;-------------------------------------------------------------------------------
; Contains:
;   v3d_light_createobj:   Create light objects
;   v3d_light_event:       Handle all light editor events
;   v3d_light_starteditor: Start light editor widget
;   v3d_light_init:        Initialize light structure
;-------------------------------------------------------------------------------


pro v3d_light_createobj, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_light_createobj ('v3d_light_create_object')
;
; PURPOSE:
;   Create light objects.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_light_createobj, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; HISTORY:
;   2005-09-07 (B.F.) Put into own routine in light module
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  (*sState)[index].oView->GetProperty, VIEWPLANE_RECT=viewplane_rect, $
                                       ZCLIP         =zclip

;  location=[40.0, 40.0, 40.0]
;  location=[viewplane_rect[0]+viewplane_rect[2], $
;            viewplane_rect[1]+viewplane_rect[3], $
;            zclip[0]]
  location=[(*sState)[index].box.xrange[1], $
            (*sState)[index].box.yrange[1], $
            zclip[0] + 0.02*(zclip[1]-zclip[0])]

  (*sState)[index].oLight1=OBJ_NEW('IDLgrLight', $
                                   NAME     ='Light A', $
                                   HIDE     =(*sState)[index].hidelights, $
                                   TYPE     =1, $
                                   LOCATION =location, $
                                   INTENSITY=0.5)
  (*sState)[index].oLight2=OBJ_NEW('IDLgrLight', $
                                   NAME     ='Light B', $
                                   HIDE     =(*sState)[index].hidelights, $
                                   TYPE     =0, $
                                   INTENSITY=0.5, $
                                   COLOR    =[255B, 255B ,255B])

end ; v3d_light_createobj


function v3d_light_event, sEvent
;-------------------------------------------------------------------------------
; NAME:
;   v3d_light_event ('v3d_light_event')
;
; PURPOSE:
;   Handle all light editor events.
;
; CATEGORY:
;   3D Visualization
;
; INPUT:
;   sEvent:  ('structure_Event') structure with information about event
;
; MODIFICATION HISTORY:
;   2005-08-04 (B.F.) Written
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  widget_control, sEvent.top    , GET_UVALUE=sState
  widget_control, sEvent.handler, GET_UVALUE=index
  ;widget_control, sEvent.id     , GET_UVALUE=uval
  ;help, /str, sEvent
  ;help, sState, index

  (*sState)[index].oWindow->Draw, (*sState)[index].oView

  return, 1

end ; v3d_light_event


pro v3d_light_starteditor, sState, index
;-------------------------------------------------------------------------------
; NAME:
;   v3d_light_starteditor ('v3d_light_start_editor')
;
; PURPOSE:
;   Start light editor widget with proper parameters.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_light_starteditor, sState, index
;
; INPUT:
;   index:  ('index') current view index
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; HISTORY:
;   2005-09-07 (B.F.) Put into own routine in light module
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  @v3d_parameters

  strindex=string(index+1, format='(I0)')

  wBaseLight=widget_base(TITLE='Settings Light ' + strindex, $
                         UVALUE=sState)
  wHandleLight=widget_base(wBaseLight, $
                           UVALUE=index, $
                           EVENT_FUNC='v3d_light_event')
  wLight=CW_LIGHT_EDITOR(wHandleLight, $
                         LIGHT=[(*sState)[index].oLight1, (*sState)[index].oLight2], $
                         XSIZE=v3dpar_LightEditor_size, YSIZE=v3dpar_LightEditor_size, $
                         XRANGE=(*sState)[index].box.xrange, $
                         YRANGE=(*sState)[index].box.yrange, $
                         ZRANGE=(*sState)[index].box.zrange, $
                         UVALUE='LIGHT' + strindex)
  widget_control, wBaseLight, /REALIZE
  xmanager, 'vis3d', wBaseLight, /NO_BLOCK

end ; v3d_light_starteditor


function v3d_light_init
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_light_init ('v3d_light_initialize')
;
; PURPOSE:
;   Initialize light structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   light=v3d_light_init()
;
; MODIFICATION HISTORY:
;   2005-09-07 Written by Bernd Freytag, L.A.
;-
;-------------------------------------------------------------------------------

  compile_opt hidden, strictarr

  return, 0

end ; v3d_light_init
