;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_parameters ('v3d_parameters')
;
; PURPOSE:
;   Specify default values for a number of parameters for vis3d.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   @v3d_parameters
;
; MODIFICATION HISTORY:
;   2005-07-15 (B.F. LA) File created
;   2005-07-15 (B.F.) v3dpar_box_Color added
;   2006-06-06 (B.F.) Some hide entries
;   2010-03-23 (B.F.) Explicit default path to color table
;-
;-------------------------------------------------------------------------------

; --- Graphics window size ---

; Default horizontal graphics window size in pixels
v3dpar_xsize_default=400

; Minimum horizontal graphics window size in pixels 
v3dpar_xsize_min=200

; Maximum horizontal graphics window size in pixels 
v3dpar_xsize_max=1600

; Default vertical graphics window size in pixels (should equal horizontal size)
v3dpar_ysize_default=400

; Minimum vertical graphics window size in pixels 
v3dpar_ysize_min=200

; Maximum vertical graphics window size in pixels 
v3dpar_ysize_max=1024

; Padding amount (larger values give broader frame around widgets)
v3dpar_Widget_xpad=0
v3dpar_Widget_ypad=0


; --- Light ---

; Size (x and y) of light editor window
v3dpar_LightEditor_size=300

; Show/hide lights
v3dpar_Light_hide=1


; --- Colorbar ---

; Left margin of colorbar, relative to colorbar window width
v3dpar_colorbar_LeftMargin=0.03

; Width of colorbar window in pixels
v3dpar_colorbar_WindowWidth=100

; Width of colorbar, relative to colorbar window width
v3dpar_colorbar_Width=0.25

; Height of colorbar, relative to colorbar window width
v3dpar_colorbar_Height=0.94

; Number of major tickmarks
v3dpar_colorbar_Major=6

; Number of minor tickmarks
v3dpar_colorbar_Minor=1

; Tick length
v3dpar_colorbar_TickLen=2.8

; Sub tick length
v3dpar_colorbar_SubTickLen=0.7

; Frame color
v3dpar_colorbar_Color=[255,255,255]

; Background color
v3dpar_colorbar_BackgroundColor=[0,0,0]

; Font (size: 7 for Linux, 12 for Mac)
v3dpar_colorbar_Font    ='Helvetica'
v3dpar_colorbar_FontSize=7


; --- Box ---

; Box colors
v3dpar_box_Color =[255B,   0B, 0B]
v3dpar_box_Color2=[255B, 140B, 0B]

; Show/hide box
v3dpar_box_hide=0


; --- Pre-rotation ---

; Default rotation axis (for pre-rotation)
v3dpar_RotateAxis_default=[0.0, 1.0, 0.0]

; Default rotation angle (for pre-rotation)
v3dpar_RotateAngle_default=0.0


; --- Views: palette, colors, scale ---

; Default IDL palette value (0: greyscale)
v3dpar_Palette_default=0

; Default background color for dataset views
v3dpar_ViewBackgroundColor_default=[0B, 0B, 70B]

; Default solid color
v3dpar_SolidColor_default=[127B, 127B, 127B]

; Default scale value
v3dpar_Scale_default=1.25

; Renderer (0: hardware, 1: software)
v3dpar_Renderer_default=0

; Projection (1=Orthogonal  2=Perspective)
v3dpar_Projection_default=2

; Path to external color table
v3dpar_ColorTablePath_default="."

; --- Iso surface ---

; Default value for decimation (in %)
v3dpar_Decimation_default=100

; Polygon shading (0: coarse flat, 1: smooth Gouraud)
v3dpar_PolygonShading_default=1

; Polygon style (0: points, 1: lines, 2: filled)
v3dpar_PolygonStyle_default=2

; Default smooth parameter (was: 50)
v3dpar_SmoothIterations_default=10

; Default smooth parameter
v3dpar_SmoothLambda_default=0.8

; Default color smooth parameter
v3dpar_SmoothColorIterations_default=15

; Default color smooth parameter
v3dpar_SmoothColorLambda_default=0.05

; Show/hide iso-surfaces (0: show, 1: hide)
v3dpar_Iso_hide=1


; --- Slices ---

; Show/hide slices (0: show, 1: hide)
v3dpar_Slices_hide=1


; --- Dots ---

; Number of dots
v3dpar_Dots_Number_default=10000L

; Show/hide dots (0: show, 1: hide)
v3dpar_Dots_hide=1


; --- Volume rendering ---

; Show/hide rendered volume (0: show, 1: hide)
v3dpar_Vol_hide=1


; --- Streamlines ---

; Number of streamlines
v3dpar_Stream_NLines_default=27L

; Number of points on streamline
v3dpar_Stream_NPoints_default=20L


; --- Export ---

; Default path: current directory
v3dpar_ExportPath_default='./'

; Default file name
v3dpar_ExportFileName_default='vis3d_image'

; Default file type
v3dpar_ExportFileType_default='jpg'

v3dpar_WriteImageFileName_default='vis3d_image.jpg'

; --- Movies ---
v3dpar_Movie_nFrames_default=         30
v3dpar_Movie_Mode_default=            1
v3dpar_Movie_OscillationAngle_default=10.0
v3dpar_Movie_RotateAxis_default=      2
v3dpar_Movie_TransSystem_default=     1
v3dpar_Movie_FrameRate_default=       5
v3dpar_Movie_Quality_default=         75
v3dpar_Movie_AntiAliasFactor_default= 1
v3dpar_Movie_Format_default=          0
v3dpar_Movie_ImageFormat_default=     0
v3dpar_Movie_FileName_default=        'vis3d_anim.mpg'
