pro v3d_start, sState, initialize=initialize
;-------------------------------------------------------------------------------
;+
; NAME:
;   v3d_start ('v3d_start')
;
; PURPOSE:
;   Start Vis3D session with initialized sState structure.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   v3d_start, sState
;
; INPUT/OUTPUT:
;   sState: ('structure_State') array of structures with state information
;
; KEYWORDS:
;   None
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2006-06-26 (B.F.) Put into separate routine and file
;-
;-------------------------------------------------------------------------------

  compile_opt strictarr

  nrofviews=n_elements(*sState)

  ; === Create and realize widgets and menus =================================
  ;
  ; --- Create widgets ---
  wBase=v3d_draw_createwidget(sState)
  ; --- Realize the widget hierarchy (with empty view, up to now) ---
  widget_control, wBase, /REALIZE
  ;
  ; Create controls by default.
  if ((*sState)[0].no_controls eq 0) then begin
    wBaseControl=v3d_createcontrolwidget(sState)
    (*sState)[*].wBaseControl=wBaseControl
    widget_control, wBaseControl, /REALIZE
  endif
  ;
  ; === Create object hierarchy for all views ================================
  ;
  if (keyword_set(initialize)) then begin
    v3d_createobjects, sState
  endif
  ;
  ; === Fill text/info widgets ===============================================
  ;
  v3d_updatefields, sState
  ;
  ; === Start xmanager =======================================================
  ;
  for i=0, nrofviews-1 do begin
    ; Get object reference to all drawables
    widget_control, (*sState)[i].wDraw, GET_VALUE=oWindow
    (*sState)[i].oWindow = oWindow
    ;
    if ((*sState)[i].no_colorbar eq 0) then begin
      widget_control, (*sState)[i].wColorbar, GET_VALUE=oCbarDrawable
      (*sState)[i].oCbarDrawable = oCbarDrawable
    endif
  endfor
  ;
  widget_control, wBase, SET_UVALUE=sState
  xmanager, 'vis3d', wBase, /NO_BLOCK, EVENT_HANDLER='v3d_drawwindow_event'
  ;
  if ((*sState)[0].no_controls eq 0) then begin
    widget_control, wBaseControl, SET_UVALUE=sState
    xmanager, 'vis3d', wBaseControl, /NO_BLOCK
  endif

end ; v3d_start
