function v3dgr_dots, d1, c1, $
                     NDOTS=ndots0, HIDE=hide, rvind=rvind

compile_opt strictarr

; --- Check keywords ---

if (not keyword_set(ndots0)) then begin
  ndots=100000L
endif else begin
  ndots=1L > long(ndots0)
endelse

; --- Check input arrays ---

exist_d1=0
exist_c1=0

if ((size(d1, /n_dim) ne 3) and (size(c1, /n_dim) ne 3)) then begin
  message, 'ERROR: No 3D input arrays supplied.'
endif

if (size(d1, /n_dim) eq 3) then begin
  exist_d1=1
  d1_size=size(d1, /dim)
  n1=d1_size[0]-1L
  n2=d1_size[1]-1L
  n3=d1_size[2]-1L
endif

if (size(c1, /n_dim) eq 3) then begin
  exist_c1=1
  c1_size=size(c1, /dim)
  if (exist_d1) then begin
    if (not array_equal(d1_size, c1_size)) then begin
      message, 'ERROR: Input arrays have incompatible dimensions.'
    endif
  endif
  n1=c1_size[0]-1L
  n2=c1_size[1]-1L
  n3=c1_size[2]-1L
endif

nvec=[n1+1L, n2+1L, n3+1L]

help, exist_d1, exist_c1, n1,n2,n3

; ---

time0=systime(1)

nc    =n_elements(d1)
d1_cum=total(reform(d1 > 0.0, nc), /CUMULATIVE)
rampl =d1_cum[nc-1L]
rval  =rampl*randomu(seed, ndots)
rpos  =interpol(findgen(nc), d1_cum, rval)
rind  =long(rpos)

rvind =lonarr(3, ndots)
factor=1L
for i=0, 2 do begin
  rvind[i, *]=(rind / factor) mod nvec[i]
  factor=factor*nvec[i]
endfor ; i

rvpos=float(rvind)

rvpos=rvpos + randomu(seed, 3, ndots) - 0.5

oDots=obj_new('IDLgrPolygon', $
              rvpos, $
              COLOR=[255B, 255B, 255B], $
              STYLE=0, $
              HIDE=hide)

message, 'Dot cloud generation took ' + string(systime(1)-time0, format='(G0)') + ' sec', $
         /INFORMATIONAL

return, oDots

end ; v3dgr_dots

