pro vis3d, d1, c1, $
           d2, c2, $
           d3, c3, $
           d4, c4, $
           NO_COPY=no_copy, $
           oUser1=oUser1, oUserLow1=oUserLow1, $
           oUser2=oUser2, oUserLow2=oUserLow2, $
           oUser3=oUser3, oUserLow3=oUserLow3, $
           oUser4=oUser4, oUserLow4=oUserLow4, $
           ;
           RENDERER    =renderer, $
           PROJECTION  =projection, $
           SCALE       =scale, $
           XSIZE       =xsize, $
           YSIZE       =ysize, $
           ROTATEAXIS  =rotateaxis, $
           ROTATEANGLE =rotateangle, $
           ROTATEMATRIX=rotatematrix, $
           VIEWCOLOR   =viewcolor, $
           ;
           IDLPAL      =IDLpal, $
           PALETTERANGE=paletterange, $
           ;
           ISOVALUES             =isoval_arr, $
           DECIMATION            =decim_arr, $
           SMOOTH_ON             =smooth_on, $
           SMOOTH_ITERATIONS     =smooth_iterations, $
           SMOOTH_LAMBDA         =smooth_lambda, $
           COLORSMOOTH_ON        =colorsmooth_on, $
           COLORSMOOTH_LAMBDA    =colorsmooth_lambda, $
           COLORSMOOTH_ITERATIONS=colorsmooth_iterations, $
           ;
           LIGHTS  =lights, $
           SHOWAXIS=showaxis, $
           ;
           SIMPLE_GUI =simple_GUI, $
           NO_CONTROLS=no_controls, $
           NO_COLORBAR=no_colorbar, $
           NO_MENU    =no_menu, $
           TITLE      =title, $
           QUIET      =quiet, $
           VERBOSE    =verbose, $
           ;
           EXPORT_ON        =export_on, $
           EXPORT_FILENAME  =export_filename, $
           EXPORT_FILENR    =export_filenr, $
           EXPORT_PATH      =export_path, $
           EXPORT_TYPE      =export_type, $
           EXPORT_DIMENSIONS=export_dimensions, $
           EXPORT_SHOW      =export_show, $
           EXPORT_GETIMAGE  =export_getimage, $
           COMMONBLOCK      =commonblock
;-------------------------------------------------------------------------------
;+
; NAME:
;   vis3d ('visualization_3_dimensions')
;
; PURPOSE:
;   Visualize 3D data arrays side by side using iso surfaces and other tools.
;
; CATEGORY:
;   3D Visualization
;
; CALLING SEQUENCE:
;   vis3d, d1,c1, d2,c2, d3,c3, d4,c4, $
;     NO_COPY=no_copy,
;     RENDERER=renderer,                       PROJECTION=projection, $
;     ISOVALUES=isoval_arr,                    DECIMATION=decim_arr, $
;     IDLPAL=IDLpal,                           PALETTERANGE=paletterange, $
;     SCALE=scale, $
;     XSIZE=xsize,                             YSIZE=ysize, $
;     VIEWCOLOR=viewcolor,                     LIGHTS=lights, $
;     SHOWAXIS=showaxis,                       ROTATEAXIS=rotateaxis, $
;     ROTATEANGLE=rotateangle,                 ROTATEMATRIX=rotatematrix, $
;     SMOOTH_ON=smooth_on, $
;     SMOOTH_ITERATIONS=smooth_iterations,     SMOOTH_LAMBDA=smooth_lambda, $
;     COLORSMOOTH_ON=colorsmooth_on, $
;     COLORSMOOTH_LAMBDA=colorsmooth_lambda, $
;     COLORSMOOTH_ITERATIONS=colorsmooth_iterations, $
;     simple_GUI=simple_GUI,                   NO_CONTROLS=no_controls, $
;     NO_COLORBAR=no_colorbar,                 NO_MENU=no_menu, $
;     TITLE=title, $
;     QUIET=quiet,                             VERBOSE=verbose, $
;     EXPORT_ON=export_on,                     EXPORT_FILENAME=export_filename, $
;     EXPORT_FILENR=export_filenr,             EXPORT_PATH=export_path, $
;     EXPORT_TYPE=export_type,                 EXPORT_DIMENSIONS=export_dimensions, $
;     EXPORT_GETIMAGE=export_getimage, $
;     COMMONBLOCK=commonblock, $
;     oUser1=oUser1, oUser2=oUser2, oUser3=oUser3, oUser4=oUser4, $
;     oUserLow1=oUserLow1, oUserLow2=oUserLow2, oUserLow3=oUserLow3, oUserLow4=oUserLow4
;
; INPUT:
;   d1:         ('data_1')  3D array 1 used to define the isosurface ("iso dataset")
;   c1:         ('color_1') 3D array 1 for vertex coloring ("color dataset")
;   d2,c2, d3,c3, d4,c4: correspondingly
;
; KEYWORDS:
;   no_copy:    ('no_copy') if set, no copy of the 3D input arrays is made which
;               will become undefined at exit
;   renderer:   ('renderer') 0: hardware rendering, 1: software rendering
;   projection: ('projection') 1: orthogonal, 2: perspective
;   isovalues:  ('iso_surface_values') float vector with iso values, one for each
;               iso data array (number of elements = number of iso data arrays = nrofviews)
;   decimation: ('decimation') Integer vector with decimation factors (in percent),
;               default: 5%
;   IDLpal:     ('IDL_palette') Integer vector with IDL palette numbers,
;               default: 0 (grey)
;   paletterange: ('palette_range') Double (2 x nrofviews) array with range that
;               should be covered by each palette
;   scale:      ('scale') float scaling factor (the same for all views, typically 1.25)
;   xsize:      ('x_size') horizontal size of draw window
;   ysize:      ('y_size') vertical size of draw window
;   viewcolor:  ('view_color') 3 byte vector with view background color
;   lights:     ('lights') Vector to switch on lights individually for each view,
;               default: off
;   showaxis:   ('show_axis') if set a box outlining the data volume is shown
;   rotateangle:('rotate_angle') angle of initial rotation
;   rotateaxis: ('rotate_axis') axis vector for initial rotation
;   rotatematrix:('rotate_matrix') 4x4 float matrix describing the
;               transformation (and projection) from model to device coordinates.
;   smooth_on:  ('smooth_on') if set the polygon vertex coordinates are smoothed
;               to reduce noice or grid artifacts, default=0 (off)
;   smooth_iterations: ('smooth_iterations') Number of vertex smooth iterations,
;               default=10
;   smooth_lambda: ('smooth_lambda') Amount of smoothing in individual step,
;               default=0.8
;   colorsmooth_on: ('color_smooth_on') if set the polygon color values are
;               smoothed to reduce noice or grid artifacts, default=0 (off)
;   colorsmooth_iterations: ('color_smooth_iterations') Number of color smooth
;               iterations, default=15
;   colorsmooth_lambda: ('color_smooth_lambda') Amount of color smoothing in
;               individual step, default=0.05
;   simple_GUI: ('simple_GUI') Set the no_controls, no_colorbar, no_menu keywords
;   no_controls:('no_controls') If set the control widget below the draw window
;               is not shown, default=0 (show controls)
;   no_colorbar:('no_colorbar') If set the colorbar is not shown,
;               default=0 (show conlorbar)
;   no_menu:    ('no_menu') If set the menu bar is not shown,
;               default=0 (show menu bar)
;   title:      ('title') Title string for main window
;   quiet:      ('quiet') Set the !QUIET system variable to 1 and be quiet,
;               default: untouched
;   verbose:    ('verbose') Set the !QUIET system variable to 0 and be verbose
;               default: untouched
;   export_on:        ('export_on') Activate the non-interactive export mode to write
;                     images directly e.g. into JPEG file
;   export_filename:  ('export_file_name') Name of file to write into
;   export_filenr:    ('export_file_number') Number appended to the file name
;   export_path:      ('export_path') Path where the output file should be located
;   export_type:      ('export_type') Type of output file ('bmp', 'jpg', 'png', 'tiff')
;   export_dimensions:('export_dimensions') Vector of two integers with output
;                     e.g. [800,600]
;   export_getimage:  ('export_get_image') Output: the generated image is given here
;   commonblock:('common_block') index of common block (so far: 1 or 2)
;   oUser1      ('object_User_1') User supplied IDL graphics object 1,
;               high resolution version for still images
;   oUserLow1   ('object_User_Low_1') User supplied IDL graphics object 1,
;               low resolution version to be shown during rotation
;   oUser2,3,4: correspondingly
;   oUserLow2,3,4:correspondingly
;
; EXAMPLE:
;   ; --- Create sample data ---
;   n   =40
;   axis=2.0*findgen(n)/float(n-1)-1.0
;   x   =rebin(reform(axis, n,1,1), n,n,n)
;   y   =rebin(reform(axis, 1,n,1), n,n,n)
;   z   =rebin(reform(axis, 1,1,n), n,n,n)
;   r   =sqrt(x^2+y^2+z^2)
;   d1  =r+0.1*cos(6.0*x)+0.05*cos(25.0*y)
;   c1  =r
;   ; --- Start visualization ---
;   vis3d, d1,c1, c1,d1
;
; MODIFICATION HISTORY:
;   2002-06-09 Written by Sigurdur Finnsson and Bernd Freytag, Uppsala
;   ...
;   2006-07-06 (B.F.) Check for custom color palette file
;-
;
; MODIFICATION HISTORY (DETAILED):
;   2002-06-09 Written by Sigurdur Finnsson, Uppsala
;   2002-12-02 (B.F.) Check if scale keyword is actually given
;   2005-07-08 (B.F.) Small changes in colorbar layout; "other buttons" in extra row
;   2005-07-09 (B.F.) Buttons: "Regenerate mesh" -> "Apply: Mesh", "Change" -> "Apply",
;                     Keywords "oUser" objects added
;   2005-07-10 (B.F.) "oUser" objects fully implemented
;   2005-07-13 (B.F.) More comments
;   2005-07-14 (B.F.) Calls to 'v3d_findtag', 'v3d_addtag', 'v3d_getcenter' removed;
;                     Button for rotation, zoom, pan
;   2005-07-15 (B.F.) Include master parameter file
;   2005-07-16 (B.F.) More parameters put into master parameter file;
;                     'Help' menu with 'about' and 'doc_library' entry;
;                     'compile_opt hidden' inserted in some routines
;   2005-07-20 (B.F.) Some routines into separate files
;   2005-07-26 (B.F.) Some routines into separate files
;   2005-07-27 (B.F.) Settings menu
;   2005-07-28 (B.F.) Rotation/zoom/pan via mouse buttons
;   2005-07-29 (B.F.) Keep aspect ratio of object even for unequal xsize and ysize,
;                     keyword renderer added
;   2005-07-30 (B.F.) v3d_create_info -> v3d_createinfo, center: structure -> vector
;   2005-07-31 (B.F.) Allow resizing of window (preliminary version)
;   2005-08-03 (B.F.) Keywords "oUserLow" objects added
;   2005-08-04 (B.F.) Light editor widget
;   2005-08-05 (B.F.) Reorganization of state variable initialization
;   2005-08-06 (B.F.) Quickstart: realize the widget hierarchy as soon as it is created 
;   2005-08-08 (B.F.) Constrained rotation, controlled via keyboard commands
;   2005-08-13 (B.F.) State variable becomes pointer
;   2005-08-15 (B.F.) data3D structure split into 3, redesign of v3d_generate3d
;   2005-08-16 (B.F.) d0->d1, c0->c1,...; keyword rotationmatrix -> rotatematrix,
;                     keyword projection added, v3d_generate3d reorganized
;   2005-08-17 (B.F.) Keyword commonblock added
;   2005-08-19 (B.F.) New layout for iso controls
;   2005-08-21 (B.F.) New layout for select controls
;   2005-08-22 (B.F.) New layout for color controls
;   2005-08-23 (B.F.) Event handler functions redistributed
;   2005-08-24 (B.F.) Some re-organization: v3d_createobjects routine split of
;   2005-08-26 (B.F.) Export as MPEG
;   2005-08-29 (B.F.) Menu moved to Control window
;   2005-09-01 (B.F.) Volume rendering with dots
;   2005-09-02 (B.F.) All isosurface routines put into common file and renamed
;   2005-09-06 (B.F.) x slice
;   2005-09-07 (B.F.) light module
;   2005-09-08 (B.F.) y and z slice
;   2005-09-11 (B.F.) Call subroutines to insert input data into box structure
;   2005-09-14 (B.F.) First version of streamlines module
;   2005-09-22 (B.F.) Color of streamlines adjustable
;   2005-11-21 (B.F.) Menus under main tool buttons
;   2006-05-31 (B.F.) Bugfix concerning Control refresh, slice color button, mesh statistics
;   2006-06-06 (B.F.) Some minor changes, e.g. some additional hide entries in parameter file
;   2006-06-09 (B.F.) Volume rendering
;   2006-06-26 (B.F.) Call v3d_start
;   2006-06-27 (B.F.) Bugfixes in volume rendering module
;                     Setting of background color moved into color settings widget
;   2006-06-28 (B.F.) trim -> strtrim
;   2006-06-30 (B.F.) Renamed: data3D -> iso; do not destroy and recreate iso object
;   2006-07-06 (B.F.) Check for custom color palette file
;   2007-04-12 (B.F.) Slices: automatic positioning to minimum or maximum
;   2009-01-30 (B.F.) Check if iso value is within iso data range and correct value
;   2011-02-21 (B.F.) Option to mask color in slices with iso dataset values
;   2011-04-01 (B.F.) Set color palette to full range (not the range correspongind to the iso value)
;-------------------------------------------------------------------------------

  compile_opt strictarr

  ; Default constants
  @v3d_parameters

  ; === Check input parameters and keywords and put everuthing into state variable =================

  ; Check VERBOSE and QUIET keywords:
  ; two separate keywords so user can set !QUIET outside of program
  if (KEYWORD_SET(quiet)) then begin
    !QUIET=1
  endif
  if (KEYWORD_SET(verbose)) then begin
    !QUIET=0
  endif

  ; Number of input parameters: 2 3D arrays for each view
  npar=n_params()

  if (npar eq 0) then begin
    ; --- Demo mode ---
    ;
    message, 'No input data provided: creating demo data', /INFORMATIONAL
    ;
    ; Edge length of data cube
    n=40
    ;
    ; Axis vector, cube with radius values and size n x n x n
    axis=2.0*findgen(n)/float(n-1)-1.0
    x=rebin(reform(axis, n,1,1), n,n,n)
    y=rebin(reform(axis, 1,n,1), n,n,n)
    z=rebin(reform(axis, 1,1,n), n,n,n)
    r=sqrt(x^2+y^2+z^2)
    ;
    ; Sample data array (d1) and color array (c1)
    ;d1=r+0.1*cos(6.0*x)+0.05*cos(20.0*y)+0.03*cos(35.0*z)
    ;d1=(0.5/(0.5+r+0.1*cos(6.0*x)+0.05*cos(20.0*y)+0.03*cos(35.0*z)))^4
    ;c1=(0.5/(0.5+temporary(r)))^4
    d1=exp(-2.0*(0.1+r+0.1*cos(6.0*x)+0.05*cos(20.0*y)+0.03*cos(35.0*z))^4)
    c1=exp(-2.0*(0.1+temporary(r))^4)
    ;d1=exp(-2.0*r^4)*(0.2+0.1*cos(6.0*x)+0.05*cos(20.0*y)+0.03*cos(35.0*z))
    ;c1=exp(-2.0*r^4)
    d2=c1
    c2=d1
    npar=4
  endif

  if ((npar ne 2) AND (npar ne 4) AND (npar ne 6) AND (npar ne 8)) then begin
    message, 'ERROR: Invalid number of parameters.'
  endif

  ; Number of views
  nrofviews=npar/2

  ; Initialize state variable (pointer to array of structures)
  sState=v3d_sstate_init(nrofviews, export_on=export_on)

  if (keyword_set(commonblock)) then begin
    v3d_sstate_putcommon, sState, commonblock
  endif

  ; Check number of dataset pairs
  switch npar of 
    8: begin
         index=3
         ;
         v3d_box_insertisodata,   sState, index, d4, NO_COPY=no_copy
         v3d_box_insertcolordata, sState, index, c4, NO_COPY=no_copy
       end
    6: begin
         index=2
         ;
         v3d_box_insertisodata,   sState, index, d3, NO_COPY=no_copy
         v3d_box_insertcolordata, sState, index, c3, NO_COPY=no_copy
       end
    4: begin
         index=1
         ;
         v3d_box_insertisodata,   sState, index, d2, NO_COPY=no_copy
         v3d_box_insertcolordata, sState, index, c2, NO_COPY=no_copy
       end
    2: begin
         index=0
         ;
         v3d_box_insertisodata,   sState, index, d1, NO_COPY=no_copy
         v3d_box_insertcolordata, sState, index, c1, NO_COPY=no_copy
       end
  endswitch ; End of 'switch npar of', checking number of parameters

  ; User defined graphics objects
  if (OBJ_VALID(oUser1   )) then (*sState)[0].oUserHigh=oUser1
  if (OBJ_VALID(oUserLow1)) then (*sState)[0].oUser    =oUserLow1
  if (OBJ_VALID(oUser2   )) then (*sState)[1].oUserHigh=oUser2
  if (OBJ_VALID(oUserLow2)) then (*sState)[1].oUser    =oUserLow2
  if (OBJ_VALID(oUser3   )) then (*sState)[2].oUserHigh=oUser3
  if (OBJ_VALID(oUserLow3)) then (*sState)[2].oUser    =oUserLow3
  if (OBJ_VALID(oUser4   )) then (*sState)[3].oUserHigh=oUser4
  if (OBJ_VALID(oUserLow4)) then (*sState)[3].oUser    =oUserLow4

  ; Check arrays for isovalues and decimation values.
  ; If no isovalues were given, then default isovalues will be used.
  if ((N_ELEMENTS(isoval_arr) eq nrofviews)) then begin
    (*sState)[*].iso.curriso=isoval_arr
    for i=0, nrofviews-1 do begin
      ; --- Check if iso value is within iso data range and correct value ---
      if (((*sState)[i].iso.curriso lt (*sState)[i].box.fullisorange[0]) or $
          ((*sState)[i].iso.curriso gt (*sState)[i].box.fullisorange[1])) then begin
        (*sState)[i].iso.curriso=0.5*((*sState)[i].box.fullisorange[0] + $
                                      (*sState)[i].box.fullisorange[1]  )
      endif
    endfor ; i
  endif else if (N_ELEMENTS(isoval_arr) eq 0) then begin 
    (*sState)[*].iso.curriso=0.5*((*sState)[*].box.fullisorange[0] + $
                                  (*sState)[*].box.fullisorange[1]  )
  endif else begin
    message, 'ERROR: Invalid number of isovalues.'
  endelse

  ; Default values for decimation if not defined
  case N_ELEMENTS(decim_arr) of
    0:         decim_arr=REPLICATE(v3dpar_Decimation_default, nrofviews)
    1:         decim_arr=REPLICATE(1 > decim_arr < 100, nrofviews)
    nrofviews: decim_arr=1 > decim_arr < 100
    else:      message, 'ERROR: Incorrect number of values for keyword DECIMATION.'
  endcase
  (*sState)[*].iso.decim=decim_arr

  ; Checking keyword LIGHTS
  case N_ELEMENTS(lights) of
    0:         hidelight_arr=REPLICATE(v3dpar_Light_hide, nrofviews)
    1:         hidelight_arr=REPLICATE(1-(fix(lights) mod 2), nrofviews)
    nrofviews: hidelight_arr=1-(fix(lights) mod 2)
    else:      message, 'ERROR: Incorrect number of values for keyword LIGHTS.'
  endcase
  (*sState)[*].hidelights=hidelight_arr
  ; end checking keyword LIGHTS

  ; Checking keywords COLORSMOOTH_*
  case (N_ELEMENTS(colorsmooth_on)) of
    0:         colorsmooth_on_arr=REPLICATE(0, nrofviews)
    1:         colorsmooth_on_arr=REPLICATE(KEYWORD_SET(colorsmooth_on), nrofviews)
    nrofviews: colorsmooth_on_arr=colorsmooth_on
    else:      message, 'ERROR: Incorrect number of elements in keyword parameter COLORSMOOTH_ON'
  endcase
  (*sState)[*].iso.colorsmooth_on=colorsmooth_on_arr

  case (N_ELEMENTS(colorsmooth_lambda)) of
    0:         colorsmooth_lambda_arr=colorsmooth_on_arr * v3dpar_SmoothColorLambda_default
    1:         colorsmooth_lambda_arr=colorsmooth_on_arr * colorsmooth_lambda
    nrofviews: colorsmooth_lambda_arr=colorsmooth_on_arr * colorsmooth_lambda
    else: begin
        if (N_ELEMENTS(colorsmooth_on) ne 0) then begin
          message, 'ERROR: Incorrect number of elements in keyword parameter COLORSMOOTH_LAMBDA.'
        endif
      end
  endcase
  (*sState)[*].iso.colorsmooth_lambda=colorsmooth_lambda_arr

  case (N_ELEMENTS(colorsmooth_iterations)) of
    0:         colorsmooth_iterations_arr=colorsmooth_on_arr * v3dpar_SmoothColorIterations_default
    1:         colorsmooth_iterations_arr=colorsmooth_on_arr * colorsmooth_iterations
    nrofviews: colorsmooth_iterations_arr=colorsmooth_on_arr * colorsmooth_iterations
    else: begin
        if (N_ELEMENTS(colorsmooth_on) ne 0) then begin
          message, 'ERROR: Incorrect number of elements in keyword COLORSMOOTH_ITERATIONS.'
        endif
      end
  endcase
  (*sState)[*].iso.colorsmooth_iterations=colorsmooth_iterations_arr
  ; End checking keywords COLORSMOOTH_*

  ; Check keyword SMOOTH
  case (N_ELEMENTS(smooth_on)) of
    0:         smooth_on_arr=REPLICATE(0, nrofviews)
    1:         smooth_on_arr=REPLICATE(KEYWORD_SET(smooth_on), nrofviews)
    nrofviews: smooth_on_arr=smooth_on
    else:      message, 'ERROR: Incorrect number of elements in keyword parameter SMOOTH_ON'
  endcase
  (*sState)[*].iso.smooth_on=smooth_on_arr

  case (N_ELEMENTS(smooth_lambda)) of
    0:         smooth_lambda_arr=smooth_on_arr * v3dpar_SmoothLambda_default
    1:         smooth_lambda_arr=smooth_on_arr * smooth_lambda
    nrofviews: smooth_lambda_arr=smooth_on_arr * smooth_lambda
    else: begin
        if (N_ELEMENTS(smooth_on) ne 0) then begin
          message, 'ERROR: Incorrect number of elements in keyword parameter SMOOTH_LAMBDA.'
        endif
      end
  endcase
  (*sState)[*].iso.smooth_lambda=smooth_lambda_arr

  case (N_ELEMENTS(smooth_iterations)) of
    0:         smooth_iterations_arr=smooth_on_arr * v3dpar_SmoothIterations_default
    1:         smooth_iterations_arr=smooth_on_arr * smooth_iterations
    nrofviews: smooth_iterations_arr=smooth_on_arr * smooth_iterations
    else: begin
        if (N_ELEMENTS(smooth_on) ne 0) then begin
          message, 'ERROR: Incorrect number of elements in keyword parameter SMOOTH_ITERATIONS.'
        endif
      end
  endcase
  (*sState)[*].iso.smooth_iterations=smooth_iterations_arr
  ; END OF: SMOOTH Keyword checking

  ; keyword PALETTE=IDLpal:
  ;  IDLpal_arr will be used as the IDL palette index for each view.
  ; 
  ; if IDLpal has no elements => Use default palette 
  ; if IDLpal has 1 element   => all views get that palette
  ; otherwise, if number of elements in IDLpal are equal to number
  ; of views then each entry is a palette index for that view.
  ; If none of above are true exit with an error
  case N_ELEMENTS(IDLpal) of
    0:         IDLpal_arr=REPLICATE(v3dpar_Palette_default, nrofviews)
    1:         IDLpal_arr=REPLICATE(IDLpal, nrofviews)
    nrofviews: IDLpal_arr=IDLpal
    else:      message, 'ERROR: Incorrect number of values for keyword IDLPAL.'
  endcase
  (*sState)[*].color.idlpal=IDLpal_arr
  for i=0, nrofviews-1 do begin
    ; Set requested IDL palette 
    v3d_color_loadpalette, sState, i 
  endfor ; i

  ; START OF: PALETTERANGE keyword checking
  ; Is PALETTERANGE defined? If not use default
  ; Which is entire range of physical values on iso-surface
  ; Otherwise use given ranges for each view. 
  ; NOTE: To use default range for some views give [0,0] as 
  ;       the palette range for that view. 
  ; Use default value?
  if (N_ELEMENTS(paletterange) eq 0) then begin
    ; Default range is denoted by using zero in both min and max values of palette range.
    paletterange=MAKE_ARRAY(2, nrofviews, VALUE=0.0, /DOUBLE)
    ;
  ; Set palette ranges to full color ranges
    for i=0, nrofviews-1 do begin
  ;    if (((*sState)[i].color.palette_bottom eq 0) AND $
  ;        ((*sState)[i].color.palette_top    eq 0)    ) then begin
  ;      (*sState)[i].color.palette_bottom=(*sState)[i].box.fullcolorrange[0]
  ;      (*sState)[i].color.palette_top   =(*sState)[i].box.fullcolorrange[1]
       paletterange[0, i]=(*sState)[i].box.fullcolorrange[0]
       paletterange[1, i]=(*sState)[i].box.fullcolorrange[1]
  ;    endif 
    endfor ; i
  endif else begin
    if (nrofviews eq 1) then begin
      if (NOT ARRAY_EQUAL(size(paletterange, /DIMENSIONS), 2)) then begin
        message, 'ERROR: For one view only two elements should be in PALETTERANGE.'
      endif 
    endif else begin
      if (NOT ARRAY_EQUAL(SIZE(paletterange, /DIMENSIONS), [2,nrofviews])) then begin
        message, 'ERROR: Invalid number of PALETTERANGE entries.' 
      endif 
    endelse
  endelse 
  for i=0, nrofviews-1 do begin
    (*sState)[i].color.palette_bottom=paletterange[0, i]
    (*sState)[i].color.palette_top   =paletterange[1, i]
  endfor ; i
  ; END OF: PALETTERANGE keyword checking

  ; Checking keyword VIEWCOLOR
  if (N_ELEMENTS(viewcolor) eq 0) then begin
    viewcolor=v3dpar_ViewBackgroundColor_default
  endif else begin
    if ( (SIZE(viewcolor, /N_DIMENSIONS) ne 1) OR $
         (N_ELEMENTS(viewcolor) ne 3)            ) then begin
      message, 'ERROR: VIEWCOLOR should only have three color indices [R, G, B].'
    endif
  endelse
  for i=0, nrofviews-1 do begin
    (*sState)[i].color.view_color=viewcolor
  endfor ; i
  ; END checking keyword VIEWCOLOR

  ; Hardware versus software rendering
  if (N_ELEMENTS(renderer) ne 0) then begin
    (*sState)[*].renderer=renderer
  endif

  ; Projection method
  if (N_ELEMENTS(projection) eq 1) then begin
    ; Allow only values 1 and 2
    (*sState)[*].projection=1 > fix(projection) < 2
  endif

  ; Checking keyword TITLE
  if (N_ELEMENTS(title) ne 0) then begin
    (*sState)[*].title=title
  endif
  ; End checking keyword TITLE

  ; START OF: SCALE keyword checking
  if (N_ELEMENTS(scale) ne 0) then begin
    if (SIZE(scale, /N_DIMENSIONS) gt 0) then begin
      message, 'ERROR: SCALE should be a scalar, arrays not allowed.'
    endif
    (*sState)[*].scale=scale  ; Ensure it is passed by value
  endif else begin
    (*sState)[*].scale=v3dpar_Scale_default
  endelse
  ; END OF: SCALE keyword checking

  ; START OF: ROTATEANGLE, ROTATEAXIS, and ROTATEMATRIX keywords checking
  if (N_ELEMENTS(rotateaxis) ne 0) then begin
    if (NOT ARRAY_EQUAL(SIZE(rotateaxis, /DIMENSIONS), [3])) then begin
      message, 'ERROR: ROTATEAXIS has invalid value, should be a vector of 3 elements.'
    endif
    for i=0, nrofviews-1 do begin
      (*sState)[i].rotateaxis=rotateaxis
    endfor ; i
  endif
  if (N_ELEMENTS(rotateangle) ne 0) then begin
    if (SIZE(rotateangle, /N_DIMENSIONS) gt 0) then begin
      message, 'ERROR: ROTATEANGLE should be a scalar, arrays not allowed.'
    endif
    (*sState)[*].rotateangle=rotateangle
  endif
  if (N_ELEMENTS(rotatematrix) ne 0) then begin
    if (NOT ARRAY_EQUAL(size(rotatematrix, /DIMENSIONS), [4,4])) then begin
      message, 'ERROR: ROTATEMATRIX not correct'
    endif
    for i=0, nrofviews-1 do begin
      (*sState)[i].rotatematrix=rotatematrix
    endfor ; i
  endif
  ; END OF: ROTATEANGLE, ROTATEAXIS, and ROTATEMATRIX keywords checking

  ; START OF: XSIZE, YSIZE keywords checking
  ; Setting minimum and maximum display size
  ; NOTE: I can't use the X-Windows current display size since on some computers
  ; with multiple monitors it reports total screen dimension (over all monitors)
  ; but neverthelees it isn't garanteed that the 3D will work on such a large window
  ; (on HPUX it generates errors).
  if KEYWORD_SET(xsize) then begin
    (*sState)[*].viewxsize=v3dpar_xsize_min > xsize < v3dpar_xsize_max
  endif else begin
    (*sState)[*].viewxsize=v3dpar_xsize_default
  endelse
  if KEYWORD_SET(ysize) then begin
    (*sState)[*].viewysize=v3dpar_ysize_min > ysize < v3dpar_ysize_max
  endif else begin
    (*sState)[*].viewysize=v3dpar_ysize_default
  endelse
  ; END OF: XSIZE, YSIZE keywords checking

  ; START OF: SHOWAXIS keyword checking,
  ; hideaxis will be set to logical inverse of showaxis
  (*sState)[*].hideaxis=(KEYWORD_SET(showaxis)) ? 0 : v3dpar_box_hide
  ; END OF: SHOWAXIS keyword checking

  ; GUI
  (*sState)[*].no_controls=KEYWORD_SET(no_controls)
  (*sState)[*].no_menu    =KEYWORD_SET(no_menu)
  (*sState)[*].no_colorbar=KEYWORD_SET(no_colorbar)
  ; simple_GUI sets all keywords that simplify the user interface
  if (KEYWORD_SET(simple_GUI)) then begin
    (*sState)[*].no_controls=1
    (*sState)[*].no_menu    =1
    (*sState)[*].no_colorbar=1
  endif

  ; For (possible) 3rd and 4th view: hide controls initially
  if (nrofviews ge 3) then begin
    (*sState)[2:nrofviews-1].hideControlColor=1
    (*sState)[2:nrofviews-1].hideControlIso  =1
  endif

  ; By now, all input data should be in sState and nrofviews,
  ; exceptions are the export_XXX keywords.

  if (NOT KEYWORD_SET(export_on)) then begin
    ; === Create and realize widgets and menus, create objects, start xmanager ==
    ;
    v3d_start, sState, /INITIALIZE
    ;
  endif else begin 
    ; === EXPORT mode is on, export images to disk =============================
    ;
    ; === Create object hierarchy for all views ================================
    ;
    v3d_createobjects, sState, EXPORT_ON=export_on
    ;
    ; In export mode don't realize the widgets and don't start xmanager.
    ;
    v3d_exportimage, sState, nrofviews, $
                     NO_COLORBAR=      no_colorbar, $
                     EXPORT_FILENAME=  export_filename, $
                     EXPORT_FILENR=    export_filenr, $
                     EXPORT_PATH=      export_path, $
                     EXPORT_TYPE=      export_type, $
                     EXPORT_DIMENSIONS=export_dimensions, $
                     EXPORT_SHOW=      export_show, $
                     EXPORT_GETIMAGE=  export_getimage
    ;
    ; Free heap variables
    v3d_sstate_freemem, sState
    ;
  endelse ; End of Export images

end ; vis3d
