pro vis3d_addct, file=file
;-------------------------------------------------------------------------------
; NAME:
;   vis3d_addct ('vis3d_add_color_table')
;
; PURPOSE:
;   Put additional color palettes into specified file.
;
; CALLING SEQUENCE:
;   vis3d_addct[, file=file]
;
; INPUT:
;   None
;
; OUTPUT:
;   None
;
; KEYWORDS:
;   file:   ('file') string with name of file to be modified (default 'colors1.tbl')
;
; PROCEDURE:
;   Straightforward
;
; SIDE EFFECTS:
;   A file is modified
;
; EXAMPLE:
;   ; --- Copy the IDL system color palette file into working directory and extend it.
;   $ cp /usr/local/rsi/idl_6.3/resource/colors/colors1.tbl .
;   ; $ cp /usr/local/itt/idl/resource/colors/colors1.tbl .
;   $ ls -og colors1.tbl
;   vis3d_addct, file='colors1.tbl'
;   $ ls -og colors1.tbl
;   xloadct, file='colors1.tbl'
;
; HISTORY:
;   2006-07-07 Written by B.F.
;   2012-02-10 (B.F.) Double-Linear-White palettes addes
;-------------------------------------------------------------------------------

compile_opt hidden, strictarr

if (not keyword_set(file)) then begin
  file='colors1.tbl'
endif

ZeroConst        =bytarr(256)
OneConst         =bytarr(256)+255B
RampLinear_up    =bindgen(256)
RampLinear_do    =255B-bindgen(256)

RampLinearHalf_up=[bytarr(128)         ,1B+2B*bindgen(128)]
RampLinearHalf_do=reverse(RampLinearHalf_up)
RampLinearHalf_du=RampLinearHalf_do + RampLinearHalf_up
RampLinearHalf_ud=255B-RampLinearHalf_du
RampLinearHalf_ip=255B-RampLinearHalf_do
RampLinearHalf_di=255B-RampLinearHalf_up

RampLinearTwoThirds_up=[bytarr(84)         ,byte(255.0/171.0*findgen(172))]
RampLinearTwoThirds_do=reverse(RampLinearTwoThirds_up)

RampSqrtHalf_up  =[bytarr(128)         ,byte(sqrt((1.0+2.0*findgen(128))/255.0)  *255.0)]
RampSqrtHalf_do  =reverse(RampSqrtHalf_up)

RampSquaHalf_up  =[bytarr(128)         ,byte(    ((1.0+2.0*findgen(128))/255.0)^2*255.0)]
RampSquaHalf_do  =reverse(RampSquaHalf_up)
RampSquaHalf_du  =RampSquaHalf_do + RampSquaHalf_up
RampSquaHalf_ud  =[byte(((1.0+2.0*findgen(128))/255.0)^2*255.0), bytarr(128)]
RampSquaHalf_ud  =RampSquaHalf_ud+reverse(RampSquaHalf_ud)

;RampCir1Half_up  =[bytarr(128)         ,byte(    sqrt(1.0-(1.0- findgen(128)/127.0)^2)   *255.0)]
RampCir1Half_up  =[bytarr(128)         ,byte(    sqrt(1.0-(1.0- (1.0+findgen(128))/128.0)^2)   *255.0)]
RampCir1Half_do  =reverse(RampCir1Half_up)
RampCir2Half_up  =[bytarr(128)         ,byte((1.0-sqrt(     1.0-(findgen(128)/127.0)^2))  *255.0)]
;RampCir2Half_up  =[bytarr(128)         ,byte((1.0-sqrt(     1.0-((1.0+findgen(128))/128.0)^2))  *255.0)]
RampCir2Half_do  =reverse(RampCir2Half_up)
RampCir2Half_du  =RampCir2Half_do + RampCir2Half_up
RampCir2Half_ud  =[byte((1.0-sqrt(     1.0-(findgen(128)/127.0)^2))  *255.0), bytarr(128)]
RampCir2Half_ud  =RampCir2Half_ud+reverse(RampCir2Half_ud)

RampLeft_1  =byte(interpol(float([255, 255, 170,  90,   0,   0,  40,   0,   0]), $
                           float([  0,  35,  95, 122, 128, 160, 180, 220, 255]),findgen(256)))
RampRight_1 =reverse(RampLeft_1)
RampCenter_1=byte(interpol(float([210,  70,  20,  0,  50,  50,   0,  20,  70, 210]), $
                           float([  0,  18,  35, 90, 127, 128, 165, 220, 237, 255]),findgen(256)))

ict=40

prefix='v3d: '

name='RampLin: '

ict=ict+1<255
modifyct, ict, prefix + name + 'White',   RampLinear_up, RampLinear_up, RampLinear_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red',     RampLinear_up, ZeroConst,     ZeroConst,     file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Yellow',  RampLinear_up, RampLinear_up, ZeroConst,     file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Green',   ZeroConst,     RampLinear_up, ZeroConst,     file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Cyan',    ZeroConst,     RampLinear_up, RampLinear_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Blue',    ZeroConst,     ZeroConst,     RampLinear_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Magenta', RampLinear_up, ZeroConst,     RampLinear_up, file=file

name='RampDubLinB: '

ict=ict+1<255
modifyct, ict, prefix + name + 'White-White',     RampLinearHalf_du, RampLinearHalf_du,  RampLinearHalf_du, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Red',       RampLinearHalf_du, RampLinearHalf_do,  RampLinearHalf_do, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Yellow',    RampLinearHalf_du, RampLinearHalf_du,  RampLinearHalf_do, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Green',     RampLinearHalf_do, RampLinearHalf_du,  RampLinearHalf_do, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Cyan',      RampLinearHalf_do, RampLinearHalf_du,  RampLinearHalf_du, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Blue',      RampLinearHalf_do, RampLinearHalf_do,  RampLinearHalf_du, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'White-Magenta',   RampLinearHalf_du, RampLinearHalf_do,  RampLinearHalf_du, file=file

ict=ict+1<255
modifyct, ict, prefix + name + 'Red-White',       RampLinearHalf_du, RampLinearHalf_up,  RampLinearHalf_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Red',         RampLinearHalf_du, ZeroConst,          ZeroConst,         file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Yellow',      RampLinearHalf_du, RampLinearHalf_up,  ZeroConst,         file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Green',       RampLinearHalf_do, RampLinearHalf_up,  ZeroConst,         file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Cyan',        RampLinearHalf_do, RampLinearHalf_up,  RampLinearHalf_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampLinearHalf_do, ZeroConst,          RampLinearHalf_up, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Magenta',     RampLinearHalf_du, ZeroConst,          RampLinearHalf_up, file=file

name='RampDubSqrt: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampSqrtHalf_do  , ZeroConst,          RampSqrtHalf_up,   file=file

name='RampDubX1: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampLinearHalf_do, RampLinearHalf_ud/3,RampLinearHalf_up, file=file

name='RampDubX2: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampSqrtHalf_do, RampSquaHalf_du/2,    RampSqrtHalf_up, file=file

name='RampDubX3: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampSqrtHalf_do, RampSquaHalf_du/2+64B,RampSqrtHalf_up, file=file

name='RampDubX4: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampSqrtHalf_do, RampSquaHalf_du/2+RampSquaHalf_ud/3, $
                                                                                         RampSqrtHalf_up, file=file
name='RampDubX5: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampCir1Half_do, RampCir2Half_du+RampCir2Half_ud/4, $
                                                                                         RampCir1Half_up, file=file
name='RampDubX6: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampLeft_1, RampCenter_1, RampRight_1, file=file

name='RampDubX7: '
ict=ict+1<255
;modifyct, ict, prefix + name + 'Red-Blue',        RampLinearTwoThirds_do, 65+RampCir2Half_du/2 , RampLinearTwoThirds_up, file=file
;modifyct, ict, prefix + name + 'Red-Blue',   RampLinear_do, 127+RampCir2Half_du/4, RampLinear_up, file=file
;modifyct, ict, prefix + name + 'Red-Blue',        byte(255.0*(RampLinearTwoThirds_do/255.0)^0.8), $
;                                                  28+RampCir2Half_du/6+RampCir2Half_ud/4, $
;                                                  byte(255.0*(RampLinearTwoThirds_up/255.0)^0.8), file=file
red =byte(255.0*(RampLinearTwoThirds_do/255.0)^1.0)
blue=byte(255.0*(RampLinearTwoThirds_up/255.0)^1.0)
modifyct, ict, prefix + name + 'Red-Blue',        red, $
                                                  ((-5+3*red[127]-(red+blue)) > 0) + RampCir2Half_du/4,  $
                                                  blue, file=file
name='RampDubX8: '
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampLinearHalf_do, RampSquaHalf_ud,RampLinearHalf_up, file=file


name='RampDubLinW: '

ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Black',       RampLinearHalf_di, RampLinearHalf_ud,  RampLinearHalf_ud, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Red',         OneConst         , RampLinearHalf_ud,  RampLinearHalf_ud, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Yellow',      OneConst         , RampLinearHalf_ip,  RampLinearHalf_ud, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Green',       RampLinearHalf_di, RampLinearHalf_ip,  RampLinearHalf_ud, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Cyan',        RampLinearHalf_di, RampLinearHalf_ip,  RampLinearHalf_ip, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Blue',        RampLinearHalf_di, RampLinearHalf_ud,  RampLinearHalf_ip, file=file
ict=ict+1<255
modifyct, ict, prefix + name + 'Red-Magenta',     OneConst         , RampLinearHalf_ud,  RampLinearHalf_ip, file=file


end ; vis3d_addct
