;+
; NAME:
;   vis3d_demo ('visualization_3_dimensions_demonstration')
;
; PURPOSE:
;   Sample script to demonstrate the use of the vis3d package by Sigurdur Finnsson.
;
; CALLING SEQUENCE:
;   @vis3d_demo
; 
; PROCEDURE:
;   This script produces sample data cubes and calls vis3d to visualize them.
;
;   Once the vis3d window is there (and all data is loaded), you can left-click
;   in the image-panel and move the mouse (mouse button still down) to
;   rotate the object.
;
;   Try to change the value of the iso surface displayed in the "ISO" field.
;   The changes only become active after the click on the "Apply: Mesh" button.
;
;   After modifying the iso surface you might adjust the color palette by clicking
;   on the "Full" button. Or you specify the palette range explicitely with
;   "PalMax" and "PalMin". The changes become active after clicking on the
;   "Apply" button.
;
;   Depending on the size of the data sets, the number of views, and the
;   support for 3D acceleration on your computer the rotation of the objects
;   can be very slow. The trick in vis3d is to use a coarse mesh during the
;   rotation and to use the full mesh if the mouse button is released. The coarseness
;   can be adjusted with the "Dec" field. It accepts the decimation factor
;   between the grids in %. The default is 5%. A Value of 100% means no decimation.
;
; EXAMPLE:
;   @vis3d_demo
; 
;-

; --- Edge length of data cube ---
n=50

; --- Axis vector, cube with radius values and size n x n x n ---
axis=2.0*findgen(n)/float(n-1)-1.0
x=rebin(reform(axis, n,1,1), n,n,n)
y=rebin(reform(axis, 1,n,1), n,n,n)
z=rebin(reform(axis, 1,1,n), n,n,n)
r=sqrt(x^2+y^2+z^2)

; --- Sample data array (d1) and color array (c1) ---
;d1=sqrt(x^2+0.8*y^2+1.5*z^2)
;d1=r+0.1*cos(6.0*x)+0.05*cos(25.0*y)
d1=r+0.1*cos(6.0*x)+0.05*cos(20.0*y)+0.03*cos(35.0*z)
;d1=sqrt(sqrt(exp(-(r/0.3)^2)*cos(12.0*x)^2*cos(12.0*y)^2*cos(12.0*z)^2))
c1=r

v=fltarr(n,n,n,3)
v[*,*,*,0]=cos(4.0*x)
v[*,*,*,1]=cos(6.0*y)
v[*,*,*,2]=cos(8.0*z)

delvar, x,y,z, r

; === Visualize ===

; --- Most simple call: one data array, one color array, no keywords ---
;vis3d, d1,c1

; --- Increase grid quality during rotation ---
;vis3d, d1,c1, decimation=[20]

; --- Large window, no decimation ---
;vis3d, d1,c1, xsize=800, ysize=600, decimation=[100]

; --- 2 array pairs: data and color arrays swapped ---
;vis3d, d1,c1, c1,d1

; --- 2 array pairs: second pair has vector field ---
vis3d, d1,c1, v, c1

; --- 2 array pairs: data and color arrays swapped ---
;vis3d, d1,c1, c1,d1, decimation=[20,50]

; --- 2 array pairs: same data and color, different iso values ---
;vis3d, d1,c1, d1,c1, isovalue=[0.8,0.95]

; --- 2 array pairs: data and color arrays swapped; colors and lights... ---
;vis3d, d1,c1, c1,d1, decimation=[20,50], idlpal=[3,6], viewcolor=[0,150,150], lights=[0,1]

; --- 2 array pairs: data and color arrays swapped; larger, no colorbar ---
;vis3d, d1,c1, c1,d1, decimation=[20,50], /no_colorbar, xsize=500, ysize=500

; --- 2 array pairs: data and color arrays swapped; larger, no decimation ---
;vis3d, d1,c1, c1,d1, decimation=[100,100], xsize=500, ysize=500

; --- 4 array pairs: same data and color, different iso values ---
;vis3d, d1,c1, d1,c1, d1,c1, d1,c1, isovalue=[0.4,0.6,0.8,0.95], /no_colorbar, xsize=300, ysize=300
