program matrix
!
!------------------------------------------------------------------------------
use matinv_gj_module                 ! --- Module for matrix inversion ---
use bicubic_module                   ! --- Module for bicubic interpolation ---
!
implicit none
!
integer, parameter                             :: rk=kind(1.0)
integer, parameter                             :: n=8
real(kind=rk), dimension(n,n)                  :: A, Ainv, Amult
integer                                        :: i, j
!
real(kind=rk), dimension(1:2)                  :: x1, x2
real(kind=rk), dimension(1:2,1:2)              :: y, y1, y2, y12
real(kind=rk), dimension(1:4,1:4)              :: C
integer                                        :: i1, i2
real(kind=rk)                                  :: x1i, x2i, yi, y1i, y2i
!
!------------------------------------------------------------------------------
!
! --- Initialize matrix a ---
do i=1,n
  do j=1,n
    call random_number(A(i,j))
    if (i == j) A(i,j)=10.0_rk*j
  end do ! j
end do ! i
write(6,*) 'A='
do j=1,n
  write(6,*) (A(i,j),i=1,n)
end do ! j
write(6,*)
!
! --- Invert matrix ---
Ainv=A
call matinv_gj(Ainv)
write(6,*) 'Ainv='
do j=1,n
  write(6,*) (Ainv(i,j),i=1,n)
end do ! j
write(6,*)
!
! --- Test inversion ---
write(6,*) 'A*Ainv='
Amult=matmul(A,Ainv)
do j=1,n
  write(6,*) (Amult(i,j),i=1,n)
end do ! j
write(6,*)
!
! =============================================================================
!
! --- Define function values in cell ---
write(6,*) 'Define function values'
x1(1)=0.0
x1(2)=1.0
x2(1)=0.5
x2(2)=1.2
do i1=1,2
  do i2=1,2
    y(  i1,i2)=                sin(x1(i1)*x2(i2))
    y1( i1,i2)=       x2(i2)*( cos(x1(i1)*x2(i2)))
    y2( i1,i2)=       x1(i1)*( cos(x1(i1)*x2(i2)))
    y12(i1,i2)=x1(i1)*x2(i2)*(-sin(x1(i1)*x2(i2))) + cos(x1(i1)*x2(i2))
!    y(  i1,i2)=x2(i2)
!    y1( i1,i2)=0._rk
!    y2( i1,i2)=1._rk
!    y12(i1,i2)=0._rk
  end do ! i2
end do ! i1
!
! --- Compute coefficients ---
write(6,*) 'Compute coefficients'
call bicubic_coeff_1(x1,x2,y,y1,y2,y12, C)
!
do
  write(6,'(A)',advance='no') 'x1i, x2i? '
  read(5,*) x1i, x2i
  if (x1i == -1000.) exit
  call bicubic_interpol_1(x1i,x2i,C, yi,y1i,y2i)
  write(6,*) 'y , yi =',sin(x1i*x2i),yi
  write(6,*) 'y1, y1i=',x2i*( cos(x1i*x2i)),y1i
  write(6,*) 'y2, y2i=',x1i*( cos(x1i*x2i)),y2i
end do
!
end program matrix
