function ddot(n, dx, incx, dy, incy) 
!---------------------------------------------------------------------
!                                                                       
!     forms the dot product of two vectors.                             
!     uses unrolled loops for increments equal to one.                  
!     jack dongarra, linpack, 3/11/78.                                  
!     modified 12/3/93, array(1) declarations changed to array(*)       
!                                                                       
!---------------------------------------------------------------------
!
implicit none 
!
! --- I/O ---
integer,                          intent(in)    :: incx, incy, n 
double precision, dimension(1:n), intent(inout) :: dx, dy
double precision                                :: ddot
!
! --- internal variables ---
integer          :: i,ix,iy,m,mp1
double precision :: dtemp
!
!---------------------------------------------------------------------
!
ddot = 0.0d0 
dtemp = 0.0d0 
if(n.le.0)return 

if(incx.eq.1.and.incy.eq.1)go to 20 
!                                                                       
! --- code for unequal increments or equal increments ---
!          not equal to 1                                               
!                                                                       
   ix = 1 
   iy = 1 
   if(incx.lt.0)ix = (-n+1)*incx + 1 
   if(incy.lt.0)iy = (-n+1)*incy + 1 

   do i = 1,n 
     dtemp = dtemp + dx(ix)*dy(iy) 
     ix = ix + incx 
     iy = iy + incy 
   end do
   ddot = dtemp 
   return 
!                                                                       
! --- code for both increments equal to 1 ---
!                                                                       
!                                                                       
!        clean-up loop                                                  
!                                                                       
   20 m = mod(n,5) 
      if( m .eq. 0 ) go to 40 
      do 30 i = 1,m 
        dtemp = dtemp + dx(i)*dy(i) 
   30 continue 
      if( n .lt. 5 ) go to 60 
   40 mp1 = m + 1 
      do 50 i = mp1,n,5 
        dtemp = dtemp + dx(i)     * dy(i)     + & 
                        dx(i + 1) * dy(i + 1) + &
                        dx(i + 2) * dy(i + 2) + & 
                        dx(i + 3) * dy(i + 3) + & 
                        dx(i + 4) * dy(i + 4)
   50 continue 
   60 ddot = dtemp 
      return                                           
end function ddot
!---------------------------------------------------------------------

