subroutine dgefa(a,lda,n,ipvt,info) 
!---------------------------------------------------------------------
!
!     dgefa factors a double precision matrix by gaussian elimination.  
!                                                                       
!     dgefa is usually called by dgeco, but it can be called            
!     directly with a saving in time if  rcond  is not needed.          
!     (time for dgeco) = (1 + 9/n)*(time for dgefa) .                   
!                                                                       
!     on entry                                                          
!                                                                       
!        a       double precision(lda, n)                               
!                the matrix to be factored.                             
!                                                                       
!        lda     integer                                                
!                the leading dimension of the array  a .                
!                                                                       
!        n       integer                                                
!                the order of the matrix  a .                           
!                                                                       
!     on return                                                         
!                                                                       
!        a       an upper triangular matrix and the multipliers         
!                which were used to obtain it.                          
!                the factorization can be written  a = l*u  where       
!                l  is a product of permutation and unit lower          
!                triangular matrices and  u  is upper triangular.       
!                                                                       
!        ipvt    integer(n)                                             
!                an integer vector of pivot indices.                    
!                                                                       
!        info    integer                                                
!                = 0  normal value.                                     
!                = k  if  u(k,k) .eq. 0.0 .  this is not an error       
!                     condition for this subroutine, but it does        
!                     indicate that dgesl or dgedi will divide by zero  
!                     if called.  use  rcond  in dgeco for a reliable   
!                     indication of singularity.                        
!                                                                       
!     linpack. this version dated 08/14/78 .                            
!     cleve moler, university of new mexico, argonne national lab.      
!                                                                       
!     subroutines and functions                                         
!                                                                       
!     blas daxpy,dscal,idamax                                           
!                                                                       
!---------------------------------------------------------------------
!
implicit none 
!
! --- I/O ---                                                          
integer,                                intent(in)    :: lda, n
integer,                                intent(out)   :: info
integer,          dimension( 1:n),      intent(out)   :: ipvt
double precision, dimension(1:lda,1:n), intent(inout) :: a
!                                                                       
# ifdef rhd_la_tune01 
integer :: idamaxres
# else 
integer :: idamax
# endif
!
! --- internal variables ---
double precision :: t 
integer          :: j, k, kp1, l, nm1 
!
!---------------------------------------------------------------------
!                                                                       
! --- gaussian elimination with partial pivoting ---
!                                                                       
      info = 0 
      nm1 = n - 1 
      if (nm1 .lt. 1) go to 70 
      do 60 k = 1, nm1 
         kp1 = k + 1 
!                                                                       
!        find l = pivot index                                           
!                                                                       
# ifdef rhd_la_tune01
         call idamax(idamaxres, n-k+1,a(k,k),1)
         l = idamaxres + k - 1 
# else 
         l = idamax(n-k+1,a(k,k),1) + k - 1 
# endif
         ipvt(k) = l 
!                                                                       
!        zero pivot implies this column already triangularized          
!                                                                       
         if (a(l,k) .eq. 0.0d0) go to 40 
!                                                                       
!           interchange if necessary                                    
!                                                                       
            if (l .eq. k) go to 10 
               t = a(l,k) 
               a(l,k) = a(k,k) 
               a(k,k) = t 
   10       continue 
!                                                                       
!           compute multipliers                                         
!                                                                       
            t = -1.0d0/a(k,k) 
            call dscal(n-k,t,a(k+1,k),1) 
!                                                                       
!           row elimination with column indexing                        
!                                          
            do 30 j = kp1, n 
               t = a(l,j) 
               if (l .eq. k) go to 20 
                  a(l,j) = a(k,j) 
                  a(k,j) = t 
   20          continue 
               call daxpy(n-k,t,a(k+1,k),1,a(k+1,j),1) 
   30       continue 
         go to 50 
   40    continue 
            info = k 
   50    continue 
   60 continue 
   70 continue 
      ipvt(n) = n 
      if (a(n,n) .eq. 0.0d0) info = n 
      return 
end subroutine dgefa                                         
!---------------------------------------------------------------------
