subroutine dgesl(a, lda, n, ipvt, b, job) 
!---------------------------------------------------------------------
!                                                                       
!     dgesl solves the double precision system                          
!     a * x = b  or  trans(a) * x = b                                   
!     using the factors computed by dgeco or dgefa.                     
!                                                                       
!     on entry                                                          
!                                                                       
!        a       double precision(lda, n)                               
!                the output from dgeco or dgefa.                        
!                                                                       
!        lda     integer                                                
!                the leading dimension of the array  a .                
!                                                                       
!        n       integer                                                
!                the order of the matrix  a .                           
!                                                                       
!        ipvt    integer(n)                                             
!                the pivot vector from dgeco or dgefa.                  
!                                                                       
!        b       double precision(n)                                    
!                the right hand side vector.                            
!                                                                       
!        job     integer                                                
!                = 0         to solve  a*x = b ,                        
!                = nonzero   to solve  trans(a)*x = b  where            
!                            trans(a)  is the transpose.                
!                                                                       
!     on return                                                         
!                                                                       
!        b       the solution vector  x .                               
!                                                                       
!     error condition                                                   
!                                                                       
!        a division by zero will occur if the input factor contains a   
!        zero on the diagonal.  technically this indicates singularity  
!        but it is often caused by improper arguments or improper       
!        setting of lda .  it will not occur if the subroutines are     
!        called correctly and if dgeco has set rcond .gt. 0.0           
!        or dgefa has set info .eq. 0 .                                 
!                                                                       
!     to compute  inverse(a) * c  where  c  is a matrix                 
!     with  p  columns                                                  
!           call dgeco(a,lda,n,ipvt,rcond,z)                            
!           if (rcond is too small) go to ...                           
!           do 10 j = 1, p                                              
!              call dgesl(a,lda,n,ipvt,c(1,j),0)                        
!        10 continue                                                    
!                                                                       
!     linpack. this version dated 08/14/78 .                            
!     cleve moler, university of new mexico, argonne national lab.      
!                                                                       
!     subroutines and functions                                         
!                                                                       
!     blas daxpy,ddot                                                   
!                                                                       
!---------------------------------------------------------------------
!
implicit none 
!
! --- I/O ---
integer,                              intent(in)  :: lda, n, job
integer,          dimension(1:n),     intent(in)  :: ipvt
double precision, dimension(lda,1:n), intent(in)  :: a
double precision, dimension(1:n),     intent(out) :: b
!
! --- internal variables ---
integer          :: k,kb,l,nm1 
double precision :: t, ddot
!
!---------------------------------------------------------------------
!                                                                       
      nm1 = n - 1 
      if (job .ne. 0) go to 50 
!                                                                       
!        job = 0 , solve  a * x = b                                     
!        first solve  l*y = b                                           
!                                                                       
         if (nm1 .lt. 1) go to 30 
         do 20 k = 1, nm1 
            l = ipvt(k) 
            t = b(l) 
            if (l .eq. k) go to 10 
               b(l) = b(k) 
               b(k) = t 
   10       continue 
            call daxpy(n-k,t,a(k+1,k),1,b(k+1),1) 
   20    continue 
   30    continue 
!                                                                       
!        now solve  u*x = y                                             
!                                                                       
         do 40 kb = 1, n 
            k = n + 1 - kb 
            b(k) = b(k)/a(k,k) 
            t = -b(k) 
            call daxpy(k-1,t,a(1,k),1,b(1),1) 
   40    continue 
      go to 100 
   50 continue 
!                                                                       
!        job = nonzero, solve  trans(a) * x = b                         
!        first solve  trans(u)*y = b                                    
!                                                                       
         do 60 k = 1, n 
            t = ddot(k-1,a(1,k),1,b(1),1) 
            b(k) = (b(k) - t)/a(k,k) 
   60    continue 
!                                                                       
!        now solve trans(l)*x = y                                       
!                                                                       
         if (nm1 .lt. 1) go to 90 
         do 80 kb = 1, nm1 
            k = n - kb 
            b(k) = b(k) + ddot(n-k,a(k+1,k),1,b(k+1),1) 
            l = ipvt(k) 
            if (l .eq. k) go to 70 
               t = b(l) 
               b(l) = b(k) 
               b(k) = t 
   70       continue 
   80    continue 
   90    continue 
  100 continue 
      return 
end subroutine dgesl                                          
