!---------------------------------------------------------------------
! --- The routine idamax is available as function or as subroutine.
!     Use switch rhd_la_tune01 to choose.
!     The default function is used if the switch is not set.
!---------------------------------------------------------------------
!
# ifdef rhd_la_tune01
!
subroutine idamax(idamaxarg, n,dx,incx) 
!---------------------------------------------------------------------
!                                                                       
!     finds the index of element having max. absolute value.            
!     jack dongarra, linpack, 3/11/78.                                  
!     modified 3/93 to return if incx .le. 0.                           
!     modified 12/3/93, array(1) declarations changed to array(*)       
!
!---------------------------------------------------------------------
!
implicit none 
!
! --- I/O ---
integer idamaxarg
integer,                          intent(in)    :: incx,n 
double precision, dimension(1:n), intent(inout) :: dx
!
! --- internal variables ---
integer          :: i,ix
double precision :: dmax 
!
!---------------------------------------------------------------------
!                                                                       
idamaxarg = 0 
!
if ((n .lt. 1) .or. (incx .le. 0)) return 
!
idamaxarg = 1 
!
if (n == 1) return 
!
if (incx == 1) then 
   !
   ! --- code for increment equal to 1 ---
   !
   dmax = abs(dx(1)) 
   do i = 2,n 
      if (abs(dx(i)) .gt. dmax) then
         idamaxarg = i 
         dmax = abs(dx(i)) 
      end if
   end do
else 
   !
   ! --- code for increment not equal to 1 ---
   !
   ix = 1 
   dmax = abs(dx(1)) 
   ix = ix + incx 
   !
   do i = 2,n 
      if (abs(dx(ix)) .gt. dmax) then 
         idamaxarg = i 
         dmax = abs(dx(ix)) 
      end if
      ix = ix + incx 
   end do
end if
!
return 
!
end subroutine idamax                                          
!---------------------------------------------------------------------
!
# else 
!
!---------------------------------------------------------------------
integer function idamax(n,dx,incx) 
!---------------------------------------------------------------------
!                                                                       
!     finds the index of element having max. absolute value.            
!     jack dongarra, linpack, 3/11/78.                                  
!     modified 3/93 to return if incx .le. 0.                           
!     modified 12/3/93, array(1) declarations changed to array(*)       
!                                                                       
!---------------------------------------------------------------------
!
implicit none 
!
! --- I/O ---
integer,                          intent(in)    :: incx,n 
double precision, dimension(1:n), intent(inout) :: dx
!
! --- internal variables ---
integer          :: i,ix
double precision :: dmax 
!
!---------------------------------------------------------------------
!                                                                       
idamax = 0 
!
if ((n .lt. 1) .or. (incx .le. 0)) return 
!
idamax = 1 
!
if (n == 1) return 
!
if (incx == 1) then 
   !
   ! --- code for increment equal to 1 ---
   !
   dmax = abs(dx(1)) 
   do i = 2,n 
      if (abs(dx(i)) .gt. dmax) then
         idamax = i 
         dmax = abs(dx(i)) 
      end if
   end do
else 
   !
   ! --- code for increment not equal to 1 ---
   !
   ix = 1 
   dmax = abs(dx(1)) 
   ix = ix + incx 
   !
   do i = 2,n 
      if (abs(dx(ix)) .gt. dmax) then 
         idamax = i 
         dmax = abs(dx(ix)) 
      end if
      ix = ix + incx 
   end do
end if
!
return 
!
end function idamax          
!
!---------------------------------------------------------------------
#endif                                
!---------------------------------------------------------------------
