!--- 2D/3D radiative transfer module ( M.S., B.F., HGL. 16-Dec-1997 20:19:16 )
!--- Last modification: 2017 Oct 26 by M. Steffen
!---------**********------------------------------------------------+-------
   MODULE comrd0_def
!
   IMPLICIT NONE
   SAVE
!
   INTEGER, PARAMETER :: k8=SELECTED_REAL_KIND(12)
!
   END MODULE comrd0_def
!-------------**********--------------------------------------------+-------

!---------**********------------------------------------------------+-------
   MODULE comrd1_def
!
   USE comrd0_def
!
   IMPLICIT NONE
   SAVE
!
   REAL (kind=k8), PARAMETER  :: pi_drk = 3.1415926535897932384626433D0
!
!  --- Define fundamental constants ---
   REAL, PARAMETER :: PI=pi_drk, PI05=0.5D0*pi_drk, PI15=1.5D0*pi_drk, &
                      PI20=2.0D0*pi_drk, PI40=4.0D0*pi_drk, &
                      SIGMA=5.6705E-5, C5=SIGMA/PI, C8=8.0D0*pi_drk/3.0D0
!
   END MODULE comrd1_def
!-------------**********--------------------------------------------+-------

!---------**********------------------------------------------------+-------
   MODULE comrd2_def
!
   USE comrd0_def
!
   IMPLICIT NONE
   SAVE
!
   INTEGER,       PARAMETER :: ksbmax=4, maxomu=4, nphmax=6, nthmax=5 
!
   INTEGER                                    :: nfifac
   INTEGER,       DIMENSION(0:nthmax)         :: NFILL, NSTEP, KRMAX
   INTEGER,       DIMENSION(:,:), POINTER     :: JK
   REAL(KIND=k8)                              :: FXY, WXY
   REAL(KIND=k8), DIMENSION(0:nthmax)         :: CTH, WTH
   REAL(KIND=k8), DIMENSION(nphmax)           :: SPH, WPH
   REAL(KIND=k8), DIMENSION(:,:), POINTER     :: DSRAY, DXR
   REAL(KIND=k8), DIMENSION(maxomu)           :: WN1, WN2
   REAL(KIND=k8), DIMENSION(:,:), POINTER     :: WJ1, WJ2
!
   END MODULE comrd2_def
!-------------**********--------------------------------------------+-------

!---------**********------------------------------------------------+-------
   MODULE comrd3_def
!
   IMPLICIT NONE
   SAVE
!
   INTEGER            :: nthick, ntheta, nphi, ksub
   INTEGER, PARAMETER :: IJMB=1, nthin=4, MRAD=1, MRAD_1=0, MRAD_2=0
!
!  MRAD   = 0:   emergent intensities only
!  MRAD   = 1:   emergent intensities and div Frad
!  MRAD   = 2:   emergent intensities and 4*pi*(J) [Meth.1]
!
!  MRAD_1 = 0: linear           interpolation of source function in rsys2d
!  MRAD_1 = 1: opacity-weighted interpolation of source function in rsys2d
!
!  MRAD_2 = 1: emergent intensities and 4*pi*(J) [Meth.1] (only for MRAD=2)
!  MRAD_2 = 2: emergent intensities and 4*pi*(J) [Meth.2] (only for MRAD=2)
!
   END MODULE comrd3_def
!-------------**********--------------------------------------------+-------

!---------**********------------------------------------------------+-------
   MODULE comrd4_def
!
   USE comrd0_def
!
   IMPLICIT NONE
   SAVE
!
   INTEGER                            :: m1, n1, m2, n2, m3, n3
   INTEGER                            :: i1max, i2max, iimax 
   INTEGER                            :: jmin, jmax, kmax, mmmax
   INTEGER, DIMENSION(:), POINTER     :: I0X, I0Y, I1X, I1Y
   REAL                               :: DELR, DCX, DCY
   REAL,    DIMENSION(:), POINTER     :: AFAC, DELTAZ, DELZ, DCZ, &
                                         GFAC, WTFAC, W1FAC, W2FAC, Z
   REAL(KIND=k8)                      :: A12
!
   END MODULE comrd4_def
!-------------**********--------------------------------------------+-------

!---------**********------------------------------------------------+-------
   MODULE comrd5_def
!
   USE comrd0_def
!
   IMPLICIT NONE
   SAVE
!
   INTEGER       :: mradbc, mthint
   REAL          :: GRAV, HTAU0, Tsurf, Teff, FREF
   REAL          :: Tinci, Dinci
   REAL(KIND=k8) :: DIPF, FARAD, GNRAD, HMRAD, HNRAD
!
   END MODULE comrd5_def
!-------------**********--------------------------------------------+-------
!
! /* --- Default values for preprocessor macros ------------------------- */
!
! /* --- raytas version: 0:'IF (default)', 1:'ABS,SIGN', 2:'MIN,MAX'  --- */
# ifndef LCFrad_raytas
#   define LCFrad_raytas 0
# endif
!
!-------------------------------------------------------------------+-------
!  _     ____ _____ ____      _    ____  
! | |   / ___|  ___|  _ \    / \  |  _ \
! | |  | |   | |_  | |_) |  / _ \ | | | |
! | |__| |___|  _| |  _ <  / ___ \| |_| |
! |_____\____|_|   |_| \_\/_/   \_\____/ _module
! 
!---------*************---------------------------------------------+-------
   MODULE lcfrad_module
!*******************************************************************************
! Radiation Transport Routines
!*******************************************************************************
! SUBROUTINES: (contained)
! lcfrad_switchinfo: Print information about used compiler switches
!     raytas: computes the optical depth increments on MSTOP rays 
!             with KRSTOP depth points.
!    rayvec5: Solves the equation of radiative transfer dI/dtau = (I - B)
!             along a number of rays employing the Feautrier difference scheme.
!    raysca5: Generalized version of subroutine rayvec5, allowing to include 
!             approximate scattering
!     rsys2d: Solves the two-dimensional radiation transfer
!             problem for oblique rays in Cartesian (slab) geometry.
!    lcfrad3: Solves the radiative transfer equation along a number
!             of rays with ntheta different inclination angles theta (2D/3D).
! lcfrad_ini: Defines constants and geometrical properties of the rays system.
! lcfrad_cls: Deallocates the arrays allocated by lcfrad_ini.
!    angles1: Provides information about number  of  angles used for ray system
!    angles2: Provides information about angles and weights used for ray system
!     lstar1: computes the lambda_star operator for given delta tau. (not used)
!     lstar2: computes the lambda_star operator for given geometrical depth 
!             along the ray system and opacity per unit length.      (not used)
!*******************************************************************************

   CONTAINS

!----------*****************----------------------------------------+-------
subroutine lcfrad_switchinfo(ncp)
!----------*****************----------------------------------------+-------
! NAME:
!   lcfrad_switchinfo ('lcfrad_switch_information')
!
! PURPOSE:
!   Print information about used compiler switches.
!   
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!   call lcfrad_switchinfo(ncp)
!
! INPUT:
!   ncp:   ('number_channel_print') integer with output channel number (usually 6)
!
! OUTPUT:
!   None
!
! LOCAL VARIABLES:
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!   If another compiler switch is used anywhere in the module it has to be explicitly
!   included in this routine.
!
! PROCEDURE:
!   Just print the value of all compiler switches used for the module.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12-Nov-2002 (M.S.) Written
!-------------------------------------------------------------------+-------
implicit none
!
! --- I/O parameters ---
integer,                  intent(in)    :: ncp
!
! --- Local variables ---
character(len=*), parameter             :: defstr='  defined', undefstr='undefined'
!-------------------------------------------------------------------+-------
!
write(ncp,'(A)')    'Compiler switches: lcfrad_module .........................'
!
write(ncp,'(A,I6)') '  LCFrad' // '_raytas:            ', LCFrad_raytas
!
# ifdef LCFrad_sub_t01
write(ncp,'(A,A)' ) '  LCFrad' // '_sub_t01:           ', defstr
# else
write(ncp,'(A,A)' ) '  LCFrad' // '_sub_t01:           ', undefstr
# endif
!
end subroutine lcfrad_switchinfo
!--------------*****************------------------------------------+-------

!-------------******------------------------------------------------+-------
   SUBROUTINE raytas(MSTOP, KRSTOP, DSRAY, XMK, DXDS, TMUNU, MODE)
!-------------------------------------------------------------------+-------
!
! Version: 23-MAR-1993 14:22:15
!
! Author : HGL
! f90-Version,  MS (14-JUN-1999)
!
! Description: raytas computes the optical depth increments on
!   MSTOP rays with KRSTOP depth points. It assume that the geometrical grid
!   on which the opacity is specified is the same on every ray. 
!   raytas takes the geometrical grid from DSRAY and sets special spatial
!   opacity derivatives.
! 
! Arguments, access, type, dimension, description:
!   MSTOP  R I : Number of rays
!   KRSTOP R I : Number of depth points
!   DSRAY  R R : Geometrical grid width
!   XMK    R R2: Opacity per Volume in the form XMK(#ray, #depth)
!   DXDS   W R2: Work array used for the computation of spatial opacity 
!                derivatives. 
!   TMUNU  * R2: Optical depth increments in the form TMUNU(#ray, #depth).
!                TMUNU(#ray, 1) = optical depth at top boundary must be 
!                specified by the calling routine.
!                The depth index K increases from TOP (1) to bottom (KRSTOP)
!                TMUNU(I,K) = tau(I,K) - tau(I,K-1)
!
!
! Common blocks (access): None
!
! External procedures   : None
!
! File units (access)   : None
!
!
! Comments: None
!
! Legend:
!   Access: (R)ead only, (W)rite only, (*)read/write
!   Type  : (C)omplex, (D)ouble precision, (E)xternal, (I)nteger, &
!           (L)ogical, (R)eal, (S)tring, (*)label
!
! MODIFICATION HISTORY:
!   23. 3.93 Written by HGL
!   17. 6.97 (B.F.) More comments, slight rearrangement of TMUNU = ...
!   04. 4.13 (M.S.) TMUNU now contains optical depth INCREMENTS:
!                   TMUNU(I,K) = tau(I,K) - tau(I,K-1)
!   17.03.16 (M.S.) MODE=0/1: monotonic cubic / linear integration
!
!-------------------------------------------------------------------+-------
   IMPLICIT NONE
   INTEGER,       PARAMETER :: k8=SELECTED_REAL_KIND(12)
   REAL(KIND=k8), PARAMETER :: F05=0.5D0, F15=1.5D0, X12=1.D0/12.D0
!  
!--- Dummy arguments:
   INTEGER,                       INTENT(IN)     :: MSTOP, KRSTOP, MODE 
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)     :: DSRAY 
   REAL(KIND=k8), DIMENSION(:,:), INTENT(IN)     :: XMK
   REAL(KIND=k8), DIMENSION(:,:), INTENT(OUT)    :: DXDS
   REAL(KIND=k8), DIMENSION(:,:), INTENT(INOUT)  :: TMUNU
!
!--- Local Variables:
   INTEGER        :: K, M
   REAL(KIND=k8)  :: S1, S2, S3, S4, S5
!
!-------------------------------------------------------------------+-------
! Computation of depth scale of rays passing through top and bottom.
!-------------------------------------------------------------------+-------
!
!     --- Compute optical depth increments for all internal depth intervals ---
 IF (ABS(MODE) == 1) THEN
! MODE == 1: linear tau integration:
   DO K=2,KRSTOP
     DO M=1,MSTOP
       TMUNU(M,K) = DSRAY(K) * F05 * ( XMK(M,K-1)+ XMK(M,K) )
     ENDDO
   ENDDO
   RETURN
 ELSEIF (MODE == 0) THEN
! MODE # 1: monotonic cubic tau integration:
!     --- Compute d(kappa)/dz => DXDS(M,K) ---
!
   DO K=2,KRSTOP
     TMUNU(1:MSTOP,K)=(XMK(1:MSTOP,K)-XMK(1:MSTOP,K-1))/DSRAY(K)
   ENDDO
!
!     --- INTERIOR ---
# if (LCFrad_raytas == 1) /* 'ABS,SIGN' */
   DO K=2,KRSTOP-1
     DO M=1,MSTOP
       S1 = TMUNU(M,K)
       S2 = TMUNU(M,K+1)
       S3 = MIN(ABS(S1), ABS(S2), &
                ABS(F05*(S1*DSRAY(K+1)+S2*DSRAY(K))/(DSRAY(K)+DSRAY(K+1))))
       DXDS(M,K) = SIGN(S3,S1)+SIGN(S3,S2)
     ENDDO
   ENDDO
# elif (LCFrad_raytas == 2) /* 'MIN,MAX' */
   DO K=2,KRSTOP-1
     DO M=1,MSTOP
       S1 = TMUNU(M,K)
       S2 = TMUNU(M,K+1)
       S3 = F05*(S1*DSRAY(K+1)+S2*DSRAY(K))/(DSRAY(K)+DSRAY(K+1))
       S4 = MIN(S1,S2,S3)
       S5 = MAX(S1,S2,S3)
       DXDS(M,K) = 2.D0*(MAX(S4,0.D0)+MIN(S5,0.D0))
     ENDDO
   ENDDO
# else /* LCFrad_raytas == 0: 'IF (default)' */
   DO K=2,KRSTOP-1
     DO M=1,MSTOP
       S1 = TMUNU(M,K)
       S2 = TMUNU(M,K+1)
       S3 = (S1*DSRAY(K+1)+S2*DSRAY(K)) / (DSRAY(K)+DSRAY(K+1))
       IF (S1*S2 <= 0.D0) THEN 
         DXDS(M,K) = 0.D0
       ELSEIF (S3 > 0.D0) THEN
         DXDS(M,K) = 2.D0*MIN(S1, S2, F05*S3)
       ELSE
         DXDS(M,K) = 2.D0*MAX(S1, S2, F05*S3)
       ENDIF
     ENDDO
   ENDDO
# endif /* LCFrad_raytas */
!
!     --- TOP:    d2(kappa)/dz2 = 0 ---
   DXDS(1:MSTOP,1)     =F15*TMUNU(1:MSTOP,2)     -F05*DXDS(1:MSTOP,2)
!
!     --- BOTTOM: d2(kappa)/dz2 = 0 ---
   DXDS(1:MSTOP,KRSTOP)=F15*TMUNU(1:MSTOP,KRSTOP)-F05*DXDS(1:MSTOP,KRSTOP-1)
!
!     --- Optical depth at upper boundary is set in calling routine ---
!
!     --- Compute optical depth increments for all internal depth intervals ---
   DO K=2,KRSTOP
     DO M=1,MSTOP
       TMUNU(M,K) = &
       DSRAY(K) * ( F05 * ( XMK(M,K-1)+ XMK(M,K)) + &
       DSRAY(K) *   X12 * (DXDS(M,K-1)-DXDS(M,K)) )
     ENDDO
   ENDDO
   RETURN
 ELSE
   WRITE(*,*) "RAYTAS: illegal value of MODE:", MODE
   STOP
 ENDIF
!
   END SUBROUTINE raytas
!-----------------******--------------------------------------------+-------

!-------------*******-----------------------------------------------+-------
   SUBROUTINE rayvec5(NR, NP, DSRAY, DTRAY, OPC, PLK, B0, GN, HM, HN, &
                      YIM0, DIPN, IJMB, RHS, VRT, VRB, YIP0)
!-------------------------------------------------------------------+-------
!
!    ==============================================================
!    F90-Version,  MS (17-MAR-2016)
!    Solution of the Feautrier equation on the "geometrical scale"
!    ==============================================================
!
!    rayvec5 solves the equation of radiative transfer dI/dtau = (I - B)
!    along a number of rays employing the Feautrier difference scheme.
!    The geometrical depth scale is identical for all rays, but arbitrary 
!    opacities and source functions may be present on different rays. 
!
!    Depth index L increases from top (L=1) to bottom (L=NP). 
!
!    For details about the numerial solution see the document 
!    *feautrier_gs.pdf* by M. Steffen (2016).
!
!    Variables and their meaning:
!    ----------------------------
!       NR       = Number of rays treated simultaneously
!       NP       = Number of depth points along a ray
!       DSRAY(  L) = geometrical depth INCREMENT   s(L)-  s(L-1) on all rays
!       DTRAY(I,L) =     optical depth INCREMENT tau(L)-tau(L-1)
!       DTRAY(I,1) = optical depth at top (L=1)
!       OPC(  I,L) = Opacity [cm^-1] at depth point L of ray I
!       PLK(  I,L) = Source (= Planck) function at depth point L of ray I
!       RHS(  I,L) = Result at depth point L of ray I,
!                    either  = [ I(+) + I(-) ] / 2 - PLK   (IJMB >  0)
!                    or      = [ I(+) - I(-) ] / 2         (IJMB <= 0)
!
!       YIM0(I)  = incident intensity at tau=0
!       YIP0(I)  = emergent intensity at tau=0
!
!
!    Boundary conditions:
!    --------------------
!       Top0         : tau = 0, B = B0, I(-) = YIM0 
!       Top    (L=1 ): v1 = q1*u1 - g0*B0 - g1*B1 - h0*YIM0
!                      q1, g0, g1, h0, are functions of tau(L=1)
!       Bottom (L=NP): GN*v(NP-1/2) + HM*ds*[OPC*(u-B)]_(NP-1) + 
!                                     HN*ds*[OPC*(u-B)]_(NP  ) = DIPN/2
!-------------------------------------------------------------------+-------
!
!
   IMPLICIT NONE
   INTEGER,       PARAMETER :: k8=SELECTED_REAL_KIND(12)
   REAL(KIND=k8), PARAMETER :: F05=0.5D0, F10=1.0D0, F20=2.0D0
!
!--- Dummy arguments:
   INTEGER,                       INTENT(IN)  :: NR, NP, IJMB
   REAL(KIND=k8),                 INTENT(IN)  :: B0, GN, HM, HN
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: YIM0, DIPN
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: DSRAY
   REAL(KIND=k8), DIMENSION(:,:), INTENT(IN)  :: DTRAY, OPC, PLK
   REAL(KIND=k8), DIMENSION(:,:), INTENT(OUT) :: RHS
   REAL(KIND=k8), DIMENSION(:),   INTENT(OUT) :: VRT, VRB, YIP0
!
!--- Local scalar variables:
   INTEGER       :: I, L
   REAL(KIND=k8), DIMENSION(NR) :: TH0, TG0, TG1 
   REAL(KIND=k8)                :: EQ1, EH0, EG0, EG1, THM, THN, &
                                   CINV, ADVC, DTAU0, DUM, EXPF0
!--- Local (automatic) array:
!   REAL(KIND=k8), DIMENSION(NR,NP) :: ALF, BET
!--- Local allocatable arrays:
   REAL(KIND=k8), DIMENSION(:,:), ALLOCATABLE :: ALF, BET
!
!-------------------------------------------------------------------+-------
! --- Allocate arrays ---
   allocate(alf(NR,NP))
   allocate(bet(NR,NP))
!
!-------------------------------------------------------------------+-------
!
!=======================================================================
!--- Set up Feautrier factors proceeding from top to bottom ---
!=======================================================================
!
!  --- Factors h0, g0, (g1-q1)  at uppermost (L=1) depth point ---
!  --> factors alpha and beta   at uppermost (L=1) depth point ---
   DO I=1,NR
      DTAU0    = DTRAY(I,1) + 1.0D-6
      EXPF0    = EXP(-DTAU0)
      TH0(I)   = EXPF0
      TG0(I)   =  (F10 - EXPF0)/DTAU0 - EXPF0
      TG1(I)   = -(F10 - EXPF0)/DTAU0 
      DUM      = DTRAY(I,2)*(F05*DSRAY(2)*OPC(I,1) + F10)
      ALF(I,1) = DUM / (F10 + DUM)
      BET(I,1) = (F10 - ALF(I,1)) * DTRAY(I,2) * &
                 ( TH0(I)*YIM0(I) + TG0(I)*B0 + TG1(I)*PLK(I,1) ) - &
                 ALF(I,1)*( PLK(I,2) - PLK(I,1) )
   ENDDO
!
!     --- Factors alpha and beta for intermediate depth points ---
   DO L=2,NP-1
     DO I=1,NR
       CINV     =  F05 * DTRAY(I,L+1) * OPC(I,L)*(DSRAY(L)+DSRAY(L+1))
       ADVC     =        DTRAY(I,L+1) /  DTRAY(I,L)
       DUM      =  CINV +  ALF(I,L-1) * ADVC 
       ALF(I,L) =  DUM / (F10 + DUM )
       BET(I,L) = (F10 - ALF(I,L)) * BET(I,L-1)*ADVC - &
                  ALF(I,L)*( PLK(I,L+1) - PLK(I,L) )
     ENDDO
   ENDDO
!
!=======================================================================
!--- Computation of Feautrier factors finished ---
!=======================================================================
!
!
!=======================================================================
!--- Computation of (u-PLK) for all points and v at lower boundary ---
!=======================================================================
!
!  --- Compute (u - PLK) at lower boundary ---
   DO I=1,NR
      THM = HM*DSRAY(NP)*OPC(I,NP-1)
      THN = HN*DSRAY(NP)*OPC(I,NP)
      DUM = GN/DTRAY(I,NP) - THM
      RHS(I,NP) = &
        ( DUM*BET(I,NP-1) + F05*DIPN(I) - THM*(PLK(I,NP)-PLK(I,NP-1)) ) / &
        ( DUM*ALF(I,NP-1) + THM + THN )

   ENDDO
!
!  --- Compute (u - PLK) for depth points NP-1...1 ---
   DO L=NP-1,1,-1
     DO I=1,NR
       RHS(I,L) = (F10 - ALF(I,L)) * RHS(I,L+1) + BET(I,L) + &
                   PLK(I,L+1) - PLK(I,L)
     ENDDO
   ENDDO
!
   DO I=1,NR
!  --- Compute v at uppermost (L=1) depth point ---
     VRT(I) = (RHS(I,2)+PLK(I,2)-RHS(I,1)-PLK(I,1)) / DTRAY(I,2) - &
               F05 * DSRAY(2) * OPC(I,1) * RHS(I,1)
! --- factors 2Q1, 2G0, 2(G1+Q1), (2H0-1) --> emergent intensity at tau=0 ---
     EQ1    = F20*TH0(I)
     EH0    =-TH0(I)*TH0(I)
     EG0    = TG1(I) - TG0(I)*TH0(I) + F10
     EG1    = TG0(I) - TG1(I)*TH0(I) + TH0(I)
     YIP0(I)= EQ1*RHS(I,1) + EG0*B0 + EG1*PLK(I,1) + EH0*YIM0(I)
!
!  --- Compute v at lowest (L=NP) depth point   ---
     VRB(I) = (RHS(I,NP)+PLK(I,NP)-RHS(I,NP-1)-PLK(I,NP-1)) / DTRAY(I,NP) + &
               F05 * DSRAY(NP) * OPC(I,NP) * RHS(I,NP)
   ENDDO
!
!=======================================================================
!--- Computation of (u-PLK) and v (upper and lower boundary) finished ---
!=======================================================================
!
!  --- Finish subroutine if flux computation was not requested ---
   IF (IJMB <= 0) THEN
!
!=======================================================================
!--- Computation of flux v for depth points NP...1 ---
!=======================================================================
!
!  --- Store v for uppermost and lowest depth point in ALF array ---
     ALF(1:NR,1 ) = VRT(1:NR)
     ALF(1:NR,NP) = VRB(1:NR)
!
!     --- Compute v for intermediate depth points NP-1...2 ---
     DO L=NP-1,2,-1
       ALF(1:NR,L) = ALF(1:NR,L+1) - F05 * DSRAY(L+1) * &
                   ( OPC(1:NR,L+1)*RHS(1:NR,L+1) + &
                     OPC(1:NR,L  )*RHS(1:NR,L  ) )
     ENDDO
!
!     --- Store flux v in RHS array ---
     RHS(1:NR,1:NP) = ALF(1:NR,1:NP)
!
   ENDIF
!=======================================================================
!--- Computation of flux v finished ---
!=======================================================================
!
!-------------------------------------------------------------------+-------
! --- Deallocate arrays ---
   deallocate(alf)
   deallocate(bet)
!
!-------------------------------------------------------------------+-------
!
   END SUBROUTINE rayvec5
!-----------------*******-------------------------------------------+-------

!-------------*******-----------------------------------------------+-------
   SUBROUTINE raysca5(NR, NP, DSRAY, DTRAY, OPC, EPS, PLK, B0, GN, HM, HN, &
                      YIM0, DIPN, IJMB, RHS, VRT, VRB, YIP0)
!-------------------------------------------------------------------+-------
!
!    ==============================================================
!    Generalized version of subroutine rayvec5
!    allowing to include approximate scattering
!    F90-Version,  MS (15-JUL-2016)
!    Solution of the Feautrier equation on the "geometrical scale"
!    ==============================================================
!
!    raysca5 solves the equation of radiative transfer dI/dtau = (I - S)
!    along a number of rays employing the Feautrier difference scheme.
!    The source function is defined as S = (1-EPS)*u + EPS*B, where
!    EPS is the thermalization parameter, EPS = kappa/(kappa+sigma)
!
!    The geometrical depth scale is identical for all rays, but arbitrary 
!    opacities (OPC, EPS) and source functions (B) may be present on 
!    different rays. 

!    The Feautrier factor B(d) is computed from the relation  
!    -A(d) + B(d) - C(d) = EPS(d).
!
!    Depth index L increases from top (L=1) to bottom (L=NP). 
!
!    For details about the numerial solution see the document 
!    *feautrier_gs.pdf* by M. Steffen (2016).
!
!
!    Variables and their meaning:
!    ----------------------------
!       NR       = Number of rays treated simultaneously
!       NP       = Number of depth points along a ray
!       DSRAY(  L) = geometrical depth INCREMENT   s(L)-  s(L-1) on all rays
!       DTRAY(I,L) =     optical depth INCREMENT tau(L)-tau(L-1)
!       DTRAY(I,1) = optical depth at top (L=1)
!       OPC(  I,L) = Opacity [cm^-1] at depth point L of ray I
!       EPS(  I,L) = Thermalization probability at depth point L of ray I
!       PLK(  I,L) = Planck function B at depth point L of ray I
!       RHS(  I,L) = Result at depth point L of ray I,
!                    either  = [ I(+) + I(-) ] / 2 - PLK   (IJMB >  0)
!                    or      = [ I(+) - I(-) ] / 2         (IJMB <= 0)
!
!       YIM0(I)  = incident intensity at tau=0
!       YIP0(I)  = emergent intensity at tau=0
!
!
!    Boundary conditions:
!    --------------------
!       Top0         : tau = 0, B = B0, eps = eps0 = eps1, I(-) = YIM0 
!       Top    (L=1 ): v1 = q1*u1 - g0*B0 - g1*B1 - h0*YIM0
!                      q1, g0, g1, h0, are functions of tau1(L1), eps1
!       Bottom (L=NP): GN*v(NP-1/2) + 
!                      HM*ds*[OPC*EPS*(u-B)]_(NP-1) + 
!                      HN*ds*[OPC*EPS*(u-B)]_(NP  ) = DIPN/2
!-------------------------------------------------------------------+-------
!
!
   IMPLICIT NONE
   INTEGER,       PARAMETER :: k8=SELECTED_REAL_KIND(12)
   REAL(KIND=k8), PARAMETER :: F05=0.5D0, F10=1.0D0, F20=2.0D0
!
!--- Dummy arguments:
   INTEGER,                       INTENT(IN)  :: NR, NP, IJMB
   REAL(KIND=k8),                 INTENT(IN)  :: B0, GN, HM, HN
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: YIM0, DIPN
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: DSRAY
   REAL(KIND=k8), DIMENSION(:,:), INTENT(IN)  :: DTRAY, OPC, EPS, PLK
   REAL(KIND=k8), DIMENSION(:,:), INTENT(OUT) :: RHS
   REAL(KIND=k8), DIMENSION(:),   INTENT(OUT) :: VRT, VRB, YIP0
!
!--- Local scalar variables:
   INTEGER       :: I, L
   REAL(KIND=k8) :: EPS0, SEP0
   REAL(KIND=k8), DIMENSION(NR) :: TQ1, TH0, TG0, TG1
   REAL(KIND=k8)                :: EQ1, EH0, EG0, EG1, THM, THN, &
                                   CINV, ADVC, DTAU0, DUM, DUM1, DUM2, EXPFX
!--- Local (automatic) array:
!   REAL(KIND=k8), DIMENSION(SIZE(TAU,1),SIZE(TAU,2)) :: ALF, BET
!--- Local allocatable arrays:
   REAL(KIND=k8), DIMENSION(:,:), ALLOCATABLE :: ALF, BET
!
!-------------------------------------------------------------------+-------
! --- Allocate arrays ---
   allocate(alf(NR,NP))
   allocate(bet(NR,NP))
!
!-------------------------------------------------------------------+-------
!
!=======================================================================
!--- Set up Feautrier factors proceeding from top to bottom ---
!=======================================================================
!
!  --- Factors q1, h0, g0, (g1-q1)  at uppermost (L=1) depth point ---
!  --> factors   alpha and beta     at uppermost (L=1) depth point ---
   DO I=1,NR
      EPS0     = EPS(I,1)
      SEP0     = SQRT(EPS0)
      DTAU0    = SEP0*DTRAY(I,1) + 1.0D-6
      EXPFX    = EXP(-DTAU0)
      DUM1     = F05*(F10 + SEP0)
      DUM2     = F05*(F10 - SEP0)*EXPFX*EXPFX
      DUM      = DUM1 - DUM2
      TQ1(I)   = (DUM1 + DUM2)*SEP0/DUM
      TH0(I)   =         EXPFX*SEP0/DUM
      DUM      = DUM*DTAU0
      TG0(I)   = (DUM1 - DUM2 - EXPFX*(DTAU0+SEP0))*SEP0/DUM
      TG1(I)   = (DUM2 - DUM1 + EXPFX*SEP0        )*SEP0/DUM
      DUM      = DTRAY(I,2)*(F05*DSRAY(2)*OPC(I,1)*EPS0 + TQ1(I))
      ALF(I,1) = DUM / (F10 + DUM)
      BET(I,1) = (F10 - ALF(I,1)) * DTRAY(I,2) * &
                 ( TH0(I)*YIM0(I) + TG0(I)*B0 + TG1(I)*PLK(I,1) ) - &
                 ALF(I,1)*( PLK(I,2) - PLK(I,1) )
   ENDDO
!
!     --- Factors alpha and beta for intermediate depth points ---
   DO L=2,NP-1
     DO I=1,NR
       CINV     =  F05 * DTRAY(I,L+1) * OPC(I,L)*(DSRAY(L)+DSRAY(L+1))
       ADVC     =        DTRAY(I,L+1) /  DTRAY(I,L)
       DUM      =  EPS(I,L)*CINV + ALF(I,L-1)*ADVC 
       ALF(I,L) =  DUM / (F10 + DUM )
       BET(I,L) = (F10 - ALF(I,L)) * BET(I,L-1)*ADVC - &
                  ALF(I,L)*( PLK(I,L+1) - PLK(I,L) )
     ENDDO
   ENDDO
!
!=======================================================================
!--- Computation of Feautrier factors finished ---
!=======================================================================
!
!
!=======================================================================
!--- Computation of (u-PLK) for all points and v at lower boundary ---
!=======================================================================
!
!  --- Compute (u - PLK) at lower boundary ---
   DO I=1,NR
      THM = HM*DSRAY(NP)*OPC(I,NP-1)*EPS(I,NP-1)
      THN = HN*DSRAY(NP)*OPC(I,NP  )*EPS(I,NP  )
      DUM = GN/DTRAY(I,NP) - THM
      RHS(I,NP) = &
      ( DUM*BET(I,NP-1) + F05*DIPN(I) - THM*(PLK(I,NP)-PLK(I,NP-1)) ) / &
      ( DUM*ALF(I,NP-1) + THM + THN )
   ENDDO
!
!  --- Compute (u - PLK) for depth points NP-1...1 ---
   DO L=NP-1,1,-1
     DO I=1,NR
       RHS(I,L) = (F10 - ALF(I,L)) * RHS(I,L+1) + BET(I,L) + &
                   PLK(I,L+1) - PLK(I,L)
     ENDDO
   ENDDO
!
!  --- Compute v at uppermost (L=1) depth point ---
   DO I=1,NR
     VRT(I) = (RHS(I,2)+PLK(I,2)-RHS(I,1)-PLK(I,1)) / DTRAY(I,2) - &
               F05 * DSRAY(2) * OPC(I,1) * EPS(I,1) * RHS(I,1)
! --- factors 2Q1, 2G0, 2(G1+Q1), (2H0-1) --> emergent intensity at tau=0 ---
     DUM    = F10+TQ1(I)
     EQ1    = F20*TH0(I)
     EH0    = (F10-TQ1(I)-F20*TH0(I)*TH0(I))/DUM
     EG0    = F20*(TG1(I) - TG0(I)*TH0(I) + TQ1(I))/DUM
     EG1    = F20*(TG0(I) - TG1(I)*TH0(I) + TH0(I))/DUM
     YIP0(I)= EQ1*RHS(I,1) + EG0*B0 + EG1*PLK(I,1) + EH0*YIM0(I)
!
!  --- Compute v at lowest (L=NP) depth point   ---
     VRB(I) = (RHS(I,NP)+PLK(I,NP)-RHS(I,NP-1)-PLK(I,NP-1)) / DTRAY(I,NP) + &
               F05 * DSRAY(NP) * OPC(I,NP) * EPS(I,NP) * RHS(I,NP)
   ENDDO
!
!=======================================================================
!--- Computation of (u-PLK) and v (upper and lower boundary) finished ---
!=======================================================================
!
!  --- Finish subroutine if flux computation was not requested ---
   IF (IJMB <= 0) THEN
!
!=======================================================================
!--- Computation of flux v for depth points NP...1 ---
!=======================================================================
!
!  --- Store v for uppermost and lowest depth point in ALF array ---
     ALF(1:NR,1 ) = VRT(1:NR)
     ALF(1:NR,NP) = VRB(1:NR)
!
!     --- Compute v for intermediate depth points NP-1...2 ---
     DO L=NP-1,2,-1
       ALF(1:NR,L) = ALF(1:NR,L+1) - F05 * DSRAY(L+1) * &
                   ( OPC(1:NR,L+1)*EPS(1:NR,L+1)*RHS(1:NR,L+1) + &
                     OPC(1:NR,L  )*EPS(1:NR,L  )*RHS(1:NR,L  ) )
     ENDDO
!
!     --- Store flux v in RHS array ---
     RHS(1:NR,1:NP) = ALF(1:NR,1:NP)
!
   ENDIF
!=======================================================================
!--- Computation of flux v finished ---
!=======================================================================
!
!-------------------------------------------------------------------+-------
! --- Deallocate arrays ---
   deallocate(alf)
   deallocate(bet)
!
!-------------------------------------------------------------------+-------
!
   END SUBROUTINE raysca5
!-----------------*******-------------------------------------------+-------

!-------------******------------------------------------------------+-------
   SUBROUTINE rsys2d(SPHI, WPHI, DIPN, HTAU1, &
                     MFRT, NNRT, NFILL, NSTEP, KRMAX, KZERO, JK, &
                     DSRAY, DXR, HDSFN, HDKAP, OMUNU, &
                     UMUNU, SMUNU, TMUNU, XMUNU, HDQRT, &
                     VRTOP, VRBOT, Y1AVG, Y2AVG, SOURC0, I00, imax)
!
!-------------------------------------------------------------------+-------
!
!    ==================================
!    Last Update:  28-Jul-1997 13:47:20
!    f90-Version,  MS (14-JUN-1999)
!    Modified,     MS (30-NOV-1999)
!    Modified,     MS (10-MAR-2000)
!    Modified,     MS (10-APR-2000)
!    Modified,     MS (06-NOV-2000)
!    Modified,     MS (28-NOV-2000)
!    Modified,     MS (08-Oct-2001) F1TAU1, F2TAU1
!    Modified,     MS (08-Oct-2001) Y1AVG, Y2AVG
!    Modified,     MS (12-Jul-2011) Y1AVG, Y2AVG
!    Modified,     MS (05-Apr-2013) +/- phi separated
!    Modified,     MS (24-Oct-2013) bug related to SFAC1 corrected
!    Modified,     MS (04-Feb-2015) W1=B1, W2=B2, NNRT
!    Modified,     MS (18-MAR-2016) MFRT
!    ==================================
!
!    This subroutine solves the two-dimensional radiation transfer
!    problem for oblique rays in Cartesian (slab) geometry.
!
!    First all required data (Planck function and opacity kappa) are
!    interpolated from HD coordinates onto the ray system, and 
!    optical depths along the rays are computed.
!
!    Then rayvec is called.
!
!    Finally, the results (mainly kappa*(J-B)) are interpolated
!    back to HD coordinates.
!
! MODIFICATION HISTORY:
! Last modified 28.11.2000 by M.S.
!-------------------------------------------------------------------+-------
!
   USE comrd3_def
   USE comrd4_def
   USE comrd5_def
!
!
   IMPLICIT NONE
   REAL(KIND=k8), PARAMETER :: F00=0.0D0, F05=0.5D0, F10=1.0D0, F20=2.0D0
!
!--- Dummy arguments:
!
   INTEGER,                       INTENT(IN)  :: imax, MFRT, NNRT, NFILL, &
                                                 NSTEP, KRMAX, KZERO
   INTEGER,       DIMENSION(:),   INTENT(IN)  :: I00, JK
   REAL(KIND=k8),                 INTENT(IN)  :: SOURC0
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: SPHI, WPHI
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: DSRAY, DXR
   REAL(KIND=k8), DIMENSION(:),   INTENT(IN)  :: DIPN, HTAU1
   REAL(KIND=k8),                 INTENT(OUT) :: Y1AVG, Y2AVG
   REAL(KIND=k8), DIMENSION(:),   INTENT(OUT) :: VRTOP, VRBOT
   REAL(KIND=k8), DIMENSION(:,:), INTENT(IN)  :: HDSFN, HDKAP
   REAL(KIND=k8), DIMENSION(:,:), INTENT(OUT) :: OMUNU,SMUNU,TMUNU,UMUNU,XMUNU
   REAL(KIND=k8), DIMENSION(:,jmin:), INTENT(OUT)   :: HDQRT
!
!--- Local variables:
!
   INTEGER       :: I1, I2, J, JJ, K, K1, K2, ks, MM1, MM2, MSTOP, NP, NP2
   REAL(KIND=k8) :: DROR, F0, OFN, RODR, RTOT, W12, WJ1, WJ2
   REAL(KIND=k8) :: DXRK, FKS, SP
   REAL(KIND=k8) :: dtauk, sum1, sum2
!
!--- Local (automatic) arrays:
!
   INTEGER, DIMENSION(ksub)               :: I1K, L1, L2 
   INTEGER, DIMENSION(ksub,krmax)         :: I0
   REAL(KIND=k8), DIMENSION(ksub)         :: XC 
   REAL(KIND=k8), DIMENSION(ksub)         :: RXODR, XR 
   REAL(KIND=k8), DIMENSION(ksub,krmax)   :: W1, W2, B1, B2
   REAL(KIND=k8), DIMENSION(imax)         :: YK1
   REAL(KIND=k8), DIMENSION(imax*ksub)    :: YMKA1, YMKA2, YMKA3 
   REAL(KIND=k8), DIMENSION(imax*ksub)    :: YIM0, YIP0
   REAL(KIND=k8), DIMENSION(imax*ksub)    :: YTOP, YBOT, DIPNR, HTAU1R
   REAL(KIND=k8), DIMENSION(krmax)        :: UAVG, SAVG, DSDS, DUDS
   REAL(KIND=k8), DIMENSION(krmax)        :: CFAC1, SFAC1, SMIN, FMIN
   REAL(KIND=k8), DIMENSION(krmax)        :: CFACA, SFACA
   REAL(KIND=k8), DIMENSION(imax,krmax)   :: HDSFN1, HDKAP1, HDQRT1
!
!-------------------------------------------------------------------+-------
!
!  --- Inverse grid point filling factor ---
   OFN = F10 / REAL(NFILL,k8)
   FKS = REAL(ksub,k8)
!
!  --- Geometrical constants ---
   RODR  = REAL(imax,k8)
   DROR  = F10  / RODR
   RTOT  = DELR * RODR
!
!  --- Planck function and opacity kappa are interpolated ---
!  --- from HD coordinates onto the (refined) intermediate grid ---
   HDSFN1(1:imax,1:KRMAX) = HDSFN(1:imax,1:kmax:NSTEP)
   HDKAP1(1:imax,1:KRMAX) = HDKAP(1:imax,1:kmax:NSTEP)
!
!  --- Clear HDQRT1 array ---
   HDQRT1(1:imax,1:KRMAX) = F00
!
!  --- Clear CFACA, SFACA arrays ---
   CFACA(1:KRMAX) = F00
   SFACA(1:KRMAX) = F00
!
!  --- Clear VRTOP and VRBOT arrays ---
   VRTOP(1:imax) = F00
   VRBOT(1:imax) = F00
!
   Y1AVG = F00
   Y2AVG = F00
!
   MSTOP  = imax * ksub
!
!***********************************************************************
!--- Loop through all PHI-angles and sub rays---
!***********************************************************************
!
   DO 500 NP2=1,2*nphi
   NP = (NP2+1)/2
!  SP = 1.0 for rays with phi > 0 (NP2=1,3,5)
!  SP =-1.0 for rays with phi < 0 (NP2=2,4,6)
   SP = REAL(2*MOD(NP2,2)-1,k8)
!  --- Weight of each ray ---
   F0 = F05 * WPHI(NP) / FKS
!
! --- Ray position at top (x coordinate in units of Rtot) ---
   XC(1:ksub) = (/ (DROR*REAL(ks-1,k8)/FKS, ks=1,ksub) /)
!
!=======================================================================
!--- Loop through all rays for this PHI and ksub ---
!=======================================================================
!
     DO 400 K=1,KRMAX
!-----------------------------------------------------------------------
!--- Loop through all depth points (HD grid) ---
!-----------------------------------------------------------------------
!
! --- Ray positions (x coordinate in units of Rtot) for this height ---
       DXRK              = DXR(K) - DXR(KZERO)
       XR(     1:  ksub) = XC(1:ksub) + SP*SPHI(NP)*DXRK/RTOT
!
!      --- Find equivalent X position:
!      --- 0.0 <= XR <= 1.0
!
       XR = XR - ANINT(XR-F05)
!
!      --- Save X-position in DELR units ---
       RXODR = RODR*ABS(XR)
!
!      0 <= I0 <= idim
!
       I0(1:ksub,K) = INT(RXODR)
!      --- Weighting factors for interpolation ---
       W2(1:ksub,K) = RXODR - REAL(I0(1:ksub,K),k8)
       W1(1:ksub,K) = F10   - W2(1:ksub,K)
!
!      --- Weighting factors for back interpolation ---
!      --- B1, B2 differ from W1, W2 above only if ksub > 1 ---
!      --- With B1, B2, back interpolation is more concentrated towards 
!          the nearest HD grid point (weights defined by intersection of 
!          geometrical cross sections of HD grid points and rays):
!          B2 = 0                  for 0 <= x <= 1/2 - 1/(2*ksub)
!          B2 = 1/2 + ksub*(x-1/2) for 1/2 - 1/(2*ksub) <= x <= 1/2 + 1/(2*ksub)
!          B2 = 1                  for 1/2 + 1/(2*ksub) <= x <= 1
!
       B2(1:ksub,K) = MIN(F10, MAX(F00, F05 + FKS*(W2(1:ksub,K)-F05)))
       B1(1:ksub,K) = F10 - B2(1:ksub,K)
!
! --- Use same weights for forward and backward interpolation:
       W1(1:ksub,K) = B1(1:ksub,K)
       W2(1:ksub,K) = B2(1:ksub,K)
!
       IF (NNRT == 1) THEN
!      --- nearest neighbor transfer ---
         W1(1:ksub,k) = ANINT(W1(1:ksub,k),k8)
         W2(1:ksub,k) = ANINT(W2(1:ksub,k),k8)
         B1(1:ksub,k) = ANINT(B1(1:ksub,k),k8)
         B2(1:ksub,k) = ANINT(B2(1:ksub,k),k8)
       ENDIF
!
! --- LINEAR interpolation of opacity onto ray point ---
! --- LINEAR interpolation of source function onto ray point ---
!
       DO ks=1,ksub
!
!      --- Rays with phi > 0 (M=1,imax*ksub):
!      --- Indices of HD-GRID between which ray point is located ---
!
         I1=I0(ks,K)
         I2=I1+1
         MM1=imax*(ks-1) + 1
         MM2=imax* ks
         OMUNU(MM1:MM2,K) =  W1(ks,K) * HDKAP1(I00(I1+1:I1+imax),K) + &
                             W2(ks,K) * HDKAP1(I00(I2+1:I2+imax),K)
         if (MRAD_1 == 0) THEN
! Linear interpolation of source function
         SMUNU(MM1:MM2,K) =  W1(ks,K) * HDSFN1(I00(I1+1:I1+imax),K) + &
                             W2(ks,K) * HDSFN1(I00(I2+1:I2+imax),K)
         else
! Opacity-weighted interpolation of source function
         SMUNU(MM1:MM2,K) =( W1(ks,K) * HDKAP1(I00(I1+1:I1+imax),K) *   &
                                        HDSFN1(I00(I1+1:I1+imax),K) +   &
                             W2(ks,K) * HDKAP1(I00(I2+1:I2+imax),K) *   &
                                        HDSFN1(I00(I2+1:I2+imax),K) ) / &
                             OMUNU(MM1:MM2,K) 
         endif
         IF (K ==     1) THEN
           HTAU1R(MM1:MM2) = W1(ks,K) *HTAU1(I00(I1+1:I1+imax)) + &
                             W2(ks,K) *HTAU1(I00(I2+1:I2+imax))
         ENDIF
         IF (K == KRMAX) THEN
            DIPNR(MM1:MM2) = W1(ks,K) * DIPN(I00(I1+1:I1+imax)) + &
                             W2(ks,K) * DIPN(I00(I2+1:I2+imax))
         ENDIF
       ENDDO
!       
400  CONTINUE
!
!-----------------------------------------------------------------------
!--- Loop through all depth points (HD grid) finished ---
!-----------------------------------------------------------------------
!
!=======================================================================
!--- Loop through all rays for this PHI-angle and sub ray finished ---
!=======================================================================
!
!    --- Optical depth tau = kappa * HTAU0 / mu at upper boundary
     TMUNU(1:mstop,1) = OMUNU(1:mstop,1) * HTAU1R(1:mstop)
!
!    --- Compute optical depth scale ---
     CALL raytas(MSTOP, KRMAX, DSRAY, OMUNU, UMUNU, TMUNU, MFRT)
!
!    --- Incident intensity at upper boundary ---
     YIM0(1:mstop) = F00
!
     IF (MFRT <= 0) THEN
!--- Compute mean opacity XMUNU = dtau/dz on ray system:
       K = 1
         XMUNU(1:mstop,K)= TMUNU(1:mstop,K+1) / DSRAY(K+1)
       K = KRMAX
         XMUNU(1:mstop,K)= TMUNU(1:mstop,K  ) / DSRAY(K)
       DO K=2,KRMAX-1
         XMUNU(1:mstop,K)=(TMUNU(1:mstop,K+1) + TMUNU(1:mstop,K)) / &
                          (DSRAY(K) + DSRAY(K+1))
       ENDDO
     ELSEIF (MFRT == 1) THEN
!--- Compute mean opacity XMUNU = local opacity OMUNU on ray system:
       DO K=1,KRMAX
         XMUNU(1:mstop,K) = OMUNU(1:mstop,K)
       ENDDO
     ELSE
       WRITE(*,*) "rsys2d: illegal value of MFRT:", MFRT
       STOP
     ENDIF
!
!    --- Solve radiation transfer through rayvec call ---
     CALL rayvec5(MSTOP, KRMAX, DSRAY, TMUNU, XMUNU, SMUNU, SOURC0, &
                  GNRAD, HMRAD, HNRAD, YIM0, DIPNR, IJMB, &
                  UMUNU, YTOP, YBOT, YIP0 )
!
!--- Compute emergent I(+) at tau = 0 ---
!    ... returned by rayvec5 ...
!
     Y1AVG = Y1AVG + F0*SUM(YIP0(1:mstop)   )
     Y2AVG = Y2AVG + F0*SUM(YIP0(1:mstop)**2)
!
!-------------------------------------------------------------------+-------
     IF (MRAD == 2 .AND. MRAD_2 == 1) THEN
! --- Reference function for ratio method [bug SFAC1, CFAC1, don't use] ---
!-------------------------------------------------------------------+-------
       DO K=1,KRMAX
         SAVG(K) = SUM(SMUNU(1:MSTOP,K))/REAL(MSTOP,k8)
         UAVG(K) = SUM(UMUNU(1:MSTOP,K))/REAL(MSTOP,k8)
         DSDS(K) = SUM((SMUNU(1:MSTOP,K)-SAVG(K))*(SMUNU(1:MSTOP,K)-SAVG(K)))
         DUDS(K) = SUM((UMUNU(1:MSTOP,K)-UAVG(K))*(SMUNU(1:MSTOP,K)-SAVG(K)))
         IF (DUDS(K) <= -DSDS(K)) THEN
            SFAC1(K) = 1.0D0
            CFAC1(K) = UAVG(K) + SAVG(K)
         ELSEIF (DUDS(K) >= DSDS(K)*UAVG(K)/SAVG(K)) THEN
            SFAC1(K) = -UAVG(K)/SAVG(K)
            CFAC1(K) = 0.0D0
         ELSE
            SFAC1(K) = -DUDS(K)/DSDS(K)
            CFAC1(K) = UAVG(K) + SFAC1(K)*SAVG(K)
         ENDIF
       ENDDO
     ENDIF
!
!-------------------------------------------------------------------+-------
     IF (MRAD == 2 .AND. MRAD_2 == 2) THEN
! --- Reference function for difference method (regression) ---
!-------------------------------------------------------------------+-------
       DO K=1,KRMAX
         SAVG(K) = SUM(SMUNU(1:MSTOP,K))/REAL(MSTOP,k8)
         UAVG(K) = SUM(UMUNU(1:MSTOP,K))/REAL(MSTOP,k8)
         DSDS(K) = SUM((SMUNU(1:MSTOP,K)-SAVG(K))*(SMUNU(1:MSTOP,K)-SAVG(K)))
         DUDS(K) = SUM((UMUNU(1:MSTOP,K)-UAVG(K))*(SMUNU(1:MSTOP,K)-SAVG(K)))
         SMIN(K) = 0.5D0*MINVAL(HDSFN1(1:imax,K))
         FMIN(K) = MINVAL( (UMUNU(1:MSTOP,K)+SMIN(K)) / &
                           (SMUNU(1:MSTOP,K)-SMIN(K)) )
         IF (DUDS(K) <= -DSDS(K)) THEN
            SFAC1(K) = 1.0D0
         ELSEIF (DUDS(K) >= DSDS(K)*FMIN(K)) THEN
            SFAC1(K) = -FMIN(K)
         ELSE
            SFAC1(K) = -DUDS(K)/DSDS(K)
         ENDIF
       ENDDO
     ENDIF
!
!=======================================================================
!--- Back interpolation of result onto HD-grid ---
!=======================================================================
!
!-----------------------------------------------------------------------
!--- Uppermost depth point ---
!-----------------------------------------------------------------------
!
      K=1
!
      I1K = I00(I0(1:ksub,K)+1) 
      L1 =  2 + imax - I1K
      L2 = L1 - 1
!
      IF (MRAD == 1) THEN
!     --- Compute kappa * (u-B) ---
          YMKA1(1:MSTOP) = F0 * UMUNU(1:MSTOP,K) * XMUNU(1:MSTOP,K)
      ENDIF
      IF (MRAD == 2 .AND. MRAD_2 == 1) THEN
!     --- Compute (u / u_fit) ---
          YMKA1(1:MSTOP) = F0 * ((UMUNU(1:MSTOP,K) + SMUNU(1:MSTOP,K)) / &
                                 (CFAC1(K) + (1.0-SFAC1(K))*SMUNU(1:MSTOP,K)))
          CFACA(K) = CFACA(K) + F0 * CFAC1(K)
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
      ENDIF
      IF (MRAD == 2 .AND. MRAD_2 == 2) THEN
!     --- Compute (u - u_fit) ---
          YMKA1(1:MSTOP) = F0 * (UMUNU(1:MSTOP,K) + SFAC1(K)*SMUNU(1:MSTOP,K))
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
      ENDIF
!       --- Flux F at top ---
      YMKA3(1:MSTOP) = F0 * YTOP(1:MSTOP)
!
!     --- Distribute YMK among the two nearest HD grid points 
      DO ks=1,ksub
        MM1=(ks-1)*imax
        HDQRT1(1:imax,K) = HDQRT1(1:imax,K) + &
                           B1(ks,K)*YMKA1(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                           B2(ks,K)*YMKA1(I00(L2(ks):L2(ks)+imax-1)+MM1)
        VRTOP(1:imax)    = VRTOP(1:imax) + &
                           B1(ks,K)*YMKA3(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                           B2(ks,K)*YMKA3(I00(L2(ks):L2(ks)+imax-1)+MM1)
      ENDDO
!
!-----------------------------------------------------------------------
!--- Lowest depth point ---
!-----------------------------------------------------------------------
!
      K = KRMAX
!
      I1K = I00(I0(1:ksub,K)+1) 
      L1 =  2 + imax - I1K
      L2 = L1 - 1
!
      IF (MRAD == 1) THEN
!     --- Compute kappa * (u-B) ---
          YMKA1(1:MSTOP) = F0 * UMUNU(1:MSTOP,K) * XMUNU(1:MSTOP,K)
      ENDIF
      IF (MRAD == 2 .AND. MRAD_2 == 1) THEN
!     --- Compute (u / u_fit) ---
          YMKA1(1:MSTOP) = F0 * ((UMUNU(1:MSTOP,K) + SMUNU(1:MSTOP,K)) / &
                                 (CFAC1(K) + (1.0-SFAC1(K))*SMUNU(1:MSTOP,K)))
          CFACA(K) = CFACA(K) + F0 * CFAC1(K)
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
      ENDIF
      IF (MRAD == 2 .AND. MRAD_2 == 2) THEN
!     --- Compute (u - u_fit) ---
          YMKA1(1:MSTOP) = F0 * (UMUNU(1:MSTOP,K) + SFAC1(K)*SMUNU(1:MSTOP,K))
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
      ENDIF
!
!       --- Flux F at bottom ---
      YMKA3(1:MSTOP) = F0 * YBOT(1:MSTOP)
!
!     --- Distribute YMK among the two nearest HD grid points 
      DO ks=1,ksub
        MM1=(ks-1)*imax
        HDQRT1(1:imax,K) = HDQRT1(1:imax,K) + &
                           B1(ks,K)*YMKA1(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                           B2(ks,K)*YMKA1(I00(L2(ks):L2(ks)+imax-1)+MM1)
        VRBOT(1:imax)    = VRBOT(1:imax) + &
                           B1(ks,K)*YMKA3(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                           B2(ks,K)*YMKA3(I00(L2(ks):L2(ks)+imax-1)+MM1)
      ENDDO
!
!-----------------------------------------------------------------------
!--- Interior points ---
!-----------------------------------------------------------------------
!
      IF (MRAD == 1) THEN
!     --- Compute kappa * (u-B) ---
        DO K=2,KRMAX-1
          I1K = I00(I0(1:ksub,K)+1) 
          L1 =  2 + imax - I1K
          L2 = L1 - 1
          YMKA1(1:MSTOP) = F0 * UMUNU(1:MSTOP,K) * XMUNU(1:MSTOP,K)
!         --- Distribute YMK among the two nearest HD grid points 
          DO ks=1,ksub
            MM1=(ks-1)*imax
            HDQRT1(1:imax,K) = HDQRT1(1:imax,K) + &
                               B1(ks,K)*YMKA1(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                               B2(ks,K)*YMKA1(I00(L2(ks):L2(ks)+imax-1)+MM1)
          ENDDO
        ENDDO
      ELSEIF (MRAD == 2 .AND. MRAD_2 == 1) THEN
!     --- Compute (u / u_fit) ---
        DO K=2,KRMAX-1
          I1K = I00(I0(1:ksub,K)+1) 
          L1 =  2 + imax - I1K
          L2 = L1 - 1
          YMKA1(1:MSTOP) = F0 * ((UMUNU(1:MSTOP,K) + SMUNU(1:MSTOP,K)) / &
                                 (CFAC1(K) + (1.0-SFAC1(K))*SMUNU(1:MSTOP,K)))
          CFACA(K) = CFACA(K) + F0 * CFAC1(K)
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
!         --- Distribute YMK among the two nearest HD grid points 
          DO ks=1,ksub
            MM1=(ks-1)*imax
            HDQRT1(1:imax,K) = HDQRT1(1:imax,K) + &
                               B1(ks,K)*YMKA1(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                               B2(ks,K)*YMKA1(I00(L2(ks):L2(ks)+imax-1)+MM1)
          ENDDO
        ENDDO
      ELSEIF (MRAD == 2 .AND. MRAD_2 == 2) THEN
!     --- Compute (u - u_fit) ---
        DO K=2,KRMAX-1
          I1K = I00(I0(1:ksub,K)+1) 
          L1 =  2 + imax - I1K
          L2 = L1 - 1
          YMKA1(1:MSTOP) = F0 * (UMUNU(1:MSTOP,K) + SFAC1(K)*SMUNU(1:MSTOP,K))
          SFACA(K) = SFACA(K) + F0 * SFAC1(K)
!         --- Distribute YMK among the two nearest HD grid points 
          DO ks=1,ksub
            MM1=(ks-1)*imax
            HDQRT1(1:imax,K) = HDQRT1(1:imax,K) + &
                               B1(ks,K)*YMKA1(I00(L1(ks):L1(ks)+imax-1)+MM1) + &
                               B2(ks,K)*YMKA1(I00(L2(ks):L2(ks)+imax-1)+MM1)
          ENDDO
        ENDDO
      ENDIF
!
!=======================================================================
!--- Back interpolation of result onto HD-grid finished ---
!=======================================================================
!
500 CONTINUE
!
!***********************************************************************
!--- Loop through all PHI-angles and sub rays finished ---
!***********************************************************************
!
   IF (MRAD == 1) THEN
      IF (NFILL == 1) THEN
!--- Compute J * kappa [1/cm] for normal depth points, finish ---
        DO J=jmin,jmax
          K = (jmax-J) + 1 
          HDQRT(1:imax,J) = HDQRT1(1:imax,K)
        ENDDO
      ELSE
!--- Compute J * kappa [1/cm] for normal depth points ---
        DO J=jmin,jmax
          K = (jmax-J)*NFILL + 1
          HDQRT(1:imax,J) = OFN*HDQRT1(1:imax,K)
        ENDDO
!---Compute J * kappa [1/cm] for intermediate depth points ---
!--- Interpolation of results onto HD grid ---
        DO JJ=1,NFILL/2
          W12 = OFN
          IF (2*JJ == NFILL) W12 = F05*W12
!--- Upper boundary:
            J  = jmax
           K2  = 1 + JJ
           WJ2 = W12 * F20
           YK1(1:imax)     = WJ2*HDQRT1(1:imax,K2)
           HDQRT(1:imax,J) =     HDQRT (1:imax,J)  +   YK1(1:imax)
!--- Lower boundary:
            J  = jmin
           K1  = (jmax-jmin)*NFILL + 1 - JJ
           WJ1 = W12 * F20
           YK1(1:imax)     = WJ1*HDQRT1(1:imax,K1)
           HDQRT(1:imax,J) =     HDQRT (1:imax,J)  +   YK1(1:imax)
 !--- Interior points:
           DO J=jmin+1,jmax-1
             K1  = (jmax-J)*NFILL + 1 - JJ
             K2  = (jmax-J)*NFILL + 1 + JJ
             WJ1 = W12 * DELTAZ(J)   * WTFAC(J)
             WJ2 = W12 * DELTAZ(J-1) * WTFAC(J)
             YK1(1:imax) = WJ1*HDQRT1(1:imax,K1) + WJ2*HDQRT1(1:imax,K2)
             HDQRT(1:imax,J) = HDQRT (1:imax,J) + YK1(1:imax)
           ENDDO
        ENDDO
      ENDIF
   ELSEIF (MRAD == 2 .AND. MRAD_2 == 1) THEN
        DO J=jmin,jmax
           K = (jmax-J)*NFILL + 1
           HDQRT(1:imax,J) = HDQRT1(1:imax,K) * &
                             (CFACA(K)+(1.0-SFACA(K))*HDSFN1(1:imax,K))
        ENDDO
   ELSEIF (MRAD == 2 .AND. MRAD_2 == 2) THEN
        DO J=jmin,jmax
           K = (jmax-J)*NFILL + 1
           HDQRT(1:imax,J) = HDQRT1(1:imax,K) + (1.0-SFACA(K))*HDSFN1(1:imax,K)
        ENDDO
   ENDIF
!
   END SUBROUTINE rsys2d
!-----------------******--------------------------------------------+-------

!-------------*******-----------------------------------------------+-------
   SUBROUTINE lcfrad3(NU1, NU2, MFRT, NNRT, PLOG, T, RHO, QRAD3, TRAD3, &
                      BLCLAM, OPCLAM, YSURF, FRZ1, FRZ2, &
                      FRBOT0, FRBOT1, FRBOT2, FRTOP0, FRTOP1, FRTOP2, FRTAU0, &
                      Y3AVG, Y3RMS, BFLX, &
                      ifco, ifdust, icount, nquc, quc, output_flag)
!-------------------------------------------------------------------+-------
!
!    ==================================
!    Last Update:   6-Oct-1997 17:22:29
!    f90-Version,  MS (14-JUN-1999)
!    3D-Version,   MS (15-APR-2000)
!    3D-Version,   MS (06-NOV-2000)
!    OMP-Version,  MS (23-MAR-2001)
!    Opacity calculation inside lcfrad3, MS (28-Nov-2002)
!    Timing calls inside lcfrad3,
!    OMP DO SCHELDUE(RUNTIME)         MS (23-Mar-2004)
!    OpenMP: DEFAULT(NONE)            MS (02-Aug-2007)
!    Local 3D arrays allocatable,
!    qrad3 array no longer used in call of op_s0
!                                     MS (03-Jan-2009)
!    dust opacity handled by subroutine op_dust
!                                     MS (22-DEC-2009)
!    OpenMP bug related to summation in y1avg2, y2avg2 
!    corrected                        MS (12-Jul-2011)
!    Modified, NNRT                   MS (04-Feb-2015)
!    Modified, MFRT                   MS (18-MAR-2016)
!    ==================================
!
!    This routine solves the radiative transfer equation along a number
!    of rays with ntheta different inclination angles theta.
!
!    The radiation transfer along vertical rays (theta = 0) is solved
!    by RAD itself. RSYS-Routines are called for the solution of the
!    transfer equation along oblique rays. These routines call rayvec.
!
! INPUT:
!   NU1:  ('frequency_1') integer, first index of frequency loop
!   NU2:  ('frequency_2') integer, last  index of frequency loop
!   PLOG: ('log P')       real array(m1:n1,m2:n2,m3:n3), log gas pressure
!   T:    ('T')           real array(m1:n1,m2:n2,m3:n3), temperature
!   RHO:  ('rho')         real array(m1:n1,m2:n2,m3:n3), gas density
!   BFLX: ('bottom_flux') real,    radiative flux enforced at bottom is
!                                  BFLX * sigma * Teff^4
!   IFCO: ('CO switch')   IFCO=0: Standard case, use grey or multi-band scheme
!                         IFCO=1: Use CO 2-band scheme, with equilibrium CO concentrations
!                         IFCO=2: Use CO 2-band scheme,  non-equilibrium CO concentrations
! IFDUST: ('DUST switch') IFDUST=0: Standard case, use grey or multi-band scheme
!                         IFDUST=1: Grey dust opacity is added to opacity of all bins
!
!   output_flag           logical, if .true., intensity maps are stored in YSURF.
!                                  (default is .false.)
!
! OUTPUT:
!   QRAD3:  ('div_Frad')    real array(idim1,idim2,jdim), output of lcfrad3:
!                           4*Pi*kappa*(J-B)[MRAD==1]  or 
!                           4*Pi*(J)_1      [MRAD==2, MRAD_2==1]  or
!                           4*Pi*(J)_2      [MRAD==2, MRAD_2==2]
!   TRAD3:  ('time_rad')    real array(idim1,idim2,jdim), output of lcfrad3:
!                           term necessary to compute radiative time scale
!
!   YSURF: ('I_surface')    real array(idim1,idim2,0:nu2), contains
!                           frequency-integrated emergent intensity as well as
!                           emergent intensity in all frequency bins for vertical ray system
!   FRZ1:   ('Frad_z_1')    real array(1:jdim), horizontally averaged radiative
!                                               flux at z(j) (cell centers)
!   FRZ2:   ('Frad_z_2')    real array(0:jdim), horizontally averaged radiative
!                                               flux between z(j) (cell boundaries)
!   FRBOT0: ('Frad_bottom') real, normalized radiative flux at z(jmin)
!   FRBOT1: ('Frad_bottom') real, normalized radiative flux at z(1)-0.5*deltaz(1)
!   FRBOT2: ('Frad_bottom') real, normalized radiative flux at z(1)+0.5*deltaz(1)
!   FRTOP0: ('Frad_top')    real, normalized radiative flux at z(jdim) from Feautrier
!                                 flux v at upper boundary
!   FRTOP1: ('Frad_top')    real, normalized radiative flux at z(jdim) from trapezoidal
!                                 integration of div Frad = FRZ1(jdim)
!   FRTOP2: ('Frad_top')    real, normalized radiative flux at z(jdim)+0.5*deltaz(jdim) 
!                                 from piecewise constant integration
!                                 of div Frad = FRZ2(jdim)
!   FRTAU0: ('Frad_top')    real, normalized radiative flux at tau=0 from emergent
!                                 ray intensities
!   Y3AVG: ('Intens_avg')   real, array(0:ntheta,0:nbin) horizontally averaged emergent 
!                                 intensity as a function of theta angle and opacity bin
!   Y3RMS: ('Intens_rms')   real, array(0:ntheta,0:nbin) corresponding intensity contrast 
!                                 as a function of theta angle and opacity bin
!
!-------------------------------------------------------------------+-------
!
   USE comrd1_def
   USE comrd2_def
   USE comrd3_def
   USE comrd4_def
   USE comrd5_def
   USE op_sf0_module
   USE timing_module
!
   IMPLICIT NONE
!
!--- Dummy arguments:
!
   INTEGER,                 INTENT(IN)    :: NU1, NU2, MFRT, NNRT, &
                                             IFCO, IFDUST, ICOUNT
   INTEGER,                 INTENT(IN)    :: nquc
   REAL,                    INTENT(IN)    :: BFLX
   REAL,                    INTENT(OUT)   :: FRBOT0, FRBOT1, FRBOT2, &
                                             FRTOP0, FRTOP1, FRTOP2, FRTAU0
   REAL, DIMENSION(0:,0:),  INTENT(OUT)   :: Y3AVG, Y3RMS
   REAL, DIMENSION(:,:,0:), INTENT(OUT)   :: YSURF
   REAL, DIMENSION(:),      INTENT(OUT)   :: FRZ1
   REAL, DIMENSION(0:),     INTENT(OUT)   :: FRZ2

   REAL, DIMENSION(m1:n1,m2:n2,m3:n3), INTENT(OUT)  :: QRAD3, TRAD3
   REAL, DIMENSION(m1:n1,m2:n2,m3:n3), INTENT(IN)   :: PLOG, T, RHO
   REAL, DIMENSION(m1:n1,m2:n2,m3:n3), INTENT(OUT)  :: BLCLAM, OPCLAM
!
   REAL, DIMENSION(m1:n1,m2:n2,m3:n3,1:nquc), &
            OPTIONAL,                  INTENT(IN)   :: quc
   LOGICAL, OPTIONAL,                  INTENT(IN)   :: output_flag
!
!--- Local (automatic) arrays:
!      
   INTEGER,       DIMENSION(kmax)            :: JK_NT
   REAL(KIND=k8), DIMENSION(kmax)            :: DSRAY_NT, DXR_NT
   REAL(KIND=k8), DIMENSION(iimax,jmin:jmax) :: HDQRT
   REAL(KIND=k8), DIMENSION(iimax,kmax)      :: HDSFN,HDKAP
   REAL(KIND=k8), DIMENSION(mmmax,kmax)      :: OMUNU,SMUNU,TMUNU,UMUNU,XMUNU
   REAL(KIND=k8), DIMENSION(iimax)           :: VRTOP,VRBOT
   REAL(KIND=k8), DIMENSION(1:jmax)          :: DIVFR,FRADZ1
   REAL(KIND=k8), DIMENSION(0:jmax)          :: FRADZ2
!
   REAL(KIND=k8), DIMENSION(i1max)                   :: YIM0, YIP0
   REAL(KIND=k8), DIMENSION(i1max)                   :: DFX
   REAL(KIND=k8), DIMENSION(i2max)                   :: DFY
   REAL(KIND=k8), DIMENSION(i1max)                   :: DIPNX, HTAU0X
   REAL(KIND=k8), DIMENSION(i2max)                   :: DIPNY, HTAU0Y
   REAL(KIND=k8), DIMENSION(i1max,i2max)             :: DIPN2, HTAU02
   REAL(KIND=k8), DIMENSION(0:jmin)                  :: FZDAVG
   REAL(KIND=k8), DIMENSION(0:ntheta)                :: Y1AVGT, Y2AVGT, Y0AVG
   REAL(KIND=k8), DIMENSION(0:ntheta, nu1:nu2)       :: Y1AVG2, Y2AVG2
   REAL(KIND=k8), DIMENSION(m1:n1,m2:n2,nu1:nu2)     :: Qjmin      
!
!--- Local allocatable 3D arrays:
!      
   REAL(KIND=k8), DIMENSION(:,:,:), ALLOCATABLE :: DWORK1, FZDIFF, YS1
   REAL,          DIMENSION(:,:,:), ALLOCATABLE :: DTRAD3
!
!
!--- Local scalar variables:
!
   INTEGER :: i, i1, i2, i3, J, JJ, K, K1, K2, KR00, KRN, &
              NF0, NFN, NS0, NT, NNU, NU
   INTEGER :: NFILL_NT, NSTEP_NT, KRMAX_NT, KZERO_NT
   REAL(KIND=k8), PARAMETER :: F00=0.0D0, F05=0.5D0, F10=1.0D0, F20=2.0D0
   REAL(KIND=k8) :: HTOP, HBOT, SOURC0, W04PI, WN4PI, OF0, &
                    Y1AVG, Y2AVG, SUM120, SUM12N

   REAL          :: blinci, blsurf, dipnwt, Tbotm
!
!-------------------------------------------------------------------+-------
!
!-----------------------------------------------------------------------
!--- Preparations ---
!-----------------------------------------------------------------------
!
   nnu   = nu2 - nu1 + 1
   W04PI = PI40 * WTH(0)
!
!  --- Clear HTOP and HBOT ---
   HTOP = F00
   HBOT = F00
!
!  --- Clear y1avg2, y2avg2 arrays ---
   y1avg2(0:ntheta,nu1:nu2) = F00
   y2avg2(0:ntheta,nu1:nu2) = F00
!
   NS0 = NSTEP(0)
   NF0 = NFILL(0)
   NFN = NFILL(ntheta)
   OF0 = F10 / REAL(NF0,k8)
   KR00= KRMAX(0)
   KRN = KRMAX(ntheta)
!
!--- Allocate work arrays:
!      
   if( .NOT. allocated(YS1))    allocate(   YS1(i1max,i2max, nu1: nu2))
   if( .NOT. allocated(DWORK1)) allocate(DWORK1(i1max,i2max,   1:jmax))
   if( .NOT. allocated(FZDIFF)) allocate(FZDIFF(i1max,i2max,   0:jmin))
   if( .NOT. allocated(DTRAD3)) allocate(DTRAD3(m1:n1,m2:n2,m3:n3))
!
!-----------------------------------------------------------------------
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(QRAD3, TRAD3, m1,n1, m2,n2, m3,n3)
!-----------------------------------------------------------------------
!
!  --- Clear QRAD3 and TRAD3 arrays ---
  !$OMP DO PRIVATE(i3)
   I3LOOP_0: do i3=m3,n3
      QRAD3(m1:n1,m2:n2,i3) = F00
      TRAD3(m1:n1,m2:n2,i3) = F00
   end do I3LOOP_0
  !$OMP END DO
!
!-----------------------------------------------------------------------
!$OMP END PARALLEL
!-----------------------------------------------------------------------
!  --- Clear FZDAVG and DIVFR arrays ---
   FZDAVG(0:jmin) = F00
   DIVFR( 1:jmax) = F00
!
!--- Preparations finished ---
!-----------------------------------------------------------------------
!
!***********************************************************************
!--- Frequency loop. RAD cycles through this loop once (gray case) 
!--- NBAND times (frequency-dependent case) ---
!***********************************************************************
!
! --- Compute approximate mean temperature at bottom ---
   Tbotm=sum(T(m1:n1,m2:n2,m3))/real((n1-m1+1)*(n2-m2+1))
!
   NULOOP: DO NU = NU1, NU2
!
   CALL timing_start('lcfrad3 opacity / sourcef')
!
! --- Compute opacity and source function ---
!
   IF ((ifdust == 1 .or. ifco == 2) .and. PRESENT(quc)) THEN
      CALL op_s1(Tinci, Tsurf, Tbotm, T, Plog, rho, gfac, &
                 nu, ifco, ifdust, m1,n1, m2,n2, m3,n3, nquc, &
                 blinci, blsurf, dipnwt, blclam, opclam, dtrad3, quc=quc)
   ELSE
      CALL op_s1(Tinci, Tsurf, Tbotm, T, Plog, rho, gfac, &
                 nu, ifco, ifdust, m1,n1, m2,n2, m3,n3, nquc, &
                 blinci, blsurf, dipnwt, blclam, opclam, dtrad3)
   ENDIF
!
   sourc0 = blsurf
!
   CALL timing_stop('lcfrad3 opacity / sourcef')
!
!=======================================================================
!--- Radiative transfer in diffusion approximation (nthick > 0) ---
!=======================================================================
!
   IF (nthick > 0) THEN
!
     CALL timing_start('lcfrad3 Diffusion RT')
!
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(BLCLAM, DCX, DCY, DCZ, DWORK1, FZDIFF, OPCLAM, &
!$OMP        I1X, I1Y, i1max, i2max, m1, m2, m3, jmin) & 
!$OMP PRIVATE(DFX, DFY) 
!
     !$OMP DO PRIVATE(i1,i2,i3)
      I3LOOP_a: do i3=1,jmin
        I1LOOP_a: do i1=1,i1max
          DFY(1:i2max) = (blclam(m1-1+i1, m2-1+I1Y(2:i2max+1),m3-1+i3) - &
                          blclam(m1-1+i1, m2-1+I1Y(1:i2max),  m3-1+i3))/ &
                         (opclam(m1-1+i1, m2-1+I1Y(2:i2max+1),m3-1+i3) + &
                          opclam(m1-1+i1, m2-1+I1Y(1:i2max),  m3-1+i3))
          IF (MRAD == 1) THEN
          DWORK1(i1,1:i2max,i3) = DCY*(DFY(I1Y(1:i2max))-DFY(I1Y(0:i2max-1)))
          ENDIF
          IF (MRAD == 2) THEN
          DWORK1(i1,1:i2max,i3) = &
                           4.D0*DCY*(DFY(I1Y(1:i2max))-DFY(I1Y(0:i2max-1)))/&
                       (opclam(m1-1+i1,m2-1+I1Y(0:i2max-1),m3-1+i3) + &
                   2.D0*opclam(m1-1+i1,m2-1+I1Y(1:i2max  ),m3-1+i3) + &
                        opclam(m1-1+i1,m2-1+I1Y(2:i2max+1),m3-1+i3))
          ENDIF
        end do I1LOOP_a
        I2LOOP_a: do i2=1,i2max
          DFX(1:i1max) = (blclam(m1-1+I1X(2:i1max+1),m2-1+i2,m3-1+i3) - &
                          blclam(m1-1+I1X(1:i1max),  m2-1+i2,m3-1+i3))/ &
                         (opclam(m1-1+I1X(2:i1max+1),m2-1+i2,m3-1+i3) + &
                          opclam(m1-1+I1X(1:i1max),  m2-1+i2,m3-1+i3))
          IF (MRAD == 1) THEN
          DWORK1(1:i1max,i2,i3) = DWORK1(1:i1max,i2,i3) + &
                                  DCX*(DFX(I1X(1:i1max))-DFX(I1X(0:i1max-1)))
          ENDIF
          IF (MRAD == 2) THEN
          DWORK1(1:i1max,i2,i3) = DWORK1(1:i1max,i2,i3) + &
                           4.D0*DCX*(DFX(I1X(1:i1max))-DFX(I1X(0:i1max-1)))/&
                       (opclam(m1-1+I1X(0:i1max-1),m2-1+i2,m3-1+i3) + &
                   2.D0*opclam(m1-1+I1X(1:i1max  ),m2-1+i2,m3-1+i3) + &
                        opclam(m1-1+I1X(2:i1max+1),m2-1+i2,m3-1+i3))
          ENDIF
        end do I2LOOP_a
        DO i1=1,i1max
          DO i2=1,i2max
          FZDIFF(i1,i2,i3) = DCZ(i3) * &
          (blclam(m1+i1-1,m2+i2-1,m3+i3) - blclam(m1+i1-1,m2+i2-1,m3+i3-1)) / &
          (opclam(m1+i1-1,m2+i2-1,m3+i3) + opclam(m1+i1-1,m2+i2-1,m3+i3-1))
          ENDDO
        ENDDO
! --- This more elegant construct does not always work ---
!        FZDIFF(1:i1max,1:i2max,i3) = DCZ(i3) * &
!              (blclam(m1:n1,m2:n2,m3+i3) - blclam(m1:n1,m2:n2,m3-1+i3)) / &
!              (opclam(m1:n1,m2:n2,m3+i3) + opclam(m1:n1,m2:n2,m3-1+i3))
      end do I3LOOP_a
     !$OMP END DO
!
!$OMP END PARALLEL
!
!=======================================================================
!-- Lower boundary condition for radiative flux in diffusion approximation
!
     IF (mradbc == 0) THEN
       FZDIFF(1:i1max,1:i2max,0) = FZDIFF(1:i1max,1:i2max,1)
     ELSE
       FZDIFF(1:i1max,1:i2max,0) = -FREF * dipnwt * BFLX
     ENDIF
     FZDAVG(0) = FZDAVG(0) - A12*SUM(FZDIFF(1:i1max,1:i2max,0))
!=======================================================================
! 
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(A12, DWORK1, FZDIFF, FZDAVG, BLCLAM, OPCLAM, WTFAC, &
!$OMP        i1max, i2max, jmin, m1, m2, m3, n1, n2)
!
!  Add flux diffusion in the vertical direction
!  Compute horizontally averaged fluxes
     IF (MRAD == 1) THEN
    !$OMP DO PRIVATE(i3)
     I3LOOP_b1: do i3=1,jmin
       DWORK1(1:i1max,1:i2max,i3) = DWORK1(1:i1max,1:i2max,i3) + & 
      (FZDIFF(1:i1max,1:i2max,i3) - FZDIFF(1:i1max,1:i2max,i3-1)) * WTFAC(i3)
     end do I3LOOP_b1
    !$OMP END DO
     ENDIF
!
     IF (MRAD == 2) THEN
       !$OMP MASTER
       DWORK1(1:i1max,1:i2max,1) = DWORK1(1:i1max,1:i2max,1)  + & 
      (FZDIFF(1:i1max,1:i2max,1) - FZDIFF(1:i1max,1:i2max,0)) * &
       WTFAC(1) / opclam(m1:n1,m2:n2,m3)
       !$OMP END MASTER
       IF (jmin > 1) THEN
        !$OMP DO PRIVATE(i3)
           I3LOOP_b2: do i3=2,jmin
              DWORK1(1:i1max,1:i2max,i3) = DWORK1(1:i1max,1:i2max,i3) + & 
             (FZDIFF(1:i1max,1:i2max,i3) - FZDIFF(1:i1max,1:i2max,i3-1)) * &
             4.D0*WTFAC(i3) / &
                 (opclam(m1:n1,m2:n2,m3-1+i3-1) + &
             2.D0*opclam(m1:n1,m2:n2,m3-1+i3  ) + &
                  opclam(m1:n1,m2:n2,m3-1+i3+1))
           end do I3LOOP_b2
        !$OMP END DO
       ENDIF
! --- change contents of DWORK1: 4 Pi (J-B) --> 4 Pi J ---
        !$OMP DO PRIVATE(i3)
           I3LOOP_b2b: do i3=1,jmin
              DWORK1(1:i1max,1:i2max,i3) = DWORK1(1:i1max,1:i2max,i3) + & 
                                           PI40*blclam(m1:n1,m2:n2,m3-1+i3)
           end do I3LOOP_b2b
        !$OMP END DO
     ENDIF
!
!  Compute horizontally averaged fluxes
    !$OMP DO PRIVATE(i3)
     I3LOOP_b3: do i3=1,jmin
       FZDAVG(i3)=FZDAVG(i3)-A12*SUM(FZDIFF(1:i1max,1:i2max,i3))
     end do I3LOOP_b3
    !$OMP END DO
!
!$OMP END PARALLEL
!
! --- Save diffusion approximation results at interface jmin ---
!
     Qjmin(m1:n1,m2:n2,nu) = DWORK1(1:i1max,1:i2max,jmin) 
!
     CALL timing_stop('lcfrad3 Diffusion RT')
!
   ENDIF
!
!=======================================================================
!--- Radiative transfer in diffusion approximation done  ---
!=======================================================================
!
   CALL timing_start('lcfrad3 raysystem RT')
!
!-----------------------------------------------------------------------
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(A12, BLCLAM, BFLX, CTH, DIPF, DIPNWT, DSRAY, DXR, &
!$OMP        DWORK1, FARAD, GNRAD, GRAV, HMRAD, HNRAD, HTOP, HBOT, &
!$OMP        HTAU0, OPCLAM, OF0, SOURC0, TINCI, DINCI, BLINCI, &
!$OMP        SPH, WPH, W04PI, WJ1, WJ2, WN1, WN2, &
!$OMP        WTH, WXY, Y1AVG2, Y2AVG2, YS1, &
!$OMP        DIPN2, HTAU02, W1FAC, W2FAC, FZDIFF, Plog, RHO, &
!$OMP        I0X, I0Y, I1MAX, I2MAX, ICOUNT, JK, JMIN, JMAX, &
!$OMP        m1, m2, m3, n1, n2, n3, MFRT, NNRT, NFILL, NFN, NF0, NS0, &
!$OMP        NSTEP, MTHINT, NTHETA, NTHICK, NU, KRN, KR00, KMAX, KRMAX) &
!$OMP PRIVATE(HDKAP, HDSFN, OMUNU, SMUNU, TMUNU, UMUNU, XMUNU, &
!$OMP         HDQRT, YIM0, YIP0, VRTOP, VRBOT, DIPNX, DIPNY, &
!$OMP         HTAU0X, HTAU0Y, WN4PI, Y1AVGT, Y2AVGT, Y1AVG, Y2AVG, &
!$OMP         I, J, JJ, K, K1, K2, NT, NFILL_NT, NSTEP_NT, KRMAX_NT, KZERO_NT, &
!$OMP         JK_NT, DSRAY_NT, DXR_NT)
!-----------------------------------------------------------------------
!
# ifdef LCFrad_sub_t01
   CALL timing_start('lcfrad3 I2LOOP')
# endif
!
!  --- Clear thread-private arrays y1avgt, y2avgt ---
   y1avgt(0:ntheta) = F00
   y2avgt(0:ntheta) = F00
!
!$OMP DO PRIVATE(i2) REDUCTION(+: HTOP,HBOT) SCHEDULE(RUNTIME)
   I2LOOP: DO i2 = 1, i2max
!
!=======================================================================
!--- Store opacities HDKAP = 1/cm; and Source Function HDSFN ---
!=======================================================================
!-----------------------------------------------------------------------
!--- Attention: Second index of HDAKP, HDSFN increases from top to bottom
!-----------------------------------------------------------------------
!
!--- Set up opacity and source function in planes parallel to x-direction ---
!
     IF (NFN == 1) THEN
       HDSFN(1:i1max,1:KRN) = blclam(m1:n1,i2+m2-1,n3:m3+jmin-1:-1) 
       HDKAP(1:i1max,1:KRN) = opclam(m1:n1,i2+m2-1,n3:m3+jmin-1:-1)
     ELSE
       HDSFN(1:i1max,1:KRN:NFN) = blclam(m1:n1,i2+m2-1,n3:m3+jmin-1:-1)
       HDKAP(1:i1max,1:KRN:NFN) = opclam(m1:n1,i2+m2-1,n3:m3+jmin-1:-1)
       DO JJ=1,NFN-1
         K1 = NFN+1-JJ
         K2 = KRN-JJ
         HDSFN(1:i1max,K1:K2:NFN) = WN1(JJ)*HDSFN(1:i1max,1+NFN:KRN:NFN) + &
                                    WN2(JJ)*HDSFN(1:i1max,1:KRN-NFN:NFN)
! --- Exponential interpolation of kappa ---------------------------------
         HDKAP(1:i1max,K1:K2:NFN) = HDKAP (1:i1max,1+NFN:KRN:NFN)* &
                                   (HDKAP (1:i1max,1:KRN-NFN:NFN)/ &
                                    HDKAP (1:i1max,1+NFN:KRN:NFN))**WN2(JJ)
! ------------------------------------------------------------------------
       ENDDO
     ENDIF
!
!=======================================================================
!--- Computation of opacities and source function finished ---
!=======================================================================
!
!=======================================================================
!--- Computation of radiative transfer along vertical rays ---
!=======================================================================
!
!    --- Set kappa and source function ---
     SMUNU(1:i1max,1:KR00) = HDSFN(1:i1max,1:kmax:NS0)
     OMUNU(1:i1max,1:KR00) = HDKAP(1:i1max,1:kmax:NS0) 
!
!-----------------------------------------------------------------------
!--- Attention: Second index of TMUNU array increases from top to bottom
!-----------------------------------------------------------------------
!
     IF (HTAU0 >= 0.0) THEN
!    --- Optical depth at upper boundary: tau0 = kappa * HTAU0 ---
       HTAU0X(1:i1max) = HTAU0
     ELSE
!    --- Optical depth at upper boundary: tau0 = kappa * HP0 * |HTAU0| ---
       HTAU0X(1:i1max) = -HTAU0 * EXP(Plog(m1:n1,i2+m2-1,n3)) / &
                         (GRAV  *     RHO( m1:n1,i2+m2-1,n3))
     ENDIF
     HTAU02(1:i1max,i2)= HTAU0X
!
     TMUNU (1:i1max,1) = OMUNU(1:i1max,1) * HTAU0X(1:i1max)
     DSRAY_NT          = DSRAY(1:KMAX,0)
!
!     --- Compute optical depth scale along vertical rays ---
     CALL raytas(i1max, KR00, DSRAY_NT, OMUNU, UMUNU, TMUNU, MFRT)
!
!     --- Set lower boundary condition for radiative transfer ---
     IF (nthick == 0) THEN
!    --- DIPN = 0 [mradbc==0] or 2*v(N) from specified Teff [mradbc==1] ---
        DIPNX(1:i1max) = BFLX * DIPF * dipnwt
     ELSE
!    --- Compute DIPN = 2*v(N) from diffusion approximation ---
! --- 2*v = 1.5*F/PI ---
        DIPNX(1:i1max) = -DIPF*(W2FAC(jmin)*FZDIFF(1:i1max,i2,jmin-1)  + &
                                W1FAC(jmin)*FZDIFF(1:i1max,i2,jmin  )) 
     ENDIF
!
!    --- Incident intensity at tau = 0 ---
     YIM0(1:i1max) = F00
! --- Incident radiation from external source ---
     IF ((Tinci > 0.0) .AND. (Dinci > 0.0) .AND. (mthint /= 2)) THEN
       YIM0(1:i1max) = YIM0(1:i1max) + 0.5D0*blinci*Dinci/WTH(0)
     ENDIF
!
     IF (MFRT <= 0) THEN
!--- Compute mean opacity XMUNU = dtau/dz on ray system:
       K = 1
       XMUNU(1:i1max,K)= TMUNU(1:i1max,K+1) / DSRAY_NT(K+1)
       K = KR00
       XMUNU(1:i1max,K)= TMUNU(1:i1max,K  ) / DSRAY_NT(K)
       DO K=2,KR00-1
         XMUNU(1:i1max,K) = (TMUNU(1:i1max,K+1) + TMUNU(1:i1max,K)) / &
                            (DSRAY_NT(K+1)+DSRAY_NT(K))
       ENDDO
     ELSEIF (MFRT == 1) THEN
!--- Compute mean opacity XMUNU = local opacity OMUNU on ray system:
       DO K=1,KR00
         XMUNU(1:i1max,K) = OMUNU(1:i1max,K)
       ENDDO
     ELSE
       WRITE(*,*) "lcfrad3: illegal value of MFRT:", MFRT
       STOP
     ENDIF
!
!    --- Solve radiation transfer along vertical rays ---
     CALL rayvec5(i1max, KR00, DSRAY_NT, TMUNU, XMUNU, SMUNU, SOURC0, &
                  GNRAD, HMRAD, HNRAD, YIM0, DIPNX, IJMB, &
                  UMUNU, VRTOP, VRBOT, YIP0 )
!
!--- Compute emergent I(+) at tau = 0 ---
     YS1 (1:i1max,i2,nu) = YIP0(1:i1max) 
!
! --- Compute fluxes at top and at bottom boundaries ---
     HTOP = HTOP + W04PI*CTH(0)*A12*SUM(VRTOP(1:i1max))
     HBOT = HBOT + W04PI*CTH(0)*A12*SUM(VRBOT(1:i1max))
!
     DIPN2(1:i1max,i2) = DIPNX
!
!-------------------------------------------------------------------+-------
     IF (MRAD == 1) THEN
!-------------------------------------------------------------------+-------
!
!    --- Multiply results with weighting factor W04PI, save as DWORK1 ---
!    --- DWORK1: 4*Pi*Xkappa*(J-B):
       IF (NF0 == 1) THEN
         DO J=jmin,jmax
           K = (jmax-J) + 1 
           DWORK1(1:i1max,i2,J) = W04PI*UMUNU(1:i1max,K)*XMUNU(1:i1max,K)
         ENDDO
       ELSE
         DO J=jmin,jmax
           K = (jmax-J)*NF0 + 1
           DWORK1(1:i1max,i2,J) = OF0*W04PI*UMUNU(1:i1max,K)*XMUNU(1:i1max,K)
         ENDDO
!
         DO JJ=1,NF0/2
!--- Upper boundary:
            J  = jmax
           K2  = 1 + JJ
           DWORK1(1:i1max,i2,J) =  DWORK1(1:i1max,i2,J) + &
                          WJ2(JJ,J)*UMUNU(1:i1max,K2)*XMUNU(1:i1max,K2)
!--- Lower boundary:
            J  = jmin
           K1  = (jmax-jmin)*NF0 + 1 - JJ
           DWORK1(1:i1max,i2,J) =  DWORK1(1:i1max,i2,J) + &
                          WJ1(JJ,J)*UMUNU(1:i1max,K1)*XMUNU(1:i1max,K1)
 !--- Interior points:
           DO J=jmin+1,jmax-1
             K1  = (jmax-J)*NF0 + 1 - JJ
             K2  = (jmax-J)*NF0 + 1 + JJ
             DWORK1(1:i1max,i2,J) =  DWORK1(1:i1max,i2,J) + &
                            WJ1(JJ,J)*UMUNU(1:i1max,K1)*XMUNU(1:i1max,K1) + &
                            WJ2(JJ,J)*UMUNU(1:i1max,K2)*XMUNU(1:i1max,K2) 
           ENDDO
         ENDDO
       ENDIF
     ENDIF
!-------------------------------------------------------------------+-------
     IF (MRAD == 2) THEN
!-------------------------------------------------------------------+-------
!    --- Multiply results with weighting factor W04PI, save as DWORK1 ---
!    --- DWORK1: 4*Pi*(J):
       DO J=jmin,jmax
         K = (jmax-J)*NF0 + 1
         DWORK1(1:i1max,i2,J) = W04PI * (UMUNU(1:i1max,K) + SMUNU(1:i1max,K))
       ENDDO
!
     ENDIF
!-------------------------------------------------------------------+-------
!
!=======================================================================
!--- Computation of radiative transfer along vertical rays finished ---
!=======================================================================
!
   IF (ntheta /= 0) THEN
!
!=======================================================================
!--- Radiative transfer along rays with inclination +/- theta ---
!=======================================================================
!
     DO NT=1,ntheta
!
!      --- Solve radiation transfer along oblique rays ---
       DIPNX  =  DIPN2(1:i1max,i2) * CTH(NT)
       HTAU0X = HTAU02(1:i1max,i2) / CTH(NT)
!
       NFILL_NT = NFILL(NT)
       NSTEP_NT = NSTEP(NT)
       KRMAX_NT = KRMAX(NT)
       KZERO_NT = NFILL_NT * MOD(icount,(jmax-jmin+1)) + 1
       JK_NT    =    JK(1:KMAX,NT)
       DXR_NT   =   DXR(1:KMAX,NT)
       DSRAY_NT = DSRAY(1:KMAX,NT)
!=======================================================================
       CALL rsys2d(SPH, WPH, DIPNX, HTAU0X, &
                   MFRT, NNRT, NFILL_NT, NSTEP_NT, KRMAX_NT, KZERO_NT, &
                   JK_NT, DSRAY_NT, DXR_NT, &
                   HDSFN, HDKAP, OMUNU, UMUNU, SMUNU, TMUNU, XMUNU, &
                   HDQRT, VRTOP, VRBOT, Y1AVG, Y2AVG, SOURC0, I0X, i1max)
!=======================================================================
!
       y1avgt(nt) = y1avgt(nt) + Y1AVG
       y2avgt(nt) = y2avgt(nt) + Y2AVG
!
       WN4PI = PI40 * WTH(NT) * WXY
!
!      --- Multiply results with weighting factor WN4PI, add to DWORK1 ---
       IF (MRAD /= 0) THEN
         DWORK1(1:i1max,i2,jmin:jmax) = DWORK1(1:i1max,i2,jmin:jmax) + &
                                   WN4PI*HDQRT(1:i1max,   jmin:jmax)
       ENDIF
!  
!      --- Add contribution to flux at top and bottom boundaries ---
       HTOP = HTOP + WN4PI*CTH(NT)*A12*SUM(VRTOP(1:i1max))
       HBOT = HBOT + WN4PI*CTH(NT)*A12*SUM(VRBOT(1:i1max))
!
     ENDDO
!
   ENDIF
   ENDDO I2LOOP
!$OMP END DO 
!
# ifdef LCFrad_sub_t01
   CALL timing_stop('lcfrad3 I2LOOP')
# endif
!
!=======================================================================
!
   IF (ntheta /= 0 .AND. i2max > 1) THEN
!
!=======================================================================
!
# ifdef LCFrad_sub_t01
   CALL timing_start('lcfrad3 I1LOOP')
# endif
!
!$OMP DO PRIVATE(i1) REDUCTION(+: HTOP,HBOT) SCHEDULE(RUNTIME)
   I1LOOP: DO i1 = 1, i1max
!
!--- Set up opacity and source function in planes parallel to y-direction ---
!
     IF (NFN == 1) THEN
       HDSFN(1:i2max,1:KRN) = blclam(i1+m1-1,m2:n2,n3:m3+jmin-1:-1) 
       HDKAP(1:i2max,1:KRN) = opclam(i1+m1-1,m2:n2,n3:m3+jmin-1:-1)
     ELSE
       HDSFN(1:i2max,1:KRN:NFN) = blclam(i1+m1-1,m2:n2,n3:m3+jmin-1:-1) 
       HDKAP(1:i2max,1:KRN:NFN) = opclam(i1+m1-1,m2:n2,n3:m3+jmin-1:-1)
       DO JJ=1,NFN-1
         K1 = NFN+1-JJ
         K2 = KRN-JJ
         HDSFN(1:i2max,K1:K2:NFN) = WN1(JJ)*HDSFN(1:i2max,1+NFN:KRN:NFN) + &
                                    WN2(JJ)*HDSFN(1:i2max,1:KRN-NFN:NFN)
! --- Exponential interpolation of kappa ---------------------------------
         HDKAP(1:i2max,K1:K2:NFN) = HDKAP (1:i2max,1+NFN:KRN:NFN)* &
                                   (HDKAP (1:i2max,1:KRN-NFN:NFN)/ &
                                    HDKAP (1:i2max,1+NFN:KRN:NFN))**WN2(JJ)
! ------------------------------------------------------------------------
       ENDDO
     ENDIF
!
!=======================================================================
!--- Radiative transfer along rays with inclination +/- theta ---
!=======================================================================
!
     DO NT=1,ntheta
!
!      --- Solve radiation transfer along oblique rays ---
       DIPNY  =  DIPN2(i1,1:i2max) * CTH(NT)
       HTAU0Y = HTAU02(i1,1:i2max) / CTH(NT)
!
       NFILL_NT = NFILL(NT)
       NSTEP_NT = NSTEP(NT)
       KRMAX_NT = KRMAX(NT)
       KZERO_NT = NFILL_NT * MOD(icount,(jmax-jmin+1)) + 1
       JK_NT    =    JK(1:KMAX,NT)
       DXR_NT   =   DXR(1:KMAX,NT)
       DSRAY_NT = DSRAY(1:KMAX,NT)
!=======================================================================
       CALL rsys2d(SPH, WPH, DIPNY, HTAU0Y, &
                   MFRT, NNRT, NFILL_NT, NSTEP_NT, KRMAX_NT, KZERO_NT, &
                   JK_NT, DSRAY_NT, DXR_NT, &
                   HDSFN, HDKAP, OMUNU, UMUNU, SMUNU, TMUNU, XMUNU, &
                   HDQRT, VRTOP, VRBOT, Y1AVG, Y2AVG, SOURC0, I0Y, i2max)
!=======================================================================
!
       y1avgt(nt) = y1avgt(nt) + Y1AVG
       y2avgt(nt) = y2avgt(nt) + Y2AVG
!
       WN4PI = PI40 * WTH(NT) * WXY
!
!      --- Multiply results with weighting factor WN4PI, add to DWORK1 ---
       IF (MRAD /= 0) THEN
         DWORK1(i1,1:i2max,jmin:jmax) = DWORK1(i1,1:i2max,jmin:jmax) + &
                                   WN4PI*HDQRT(   1:i2max,jmin:jmax)
       ENDIF
!  
!      --- Add contribution to flux at top and bottom boundaries ---
       HTOP = HTOP + WN4PI*CTH(NT)*A12*SUM(VRTOP(1:i2max))
       HBOT = HBOT + WN4PI*CTH(NT)*A12*SUM(VRBOT(1:i2max))
!
     ENDDO
!
   ENDDO I1LOOP
!$OMP END DO
!
# ifdef LCFrad_sub_t01
   CALL timing_stop('lcfrad3 I1LOOP')
# endif
!
   ENDIF
!
   IF (ntheta /= 0) THEN
!$OMP CRITICAL
       y1avg2(1:ntheta,nu) = y1avg2(1:ntheta,nu) + y1avgt(1:ntheta)
       y2avg2(1:ntheta,nu) = y2avg2(1:ntheta,nu) + y2avgt(1:ntheta)
!$OMP END CRITICAL
   ENDIF
!
!=======================================================================
!--- Radiative transfer along inclination angle +/- theta finished ---
!=======================================================================
!$OMP END PARALLEL
!
   CALL timing_stop('lcfrad3 raysystem RT')
!
!
! --- Return results, conversion to lower precision ---
!
!  --- Compute QRAD3 = - div(Frad)  or   4*Pi*J 
!  --- Compute DIVFR =  <div(Frad)> or -<4*Pi*J> ---
! 
!$OMP PARALLEL DEFAULT(NONE) &
!$OMP SHARED(A12, DIVFR, DWORK1, QRAD3, DTRAD3, TRAD3, &
!$OMP        i1max, i2max, jmax, m1, m2, m3, n1, n2)
!
   !$OMP DO PRIVATE(i3)
    DO i3=1,jmax
      DIVFR(i3) = DIVFR(i3)-A12*SUM(DWORK1(1:i1max,1:i2max,i3))
      QRAD3(m1:n1,m2:n2,m3-1+i3) = QRAD3(m1:n1,m2:n2,m3-1+i3) + &
                                   DWORK1(1:i1max,1:i2max,i3) 
      TRAD3(m1:n1,m2:n2,m3-1+i3) =  TRAD3(m1:n1,m2:n2,m3-1+i3) + &
                                   DTRAD3(m1:n1,m2:n2,m3-1+i3)
    ENDDO
   !$OMP END DO
!
!$OMP END PARALLEL
!
   ENDDO NULOOP
!
!***********************************************************************
!--- Frequency loop finished ---
!***********************************************************************
!
! --- Qrad3(jmin) must be corrected if (nthick > 0),  
!     combining the weighted contributions of both the diffusion RT 
!     and the ray system RT; DIVFR(jmin) remains unchanged ---
!
  IF (nthick > 0) THEN
     QRAD3(m1:n1,m2:n2,jmin) = W2FAC(jmin)*QRAD3(m1:n1,m2:n2,jmin) + &
                               W1FAC(jmin)*SUM(Qjmin,3)
  ENDIF

  IF (present(output_flag)) THEN
    IF (output_flag) THEN
    ! YSURF(:,:,0)     is the frequency-integrated emergent intensity
      YSURF(1:i1max,1:i2max,0)  =  SUM(YS1(1:i1max,1:i2max,nu1:nu2),3)
    ! YSURF(:,:,1:nnu) is the emergent intensity in frequency bins nu1:nu2
      YSURF(1:i1max,1:i2max,1:nnu) = YS1(1:i1max,1:i2max,nu1:nu2)
    ENDIF
  ENDIF
  !
  sum120 = REAL(i1max*i2max,k8)
  sum12n = FXY*sum120
  do nu=nu1,nu2
    y1avg2(0,nu)=SUM(ys1(1:i1max,1:i2max,nu))
    y2avg2(0,nu)=SUM((sum120*ys1(1:i1max,1:i2max,nu) - y1avg2(0,nu))* &
                     (sum120*ys1(1:i1max,1:i2max,nu) - y1avg2(0,nu)))
    y3avg(0,nu) = y1avg2(0,nu)/sum120
    y3rms(0,nu) = SQRT(y2avg2(0,nu)/sum120) / y1avg2(0,nu)
    IF (ntheta > 0) THEN
  ! Convert mean square intensities stored in y2avg2 into contrast
      do nt=1,ntheta
        y3avg(nt,nu) = y1avg2(nt,nu)/sum12n
        y3rms(nt,nu) = SQRT( MAX( F00, &
                       sum12n*y2avg2(nt,nu)-y1avg2(nt,nu)*y1avg2(nt,nu) ) ) / &
                       y1avg2(nt,nu)
      enddo
    ENDIF
  enddo
  y0avg(0) = SUM(y3avg(0,nu1:nu2))
  IF (ntheta > 0) THEN
    do nt=1,ntheta
      y0avg(nt) = SUM(y3avg(nt,nu1:nu2))
    enddo
  ENDIF
!
  FRTAU0 = PI20*SUM(WTH(0:ntheta)*CTH(0:ntheta)*y0avg(0:ntheta))/FREF
!
!=======================================================================
!--- Computation of model flux at top and bottom boundary ---
!=======================================================================
!
!  --- Starting value for integration of emergent flux is HBOT
!
!  --- Fluxes from ray system ---
   FRADZ1(jmin)   = HBOT
   FRADZ2(jmin)   = HBOT + F05*DIVFR(jmin)*DELTAZ(jmin  )
   FRADZ2(jmin-1) = HBOT - F05*DIVFR(jmin)*DELTAZ(jmin-1)
   DO J=jmin+1,jmax
     FRADZ1(J) = FRADZ1(J-1) + F05 * ( DIVFR(J-1) +  DIVFR(J) ) * DELTAZ(J-1)
     FRADZ2(J) = FRADZ2(J-1) + F05 * (DELTAZ(J-1) + DELTAZ(J) ) *  DIVFR(J) 
   ENDDO
!
!  --- Fluxes from diffusion approximation ---
!
   IF (nthick > 0) THEN
     DO J=jmin-1,0,-1
       FRADZ2(J) = FZDAVG(J)
     ENDDO
   ENDIF
   IF (nthick > 1) THEN
     DO J=jmin-1,1,-1
       FRADZ1(J) = FRADZ2(J) - F05 * DELTAZ(J) * DIVFR(J)
     ENDDO
   ENDIF
!
!  --- Flux at lower boundary ---
   FRBOT0 = HBOT / FREF
   FRBOT1 = FRADZ2(0) / FREF
   FRBOT2 = FRADZ2(1) / FREF
!
!  --- Flux at upper boundary computed with old method ---
   FRTOP0 = HTOP / FREF
!
!  --- Set flux at upper boundary computed from FRADZ1  ---
   FRTOP1 = FRADZ1(jmax) / FREF
!  --- Set flux at upper boundary computed from FRADZ2  ---
   FRTOP2 = FRADZ2(jmax) / FREF
!
   FRZ1(1:jmax) = FRADZ1(1:jmax)
   FRZ2(0:jmax) = FRADZ2(0:jmax)
!
!=======================================================================
!--- Computation of flux at boundaries completed ---
!=======================================================================
!
! Note: Rosseland mean opacities must always be computed and returned 
!       by lcfrad3 for further processing by rhd_rad_output
!
! --- Compute Rosseland mean opacities if not yet available ---
   IF (NU1 /= 0) THEN
     CALL timing_start('lcfrad3 opacity / sourcef')
!
   IF (ifdust == 1 .and. PRESENT(quc)) THEN
     CALL op_s0(T, Plog, rho, ifdust, m1,n1, m2,n2, m3,n3, nquc, opclam, &
                quc=quc)
   ELSE
     CALL op_s0(T, Plog, rho, ifdust, m1,n1, m2,n2, m3,n3, nquc, opclam)
   ENDIF
!
     CALL timing_stop('lcfrad3 opacity / sourcef')
   ENDIF
!
!--- Deallocate work arrays:
!      
   if(allocated(YS1))    deallocate(YS1)
   if(allocated(DWORK1)) deallocate(DWORK1)
   if(allocated(FZDIFF)) deallocate(FZDIFF)
   if(allocated(DTRAD3)) deallocate(DTRAD3)
!
!-------------------------------------------------------------------+-------
   END SUBROUTINE lcfrad3
!-----------------*******-------------------------------------------+-------

!-------------**********--------------------------------------------+-------
   SUBROUTINE lcfrad_ini(m1_, n1_, m2_, n2_, m3_, n3_, DELR_, Z_, DELTAZ_, &
                         GRAV_, HTAU0_, Tsurf_, Teff_, Tinci_, Dinci_, &
                         nthick_, ntheta_, nphi_, ksub_, &
                         nfifac_, mradbc_, mthint_)
!
!-------------------------------------------------------------------+-------
!
!    ==================================
!    Last Update:   2-Oct-1997 11:08:17
!    f90-Version,  MS (14-JUN-1999)
!    Modified:     MS (29-DEC-2000)
!    Modified:     MS (05-APR-2031) mmmax reduced by factor 2
!    ==================================
!
!    Subroutine lcfrad_ini defines constants and geometrical properties 
!    of the rays system. It is called once at the beginning of the
!    computation and performs all necessary preparations for subsequent 
!    calls of the radiation subroutine lcfrad3.
!
!-------------------------------------------------------------------+-------
!
   USE comrd1_def
   USE comrd2_def
   USE comrd3_def
   USE comrd4_def
   USE comrd5_def
!
!
   IMPLICIT NONE
!
!--- Dummy arguments:
!
   INTEGER, INTENT(IN)    :: m1_, n1_, m2_, n2_, m3_, n3_, mradbc_, mthint_
   INTEGER, INTENT(IN)    :: nfifac_, nthick_, ntheta_, nphi_, ksub_
   REAL,    INTENT(IN)    :: DELR_, GRAV_, HTAU0_, Tsurf_, Teff_, Tinci_, Dinci_
   REAL, DIMENSION(1:), INTENT(IN)  :: Z_
   REAL, DIMENSION(0:), INTENT(IN)  :: DELTAZ_
!
!--- Local arrays:
!      
   INTEGER,       DIMENSION(:), ALLOCATABLE :: JD
   REAL(KIND=k8), DIMENSION(nthmax,nthmax)  :: WTLBT, XNLBT
   REAL(KIND=k8), DIMENSION(nthmax,nthmax)  :: WTDBG, XNDBG
   REAL(KIND=k8), DIMENSION(nthmax,nthmax)  :: WTDBR, XNDBR
   REAL(KIND=k8), DIMENSION(6,6)            :: WTGSS, XNGSS
!
!--- Local scalar variables:
!
   INTEGER :: I, J, JJ, K, NP, NT, NF0, NFN
   REAL(KIND=k8) :: OF0, OFN, TTHOR, W04PI, W12, WTSUM, ZZ
!
   REAL(KIND=k8), PARAMETER :: F00=0.0D0, F05=0.5D0, F10=1.0D0, F20=2.0D0
!--- SPHI3D=0.0/1.0: 1.5D / 3D RT
   REAL(KIND=k8), PARAMETER :: SPHI3D=1.0D0 
!
!
!  --- Constants for LOBATTO quadrature ---
!
   DATA (WTLBT(NT,1), NT=1,1) / 0.83333333D0 /
   DATA (XNLBT(NT,1), NT=1,1) / 0.44721360D0 /
!
   DATA (WTLBT(NT,2), NT=1,2) / 0.37847496D0, 0.55485838D0 /
   DATA (XNLBT(NT,2), NT=1,2) / 0.76505532D0, 0.28523152D0 /
!
   DATA (WTLBT(NT,3), NT=1,3) / 0.21070422D0, 0.34112270D0, 0.41245880D0 /
   DATA (XNLBT(NT,3), NT=1,3) / 0.87174015D0, 0.59170018D0, 0.20929922D0 /
!
   DATA (WTLBT(NT,4), NT=1,4) / 0.1333059908D0, 0.2248893420D0, &
                                0.2920426836D0, 0.3275397612D0 /
   DATA (XNLBT(NT,4), NT=1,4) / 0.9195339082D0, 0.7387738651D0, &
                                0.4779249498D0, 0.1652789577D0 /
!
   DATA (WTLBT(NT,5), NT=1,5) / 0.091684517440D0, 0.157974705655D0, &
                                0.212508417834D0, 0.251275603210D0, &
                                0.271405240910D0 /
   DATA (XNLBT(NT,5), NT=1,5) / 0.944899272223D0, 0.819279321644D0, &
                                0.632876153032D0, 0.399530940965D0, &
                                0.136552932855D0 /
!
!  --- Constants for Double Gauss quadrature ---
!
   DATA (WTDBG(NT,1), NT=1,1) / 1.00000000D0 /
   DATA (XNDBG(NT,1), NT=1,1) / 0.50000000D0 /
!
   DATA (WTDBG(NT,2), NT=1,2) / 0.50000000D0, 0.50000000D0 /
   DATA (XNDBG(NT,2), NT=1,2) / 0.78867513D0, 0.21132487D0 /
!
   DATA (WTDBG(NT,3), NT=1,3) / 0.27777778D0, 0.44444444D0, 0.27777778D0 /
   DATA (XNDBG(NT,3), NT=1,3) / 0.88729833D0, 0.50000000D0, 0.11270167D0 /
!
   DATA (WTDBG(NT,4), NT=1,4) / 0.1739274226D0, 0.3260725774D0, &
                                0.3260725774D0, 0.1739274226D0 /
   DATA (XNDBG(NT,4), NT=1,4) / 0.9305681558D0, 0.6699905218D0, &
                                0.3300094782D0, 0.0694318442D0 /
!
   DATA (WTDBG(NT,5), NT=1,5) / 0.1184634425D0, 0.2393143352D0, &
                                0.2844444444D0, 0.2393143352D0, &
                                0.1184634425D0 /
   DATA (XNDBG(NT,5), NT=1,5) / 0.9530899230D0, 0.7692346551D0, &
                                0.5000000000D0, 0.2307653449D0, &
                                0.0469100770D0 /
!
!  --- Constants for Double Gauss-Radau quadrature ---
!
   DATA (WTDBR(NT,1), NT=1,1) / 0.7500000000D0 /
   DATA (XNDBR(NT,1), NT=1,1) / 0.3333333333D0 /
!
   DATA (WTDBR(NT,2), NT=1,2) / 0.5124858262D0, 0.3764030627D0 /
   DATA (XNDBR(NT,2), NT=1,2) / 0.6449489743D0, 0.1550510257D0 /
!
   DATA (WTDBR(NT,3), NT=1,3) / 0.3288443200D0, 0.3881934688D0, 0.2204622112D0 /
   DATA (XNDBR(NT,3), NT=1,3) / 0.7876594618D0, 0.4094668644D0, 0.0885879595D0 /
!
   DATA (WTDBR(NT,4), NT=1,4) / 0.2231039011D0, 0.3118265230D0, &
                                0.2813560151D0, 0.1437135608D0 /
   DATA (XNDBR(NT,4), NT=1,4) / 0.8602401357D0, 0.5835904324D0, &
                                0.2768430136D0, 0.0571041961D0 /
!
   DATA (WTDBR(NT,5), NT=1,5) / 0.1598203766D0, 0.2426935942D0, &
                                0.2604633916D0, 0.2084506673D0, &
                                0.1007941926D0 /
   DATA (XNDBR(NT,5), NT=1,5) / 0.9014649142D0, 0.6954642734D0, &
                                0.4379748102D0, 0.1980134179D0, &
                                0.0398098571D0/
!
!  --- Constants for GAUSS quadrature ---
!
   DATA (WTGSS(NP,1), NP=1,1) / 1.000000000000000D0 /
   DATA (XNGSS(NP,1), NP=1,1) / 0.577350269189626D0 /
!
   DATA (WTGSS(NP,2), NP=1,2) / 0.652145154862546D0, &
                                0.347854845137454D0 /
   DATA (XNGSS(NP,2), NP=1,2) / 0.339981043584856D0, &
                                0.861136311594053D0 /
!
   DATA (WTGSS(NP,3), NP=1,3) / 0.467913934572691D0, &
                                0.360761573048139D0, &
                                0.171324492379170D0 /
   DATA (XNGSS(NP,3), NP=1,3) / 0.238619186083197D0, &
                                0.661209386466265D0, &
                                0.932469514203152D0 /
!
   DATA (WTGSS(NP,4), NP=1,4) / 0.362683783378362D0, &
                                0.313706645877887D0, &
                                0.222381034453374D0, &
                                0.101228536290376D0 /
   DATA (XNGSS(NP,4), NP=1,4) / 0.183434642495650D0, &
                                0.525532409916329D0, &
                                0.796666477413627D0, &
                                0.960289856497536D0 /
   DATA (WTGSS(NP,5), NP=1,5) / 0.295524224714753D0, &
                                0.269266719309996D0, &
                                0.219086362515982D0, &
                                0.149451349150581D0, &
                                0.066671344308688D0 /
   DATA (XNGSS(NP,5), NP=1,5) / 0.148874338981631D0, &
                                0.433395394129247D0, &
                                0.679409568299024D0, &
                                0.865063366688985D0, &
                                0.973906528517172D0 /
   DATA (WTGSS(NP,6), NP=1,6) / 0.249147045813403D0, &
                                0.233492536538355D0, &
                                0.203167426723066D0, &
                                0.160078328543346D0, &
                                0.106939325995318D0, &
                                0.047175336386512D0 /
   DATA (XNGSS(NP,6), NP=1,6) / 0.125233408511469D0, &
                                0.367831498998180D0, &
                                0.587317954286617D0, &
                                0.769902674194305D0, &
                                0.904117256370475D0, &
                                0.981560634246719D0 /
!-------------------------------------------------------------------+-------
!
!
   i1max  = n1_ - m1_ + 1
   i2max  = n2_ - m2_ + 1
   iimax  = MAX(i1max,i2max)
   jmax   = n3_ - m3_  + 1
   nthick = nthick_
!  --- 0 <= nthick <= jmax-nthin ---
   IF (nthick < 0         ) nthick = 0
   IF (nthick > jmax-nthin) nthick = jmax-nthin
   jmin   = MAX(1, nthick)
!
! --- Weights for computing azimuthal averages ---
   IF (i2max > 1) THEN
     FXY=F20
     WXY=F05
   ELSE
     FXY=F10
     WXY=F10
   ENDIF
!
!  --- Reduce  ksub  if greater than ksbmax ---
!  --- Reduce  nphi  if greater than nphmax ---
!  --- Reduce ntheta if greater than nthmax ---

   ksub   = ksub_
   IF (ksub > ksbmax) ksub = ksbmax
   nphi   = nphi_
   IF (nphi > nphmax) nphi = nphmax
! In case of 3D geometry nphi must be -1
   IF (i2max > 1) nphi = -1
! --- enforce 0 <= ntheta <= nthmax ---
   ntheta = MIN(ABS(ntheta_),nthmax)
   nfifac = nfifac_
!
   mmmax  = iimax*ksub
!
   ALLOCATE( DELTAZ(0:jmax) )
   ALLOCATE( AFAC(jmax), DELZ(jmax), GFAC(jmax), WTFAC(jmax), Z(jmax) )
   ALLOCATE( W1FAC(jmax), W2FAC(jmax) )
   ALLOCATE( I0X(2*i1max+2), I0Y(2*i2max+2) )  
   ALLOCATE( I1X(0:i1max+1), I1Y(0:i2max+1) )  
   ALLOCATE( DCZ(1:jmin) )
   ALLOCATE( WJ1(maxomu, jmin:jmax), WJ2(maxomu, jmin:jmax) )
!
   m1     = m1_
   n1     = n1_
   m2     = m2_
   n2     = n2_
   m3     = m3_
   n3     = n3_
   DELR   = DELR_
   Z      = Z_
   DELTAZ = DELTAZ_
!
   GRAV   = GRAV_
   HTAU0  = HTAU0_
   Tsurf  = Tsurf_
   Teff   = Teff_
   Tinci = Tinci_
   Dinci = Dinci_
!
   FREF   = SIGMA*Teff*Teff*Teff*Teff
!
   mradbc = mradbc_
   mthint = mthint_
!
   IF (nthick > 0) THEN
!-- Normal upper boundary, 
!   flux through lower boundary from diffusion approximation
     FARAD =  0.0D0
     GNRAD =  1.0D0
     HMRAD =  0.0D0
     HNRAD =  0.5D0
     DIPF  =  1.5D0 / PI
   ELSEIF (mradbc == 1) THEN
!-- Normal upper boundary, enforce total flux through lower boundary
     FARAD =  0.0D0
     GNRAD =  1.0D0
     HMRAD =  0.0D0
     HNRAD =  0.5D0
     DIPF  =  1.5D0 * FREF / PI
   ELSE
!-- Normal upper boundary, "neutral" lower boundary (J=B)
     FARAD =  0.0D0
     GNRAD =  0.0D0
     HMRAD =  0.0D0
     HNRAD =  1.0D0
     DIPF  =  0.0D0
   ENDIF
!
   A12 = 1.0D0/REAL(i1max*i2max,k8)
!
   I0X(      1:  i1max) = (/ (I,I=1,i1max) /)
   I0X(i1max+1:2*i1max) = (/ (I,I=1,i1max) /)
   I0X(2*i1max+1)       = I0X(1)
   I0X(2*i1max+2)       = I0X(2)
!
   I0Y(      1:  i2max) = (/ (I,I=1,i2max) /)
   I0Y(i2max+1:2*i2max) = (/ (I,I=1,i2max) /)
   I0Y(2*i2max+1)       = I0Y(1)
   I0Y(2*i2max+2)       = I0Y(2)
!
   I1X(      1:  i1max) = (/ (I,I=1,i1max) /)
   I1X(0)               =  i1max
   I1X(i1max+1)         =  1
!
   I1Y(      1:  i2max) = (/ (I,I=1,i2max) /)
   I1Y(0)               =  i2max
   I1Y(i2max+1)         =  1
!
! --- Factors needed for the computation of diffusive fluxes ---
!
   DCZ(1:jmin) = C8/DELTAZ(1:jmin)
   DCX         = C8/(DELR*DELR)
   DCY         = C8/(DELR*DELR)
!
! --- Weighting factors for vertical averaging to be used by ---
! --- lcfrad3 and rsys2d if filling factor (NFILL) > 1          ---
   WTFAC(1)        = F10 /  DELTAZ(1)
   WTFAC(2:jmax-1) = F20 / (DELTAZ(1:jmax-2) + DELTAZ(2:jmax-1))
   WTFAC(jmax)     = F10 /  DELTAZ(jmax-1)
!
! --- Weighting factors for linear vertical interpolation used by ---
! --- lcfrad3 in case (NTHICK) > 0          ---
   W1FAC= DELTAZ(0:jmax-1)/(DELTAZ(0:jmax-1)+DELTAZ(1:jmax))
   W2FAC= DELTAZ(1:jmax  )/(DELTAZ(0:jmax-1)+DELTAZ(1:jmax))
!
! --- Geometrical factor for radiative time scale estimation ---
   DELZ = 0.5D0*(DELTAZ(0:jmax-1)+DELTAZ(1:jmax))
   AFAC = (DELR/DELZ)*(DELR/DELZ)
   GFAC = 1.5D0*AFAC/(2.0D0+AFAC)*DELZ*DELZ
!
!***********************************************************************
!--- Computation geometric properties of ray system ---
!***********************************************************************
!  --- Set up constants for Gaussian quadrature in PHI-direction
!
   IF     (nphi == -1) THEN
     nphi = 1
     WPH(1) = F10
     SPH(1) = SPHI3D
   ELSEIF (nphi == -2) THEN
     nphi = 2
     WPH(1) = F05
     WPH(2) = F05
     SPH(1) = F00
     SPH(2) = F10
   ELSE
     WPH(1:nphi) =          WTGSS(1:nphi,nphi)
     SPH(1:nphi) = SIN(PI05*XNGSS(1:nphi,nphi))
   ENDIF
!
!  --- Set up constants used for Lobatto Integration ---
!
   ZZ    = Z(jmax)
   WTSUM = F00
!     --- Constants for oblique rays ---
   DO NT=0,ntheta
     IF (NT == 0) THEN
       CTH(0) = F10
       WTH(0) = F00
     ELSEIF (NT == 1 .AND. ntheta_ == -1) THEN
!      Special case: Aake's 2 ray  quadrature:
       WTH(NT) = 0.75D0
       CTH(NT) = 1.D0/3.D0
     ELSEIF (mthint == 1) THEN
       WTH(NT) = WTLBT(NT,ntheta)
       CTH(NT) = XNLBT(NT,ntheta)
     ELSEIF (mthint == 2) THEN
       WTH(NT) = WTDBG(NT,ntheta)
       CTH(NT) = XNDBG(NT,ntheta)
     ELSEIF (mthint == 3) THEN
       WTH(NT) = WTDBR(NT,ntheta)
       CTH(NT) = XNDBR(NT,ntheta)
     ELSE
       WRITE(6,*) "lcfrad_ini Error: illegal value of mthint:", mthint
       STOP
     ENDIF
     WTSUM   = WTSUM + WTH(NT)
!
!    --- Set ray filling factors ---
     IF (nfifac > 0) THEN
       NFILL(NT) = MIN0 (maxomu, nfifac)
     ELSE
       NFILL(NT) = MIN0 (maxomu, INT( F10 / CTH(NT) ))
     ENDIF
   ENDDO
!
   DO NT=0,ntheta
!    --- NFILL(ntheta) must be NFILL(NT)*NSTEP(NT), NSTEP(NT) integer ---
     NSTEP(NT) = NFILL(ntheta)/NFILL(NT)
     NFILL(NT) = NFILL(ntheta)/NSTEP(NT)
!    --- Number of grid points along ray system theta ---
     KRMAX(NT) = (jmax-jmin) * NFILL(NT) + 1
   ENDDO
!
!     --- Weight for vertical ray ---
!     --- Set weight to 2/5 if only vertical rays are used ---
!     --- Otherwise the sum of all weights must be 1
   IF (ntheta == 0) THEN
      WTH(0) = 4.0D-01
   ELSE
      WTH(0) = F10 - WTSUM
   ENDIF
!
   kmax   = MAXVAL(KRMAX(0:ntheta))
   ALLOCATE( DSRAY(kmax,0:nthmax), DXR(kmax,0:nthmax) )
   ALLOCATE( JK(   kmax,0:nthmax) )
   ALLOCATE( JD(   kmax) )
!
! --- Initialize arrays DSRAY, DXR, JK ---
!
   DSRAY(:,:) = 0.0D0
   DXR(  :,:) = 0.0D0
   JK(   :,:) = 0
!
   DO NT=0,ntheta
!    --- Compute geometrical distances along ray system theta ---
     DO K=2,KRMAX(NT)
       JJ = KRMAX(NT) - K + 1
       J  = (JJ-1) / NFILL(NT) + jmin
       DSRAY(K,NT) = DELTAZ(J) / ( CTH(NT) * REAL(NFILL(NT),k8) )
     ENDDO
!
!  --- Inverse grid point filling factor ---
     OFN = F10 / REAL(NFILL(NT),k8)
!
!  --- Geometrical constants ---
     IF (NT == 0) THEN
       TTHOR = F00
     ELSE
       TTHOR = SQRT (F10 - CTH(NT)*CTH(NT))/CTH(NT)
     ENDIF
!
     JD (1:KRMAX(NT))    = (/ (KRMAX(NT) - K, K=1,KRMAX(NT)) /)
     JK (1:KRMAX(NT),NT) = JD(1:KRMAX(NT))/NFILL(NT) + jmin
     JD (1:KRMAX(NT))    = MOD(JD(1:KRMAX(NT)),NFILL(NT))
     DXR(1:KRMAX(NT),NT) = TTHOR*(ZZ - Z(JK(1:KRMAX(NT),NT)) - &
               DELTAZ(JK(1:KRMAX(NT),NT))*OFN*REAL(JD(1:KRMAX(NT)),k8))
!
   ENDDO
!
   NF0   = NFILL(0)
   NFN   = NFILL(ntheta)
   OF0   = F10 / REAL(NF0,k8)
   OFN   = F10 / REAL(NFN,k8)
   W04PI = PI40 * WTH(0)
!
   IF (NF0 > 1) THEN
     DO JJ=1,NF0/2
       W12 = OF0
       IF (2*JJ == NF0) W12 = F05*W12
       WJ1(JJ,jmin) = W12 * F20 * W04PI
       WJ2(JJ,jmax) = W12 * F20 * W04PI
       DO J=jmin+1,jmax-1
         WJ1(JJ,J) = W12 * DELTAZ(J)   * WTFAC(J) * W04PI
         WJ2(JJ,J) = W12 * DELTAZ(J-1) * WTFAC(J) * W04PI
       ENDDO
     ENDDO
   ENDIF
!
   IF (NFN > 1) THEN
     DO JJ=1,NFN-1
       WN2(JJ) = OFN*REAL(JJ,k8)
       WN1(JJ) = F10 - WN2(JJ)
     ENDDO
   ENDIF
!
   IF (i2max == 1) THEN
      WRITE (6, '(//, " USING i1max + ntheta x 2*nphi x i1max x ksub")')
      WRITE (6, '(8X,I4,6X,I3,6X,I3,4X,I4,4X,I3," =",I7," RAYS"/)') &
          i1max, ntheta, 2*nphi, i1max, ksub, i1max+ntheta*2*nphi*i1max*ksub
   ELSE
      WRITE (6, '(//, " USING i1max*i2max + i2max x ntheta x 2*i1max x ksub")')
      WRITE (6, '(    "                   + i1max x ntheta x 2*i2max x ksub")')
      WRITE (6, '(10X, I8, 4X, I4, 6X, I3, 5X, I5, 4X, I3)') &
          i1max*i2max, i2max, ntheta, 2*i1max, ksub
      WRITE (6, '(22X, I4, 6X, I3, 5X, I5, 4X, I3, " =", I9, " RAYS"/)') &
                       i1max, ntheta, 2*i2max, ksub, &
          i1max*i2max + i2max*ntheta*2*i1max*ksub + i1max*ntheta*2*i2max*ksub
   ENDIF
!
   IF (nthick == 0) THEN
     WRITE (6, '(" Ray system RT from level 1 to ", I3)') jmax
     WRITE (6, '(" Diffusion approximation: nowhere"//)')
   ELSE
     WRITE (6, '(" Ray system RT           from level ", I3, " to ", I3)') jmin, jmax
     WRITE (6, '(" Diffusion approximation from level   1 to ", I3//)') jmin
   ENDIF
!
     WRITE (6, '(" Ray system information:")')
     WRITE (6, '(" Ray system    mu      weight  nfill  nstep")')
     DO NT=0,ntheta
       WRITE (6, '(5X, I1, 5X, F8.4, 2X, F8.4, 4X, I1, 6X, I1)') &
       NT, CTH(NT), WTH(NT), NFILL(NT), NSTEP(NT)
     ENDDO
!
   DEALLOCATE (JD )
!   
!=======================================================================
!--- Computation of rectangle sizes finished ---
!=======================================================================
!
   END SUBROUTINE lcfrad_ini
!-----------------**********----------------------------------------+-------

!-------------**********--------------------------------------------+-------
   SUBROUTINE lcfrad_cls
!-------------------------------------------------------------------+-------
!
!    ==================================
!    f90-Version,  MS (14-JUN-1999)
!    ==================================
!
!    Subroutine lcfrad_cls deallocates the arrays allocated by lcfrad_ini
!-------------------------------------------------------------------+-------
!
   USE comrd2_def
   USE comrd4_def
!
   IMPLICIT NONE
!
   DEALLOCATE( DELTAZ, WTFAC, Z )
   DEALLOCATE( W1FAC, W2FAC )
   DEALLOCATE( AFAC, DELZ, GFAC )
   DEALLOCATE( DSRAY, DXR )
   DEALLOCATE( JK )
   DEALLOCATE( I0X, I0Y )  
   DEALLOCATE( I1X, I1Y )  
   DEALLOCATE( DCZ )  
   DEALLOCATE( WJ1, WJ2 )  
!
   END SUBROUTINE lcfrad_cls
!-----------------**********----------------------------------------+-------

!-------------*******-----------------------------------------------+-------
   SUBROUTINE angles1(NTHX, NPHX)
!  --- Provides information about number of angles used for ray system
!
   USE comrd3_def
!
   IMPLICIT NONE
!--- Dummy arguments:
   INTEGER,             INTENT(OUT) :: NTHX, NPHX
!
   NTHX = ntheta
   NPHX = nphi
!
   END SUBROUTINE angles1
!-----------------*******-------------------------------------------+-------

!-------------*******-----------------------------------------------+-------
   SUBROUTINE angles2(CTHX, WTHX, SPHX, WPHX)
!  --- Provides information about angles and weights used for ray system
!
   USE comrd2_def
   USE comrd3_def
!
   IMPLICIT NONE
!--- Dummy arguments:
   REAL, DIMENSION(0:), INTENT(OUT) :: CTHX, WTHX
   REAL, DIMENSION(1:), INTENT(OUT) :: SPHX, WPHX
!
   CTHX(0:ntheta) = CTH(0:ntheta)
   WTHX(0:ntheta) = WTH(0:ntheta)
   SPHX(1:nphi)   = SPH(1:nphi)
   WPHX(1:nphi)   = WPH(1:nphi)
!
   END SUBROUTINE angles2
!-----------------*******-------------------------------------------+-------

!-------------******------------------------------------------------+-------
   SUBROUTINE lstar1(DTAUMK, OPLSMK, mmax, mstop, kmax, kstop, Gamma)
!  --- computes the lambda_star operator (OPLSMK) for given delta tau (DTAUMK)
!  --- first  index m=1,mstop (horizontal direction)
!  --- second index k=1,kstop (increasing optical depth)
!  --- see Heft 30, 260, lambda_star4
!
   USE comrd0_def
!
   IMPLICIT NONE
!--- Dummy arguments:
   INTEGER,                                 INTENT(IN)  :: mmax, mstop, &
                                                           kmax, kstop
   REAL(KIND=k8),                           INTENT(IN)  :: Gamma
   REAL(KIND=k8), DIMENSION(1:mmax,1:kmax), INTENT(IN)  :: DTAUMK
   REAL(KIND=k8), DIMENSION(1:mmax,1:kmax), INTENT(OUT) :: OPLSMK
!
!--- Local scalar variables:
!      
   INTEGER                            :: K  
   REAL(KIND=k8)                      :: SG1
!
!--- Local arrays:
!      
   REAL(KIND=k8), DIMENSION(mstop)    :: dtau1,ftau1,atau1
   REAL(KIND=k8), DIMENSION(mstop)    :: dtau2,ftau2,atau2
!
     SG1 = 1.0D0/Gamma    ! empirical adjustment parameter
!
!--- Compute lambda_star on ray system:
!
     DO K=2,kstop-1
       dtau1(1:mstop) = SG1*DTAUMK(1:mstop,K)
       dtau2(1:mstop) = SG1*DTAUMK(1:mstop,K+1)
       atau1(1:mstop) = 1.0D0 + 0.5D0*dtau1(1:mstop)
       atau2(1:mstop) = 1.0D0 + 0.5D0*dtau2(1:mstop)
       ftau1(1:mstop) = 1.0D0 + dtau1(1:mstop)*atau1(1:mstop)
       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
       OPLSMK(1:mstop,K) = 1.0D0/ &
                          (1.0D0 + 2.0D0*(atau1(1:mstop)/ftau1(1:mstop) + &
                                          atau2(1:mstop)/ftau2(1:mstop))/ &
                                         (dtau1(1:mstop)+dtau2(1:mstop)))
     ENDDO
!
! --- top boundary: lambda*_4 with dtau1 = dtau2 ---
!     K = 1
!       dtau2(1:mstop) = SG1*DTAUMK(1:mstop,K+1)
!       atau2(1:mstop) = 1.0D0 + 0.5D0*dtau2(1:mstop)
!       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
!       OPLSMK(1:mstop,K) = 1.0D0/ &
!                          (1.0D0 + 2.0D0*atau2(1:mstop)/ftau2(1:mstop)/ &
!                                                        dtau2(1:mstop))
!
! --- top boundary: lambda*_6a = lambda*_6b with dtau1 = dtau2 ---
     K = 1
       dtau2(1:mstop) = SG1*DTAUMK(1:mstop,K+1)
       atau2(1:mstop) = 1.0D0 + 0.5D0*dtau2(1:mstop)
       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
       OPLSMK(1:mstop,K) = (ftau2(1:mstop) - 1.0D0) / &
                           (ftau2(1:mstop) + 1.0D0)
!
! --- top boundary: lambda*_6b ---
!     K = 1
!       dtau1(1:mstop) = SG1*DTAUMK(1:mstop,K)
!       atau1(1:mstop) = 1.0D0 + 0.5D0*dtau1(1:mstop)
!       dtau2(1:mstop) = SG1*DTAUMK(1:mstop,K+1)
!       atau2(1:mstop) = 1.0D0 + 0.5D0*dtau2(1:mstop)
!       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
!       OPLSMK(1:mstop,K) = (ftau2(1:mstop) - ftau2(1:mstop)/ftau1(1:mstop)  * &
!                                             atau1(1:mstop)/atau2(1:mstop)) / &
!                           (1.0D0 + ftau2(1:mstop))
!
! --- bottom boundary: lambda*_4 with dtau1 = dtau2 ---
     K = kstop
       dtau1(1:mstop) = SG1*DTAUMK(1:mstop,K)
       atau1(1:mstop) = 1.0D0 + 0.5D0*dtau1(1:mstop)
       ftau1(1:mstop) = 1.0D0 + dtau1(1:mstop)*atau1(1:mstop)
       OPLSMK(1:mstop,K) = 1.0D0/ &
                          (1.0D0 + 2.0D0*atau1(1:mstop)/ftau1(1:mstop)/ &
                                                        dtau1(1:mstop))
!
   END SUBROUTINE lstar1
!-----------------******--------------------------------------------+-------

!-------------******------------------------------------------------+-------
   SUBROUTINE lstar2(DSRAY, OPC, OPLSMK, mmax, mstop, kmax, kstop, Gamma)
!  --- computes the lambda_star operator (OPLSMK) for given geometrical 
!      depth along the ray system (DSRAY) and opacity per unit length (OPC) --
!  --- first  index m=1,mstop (horizontal direction)
!  --- second index k=1,kstop (increasing geometrical depth)
!  --- see *feautrier_gs.pdf* by M. Steffen (2016) for details
!
   USE comrd0_def
!
   IMPLICIT NONE
!--- Dummy arguments:
   INTEGER,                                 INTENT(IN)  :: mmax, mstop, &
                                                           kmax, kstop
   REAL(KIND=k8),                           INTENT(IN)  :: Gamma
   REAL(KIND=k8), DIMENSION(       1:kmax), INTENT(IN)  :: DSRAY
   REAL(KIND=k8), DIMENSION(1:mmax,1:kmax), INTENT(IN)  :: OPC
   REAL(KIND=k8), DIMENSION(1:mmax,1:kmax), INTENT(OUT) :: OPLSMK
!
!--- Local scalar variables:
!      
   INTEGER                            :: K  
   REAL(KIND=k8)                      :: SG1, SG2
!
!--- Local arrays:
!      
   REAL(KIND=k8), DIMENSION(mstop)    :: dtauc
   REAL(KIND=k8), DIMENSION(mstop)    :: dtaum,dtau1,ftau1,atau1
   REAL(KIND=k8), DIMENSION(mstop)    :: dtaup,dtau2,ftau2,atau2
!
     SG1 = 1.0D0/Gamma    ! empirical adjustment parameter
     SG2 = 0.5D0/Gamma    ! empirical adjustment parameter/2
!
!--- Compute lambda_star on ray system:
!
     DO K=2,kstop-1
       dtaum(1:mstop) = SG1*DSRAY(K  )*OPC(1:mstop,K-1)
       dtaup(1:mstop) = SG1*DSRAY(K+1)*OPC(1:mstop,K+1)
       dtau1(1:mstop) = SG2*DSRAY(k  )*(OPC(1:mstop,K)+OPC(1:mstop,K-1))
       dtau2(1:mstop) = SG2*DSRAY(k+1)*(OPC(1:mstop,K)+OPC(1:mstop,K+1))
       dtauc(1:mstop) = SG2*(DSRAY(K)+DSRAY(K+1)) * OPC(1:mstop,K  )
       atau1(1:mstop) = 1.0D0 + 0.5D0*dtaum(1:mstop)
       atau2(1:mstop) = 1.0D0 + 0.5D0*dtaup(1:mstop)
       ftau1(1:mstop) = 1.0D0 + dtau1(1:mstop)*atau1(1:mstop)
       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
       OPLSMK(1:mstop,K) = 1.0D0/ &
                         ( 1.0D0 + (atau1(1:mstop)/ftau1(1:mstop) + &
                                    atau2(1:mstop)/ftau2(1:mstop))/ &
                                                   dtauc(1:mstop) )
     ENDDO
!
! --- top boundary: interior lambda* with dtau1 = dtau2 ---
     K = 1
       dtau2(1:mstop) = SG1*DSRAY(K+1)*OPC(1:mstop,K+1)
       atau2(1:mstop) = 1.0D0 + 0.5D0*dtau2(1:mstop)
       ftau2(1:mstop) = 1.0D0 + dtau2(1:mstop)*atau2(1:mstop)
       OPLSMK(1:mstop,K) = 1.0D0/ &
                         ( 1.0D0 + 2.0D0*atau2(1:mstop)/ftau2(1:mstop)/ &
                                                        dtau2(1:mstop) )
!
! --- bottom boundary: interior lambda*_ with dtau1 = dtau2 ---
     K = kstop
       dtau1(1:mstop) = SG1*DSRAY(K)*OPC(1:mstop,K)
       atau1(1:mstop) = 1.0D0 + 0.5D0*dtau1(1:mstop)
       ftau1(1:mstop) = 1.0D0 + dtau1(1:mstop)*atau1(1:mstop)
       OPLSMK(1:mstop,K) = 1.0D0/ &
                         ( 1.0D0 + 2.0D0*atau1(1:mstop)/ftau1(1:mstop)/ &
                                                        dtau1(1:mstop))
!
   END SUBROUTINE lstar2
!-----------------******--------------------------------------------+-------
!
   END MODULE lcfrad_module
!-------------*************-----------------------------------------+-------
