C------------*******---------------------------------------------------+-------
      MODULE dim_def
C
      IMPLICIT NONE
      SAVE
C
C     --- Define some dimensions ---
      INTEGER, PARAMETER :: NDIM=50, NDIM2=2*NDIM, NDIM4=4*NDIM
C
      END MODULE dim_def
C----------------*******-----------------------------------------------+-------

C------------***********-----------------------------------------------+-------
      MODULE iondis1_com
C
      USE dim_def
C
      IMPLICIT NONE
      SAVE
C
      INTEGER                  :: MAI
      INTEGER, DIMENSION(NDIM) :: NAMI
      REAL,    DIMENSION(NDIM) :: ABUI, FRACI, WTI
C
      END MODULE iondis1_com
C----------------***********-------------------------------------------+-------

C------------***********-----------------------------------------------+-------
      MODULE iondis2_com
C
      USE dim_def
C
      IMPLICIT NONE
      SAVE
C
      INTEGER, DIMENSION(NDIM4) :: NAMJ
      REAL,    DIMENSION(NDIM4) :: WTJ, CHIJ, FRACJ, ZETAJ
      REAL,    DIMENSION(NDIM2) :: JIZ
C
      END MODULE iondis2_com
C----------------***********-------------------------------------------+-------

C------------***********-----------------------------------------------+-------
      MODULE iondis3_com
C
      USE dim_def
C
      IMPLICIT NONE
      SAVE
C
      REAL, DIMENSION(NDIM4) :: PARTFJ, SAHAJ
C
      END MODULE iondis3_com
C----------------***********-------------------------------------------+-------

C------------***********-----------------------------------------------+-------
      MODULE iondis4_com
C
      IMPLICIT NONE
      SAVE
C
      INTEGER :: JATI, JNEGI, JMOL2
C
      END MODULE iondis4_com
C----------------***********-------------------------------------------+-------

C------------*******---------------------------------------------------+-------
      MODULE op1_com
C
      IMPLICIT NONE
      SAVE
C
      REAL :: ATHOMS, HNU, AKT, RYD, BLAM
C
      END MODULE op1_com
C----------------*******-----------------------------------------------+-------

C------------*******---------------------------------------------------+-------
      MODULE op2_com
C
      IMPLICIT NONE
      SAVE
C
      INTEGER  KSCAT, KABS
      INTEGER, DIMENSION(40) :: NAMK
      REAL,    DIMENSION(40) :: OPAC
C
      END MODULE op2_com
C----------------*******-----------------------------------------------+-------

C------------*********-------------------------------------------------+-------
      MODULE constants
C
      IMPLICIT NONE
      SAVE
C
C     --- Define some constants ---
      REAL, PARAMETER :: BOLTZK=1.38062E-16, HPLANC=6.62620E-27,
     &                   CLIGHT=2.997925E+10, ALN10=2.302585092994046
C
      END MODULE constants
C----------------*********---------------------------------------------+-------

C------------********************--------------------------------------+-------
      MODULE iondis_opalam_module

      CONTAINS

C----------------***********-------------------------------------------+-------
      SUBROUTINE iondis_init(ABUFILE)
C
C --- M. Steffen, July 1997
C
      USE dim_def
      USE iondis1_com
      USE iondis2_com
      USE constants
C
      IMPLICIT NONE
C
C --- Dummy variables:
      CHARACTER(LEN=*), INTENT(IN) :: ABUFILE
C
C --- Local variables:
      INTEGER :: I
      REAL    :: ASUM, A, A1, AVEMAT
C
C     READ INPUT COMPOSITION
C
      OPEN(UNIT=1, FILE=ABUFILE, STATUS='OLD', ACTION='READ')
         READ(1,"(I7)") MAI
         ASUM=0.
      DO I=1,MAI
         READ(1, "(I7, F9.4, F7.3)") NAMI(I),WTI(I),A
         ABUI(I)=A
         A1=EXP(ALN10*A)
         FRACI(I)=A1
         ASUM=ASUM+A1
      ENDDO
C
      CLOSE(UNIT=1)
C
      FRACI(1:MAI) = FRACI(1:MAI)/ASUM
      AVEMAT       = 1.660531E-24*SUM(WTI(1:MAI)*FRACI(1:MAI))
C
C--- FORMATTING AND PRINTING
C
  100 FORMAT(' INPUT COMPOSITION (', I3, ' ELEMENTS):' //
     >       ' ELEMENT    WTI    ABUI   FRACI' /)
C
      WRITE(*,"(A)") ' --- IONDIS_INIT KIEL 1997 ---'
      WRITE(*,100) MAI
      DO I=1,MAI
         WRITE(*,"(I7,F9.4,F7.3,ES10.3)")
     &        NAMI(I),WTI(I),ABUI(I),FRACI(I)
      ENDDO
      WRITE(*,"(/1X,'  AVEMAT=',ES12.5)")AVEMAT
C
      END SUBROUTINE iondis_init
C--------------------***********---------------------------------------+-------

C----------------******------------------------------------------------+-------
      SUBROUTINE P_COMP(pgas,tgas,pe00,pele,dnc,slope0,dlpgpe,itnum)
C
C --- COMPUTES PELE AND DENSNC FOR GIVEN T AND PGAS
C --- IN ADDITION (D LOG PG / D LOG PE)|T IS DETERMINED
C
C --- M. Steffen, July 1997
C
      USE dim_def
      USE iondis1_com
      USE iondis2_com
      USE constants
C
      IMPLICIT NONE
C
C--- Dummy variables:
      INTEGER, INTENT(OUT) :: itnum
      REAL, INTENT(IN)     :: pgas, tgas, pe00, slope0
      REAL, INTENT(OUT)    :: pele, dnc, dlpgpe
C--- Local variables:
      INTEGER, PARAMETER   :: mode=1, iprint=1, itmax=20
      REAL,    PARAMETER   :: delta=1.e-6
      REAL                 :: temp, pe0, pe1, pg0, pg1, dlpe, 
     &                        dlpg0, dlpg1, dnc0, dnc1, slope1
C
      temp  = tgas
      pe0   = pe00
C
      CALL IONDIS(temp, pe0, mode, iprint, pg0, dnc0)
C
      itnum = 1
      dlpg0 = ALOG(pg0/pgas)
      IF(ABS(dlpg0) .LT. delta) THEN
C--- First guess of PE was correct!
C         WRITE(*,"(1X,I4,5ES14.5, '   **')") 
C     &         itnum, temp, pgas, pg0, pe0, dlpg0
         pele   = pe0
         dnc    = dnc0
         dlpgpe = slope0
         RETURN
      ENDIF
C
C--- First guess of PE has to be improved:
C
      dlpe = dlpg0/slope0
      IF (ABS(dlpe) .LT. 0.1*delta)dlpe=SIGN(0.1*delta,dlpg0) 
      pe1 = pe0*exp(-dlpe)
C
C   ***********************************************************
C   START OF LOOP
      DO itnum = 2, itmax
         CALL IONDIS(temp, pe1, mode, iprint, pg1, dnc1)
         dlpg1  = ALOG(pg1/pgas)
         slope1 = (dlpg0-dlpg1)/dlpe
         IF(ABS(delta*slope1) .GT. ABS(dlpg1)) EXIT
         pe0    = pe1
         dlpg0  = dlpg1
         dlpe   = dlpg1/slope1
         pe1    = pe0*exp(-dlpe)
      ENDDO
C   *******************************************************
      IF (itnum .LE. itmax) THEN
C        WRITE(*,"(1X,I4,5ES14.5)")itnum, temp, pgas, pg1, pe1, dlpg1
        pele   = pe1
        dnc    = dnc1
        dlpgpe = slope1
      ELSE
        WRITE(*,750)itnum, temp, pe0, pe1, dlpg0, dlpg1
  750   FORMAT(1X, 'P_COMP: SLOW CONVERGENCE! '/ 
     &         1X,I4,ES14.5, 2X, 4ES14.5)
        STOP
      ENDIF
C-----------------------------------------------------------------------
      END SUBROUTINE P_COMP
C--------------------******--------------------------------------------+-------

      SUBROUTINE IONDIS(TEMP,PELE,MODE,IPRINT,PGAS,DENSNC)
C
      USE iondis1_com
      USE iondis2_com
      USE iondis3_com
      USE iondis4_com
      USE constants
C
      REAL, PARAMETER  :: RLIMIT=1.0E-37
C
      DIMENSION IATIJ(NDIM4),FRACJ2(NDIM4),FREEJ(NDIM4),FRACJL(NDIM4)
C
C     IONISATION RATIO SAHAJ:NJ/SUM(NJ),ELEMENT I
C     FOUR STAGES PER ELEMENT:I,II,III,IV
C
      DO 15 I = 1, 200
         FRACJL(I) = 0.0
 15   CONTINUE
      TH=5040./TEMP
      TN=-TH*ALN10
      JATI=0
      CALL QATI1(TEMP)
      IF(MODE.EQ.2)CALL QATI2(TEMP)
      CALL DCHI(TH,PELE)
      SA=1.2023E09/TH/TH/SQRT(TH)/PELE
      I1=JATI/4
      DO 10 I=1,I1
      J1=4*I-3
      J2=J1+1
      J3=J2+1
      J4=J3+1
      K=NAMJ(J1)/100
      JIZ(K)=J1
      X1=PARTFJ(J1)
      X2=PARTFJ(J2)
      X3=PARTFJ(J3)
      X4=PARTFJ(J4)
      A2=SA*EXP(TN*CHIJ(J1))/X1*X2
      A3=A2*SA*EXP(AMAX1(-400.,TN*CHIJ(J2)))/X2*X3
      A4=A3*SA*EXP(AMAX1(-400.,TN*CHIJ(J3)))/X3*X4
      AS=1.+A2+A3+A4
      SAHAJ(J1)=1./AS
      SAHAJ(J2)=A2/AS
      SAHAJ(J3)=AMAX1(RLIMIT,A3/AS)
   10 SAHAJ(J4)=AMAX1(RLIMIT,A4/AS)
C
C     NEGATIVE IONS:IONISATION RATIO SAHAJ,RENORMALISATION
      J=JATI
C     H MINUS
      J=J+1
      NAMJ(J)=-101
      CHIJ(J)=0.754
      PARTFJ(J)=1.
      J1=JIZ(1)
      A1=PARTFJ(J)/PARTFJ(J1)/SA/EXP(CHIJ(J)*TN)*SAHAJ(J1)
      X=1.+A1
      SAHAJ(J)=A1/X
      SAHAJ(J1)=SAHAJ(J1)/X
      SAHAJ(J1+1)=SAHAJ(J1+1)/X
C     C MINUS
      J=J+1
      NAMJ(J)=-601
      CHIJ(J)=1.25
      PARTFJ(J)=4.
      J1=JIZ(6)
      A1=PARTFJ(J)/PARTFJ(J1)/SA/EXP(CHIJ(J)*TN)*SAHAJ(J1)
      X=1.+A1
      SAHAJ(J)=A1/X
      SAHAJ(J1)=SAHAJ(J1)/X
      SAHAJ(J1+1)=SAHAJ(J1+1)/X
      SAHAJ(J1+2)=SAHAJ(J1+2)/X
      SAHAJ(J1+3)=SAHAJ(J1+3)/X
C     O MINUS
      J=J+1
      NAMJ(J)=-801
      CHIJ(J)=1.46
      PARTFJ(J)=6.
      J1=JIZ(8)
      A1=PARTFJ(J)/PARTFJ(J1)/SA/EXP(CHIJ(J)*TN)*SAHAJ(J1)
      X=1.+A1
      SAHAJ(J)=A1/X
      SAHAJ(J1)=SAHAJ(J1)/X
      SAHAJ(J1+1)=SAHAJ(J1+1)/X
      SAHAJ(J1+2)=SAHAJ(J1+2)/X
      SAHAJ(J1+3)=SAHAJ(J1+3)/X
      JNEGI=J
C
C     PARTITION FUNCTIONS:MOLECULES
C     DISSOCIATION CONSTANTS SAHAJ=P(A)*P(B)/P(AB)
      CALL QMOL(TEMP)
C
C     CROSS-REFERENCE TABLE IATIJ
C     ELEMENT INDEX I OF ATOM OR ION J
      DO 30 J=1,JNEGI
      J1=IABS(NAMJ(J))
      J2=(J1/100)*100
      DO 40 I=1,MAI
      IF(NAMI(I).EQ.J2)GO TO 30
   40 CONTINUE
   30 IATIJ(J)=I
      J1=JNEGI+1
      DO 32 J=J1,JMOL2
   32 IATIJ(J)=0
C     ATOMIC WEIGHT OF ATOM OR ION J
      DO 36 J=1,JNEGI
   36 WTJ(J)=WTI(IATIJ(J))
C
C     STARTING GUESS FRACTIONAL ABUNDANCES
C     ATOMS,POS.IONS,ELECTRONS:MOLECULES AND NEG.IONS SUPPRESSED
      FRACEL=0.
      DO 50 J=1,JATI
      FRACJ(J)=SAHAJ(J)*FRACI(IATIJ(J))
   50 FRACEL=FRACEL+FRACJ(J)*NCHARG(NAMJ(J))
      PNUC=PELE/FRACEL
      J1=JATI+1
C     STARTING GUESS MOLECULES
      J1=JNEGI+1
      DO 70 J=J1,JMOL2
      CALL DECOM2(NAMJ(J),JA,JB)
      Z=FRACJ(JA)*FRACJ(JB)*PNUC/SAHAJ(J)
      FRACJ(J)=Z
      A=FRACI(IATIJ(JA))
      B=FRACI(IATIJ(JB))
      IF(Z.LT.A.AND.Z.LT.B)GO TO 70
C     SECOND GUESS IF MOLECULE FORMATION IS EXHAUSTIVE
      IF(JA.EQ.JB)GO TO 72
      IF(A.LE.B)GO TO 74
      GO TO 76
   72 J2=JA
      A1=SQRT(A/Z)/3.
      GO TO 78
   74 J2=JA
      A1=A/Z*0.5
      GO TO 78
   76 J2=JB
      A1=B/Z*0.5
      GO TO 78
   78 FRACJ(J2)=FRACJ(J2)*A1
      FRACJ(J2+1)=FRACJ(J2+1)*A1
      FRACJ(J2+2)=FRACJ(J2+2)*A1
      FRACJ(J2+3)=FRACJ(J2+3)*A1
      FRACJ(J)=FRACJ(JA)*FRACJ(JB)*PNUC/SAHAJ(J)
   70 CONTINUE
C     SECOND GUESS ELECTRONS,PNUC;ANORM
      FRACEL=0.
      ANORM=0.
      DO 80 J=1,JATI
      FRACEL=FRACEL+FRACJ(J)*NCHARG(NAMJ(J))
   80 ANORM=ANORM+FRACJ(J)
      J1=JNEGI+1
      DO 90 J=J1,JMOL2
      FRACEL=FRACEL+FRACJ(J)*NCHARG(NAMJ(J))
   90 ANORM=ANORM+2.*FRACJ(J)
      PNUC=PELE/FRACEL
C
C     MAXIMUM NUMBER OF ITERATIONS;CONVERGENCE CRITERION
      ITMA=30
      DELTA=0.01
C
C     FRACTIONAL ABUNDANCES FRACJ OF DIATOMIC MOLECULES
C     GIVEN FRACTIONAL ABUNDANCES FRACJ OF ATOMS
      IT=0
  100 ITER=0
      IT=IT+1
      J1=JNEGI+1
      DO 110 J=J1,JMOL2
      CALL DECOM2(NAMJ(J),JA,JB)
  110 FRACJ(J)=FRACJ(JA)*FRACJ(JB)*PNUC/SAHAJ(J)
C
C     FRACTIONAL ABUNDANCES FRACJ2 OF ATOMS AND POS.IONS
C     GIVEN FRACTIONAL ABUNDANCES FRACJ OF DIATOMIC MOLECULES
      DO 120 J=1,JATI
      I1=IATIJ(J)
      X=0.
      J2=JNEGI+1
      DO 130 J1=J2,JMOL2
      CALL DECOM2(NAMJ(J1),JA,JB)
      IF(IATIJ(JA).EQ.I1)X=X+FRACJ(J1)
      IF(IATIJ(JB).EQ.I1)X=X+FRACJ(J1)
  130 CONTINUE
      FREEJ(J)=AMAX1(1.-X/FRACI(I1),RLIMIT)
      FRACJ2(J)=SAHAJ(J)*FRACI(I1)*FREEJ(J)
  120 CONTINUE
C     DUMMY FRACJ2,FREEJ FOR MOLECULES
      J1=JNEGI+1
      DO 140 J=J1,JMOL2
      FREEJ(J)=RLIMIT
  140 FRACJ2(J)=RLIMIT
C     FRACTIONAL ABUNDANCES FRACJ=FRACJ2 OF NEGATIVE IONS
C     GIVEN DISSOCIATION FRACTION FREEJ OF ATOMS
      J1=JATI+1
      DO 60 J=J1,JNEGI
      IF(IT.EQ.1)FRACJL(J)=RLIMIT
      IF(IT.GT.1)FRACJL(J)=FRACJ(J)
      I1=IATIJ(J)
      J1=JIZ(NAMI(I1)/100)
      FREEJ(J)=FREEJ(J1)
      X=SAHAJ(J)*FRACI(I1)*FREEJ(J)
      FRACJ(J)=X
      FRACJ2(J)=X
   60 CONTINUE
C
C     CONVERGENCE CHECK (NEUTRAL ATOMS)
C     WHEN IT=1 CHECK IF FRACJ AND FRACJ2 AGREE
C     WHEN IT>1 CHECK IF FRACJ AND FRACJL (=FRACJ OF LAST IT.) AGREE
      IF(IT.EQ.ITMA)GO TO 151
      DO 150 J=1,JATI,4
      IF(IT.EQ.1)Z=FRACJ2(J)
      IF(IT.GT.1)Z=FRACJL(J)
      IF(Z.LE.RLIMIT)GO TO 150
      IF(DELTA.LT.ABS(FRACJ(J)/Z-1.))ITER=1
  150 CONTINUE
  151 CONTINUE
C
C     ITERATION OF FRACTIONAL ABUNDANCES FRACJ:ATOMS,POS.IONS
C     THREE POSITIVE IONS PER NEUTRAL ATOM
      IF(ITER.EQ.0)GO TO 190
      Z=1.
      IF(TEMP.LT.3360..AND.PELE.GT.1.26)Z=0.1
      IF(TEMP.LT.3360..AND.PELE.GT.0.10)Z=0.3
      IF(TEMP.LT.5600..AND.PELE.GT.100.)Z=0.3
      J1=JATI-3
      DO 160 J=1,J1,4
      FRACJL(J)=FRACJ(J)
      FRACJL(J+1)=FRACJ(J+1)
      FRACJL(J+2)=FRACJ(J+2)
      FRACJL(J+3)=FRACJ(J+3)
      U=FRACJ(J)
C     NEUTRAL ATOMS
      IF(U.LT.RLIMIT)U=RLIMIT
      V=FRACJ2(J)
      W=V/U
      X=FREEJ(J)
      IF(V.LT.RLIMIT)GO TO 170
      IF(W.LT.2.)GO TO 172
      A1=FRACI(IATIJ(J))
      IF(W.LT.1.E-5.AND.A1.GT.0.2)GO TO 174
      IF(X.LT.0.2.AND.A1.LE.0.2)GO TO 174
      IF(A1.GT.0.2)GO TO 176
      IF(A1.LE.0.2.AND.X.LT.0.99)GO TO 177
      GO TO 178
  170 FRACJ(J)=U*0.95
      GO TO 180
  172 FRACJ(J)=U+(V-U)*X/1.5
      GO TO 180
  174 FRACJ(J)=U*(1.+Z*X)
      GO TO 180
  176 FRACJ(J)=U*(1.+1.5*X)
      GO TO 180
  177 FRACJ(J)=U*(1.+2.5*Z*X)
      GO TO 180
  178 FRACJ(J)=U*5.
      GO TO 180
  180 A=FRACJ(J)/FRACJL(J)
C     POSITIVE IONS
      FRACJ(J+1)=FRACJL(J+1)*A
      FRACJ(J+2)=FRACJL(J+2)*A
      FRACJ(J+3)=FRACJL(J+3)*A
  160 CONTINUE
C
C     COLLECTIVE QUANTITIES,SOME SERVING AS INPUT FOR NEXT ITERATION
  190 FRACEL=0.
      PGAS=0.
      ANORM=0.
      DO 192 J=1,JNEGI
      FRACEL=FRACEL+FRACJ(J)*NCHARG(NAMJ(J))
  192 ANORM=ANORM+FRACJ(J)
      J1=JNEGI+1
      DO 200 J=J1,JMOL2
      FRACEL=FRACEL+FRACJ(J)*NCHARG(NAMJ(J))
  200 ANORM=ANORM+2.*FRACJ(J)
      IF(FRACEL.GE.RLIMIT)GO TO 210
      IF(IT.GT.3)FRACEL=RLIMIT
      IF(IT.LE.3)FRACEL=ABS(FRACEL)*0.9
  210 PNUC=PELE/FRACEL
C     EXIT TO NEXT ITERATION
      IF(IT.LT.ITMA.AND.ITER.EQ.1)GO TO 100
      X=0.
      Y=0.
      DO 220 J=1,JMOL2
      A=FRACJ(J)
      ZETAJ(J)=A/PARTFJ(J)
      IF(A.LE.RLIMIT)ZETAJ(J)=RLIMIT
      X=X+A
  220 Y=Y+WTJ(J)*A
      X=X+FRACEL
      Y=Y+FRACEL/1822.
      PGAS=PNUC*X
      EFFMY=Y/X
      DENSNC=PNUC/(BOLTZK*TEMP)
      IF(IPRINT.NE.2)GO TO 900
C
C     PRINTOUT IONDIS
 9998 FORMAT(3X, 'THETA', 5X, 'TEMP', 5X, 'LOG PE', 4X, 'LOG PG', 4X,
     >       'LOG PN', 5X, 'EFFMY', 5X, 'ANORM', 2X, 'IT', 6X, 'DENSNC')
 9997 FORMAT(1X,F7.4,F10.0,3F10.3,F10.4,F10.4,I4,1PE12.3/)
 9996 FORMAT(4X, 'J', 5X, 'NAMJ', 7X, 'WTJ', 6X, 'CHIJ', 2X, 'L PARTFJ',
     >       3X, 'L SAHAJ', 3X, 'L FRACJ', 2X, 'L FRACJ2', 3X, 
     >       'L FREEJ', 2X, 'L FRACJL', 3X, 'L ZETAJ', 1X, 'IATIJ', 1X,
     >       'FREE EL CONTR.' /)
 9995 FORMAT(1X,I4,I9,9F10.3,I4,I7)
      WRITE(3,9998)
      A1=ALOG10(PELE)
      A2=ALOG10(PGAS)
      A3=ALOG10(PNUC)
      WRITE(3,9997)TH,TEMP,A1,A2,A3,EFFMY,ANORM,IT,DENSNC
      WRITE(3,9996)
      J=-1
      A2=-35.
      A3=1./1823.
      A1=ALOG10(FRACEL)
      WRITE(3,9995)J,J,A3,A2,A2,A2,A1
      DO 9901 J=1,JMOL2
      A1=ALOG10(PARTFJ(J))
      A2=ALOG10(SAHAJ(J))
      A3=FRACJ(J)
      IF(A3.LE.RLIMIT)A3=RLIMIT
      A3=ALOG10(A3)
      A4=ALOG10(ZETAJ(J))
      A5=FRACJ2(J)
      IF(A5.LE.RLIMIT)A5=RLIMIT
      A5=ALOG10(A5)
      A6=ALOG10(FREEJ(J))
      A7=FRACJL(J)
      IF(A7.LE.RLIMIT.OR.IT.EQ.1)A7=RLIMIT
      A7=ALOG10(A7)
      I=IATIJ(J)
      A=NCHARG(NAMJ(J))*FRACJ(J)/FRACEL*1000.
      K=IFIX(A)
 9901 WRITE(3,9995)J,NAMJ(J),WTJ(J),CHIJ(J),A1,A2,A3,A5,A6,A7,A4,I,K
  900 CONTINUE
      RETURN
      END SUBROUTINE IONDIS
C
C
C
      SUBROUTINE QATI1(TEMP)
C
      USE iondis2_com
      USE iondis3_com
      USE iondis4_com
      USE constants
C
      REAL, PARAMETER  :: RLIMIT=1.0E-37
C
C     PARTITION FUNCTIONS:ATOMS AND POSITIVE IONS
C     EXCLUDING RARE ELEMENTS:MODE=1
C
C
      TN=-ALN10*5040./TEMP
      J=JATI
C     HYDROGEN
      J=J+1
      NAMJ(J)=100
      CHIJ(J)=13.595
      PARTFJ(J)=2.+20.498*EXP(10.853*TN)+747.50*EXP(13.342*TN)
      J=J+1
      NAMJ(J)=101
      CHIJ(J)=100.
      PARTFJ(J)=1.
      J=J+1
      NAMJ(J)=102
      CHIJ(J)=100.
      PARTFJ(J)=RLIMIT
      J=J+1
      NAMJ(J)=103
      CHIJ(J)=100.
      PARTFJ(J)=RLIMIT
C
C     HELIUM
      J=J+1
      NAMJ(J)=200
      CHIJ(J)=24.580
      PARTFJ(J)=1.+28.170*EXP(21.170*TN)+527.83*EXP(24.125*TN)
      J=J+1
      NAMJ(J)=201
      CHIJ(J)=54.403
      PARTFJ(J)=2.+22.281*EXP(43.708*TN)+987.72*EXP(53.542*TN)
      J=J+1
      NAMJ(J)=202
      CHIJ(J)=100.
      PARTFJ(J)=1.
      J=J+1
      NAMJ(J)=203
      CHIJ(J)=100.
      PARTFJ(J)=RLIMIT
C
C     CARBON
      J=J+1
      NAMJ(J)=600
      CHIJ(J)=11.256
      PARTFJ(J)=1.+8.0158*EXP(0.004*TN)+5.8833*EXP(1.359*TN)
     1 +33.752*EXP(6.454*TN)+595.34*EXP(10.376*TN)
      J=J+1
      NAMJ(J)=601
      CHIJ(J)=24.376
      PARTFJ(J)=2.+4.0003*EXP(0.008*TN)+62.29*EXP(16.17*TN)
     1 +82.915*EXP(21.614*TN)+15.981*EXP(5.688*TN)+435.81*EXP(26.269*TN)
      J=J+1
      NAMJ(J)=602
      CHIJ(J)=47.871
      PARTFJ(J)=1.+10.028*EXP(6.691*TN)+15.757*EXP(25.034*TN)
     1 +15.413*EXP(17.60*TN)+242.2*EXP(38.58*TN)+243.63*EXP(47.133*TN)
      J=J+1
      NAMJ(J)=603
      CHIJ(J)=64.476
      PARTFJ(J)=2.
C
C     NITROGEN
      J=J+1
      NAMJ(J)=700
      CHIJ(J)=14.529
      PARTFJ(J)=4.+14.050*EXP(2.554*TN)+30.801*EXP(9.169*TN)
     1 +899.1*EXP(13.72*TN)+10.000*EXP(12.353*TN)+64.000*EXP(14.874*TN)
      J=J+1
      NAMJ(J)=701
      CHIJ(J)=29.593
      PARTFJ(J)=1.+8.0462*EXP(0.014*TN)+6.2669*EXP(2.131*TN)
     1 +51.01*EXP(15.00*TN)+282.81*EXP(24.949*TN)
     2 +7.3751*EXP(6.376*TN)+215.48*EXP(29.465*TN)
      J=J+1
      NAMJ(J)=702
      CHIJ(J)=47.426
      PARTFJ(J)=2.+4.0003*EXP(0.022*TN)+113.7*EXP(32.82*TN)
     1 +505.6*EXP(46.02*TN)+13.100*EXP(7.212*TN)+19.642*EXP(15.228*TN)
      J=J+1
      NAMJ(J)=703
      CHIJ(J)=77.450
      PARTFJ(J)=1.
C
C     OXYGEN
      J=J+1
      NAMJ(J)=800
      CHIJ(J)=13.614
      PARTFJ(J)=5.+4.0029*EXP(0.022*TN)+5.3656*EXP(2.019*TN)
     1 +36.285*EXP(9.812*TN)+1175.4*EXP(13.45*TN)
     2 +962.1*EXP(16.09*TN)+14.853*EXP(14.293*TN)
      J=J+1
      NAMJ(J)=801
      CHIJ(J)=35.108
      PARTFJ(J)=4.+12.784*EXP(3.472*TN)+5.6828*EXP(7.437*TN)
     1 +98.092*EXP(22.579*TN)+1134.5*EXP(33.03*TN)+53.0*EXP(27.95*TN)
      J=J+1
      NAMJ(J)=802
      CHIJ(J)=54.886
      PARTFJ(J)=1.+8.0703*EXP(0.032*TN)+5.7144*EXP(2.760*TN)
     1 +84.116*EXP(35.328*TN)+640.5*EXP(45.5*TN)+674.0*EXP(53.6*TN)
      J=J+1
      NAMJ(J)=803
      CHIJ(J)=77.394
      PARTFJ(J)=6.
C
C     NEON
      J=J+1
      NAMJ(J)=1000
      CHIJ(J)=21.559
      PARTFJ(J)=1.+51.08*EXP(17.84*TN)+548.9*EXP(20.79*TN)
      J=J+1
      NAMJ(J)=1001
      CHIJ(J)=41.071
      PARTFJ(J)=4.+2.0007*EXP(0.097*TN)+116.0*EXP(30.90*TN)
     1 +444.0*EXP(37.39*TN)
      J=J+1
      NAMJ(J)=1002
      CHIJ(J)=63.729
      PARTFJ(J)=5.+4.0342*EXP(0.092*TN)+5.6162*EXP(3.424*TN)
     1 +11.518*EXP(24.806*TN)+152.6*EXP(46.9*TN)+208.3*EXP(55.8*TN)
      J=J+1
      NAMJ(J)=1003
      CHIJ(J)=97.162
      PARTFJ(J)=4.
C
C     SODIUM
      J=J+1
      NAMJ(J)=1100
      CHIJ(J)=5.138
      PARTFJ(J)=2.+11.635*EXP(2.400*TN)+158.36*EXP(4.552*TN)
      J=J+1
      NAMJ(J)=1101
      CHIJ(J)=47.290
      PARTFJ(J)=1.+31.2*EXP(34.52*TN)+76.7*EXP(40.67*TN)
      J=J+1
      NAMJ(J)=1102
      CHIJ(J)=71.647
      PARTFJ(J)=4.+2.0019*EXP(0.170*TN)+38.057*EXP(44.554*TN)
     1 +28.311*EXP(51.689*TN)+199.6*EXP(58.8*TN)
      J=J+1
      NAMJ(J)=1103
      CHIJ(J)=98.880
      PARTFJ(J)=9.
C
C     MAGNESIUM
      J=J+1
      NAMJ(J)=1200
      CHIJ(J)=7.644
      PARTFJ(J)=1.+10.744*EXP(2.805*TN)+291.51*EXP(6.777*TN)
     1 +53.749*EXP(7.254*TN)
      J=J+1
      NAMJ(J)=1201
      CHIJ(J)=15.031
      PARTFJ(J)=2.+6.2270*EXP(4.459*TN)+31.129*EXP(9.789*TN)
     1 +132.64*EXP(13.137*TN)
      J=J+1
      NAMJ(J)=1202
      CHIJ(J)=80.117
      PARTFJ(J)=1.+40.438*EXP(57.413*TN)+239.2*EXP(71.46*TN)
     1 +20.384*EXP(58.010*TN)
      J=J+1
      NAMJ(J)=1203
      CHIJ(J)=109.294
      PARTFJ(J)=4.
C
C     ALUMINIUM
      J=J+1
      NAMJ(J)=1300
      CHIJ(J)=5.984
      PARTFJ(J)=2.+4.0009*EXP(0.014*TN)+25.44*EXP(3.784*TN)
     1 +142.22*EXP(5.420*TN)+96.337*EXP(8.833*TN)
      J=J+1
      NAMJ(J)=1301
      CHIJ(J)=18.823
      PARTFJ(J)=1.+10.081*EXP(4.749*TN)+64.27*EXP(11.61*TN)
     1 +344.6*EXP(17.4*TN)
      J=J+1
      NAMJ(J)=1302
      CHIJ(J)=28.441
      PARTFJ(J)=2.+6.3277*EXP(6.751*TN)+29.509*EXP(16.681*TN)
     1 +134.16*EXP(24.151*TN)
      J=J+1
      NAMJ(J)=1303
      CHIJ(J)=119.957
      PARTFJ(J)=1.
C
C     SILICON
      J=J+1
      NAMJ(J)=1400
      CHIJ(J)=8.149
      PARTFJ(J)=1.+7.9658*EXP(0.020*TN)+4.6762*EXP(0.752*TN)
     1 +1.3512*EXP(1.614*TN)+123.23*EXP(5.831*TN)+443.78*EXP(7.431*TN)
      J=J+1
      NAMJ(J)=1401
      CHIJ(J)=16.339
      PARTFJ(J)=2.+4.0000*EXP(0.036*TN)+7.4186*EXP(8.795*TN)
     1 +24.175*EXP(11.208*TN)+86.91*EXP(14.14*TN)
     2 +14.469*EXP(5.418*TN)+11.972*EXP(7.825*TN)+269.05*EXP(19.412*TN)
      J=J+1
      NAMJ(J)=1402
      CHIJ(J)=33.459
      PARTFJ(J)=1.+9.1793*EXP(6.572*TN)+4.8766*EXP(11.449*TN)
     1 +42.41*EXP(17.05*TN)+88.84*EXP(26.23*TN)+180.69*EXP(34.599*TN)
      J=J+1
      NAMJ(J)=1403
      CHIJ(J)=45.130
      PARTFJ(J)=2.
C
C     PHOSPHORUS
      J=J+1
      NAMJ(J)=1500
      CHIJ(J)=10.474
      PARTFJ(J)=4.+13.521*EXP(1.514*TN)+22.213*EXP(5.575*TN)
     1 +503.3*EXP(9.99*TN)+10.000*EXP(8.076*TN)
      J=J+1
      NAMJ(J)=1501
      CHIJ(J)=19.720
      PARTFJ(J)=1.+8.0241*EXP(0.043*TN)+5.8085*EXP(1.212*TN)
     1 +51.754*EXP(8.545*TN)+252.40*EXP(15.525*TN)
      J=J+1
      NAMJ(J)=1502
      CHIJ(J)=30.156
      PARTFJ(J)=2.+4.0021*EXP(0.074*TN)+20.798*EXP(7.674*TN)
     1 +62.419*EXP(16.639*TN)+200.78*EXP(25.118*TN)
      J=J+1
      NAMJ(J)=1503
      CHIJ(J)=51.354
      PARTFJ(J)=1.
C
C     SULPHUR
      J=J+1
      NAMJ(J)=1600
      CHIJ(J)=10.357
      PARTFJ(J)=5.+3.9615*EXP(0.053*TN)+5.0780*EXP(1.121*TN)
     1 +15.094*EXP(5.812*TN)+426.5*EXP(9.32*TN)+544.4*EXP(11.91*TN)
      J=J+1
      NAMJ(J)=1601
      CHIJ(J)=23.405
      PARTFJ(J)=4.+11.438*EXP(1.892*TN)+5.5126*EXP(3.646*TN)
     1 +141.00*EXP(13.550*TN)+33.052*EXP(16.253*TN)+381.0*EXP(20.22*TN)
      J=J+1
      NAMJ(J)=1602
      CHIJ(J)=35.047
      PARTFJ(J)=1.+8.13*EXP(0.08*TN)+5.7245*EXP(1.590*TN)
     1 +144.64*EXP(13.712*TN)+106.49*EXP(22.050*TN)
      J=J+1
      NAMJ(J)=1603
      CHIJ(J)=47.292
      PARTFJ(J)=6.
C
C     ARGON
      J=J+1
      NAMJ(J)=1800
      CHIJ(J)=15.755
      PARTFJ(J)=1.+64.5*EXP(12.74*TN)+487.5*EXP(15.05*TN)
      J=J+1
      NAMJ(J)=1801
      CHIJ(J)=27.619
      PARTFJ(J)=4.+2.0026*EXP(0.178*TN)+200.2*EXP(19.0*TN)
     1 +407.7*EXP(24.4*TN)
      J=J+1
      NAMJ(J)=1802
      CHIJ(J)=40.899
      PARTFJ(J)=5.+4.0495*EXP(0.151*TN)+14.447*EXP(1.561*TN)
     1 +46.823*EXP(17.399*TN)+543.5*EXP(33.8*TN)+253.1*EXP(25.9*TN)
      J=J+1
      NAMJ(J)=1803
      CHIJ(J)=59.793
      PARTFJ(J)=4.
C
C     POTASSIUM
      J=J+1
      NAMJ(J)=1900
      CHIJ(J)=4.339
      PARTFJ(J)=2.+12.978*EXP(1.871*TN)+148.67*EXP(3.713*TN)
     1 +6.3493*EXP(4.0*TN)
      J=J+1
      NAMJ(J)=1901
      CHIJ(J)=31.810
      PARTFJ(J)=1.+79.8*EXP(22.45*TN)+4.0001*EXP(20.59*TN)
     1 +148.2*EXP(28.12*TN)
      J=J+1
      NAMJ(J)=1902
      CHIJ(J)=45.738
      PARTFJ(J)=4.+2.0171*EXP(0.273*TN)+182.1*EXP(29.3*TN)
     1 +1149.9*EXP(40.5*TN)
      J=J+1
      NAMJ(J)=1903
      CHIJ(J)=60.897
      PARTFJ(J)=9.
C
C     CALCIUM
      J=J+1
      NAMJ(J)=2000
      CHIJ(J)=6.111
      PARTFJ(J)=1.+18.237*EXP(2.050*TN)+27.501*EXP(3.349*TN)
     1 +149.26*EXP(5.321*TN)+94.524*EXP(4.873*TN)+705.47*EXP(7.017*TN)
      J=J+1
      NAMJ(J)=2001
      CHIJ(J)=11.868
      PARTFJ(J)=2.+11.871*EXP(1.769*TN)+14.071*EXP(5.109*TN)
     1 +106.05*EXP(9.524*TN)
      J=J+1
      NAMJ(J)=2002
      CHIJ(J)=51.207
      PARTFJ(J)=1.+87.05*EXP(28.22*TN)+164.9*EXP(41.85*TN)
      J=J+1
      NAMJ(J)=2003
      CHIJ(J)=67.181
      PARTFJ(J)=6.
C
C     TITANIUM
      J=J+1
      NAMJ(J)=2200
      CHIJ(J)=6.818
      PARTFJ(J)=5.+7.0887*EXP(0.021*TN)+8.9186*EXP(0.048*TN)
     1 +17.563*EXP(1.029*TN)+206.68*EXP(2.183*TN)+438.57*EXP(4.109*TN)
     2 +1486.2*EXP(5.786*TN)+38.046*EXP(0.846*TN)+69.627*EXP(1.792*TN)
     3 +364.28*EXP(3.836*TN)+98.856*EXP(2.561*TN)
     4 +57.993*EXP(4.869*TN)+442.15*EXP(6.340*TN)
      J=J+1
      NAMJ(J)=2201
      CHIJ(J)=13.635
      PARTFJ(J)=4.+19.784*EXP(0.023*TN)+32.064*EXP(0.124*TN)
     1 +37.089*EXP(0.774*TN)+110.67*EXP(1.810*TN)+322.5*EXP(4.95*TN)
     2 +521.88*EXP(9.585*TN)+10.000*EXP(1.082*TN)+120.00*EXP(11.279*TN)
      J=J+1
      NAMJ(J)=2202
      CHIJ(J)=28.137
      PARTFJ(J)=5.+16.169*EXP(0.041*TN)+22.355*EXP(1.375*TN)
     1 +24.165*EXP(4.768*TN)+83.513*EXP(10.985*TN)+222.80*EXP(19.769*TN)
      J=J+1
      NAMJ(J)=2203
      CHIJ(J)=43.236
      PARTFJ(J)=10.
C
C     CHROMIUM
      J=J+1
      NAMJ(J)=2400
      CHIJ(J)=6.763
      PARTFJ(J)=7.+30.184*EXP(0.993*TN)+79.285*EXP(3.070*TN)
     1 +149.53*EXP(5.673*TN)+215.37*EXP(3.339*TN)+119.20*EXP(4.801*TN)
     2 +1525.9*EXP(7.42*TN)+184.99*EXP(2.829*TN)
     3 +1352.5*EXP(4.990*TN)
      J=J+1
      NAMJ(J)=2401
      CHIJ(J)=16.493
      PARTFJ(J)=6.+46.619*EXP(1.645*TN)+160.14*EXP(3.727*TN)
     1 +488.04*EXP(7.181*TN)+657.19*EXP(12.299*TN)+47.174*EXP(2.902*TN)
     2 +267.03*EXP(4.273*TN)+441.13*EXP(8.569*TN)+150.66*EXP(14.912*TN)
      J=J+1
      NAMJ(J)=2402
      CHIJ(J)=30.950
      PARTFJ(J)=1.+24.377*EXP(0.047*TN)+122.84*EXP(2.556*TN)
     1 +285.51*EXP(9.441*TN)+794.16*EXP(21.198*TN)
      J=J+1
      NAMJ(J)=2403
      CHIJ(J)=49.580
      PARTFJ(J)=28.
C
C     MANGANESE
      J=J+1
      NAMJ(J)=2500
      CHIJ(J)=7.432
      PARTFJ(J)=6.+53.911*EXP(2.527*TN)+225.58*EXP(4.181*TN)
     1 +954.5*EXP(6.96*TN)+45.618*EXP(2.285*TN)
     2 +298.44*EXP(5.631*TN)+2410.9*EXP(8.448*TN)
      J=J+1
      NAMJ(J)=2501
      CHIJ(J)=15.636
      PARTFJ(J)=7.+22.638*EXP(1.496*TN)+230.9*EXP(3.76*TN)
     1 +168.68*EXP(6.054*TN)+513.0*EXP(8.84*TN)+1680.8*EXP(14.21*TN)
      J=J+1
      NAMJ(J)=2502
      CHIJ(J)=33.690
      PARTFJ(J)=6.+70.193*EXP(3.531*TN)+72.337*EXP(6.967*TN)
     1 +213.95*EXP(15.222*TN)+539.52*EXP(25.069*TN)
      J=J+1
      NAMJ(J)=2503
      CHIJ(J)=53.001
      PARTFJ(J)=25.
C
C     IRON
      J=J+1
      NAMJ(J)=2600
      CHIJ(J)=7.896
      PARTFJ(J)=9.+14.410*EXP(0.066*TN)+2.7050*EXP(0.339*TN)
     1 +421.66*EXP(2.897*TN)+940.15*EXP(6.585*TN)
     2 +36.219*EXP(0.923*TN)+22.888*EXP(1.679*TN)
     3 +239.60*EXP(4.620*TN)+825.29*EXP(7.053*TN)
     4 +110.02*EXP(4.249*TN)+992.30*EXP(5.875*TN)
     5 +640.67*EXP(7.781*TN)
      J=J+1
      NAMJ(J)=2601
      CHIJ(J)=16.178
      PARTFJ(J)=10.+17.049*EXP(0.062*TN)+32.378*EXP(0.283*TN)
     1 +34.318*EXP(1.504*TN)+420.96*EXP(5.430*TN)+1067.2*EXP(11.210*TN)
     2 +154.01*EXP(2.792*TN)+462.11*EXP(7.627*TN)+329.86*EXP(13.623*TN)
      J=J+1
      NAMJ(J)=2602
      CHIJ(J)=30.640
      PARTFJ(J)=9.+15.791*EXP(0.077*TN)+47.119*EXP(3.723*TN)
     1 +279.93*EXP(12.137*TN)+1528.8*EXP(24.64*TN)
     2 +91.021*EXP(2.688*TN)+206.31*EXP(7.595*TN)+706.99*EXP(15.444*TN)
      J=J+1
      NAMJ(J)=2603
      CHIJ(J)=56.001
      PARTFJ(J)=6.
C
C     NICKEL
      J=J+1
      NAMJ(J)=2800
      CHIJ(J)=7.633
      PARTFJ(J)=9.+7.1268*EXP(0.026*TN)+12.449*EXP(0.137*TN)
     1 +11.995*EXP(0.315*TN)+10.055*EXP(1.778*TN)+391.21*EXP(6.621*TN)
     2 +26.391*EXP(2.249*TN)+328.0*EXP(4.036*TN)+938.79*EXP(7.621*TN)
      J=J+1
      NAMJ(J)=2801
      CHIJ(J)=18.147
      PARTFJ(J)=6.+4.1421*EXP(0.191*TN)+37.378*EXP(1.235*TN)
     1 +25.971*EXP(3.358*TN)+333.34*EXP(8.429*TN)+447.9*EXP(16.83*TN)
     2 +33.103*EXP(3.472*TN)+184.19*EXP(9.065*TN)
      J=J+1
      NAMJ(J)=2802
      CHIJ(J)=35.161
      PARTFJ(J)=9.+11.191*EXP(0.194*TN)+5.4174*EXP(1.305*TN)
     1 +53.679*EXP(5.813*TN)+460.68*EXP(14.172*TN)+380.01*EXP(26.169*TN)
      J=J+1
      NAMJ(J)=2803
      CHIJ(J)=56.025
      PARTFJ(J)=28.
C
      JATI=J
      RETURN
      END SUBROUTINE QATI1
C
C
C
      SUBROUTINE QATI2(TEMP)
C     PARTITION FUNCTIONS:ATOMS AND POSITIVE IONS
C     RARE ELEMENTS:MODE=2
C     WILL BE COMBINED WITH QATI1 IF MODE=2
C
      USE iondis2_com
      USE iondis3_com
      USE iondis4_com
      USE constants
C
      TN=-ALN10*5040./TEMP
      J=JATI
C
C     LITHIUM
      J=J+1
      NAMJ(J)=300
      CHIJ(J)=5.39
      PARTFJ(J)=2.+8.4915*EXP(2.022*TN)+97.5015*EXP(4.604*TN)
      J=J+1
      NAMJ(J)=301
      CHIJ(J)=75.919
      PARTFJ(J)=1.+23.3299*EXP(62.032*TN)+192.6701*EXP(72.624*TN)
      J=J+1
      NAMJ(J)=302
      CHIJ(J)=122.420
      PARTFJ(J)=2.+16.0047*EXP(96.149*TN)+247.9952*EXP(118.333*TN)
      J=J+1
      NAMJ(J)=303
      CHIJ(J)=200.
      PARTFJ(J)=1.
C
C     SCANDIUM
      J=J+1
      NAMJ(J)=2100
      CHIJ(J)=6.538
      PARTFJ(J)=4.+6.0014*EXP(0.021*TN)+83.196*EXP(2.056*TN)
     1 +67.367*EXP(3.551*TN)+329.44*EXP(5.465*TN)+44.079*EXP(1.535*TN)
     2 +170.00*EXP(3.797*TN)+533.92*EXP(6.203*TN)
     3 +34.164*EXP(2.389*TN)+124.85*EXP(4.858*TN)+228.99*EXP(7.141*TN)
      J=J+1
      NAMJ(J)=2101
      CHIJ(J)=12.891
      PARTFJ(J)=3.+11.998*EXP(0.011*TN)+16.928*EXP(0.430*TN)
     1 +28.478*EXP(1.156*TN)+82.042*EXP(3.711*TN)+234.54*EXP(8.863*TN)
      J=J+1
      NAMJ(J)=2102
      CHIJ(J)=24.752
      PARTFJ(J)=4.+6.0042*EXP(0.025*TN)+2.7101*EXP(3.499*TN)
     1 +13.980*EXP(10.463*TN)+65.304*EXP(18.606*TN)
      J=J+1
      NAMJ(J)=2103
      CHIJ(J)=74.090
      PARTFJ(J)=1.
C
C     VANADIUM
      J=J+1
      NAMJ(J)=2300
      CHIJ(J)=6.738
      PARTFJ(J)=4.+15.263*EXP(0.026*TN)+23.987*EXP(0.145*TN)
     1 +51.305*EXP(0.718*TN)+570.34*EXP(2.586*TN)+2560.*EXP(5.778*TN)
     2 +162.28*EXP(2.171*TN)+298.83*EXP(4.153*TN)
      J=J+1
      NAMJ(J)=2301
      CHIJ(J)=14.205
      PARTFJ(J)=1.+23.674*EXP(0.009*TN)+37.162*EXP(0.366*TN)
     1 +86.801*EXP(1.504*TN)+300.74*EXP(5.294*TN)+2448.*EXP(11.76*TN)
     2 +57.896*EXP(1.796*TN)+141.3*EXP(2.456*TN)+407.7*EXP(6.331*TN)
     3 +64.084*EXP(3.674*TN)
      J=J+1
      NAMJ(J)=2302
      CHIJ(J)=29.748
      PARTFJ(J)=4.+23.812*EXP(0.045*TN)+68.249*EXP(1.684*TN)
     1 +135.06*EXP(8.162*TN)+536.76*EXP(21.262*TN)
      J=J+1
      NAMJ(J)=2303
      CHIJ(J)=48.464
      PARTFJ(J)=21.
C
C     COBALT
      J=J+1
      NAMJ(J)=2700
      CHIJ(J)=7.863
      PARTFJ(J)=10.+11.912*EXP(0.112*TN)+20.442*EXP(0.341*TN)
     1 +28.386*EXP(0.809*TN)+369.9*EXP(3.646*TN)+1578.*EXP(6.97*TN)
     2 +66.0*EXP(2.071*TN)+55.540*EXP(2.405*TN)
     3 +402.7*EXP(5.212*TN)+727.1*EXP(8.262*TN)
      J=J+1
      NAMJ(J)=2701
      CHIJ(J)=17.052
      PARTFJ(J)=9.+11.259*EXP(0.135*TN)+38.224*EXP(0.517*TN)
     1 +22.996*EXP(1.606*TN)+261.35*EXP(6.772*TN)+637.15*EXP(12.622*TN)
     2 +23.023*EXP(2.512*TN)+41.660*EXP(4.348*TN)
     3 +264.65*EXP(8.253*TN)+181.67*EXP(15.377*TN)
      J=J+1
      NAMJ(J)=2702
      CHIJ(J)=33.491
      PARTFJ(J)=10.+16.036*EXP(0.132*TN)+7.8633*EXP(0.863*TN)
     1 +70.316*EXP(3.086*TN)+423.35*EXP(11.789*TN)+742.35*EXP(23.263*TN)
      J=J+1
      NAMJ(J)=2703
      CHIJ(J)=53.001
      PARTFJ(J)=20.
C
C     COPPER
      J=J+1
      NAMJ(J)=2900
      CHIJ(J)=7.724
      PARTFJ(J)=2.+11.055*EXP(4.212*TN)+268.9*EXP(7.154*TN)
     1 +10.308*EXP(1.493*TN)+126.30*EXP(5.859*TN)+1183.*EXP(9.734*TN)
      J=J+1
      NAMJ(J)=2901
      CHIJ(J)=20.286
      PARTFJ(J)=1.+19.298*EXP(2.865*TN)+50.597*EXP(8.260*TN)
     1 +823.4*EXP(14.536*TN)+1216.9*EXP(18.292*TN)
     2 +48.305*EXP(9.650*TN)+320.49*EXP(24.320*TN)
      J=J+1
      NAMJ(J)=2902
      CHIJ(J)=36.826
      PARTFJ(J)=6.+4.0155*EXP(0.337*TN)+70.326*EXP(8.520*TN)
     1 +313.12*EXP(16.925*TN)+536.53*EXP(28.342*TN)
      J=J+1
      NAMJ(J)=2903
      CHIJ(J)=61.975
      PARTFJ(J)=21.
C
C     ZINC
      J=J+1
      NAMJ(J)=3000
      CHIJ(J)=9.391
      PARTFJ(J)=1.+15.988*EXP(4.546*TN)+484.00*EXP(8.840*TN)
     1 +21.59*EXP(10.48*TN)+312.4*EXP(16.47*TN)
      J=J+1
      NAMJ(J)=3001
      CHIJ(J)=17.959
      PARTFJ(J)=2.+6.1902*EXP(6.113*TN)+128.3*EXP(13.30*TN)
     1 +234.9*EXP(16.36*TN)+10.259*EXP(7.926*TN)+498.4*EXP(24.63*TN)
      J=J+1
      NAMJ(J)=3002
      CHIJ(J)=39.701
      PARTFJ(J)=1.+24.690*EXP(10.291*TN)+106.75*EXP(20.689*TN)
     1 +439.56*EXP(32.077*TN)
      J=J+1
      NAMJ(J)=3003
      CHIJ(J)=65.074
      PARTFJ(J)=10.
C
C     STRONTIUM
      J=J+1
      NAMJ(J)=3800
      CHIJ(J)=5.693
      PARTFJ(J)=1.+24.315*EXP(2.015*TN)+47.434*EXP(3.777*TN)
     1 +668.25*EXP(5.348*TN)+89.40*EXP(4.429*TN)+425.6*EXP(6.964*TN)
      J=J+1
      NAMJ(J)=3801
      CHIJ(J)=11.027
      PARTFJ(J)=2.+6.2782*EXP(0.675*TN)+13.784*EXP(2.044*TN)
     1 +36.015*EXP(6.735*TN)+313.92*EXP(9.792*TN)
      J=J+1
      NAMJ(J)=3802
      CHIJ(J)=43.001
      PARTFJ(J)=1.
      J=J+1
      NAMJ(J)=3803
      CHIJ(J)=57.017
      PARTFJ(J)=4.+2.0349*EXP(1.232*TN)
C
C     YTTRIUM
      J=J+1
      NAMJ(J)=3900
      CHIJ(J)=6.377
      PARTFJ(J)=4.+6.0310*EXP(0.066*TN)+102.09*EXP(1.620*TN)
     1 +230.69*EXP(3.139*TN)+349.19*EXP(5.015*TN)
      J=J+1
      NAMJ(J)=3901
      CHIJ(J)=12.233
      PARTFJ(J)=1.+13.373*EXP(0.133*TN)+10.274*EXP(0.447*TN)
     1 +43.589*EXP(1.509*TN)+88.047*EXP(4.379*TN)+467.8*EXP(9.89*TN)
     2 +9.2791*EXP(2.999*TN)+25.574*EXP(7.394*TN)
      J=J+1
      NAMJ(J)=3902
      CHIJ(J)=20.514
      PARTFJ(J)=4.+6.0039*EXP(0.090*TN)+2.0213*EXP(0.936*TN)
     1 +6.6309*EXP(5.454*TN)+25.343*EXP(11.911*TN)
      J=J+1
      NAMJ(J)=3903
      CHIJ(J)=76.105
      PARTFJ(J)=1.
C
C     ZIRCONIUM
      J=J+1
      NAMJ(J)=4000
      CHIJ(J)=6.835
      PARTFJ(J)=5.+11.6004*EXP(0.084*TN)+24.5594*EXP(0.401*TN)
     1 +111.53*EXP(1.170*TN)+570.87*EXP(2.864*TN)+1185.4*EXP(5.852*TN)
      J=J+1
      NAMJ(J)=4001
      CHIJ(J)=14.028
      PARTFJ(J)=4.+17.788*EXP(0.068*TN)+53.673*EXP(0.448*TN)
     1 +148.53*EXP(3.200*TN)+1192.2*EXP(11.56*TN)+26.848*EXP(0.687*TN)
     2 +101.36*EXP(1.728*TN)+155.27*EXP(5.735*TN)
      J=J+1
      NAMJ(J)=4002
      CHIJ(J)=22.980
      PARTFJ(J)=5.+11.776*EXP(0.102*TN)+9.511*EXP(0.401*TN)
     1 +34.725*EXP(1.655*TN)+100.91*EXP(7.737*TN)+477.03*EXP(17.096*TN)
      J=J+1
      NAMJ(J)=4003
      CHIJ(J)=34.330
      PARTFJ(J)=10.+2.3714*EXP(4.962*TN)
C
C     BARIUM
      J=J+1
      NAMJ(J)=5600
      CHIJ(J)=5.210
      PARTFJ(J)=1.+24.664*EXP(1.236*TN)+18.848*EXP(2.641*TN)
     1 +413.5*EXP(4.584*TN)+69.369*EXP(2.939*TN)+443.62*EXP(5.461*TN)
      J=J+1
      NAMJ(J)=5601
      CHIJ(J)=10.001
      PARTFJ(J)=2.+9.8234*EXP(0.654*TN)+6.0628*EXP(2.523*TN)
     1 +83.389*EXP(6.575*TN)+270.72*EXP(9.413*TN)
      J=J+1
      NAMJ(J)=5602
      CHIJ(J)=37.001
      PARTFJ(J)=1.
      J=J+1
      NAMJ(J)=5603
      CHIJ(J)=61.975
      PARTFJ(J)=4.
C
C     RARE EARTHS:CHEBYSHEV APPROX.(VAN DIEST 1980)
      X=TEMP
      IF(X.GT.9000.)X=9000.
      IF(X.LT.1000.)X=1000.
      T=(X-5000.)/4000.
      AC=ACOS(T)
      C1=T
      C2=COS(2.*AC)
      C3=COS(3.*AC)
      C4=COS(4.*AC)
      C5=COS(5.*AC)
      C6=COS(6.*AC)
      C7=COS(7.*AC)
      C8=COS(8.*AC)
C
C     LANTHANUM
      J=J+1
      NAMJ(J)=5700
      CHIJ(J)=5.5770
      PARTFJ(J)=32.8190+32.6313*C1+5.9452*C2+0.6352*C3-0.0217*C4
     1 -0.0600*C5+0.0214*C6
      J=J+1
      NAMJ(J)=5701
      CHIJ(J)=11.060
      PARTFJ(J)=28.8005+19.1464*C1-0.7910*C2+0.6407*C3-0.1051*C4
     1 -0.0308*C5+0.0296*C6-0.0196*C7
      J=J+1
      NAMJ(J)=5702
      CHIJ(J)=19.1774
      PARTFJ(J)=8.9772+4.0607*C1-0.2753*C2+0.0128*C3-0.0229*C4
     1 +0.0126*C5
      J=J+1
      NAMJ(J)=5703
      CHIJ(J)=49.95
      PARTFJ(J)=1.
C
C     CERIUM
      J=J+1
      NAMJ(J)=5800
      CHIJ(J)=5.5387
      PARTFJ(J)=245.5966+283.8846*C1+55.4148*C2-0.7151*C3-0.8115*C4
     1 -0.2495*C5+0.1380*C6-0.0125*C7-0.0124*C8
      J=J+1
      NAMJ(J)=5801
      CHIJ(J)=10.85
      PARTFJ(J)=204.9558+215.0671*C1+16.2455*C2-6.5404*C3+1.5321*C4
     1 -0.1775*C5-0.0281*C6+0.0224*C7
      J=J+1
      NAMJ(J)=5802
      CHIJ(J)=20.198
      PARTFJ(J)=45.9857+37.3066*C1+0.0754*C2-1.2149*C3+0.4384*C4
     1 -0.1034*C5+0.0085*C6
      J=J+1
      NAMJ(J)=5803
      CHIJ(J)=36.758
      PARTFJ(J)=14.
C
C     NEODYMIUM
      J=J+1
      NAMJ(J)=6000
      CHIJ(J)=5.5250
      PARTFJ(J)=160.8704+207.4941*C1+64.9327*C2+3.9955*C3-2.6214*C4
     1 +0.1961*C5+0.1002*C6-0.0608*C7+0.0262*C8
      J=J+1
      NAMJ(J)=6001
      CHIJ(J)=10.73
      PARTFJ(J)=157.2243+179.3331*C1+39.8991*C2+0.7263*C3-1.1818*C4
     1 +0.1963*C5+0.0329*C6-0.0413*C7+0.0160*C8
      J=J+1
      NAMJ(J)=6002
      CHIJ(J)=22.1
      PARTFJ(J)=32.3431+19.5901*C1-0.8376*C2+0.4982*C3+0.0649*C4
     1 -0.1065*C5+0.0431*C6
      J=J+1
      NAMJ(J)=6003
      CHIJ(J)=40.4
      PARTFJ(J)=52.
C
C     SAMARIUM
      J=J+1
      NAMJ(J)=6200
      CHIJ(J)=5.6437
      PARTFJ(J)=58.2257+67.9619*C1+19.5360*C2+3.3723*C3-0.8480*C4
     1 -0.1256*C5+0.1056*C6-0.0328*C7
      J=J+1
      NAMJ(J)=6201
      CHIJ(J)=11.07
      PARTFJ(J)=68.3415+67.7946*C1+9.1011*C2+1.2232*C3-0.1001*C4
     1 -0.0800*C5+0.0392*C6-0.0238*C7+0.0127*C8
      J=J+1
      NAMJ(J)=6202
      CHIJ(J)=23.4
      PARTFJ(J)=23.9023+15.5772*C1-2.0164*C2+0.6092*C3+0.0224*C4
     1 -0.0637*C5+0.0322*C6-0.0136*C7
      J=J+1
      NAMJ(J)=6203
      CHIJ(J)=41.4
      PARTFJ(J)=66.
C
C     EUROPIUM
      J=J+1
      NAMJ(J)=6300
      CHIJ(J)=5.67045
      PARTFJ(J)=16.7153+13.5109*C1+5.9698*C2+1.1638*C3-0.0197*C4
      J=J+1
      NAMJ(J)=6301
      CHIJ(J)=11.241
      PARTFJ(J)=17.2214+8.6405*C1+1.3429*C2+0.1994*C3-0.0717*C4
     1 +0.0334*C5-0.0016*C6-0.0099*C7
      J=J+1
      NAMJ(J)=6302
      CHIJ(J)=24.92
      PARTFJ(J)=8.3114+0.5326*C1+0.3244*C2+0.1332*C3+0.0274*C4
      J=J+1
      NAMJ(J)=6303
      CHIJ(J)=42.7
      PARTFJ(J)=49.
C
C     POLYNOMIAL APPPROX.(IRWIN 1981)
      X=TEMP
      IF(X.GT.16000.)X=16000.
      IF(X.LT.1000.)X=1000.
      C1=ALOG(X)
      C2=C1*C1
      C3=C2*C1
      C4=C2*C2
      C5=C4*C1
C
C     TUNGSTEN
C     I.P. FROM ALLEN(1976)
      J=J+1
      NAMJ(J)=7400
      CHIJ(J)=7.98
      X=2042.66501-1280.61781*C1+317.708016*C2-39.0490363*C3
     1 +2.38059556*C4-5.7586891E-2*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7401
      CHIJ(J)=18.0
      X=13311.7880-7788.61299*C1+1815.23243*C2-210.756299*C3
     1 +12.1955751*C4-0.281435906*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7402
      CHIJ(J)=24.0
      X=10225.2650-5854.18868*C1+1333.98954*C2-151.328950*C3
     1 +8.55153740*C4-0.192642598*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7403
      CHIJ(J)=35.0
      PARTFJ(J)=1.
C
C     OSMIUM
C     I.P. FROM ALLEN(1976)
      J=J+1
      NAMJ(J)=7600
      CHIJ(J)=8.7
      X=-466.348695+240.124797*C1-47.7885296*C2+4.56354983*C3
     1 -0.205654128*C4+3.42399283E-3*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7601
      CHIJ(J)=17.
      X=1452.88057-870.843194*C1+208.235756*C2-24.8196634*C3
     1 +1.47535190*C4-3.49765722E-2*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7602
      CHIJ(J)=25.
      X=-3130.86355+1976.38153*C1-496.399961*C2+62.0305075*C3
     1 -3.85705885*C4+9.55107973E-2*C5
      PARTFJ(J)=EXP(X)
      J=J+1
      NAMJ(J)=7603
      CHIJ(J)=40.
      PARTFJ(J)=1.
C
C     THORIUM
C     ZALUBAS(1968),ZALUBAS&CORLISS(1974),KLINKENBERG(1950)
      J=J+1
      NAMJ(J)=9000
      CHIJ(J)=6.08
      PARTFJ(J)=5.+8.0*EXP(0.34*TN)+8.0*EXP(0.47*TN)+12.0*EXP(0.65*TN)
     1 +10.0*EXP(0.84*TN)+39.0*EXP(1.00*TN)+66.0*EXP(1.38*TN)
     2 +88.0*EXP(1.71*TN)+174.0*EXP(2.03*TN)+147.0*EXP(2.31*TN)
     3 +115.0*EXP(2.62*TN)+324.0*EXP(2.92*TN)+387.0*EXP(3.28*TN)
     4 +485.0*EXP(3.66*TN)+491.0*EXP(4.03*TN)+603.0*EXP(4.40*TN)
     5 +467.0*EXP(4.77*TN)+329.0*EXP(5.2*TN)
      J=J+1
      NAMJ(J)=9001
      CHIJ(J)=11.5
      PARTFJ(J)=4.+10.0*EXP(0.21*TN)+20.0*EXP(0.53*TN)+24.0*EXP(0.78*TN)
     1 +22.0*EXP(0.89*TN)+22.0*EXP(1.04*TN)+52.0*EXP(1.17*TN)
     2 +60.0*EXP(1.33*TN)+102.0*EXP(1.64*TN)+104.0*EXP(1.90*TN)
     3 +88.0*EXP(2.19*TN)+58.0*EXP(2.40*TN)+128.0*EXP(2.62*TN)
     4 +138.0*EXP(2.94*TN)+140.0*EXP(3.25*TN)+128.0*EXP(3.56*TN)
     5 +344.0*EXP(4.03*TN)+596.0*EXP(4.65*TN)+46.0*EXP(5.27*TN)
     6 +878.0*EXP(6.4*TN)
      J=J+1
      NAMJ(J)=9002
      CHIJ(J)=20.0
      PARTFJ(J)=14.+5.0*EXP(0.06*TN)+21.0*EXP(0.37*TN)+41.0*EXP(0.58*TN)
     1 +35.0*EXP(0.84*TN)+44.0*EXP(1.06*TN)+42.0*EXP(1.35*TN)
     2 +26.0*EXP(1.85*TN)+60.0*EXP(2.47*TN)+31.0*EXP(3.57*TN)
     3 +41.0*EXP(4.51*TN)+84.0*EXP(5.66*TN)+37.0*EXP(6.79*TN)
      J=J+1
      NAMJ(J)=9003
      CHIJ(J)=28.6
      PARTFJ(J)=14.0*EXP(0.27*TN)
C
      JATI=J
      RETURN
      END SUBROUTINE QATI2

C
C
C
      SUBROUTINE DCHI(THETA,PELE)
C     LOWERING OF IONISATION ENERGY
C
      USE iondis2_com
      USE iondis4_com
C
      Z=4.98E-4*THETA*SQRT(PELE)
      J1=JATI-3
      DO 1 J=1,J1,4
      CHIJ(J)=CHIJ(J)-Z
      CHIJ(J+1)=CHIJ(J+1)-2.*Z
      CHIJ(J+2)=CHIJ(J+2)-3.*Z
    1 CHIJ(J+3)=CHIJ(J+3)-4.*Z
      RETURN
      END SUBROUTINE DCHI

C
C
C
      SUBROUTINE QMOL(TEMP)
C     PARTITION FUNCTIONS:MOLECULES
C     DISSOCIATION CONSTANTS SAHAJ:P(A)*P(B)/P(AB)
C
      USE iondis2_com
      USE iondis3_com
      USE iondis4_com
      USE constants
C
      DIMENSION QH2(71),QCH(71),QC2(71),QCN(71),QCO(71)
C
C
      TH=5040./TEMP
      TN=-TH*ALN10
      T25=4.6968E13/TH/TH/SQRT(TH)
      J=JNEGI
C
C     H2 TATUM(1966) TEMP=1000(100)8000 K
      J=J+1
      NAMJ(J)=01000100
      WTJ(J)=2.016
      CHIJ(J)=15.43
      RM=0.504
      D0=4.477
      DATA(QH2(K),K=1,71)/5.870,6.470,7.080,7.700,8.320,8.960,9.610,
     1 1.028E1,1.096E1,1.166E1,1.238E1,1.311E1,1.386E1,1.463E1,1.542E1,
     2 1.623E1,1.707E1,1.792E1,1.879E1,1.968E1,2.060E1,2.153E1,2.249E1,
     3 2.347E1,2.448E1,2.550E1,2.655E1,2.762E1,2.872E1,2.984E1,3.099E1,
     4 3.216E1,3.335E1,3.457E1,3.582E1,3.709E1,3.838E1,3.971E1,4.106E1,
     5 4.243E1,4.384E1,4.527E1,4.673E1,4.822E1,4.974E1,5.128E1,5.286E1,
     6 5.447E1,5.611E1,5.778E1,5.948E1,6.122E1,6.299E1,6.479E1,6.663E1,
     7 6.850E1,7.041E1,7.236E1,7.434E1,7.636E1,7.842E1,8.052E1,8.266E1,
     8 8.483E1,8.705E1,8.932E1,9.162E1,9.397E1,9.636E1,9.880E1,1.013E2/
      I1=INT(TEMP/100.)
      I=I1-9
      IF(I.LT.1)X=QH2(1)
      IF(I.GT.70)X=QH2(71)
      IF(I.GE.1.AND.I.LE.70)
     1 X=QH2(I)+(QH2(I+1)-QH2(I))*(TEMP-I1*100.)/100.
      PARTFJ(J)=X
      J1=JIZ(1)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/X*EXP(D0*TN)
C
C     CH TATUM(1966) TEMP=1000(100)8000 K
      J=J+1
      NAMJ(J)=06000100
      WTJ(J)=13.019
      CHIJ(J)=10.64
      RM=0.930
      D0=3.47
      DATA(QCH(K),K=1,71)/2.001E2,2.222E2,2.449E2,2.685E2,2.930E2,
     1 3.184E2,3.447E2,3.721E2,4.004E2,4.298E2,4.602E2,4.917E2,5.243E2,
     2 5.581E2,5.929E2,6.288E2,6.659E2,7.042E2,7.437E2,7.843E2,8.261E2,
     3 8.962E2,9.135E2,9.590E2,1.006E3,1.054E3,1.103E3,1.154E3,1.206E3,
     4 1.260E3,1.314E3,1.371E3,1.428E3,1.487E3,1.548E3,1.610E3,1.674E3,
     5 1.739E3,1.806E3,1.875E3,1.945E3,2.017E3,2.091E3,2.166E3,2.244E3,
     6 2.323E3,2.405E3,2.488E3,2.573E3,2.661E3,2.750E3,2.842E3,2.936E3,
     7 3.032E3,3.131E3,3.232E3,3.335E3,3.441E3,3.550E3,3.661E3,3.774E3,
     8 3.891E3,4.010E3,4.132E3,4.265E3,4.384E3,4.515E3,4.648E3,4.785E3,
     9 4.924E3,5.067E3/
      I1=INT(TEMP/100.)
      I=I1-9
      IF(I.LT.1)X=QCH(1)
      IF(I.GT.70)X=QCH(71)
      IF(I.GE.1.AND.I.LE.70)
     1 X=QCH(I)+(QCH(I+1)-QCH(I))*(TEMP-I1*100.)/100.
      PARTFJ(J)=X
      J1=JIZ(6)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/X*EXP(D0*TN)
C
C     C2 TATUM(1966) TEMP=1000(100)8000 K
      J=J+1
      NAMJ(J)=06000600
      WTJ(J)=24.022
      CHIJ(J)=12.0
      RM=6.003
      D0=6.2
      DATA(QC2(K),K=1,71)/7.996E2,9.576E2,1.129E3,1.313E3,1.510E3,
     1 1.721E3,1.945E3,2.183E3,2.436E3,2.703E3,2.984E3,3.281E3,3.594E3,
     2 3.922E3,4.266E3,4.627E3,5.005E3,5.400E3,5.813E3,6.244E3,6.693E3,
     3 7.162E3,7.649E3,8.156E3,8.683E3,9.231E3,9.799E3,1.039E4,1.100E4,
     4 1.163E4,1.229E4,1.297E4,1.367E4,1.439E4,1.514E4,1.591E4,1.671E4,
     5 1.753E4,1.838E4,1.925E4,2.015E4,2.108E4,2.204E4,2.302E4,2.403E4,
     6 2.507E4,2.613E4,2.723E4,2.835E4,2.951E4,3.070E4,3.191E4,3.316E4,
     7 3.444E4,3.575E4,3.710E4,3.848E4,3.989E4,4.133E4,4.281E4,4.433E4,
     8 4.588E4,4.746E4,4.908E4,5.074E4,5.244E4,5.417E4,5.594E4,5.775E4,
     9 5.960E4,6.148E4/
      I1=INT(TEMP/100.)
      I=I1-9
      IF(I.LT.1)X=QC2(1)
      IF(I.GT.70)X=QC2(71)
      IF(I.GE.1.AND.I.LE.70)
     1 X=QC2(I)+(QC2(I+1)-QC2(I))*(TEMP-I1*100.)/100.
      PARTFJ(J)=X
      J1=JIZ(6)
      J2=JIZ(6)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/X*EXP(D0*TN)
C
C     CN TATUM(1966) TEMP=100(100)8000 K
      J=J+1
      NAMJ(J)=06000700
      WTJ(J)=26.018
      CHIJ(J)=14.0
      RM=6.465
      D0=7.8
      DATA(QCN(K),K=1,71)/7.713E2,8.621E2,9.571E2,1.057E3,1.161E3,
     1 1.270E3,1.384E3,1.503E3,1.628E3,1.758E3,1.894E3,2.037E3,2.187E3,
     2 2.343E3,2.506E3,2.677E3,2.856E3,3.043E3,3.238E3,3.442E3,3.655E3,
     3 3.877E3,4.110E3,4.352E3,4.602E3,4.868E3,5.142E3,5.427E3,5.723E3,
     4 6.032E3,6.352E3,6.685E3,7.030E3,7.387E3,7.758E3,8.142E3,8.540E3,
     5 8.951E3,9.376E3,9.815E3,1.027E4,1.074E4,1.122E4,1.172E4,1.223E4,
     6 1.276E4,1.330E4,1.386E4,1.444E4,1.503E4,1.564E4,1.626E4,1.690E4,
     7 1.756E4,1.824E4,1.893E4,1.964E4,2.036E4,2.111E4,2.187E4,2.265E4,
     8 2.345E4,2.427E4,2.511E4,2.596E4,2.684E4,2.773E4,2.864E4,2.958E4,
     9 3.053E4,3.150E4/
      I1=INT(TEMP/100.)
      I=I1-9
      IF(I.LT.1)X=QCN(1)
      IF(I.GT.70)X=QCN(71)
      IF(I.GE.1.AND.I.LE.70)
     1 X=QCN(I)+(QCN(I+1)-QCN(I))*(TEMP-I1*100.)/100.
      PARTFJ(J)=X
      J1=JIZ(6)
      J2=JIZ(7)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/X*EXP(D0*TN)
C
C     CO TATUM(1966) TEMP=1000(100)8000 K
      J=J+1
      NAMJ(J)=06000800
      WTJ(J)=28.010
      CHIJ(J)=14.01
      RM=6.859
      D0=11.09
      DATA(QCO(K),K=1,71)/3.791E2,4.237E2,4.703E2,5.190E2,5.700E2,
     1 6.231E2,6.786E2,7.363E2,7.964E2,8.589E2,9.237E2,9.909E2,1.061E3,
     2 1.133E3,1.207E3,1.284E3,1.363E3,1.445E3,1.529E3,1.616E3,1.705E3,
     3 1.797E3,1.891E3,1.987E3,2.087E3,2.188E3,2.292E3,2.399E3,2.508E3,
     4 2.620E3,2.734E3,2.851E3,2.970E3,3.092E3,3.217E3,3.344E3,3.474E3,
     5 3.606E3,3.741E3,3.878E3,4.018E3,4.161E3,4.306E3,4.454E3,
     6 4.605E3,4.758E3,4.914E3,5.072E3,5.233E3,5.397E3,5.564E3,5.733E3,
     7 5.905E3,6.080E3,6.257E3,6.437E3,6.620E3,6.805E3,6.994E3,7.185E3,
     8 7.378E3,7.575E3,7.774E3,7.976E3,8.181E3,8.389E3,8.600E3,8.813E3,
     9 9.029E3,9.248E3,9.470E3/
      I1=INT(TEMP/100.)
      I=I1-9
      IF(I.LT.1)X=QCO(1)
      IF(I.GT.70)X=QCO(71)
      IF(I.GE.1.AND.I.LE.70)
     1 X=QCO(I)+(QCO(I+1)-QCO(I))*(TEMP-I1*100.)/100.
      PARTFJ(J)=X
      J1=JIZ(6)
      J2=JIZ(8)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/X*EXP(D0*TN)
C
C     MGH
C     DISSOCIATION CONSTANT:TSUJI(1973)
C     MODIFIED TO ACCOUNT FOR REVISED DISSOCIATION ENERGY D0
C     D0=1.33 EV (BALFOUR&CARTWRIGHT 1976)
      J=J+1
      NAMJ(J)=12000100
      WTJ(J)=25.313
      CHIJ(J)=7.6
      RM=0.968
      PARTFJ(J)=1.
      D0OLD=2.00
      D0=1.33
      X=TH
      IF(X.GT.5.04)X=5.04
      IF(X.LT.0.63)X=0.63
      Y=X*X
      S=11.285-2.7164*X+0.19658*Y-0.027310*X*Y+0.0013816*Y*Y
      Y=S-(D0-D0OLD)*X
      SAHAJ(J)=EXP(Y*ALN10)
C
      JMOL2=J
      RETURN
      END SUBROUTINE QMOL
C
C
C
      SUBROUTINE DECOM2(NAME,JA,JB)
C     INDEX JA,JB OF COMPONENTS OF NEUTRAL MOLECULE NAME
C
      USE iondis2_com
C
      II=NAME/1000000
      JA=JIZ(II)
      JB=JIZ(MOD(NAME,1000000)/100)
      RETURN
      END SUBROUTINE DECOM2
C
C
C
      FUNCTION NCHARG(NAME)
C     CHARGE OF ATOM,ION,MOLECULE NAME
      J1=IABS(NAME)
      J1=MOD(J1,100)
      NCHARG=ISIGN(J1,NAME)
      RETURN
      END  FUNCTION NCHARG


      SUBROUTINE OPALAM(ALAM,TEMP,PELE,DENSNC,MODE,OKAPPA,OSIGMA)
C
      USE iondis2_com
      USE iondis4_com
      USE op1_com
      USE op2_com
      USE constants
C
      HNU=HPLANC*CLIGHT/ALAM/1.E-8
      AKT=BOLTZK*TEMP
      ALAM3=ALAM*ALAM*ALAM
      X=EXP(-HNU/AKT)
      STIMEM=1.-X
      BLAM=2.*HPLANC*CLIGHT*CLIGHT/(1./X-1.)*1.E20/ALAM3*1.E20/ALAM/ALAM
      DENSEL=PELE/AKT
      ATHOMS=6.65246E-25
      RYD=2.17992E-11
      ERGEV=1.602192E-12
      K=0
C
C     SCATTERING:K=1...KSCAT
C     THOMSON SCATTERING OF FREE ELECTRONS(REST FRAME)
      K=K+1
      NAMK(K)=-1
      OPAC(K)=DENSEL/DENSNC
C
C     RAYLEIGH SCATTERING OF HI,HEI,H2
C     HYDROGEN
      K=K+1
      J=JIZ(1)
      G=2.
      NAMK(K)=NAMJ(J)
      OPAC(K)=ARAYL(NAMJ(J),ALAM)*G*ZETAJ(J)
C     HELIUM
      K=K+1
      J=JIZ(2)
      G=1.
      NAMK(K)=NAMJ(J)
      OPAC(K)=ARAYL(NAMJ(J),ALAM)*G*ZETAJ(J)
C     MOLECULAR HYDROGEN
      K=K+1
      J1=JNEGI+1
      DO 10 J=J1,JMOL2
      IF(NAMJ(J).EQ.1000100)J2=J
   10 CONTINUE
      NAMK(K)=NAMJ(J2)
      OPAC(K)=ARAYL(NAMJ(J2),ALAM)*FRACJ(J2)
C
      KSCAT=K
C
C     TRUE ABSORPTION
C
C     FREE-FREE ABSORPTION:HYDROGEN
      K=K+1
      J=JIZ(1)
      NAMK(K)=NAMJ(J)
      OPAC(K)=AHYLFF(1.,ALAM,TEMP)*STIMEM*DENSEL*FRACJ(J+1)
C
C     FREE-FREE ABSORPTION:HELIUM THROUGH NICKEL
C     ASSUMING FOUR STAGES PER ELEMENT:I,II,III,IV
      K=K+1
      A1=AHYLFF(1.,ALAM,TEMP)
      A2=AHYLFF(2.,ALAM,TEMP)
      A3=AHYLFF(3.,ALAM,TEMP)
      X=0.
      J1=JIZ(2)
      J2=JIZ(28)
      DO 20 J=J1,J2,4
   20 X=X+A1*FRACJ(J+1)+A2*FRACJ(J+2)+A3*FRACJ(J+3)
      NAMK(K)=NAMJ(J2)
      OPAC(K)=X*STIMEM*DENSEL
C
C     FREE-FREE ABSORPTION:NEGATIVE HYDROGEN ION
      K=K+1
      J=JIZ(1)
      NAMK(K)=-101
      OPAC(K)=AHMIFF(ALAM,TEMP)*2.*PELE*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:HYDROGEN
      K=K+1
      J=JIZ(1)
      EI=RYD
C     HYDROGEN:N=1...14
      X=0.
      DO 60 N=1,14
      A2=FLOAT(N*N)
      EN=EI*(1.-1./A2)
      HNUN=EI-EN
      N1=N
      IF(HNU.GE.HNUN)X=X+AHYLBF(1.,ALAM3,N1)*A2*EXP(-EN/AKT)
   60 CONTINUE
      NAMK(K)=NAMJ(J)
      OPAC(K)=X*STIMEM*2.*ZETAJ(J)
C     HYDROGEN:N=15
      K=K+1
      NAMK(K)=NAMJ(J)+15
      EN=EI*(1.-1./225.)
      HNUN=EI-EN
      OPAC(K)=0.
      IF(HNU.GE.HNUN)OPAC(K)=
     1 AHYLBF(1.,ALAM3,15)*225.*EXP(-EN/AKT)*STIMEM*2.*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:NEGATIVE HYDROGEN ION
      K=K+1
      J1=JATI+1
      DO 70 J=J1,JNEGI
      IF(NAMJ(J).EQ.-101)J2=J
   70 CONTINUE
      NAMK(K)=NAMJ(J2)
      OPAC(K)=AHMIBF(ALAM)*STIMEM*FRACJ(J2)
C
C     HYDROGEN QUASI-MOLECULE H2
      K=K+1
      NAMK(K)=1000100
      J=JIZ(1)
      OPAC(K)=AH2Q(ALAM,TEMP)*DENSNC*STIMEM*FRACJ(J)*FRACJ(J)
C
C     HYDROGEN MOLECULAR ION H2+
      K=K+1
      NAMK(K)=1000101
      J=JIZ(1)
      OPAC(K)=AH2P(ALAM,TEMP)*DENSNC*FRACJ(J+1)*2.*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:HELIUM I
      K=K+1
      J=JIZ(2)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 75 M=1,9
      I=M
      CALL AHE1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
   75 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:HELIUM II
      K=K+1
      J=JIZ(2)+1
      EI=RYD*4.
      X=0.
      DO 65 N=1,15
      A2=FLOAT(N*N)
      EN=EI*(1.-1./A2)
      HNUN=EI-EN
      N1=N
      IF(HNU.GE.HNUN)X=X+AHYLBF(16.,ALAM3,N1)*A2*EXP(-EN/AKT)
   65 CONTINUE
      NAMK(K)=NAMJ(J)
      OPAC(K)=X*STIMEM*2.*ZETAJ(J)
C
C     FREE-FREE ABSORPTION:NEGATIVE HELIUM ION
      K=K+1
      J=JIZ(2)
      NAMK(K)=-201
      OPAC(K)=AHEMFF(ALAM,TEMP)*ZETAJ(J)*PELE
C
C     BOUND-FREE ABSORPTION:NEGATIVE CARBON ION
      K=K+1
      J1=JATI+1
      DO 80 J=J1,JNEGI
      IF(NAMJ(J).EQ.-601)J2=J
   80 CONTINUE
      NAMK(K)=NAMJ(J2)
      OPAC(K)=ACMI(ALAM)*STIMEM*FRACJ(J2)
C
C     BOUND-FREE ABSORPTION:CARBON I
      K=K+1
      J=JIZ(6)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 90 M=1,13
      I=M
      CALL AC1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
   90 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:NITROGEN I
      K=K+1
      J=JIZ(7)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 100 M=1,13
      I=M
      CALL AN1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  100 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:OXYGEN I
      K=K+1
      J=JIZ(8)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 110 M=1,12
      I=M
      CALL AO1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  110 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:NEON I
      K=K+1
      J=JIZ(10)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 120 M=1,1
      I=M
      CALL ANE1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  120 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:SODIUM I
      K=K+1
      J=JIZ(11)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 130 M=1,5
      I=M
      CALL ANA1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  130 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:MAGNESIUM I
      K=K+1
      J=JIZ(12)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 140 M=1,5
      I=M
      CALL AMG1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  140 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:MAGNESIUM II
      K=K+1
      J=JIZ(12)+1
      NAMK(K)=NAMJ(J)
      X=0.
      DO 150 M=1,2
      I=M
      CALL AMG2BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  150 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:ALUMINIUM I
      K=K+1
      J=JIZ(13)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 160 M=1,2
      I=M
      CALL AAL1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  160 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:SILICON I
      K=K+1
      J=JIZ(14)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 170 M=1,5
      I=M
      CALL ASI1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  170 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:SILICON II
      K=K+1
      J=JIZ(14)+1
      NAMK(K)=NAMJ(J)
      X=0.
      DO 180 M=1,1
      I=M
      CALL ASI2BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  180 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:CALCIUM I
      K=K+1
      J=JIZ(20)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 190 M=1,3
      I=M
      CALL ACA1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  190 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:CALCIUM II
      K=K+1
      J=JIZ(20)+1
      NAMK(K)=NAMJ(J)
      X=0.
      DO 200 M=1,3
      I=M
      CALL ACA2BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  200 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C     BOUND-FREE ABSORPTION:IRON I
      K=K+1
      J=JIZ(26)
      NAMK(K)=NAMJ(J)
      X=0.
      DO 210 M=1,1
      I=M
      CALL AFE1BF(ALAM,I,ALAMM,E,G,A)
      IF(ALAM.LE.ALAMM)X=X+A*G*EXP(-E*ERGEV/AKT)
  210 CONTINUE
      OPAC(K)=X*STIMEM*ZETAJ(J)
C
C
      KABS=K
      OSIGMA=0.
      DO 40 K=1,KSCAT
   40 OSIGMA=OSIGMA+OPAC(K)
      OSIGMA=OSIGMA*ATHOMS
      J1=KSCAT+1
      OKAPPA=0.
      DO 42 K=J1,KABS
   42 OKAPPA=OKAPPA+OPAC(K)
      OKAPPA=OKAPPA*ATHOMS
C
C
C     PRINTOUT IF MODE=2
      IF(MODE.NE.2)GO TO 50
 9998 FORMAT(///// '*    LAMBDA', 6X, 'TEMP', 4X, 'LOG PE', 6X,
     >       'DENSNC', 6X, 'STIMEM', 6X, 'BLAM')
 9997 FORMAT('*', F10.1, F10.0, F10.3, 1P, 3E12.3)
 9996 FORMAT('*    K     NAMK    L OPAC    L FRAC')
 9995 FORMAT('*SCATTERING:')
 9994 FORMAT('*TRUE ABSORPTION:')
 9993 FORMAT('*',I5,I9,2F10.3)
      WRITE(3,9998)
      A1=ALOG10(PELE)
      WRITE(3,9997)ALAM,TEMP,A1,DENSNC,STIMEM,BLAM
      WRITE(3,9996)
      WRITE(3,9995)
      A1=(OSIGMA+OKAPPA)/ATHOMS
      DO 9901 K=1,KSCAT
      Y=OPAC(K)
      A2=AMAX1(-37.,ALOG10(Y))
      A3=AMAX1(-37.,ALOG10(Y/A1))
 9901 WRITE(3,9993)K,NAMK(K),A2,A3
      J1=KSCAT+1
      WRITE(3,9994)
      DO 9902 K=J1,KABS
      Y=OPAC(K)
      A2=AMAX1(-37.,ALOG10(Y))
      A3=AMAX1(-37.,ALOG10(Y/A1))
 9902 WRITE(3,9993)K,NAMK(K),A2,A3
 9992 FORMAT(/ '*       SIGMA', 7X, 'KAPPA', 3X, 'LOG (S+K)', 1X,
     >       'LOG SCATFRAC')
 9991 FORMAT('*', 1P, 2E12.4, 0P, 2F10.3)
      A=OSIGMA+OKAPPA
      A1=ALOG10(A)
      A2=ALOG10(OSIGMA/A)
      WRITE(3,9992)
      WRITE(3,9991)OSIGMA,OKAPPA,A1,A2
   50 CONTINUE
      RETURN
      END SUBROUTINE OPALAM
C
C
C
      FUNCTION ARAYL(NAMJ,ALAM)
C     RAYLEIGH SCATTERING OF HI,HEI,H2
C     DALGARNO&KINGSTON(1960),DALGARNO&WILLIAMS(1962)
C     BASCHEK&SCHOLZ(1980)
      ARAYL=1.E-35
      IF(NAMJ.EQ.100)GO TO 1
      IF(NAMJ.EQ.200)GO TO 2
      IF(NAMJ.EQ.1000100)GO TO 3
      GO TO 5
    1 A=1.
      A0=966./ALAM
      IF(A0.GE.1.)GO TO 5
      A2=1566./ALAM
      A4=1480./ALAM
      GO TO 4
    2 A=4.
      A0=379./ALAM
      IF(A0.GE.1.)GO TO 5
      A2=669./ALAM
      A4=641./ALAM
      GO TO 4
    3 A=4.
      A0=744./ALAM
      IF(A0.GE.1.)GO TO 5
      A2=1254./ALAM
      A4=1186./ALAM
    4 ARAYL=A*A0*A0*A0*A0*(1.+A2*A2+A4*A4*A4*A4)
    5 CONTINUE
      RETURN
      END FUNCTION ARAYL
C
C
      FUNCTION AHYLFF(ZION,ALAM,TEMP)
C     FREE-FREE ABSORPTION:HYDROGENIC ION OF CHARGE ZION
C
      USE op1_com
C
      AHYLFF=1.370E-20/ATHOMS*1.E-27*ZION*ZION*ALAM*ALAM*ALAM/SQRT(TEMP)
     1 *GAUHFF(ZION,ALAM,TEMP)
      RETURN
      END FUNCTION AHYLFF
C
C
C
      FUNCTION GAUHFF(ZION,ALAM,TEMP)
C     GAUNT FACTOR:HYDROGENIC ION OF CHARGE ZION
C     KARZAS&LATTER(1961),NO EXTRAPOLATION
C
      USE op1_com
C
      DIMENSION ALU(12),ALG2(5),G(12,5)
      AALU=ALOG10(HNU/AKT)
      IF(AALU.LT.-4.0)AALU=-4.0
      IF(AALU.GT.+1.5)AALU=+1.5
      AALG2=ALOG10(ZION*ZION*RYD/AKT)
      IF(AALG2.LT.-3.0)AALG2=-3.0
      IF(AALG2.GT.+2.0)AALG2=+2.0
      DATA(ALU(LU),LU=1,12)/-4.0,-3.5,-3.0,-2.5,-2.0,-1.5,-1.0,-0.5,0.0,
     1                      +0.5,+1.0,+1.5/
      DATA(ALG2(LG2),LG2=1,5)/2.,1.,0.,-1.,-3./
      DATA(G(LU,1),LU=1,12)/3.8,3.2,2.7,2.2,1.8,1.5,1.3,1.2,1.1,
     1                      1.08,1.08,1.08/
      DATA(G(LU,2),LU=1,12)/4.5,3.8,3.2,2.7,2.2,1.8,1.5,1.3,1.2,
     1                      1.16,1.13,1.08/
      DATA(G(LU,3),LU=1,12)/5.0,4.4,3.8,3.2,2.6,2.2,1.8,1.5,1.3,
     1                      1.16,0.98,0.74/
      DATA(G(LU,4),LU=1,12)/5.4,4.8,4.2,3.5,2.9,2.4,1.9,1.4,1.2,
     1                      0.86,0.59,0.38/
      DATA(G(LU,5),LU=1,12)/5.5,4.9,4.3,3.6,3.0,2.4,1.9,1.3,0.9,
     1                      0.54,0.32,0.19/
      DO 1 L=1,4
      IF(AALG2.LE.ALG2(L).AND.AALG2.GE.ALG2(L+1))LG2=L
    1 CONTINUE
      DO 2 L=1,11
      IF(AALU.GE.ALU(L).AND.AALU.LE.ALU(L+1))LU=L
    2 CONTINUE
      Y=(AALG2-ALG2(LG2))/(ALG2(LG2+1)-ALG2(LG2))
      GLEFT=G(LU,LG2)+(G(LU,LG2+1)-G(LU,LG2))*Y
      GRIGHT=G(LU+1,LG2)+(G(LU+1,LG2+1)-G(LU+1,LG2))*Y
      GAUHFF=GLEFT+(GRIGHT-GLEFT)*(AALU-ALU(LU))/(ALU(LU+1)-ALU(LU))
      RETURN
      END FUNCTION GAUHFF
C
C
C
      FUNCTION AHMIFF(ALAM,TEMP)
C     FREE-FREE ABSORPTION:NEGATIVE HYDROGEN ION
C     STILLEY&CALLAWAY(1970):DIPOLE-LENGTH;NO EXTRAPOLATION
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C   H- frei-frei AHMIFF: Extrapolationsfehler lambda > 91126.7 A behoben
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C
      USE op1_com
C
      DIMENSION DELK2(16),THETA(16),AHM(16,16)
      DATA(DELK2(M),M=1,16)/0.01,0.02,0.03,0.04,0.05,0.06,0.07,0.08,
     1 0.09,0.10,0.12,0.14,0.16,0.18,0.20,0.30/
      DATA(THETA(N),N=1,16)/0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,1.3,1.4,
     1 1.5,1.6,1.7,1.8,1.9,2.0/
      DATA(AHM(M,1),M=1,16)/27.5,6.92,3.10,1.77,1.14,.798,.592,.458,
     1 .305,.298,.210,.156,.121,.0959,.0780,.0344/
      DATA(AHM(M,2),M=1,16)/32.9,8.27,3.71,2.11,1.36,.958,.711,.550,
     1 .439,.359,.253,.188,.145,.116,.0941,.0418/
      DATA(AHM(M,3),M=1,16)/38.0,9.56,4.29,2.44,1.58,1.11,.824,.637,
     1 .509,.416,.293,.218,.169,.135,.110,.0491/
      DATA(AHM(M,4),M=1,16)/42.8,10.8,4.84,2.75,1.78,1.25,.931,.721,
     1 .575,.470,.332,.247,.192,.153,.125,.0565/
      DATA(AHM(M,5),M=1,16)/47.5,11.9,5.37,3.05,1.98,1.39,1.03,.800,
     1 .639,.522,.369,.276,.214,.172,.140,.0639/
      DATA(AHM(M,6),M=1,16)/51.9,13.1,5.87,3.34,2.17,1.52,1.13,.876,
     1 .700,.573,.406,.303,.236,.190,.156,.0713/
      DATA(AHM(M,7),M=1,16)/56.2,14.2,6.36,3.62,2.34,1.65,1.23,.949,
     1 .758,.621,.441,.331,.258,.208,.171,.0787/
      DATA(AHM(M,8),M=1,16)/60.4,15.2,6.83,3.89,2.52,1.77,1.32,1.02,
     1 .815,.668,.475,.357,.280,.225,.186,.0862/
      DATA(AHM(M,9),M=1,16)/64.5,16.2,7.28,4.14,2.68,1.89,1.40,1.09,
     1 .871,.715,.509,.384,.301,.243,.201,.0936/
      DATA(AHM(M,10),M=1,16)/68.4,17.2,7.72,4.39,2.84,2.00,1.49,1.15,
     1 .925,.760,.543,.410,.322,.261,.216,.101/
      DATA(AHM(M,11),M=1,16)/72.3,18.2,8.14,4.63,3.00,2.11,1.57,1.22,
     1 .977,.804,.576,.436,.343,.278,.231,.108/
      DATA(AHM(M,12),M=1,16)/76.0,19.1,8.55,4.86,3.15,2.21,1.65,1.28,
     1 1.03,.847,.608,.462,.364,.296,.245,.116/
      DATA(AHM(M,13),M=1,16)/79.7,20.0,8.95,5.08,3.29,2.32,1.73,1.34,
     1 1.08,.890,.640,.487,.385,.313,.260,.123/
      DATA(AHM(M,14),M=1,16)/83.2,20.9,9.33,5.30,3.43,2.42,1.80,1.40,
     1 1.13,.932,.672,.512,.406,.330,.275,.130/
      DATA(AHM(M,15),M=1,16)/86.7,21.7,9.71,5.51,3.57,2.51,1.88,1.46,
     1 1.18,.973,.703,.537,.426,.347,.289,.138/
      DATA(AHM(M,16),M=1,16)/90.1,22.6,10.1,5.71,3.70,2.61,1.95,1.52,
     1 1.23,1.02,.734,.562,.446,.364,.303,.145/
      DK2=911.267/ALAM
      THE=5040./TEMP
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C     IF(DK2.LT.0.01)DK2=0.01       inaktivieren, stattdessen m=1 setzen (s.u.)
C 	              d.h. Extrapolation gegen dk2 = 0 (alam > 91127 A) erlaubt
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      IF(DK2.GT.0.30)DK2=0.30
      IF(THE.LT.0.50)THE=0.50
      IF(THE.GT.2.00)THE=2.00
      DO 1 L=1,15
      IF(DK2.GE.DELK2(L).AND.DK2.LE.DELK2(L+1))M=L
    1 CONTINUE
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C            am Tabellenrand dk2 = 0.01 Extrapolation vom letzten Intervall aus
      if (dk2.lt.0.01) m=1
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      DO 2 L=1,15
      IF(THE.GE.THETA(L).AND.THE.LE.THETA(L+1))N=L
    2 CONTINUE
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C     X=ALAM*ALAM						  ersetzt durch
      z = alam
      if (dk2.lt.0.01) z = 91126.7
      x = z * z
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      A11=AHM(M,N)/X
      A12=AHM(M,N+1)/X
      A21=AHM(M+1,N)/X
      A22=AHM(M+1,N+1)/X
      Y=(DK2-DELK2(M))/(DELK2(M+1)-DELK2(M))
      ALEFT=A11+(A21-A11)*Y
      ARIGHT=A12+(A22-A12)*Y
      A=ALEFT+(ARIGHT-ALEFT)*(THE-THETA(N))/(THETA(N+1)-THETA(N))
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C     AHMIFF=A*X*1.E-26/ATHOMS					  ersetzt durch
      ahmiff = a * alam * alam * 1e-26 / athoms
C%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      RETURN
      END FUNCTION AHMIFF
C
C
C
      FUNCTION AHYLBF(ZION4,ALAM3,N)
C     BOUND-FREE ABSORPTION:HYDROGENIC ION
C     ZION4=CHARGE**4,ALAM3=WAVELENGTH**3,LEVEL N
C
      USE op1_com
C
      A=FLOAT(N)
      AHYLBF=1.046E-26/ATHOMS*ZION4*ALAM3/A/A/A/A/A*GAUHBF(N)
      RETURN
      END FUNCTION AHYLBF
C
C
C
      FUNCTION GAUHBF(N)
C     GAUNT FACTOR:HYDROGENIC ION,LEVEL N
C     KARZAS&LATTER(1961)
C     ASSUMIN FREQUENCY-INDEPENDENT G=G AT THRESHOLD
      DIMENSION G(15)
      DATA(G(M),M=1,15)/0.7973,0.8761,0.9075,0.9247,0.9358,0.9436,
     1 0.9494,0.9540,0.9576,0.9606,0.9632,0.9653,0.9672,0.9689,0.9703/
      GAUHBF=G(N)
      RETURN
      END FUNCTION GAUHBF
C
C
C
      FUNCTION AHMIBF(ALAM)
C     BOUND-FREE ABSORPTION:NEGATIVE HYDROGEN ION
C     WISHART(1979)
C
      USE op1_com
C
      DIMENSION ALU(65),AHM(65)
      DATA(ALU(L),L=1,65)/16444.,16300.,16200.,16100.,16000.,
     1 15750.,15500.,15250.,15000.,14750.,14500.,14250.,14000.,
     2 13750.,13500.,13250.,13000.,12750.,12500.,12250.,12000.,
     3 11750.,11500.,11250.,11000.,10750.,10500.,10250.,10000.,
     4 9750.,9500.,9250.,9000.,8750.,8500.,8250.,8000.,
     5 7750.,7500.,7250.,7000.,6750.,6500.,6250.,6000.,
     6 5750.,5500.,5250.,5000.,4750.,4500.,4250.,4000.,
     7 3750.,3500.,3250.,3000.,2750.,2500.,2250.,2000.,
     8 1750.,1500.,1250.,0./
      DATA(AHM(L),L=1,65)/.0000,.1989,.4974,.8697,1.302,
     1 2.575,4.052,5.677,7.407,9.211,11.07,12.95,14.85,
     2 16.74,18.62,20.46,22.26,24.02,25.71,27.33,28.87,
     3 30.34,31.72,33.01,34.19,35.28,36.25,37.13,37.89,
     4 38.53,39.06,39.48,39.77,39.95,40.01,39.95,39.77,
     5 39.48,39.07,38.54,37.91,37.17,36.32,35.37,34.32,
     6 33.17,31.94,30.62,29.23,27.77,26.24,24.65,23.02,
     7 21.35,19.65,17.92,16.19,14.46,12.75,11.08,9.453,
     8 7.918,6.512,5.431,.0/
      AHMIBF=0.
      IF(ALAM.GT.16444.)GO TO 1
      DO 2 M=1,64
      IF(ALAM.LE.ALU(M).AND.ALAM.GE.ALU(M+1))L=M
    2 CONTINUE
      X=AHM(L)+(AHM(L+1)-AHM(L))*(ALAM-ALU(L))/(ALU(L+1)-ALU(L))
      AHMIBF=X*1.E-18/ATHOMS
    1 CONTINUE
      RETURN
      END FUNCTION AHMIBF
C
C
C
      FUNCTION AHEMFF(ALAM,TEMP)
C     FREE-FREE ABSORPTION:NEGATIVE HELIUM ION
C     JOHN(1968),CARBON,GINGERICH&LATHAM(1969)
C
      USE op1_com
C
      Y=5040./TEMP
      X=2.46E-4+(-1.26E-7+5.67E-10*ALAM)*ALAM
     1  +(-5.92E-4+(5.12E-7+1.30E-09*ALAM)*ALAM)*Y
     2  +(+1.45E-2-(8.30E-7+1.80E-10*ALAM)*ALAM)*Y*Y
      AHEMFF=X*1.E-26/ATHOMS
      RETURN
      END FUNCTION AHEMFF
C
C
C
      FUNCTION AH2Q(ALAM,TEMP)
C     CONTINUOUS ABSORPTION:HYDROGEN QUASI-MOLECULE
C     DOYLE(1968)
C
      USE op1_com
C
      DIMENSION AL(9),AT(7),AMN(9,7)
      DATA(AL(M),M=1,9)/1540.,1750.,2000.,2222.,2500.,
     1 3000.,3640.,4500.,6667./
      DATA(AT(N),N=1,7)/2500.,3000.,4000.,5000.,6000.,8000.,10000./
      DATA(AMN(M,1),M=1,9)/2.600E-2,3.270E-3,2.060E-4,2.250E-5,1.882E-6,
     1 5.060E-8,1.500E-9,3.990E-11,1.380E-13/
      DATA(AMN(M,2),M=1,9)/3.440E-2,6.230E-3,5.930E-4,9.100E-5,1.128E-5,
     1 5.500E-7,2.890E-8,1.330E-9,9.790E-12/
      DATA(AMN(M,3),M=1,9)/4.750E-2,1.388E-2,2.220E-3,5.230E-4,1.055E-4,
     1 1.070E-5,1.154E-6,1.061E-7,2.100E-9/
      DATA(AMN(M,4),M=1,9)/5.630E-2,2.210E-2,4.900E-3,1.490E-3,4.020E-4,
     1 6.300E-5,1.041E-5,1.470E-6,5.400E-8/
      DATA(AMN(M,5),M=1,9)/6.160E-2,2.950E-2,8.190E-3,2.960E-3,9.690E-4,
     1 2.040E-4,4.450E-5,8.400E-6,4.710E-7/
      DATA(AMN(M,6),M=1,9)/6.500E-2,4.050E-2,1.510E-2,6.770E-3,2.830E-3,
     1 8.600E-4,2.660E-4,7.240E-5,7.000E-6/
      DATA(AMN(M,7),M=1,9)/6.490E-2,4.690E-2,2.080E-2,1.066E-2,5.180E-3,
     1 1.970E-3,7.520E-4,2.550E-4,3.420E-5/
      XL=ALAM
      XT=TEMP
      AH2Q=0.
      IF(XL.LT.AL(1))XL=AL(1)
      IF(XL.GT.AL(9))GO TO 1
      IF(XT.LT.AT(1))XT=AT(1)
      IF(XT.GT.AT(7))XT=AT(7)
      DO 2 L=1,8
      IF(XL.GE.AL(L).AND.XL.LE.AL(L+1))M=L
    2 CONTINUE
      DO 3 L=1,6
      IF(XT.GE.AT(L).AND.XT.LE.AT(L+1))N=L
    3 CONTINUE
      A11=ALOG(AMN(M,N))
      A12=ALOG(AMN(M,N+1))
      A21=ALOG(AMN(M+1,N))
      A22=ALOG(AMN(M+1,N+1))
      Y=(1./XL-1./AL(M))/(1./AL(M+1)-1./AL(M))
      ALEFT=A11+(A21-A11)*Y
      ARIGHT=A12+(A22-A12)*Y
      A=ALEFT+(ARIGHT-ALEFT)*(XT-AT(N))/(AT(N+1)-AT(N))
      AH2Q=EXP(A)/ATHOMS*1.E-20*1.E-20
    1 CONTINUE
      RETURN
      END FUNCTION AH2Q
C
C
C
      FUNCTION AH2P(ALAM,TEMP)
C     CONTINUOUS ABSORPTION:HYDROGEN MOLECULAR ION H2+
C     BOGGESS(1959);ALAM>10000 EXTRAPOLATED
C
      USE op1_com
      USE constants
C
      DIMENSION AL(14),AA(14),AB(14)
      DATA(AL(M),M=1,14)/400.,500.,1000.,1500.,2000.,2500.,3000.,
     1 4000.,5000.,6000.,7000.,8000.,9000.,10000./
      DATA(AA(M),M=1,14)/-19.00,-19.14,-19.69,-20.20,-20.59,-20.91,
     1 -21.18,-21.63,-21.98,-22.29,-22.54,-22.77,-22.97,-23.16/
      DATA(AB(M),M=1,14)/200000.,134600.,48420.,28770.,20040.,15220.,
     1 12220.,8650.,6650.,5380.,4500.,3860.,3380.,3000./
      AH2P=0.
      IF(ALAM.LT.400.)GO TO 1
      IF(ALAM.LE.10000.)GO TO 2
      C=-23.16-4.0*ALOG10(ALAM/10000.)
      D=3000.*(ALAM/10000.)**(-1.15)
      GO TO 3
    2 CONTINUE
      DO 4 L=1,13
      IF(ALAM.GE.AL(L).AND.ALAM.LE.AL(L+1))M=L
    4 CONTINUE
      XL=ALOG(ALAM)
      A1=ALOG(AL(M))
      A2=ALOG(AL(M+1))
      Y=(XL-A1)/(A2-A1)
      C=AA(M)+(AA(M+1)-AA(M))*Y
      A1=ALOG(AB(M))
      A2=ALOG(AB(M+1))
      X=A1+(A2-A1)*Y
      D=EXP(X)
    3 CONTINUE
      F=EXP(ALN10*(C-D/TEMP))
      AH2P=F/ATHOMS/12.5663706/BLAM
    1 CONTINUE
      RETURN
      END FUNCTION AH2P
C
C
C
      FUNCTION ACMI(ALAM)
C     BOUND-FREE ABSORPTION:NEGATIVE CARBON ION
C     SEMAN&BRANSCOMB(1962);ALAM<4000A EXTRAPOLATED
C
      USE op1_com
C
      DIMENSION AL(9),AA(9)
      DATA(AL(M),M=1,9)/4000.,5000.,6000.,7000.,8000.,8500.,
     1 9000.,9500.,9919./
      DATA(AA(M),M=1,9)/1.30,1.34,1.39,1.42,1.46,1.45,1.38,1.12,0.00/
      ACMI=0.
      IF(ALAM.GT.9919.)GO TO 1
      X=AMAX1(ALAM,4000.)
      DO 2 L=1,8
      IF(X.GE.AL(L).AND.X.LE.AL(L+1))M=L
    2 CONTINUE
      A=AA(M)+(AA(M+1)-AA(M))*(X-AL(M))/(AL(M+1)-AL(M))
      ACMI=A*1.E-17/ATHOMS
    1 CONTINUE
      RETURN
      END FUNCTION ACMI
C
C
C
      SUBROUTINE AC1BF(ALAM,M,ALAMM,EM,GM,AM)
C     C I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     HOFSAESS(1979):STF
C     EXCITED LEVELS M>1 SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(13),AA(13),AB(13),AC(13),AL1(23),AA1(23)
      DATA(AL(L),L=1,13)/1100.,1239.,1444.,998.,3280.,3459.,1433.,983.,
     1 4552.,4733.,4965.,5127.,5472./
      DATA(AA(L),L=1,13)/1.395E-17,1.478E-17,1.733E-17,1.313E-17,
     1 1.455E-19,1.068E-18,1.203E-17,7.269E-18,2.829E-18,1.764E-18,
     2 2.257E-18,2.436E-18,3.848E-18/
      DATA(AB(L),L=1,13)/0.00,1.26,2.68,4.18,7.49,7.68,7.95,7.95,
     1 8.54,8.64,8.77,8.85,9.00/
      DATA(AC(L),L=1,13)/9.,5.,1.,5.,9.,3.,15.,15.,3.,15.,3.,9.,5./
      IF(M.LT.1.OR.M.GT.13)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      IF(M.EQ.1)GO TO 11
      X=ALAM/ALAMM
      AM=AA(M)/ATHOMS*X*X*X
      GO TO 2
   11 CONTINUE
      DATA(AL1(L),L=1,23)/1100.,1087.,1038.,982.,886.,807.,742.,686.,
     1 638.,596.,559.,527.,498.,449.,409.,375.,346.,322.,300.,282.,
     2 265.,251.,238./
      DATA(AA1(L),L=1,23)/1.395E-17,1.396E-17,1.390E-17,1.370E-17,
     1 1.289E-17,1.175E-17,1.054E-17,9.380E-18,8.336E-18,7.385E-18,
     2 6.567E-18,5.849E-18,5.241E-18,4.223E-18,3.420E-18,2.833E-18,
     3 2.347E-18,1.992E-18,1.683E-18,1.436E-18,1.239E-18,1.075E-18,
     4 9.534E-19/
      DO 12 K=1,22
      IF(ALAM.LE.AL1(K).AND.ALAM.GE.AL1(K+1))L=K
   12 CONTINUE
      A=AA1(L)+(AA1(L+1)-AA1(L))*(ALAM-AL1(L))/(AL1(L+1)-AL1(L))
      AM=A/ATHOMS
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AC1BF
C
C
C
      SUBROUTINE AN1BF(ALAM,M,ALAMM,EM,GM,AM)
C     N I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     HOFSAESS(1979):STF
C     EXCITED LEVELS M>1 SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(13),AA(13),AB(13),AC(13),AL1(23),AA1(23)
      DATA(AL(L),L=1,13)/852.,882.,1130.,963.,825.,2942.,3213.,2245.,
     1 4213.,4449.,4586.,4872.,5122./
      DATA(AA(L),L=1,13)/1.390E-17,6.642E-18,1.025E-17,4.200E-18,
     1 3.005E-18,2.164E-19,3.467E-19,5.681E-17,4.280E-18,
     2 3.658E-18,2.516E-18,2.926E-18,1.782E-18/
      DATA(AB(L),L=1,13)/0.00,2.38,3.58,3.58,3.58,10.33,10.67,
     1 10.93,11.60,11.76,11.84,12.00,12.13/
      DATA(AC(L),L=1,13)/4.,10.,6.,6.,6.,12.,6.,12.,2.,20.,12.,14.,6./
      IF(M.LT.1.OR.M.GT.13)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      IF(M.EQ.1)GO TO 11
      X=ALAM/ALAMM
      AM=AA(M)/ATHOMS*X*X*X
      GO TO 2
   11 CONTINUE
      DATA(AL1(L),L=1,23)/852.,844.,814.,779.,718.,665.,620.,580.,
     1 545.,515.,487.,462.,440.,401.,369.,341.,317.,296.,278.,262.,
     2 248.,235.,223./
      DATA(AA1(L),L=1,23)/1.390E-17,1.415E-17,1.482E-17,1.529E-17,
     1 1.538E-17,1.477E-17,1.380E-17,1.270E-17,1.162E-17,1.062E-17,
     2 9.671E-18,8.748E-18,8.012E-18,6.631E-18,5.556E-18,4.689E-18,
     3 4.000E-18,3.436E-18,3.021E-18,2.598E-18,2.278E-18,2.010E-18,
     4 1.748E-18/
      DO 12 K=1,22
      IF(ALAM.LE.AL1(K).AND.ALAM.GE.AL1(K+1))L=K
   12 CONTINUE
      A=AA1(L)+(AA1(L+1)-AA1(L))*(ALAM-AL1(L))/(AL1(L+1)-AL1(L))
      AM=A/ATHOMS
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AN1BF
C
C
C
      SUBROUTINE AO1BF(ALAM,M,ALAMM,EM,GM,AM)
C     O I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     HOFSAESS(1979):STF
C     LEVELS M>1 SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(12),AA(12),AB(12),AC(12),AL1(23),AA1(23)
      DATA(AL(L),L=1,12)/911.,732.,665.,827.,743.,858.,2772.,3026.,
     1 4308.,4715.,6963.,7346./
      DATA(AA(L),L=1,12)/4.839E-18,4.159E-18,2.343E-18,9.125E-18,
     1 2.794E-18,1.399E-17,7.203E-21,2.402E-20,4.686E-18,2.151E-18,
     2 4.446E-20,2.743E-19/
      DATA(AB(L),L=1,12)/0.01,0.01,0.01,1.97,1.97,4.19,9.15,9.52,
     1 10.74,10.99,11.84,11.93/
      DATA(AC(L),L=1,12)/9.,9.,9.,5.,5.,1.,5.,3.,15.,9.,5.,3./
      IF(M.LT.1.OR.M.GT.12)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      IF(M.EQ.1)GO TO 11
      X=ALAM/ALAMM
      AM=AA(M)/ATHOMS*X*X*X
      GO TO 2
   11 CONTINUE
      DATA(AL1(L),L=1,23)/911.,902.,867.,828.,759.,700.,650.,607.,569.,
     1 535.,506.,479.,455.,414.,379.,350.,325.,303.,284.,268.,253.,
     2 239.,227./
      DATA(AA1(L),L=1,23)/4.839E-18,4.890E-18,5.051E-18,5.164E-18,
     1 5.242E-18,5.177E-18,5.059E-18,4.910E-18,4.724E-18,4.540E-18,
     2 4.323E-18,4.084E-18,3.879E-18,3.460E-18,3.075E-18,2.756E-18,
     3 2.437E-18,2.187E-18,1.941E-18,1.754E-18,1.576E-18,1.410E-18,
     4 1.275E-18/
      DO 12 K=1,22
      IF(ALAM.LE.AL1(K).AND.ALAM.GE.AL1(K+1))L=K
   12 CONTINUE
      A=AA1(L)+(AA1(L+1)-AA1(L))*(ALAM-AL1(L))/(AL1(L+1)-AL1(L))
      AM=A/ATHOMS
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AO1BF
C
C
C
      SUBROUTINE AHE1BF(ALAM,M,ALAMM,EM,GM,AM)
C     HE I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(9),AA(9),AB(9),AC(9)
      DATA(AL(L),L=1,9)/504.,2601.,3122.,3422.,3680.,6634.,7437.,
     1 7846.,8230./
      DATA(AA(L),L=1,9)/8.4,5.1,8.6,15.0,14.0,8.8,16.0,31.0,47.0/
      DATA(AB(L),L=1,9)/0.00,19.82,20.62,20.96,21.22,22.72,
     1 22.92,23.01,23.08/
      DATA(AC(L),L=1,9)/1.,3.,1.,9.,3.,3.,1.,9.,23./
      IF(M.LT.1.OR.M.GT.9)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AHE1BF
C
C
C
      SUBROUTINE ANE1BF(ALAM,M,ALAMM,EM,GM,AM)
C     NE I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(1),AA(1),AB(1),AC(1)
      DATA(AL(L),L=1,1)/574./
      DATA(AA(L),L=1,1)/5.4/
      DATA(AB(L),L=1,1)/0.00/
      DATA(AC(L),L=1,1)/1./
      IF(M.LT.1.OR.M.GT.1)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ANE1BF
C
C
C
      SUBROUTINE ANA1BF(ALAM,M,ALAMM,EM,GM,AM)
C     NA I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(5),AA(5),AB(5),AC(5)
      DATA(AL(L),L=1,5)/2411.,4081.,6360.,8139.,8936./
      DATA(AA(L),L=1,5)/0.13,8.2,0.9,17.0,19.0/
      DATA(AB(L),L=1,5)/0.00,2.10,3.19,3.62,3.75/
      DATA(AC(L),L=1,5)/2.,6.,2.,10.,6./
      IF(M.LT.1.OR.M.GT.5)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ANA1BF
C
C
C
      SUBROUTINE AMG1BF(ALAM,M,ALAMM,EM,GM,AM)
C     MG I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(5),AA(5),AB(5),AC(5)
      DATA(AL(L),L=1,5)/1621.,2513.,3756.,4884.,5504./
      DATA(AA(L),L=1,5)/1.2,35.0,12.0,1.7,0.4/
      DATA(AB(L),L=1,5)/0.00,2.71,4.35,5.11,5.39/
      DATA(AC(L),L=1,5)/1.,9.,3.,3.,1./
      IF(M.LT.1.OR.M.GT.5)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AMG1BF
C
C
C
      SUBROUTINE AMG2BF(ALAM,M,ALAMM,EM,GM,AM)
C     MG II LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(2),AA(2),AB(2),AC(2)
      DATA(AL(L),L=1,2)/825.,1170./
      DATA(AA(L),L=1,2)/0.14,0.5/
      DATA(AB(L),L=1,2)/0.00,4.43/
      DATA(AC(L),L=1,2)/2.,6./
      IF(M.LT.1.OR.M.GT.2)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AMG2BF
C
C
C
      SUBROUTINE AAL1BF(ALAM,M,ALAMM,EM,GM,AM)
C     AL I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(2),AA(2),AB(2),AC(2)
      DATA(AL(L),L=1,2)/2076.,4361./
      DATA(AA(L),L=1,2)/19.0,2.0/
      DATA(AB(L),L=1,2)/0.01,3.14/
      DATA(AC(L),L=1,2)/6.,2./
      IF(M.LT.1.OR.M.GT.2)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AAL1BF
C
C
C
      SUBROUTINE ASI1BF(ALAM,M,ALAMM,EM,GM,AM)
C     SI I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(5),AA(5),AB(5),AC(5)
      DATA(AL(L),L=1,5)/1525.,1682.,1986.,3862.,4040./
      DATA(AA(L),L=1,5)/26.0,23.0,39.0,1.2,8.1/
      DATA(AB(L),L=1,5)/0.02,0.78,1.91,4.93,5.08/
      DATA(AC(L),L=1,5)/9.,5.,1.,9.,3./
      IF(M.LT.1.OR.M.GT.5)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ASI1BF
C
C
C
      SUBROUTINE ASI2BF(ALAM,M,ALAMM,EM,GM,AM)
C     SI II LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(1),AA(1),AB(1),AC(1)
      DATA(AL(L),L=1,1)/760./
      DATA(AA(L),L=1,1)/2.3/
      DATA(AB(L),L=1,1)/0.03/
      DATA(AC(L),L=1,1)/6./
      IF(M.LT.1.OR.M.GT.1)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ASI2BF
C
C
C
      SUBROUTINE ACA1BF(ALAM,M,ALAMM,EM,GM,AM)
C     CA I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(3),AA(3),AB(3),AC(3)
      DATA(AL(L),L=1,3)/2028.,2938.,3454./
      DATA(AA(L),L=1,3)/0.83,6.1,5.0/
      DATA(AB(L),L=1,3)/0.00,1.89,2.52/
      DATA(AC(L),L=1,3)/1.,9.,15./
      IF(M.LT.1.OR.M.GT.3)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ACA1BF
C
C
C
      SUBROUTINE ACA2BF(ALAM,M,ALAMM,EM,GM,AM)
C     CA II LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(3),AA(3),AB(3),AC(3)
      DATA(AL(L),L=1,3)/1045.,1219.,1421./
      DATA(AA(L),L=1,3)/0.054,16.0,4.1/
      DATA(AB(L),L=1,3)/0.00,1.70,3.14/
      DATA(AC(L),L=1,3)/2.,10.,6./
      IF(M.LT.1.OR.M.GT.3)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE ACA2BF
C
C
C
      SUBROUTINE AFE1BF(ALAM,M,ALAMM,EM,GM,AM)
C     FE I LEVEL M:EDGE ALAMM,ENERGY EM,WEIGHT GM,B-F CROSS-SECTION AM
C     CROSS-SECTION AT EDGE AL:BASCHEK&SCHOLZ(1981)
C     SHORTWARD OF EDGE HYDROGENIC
C
      USE op1_com
C
      DIMENSION AL(1),AA(1),AB(1),AC(1)
      DATA(AL(L),L=1,1)/1570./
      DATA(AA(L),L=1,1)/4.1/
      DATA(AB(L),L=1,1)/0.03/
      DATA(AC(L),L=1,1)/25.0/
      IF(M.LT.1.OR.M.GT.1)GO TO 1
      ALAMM=AL(M)
      EM=AB(M)
      GM=AC(M)
      AM=0.
      IF(ALAM.GT.ALAMM)GO TO 2
      X=ALAM/ALAMM
      AM=AA(M)*1.E-18/ATHOMS*X*X*X
      GO TO 2
    1 EM=0.
      GM=0.
      AM=0.
      ALAMM=0.
    2 CONTINUE
      RETURN
      END SUBROUTINE AFE1BF

C--------------*****---------------------------------------------------+-------
      FUNCTION B_lam(temp, alam)
C
      USE constants
C
      IMPLICIT NONE
C
      REAL :: B_lam
C--- Dummy variables:
      REAL, INTENT(IN) :: temp, alam
C--- Local variables:
      REAL :: hnu, akt, alam5, x
C 
      hnu=hplanc*clight/alam/1.e-8
      akt=boltzk*temp
      alam5=1.e-20*alam*alam*alam*1.e-20*alam*alam
      x=EXP(-hnu/akt)
      B_lam=2.*hplanc*clight*clight/(1./x-1.)/alam5

      END FUNCTION B_lam
C------------------*****-----------------------------------------------+-------

C----------------****--------------------------------------------------+-------
      SUBROUTINE OS2D(nclam, clam, istop, jstop, pij, tij, t0, b0,
     &                blclam, opclam)

C----------------------------------------------------------------------+-------
C
C Version: 11-Dec-1997 17:40:06
C
C Author : MS
C
C Description: OS2D computes the monochromatic continuous opacity 
C at wavelength CLAM based on the Kiel routines IONDIS and OPALAM
C
C***********************************************************************
C
      IMPLICIT NONE
C--- Dummy arguments:
      INTEGER, INTENT(IN)  :: nclam, istop, jstop
      REAL,    INTENT(IN)  :: t0
      REAL,    INTENT(OUT), DIMENSION(:)      :: b0
      REAL,    INTENT(IN),  DIMENSION(:)      :: clam
      REAL,    INTENT(IN),  DIMENSION(:,:)    :: pij, tij      
      REAL,    INTENT(OUT), DIMENSION(:,:,:)  :: blclam, opclam
C--- Local variables:
      REAL, PARAMETER :: Tdiff=100.0
      INTEGER         :: i, j, itnum, nl
      REAL            :: dlpgpe, dnc, pe0, pe1, oka, osi, slope0,
     &                   dlpe, dlpedT, pgas0, pgas1, temp0, temp1
C
      pe0    = 1.0E-4
      slope0 = 1.5
C
      DO j=jstop,1,-1
        pgas0  = pij(1,j)
        temp0  = tij(1,j)
        temp1  = temp0 + Tdiff
        CALL P_COMP(pgas0, temp1, pe0, pe1, dnc, slope0, dlpgpe, itnum)
        CALL P_COMP(pgas0, temp0, pe1, pe0, dnc, slope0, dlpgpe, itnum)
        dlpedT = 2.0*(pe1-pe0)/(pe1+pe0)/(temp1-temp0)
        slope0 = dlpgpe
        DO i=1,istop
          pgas1 = pij(i,j)
          temp1 = tij(i,j)
          dlpe  = dlpedT*(temp1-temp0) + 
     &            2.0*(pgas1-pgas0)/(pgas1+pgas0)/slope0
          pe0   = pe0*exp(dlpe)
C***********************************************************************
          CALL P_COMP(pgas1, temp1, pe0, pe1, dnc, 
     &                slope0, dlpgpe, itnum)
          DO nl=1,nclam
            CALL OPALAM(clam(nl), temp1, pe1, dnc, 1, oka, osi)
            opclam(nl,i,j) = dnc*(oka+osi)
            blclam(nl,i,j) = B_lam(temp1, clam(nl)) 
          ENDDO
C***********************************************************************
C          WRITE(*,"(1X,2I4,2ES15.6,I3)")i, j, dlpedT, dlpgpe, itnum
C
          pgas0  = pgas1
          temp0  = temp1
          pe0    = pe1
          slope0 = dlpgpe
C
        ENDDO
      ENDDO
C
      DO nl=1,nclam
        b0(nl) = B_lam(t0, clam(nl)) 
      ENDDO
C
      END SUBROUTINE OS2D
C--------------------****----------------------------------------------+-------

      END MODULE iondis_opalam_module
C----------------********************----------------------------------+-------
