!**************************************************************************************************
!
!   #####   #    #  #####           
!   #    #  #    #  #    #          
!   #    #  ######  #    #          
!   #####   #    #  #    #          
!   #   #   #    #  #    #          
!   #    #  #    #  #####  #######  chromorad_module
!
!**************************************************************************************************
!   Fortran90
!   Sven Wedemeyer-Boehm; Freiburg, Germany | Jorrit Leenaarts, Utrecht, The Netherlands
!   11.02.2005
!--- Last modification: 11.02.2005 by S. Wedemeyer-Boehm
!**************************************************************************************************
! Modules:
!   rhd_chromorad_module: Chromospheric radiative transfer routines for rhd
!**************************************************************************************************
! /* --- Default values for preprocessor macros ------------------------- */
!
!------********************------------------------------------------------------------------------
module rhd_chromorad_module
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_chromorad_module ('rhd_chromospheric_radiation_module')
!
! PURPOSE:
!   Chromospheric radiative transfer routines for rhd.
!
! CATEGORY:
!   Radiation transport
!
! CALLING SEQUENCE:
!   use rhd_chromorad_module
!
! VARIABLES:
!
! ROUTINES: (contained)
!
! MODIFICATION HISTORY:
!   11. 2.05 Written by Sven Wedemeyer-Boehm, KIS, Freiburg
!--------------------------------------------------------------------------------------------------
!
implicit none
!save
!
contains
!
!----------******************----------------------------------------------------------------------------
subroutine rhd_chromorad_qrad(model, qrad, n3chromo)
!--------------------------------------------------------------------------------------------------
!--------------------------------------------------------------------------------------------------
! NAME:
!   rhd_chromorad_qrad ('rhd_chromospheric_radiation_transport_get_Qrad')
!
! PURPOSE:
!   calculate/update net radiative heating rate Qrad
!
! CATEGORY:
!   Radiative transfer
!
! CALLING SEQUENCE:
!
! INPUT:
!   none
!
! OUTPUT:
!   none
!
! ROUTINES:
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   11. 2.05 Written by Sven Wedemeyer-Boehm, KIS
!--------------------------------------------------------------------------------------------------
use rhd_box_module
!
implicit none
!
! --- I/O parameters ---
type(box_type),           intent(inout) :: model
real, dimension(model%m1:model%n1,  &
                model%m2:model%n2,  &
                model%m3:model%n3), &
                intent(inout)           :: qrad
integer, intent(in)                     :: n3chromo

integer                                 :: i1, i2, i3, &
                                           n,m1,n1,m2,n2,m3,n3
!
!--------------------------------------------------------------------------------------------------
! --- Abbreviations ---
m1=model%m1
n1=model%n1
m2=model%m2
n2=model%n2
m3=model%m3
n3=model%n3
!
!$OMP PARALLEL DEFAULT(SHARED)
!$OMP DO PRIVATE(i1,i2)
do i3=1,n3-n3chromo
  do i2=m2,n2
    do i1=m1,n1
      ! --- calculate net radiative heating rate based on chromospheric RT ---
      !
      qrad(i1,i2,i3)=qrad(i1,i2,i3)
      !
    end do
  end do
end do
!$OMP END DO
!$OMP END PARALLEL
!
!return
end subroutine rhd_chromorad_qrad
!
end module rhd_chromorad_module
