!***************************************************************************************************
!
!   #    # #  ####        ###### # #      ###### #####  ###### ###### 
!   #    # # #    #       #      # #      #      #    # #      #      
!   #    # # #    #       #####  # #      #####  #    # #####  #####  
!   #    # # #    #       #      # #      #      #    # #      #      
!   #    # # #    #       #      # #      #      #    # #      #      
!    ####  #  #### ###### #      # ###### ###### #####  ###### #     _module 
!
!   Universal Input/Output Routines: file label definitions
!
!***************************************************************************************************
!   This set of input and output routines allows the reading and writing of scalar, array, and
!   table data in header-data units in formatted or unformatted Fortran file format.
!***************************************************************************************************
!   Fortran90
!   2016-03-16
!***************************************************************************************************
! With contributions from:
!   Bernd Freytag:         Kiel, Copenhagen, Lyon, Uppsala
!***************************************************************************************************
! MODULES:
!   uio_filedef_module:    Parameters for standard UIO file descriptors and labels
!
!***************************************************************************************************

 
!------******************---------------------------------------------------------------------------
module uio_filedef_module
!---------------------------------------------------------------------------------------------------
! NAME:
!   uio_filedef_module ('uio_file_definition_module')
!
! PURPOSE:
!   Contain parameters for standard file descriptors and labels.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_filedef_module
!
! VARIABLES:
!   box_ident:    ('box_identifier') character, label for start of box
!   endbox_ident: ('end_box_identifier') character, label for end of box
!   dataset_ident: ('data_set_identifier') character, label for start of dataset
!   enddataset_ident: ('end_data_set_identifier') character, label for end of dataset
!
! MODIFICATION HISTORY:
!   1998-11-17 (B.F. Copenhagen) Written
!   1998-12-14 (B.F.) box_id, dataset_id
!   2013-08-24 (B.F.) Change layout of comments
!---------------------------------------------------------------------------------------------------
implicit none
!
character(len=*), parameter             :: file_id_ident         ='file_id'    , &
                                           file_version_ident    ='version'    , &
                                           file_description_ident='description', &
                                           file_history_ident    ='history'    , &
                                           dataset_ident         ='dataset'    , &
                                           enddataset_ident      ='enddataset' , &
                                           dataset_id_ident      ='dataset_id' , &
                                           box_ident             ='box'        , &
                                           endbox_ident          ='endbox'     , &
                                           box_id_ident          ='box_id'
!
end module uio_filedef_module
