!******************************************************************************
!       Universal Input/Output Routines
!       uio flexible variable type: Fortran90 version
!       27-Mar-1998 18:18:36
!       11.10.2001
!******************************************************************************
! This set of routines allows the basic handling of uio flexible variables.
!******************************************************************************
! Non-standard FORTRAN:
!   Hopefully none!
!******************************************************************************


!------**************----------------------------------------------------------
module uio_var_module
!------------------------------------------------------------------------------
! NAME:
!   uio_var_module ('uio_variable_module')
!
! PURPOSE:
!   Contain uio-routines for uio flexible variable type.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   use uio_var_module
!
! ROUTINES: (CONTAINED)
!   uio_var_askhead:   Check header information of uio flexible variable
!   uio_var_changehead:Change header information of uio flexible variable
!   uio_var_copy:      Copy uio flexible variable. Account for pointers properly
!   uio_var_cut:       Cut block out of (4D) array
!   uio_var_real:      Change type of uio flexibel variable to real
!   uio_var_3Dto4D:    Transform array in uio flexibel variable from 3D (or scalar) to 4D
!   uio_var_delete:    Delete uio flexible variable
!   uio_var_null:      Set uio flexible variable to zero (initialization)
!   uio_var_glue:      Enlarge (4D) array in uio flexible variable
!   uio_var_get:       Get data set from uio flexible variable
!   uio_var_pointget:  Point to data set in uio flexible variable
!   uio_var_pointput:  Put data set into uio flexible variable via pointing
!   uio_var_printhead: Print header of uio flexible variable
!   uio_var_put:       Put data set into uio flexible variable
!   uio_var_putfifo:   Put fileform data set into uio flexible variable
!   uio_var_putlabl:   Put label into uio flexible variable
!   uio_var_puttab:    Put table into uio flexible variable
!   uio_var_rd:        Read uio file entry into uio flexible variable
!   uio_var_wr:        Write uio flexible variable into uio file
!
! PARAMETERS:
!   uio_var_l: Maximum string length in uio flexible variable
!
! TYPES:
!   uio_var_type: uio flexible variable
!
! MODULES:
!   uio_siz_module:
!   uio_table_module:
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   29. 1.98 (B.F.)
!   13. 2.98 (B.F.) size -> shape
!------------------------------------------------------------------------------
use uio_siz_module
use uio_table_module
!
implicit none
!
integer, parameter                      :: uio_var_l=256
!
type uio_var_type
  character*(let)                       :: type, ident, format
  integer                               :: kind, status
  integer                               :: ntt
  character*(let), dimension(2,nttmx)   :: termt
  integer                               :: ndim
  integer,              pointer                     :: integer0
  integer,              pointer, dimension(:)       :: integer1
  integer,              pointer, dimension(:,:)     :: integer2
  integer,              pointer, dimension(:,:,:)   :: integer3
  integer,              pointer, dimension(:,:,:,:) :: integer4
  real(kind(0.0E0)),    pointer                     :: real0
  real(kind(0.0E0)),    pointer, dimension(:)       :: real1
  real(kind(0.0E0)),    pointer, dimension(:,:)     :: real2
  real(kind(0.0E0)),    pointer, dimension(:,:,:)   :: real3
  real(kind(0.0E0)),    pointer, dimension(:,:,:,:) :: real4
  real(kind(0.0D0)),    pointer                     :: dbreal0
  real(kind(0.0D0)),    pointer, dimension(:)       :: dbreal1
  real(kind(0.0D0)),    pointer, dimension(:,:)     :: dbreal2
  real(kind(0.0D0)),    pointer, dimension(:,:,:)   :: dbreal3
  real(kind(0.0D0)),    pointer, dimension(:,:,:,:) :: dbreal4
  complex(kind(0.0E0)), pointer                     :: complex0
  complex(kind(0.0E0)), pointer, dimension(:)       :: complex1
  complex(kind(0.0E0)), pointer, dimension(:,:)     :: complex2
  complex(kind(0.0E0)), pointer, dimension(:,:,:)   :: complex3
  complex(kind(0.0E0)), pointer, dimension(:,:,:,:) :: complex4
  character(uio_var_l), pointer                     :: character0
  character(uio_var_l), pointer, dimension(:)       :: character1
  character(uio_var_l), pointer, dimension(:,:)     :: character2
  character(uio_var_l), pointer, dimension(:,:,:)   :: character3
  character(uio_var_l), pointer, dimension(:,:,:,:) :: character4
  type(uio_table_type), pointer                     :: table
end type uio_var_type
!
interface uio_var_pointget
  module procedure &
        uio_var_pointgeti0, uio_var_pointgeti1, uio_var_pointgeti2, &
        uio_var_pointgeti3, uio_var_pointgeti4, &
        uio_var_pointgetr0, uio_var_pointgetr1, uio_var_pointgetr2, &
        uio_var_pointgetr3, uio_var_pointgetr4, &
        uio_var_pointgetd0, uio_var_pointgetd1, uio_var_pointgetd2, &
        uio_var_pointgetd3, uio_var_pointgetd4, &
        uio_var_pointgetc0, uio_var_pointgetc1, uio_var_pointgetc2, &
        uio_var_pointgetc3, uio_var_pointgetc4, &
        uio_var_pointgets0, uio_var_pointgets1, uio_var_pointgets2, &
        uio_var_pointgets3, uio_var_pointgets4
end interface ! uio_var_pointget
!
private &
        uio_var_pointgeti0, uio_var_pointgeti1, uio_var_pointgeti2, &
        uio_var_pointgeti3, uio_var_pointgeti4, &
        uio_var_pointgetr0, uio_var_pointgetr1, uio_var_pointgetr2, &
        uio_var_pointgetr3, uio_var_pointgetr4, &
        uio_var_pointgetd0, uio_var_pointgetd1, uio_var_pointgetd2, &
        uio_var_pointgetd3, uio_var_pointgetd4, &
        uio_var_pointgetc0, uio_var_pointgetc1, uio_var_pointgetc2, &
        uio_var_pointgetc3, uio_var_pointgetc4, &
        uio_var_pointgets0, uio_var_pointgets1, uio_var_pointgets2, &
        uio_var_pointgets3, uio_var_pointgets4
!
interface uio_var_get
  module procedure &
        uio_var_geti0, uio_var_geti1, uio_var_geti2, &
        uio_var_geti3, uio_var_geti4, &
        uio_var_getr0, uio_var_getr1, uio_var_getr2, &
        uio_var_getr3, uio_var_getr4, &
        uio_var_getd0, uio_var_getd1, uio_var_getd2, &
        uio_var_getd3, uio_var_getd4, &
        uio_var_getc0, uio_var_getc1, uio_var_getc2, &
        uio_var_getc3, uio_var_getc4, &
        uio_var_gets0, uio_var_gets1, uio_var_gets2, &
        uio_var_gets3, uio_var_gets4
end interface ! uio_var_get
!
private &
        uio_var_geti0, uio_var_geti1, uio_var_geti2, &
        uio_var_geti3, uio_var_geti4, &
        uio_var_getr0, uio_var_getr1, uio_var_getr2, &
        uio_var_getr3, uio_var_getr4, &
        uio_var_getd0, uio_var_getd1, uio_var_getd2, &
        uio_var_getd3, uio_var_getd4, &
        uio_var_getc0, uio_var_getc1, uio_var_getc2, &
        uio_var_getc3, uio_var_getc4, &
        uio_var_gets0, uio_var_gets1, uio_var_gets2, &
        uio_var_gets3, uio_var_gets4
!
interface uio_var_pointput
  module procedure &
        uio_var_pointputi0, uio_var_pointputi1, uio_var_pointputi2, &
        uio_var_pointputi3, uio_var_pointputi4, &
        uio_var_pointputr0, uio_var_pointputr1, uio_var_pointputr2, &
        uio_var_pointputr3, uio_var_pointputr4, &
        uio_var_pointputd0, uio_var_pointputd1, uio_var_pointputd2, &
        uio_var_pointputd3, uio_var_pointputd4, &
        uio_var_pointputc0, uio_var_pointputc1, uio_var_pointputc2, &
        uio_var_pointputc3, uio_var_pointputc4, &
        uio_var_pointputs0, uio_var_pointputs1, uio_var_pointputs2, &
        uio_var_pointputs3, uio_var_pointputs4
end interface ! uio_var_pointput
!
private &
        uio_var_pointputi0, uio_var_pointputi1, uio_var_pointputi2, &
        uio_var_pointputi3, uio_var_pointputi4, &
        uio_var_pointputr0, uio_var_pointputr1, uio_var_pointputr2, &
        uio_var_pointputr3, uio_var_pointputr4, &
        uio_var_pointputd0, uio_var_pointputd1, uio_var_pointputd2, &
        uio_var_pointputd3, uio_var_pointputd4, &
        uio_var_pointputc0, uio_var_pointputc1, uio_var_pointputc2, &
        uio_var_pointputc3, uio_var_pointputc4, &
        uio_var_pointputs0, uio_var_pointputs1, uio_var_pointputs2, &
        uio_var_pointputs3, uio_var_pointputs4
!
interface uio_var_put
  module procedure &
        uio_var_puti0, uio_var_puti1, uio_var_puti2, &
        uio_var_puti3, uio_var_puti4, &
        uio_var_putr0, uio_var_putr1, uio_var_putr2, &
        uio_var_putr3, uio_var_putr4, &
        uio_var_putd0, uio_var_putd1, uio_var_putd2, &
        uio_var_putd3, uio_var_putd4, &
        uio_var_putc0, uio_var_putc1, uio_var_putc2, &
        uio_var_putc3, uio_var_putc4, &
        uio_var_puts0, uio_var_puts1, uio_var_puts2, &
        uio_var_puts3, uio_var_puts4
end interface ! uio_var_put
!
private &
        uio_var_puti0, uio_var_puti1, uio_var_puti2, &
        uio_var_puti3, uio_var_puti4, &
        uio_var_putr0, uio_var_putr1, uio_var_putr2, &
        uio_var_putr3, uio_var_putr4, &
        uio_var_putd0, uio_var_putd1, uio_var_putd2, &
        uio_var_putd3, uio_var_putd4, &
        uio_var_putc0, uio_var_putc1, uio_var_putc2, &
        uio_var_putc3, uio_var_putc4, &
        uio_var_puts0, uio_var_puts1, uio_var_puts2, &
        uio_var_puts3, uio_var_puts4
!
interface assignment (=)
  module procedure uio_var_copy
end interface ! assignment (=)
!
contains
!
!----------***************-----------------------------------------------------
subroutine uio_var_askhead(var, termt,ntt, ident, type, nkind, status, ndim, &
                           ilow,iup, format, name, unit, tran, date, nbytes, &
                           outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_askhead ('uio_variable_ask_head')
!
! PURPOSE:
!   Check header information of uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_askhead(var, termt=termt,ntt=ntt, ident=ident, type=type, &
!                        nkind=nkind, status=status, ndim=ndim, &
!                        ilow=ilow,iup=iup, format=format, name=name, &
!                        unit=unit, tran=tran, date=date, nbytes=nbytes, &
!                        outstr=outstr,ierr=ierr)
!
! INPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be checked
!
! OUTPUT: (optional)
!   termt:  ('term_table') character array (2,nttmx), optional, complete
!           keyword table
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!   ident:  ('identifier') character, optional, identifier
!   type:   ('type') character, optional, type
!   nkind:  ('number_kind') integer, optional, kind of variable
!   status: ('status') integer, optional, status: 0: empty, 1: defined
!   ndim:   ('number_dimension') integer, optional, number of dimensions
!   ilow:   ('index_low') integer vector, optional, lower indices to use
!   iup:    ('index_up') integer vector, optional, upper indices to use
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!   nbytes: ('number_bytes') integer, optional, number of bytes of component
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_exkeyw: Extract value of keyword from table
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Check header and give several quantities describing the
!   uio flexible variable.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   14. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!    6. 3.98 (B.F.) Account for parameters 'type' and 'nkind'
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(in)     :: var
character*(*), optional, intent(out)    :: termt(2,nttmx)
integer,       optional, intent(out)    :: ntt
integer,       optional, intent(out)    :: nkind, status, ndim, nbytes
character*(*), optional, intent(out)    :: ident, type, &
                                           format, name, unit, tran, date
integer, dimension(:), &
               optional, intent(out)    :: ilow, iup
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(let)                         :: termtt(2,nttmx)
character*(let)                         :: date0, dummy
integer                                 :: nttt
integer                                 :: ierr0
character*(80)                          :: outstr0
integer, dimension(4)                   :: ilow0, iup0
!------------------------------------------------------------------------------
!
if (present(status)) status=var%status
!
! --- Check status ---
if (var%status > 0) then
  ! --- Change explicit header data ---
  if (present(type  )) type  =var%type
  if (present(ident )) ident =var%ident
  if (present(format)) format=var%format
  if (present(ndim  )) ndim  =var%ndim
  if (present(nkind )) nkind =var%kind
  !
  ! --- Copy term table into temporary variable ---
  nttt=var%ntt
  termtt(:,1:nttt)=var%termt(:,1:var%ntt)
  !
  ! --- Give data in term table ---
  if (present(name)) call uio_exkeyw(termtt,nttt, namena ,name, mode=12)
  if (present(unit)) call uio_exkeyw(termtt,nttt, unitna ,unit, mode=12)
  if (present(tran)) call uio_exkeyw(termtt,nttt, tranna ,tran, mode=12)
  if (present(date)) call uio_exkeyw(termtt,nttt, datena ,date, mode=12)
  !
  ! --- Number of bytes (lengths of string) ---
  if (present(nbytes)) then
    select case(var%type)
      case(intena)                    ! --- Integer ---
        inquire(iolength=nbytes) 0
      case(realna)                    ! --- Real ---
        select case(var%kind)
          case(kind(0.0E0))               ! --- Single precision ---
            inquire(iolength=nbytes) 0.0E0
          case(kind(0.0D0))               ! --- Double precision ---
            inquire(iolength=nbytes) 0.0D0
        end select ! var%kind
      case(compna)                    ! --- Complex ---
        inquire(iolength=nbytes) cmplx(0.0E0)
      case(charna)                    ! --- Character ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            nbytes=len(var%character0)
          case(1)                         ! --- 1D ---
            nbytes=len(var%character1)
          case(2)                         ! --- 2D ---
            nbytes=len(var%character2)
          case(3)                         ! --- 3D ---
            nbytes=len(var%character3)
          case(4)                         ! --- 4D ---
            nbytes=len(var%character4)
        end select ! var%ndim
      case default                    ! --- Rest ---
        nbytes=0
    end select ! var%type
  endif
  !
  ! --- Give term table ---
  if (present(termt)) termt(:,1:nttt)=termtt(:,1:nttt)
  if (present(ntt)) ntt=nttt
  !
  ! --- Give lower and upper boundaries of array ---
  if (present(ilow) .or. present(iup)) then
    if (var%ndim == 0) then
      select case(var%type)
        case(intena, realna, compna, charna)   ! --- Standard types ---
          if (present(ilow)) ilow=1
          if (present(iup)) iup=1
        case default                           ! --- Rest ---
          if (present(ilow)) ilow=0
          if (present(iup)) iup=0
      end select ! var%type
    else
      select case(var%type)
        case(intena)                    ! --- Integer ---
          select case(var%ndim)
            case(1)                         ! --- 1D ---
              ilow0(1:1)=lbound(var%integer1)
              iup0( 1:1)=ubound(var%integer1)
            case(2)                         ! --- 2D ---
              ilow0(1:2)=lbound(var%integer2)
              iup0( 1:2)=ubound(var%integer2)
            case(3)                         ! --- 3D ---
              ilow0(1:3)=lbound(var%integer3)
              iup0( 1:3)=ubound(var%integer3)
            case(4)                         ! --- 4D ---
              ilow0(1:4)=lbound(var%integer4)
              iup0( 1:4)=ubound(var%integer4)
          end select ! var%ndim
        case(realna)                    ! --- Real ---
          select case(var%kind)
            case(kind(0.0E0))               ! --- Single precision ---
              select case(var%ndim)
                case(1)                         ! --- 1D ---
                  ilow0(1:1)=lbound(var%real1)
                  iup0( 1:1)=ubound(var%real1)
                case(2)                         ! --- 2D ---
                  ilow0(1:2)=lbound(var%real2)
                  iup0( 1:2)=ubound(var%real2)
                case(3)                         ! --- 3D ---
                  ilow0(1:3)=lbound(var%real3)
                  iup0( 1:3)=ubound(var%real3)
                case(4)                         ! --- 4D ---
                  ilow0(1:4)=lbound(var%real4)
                  iup0( 1:4)=ubound(var%real4)
              end select ! var%ndim
            case(kind(0.0D0))               ! --- Double precision ---
              select case(var%ndim)
                case(1)                         ! --- 1D ---
                  ilow0(1:1)=lbound(var%dbreal1)
                  iup0( 1:1)=ubound(var%dbreal1)
                case(2)                         ! --- 2D ---
                  ilow0(1:2)=lbound(var%dbreal2)
                  iup0( 1:2)=ubound(var%dbreal2)
                case(3)                         ! --- 3D ---
                  ilow0(1:3)=lbound(var%dbreal3)
                  iup0( 1:3)=ubound(var%dbreal3)
                case(4)                         ! --- 4D ---
                  ilow0(1:4)=lbound(var%dbreal4)
                  iup0( 1:4)=ubound(var%dbreal4)
              end select ! var%ndim
          end select ! var%kind
        case(compna)                    ! --- Complex ---
              select case(var%ndim)
                case(1)                         ! --- 1D ---
                  ilow0(1:1)=lbound(var%complex1)
                  iup0( 1:1)=ubound(var%complex1)
                case(2)                         ! --- 2D ---
                  ilow0(1:2)=lbound(var%complex2)
                  iup0( 1:2)=ubound(var%complex2)
                case(3)                         ! --- 3D ---
                  ilow0(1:3)=lbound(var%complex3)
                  iup0( 1:3)=ubound(var%complex3)
                case(4)                         ! --- 4D ---
                  ilow0(1:4)=lbound(var%complex4)
                  iup0( 1:4)=ubound(var%complex4)
              end select ! var%ndim
        case(charna)                    ! --- Character ---
          select case(var%ndim)
            case(1)                         ! --- 1D ---
              ilow0(1:1)=lbound(var%character1)
              iup0( 1:1)=ubound(var%character1)
            case(2)                         ! --- 2D ---
              ilow0(1:2)=lbound(var%character2)
              iup0( 1:2)=ubound(var%character2)
            case(3)                         ! --- 3D ---
              ilow0(1:3)=lbound(var%character3)
              iup0( 1:3)=ubound(var%character3)
            case(4)                         ! --- 4D ---
              ilow0(1:4)=lbound(var%character4)
              iup0( 1:4)=ubound(var%character4)
          end select ! var%ndim
      end select ! var%type
    endif
    !
    if (present(ilow)) ilow=ilow0(1:var%ndim)
    if (present(iup )) iup =iup0( 1:var%ndim)
    !
  endif
  !
  outstr0='done'
  ierr0=0
else
  outstr0='undefined variable'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_askhead


!----------******************--------------------------------------------------
subroutine uio_var_changehead(var, termt,ntt, ident, &
                              format, name, unit, tran, date, &
                              outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_changehead ('uio_variable_change_head')
!
! PURPOSE:
!   Change header information of uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!
! INPUT:
!   ident:  ('identifier') character, optional, identifier
!   format: ('format') character, optional, format string
!   name:   ('name') character, optional, name of variable
!   unit:   ('unit') character, optional, units
!   tran:   ('transformation') character, optional, transformation
!   date:   ('date') character, optional, date
!
! INPUT/OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be checked
!
! OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), optional, complete
!           keyword table
!   ntt:    ('number_term_table') integer, optional, number of non-empty lines
!           of termt
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_adkeyw: Add one keyword to term table with keyword-value or default
!   uio_dattim: Get string with current date and time
!   uio_exkeyw: Extract value of keyword from table
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   15. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(inout)  :: var
character*(*), optional, intent(in)     :: termt(2,nttmx)
integer,       optional, intent(in)     :: ntt
character*(*), optional, intent(in)     :: ident, format, name, unit, tran, date
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(let)                         :: termtt(2,nttmx), dummy, date0
integer                                 :: nttt, nttn
integer                                 :: ierr0
character*(80)                          :: outstr0
!------------------------------------------------------------------------------
!
! --- Change explicit header data ---
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
!
nttt=0
if (present(name)) then
  call uio_exkeyw(var%termt,var%ntt, namena, dummy, mode=12)
  call uio_adkeyw(termtt,   nttt,    namena, keyval=name)
endif
if (present(unit)) then
  call uio_exkeyw(var%termt,var%ntt, unitna, dummy, mode=12)
  call uio_adkeyw(termtt,   nttt,    unitna, keyval=unit)
endif
if (present(tran)) then
  call uio_exkeyw(var%termt,var%ntt, tranna, dummy, mode=12)
  call uio_adkeyw(termtt,   nttt,    tranna, keyval=tran)
endif
if (present(date)) then
  call uio_exkeyw(var%termt,var%ntt, datena, dummy, mode=12)
  if (date == 'now') then
    call uio_dattim(date0)
  else
    date0=date
  endif
  call uio_adkeyw(termtt,nttt, datena, keyval=date0)
endif
!
! --- Concatenate term tables ---
if (present(termt) .and. present(ntt)) then
  nttn=min(nttt+ntt,nttmx)
  termtt(:,nttt+1:nttn)=termt(:,1:nttn-nttt)
  nttt=nttn
else
  nttn=min(nttt+var%ntt,nttmx)
  termtt(:,nttt+1:nttn)=var%termt(:,1:nttn-nttt)
  nttt=nttn
endif
!
! --- Put new term table into uio flexible variable ---
var%ntt=nttt
var%termt(:,1:var%ntt)=termtt(:,1:nttt)
!
outstr0='done'
ierr0=0
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_changehead
!
!----------************--------------------------------------------------------
subroutine uio_var_copy(varleft, varright)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_copy ('uio_variable_copy')
!
! PURPOSE:
!   Copy uio flexible variable. Account for pointers properly.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_copy(varleft, varright)
!
! INPUT:
!   varright: ('variable_right') uio_var_type, input
!
! OUTPUT:
!   varleft:  ('variable_left') uio_var_type, copy of varright
!
! ROUTINES:
!   None
!
! MODULES:
!   None
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!   Copy all header items. Copy pointer targets properly.
!   Effect: varleft=varright.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   14. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(in)     :: varright
type(uio_var_type)     , intent(out)    :: varleft
!
! --- Local variables ---
!------------------------------------------------------------------------------
!
! --- Copy header ---
varleft%type  =varright%type
varleft%ident =varright%ident 
varleft%format=varright%format
varleft%kind  =varright%kind
varleft%status=varright%status
varleft%ntt   =varright%ntt
varleft%termt(:,1:varleft%ntt)=varright%termt(:,1:varright%ntt)
varleft%ndim  =varright%ndim
!
! --- Copy data ---
! --- Integer ---
if (associated(varright%integer0)) then
  allocate(varleft%integer0)
  varleft%integer0=varright%integer0
endif
if (associated(varright%integer1)) then
  allocate(varleft%integer1( &
                   lbound(varright%integer1,1):ubound(varright%integer1,1) ))
  varleft%integer1=varright%integer1
endif
if (associated(varright%integer2)) then
  allocate(varleft%integer2( &
                   lbound(varright%integer2,1):ubound(varright%integer2,1), &
                   lbound(varright%integer2,2):ubound(varright%integer2,2) ))
  varleft%integer2=varright%integer2
endif
if (associated(varright%integer3)) then
  allocate(varleft%integer3( &
                   lbound(varright%integer3,1):ubound(varright%integer3,1), &
                   lbound(varright%integer3,2):ubound(varright%integer3,2), &
                   lbound(varright%integer3,3):ubound(varright%integer3,3) ))
  varleft%integer3=varright%integer3
endif
if (associated(varright%integer4)) then
  allocate(varleft%integer4( &
                   lbound(varright%integer4,1):ubound(varright%integer4,1), &
                   lbound(varright%integer4,2):ubound(varright%integer4,2), &
                   lbound(varright%integer4,3):ubound(varright%integer4,3), &
                   lbound(varright%integer4,4):ubound(varright%integer4,4) ))
  varleft%integer4=varright%integer4
endif
!
! --- Real (single precision) ---
if (associated(varright%real0)) then
  allocate(varleft%real0)
  varleft%real0=varright%real0
endif
if (associated(varright%real1)) then
  allocate(varleft%real1( &
                   lbound(varright%real1,1):ubound(varright%real1,1) ))
  varleft%real1=varright%real1
endif
if (associated(varright%real2)) then
  allocate(varleft%real2( &
                   lbound(varright%real2,1):ubound(varright%real2,1), &
                   lbound(varright%real2,2):ubound(varright%real2,2) ))
  varleft%real2=varright%real2
endif
if (associated(varright%real3)) then
  allocate(varleft%real3( &
                   lbound(varright%real3,1):ubound(varright%real3,1), &
                   lbound(varright%real3,2):ubound(varright%real3,2), &
                   lbound(varright%real3,3):ubound(varright%real3,3) ))
  varleft%real3=varright%real3
endif
if (associated(varright%real4)) then
  allocate(varleft%real4( &
                   lbound(varright%real4,1):ubound(varright%real4,1), &
                   lbound(varright%real4,2):ubound(varright%real4,2), &
                   lbound(varright%real4,3):ubound(varright%real4,3), &
                   lbound(varright%real4,4):ubound(varright%real4,4) ))
  varleft%real4=varright%real4
endif
!
! --- Real (double precision) ---
if (associated(varright%dbreal0)) then
  allocate(varleft%dbreal0)
  varleft%dbreal0=varright%dbreal0
endif
if (associated(varright%dbreal1)) then
  allocate(varleft%dbreal1( &
                   lbound(varright%dbreal1,1):ubound(varright%dbreal1,1) ))
  varleft%dbreal1=varright%dbreal1
endif
if (associated(varright%dbreal2)) then
  allocate(varleft%dbreal2( &
                   lbound(varright%dbreal2,1):ubound(varright%dbreal2,1), &
                   lbound(varright%dbreal2,2):ubound(varright%dbreal2,2) ))
  varleft%dbreal2=varright%dbreal2
endif
if (associated(varright%dbreal3)) then
  allocate(varleft%dbreal3( &
                   lbound(varright%dbreal3,1):ubound(varright%dbreal3,1), &
                   lbound(varright%dbreal3,2):ubound(varright%dbreal3,2), &
                   lbound(varright%dbreal3,3):ubound(varright%dbreal3,3) ))
  varleft%dbreal3=varright%dbreal3
endif
if (associated(varright%dbreal4)) then
  allocate(varleft%dbreal4( &
                   lbound(varright%dbreal4,1):ubound(varright%dbreal4,1), &
                   lbound(varright%dbreal4,2):ubound(varright%dbreal4,2), &
                   lbound(varright%dbreal4,3):ubound(varright%dbreal4,3), &
                   lbound(varright%dbreal4,4):ubound(varright%dbreal4,4) ))
  varleft%dbreal4=varright%dbreal4
endif
!
! --- Complex (single precision) ---
if (associated(varright%complex0)) then
  allocate(varleft%complex0)
  varleft%complex0=varright%complex0
endif
if (associated(varright%complex1)) then
  allocate(varleft%complex1( &
                   lbound(varright%complex1,1):ubound(varright%complex1,1) ))
  varleft%complex1=varright%complex1
endif
if (associated(varright%complex2)) then
  allocate(varleft%complex2( &
                   lbound(varright%complex2,1):ubound(varright%complex2,1), &
                   lbound(varright%complex2,2):ubound(varright%complex2,2) ))
  varleft%complex2=varright%complex2
endif
if (associated(varright%complex3)) then
  allocate(varleft%complex3( &
                   lbound(varright%complex3,1):ubound(varright%complex3,1), &
                   lbound(varright%complex3,2):ubound(varright%complex3,2), &
                   lbound(varright%complex3,3):ubound(varright%complex3,3) ))
  varleft%complex3=varright%complex3
endif
if (associated(varright%complex4)) then
  allocate(varleft%complex4( &
                   lbound(varright%complex4,1):ubound(varright%complex4,1), &
                   lbound(varright%complex4,2):ubound(varright%complex4,2), &
                   lbound(varright%complex4,3):ubound(varright%complex4,3), &
                   lbound(varright%complex4,4):ubound(varright%complex4,4) ))
  varleft%complex4=varright%complex4
endif
!
! --- Character ---
if (associated(varright%character0)) then
  allocate(varleft%character0)
  varleft%character0=varright%character0
endif
if (associated(varright%character1)) then
  allocate(varleft%character1( &
                 lbound(varright%character1,1):ubound(varright%character1,1) ))
  varleft%character1=varright%character1
endif
if (associated(varright%character2)) then
  allocate(varleft%character2( &
                 lbound(varright%character2,1):ubound(varright%character2,1), &
                 lbound(varright%character2,2):ubound(varright%character2,2) ))
  varleft%character2=varright%character2
endif
if (associated(varright%character3)) then
  allocate(varleft%character3( &
                 lbound(varright%character3,1):ubound(varright%character3,1), &
                 lbound(varright%character3,2):ubound(varright%character3,2), &
                 lbound(varright%character3,3):ubound(varright%character3,3) ))
  varleft%character3=varright%character3
endif
if (associated(varright%character4)) then
  allocate(varleft%character4( &
                 lbound(varright%character4,1):ubound(varright%character4,1), &
                 lbound(varright%character4,2):ubound(varright%character4,2), &
                 lbound(varright%character4,3):ubound(varright%character4,3), &
                 lbound(varright%character4,4):ubound(varright%character4,4) ))
  varleft%character4=varright%character4
endif
! --- Table ---
if (associated(varright%table)) then
  allocate(varleft%table)
  varleft%table=varright%table
endif
!
end subroutine uio_var_copy


!----------************--------------------------------------------------------
subroutine uio_var_cut(var, dimension_cut, outstr, ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_cut ('uio_variable_cut')
!
! PURPOSE:
!   Cut block out of (4D) array
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_cut(var, dimension_cut, outstr=outstr, ierr=ierr)
!
! INPUT:
!   dimension_cut: ('dimension_cut') integer array with new dimension
!                  specification:
!                  ((1_min,1_max),(2_min,2_max),(3_min,3_max),(4_min,4_max))
!                  d_min > d_max means: no cut in this dimension
!
! INPUT/OUTPUT:
!   var: ('variable') uio_var_type, variable to be cut
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_askhead:
!   uio_exkeyw:
!   uio_st2dim:
!
! MODULES:
!   uio_siz_module:
!   uio_nam_module:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Currently restricted to 4D arrays only.
!
! PROCEDURE:
!   Check dimension specifications.
!   If a cut has to be performed, allocate new smaller array,
!   copy extracted data into it, deallocate old data, and redirect pointer.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12. 3.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, dimension(:,:), intent(in)     :: dimension_cut
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: ndim_box, ndim, ndim_shift, nkind
integer                                 :: n
integer, dimension(7)                   :: ilow, iup, ilow_new, iup_new, &
                                           ilow_shift, iup_shift
integer, dimension(2,7)                 :: dimension
character*(let)                         :: type, ident, format, shistr
character*(let)                         :: termt(2,nttmx)
integer                                 :: ntt
integer                                 :: ierr0
character*(80)                          :: outstr0
integer,              pointer, dimension(:)       :: integer1
integer,              pointer, dimension(:,:)     :: integer2
integer,              pointer, dimension(:,:,:)   :: integer3
integer,              pointer, dimension(:,:,:,:) :: integer4
real(kind(0.0E0)),    pointer, dimension(:)       :: real1
real(kind(0.0E0)),    pointer, dimension(:,:)     :: real2
real(kind(0.0E0)),    pointer, dimension(:,:,:)   :: real3
real(kind(0.0E0)),    pointer, dimension(:,:,:,:) :: real4
real(kind(0.0D0)),    pointer, dimension(:)       :: dbreal1
real(kind(0.0D0)),    pointer, dimension(:,:)     :: dbreal2
real(kind(0.0D0)),    pointer, dimension(:,:,:)   :: dbreal3
real(kind(0.0D0)),    pointer, dimension(:,:,:,:) :: dbreal4
complex(kind(0.0E0)), pointer, dimension(:)       :: complex1
complex(kind(0.0E0)), pointer, dimension(:,:)     :: complex2
complex(kind(0.0E0)), pointer, dimension(:,:,:)   :: complex3
complex(kind(0.0E0)), pointer, dimension(:,:,:,:) :: complex4
character(uio_var_l), pointer, dimension(:)       :: character1
character(uio_var_l), pointer, dimension(:,:)     :: character2
character(uio_var_l), pointer, dimension(:,:,:)   :: character3
character(uio_var_l), pointer, dimension(:,:,:,:) :: character4
!------------------------------------------------------------------------------
!
! --- Analyse entry header ---
call uio_var_askhead(var, termt=termt, ntt=ntt, format=format, &
                          type=type, nkind=nkind, ident=ident, &
                          ndim=ndim, ilow=ilow, iup=iup)
if (type == lablna) then
  outstr0='label'
  ierr0=0
  goto 900
else if (type == tabna) then
  outstr0='table'
  ierr0=0
  goto 900
endif
! --- Check cut dimension specification ---
if (size(dimension_cut,1) < 2) then
  outstr0='malformatted cut dimension specification'
  ierr0=1
  goto 900
endif
if (size(dimension_cut,2) /= ndim) then
  outstr0='cut and array dimension specification do not match'
  ierr0=2
  goto 900
endif
call uio_exkeyw(termt,ntt, 'ds', shistr)
if (shistr == '') then
  ilow_shift=0
  iup_shift=0
  ndim_shift=ndim
else
  call uio_st2dim(shistr, ilow_shift, iup_shift, ndim=ndim_shift)
  if (ndim_shift /= ndim) then
    outstr0='shift and array dimension specification do not match'
    ierr0=2
    goto 900
  endif
endif
!
! --- Compute new lower and upper dimension ---
where (dimension_cut(1,1:ndim) <= dimension_cut(2,1:ndim))
  ilow_new(1:ndim)=max(ilow(1:ndim), &
                       dimension_cut(1,1:ndim)-ilow_shift(1:ndim))
  iup_new( 1:ndim)=min(iup( 1:ndim), &
                       dimension_cut(2,1:ndim)+iup_shift( 1:ndim))
elsewhere
  ilow_new(1:ndim)=ilow(1:ndim)
  iup_new( 1:ndim)=iup( 1:ndim)
end where
!
! --- Check if lower index is indeed below upper index, ---
! --- otherwise no cutting in this direction            ---
do n=1,ndim
  if (iup_new(n) < ilow_new(n)) then
    ilow_new(n)=ilow(n)
    iup_new(n) =iup(n)
  endif
end do ! n
!
if (maxval(abs(ilow_new(1:ndim)-ilow(1:ndim))+ &
           abs(iup_new( 1:ndim)-iup( 1:ndim))) > 0) then
  ! --- Index range is new ---
  if (type == intena) then
    ! --- 4D integer ---
    allocate(integer4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                      ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    integer4=var%integer4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                          ilow_new(3):iup_new(3), ilow_new(4):iup_new(4))
    deallocate(var%integer4)
    var%integer4=>integer4
    nullify(integer4)
  else if ((type == realna) .and. (nkind == kind(0.0E0))) then
    ! --- 4D single precision real ---
    allocate(real4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                   ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    real4=var%real4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                    ilow_new(3):iup_new(3), ilow_new(4):iup_new(4))
    deallocate(var%real4)
    var%real4=>real4
    nullify(real4)
  else if ((type == realna) .and. (nkind == kind(0.0D0))) then
    ! --- 4D double precision real ---
    allocate(dbreal4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                     ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    dbreal4=var%dbreal4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                        ilow_new(3):iup_new(3), ilow_new(4):iup_new(4))
    deallocate(var%dbreal4)
    var%dbreal4=>dbreal4
    nullify(dbreal4)
  else if (type == compna) then
    ! --- 4D complex ---
    allocate(complex4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                      ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    complex4=var%complex4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                          ilow_new(3):iup_new(3), ilow_new(4):iup_new(4))
    deallocate(var%complex4)
    var%complex4=>complex4
    nullify(complex4)
  else if (type == charna) then
    ! --- 4D character ---
    allocate(character4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                        ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    character4=var%character4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                              ilow_new(3):iup_new(3), ilow_new(4):iup_new(4))
    deallocate(var%character4)
    var%character4=>character4
    nullify(character4)
  endif
endif
!
outstr0='done'
ierr0=0
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_cut


!----------*************-------------------------------------------------------
subroutine uio_var_real(var, nkind, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_real ('uio_variable_real')
!
! PURPOSE:
!   Change type of uio flexibel variable to real.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_real(var, nkind=nkind, outstr=outstr,ierr=ierr)
!
! INPUT:
!   nkind:  ('number_kind') integer, optional, kind of variable, 
!           default:kind(0.0E0)
!
! INPUT/OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be checked
!
! OUTPUT: (optional)
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_askhead:
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   27. 3.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(inout)  :: var
integer,       optional, intent(in)     :: nkind
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: nkind0, i1, i2, i3, i4
integer                                 :: ierr0
character*(80)                          :: outstr0
integer, dimension(7)                   :: ilow, iup
!------------------------------------------------------------------------------
!
! --- Check status ---
if (var%status > 0) then
  !
  if (present(nkind)) then
    nkind0=nkind
  else
    nkind0=kind(0.0E0)
  endif
  !
  call uio_var_askhead(var, ilow=ilow, iup=iup)
  !
  ! --- Give lower and upper boundaries of array ---
  if (nkind0 == kind(0.0E0)) then     ! --- Make single precision real ---
    select case(var%type)
      case(intena)                    ! --- Integer ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%real0)
            var%real0=real(var%integer0, kind=kind(0.0E0))
            deallocate(var%integer0)
          case(1)                         ! --- 1D ---
            allocate(var%real1(ilow(1):iup(1)))
            var%real1=real(var%integer1, kind=kind(0.0E0))
            deallocate(var%integer1)
          case(2)                         ! --- 2D ---
            allocate(var%real2(ilow(1):iup(1), ilow(2):iup(2)))
            var%real2=real(var%integer2, kind=kind(0.0E0))
            deallocate(var%integer2)
          case(3)                         ! --- 3D ---
            allocate(var%real3(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3)))
            var%real3=real(var%integer3, kind=kind(0.0E0))
            deallocate(var%integer3)
          case(4)                         ! --- 4D ---
            allocate(var%real4(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3), ilow(4):iup(4)))
            var%real4=real(var%integer4, kind=kind(0.0E0))
            deallocate(var%integer4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0E0)
      case(realna)                    ! --- Real ---
        select case(var%kind)
          case(kind(0.0E0))               ! --- Single precision ---
            ! --- Do nothing ---
          case(kind(0.0D0))               ! --- Double precision ---
            select case(var%ndim)
              case(0)                         ! --- 0D ---
                allocate(var%real0)
                var%real0=real(var%dbreal0, kind=kind(0.0E0))
                deallocate(var%dbreal0)
              case(1)                         ! --- 1D ---
                allocate(var%real1(ilow(1):iup(1)))
                var%real1=real(var%dbreal1, kind=kind(0.0E0))
                deallocate(var%dbreal1)
              case(2)                         ! --- 2D ---
                allocate(var%real2(ilow(1):iup(1), ilow(2):iup(2)))
                var%real2=real(var%dbreal2, kind=kind(0.0E0))
                deallocate(var%dbreal2)
              case(3)                         ! --- 3D ---
                allocate(var%real3(ilow(1):iup(1), ilow(2):iup(2), &
                                   ilow(3):iup(3)))
                var%real3=real(var%dbreal3, kind=kind(0.0E0))
                deallocate(var%dbreal3)
              case(4)                         ! --- 4D ---
                allocate(var%real4(ilow(1):iup(1), ilow(2):iup(2), &
                                   ilow(3):iup(3), ilow(4):iup(4)))
                var%real4=real(var%dbreal4, kind=kind(0.0E0))
                deallocate(var%dbreal4)
            end select ! var%ndim
            var%kind  =kind(0.0E0)
        end select ! var%kind
      case(compna)                    ! --- Complex ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%real0)
            var%real0=real(var%complex0, kind=kind(0.0E0))
            deallocate(var%complex0)
          case(1)                         ! --- 1D ---
            allocate(var%real1(ilow(1):iup(1)))
            var%real1=real(var%complex1, kind=kind(0.0E0))
            deallocate(var%complex1)
          case(2)                         ! --- 2D ---
            allocate(var%real2(ilow(1):iup(1), ilow(2):iup(2)))
            var%real2=real(var%complex2, kind=kind(0.0E0))
            deallocate(var%complex2)
          case(3)                         ! --- 3D ---
            allocate(var%real3(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3)))
            var%real3=real(var%complex3, kind=kind(0.0E0))
            deallocate(var%complex3)
          case(4)                         ! --- 4D ---
            allocate(var%real4(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3), ilow(4):iup(4)))
            var%real4=real(var%complex4, kind=kind(0.0E0))
            deallocate(var%complex4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0E0)
      case(charna)                    ! --- Character ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%real0)
            read(var%character0,*) var%real0
            deallocate(var%character0)
          case(1)                         ! --- 1D ---
            allocate(var%real1(ilow(1):iup(1)))
            do i1=ilow(1),iup(1)
              read(var%character1(i1),*) var%real1(i1)
            end do ! i1
            deallocate(var%character1)
          case(2)                         ! --- 2D ---
            allocate(var%real2(ilow(1):iup(1), ilow(2):iup(2)))
            do i2=ilow(2),iup(2)
              do i1=ilow(1),iup(1)
                read(var%character2(i1,i2),*) var%real2(i1,i2)
              end do ! i1
            end do ! i2
            deallocate(var%character2)
          case(3)                         ! --- 3D ---
            allocate(var%real3(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3)))
            do i3=ilow(3),iup(3)
              do i2=ilow(2),iup(2)
                do i1=ilow(1),iup(1)
                  read(var%character3(i1,i2,i3),*) var%real3(i1,i2,i3)
                end do ! i1
              end do ! i2
            end do ! i3
            deallocate(var%character3)
          case(4)                         ! --- 4D ---
            allocate(var%real4(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3), ilow(4):iup(4)))
            do i4=ilow(4),iup(4)
              do i3=ilow(3),iup(3)
                do i2=ilow(2),iup(2)
                  do i1=ilow(1),iup(1)
                    read(var%character4(i1,i2,i3,4),*) var%real4(i1,i2,i3,4)
                  end do ! i1
                end do ! i2
              end do ! i3
            end do ! i4
            deallocate(var%character4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0E0)
    end select ! var%type
  else                              ! --- Make double precision real ----------
    select case(var%type)
      case(intena)                    ! --- Integer ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%dbreal0)
            var%dbreal0=real(var%integer0, kind=kind(0.0D0))
            deallocate(var%integer0)
          case(1)                         ! --- 1D ---
            allocate(var%dbreal1(ilow(1):iup(1)))
            var%dbreal1=real(var%integer1, kind=kind(0.0D0))
            deallocate(var%integer1)
          case(2)                         ! --- 2D ---
            allocate(var%dbreal2(ilow(1):iup(1), ilow(2):iup(2)))
            var%dbreal2=real(var%integer2, kind=kind(0.0D0))
            deallocate(var%integer2)
          case(3)                         ! --- 3D ---
            allocate(var%dbreal3(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3)))
            var%dbreal3=real(var%integer3, kind=kind(0.0D0))
            deallocate(var%integer3)
          case(4)                         ! --- 4D ---
            allocate(var%dbreal4(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3), ilow(4):iup(4)))
            var%dbreal4=real(var%integer4, kind=kind(0.0D0))
            deallocate(var%integer4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0D0)
      case(realna)                    ! --- Real ---
        select case(var%kind)
          case(kind(0.0E0))               ! --- Single precision ---
            select case(var%ndim)
              case(0)                         ! --- 0D ---
                allocate(var%dbreal0)
                var%dbreal0=real(var%real0, kind=kind(0.0D0))
                deallocate(var%real0)
              case(1)                         ! --- 1D ---
                allocate(var%dbreal1(ilow(1):iup(1)))
                var%dbreal1=real(var%real1, kind=kind(0.0D0))
                deallocate(var%real1)
              case(2)                         ! --- 2D ---
                allocate(var%dbreal2(ilow(1):iup(1), ilow(2):iup(2)))
                var%dbreal2=real(var%real2, kind=kind(0.0D0))
                deallocate(var%real2)
              case(3)                         ! --- 3D ---
                allocate(var%dbreal3(ilow(1):iup(1), ilow(2):iup(2), &
                                     ilow(3):iup(3)))
                var%dbreal3=real(var%real3, kind=kind(0.0D0))
                deallocate(var%real3)
              case(4)                         ! --- 4D ---
                allocate(var%dbreal4(ilow(1):iup(1), ilow(2):iup(2), &
                                     ilow(3):iup(3), ilow(4):iup(4)))
                var%dbreal4=real(var%real4, kind=kind(0.0D0))
                deallocate(var%real4)
            end select ! var%ndim
            var%kind  =kind(0.0D0)
          case(kind(0.0D0))               ! --- Double precision ---
            ! --- Do nothing ---
        end select ! var%kind
      case(compna)                    ! --- Complex ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%dbreal0)
            var%dbreal0=real(var%complex0, kind=kind(0.0D0))
            deallocate(var%complex0)
          case(1)                         ! --- 1D ---
            allocate(var%dbreal1(ilow(1):iup(1)))
            var%dbreal1=real(var%complex1, kind=kind(0.0D0))
            deallocate(var%complex1)
          case(2)                         ! --- 2D ---
            allocate(var%dbreal2(ilow(1):iup(1), ilow(2):iup(2)))
            var%dbreal2=real(var%complex2, kind=kind(0.0D0))
            deallocate(var%complex2)
          case(3)                         ! --- 3D ---
            allocate(var%dbreal3(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3)))
            var%dbreal3=real(var%complex3, kind=kind(0.0D0))
            deallocate(var%complex3)
          case(4)                         ! --- 4D ---
            allocate(var%dbreal4(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3), ilow(4):iup(4)))
            var%dbreal4=real(var%complex4, kind=kind(0.0D0))
            deallocate(var%complex4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0D0)
      case(charna)                    ! --- Character ---
        select case(var%ndim)
          case(0)                         ! --- 0D ---
            allocate(var%dbreal0)
            read(var%character0,*) var%dbreal0
            deallocate(var%character0)
          case(1)                         ! --- 1D ---
            allocate(var%dbreal1(ilow(1):iup(1)))
            do i1=ilow(1),iup(1)
              read(var%character1(i1),*) var%dbreal1(i1)
            end do ! i1
            deallocate(var%character1)
          case(2)                         ! --- 2D ---
            allocate(var%dbreal2(ilow(1):iup(1), ilow(2):iup(2)))
            do i2=ilow(2),iup(2)
              do i1=ilow(1),iup(1)
                read(var%character2(i1,i2),*) var%dbreal2(i1,i2)
              end do ! i1
            end do ! i2
            deallocate(var%character2)
          case(3)                         ! --- 3D ---
            allocate(var%dbreal3(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3)))
            do i3=ilow(3),iup(3)
              do i2=ilow(2),iup(2)
                do i1=ilow(1),iup(1)
                  read(var%character3(i1,i2,i3),*) var%dbreal3(i1,i2,i3)
                end do ! i1
              end do ! i2
            end do ! i3
            deallocate(var%character3)
          case(4)                         ! --- 4D ---
            allocate(var%dbreal4(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3), ilow(4):iup(4)))
            do i4=ilow(4),iup(4)
              do i3=ilow(3),iup(3)
                do i2=ilow(2),iup(2)
                  do i1=ilow(1),iup(1)
                    read(var%character4(i1,i2,i3,4),*) var%dbreal4(i1,i2,i3,4)
                  end do ! i1
                end do ! i2
              end do ! i3
            end do ! i4
            deallocate(var%character4)
        end select ! var%ndim
        var%format=''
        var%type  =realna
        var%kind  =kind(0.0D0)
    end select ! var%type
  endif
  !
  outstr0='done'
  ierr0=0
else
  outstr0='undefined variable'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_real


!----------**************------------------------------------------------------
subroutine uio_var_3Dto4D(var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_3Dto4D ('uio_variable_3D_to_4D')
!
! PURPOSE:
!   Transform array in uio flexibel variable from 3D (or scalar) to 4D.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_3Dto4D(var, outstr=outstr,ierr=ierr)
!
! INPUT:
!
! INPUT/OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable to be checked
!
! OUTPUT: (optional)
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_askhead:
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   None
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   11.10.01 Written by Bernd Freytag, Astronomical Observatory Uppsala
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(inout)  :: var
!integer,       optional, intent(in)     :: makereal
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: i1,i2,i3,i4
!integer                                 :: makereal0
integer                                 :: ierr0
character*(80)                          :: outstr0
integer, dimension(7)                   :: ilow, iup
!------------------------------------------------------------------------------
!
! --- Arbitrary indices for additional dimensions ---
i1=1
i2=1
i3=1
i4=1
!
!!$if (present(makereal)) then
!!$  makereal0=makereal
!!$else
!!$  makereal0=0
!!$endif
!
! --- Check status ---
if (var%status > 0) then
  !
  if (var%ndim == 0) then
    ! --- Transform Scalars ---
    !
    var%ndim=4
    !
    select case(var%type)
      case(intena)                                            ! --- Integer ---
!!$        if (makereal0 == 1) then
!!$          allocate(var%real4(i1:i1,i2:i2,i3:i3,i4:i4))
!!$          var%real4(i1,i2,i3,i4)=real(var%integer0)
!!$          var%type=realna
!!$          var%nkind=kind(0.0E+00)
!!$        else
          allocate(var%integer4(i1:i1,i2:i2,i3:i3,i4:i4))
          var%integer4(i1,i2,i3,i4)=var%integer0
!!$        endif
        deallocate(var%integer0)
      case(realna)                                            ! --- Real ---
        select case(var%kind)
          case(kind(0.0E0))                                   ! --- Single precision ---
            allocate(var%real4(i1:i1,i2:i2,i3:i3,i4:i4))
            var%real4(i1,i2,i3,i4)=var%real0
            deallocate(var%real0)
          case(kind(0.0D0))                                   ! --- Single precision ---
            allocate(var%dbreal4(i1:i1,i2:i2,i3:i3,i4:i4))
            var%dbreal4(i1,i2,i3,i4)=var%dbreal0
            deallocate(var%dbreal0)
        end select ! var%kind
      case(compna)                                            ! --- Complex ---
        allocate(var%complex4(i1:i1,i2:i2,i3:i3,i4:i4))
        var%complex4(i1,i2,i3,i4)=var%complex0
        deallocate(var%complex0)
      case(charna)                                            ! --- Character ---
        allocate(var%character4(i1:i1,i2:i2,i3:i3,i4:i4))
        var%character4(i1,i2,i3,i4)=var%character0
        deallocate(var%character0)
    end select ! var%type
    !
  else if (var%ndim == 3) then
    ! --- Transform 3D arrays ---
    !
    call uio_var_askhead(var, ilow=ilow, iup=iup)
    !
    var%ndim=4
    !
    select case(var%type)
      case(intena)                                            ! --- Integer ---
        allocate(var%integer4(ilow(1):iup(1), ilow(2):iup(2), &
                              ilow(3):iup(3), i4:i4))
        var%integer4(:,:,:,i4)=var%integer3
        deallocate(var%integer3)
      case(realna)                                            ! --- Real ---
        select case(var%kind)
          case(kind(0.0E0))                                   ! --- Single precision ---
            allocate(var%real4(ilow(1):iup(1), ilow(2):iup(2), &
                               ilow(3):iup(3), i4:i4))
            var%real4(:,:,:,i4)=var%real3
            deallocate(var%real3)
          case(kind(0.0D0))                                   ! --- Single precision ---
            allocate(var%dbreal4(ilow(1):iup(1), ilow(2):iup(2), &
                                 ilow(3):iup(3), i4:i4))
            var%dbreal4(:,:,:,i4)=var%dbreal3
            deallocate(var%dbreal3)
        end select ! var%kind
      case(compna)                                            ! --- Complex ---
        allocate(var%complex4(ilow(1):iup(1), ilow(2):iup(2), &
                              ilow(3):iup(3), i4:i4))
        var%complex4(:,:,:,i4)=var%complex3
        deallocate(var%complex3)
      case(charna)                                            ! --- Character ---
        allocate(var%character4(ilow(1):iup(1), ilow(2):iup(2), &
                                ilow(3):iup(3), i4:i4))
        var%character4(:,:,:,i4)=var%character3
        deallocate(var%character3)
    end select ! var%type
  endif
  !
  outstr0='done'
  ierr0=0
else
  outstr0='undefined variable'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_3Dto4D


!----------************--------------------------------------------------------
subroutine uio_var_glue(var, dimension_glue, outstr, ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_glue ('uio_variable_glue')
!
! PURPOSE:
!   Enlarge (4D) array in uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_glue(var, dimension_glue, outstr=outstr, ierr=ierr)
!
! INPUT:
!   dimension_glue: ('dimension_glue') integer array with additional dimension
!                   specification:
!                  ((1_min,1_max),(2_min,2_max),(3_min,3_max),(4_min,4_max))
!                  d_min > d_max means: no glue in this dimension
!
! INPUT/OUTPUT:
!   var: ('variable') uio_var_type, variable to be expanded
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_var_askhead:
!   uio_exkeyw:
!   uio_st2dim:
!
! MODULES:
!   uio_siz_module:
!   uio_nam_module:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Currently restricted to 4D arrays only.
!
! PROCEDURE:
!   Check dimension specifications.
!   If a glueing has to be performed, allocate new larger array,
!   fill it with zeros,
!   copy old data into it, deallocate old data, and redirect pointer.
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12. 3.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, dimension(:,:), intent(in)     :: dimension_glue
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: ndim_box, ndim, ndim_shift, nkind
integer                                 :: n
integer, dimension(7)                   :: ilow, iup, ilow_new, iup_new, &
                                           ilow_shift, iup_shift
integer, dimension(2,7)                 :: dimension
character*(let)                         :: type, ident, format, shistr
character*(let)                         :: termt(2,nttmx)
integer                                 :: ntt
integer                                 :: ierr0
character*(80)                          :: outstr0
integer,              pointer, dimension(:)       :: integer1
integer,              pointer, dimension(:,:)     :: integer2
integer,              pointer, dimension(:,:,:)   :: integer3
integer,              pointer, dimension(:,:,:,:) :: integer4
real(kind(0.0E0)),    pointer, dimension(:)       :: real1
real(kind(0.0E0)),    pointer, dimension(:,:)     :: real2
real(kind(0.0E0)),    pointer, dimension(:,:,:)   :: real3
real(kind(0.0E0)),    pointer, dimension(:,:,:,:) :: real4
real(kind(0.0D0)),    pointer, dimension(:)       :: dbreal1
real(kind(0.0D0)),    pointer, dimension(:,:)     :: dbreal2
real(kind(0.0D0)),    pointer, dimension(:,:,:)   :: dbreal3
real(kind(0.0D0)),    pointer, dimension(:,:,:,:) :: dbreal4
complex(kind(0.0E0)), pointer, dimension(:)       :: complex1
complex(kind(0.0E0)), pointer, dimension(:,:)     :: complex2
complex(kind(0.0E0)), pointer, dimension(:,:,:)   :: complex3
complex(kind(0.0E0)), pointer, dimension(:,:,:,:) :: complex4
character(uio_var_l), pointer, dimension(:)       :: character1
character(uio_var_l), pointer, dimension(:,:)     :: character2
character(uio_var_l), pointer, dimension(:,:,:)   :: character3
character(uio_var_l), pointer, dimension(:,:,:,:) :: character4
!------------------------------------------------------------------------------
!
! --- Analyse entry header ---
call uio_var_askhead(var, termt=termt, ntt=ntt, format=format, &
                          type=type, nkind=nkind, ident=ident, &
                          ndim=ndim, ilow=ilow, iup=iup)
if (type == lablna) then
  outstr0='label'
  ierr0=0
  goto 900
else if (type == tabna) then
  outstr0='table'
  ierr0=0
  goto 900
endif
! --- Check cut dimension specification ---
if (size(dimension_glue,1) < 2) then
  outstr0='malformatted glue dimension specification'
  ierr0=1
  goto 900
endif
if (size(dimension_glue,2) /= ndim) then
  outstr0='glue and array dimension specification do not match'
  ierr0=2
  goto 900
endif
!call uio_exkeyw(termt,ntt, 'ds', shistr)
!if (shistr == '') then
!  ilow_shift=0
!  iup_shift=0
!  ndim_shift=ndim
!else
!  call uio_st2dim(shistr, ilow_shift, iup_shift, ndim=ndim_shift)
!  if (ndim_shift /= ndim) then
!    outstr0='shift and array dimension specification do not match'
!    ierr0=2
!    goto 900
!  endif
!endif
!
! --- Compute new lower and upper dimension ---
ilow_new(1:ndim)=ilow(1:ndim)-max(dimension_glue(1,1:ndim),0)
iup_new( 1:ndim)=iup( 1:ndim)+max(dimension_glue(2,1:ndim),0)
!
if (maxval(abs(ilow_new(1:ndim)-ilow(1:ndim))+ &
           abs(iup_new( 1:ndim)-iup( 1:ndim))) > 0) then
  ! --- Index range is new ---
  if (type == intena) then
    ! --- 4D integer ---
    allocate(integer4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                      ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    integer4=0
    integer4(ilow(1):iup(1), ilow(2):iup(2), ilow(3):iup(3), ilow(4):iup(4)) &
            =var%integer4
    deallocate(var%integer4)
    var%integer4=>integer4
    nullify(integer4)
  else if ((type == realna) .and. (nkind == kind(0.0E0))) then
    ! --- 4D single precision real ---
    allocate(real4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                   ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    real4=0.0E0
    real4(ilow(1):iup(1), ilow(2):iup(2), ilow(3):iup(3), ilow(4):iup(4)) &
         =var%real4
    deallocate(var%real4)
    var%real4=>real4
    nullify(real4)
  else if ((type == realna) .and. (nkind == kind(0.0D0))) then
    ! --- 4D double precision real ---
    allocate(dbreal4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                     ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    dbreal4=0.0D0
    dbreal4(ilow(1):iup(1), ilow(2):iup(2), ilow(3):iup(3), ilow(4):iup(4)) &
           =var%dbreal4
    deallocate(var%dbreal4)
    var%dbreal4=>dbreal4
    nullify(dbreal4)
  else if (type == compna) then
    ! --- 4D complex ---
    allocate(complex4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                      ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    complex4=0.0E0
    complex4(ilow(1):iup(1), ilow(2):iup(2), ilow(3):iup(3), ilow(4):iup(4)) &
            =var%complex4
    deallocate(var%complex4)
    var%complex4=>complex4
    nullify(complex4)
  else if (type == charna) then
    ! --- 4D character ---
    allocate(character4(ilow_new(1):iup_new(1), ilow_new(2):iup_new(2), &
                        ilow_new(3):iup_new(3), ilow_new(4):iup_new(4)))
    character4=''
    character4(ilow(1):iup(1), ilow(2):iup(2), ilow(3):iup(3), ilow(4):iup(4)) &
              =var%character4
    deallocate(var%character4)
    var%character4=>character4
    nullify(character4)
  endif
endif
!
outstr0='done'
ierr0=0
!
900 if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_glue


!----------**************------------------------------------------------------
subroutine uio_var_delete(var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_delete ('uio_variable_delete')
!
! PURPOSE:
!   Delete uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_delete(var, outstr=outstr,ierr=ierr)
!
! INPUT/OUTPUT:
!   var:    ('variable') uio_var_type
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   Pointers are deallocated.
!
! RESTRICTIONS:
!   Pointers are deallocated. Watch for side effects.
!
! PROCEDURE:
!   Character variables are set to empty strings.
!   Integers are zet to zero.
!   Pointers are deallocated.
!
! EXAMPLE:
!   type(uio_var_type) :: a
!   call uio_var_delete(a)
!
! MODIFICATION HISTORY:
!   14. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(inout)  :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
!------------------------------------------------------------------------------
!
! --- Delete header: set variables to zero or empty strings, respectively ---
var%type  =''
var%ident =''
var%format=''
var%kind  =0
var%status=0
var%ntt   =0
var%ndim  =0
!
! --- Delete data: deallocate pointers ---
if (associated(var%integer0)) deallocate(var%integer0)
if (associated(var%integer1)) deallocate(var%integer1)
if (associated(var%integer2)) deallocate(var%integer2)
if (associated(var%integer3)) deallocate(var%integer3)
if (associated(var%integer4)) deallocate(var%integer4)
if (associated(var%real0)) deallocate(var%real0)
if (associated(var%real1)) deallocate(var%real1)
if (associated(var%real2)) deallocate(var%real2)
if (associated(var%real3)) deallocate(var%real3)
if (associated(var%real4)) deallocate(var%real4)
if (associated(var%dbreal0)) deallocate(var%dbreal0)
if (associated(var%dbreal1)) deallocate(var%dbreal1)
if (associated(var%dbreal2)) deallocate(var%dbreal2)
if (associated(var%dbreal3)) deallocate(var%dbreal3)
if (associated(var%dbreal4)) deallocate(var%dbreal4)
if (associated(var%complex0)) deallocate(var%complex0)
if (associated(var%complex1)) deallocate(var%complex1)
if (associated(var%complex2)) deallocate(var%complex2)
if (associated(var%complex3)) deallocate(var%complex3)
if (associated(var%complex4)) deallocate(var%complex4)
if (associated(var%character0)) deallocate(var%character0)
if (associated(var%character1)) deallocate(var%character1)
if (associated(var%character2)) deallocate(var%character2)
if (associated(var%character3)) deallocate(var%character3)
if (associated(var%character4)) deallocate(var%character4)
! --- Table (pointers in table?) ---
!if (associated(var%table)) deallocate(var%table)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_var_delete
!
!----------************--------------------------------------------------------
subroutine uio_var_null(var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_null ('uio_variable_null')
!
! PURPOSE:
!   Initialization procedure for uio flexible variable routines:
!   Set uio flexible variable to zero (and pointers to defined initial status).
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_null(var, outstr=outstr,ierr=ierr)
!
! INPUT/OUTPUT:
!   var:    ('variable') uio_var_type
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!   Do not use this routine for already defined variables.
!   Pointers are nullified and not deallocated.
!
! PROCEDURE:
!   Character variables are set to empty strings.
!   Integers are zet to zero.
!   Pointers are nullified.
!
! EXAMPLE:
!   type(uio_var_type) :: a
!   call uio_var_null(a)
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(inout)  :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
!------------------------------------------------------------------------------
!
! --- Delete header: set variables to zero or empty strings, respectively ---
var%type  =''
var%ident =''
var%format=''
var%kind  =0
var%status=0
var%ntt   =0
var%ndim  =0
!
! --- Delete data: nullify pointers ---
nullify(var%integer0)
nullify(var%integer1)
nullify(var%integer2)
nullify(var%integer3)
nullify(var%integer4)
nullify(var%real0)
nullify(var%real1)
nullify(var%real2)
nullify(var%real3)
nullify(var%real4)
nullify(var%dbreal0)
nullify(var%dbreal1)
nullify(var%dbreal2)
nullify(var%dbreal3)
nullify(var%dbreal4)
nullify(var%complex0)
nullify(var%complex1)
nullify(var%complex2)
nullify(var%complex3)
nullify(var%complex4)
nullify(var%character0)
nullify(var%character1)
nullify(var%character2)
nullify(var%character3)
nullify(var%character4)
nullify(var%table)
!
if (present(outstr)) outstr='done'
if (present(ierr)) ierr=0
!
end subroutine uio_var_null
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_geti0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_geti0 ('uio_variable_get_integer_0')
!
! PURPOSE:
!   Give out data set from uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_geti0(var,val)
!
! INPUT:
!   var:    ('variable') uio_var_type
!
! OUTPUT:
!   val:    ('value') integer, real, complex, character; scalar or array
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   13. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, intent(out)                    :: val
!------------------------------------------------------------------------------
val=var%integer0
!
end subroutine uio_var_geti0
!
!----------*************-------------------------------------------------------
subroutine uio_var_geti1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_geti1 ('uio_variable_get_integer_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:), intent(out)      :: val
!------------------------------------------------------------------------------
val=var%integer1
!
end subroutine uio_var_geti1
!
!----------*************-------------------------------------------------------
subroutine uio_var_geti2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_geti2 ('uio_variable_get_integer_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:), intent(out)    :: val
!------------------------------------------------------------------------------
val=var%integer2
!
end subroutine uio_var_geti2
!
!----------*************-------------------------------------------------------
subroutine uio_var_geti3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_geti3 ('uio_variable_get_integer_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:,:), intent(out)  :: val
!------------------------------------------------------------------------------
val=var%integer3
!
end subroutine uio_var_geti3
!
!----------*************-------------------------------------------------------
subroutine uio_var_geti4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_geti4 ('uio_variable_get_integer_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:,:,:), intent(out):: val
!------------------------------------------------------------------------------
val=var%integer4
!
end subroutine uio_var_geti4
!
!----------*************-------------------------------------------------------
subroutine uio_var_getr0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getr0 ('uio_variable_get_real0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), intent(out)          :: val
!------------------------------------------------------------------------------
val=var%real0
!
end subroutine uio_var_getr0
!
!----------*************-------------------------------------------------------
subroutine uio_var_getr1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getr1 ('uio_variable_get_real_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%real1
!
end subroutine uio_var_getr1
!
!----------*************-------------------------------------------------------
subroutine uio_var_getr2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getr2 ('uio_variable_get_real_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%real2
!
end subroutine uio_var_getr2
!
!----------*************-------------------------------------------------------
subroutine uio_var_getr3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getr3 ('uio_variable_get_real_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:,:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%real3
!
end subroutine uio_var_getr3
!
!----------*************-------------------------------------------------------
subroutine uio_var_getr4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getr4 ('uio_variable_get_real_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:,:,:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%real4
!
end subroutine uio_var_getr4
!
!----------*************-------------------------------------------------------
subroutine uio_var_getd0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getd0 ('uio_variable_get_double_real0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), intent(out)          :: val
!------------------------------------------------------------------------------
val=var%dbreal0
!
end subroutine uio_var_getd0
!
!----------*************-------------------------------------------------------
subroutine uio_var_getd1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getd1 ('uio_variable_get_double_real_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%dbreal1
!
end subroutine uio_var_getd1
!
!----------*************-------------------------------------------------------
subroutine uio_var_getd2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getd2 ('uio_variable_get_double_real_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%dbreal2
!
end subroutine uio_var_getd2
!
!----------*************-------------------------------------------------------
subroutine uio_var_getd3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getd3 ('uio_variable_get_double_real_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%dbreal3
!
end subroutine uio_var_getd3
!
!----------*************-------------------------------------------------------
subroutine uio_var_getd4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getd4 ('uio_variable_get_double_real_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:,:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%dbreal4
!
end subroutine uio_var_getd4
!
!----------*************-------------------------------------------------------
subroutine uio_var_getc0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getc0 ('uio_variable_get_complex0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), intent(out)       :: val
!------------------------------------------------------------------------------
val=var%complex0
!
end subroutine uio_var_getc0
!
!----------*************-------------------------------------------------------
subroutine uio_var_getc1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getc1 ('uio_variable_get_complex_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%complex1
!
end subroutine uio_var_getc1
!
!----------*************-------------------------------------------------------
subroutine uio_var_getc2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getc2 ('uio_variable_get_complex_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%complex2
!
end subroutine uio_var_getc2
!
!----------*************-------------------------------------------------------
subroutine uio_var_getc3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getc3 ('uio_variable_get_complex_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%complex3
!
end subroutine uio_var_getc3
!
!----------*************-------------------------------------------------------
subroutine uio_var_getc4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_getc4 ('uio_variable_get_complex_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:,:,:), &
                             intent(out):: val
!------------------------------------------------------------------------------
val=var%complex4
!
end subroutine uio_var_getc4
!
!----------*************-------------------------------------------------------
subroutine uio_var_gets0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_gets0 ('uio_variable_get_string_0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(*), intent(out)               :: val
!------------------------------------------------------------------------------
val=var%character0
!
end subroutine uio_var_gets0
!
!----------*************-------------------------------------------------------
subroutine uio_var_gets1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_gets1 ('uio_variable_get_string_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(*), dimension(:), intent(out) :: val
!------------------------------------------------------------------------------
val=var%character1
!
end subroutine uio_var_gets1
!
!----------*************-------------------------------------------------------
subroutine uio_var_gets2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_gets2 ('uio_variable_get_string_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(*), dimension(:,:), &
                           intent(out)  :: val
!------------------------------------------------------------------------------
val=var%character2
!
end subroutine uio_var_gets2
!
!----------*************-------------------------------------------------------
subroutine uio_var_gets3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_gets3 ('uio_variable_get_string_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(*), dimension(:,:,:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%character3
!
end subroutine uio_var_gets3
!
!----------*************-------------------------------------------------------
subroutine uio_var_gets4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_gets4 ('uio_variable_get_string_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(*), dimension(:,:,:,:), &
                            intent(out) :: val
!------------------------------------------------------------------------------
val=var%character4
!
end subroutine uio_var_gets4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgeti0(var,val)
!------------------------------------------------------------------------------
! NAME:0
!   uio_var_pointgeti0 ('uio_variable_point_get_integer_0')
!
! PURPOSE:
!   Point to data set in uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_pointgeti0(var,val)
!
! INPUT:
!   var:    ('variable') uio_var_type
!
! OUTPUT:
!   val:    ('value') integer, real, complex, character; scalar or array
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   13. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, pointer                        :: val
!------------------------------------------------------------------------------
val=>var%integer0
!
end subroutine uio_var_pointgeti0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgeti1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgeti1 ('uio_variable_point_get_integer_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:), pointer          :: val
!------------------------------------------------------------------------------
val=>var%integer1
!
end subroutine uio_var_pointgeti1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgeti2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgeti2 ('uio_variable_point_get_integer_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:), pointer        :: val
!------------------------------------------------------------------------------
val=>var%integer2
!
end subroutine uio_var_pointgeti2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgeti3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgeti3 ('uio_variable_point_get_integer_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:,:), pointer      :: val
!------------------------------------------------------------------------------
val=>var%integer3
!
end subroutine uio_var_pointgeti3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgeti4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgeti4 ('uio_variable_point_get_integer_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
integer, dimension(:,:,:,:), pointer    :: val
!------------------------------------------------------------------------------
val=>var%integer4
!
end subroutine uio_var_pointgeti4
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetr0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetr0 ('uio_variable_point_get_real0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), pointer              :: val
!------------------------------------------------------------------------------
val=>var%real0
!
end subroutine uio_var_pointgetr0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetr1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetr1 ('uio_variable_point_get_real_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:), pointer:: val
!------------------------------------------------------------------------------
val=>var%real1
!
end subroutine uio_var_pointgetr1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetr2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetr2 ('uio_variable_point_get_real_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%real2
!
end subroutine uio_var_pointgetr2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetr3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetr3 ('uio_variable_point_get_real_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%real3
!
end subroutine uio_var_pointgetr3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetr4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetr4 ('uio_variable_point_get_real_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0E0)), dimension(:,:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%real4
!
end subroutine uio_var_pointgetr4
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetd0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetd0 ('uio_variable_point_get_double_real0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), pointer              :: val
!------------------------------------------------------------------------------
val=>var%dbreal0
!
end subroutine uio_var_pointgetd0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetd1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetd1 ('uio_variable_point_get_double_real_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:), pointer:: val
!------------------------------------------------------------------------------
val=>var%dbreal1
!
end subroutine uio_var_pointgetd1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetd2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetd2 ('uio_variable_point_get_double_real_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%dbreal2
!
end subroutine uio_var_pointgetd2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetd3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetd3 ('uio_variable_point_get_double_real_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%dbreal3
!
end subroutine uio_var_pointgetd3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetd4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetd4 ('uio_variable_point_get_double_real_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
real(kind(0.0D0)), dimension(:,:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%dbreal4
!
end subroutine uio_var_pointgetd4
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetc0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetc0 ('uio_variable_point_get_complex0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), pointer           :: val
!------------------------------------------------------------------------------
val=>var%complex0
!
end subroutine uio_var_pointgetc0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetc1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetc1 ('uio_variable_point_get_complex_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%complex1
!
end subroutine uio_var_pointgetc1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetc2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetc2 ('uio_variable_point_get_complex_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%complex2
!
end subroutine uio_var_pointgetc2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetc3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetc3 ('uio_variable_point_get_complex_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%complex3
!
end subroutine uio_var_pointgetc3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgetc4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgetc4 ('uio_variable_point_get_complex_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
complex(kind(0.0E0)), dimension(:,:,:,:), &
                                 pointer:: val
!------------------------------------------------------------------------------
val=>var%complex4
!
end subroutine uio_var_pointgetc4
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgets0(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgets0 ('uio_variable_point_get_string_0')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(uio_var_l), pointer           :: val
!------------------------------------------------------------------------------
val=>var%character0
!
end subroutine uio_var_pointgets0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgets1(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgets1 ('uio_variable_point_get_string_1')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(uio_var_l), dimension(:), &
                             pointer    :: val
!------------------------------------------------------------------------------
val=>var%character1
!
end subroutine uio_var_pointgets1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgets2(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgets2 ('uio_variable_point_get_string_2')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(uio_var_l), dimension(:,:), &
                               pointer  :: val
!------------------------------------------------------------------------------
val=>var%character2
!
end subroutine uio_var_pointgets2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgets3(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgets3 ('uio_variable_point_get_string_3')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(uio_var_l), dimension(:,:,:), &
                                pointer :: val
!------------------------------------------------------------------------------
val=>var%character3
!
end subroutine uio_var_pointgets3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointgets4(var,val)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointgets4 ('uio_variable_point_get_string_4')
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type), intent(in)          :: var
character(uio_var_l), dimension(:,:,:,:), &
                                pointer :: val
!------------------------------------------------------------------------------
val=>var%character4
!
end subroutine uio_var_pointgets4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputi0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputi0 ('uio_variable_point_put_integer_0')
!
! PURPOSE:
!   Put data set into uio flexible variable via pointing.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_pointputi0(var,val)
!
! INPUT:
!   val:    ('value') scalar or array to be put into uio flexible variable
!   ident:  ('identifier') character, optional, identifier
!   format: ('format') character, optional, format string
!
! OUTPUT:
!   var:    ('variable') uio_var_type
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   14. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, pointer                        :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%integer0=>val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_pointputi0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputi1(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputi1 ('uio_variable_point_put_integer_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, pointer, dimension(:)          :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%integer1=>val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_pointputi1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputi2(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputi2 ('uio_variable_point_put_integer_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, pointer, dimension(:,:)        :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%integer2=>val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_pointputi2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputi3(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputi3 ('uio_variable_point_put_integer_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, pointer, dimension(:,:,:)      :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%integer3=>val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_pointputi3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputi4(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputi4 ('uio_variable_point_put_integer_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, pointer, dimension(:,:,:,:)    :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%integer4=>val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_pointputi4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputr0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputr0 ('uio_variable_point_put_real_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), pointer              :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%real0=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_pointputr0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputr1(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputr1 ('uio_variable_point_put_real_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), pointer, &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%real1=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_pointputr1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputr2(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputr2 ('uio_variable_point_put_real_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), pointer, &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%real2=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_pointputr2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputr3(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputr3 ('uio_variable_point_put_real_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), pointer, &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%real3=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_pointputr3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputr4(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputr4 ('uio_variable_point_put_real_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), pointer, &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%real4=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_pointputr4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputd0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputd0 ('uio_variable_point_put_real_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), pointer              :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%dbreal0=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_pointputd0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputd1(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputd1 ('uio_variable_point_put_real_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), pointer, &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%dbreal1=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_pointputd1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputd2(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputd2 ('uio_variable_point_put_real_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), pointer, &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%dbreal2=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_pointputd2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputd3(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputd3 ('uio_variable_point_put_real_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), pointer, &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%dbreal3=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_pointputd3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputd4(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputd4 ('uio_variable_point_put_real_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), pointer, &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%dbreal4=>val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_pointputd4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputc0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputc0 ('uio_variable_point_put_complex_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), pointer           :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%complex0=>val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_pointputc0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputc1(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputc1 ('uio_variable_point_put_complex_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), pointer, &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%complex1=>val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_pointputc1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputc2(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputc2 ('uio_variable_point_put_complex_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), pointer, &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%complex2=>val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_pointputc2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputc3(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputc3 ('uio_variable_point_put_complex_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), pointer, &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%complex3=>val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_pointputc3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputc4(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputc4 ('uio_variable_point_put_complex_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), pointer, &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%complex4=>val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_pointputc4
!
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputs0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputs0 ('uio_variable_point_put_string_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character(uio_var_l), pointer           :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%character0=>val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_pointputs0
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputs1(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputs1 ('uio_variable_point_put_string_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character(uio_var_l), pointer, &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%character1=>val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_pointputs1
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputs2(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputs2 ('uio_variable_point_put_string_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character(uio_var_l), pointer, &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%character2=>val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_pointputs2
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputs3(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputs3 ('uio_variable_point_put_string_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character(uio_var_l), pointer, &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%character3=>val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_pointputs3
!
!----------******************--------------------------------------------------
subroutine uio_var_pointputs4(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_pointputs4 ('uio_variable_point_put_string_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character(uio_var_l), pointer, &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%character4=>val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_pointputs4
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_puti0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puti0 ('uio_variable_put_integer_0')
!
! PURPOSE:
!   Put data set into uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_puti0(var,val)
!
! INPUT:
!   val:    ('value') scalar or array to be put into uio flexible variable
!   ident:  ('identifier') character, optional, identifier
!   format: ('format') character, optional, format string
!
! OUTPUT:
!   var:    ('variable') uio_var_type
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   14. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer,                 intent(in)     :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%integer0)
var%integer0=val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_puti0
!
!----------*************-------------------------------------------------------
subroutine uio_var_puti1(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puti1 ('uio_variable_put_integer_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, dimension(:),   intent(in)     :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(1), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(1)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%integer1(ilow(1):iup(1)))
var%integer1=val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_puti1
!
!----------*************-------------------------------------------------------
subroutine uio_var_puti2(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puti2 ('uio_variable_put_integer_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, dimension(:,:), intent(in)     :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(2), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(2)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%integer2(ilow(1):iup(1),ilow(2):iup(2)))
var%integer2=val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_puti2
!
!----------*************-------------------------------------------------------
subroutine uio_var_puti3(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puti3 ('uio_variable_put_integer_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),        intent(inout):: var
integer, dimension(:,:,:), intent(in)   :: val
character*(*), optional,   intent(in)   :: ident, format
integer, dimension(3), &
                 optional, intent(in)   :: ilb
!
! --- Local variables ---
integer, dimension(3)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%integer3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
var%integer3=val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_puti3
!
!----------*************-------------------------------------------------------
subroutine uio_var_puti4(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puti4 ('uio_variable_put_integer_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
integer, dimension(:,:,:,:), intent(in) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(4), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%integer4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                      ilow(4):iup(4)))
var%integer4=val
!
var%type=intena
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_puti4
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_putr0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putr0 ('uio_variable_put_real_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)),       intent(in)     :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%real0)
var%real0=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_putr0
!
!----------*************-------------------------------------------------------
subroutine uio_var_putr1(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putr1 ('uio_variable_put_real_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)),       intent(in), &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(1), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(1)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%real1(ilow(1):iup(1)))
var%real1=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_putr1
!
!----------*************-------------------------------------------------------
subroutine uio_var_putr2(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putr2 ('uio_variable_put_real_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)),       intent(in), &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(2), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(2)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%real2(ilow(1):iup(1),ilow(2):iup(2)))
var%real2=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_putr2
!
!----------*************-------------------------------------------------------
subroutine uio_var_putr3(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putr3 ('uio_variable_put_real_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)),       intent(in), &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(3), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(3)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%real3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
var%real3=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_putr3
!
!----------*************-------------------------------------------------------
subroutine uio_var_putr4(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putr4 ('uio_variable_put_real_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0E0)), intent(in), &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(4), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%real4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                   ilow(4):iup(4)))
var%real4=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_putr4
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_putd0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putd0 ('uio_variable_put_real_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), intent(in)           :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%dbreal0)
var%dbreal0=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_putd0
!
!----------*************-------------------------------------------------------
subroutine uio_var_putd1(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putd1 ('uio_variable_put_real_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), intent(in), &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(1), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(1)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%dbreal1(ilow(1):iup(1)))
var%dbreal1=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_putd1
!
!----------*************-------------------------------------------------------
subroutine uio_var_putd2(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putd2 ('uio_variable_put_real_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), intent(in), &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(2), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(2)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%dbreal2(ilow(1):iup(1),ilow(2):iup(2)))
var%dbreal2=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_putd2
!
!----------*************-------------------------------------------------------
subroutine uio_var_putd3(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putd3 ('uio_variable_put_real_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), intent(in), &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(3), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(3)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%dbreal3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
var%dbreal3=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_putd3
!
!----------*************-------------------------------------------------------
subroutine uio_var_putd4(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putd4 ('uio_variable_put_real_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
real(kind(0.0D0)), intent(in), &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(4), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%dbreal4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                     ilow(4):iup(4)))
var%dbreal4=val
!
var%type=realna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_putd4
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_putc0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putc0 ('uio_variable_put_complex_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), intent(in)        :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%complex0)
var%complex0=val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_putc0
!
!----------*************-------------------------------------------------------
subroutine uio_var_putc1(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putc1 ('uio_variable_put_complex_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), intent(in), &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(1), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(1)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%complex1(ilow(1):iup(1)))
var%complex1=val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_putc1
!
!----------*************-------------------------------------------------------
subroutine uio_var_putc2(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putc2 ('uio_variable_put_complex_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), intent(in), &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(2), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(2)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%complex2(ilow(1):iup(1),ilow(2):iup(2)))
var%complex2=val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_putc2
!
!----------*************-------------------------------------------------------
subroutine uio_var_putc3(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putc3 ('uio_variable_put_complex_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), intent(in), &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(3), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(3)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%complex3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
var%complex3=val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_putc3
!
!----------*************-------------------------------------------------------
subroutine uio_var_putc4(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putc4 ('uio_variable_put_complex_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
complex(kind(0.0E0)), intent(in), &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(4), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%complex4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                      ilow(4):iup(4)))
var%complex4=val
!
var%type=compna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_putc4
!
!
!----------*************-------------------------------------------------------
subroutine uio_var_puts0(var, val, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puts0 ('uio_variable_put_string_0')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*),           intent(in)     :: val
character*(*), optional, intent(in)     :: ident, format
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%character0)
var%character0=val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=0
!
end subroutine uio_var_puts0
!
!----------*************-------------------------------------------------------
subroutine uio_var_puts1(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puts1 ('uio_variable_put_string_1')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*),           intent(in), &
                           dimension(:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(1), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(1)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%character1(ilow(1):iup(1)))
var%character1=val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=1
!
end subroutine uio_var_puts1
!
!----------*************-------------------------------------------------------
subroutine uio_var_puts2(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puts2 ('uio_variable_put_string_2')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*), intent(in), &
                         dimension(:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(2), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(2)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%character2(ilow(1):iup(1),ilow(2):iup(2)))
var%character2=val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=2
!
end subroutine uio_var_puts2
!
!----------*************-------------------------------------------------------
subroutine uio_var_puts3(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puts3 ('uio_variable_put_string_3')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*),           intent(in), &
                       dimension(:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(3), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(3)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%character3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
var%character3=val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=3
!
end subroutine uio_var_puts3
!
!----------*************-------------------------------------------------------
subroutine uio_var_puts4(var, val, ilb, ident, format)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puts4 ('uio_variable_put_string_4')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*), intent(in), &
                     dimension(:,:,:,:) :: val
character*(*), optional, intent(in)     :: ident, format
integer, dimension(4), &
                   optional, intent(in) :: ilb
!
! --- Local variables ---
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
if (present(ilb)) then
  ilow=ilb
  iup =ilb+shape(val)-1
else
  ilow=1
  iup =shape(val)
endif
allocate(var%character4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                        ilow(4):iup(4)))
var%character4=val
!
var%type=charna
if (present(ident )) var%ident =ident
if (present(format)) var%format=format
var%kind=kind(val)
var%status=1
var%ndim=4
!
end subroutine uio_var_puts4
!
!----------**************------------------------------------------------------
subroutine uio_var_puttab(var, val, ident)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_puttab ('uio_variable_put_table')
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
type(uio_table_type),    intent(in)     :: val
character*(*), optional, intent(in)     :: ident
!
! --- Local variables ---
integer                                 :: ncol
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
allocate(var%table)
call uio_tabnull(var%table)
!
var%table=val
!
var%type=tabna
call uio_tabaskhead(val, ncol=ncol)
if (ncol > 0) then
  call uio_tabaskhead(val, icol=1, ident=var%ident)
else
  if (present(ident)) var%ident =ident
endif
var%kind=0
var%status=1
var%ndim=0
!
end subroutine uio_var_puttab
!
!
!----------***************-----------------------------------------------------
subroutine uio_var_putfifo(var)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putfifo ('uio_variable_put_fileform')
!
! PURPOSE:
!   Put fileform data set into uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_putfifo(var)
!
! INPUT:
!
! OUTPUT:
!   var:    ('variable') uio_var_type
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   22. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%type =fifona
var%ident=uiona
var%kind=0
var%status=1
var%ndim=0
!
end subroutine uio_var_putfifo
!
!
!----------***************-----------------------------------------------------
subroutine uio_var_putlabl(var, ident)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_putlabl ('uio_variable_put_label')
!
! PURPOSE:
!   Put label into uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   uio_var_putlabl(var, ident=ident)
!
! INPUT:
!   ident:  ('identifier') character, optional, identifier
!
! OUTPUT:
!   var:    ('variable') uio_var_type
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   22. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
type(uio_var_type),      intent(inout)  :: var
character*(*), optional, intent(in)     :: ident
!------------------------------------------------------------------------------
if (var%status > 0) call uio_var_delete(var)
!
var%type=lablna
if (present(ident)) var%ident=ident
var%kind=0
var%status=1
var%ndim=0
!
end subroutine uio_var_putlabl
!
!
!----------*****************---------------------------------------------------
subroutine uio_var_printhead(var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_printhead ('uio_variable_printhead')
!
! PURPOSE:
!   Print header of uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!   call uio_var_printhead(var, outstr=outstr,ierr=ierr)
!
! INPUT:
!   var:    ('variable') uio_var_type, uio flexible variable whose header is
!           to be printed
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!
! SIDE EFFECTS:
!   Print data on standard output.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!   type(uio_var_type) :: a
!   call uio_var_printhead(a)
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
implicit none
!
! --- I/O parameters ---
type(uio_var_type)     , intent(in)     :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
integer                                 :: ierr0
character*(80)                          :: outstr0
integer                                 :: nttt, nttn
character*(let), dimension(2,nttmx)     :: termtt
!------------------------------------------------------------------------------
!
if (var%status > 0) then
  !
  nttt=0
  !
  ! --- Type and identifier ---
  if (var%type == '') then
    call uio_adkeyw(termtt,nttt,'notype')
  else
    call uio_adkeyw(termtt,nttt,var%type)
  endif
  if (var%ident == '') then
    call uio_adkeyw(termtt,nttt,'??')
  else
    call uio_adkeyw(termtt,nttt,var%ident)
  endif
  !
  ! --- Concatenate term tables ---
  nttn=min(nttt+var%ntt,nttmx)
  termtt(:,nttt+1:nttn)=var%termt(:,1:nttn-nttt)
  nttt=nttn
  !
  ! --- Print term table ---
  call uio_pptrmt(termtt, nttt, mode=0)
  !
  outstr0='done'
  ierr0=0
else
  write(6,'(A)') 'Undefined variable'
  !
  outstr0='undefined variable'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_printhead
!
!
!----------**********----------------------------------------------------------
subroutine uio_var_rd(nc,termt,ntt, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_rd ('uio_variable_read')
!
! PURPOSE:
!   Read file entry into uio flexible variable.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!
! INPUT/OUTPUT:
!   termt:  ('term_table') character array (2,nttmx), complete keyword table
!   ntt:    ('number_term_table') integer, number of non-empty lines of termt
!
! OUTPUT:
!   var:    ('variable') uio_var_type, uio flexible variable read from uio file
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   None
!
! MODULES:
!   uio_cst_module: channel status information
!   uio_cvl_module: convert type list of current machine
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   Print data on standard output.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   27. 1.98 (B.F.) uio table included
!------------------------------------------------------------------------------
use uio_nam_module
use uio_cst_module
use uio_cvl_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
character*(*),           intent(inout)  :: termt(2,nttmx)
integer,                 intent(inout)  :: ntt
type(uio_var_type),      intent(inout)  :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(let)                         :: dimstr, bytes, format
character*(80)                          :: outstr0
integer                                 :: ierr0
integer                                 :: nbytes, nbtrels
integer, dimension(4)                   :: ilow, iup
!------------------------------------------------------------------------------
outstr0=''
ierr0=0
!
if (var%status > 0) call uio_var_delete(var)
!
! --- Analyse header ---
! --- Type and identifier ---
var%type=termt(1,1)
!
! --- Dimension ---
call uio_exkeyw(termt,ntt, dimna ,dimstr, mode=0)
if (dimstr /= '') then
  call uio_st2dim(dimstr, ilow, iup, ndim=var%ndim)
  if (var%type == tabna) var%ndim=0
else
  var%ndim=0
endif
!
! --- Number of bytes ---
call uio_exkeyw(termt,ntt, bytena ,bytes, mode=0)
if (bytes == '') then
  nbytes=0
else
  read(bytes,'(I6)') nbytes
endif
read(btrels(cvnum(nc)),'(I6)') nbtrels
!
! --- Allocate memory and read variable for correct type and dimension ---
select case(var%type)
  case(intena)                    ! --- Integer ---
    var%kind=kind(0)
    select case(var%ndim)
      case(0)                         ! --- Scalar ---
        allocate(var%integer0)
        call uio_rd(nc, termt,ntt, var%integer0, var%ident, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(1)                         ! --- 1D ---
        allocate(var%integer1(ilow(1):iup(1)))
        call uio_rd(nc, termt,ntt, var%integer1, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(2)                         ! --- 2D ---
        allocate(var%integer2(ilow(1):iup(1),ilow(2):iup(2)))
        call uio_rd(nc, termt,ntt, var%integer2, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(3)                         ! --- 3D ---
        allocate(var%integer3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
        call uio_rd(nc, termt,ntt, var%integer3, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(4)                         ! --- 4D ---
        allocate(var%integer4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                              ilow(4):iup(4)))
        call uio_rd(nc, termt,ntt, var%integer4, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
    end select ! var%ndim
  case(realna)                    ! --- Real ---
    if (nbytes <= nbtrels) then       ! --- Single precision ---
      var%kind=kind(0.0E0)
      select case(var%ndim)
        case(0)                         ! --- Scalar ---
          allocate(var%real0)
          call uio_rd(nc, termt,ntt, var%real0, var%ident, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(1)                         ! --- 1D ---
          allocate(var%real1(ilow(1):iup(1)))
          call uio_rd(nc, termt,ntt, var%real1, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(2)                         ! --- 2D ---
          allocate(var%real2(ilow(1):iup(1),ilow(2):iup(2)))
          call uio_rd(nc, termt,ntt, var%real2, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(3)                         ! --- 3D ---
          allocate(var%real3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
          call uio_rd(nc, termt,ntt, var%real3, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(4)                         ! --- 4D ---
          allocate(var%real4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                             ilow(4):iup(4)))
          call uio_rd(nc, termt,ntt, var%real4, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
      end select ! var%ndim
    else                              ! --- Double precision ---
      var%kind=kind(0.0D0)
      select case(var%ndim)
        case(0)                         ! --- Scalar ---
          allocate(var%dbreal0)
          call uio_rd(nc, termt,ntt, var%dbreal0, var%ident, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(1)                         ! --- 1D ---
          allocate(var%dbreal1(ilow(1):iup(1)))
          call uio_rd(nc, termt,ntt, var%dbreal1, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(2)                         ! --- 2D ---
          allocate(var%dbreal2(ilow(1):iup(1),ilow(2):iup(2)))
          call uio_rd(nc, termt,ntt, var%dbreal2, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(3)                         ! --- 3D ---
          allocate(var%dbreal3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
          call uio_rd(nc, termt,ntt, var%dbreal3, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
        case(4)                         ! --- 4D ---
          allocate(var%dbreal4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                               ilow(4):iup(4)))
          call uio_rd(nc, termt,ntt, var%dbreal4, var%ident, mode=1, &
                      format=var%format, outstr=outstr0, ierr=ierr0)
          var%status=1
      end select ! var%ndim
    endif
  case(compna)                    ! --- Complex ---
    var%kind=kind(cmplx(0.0E0))
    select case(var%ndim)
      case(0)                         ! --- Scalar ---
        allocate(var%complex0)
        call uio_rd(nc, termt,ntt, var%complex0, var%ident, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(1)                         ! --- 1D ---
        allocate(var%complex1(ilow(1):iup(1)))
        call uio_rd(nc, termt,ntt, var%complex1, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(2)                         ! --- 2D ---
        allocate(var%complex2(ilow(1):iup(1),ilow(2):iup(2)))
        call uio_rd(nc, termt,ntt, var%complex2, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(3)                         ! --- 3D ---
        allocate(var%complex3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
        call uio_rd(nc, termt,ntt, var%complex3, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(4)                         ! --- 4D ---
        allocate(var%complex4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                              ilow(4):iup(4)))
        call uio_rd(nc, termt,ntt, var%complex4, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
    end select ! var%ndim
  case(charna)                    ! --- Character ---
    var%kind=kind(0)
    select case(var%ndim)
      case(0)                         ! --- Scalar ---
        allocate(var%character0)
        call uio_rd(nc, termt,ntt, var%character0, var%ident, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(1)                         ! --- 1D ---
        allocate(var%character1(ilow(1):iup(1)))
        call uio_rd(nc, termt,ntt, var%character1, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(2)                         ! --- 2D ---
        allocate(var%character2(ilow(1):iup(1),ilow(2):iup(2)))
        call uio_rd(nc, termt,ntt, var%character2, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(3)                         ! --- 3D ---
        allocate(var%character3(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3)))
        call uio_rd(nc, termt,ntt, var%character3, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
      case(4)                         ! --- 4D ---
        allocate(var%character4(ilow(1):iup(1),ilow(2):iup(2),ilow(3):iup(3), &
                                ilow(4):iup(4)))
        call uio_rd(nc, termt,ntt, var%character4, var%ident, mode=1, &
                    format=var%format, outstr=outstr0, ierr=ierr0)
        var%status=1
    end select ! var%ndim
  case(fifona)                    ! --- Fileform entry ---
    call uio_rdfifo(nc, termt,ntt, outstr=outstr0, ierr=ierr0)
      var%ident=uiona      
      var%status=1
  case(lablna)                    ! --- Label ---
    call uio_rdlabl(nc, termt,ntt, var%ident, outstr=outstr0, ierr=ierr0)
      var%status=1
  case(tabna)                     ! --- Table ---
    allocate(var%table)
    call uio_tabnull(var%table)
    call uio_rdtab(nc, termt,ntt, var%table, var%ident)
    var%format=''
    var%status=1
end select ! var%type
!
var%ntt=ntt
var%termt(:,1:ntt)=termt(:,1:ntt)
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_rd
!
!----------**********----------------------------------------------------------
subroutine uio_var_wr(nc, var, outstr,ierr)
!------------------------------------------------------------------------------
! NAME:
!   uio_var_wr ('uio_variable_write')
!
! PURPOSE:
!   Write uio flexible variable into uio file.
!
! CATEGORY:
!   Input/Output
!
! CALLING SEQUENCE:
!
! INPUT:
!   nc:     ('number_channel') integer, channel unit number
!   var:    ('variable') uio_var_type
!
! OUTPUT:
!   outstr: ('output_string') character, optional
!   ierr:   ('integer_error') integer, optional, contains exit error status
!            0: correct operation
!           >0: occurance of an error
!
! ROUTINES:
!   uio_wr:     Write integer/real/double real/complex/character scalar or array
!
! MODULES:
!   uio_nam_module: definition of names
!
! SIDE EFFECTS:
!   Write data to file.
!
! RESTRICTIONS:
!
! PROCEDURE:
!
! EXAMPLE:
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!------------------------------------------------------------------------------
use uio_nam_module
!
implicit none
!
! --- I/O parameters ---
integer,                 intent(in)     :: nc
type(uio_var_type)     , intent(in)     :: var
character*(*), optional, intent(out)    :: outstr
integer,       optional, intent(out)    :: ierr
!
! --- Local variables ---
character*(80)                          :: outstr0
integer                                 :: ierr0
character*(let)                         :: trans
integer                                 :: iostat, nbytes
integer                                 :: nttt
character*(let), dimension(2,nttmx)     :: termtt
!------------------------------------------------------------------------------
!
if (var%status > 0) then
  ! --- Write variable with correct type and dimension ---
  select case(var%type)
    case(intena)                    ! --- Integer ---
      select case(var%ndim)
        case(0)                         ! --- Scalar ---
          call uio_wr(nc, var%integer0, var%ident, format=var%format, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(1)                         ! --- 1D ---
          call uio_wr(nc, var%integer1, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%integer1), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(2)                         ! --- 2D ---
          call uio_wr(nc, var%integer2, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%integer2), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(3)                         ! --- 3D ---
          call uio_wr(nc, var%integer3, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%integer3), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(4)                         ! --- 4D ---
          call uio_wr(nc, var%integer4, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%integer4), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
      end select ! var%ndim
    case(realna)                    ! --- Real ---
      nttt=var%ntt
      termtt(:,1:nttt)=var%termt(:,1:var%ntt)
      call uio_exkeyw(termtt,nttt, tranna, trans, mode=13)
      if (var%kind == kind(0.0E0)) then ! --- Single precision ---
        select case(var%ndim)
          case(0)                         ! --- Scalar ---
            call uio_wr(nc, var%real0, var%ident, format=var%format, &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(1)                         ! --- 1D ---
            call uio_wr(nc, var%real1, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%real1), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(2)                         ! --- 2D ---
            call uio_wr(nc, var%real2, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%real2), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(3)                         ! --- 3D ---
            call uio_wr(nc, var%real3, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%real3), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(4)                         ! --- 4D ---
            call uio_wr(nc, var%real4, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%real4), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
        end select ! var%ndim
      else                              ! --- Double precision ---
        select case(var%ndim)
          case(0)                         ! --- Scalar ---
            call uio_wr(nc, var%dbreal0, var%ident, format=var%format, &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(1)                         ! --- 1D ---
            call uio_wr(nc, var%dbreal1, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%dbreal1), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(2)                         ! --- 2D ---
            call uio_wr(nc, var%dbreal2, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%dbreal2), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(3)                         ! --- 3D ---
            call uio_wr(nc, var%dbreal3, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%dbreal3), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
          case(4)                         ! --- 4D ---
            call uio_wr(nc, var%dbreal4, var%ident, format=var%format, &
                        mode=0, ilb=lbound(var%dbreal4), &
                        termt=termtt, ntt=nttt, tran=trans, &
                        outstr=outstr0, ierr=ierr0)
        end select ! var%ndim
      endif
    case(compna)                    ! --- Complex ---
      select case(var%ndim)
        case(0)                         ! --- Scalar ---
          call uio_wr(nc, var%complex0, var%ident, format=var%format, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(1)                         ! --- 1D ---
          call uio_wr(nc, var%complex1, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%complex1), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(2)                         ! --- 2D ---
          call uio_wr(nc, var%complex2, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%complex2), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(3)                         ! --- 3D ---
          call uio_wr(nc, var%complex3, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%complex3), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(4)                         ! --- 4D ---
          call uio_wr(nc, var%complex4, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%complex4), &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
      end select ! var%ndim
    case(charna)                    ! --- Character ---
      if (var%format /= '') then
        ! --- Try to determine the length of the strings from format statement -
        read(var%format, '(1X,I6)', iostat=iostat) nbytes
        if (iostat /= 0) nbytes=-1
      else
        nbytes=-1
      endif
      select case(var%ndim)
        case(0)                         ! --- Scalar ---
          call uio_wr(nc, var%character0, var%ident, format=var%format, &
                      termt=var%termt, ntt=var%ntt, nbytes=nbytes, &
                      outstr=outstr0, ierr=ierr0)
        case(1)                         ! --- 1D ---
          call uio_wr(nc, var%character1, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%character1), nbytes=nbytes, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(2)                         ! --- 2D ---
          call uio_wr(nc, var%character2, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%character2), nbytes=nbytes, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(3)                         ! --- 3D ---
          call uio_wr(nc, var%character3, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%character3), nbytes=nbytes, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
        case(4)                         ! --- 4D ---
          call uio_wr(nc, var%character4, var%ident, format=var%format, &
                      mode=0, ilb=lbound(var%character4), nbytes=nbytes, &
                      termt=var%termt, ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
      end select ! var%ndim
    case(fifona)                    ! --- Fileform entry ---
      call uio_wrfifo(nc, outstr=outstr0, ierr=ierr0)
    case(lablna)                    ! --- Label ---
      call uio_wrlabl(nc, var%ident, termt=var%termt,ntt=var%ntt, &
                      outstr=outstr0, ierr=ierr0)
    case(tabna)                     ! --- Table ---
      call uio_wrtab(nc, var%table, outstr=outstr0, ierr=ierr0)
  end select ! var%type
else
  outstr0='undefined variable'
  ierr0=1
endif
!
if (present(outstr)) outstr=outstr0
if (present(ierr)) ierr=ierr0
!
end subroutine uio_var_wr
!
end module uio_var_module
