program uio_var_test
!------------------------------------------------------------------------------
! NAME:
!   uio_var_test ('universal_variable_test')
!
! PURPOSE:
!   Test uio flexible variable routines.
!
! CATEGORY:
!   Input/Output
!
! SUBROUTINES:
!   uio_init:   Initialization procedure for input/output routines
!   uio_openrd: Open file for reading, read header
!
! PROCEDURE:
!   Open file, read and print all entry headers, skip data blocks.
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   27. 1.98 (B.F.)
!------------------------------------------------------------------------------
! Non-standard FORTRAN:
!   Hopefully none.
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_table_module
use uio_var_module
!
implicit none
!
! --- Local variables ---
integer                                 :: ncin, ncout, ntt, ierr, status, &
                                           nbytes
character*(80)                          :: outstr, file, name, ident
character*(let)                         :: termt(2,nttmx)
type(uio_var_type)                      :: a_var, b_var, c_var
integer, pointer                            :: i0
integer, pointer, dimension(:)              :: i1, i1a
integer, pointer, dimension(:,:,:,:)        :: i4a
real(kind(0.0E0)), pointer                  :: r0
real(kind(0.0E0)), pointer, dimension(:)    :: r1, r1a
character(uio_var_l), pointer               :: s0
character(uio_var_l), pointer, dimension(:) :: s1, s1a
integer, dimension(0:2)                     :: ii1
real                                        :: rr0
real, dimension(0:3)                        :: rr1
type(uio_table_type)                        :: table
!------------------------------------------------------------------------------
!
! write(6,*)
! write(6,*) 'UIO_VAR_TEST'
! write(6,*) '============'
! write(6,*)
!
! --- uio initialization ---
call uio_init(progrm='uio_var_test')
!
! --- Open files ---
call uio_openrd(ncin,  file='uio_var_test.txt')
call uio_openwr(ncout, file='uio_var_test.out', form='formatted', conv='ieee_4')
!
call uio_var_null(a_var)
call uio_var_null(b_var)
call uio_var_null(c_var)

entryloop: do
  call uio_rdhd(ncin, termt,ntt, outstr=outstr,ierr=ierr)
  !
  if (outstr == 'eof') exit entryloop
  !
  ! --- Copy entry ---
  call uio_var_rd(ncin, termt,ntt, a_var)
!  call uio_var_printhead(a_var)
  call uio_var_wr(6, a_var)
  call uio_var_askhead(a_var, status=status, ident=ident, name=name, &
                       nbytes=nbytes)
  write(6,*) status, nbytes
  select case(ident)
    case('i1')
      call uio_var_pointget(a_var, i1)
      write(6,*) i1
      ii1=420
      call uio_var_put(a_var, ii1, ilb=(/0/), ident='i1')
    case('r1')
      call uio_var_get(a_var, rr1)
      write(6,*) rr1
      allocate(r1a(1:size(rr1,1)))
      r1a=420.0
      call uio_var_pointput(a_var, r1a, ident='r1')
    case('s1')
      call uio_var_pointget(a_var, s1)
      write(6,'(A79)') S1
      allocate(s1a(1:size(s1,1)))
      s1a='fortytwo'
      call uio_var_pointput(a_var, s1a, ident='s1', format='A10')
  end select ! ident
  call uio_var_wr(ncout, a_var)
  call uio_var_delete(a_var)
  write(6,*)
end do entryloop
!
allocate(i4a(2,3,2,2))
i4a=4200
call uio_var_pointput(a_var, i4a, ident='i4', format='I5')
call uio_var_wr(ncout, a_var)
call uio_var_delete(a_var)


!call uio_var_askhead(a_var, status=status, ident=ident, name=name)
!write(6,*) status
!write(6,*) '('//trim(ident)//')' // '('//trim(name)//')'

!call uio_var_changehead(a_var, name='1D integer array')
!call uio_var_askhead(a_var, status=status, ident=ident, name=name)
!write(6,*) status
!write(6,*) '('//trim(ident)//')' // '('//trim(name)//')'
!call uio_var_printhead(a_var)

!call uio_var_pointget(a_var,i1)
!write(6,*) i1
!write(6,*) lbound(i1)
!write(6,*) ubound(i1)

!b_var=a_var
!call uio_var_wr(6, b_var)
!call uio_var_delete(a_var)
!call uio_var_wr(6, b_var)

!
! --- Close files ---
call uio_closrd(ncin)
call uio_closwr(ncout)
!
end program uio_var_test
