program uio_varfile_test
!------------------------------------------------------------------------------
! NAME:
!   uio_varfile_test ('universal_variable_file_test')
!
! PURPOSE:
!   Test uio flexible variable file routines.
!
! CATEGORY:
!   Input/Output
!
! SUBROUTINES:
!   uio_init:   Initialization procedure for input/output routines
!   uio_openrd: Open file for reading, read header
!
! PROCEDURE:
!   Open file, read and print all entry headers, skip data blocks.
!
! MODIFICATION HISTORY:
!   21. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   29. 1.98 (B.F.)
!------------------------------------------------------------------------------
! Non-standard FORTRAN:
!   Hopefully none.
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
use uio_varfile_module
!
implicit none
!
! --- Local variables ---
integer                                 :: ncin, ncout, ntt, ierr, status, &
                                           nbytes, nentry, position
character*(80)                          :: outstr, file
character*(let)                         :: type, ident, name
character*(let)                         :: termt(2,nttmx)
type(uio_var_type)                      :: a_var, b_var, c_var
type(uio_var_type), pointer             :: p_var
type(uio_varfile_type)                  :: varfile

integer, pointer                            :: i0
integer, pointer, dimension(:)              :: i1, i1a
integer, pointer, dimension(:,:,:,:)        :: i4a
real(kind(0.0E0)), pointer                  :: r0
real(kind(0.0E0)), pointer, dimension(:)    :: r1, r1a
character(uio_var_l), pointer               :: s0
character(uio_var_l), pointer, dimension(:) :: s1, s1a
integer, dimension(0:2)                     :: ii1
real                                        :: rr0
real, dimension(0:3)                        :: rr1
!------------------------------------------------------------------------------

write(6,*)
write(6,*) 'UIO_VARFILE_TEST'
write(6,*) '================'
write(6,*)
!
! --- uio initialization ---
call uio_init(progrm='uio_var_test')
!
! --- Initialize uio varfile ---
call uio_varfile_null(varfile)
!
call uio_openrd(ncin, file='uio_var_test.txt')
call uio_varfile_rd(ncin, varfile)
call uio_closrd(ncin)

!call uio_var_null(a_var)
!rr1=420.0
!call uio_var_put(a_var, rr1, ident='r1', ilb=lbound(rr1))
!call uio_varfile_put(varfile, a_var)
!
call uio_varfile_goto(varfile, 'rewind')

call uio_var_delete(a_var)
call uio_varfile_get(varfile, a_var)
call uio_var_wr(6, a_var)
call uio_varfile_pointget(varfile, p_var)
call uio_var_wr(6, p_var)

call uio_varfile_askhead(varfile, nentry=nentry, position=position, &
                         type=type, ident=ident)

call uio_varfile_goto(varfile, 'rewind')

write(6,*) nentry, position
write(6,*) trim(type) // ' ' // trim(ident)
write(6,*)


call uio_varfile_wr(6, varfile)


end program uio_varfile_test
