!-------******-----------------------------------------------------------------
program uiocop
!------------------------------------------------------------------------------
! NAME:
!   uiocop ('universal_input_output_copy')
!
! PURPOSE:
!   Read a file in uio form make a copy (in a different format/conversion type).
!
! CATEGORY:
!   Input/Output
!
! SUBROUTINES:
!   uio_closrd: Close file after reading
!   uio_closwr: Close file after writing
!   uio_cpentr: Copy entry
!   uio_init:   Initialization procedure for input/output routines
!   uio_openrd: Open file for reading, read header
!   uio_openwr: Open file for writing, write header
!   uio_rdfifo: Read file header
!   uio_srhd:   Search entry header in file or in entry list
!   uio_wrfifo: Write file header
!
! PROCEDURE:
!   Read parameter list, open input and output file, call uio_srhd and
!   uio_cpentr repeatedly and close files again.
!
! MODIFICATION HISTORY:
!    4. 1.96 Written by Bernd Freytag, Astrophysik Kiel
!    5. 1.96 (B.F.) Changes
!    8. 1.96 (B.F.) Changes, 1. working version
!    9. 1.96 (B.F.) Parameter abb removed, tran added
!   11. 1.96 (B.F.) labels and fileforms allowed
!   27. 1.96 (B.F.) wrra -> wrr1 ..., tran shifted
!   30. 1.96 (B.F.) uio_openrd, uio_rdfifo: parameter langua added
!    9. 2.96 (B.F.) (lt) -> (let), (ll) -> (lel)
!   20. 2.96 (B.F.) No explicit questions on terminal
!   21. 2.96 (B.F.) More types, uiocst.inc & uiocvl.inc included
!            arbitrary number of input files
!   23. 2.96 (B.F.) Different input parameters,
!            output to stdout possible, copylist added
!   15.10.96 (B.F.) Ignore all old keywords from fileform during copying
!   19.10.96 (B.F.) Copying done by subroutine cpentr
!   16. 2.97 (B.F.) Fortran90 version
!   20. 2.97 (B.F.) Watch conversion type also for '>stdout'
!   23. 2.97 (B.F.) Set conversion type of stdout
!   23. 1.98 (B.F.) uio_copy_module removed
!------------------------------------------------------------------------------
! Non-standard FORTRAN:
!   Hopefully none
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_table_module
!
implicit none
!
! --- Local variables ---
! --- Maximum number of input files ---
integer, parameter                      :: ninmax=1000
integer                                 :: ncin, ncout, ierr, ntt
integer                                 :: n, nin
character*(80)                          :: outstr
character*(let)                         :: infile(ninmax)
character*(let)                         :: inform, inconv, &
                                           oufile, ouform, ouconv
character*(let)                         :: termt(2,nttmx)
! --- Copy list ---
integer                                 :: ncopy
integer, parameter                      :: mxcopy=20
character*(let)                         :: copyls(2,mxcopy)
!
!------------------------------------------------------------------------------
!
! write(6,*)
! write(6,*) 'UIOCOP'
! write(6,*) '======'
! write(6,*)
!
! --- uio initialization ---
call uio_init(progrm='uiocop')
!
! --- Read parameters from standard input ---
!
! write(6,*) 'Output file name?'
read(5,'(A)') oufile
! write(6,*) 'Output file format?'
read(5,'(A)') ouform
! write(6,*) 'Output file conversion type?'
read(5,'(A)')  ouconv
!if (ouform == 'formatted') then
!  ouconv=''
!endif
! --- Read copy list until search tag='- -' ---
ncopy=1
do
  read(5,'(A)') copyls(1,ncopy)
  read(5,'(A)') copyls(2,ncopy)
!  write(6,'(A') '(' // trim(copyls(1,ncopy)) // ') (' // trim(copyls(2,ncopy)) // ')'
  if (copyls(1,ncopy) == '*') copyls(1,ncopy)=''
  if (copyls(2,ncopy) == '*') copyls(2,ncopy)=''
  if ((copyls(1,ncopy) /= '-') .or. (copyls(2,ncopy) /= '-')) then
    ncopy=min(ncopy+1,mxcopy)
  else
    ncopy=ncopy-1
    exit
  endif
end do
!
! --- Read file names ---
! write(6,*) 'Number of input files?'
read(5,*) nin
if (nin >= ninmax) then
  outstr='too many input files'
  ierr=1
  call uio_err(outstr=outstr,ierr=ierr, routine='uiocop', errlevel=3)
endif
if (nin >= 1) then
  do n=1,nin
    read(5,'(A)') infile(n)
  end do ! n
else
  nin=1
  infile(nin)='<<stdin'
endif
!
! === Loop over all files =========
fileloop: do n=1,nin
  !
  if (infile(n) == '<<stdin') then
    ! --- Input from stdin ---
    ncin=5
    call uio_rdhd(ncin, termt,ntt)
    call uio_rdfifo(ncin, termt,ntt, form=inform, conv=inconv)
  else
    ! --- Open input file ---
    call uio_openrd(ncin, infile(n), form=inform, conv=inconv)
  endif
  !
  if (n == 1) then
    ! --- Output channel ---
    if (ouconv == '') ouconv=inconv
    if (oufile == '>stdout') then
      ! --- Output to stdout ---
      ncout=6
      ouform='formatted'
      call uio_chconv(6, conv=ouconv)
      call uio_wrfifo(ncout)
    else
      ! --- Open output file ---
      call uio_openwr(ncout, oufile, form=ouform,conv=ouconv)
    endif
  endif
  !
  ! --- Repeat: copy all entries ---
  !
  ! --- Read header and extract keywords ---
  entryloop: do
    call uio_srhd(ncin, termt,ntt, srchls=copyls,nsrc=ncopy, &
                  outstr=outstr,ierr=ierr)
    if (outstr == 'eof') exit entryloop
    ! --- Copy entry ---
    call uio_cpentr(ncin, termt,ntt, ncout, outstr=outstr,ierr=ierr)
  end do entryloop
  !
  ! --- Close input file ---
  if (ncin /= 5) call uio_closrd(ncin)
  !
end do fileloop
!
! --- Close output file ---
if (ncout /= 6) call uio_closwr(ncout)
!
end program uiocop
