!-------******-----------------------------------------------------------------
program uioinf
!------------------------------------------------------------------------------
! NAME:
!   uioinf ('universal_input_output_information')
!
! PURPOSE:
!   Write information about machine dependent features.
!
! CATEGORY:
!   Input/Output
!
! SUBROUTINES:
!   uio_init:   Initialization procedure for input/output routines
!
! MODULES:
!   uio_cvl_module: convert type list of current machine
!   uio_nam_module: definition of names
!   uio_siz_module: string length, table size
!
! PROCEDURE:
!
! MODIFICATION HISTORY:
!   12. 1.98 Written by Bernd Freytag, Astrophysik Kiel
!   27. 2.98 (B.F.) Print more information
!   2010-05-05 (B.F. Lyon) Call uio_SwitchInfo
!------------------------------------------------------------------------------
use uio_cvl_module
use uio_siz_module
use uio_nam_module
use uio_bulk_module
!
implicit none
!
! --- Local variables ---
integer                                 :: ierr, n
character*(80)                          :: outstr
character*(12)                          :: defstr
character*(let)                         :: system, machin, osrels, osvers, &
                                           hrdwar, langua
!------------------------------------------------------------------------------
!
write(6,*)
write(6,*) 'UIOINF'
write(6,*) '======'
write(6,*)
!
call uio_SwitchInfo(6)
!
! --- uio initialization ---
call uio_init(progrm='uioinf')
!
! --- Get information about environment ---
call uio_getenv(system=system, machin=machin, osrels=osrels, &
                osvers=osvers, hrdwar=hrdwar, langua=langua, &
                outstr=outstr,ierr=ierr)
!
!write(6,'(A)') 'date=   '//trim(  date)
write(6,'(A)') 'Environment:'
write(6,'(A)') 'System=       '//trim(system)
write(6,'(A)') 'Machine name= '//trim(machin)
write(6,'(A)') 'OS release=   '//trim(osrels)
write(6,'(A)') 'OS version=   '//trim(osvers)
write(6,'(A)') 'Hardware=     '//trim(hrdwar)
write(6,'(A)') 'Language=     '//trim(langua)
!
write(6,*)
!
write(6,'(A)') 'List of conversion types:'
do n=1,ncv
  if (n == ncvdef) then
    defstr=' <= default'
  else
    defstr=''
  endif
!  write(6,'(I2,1X,A)') n, ' '//cvnals(n)//' '//btrels(n)//' '//forels(n)//defstr
  write(6,'(I2,1X,A)') n, ' '//cvnals(n)//'  '//btrels(n)//' '//forels(n)// &
                                         '  '//btdbls(n)//' '//fodbls(n)// &
                                         '  '//btinls(n)//' '//foinls(n)// &
                         defstr
end do ! n
!
end program uioinf
