!-------******-----------------------------------------------------------------
program uiolok
!------------------------------------------------------------------------------
! NAME:
!   uiolok ('universal_input_output_look')
!
! PURPOSE:
!   Scan file and print headers.
!
! CATEGORY:
!   Input/Output
!
! SUBROUTINES:
!   uio_chpos:  Give current file position or jump to specified position
!   uio_chunit: Initialize, store and actualize a list of unit numbers
!   uio_closrd: Close file after reading
!   uio_exmtrm: Transform a list of items into its components
!   uio_init:   Initialization procedure for input/output routines
!   uio_openrd: Open file for reading, read header
!   uio_pptrmt: Print term table in pretty form
!   uio_rdhdln: Read header, output: line table
!   uio_skipda: Skip data block
!   uio_wrhdln: Write header, input: line table
!
! PROCEDURE:
!   Open file, read and print all entry headers, skip data blocks.
!
! MODIFICATION HISTORY:
!    9. 1.96 Written by Bernd Freytag, Astrophysik Kiel
!    4. 2.96 (B.F.) inlang,inprog added in call of uio_openrd
!    9. 2.96 (B.F.) (ll) -> (lel), (lt) -> (let)
!   19. 2.96 (B.F.) Different usage of pptrmt, input from stdin
!   20. 2.96 (B.F.)
!   22. 2.96 (B.F.) Parameter mode in pptrmt added,
!            output in pretty or compressed form
!   28.10.96 (B.F.) Renaming
!   16. 2.97 (B.F.) Fortran90 version
!------------------------------------------------------------------------------
! Non-standard FORTRAN:
!   Hopefully none.
!------------------------------------------------------------------------------
use uio_siz_module
use uio_nam_module
use uio_bulk_module
!
implicit none
!
! --- Local variables ---
integer                                 :: ppmode, ncin, ntt, nlt, ierr
character*(lel)                         :: outstr
character*(let)                         :: form, infile
character*(let)                         :: termt(2,nttmx)
character*(lel)                         :: linet(nltmx)
!
!------------------------------------------------------------------------------
!
! write(6,*)
! write(6,*) 'UIOLOK'
! write(6,*) '======'
! write(6,*)
!
! --- Read file names ---
! write(6,*) 'Input file name?'
read(5,'(A)') infile
! write(6,*) 'Pretty print?'
read(5,*) ppmode
!
! --- uio initialization ---
call uio_init(progrm='uiolok')
!
if (infile == '<<stdin') then
  ! --- Input from stdin ---
  ncin=5
else
  ! --- Open input file ---
  call uio_openrd(ncin, infile)
  call uio_chpos(ncin, mode='rewind')
endif
!
! --- Write information about input file to standard output ---
!
! --- Get format of input file ---
call uio_chunit('ask', nc1in=ncin, outstr=form)
!
! --- Repeat: Scan all entries ---
do
  ! --- Read entry header ---
  call uio_rdhdln(ncin,form,linet,nlt, outstr=outstr)
  if (outstr == 'eof') exit
  ! --- Put keywords into table ---
  call uio_exmtrm(linet,nlt, termt,ntt, outstr,ierr)
  if (ppmode == 1) then
    ! --- Pretty print ---
    call uio_pptrmt(termt,ntt)
  else
    ! --- Write header in compressed form to standard output ---
    call uio_wrhdln(6,'formatted',linet,nlt)
  endif
  ! --- Skip data ---
  call uio_skipda(ncin, termt,ntt)
end do
!
! --- Close input file ---
if (ncin /= 5) call uio_closrd(ncin)
!
end
