pro rhd_mpg, modelname, q_id, $
  datatype=datatype0, fileincrement=fileincrement, $
  modelident=modelident0, modeldisk=modeldisk0, modelfile=modelfile0, $
  eos=eos, par=par, xfac=xfac0, $
  q_name=q_name0, $
    q_title=q_title0, q_meansub=q_meansub, $
    q_execute=q_execute, q_log10=q_log10, q_scale=q_scale, q_smooth=q_smooth, q_median=q_median, $
    q_atan=q_atan, $
  a_q_id=a_q_id0, $
    a_q_meansub=a_q_meansub, $
    a_q_execute=a_q_execute, a_q_log10=a_q_log10, a_q_scale=a_q_scale, a_q_smooth=a_q_smooth, a_q_median=a_q_median, $
    a_q_atan=a_q_atan, $
  c_q_id=c_q_id0, $
    c_nlevels=c_nlevels0, c_levels=c_levels0, $
    c_annotation=c_annotation, c_labels=c_labels, c_fill=c_fill, $
    c_charsize=c_charsize, c_thick=c_thick0, $
    c_color=c_color, c_linestyle=c_linestyle0, c_q_meansub=c_q_meansub, c_smooth=c_smooth, $
  s_q_id=s_q_id0, $
    s_randomstart=s_randomstart, s_rows=s_rows, s_columns=s_columns, $
    s_timestep=s_timestep, s_nsteps=s_nsteps, s_norder=s_norder, $
    s_thick=s_thick, s_color=s_color, s_arrowsize=s_arrowsize, $
  bubble=bubble0, $
    b_scale=b_scale0, b_rgb=b_rgb0, $
  MPG_file=MPG_file0, mpg_quality=mpg_quality0, $
  jpg_flag=jpg_flag0, single_frame=single_frame, $
  drange=drange0, $
  n_subframe=n_subframe0, q_weightmode=q_weightmode0, $
  n_timestep=n_timestep0, n_firststep=n_firststep0, $
  t_colorrange=t_colorrange, t_sample=t_sample, $
  a_circle=a_circle, $
  w_xsize=w_xsize0, w_ysize=w_ysize0, w_swap=w_swap, $
  device=device0, nmlist=nmlist0
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_mpg ('rhd_mpg')
;
; PURPOSE:
;   Produce MPEG animation from data given by file list.
;
; CATEGORY:
;   RHD, animation
;
; CALLING SEQUENCE:
;   rhd_mpg, modelname, q_id, $
;     datatype=datatype, fileincrement=fileincrement, $
;     modelident=modelident, modeldisk=modeldisk, modelfile=modelfile, $
;     eos=eos, par=par, xfac=xfac, $
;     q_name=q_name, $
;       q_title=q_title, q_meansub=q_meansub, $
;       q_execute=q_execute, q_log10=q_log10, q_scale=q_scale, q_smooth=q_smooth, q_median=q_median, $
;       q_atan=q_atan, $
;     c_q_id=c_q_id, $
;       c_nlevels=c_nlevels, c_levels=c_levels, $
;       c_annotation=c_annotation, c_labels=c_labels, c_fill=c_fill, $
;       c_charsize=c_charsize, c_thick=c_thick, $
;       c_color=c_color, c_linestyle=c_linestyle, c_q_meansub=c_q_meansub, c_smooth=c_smooth, $
;     s_q_id=s_q_id, $
;       s_randomstart=s_randomstart, s_rows=s_rows, s_columns=s_columns, $
;       s_timestep=s_timestep, s_nsteps=s_nsteps, s_norder=s_norder, $
;       s_thick=s_thick, s_color=s_color, s_arrowsize=s_arrowsize, $
;     bubble=bubble, $
;       b_scale=b_scale, b_rgb=b_rgb, $
;     MPG_file=MPG_file, mpg_quality=mpg_quality, $
;     jpg_flag=jpg_flag, single_frame=single_frame, $
;     drange=drange, $
;     n_subframe=n_subframe, q_weightmode=q_weightmode, $
;     n_timestep=n_timestep, n_firststep=n_firststep, $
;     t_colorrange=t_colorrange, t_sample=t_sample, $
;     a_circle=a_circle, $
;     w_xsize=w_xsize, w_ysize=w_ysize, w_swap=w_swap, $
;     device=device, nmlist=nmlist
;
; INPUT:
;   modelname:    ('model_name') String with model name, used to generate output filenames.
;   q_id:         ('quantity_identifier') String with quantity to visualize as color plot,
;                 e.g. 'I1bl', 'I3br', 'T', 'rho', see combox.pro for a list.
;
; OUTPUT:
;   None (An MPG file or lots of JPEG files are produced)
;
; KEYWORDS:
;   datatype:     ('data_type') String with data type: 'uio' (default), 'rage', 'flash'
;   fileincrement: ('file_increment') Integer with file increment ( >1 means files are skipped,
;                 for 'rage' or 'flash' data)
;   modelident:   ('model_identifier') String to specify group of input files, default: '_*'
;   modeldisk:    ('model_disk') String with path to model files,
;                 default: getenv('HOME') + '/dat/rhd/d' + modelname + '/'
;   modelfile:    ('model_file') String with full file specification,
;                 default: modeldisk + modelname + modelident + '.mean'
;                 If modelfile='end', 'chu1', 'chu2', 'chu3', 'full', or 'mean' then
;                   modelfile=modeldisk + modelname + modelident + '.' + modelfile
;   eos:          ('equation_of_state') Structure with EOS information
;   par:          ('par') Structure with CO5BOLD parameter file
;   xfac:         ('x_factor') Real with scaling factor for axes (default: 1.0E-05 to get from cm to km)
;   q_name:       ('quantity_name') String with name of quantity to be used in title and output file name(s),
;                 default: q_id
;   q_title:      ('quantity_title') String with title used in movie. The model name and a time value are added.
;   q_meansub:    ('quantity_mean_subtract') Subtract horizontal mean of quantity, done in combox.pro.
;   q_execute:    ('quantity_execute') String with command to be processed by 'execute' to modify quantity,
;                 e.g. 'q=sqrt(sqrt(q/5.0E+15))'
;   q_log10:      ('quantity_logarithm_10') If set, q=alog10(q) is applied.
;   q_scale:      ('quantity_scale') If set, q=q_scale*q is applied.
;   q_smooth:     ('quantity_smooth') If set, q=smooth2D(q, q_smooth=q_smooth)
;                 OLD: If set, q=convol(q, kernel, /edge_truncate, /center) is applied
;                 with a 3x3 smoothing kernel.
;   q_median:     ('quantity_median') If set, q=median(q, q_median) is applied.
;   q_atan:       ('quantity_arcus_tangens') If set, q=(2.0/!pi)*atan(q/(2.0/!pi)) is applied, after time interpolation.
;   c_q_id:       ('contour_quantity_identifier') String with quantity to be visualized as contour plot,
;                 analog to q_id.
;     c_*:        ('contour_*') Keywords for contour plotting, see plotfield.pro.
;     c_nlevels:  ('contour_number_levels') integer, number of levels (default=10)
;     c_levels:   ('contour_levels') real vector with level values (default=automatic)
;     c_annotation: ('contour_annotation') character vector with the labels
;                 to be drawn on the contours (default=value)
;     c_charsize: ('contour_character_size') real, the size of the characters
;                 used to annotate contour labels
;     c_labels:   ('contour_labels') specifies which contour levels should be labeled
;     c_fill:     ('contour_fill') logical (0/1), if set -> filled contour
;     c_color:    ('contour_colour') integer vector (or scalar) with the color indices
;                 used to draw the contours (default=!P.COLOR)
;     c_linestyle: ('contour_line_style') integer vector with the linestyles
;                 used to draw the contours
;     c_thick:    ('contour_thick') integer vector of line thickness values
;     c_smooth:   ('contour_smooth') integer  to activate smoothing of contour lines
;                 useful for MPEG movie production, default=0, reasonable=1,2,3
;   s_q_id:       ('streamlines_quantity_identifier') String with vector quantity to be visualized as
;                 pseudo-streamline plot. So far 'v' or 'Bc' are possible.
;   s_*:          ('streamlines_*') Keywords for streamline plotting, see plotfield.pro.
;     s_randomstart: ('stream_lines_random_start') real, start with random initial positions
;     s_rows:     ('stream_lines_rows') integer, number of tracer particle rows
;     s_columns:  ('stream_lines_columns') integer, number of tracer particle columns
;     s_timestep: ('stream_lines_time_step') real, timestep
;     s_nsteps:   ('stream_lines_number_steps') integer, number of points
;                 for one streamline (typical 4)
;     s_norder:   ('stream_lines_number_order') integer, order of integration (1 or 2)
;     s_thick:    ('stream_lines_thick') integer with thickness of streamlines (and arrows)
;     s_color:    ('stream_lines_color') integer with color index of streamlines
;     s_arrowsize:('stream_lines_arrow_size') real with size of arrow head
;   bubble:       ('bubble') Keyword to activate the overplotting of bubbles that follow the flow field.
;   b_scale:      ('bubble_scale') Without effect, so far.
;   b_rgb:        ('bubble_red_green_blue') Float vector with 3 color factors, default: [40.0, 40.0, 40.0],
;                 e.g.: [50.0, 90.0, 120.0]; larger numbers give more visible bubbles.
;   MPG_file:     ('MPEG_file') String with name of output MPEG file; default: modelname + '_' + q_name + '.mpg'
;   mpg_quality:  ('MPEG_quality') Integer with MPEG quality; default: 85; the more the better (<= 100)
;   jpg_flag:     ('JPEG_flag') If set the first frame is written as a JPEG file, useful to generate
;                 a thumbnail for the movie.
;   single_frame: ('single_frame') If set, each image is written as single (JPEG) file into
;                 'tmp' subdirectory of current working directory
;   drange:       ('data_range') Two element float vector with data range to be considered for color table
;   n_subframe:   ('number_sub_frame') Integer with number of plotted frames per dataset.
;                 The default (2) means that one interpolated frame is inserted between each original frame.
;   q_weightmode: ('quantity_weight_mode') Integer to specify how up to 3 original frames are weighted
;                 for the interpolation.
;   n_timestep:   ('number_time_step') Integer with number of snapshots to make movie from; default: 30000.
;   n_firststep:  ('number_first_step') Integer with number of first snapshot to actually use, default: 0.
;   t_colorrange: ('tv_color_range') Specifies color range, see plotfield.pro.
;                 On 8-bit terminals: Two-element vector with index range in color table to be used.
;                 On 24-bit terminals: n x 3 byte array with color table (with n colors).
;   t_sample:     ('tv_sample') If set nearest neighbor sampling instead of bilinear interpolation is used
;   a_circle:     ('a_circle') If set a set with radius par.r0_grav is drawn in the center of the image
;   w_xsize:      ('window_x_size') Integer with window width in pixels
;   w_ysize:      ('window_y_size') Integer with window height in pixels
;   w_swap:       ('window_swap') If set the image is written transposes into the output file (rarely used).
;   device:       ('device') String with output device, usually 'X' (default); 'PS' is possible (rarely used).
;   nmlist:       ('number_model_list') Vector with integers of frames to plot into PS file.
;
; COMMON BLOCKS:
;   None
;
; ROUTINES:
;   combox:
;   derivbox:
;   rd_datasetlist:
;   bubble2d_init:
;   bubble2d_sequence:
;
; SIDE EFFECTS:
;   Window 0 is opened and used.
;   A file is written.
;
; RESTRICTIONS:
;   The routine is useful for data in the form provided by CO5BOLD.
;   However, some data from RAGE or FLASH can be forced to be in that format, too. See rd_datasetlist.pro.
;
; PROCEDURE:
;   After producing a list of images encode manually e.g. with
;     convert -quality 95 infile_*.jpg outfile.mpg
;     mencoder 'mf://infile_*.jpg' -o outfile.avi -ovc lavc -lavcopts vcodec=mjpeg
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;    4. 4.02 Written by Bernd Freytag, Astronomical Observatory Uppsala
;   10. 2.03 (B.F.) q_weightmode=12
;   17. 3.04 (B.F.) Compute vorticity with derivbox
;    6. 4.04 (B.F.) x1p
;    5. 7.05 (B.F. LA) Keyword datatype added
;   31. 8.05 (B.F. LA) Keyword mpg_quality added
;   2006-05-30 (B.F. LA) Bugfix: mpq_quality -> mpg_quality
;   2006-05-31 (B.F.) absvort
;   2007-01-24 (B.F. Lyon) Keyword single_frame added
;   2007-04-13 (B.F.) Handle keyword modelfile='mean', 'full', 'end'; keyword xfac
;   2007-08-23 (B.F.) Time units 'd' possible
;   2012-04-23 (B.F.) Handle keyword modelfile='chu1'
;   2012-10-08 (B.F.) Replace convol by smooth2d: allow more than one smoothing step
;   2014-08-07 (B.F. Uppsala) Use r0_core for circle if not zero.
;-
;------------------------------------------------------------------------------

; --- Set default values for keywords ---
if (keyword_set(xfac0)) then begin
  xfac=xfac0
endif else begin
  xfac=1.0E-05
endelse

if (n_elements(modelident0) ne 0) then begin
  modelident=modelident0
endif else begin
  modelident='_*'
endelse

if (n_elements(modeldisk0) ne 0) then begin
  modeldisk=modeldisk0
endif else begin
  modeldisk=getenv('HOME') + '/dat/rhd/d' + modelname + '/'
endelse

if (keyword_set(modelfile0)) then begin
  if (modelfile0 eq 'mean') then begin
    modelfile=modeldisk + modelname + modelident + '.mean'
  endif else if (modelfile0 eq 'full') then begin
    modelfile=modeldisk + modelname + modelident + '.full'
  endif else if (modelfile0 eq 'end') then begin
    modelfile=modeldisk + modelname + modelident + '.end'
  endif else if (modelfile0 eq 'chu1') then begin
    modelfile=modeldisk + modelname + modelident + '.chu1'
  endif else if (modelfile0 eq 'chu2') then begin
    modelfile=modeldisk + modelname + modelident + '.chu2'
  endif else if (modelfile0 eq 'chu3') then begin
    modelfile=modeldisk + modelname + modelident + '.chu3'
  endif else begin
    modelfile=modelfile0
  endelse
endif else begin
  modelfile=modeldisk + modelname + modelident + '.mean'
endelse

help, modelfile

if (keyword_set(datatype0)) then begin
  datatype=datatype0
;endif else if (keyword_set(listdata)) then begin
;  datatype=listdata.type
endif else begin
  datatype='uio'
endelse

if (keyword_set(q_name0)) then begin
  q_name=q_name0
endif else begin
  q_name=q_id
endelse

if (n_elements(q_weightmode0) ne 0) then begin
  q_weightmode=q_weightmode0
endif else begin
  q_weightmode=2
endelse
help, q_weightmode
;
if (keyword_set(c_q_id0)) then begin
  c_q_id=c_q_id0
endif else begin
  c_q_id=''
endelse
;
if (keyword_set(a_q_id0)) then begin
  a_q_id=a_q_id0
endif else begin
  a_q_id=''
endelse
;
if (keyword_set(s_q_id0)) then begin
  s_q_id=s_q_id0
endif else begin
  s_q_id=''
endelse
;
if (keyword_set(drange0)) then begin
  drange=drange0
endif else begin
  drange=[-1.0, 1.0]
endelse
;
if (n_elements(n_subframe0) ne 0) then begin
  n_subframe=fix(n_subframe0) > 1
endif else begin
  n_subframe=2
endelse
help,n_subframe
;
if (keyword_set(n_timestep0)) then begin
  n_timestep=n_timestep0
endif else begin
  n_timestep=30000L
endelse
;
if (keyword_set(n_firststep0)) then begin
  n_firststep=n_firststep0
endif else begin
  n_firststep=0L
endelse
;
if (keyword_set(MPG_file0)) then begin
  MPG_file=MPG_file0
  if (keyword_set(jpg_flag0)) then begin
    jpg_file=MPG_file + '.jpg'
  endif else begin
    jpg_file=''
  endelse
endif else begin
  MPG_file=modelname + '_' + q_name + '.mpg'
  if (keyword_set(jpg_flag0)) then begin
    jpg_file=modelname + '_' + q_name + '.jpg'
  endif else begin
    jpg_file=''
  endelse
endelse
;
if (keyword_set(mpg_quality0)) then begin
  mpg_quality=mpg_quality0
endif else begin
  mpg_quality=85
endelse
;
if (keyword_set(bubble0)) then begin
  bubble=bubble0
endif else begin
  bubble=0
endelse
;
if (keyword_set(b_scale0)) then begin
  b_scale=fix(b_scale0) > 1
endif else begin
  b_scale=2
endelse
;
if (keyword_set(b_rgb0)) then begin
  b_rgb=b_rgb0
endif else begin
  b_rgb=[40.0, 40.0, 40.0]
endelse
b_f=(1.0-mean(b_rgb)/255.0) > 0.5
;
if (keyword_set(w_xsize0)) then begin
  w_xsize=((fix(w_xsize0)/4)*4) > 4
endif else begin
  w_xsize=4*127
endelse
;
if (keyword_set(w_ysize0)) then begin
  w_ysize=((fix(w_ysize0)/4)*4) > 4
endif else begin
  w_ysize=w_xsize+24
endelse
;
if (keyword_set(device0)) then begin
  device=strlowcase(device0)
  if (device ne 'ps') then device='x'
endif else begin
  device='x'
endelse
mpg_flag=(device eq 'x')
mpgOut_flag=(MPG_quality ge 0.0)
;
if (keyword_set(nmlist)) then begin
  nmlist=nmlist0
endif else begin
  nmlist=[0,1,2,3]
endelse
;
; --- Define smoothing kernel ---
kernel=[[1.0, 2.0, 1.0], $
        [2.0, 4.0, 2.0], $
        [1.0, 2.0, 1.0]]
kernel=kernel/total(kernel)
;
; --- Circle ---
nangle=19
angle=(findgen(nangle)) * (!PI*2./(nangle-1))
xcirc=cos(angle) & ycirc=sin(angle)
;
if (datatype eq 'uio') then begin
  flux0=!con.sigmasb*par.Teff^4
endif
;
; --- Open window ---
win_id=0
window, win_id, xsize=w_xsize, ysize=w_ysize, retain=2
;
if (mpg_flag and mpgOut_flag) then begin
  ; --- Open MPEG file ---
  print, 'Open MPEG file'
  if (keyword_set(w_swap)) then begin
    MPG_ID=MPEG_open([w_ysize, w_xsize], quality=mpg_quality)
  endif else begin
    MPG_ID=MPEG_open([w_xsize, w_ysize], quality=mpg_quality)
  endelse
endif
;
if (bubble ne 0) then begin
  ; --- Initialization necessary for bubble plot ---
  ;
  ; --- List first dataset ---
  ;ful=uio_datasetlist_rd(modelfile, listdata=listdatainit, ierr=ierr)
  ;uio_closrd, listdatainit.channel
  ful=rd_datasetlist(modelfile, datatype=datatype, listdata=listdatainit, ierr=ierr)
  if (datatype eq 'uio') then begin
    uio_closrd, listdatainit.channel
  endif
  ;
  ; --- Dataset size ---
  n1=n_elements(ful.z.xc1)
  n2=n_elements(ful.z.xc2)
  n3=n_elements(ful.z.xc3)
  ;
  if ((n1 gt 1) and (n2 gt 1) and (n3 gt 1)) then begin
    ; --- Take slice out of the 3D array ---
    n2org=n2
    ful=chunkbox(ful, [[0,99999], [n2org/2L,n2org/2L], [0,99999L]])
    n1=n_elements(ful.z.xc1)
    n2=n_elements(ful.z.xc2)
    n3=n_elements(ful.z.xc3)
    chunk_flag=1
  endif else begin
    chunk_flag=0
  endelse
  ;
  if (n1 gt 1) then begin
    xc=reform(ful.z.xc1)
    if (n2 gt 1) then begin
      yc=reform(ful.z.xc2)
    endif else begin
      yc=reform(ful.z.xc3)
    endelse
  endif else begin
    xc=reform(ful.z.xc2)
    yc=reform(ful.z.xc3)
  endelse

  ;
  nx=n_elements(xc)
  ny=n_elements(yc)
  ;
  if ( ((yc(ny-1)-yc(0)) / (xc(nx-1)-xc(0))) lt (float(w_ysize)/float(w_xsize)) ) then begin
    aspect_xsize=w_xsize
    aspect_ysize=fix(float(w_xsize) * (yc(ny-1)-yc(0)) / (xc(nx-1)-xc(0)))
  endif else begin
    aspect_xsize=fix(float(w_ysize) * (xc(nx-1)-xc(0)) / (yc(ny-1)-yc(0)))
    aspect_ysize=w_ysize
  endelse
  ;
  ix_new=findgen(aspect_xsize)/float(aspect_xsize-1)*float(b_scale*nx-1)
  iy_new=findgen(aspect_ysize)/float(aspect_ysize-1)*float(b_scale*ny-1)
  ;

  ix_old0=findgen(aspect_xsize)/float(aspect_xsize-1)*float(nx-1)
  iy_old0=findgen(aspect_ysize)/float(aspect_ysize-1)*float(ny-1)
  xcn=(xc-min(xc))/(max(xc)-min(xc))*(nx-1)
  ycn=(yc-min(yc))/(max(yc)-min(yc))*(ny-1)
  ix_old=interpol(findgen(nx), xcn, ix_old0)
  iy_old=interpol(findgen(ny), ycn, iy_old0)

print, format='(A,I5,2X,6F7.2)', $
  'ix_new: ', n_elements(ix_new), ix_new(0:2),ix_new(n_elements(ix_new)-3:n_elements(ix_new)-1)
print, format='(A,I5,2X,6F7.2)', $ 
  'iy_new: ', n_elements(iy_new), iy_new(0:2),iy_new(n_elements(iy_new)-3:n_elements(iy_new)-1)
print, format='(A,I5,2X,6F7.2)', $
  'ix_old: ', n_elements(ix_old), ix_old(0:2),ix_old(n_elements(ix_old)-3:n_elements(ix_old)-1)
print, format='(A,I5,2X,6F7.2)', $ 
  'iy_old: ', n_elements(iy_old), iy_old(0:2),iy_old(n_elements(iy_old)-3:n_elements(iy_old)-1)
  ;
  ; --- Initialize bubble amplitude array ---
;  bubble2d_init, b_scale*nx, b_scale*ny, u0
  bubble2d_init, n_elements(ix_old), n_elements(iy_old), u0
  u=u0
help, u0
  ;
endif

starttimestep=(n_firststep-1L) > 0L
if (datatype ne 'uio') then begin
  ful=rd_datasetlist(modelfile, datatype=datatype, fileincrement=fileincrement, listdata=listdata, ierr=ierr)
  listdata.open_number=starttimestep
endif

; --- Initialize some counters ---
i_frame=-1
i_timestep=starttimestep
ierr=0
itime_last=-10000L
;
; --- Main loop over time steps ------------------------------------------------
while ((ierr eq 0) and (i_timestep lt n_timestep)) do begin
  ; --- Read next dataset ---
  ;ful=uio_datasetlist_rd(modelfile, listdata=listdata, ierr=ierr)
  ful=rd_datasetlist(modelfile, datatype=datatype, fileincrement=fileincrement, listdata=listdata, ierr=ierr)
  ;
  ; --- Check error status (end of dataset list) ---
  if (ierr eq 0) then begin
    if ((ful.z.itime gt (itime_last+1)) or (strlowcase(datatype) ne 'uio')) then begin
      ; --- Dataset size ---
      n1=n_elements(ful.z.xc1)
      n2=n_elements(ful.z.xc2)
      n3=n_elements(ful.z.xc3)
      ;
      if ((n1 gt 1) and (n2 gt 1) and (n3 gt 1)) then begin
        ; --- Take slice out of the 3D array ---
        n2org=n2
        ful=chunkbox(ful, [[0,99999], [n2org/2L,n2org/2L], [0,99999L]])
        n1=n_elements(ful.z.xc1)
        n2=n_elements(ful.z.xc2)
        n3=n_elements(ful.z.xc3)
        chunk_flag=1
      endif else begin
        chunk_flag=0
      endelse
      ;
      ; --- Compute EOS (if supplied) ---
      if (n_elements(EOS) ne 0) then eosbox, ful, eos=EOS, /opa
      ;
      print, '--- ', i_timestep, ful.z.itime, ful.z.time, ' ---' , format='(A,I4,I8,E15.8,A)'
      ;
      ; --- Compute quantity to display ---
      x1p=0.0
      if            (strlowcase(q_id) eq 'i1bl') then begin
        ; --- Surface intensities ---
        ;
        q=reform(abs(ful.rad.intb1_l))/flux0
        q_titlet='Surface Intensity(1l)'
        x1p=reform(ful.rad.xc2)
        x2p=reform(ful.rad.xc3)
      endif else if (strlowcase(q_id) eq 'i1br') then begin
        q=reform(abs(ful.rad.intb1_r))/flux0
        q_titlet='Surface Intensity(1r)'
        x1p=reform(ful.rad.xc2)
        x2p=reform(ful.rad.xc3)
      endif else if (strlowcase(q_id) eq 'i2bl') then begin
        q=reform(abs(ful.rad.intb2_l))/flux0
        q_titlet='Surface Intensity(2l)'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc3)
      endif else if (strlowcase(q_id) eq 'i2br') then begin
        q=reform(abs(ful.rad.intb2_r))/flux0
        q_titlet='Surface Intensity(2r)'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc3)
      endif else if (strlowcase(q_id) eq 'i3bl') then begin
        q=reform(abs(ful.rad.intb3_l))/flux0
        q_titlet='Surface Intensity(3l)'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i3br') then begin
        q=reform(abs(ful.rad.intb3_r))/flux0
        q_titlet='Surface Intensity(3r)'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i013br') then begin
        q=reform(abs(ful.rad.int01b3_r))/flux0
        q_titlet='Surface Intensity(3r) bin1'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i023br') then begin
        q=reform(abs(ful.rad.int02b3_r))/flux0
        q_titlet='Surface Intensity(3r) bin2'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i033br') then begin
        q=reform(abs(ful.rad.int03b3_r))/flux0
        q_titlet='Surface Intensity(3r) bin3'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i043br') then begin
        q=reform(abs(ful.rad.int04b3_r))/flux0
        q_titlet='Surface Intensity(3r) bin4'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
      endif else if (strlowcase(q_id) eq 'i053br') then begin
        q=reform(abs(ful.rad.int05b3_r))/flux0
        q_titlet='Surface Intensity(3r) bin5'
        x1p=reform(ful.rad.xc1)
        x2p=reform(ful.rad.xc2)
        ;
      endif else if (strlowcase(q_id) eq 'divv') then begin
        ; --- Divergence of velocity ---
        ;
        n1=n_elements(ful.z.xc1)
        n2=n_elements(ful.z.xc2)
        q=(              shift(reform(ful.z.v1 ),-1,0)-shift(reform(ful.z.v1 ),1,0) ) / $
            rebin(reform(shift(reform(ful.z.xc1),-1  )-shift(reform(ful.z.xc1),1  ),n1,1),n1,n2) $
           + $
          (              shift(reform(ful.z.v2 ),0,-1)-shift(reform(ful.z.v2 ),0,1) ) / $
            rebin(reform(shift(reform(ful.z.xc2),  -1)-shift(reform(ful.z.xc2),  1),1,n2),n1,n2)
        q(0,*)=q(1,*)
        q(n1-1,*)=q(n1-2,*)
        q(*,0)=q(*,1)
        q(*,n2-1)=q(*,n2-2)
        q_titlet='div(V12)'
        ;
      endif else if (strlowcase(q_id) eq 'vort') then begin
        ; --- Vorticity (velocity) ---
        ;
        n1=n_elements(ful.z.xc1)
        n2=n_elements(ful.z.xc2)
        n3=n_elements(ful.z.xc3)
        if (n3 eq 1) then begin
          q=reform(derivbox('rotv3', ful))
          q_titlet='rot(V12)'
          x1p=reform(ful.z.xc1)
          x2p=reform(ful.z.xc2)
        endif else if (n1 eq 1) then begin
          q=reform(derivbox('rotv1', ful))
          q_titlet='rot(V23)'
          x1p=reform(ful.z.xc2)
          x2p=reform(ful.z.xc3)
        endif else begin
          q=reform(derivbox('rotv2', ful))
          q_titlet='rot(V13)'
          x1p=reform(ful.z.xc1)
          x2p=reform(ful.z.xc3)
        endelse
        ;
      endif else if (strlowcase(q_id) eq 'absvort') then begin
        ; --- Vorticity (velocity) ---
        ;
        q=reform(derivbox('absrotv', ful))
        q_titlet='abs(rot(V))'
        if (n3 eq 1) then begin
          x1p=reform(ful.z.xc1)
          x2p=reform(ful.z.xc2)
        endif else if (n1 eq 1) then begin
          x1p=reform(ful.z.xc2)
          x2p=reform(ful.z.xc3)
        endif else begin
          x1p=reform(ful.z.xc1)
          x2p=reform(ful.z.xc3)
        endelse
        ;
      endif else begin
        ; --- General quantity ---
        ;
        q=reform(combox(q_id, ful, meansub=q_meansub))
        q_titlet=q_name
        print, 'total(' + q_name + '):', total(q)
        ;
      endelse
      ;
      if (keyword_set(q_title0)) then begin
        q_title=q_title0
      endif else begin
        q_title=q_titlet
      endelse 
      ;
      ; --- Process quantity further ---
      if (keyword_set(q_execute)) then begin
        result=execute(q_execute)
      endif
      if (keyword_set(q_log10)) then begin
        q=alog10(q)
      endif
      if (keyword_set(q_scale)) then begin
        q=q_scale*q
      endif
      if (keyword_set(q_smooth)) then begin
        ;q=convol(q, kernel, /edge_truncate, /center, q_smooth=q_smooth)
        q=smooth2d(q, q_smooth=q_smooth, periodic=1)
      endif
      if (keyword_set(q_median)) then begin
        if (q_median gt 1) then begin
          q=median(q, q_median)
        endif
      endif
      ;
      ; === Additional quantity ===
      if (a_q_id ne '') then begin
        ; --- General quantity ---
        a_q=reform(combox(a_q_id, ful, meansub=q_meansub))
      endif else begin
        a_q=0.0
      endelse
      ; --- Process quantity further ---
      if (keyword_set(a_q_execute)) then begin
        result=execute(a_q_execute)
      endif
      if (keyword_set(a_q_log10)) then begin
        a_q=alog10(a_q)
      endif
      if (keyword_set(a_q_scale)) then begin
        a_q=a_q_scale*a_q
      endif
      if (keyword_set(a_q_smooth)) then begin
        ; a_q=convol(a_q, kernel, /edge_truncate, /center, q_smooth=q_smooth)
        a_q=smooth2d(a_q, q_smooth=q_smooth, periodic=1)
      endif
      if (keyword_set(a_q_median)) then begin
        if (a_q_median gt 1) then begin
          a_q=median(a_q, a_q_median)
        endif
      endif
      ;
      ; --- Keep last three time steps ---
      if (i_timestep eq starttimestep) then begin
        q1   =q          & q2   =q          & q3   =q
        a_q1 =a_q        & a_q2 =a_q        & a_q3 =a_q
        time1=ful.z.time & time2=ful.z.time & time3=ful.z.time
      endif else begin
        q1   =q2         & q2   =q3         & q3   =q
        a_q1 =a_q2       & a_q2 =a_q3       & a_q3 =a_q
        time1=time2      & time2=time3      & time3=ful.z.time
      endelse
      ;
      if (bubble ne 0) then begin
;         v1_in=rebin(reform(ful.z.v1),  b_scale*n1, b_scale*n2)
;         x1_in=rebin(reform(ful.z.xc1), b_scale*n1            )
;         if (n2 gt 1) then begin
;           v2_in=rebin(reform(ful.z.v2),  b_scale*n1, b_scale*n2)
;           x2_in=rebin(reform(ful.z.xc2),             b_scale*n2)
;         endif else begin
;           v2_in=rebin(reform(ful.z.v3),  b_scale*n1, b_scale*n2)
;           x2_in=rebin(reform(ful.z.xc3),             b_scale*n2)
;         endelse

        if (n3 eq 1) then begin
          v1_in=interpolate(reform(ful.z.v1) , ix_old, iy_old, /grid)
          x1_in=interpolate(reform(ful.z.xc1), ix_old               )
          v2_in=interpolate(reform(ful.z.v2 ), ix_old, iy_old, /grid) 
          x2_in=interpolate(reform(ful.z.xc2),         iy_old       )
        endif else if (n1 eq 1) then begin
          v1_in=interpolate(reform(ful.z.v2) , ix_old, iy_old, /grid)
          x1_in=interpolate(reform(ful.z.xc2), ix_old               )
          v2_in=interpolate(reform(ful.z.v3 ), ix_old, iy_old, /grid) 
          x2_in=interpolate(reform(ful.z.xc3),         iy_old       )
        endif else begin
          v1_in=interpolate(reform(ful.z.v1) , ix_old, iy_old, /grid)
          x1_in=interpolate(reform(ful.z.xc1), ix_old               )
          v2_in=interpolate(reform(ful.z.v3 ), ix_old, iy_old, /grid) 
          x2_in=interpolate(reform(ful.z.xc3),         iy_old       )
        endelse
        ;
        bubble2d_sequence, u, n_subframe, par.dtime_out_mean, $
          x1_in, x2_in, $
          v1_in, v2_in, $
          u_arr
      endif
      ;
      if (i_timestep ge n_firststep) then begin
        ; --- Loop over sub time steps ---
        for i_subframe=0,n_subframe do begin
          ; --- Weights to average the three available time steps ---
          if            (q_weightmode eq 0) then begin
            w1=0.0          & w2=0.0    & w3=1.0
          endif else if (q_weightmode eq 1) then begin
            mu=float(i_subframe  )/float(n_subframe)
            w1=0.0          & w2=1.0-mu & w3=mu
          endif else if (q_weightmode eq 2) then begin
            mu=float(i_subframe  )/float(n_subframe)
            w1=0.5*(1.0-mu) & w2=0.5    & w3=0.5*mu
          endif else if ((q_weightmode ge 10) and (q_weightmode le 14)) then begin
            mu=float(i_subframe  )/float(n_subframe)
            w1=0.0          & w2=1.0-mu & w3=mu
          endif else begin
            w1=0.0          & w2=0.0    & w3=1.0
          endelse
          ;
          if ((i_timestep eq starttimestep) and (i_subframe eq 0)) then begin
;            plot_flag=1
            plot_flag=0
;            if (bubble ne 0) then ut=interpolate(reform(u0), ix_new, iy_new, /grid)
            if (bubble ne 0) then ut=reform(u0)
          endif else if (i_subframe gt 0) then begin
            plot_flag=1
;            if (bubble ne 0) then ut=interpolate(reform(u_arr(*,*,i_subframe-1)), ix_new, iy_new, /grid)
            if (bubble ne 0) then ut=reform(u_arr(*,*,i_subframe-1))
          endif else begin
            plot_flag=0
          endelse
          if (not mpg_flag) then begin
            plot_flag=(plot_flag) and (where(i_timestep eq nmlist) ge 0)
          endif
          ;
          if (plot_flag eq 1) then begin
            ; --- Plot and extract next frame ---
            i_frame=i_frame+1
            print, '--- ', i_timestep, i_subframe, i_frame, systime(0), ' ---' , $
              format='(A,I5,I4,I6,2X,2A)'
            ;
            ; --- Average (interpolate) quantity using the three time steps ---
            print, 'w1,w2,w3: ', w1,w2,w3
            if (q_weightmode eq 10) then begin
              q    = (w2*q2+w3*q3) - (w2*q1+w3*q2)
            endif else if (q_weightmode eq 11) then begin
              q    = 1.0E-04*reform(ful.eos.T) * ((w2*q2+w3*q3) - (w2*q1+w3*q2))
            endif else if (q_weightmode eq 12) then begin
              q    = (1.0E-04*reform(ful.eos.T))^2 * ((w2*q2+w3*q3) - (w2*q1+w3*q2))
            endif else if (q_weightmode eq 13) then begin
              q    = (1.0E-04*reform(ful.eos.T))^3 * ((w2*q2+w3*q3) - (w2*q1+w3*q2))
            endif else if (q_weightmode eq 14) then begin
              q    = (1.0E-04*reform(ful.eos.T))^4 * ((w2*q2+w3*q3) - (w2*q1+w3*q2))
            endif else begin
              q    = w1*q1           + w2*q2         + w3*q3
            endelse
            a_q  = w1*a_q1           + w2*a_q2       + w3*a_q3
            time = w1*time1          + w2*time2      + w3*time3
            stime=(w1*(i_timestep-1) + w2*i_timestep + w3*(i_timestep+1)) > 0.0
            ;
            if (keyword_set(q_atan)) then begin
              q=(2.0/!pi)*atan(q/(2.0/!pi))
            endif
            ;
            if (keyword_set(a_q_atan)) then begin
              a_q=(2.0/!pi)*atan(a_q/(2.0/!pi))
            endif
            ;
            if ((max(abs(ful.z.xb1)) > max(abs(ful.z.xb2)) > max(abs(ful.z.xb3))) gt 5.0E12) then begin
              time_title='time(' + string(stime,format='(I4)') + ')=' + $
                         string(time/(3600.0*24.0*365.2425), format='(F6.3)') + $
                        ' yrs'
            endif else if ((max(abs(ful.z.xb1)) > max(abs(ful.z.xb2)) > max(abs(ful.z.xb3))) gt 1.0E10) then begin
              time_title='time(' + string(stime,format='(I4)') + ')=' + $
                         string(time/(3600.0*24.0), format='(F7.3)') + $
                        ' d'
            endif else if ((max(abs(ful.z.xb1)) > max(abs(ful.z.xb2)) > max(abs(ful.z.xb3))) gt 1.0E9) then begin
              time_title='time(' + string(stime,format='(I4)') + ')=' + $
                         string(time/(3600.0), format='(F7.3)') + $
                        ' h'
            endif else begin
              time_title='time(' + strmid(string(stime,format='(I4)'),5,/reverse_offset) + ')=' + $
                         string(time, format='(F8.1)') + $
                        ' s'
            endelse
            ;
            if ((w_xsize eq 320) and (w_ysize eq 240)) then begin
              xytitle=[modelname , $
                       q_title , $
                       time_title]

;              xytitle=modelname + '!C' + $
;                      q_title + '!C' + $
;                      time_title
              title=''
              ;
;              title=modelname + '!C' + q_title + '!C' + time_title
            endif else if ((w_xsize eq 240) and (w_ysize eq 320)) then begin
              xytitle=''
              title='model: ' + modelname + '!C' + $
                    q_title + '!C' + $
                    time_title
            endif else begin
              ;
              xytitle=''
              title=modelname + ': ' + q_title + ', ' + time_title
            endelse
            ;
            ; --- Plot scalar quantity ---
            if (n_elements(x1p) le 1) then begin
              if (n_elements(ful.z.xc1) gt 1) then begin
                x1p=reform(ful.z.xc1)
                if (n_elements(ful.z.xc2) gt 1) then begin
                  x2p=reform(ful.z.xc2)
                endif else begin
                  x2p=reform(ful.z.xc3)
                endelse
              endif else begin
                x1p=reform(ful.z.xc2)
                x2p=reform(ful.z.xc3)
              endelse
            endif
            ;
;help, a_q
;print,minmax(a_q)
            plotfield, x1p*xfac, x2p*xfac, q, datamask=a_q, $
              title=title, $
              drange=drange, $
              /tv, t_col=t_colorrange, t_sample=t_sample, $
              device=device, /psnoinit, /noclose
            ;
            if ((w_xsize eq 320) and (w_ysize eq 240)) then begin
;              xyouts, /dev, w_ysize, w_ysize-10, xytitle
;              xyouts, /dev, alignment=1.0, w_xsize-1, w_ysize-10, xytitle
              noclip_old=!P.NOCLIP
              !P.NOCLIP=1
              xwindow_old=!X.WINDOW
              ywindow_old=!Y.WINDOW
              !X.WINDOW=[0.0,1.0]
              !Y.WINDOW=[0.0,1.0]
              legend, xytitle, xalignment=1.0, margin=0.0, box=0
              !P.NOCLIP=noclip_old
              !X.WINDOW=xwindow_old
              !Y.WINDOW=ywindow_old
            endif
            ;
            if (a_q_id ne '') then begin
              ; --- Reduce brightness ---
              ;mask=0.0 > q < 1.0
              ; ...
            endif
            ;
            if ((bubble ne 0) and mpg_flag) then begin
              ; --- Overlay bubbles ---
              image=tvrd(0, 0, !D.X_VSIZE, !D.Y_VSIZE, true=3)
;              image[0:aspect_xsize-1, 0:aspect_ysize-1,0]=byte((float(image[0:aspect_xsize-1, 0:aspect_ysize-1,0]) + $
;                                                                (b_rgb[0]*(0.5 + (-0.5>ut<1.5))))<255.0)
;              image[0:aspect_xsize-1, 0:aspect_ysize-1,1]=byte((float(image[0:aspect_xsize-1, 0:aspect_ysize-1,1]) + $
;                                                                (b_rgb[1]*(0.5 + (-0.5>ut<1.5))))<255.0)
;              image[0:aspect_xsize-1, 0:aspect_ysize-1,2]=byte((float(image[0:aspect_xsize-1, 0:aspect_ysize-1,2]) + $
;                                                                (b_rgb[2]*(0.5 + (-0.5>ut<1.5))))<255.0)
              image[0:aspect_xsize-1, 0:aspect_ysize-1,0]=byte((b_f*float(image[0:aspect_xsize-1, 0:aspect_ysize-1,0]) + $
                                                                (b_rgb[0]*(0.5 + (-0.5>ut<1.5))))<255.0)
              image[0:aspect_xsize-1, 0:aspect_ysize-1,1]=byte((b_f*float(image[0:aspect_xsize-1, 0:aspect_ysize-1,1]) + $
                                                                (b_rgb[1]*(0.5 + (-0.5>ut<1.5))))<255.0)
              image[0:aspect_xsize-1, 0:aspect_ysize-1,2]=byte((b_f*float(image[0:aspect_xsize-1, 0:aspect_ysize-1,2]) + $
                                                                (b_rgb[2]*(0.5 + (-0.5>ut<1.5))))<255.0)
              tv, image, true=3
            endif
            ;
            if (keyword_set(a_circle)) then begin
              if (total(tag_names(par) eq 'R0_CORE') gt 0.0) then begin
                if (par.r0_core gt 0.0) then begin
                  r_circle=par.r0_core*xfac
                endif else begin
                  r_circle=par.r0_grav*xfac
                endelse
              endif else begin
                r_circle=par.r0_grav*xfac
              endelse
              oplot, r_circle*xcirc, r_circle*ycirc, color=!ctv.white
              oplot, r_circle*xcirc, r_circle*ycirc, color=!ctv.black, linestyle=2
            endif
            ;
            if (c_q_id ne '') then begin
              ; --- Contour lines (from current time step) ---
              c_q=reform(combox(c_q_id, ful, meansub=c_q_meansub, par=par))
              plotfield, x1p*xfac, x2p*xfac, c_q, $
                /contour, /over, $
                  c_nlevels=c_nlevels0, c_levels=c_levels0, $
                  c_annotation=c_annotation, c_labels=c_labels, c_fill=c_fill, $
                  c_charsize=c_charsize, c_thick=c_thick0, $
                  c_color=c_color, c_linestyle=c_linestyle0, c_smooth=c_smooth, $
                device=device, /psnoinit, /noclose
            endif
            ;
            if (s_q_id ne '') then begin
              ; --- Stream lines (from current time step) ---
              s_q1=reform(combox(s_q_id+'1', ful))
              s_q2=reform(combox(s_q_id+'3', ful))
              plotfield, x1p*xfac, x2p*xfac, s_q1, xvec=s_q1, yvec=s_q2, $
                /str, /over, $
                  s_randomstart=s_randomstart, s_rows=s_rows, s_columns=s_columns, $
                  s_timestep=s_timestep, s_nsteps=s_nsteps, s_norder=s_norder, $
                  s_thick=s_thick, s_color=s_color, s_arrowsize=s_arrowsize, $
                device=device, /psnoinit, /noclose
            endif
            ;
            ;
            if (mpg_flag) then begin
              ; --- Use image as MPEG frame ---
              if (keyword_set(w_swap)) then begin
                image=tvrd(0, 0, !D.X_VSIZE, !D.Y_VSIZE, true=1)
                image1=bytarr(3, !D.Y_VSIZE, !D.X_VSIZE)
                image1(0,*,*)=rotate(reform(image(0,*,*)), 3)
                image1(1,*,*)=rotate(reform(image(1,*,*)), 3)
                image1(2,*,*)=rotate(reform(image(2,*,*)), 3)
              endif else begin
                image1=tvrd(0, 0, !D.X_VSIZE, !D.Y_VSIZE, true=1)
              endelse
              if (mpgOut_flag) then MPEG_put, MPG_ID, image=image1, frame=i_frame, /color, /order
              ;
              if (keyword_set(single_frame)) then begin
                str_i_frame='0000' + strtrim(string(i_frame),2)
                str_i_frame=strmid(str_i_frame,strlen(str_i_frame)-5,5)
                image_file='tmp/' + modelname + '_' + q_name + '_' + str_i_frame + '.jpg'
                write_jpeg, image_file, image1, order=0, quality=90, true=1
              endif
              ;
              if ((i_frame eq 0) and keyword_set(jpg_flag0) and (jpg_file ne '')) then begin
                write_jpeg, jpg_file, image1, order=0, quality=90, true=1
              endif
              ;
            endif ; (mpg_flag)
          endif ; (plot_flag eq 1)
        endfor ; i_subframe
      endif ; (i_timestep ge n_firststep)
      ;
      i_timestep=i_timestep+1
      if (ierr eq 0) then itime_last=ful.z.itime
      ;
    endif ; ((ful.z.itime gt (itime_last+1)) or (strlowcase(datatype) ne 'uio'))
  endif ; (ierr eq 0)
endwhile ; ((ierr eq 0) and (i_timestep lt n_timestep))
; --- End of main loop over time steps -----------------------------------------
;
if (mpg_flag and mpgOut_flag) then begin
  print, 'Make MPEG file ' + MPG_file
  MPEG_save, MPG_ID, filename=MPG_file
  MPEG_close, MPG_ID
endif
;
end ; rhd_mpg
