; pro rhd_mpg_savmake
;
;------------------------------------------------------------------------------
;+
; NAME:
;   rhd_mpg_savmake ('rhd_mpeg_save-file_make')
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;   @rhd_mpg_savmake
;
; INPUTS:
;
; OUTPUTS:
;
; KEYWORDS:
;
; COMMON BLOCKS:
;
; ROUTINES:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;   Before first use type
;     delvar, eos
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;   17. 5.99 Written by Bernd Freytag, Astronomical Observatory Copenhagen
;    3. 4.01 (B.F. Uppsala) rhd_plot -> rhd_movie
;    4. 4.02 (B.F.) rhd_movie -> rhd_mpg_make
;   2009-07-29 (B.F.) rhd_mpg_make -> rhd_mpg_savmake
;-
;------------------------------------------------------------------------------
;
; ================================
if (not keyword_set(external_control)) then begin &$
  ;stop &$
  ;
  ;Model='mt15g50mm00n06' &$
  Model='mt22g50mm00n08' &$
  ;SavIdent='surf_Pfac10' & qid='v3mass' &$
  ;SavIdent='surf_Pfac10' & qid='s' &$
  SavIdent='intens' & qid='I3r' &$
  ;
  ;n_subframe=2 & q_weightmode=2
  ;n_subframe=1 & q_weightmode=0 &$
  ;w_xsize=320 & w_ysize=240 & w_swap=0
  ;w_xsize=240 & w_ysize=320 & w_swap=1
  ;w_xsize=320 & w_ysize=200 & w_swap=0
  w_xsize=512 & w_ysize=532 & w_swap=0 &$
  ;w_xsize=640 & w_ysize=380 & w_swap=0
  ;
  ;Irange=[0.15, 0.6]    ; --- at80g40n20
  ;Irange=[0.15, 0.7]    ; --- at80g40n21/n22
  ;Irange=[0.15, 0.33]    ; --- at80g44n10
  ;Irange=[0.15, 0.50]    ; --- at85g44n10_3D
  ;Irange=[0.2, 0.55]    ; --- at85g44n13
  ;Irange=[0.15, 0.55]    ; --- at85g40n01
  ;Irange=[0.25, 0.48]    ; --- at85g40n02
  ;Irange=[0.0, 0.9]
  ;Irange=[0.0, 1.0]
  ;Irange=[0.0, 1.1]
  ;Irange=[0.0, 1.2] &$
  qRange=[0.0, 0.0] &$
  ;q_smooth=1 &$
  ;q_median=5 &$
  ;q_median=3 &$
  ;q_median=0 &$
  n_timestep=10000L &$
  n_firststep=0L &$
  d_timestep=1L &$
  ;
  ;t_colorrange=!ctv.ct0 &$
  ;t_colorrange=!ctv.ct3 &$
  t_colorrange=!ctv.ct8 &$
  ;jpg_flag=0 &$
  single_frame_flag=1 &$
  mpg_quality=95 &$
  ;
  xfac=1.0E-05 & xunit='[km]' &$
endif
; ================================


;modeldisk=getenv('HOME') + '/dat/rhd/d' + model + '/'
;eosdisk=getenv('HOME') + '/for/eos/dat/'
;opadisk=getenv('HOME') + '/for/opa/dat/'

; --- Read EOS file ---
;par=uio_struct_rd(modeldisk + model + parmodelident + '.par')
;if (n_elements(EOS) eq 0) then begin &$
;  tabinter_rdcoeff, eosdisk + par.eosfile, EOS &$
;  dfopta, opadisk + par.opafile &$
;endif

RestoreFile=Model + '_' + SavIdent + '.sav'
surf=0
intens=0
restore, RestoreFile, /verbose

!P.BACKGROUND=!ctv.black
!P.CHARSIZE=1.2
;!P.CHARSIZE=0.9
!X.STYLE=0      & !Y.STYLE=0
!X.MARGIN=[7,1] & !Y.MARGIN=[4,1]
;!X.STYLE=5      & !Y.STYLE=5
;!X.MARGIN=[0,0] & !Y.MARGIN=[0,0]

; --- Open window ---
win_id=0
window, win_id, xsize=w_xsize, ysize=w_ysize, retain=2

; --- Open MPEG file ---
print, 'Open MPEG file'
MPG_ID=MPEG_open([w_xsize, w_ysize], quality=mpg_quality)

if (max(abs(qrange)) eq 0.0) then begin &$
  ; --- Determine range of data values ---
  if (qid eq 'v3mass') then begin &$
    qrange=minmax(surf.z.v3mass) &$
    nTime=n_elements(surf.time) &$
    x1p=reform(surf.z.xc1)*xfac &$
    x2p=reform(surf.z.xc2)*xfac &$
    time=surf.z.time &$
    itimea=surf.z.itime &$
  endif else if (qid eq 's') then begin &$
    qrange=minmax(surf.z.s) &$
    nTime=n_elements(surf.time) &$
    x1p=reform(surf.z.xc1)*xfac &$
    x2p=reform(surf.z.xc2)*xfac &$
    time=surf.z.time &$
    itimea=surf.z.itime &$
  endif else if (qid eq 'I3r') then begin &$
    qrange=minmax(intens.i3r) &$
    nTime=n_elements(intens.time) &$
    x1p=reform(intens.mean.z1.xc1)*xfac &$
    x2p=reform(intens.mean.z2.xc2)*xfac &$
    time=intens.time &$
    itimea=intens.itime &$
  endif else begin &$
    print, 'Unknown quantity' &$
    stop &$
  endelse &$
endif


; --- Loop over all selected time steps ---
i_frame=0L
for iTime=(n_firststep > 0L),(nTime-1L < n_timestep),d_timestep do begin &$
  print, iTime &$
  ;
  if (qid eq 'v3mass') then begin &$
    q=reform(surf.z.v3mass[*,*,iTime]) &$
  endif else if (qid eq 's') then begin &$
    q=reform(surf.z.s[     *,*,iTime]) &$
  endif else if (qid eq 'I3r') then begin &$
    q=reform(intens.i3r[   *,*,iTime]) &$
  endif else begin &$
    print, 'Unknown quantity' &$
    stop &$
  endelse &$
  ;
;  title=Model + $
;        '  ' + qid + $
;        '  time(' + strmid(string(surf.z.itime[iTime],format='(F7.1)'),5,/reverse_offset) + ')=' + $
;        string(surf.z.time[iTime], format='(F8.1)') + ' s' &$
  title=Model + $
        ': ' + qid + $
        ', time(' + strmid(string(i_frame,format='(I4)'),5,/reverse_offset) + ')=' + $
        string(time[iTime], format='(I8)') + ' s' &$
  plotfield, x1p, x2p, q, $
    xtitle='x ' + xunit, ytitle='y ' + xunit, title=title, $
    /tv, t_colorrange=t_colorrange, $
    device='X' &$
  ;
  image1=tvrd(0, 0, !D.X_VSIZE, !D.Y_VSIZE, true=1) &$
  MPEG_put, MPG_ID, image=image1, frame=i_frame, /color, /order &$
  if (keyword_set(single_frame_flag)) then begin &$
    str_i_frame='0000' + strtrim(string(i_frame),2) &$
    str_i_frame=strmid(str_i_frame,strlen(str_i_frame)-5,5) &$
    image_file='tmp/' + Model + '_surf_' + qid + '_' + str_i_frame + '.jpg' &$
    write_jpeg, image_file, image1, order=0, quality=90, true=1 &$
  endif &$
  ;
  i_frame=i_frame+1L &$
  ;
endfor ; iTime

MPG_file=Model + '_surf_' + qid + '.mpg'
print, 'Make MPEG file ' + MPG_file
MPEG_save, MPG_ID, filename=MPG_file
MPEG_close, MPG_ID

!P.BACKGROUND=0
!P.CHARSIZE=0
!X.STYLE=0       & !Y.STYLE=0
!X.MARGIN=[12,3] & !Y.MARGIN=[4,2]

; convert -resize 160x160 -quality 90 st35gm04n26_Tbubble.jpg st35gm04n26_Tbubble_thumb.jpg

; end ; rhd_mpg_savmake
