cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
      subroutine rossow78_mod(p,t,grav,ro,ros,rov,size,  mcrit,rnucl,
     &     rcond,rcoag,rcoal)
c     Version for CO5BOLD: 07/11/2006
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Calcul of the caracteristical rates of cloud formation           c
c                  (after Rossow (1978), Icarus 30, 1-50)              c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Auteur: T. Guillot, F. Allard
c Entrees:
c     p,t,ro: pressure, temperature, gas density (cgs)
c     ros: mass density of grains (g/cm^-3)
c     rov: mass density of monomer in the gas phase (g cm^-3) 
c     grav: gravity (en cm/s2)
c     size: radius of the grains (cm)
c Sorties:
c     r... : rates of nucleation, condensation (g cm^-3 sec^-1)
c            coalescence and coagulation (g cm^-3 sec^-1)?
c     mcrit (g) : critical mass for a grain to be viable

      implicit none

      real*8 p,t,ro,grav,ros,rov,rovs
      real*8 size,rnucl,rcond,rcoag,rcoal,revap
      integer i,j
      real*8 nu,f,smax,alpha,sig,mu,rn,lambda,a70,rop
      real*8 logamin,logamax,kn,re,nn
      real*8 stock,stock1,stock2
      real*8 mv,mv1,mv2,ms,kgas,gibbs
      real*8 dgv,dgvcrit,rcrit,ncrit,mcrit,omega,gamma

      character*18 cname
      real*8 pi,kbol,amu,rgas
      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
cfa06 Rgas added for new viscosity from Bernd Freytag
      data rgas/8.3145d7/
      real*8 coef(5)
      real*8 dklog,dkgas,t1,su,th,th1,th2,y1,y2
      data cname,coef/'Mg2SiO4/c',7.52334e+04,-9.38369e+05,2.47581e+02,
     &                           -3.14980e-03, 0.00000e+00/                     


c-----------------------------------------------------------------------
 2000 format(7(1x,1pd10.3))

c     initialize outputs: 
      mcrit = 0.d0
      rnucl = 0.d0
      rcond = 0.d0
      rcoal = 0.d0
      rcoag = 0.d0

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     input concerning the type of grain or monomer:                   c
c     -Averaged grain material density (gr/cm^3) [rop]                 c
c     -Molar mass of a monomer (g) [ms]                                c
c     -Molar mass of constituting elements (g) [mv,mv1,mv2]            c
c     -Surface energy per unit area or flux units (erg cm^-2) [gamma]  c
c     -Stoichiometric coefficients of elements in monomer [stock]      c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rop = 3.3d0               !for dirty forsterite (Mg2SiO4)
      stock = 2.d0             !for Mg in forsterite
      stock1 = 1.d0            !for Si in forsterite
      stock2 = 4.d0            !for O in forsterite
      eps = 1.2427d-5           !for Mg in forsterite
      eps1= 1.1868d-5           !for Si in forsterite
      eps2= 1.6764d-4           !for O in forsterite
      mv = 24.31d0*amu          !for Mg in forsterite
      mv1= 28.09d0*amu          !for Si in forsterite
      mv2= 16.d0*amu            !for O in forsterite
      ms = 140.71d0*amu         !for Mg2SiO4
      rs = ( 3.d0*(stock*mv+stock1*mv1+stock2*mv2)/
     .     (rop*4.d0*pi) )**(1.d0/3.d0)
      gamma = 250.d0            !surface tension

c     compute partial pressures of constituting elements from
c     that of Mg2SiO4 monomer:
c     cte that doesn't participate    condensable vapor
      p0=                             stock*rov*kbol*t/ms
      p1= (eps1-stock1/stock*eps)*p + stock1*rov*kbol*t/ms
      p2 =(eps2-stock2/stock*eps)*p + stock2*rov*kbol*t/ms

c     log10 of right hand of equation keq = Pa * Pb * Pc ...
      dkgas= stock*log10(p0) + stock1*log10(p1) + stock2*log10(p2)

c     compute log10(keq) from polynomial coeffs by Sharp & Huebner (1992): 
      dklog= 99.d0
      if( t.gt.6000.d0 ) goto 10
      if(t.ge.1000.d0) then
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t**(j-3)
         enddo
         dklog = su/1.98717d0 * 0.4342944819d0
      else ! extrapolate linearly in theta
         t1= 1100.d0
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t1**(j-3)
         enddo
         y1 = su/1.98717d0 * 0.4342944819d0
         th1= 5040.d0/t1
         t1= 1000.d0
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t1**(j-3)
         enddo
         y2 = su/1.98717d0 * 0.4342944819d0
         th2= 5040.d0/t1
         t1 = t
         th= 5040.d0/t1
         dklog= (y1-y2)/(th1-th2)*(th-th2)+y2
      endif
 10   continue
c      if(dklog.eq.99.d0) then ! should be no grains
c         ros = 0.d0
c         size= 0.d0
c         return    
c      endif
             
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     rovs is the equilibrium value of rov, i.e.  when                 c 
c          keq(Mg2SiO4) = P(Mg)^2 * P(Si) * P(O)^4                     c
c     We can therefore obtain rovs from the equations derived above    c
c     for dkgas if we suppose that rovs*kt/ms <<< delta Epsilon*Pgas   c 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rovs = sqrt( 10**dklog / ( 4.d0*(kbol*t/ms)**2 * (eps1-0.5d0*eps)*
     &     (eps2-2.d0*eps)*p**2 ) )
c     or assume it equal to the grain mass density:     
c      rovs = ros

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Parameters:                                                      c
c     -Correction of finite mass [f]	 			       c
c     -Collage efficiency [alpha]				       c
c     -Cross-section of the gaz [sig]				       c
c     -Mean molecular weight of the gaz [mu]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      f=5.d0
      alpha=1.d0
      sig= pi*(2.d-8)**2     !cfa06: radius of H2 molecule, not HI!
c      sig=pi*(1.d-8)**2
      mu=ro*kbol*t/(p*amu)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Number density of monomers in the condensed phase: nn            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       nn=rovs/(4.d0/3.d0*pi*size**3*rop)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Atmospheric density by number of particules/cm3 [rn]	       c
c     -mean free path of the gaz [lambda]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rn=ro/mu/amu
      lambda=sqrt(3.d0*pi)/(8.d0*rn*sig)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -dynamical viscosity [nu]					       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      nu=ro/sqrt(t)
cfa06 une Bernd's viscosity!
      nu=ro*lambda*sqrt(8.d0*rgas*t/(pi*mu))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Size of particules for which Re=70                               c
c     Corrected according to Carlson et al. 1988                       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      a70=(9.d0*270.d0*nu*nu/(4.d0*rop*ro*grav))**(1.d0/3.d0)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Knudsen number [kn]                                             c
c     -Reynolds number [re]                                            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      kn=lambda/size
      re=70.d0*size/a70

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Supersaturation (Rossow p.2,3)        			       c
c         s>0: growth ; s<0: evaporation                               c
c      (here topped at 1.d-2 like in Phx but can be changed)           c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      smax = 1.d-2             
      if (rovs.ne.0.d0) then
         smax = rov/rovs-1.d0
      endif

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -change in free energy per unit volume  (erg/cm**2) [dgv]        c
c     -Gibbs free energy of formation of the grain (pressure) [dklog]  c
c        Keq/kgas < 1 for grains to form ==> dgv & dgcrit < 0          c
c     -barrier for vapor going to critical nucleus (erg) [dgcrit]      c
c     -critical number density of seed molecules (cm^-3)   [ncrit]     c
c     -critical radius for grains to start nucleating (cm)   [rcrit]   c
c     -flux of gas molecules/atoms onto grains (cm^-2 sec^-1) [omega]  c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cout  gibbs=kbol*t*log(rov/rovs)    !according to lecture
      gibbs = kbol*t*log(1.d0+smax) !according to Arizona Cloud code
c     gibbs=kbol*t*(dklog-dkgas)    !according to derivation
      dgv= gibbs/(4.d0*pi*rs**3/3.d0) 
      dgcrit= 16.d0*pi*gamma**3/(3.d0*dgv**2)
      ncrit=(rov/ms)*exp(-dgcrit/(kbol*t))   
      rcrit=-2.d0*gamma/dgv                                
      mcrit=rop*(4.d0*pi*rcrit**3/3.d0)
      omega=alpha*(rov-rovs)/ms*sqrt(rgas*t/(2.d0*pi*mv))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     rates of nucleation, condensation, sedimentation, coagulation,   c 
c     and coalescence                                                  c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     nucleation rate:  dros/dt
      rnucl=ncrit *4.d0*pi*rcrit**2 *omega *ms
      if (size.le.0.d0) size = rcrit

c     condensation rate: dros/dt
      if (kn.lt.alpha) then
c        laminar viscous flow (stokes)
         rcond=ros*(2.d0*f*nu*rovs*smax/rop)/(ro*size**2)
         if (re.gt.1.d0) then
c           turbulent flow (Newton)
            rcond=rcond*(1.d0+2.d-1*sqrt(re))
         endif
      else
c        subsonic free molecular flow
         rcond=ros*sqrt(2.d0*kbol*t/(pi*amu*mu))*
     &        (3.d0*alpha*f*rovs*smax/(2.d0*rop))/size
      endif

c     coagulation rate = dros/dt:
      if (kn.lt.1.d0) then
c        laminar viscous flow (stokes)
         rcoag=ros*(4.d0*kbol*t/(3.d0*nu))*nn
      else
c        subsonic free molecular flow
         rcoag=ros*4.d0*sqrt(3.d0*size*kbol*t/rop)*nn
      endif

c     coalescence rate = dros/dt:
      if (kn.gt.1.d0) then
c        subsonic free molecular flow
         rcoal=ros*(4.d0*pi*rop*grav/(27.d0*ro))*
     &       sqrt(pi*amu*mu/(2.d0*kbol*t))*size**3*nn
      elseif (re.gt.70.d0) then
c        turbulent flow (Newton)
         rcoal=ros*sqrt(10.d0*pi*pi*rop*grav*size**5/(3.d0*ro))*nn
      else
c        laminar viscous flow (stokes)
         rcoal=ros*(pi*rop*grav/(9.d0*nu))*size**4*nn
      endif
c     write(*,'(1p,8d10.3)')p,t,size,rnucl,rcond,rcoag,rcoal
      end
