cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
      subroutine rossow78_mod(p,t,grav,ro,ros,rov,  size,rnucl,rcond,
     &     rcoag,rcoal)
c     Version for CO5BOLD: 07/11/2006
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Calcul of the caracteristical rates of cloud formation           c
c                  (after Rossow (1978), Icarus 30, 1-50)              c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Auteur: T. Guillot, F. Allard
c Entrees:
c     p,t,ro: pressure, temperature, gas density (cgs)
c     ros: mass density of grains
c     rov: mass density of the constituting element in the gas phase 
c     grav: gravity (en cm/s2)
c Sorties:
c     size: radius of the grains
c     r(...) sec^-1: rates of nucleation, condensation, coagulation, 
c                    coalescence, and evaporation

      implicit none

      real*8 p,t,ro,grav,ros,rov
      real*8 size,rnucl,rcond,rcoag,rcoal,revap

      integer i
      real*8 nu,f,smax,alpha,sig,mu,rn,lambda,a70,rop
      real*8 logamin,logamax,kn,re,nn
      real*8 stock,stock1,stock2,stock_tot
      real*8 mv,mv1,mv2,ms,kgas,gibbs
      real*8 dgv,dgvcrit,rcrit,ncrit,omega,gamma

      character*18 cname
      real*8 pi,kbol,amu,rgas
      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
cfa06 Rgas added for new viscosity from Bernd Freytag
      data rgas/8.3145d7/
      real*8 coef(5),keq
      real*8 dklog,t1,su,th1,th2,y1,y2
      data cname,coef/'Mg2SiO4/c',7.52334e+04,-9.38369e+05,2.47581e+02,
     &                           -3.14980e-03, 0.00000e+00/                     


c-----------------------------------------------------------------------
 2000 format(7(1x,1pd10.3))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     input concerning the type of grain or monomer:                   c
c       (to be later replaced by data statements for several grains)   c
c     -Averaged grain material density in gr/cm^3 [rop]                c
c     -Molar mass of monomer [ms]                                      c
c     -surface energy per unit area or flux units [gamma]              c
c     -Stockiometric coefficient of element in monomer [stock]         c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rop = 3.8d0               !for dirty forsterite (MgFeSiO4)
      stock = 2.d0              !for Mg in forsterite
      stock1 = 1.d0             !for Si in forsterite
      stock2 = 4.d0             !for O in forsterite
      stock_tot = 7.d0          !for Mg2SiO4 in forsterite
      mv = 24.31d0*amu          !for Mg in forsterite
      mv1= 28.09d0*amu          !for Mg in forsterite
      mv2= 16.d0*amu            !for Mg in forsterite
      ms = 140.71d0*amu         !for Mg2SiO4
      gamma = 1.28d3            !g sec^-2  not sure of the units (1.28Jm2)

c     compute log10 of keq from polynomial coefficients by Sharp & Huebner (1992): 
      dklog= 99.d0
      if( t.gt.6000.d0 ) goto 10
      if(t.ge.1000.d0) then
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t**(j-3)
         enddo
         dklog = su/1.98717d0 * 0.4342944819d0
      else ! extrapolate linearly in theta
         t1= 1100.d0
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t1**(j-3)
         enddo
         y1 = su/1.98717d0 * 0.4342944819d0
         th1= 5040.d0/t1
         t1= 1000.d0
         su = 0.d0
         do j=1,5
            su = su + coef(j) * t1**(j-3)
         enddo
         y2 = su/1.98717d0 * 0.4342944819d0
         th2= 5040.d0/t1
         t1 = t
         th= 5040.d0/t1
         dklog= (y1-y2)/(th1-th2)*(th-th2)+y2
      endif
 10   continue

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Parameters:                                                      c
c     -Correction of finite mass [f]	 			       c
c     -Collage efficiency [alpha]				       c
c     -Cross-section of the gaz [sig]				       c
c     -Mean molecular weight of the gaz [mu]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      f=5.d0
      alpha=1.d0
      sig= pi*(2.d-8)**2.d0     !cfa06: radius of H2 molecule, not HI!
c      sig=pi*(1.d-8)**2
      mu=ro*kbol*t/(p*amu)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Number density of monomers: nn                                   c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       nn=ros/(4.d0/3.d0*pi*size**3.d0*rop)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Atmospheric density by number of particules/cm3 [rn]	       c
c     -mean free path of the gaz [lambda]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rn=ro/mu/amu
      lambda=sqrt(3.d0*pi)/(8.d0*rn*sig)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -dynamical viscosity [nu]					       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      nu=ro/sqrt(t)
cfa06 une Bernd's viscosity!
      nu=ro*lambda*sqrt(8.d0*rgas*t/(pi*mu))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Size of particules for which Re=70                               c
c     Corrected according to Carlson et al. 1988                       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      a70=(9.d0*270.d0*nu*nu/(4.d0*rop*ro*grav))**(1.d0/3.d0)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Knudsen number [kn]                                             c
c     -Reynolds number [re]                                            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      kn=lambda/size
      re=70.d0*size/a70

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Supersaturation           				       c
c         s>0: growth ; s<0: evaporation                               c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      smax=1d-2             
      smax = (rov/stock)/ros - 1.d0


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -change in free energy per unit volume  (erg/cm**2) [dgv]        c
c     -Gibbs free energy of formation of the grain (pressure) [dklog]  c
c        Keq/kgas < 1 for grains to form ==> dgv & dgcrit < 0          c
c     -barrier for vapor going to critical nucleus (erg) [dgcrit]      c
c     -critical number density of seed molecules (cm^-3)   [ncrit]     c
c     -critical radius for grains to start nucleating (cm)   [rcrit]   c
c     -flux of gas molecules/atoms onto grains (cm^-2 sec^-1) [omega]  c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       kgas= (rov/mv)**stock *(rov1/m1)**stock1 *(rov2/mv2)**stock2
     &      *(kbol*t)**stock_tot
cout   gibbs=kbol*t*log(ros/(rov/stock))  ! heterogeneous (growth on grains)
       gibbs=-kbol*t*(dklog-log10(kgas))  ! chemical (phase transition)
       dgv= gibbs/(4.d0*pi*lambda**3.d0/3.d0) 
       dgcrit= 16.d0*pi*gamma**3.d0/(3.d0*dgv**2.d0)
       ncrit=(rov/stock/mv)*exp(-dgcrit/(kbol*t))   
       rcrit=-2.d0*gamma/dgv                                
       omega=alpha*(rov/stock/mv-ros/ms)*sqrt(rgas*t/(2.d0*pi*mv))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     rates of nucleation, condensation, sedimentation, coagulation,   c 
c     and coalescence                                                  c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      nucleation rate (heterogeneous):  dN/dt; N= number density of grains
c      of size > rcrit
       rnucl=ncrit*4.d0*pi*rcrit**2.d0*omega
       if (size.le.0.d0) size = rcrit

c      condensation rate: 1/M dM/dt;  M= total mass of all grains
       if (kn.lt.alpha) then
c         laminar viscous flow (stokes)
          rcond=(2.d0*f*nu*ros*smax/rop)/(ro*size*size)
          if (re.gt.1.d0) then
c            turbulent flow (Newton)
             rcond=rcond*(1.d0+2.d-1*sqrt(re))
          endif
       else
c       subsonic free molecular flow
        rcond=sqrt(2.d0*kbol*t/(pi*amu*mu))*(3.d0*alpha*f*
     &             ros*smax/(2.d0*rop))/size
       endif

c      coagulation rate = (1/N) dN/dt:
       if (kn.lt.1.d0) then
c       laminar viscous flow (stokes)
        rcoag=(4.d0*kbol*t/(3.d0*nu))*nn
       else
c       subsonic free molecular flow
        rcoag=4.d0*sqrt(3.d0*size*kbol*t/rop)*nn
       endif

c      coalescence rate:
       if (kn.gt.1.d0) then
c       subsonic free molecular flow
        rcoal=(4.d0*pi*rop*grav/(27.d0*ro))*
     &       sqrt(pi*amu*mu/(2.d0*kbol*t))*size*size*size*nn
       elseif (re.gt.70.d0) then
c       turbulent flow (Newton)
        rcoal=sqrt(10.d0*pi*pi*rop*grav*size**5.d0/(3.d0*ro))*nn
       else
c       laminar viscous flow (stokes)
        rcoal=(pi*rop*grav/(9.d0*nu))*size**4.d0*nn
       endif
c       write(*,'(1p,8d10.3)')p,t,size,rnucl,rcond,rcoag,rcoal
      end
