cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
      subroutine rossow78_tfall(p,t,ro,grav,size, vfall)
c     Version: 11/10/06 by F. Allard
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Calcul of the sedimentation velocity in the cloud                c
c     (according to Rossow (1978), Icarus 30, 1-50)                    c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Auteur: T. Guillot d'origine (Avril 2000)                        c
c Entrees:                                                             c
c     p,t,ro: pression, temperature, gas densite (cgs)                 c
c     grav: gravity (en cm/s2)                                         c
c     size: radius of the grain en cm                                  c
c Sorties:                                                             c
c     vfall: sedimentation velocity                                    c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none

      real*8 p,t,ro,grav,size,vfall
      real*8 nu,sig,mu,rn,lambda,a70,rop,kn,re

c     constants:
      real*8 pi,kbol,amu,rgas
      data pi/3.1415926539d0/,kbol/1.380658d-16/,amu/1.6605402d-24/
      data rgas/8.3145d7/

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Averaged grain material density in gr/cm^3 [rop]                c
c     -Section efficace du gaz [sig]				       c
c     -Masse molaire moyenne du gaz [mu]			       c
c     -Atmospheric density by number of particules/cm3 [rn]	       c
c     -mean free path of the gaz [lambda]			       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      rop = 3.8d0               !for dirty forsterite (MgFeSiO4)
      sig= pi*(2.d-8)**2.d0     !cfa06: radius of H2 molecule, not HI!
      mu=ro*kbol*t/(p*amu)
      rn=ro/mu/amu
      lambda=sqrt(3.d0*pi)/(8.d0*rn*sig)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -dynamical viscosity [nu]					       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      nu=ro/sqrt(t)
cfa06 une Bernd's viscosity!
      nu=ro*lambda*sqrt(8.d0*rgas*t/(pi*mu))

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Size of particules for which Re=70                               c
c     Corrected according to Carlson et al. 1988                       c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      a70=(9.d0*270.d0*nu*nu/(4.d0*rop*ro*grav))**(1.d0/3.d0)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     -Knudsen number [kn]                                             c
c     -Reynolds number [re]                                            c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      kn=lambda/size
      re=70.d0*size/a70

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      sedimentation velocity:
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       if (kn.gt.1.d0) then
c       subsonic free molecular flow
        vfall=(16.d0*rop*grav*size)*sqrt(pi*mu*amu/(8.d0*kbol*t))
     &         /(27d0*ro)
       elseif (re.gt.70.d0) then
c       turbulent flow (Newton)
        vfall=sqrt(4.d0*rop*grav*grav*size))/(3.d0*ro)
       else
c       laminar viscous flow (stokes)
        vfall=(2.d0*rop*grav*size*size)/(9.d0*nu)
       endif
c      write(*,'(1p,4d10.3)')p,t,size,vfall
      return
      end
